//
//  JRF5NSTextView.m
//
//  Created by Jay Freeman on 06/14/2010
//  Copyright 2007 Jay Reynolds Freeman. All rights reserved.
//

//  Specialized for the Pixie Scheme II main view.
/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628
*/

#import "JRF5NSTextView.h"
#import "PixieSchemeIIController.h"
#import "ASCIITypes.h"
#import "SchemeCType.h"

@implementation JRF5NSTextView

- (void) awakeFromNib {
    allowedTypes = [[NSArray arrayWithObjects: NSStringPboardType, nil] retain];
    typingRange.location = 0;
    typingRange.length = 0;
    needsNewTypingRange = TRUE;
    [self setAllowsUndo:YES];
    [self setUsesFindPanel:YES];
    }

- (void) setDealWithText:(SEL)aMethod { dealWithText = aMethod; }


- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender {

    NSRange selectionRange;

    bool result = [super performDragOperation:sender];

    if( result ) {
        selectionRange = [[[self selectedRanges] objectAtIndex:0] rangeValue];
        [self setNeedsNewTypingRange:YES];
        [theController dealWithSelection:YES];
//        [self showSelectedRange];
//        [self setSelectedRange:selectionRange];
        }
    return result;
    }

- (void) paste:(id)sender {

    NSInteger startLocation = [[[self selectedRanges] objectAtIndex:0] rangeValue].location;
    NSInteger endLocation = 0;

    [super paste:sender];

    endLocation = [[[self selectedRanges] objectAtIndex:0] rangeValue].location;

    typingRange.location = startLocation;
    typingRange.length = endLocation - startLocation;
    [self setNeedsNewTypingRange:NO];

    [self showSelectedRange];
    [theController dealWithTextRange:[self hasTypingRange]];
    }

- (void) appendNewline { [self appendString:@"\n"]; }

- (void) appendString:(NSString *)aString {

    NSRange endRange;

    endRange.location = [[self string] length];
    endRange.length = 0;
    [self replaceCharactersInRange:endRange withString:aString];

    }

- (void) insertString:(NSString *)aString {

    NSArray *selRanges = [self selectedRanges];
    NSRange selectionRange;

    if( [selRanges count] != 0 ) {
        selectionRange = [[selRanges objectAtIndex:0] rangeValue];
        [self replaceCharactersInRange:selectionRange withString:aString];
        }
    [self showSelectedRange];
    }

- (NSRange) getSelectedRange {

    NSArray *selRanges = [self selectedRanges];
    NSRange selectionRange;

    selectionRange.location = 0;
    selectionRange.length = 0;

    if( [selRanges count] != 0 )
        selectionRange = [[selRanges objectAtIndex:0] rangeValue];

    return selectionRange;
    }

- (void) showEndRange {

    NSRange endRange;

    endRange.location = [[self string] length];
    endRange.length = 0;
    [self setSelectedRange:endRange];
    [self scrollRangeToVisible:endRange];

    }

- (void) showSelectedRange {

    NSArray *selRanges = [self selectedRanges];

    if( [selRanges count] != 0 ) {
        [self scrollRangeToVisible:[[selRanges objectAtIndex:0] rangeValue]];

        // This next if statement fixes slight glitchiness in where in the
        // window the text is displayed, depending on whether we got to it by
        // scrolling up or down.  There is an off-by-one.  Best aesthetics
        // turns out to be by scrolling down, so we force that when possible.

        if( ![self inFirstLine] ) {
            [super moveUp:self];
            [super moveDown:self];
            }
        }
    }

- (bool) inFirstLine {

    NSArray *selRanges = [self selectedRanges];
    NSRange selectedRange;
    NSInteger counter;
    NSString *allText = [self string];
    NSInteger lnth = [allText length];

    if( [selRanges count] != 0 ) {
        selectedRange = [[selRanges objectAtIndex:0] rangeValue];
        if( lnth > 0 )
            for( counter = selectedRange.location - 1; counter >= 0; --counter )
                if( [allText characterAtIndex:counter ] == NEWLINE )
                    return NO;
        return YES;
        }
    return NO;
    }

- (bool) inLastLine {

    NSArray *selRanges = [self selectedRanges];
    NSRange selectedRange;
    NSInteger counter;
    NSString *allText = [self string];
    NSInteger lnth = [allText length];

    if( [selRanges count] != 0 ) {
        selectedRange = [[selRanges objectAtIndex:0] rangeValue];
        for( counter = selectedRange.location; counter < lnth; ++counter )
            if( [allText characterAtIndex:counter ] == NEWLINE )
                return NO;
        return YES;
        }
    return NO;
    }

- (NSRange) currentLinesRange {

    NSArray *selRanges = [self selectedRanges];
    NSRange selectedRange = [[selRanges objectAtIndex:0] rangeValue];
    NSString *allText = [self string];
    NSInteger textLength = [allText length];
    NSRange answer;
    NSInteger answerEnd;

    if( textLength == 0 ) {
        answer.location = 0;
        answer.length = 0;
        return answer;
        }

    answer.location = selectedRange.location;
    if( (answer.location >= textLength) || ([allText characterAtIndex:answer.location] == NEWLINE) )
        --(answer.location);
    for( ;
         (answer.location > 0) && ([allText characterAtIndex:answer.location] != NEWLINE);
         --(answer.location) )
        ;
    if( [allText characterAtIndex:answer.location] == NEWLINE )
        ++(answer.location);


    for( answerEnd = selectedRange.location + selectedRange.length;
         (answerEnd < textLength) && ([allText characterAtIndex:answerEnd] != NEWLINE);
         ++answerEnd )
        ;

    answer.length = answerEnd - answer.location;

    return answer;
    }

- (IBAction) selectCurrentLines:(id)sender {

    [self setSelectedRange:[self currentLinesRange]];
    [self setNeedsNewTypingRange:YES];
    [theController dealWithTextRange:NO];
    [theController dealWithSelection:YES];
    }

- (bool) atFirstCharacterInLine {

    NSArray *selRanges = [self selectedRanges];
    NSRange selectedRange;
    NSString *allText = [self string];

    if( [selRanges count] != 0 ) {
        selectedRange = [[selRanges objectAtIndex:0] rangeValue];
        if( selectedRange.location == 0 )
            return YES;
        if( [allText characterAtIndex:(selectedRange.location - 1)] == NEWLINE )
            return YES;
        }
    return NO;
    }

- (bool) atLastCharacterInLine {

    NSArray *selRanges = [self selectedRanges];
    NSRange selectedRange;
    NSString *allText = [self string];
    NSUInteger lnth = [allText length];

    if( [selRanges count] != 0 ) {
        selectedRange = [[selRanges objectAtIndex:0] rangeValue];
        if( selectedRange.location == lnth )
            return YES;
        if( [allText characterAtIndex:selectedRange.location] == NEWLINE )
            return YES;
        }
    return NO;
    }

- (void) lineStartsAt:(NSUInteger *)lineLocation withLength:(NSUInteger *)lineLength withSelectionOffset:(NSInteger *)selOffset {

    NSArray *selRanges = [self selectedRanges];
    NSRange selectedRange;
    NSString *allText = [self string];
    NSInteger lnth = [allText length];
    NSInteger pastChar;

    if( [selRanges count] != 0 ) {
        selectedRange = [[selRanges objectAtIndex:0] rangeValue];
        for( *lineLocation = selectedRange.location; *lineLocation >= 1; --(*lineLocation) )
            if( [allText characterAtIndex:((*lineLocation) - 1)] == NEWLINE )
                break;
        *selOffset = selectedRange.location - *lineLocation;
        for( pastChar = selectedRange.location; pastChar < lnth; ++pastChar )
            if( [allText characterAtIndex:pastChar] == NEWLINE )
                break;
        *lineLength = pastChar - *lineLocation;
        }
    }

- (void) lastLineStartsAt:(NSUInteger *)lineLocation withLength:(NSUInteger *)lineLength {

    NSRange selectedRange;
    NSString *allText = [self string];
    NSUInteger lnth = [allText length];
    NSUInteger pastChar;

    selectedRange.location = lnth;
    selectedRange.length = 0;
    for( *lineLocation = selectedRange.location; *lineLocation >= 1; --(*lineLocation) )
        if( [allText characterAtIndex:((*lineLocation) - 1)] == NEWLINE )
            break;
    for( pastChar = selectedRange.location; pastChar < lnth; ++pastChar )
        if( [allText characterAtIndex:pastChar] == NEWLINE )
            break;
    *lineLength = pastChar - *lineLocation;
    }

- (void) scrollToBottom {

    NSClipView *theClipView = nil;
    NSRange endRange;
    NSPoint newPoint;
    NSRect clipViewBounds;
    NSRect mainBounds;
    NSInteger lnth = [[self string] length];

    endRange.location = lnth;
    if( endRange.location < 0 )
        endRange.location = 0;
    endRange.length = 0;

    [self scrollRangeToVisible:endRange];   // Doesn't work if last two characters are both 0x0a; erratic.  Fix-up follows:

    mainBounds = [self bounds];
    theClipView = (NSClipView *)[self superview];
    clipViewBounds = [theClipView bounds];

    newPoint.x = 0;
    newPoint.y = mainBounds.size.height - clipViewBounds.size.height;

    if( newPoint.y < 0 )
        newPoint.y = 0;

    [theClipView scrollToPoint:newPoint];
    }

- (void) disableShortcuts {

    [iMPMenuItem setEnabled:NO];
    [sSWPMenuItem setEnabled:NO];
    [iPOQMMenuItem setEnabled:NO];
    [sSWQMMenuItem setEnabled:NO];
    }

- (void) enableShortcuts {

    [iMPMenuItem setEnabled:YES];
    [sSWPMenuItem setEnabled:YES];
    [iPOQMMenuItem setEnabled:YES];
    [sSWQMMenuItem setEnabled:YES];
    }

- (void) transmitAll:(NSString *)theString {

    if( [theString length] > 0 ) {
        [theController performSelector:dealWithText withObject:theString];
        }
    }

- (IBAction) insertMatchedParentheses:(id)sender {

    if( [[theWindowController getMainWindow] firstResponder] == self ) {
        [self insertString:@"()"];
        NSArray *selRanges = [self selectedRanges];
        NSRange selectionRange = [[selRanges objectAtIndex:0] rangeValue];
        selectionRange.location = selectionRange.location - 1;
        [self setSelectedRange:selectionRange];
        [self setNeedsNewTypingRange:YES];
        [theController dealWithTextRange:NO];
        }
    else
        NSBeep();
    }

- (IBAction) surroundSelectionWithParentheses:(id)sender {

    NSArray *selRanges = [self selectedRanges];
    NSRange selectionRange = [[selRanges objectAtIndex:0] rangeValue];
    NSRange newRange;
    NSInteger savedLength = 0;    

    if( selectionRange.length == 0 )
        [self insertMatchedParentheses:sender];
    else {
        savedLength = selectionRange.length;
        newRange.location = selectionRange.location;
        newRange.length = 0;
        [self setSelectedRange:newRange];
        [self insertString:@"("];
        newRange.location = newRange.location + savedLength + 1;
        [self setSelectedRange:newRange];
        [self insertString:@")"];
        [self setSelectedRange:newRange];
        [self setNeedsNewTypingRange:YES];
        [theController dealWithTextRange:NO];
        }
    }

- (IBAction) insertPairOfQuotationMarks:(id)sender {

    if( [[theWindowController getMainWindow] firstResponder] == self ) {
        [self insertString:@"\"\""];
        NSArray *selRanges = [self selectedRanges];
        NSRange selectionRange = [[selRanges objectAtIndex:0] rangeValue];
        selectionRange.location = selectionRange.location - 1;
        [self setSelectedRange:selectionRange];
        [self setNeedsNewTypingRange:YES];
        [theController dealWithTextRange:NO];
        }
    else
        NSBeep();
    }

- (IBAction) surroundSelectionWithQuotationMarks:(id)sender {

    NSArray *selRanges = [self selectedRanges];
    NSRange selectionRange = [[selRanges objectAtIndex:0] rangeValue];
    NSRange newRange;
    NSInteger savedLength = 0;    

    if( selectionRange.length == 0 )
        [self insertPairOfQuotationMarks:sender];
    else {
        savedLength = selectionRange.length;
        newRange.location = selectionRange.location;
        newRange.length = 0;
        [self setSelectedRange:newRange];
        [self insertString:@"\""];
        newRange.location = newRange.location + savedLength + 1;
        [self setSelectedRange:newRange];
        [self insertString:@"\""];
        [self setSelectedRange:newRange];
        [self setNeedsNewTypingRange:YES];
        [theController dealWithTextRange:NO];
        }
    }



// Overridden to allow checking for built-in Scheme names.

- (NSArray *)completionsForPartialWordRange:(NSRange)charRange indexOfSelectedItem:(NSInteger *)i {

    NSArray *answer = [theController spellingCompletionsForPartialWord:[[self string] substringWithRange:charRange]];

    if( [answer count] == 1 )
        *i = 0;
    else
        *i = -1;

    return answer;

    }

- (NSRange)rangeForUserCompletion {

    NSArray *selRanges = [self selectedRanges];
    NSRange selectionRange = [[selRanges objectAtIndex:0] rangeValue];
    NSString *theString = [self string];
    NSRange destinationRange = selectionRange;
    SInt32 c = 0;    
    SInt64 idx = 0;

    for( idx = selectionRange.location - 1; idx >= 0; --idx ) {
        c = [theString characterAtIndex:idx];
        if( isSubsequent_M( c ) || isInitial_M( c ) ) {
            destinationRange.location = idx;
            (destinationRange.length)++;
            }
        else
           break;
        }
    return destinationRange;
    }

- (IBAction)markOrTransmitSelection:(id)sender {

    if( [self hasTypingRange] ) {
        [self setNeedsNewTypingRange:YES];
        [self setSelectedRange:typingRange];
        [theController dealWithSelection:YES];
        }
    else if( [self hasSelectedText] ) {
        [self setNeedsNewTypingRange:YES];
        [theController dealWithTextRange:NO];
    	[self transmitSelection:sender];
        }
    }

- (IBAction)transmitSelection:(id)sender {

    NSArray *selRanges = [self selectedRanges];
    NSRange selectionRange = [[selRanges objectAtIndex:0] rangeValue];
    NSRange newSelectionRange;

    if( selectionRange.length == 0 )
        NSBeep();
    else {
        newSelectionRange.location = selectionRange.location + selectionRange.length;
        newSelectionRange.length = 0;
        [self replaceCharactersInRange:newSelectionRange withString:@"\n"];
        newSelectionRange.location += 1;
        [self setSelectedRange:newSelectionRange];
        [self transmitAll:[[self string] substringWithRange:selectionRange]];
        }
    }

- (bool) hasSelectedText {

    NSArray *selRanges = [self selectedRanges];
   
    if( [selRanges count] == 0 )
        return NO;
    else
        return ([[selRanges objectAtIndex:0] rangeValue].length > 0);
    }

- (bool) hasTypingRange { return (! needsNewTypingRange); }

- (void) setNeedsNewTypingRange:(bool)value {

    needsNewTypingRange = value;
    }

- (void) startTypingRange {

    NSArray *selRanges = [self selectedRanges];

    if( (! [self hasTypingRange]) && ([selRanges count] > 0) ) {
       typingRange.location = [[selRanges objectAtIndex:0] rangeValue].location - 1;
       typingRange.length = 0;
       [self setNeedsNewTypingRange:NO];
       }
    }

- (void) adjustTypingRange { 

    NSArray *selRanges = [self selectedRanges];

    if( [self hasTypingRange] )
        if( [selRanges count] > 0 )
           typingRange.length = [[selRanges objectAtIndex:0] rangeValue].location - typingRange.location;
    }

- (void)keyDown:(NSEvent *)theEvent {

    startLengthForKeyPress = [[self string] length];
    [super keyDown:theEvent];
    endLengthForKeyPress =  [[self string] length];

    if(    (endLengthForKeyPress > startLengthForKeyPress)          // Key inserted, for sure ...
        || isprint( [[theEvent characters] characterAtIndex:0] )    //   ... or maybe we replaced a single selected character.
        )
        if( ! [self hasTypingRange] )
            [self startTypingRange];
    }

- (void) keyUp:(NSEvent *)theEvent {

    [super keyUp:theEvent];

    if(    (endLengthForKeyPress != startLengthForKeyPress)          // Key inserted, for sure ...
        || isprint( [[theEvent characters] characterAtIndex:0] )     //   ... or maybe we replaced a single selected character.
        ) {
        [self adjustTypingRange];
        }
    else if( [[theEvent characters] characterAtIndex:0] == ESCAPE )  // Used for typing completion, do NOT end the range if
        [self adjustTypingRange];                                    //  completion fails.
    else
        [self setNeedsNewTypingRange:YES];
    
    if( [self hasSelectedText] ) {
        [theController dealWithSelection:YES];
        [self setNeedsNewTypingRange:YES];
        }
    else
        [theController dealWithTextRange:[self hasTypingRange]];

    }
    
- (void) mouseDown:(NSEvent *)theEvent {

    [super mouseDown:theEvent];
    if( [self hasSelectedText] ) {
        [theController dealWithSelection:YES];
        [self setNeedsNewTypingRange:YES];
        }
    else {
        [self setNeedsNewTypingRange:YES];
        [theController dealWithTextRange:NO];
        }
    }
    
//- (BOOL)acceptsFirstResponder {
//
//    return YES;
//    }

- (void)selectAll:(id)sender {

    [super selectAll:sender];
    [theController dealWithSelection:[self hasSelectedText]];
    }

- (IBAction)copy:(id)sender {

    [super copy:sender];
    [theController dealWithSelection:[self hasSelectedText]];
    }

- (IBAction)cut:(id)sender {

    [super cut:sender];
    [theController dealWithSelection:[self hasSelectedText]];
    }

/****

- (IBAction)foo:(id)sender {

    [super foo:sender];
    [theController dealWithSelection:[self hasSelectedText]];
    }

****/

//- (void)setSelectedRange:(NSRange)charRange affinity:(NSSelectionAffinity)affinity stillSelecting:(BOOL)flag {
//
//    [super setSelectedRange:charRange affinity:affinity stillSelecting:flag];
//
//    if( !flag )
//        [theController dealWithSelection:YES];
//    }

- (BOOL) isOpaque { return YES; }

@end
