 
//
//  PixieIIWindowController.m
//
//  Created by Jay Freeman on 06/13/2010.
//  Copyright 2006 Jay Reynolds Freeman. All rights reserved.
//
/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628
*/

#import "PixieIIWindowController.h"
#import "JRF5NSTextView.h"
#define PIXIE_SCHEME_II_FONT @"Monaco"
#define PIXIE_SCHEME_II_BOLD_FONT @"Monaco"
#define INITIAL_MAIN_BUFFER_SIZE 4096

extern char releaseDate[];
extern int usleep(useconds_t useconds);

@implementation PixieIIWindowController

- (void)awakeFromNib {

    specialYellow = [NSColor colorWithCalibratedRed:1.00  green:1.00  blue:0.67450980392  alpha:1.00];
    black = [NSColor blackColor];

    applicationFont = [mainView font];
    currentFontSize = [applicationFont pointSize];
    currentLineSize = matchingLineHeight( currentFontSize );

//    disasterMeow = [NSSound soundNamed:@"DisasterMeow"];  // FIXME
    

    [mainWindow center];
    [mainView scrollToBottom];
    
    // The next line is a hack:  I cannot get the interface builder to set the autosize
    //   properties of just this one field to be the way I want them.  All the other fields
    //   behave as I expect, but this one is the odd entity out ... ?
    
    [fontBiggerItem   setEnabled:YES];
    [fontSmallerItem  setEnabled:YES];

    [self setMainViewText:@""];

    [mainView          setBackgroundColor:        specialYellow];
    [mainView          setTextColor:              black];

    mainDisplayBuffer = [[[NSMutableString alloc] initWithCapacity:INITIAL_MAIN_BUFFER_SIZE] retain];

    }
    
- (void)displayAll { [mainView display]; }

// MainView

- (void)displayStringInMainView:(NSString *)str {

//    [mainView setString:[[mainView string] stringByAppendingString:str]];
//    [mainView setNeedsNewTypingRange:YES];
//    [mainView scrollToBottom];

    NSArray *selRanges = [mainView selectedRanges];
    NSRange selectionRange = [[selRanges objectAtIndex:0] rangeValue];
    [mainView replaceCharactersInRange:selectionRange withString:str];
    [mainView showSelectedRange];
    [mainView setNeedsDisplay:YES];
    }

- (void)setMainViewText:(NSString *)msg {
 
    [mainView setString:msg];
    [mainView setFont:[NSFont fontWithName:PIXIE_SCHEME_II_FONT size:currentFontSize]];
    [mainView scrollToBottom];
    }

- (void)setFontsToSize:(CGFloat)fontSize {
    [mainView setFont:[NSFont fontWithName:PIXIE_SCHEME_II_FONT size:fontSize]];
    }

- (CGFloat) currentFontSize { return currentFontSize; }

- (void) setCurrentFontSize:(CGFloat)fontSize {
    currentFontSize = fontSize;
    [self setFontsToSize:fontSize];
    }

- (IBAction)playDisasterMeow:(id)sender {

  if( ! [disasterMeow isPlaying] ) {
        [disasterMeow play];
        while( [disasterMeow isPlaying] )
            ;
        }
    }

- (IBAction)playDisasterMeowAndWait:(id)sender {

    if( ! [disasterMeow isPlaying] )
        [disasterMeow play];
    while( [disasterMeow isPlaying] )
	;
    }

// Lots of stuff for changing font size and modifying the height of the various
// panels of the Pixie II Scheme window in consequence.  Here is where a lot of
// work would need to be done to make things behave well if resolution independence
// goes through in the manner described in the pre-release days of Leopard.

- (IBAction)makeFontBigger:(id)sender {

    CGFloat newFontSize = nextLargestFontSize( currentFontSize );
    CGFloat newLineSize = matchingLineHeight( newFontSize );
    
    currentFontSize = newFontSize;
    currentLineSize = newLineSize;

    [self setFontsToSize:newFontSize ];
    [mainView showSelectedRange];

    if( currentFontSize >= 30. )
        [fontBiggerItem setEnabled:NO];
    else
        [fontBiggerItem setEnabled:YES];
    if( currentFontSize >= 9.5 )
        [fontSmallerItem setEnabled:YES];
    else
        [fontSmallerItem setEnabled:NO];
    }

- (IBAction)makeFontSmaller:(id)sender {

    CGFloat newFontSize = nextSmallestFontSize( currentFontSize );
    CGFloat newLineSize = matchingLineHeight( newFontSize );

    currentFontSize = newFontSize;
    currentLineSize = newLineSize;

    [self setFontsToSize:newFontSize ];
    [mainView showSelectedRange];

    if( currentFontSize <= 30. )
        [fontBiggerItem setEnabled:YES];
    else
        [fontBiggerItem setEnabled:NO];
    if( currentFontSize <= 9.5 )
        [fontSmallerItem setEnabled:NO];
    else
        [fontSmallerItem setEnabled:YES];
    }

CGFloat nextLargestFontSize( CGFloat fontSize ) {
    if( fontSize < 9.5 )
        return 10.;
    else if( fontSize < 11. )
        return 12.;
    else if( fontSize < 13. )
        return 14.;
    else if( fontSize < 16. )
        return 18.;
    else if( fontSize < 21. )
        return 24.;
    else
        return 36.;
    }
    
CGFloat nextSmallestFontSize( CGFloat fontSize ) {
    if( fontSize > 30 )
        return 24.;
    else if( fontSize > 21. )
        return 18.;
    else if( fontSize > 16. )
        return 14.;
    else if( fontSize > 13. )
        return 12.;
    else if( fontSize > 11. )
        return 10.;
    else
        return 9.;
    }

CGFloat matchingLineHeight( CGFloat fontSize ) {
    if( fontSize < 9.5 )
        return 12.;
    else if( fontSize < 11. )
        return 14;
    else if( fontSize < 13. )
        return 16.;
    else if( fontSize < 16. )
        return 19.;
    else if( fontSize < 21. )
        return 25.;
    else if( fontSize < 30. )
        return 32.;
    else
        return 48.;
    }

- (NSWindow *)getMainWindow { return mainWindow; }

@end

void displayRect( char *msg, NSRect aRect )
{
    printf( "Rect \"%s\": origin %f %f size %f %f\n",
            msg, aRect.origin.x, aRect.origin.y, aRect.size.width, aRect.size.height );
    }

