
//
//  PixieSchemeIIController.h
//
//  Created by Jay Freeman on 06/13/2010
//  Copyright 2006 Jay Reynolds Freeman. All rights reserved.
//
/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628
*/

#import <sys/types.h>
#import <sys/stat.h>

#define WORD_MASK 0xffffffff

#import <Cocoa/Cocoa.h>
#import "CocoaInterface.h"
#import "CocoaIO.h"
#import "UserSetup.h"
#import "MainAux.e"

@class PixieIIWindowController;
@class StatusLightController;
@class PixieIISenseLightArrayController;
@class JRF5NSTextView;
@class CustomDockIconView;

@interface PixieSchemeIIController : NSObject {

    bool evaluating;
    bool displayingMessage;
    bool stopEvaluatingPending;
    bool gcing;
    bool outputEnabled;
    bool wantsToQuit;

    bool expectMoreInput;

    int nextTask;
    long messageDisplayCounter;

    NSInteger mainMemorySizeInUse;
    NSInteger generationalMemorySizeInUse;
            
//    NSMutableArray *symbolsInObList;
    NSArray *symbolsInPixieSchemeDictionary;

    IBOutlet NSWindow *myParentWindow;
    IBOutlet PixieIIWindowController *mWController;

    IBOutlet NSLevelIndicator *MyLevelIndicator;

    IBOutlet StatusLightController *eVBController;
    IBOutlet StatusLightController *gCBController;

    IBOutlet PixieIISenseLightArrayController *sLAController;

    IBOutlet NSMenuItem *cDMenuItem;
    IBOutlet NSMenuItem *sFPMenuItem;
    IBOutlet NSMenuItem *cRMenuItem;

             bool showingGenerationalInformation;

    IBOutlet NSScrollView *mainScrollView;
        
    IBOutlet JRF5NSTextView *mainView;
             bool showingExpectView;
             bool cyclingExpectView;
    IBOutlet NSTextField *expectStuff;

    IBOutlet NSPanel *myDonationSheet;

    IBOutlet NSPanel *userDialog;
    IBOutlet NSTextField *dialogPrompt;
    IBOutlet NSTextField *dialogResponse;

    IBOutlet NSImageView *donationSheetImageView;
             NSImage *sadKitty;
             NSImage *pixieFace;
    IBOutlet NSTextField *boldField;
    IBOutlet NSTextField *normalField;
    IBOutlet NSButton *closeButton;
    IBOutlet NSButton *markOrEvalButton;
    IBOutlet NSButton *resetSchemeButton;
    }

- (id) init;
//- (void)loadMutableArray:(NSMutableArray *)nsa fromCStringArrayEndingInZero:(char **)csa startingWith:(int)start;
- (void)didUndo:(NSNotification *)notification;
- (void)didRedo:(NSNotification *)notification;
- (NSArray *)spellingCompletionsForPartialWord:(NSString *)partWord;
- (NSArray *) createCompletionArray;
- (void) beginInitLevel;
- (void) updateLevel:( double )newValue;
- (void) endInitLevel;
- (void) doControllerInitializationTask:(NSTimer *)theTimer;
- (bool) controllerInitializationTask;
- (void)setDefaultControllerSetup;
- (void)setControllerSetup;
- (void) donationSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo;
- (IBAction) closeDonationSheet:(id)sender;
- (IBAction) runReloadWorldSheet:(id)sender;
- (void) reloadWorldSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo;
- (IBAction) openForWorldLoad:(id)sender;
- (void) worldLoadSucceeded:(const char *)path;
- (IBAction) openForWorldSave:(id)sender;
- (IBAction) doQuit:(id)sender;
- (void) reportDisasterAndQuitDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo;
- (void) reportDisasterAndQuit:(const char *)errorMessage;
- (void) reportEarlyDisasterAndQuitDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo;
- (void) reportEarlyDisasterAndQuit;
- (IBAction) runDisableOutputSheet:(id)sender;
- (void) disableOutputSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo;
- (IBAction) runDonationSheet:(id)sender;
- (void) runEvaluator:(id)theObject;
- (void) setNormalLights;
- (void) setForForgettingTyping;
- (void) turnExpectStuffOn;
- (void) turnExpectStuffOff;
- (void) setExpectStuffHidden;
- (void) setExpectStuffVisible;
- (void) setForEvaluating;
- (void) setForNotEvaluating;
- (void) setForGCing;
- (void) setForNotGCing;
- (void)showDialogSheet:(NSWindow *)window;
- (IBAction)closeDialogSheet:(id)sender;
- (void)didEndUserDialogSheet:(NSWindow *)sheet returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo;
- (void) startDialog:(NSString *)thePrompt;
- (bool) dealWithResponseText:(NSString *)theText;
- (void) startNormalInput;
- (IBAction) toggleCompileDefines:(id)sender;
- (IBAction) toggleFullNumericPrecision:(id)sender;
- (IBAction) requestReset:(id)sender;
- (void) tellEvaluator:(int)mID withLong:(long)mLong withString:(const char *)mString;
- (void) postMakeFinalWorld;
- (bool) pollEvaluatorOnce;
- (void) localError:(NSString *)message;
- (void) localError:(NSString *)message onWindow:(NSWindow *)window;
- (IBAction) openDictionaryFile:(id)sender;
- (IBAction) openHelpFile:(id)sender;
- (IBAction) openGNU:(id)sender;
- (void) provideUserSetup;
- (void) newViewString:(const char *)newString forView:(NSTextField *)theView;
- (IBAction) openSourceCode1:(id)sender;
- (IBAction) openSourceCode2:(id)sender;
- (IBAction) openSourceCode3:(id)sender;
- (IBAction) openSourceCode4:(id)sender;
- (IBAction) openSourceCode5:(id)sender;
- (IBAction) openSourceCode6:(id)sender;
- (IBAction) openSourceCode7:(id)sender;
- (IBAction) openSourceCode8:(id)sender;
- (IBAction) openSourceCode9:(id)sender;
- (IBAction) openSourceCode10:(id)sender;
- (IBAction) openReadMe:(id)sender;

- (void) dealWithSelection:(bool)value;
- (void) dealWithTextRange:(bool)value;


- (IBAction)findInPixieSchemeDictionary:(id)sender;

- (IBAction)Test1:(id)sender;
- (IBAction)Test2:(id)sender;
- (IBAction)Test3:(id)sender;
- (IBAction)Test4:(id)sender;
- (IBAction)Test5:(id)sender;
- (IBAction)Test6:(id)sender;
- (IBAction)Test7:(id)sender;
- (IBAction)Test8:(id)sender;

@end
