//
//  PixieSchemeIIController.mm
//
//  Created by Jay Freeman on 06/13/2010.
//  Copyright 2006 Jay Reynolds Freeman.
//
/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628
*/

// #defines for initialization waits:

#define WAIT_FOR_EVALUATOR_DELAY 1000

// #defines for debugging:

#define SHOW_THREAD_INTERACTIONS FALSE      // Make TRUE to get log of thread-geraction details.

// During initialization, every so often a timer fires to attempt a controller/evaluator
// thread interaction that requires their cooperation to get it done.  This next #define 
// is the delay between such timer firings.  Changing this probably requires a 

#define STANDARD_INITIALIZATION_TIMER_INTERVAL 0.1
#define EXPECTS_VIEW_TIMER 2.0

// The size of string passed from the controller to the evaluator is variable; the
// next #define is the length started out with.  (Old string is freed and new calloced
// when necessary.)

#define START_STRING_SIZE 512

#import "PixieSchemeIIController.h"
#import "PixieIIWindowController.h"
#import "StatusLightController.h"
#import "PixieIISenseLightArrayController.h"
#import "JRF5NSTextView.h"
#import "UserSetup.h"

#import "Globals.e"

extern void pollControllerOnce();
extern int runSchemeWithPathAndMemory( const char *argv0, long memorySizeProvided );
extern void cleanUpForExit();
extern char *messagesFromControllerNames[];
extern char *messagesFromEvaluatorNames[];
extern void reportGUIInitialized();
extern void increaseBlocking();
extern void setFrontKitten( int aKitten, int aValue );
extern int getFrontKitten( int aKitten );
extern Boolean fatalErrorOccurred;
//extern int evaluatorHasInitialized;
extern LongWord progressFractionInThousandths;
extern Boolean usingGenerationalGC;

threadMessage toEvaluator;
threadMessage toController;

SInt32 outputEnabled = 1;
SInt32 waitingToExit = 0;
SInt32 clowderIsOperating = 0;
SInt32 worldIsLoaded = 0;
SInt32 controllerReadyForMessages = 0;
SInt32 evaluatorHasInitialized = 0;
SInt32 GUIInitialized = 0;
SInt32 readyToRead = 0;

extern volatile int controllerOutboundMailboxFlag;
extern Text appSupportWorldFileName[];
extern bool isOriginalWorld;

NSLock *controllersMailbox = nil;
NSLock *evaluatorsMailbox = nil;

PixieSchemeIIController *thePixieSchemeController;

int worldFileID;

#define BUFSIZE_FOR_PATHNAME 1000

Text worldFilePathName[ BUFSIZE_FOR_PATHNAME + 1 ];

//########  Stuff to set metadata for saved worlds.

// const OSType PixieSchemeIICreatorCode      = 'cld4';  // Not required for Pixie Scheme II.I
// const OSType PixieSchemeIIDocumentTypeCode = 'kdl4';  // Not required for Pixie Scheme II.I

void setWorldAttributesForPath( const char *path ) 
{
    // Pixie Scheme II does not use any worlds except those at known positions in the file system.
    }

//################################################################################
//
//########  Stuff used during initialization.

SInt32 localArgc = 0;
char *localArgv[ 12 ] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };

@implementation PixieSchemeIIController


//########  "init" mostly sets up controller state variables.

- (id)init {

    self = [super  init];

    worldFileID = 0;

    evaluating = false;
    displayingMessage = false;
    stopEvaluatingPending = false;
    gcing = false;
    outputEnabled = TRUE;
    wantsToQuit = false;

    expectMoreInput = false;

    nextTask = 1;
    
    toEvaluator.messageID = C_NO_MESSAGE_YET;
    toEvaluator.theString = 0;
    toEvaluator.theLong = 0;

    toController.messageID = E_NO_MESSAGE_YET;
    toController.theString = 0;
    toController.theLong = 0;

    controllersMailbox = [[[NSLock alloc] init] retain];
    evaluatorsMailbox = [[[NSLock alloc] init] retain];

    srandom( (SInt32)(time( 0 ) & WORD_MASK));

    return self;
    }


//################################################################################
//
//########  Stuff to control a level indicator.

- (void)beginInitLevel {

    [MyLevelIndicator setDoubleValue:0.0];
    [MyLevelIndicator setHidden:NO];
    [MyLevelIndicator display];
    }

- (void)updateLevel:( double )newValue {

    [MyLevelIndicator setDoubleValue:newValue];
    [MyLevelIndicator setHidden:NO];
    [MyLevelIndicator display];
    }

- (void)endInitLevel {

    [MyLevelIndicator setHidden:YES];
    [MyLevelIndicator display];
    }


- (NSArray *)spellingCompletionsForPartialWord:(NSString *)partWord {

    NSPredicate *aPredicate = [NSPredicate predicateWithFormat:@"SELF beginswith[c] %@", partWord ];

//    return [symbolsInObList filteredArrayUsingPredicate:aPredicate];
    return [symbolsInPixieSchemeDictionary filteredArrayUsingPredicate:aPredicate];
    }



//- (void)loadMutableArray:(NSMutableArray *)nsa fromCStringArrayEndingInZero:(char **)csa startingWith:(int)start {
//
//    for( int counter = start; csa[ counter ] != 0; ++counter )
//        [nsa addObject:[NSString stringWithCString:csa[ counter ] encoding:NSASCIIStringEncoding]];
//    }


//########   "applicationDidFinishLaunching" is lengthy but straightforward, up till the final
//########   call, to "arrangeForcontrollerInitializationTasksForEvaluator", which see ...

extern const unsigned long PIXIE_SCHEMES_MEM_SIZE_MBYTES;

- (void) applicationDidFinishLaunching:(NSNotification *)aNotification {

    thePixieSchemeController = self;

    double progress = 0.0;

    // Set up the submenu to load recent files ...

    symbolsInPixieSchemeDictionary = [[self createCompletionArray] retain];

    [MyLevelIndicator                setHidden:    NO];
    [MyLevelIndicator           setDoubleValue:    progress];

    [mWController displayAll];
    [mainView                     setDealWithText:    @selector(dealWithInputText2:)];
    [mainScrollView setDocumentView:mainView];     // TEST 2010.07.06
    [self setNormalLights];
    [sLAController initialize];
    [markOrEvalButton setTitle:@"Select/Eval"];
    [markOrEvalButton  setEnabled:NO];
    [resetSchemeButton setEnabled:NO];

    pixieFace = [[NSImage imageNamed:@"PixieIIFace.png"] retain];
    sadKitty = [[NSImage imageNamed:@"PixieIIFaceSadKitty.png"] retain];

    mainMemorySizeInUse         = PIXIE_SCHEMES_MEM_SIZE_MBYTES * 1048576;
    generationalMemorySizeInUse =                             1 * 1048576;

    showingExpectView = false;
    cyclingExpectView = false;
    [expectStuff setStringValue: @""];
    [expectStuff setHidden:YES];

    theNumberOfKittens = 1;
    [self setForEvaluating];                    //  To protect world load.
    [self setForNotGCing];                      //  To initialize the light.

    [myParentWindow setTitle:@"Pixie Scheme II"];

    //  And away we go ... (this next dispatches a thread that runs the evaluator).

    [NSThread detachNewThreadSelector:@selector(runEvaluator:) toTarget:self withObject:nil];

    while( ! worldIsLoaded ) {

        usleep( WAIT_FOR_EVALUATOR_DELAY );                
        if( fatalErrorOccurred )
            [self reportEarlyDisasterAndQuit];
        }

    [[NSNotificationCenter defaultCenter]
        addObserver: self
           selector: @selector(didRedo:)
               name: NSUndoManagerDidRedoChangeNotification
             object: nil];

    [[NSNotificationCenter defaultCenter]
        addObserver: self
           selector: @selector(didUndo:)
               name: NSUndoManagerDidUndoChangeNotification
             object: nil];

    if( fatalErrorOccurred )
        [self reportEarlyDisasterAndQuit];

    //  On your mark ... get set ... (this starts a series of events that must synchronize with evaluator initialization).

    // To avoid a "while( TRUE )" loop in "applicationDidFinishLaunching", we arrange to
    // fire a timer which calls a function that checks whether the evaluator has initialized, and does some tasks
    // for it if so; if not, it fires another timer to do the same thing,

    [NSTimer scheduledTimerWithTimeInterval  :  STANDARD_INITIALIZATION_TIMER_INTERVAL
                                     target  :  self
                                   selector  :  @selector(doControllerInitializationTask:)
                                   userInfo  :  nil
                                    repeats  :  NO];

//    [self Test1:self];    // For testing fatal error handling.

    }


- (void)didRedo:(NSNotification *)notification {

    NSRange selRange = [mainView getSelectedRange];

    selRange.location += selRange.length;
    selRange.length = 0;
    [mainView setSelectedRange:selRange];
    [mainView setNeedsNewTypingRange:YES];
    [self dealWithSelection:NO];
    }

- (void)didUndo:(NSNotification *)notification {

    NSRange selRange = [mainView getSelectedRange];

    selRange.location += selRange.length;
    selRange.length = 0;
    [mainView setSelectedRange:selRange];
    [mainView setNeedsNewTypingRange:YES];
    [self dealWithSelection:NO];
    }

//########  This is the code that runs during the initialization-timer events.
//########  It runs first on the basis of "arrangeForControllerInitializationTasksForEvaluator"'s timer,
//########  then on the basis of a timer which it itself sets.

- (void) doControllerInitializationTask:(NSTimer *)theTimer {

    if( [self controllerInitializationTask] )
        ;
    else
        [NSTimer scheduledTimerWithTimeInterval  :  STANDARD_INITIALIZATION_TIMER_INTERVAL
                                         target  :  self
                                       selector  :  @selector(doControllerInitializationTask:)
                                       userInfo  :  nil
                                        repeats  :  NO];
    }


- (void)setDefaultControllerSetup {
    }


- (void)setControllerSetup {

    int counter = 0;

    bool *senseLightState = [sLAController getState];
    bool *senseLightVisible = [sLAController getVisible];
    int *senseLightIllumination = [sLAController getIllumination];

    NSPoint scrollerOrigin;
    NSRange selectedRange;

    userSetup aSetup;
    NSRect windowRect;

    theWorldHandler->retrieveUserSetup( &aSetup );

    [mWController setCurrentFontSize:(aSetup.fontSize)];
    windowRect.origin.x = aSetup.mainWindowX;
    windowRect.origin.y = aSetup.mainWindowY;
    windowRect.size.width = aSetup.mainWindowWidth;
    windowRect.size.height = aSetup.mainWindowHeight;

    [mainView setString:[NSString stringWithCString:theWorldHandler->getTheString() encoding:NSASCIIStringEncoding]];
    theWorldHandler->freeTheString();

    scrollerOrigin.x = aSetup.scrollPointX;
    scrollerOrigin.y = aSetup.scrollPointY;
    [[mainScrollView documentView] scrollPoint:scrollerOrigin];  // For some reason I cannot get this to work.  (2010.07.06)

    selectedRange.location = aSetup.selectionLocation;
    selectedRange.length   = aSetup.selectionLength;

    [mainView setSelectedRange:selectedRange];
    [myParentWindow makeFirstResponder:mainView];

    for( counter = 0; counter < N_SENSE_LIGHTS; ++counter ) {
        senseLightState[ counter ] = aSetup.state[ counter ];
        senseLightVisible[ counter ] = aSetup.visible[ counter ];
        senseLightIllumination[ counter ] = aSetup.illumination[ counter ];
        [sLAController setLightNFromRecordedState:counter];
        }

    [NSAnimationContext beginGrouping];
    [[NSAnimationContext currentContext] setDuration:0.25]; 
    [[myParentWindow animator] setFrame:windowRect display:YES];
    [NSAnimationContext endGrouping];

    [mainView scrollRangeToVisible:selectedRange];
    if( [mainView hasSelectedText] ) {
        [self dealWithSelection:YES];
        [mainView setNeedsNewTypingRange:YES];
        }
    }


//########  The problem here is potential deadlock if we do two "tellEvaluators" in the same timer-firing.  This is the
//########  thread that does GUI I/O, and the evaluator must call the thread that does GUI I/O in order to respond to
//########  some of the "tellEvaluator" commands.  But if there are two "tellEvaluators" in a row, the GUI I/O thread
//########  will be waiting for the second "tellEvaluator" to respond while the evaluator thread is waiting for the
//########  GUI I/O thread to perform some I/O that was consequent to the first "tellEvaluator".  The fix is to
//########  (1) *DO* *AT* *MOST* *ONE* "tellEvaluator" per timer firing -- the code following is so written -- and
//########  (2) Make sure not even to call "controllerInitializationTask" unless it is the controller's turn to use
//########  the controller/evaluator interface (guaranteed by test in "doControllerInitializationTask").

- (bool)controllerInitializationTask {

    // Rule here -- at most one "tellEvaluator" per task.
    // This code allows for as many as nine tasks, and it is easy to add more.
    // Not all task opportunities are presently used for anything.


    // Synchronize threads  --  getting past the first "if" means that the evaluator is set up for
    // kitten-to-kitten communication (the Clowder is running), and also means that the
    // GUI thread has advised the evaluator thread that the GUI is ready for messages from the 
    // evaluator.

    switch( nextTask ) {
        case 1:
            reportGUIInitialized();
            nextTask = 2;
            break;
        case 2:
            if( evaluatorHasInitialized )
                nextTask = 3;
            break;
        case 3:
            [self tellEvaluator:C_PLEASE_USE_HIGH_PRECISION withLong:0 withString:""];
            nextTask = 4;
            break;
        case 4:
            [self tellEvaluator:C_PLEASE_COMPILE_DEFINES withLong:0 withString:""];
            nextTask = 5;
            break;
        case 5:
            if( allKittensAreActive() )
                nextTask = 6;
            break;
        case 6:
            if( ! isOriginalWorld )
                [self setControllerSetup];
            nextTask = 7;
            break;
        case 7:
            readyToRead = 1;
            nextTask = 8;
            break;
        default:
            return YES;       // Done with initialization tasks.
        }
    return NO;                // Not done with initialization tasks.
    }    

- (void) setNormalLights {

    [eVBController setOnImage:[NSImage imageNamed:@"MagentaLight.png"]];
    [eVBController setAlternateOnImage:[NSImage imageNamed:@"MagentaLightAlternate.png"]];
    [eVBController setOffImage:[NSImage imageNamed:@"OffLight.png"]];
    [gCBController setOnImage:[NSImage imageNamed:@"RedLight.png"]];
    [gCBController setAlternateOnImage:[NSImage imageNamed:@"RedLightAlternate.png"]];
    [gCBController setOffImage:[NSImage imageNamed:@"OffLight.png"]];
    }

//########  Run the evaluator, in a thread dispatched for that purpose:

extern char *globalArgvZero;

- (void) runEvaluator:(id)theObject {

    NSAutoreleasePool *thePool = [[NSAutoreleasePool alloc] init];

    runSchemeWithPathAndMemory( globalArgvZero, mainMemorySizeInUse, generationalMemorySizeInUse );  // Never returns ...

    [thePool release];  // It never returns, but just in case ...

    }
    

- (void) setForForgettingTyping {

    [resetSchemeButton  setEnabled:YES];
    [resetSchemeButton  setTitle:@"Forget Typing"];
    }

- (void) turnExpectStuffOff {

    [NSAnimationContext beginGrouping];
    [[NSAnimationContext currentContext] setDuration:0.7]; 
    [[expectStuff animator] setAlphaValue:0.0];
    [NSAnimationContext endGrouping];
    }

- (void) turnExpectStuffOn {

    [NSAnimationContext beginGrouping];
    [[NSAnimationContext currentContext] setDuration:0.7]; 
    [[expectStuff animator] setAlphaValue:1.0];
    [NSAnimationContext endGrouping];
    }

- (void) setExpectStuffVisible  {

    [expectStuff setHidden:NO];
	showingExpectView = false;
    cyclingExpectView = true;
    [self turnExpectStuffOn];
    }

- (void) setExpectStuffHidden {

    [expectStuff setHidden:YES];
    cyclingExpectView = false;
    [self turnExpectStuffOff];
    }


- (void) setForEvaluating {

    evaluating = TRUE;

    [self setExpectStuffHidden];
    [resetSchemeButton  setEnabled:YES];
    [resetSchemeButton  setTitle:@"Reset Scheme"];
    [cDMenuItem         setEnabled:NO];
    [sFPMenuItem        setEnabled:NO];
    [cRMenuItem         setEnabled:NO];
    [eVBController      turnOn];     
    [mainView           setEditable:NO];
    [mainView           disableShortcuts];
    }

- (void) setForNotEvaluating {

    evaluating = FALSE;

    stopEvaluatingPending = false;

    [self setExpectStuffHidden];
    [resetSchemeButton  setEnabled:NO];
    [resetSchemeButton  setTitle:@"Reset Scheme"];
    [cDMenuItem         setEnabled:YES];
    [sFPMenuItem        setEnabled:YES];
    [cRMenuItem         setEnabled:YES];
    [eVBController      turnOff];     
    [mainView           setEditable:YES];
    [mainView           enableShortcuts];
    }

- (void) setForGCing {

    gcing = TRUE;

    [resetSchemeButton  setEnabled:YES];
    [cDMenuItem         setEnabled:NO];
    [sFPMenuItem        setEnabled:NO];
    [gCBController      turnOn];     
    }

- (void) setForNotGCing {

    gcing = FALSE;

    [gCBController       turnOff];     

    if( evaluating )
        [self setForEvaluating];
    else
        [self setForNotEvaluating];
    }


//################################################################################
//
//########  Stuff used during quitting and shutdown.  Includes fatal error-message handling.


- (IBAction) doQuit:(id)sender {

    waitingToExit = 1;    // Tested when evaluator reports "E_WORLD_SAVE_SUCCEEDED" and "REINITIALIZATION_COMPLETE".
    [self requestReset:nil];
    }

//######## Sheet display for reporting fatal error.

- (void) reportDisasterAndQuit:(const char *)errorMessage {

    wantsToQuit = true;
    NSBeginAlertSheet(
        @"Pixie Scheme II must quit because:",
        @"Sorry!", NULL, NULL,
        myParentWindow,
        self,
        @selector( reportDisasterAndQuitDidEnd:returnCode:contextInfo: ),
        NULL,
        NULL,
        [NSString stringWithCString:errorMessage encoding:NSASCIIStringEncoding],
        NULL
        );
    }


//######## Sheet did end handler for reporting fatal error.

- (void) reportDisasterAndQuitDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo {

    [sheet orderOut:nil];
    cleanUpForExit();
    [NSApp terminate:self];
    }


//######## Sheet display for reporting fatal error in early initialization.

- (void) reportEarlyDisasterAndQuit {

    if( wantsToQuit ) {
        usleep( 15000000 );
        exit( 1 );
        }

    wantsToQuit = true;
    NSBeginAlertSheet(
        @"Pixie Scheme II must quit because of an implementation error while starting up:",
        @"Sorry!", NULL, NULL,
        myParentWindow,
        self,
        @selector( reportEarlyDisasterAndQuitDidEnd:returnCode:contextInfo: ),
        NULL,
        NULL,
        @"This error is not your fault.  Please report it to Jay_Reynolds_Freeman@mac.com.  For details, see the end of the console log file, somewhere in the \"Console\" folder of the \"Logs\" folder of the \"Library\" folder of your main disk drive.  Thank you.\n\nPixie Scheme II's normal error-handling mechanism isn't running yet.",
        NULL
        );
    }

//######## Sheet did end handler for reporting fatal error.

- (void) reportEarlyDisasterAndQuitDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo {
    [sheet orderOut:nil];
    exit( 1 );
    }



//################################################################################
//
//########  Stuff for handling file I/O.  Includes world loads and saves.


//######## Reloading the default world:

- (IBAction) runReloadWorldSheet:(id)sender {

    NSBeginAlertSheet(
        @"Really FORGET ALL YOUR WORK and restart Pixie Scheme II?\n\nYou cannot undo this action.",
        @"Cancel", 
        NULL,
        @"Restart",
        myParentWindow,
        self,
        @selector( reloadWorldSheetDidEnd:returnCode:contextInfo: ),
        NULL,
        NULL,
        @"",
//        @"Pixie Scheme II is about to forget everything you have done.  You cannot undo this action.",
        NULL
        );
    }

- (void) reloadWorldSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo {

    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "%sPixie Scheme II.app/Contents/Resources/PixieSchemeII.world", InitFilePath );

    if( returnCode == NSAlertDefaultReturn ) {    // Cancel.
        [sheet orderOut:nil];
        }
    else {                                        // Reload the world.
        [self setForEvaluating];
        [sLAController clearSenseLights];
        [self tellEvaluator:C_INPUT_READY
                   withLong:0
                 withString:[[NSString stringWithFormat: @"(c::load-world! \"%s\")\n", buffer]
                             cStringUsingEncoding:NSASCIIStringEncoding]];
        }
    }
    

//######## Loading a world:

- (IBAction) openForWorldLoad:(id)sender {
    static NSString *worldToLoad = nil;
    NSInteger result;
    const char *theString = 0;
    NSOpenPanel *op = [NSOpenPanel openPanel];

    [op setTitle:@"Load What World?"];
    [op setMessage:@"Choose a saved world for Pixie Scheme II to use.\n\nCAUTION: World loads overwrite all of Pixie Scheme II's memory:\nThat is, you are about to lose any work that you have not saved."];
    result = [op runModalForDirectory:nil file:nil types:nil];
    if (result == NSOKButton) {
        worldToLoad = [[op filenames] objectAtIndex:0];
        theString = [worldToLoad cStringUsingEncoding:NSASCIIStringEncoding];
        if( theString && (strlen( theString ) > 0 ) ) {
            [self setForEvaluating];
            [self tellEvaluator:C_INPUT_READY
                       withLong:0
                     withString:[[NSString stringWithFormat: @"(e::load-world! \"%s\")\n", theString]
                                 cStringUsingEncoding:NSASCIIStringEncoding]];
            }
        else
            [self localError:@"\nThe path (including the file name) to the world file to load includes some non-ASCII characters.\n\nScheme is supposed to use only ASCII characters.  They are pretty much the \"regular\" keyboard characters, entered without the \"command-\" and \"option-\" modifier keys.\n\nTo load the world file you selected, you must change its name, or the names of the directories in the path to the file, or both, to contain only ASCII characters."];
        }
    }
    

//######## Loading a world: Succeeded

- (void) worldLoadSucceeded:(const char *)path {
    }


//######## Saving a world:

- (IBAction) openForWorldSave:(id)sender {
    static NSString *worldToSave = nil;
    NSInteger result;
    const char *theString = 0;
    NSSavePanel *sp = [NSSavePanel savePanel];

    [sp setTitle:@"Save as What World File?"];
    [sp setMessage:@"Provide the name of a world file to contain all of Pixie Scheme II's memory."];
    result = [sp runModalForDirectory:nil file:nil];
    if (result == NSOKButton) {
        worldToSave = [sp filename];
        theString = [worldToSave cStringUsingEncoding:NSASCIIStringEncoding];
        if( theString && (strlen( theString ) > 0 ) ) {
            [self setForEvaluating];
            [self tellEvaluator:C_INPUT_READY
                       withLong:0
                     withString:[[NSString stringWithFormat: @"(e::save-world \"%s\")\n", theString]
                                 cStringUsingEncoding:NSASCIIStringEncoding]];
            }
        else
            [self localError:@"\nThe path (including the file name) to the file in which to save the world includes some non-ASCII characters.\n\nScheme is supposed to use only ASCII characters.  They are pretty much the \"regular\" keyboard characters, entered without the \"command-\" and \"option-\" modifier keys.\n\nPlease use the \"Save World...\" menu item again, and choose a path and enter a file name that contain only ASCII characters."];
        }
    }


//######## Opening the Pixie Scheme II dictionary.

- (IBAction) openDictionaryFile:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme II.app/Contents/Resources/English.lproj/Pixie Scheme II Help/Pixie Scheme II Dictionary.html\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme II could not open the Pixie Scheme II Dictionary with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }


//######## Opening the Pixie Scheme II help file in a browser.

- (IBAction) openHelpFile:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme II.app/Contents/Resources/English.lproj/Pixie Scheme II Help/Pixie Scheme II Help.html\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme II could not open the Pixie Scheme II Help File with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }


//######## Opening the Pixie Scheme II README file.

- (IBAction) openReadMe:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme II.app/Contents/Resources/English.lproj/README.txt\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme II could not open the Pixie Scheme II \"README.txt\" file with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

//######## Opening GNU General Public License information.

- (IBAction) openGNU:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme II.app/Contents/Resources/English.lproj/GNUEtcetera.txt\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme II could not open the GNU warranty and license file with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }


//########  Opening various files of source code that are built-in to Pixie Scheme II.

//########  Easy to add more.  I could probably store the file name to be loaded in connection
//########  with the menu item that accesses code, and write one procedure to do it all.

- (IBAction) openSourceCode1:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme II.app/Contents/Resources/English.lproj/SampleSourceCode/Checkbook.txt\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme II could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

- (IBAction) openSourceCode2:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme II.app/Contents/Resources/English.lproj/SampleSourceCode/FunctionalProgramming.txt\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme II could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

- (IBAction) openSourceCode3:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme II.app/Contents/Resources/English.lproj/SampleSourceCode/ByAnyOtherName.txt\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme II could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

- (IBAction) openSourceCode4:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme II.app/Contents/Resources/English.lproj/SampleSourceCode/ShellEmulator.txt\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme II could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

- (IBAction) openSourceCode5:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme II.app/Contents/Resources/Demo.txt\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme II could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

- (IBAction) openSourceCode6:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme II.app/Contents/Resources/English.lproj/SampleSourceCode/MMapDemo.c++\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme II could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

- (IBAction) openSourceCode7:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme II.app/Contents/Resources/English.lproj/SampleSourceCode/MMapInterruptDemo.c++\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme II could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }


- (IBAction) openSourceCode8:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme II.app/Contents/Resources/English.lproj/SampleSourceCode/SocketClient.c++\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme II could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }


- (IBAction) openSourceCode9:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme II.app/Contents/Resources/English.lproj/SampleSourceCode/SocketServer.c++\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme II could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }


- (IBAction) openSourceCode10:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme II.app/Contents/Resources/English.lproj/SampleSourceCode/SenseLightDemonstrations.txt\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme II could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }


//################################################################################
//
//########  Miscellaneous sheets.


//########  Sheet for verifying the user wants to disable console output:

- (void) runDisableOutputSheet:(id)sender {
    outputEnabled = FALSE;
    NSBeginAlertSheet(
        @"Text output to the Pixie Scheme II main window is disabled.",
        @"Provide Normal Output", NULL, NULL,
        myParentWindow,
        self,
        @selector( disableOutputSheetDidEnd:returnCode:contextInfo: ),
        NULL,
        NULL,
        @"Text that would have been printed is lost beyond recovery.  Output to files, including transcript files, is not affected.",
        NULL
        );
    }


//########  Sheet for verifying the user wants to disable console output: Did-end handler.

- (void) disableOutputSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo {
    outputEnabled = TRUE;
    [sheet orderOut:nil];
    }


//########  Sheet for reminding the user how to make a shareware donation.

- (IBAction) runDonationSheet:(id)sender {

    [boldField setHidden:FALSE];
    [normalField setHidden:FALSE];
    [closeButton setHidden:FALSE];
    [donationSheetImageView setImage:pixieFace];
    [donationSheetImageView setAlphaValue:1.0];        

    [NSApp beginSheet: myDonationSheet
       modalForWindow: myParentWindow
        modalDelegate: self
       didEndSelector: @selector(donationSheetDidEnd:returnCode:contextInfo:)
          contextInfo: nil];
    }


//########  Sheet for reminding the user how to make a shareware donation: Did-end handler.

- (void) donationSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo {

    [donationSheetImageView setImage:sadKitty];
    [donationSheetImageView display];
    [boldField setHidden:TRUE];
    [myDonationSheet display];
    usleep( 400000  );
    [normalField setHidden:TRUE];
    [myDonationSheet display];
    usleep( 600000 );
    [closeButton setHidden:TRUE];
    [myDonationSheet display];

    [NSAnimationContext beginGrouping];

    [[NSAnimationContext currentContext] setDuration:0.60]; 
    [[donationSheetImageView animator] setAlphaValue:0.0];        

    [NSAnimationContext endGrouping];

    usleep( 600000 );

    [sheet orderOut:self];
    }

- (IBAction) closeDonationSheet:(id)sender {

    [NSApp endSheet:myDonationSheet];
    }


//######## Sheet for handling non-local errors:

- (void) localError:(NSString *)message {
    NSBeginAlertSheet(
        message,
        @"Okay", NULL, NULL,
        myParentWindow,
        self,
        NULL,
        NULL,
        NULL,
        @"",
        NULL
        );
    }

- (void) localError:(NSString *)message onWindow:(NSWindow *)window {
    NSBeginAlertSheet(
        message,
        @"Okay", NULL, NULL,
        window,
        self,
        NULL,
        NULL,
        NULL,
        @"",
        NULL
        );
    }


//################################################################################
//
//########  Mode changes:  Toggling this and that.


//########  Toggle compile-defines flag.

- (IBAction) toggleCompileDefines:(id)sender {
    [self tellEvaluator:([sender state] ? C_PLEASE_DO_NOT_COMPILE_DEFINES : C_PLEASE_COMPILE_DEFINES ) withLong:0 withString:""];
    }
    

//########  Toggle use-full-numeric-precision flag.

- (IBAction) toggleFullNumericPrecision:(id)sender {
    [self tellEvaluator:([sender state] ? C_PLEASE_USE_LOW_PRECISION : C_PLEASE_USE_HIGH_PRECISION ) withLong:0 withString:""];
    }


//################################################################################
//
//########  Resetting:

- (IBAction) requestReset:(id)sender {

    if( evaluatorHasInitialized && (! stopEvaluatingPending ) ) {
        [resetSchemeButton     setEnabled:NO];
        stopEvaluatingPending = true;
        [self tellEvaluator:C_PLEASE_STOP_EVALUATING withLong:0 withString:""];
//        [mWController        setPrompt:    @"Pixie Scheme II may ask questions here ..."];
//        [mWController      setResponse:    @"... and will expect you to enter your answers here."];
        [self startNormalInput];
        }
    }


//################################################################################
//
//########  Use of the dialog panel:


- (void)showDialogSheet:(NSWindow *)window {

    [NSApp      beginSheet: userDialog
            modalForWindow: window
             modalDelegate: self
            didEndSelector: @selector(didEndUserDialogSheet:returnCode:contextInfo:)
               contextInfo: nil];
    }

- (IBAction)closeDialogSheet:(id)sender {

    bool responseWasOkay = [self dealWithResponseText:[dialogResponse stringValue]];
        
    if( responseWasOkay ) {
        [mainView setEditable:YES];
        [NSApp endSheet:userDialog];
        }
    }

- (void)didEndUserDialogSheet:(NSWindow *)sheet returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo {

    [sheet orderOut:self];

    }

//########  Get a dialog going:

- (void) startDialog:(NSString *)thePrompt {

    [dialogPrompt   setStringValue:thePrompt];
    [dialogResponse setStringValue:@""];
    [mainView setEditable:NO];    
    [self showDialogSheet:myParentWindow];

    }


//########  Deal with the response to a dialog:

- (bool) dealWithResponseText:(NSString *)theText {

    const char *theString = 0;
    const char *theCString = "";
    NSInteger theLength = 0;

    if( theText == nil )
        [self tellEvaluator:C_DIALOG_RESPONSE_READY withLong:0 withString:""];
    else if( [theText canBeConvertedToEncoding:NSASCIIStringEncoding] ) {
        theString = [theText cStringUsingEncoding:NSASCIIStringEncoding];
        if( theString && (strlen( theString ) > 0 ) ) {
            [self startNormalInput];
            if( (theLength = [theText length]) > 0 ) {
                theCString = [theText cStringUsingEncoding:NSASCIIStringEncoding];
                if( theCString[ theLength - 1 ] == NEWLINE )
                    theCString = [[theText substringToIndex:(theLength - 1)] cStringUsingEncoding:NSASCIIStringEncoding];
                }
            [self tellEvaluator:C_DIALOG_RESPONSE_READY withLong:0 withString:theCString];
            }
        else
            [self tellEvaluator:C_DIALOG_RESPONSE_READY withLong:0 withString:""];
        }
    else {
        [self localError:@"\nYour response includes some non-ASCII characters.\n\nScheme is supposed to use only ASCII characters.  They are pretty much the \"regular\" keyboard characters, entered without the \"command-\" and \"option-\" modifier keys.\n\nPlease retype your response to the dialog."
                onWindow:userDialog];
        return false;
        }

    return true;
    }


//########  Return to getting input after a dialog.  (Also used in resetting.)

- (void) startNormalInput {
    }


//################################################################################
//
//########  Stuff for dealing with controller/evaluator interactions after initialization:


void doPollEvaluatorOnce()
{
    [thePixieSchemeController performSelectorOnMainThread:@selector(pollEvaluatorOnce) withObject:nil waitUntilDone:FALSE];
    }


- (void) postMakeFinalWorld {
    [self tellEvaluator:C_INPUT_READY
               withLong:0
             withString:[[NSString stringWithFormat: @"(c::save-world \"%s\")\n", appSupportWorldFileName]
                               cStringUsingEncoding:NSASCIIStringEncoding]];
    }


extern volatile int evaluatorOutboundMailboxFlag;

//########  Get just one request from the evaluator, and act on it:  Critical-section-access and lock slime, and a big switch.

- (bool) pollEvaluatorOnce {

    SInt32 newMessage;
    long newLong;
    NSString *newNSString;
    bool returnValue;

    OSMemoryBarrier();     // Defensive programming

    if( evaluatorOutboundMailboxFlag == MAILBOX_FULL ) {

        newMessage  = toController.messageID;
        newLong     = toController.theLong;
        newNSString = [[NSString stringWithCString:toController.theString encoding:NSASCIIStringEncoding] copy];
     
        switch( newMessage ) {

            // System administration.

            case E_NO_MESSAGE_YET: break;                   // Initialization state; evaluator has not sent any messages.
            case E_COMPLETED_INITIALIZATION:
                [myParentWindow display];
                break;
            case E_HELLO_THERE: break;                      // No action required.
            case E_FATAL_PROBLEM:
                [self reportDisasterAndQuit:toController.theString];
                break;
            case E_PLEASE_TIDY_UP_FOR_EXIT: break;
            case E_TIDIED_UP: break;
            case E_COGNITO_ERGO_SUM: break;
            case E_MAKE_FRONT:
                if( [myParentWindow isMiniaturized] )
                    [myParentWindow deminiaturize:self];
                [NSApp activateIgnoringOtherApps:YES];
                break;
            case E_NEW_OBLIST_SYMBOL:
//                [symbolsInObList addObject:newNSString];
                break;
            case E_CLEAR_OBLIST_SYMBOLS:
//                [symbolsInObList removeAllObjects];
                break;

            // Simple state transitions.

            case E_EVALUATING:                 
                [self setForEvaluating];
                break;
            case E_NOT_EVALUATING:
                [self setForNotEvaluating];
                break;
            case E_ACKNOWLEDGE_RESET:
                [self setForNotEvaluating];
                [self setForNotGCing];
                if( waitingToExit ) {
                    [self setForEvaluating];
                    [self provideUserSetup];
                    [self performSelectorOnMainThread:@selector(postMakeFinalWorld) withObject:nil waitUntilDone:FALSE];
                    }
                break;
            case E_GARBAGE_COLLECTING:         [self setForGCing];               break;
            case E_NOT_GARBAGE_COLLECTING:     [self setForNotGCing];            break;

            case E_SLEEPING:                   /* [sBController turnOn];  */           break;
            case E_WAKED_UP:                   /* [sBController turnOff]; */           break;
            case E_COMPILING_DEFINES:
                [cDMenuItem setState:NSOnState];
                break;
            case E_NOT_COMPILING_DEFINES:
                [cDMenuItem setState:NSOffState];
                break;
            case E_USING_LOW_PRECISION:
                [sFPMenuItem setState:NSOffState];
                break;
            case E_USING_HIGH_PRECISION:
                [sFPMenuItem setState:NSOnState];
                break;

            // Routine I/O.

            case E_OUTPUT_READY: 
                if( outputEnabled )
                    [mWController displayStringInMainView:newNSString];
                break;
            case E_MESSAGE_TO_DISPLAY:
                messageDisplayCounter = 0;
                displayingMessage = true;
//                [mWController setMessageViewText:newNSString];
                break;
            case E_DIALOG_TO_START:
                [NSApp requestUserAttention:NSInformationalRequest];
                if( [newNSString length] > 0 )
                    [self startDialog:newNSString];
                break;
            case E_FILE_LOAD_BEGINNING:
                break;
            case E_FILE_LOAD_SUCCEEDED:
                break;
            case E_FILE_LOAD_FAILED:
                break;
            case E_WORLD_LOAD_SUCCEEDED:
                [self worldLoadSucceeded:[newNSString cStringUsingEncoding:NSASCIIStringEncoding]];
                break;
            case E_WORLD_LOAD_FAILED:    break;    // Nothing to do.
            case E_WORLD_SAVE_SUCCEEDED:
                if( waitingToExit ) {
                    cleanUpForExit();
                    [NSApp terminate:self];
                    }
                break;
            case E_WORLD_SAVE_FAILED:
                waitingToExit = 0;
                break;

            // Input parsing.

            case E_EXPECT_MORE_INPUT:
                expectMoreInput = true;
                [self setExpectStuffVisible];
                [self newViewString:"Expect ?" forView:expectStuff];
                [self setForForgettingTyping];
                break;

            case E_DO_NOT_EXPECT_MORE_INPUT:
                expectMoreInput = false;
                [self newViewString:"" forView:expectStuff];
//                [self setExpectStuffHidden];
                break;

            case E_EXPECT_PARENS:
                [self setForForgettingTyping];
                switch( newLong ) {
                    case 0:  [self newViewString:""           forView:expectStuff]; break;
                    case 1:  [self newViewString:"Expect )"   forView:expectStuff]; break;
                    case 2:  [self newViewString:"Expect ))"  forView:expectStuff]; break;
                    case 3:  [self newViewString:"Expect )))" forView:expectStuff]; break;
                    default: [self newViewString:"Expect ))s" forView:expectStuff]; break;
                    }
                [self setExpectStuffVisible];
                break;

            case E_EXPECT_DOUBLE_QUOTE:
                [self setForForgettingTyping];
                [self newViewString:"Expect \""   forView:expectStuff];
                [self setExpectStuffVisible];
                break;

            case E_DISCARDED_INPUT: break;

            // Status information, possibly for display.

            case E_NEW_GC_COUNT:               /* [self newViewNumber:newLong forView:    numberOfGCsView             ] */;   break;
            case E_NEW_GENERATIONAL_GC_COUNT:  /* [self newViewNumber:newLong forView:    numberOfGenerationalGCsView ] */;   break;
            case E_MEMORY_SIZE:                /* [self newViewNumber:newLong forView:    memorySizeView              ] */;   break;
            case E_MEMORY_IN_USE:              /* [self newViewNumber:newLong forView:    memoryInUseView             ] */;   break;
            case E_GENERATIONAL_MEMORY_SIZE:
                                  /* [self newViewParenthesizedNumber:newLong forView:    generationalMemorySizeView  ] */;   break;
            case E_GENERATIONAL_MEMORY_IN_USE:
                                  /* [self newViewParenthesizedNumber:newLong forView:    generationalMemoryInUseView ] */;   break;
            case E_LIST_PRINT_LENGTH:          /* [self newViewNumber:newLong forView:    listPrintLengthView         ] */;   break;
            case E_LIST_PRINT_DEPTH:           /* [self newViewNumber:newLong forView:    listPrintDepthView          ] */;   break;
            case E_VECTOR_PRINT_LENGTH:        /* [self newViewNumber:newLong forView:    vectorPrintLengthView       ] */;   break;
            case E_VECTOR_PRINT_DEPTH:         /* [self newViewNumber:newLong forView:    vectorPrintDepthView        ] */;   break;

            case E_SENSE_LIGHT_COMMAND:        [sLAController interpretLongCommand:    newLong                     ];   break;
            case E_SENSE_LIGHT_ROTATE:         [sLAController rotateLights:            newLong                     ];   break;
            case E_TOTAL_MEMORY_IN_USE:
              [self updateLevel:( (((double)newLong) * 1000.0)/ ((double)( 40 * MEGABYTE)) )];
              break;

            // Level indicator manipulation.

            case E_INIT_PROGRESS:        /* [self beginInitLevel] */;                      break;
            case E_INCREMENT_PROGRESS:   /* [self updateLevel:( newLong / 1000)] */;       break;
            case E_WRAPUP_PROGRESS:      /* [self endInitLevel] */;                        break;

            default:
                [self reportDisasterAndQuit:
                          [[NSString stringWithFormat:@"Controller got unknown message, number %d.", newMessage]
                              cStringUsingEncoding:NSASCIIStringEncoding]];
                break;
            }
        returnValue = TRUE;

        [newNSString release];     // TEST

        OSMemoryBarrier();     // Defensive programming
        evaluatorOutboundMailboxFlag = MAILBOX_EMPTY;
        }
    else
        returnValue = FALSE;

    return returnValue;
                
    }


//########  Send a request to the evaluator.

- (void) tellEvaluator:(SInt32)mID withLong:(long)mLong withString:(char *)mString; {

    static char *stringToPass = 0;
    static SInt32 currentLength = 0;
    SInt32 newBufferSize = 0;

    OSMemoryBarrier();     // Defensive programming

    LongWord lengthNeeded = strlen( mString ) + 1;

    if( lengthNeeded > (currentLength - 1) ) {  //  Need a bigger hammer ... er, buffer.
        if( currentLength != 0 )
            free( stringToPass );
        for( newBufferSize = START_STRING_SIZE; newBufferSize < lengthNeeded; newBufferSize = newBufferSize * 2 )
             ;
        stringToPass = (Text *)calloc( newBufferSize, sizeof( char ) );
        if( stringToPass == 0 )
            [self reportDisasterAndQuit:"Cannot allocate memory for a communication buffer."];
        currentLength = newBufferSize;
        }

#if SHOW_THREAD_INTERACTIONS
    printf("Kitten %d: C: tellEvaluator called with message \"%s\".\n",      // DEBUG
           theKittenNumber, messagesFromControllerNames[ mID ] );            // DEBUG
#endif

    if( ! evaluatorHasInitialized )
        return;

#if SHOW_THREAD_INTERACTIONS
    printf("Kitten %d: C: tellEvaluator attempting critical section with message \"%s\".\n",      // DEBUG
           theKittenNumber, messagesFromControllerNames[ mID ] );                                 // DEBUG
#endif

    while( controllerOutboundMailboxFlag == MAILBOX_FULL )
        ;

#if SHOW_THREAD_INTERACTIONS
    printf("Kitten %d: C: tellEvaluator has the evaluator's mailbox with message \"%s\".\n",      // DEBUG
           theKittenNumber, messagesFromControllerNames[ mID ] );                                 // DEBUG
#endif

    if( mString != 0 )
        strncpy( stringToPass, mString, currentLength - 1 );
    else
        stringToPass[0] = 0;
    toEvaluator.messageID = mID;
    toEvaluator.theLong   = mLong;
    toEvaluator.theString = stringToPass;
 
    controllerOutboundMailboxFlag = MAILBOX_FULL;

    OSMemoryBarrier();     // Defensive programming

#if SHOW_THREAD_INTERACTIONS
        printf("Kitten %d: C: tellEvaluator left critical section with message  \"%s\".\n",      // DEBUG
               theKittenNumber, messagesFromControllerNames[ mID ] );                            // DEBUG
#endif
    }
    

- (void) provideUserSetup {

    int counter = 0;

    bool *senseLightState = [sLAController getState];
    bool *senseLightVisible = [sLAController getVisible];
    int *senseLightIllumination = [sLAController getIllumination];

    NSRange selectedRange = [mainView getSelectedRange];
    NSPoint scrollerOrigin = [[mainScrollView contentView] bounds].origin;

    userSetup aSetup;
    NSRect windowRect = [myParentWindow frame];

    aSetup.compilerOn          = [cDMenuItem state];
    aSetup.showsFullPrecision  = [sFPMenuItem state];
    aSetup.fontSize            = [mWController currentFontSize];

    aSetup.mainWindowX         = windowRect.origin.x;
    aSetup.mainWindowY         = windowRect.origin.y;
    aSetup.mainWindowWidth     = windowRect.size.width;
    aSetup.mainWindowHeight    = windowRect.size.height;

    aSetup.scrollPointX        = scrollerOrigin.x;
    aSetup.scrollPointY        = scrollerOrigin.y;

    for( counter = 0; counter < N_SENSE_LIGHTS; ++counter ) {
        aSetup.state[ counter ] = senseLightState[ counter ];
        aSetup.visible[ counter ] = senseLightVisible[ counter ];
        aSetup.illumination[ counter ] = senseLightIllumination[ counter ];
        }

    aSetup.selectionLocation = selectedRange.location;    
    aSetup.selectionLength   = selectedRange.length;    

    theWorldHandler->setUserSetup( &aSetup );

    theWorldHandler->setTheString( (Text *)[[mainView string] UTF8String] );
    }


//################################################################################
//
//########  Process text from the Input Panel:

- (bool) dealWithInputText2:(NSString *)theText {

    const char *theString = 0;

    if( [theText canBeConvertedToEncoding:NSASCIIStringEncoding] ) {
        theString = [theText cStringUsingEncoding:NSASCIIStringEncoding];
        if( theString && (strlen( theString ) > 0) ) {
            [self tellEvaluator:C_INPUT_READY withLong:0 withString:theString];
            }
        // else nothing to do
        }
    else {
        [self localError:@"\nYou have entered some non-ASCII characters.\n\nScheme is supposed to use only ASCII characters.  They are pretty much the \"regular\" keyboard characters, entered without the \"command-\" and \"option-\" modifier keys.\n\nPlease correct the input."];
        }
    return true;
    }


// Called by the main view whenever there might be a selection; the intent is that the
// controller will offer the user the opportunity to evaluate what is selected.

- (void) dealWithSelection:(bool)value {

    if( value ) {
        [markOrEvalButton setTitle:@"Evaluate"];
        [markOrEvalButton setEnabled:YES];
        }
    else {
        [markOrEvalButton setTitle:@"Select/Eval"];
        [markOrEvalButton setEnabled:NO];
        }
    }

// Called by the main view whenever there might be a text range; the intent is that the
// controller will offer the user the opportunity to mark any text range.

- (void) dealWithTextRange:(bool)value {

    if( value ) {
        [markOrEvalButton setTitle:@"Select That?"];
        [markOrEvalButton setEnabled:YES];
        }
    else {
        [markOrEvalButton setTitle:@"Select/Eval"];
        [markOrEvalButton setEnabled:NO];
        }
    }


//########  Wrapper to set a new string value in the given view.

- (void) newViewString:(const char *)newString forView:(NSTextField *)theView {

    [theView setStringValue:[NSString stringWithCString:newString encoding:NSASCIIStringEncoding]];
    [theView setNeedsDisplay:YES];
    [theView display];
    }



//  Code to look up a term in the dictionary.

- (IBAction)findInPixieSchemeDictionary:(id)sender {

//  Format string for the lookup.  This *is* part of the method, just formatted in a compromising manner.

const char *scriptTemplate =
"echo $'\
tell application \"Safari\"\n\
     make new document with properties {URL:\"file://%sPixie%%20Scheme%%20II.app/Contents/Resources/English.lproj/Pixie%%20Scheme%%20II%%20Help/Pixie%%20Scheme%%20II%%20Dictionary.html#%s\"}\n\
     get source of front document\n\
     if source of front document contains \"<a name=\\\\\"%s\" then\n\
        activate\n\
    else\n\
        close front window\n\
        beep\n\
    end if\n\
end tell\
' | osascript";

    NSResponder *currentFocusedView = [myParentWindow firstResponder];

    if( [currentFocusedView isEqual:mainView] ) {

        NSArray *selRanges = [((NSTextView *)currentFocusedView) selectedRanges];
        NSRange selectionRange = [[selRanges objectAtIndex:0] rangeValue];

        if( selectionRange.length > 0 ) {

            NSString *theString = [[((NSTextView *)currentFocusedView) string] substringWithRange:selectionRange];
            char buffer[ MAXPATHLEN + 3000 ];

            snprintf( buffer,
                      MAXPATHLEN + 3000,
                      scriptTemplate,
                      InitFilePath,
                      [theString UTF8String],
                      [theString UTF8String]
                      );

            if( system( buffer ) != 0 ) {
                [self localError:@"Pixie Scheme II could not find such an entry in the dictionary."];
                return;
                }

            }
        else
            NSBeep();
        }
    else
        NSBeep();
    }


- (NSArray *)createCompletionArray {

    return [NSArray arrayWithObjects:
        @"*",
        @"+",
        @"-",
        @"/",
        @"<",
        @"<=",
        @"=",
        @">",
        @">***<",
        @">**<",
        @">*<",
        @">+++<",
        @">++<",
        @">+<",
        @">-<",
        @">=",
        @"abs",
        @"acos",
        @"and",
        @"append",
        @"apply",
        @"asin",
        @"assoc",
        @"assq",
        @"assv",
        @"atan",
        @"begin",
        @"boolean?",
        @"call-with-current-continuation",
        @"call-with-values",
        @"car",
        @"cdr",
        @"caar",
        @"cadr",
        @"cdar",
        @"cddr",
        @"caaar",
        @"caadr",
        @"cadar",
        @"caddr",
        @"cdaar",
        @"cdadr",
        @"cddar",
        @"cdddr",
        @"caaaar",
        @"caaadr",
        @"caadar",
        @"caaddr",
        @"cadaar",
        @"cadadr",
        @"caddar",
        @"cadddr",
        @"cdaaar",
        @"cdaadr",
        @"cdadar",
        @"cdaddr",
        @"cddaar",
        @"cddadr",
        @"cdddar",
        @"cddddr",
        @"case",
        @"ceiling",
        @"char->integer",
        @"char-alphabetic?",
        @"char-ci<=?",
        @"char-ci&lt;?",
        @"char-ci=?",
        @"char-ci>=?",
        @"char-ci>?",
        @"char-downcase",
        @"char-lower-case?",
        @"char-numeric?",
        @"char-ready?",
        @"char-upcase",
        @"char-upper-case?",
        @"char-whitespace?",
        @"char<=?",
        @"char&lt;?",
        @"char=?",
        @"char>=?",
        @"char>?",
        @"char?",
        @"complex?",
        @"cond",
        @"cons",
        @"cos",
        @"current-input-port",
        @"current-output-port",
        @"define",
        @"define-syntax",
        @"delay",
        @"display",
        @"do",
        @"dynamic-wind",
        @"else",
        @"eof-object?",
        @"eq?",
        @"equal?",
        @"eqv?",
        @"eval",
        @"even?",
        @"exact->inexact",
        @"exact?",
        @"exp",
        @"expt",
        @"floor",
        @"for-each",
        @"force",
        @"gcd",
        @"if",
        @"inexact->exact",
        @"inexact?",
        @"input-port?",
        @"integer->char",
        @"integer?",
        @"interaction-environment",
        @"lambda",
        @"lcm",
        @"length",
        @"let",
        @"let-syntax",
        @"let*",
        @"letrec",
        @"letrec-syntax",
        @"list",
        @"list->string",
        @"list->vector",
        @"list-ref",
        @"list-tail",
        @"list?",
        @"log",
        @"make-string",
        @"make-vector",
        @"map",
        @"max",
        @"member",
        @"memq",
        @"memv",
        @"min",
        @"modulo",
        @"negative?",
        @"newline",
        @"not",
        @"null-environment",
        @"null?",
        @"number->string",
        @"number?",
        @"odd?",
        @"or",
        @"output-port?",
        @"pair?",
        @"peek-char",
        @"port?",
        @"positive?",
        @"procedure?",
        @"quasiquote",
        @"quote",
        @"quotient",
        @"rational?",
        @"read",
        @"read-char",
        @"real?",
        @"remainder",
        @"reverse",
        @"round",
        @"scheme-report-environment",
        @"set!",
        @"set-car!",
        @"set-cdr!",
        @"sin",
        @"sqrt",
        @"string->list",
        @"string->number",
        @"string->symbol",
        @"string-append",
        @"string-ci<=?",
        @"string-ci&lt;?",
        @"string-ci=?",
        @"string-ci>=?",
        @"string-ci>?",
        @"string-copy",
        @"string-fill!",
        @"string-length",
        @"string-ref",
        @"string-set!",
        @"string<=?",
        @"string<?",
        @"string=?",
        @"string>=?",
        @"string>?",
        @"string?",
        @"substring",
        @"symbol->string",
        @"symbol?",
        @"syntax-rules",
        @"tan",
        @"truncate",
        @"unquote",
        @"unquote-splicing",
        @"values",
        @"vector",
        @"vector->list",
        @"vector-fill!",
        @"vector-length",
        @"vector-ref",
        @"vector-set!",
        @"vector?",
        @"write",
        @"write-char",
        @"zero?",
        @"e::active-room",
        @"e::active-store-size",
        @"e::aging-room",
        @"e::all?",
        @"e::any?",
        @"e::atom?",
        @"e::bit-and",
        @"e::bit-not",
        @"e::bit-or",
        @"e::bit-shift-left",
        @"e::bit-shift-right-arithmetic",
        @"e::bit-shift-right-logical",
        @"e::bit-xor",
        @"e::block-any-binding",
        @"e::block-symbol-binding",
        @"e::bound-instance?",
        @"e::c-boolean->scheme-boolean?",
        @"e::clear-compiler-on!",
        @"e::clear-permanent!",
        @"e::clear-show-full-precision!",
        @"e::closed-port?",
        @"e::compiler-on?",
        @"e::compile-form",
        @"e::cons-with-continuation",
        @"e::debug",
        @"e::deep-copy",
        @"e::define-no-compile",
        @"e::entropy-death",
        @"e::error",
        @"e::error-port",
        @"e::exit",
        @"e::expand-macro",
        @"e::expand-macros-recursively",
        @"e::fixnum?",
        @"e::float?",
        @"e::forced?",
        @"e::forgettable?",
        @"e::forgettable-expiration-time",
        @"e::forgettable-forgotten?",
        @"e::forgettable-object",
        @"e::forgettable-remembered?",
        @"e::forgettable-remember-probability",
        @"e::forgettable-unexpired?",
        @"e::full-gc",
        @"e::gensym",
        @"e::get-tag",
        @"e::hide-sense-lights!",
        @"e::inf?",
        @"e::inspect",
        @"e::list-print-depth",
        @"e::list-print-length",
        @"e::set-list-print-depth!",
        @"e::set-list-print-length!",
        @"e::set-vector-print-depth!",
        @"e::set-vector-print-length!",
        @"e::vector-print-depth",
        @"e::vector-print-length",
        @"e::logic-constant?",
        @"e::macro",
        @"e::macro-body",
        @"e::macro?",
        @"e::main",
        @"e::make-forgettable",
        @"e::make-simple-class",
        @"e::money->string",
        @"e::multiple-values?",
        @"e::nan?",
        @"e::ninety-percent-used?",
        @"e::non-printing-object?",
        @"e::not-null?",
        @"e::number->string-with-n-decimals",
        @"e::original-cwcc",
        @"e::perform-applicative",
        @"e::permanent?",
        @"e::promise?",
        @"e::random",
        @"e::read-string-with-prompt",
        @"e::reset",
        @"e::room",
        @"e::rotate-sense-lights!",
        @"e::sense-light-number?",
        @"e::sense-light-illumination-number?",
        @"e::set-sense-light-illuminations!",
        @"e::set-sense-light-n-illumination!",
        @"e::set-sense-light-n-visibility!",
        @"e::set-sense-lights-off!",
        @"e::set-compiler-on!",
        @"e::set-forgettable-expiration-time!",
        @"e::set-forgettable-remember-probability!",
        @"e::set-permanent!",
        @"e::set-show-full-precision!",
        @"e::set-tag!",
        @"e::show-active-memory",
        @"e::show-aging-memory",
        @"e::show-dynamic-environment-list",
        @"e::show-environment",
        @"e::show-environment-list",
        @"e::show-full-precision?",
        @"e::show-locks",
        @"e::show-memory",
        @"e::show-room",
        @"e::show-sense-lights!",
        @"e::srandom",
        @"e::stack-depth",
        @"e::store-size",
        @"e::substitute-new-permanent-symbols",
        @"e::time",
        @"e::top-loop-code",
        @"e::usleep",
        @"e::version",
        @"e::warning",
        @"e::write-continuation",
        @"e::write-stack",
        @"c::block",
        @"c::compiling?",
        @"c::define",
        @"c::down-in-flames!!!",
        @"c::environment",
        @"c::hash-string",
        @"c::let-deadlock-time-out",
        @"c::load-from-string",
        @"c::number->string-with-c-format",
        @"c::release-block",
        @"c::release-locks",
        @"c::set-blocked-binding!",
        @"c::symeval",
        nil
        ];
    }


//################################################################################
//
//######## Historical code:  Left in place for use in a "Test" menu (not now installed) in the menu bar.

- (IBAction)Test1:(id)sender {
    [self reportDisasterAndQuit:"This is a test error message.  It is only a test.  Pixie Scheme II will quit anyway, though."];
    }

- (IBAction)Test2:(id)sender {
    [self tellEvaluator: (SInt32)((random() % C_NUMBER_OF_MESSAGES) & WORD_MASK)
               withLong: 137
             withString: "Test string for evaluator."];
    }

- (IBAction)Test3:(id)sender {
    }

- (IBAction)Test4:(id)sender {
    }

- (IBAction)Test5:(id)sender {
    }

- (IBAction)Test6:(id)sender {
    [self toggleCompileDefines:cDMenuItem];
    }

- (IBAction)Test7:(id)sender {
    }

- (IBAction)Test8:(id)sender {
    }

@end

