/****************************************************************************************

PixieIIIMainAux.c++ -- Copyright (c) by Jay Reynolds Freeman 1987, 1988, 1989, 1990, 1991, 2006

Auxiliaries "Main" for Pixie Scheme III.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import <sys/param.h>
#import <sys/sysctl.h>
#import <signal.h>
#import "WraithHeader.h"
#import "MemoryManager.h"
#import "Clowder.h"
#import "DataTypes.e"
#import "MemoryManager.h"
#import "Events.e"
#import "Globals.e"
#import "Intern.e"
#import "IO.e"
#import "MMapStuff.e"
#import "OpCodes.h"
#import "Reader.e"
#import "Registers.e"
#import "SECDIO.e"
#import "SECDMachine.e"
#import "SECDRegisters.e"
#import "WorldHandler.h"
#import "SchemeCType.e"
#import "SchemeFunctions.e"
#import "SchemeFunctionExterns.e"
#import "SchemeGlobals.e"
#import "SchemeSpecials.e"
#import "StreamHandling.e"
#import "StringLoader.e"

#define KITTEN_LAUNCH_DELAY 100000
#define MAX_LAUNCH_RETRIES  100
#define MAX_INIT_WAITS      1000

#define LOAD_WORLD_DELAY    1000  // microseconds
#define INIT_GLOBALS_DELAY  1000  // microseconds

#import "CocoaInterface.h"

// Globals

UnsignedWord theKittenNumber = 0;

UnsignedWord theNumberOfKittens = 0;
Boolean usingGenerationalGC = TRUE;

MemoryManager *theHeap = 0;
Clowder *theClowder = 0;
WorldHandler *theWorldHandler = 0;
UnsignedLongWord *theSimpleTable = 0;
Boolean bigHeap = FALSE;
Boolean BigEndianFlag = FALSE;
long long hardwareMemorySize = 0;
Text lastWorldLoadedName[ MAX_PATHNAME_LENGTH + 1];
Text InitFileName[ MAX_PATHNAME_LENGTH ];
Boolean useStdinP = FALSE;
Boolean isMomCatP = TRUE;
Boolean aWorldHasJustBeenLoaded = FALSE;

LongWord progressFractionInThousandths = 0;

Text theKittenString[ 32 ];
Text appSupDirName[ MAX_PATHNAME_LENGTH + 1 ];
Text appSupportWorldFileName[ MAX_PATHNAME_LENGTH ];
Text appSupportEditSessionName[ MAX_PATHNAME_LENGTH ];

Boolean isAlarmSet = FALSE;
LongWord alarmTime = 0;

Boolean isOriginalWorld = TRUE;

// end globals

static Boolean InitFileExists;
static LongWord nFuncs = 0;

// Some targets for longjmp.

jmp_buf debugBuffer;
jmp_buf ExitBuffer;
jmp_buf ResetErrorBuffer;

Boolean debugBufferInitialized = FALSE;

FILE *logFile = 0;

/**** Forwards: ****/

Text *loadWorldIfPossible();
Boolean momSaysReset();
void openKittenLogFile();
Boolean allKittensAreActive();

/**** Here come the functions: ****/

int getProcessorCount()
{

    int     count;
    size_t  size=sizeof(count);

    if( sysctlbyname("hw.ncpu", &count, &size, NULL, 0 ) )
        return 1;

    return count;
    }

/**** failAtOnce ****/

void failAtOnce()
{ 
     kill( getpid(), SIGBUS );     // Doesn't generate kill report, et cetera.
     }

/**** debugEntry -- "longjmp" to restart "eval" ****/

void debugEntry()
{ 
    if( debugBufferInitialized )
        longjmp( debugBuffer, 1 );
    else
        exit( 1 );
    }

/**** exitTopLoop ****/

void exitTopLoop() { longjmp( ExitBuffer, 1 ); }

/**** topLoopCode -- The canonical top-level read-eval-print loop.  Push into the
                     continuation, code that (1) reads an object, (2) pushes the object
                     that was read back into the continuation (so it can in turn be
                     evaluated), (3) writes the result (to "defaultOutputFD"), (4) writes
                     a newline (to "defaultOutputFD"), and finally (5) loads the code
                     for "topLoopCode" itself into the continuation (thus "topLoopCode"
                     tail-recurses).
*/

PROC( topLoopCode )

    depthSoFar = vDepthSoFar = 0;                   // Hackish
    SPushR();
    MOVE( TLC, R );
    CSPushProperList();
    SPopR();
    CONTINUE

END_PROC

/**** runTopLoop ****/

static Boolean InitFileLoadAttempted = FALSE;

void runTopLoop( Boolean InitFilePresent )
{
    Boolean runFirstCode = FALSE;

    depthSoFar = vDepthSoFar = 0;

    tprintf2( &CurrentOutputPort, MiscStrings[ 105 ] );     // "Top-level loop ..."
    tfflushOutput( getPortFilePointer( &CurrentOutputPort ) );

    SPushR();
    MOVE( TLC, R );
    CSPushProperList();
    SPopR();

    if( aWorldHasJustBeenLoaded ) {                         // Deal with a world load.
        aWorldHasJustBeenLoaded = FALSE;
        LOAD_R_TAG( exactShortFixnumBitsMask )
        LOAD_OBJECT( NameStrings[ 377 ], R )
        CALL( intern )
        BOUND_SYMBOL_P
        JUMP_FALSE( noCode )

        // Here following, we evaluate the function to be run
        // on behalf of the world just loaded, and set up to run it.

        EVAL_SYMBOL
        LAMBDA_PAIR_P
        JUMP_FALSE( noCode )
        NILCONS
        PUSH
		MOVE( zero, R )
        SWAP

extern void tellKittenCode();

        CALL( tellKittenCode )
        runFirstCode = TRUE;                            // The world has a function to run.
      noCode:
        runFirstCode = FALSE;                           // The world does not have a function to run.
        }

    if( runFirstCode )                                      // We are set to run the function.
        ;
    else if( ! InitFileLoadAttempted ) {                    // Check for an "init" file, and load it if applicable.
        InitFileLoadAttempted = TRUE;
        if( InitFilePresent ) {
            LOAD_R_TAG( schemeMachinePCIndexTag )
            LOAD_OBJECT( assertNotEvaluating_index, R )
            PUSH_CONTINUATION
            LOAD_R_TAG( schemeMachinePCIndexTag )
            LOAD_OBJECT( loadCode_index, R )
            PUSH_CONTINUATION
            LOAD_R_TAG( exactShortFixnumBitsMask )
            LOAD_OBJECT( InitFileName, R )
            MAKE_STRING
            PUSH
            MOVE( one, R )
            }
        }       

    setjmp( debugBuffer );                                  // This is the final position of the debug longjmp target.
    debugBufferInitialized = TRUE;                    
    eval();                                                 // ... and it never returned, and its fate is still unlearned ...
    }

/**** initializeApplicationSupportDirectory ****/

extern int errno;

void initializeApplicationSupportDirectory()
{
    int statResult = 0;
    int statError = 0;
    struct stat appDirStatBuffer;

    statResult = stat( appSupDirName, &appDirStatBuffer );
    statError = errno;
    if( (statResult == 0) ) {
        if( appDirStatBuffer.st_mode & S_IFDIR )  // Fine!
            return;
        else {
            fprintf( stderr, MiscStrings[ 392 ], appSupDirName, MiscStrings[ 508 ] );
            exit( 1 );
            }
        }
    else {
        if( mkdir( appSupDirName, 0777 ) == 0 ) { // Fine!
             return;
             }
        else {
            fprintf( stderr, MiscStrings[ 393 ], MiscStrings[ 508 ], appSupDirName );
            exit( 1 );
            }
        }
    return;
    }

/**** For developer use -- optional initialization tests to be run on entry, before something else breaks. ****/

void initializationTests()
{

    fprintf( stderr, "INITIALIZATION TESTS:\n");  fflush( stderr );

    }


/**** initialize -- master initialization portmanteau ****/

void initialize( UnsignedLongWord *memorySize, UnsignedLongWord *generationSize ) 
{
//    printf("tagged aval has size %lu, UnsignedLongWord %lu\n", sizeof( taggedAval ), sizeof( UnsignedLongWord ) );  // DEBUG
    size_t localMemorySize = (size_t)(*memorySize);

    srandom( (Word)time( NULL ) );          // Seed the random number generator.

//    initializationTests();  // Developer option -- for testing.
    initializeEvents();
    initializeLowLevelIO();

    const NXArchInfo *localArchitecture = NXGetLocalArchInfo();

    if( localArchitecture == NULL )
        fatalError( 91 );
    else if( localArchitecture->byteorder == NX_UnknownByteOrder )
        fatalError( 91 );
    else if( localArchitecture->byteorder == NX_BigEndian )
        BigEndianFlag = TRUE;
    else // NX_LittleEndian
        BigEndianFlag = FALSE;

    initializeStartupWorkingFolder();
    initializeSignalHandlers();

    initializeApplicationSupportDirectory();
    theClowder = new Clowder;
//    theClowder->initialize( clowderReserve, interruptInformationReserve );
    theClowder->initializeByMalloc();
    clowderIsOperating = 1;

    sprintf( theKittenString, MiscStrings[ 399 ], theKittenNumber );

    initializeFileDescriptorsIfNecessary();
    initializeCharTypes();
    initializeSpecialTags();
    initializeDefaultPorts();
    initializeReader();
    initializeMiscChars();

    theHeap = new MemoryManager;
    theHeap->initialize();

//    theHeap->initializeMemoryManager( BigEndianFlag, (Boolean)TRUE, &(lastWorldLoadedName[ 0 ]), MAX_PATHNAME_LENGTH,
//                                      (size_t)(*memorySize), (size_t)(*memorySize / 10), (size_t)(*memorySize / 10),
//                                      (size_t)(*generationSize), &localMemorySize,
//                                      spaceAReserve, spaceBReserve, scratchStackReserve, gGCScratchReserve );
    theHeap->initializeMemoryManagerByMalloc( BigEndianFlag, (Boolean)TRUE, &(lastWorldLoadedName[ 0 ]), MAX_PATHNAME_LENGTH,
                                      (size_t)(*memorySize), (size_t)(*memorySize / 10), (size_t)(*memorySize / 10),
                                      (size_t)(*generationSize), &localMemorySize );
    *memorySize = localMemorySize;

    theClowder->allocateBitsForProper( *memorySize );
    theClowder->setKittenEvaluatorActivity( KITTEN_ACTIVITY_INITIALIZED_MEMORY );
    theHeap->setClowder( theClowder );
    theWorldHandler = theHeap->getWorldHandler();
    theSimpleTable = theWorldHandler->theSimpleTable;
    initializePortmanteauIndices();

    initializeDataTypeTable( false, false );
    initializeDataTypeList();
    initializeMachine();
    initializeObList( TRUE );

    initializeEnvironmentList( TRUE );
    nFuncs = loadSchemeFunctions( FALSE );
    loadSchemeSpecialForms( FALSE );
    loadSchemeGlobals( FALSE );
    theClowder->setKittenEvaluatorActivity( KITTEN_ACTIVITY_INITIALIZED_GLOBALS );
    }

/**** errorRecovery ****/

void errorRecovery( Text *reinitializationAction )
{
    reinitializeMachine();
    reinitializeEvents();
    reinitializeLowLevelIO( reinitializationAction );
    initializeDefaultPorts();
    initializeReader();
    theClowder->resetMyStuff();
    theClowder->setMomSaysToEverybody( MOM_SAYS_NOTHING );
    }

/**** resetCleanUp ****/

void resetCleanUp()
{

    }

/**** loadWorldIfPossible ****/

Boolean hasAppSupportEditSession = FALSE;

Text *loadWorldIfPossible()
{
    FILE *scratch;
    struct stat sb;
    Text bundleWorldFileName[ MAX_PATHNAME_LENGTH ];
    Boolean hasBundleWorld = TRUE;    
    Boolean hasAppSupportWorld = TRUE;    

    static Text bundleWorldFileTitle[]       = "Pixie Scheme.app/PixieSchemeIII.world";
    static Text appSupportWorldFileTitle[]   = "/My Pixie Scheme III World.world";
    static Text appSupportEditSessionTitle[] = "/My Pixie Scheme III Edit Session.txt";

    strncpy( bundleWorldFileName, InitFilePath, sizeof( bundleWorldFileName ) );
    strncat( bundleWorldFileName, bundleWorldFileTitle, sizeof( bundleWorldFileName ) - strlen( bundleWorldFileName ) );
//    fprintf( stdout, "Bundle world file name is \"%s\"\n", bundleWorldFileName ); fflush( stdout );  // DEBUG
    if( stat( bundleWorldFileName, &sb ) == -1 )
        hasBundleWorld = FALSE;

    strncpy( appSupportWorldFileName, appSupDirName, sizeof( appSupportWorldFileName ) );
    strncat( appSupportWorldFileName, appSupportWorldFileTitle,
             sizeof( appSupportWorldFileName ) - strlen( appSupportWorldFileName ) );

//    fprintf( stdout, "app support world file name is \"%s\"\n", appSupportWorldFileName ); fflush( stdout );  // DEBUG
    if( stat( appSupportWorldFileName, &sb ) == -1 )
        hasAppSupportWorld = FALSE;

    strncpy( appSupportEditSessionName, appSupDirName, sizeof( appSupportEditSessionName ) );
    strncat( appSupportEditSessionName, appSupportEditSessionTitle,
             sizeof( appSupportEditSessionName ) - strlen( appSupportEditSessionName ) );
//    fprintf( stdout, "app support edit session name is \"%s\"\n", appSupportEditSessionName ); fflush( stdout );  // DEBUG
    if( stat( appSupportEditSessionName, &sb ) == -1 )
        hasAppSupportEditSession = FALSE;
    else
        hasAppSupportEditSession = TRUE;

    if( hasAppSupportWorld && theHeap->worldIsUseable( appSupportWorldFileName ) ) {
        if( (scratch = fopen( appSupportWorldFileName, "r" )) == NULL ) {
            tprintf2( &ErrorPort, MiscStrings[ 509 ] );
            if( (scratch = fopen( bundleWorldFileName, "r" )) == NULL ) {
                fatalError( 191 );
                }
            else {
                worldFD->fp = scratch;
                strcpy( worldFD->oldPathName, worldFD->pathName );
                strcpy( worldFD->pathName, bundleWorldFileName );
                strcpy( worldFD->type, "r" );
                theHeap->loadWorldGuts( worldFD, FALSE );
                isOriginalWorld = TRUE;
                }
            }
        else {
            worldFD->fp = scratch;
            strcpy( worldFD->oldPathName, worldFD->pathName );
            strcpy( worldFD->pathName, appSupportWorldFileName );
            strcpy( worldFD->type, "r" );
            theHeap->loadWorldGuts( worldFD, FALSE );
            isOriginalWorld = FALSE;
            }
        }
    else if( hasBundleWorld ) {
        if( (scratch = fopen( bundleWorldFileName, "r" )) == NULL ) {
            fatalError( 191 );
            }
        else {
            worldFD->fp = scratch;
            strcpy( worldFD->oldPathName, worldFD->pathName );
            strcpy( worldFD->pathName, bundleWorldFileName );
            strcpy( worldFD->type, "r" );
            theHeap->loadWorldGuts( worldFD, FALSE );
            isOriginalWorld = TRUE;
            }
        }
    else {
        fatalError( 192 );
        }

    return 0;
    }

/**** cleanUpForExit ****/

void cleanUpForExit()
{
    reinitializeSignalHandlers();
    cleanUpLowLevelIO();
    theHeap->releaseMemoryFiles();
    theClowder->releaseClowderFiles();
    }

/**** setEvaluatorSetup ****/

void setEvaluatorSetup( userSetup *aUserSetup )
{
    if( (CompileDefinesFlag = aUserSetup->compilerOn) )
        tellController( E_COMPILING_DEFINES, 0, "" );
    else
        tellController( E_NOT_COMPILING_DEFINES, 0, "" );

    if( (useFullNumericPrecision = aUserSetup->showsFullPrecision) )
        tellController( E_USING_HIGH_PRECISION, 0, "" );
    else
        tellController( E_USING_LOW_PRECISION, 0, "" );

    }

/**** runSchemeWithArguments ****/

void runSchemeWithArguments( const char *LocalInitFileName, const char *LocalInitFilePath,
                             UnsignedLongWord LocalMemorySize, UnsignedLongWord LocalGenSize, unsigned char LocalInitFileExists )
{
    Word exitValue = 1;
    Word resetState = 0;
    Text *loadWorldString = 0;
    userSetup aUserSetup;
    static Text finalCommandBuffer[ MAX_PATHNAME_LENGTH + 1011 ];

    strncpy( InitFileName, LocalInitFileName, sizeof( InitFileName ) - 1 );
    strncpy( InitFilePath, LocalInitFilePath, MAX_PATHNAME_LENGTH - 1 );

    InitFileExists = LocalInitFileExists;

    if( (exitValue = setjmp( ExitBuffer )) != 0 ) {  // Exit from total disaster.
        if( exitValue == 1 )
            cleanUpForExit();
        exit( --exitValue );
        }
    initialize( &LocalMemorySize, &LocalGenSize );
    if( setjmp( debugBuffer ) )
        fatalError( 133 );
    loadWorldString = loadWorldIfPossible();
    worldIsLoaded = 1;
    theClowder->setKittenEvaluatorActivity( KITTEN_ACTIVITY_LOADED_WORLD );

    while( ! GUIInitialized )
        usleep( KITTEN_LAUNCH_DELAY );

    // NOW we can do I/O to the Pixie Scheme III window and its panels ...

    tprintf2( &CurrentOutputPort, MiscStrings[ 337 ]);
    tprintf2( &CurrentOutputPort, MiscStrings[ 224 ]);
    tprintf2( &CurrentOutputPort, MiscStrings[ 309 ]);
    tprintf2( &CurrentOutputPort, MiscStrings[ 115 ] );
    tprintf2( &CurrentOutputPort, MiscStrings[ 432 ]);
    tprintf2( &CurrentOutputPort, MiscStrings[ 115 ] );
//    tprintf3( &CurrentOutputPort, MiscStrings[ 227 ], LocalMemorySize );

    if( loadWorldString != 0 )
        tprintf2( &CurrentOutputPort, loadWorldString );

    theWorldHandler->retrieveUserSetup( &aUserSetup );
    if( ! isOriginalWorld )
        setEvaluatorSetup( &aUserSetup );

    emitIOInitializationMessages();
    reportInitialEvents();

    if( (resetState = setjmp( ResetErrorBuffer )) == FINAL_RESET ) {
        snprintf( finalCommandBuffer, MAX_PATHNAME_LENGTH + 100, "(c::save-world \"%s\")\n", appSupportWorldFileName );
//        fprintf( stdout, "Final command is \"%s\"\n", finalCommandBuffer ); fflush( stdout );  // DEBUG
        errorRecovery( finalCommandBuffer );
        }
    else if( resetState != 0 ) {                 // Recover from mere calamity.
        errorRecovery( 0 );
        emitStatusMessage( MiscStrings[ 384 ] );
        reportEvent( REINITIALIZATION_COMPLETE );
        }
    runTopLoop( InitFileExists );
    }    

/**** runSchemeWithPathAndMemory ****/

int runSchemeWithPathAndMemory( const char *argv0, UnsignedLongWord memorySizeProvided, UnsignedLongWord generationSizeProvided )
{
    Text LocalInitFilePath[ MAX_PATHNAME_LENGTH ];

#define SLASHES_TO_BACK_UP 2

//    fprintf( stdout, "argv[0] is \"%s\"\n", argv0 ); fflush( stdout );  // DEBUG

//    logFile = fopen("/Users/JayFreeman/Desktop/PixieIIILog.text", "w");

    strcpy( LocalInitFilePath, argv0 );   // "<whatever>/Pixie Scheme.app/Pixie Scheme"

    long counter = strlen( LocalInitFilePath );
    int slashesFound = 0;

    while( --counter >= 0 ) {
        if( LocalInitFilePath[ counter ] == '/' )
            ++slashesFound;
        if( slashesFound == SLASHES_TO_BACK_UP )
            break;
        }
    LocalInitFilePath[ counter + 1 ] = 0;

    runSchemeWithArguments( "", LocalInitFilePath, memorySizeProvided, generationSizeProvided, FALSE );
    return 0;
    }

/**** momSaysQuit ****/

Boolean momSaysQuit() { return( theClowder->getMomSays() == MOM_SAYS_EXIT ); }

/**** momSaysReset ****/

Boolean momSaysReset() { return( theClowder->getMomSays() == MOM_SAYS_RESET ); }

/**** momSaysWorldOp ****/

Boolean momSaysWorldOp() { return( theClowder->getMomSays() == MOM_SAYS_WORLD_OPERATION ); }

/**** allKittensAreActive ****/

Boolean allKittensAreActive() { return theClowder->allKittensActiveAt( KITTEN_ACTIVITY_READY_TO_EVAL ); }

/**** reportGUIInitialized ****/

void reportGUIInitialized()
{
    theClowder->setKittenGUIActivity( KITTEN_GUI_ACTIVITY_COMPLETED_INITIALIZATION );
    GUIInitialized = 1;
    }

/**** increaseBlocking ****/

void increaseBlocking() { theClowder->increaseKittenBlock(); }

FILE *kittenLogFile = 0;

