//
//  JRF6UITextView.m
//
//  Created by Jay Freeman on 07/07/2010
//  Copyright 2010 Jay Reynolds Freeman. All rights reserved.
//

//  Specialized for the Pixie Scheme III view.
/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628
*/

#import "JRF6UITextView.h"
#import "ASCIITypes.h"
#import "SchemeCType.h"

const CGFloat xOffset = 8.F;   // Observed left inset of UITextView text.
const CGFloat yOffset = 8.F;  // Observed top inset of UITextView text.

Word keyboardSelectEvalEquivalentUsed = FALSE;

@implementation JRF6UITextView

@synthesize typingRangeIlluminated;

+ (void) _keepAtLinkTime {}

// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return YES;
    }

const NSUInteger textInitialCapacity = 8192;

- (void) applicationDidFinishLaunchingContent {

    NSString *osVersion = [[UIDevice currentDevice] systemVersion];
    osVersionFloat = [osVersion floatValue];
//    NSLog(@"osVersionFloat = %g", osVersionFloat );  // DEBUG

    color1 = [[UIColor colorWithHue:(198.0F/360.0F)  saturation:1.00F  brightness:0.60F alpha:0.24F] retain];  // "selected" blue
    color2 = [[UIColor colorWithRed:0.00F  green:1.00F  blue:0.0F alpha:0.50F] retain];                        // green
    color3 = [[UIColor colorWithRed:1.00F  green:0.50F  blue:0.0F alpha:0.50F] retain];                        // orange
    color4 = [[UIColor colorWithRed:1.00F  green:0.00F  blue:0.0F alpha:0.50F] retain];                        // red

    lastSelectedRange.location = 0;
    lastSelectedRange.length = 0;
    typingRange.location = 0;
    typingRange.length = 0;
    typingRangeIsActive = FALSE;
    typingRangeIlluminated = FALSE;
    illuminatedRange.location = 0;
    illuminatedRange.length = 0;
    [self setAutocapitalizationType:UITextAutocapitalizationTypeNone];
    [self setAutocorrectionType:UITextAutocorrectionTypeNo];
    [self setKeyboardType:UIKeyboardTypeASCIICapable];
	[self setText:[NSMutableString stringWithCapacity:textInitialCapacity]];

    theParenFinder = [[[ParenFinder alloc] init] retain];

//    NSLog(@"theMainJRFTextView is about to try to become first responder");  // DEBUG
//    [self becomeFirstResponder];            // Show the keyboard at launch.
    }

- (void) setDealWithText:(SEL)aMethod { dealWithText = aMethod; }

- (NSRange) replaceCharactersInRange:(NSRange)aRange withString:(NSString *)aString {

    NSString *theText = [self text];
    NSUInteger theLength = [theText length];
    NSRange newSelectedRange;
    
    newSelectedRange.length = 0;

    if( aString == nil ) {
        newSelectedRange.location = aRange.location + aRange.length;
        return newSelectedRange;
        }

    if( aRange.location > theLength ) {
        [(NSMutableString *)theText appendString:aString];
        newSelectedRange.location = [theText length];
        }
    else {
        [(NSMutableString *)theText replaceCharactersInRange:aRange withString:aString];
        newSelectedRange.location = aRange.location + [aString length];
        }
    
    [self performSelectorOnMainThread:@selector(setText:) withObject:theText waitUntilDone:YES];
    [self setSelectedRange:newSelectedRange];
    return newSelectedRange;
    }

- (NSRange) getSelectedRange { return [self selectedRange]; }

- (NSRange) getTypingRange { return typingRange; }

- (void) showEndRange {

    NSRange endRange;

    if( [self hasText] ) {
        endRange.location = [[self text] length] -  1;
        endRange.length = 1;
        [self scrollRangeToVisible:endRange];
        }
    }

- (void) showSelectedRange {

    NSRange workaroundRange = [self selectedRange];   // Dealing with a known bug in iOS 3.x here, "scrollRangeToVisible" does
                                                      //   not like ranges with zero length.
    if( workaroundRange.location == 2147483647 ) {
        return;
        }
    if( workaroundRange.length == 0 ) {
        if( workaroundRange.location > 0 ) {
            --workaroundRange.location;
            ++workaroundRange.length;
            }
        else if( [[self text] length] > 0 )
            ++workaroundRange.length;
        //else nothing, we have no text at all.
        }

    [self scrollRangeToVisible:workaroundRange];
    }

- (NSRange) currentLinesRange {

    NSRange selectedRange = [self selectedRange];
    NSString *allText = [self text];
    NSInteger textLength = [allText length];
    NSRange answer;
    NSInteger answerEnd;

    if( textLength == 0 ) {
        answer.location = 0;
        answer.length = 0;
        return answer;
        }

    answer.location = selectedRange.location;
    if( (answer.location >= textLength) || ([allText characterAtIndex:answer.location] == NEWLINE) )
        --(answer.location);
    for( ;
         (answer.location > 0) && ([allText characterAtIndex:answer.location] != NEWLINE);
         --(answer.location) )
        ;
    if( [allText characterAtIndex:answer.location] == NEWLINE )
        ++(answer.location);

    for( answerEnd = selectedRange.location + selectedRange.length;
         (answerEnd < textLength) && ([allText characterAtIndex:answerEnd] != NEWLINE);
         ++answerEnd )
        ;

    answer.length = answerEnd - answer.location;

    return answer;
    }

- (IBAction) selectCurrentLines:(id)sender {

    [self setSelectedRange:[self currentLinesRange]];
    [self setTypingRangeIsActive:NO];
    [theController dealWithTextRange:NO];
    [theController dealWithSelection:YES];
    }

- (void) transmitAll:(NSString *)theString {

    if( [theString length] > 0 ) {
        [theController performSelector:dealWithText withObject:theString];
        }
    }

// Overridden to allow checking for built-in Scheme names.

- (NSArray *)completionsForPartialWordRange:(NSRange)charRange inString:(NSString *)string language:(NSString *)language {

    NSLog(@"completionsForPartialWordRange called in JRF6UITextView");  // DEBUG

    return [theController spellingCompletionsForPartialWord:[string substringWithRange:charRange]];

    }

- (NSArray *)guessesForWordRange:(NSRange)range inString:(NSString *)string language:(NSString *)language {

    NSLog(@"guessesForWordRange called in JRF6UITextView");  // DEBUG

    return [theController spellingCompletionsForPartialWord:[string substringWithRange:range]];

    }

- (NSRange)rangeForUserCompletion {

    NSRange selectionRange = [self selectedRange];
    NSString *theString = [self text];
    NSRange destinationRange = selectionRange;
    SInt32 c = 0;    
    SInt32 idx = 0;

    for( idx = selectionRange.location - 1; idx >= 0; --idx ) {
        c = [theString characterAtIndex:idx];
        if( isSubsequent_M( c ) || isInitial_M( c ) ) {
            destinationRange.location = idx;
            (destinationRange.length)++;
            }
        else
           break;
        }
    return destinationRange;
    }

- (IBAction)markOrTransmitSelection:(id)sender {

    if( sender == theController )
        keyboardSelectEvalEquivalentUsed = TRUE;

    if( [self matchedParensDisplayed] ) {

//        NSLog(@"foo");  // DEBUG
        NSRange parenRange;

        parenRange.location = [[theController tintRectangle4] index];
        parenRange.length   = [[theController tintRectangle5] index] - parenRange.location + 1;

        [self setTypingRangeIsActive:NO];
        [self setSelectedRange:parenRange];
        [theController dealWithSelection:YES];
        [self illuminateRange:parenRange color:1];
        }
    else if( [self typingRangeIsActive] ) {
        [self setTypingRangeIsActive:NO];
        [self setSelectedRange:typingRange];
        [theController dealWithSelection:YES];
        [self illuminateRange:typingRange color:1];
        }
    else if( [self hasSelectedText] ) {
        [self setTypingRangeIsActive:NO];
        [theController dealWithTextRange:NO];
    	[self transmitSelection:sender];
        }
    else {  // We have no selections of any kind
        [theController dealWithTextRange:NO];
        }
    }

- (IBAction)transmitSelection:(id)sender {

    NSRange selectionRange = [self selectedRange];
    NSRange newSelectionRange;

    if( selectionRange.length == 0 )
        ;  // NSLog(@"Selection range in \"transmitSelection\" is zero.");  // DEBUG
    else {
        newSelectionRange.location = selectionRange.location + selectionRange.length;
        newSelectionRange.length = 0;
        [self replaceCharactersInRange:newSelectionRange withString:@"\n"];
        newSelectionRange.location += 1;
        [self setSelectedRange:newSelectionRange];
        [self showSelectedRange];
        [theController setRangeForNextInsertion:newSelectionRange];
        [self transmitAll:[[self text] substringWithRange:selectionRange]];
        }
    }

- (bool) hasSelectedText { return ([self selectedRange].length > 0); }

- (bool) typingRangeIsActive { return typingRangeIsActive; }
- (bool) typingRangeIsNonEmpty { return (typingRange.length > 0); }
- (void) setTypingRangeIsActive:(bool)value { typingRangeIsActive = value; }

- (NSString *)typingRangeContent {

    if( (! [self hasText]) || (typingRange.length == 0 ) )
        return @"";
    else if( typingRange.location > [[self text] length] )
        return @"<typing range location is out of range for the current text>";
    else if( (typingRange.location + typingRange.length) > [[self text] length] )
        return @"<typing range overlaps far end of current text>";
    else {
        NSString *rangeContent = [[[self text] substringWithRange:typingRange] retain];
        return [rangeContent autorelease];
        }
    }

- (NSString *)typingRangeAbbreviatedContent {

#define TYPING_RANGE_CONTENT_LIMIT 32

    if( (! [self hasText]) || (typingRange.length == 0 ) )
        return @"";
    else if( typingRange.location > [[self text] length] )
        return @"<typing range location is out of range for the current text>";
    else if( (typingRange.location + typingRange.length) > [[self text] length] )
        return @"<typing range overlaps far end of current text>";
    else if( typingRange.length <= TYPING_RANGE_CONTENT_LIMIT ) {
        NSString *rangeContent = [[[self text] substringWithRange:typingRange] retain];
        return [rangeContent autorelease];
        }
    else {
        NSRange startRange = NSMakeRange( typingRange.location, TYPING_RANGE_CONTENT_LIMIT/2 );
        NSRange endRange   = NSMakeRange( typingRange.location + typingRange.length - TYPING_RANGE_CONTENT_LIMIT/2,
                                          TYPING_RANGE_CONTENT_LIMIT/2 );
        NSString *startString = [[self text] substringWithRange:startRange];
        NSString *endString = [[self text] substringWithRange:endRange];
        NSString *answerString = [[[startString stringByAppendingString:@" ... "] stringByAppendingString:endString] retain];
        return [answerString autorelease];
        }
    }

- (void) logTypingRangeDescription {

    NSLog(@"Typing range %@ %u %u \"%@\"",
         (typingRangeIsActive ? @"active" : @"not active"),
         typingRange.location,
         typingRange.length,
         [self typingRangeAbbreviatedContent]
         );
    }

- (void) startTypingRange {

    typingRange.location = [self selectedRange].location;
    typingRange.length = 0;
    [self deilluminateRange];
    [self setTypingRangeIsActive:YES];
    }

- (void) adjustTypingRange:(NSRange)range forLength:(NSInteger)lnth isDelete:(bool)isDelete {

    if( isDelete ) {
        if( typingRange.length > 1 )
            --typingRange.length;
        else
            [self setTypingRangeIsActive:NO];
        }
    else
        typingRange.length += lnth;
    }

- (void) touchesBegan:(NSSet *)touches withEvent:(UIEvent *)theEvent {

    [super touchesBegan:touches withEvent:theEvent];
    [theController touchesBeganInView:theEvent];
    }
    
- (void)selectAll:(id)sender {

    [super selectAll:sender];
    [theController dealWithSelection:[self hasSelectedText]];
    }

- (IBAction)copy:(id)sender {

    [super copy:sender];
    [theController dealWithSelection:[self hasSelectedText]];
    }

- (IBAction)cut:(id)sender {

    [super cut:sender];
    [theController dealWithSelection:[self hasSelectedText]];
    }

- (CGPoint) topLeftCornerOfCharAtIndex:(NSUInteger)index {

    UIFont *theFont = [self font];

    CGFloat xValue = 0.F;
    CGFloat yValue = 0.F;

    NSString *content = [self text];
    NSString *stringToChar;
    
    if( (content == nil) || ([content length] == 0) )
        return CGPointMake( xOffset, yOffset );

    if( (index == 0 ) || [[NSCharacterSet newlineCharacterSet] characterIsMember:[content characterAtIndex:(index - 1)]] ) {
//        NSLog(@"Position 1");  // DEBUG
        stringToChar = [content substringToIndex:(index + 1)];
        }
    else {
//        NSLog(@"Position 2");  // DEBUG
        stringToChar = [content substringToIndex:index];
        }

    CGSize contentSize = [self contentSize];
    contentSize.width -= 2.F * xOffset;
    CGSize sizeToChar = [stringToChar sizeWithFont:theFont constrainedToSize:contentSize lineBreakMode:UILineBreakModeWordWrap];
    yValue = sizeToChar.height - [theController currentLineSize] + yOffset;

    if( (index == 0) 
        || [[NSCharacterSet newlineCharacterSet] characterIsMember:[content characterAtIndex:(index - 1)]] 
        )
        xValue = xOffset;
    else {
//        NSLog(@"Position 3");  // DEBUG
        NSString *leadString = [content substringToIndex:index];
        CGSize initialSize = [leadString sizeWithFont:theFont constrainedToSize:contentSize lineBreakMode:UILineBreakModeWordWrap];
        CGSize runningSize;
        NSInteger scan = 0;
        for( scan = index - 1; scan > 0; --scan ) {
            if( [[NSCharacterSet newlineCharacterSet] characterIsMember:[content characterAtIndex:scan]] ) {
                ++scan;
                break;
                }
            // else if scan is not pointing at the start of a word, keep backing up
            else if(    (! [[NSCharacterSet whitespaceCharacterSet] characterIsMember:[content characterAtIndex:(scan - 1)]])
                     || [[NSCharacterSet whitespaceCharacterSet] characterIsMember:[content characterAtIndex:scan]] 
                    )
                ;
            else {
//                NSLog(@"Position 4");  // DEBUG
                leadString = [content substringToIndex:scan];
                runningSize = [leadString sizeWithFont:theFont constrainedToSize:contentSize lineBreakMode:UILineBreakModeWordWrap];
                if( runningSize.height < initialSize.height ) {
                    break;
                    }
                }
            }
        if( scan == 0 ) {   // Backed up all the way to the start of the text
            if( [[NSCharacterSet newlineCharacterSet] characterIsMember:[content characterAtIndex:scan]] ) {
                ++scan;
                }
            }
        NSString *firstPartOfLine = [content substringWithRange:NSMakeRange( scan, index - scan )];
        CGSize firstPartSize = [firstPartOfLine sizeWithFont:theFont];
        xValue = xOffset + firstPartSize.width;
        }

    return CGPointMake( xValue, yValue );
    }

- (CGPoint) topRightCornerOfCharAtIndex:(NSUInteger)index {

    NSString *content = [self text];

    if( (content == nil) || ([content length] == 0) )
        return CGPointMake( xOffset, yOffset );

    if( (content == nil) || ((index + 1) < [content length]) )
        return [self topLeftCornerOfCharAtIndex:(index + 1)];
    else
        return [self topLeftJustPastEnd];
    }

- (CGPoint) topLeftJustPastEnd {

    UIFont *theFont = [self font];

    CGFloat xValue = 0.F;
    CGFloat yValue = 0.F;

    NSString *content = [self text];
    
    if( content == nil )
        return CGPointMake( xOffset, yOffset );

    NSUInteger index = [content length];

    CGSize contentSize = [self contentSize];
    contentSize.width -= 2.F * xOffset;
    CGSize initialSize = [content sizeWithFont:theFont constrainedToSize:contentSize lineBreakMode:UILineBreakModeWordWrap];
    yValue = initialSize.height - [theController currentLineSize] + yOffset;

    CGSize runningSize;
    NSString *leadString;
    NSInteger scan = 0;

    if( [[NSCharacterSet newlineCharacterSet] characterIsMember:[content characterAtIndex:(index - 1)]] ) {
        yValue += [theController currentLineSize];
        xValue = xOffset;
        }
    else {   // Last character is not a newline.
        for( scan = index - 1; scan > 0; --scan ) {
            //  Back up till we hit a newline, or a location that changes the running height.
            if( [[NSCharacterSet newlineCharacterSet] characterIsMember:[content characterAtIndex:scan]] ) {
                ++scan;
                break;
                }
            // else if scan is not pointing at the start of a word, keep backing up
            else if(    (! [[NSCharacterSet whitespaceCharacterSet] characterIsMember:[content characterAtIndex:(scan - 1)]])
                     || [[NSCharacterSet whitespaceCharacterSet] characterIsMember:[content characterAtIndex:scan]] 
                    )
                ;
            else {
                leadString = [content substringToIndex:scan];
                runningSize = [leadString sizeWithFont:theFont constrainedToSize:contentSize lineBreakMode:UILineBreakModeWordWrap];
                if( runningSize.height < initialSize.height ) {
                    break;
                    }
                }
            }
        if( scan == 0 ) {   // Backed up all the way to the start of the text
            if( [[NSCharacterSet newlineCharacterSet] characterIsMember:[content characterAtIndex:scan]] ) {
                ++scan;
                }
            }
        NSString *firstPartOfLine = [content substringWithRange:NSMakeRange( scan, index - scan )];
        CGSize firstPartSize = [firstPartOfLine sizeWithFont:theFont];
        xValue = xOffset + firstPartSize.width;
        }
    return CGPointMake( xValue, yValue );
    }

- (void) describePoint:(CGPoint)point named:(NSString *)name {

    NSLog(@"Point \"%@\" is %g %g", name, point.x, point.y );
    }

- (void) describeFrame:(CGRect)frame named:(NSString *)name {

    NSLog(@"Frame \"%@\" has origin %g %g, size %g %g", name, frame.origin.x, frame.origin.y, frame.size.width, frame.size.height );
    }

- (void) illuminateRange:(NSRange)range color:(NSInteger)colorNumber {

    if( osVersionFloat > 3.9F )
        ;
    else if( colorNumber == 1 ) {
        [[theController tintRectangle1] setBackgroundColor:color1];
        [[theController tintRectangle2] setBackgroundColor:color1];
        [[theController tintRectangle3] setBackgroundColor:color1];
        }
    else {
        [[theController tintRectangle1] setBackgroundColor:color2];
        [[theController tintRectangle2] setBackgroundColor:color2];
        [[theController tintRectangle3] setBackgroundColor:color2];
        }

    [self illuminateRange:range];
    }

- (void) illuminateRange:(NSRange) range {

    if( range.length == 0 )
        return;

    illuminatedRange = range;

    CGRect frameOne;
    CGFloat lineSize = [theController currentLineSize];
    CGSize contentSize = [self contentSize];
    contentSize.width -= 2.F * xOffset;
    CGPoint testPoint = [self topLeftCornerOfCharAtIndex:range.location];
    CGPoint testPoint3 = [self topRightCornerOfCharAtIndex:(range.location + range.length - 1)];
    frameOne.origin = testPoint;
//    NSLog(@"Test point: x %g, y %g", testPoint.x, testPoint.y );  // DEBUG

    if( osVersionFloat < 3.9F ) {
        if( testPoint3.y == testPoint.y ) { // characters are in same line
            frameOne.size = CGSizeMake( testPoint3.x - testPoint.x, lineSize );
            }
        else {
            frameOne.size = CGSizeMake( contentSize.width + 2 * xOffset - testPoint.x, lineSize );
            [[theController tintRectangle1] setFrame:frameOne];
            [[theController tintRectangle1] setHidden:NO];  
            CGRect frameThree;
            frameThree.origin = CGPointMake( xOffset, testPoint3.y );
            frameThree.size = CGSizeMake( testPoint3.x - xOffset, lineSize );
            [[theController tintRectangle3] setFrame:frameThree];
            [[theController tintRectangle3] setHidden:NO];
    
            if( testPoint3.y > (testPoint.y + lineSize) ) {  // characters are separated by one or more lines
                CGRect frameTwo;
                frameTwo.origin.x = xOffset;
                frameTwo.origin.y = testPoint.y + lineSize;
                frameTwo.size = CGSizeMake( contentSize.width + xOffset, testPoint3.y - testPoint.y - lineSize );
                [[theController tintRectangle2] setFrame:frameTwo];
                [[theController tintRectangle2] setHidden:NO];
                }
            }
        [[theController tintRectangle1] setFrame:frameOne];
        [[theController tintRectangle1] setHidden:NO];
        }

    typingRangeIlluminated = TRUE;
    }

- (void) illuminateTintedUIView:(TintedUIView *)theView colorNumber:(NSInteger)colorNum {

    switch( colorNum ) {
        case 1: [theView setBackgroundColor:color1]; [theView setHidden:FALSE]; break;
        case 2: [theView setBackgroundColor:color2]; [theView setHidden:FALSE]; break;
        case 3: [theView setBackgroundColor:color3]; [theView setHidden:FALSE]; break;
        case 4: [theView setBackgroundColor:color4]; [theView setHidden:FALSE]; break;
        default: [theView setHidden:TRUE]; break;
        }
    }

- (void) reilluminateRange { [self illuminateRange:illuminatedRange]; }

- (void) bringUpMenuForRange:(NSRange)range {

    if( range.length == 0 )
        return;

    CGFloat lineSize = [theController currentLineSize];
    CGSize contentSize = [self contentSize];
    contentSize.width -= 2.F * xOffset;
    CGPoint testPoint = [self topLeftCornerOfCharAtIndex:range.location];
    CGPoint testPoint3 = [self topRightCornerOfCharAtIndex:(range.location + range.length - 1)];

    if( testPoint3.y == testPoint.y ) { // characters are in same line
        CGRect frameOne;
        frameOne.origin = testPoint;
        frameOne.size = CGSizeMake( testPoint3.x - testPoint.x, lineSize );
        [[UIMenuController sharedMenuController] setTargetRect:frameOne inView:self];
        [[UIMenuController sharedMenuController] setMenuVisible:YES animated:YES];
        [self setNeedsDisplayInRect:frameOne];
        }
    typingRangeIlluminated = TRUE;
    }

- (void) deilluminateRange {

    [[theController tintRectangle1] setHidden:YES];
    [[theController tintRectangle2] setHidden:YES];
    [[theController tintRectangle3] setHidden:YES];

    typingRangeIlluminated = FALSE;
    }

- (NSRange) getLastSelectedRange { return lastSelectedRange; }

- (void) setSelectedRange:(NSRange)range {

    if( (NSInteger)(range.location) >= 0 )
        lastSelectedRange = range;
    [super setSelectedRange:range];
    }

- (BOOL) isOpaque { return YES; }

- (void) illuminateCharacterNumber:(NSInteger)n at:(NSUInteger)index colorNumber:(NSInteger)colorNum {

//    NSLog(@"Trying to illuminate character at %u", index );  // DEBUG

    CGPoint testPoint1 = [self topLeftCornerOfCharAtIndex:index];
    CGPoint testPoint2 = [self topRightCornerOfCharAtIndex:index];
    CGRect frameOne;
    CGFloat lineSize = [theController currentLineSize];

//    NSLog(@"Test point 1: x %g, y %g", testPoint1.x, testPoint1.y );  // DEBUG

    frameOne.origin = testPoint1;
    frameOne.size = CGSizeMake( testPoint2.x - testPoint1.x, lineSize );

    switch( n ) {
        case 1:
            [[theController tintRectangle4] setFrame:frameOne];
            [[theController tintRectangle4] setHidden:NO];
            [[theController tintRectangle4] setIndex:index];
            [self illuminateTintedUIView:[theController tintRectangle4] colorNumber:colorNum];
            break;
        case 2:
            [[theController tintRectangle5] setFrame:frameOne];
            [[theController tintRectangle5] setHidden:NO];
            [[theController tintRectangle5] setIndex:index];
            [self illuminateTintedUIView:[theController tintRectangle5] colorNumber:colorNum];
            break;
        default:
            break;
        }
    }

- (void) illuminateCharacterNumber:(NSInteger)n at:(NSUInteger)index {

    CGPoint testPoint1 = [self topLeftCornerOfCharAtIndex:index];
    CGPoint testPoint2 = [self topRightCornerOfCharAtIndex:index];
    CGRect frameOne;
    CGFloat lineSize = [theController currentLineSize];

//    NSLog(@"Test point 1: x %g, y %g", testPoint1.x, testPoint1.y );  // DEBUG

    frameOne.origin = testPoint1;
    frameOne.size = CGSizeMake( testPoint2.x - testPoint1.x, lineSize );

    switch( n ) {
        case 1:
            [[theController tintRectangle4] setFrame:frameOne];
            [[theController tintRectangle4] setHidden:NO];
            [[theController tintRectangle4] setIndex:index];
            break;
        case 2:
            [[theController tintRectangle5] setFrame:frameOne];
            [[theController tintRectangle5] setHidden:NO];
            [[theController tintRectangle5] setIndex:index];
            break;
        default:
            break;
        }
    }

- (void) deilluminateCharacterNumber:(NSInteger)n {

    switch( n ) {
        case 1: [[theController tintRectangle4] setHidden:YES]; break;
        case 2: [[theController tintRectangle5] setHidden:YES]; break;
        default: break;
        }
    }

- (BOOL) findMatchingLeftParenInCode:(NSUInteger)here at:(NSUInteger *)foundAt {

    return [theParenFinder findMatchingLeftParenInCode:here at:foundAt];
    }

- (BOOL) findMatchingRightParenInCode:(NSUInteger)here at:(NSUInteger *)foundAt {

    return [theParenFinder findMatchingRightParenInCode:here at:foundAt inText:[self text]];
    }

- (void) displayParenMatchAt:(NSUInteger)here {

    NSString *theText = [self text];
    NSUInteger foundIndex = 0;

    [self deilluminateCharacterNumber:1];
    [self deilluminateCharacterNumber:2];

    if( [theText characterAtIndex:here] == ((unichar)(')')) ) {
        [self revalidateParensTo:here];
        if( [theParenFinder findMatchingLeftParenInCode:here at:&foundIndex] ) {
            [self illuminateCharacterNumber:1 at:foundIndex colorNumber:2];
            [self illuminateCharacterNumber:2 at:here colorNumber:2];
            }
        else
            [self illuminateCharacterNumber:2 at:here colorNumber:4];
        }
    else if( [theText characterAtIndex:here] == ((unichar)('(')) ) {
        [self revalidateParensTo:here];
        if( [theParenFinder findMatchingRightParenInCode:here at:&foundIndex inText:[self text]] ) {
            [self illuminateCharacterNumber:1 at:here colorNumber:2];
            [self illuminateCharacterNumber:2 at:foundIndex colorNumber:2];
            }
        else
            [self illuminateCharacterNumber:1 at:here colorNumber:3];
        }
    }

- (void) repositionParenMatch {

    if( ![[theController tintRectangle4] isHidden] )
        [self illuminateCharacterNumber:1 at:[[theController tintRectangle4] index]];
    if( ![[theController tintRectangle5] isHidden] )
        [self illuminateCharacterNumber:2 at:[[theController tintRectangle5] index]];
    }

- (BOOL) matchedParensDisplayed {

    return(    (! [[theController tintRectangle4] isHidden])
            && (! [[theController tintRectangle5] isHidden]) );
    }

- (void) analyzeParens {

    [theParenFinder rebuildParenStackFromCodeAt:0 toButNotIncluding:[[self text] length] inText:[self text]];

    }

- (void) invalidateParensStarting:(NSUInteger)here { [theParenFinder clearBackTo:here]; }
    
- (void) revalidateParensTo:(NSUInteger)here {

    [theParenFinder rebuildParenStackToButNotIncluding:here inText:[self text]];
//    [theParenFinder describeParenStack];  // DEBUG
    }

- (void) reanalyzeParens {

    [theParenFinder clearBackTo:0];
    [theParenFinder rebuildParenStackToButNotIncluding:[[self text] length] inText:[self text]];
    }

@end
