//
//  ParenFinder.m
//
//  Created by Jay Freeman on 8/5/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628
*/

#import "ParenFinder.h"
#import "PixieSchemeIIIViewController.h"
#import "JRF6UITextView.h"

@implementation ParenFinder

@synthesize theController;
@synthesize theMainJRFView;

- (id) init{

    self = [super init];

    parenStack = [[NSMutableArray arrayWithCapacity:256] retain];

    return self;
    }

- (void) clearBackTo:(NSUInteger)there {

    ParenTriple *last = 0;

    while( (last = (ParenTriple *)[parenStack lastObject]) != nil )
        if( [last location] >= there )
            [parenStack removeLastObject];
        else
            break;
//    [self describeParenStack];  // DEBUG
    }

- (void) describeParenStack {

    NSUInteger counter = 0;

    NSLog(@"Paren stack of length %d:", [parenStack count]);

    for( counter = 0; counter < [parenStack count]; ++counter )
        [[parenStack objectAtIndex:counter] describe];
    }

- (NSUInteger) eatCommentFrom:(NSUInteger)start inText:(NSString *)theText {

    NSUInteger lnth = [theText length];
    NSUInteger currentIndex = start;

    while( ++currentIndex < lnth )
        if( [theText characterAtIndex:currentIndex] == '\n' )
            return ++currentIndex;

    return currentIndex;
    }
    
- (NSUInteger) eatStringFrom:(NSUInteger)start inText:(NSString *)theText {

    NSUInteger lnth = [theText length];
    NSUInteger currentIndex = start;

    while( (currentIndex = [self scanSkipBackslash:currentIndex inText:theText]) < lnth ) {
        switch( [theText characterAtIndex:currentIndex] ) {
            case '"'  : return ++currentIndex; break;
            case '\n' : return ++currentIndex; break;     // This is technically a Scheme syntax error, but the user probably 
                                                          //   intends the string to terminate here, so ...
            default   : break;
            }
        }
    return currentIndex;
    }
    
- (NSInteger) endLevel {

    ParenTriple *last = (ParenTriple *)[parenStack lastObject];

    if( last == nil )
        return 0;
    else if([last isLeftParen ])
        return [last level];
    else
        return([last level] - 1);
    }

- (NSUInteger) findCodeBackwardFrom:(NSUInteger)start inText:(NSString *)theText {

    NSUInteger currentIndex = start;
    unichar c = 0;
    unichar c1 = 0;

    while( currentIndex > 0 ) {
        if(    ((c = [theText characterAtIndex:(currentIndex - 1)]) == '\n')
            && (currentIndex > 1)
            && ((c1 = [theText characterAtIndex:(currentIndex - 2)]) != '\\')
            )
            return currentIndex;        
        --currentIndex;
        }
    return 0;
    }

- (BOOL) findMatchingLeftParenInCode:(NSUInteger)here at:(NSUInteger *)foundAt {

    NSUInteger lnth = [parenStack count];
    NSUInteger counter = 0;
    ParenTriple *thisTriple = 0;

    *foundAt = 0;

    if( lnth == 0 )           // No parentheses at all.
        return FALSE;

    if( ! [self lastIndexBefore:here is:(&counter) ] ) {
//        NSLog(@"findMatchingLeftParenInCode will return false from the beginning");  // DEBUG
        return FALSE;
        }
//    NSLog(@"findMatchingLeftParenInCode:%u ... counter %u", here, counter);  // DEBUG

    NSInteger levelSought = [[parenStack objectAtIndex:counter] level];
    if( ! [[parenStack objectAtIndex:counter] isLeftParen] )
        --levelSought;

//    NSLog(@"findMatchingLeftParenInCode:%u ... level sought %d, counter %u", here, levelSought, counter);  // DEBUG

    while( TRUE ) {
//        NSLog(@"Counter is %u", counter );  // DEBUG
        thisTriple = (ParenTriple *)[parenStack objectAtIndex: counter];
//        [thisTriple describe]; // DEBUG
        if( [thisTriple isLeftParen] && ([thisTriple level] == levelSought) ) {
            *foundAt = [thisTriple location];
            return TRUE;
            }
        if( counter-- == 0 )
            break;
        }

    return FALSE;
    }

- (BOOL) findMatchingRParenFrom:(NSUInteger)start at:(NSUInteger *)whereFound inText:(NSString *)theText {

    NSUInteger lnth = [theText length];
    NSUInteger currentIndex = start;
    NSInteger parenLevel = 0;

    *whereFound = lnth;

    while( currentIndex < lnth ) {
        switch( [theText characterAtIndex:currentIndex] ) {
            case '(': ++parenLevel; currentIndex = [self scanSkipBackslash:currentIndex inText:theText]; break;
            case ')':
                if( --parenLevel == 0 ) {
                    *whereFound = currentIndex;
                    return TRUE;
                    }
                currentIndex = [self scanSkipBackslash:currentIndex inText:theText];
                break;
            case ';': currentIndex = [self eatCommentFrom:currentIndex inText:theText];    break;
            case '"': currentIndex = [self eatStringFrom:currentIndex inText:theText];     break;
            default : currentIndex = [self scanSkipBackslash:currentIndex inText:theText]; break;
            }
        }    
    return FALSE;
   }

- (BOOL) findMatchingRightParenInCode:(NSUInteger)here at:(NSUInteger *)foundAt inText:(NSString *)theText {

//    NSLog(@"findMatchingRightParenInCode:%u...", here );  // DEBUG

    NSUInteger knownInCode = [self findCodeBackwardFrom:here inText:theText];
    *foundAt = 0;

    if( [self isInCode:here withCodeStart:knownInCode inText:theText] ) {
//        NSUInteger returnValue = [self findMatchingRParenFrom:here at:foundAt inText:theText]; // DEBUG
//        NSLog(@"knownInCode %u, will return %u", knownInCode, *foundAt );  // DEBUG
//        return returnValue; // DEBUG
        return [self findMatchingRParenFrom:here at:foundAt inText:theText];
        }
    else
        return FALSE;
    }

- (BOOL) isInCode:(NSUInteger)here withCodeStart:(NSUInteger)start inText:(NSString *)theText{

    NSUInteger currentIndex = start;

    if( here == start )
        return TRUE;

    while( currentIndex < here ) {
        switch( [theText characterAtIndex:currentIndex] ) {
            case ';':
                if( (currentIndex = [self eatCommentFrom:currentIndex inText:theText]) > here )
                    return FALSE;
                break;
			case '"':
                if( (currentIndex = [self eatStringFrom:currentIndex inText:theText]) > here )
                    return FALSE;
                break;
            default :
                currentIndex = [self scanSkipBackslash:currentIndex inText:theText];
                break;
            }
        if( currentIndex == here )
            return TRUE;
        }
    return FALSE;
    }        

- (BOOL) lastIndexBefore:(NSUInteger)here is:(NSUInteger *)desiredIndex {

    NSUInteger lnth = [parenStack count];
    NSUInteger counter = 0;
    ParenTriple *thisTriple = 0;

    *desiredIndex = 0;

    if( lnth == 0 )           // No parentheses at all.
        return FALSE;

    counter = lnth - 1;

    while( TRUE ) {   // position counter just before here
        thisTriple = (ParenTriple *)[parenStack objectAtIndex: counter];
        if( [thisTriple location] < here )
            break;
        if( counter-- == 0 )
            return FALSE;
        }        
    *desiredIndex = counter;
    return TRUE;
    }

- (BOOL) lastParenBefore:(NSUInteger)here at:(NSUInteger *)foundAt inText:(NSString *)theText {

    *foundAt = 0;

    if( [parenStack count] > 0 ) {
        *foundAt = [[parenStack lastObject] location];
        return TRUE;
        }
    else 
        return FALSE;;
    }

- (void) pushLeftParenAt:(NSUInteger)here {

//    NSLog(@"pushing a left paren at %u", here );  // DEBUG

    ParenTriple *newPT = [ParenTriple alloc];

    [newPT setIsLeftParen:YES];
    [newPT setLevel:([self endLevel] + 1)];
    [newPT setLocation:here];
    [parenStack addObject:newPT];

    }

- (void) pushRightParenAt:(NSUInteger)here {

//    NSLog(@"pushing a right paren at %u", here );  // DEBUG

    ParenTriple *newPT = [ParenTriple alloc];

    [newPT setIsLeftParen:NO];
    [newPT setLevel:[self endLevel]];
    [newPT setLocation:here];
    [parenStack addObject:newPT];

    }

- (void) rebuildParenStackFromCodeAt:(NSUInteger)start toButNotIncluding:(NSUInteger)end inText:(NSString *)theText {

    NSUInteger where = start;

//    NSLog(@"rebuilding paren stack from %u to %u", start, end );  // DEBUG

    while( where < end ) {
        switch( [theText characterAtIndex:where] ) {
            case '(': [self pushLeftParenAt:where];  where = [self scanSkipBackslash:where inText:theText]; break;
            case ')': [self pushRightParenAt:where]; where = [self scanSkipBackslash:where inText:theText]; break;
            case ';': where = [self eatCommentFrom:where inText:theText];    break;
            case '"': where = [self eatStringFrom:where inText:theText];     break;
            default : where = [self scanSkipBackslash:where inText:theText]; break;
            }
        }    
//   NSLog(@"Paren stack length is %u", [parenStack count]);  // DEBUG
//   [self describeParenStack];  // DEBUG
   }

- (void) rebuildParenStackToButNotIncluding:(NSUInteger)end inText:(NSString *)theText {

    NSUInteger start = 0;

    if( [self lastParenBefore:end at:(&start) inText:theText] )
        [self rebuildParenStackFromCodeAt:(start + 1) toButNotIncluding:end inText:theText];
    else
        [self rebuildParenStackFromCodeAt:0 toButNotIncluding:end inText:theText];

    }

- (NSUInteger) scanSkipBackslash:(NSUInteger)start inText:(NSString *)theText {

    NSUInteger lnth = [theText length];

    if( ++start < lnth ) {
        if( [theText characterAtIndex:start] == '\\') {
            if( (start + 2) < lnth )
                return start + 2;
            else
                return lnth;
            }
        else
           return start;
        }
    else
        return lnth;
    }            

@end
