//
//  PixieIIISenseLightArrayController.m
//  WraithCocoa
//
//  Copyright 2010 Jay Reynolds Freeman. All rights reserved.
//
/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628
*/

#import "PixieIIISenseLightArrayController.h"
#import "PixieSchemeIIIViewController.h"

@implementation PixieIIISenseLightArrayController

- (void) initialize {

    redImage     = [[UIImage imageNamed:@"RedLight.png"] retain];
    orangeImage  = [[UIImage imageNamed:@"OrangeLight.png"] retain];
    yellowImage  = [[UIImage imageNamed:@"YellowLight.png"] retain];
    greenImage   = [[UIImage imageNamed:@"GreenLight.png"] retain];
    cyanImage    = [[UIImage imageNamed:@"CyanLight.png"] retain];
    magentaImage = [[UIImage imageNamed:@"MagentaLight.png"] retain];
    offImage     = [[UIImage imageNamed:@"OffLight.png"] retain];
    blackImage   = [[UIImage imageNamed:@"BlackLight.png"] retain];

    theView[ 0 ] = button0;
    theView[ 1 ] = button1;
    theView[ 2 ] = button2;
    theView[ 3 ] = button3;
    theView[ 4 ] = button4;
    theView[ 5 ] = button5;
    theView[ 6 ] = button6;
    theView[ 7 ] = button7;

    [self setLight:0 toVisible:false];
    [self setLight:1 toVisible:false];
    [self setLight:2 toVisible:false];
    [self setLight:3 toVisible:false];
    [self setLight:4 toVisible:false];
    [self setLight:5 toVisible:false];
    [self setLight:6 toVisible:false];
    [self setLight:7 toVisible:false];

    [self setLight:0 toIllumination:SENSE_LIGHT_OFF];
    [self setLight:1 toIllumination:SENSE_LIGHT_OFF];
    [self setLight:2 toIllumination:SENSE_LIGHT_OFF];
    [self setLight:3 toIllumination:SENSE_LIGHT_OFF];
    [self setLight:4 toIllumination:SENSE_LIGHT_OFF];
    [self setLight:5 toIllumination:SENSE_LIGHT_OFF];
    [self setLight:6 toIllumination:SENSE_LIGHT_OFF];
    [self setLight:7 toIllumination:SENSE_LIGHT_OFF];

#if FALSE   // TRUE to test or for picture-taking, FALSE not to.

    [self setLight:0 toVisible:true];  // TEST
    [self setLight:1 toVisible:true];  // TEST
    [self setLight:2 toVisible:true];  // TEST
    [self setLight:3 toVisible:true];  // TEST
    [self setLight:4 toVisible:true];  // TEST
    [self setLight:5 toVisible:true];  // TEST
    [self setLight:6 toVisible:true];  // TEST
    [self setLight:7 toVisible:true];  // TEST

    [self setLight:0 toIllumination:0];   // TEST
    [self setLight:1 toIllumination:1];   // TEST
    [self setLight:2 toIllumination:2];   // TEST
    [self setLight:3 toIllumination:3];   // TEST
    [self setLight:4 toIllumination:4];   // TEST
    [self setLight:5 toIllumination:5];   // TEST
    [self setLight:6 toIllumination:6];   // TEST
    [self setLight:7 toIllumination:7];   // TEST

//    [self setLight:5 toVisible:false]; // TEST

#endif

    }


- (void) clearSenseLights {

    [self setLight:0 toVisible:false];
    [self setLight:1 toVisible:false];
    [self setLight:2 toVisible:false];
    [self setLight:3 toVisible:false];
    [self setLight:4 toVisible:false];
    [self setLight:5 toVisible:false];
    [self setLight:6 toVisible:false];
    [self setLight:7 toVisible:false];

    [self setLight:0 toIllumination:SENSE_LIGHT_OFF];
    [self setLight:1 toIllumination:SENSE_LIGHT_OFF];
    [self setLight:2 toIllumination:SENSE_LIGHT_OFF];
    [self setLight:3 toIllumination:SENSE_LIGHT_OFF];
    [self setLight:4 toIllumination:SENSE_LIGHT_OFF];
    [self setLight:5 toIllumination:SENSE_LIGHT_OFF];
    [self setLight:6 toIllumination:SENSE_LIGHT_OFF];
    [self setLight:7 toIllumination:SENSE_LIGHT_OFF];

    }


- (void) interpretLongCommand:(LongWord)c {

    int light = 0;
    unsigned char command = 0;

    for( light = 0; light < N_SENSE_LIGHTS; ++light ) {
//        fprintf( stdout, "Interpreting command 0x%016lx for light %d, to shift by %lu\n", c, light, (light * sizeof( unsigned char )) ); fflush( stdout ); // DEBUG
        command = (unsigned char)((c >> (light * BITS_IN_BYTE)) & COMMAND_MASK);
        [self interpretOneCommand:command forLight:light];
        }
    }


- (void) interpretOneCommand:(unsigned char)c forLight:(int)light {

//    fprintf( stdout, "Interpreting command 0x%02x for light %d\n", c, light ); fflush( stdout ); // DEBUG

    if( (light < 0) || (light >= N_SENSE_LIGHTS) )
        [theSchemeController reportDisasterAndQuit:"Range error for a sense light number, while interpreting a command."];

    if( c & COMMAND_BIT_MASK ) {            //  We have a command.
        if( c & COMMAND_TYPE_BIT_MASK )     //  The command is to change the color.
            [self setLight:light toIllumination:(c & ILLUMINATION_BITS_MASK)];
        else                                    //  The command is to change the visibility.
            [self setLight:light toVisible:((c & VISIBLE_BIT_MASK) ? true : false)];
        }
    }


- (void) interpretBothCommands:(unsigned char)c forLight:(LongWord)light {

//    fprintf( stderr, "Interpreting command 0x%02x for light %lld\n", c, light ); // DEBUG
    if( (light < 0) || (light >= N_SENSE_LIGHTS) )
        [theSchemeController reportDisasterAndQuit:"Range error for a sense light number, while interpreting both commands."];
    [self setLight:(int)light toIllumination:(c & ILLUMINATION_BITS_MASK)];
    [self setLight:(int)light toVisible:((c & VISIBLE_BIT_MASK) ? true : false)];
    }


extern unsigned long packOneCommand( unsigned char isCommand, unsigned char isSetIllumination,
                                     unsigned char isVisible, UnsignedLongWord color );

extern void loadCommandForLight( long *destination, unsigned long light,
                                 unsigned char isCommand, unsigned char isSetIllumination, 
                                 unsigned char isVisible, UnsignedLongWord color );

- (void) rotateLights:(LongWord)n {

    bool oldVisible[ N_SENSE_LIGHTS ];
    int oldIllumination[ N_SENSE_LIGHTS ];
    unsigned char command     = 0;
    LongWord      thisTarget  = 0;
    LongWord      light       = 0;
    unsigned char dummy       = FALSE;

    for( light = 0; light < N_SENSE_LIGHTS; ++light ) {
        oldVisible[ light ] = visible[ light ];
        oldIllumination[ light ] = illumination[ light];
        }

    for( light = 0; light < N_SENSE_LIGHTS; ++light ) {
        thisTarget = (light + n) & ILLUMINATION_BITS_MASK;
        command = (packOneCommand( dummy, dummy, oldVisible[ thisTarget ], oldIllumination[ thisTarget ] )) & COMMAND_MASK;
        [self interpretBothCommands: command forLight: light];
        }
    
    }


- (void) setLight:(int)light toIllumination:(int)newIllumination {

    UIButton *thisView;

    if( (light < 0) || (light >= N_SENSE_LIGHTS) )
        [theSchemeController reportDisasterAndQuit:"Range error for a sense light number, while setting sense light illumination."];
    if( (newIllumination < 0) || (newIllumination >= N_SENSE_LIGHT_ILLUMINATIONS) )
        [theSchemeController reportDisasterAndQuit:"Range error for a sense light illumination, while setting sense light illumination."];

    thisView = theView[ light ];

    illumination[ light ] = newIllumination;

    if( visible[ light ] ) {
        switch( newIllumination ) {
            case SENSE_LIGHT_OFF:     [thisView setImage:offImage forState:[thisView state]];      break;
            case SENSE_LIGHT_RED:     [thisView setImage:redImage forState:[thisView state]];      break;
            case SENSE_LIGHT_ORANGE:  [thisView setImage:orangeImage forState:[thisView state]];   break;
            case SENSE_LIGHT_YELLOW:  [thisView setImage:yellowImage forState:[thisView state]];   break;
            case SENSE_LIGHT_GREEN:   [thisView setImage:greenImage forState:[thisView state]];    break;
            case SENSE_LIGHT_CYAN:    [thisView setImage:cyanImage forState:[thisView state]];     break;
            case SENSE_LIGHT_MAGENTA: [thisView setImage:magentaImage forState:[thisView state]];  break;
            case SENSE_LIGHT_BLACK:   [thisView setImage:blackImage forState:[thisView state]];    break;
            default:
                [theSchemeController reportDisasterAndQuit:"Range error for a sense light illumination, while setting sense light illumination, position 2."];
                break;
            }
        }
    }


- (void) setLight:(int)light toVisible:(bool)newVisible {

    UIButton *thisView;

    if( (light < 0) || (light >= N_SENSE_LIGHTS) )
        [theSchemeController reportDisasterAndQuit:"Range error for a sense light number, while setting sense light visibility."];

    thisView = theView[ light ];

    visible[ light ] = newVisible;

    if( newVisible ) {
        switch( illumination[ light ] ) {
            case SENSE_LIGHT_OFF:     [thisView setImage:offImage forState:[thisView state]];      break;
            case SENSE_LIGHT_RED:     [thisView setImage:redImage forState:[thisView state]];      break;
            case SENSE_LIGHT_ORANGE:  [thisView setImage:orangeImage forState:[thisView state]];   break;
            case SENSE_LIGHT_YELLOW:  [thisView setImage:yellowImage forState:[thisView state]];   break;
            case SENSE_LIGHT_GREEN:   [thisView setImage:greenImage forState:[thisView state]];    break;
            case SENSE_LIGHT_CYAN:    [thisView setImage:cyanImage forState:[thisView state]];     break;
            case SENSE_LIGHT_MAGENTA: [thisView setImage:magentaImage forState:[thisView state]];  break;
            case SENSE_LIGHT_BLACK:   [thisView setImage:blackImage forState:[thisView state]];    break;
            default:
                [theSchemeController reportDisasterAndQuit:"Range error for a sense light illumination, while setting sense light visibility."];
                break;
            }
        [thisView setHidden:NO];
        }
    else {
        [thisView setImage:nil forState:[thisView state]];
        [thisView setHidden:YES];
        }
    }

- (bool *) getState { return state; }

- (bool *) getVisible { return visible; }

- (bool) anyVisible {

    for( LongWord light = 0; light < N_SENSE_LIGHTS; ++light )
        if( visible[ light ] )
            return TRUE;
    return FALSE;
    }
  
- (int *) getIllumination { return illumination; }

- (void) setLightNFromRecordedState:(int)n {
    
    [self setLight:n toIllumination:illumination[ n ]];
    [self setLight:n toVisible:visible[ n ]];
    }

@end
