//
//  PixieSchemeIIIViewController.mm
//
//  Created by Jay Freeman on 06/13/2010.
//  Copyright 2006 Jay Reynolds Freeman.
//
/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628
*/

#import <UIKit/UIKit.h>

// #defines for initialization waits:

#define WAIT_FOR_EVALUATOR_DELAY 1000

// #defines for debugging:

#define SHOW_THREAD_INTERACTIONS FALSE      // Make TRUE to get log of thread-interaction details.

// During initialization, every so often a timer fires to attempt a controller/evaluator
// thread interaction that requires their cooperation to get it done.  This next #define 
// is the delay between such timer firings.  Changing this probably requires a 

#define STANDARD_INITIALIZATION_TIMER_INTERVAL 0.1

// The size of string passed from the controller to the evaluator is variable; the
// next #define is the length started out with.  (Old string is freed and new calloced
// when necessary.)

#define START_STRING_SIZE 512

#define PIXIE_SCHEME_III_FONT @"Palatino-Roman"
#define INITIAL_MAIN_BUFFER_SIZE 4096

#import "PixieSchemeIIIViewController.h"
#import "PixieIIIStatusLightController.h"
#import "PixieIIISenseLightArrayController.h"
#import "Pixie_Scheme_IIIAppDelegate.h"
#import "JRF6UITextView.h"
#import "UserSetup.h"
#import "HelpViewController.h"
#import "DialogController.h"
#import "FeaturesViewController.h"
#import "WebDocController.h"
#import "TextDocController.h"
#import "TintedUIView.h"

#import "Globals.e"

extern void pollControllerOnce();
extern int runSchemeWithPathAndMemory( const char *argv0, long memorySizeProvided );
extern void cleanUpForExit();
extern char *messagesFromControllerNames[];
extern char *messagesFromEvaluatorNames[];
extern void reportGUIInitialized();
extern void increaseBlocking();
extern void setFrontKitten( int aKitten, int aValue );
extern int getFrontKitten( int aKitten );
extern Boolean fatalErrorOccurred;
extern LongWord progressFractionInThousandths;
extern Boolean usingGenerationalGC;

extern const UnsignedLongWord PIXIE_SCHEMES_MEM_SIZE_MBYTES;

UIColor *color1;
UIColor *color2;

threadMessage toEvaluator;
threadMessage toController;

Word outputEnabled = 1;
Word waitingToExit = 0;
Word clowderIsOperating = 0;
Word worldIsLoaded = 0;
Word controllerReadyForMessages = 0;
Word evaluatorHasInitialized = 0;
Word GUIInitialized = 0;
Word readyToRead = 0;

extern Word keyboardSelectEvalEquivalentUsed;

extern volatile int controllerOutboundMailboxFlag;
extern Text appSupportWorldFileName[];
extern Text appSupportEditSessionName[];
extern bool isOriginalWorld;
extern bool hasAppSupportEditSession;

NSLock *controllersMailbox = nil;
NSLock *evaluatorsMailbox = nil;

PixieSchemeIIIViewController *thePixieSchemeController;

int worldFileID;

#define BUFSIZE_FOR_PATHNAME 1000

Text worldFilePathName[ BUFSIZE_FOR_PATHNAME + 1 ];

//################################################################################
//
//########  Stuff used during initialization.

Word localArgc = 0;
char *localArgv[ 12 ] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };

extern char releaseDate[];
extern int usleep(useconds_t useconds);

@implementation PixieSchemeIIIViewController

@synthesize theMainJRFView;
@synthesize dialogResponseString;
@synthesize theNavigationBar;
@synthesize theToolBar;
@synthesize myParentWindow;
@synthesize markOrEvalButton;
@synthesize featuresBarButtonItem;
@synthesize helpBarButtonItem;
@synthesize featuresButton;
@synthesize helpButton;
@synthesize resetSchemeButton;
@synthesize demonstrateSchemeButton;
@synthesize fontBiggerButton;
@synthesize fontSmallerButton;
@synthesize completeRestartButton;
@synthesize saveWorldButton;
@synthesize helpFileButton;
@synthesize dictionaryFileButton;
@synthesize warrantyFileButton;
@synthesize readmeFileButton;
@synthesize dialogPrompt;
@synthesize compileDefinesSwitch;
@synthesize showFullPrecisionSwitch;
@synthesize dialogResponse;
@synthesize dialogView;
@synthesize dialogController;
@synthesize featuresController;
@synthesize helpController;
@synthesize readmeController;
@synthesize warrantyController;
@synthesize helpFileController;
@synthesize dictionaryFileController;
@synthesize theApplication;
@synthesize myLevelIndicator;
@synthesize eVBController;
@synthesize gCBController;
@synthesize sLAController;
@synthesize memoryUsedLabel;
@synthesize memoryRangeLabel;
@synthesize tintRectangle1;
@synthesize tintRectangle2;
@synthesize tintRectangle3;
@synthesize tintRectangle4;
@synthesize tintRectangle5;
@synthesize appDelegate;

- (void) actionToSaveState {

//    NSLog(@"Application in save state routine");  // DEBUG
    [self doQuit:nil];
//    NSLog(@"Application has requested reset");  // DEBUG
    while( waitingToExit )
        [self pollEvaluatorOnce];
//    NSLog(@"Application is ready to exit");  // DEBUG
    }

// The next method would need much work if there were more than one action sheet that called it ...

- (void) actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex {

    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "%sPixie Scheme.app/PixieSchemeIII.world", InitFilePath );

    if( actionSheet == currentReportDisasterAndQuitAS ) {
        cleanUpForExit();
        exit( 1 );
        }
    else if( actionSheet == currentReportEarlyDisasterAndQuitAS ) {
        exit( 1 );
        }
    else if( actionSheet == currentRunReloadWorldAS ) {
        if( buttonIndex == 0 ) { // Forget and Restart
            [self setForEvaluating];
            [sLAController clearSenseLights];
            [self tellEvaluator:C_INPUT_READY
                       withLong:0
                     withString:[[NSString stringWithFormat: @"(c::load-world! \"%s\")\n", buffer]
                                 cStringUsingEncoding:NSASCIIStringEncoding]];
            } 
        else {                   // Cancel -- nothing to do.
            ;
            }
        [featuresPopover dismissPopoverAnimated:YES];
        }
    else if( actionSheet == currentLocalErrorAS ) {
        ;
        }
    }

- (void) changeFrontControllerTo:(UIViewController *)newFrontController {
    frontController = newFrontController;
    [self presentModalViewController:newFrontController animated:YES];
    }

- (void) cleanUpForExit {
    
    }


//########   "applicationDidFinishLaunching" is lengthy but straightforward, up till the final
//########   call, to "arrangeForcontrollerInitializationTasksForEvaluator", which see ...

- (void) applicationDidFinishLaunchingContent {

//    [[NSNotificationCenter defaultCenter]
//        addObserver: self
//           selector: @selector(keyboardDidShow:)
//               name: UIKeyboardDidShowNotification
//             object: nil];

    [[NSNotificationCenter defaultCenter]
        addObserver: self
           selector: @selector(keyboardWillShow:)
               name: UIKeyboardWillShowNotification
             object: nil];

    [[NSNotificationCenter defaultCenter]
        addObserver: self
           selector: @selector(keyboardWillHide:)
               name: UIKeyboardWillHideNotification
             object: nil];

    [[NSNotificationCenter defaultCenter]
        addObserver: self
           selector: @selector(didRedo:)
               name: NSUndoManagerDidRedoChangeNotification
             object: nil];

    [[NSNotificationCenter defaultCenter]
        addObserver: self
           selector: @selector(didUndo:)
               name: NSUndoManagerDidUndoChangeNotification
             object: nil];

//    [[NSNotificationCenter defaultCenter]
//        addObserver: self
//           selector: @selector(applicationWillTerminate:)
//               name: UIApplicationWillTerminateNotification
//             object: nil];

//    [[NSNotificationCenter defaultCenter]
//        addObserver: self
//           selector: @selector(applicationWillResignActive:)
//               name: UIApplicationWillResignActiveNotification
//             object: nil];

//    [[NSNotificationCenter defaultCenter]
//        addObserver: self
//           selector: @selector(applicationDidEnterBackground:)
//               name: UIApplicationDidEnterBackgroundNotification
//             object: nil];

    //  Start former init content.

    worldFileID = 0;

    [helpFileController setParentController:self];
    [dictionaryFileController setParentController:self];

    [helpController retain];
    [featuresController retain];

    helpPopover = [[[UIPopoverController alloc] initWithContentViewController:helpController] retain];
    featuresPopover = [[[UIPopoverController alloc] initWithContentViewController:featuresController] retain];

    black = [[UIColor blackColor] retain];
    buttonTextNormalColor = [[[helpButton titleLabel] textColor] retain];
    darkGreen     = [[UIColor colorWithRed:0.00F  green:0.75F  blue:0.00F alpha:1.00F] retain];
    paleGreen     = [[UIColor colorWithRed:0.75F  green:1.00F  blue:0.75F alpha:1.00F] retain];
    red   = [[UIColor redColor] retain];
    paleRed       = [[UIColor colorWithRed:1.00F  green:0.75F  blue:0.75F alpha:1.00F] retain];
    coolBlue      = [[fontBiggerButton currentTitleColor] retain];
    paleCoolBlue  = [[[UIColor colorWithCGColor:[coolBlue CGColor]] colorWithAlphaComponent: 0.33F] retain];
    specialYellow = [[UIColor colorWithRed:1.00F  green:1.00F  blue:0.67450980392F alpha:1.00F] retain];

    [fontBiggerButton setTitleColor:coolBlue forState:UIControlStateNormal];
    [fontBiggerButton setTitleColor:coolBlue forState:UIControlStateHighlighted];
    [fontBiggerButton setTitleColor:paleCoolBlue forState:UIControlStateDisabled];

    [fontSmallerButton setTitleColor:coolBlue forState:UIControlStateNormal];
    [fontSmallerButton setTitleColor:coolBlue forState:UIControlStateHighlighted];
    [fontSmallerButton setTitleColor:paleCoolBlue forState:UIControlStateDisabled];

    [completeRestartButton setTitleColor:red forState:UIControlStateNormal];
    [completeRestartButton setTitleColor:red forState:UIControlStateHighlighted];
    [completeRestartButton setTitleColor:paleRed forState:UIControlStateDisabled];

    [demonstrateSchemeButton setTitleColor:darkGreen forState:UIControlStateNormal];
    [demonstrateSchemeButton setTitleColor:darkGreen forState:UIControlStateHighlighted];
    [demonstrateSchemeButton setTitleColor:paleGreen forState:UIControlStateDisabled];

    [saveWorldButton setTitleColor:darkGreen forState:UIControlStateNormal];
    [saveWorldButton setTitleColor:darkGreen forState:UIControlStateHighlighted];
    [saveWorldButton setTitleColor:paleGreen forState:UIControlStateDisabled];

//    [saveWorldButton setHidden:YES];    // The plan is to remove this button completely, after we are sure we are done with it.

    [[self view]             setBackgroundColor:        specialYellow];  // view is momentarily visible when keyboard shows.
    [theMainJRFView          setBackgroundColor:        specialYellow];
    [theMainJRFView          setTextColor:              black];
    [dialogView setHidden:YES];

    mainDisplayBuffer = [[[NSMutableString alloc] initWithCapacity:INITIAL_MAIN_BUFFER_SIZE] retain];

    rangeForNextInsertion.location = 0;
    rangeForNextInsertion.length = 0;

    rangeBeingChanged.location = 0;
    rangeBeingChanged.length = 0;

    stringOfChange = nil;

    evaluating = false;
    displayingMessage = false;
    stopEvaluatingPending = false;
    gcing = false;
    outputEnabled = TRUE;
    wantsToQuit = false;
    textLastRange = NSMakeRange( 0, 0 );
    
    [theMainJRFView setFont:[UIFont fontWithName:PIXIE_SCHEME_III_FONT size:currentFontSize]];

//    applicationFont = [theMainJRFView font];
//    currentFontSize = [applicationFont pointSize];
    currentFontSize = 18.0F;
    currentLineSize = matchingLineHeight( currentFontSize );
    applicationFont = [UIFont fontWithName:PIXIE_SCHEME_III_FONT size:currentFontSize];
    [theMainJRFView setFont:[applicationFont fontWithSize:currentFontSize]];

    nextTask = 1;
    
    toEvaluator.messageID = C_NO_MESSAGE_YET;
    toEvaluator.theString = 0;
    toEvaluator.theLong = 0;

    toController.messageID = E_NO_MESSAGE_YET;
    toController.theString = 0;
    toController.theLong = 0;

    controllersMailbox = [[[NSLock alloc] init] retain];
    evaluatorsMailbox = [[[NSLock alloc] init] retain];

    srandom( (Word)(time( 0 ) & WORD_MASK));

    [JRF6UITextView _keepAtLinkTime];

    //  End of former init content.

    frontController = self;

    thePixieSchemeController = self;

    [self documentsDirectoryAndSetGlobal];

    float progress = 0.0F;

    // Set up the submenu to load recent files ...

    symbolsInPixieSchemeDictionary = [[self createCompletionArray] retain];

    currentReportDisasterAndQuitAS = 0;
    currentReportEarlyDisasterAndQuitAS = 0;
    currentRunReloadWorldAS = 0;
    currentLocalErrorAS = 0;

    [myLevelIndicator          setHidden:    NO];
    [myLevelIndicator          setProgress:  progress];
    [memoryUsedLabel           setHidden:    YES];
    [memoryRangeLabel          setHidden:    YES];

    [tintRectangle1 setHidden:YES];
    [tintRectangle2 setHidden:YES];
    [tintRectangle3 setHidden:YES];
    [tintRectangle4 setHidden:YES];
    [tintRectangle5 setHidden:YES];

    [tintRectangle4 setIndex:0];

    [tintRectangle5 setIndex:0];

//    UIMenuItem *menuItem = [[[UIMenuItem alloc] init] autorelease];
//    menuItem.title = @"Select Expression";
//    menuItem.action = @selector(selectExpression:);
//    [UIMenuController sharedMenuController].menuItems = [NSArray arrayWithObject:menuItem];

    UIMenuItem *menuItem = [[[UIMenuItem alloc] init] autorelease];
    menuItem.title = @"dummy";
    menuItem.action = @selector(doDummy:);
    [UIMenuController sharedMenuController].menuItems = [NSArray arrayWithObject:menuItem];

    [self displayAll];
    [theMainJRFView                     setDealWithText:    @selector(dealWithInputText2:)];
    [self setNormalLights];
    [sLAController initialize];
    [markOrEvalButton  setEnabled:NO];
    [markOrEvalButton setTitle:@"<None>" forState:UIControlStateDisabled];
    [markOrEvalButton setHidden:YES];
    [resetSchemeButton setEnabled:NO];
    [resetSchemeButton setTitle:@"<NONE>" forState:UIControlStateDisabled];
    [resetSchemeButton setHidden:YES];

//  [theMainJRFView resignFirstResponder];    //Hide the keyboard.
//    [theMainJRFView becomeFirstResponder];  // Show the keyboard.
    
    [theMainJRFView applicationDidFinishLaunchingContent];

//    PixieFace = [[UIImage imageNamed:@"PixieIIIFace.png"] retain];    // FIXME out for development
//    SadKitty = [[UIImage imageNamed:@"PixieIIIFaceSadKitty.png"] retain];    // FIXME out for development

    mainMemorySizeInUse         = PIXIE_SCHEMES_MEM_SIZE_MBYTES * 1048576;
    generationalMemorySizeInUse =                             1 * 1048576;

    theNumberOfKittens = 1;
    [self setForEvaluating];                    //  To protect world load.
    [self setForNotGCing];                      //  To initialize the light.

    //  And away we go ... (this next dispatches a thread that runs the evaluator).

    [NSThread detachNewThreadSelector:@selector(runEvaluator:) toTarget:self withObject:nil];

    while( ! worldIsLoaded ) {

        usleep( WAIT_FOR_EVALUATOR_DELAY );                
        if( fatalErrorOccurred ) {
            [self reportEarlyDisasterAndQuit];
            }
        }

    if( fatalErrorOccurred ) {
        [self reportEarlyDisasterAndQuit];
        }

    //  On your mark ... get set ... (this starts a series of events that must synchronize with evaluator initialization).

    // To avoid a "while( TRUE )" loop in "applicationDidFinishLaunching", we arrange to
    // fire a timer which calls a function that checks whether the evaluator has initialized, and does some tasks
    // for it if so; if not, it fires another timer to do the same thing,


    [NSTimer scheduledTimerWithTimeInterval  :  STANDARD_INITIALIZATION_TIMER_INTERVAL
                                     target  :  self
                                   selector  :  @selector(doControllerInitializationTask:)
                                   userInfo  :  nil
                                    repeats  :  NO];

//    NSLog(@"Application support directory is \"%s\"", appSupDirName );  // DEBUG
//    [self Test1:self];    // For testing fatal error handling.

    }

- (void) beginInitLevel {

    [myLevelIndicator setProgress:0.0F];
    [myLevelIndicator setHidden:NO];
    [myLevelIndicator setNeedsDisplay];
    }

// Override a method in UIResponder ...

- (BOOL)canPerformAction:(SEL)action withSender:(id)sender {

    NSRange newRange;
    NSRange selRange = [theMainJRFView selectedRange];

    if( action == @selector(doDummy:) ) {
        if(    (selRange.length == 0)
            && (selRange.location == [tintRectangle4 index]) || (selRange.location == [tintRectangle5 index] + 1)
            && (! [tintRectangle4 isHidden]) && (! [tintRectangle5 isHidden])
            ) {
            newRange.location = [tintRectangle4 index];
            newRange.length = [tintRectangle5 index] - newRange.location + 1;
            [theMainJRFView setSelectedRange:newRange];
            [theMainJRFView illuminateRange:newRange color:2];
            }
        return FALSE;
        }
    else
        return [super canPerformAction:action withSender:sender];
    }

//########  The problem here is potential deadlock if we do two "tellEvaluators" in the same timer-firing.  This is the
//########  thread that does GUI I/O, and the evaluator must call the thread that does GUI I/O in order to respond to
//########  some of the "tellEvaluator" commands.  But if there are two "tellEvaluators" in a row, the GUI I/O thread
//########  will be waiting for the second "tellEvaluator" to respond while the evaluator thread is waiting for the
//########  GUI I/O thread to perform some I/O that was consequent to the first "tellEvaluator".  The fix is to
//########  (1) *DO* *AT* *MOST* *ONE* "tellEvaluator" per timer firing -- the code following is so written -- and
//########  (2) Make sure not even to call "controllerInitializationTask" unless it is the controller's turn to use
//########  the controller/evaluator interface (guaranteed by test in "doControllerInitializationTask").

- (bool) controllerInitializationTask {

    // Rule here -- at most one "tellEvaluator" per task.
    // This code allows for as many as nine tasks, and it is easy to add more.
    // Not all task opportunities are presently used for anything.


    // Synchronize threads  --  getting past the first "if" means that the evaluator is set up for
    // kitten-to-kitten communication (the Clowder is running), and also means that the
    // GUI thread has advised the evaluator thread that the GUI is ready for messages from the 
    // evaluator.

    switch( nextTask ) {
        case 1:
            reportGUIInitialized();
            nextTask = 2;
            break;
        case 2:
            if( evaluatorHasInitialized ) {
                nextTask = 3;
                }
            break;
        case 3:
            [self tellEvaluator:C_PLEASE_USE_HIGH_PRECISION withLong:0 withString:""];
            nextTask = 4;
            break;
        case 4:
            [self tellEvaluator:C_PLEASE_COMPILE_DEFINES withLong:0 withString:""];
            nextTask = 5;
            break;
        case 5:
            if( allKittensAreActive() ) {
                nextTask = 6;
                }
            break;
        case 6:
            userSetup aSetup;
            if( ! isOriginalWorld ) {
                theWorldHandler->retrieveUserSetup( &aSetup );
                [self setControllerSetup:&aSetup];
                }
            else
                [self dealWithTextRange:NO];  // Make sure the select/eval button is off.
            nextTask = 7;
            break;
        case 7:
            [self loadEditSession];
            nextTask = 8;
            break;
        case 8:
            [theMainJRFView analyzeParens];
            nextTask = 9;
            break;
        case 9:
            readyToRead = 1;
            nextTask = 10;
            break;
        case 10:
            break;
        default:
//            [self reportEarlyDisasterAndQuit];  //  For testing.
            return YES;       // Done with initialization tasks.
        }
    return NO;                // Not done with initialization tasks.
    }    

- (CGFloat) currentFontSize { return currentFontSize; }

- (CGFloat) currentLineSize { return currentLineSize; }

- (void) dealloc {
    [theMainJRFView release];
    [super dealloc];
    }

//################################################################################
//
//########  Process text from the Input Panel:

- (bool) dealWithInputText2:(NSString *)theText {

    const char *theString = 0;

    if( [theText canBeConvertedToEncoding:NSASCIIStringEncoding] ) {
        theString = [theText cStringUsingEncoding:NSASCIIStringEncoding];
        if( theString && (strlen( theString ) > 0) ) {
            [self tellEvaluator:C_INPUT_READY withLong:0 withString:theString];
            }
        // else nothing to do
        }
    else {
        [self localError:@"\nYou have entered some non-ASCII characters. Please correct the input.\n\n"];
        }
    return true;
    }

//########  Deal with the response to a dialog:

- (bool) dealWithResponseText {

    NSString *theText = [dialogResponse text];

    const char *theString = 0;
    const char *theCString = "";
    NSInteger theLength = 0;

    if( theText == nil )
        [self tellEvaluator:C_DIALOG_RESPONSE_READY withLong:0 withString:""];
    else if( [theText canBeConvertedToEncoding:NSASCIIStringEncoding] ) {
        theString = [theText cStringUsingEncoding:NSASCIIStringEncoding];
        if( theString && (strlen( theString ) > 0 ) ) {
            [self startNormalInput];
            if( (theLength = [theText length]) > 0 ) {
                theCString = [theText cStringUsingEncoding:NSASCIIStringEncoding];
                if( theCString[ theLength - 1 ] == NEWLINE )
                    theCString = [[theText substringToIndex:(theLength - 1)] cStringUsingEncoding:NSASCIIStringEncoding];
                }
            [self tellEvaluator:C_DIALOG_RESPONSE_READY withLong:0 withString:theCString];
            }
        else
            [self tellEvaluator:C_DIALOG_RESPONSE_READY withLong:0 withString:""];
        }
    else {
        [self localError:@"\nYour response includes some non-ASCII characters.\n\nScheme is supposed to use only ASCII characters.  They are pretty much the \"regular\" keyboard characters, entered without the \"command-\" and \"option-\" modifier keys.\n\nPlease retype your response to the dialog."];
        return false;
        }

    return true;
    }

// Called by the main view whenever there might be a selection; the intent is that the
// controller will offer the user the opportunity to evaluate what is selected.

- (void) dealWithSelection:(bool)value {

    if( value ) {
        [markOrEvalButton setEnabled:YES];
        [markOrEvalButton setTitle:@"  Evaluate  " forState:UIControlStateNormal];
        [markOrEvalButton setTitleColor:darkGreen forState:UIControlStateNormal];
        [markOrEvalButton setHidden:NO];
        }
    else {
        [markOrEvalButton setEnabled:NO];
        [markOrEvalButton setTitle:@"" forState:UIControlStateDisabled];
        [markOrEvalButton setHidden:YES];
        [theMainJRFView deilluminateRange];
        }
    }

- (IBAction) demonstrateScheme:(id)sender {

    const char *demoString =
"\
(begin\
  (newline)\
\
  (display \"Starting the Pixie Scheme III demonstration program ...\\\n\")\
\
  (define (add-one-to n) (+ n 1))\
  (define (hypotenuse x y) (sqrt (+ (* x x) (* y y))))\
  (define a 3)\
  \
  (define (sleep-after-display-source) (e::usleep 1500000))\
  (define (sleep-after-user-newline) (e::usleep 500000))\
  (define (sleep-after-display-result) (e::usleep 1000000))\
  \
  (define (demonstrate s-expression)\
     (let ((result '()))\
       (newline)\
       (display s-expression)\
       (sleep-after-display-source)\
       (newline)\
       (sleep-after-user-newline)\
       (set! result (e::cons-with-continuation s-expression))\
       (display result)\
       (newline)\
       (sleep-after-display-result)\
       )\
     )\
  \
  (define (comment str)\
     (let ((us (max (* (string-length str) 50000) 2000000)))\
       (newline)\
       (display str)\
       (newline)\
       (e::usleep us)\
       )\
    )\
  \
  (define (demonstrate-by-printing str s-expression)\
     (let ((result '()))\
       (newline)\
       (display str)\
       (sleep-after-display-source)\
       (newline)\
       (sleep-after-user-newline)\
       (set! result (e::cons-with-continuation s-expression))\
       (display result)\
       (newline)\
       (sleep-after-display-result)\
       )\
     )\
  \
  (define (demonstrate-define s-expression)\
     (let ((result '()))\
       (newline)\
       (display s-expression)\
       (sleep-after-display-source)\
       (newline)\
       (sleep-after-user-newline)\
       (set! result (caadr s-expression))\
       (display result)\
       (newline)\
       (sleep-after-display-result)\
       )\
     )\
(define (demo)\
  (display \"\\\n\
###############################################\\\n\
\\\n\
                        Pixie Scheme III Demonstration\\\n\
\\\n\
###############################################\\\n\"\
    )\
(comment \"[This demo takes a few minutes.  TO STOP THE DEMO, press\\\n\
the \\\"Reset Scheme\\\" button at top left.]\")\
  (comment \"Let's look at some things you can do with Pixie Scheme III.\\\n\
(Pixie Scheme III will do the typing in the demo):\")\
  (comment\
\"If you type a number, select it, and press \\\"Evaluate\\\", Scheme will\\\n\
recognize it and print it back to you, like this:\")\
  (comment \"(Notice that this demonstration program does not actually\\\n\
push any buttons or mark any text; those are things that\\\n\
only human users get to do.)\")\
  (comment \"(Note also that if you are using a Bluetooth keyboard with\\\n\
your iPad, you may press option-' (option-single-quote or\\\n\
option-apostrophe) instead of the \\\"Evaluate\\\" button.)\")\
  (comment \"Anyway, let's see what happens when we type and\\\n\
evaluate some numbers ...\")\
  (demonstrate 2)\
  (demonstrate -3)\
  (demonstrate 1.5)\
  (comment \"Fractions work, too:\")\
  (demonstrate-by-printing \"1/2\" #i1/2)\
  (comment\
\"Pixie Scheme III can't always calculate numbers with\\\n\
perfect accuracy:\")\
  (demonstrate-by-printing \"1/3\" 1/3)\
  (comment \"It's close, though!\")\
  (comment \"Here's how you use exponents:\")\
  (demonstrate-by-printing \"1.2345e4\" 12345)\
  (comment\
\"By the way, Pixie Scheme III reacts much faster than you\\\n\
might guess by looking at this demo.  The demo has things\\\n\
slowed down a lot, so you can see what is happening.\")\
  (comment\
\"You can create variables by using \\\"define\\\".\\\n\
Let's make a variable \\\"a\\\" with a value of 3.\")\
  (demonstrate-by-printing \"(define a 3)\" ''a)\
  (comment \"Did it work?  Let's type \\\"a\\\" and see ...\")\
  (demonstrate 'a)\
  (comment \"You can add.  Here's how to add two and two.\")\
  (demonstrate '(+ 2 2))\
  (comment\
\"The function \\\"+\\\" added up the numbers, and returned\\\n\
the value 4.  Pixie Scheme III printed out that value\\\n\
for you to look at.  That's how Scheme works.\")\
  (demonstrate '(+ 3 5))\
  (comment\
\"You can call things by their names when you are providing\\\n\
arguments to functions.  We've defined \\\"a\\\" to be 3, so ...\")\
  (demonstrate '(+ a 5))\
  (comment\
\"Pixie Scheme III looked up the value of \\\"a\\\" -- which was\\\n\
three -- and passed it to \\\"+\\\" along with the \\\"5\\\".\")\
  (comment\
\"You can pass the result of a function to another function.\\\n\
Let's add up the results of two additions that way.\")\
  (demonstrate '(+ (+ 2 2) (+ 3 3)))\
  (comment\
\"Pixie Scheme III evaluated \\\"(+ 2 2)\\\" and \\\"(+ 3 3)\\\" --\\\n\
getting 4 and 6 -- and passed those results to \\\"+\\\".\")\
  (comment \"You can also subtract ...\")\
  (demonstrate '(- 5 2))\
  (comment \"... or multiply or divide ...\")\
  (demonstrate '(* 3 4))\
  (demonstrate '(/ 10 5))\
  (demonstrate '(/ 10 a))\
  (comment \"... or meet the world!\")\
  (demonstrate-by-printing \"(display \\\"Hello, world!\\\")\"\
                           '(display \"Hello, world!\"))\
  (comment\
\"The function \\\"display\\\" printed out the text all by itself,\\\n\
and returned the value \\\"#t\\\".  Pixie Scheme III printed out\\\n\
the #t.  (The \\\"#t\\\" means that the function is finished with\\\n\
its work.  \\\"#t\\\" is a symbol that Scheme uses to mean \\\"true\\\"\\\n\
or \\\"yes\\\", but in this context all it really means is \\\"all\\\n\
done\\\".)\")\
  (comment \"Here's a way to dodge the extra print-out of \\\"#t\\\":\")\
  (demonstrate-by-printing \"(display \\\"Hello, world!\\\\n\\\")\"\
                           '(display \"Hello, world!\\\n\"))\
  (comment\
\"The \\\"\\\\n\\\" in the text tells the \\\"display\\\" function to\\\n\
skip to the start of the next line right then and there.\")\
  (comment\
\"Half the fun of Scheme is writing your own functions.  You\\\n\
do that with some extra parentheses in \\\"define\\\".  Here's\\\n\
a function that adds one to a number.  The function is\\\n\
named \\\"add-one-to\\\", and it takes one argument, \\\"n\\\".\")\
  (demonstrate-define '(define (add-one-to n) (+ n 1)))\
  (comment\
\"The stuff that came after \\\"(add-one-to n)\\\" was Scheme\\\n\
instructions to tell the function what to do; namely, to\\\n\
add one to its argument.  Let's see if it works.\")\
  (demonstrate '(add-one-to 1))\
  (comment \"Looks good to me ...\")\
  (demonstrate '(add-one-to 2))\
  (demonstrate '(add-one-to (add-one-to 1)))\
  (demonstrate '(add-one-to 10))\
  (comment\
\"Here's one that finds the hypotenuse of a right triangle.\\\n\
That's the square root of the sum of the squares of the\\\n\
sides ...\")\
  (demonstrate-define '(define (hypotenuse x y) (sqrt (+ (* x x) (* y y)))))\
  (demonstrate '(hypotenuse 3 4))\
  (demonstrate '(hypotenuse 5 12))\
  (demonstrate '(hypotenuse 8 15))\
  (comment \"If you want a function like ...\")\
  (comment \"(define (save-the-world) ... )\")\
  (comment\
\"... you will have to write it yourself -- but Pixie\\\n\
Scheme III has a lot of other features and functions\\\n\
to learn about, so maybe you can!\")\
  (comment \"Pixie Scheme III can save your work between\\\n\
sessions.  Just press the \\\"Features\\\" button,\\\n\
at lower left, and then press \\\"Save World\\\".\")\
  (comment\
\"To learn more about Pixie Scheme III, use the \\\"Help\\\"\\\n\
button, at lower right, and check for updates and\\\n\
other information on my web site,\\\n\
\\\n\
       http://web.mac.com/Jay_Reynolds_Freeman\\\n\
\\\n\
The \\\"Pixie Scheme III Help\\\" file -- which you obtain\\\n\
from the top item on the list that comes up when you\\\n\
press the Pixie Scheme III Help button -- contains a list of\\\n\
books about Scheme, too.  Some of them make pretty good\\\n\
reading when you are learning the language.\")\
(newline)\
(display\
\"Or just start out.  You can scroll back to review\\\n\
this demonstration.\\\n\
\\\n\
When you are ready to type, just tap the screen\\\n\
anywhere to bring up the keyboard.  Have fun!\\\n\")\
  )\
(demo)\
)\
";
    [featuresPopover dismissPopoverAnimated:YES];
    [helpPopover dismissPopoverAnimated:YES];      // Just in case it is open.
    [self tellEvaluator:C_INPUT_READY\
               withLong:0
             withString:demoString];
    }

// Called by the main view whenever there might be a text range; the intent is that the
// controller will offer the user the opportunity to mark any text range.

- (void) dealWithTextRange:(bool)value {

    if( value ) {
        [markOrEvalButton setEnabled:YES];
        [markOrEvalButton setTitle:@"Select That?" forState:UIControlStateNormal];
        [markOrEvalButton setTitleColor:buttonTextNormalColor forState:UIControlStateNormal];
        [markOrEvalButton setHidden:NO];
        }
    else {
        [markOrEvalButton setEnabled:NO];
        [markOrEvalButton setTitle:@"" forState:UIControlStateDisabled];
        [markOrEvalButton setHidden:YES];
        [theMainJRFView deilluminateRange];
        }
    }


- (void) didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
//    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
    }

- (void)didRedo:(NSNotification *)notification {

    NSRange selRange = [theMainJRFView getSelectedRange];

    selRange.location += selRange.length;
    selRange.length = 0;
    [theMainJRFView setSelectedRange:selRange];
    [theMainJRFView startTypingRange];
    [self dealWithSelection:NO];
    }

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation {

    if(    ([self interfaceOrientation] == UIInterfaceOrientationPortrait)
        || ([self interfaceOrientation] == UIInterfaceOrientationPortraitUpsideDown)
        ) {
        [memoryUsedLabel           setHidden:    YES];
        [memoryRangeLabel          setHidden:    YES];
        }
    else {
        [memoryUsedLabel           setHidden:    NO];
        [memoryRangeLabel          setHidden:    NO];
        }
    if( theMainJRFView.typingRangeIlluminated )
        [theMainJRFView reilluminateRange];
    [theMainJRFView repositionParenMatch];
    [theMainJRFView showSelectedRange];
//    [self.view setHidden:NO];
    }

- (void)didUndo:(NSNotification *)notification {

    NSRange selRange = [theMainJRFView getSelectedRange];

    selRange.location += selRange.length;
    selRange.length = 0;
    [theMainJRFView setSelectedRange:selRange];
    [theMainJRFView startTypingRange];
    [self dealWithSelection:NO];
    }

- (void) dismissModal:(id)sender { 

    [self dismissModalViewControllerAnimated:NO];
    frontController = self;
    }

- (void) displayAll { [theMainJRFView setNeedsDisplay]; }

- (IBAction) displayDictionary:(id)sender {

    char buffer[ MAXPATHLEN + 300 ];
    sprintf( buffer, "%sPixie Scheme.app/Pixie Scheme III Help/Pixie Scheme III Dictionary.html", InitFilePath );

    [self nothing:[dictionaryFileController view]];

    NSString *thePath = [NSString stringWithUTF8String:buffer];
    [dictionaryFileController createWithLoadRequest:thePath];
    [self changeFrontControllerTo:dictionaryFileController];
    [helpPopover dismissPopoverAnimated:YES];
    [featuresPopover dismissPopoverAnimated:YES];  // Just in case the features popover is up.
    }

#define UIToolbarStandardHeight 44.0F
#define UIStatusbarStandardHeight 20.0F

- (IBAction) displayHelp:(id)sender {

    char buffer[ MAXPATHLEN + 300 ];
    sprintf( buffer, "%sPixie Scheme.app/Pixie Scheme III Help/Pixie Scheme III Help.html", InitFilePath );

    [self nothing:[helpFileController view]];

    NSString *thePath = [NSString stringWithUTF8String:buffer];
    [helpFileController createWithLoadRequest:thePath];
    [self changeFrontControllerTo:helpFileController];
    [helpPopover dismissPopoverAnimated:YES];
    [featuresPopover dismissPopoverAnimated:YES];  // Just in case the features popover is up.
    }

- (IBAction) displayReadMe:(id)sender {

    char buffer[ MAXPATHLEN + 300 ];
    sprintf( buffer, "%sPixie Scheme.app/README.txt", InitFilePath );

    [self nothing:[readmeController view]];
    NSString *thePath = [NSString stringWithUTF8String:buffer];
    NSString *desiredText = [NSString stringWithContentsOfFile:thePath encoding:NSASCIIStringEncoding error:nil];

    [readmeController createWithText:desiredText];
    [self changeFrontControllerTo:readmeController];
    [helpPopover dismissPopoverAnimated:YES];
    [featuresPopover dismissPopoverAnimated:YES];  // Just in case the features popover is up.
    }

- (void) displayStringInMainView:(NSString *)str {     // FIXME -- check for retain/release of old and new strings.

    NSString *oldString = [theMainJRFView text];
    NSRange selectionRange = [theMainJRFView selectedRange];

    if( oldString ) {
        NSString *newString = [oldString stringByReplacingCharactersInRange:selectionRange withString:str];
        [theMainJRFView setText:newString];
        }
    else
        [theMainJRFView setText:str];

    [theMainJRFView showSelectedRange];
    [theMainJRFView setNeedsDisplay];
    }

- (IBAction) displayWarranty:(id)sender {

    char buffer[ MAXPATHLEN + 300 ];
    sprintf( buffer, "%sPixie Scheme.app/GNUEtcetera.txt", InitFilePath );

    [self nothing:[warrantyController view]];
    NSString *thePath = [NSString stringWithUTF8String:buffer];
    NSString *desiredText = [NSString stringWithContentsOfFile:thePath encoding:NSASCIIStringEncoding error:nil];

    [warrantyController createWithText:desiredText];
    [self changeFrontControllerTo:warrantyController];
    [helpPopover dismissPopoverAnimated:YES];
    [featuresPopover dismissPopoverAnimated:YES];  // Just in case the features popover is up.
    }

- (NSString *) documentsDirectoryAndSetGlobal {

    NSArray *paths = NSSearchPathForDirectoriesInDomains( NSDocumentDirectory, NSUserDomainMask, YES );
    NSString *documentsDirectory = [paths objectAtIndex:0];

  extern Text appSupDirName[];

    strcpy( appSupDirName, [documentsDirectory cStringUsingEncoding:NSASCIIStringEncoding]);
//    NSLog(@"Application Support Directory name set to \"%s\"", appSupDirName );  // DEBUG

    return documentsDirectory;
    }

//########  This is the code that runs during the initialization-timer events.
//########  It runs first on the basis of "arrangeForControllerInitializationTasksForEvaluator"'s timer,
//########  then on the basis of a timer which it itself sets.

- (void) doControllerInitializationTask:(NSTimer *)theTimer {

    if( [self controllerInitializationTask] )
        ;
    else
        [NSTimer scheduledTimerWithTimeInterval  :  STANDARD_INITIALIZATION_TIMER_INTERVAL
                                         target  :  self
                                       selector  :  @selector(doControllerInitializationTask:)
                                       userInfo  :  nil
                                        repeats  :  NO];
    }

- (void) doDummy:(id)sender {;}

//################################################################################
//
//########  Stuff for dealing with controller/evaluator interactions after initialization:


void doPollEvaluatorOnce()
{
    [thePixieSchemeController performSelectorOnMainThread:@selector(pollEvaluatorOnce) withObject:nil waitUntilDone:FALSE];
    }

//################################################################################
//
//########  Stuff used during quitting and shutdown.  Includes fatal error-message handling.


- (IBAction) doQuit:(id)sender {

    waitingToExit = 1;    // Tested when evaluator reports "E_WORLD_SAVE_SUCCEEDED" and "REINITIALIZATION_COMPLETE".
    [self requestReset:nil];
    }

- (void) endInitLevel {

    [myLevelIndicator setHidden:YES];
    [myLevelIndicator setNeedsDisplay];
    }

//  Code to look up a term in the dictionary.

- (IBAction) findInPixieSchemeDictionary:(id)sender {

//  Format string for the lookup.  This *is* part of the method, just formatted in a compromising manner.

const char *scriptTemplate =
"echo $'\
tell application \"Safari\"\n\
     make new document with properties {URL:\"file://%sPixie%%20Scheme%%20III.app/Contents/Resources/English.lproj/Pixie%%20Scheme%%20III%%20Help/Pixie%%20Scheme%%20III%%20Dictionary.html#%s\"}\n\
     get source of front document\n\
     if source of front document contains \"<a name=\\\\\"%s\" then\n\
        activate\n\
    else\n\
        close front window\n\
        beep\n\
    end if\n\
end tell\
' | osascript";

//    NSResponder *currentFocusedView = [myParentWindow firstResponder];    // FIXME out for development
    UITextView *currentFocusedView = theMainJRFView;   // FIXME -- needs rewrite

    if( [currentFocusedView isEqual:theMainJRFView] ) {

        NSRange selectionRange = [currentFocusedView selectedRange];

        if( selectionRange.length > 0 ) {

            NSString *theString = [[currentFocusedView text] substringWithRange:selectionRange];
            char buffer[ MAXPATHLEN + 3000 ];

            snprintf( buffer,
                      MAXPATHLEN + 3000,
                      scriptTemplate,
                      InitFilePath,
                      [theString UTF8String],
                      [theString UTF8String]
                      );

//            NSLog(@"Command is %s\n", buffer );  // DEBUG

            if( system( buffer ) != 0 ) {
                [self localError:@"Pixie Scheme III could not find such an entry in the dictionary."];
                return;
                }

            }
//        else    // FIXME out for development
//            NSBeep();    // FIXME out for development
        }
//    else    // FIXME out for development
//        NSBeep();    // FIXME out for development
    }

/*
- (id) init 
{
    return [super init];
    }
*/

/*
// The designated initializer. Override to perform setup that is required before the view is loaded.
- (id) initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if ((self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil])) {
        // Custom initialization
        }
    return self;
    }
*/

- (BOOL) keyboardShown { return keyboardShown; }

- (void) keyboardDidShow:(NSNotification *)aNotification {
    }

// Called when the UIKeyboardWillHideNotification is sent

/**************** EXPERIMENTED with these, could not get them to work satisfactorily.

- (void)keyboardWillBeHidden:(NSNotification*)aNotification {

    UIEdgeInsets contentInsets = UIEdgeInsetsZero;
    theMainJRFView.contentInset = contentInsets;
    theMainJRFView.scrollIndicatorInsets = contentInsets;

//    [theMainJRFView scrollRangeToVisible:[theMainJRFView selectedRange]];
    }

// Called when the UIKeyboardWillShowNotification is sent.

- (void)keyboardWillBeShown:(NSNotification*)aNotification {

    NSDictionary* info = [aNotification userInfo];
    CGSize kbSize = [[info objectForKey:UIKeyboardFrameBeginUserInfoKey] CGRectValue].size;
 
    UIEdgeInsets contentInsets = UIEdgeInsetsMake( 0.0F, 0.0F, kbSize.height, 0.0F );

    theMainJRFView.contentInset = contentInsets;
    theMainJRFView.scrollIndicatorInsets = contentInsets;
 
//    [theMainJRFView scrollRangeToVisible:[theMainJRFView selectedRange]];

    CGRect aRect = [theMainJRFView frame];
    aRect.size.height -= kbSize.height;
    theMainJRFView.frame = aRect;

    [theMainJRFView scrollRangeToVisible:[theMainJRFView selectedRange]];

//    if( !CGRectContainsPoint( aRect, activeField.frame.origin ) ) {
//        CGPoint scrollPoint = CGPointMake( 0.0F, activeField.frame.origin.y-kbSize.height );
//        [theMainJRFView setContentOffset:scrollPoint animated:YES];
//        }

    }
 
**************** End of EXPERIMENT ****************/

- (void)keyboardWillHide:(NSNotification *)aNotification {
//    [self keyboardWillBeHidden:aNotification];
    [self moveTextViewForKeyboard:aNotification up:NO]; 
    }

- (void)keyboardWillShow:(NSNotification *)aNotification {
//    [self keyboardWillBeShown:aNotification];
    [self moveTextViewForKeyboard:aNotification up:YES];
    }

//######## Sheet for handling non-local errors:

- (void) localError:(NSString *)message {

   UIAlertView *alert =
     [[UIAlertView alloc]
                  initWithTitle: @"Error"
                        message: message
                       delegate: self
              cancelButtonTitle: @"Okay"
              otherButtonTitles: nil];
    if( currentLocalErrorAS != nil )
        [currentLocalErrorAS release];
    currentLocalErrorAS = [alert retain];

    [alert show]; 
    [alert release];
    }

- (void) loadEditSession  {

//    NSLog(@"Loading edit session");  // DEBUG

    userSetup aSetup;

    if( ! hasAppSupportEditSession )
        return;

//    NSString *editSession
//       = [NSMutableString
//           stringWithContentsOfFile: [NSString stringWithCString: appSupportEditSessionName encoding: NSASCIIStringEncoding]
//                           encoding: NSASCIIStringEncoding
//                              error: nil];
//    if( (editSession != nil) && ([editSession length] > 0) )
//        [theMainJRFView setText:editSession];
//
//    [theMainJRFView setSelectedRange:NSMakeRange( 0, 0 )];
//    [theMainJRFView setTypingRange:NSMakeRange( 0, 0 )];

//    NSLog(@"Will read setup from %s", appSupportEditSessionName );  // DEBUG
    theWorldHandler->readUserSetupFromFileNamed( appSupportEditSessionName );

//    NSLog(@"Will retrieve setup");  // DEBUG
    theWorldHandler->retrieveUserSetup( &aSetup );
//    NSLog(@"Will set controller setup");  // DEBUG
    [self setControllerSetup:&aSetup];
//    NSLog(@"Done loading");  // DEBUG
    }

/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void) loadView {
    }
*/

void logRect( CGRect aRect, NSString *name ) { 

    NSLog( @"Rect %@: at ( %g, %g ), size ( %g, %g )",
            name, aRect.origin.x, aRect.origin.y, aRect.size.width, aRect.size.height );
    }

- (IBAction) makeFontBigger:(id)sender {

    CGFloat newFontSize = nextLargestFontSize( currentFontSize );
    CGFloat newLineSize = matchingLineHeight( newFontSize );

    currentFontSize = newFontSize;
    currentLineSize = newLineSize;

    [self setFontsToSize:newFontSize ];
    [theMainJRFView showSelectedRange];

    [theMainJRFView repositionParenMatch];
    
    if( currentFontSize >= 30. )
        [fontBiggerButton setEnabled:NO];
    else
        [fontBiggerButton setEnabled:YES];
    if( currentFontSize >= 9.5 )
        [fontSmallerButton setEnabled:YES];
    else
        [fontSmallerButton setEnabled:NO];
    }

- (IBAction) makeFontSmaller:(id)sender {

    CGFloat newFontSize = nextSmallestFontSize( currentFontSize );
    CGFloat newLineSize = matchingLineHeight( newFontSize );

    currentFontSize = newFontSize;
    currentLineSize = newLineSize;

    [self setFontsToSize:newFontSize ];
    [theMainJRFView showSelectedRange];

    [theMainJRFView repositionParenMatch];

    if( currentFontSize <= 30. )
        [fontBiggerButton setEnabled:YES];
    else
        [fontBiggerButton setEnabled:NO];
    if( currentFontSize <= 9.5 )
        [fontSmallerButton setEnabled:NO];
    else
        [fontSmallerButton setEnabled:YES];
    }

CGFloat matchingLineHeight( CGFloat fontSize ) {
    if( fontSize < 9.5F )
        return 11.F;
    else if( fontSize < 11.F )
        return 12.F;
    else if( fontSize < 13.F )
        return 14.F;
    else if( fontSize < 16.F )
        return 16.F;
    else if( fontSize < 21.F )
        return 20.F;
    else if( fontSize < 30.F )
        return 27.F;
    else
        return 40.F;
    }

- (void) moveTextViewForKeyboard:(NSNotification*)aNotification up:(BOOL)up {

    NSDictionary* userInfo = [aNotification userInfo];

    // Get animation info from userInfo

    NSTimeInterval animationDuration;
    UIViewAnimationCurve animationCurve;

    CGRect keyboardEndFrame;
    CGRect barFrame = [theToolBar frame];

    [[userInfo objectForKey:UIKeyboardAnimationCurveUserInfoKey] getValue:&animationCurve];
    [[userInfo objectForKey:UIKeyboardAnimationDurationUserInfoKey] getValue:&animationDuration];

    [[userInfo objectForKey:UIKeyboardFrameEndUserInfoKey] getValue:&keyboardEndFrame];

    // Animate up or down

    [UIView beginAnimations:nil context:nil];
    [UIView setAnimationDuration:animationDuration];
    [UIView setAnimationCurve:animationCurve];

    CGRect newFrame = theMainJRFView.frame;
    CGRect keyboardFrame = [self.view convertRect:keyboardEndFrame toView:nil];

//  logRect( newFrame, @"theMainJRFViewFrame as given" );  // DEBUG
//  logRect( keyboardFrame, @"keyboardFrame as given" );  // DEBUG
//  logRect( barFrame, @"barFrame as given" );  // DEBUG

    newFrame.size.height -= keyboardFrame.size.height * (up ? 1 : -1);
    newFrame.size.height = newFrame.size.height + barFrame.size.height * (up ? 1 : -1);
        
    theMainJRFView.frame = newFrame;
//  logRect( newFrame, @"theMainJRFViewFrame as modified" );  // DEBUG

    [UIView commitAnimations];
    }

//########  Wrapper to set a new string value in the given view.

- (void) newViewString:(const char *)newString forView:(UITextField *)theView {

    [theView setText:[NSString stringWithCString:newString encoding:NSASCIIStringEncoding]];
    [theView setNeedsDisplay];
    }

CGFloat nextLargestFontSize( CGFloat fontSize ) {
    if( fontSize < 9.5F )
        return 10.F;
    else if( fontSize < 11.F )
        return 12.F;
    else if( fontSize < 13.F )
        return 14.F;
    else if( fontSize < 16.F )
        return 18.F;
    else if( fontSize < 21.F )
        return 24.F;
    else
        return 36.F;
    }
    
CGFloat nextSmallestFontSize( CGFloat fontSize ) {
    if( fontSize > 30.F )
        return 24.F;
    else if( fontSize > 21.F )
        return 18.F;
    else if( fontSize > 16.F )
        return 14.F;
    else if( fontSize > 13.F )
        return 12.F;
    else if( fontSize > 11.F )
        return 10.F;
    else
        return 9.F;
    }

- (void) nothing:(id)thing {}

//######## Opening the Pixie Scheme III dictionary.

- (IBAction) openDictionaryFile:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme.app/Contents/Resources/English.lproj/Pixie Scheme III Help/Pixie Scheme III Dictionary.html\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme III could not open the Pixie Scheme III Dictionary with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

//######## Opening GNU General Public License information.

- (IBAction) openGNU:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme.app/Contents/Resources/English.lproj/GNUEtcetera.txt\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme III could not open the GNU warranty and license file with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

//######## Opening the Pixie Scheme III help file in a browser.

- (IBAction) openHelpFile:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme.app/Contents/Resources/English.lproj/Pixie Scheme III Help/Pixie Scheme III Help.html\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme III could not open the Pixie Scheme III Help File with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

//######## Opening the Pixie Scheme III README file.

- (IBAction) openReadMe:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme.app/Contents/Resources/English.lproj/README.txt\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme III could not open the Pixie Scheme III \"README.txt\" file with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

//########  Opening various files of source code that are built-in to Pixie Scheme III.

//########  Easy to add more.  I could probably store the file name to be loaded in connection
//########  with the menu item that accesses code, and write one procedure to do it all.

- (IBAction) openSourceCode1:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme.app/Contents/Resources/English.lproj/SampleSourceCode/Checkbook.txt\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme III could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

- (IBAction) openSourceCode2:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme.app/Contents/Resources/English.lproj/SampleSourceCode/FunctionalProgramming.txt\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme III could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

- (IBAction) openSourceCode3:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme.app/Contents/Resources/English.lproj/SampleSourceCode/ByAnyOtherName.txt\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme III could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

- (IBAction) openSourceCode4:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme.app/Contents/Resources/English.lproj/SampleSourceCode/ShellEmulator.txt\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme III could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

- (IBAction) openSourceCode5:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme.app/Contents/Resources/Demo.txt\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme III could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

- (IBAction) openSourceCode6:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme.app/Contents/Resources/English.lproj/SampleSourceCode/MMapDemo.c++\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme III could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

- (IBAction) openSourceCode7:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme.app/Contents/Resources/English.lproj/SampleSourceCode/MMapInterruptDemo.c++\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme III could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }


- (IBAction) openSourceCode8:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme.app/Contents/Resources/English.lproj/SampleSourceCode/SocketClient.c++\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme III could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }


- (IBAction) openSourceCode9:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme.app/Contents/Resources/English.lproj/SampleSourceCode/SocketServer.c++\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme III could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }


- (IBAction) openSourceCode10:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sPixie Scheme.app/Contents/Resources/English.lproj/SampleSourceCode/SenseLightDemonstrations.txt\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Pixie Scheme III could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

//########  Get just one request from the evaluator, and act on it:  Critical-section-access and lock slime, and a big switch.

extern volatile int evaluatorOutboundMailboxFlag;

- (bool) pollEvaluatorOnce {

    Word newMessage;
    LongWord newLong;
    NSString *newNSString;
    bool returnValue;

    OSMemoryBarrier();     // Defensive programming

    if( evaluatorOutboundMailboxFlag == MAILBOX_FULL ) {

        newMessage  = toController.messageID;
        newLong     = toController.theLong;
        newNSString = [[NSString stringWithCString:toController.theString encoding:NSASCIIStringEncoding] copy];
     
        switch( newMessage ) {

            // System administration.

            case E_NO_MESSAGE_YET: break;                   // Initialization state; evaluator has not sent any messages.
            case E_COMPLETED_INITIALIZATION:
                [myParentWindow setNeedsDisplay];
                break;
            case E_HELLO_THERE: break;                      // No action required.
            case E_FATAL_PROBLEM:
                [self reportDisasterAndQuit:toController.theString];
                break;
            case E_PLEASE_TIDY_UP_FOR_EXIT: break;
            case E_TIDIED_UP: break;
            case E_COGNITO_ERGO_SUM: break;
            case E_MAKE_FRONT: break;                       // No action required.
            case E_NEW_OBLIST_SYMBOL:
//                [symbolsInObList addObject:newNSString];
                break;
            case E_CLEAR_OBLIST_SYMBOLS:
//                [symbolsInObList removeAllObjects];
                break;

            // Simple state transitions.

            case E_EVALUATING:                 
                [self setForEvaluating];
                break;
            case E_NOT_EVALUATING:
                [self setForNotEvaluating];
                break;
            case E_ACKNOWLEDGE_RESET:
                [self setForNotEvaluating];
                [self setForNotGCing];
                if( waitingToExit ) {
                    [self setForEvaluating];
                    [self provideUserSetup];
                    [self performSelectorOnMainThread:@selector(postMakeFinalWorld) withObject:nil waitUntilDone:FALSE];
                    }
                break;
            case E_GARBAGE_COLLECTING:         [self setForGCing];               break;
            case E_NOT_GARBAGE_COLLECTING:     [self setForNotGCing];            break;

            case E_SLEEPING:                   /* [SBController turnOn];  */           break;
            case E_WAKED_UP:                   /* [SBController turnOff]; */           break;
            case E_COMPILING_DEFINES:
                [compileDefinesSwitch setOn:YES];
                break;
            case E_NOT_COMPILING_DEFINES:
                [compileDefinesSwitch setOn:NO];
                break;
            case E_USING_LOW_PRECISION:
                [showFullPrecisionSwitch setOn:NO];
                break;
            case E_USING_HIGH_PRECISION:
                [showFullPrecisionSwitch setOn:YES];
                break;

            // Routine I/O.

            case E_OUTPUT_READY: 
                if( outputEnabled ) {
                    rangeForNextInsertion = [theMainJRFView replaceCharactersInRange:rangeForNextInsertion withString:newNSString];
//                    [self displayStringInMainView:newNSString];
                    }
                break;
            case E_MESSAGE_TO_DISPLAY:
                messageDisplayCounter = 0;
                displayingMessage = true;
//                [self setMessageViewText:newNSString];
                break;
            case E_DIALOG_TO_START:
                if( [newNSString length] > 0 ) {
                    [self startDialog:newNSString];
                    }
                break;
            case E_FILE_LOAD_BEGINNING:
                break;
            case E_FILE_LOAD_SUCCEEDED:
                break;
            case E_FILE_LOAD_FAILED:
                break;
            case E_WORLD_LOAD_SUCCEEDED:
                [self worldLoadSucceeded:[newNSString cStringUsingEncoding:NSASCIIStringEncoding]];
                break;
            case E_WORLD_LOAD_FAILED:    break;    // Nothing to do.
            case E_WORLD_SAVE_SUCCEEDED:
//                NSLog(@"World save succeeded");  // DEBUG
                if( waitingToExit ) {
                    cleanUpForExit();
                    waitingToExit = 0;
//                    exit( 0 );
                    }
                break;
            case E_WORLD_SAVE_FAILED:
//                NSLog(@"World save failed");  // DEBUG
                if( waitingToExit ) {
                    cleanUpForExit();
                    waitingToExit = 0;
//                    exit( 0 );
                    }
                break;

            // Input parsing.

            case E_EXPECT_MORE_INPUT:
                expectMoreInput = true;
                [self setForForgettingTyping];
                break;

            case E_DO_NOT_EXPECT_MORE_INPUT:
                expectMoreInput = false;
                break;

            case E_EXPECT_PARENS:
                [self setForForgettingTyping];
                break;

            case E_EXPECT_DOUBLE_QUOTE:
                [self setForForgettingTyping];
                break;

            case E_DISCARDED_INPUT: break;

            // Status information, possibly for display.

            case E_NEW_GC_COUNT:               /* [self newViewNumber:newLong forView:    NumberOfGCsView             ] */;   break;
            case E_NEW_GENERATIONAL_GC_COUNT:  /* [self newViewNumber:newLong forView:    NumberOfGenerationalGCsView ] */;   break;
            case E_MEMORY_SIZE:                /* [self newViewNumber:newLong forView:    MemorySizeView              ] */;   break;
            case E_MEMORY_IN_USE:              /* [self newViewNumber:newLong forView:    MemoryInUseView             ] */;   break;
            case E_GENERATIONAL_MEMORY_SIZE:
                                  /* [self newViewParenthesizedNumber:newLong forView:    GenerationalMemorySizeView  ] */;   break;
            case E_GENERATIONAL_MEMORY_IN_USE:
                                  /* [self newViewParenthesizedNumber:newLong forView:    GenerationalMemoryInUseView ] */;   break;
            case E_LIST_PRINT_LENGTH:          /* [self newViewNumber:newLong forView:    ListPrintLengthView         ] */;   break;
            case E_LIST_PRINT_DEPTH:           /* [self newViewNumber:newLong forView:    ListPrintDepthView          ] */;   break;
            case E_VECTOR_PRINT_LENGTH:        /* [self newViewNumber:newLong forView:    VectorPrintLengthView       ] */;   break;
            case E_VECTOR_PRINT_DEPTH:         /* [self newViewNumber:newLong forView:    VectorPrintDepthView        ] */;   break;

            case E_SENSE_LIGHT_COMMAND:        [sLAController interpretLongCommand:    newLong                     ];   break;
            case E_SENSE_LIGHT_ROTATE:         [sLAController rotateLights:            newLong                     ];   break;
            case E_TOTAL_MEMORY_IN_USE:
              [self updateLevel:(float)(((float)newLong)/ ((float)( 40 * MEGABYTE)))];
              break;

            // Level indicator manipulation.

            case E_INIT_PROGRESS:        /* [self beginInitLevel] */;                      break;
            case E_INCREMENT_PROGRESS:   /* [self updateLevel:( newLong / 1000F)] */;      break;
            case E_WRAPUP_PROGRESS:      /* [self endInitLevel] */;                        break;

            default:
                [self reportDisasterAndQuit:
                          [[NSString stringWithFormat:@"Controller got unknown message, number %d.", newMessage]
                              cStringUsingEncoding:NSASCIIStringEncoding]];
                break;
            }
        returnValue = TRUE;

        [newNSString release];     // TEST

        OSMemoryBarrier();     // Defensive programming
        evaluatorOutboundMailboxFlag = MAILBOX_EMPTY;
        }
    else
        returnValue = FALSE;

    return returnValue;
                
    }

- (void) postMakeFinalWorld {
    [self tellEvaluator:C_INPUT_READY
               withLong:0
             withString:[[NSString stringWithFormat: @"(c::save-world \"%s\")\n", appSupportWorldFileName]
                               cStringUsingEncoding:NSASCIIStringEncoding]];
    }

- (IBAction) presentFeaturesPopover:(id)sender {

    [self nothing:[featuresController view]];

    [helpPopover dismissPopoverAnimated:YES];  // Remove this line to allow both popovers to be visible at once.
    [featuresPopover
        presentPopoverFromBarButtonItem: featuresBarButtonItem
               permittedArrowDirections: UIPopoverArrowDirectionAny
                               animated: YES];
    }

- (IBAction) presentHelpPopover:(id)sender {

    [self nothing:[helpController view]];

    [featuresPopover dismissPopoverAnimated:YES];  // Remove this line to allow both popovers to be visible at once.
    [helpPopover
         presentPopoverFromBarButtonItem:helpBarButtonItem
                permittedArrowDirections:UIPopoverArrowDirectionAny
                                animated:YES];
    }

- (void) provideUserSetup {

    int counter = 0;

    bool *senseLightState = [sLAController getState];
    bool *senseLightVisible = [sLAController getVisible];
    int *senseLightIllumination = [sLAController getIllumination];

    NSRange selectedRange = [theMainJRFView getLastSelectedRange];

//    NSLog(@"Selected range on save is %u %u", selectedRange.location, selectedRange.length );  // DEBUG

    CGPoint scrollerOrigin = [theMainJRFView contentOffset];

    userSetup aSetup;
    CGRect windowRect = [myParentWindow frame];

    aSetup.compilerOn          = [compileDefinesSwitch isEnabled];
    aSetup.showsFullPrecision  = [showFullPrecisionSwitch isEnabled];
    aSetup.fontSize            = [self currentFontSize];

    aSetup.mainWindowX         = windowRect.origin.x;
    aSetup.mainWindowY         = windowRect.origin.y;
    aSetup.mainWindowWidth     = windowRect.size.width;
    aSetup.mainWindowHeight    = windowRect.size.height;

    aSetup.scrollPointX        = scrollerOrigin.x;
    aSetup.scrollPointY        = scrollerOrigin.y;

    for( counter = 0; counter < N_SENSE_LIGHTS; ++counter ) {
        aSetup.state[ counter ] = senseLightState[ counter ];
        aSetup.visible[ counter ] = senseLightVisible[ counter ];
        aSetup.illumination[ counter ] = senseLightIllumination[ counter ];
        }

    aSetup.selectionLocation = selectedRange.location;    
    aSetup.selectionLength   = selectedRange.length;    

    theWorldHandler->setUserSetup( &aSetup );

    theWorldHandler->setTheString( (Text *)[[theMainJRFView text] UTF8String] );
    }

//################################################################################
//
//########  Resetting:

- (IBAction) requestReset:(id)sender {

    if( evaluatorHasInitialized && (! stopEvaluatingPending ) ) {
        [resetSchemeButton     setEnabled:NO];
        [resetSchemeButton  setTitle:@"Reset Scheme" forState:UIControlStateDisabled];
        [resetSchemeButton setHidden:NO];

        // The next two lines deal with the possibility that we reset when a dialog is in progress.

        [dialogResponse resignFirstResponder];
        [[dialogController view] setHidden:YES];

//        [[resetSchemeButton  titleLabel] setText:@"Reset Scheme"];
        stopEvaluatingPending = true;
        [self tellEvaluator:C_PLEASE_STOP_EVALUATING withLong:0 withString:""];
//        [self        setPrompt:    @"Pixie Scheme III may ask questions here ..."];
//        [self      setResponse:    @"... and will expect you to enter your answers here."];
        [self startNormalInput];
        }
    }

//######## Reloading the default world:

- (IBAction) runReloadWorldSheet:(id)sender {

   UIActionSheet *action =
      [[UIActionSheet alloc]
//                  initWithTitle: @"Do you really want to\n\nFORGET ALL YOUR WORK\n\nand restart Pixie Scheme III?\n  "
                  initWithTitle: @" Do you really want to                     FORGET ALL YOUR WORK                and restart Pixie Scheme III?                (You cannot undo this action.)"
                       delegate: self
              cancelButtonTitle: @"Cancel"
         destructiveButtonTitle: @"Do It"
              otherButtonTitles: nil];
    if( currentRunReloadWorldAS != nil )
        [currentRunReloadWorldAS release];
    currentRunReloadWorldAS = [action retain];

    [action setActionSheetStyle:UIActionSheetStyleDefault];
    [action showInView:theMainJRFView];
    [action release];
    }

//######## Sheet display for reporting fatal error.

- (void) reportDisasterAndQuit:(const char *)errorMessage {

//   NSLog(@"Pixie Scheme III must quit because: %s\n", errorMessage );  // DEBUG

   UIActionSheet *action =

     [[UIActionSheet alloc]
                  initWithTitle: [NSString stringWithFormat:@"Pixie Scheme III must quit because: %s\n", errorMessage]
                       delegate: self
              cancelButtonTitle: nil
         destructiveButtonTitle: nil
              otherButtonTitles: nil];

    [action setActionSheetStyle:UIActionSheetStyleDefault];
    if( currentReportDisasterAndQuitAS != nil )
        [currentReportDisasterAndQuitAS release];

    currentReportDisasterAndQuitAS = [action retain];

    [action showInView:theMainJRFView];
    [action release];
    }

//######## Sheet display for reporting fatal error in early initialization.

- (void) reportEarlyDisasterAndQuit {

   UIActionSheet *action =
      [[UIActionSheet alloc]
                  initWithTitle: @"Pixie Scheme III must quit because of an implementation error while starting up.  Sorry!!\n"
                       delegate: self
              cancelButtonTitle: nil
         destructiveButtonTitle: nil
              otherButtonTitles: nil];

    [action setActionSheetStyle:UIActionSheetStyleDefault];
    if( currentReportEarlyDisasterAndQuitAS != nil )
        [currentReportEarlyDisasterAndQuitAS release];
    currentReportEarlyDisasterAndQuitAS = [action retain];

    [action showInView:[self view]];
    [action release];
    }

//########  Run the evaluator, in a thread dispatched for that purpose:

extern char *globalArgvZero;

- (void) runEvaluator:(id)theObject {

    NSAutoreleasePool *thePool = [[NSAutoreleasePool alloc] init];

    runSchemeWithPathAndMemory( globalArgvZero, mainMemorySizeInUse, generationalMemorySizeInUse );  // Never returns ...

    [thePool release];  // It never returns, but just in case ...

    }
    
- (void) saveEditSession {

//    [[NSFileManager defaultManager]
//      createFileAtPath:[NSString stringWithCString: appSupportEditSessionName encoding: NSASCIIStringEncoding]
//              contents:[NSData dataWithBytes: [[theMainJRFView text] cStringUsingEncoding: NSASCIIStringEncoding]
//                                      length: [[theMainJRFView text] length]]
//            attributes:nil];

//    NSLog(@"Saving edit session");  // DEBUG
    [self provideUserSetup];
//    NSLog(@"Provided user setup");  // DEBUG
//    NSLog(@"Will try to save to \"%s\"", appSupportEditSessionName );  // DEBUG
    
    theWorldHandler->saveUserSetupToFileNamed( appSupportEditSessionName );

//    NSLog(@"Looks like the save worked");  // DEBUG
    }

- (IBAction) saveWorld:(id)sender {
    [self provideUserSetup];
    [self tellEvaluator:C_INPUT_READY
               withLong:0
             withString:[[NSString stringWithFormat: @"(c::save-world \"%s\")\n", appSupportWorldFileName]
                               cStringUsingEncoding:NSASCIIStringEncoding]];
    [helpPopover dismissPopoverAnimated:YES];      // Just in case it is open.
    [featuresPopover dismissPopoverAnimated:YES];
    }

- (void) saveWorldForEnd {

    evaluatorOutboundMailboxFlag = MAILBOX_EMPTY;
    [self tellEvaluator:C_PLEASE_STOP_EVALUATING withLong:0 withString:""];
    [self provideUserSetup];
    evaluatorOutboundMailboxFlag = MAILBOX_EMPTY;
    waitingToExit = TRUE;
    [self tellEvaluator:C_INPUT_READY
               withLong:0
             withString:[[NSString stringWithFormat: @"(c::save-world \"%s\")\n", appSupportWorldFileName]
                               cStringUsingEncoding:NSASCIIStringEncoding]];
    evaluatorOutboundMailboxFlag = MAILBOX_EMPTY;
    while( waitingToExit )
        [self pollEvaluatorOnce];
    }

- (void) setDefaultControllerSetup {
    }

- (void) selectExpression:(id)sender {
    
    NSRange newRange;

    newRange.location = [tintRectangle4 index];
    newRange.length = [tintRectangle5 index] - newRange.location + 1;
    [tintRectangle4 setHidden:YES];
    [tintRectangle5 setHidden:YES];
    [theMainJRFView select:sender];
    [theMainJRFView setSelectedRange:newRange];

    currentLineSize = matchingLineHeight( currentFontSize );

//    [theMainJRFView select:sender];
//    [[UIMenuController sharedMenuController] update];
//    [[UIMenuController sharedMenuController] setMenuVisible:YES animated:YES];
//    [[UIMenuController sharedMenuController] setTargetRect:CGRectMake( 100, 200, 30, 90 ) inView:theMainJRFView];
    }

- (void) setControllerSetup:(userSetup *)aSetup {

    int counter = 0;

    bool *senseLightState = [sLAController getState];
    bool *senseLightVisible = [sLAController getVisible];
    int *senseLightIllumination = [sLAController getIllumination];

    CGPoint scrollerOrigin;
    NSRange selectedRange;

    [self setCurrentFontSize:(float)(aSetup->fontSize)];
    currentLineSize = matchingLineHeight( currentFontSize );

    [theMainJRFView setText:[NSString stringWithCString:theWorldHandler->getTheString() encoding:NSASCIIStringEncoding]];
    theWorldHandler->freeTheString();

    scrollerOrigin.x = (float)(aSetup->scrollPointX);
    scrollerOrigin.y = (float)(aSetup->scrollPointY);
//    [[theMainJRFView superview] scrollPoint:scrollerOrigin];  // Does this work?  No...

    selectedRange.location = (float)(aSetup->selectionLocation);
    selectedRange.length   = (float)(aSetup->selectionLength);

    [theMainJRFView setSelectedRange:selectedRange];

    for( counter = 0; counter < N_SENSE_LIGHTS; ++counter ) {
        senseLightState[ counter ] = aSetup->state[ counter ];
        senseLightVisible[ counter ] = aSetup->visible[ counter ];
        senseLightIllumination[ counter ] = aSetup->illumination[ counter ];
        [sLAController setLightNFromRecordedState:counter];
        }

//    NSLog(@"Selected range %u %u", selectedRange.location, selectedRange.length );  // DEBUG
//
//    [theMainJRFView scrollRangeToVisible:selectedRange];
    if( [theMainJRFView hasSelectedText] ) {
        [self dealWithSelection:YES];
        [theMainJRFView setTypingRangeIsActive:NO];
        }
    }

- (void) setCurrentFontSize:(CGFloat)fontSize {
    currentFontSize = fontSize;
    [self setFontsToSize:fontSize];
    }

- (void) setFontsToSize:(CGFloat)fontSize {
    [theMainJRFView setFont:[applicationFont fontWithSize:fontSize]];
    }

- (void) setForEvaluating {

    evaluating = TRUE;

    [resetSchemeButton  setEnabled:YES];    
    [resetSchemeButton  setTitle:@"Reset Scheme" forState:UIControlStateNormal];
    [resetSchemeButton setHidden:NO];
    [compileDefinesSwitch         setEnabled:NO];
    [showFullPrecisionSwitch      setEnabled:NO];
    [completeRestartButton        setEnabled:NO];
    [demonstrateSchemeButton      setEnabled:NO];
    [saveWorldButton              setEnabled:NO];
    [eVBController      turnOn];     
//    [theMainJRFView           setEditable:NO];
    if( ! keyboardSelectEvalEquivalentUsed )
        [theMainJRFView resignFirstResponder];
    }

- (void) setForForgettingTyping {

    [resetSchemeButton  setEnabled:YES];
    [resetSchemeButton  setTitle:@"Forget Typing" forState:UIControlStateNormal];
    [resetSchemeButton setHidden:NO];
//    [[resetSchemeButton  titleLabel] setText:@"Forget Typing"];
    }

- (void) setForGCing {

    gcing = TRUE;

    [resetSchemeButton  setEnabled:YES];
    [resetSchemeButton  setTitle:@"Reset Scheme" forState:UIControlStateNormal];
    [resetSchemeButton setHidden:NO];
    [compileDefinesSwitch         setEnabled:NO];
    [showFullPrecisionSwitch      setEnabled:NO];
    [gCBController      turnOn];     
    }

- (void) setForNotEvaluating {

    evaluating = FALSE;

    stopEvaluatingPending = false;

    [resetSchemeButton  setEnabled:NO];
    [resetSchemeButton  setTitle:@"" forState:UIControlStateDisabled];
    [resetSchemeButton setHidden:YES];
    [compileDefinesSwitch         setEnabled:YES];
    [showFullPrecisionSwitch      setEnabled:YES];
    [completeRestartButton        setEnabled:YES];
    [saveWorldButton              setEnabled:YES];
    [demonstrateSchemeButton      setEnabled:YES];
    [eVBController      turnOff];     
//    [eVBController      turnOn];     // DEBUG (picture-taking)
//    [gCBController      turnOn];     // DEBUG (picture-taking)
    [theMainJRFView           setEditable:YES];
//    [theMainJRFView becomeFirstResponder];
    [theMainJRFView showSelectedRange];
    }

- (void) setForNotGCing {

    gcing = FALSE;

    [gCBController       turnOff];     

    if( evaluating )
        [self setForEvaluating];
    else
        [self setForNotEvaluating];
    }

- (void) setMainViewText:(NSString *)msg {
 
    [theMainJRFView setText:msg];
    [theMainJRFView setFont:[UIFont fontWithName:PIXIE_SCHEME_III_FONT size:currentFontSize]];
    }

- (void) setRangeForNextInsertion:(NSRange)newSelectionRange { rangeForNextInsertion = newSelectionRange; }

//########  Stuff to set metadata for saved worlds.

const OSType PixieSchemeIIICreatorCode      = 'cld6';  // FIXME
const OSType PixieSchemeIIIDocumentTypeCode = 'kdl6';  // FIXME

void setWorldAttributesForPath( const char *path ) 
{
    NSMutableDictionary *fileAttributes = [NSMutableDictionary dictionaryWithCapacity: 2];

    [fileAttributes
        setObject: [NSNumber numberWithUnsignedInt:PixieSchemeIIICreatorCode]
           forKey: NSFileHFSCreatorCode];

    [fileAttributes
        setObject: [NSNumber numberWithUnsignedInt:PixieSchemeIIIDocumentTypeCode]
           forKey: NSFileHFSTypeCode];

    [[NSFileManager defaultManager]
       setAttributes: fileAttributes
        ofItemAtPath: [NSString stringWithCString:path encoding:NSASCIIStringEncoding]
               error: 0];

//    [[NSFileManager defaultManager]
//       changeFileAttributes: fileAttributes
//                     atPath: [NSString stringWithCString:path encoding:NSASCIIStringEncoding]];
    }

- (void) setNormalLights {

    [eVBController setOnImage:[UIImage imageNamed:@"MagentaLight.png"]];
    [eVBController setOffImage:[UIImage imageNamed:@"OffLight.png"]];
    [gCBController setOnImage:[UIImage imageNamed:@"RedLight.png"]];
    [gCBController setOffImage:[UIImage imageNamed:@"OffLight.png"]];
    }

- (NSArray *)spellingCompletionsForPartialWord:(NSString *)partWord {

    NSPredicate *aPredicate = [NSPredicate predicateWithFormat:@"SELF beginswith[c] %@", partWord ];

//    return [symbolsInObList filteredArrayUsingPredicate:aPredicate];
    return [symbolsInPixieSchemeDictionary filteredArrayUsingPredicate:aPredicate];
    }

// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return YES;
    }

//########  Get a dialog going:

- (void) startDialog:(NSString *)thePrompt {

    [dialogPrompt setText:thePrompt];
    [dialogView setHidden:NO];
    [dialogResponse becomeFirstResponder];
    }

//########  Return to getting input after a dialog.  (Also used in resetting.)

- (void) startNormalInput {
    }

//########  Send a request to the evaluator.

- (void) tellEvaluator:(Word)mID withLong:(LongWord)mLong withString:(char *)mString; {

    static char *stringToPass = 0;
    static Word currentLength = 0;
    Word newBufferSize = 0;

    OSMemoryBarrier();     // Defensive programming

    LongWord lengthNeeded = strlen( mString ) + 1;

    if( lengthNeeded > (currentLength - 1) ) {  //  Need a bigger hammer ... er, buffer.
        if( currentLength != 0 )
            free( stringToPass );
        for( newBufferSize = START_STRING_SIZE; newBufferSize < lengthNeeded; newBufferSize = newBufferSize * 2 )
             ;
        stringToPass = (Text *)calloc( newBufferSize, sizeof( char ) );
        if( stringToPass == 0 )
            [self reportDisasterAndQuit:"Cannot allocate memory for a communication buffer."];
        currentLength = newBufferSize;
        }

#if SHOW_THREAD_INTERACTIONS
    NSLog( @"Kitten %d: C: tellEvaluator called with message \"%s\".\n",      // DEBUG
           theKittenNumber, messagesFromControllerNames[ mID ] );            // DEBUG
#endif

    if( ! evaluatorHasInitialized )
        return;

#if SHOW_THREAD_INTERACTIONS
    NSLog( @"Kitten %d: C: tellEvaluator attempting critical section with message \"%s\".\n",      // DEBUG
           theKittenNumber, messagesFromControllerNames[ mID ] );                                 // DEBUG
#endif

    while( controllerOutboundMailboxFlag == MAILBOX_FULL )
        ;

#if SHOW_THREAD_INTERACTIONS
    NSLog( @"Kitten %d: C: tellEvaluator has the evaluator's mailbox with message \"%s\".\n",      // DEBUG
           theKittenNumber, messagesFromControllerNames[ mID ] );                                 // DEBUG
#endif

    if( mString != 0 )
        strncpy( stringToPass, mString, currentLength - 1 );
    else
        stringToPass[0] = 0;
    toEvaluator.messageID = mID;
    toEvaluator.theLong   = mLong;
    toEvaluator.theString = stringToPass;
 
    controllerOutboundMailboxFlag = MAILBOX_FULL;

    OSMemoryBarrier();     // Defensive programming

#if SHOW_THREAD_INTERACTIONS
    NSLog( @"Kitten %d: C: tellEvaluator left critical section with message  \"%s\".\n",      // DEBUG
           theKittenNumber, messagesFromControllerNames[ mID ] );                            // DEBUG
#endif
    }
    
- (void) textViewDidChange:(UITextView *)textView {

    NSUInteger foundIndex = 0;

    [theMainJRFView invalidateParensStarting:rangeBeingChanged.location];

    if( stringOfChange == nil )
        return;

    [theMainJRFView deilluminateCharacterNumber:1];
    [theMainJRFView deilluminateCharacterNumber:2];

    if( [stringOfChange isEqualToString:@")"] ) {
//        NSLog(@"stringOfChange is right paren");  // DEBUG
        [theMainJRFView revalidateParensTo:rangeBeingChanged.location];
        if( [theMainJRFView findMatchingLeftParenInCode:rangeBeingChanged.location at:&foundIndex] ) {
            [theMainJRFView illuminateCharacterNumber:2 at:rangeBeingChanged.location colorNumber:2];
            [theMainJRFView illuminateCharacterNumber:1 at:foundIndex colorNumber:2];
            }
        else
            [theMainJRFView illuminateCharacterNumber:1 at:rangeBeingChanged.location colorNumber:4];
        }
    else if( [stringOfChange isEqualToString:@"("] ) {
        [theMainJRFView revalidateParensTo:rangeBeingChanged.location];
        if( [theMainJRFView findMatchingRightParenInCode:rangeBeingChanged.location at:&foundIndex] ) {
            [theMainJRFView illuminateCharacterNumber:1 at:rangeBeingChanged.location colorNumber:2];
            [theMainJRFView illuminateCharacterNumber:2 at:foundIndex colorNumber:2];
            }
        else
            [theMainJRFView illuminateCharacterNumber:1 at:rangeBeingChanged.location colorNumber:3];
        }
    else if( ([stringOfChange length] == 0) && (rangeBeingChanged.length == 1) ) {  // Delete ...
        NSRange selRange = [theMainJRFView getSelectedRange];
        if( ([theMainJRFView text] != nil) && (selRange.length == 0) && (selRange.location > 0) )
            [theMainJRFView displayParenMatchAt:(selRange.location - 1)];
        }

    [stringOfChange release];
    stringOfChange = nil;

    }

- (void) textViewDidChangeSelection:(UITextView *)textView {

//    NSLog(@"textViewDidChangeSelection starting");  // DEBUG entry/exit
//    NSLog(@"selectionRange %u %u", [theMainJRFView getSelectedRange].location, [theMainJRFView getSelectedRange].length ); // DEBUG
//    [theMainJRFView logTypingRangeDescription];  // DEBUG

    [theMainJRFView deilluminateCharacterNumber:1];
    [theMainJRFView deilluminateCharacterNumber:2];
    if( [theMainJRFView hasSelectedText] ) {
        [self dealWithSelection:YES];
        [theMainJRFView setTypingRangeIsActive:NO];
        }
    else if( [theMainJRFView typingRangeIsActive] ) {
        NSRange selRange = [theMainJRFView getSelectedRange];
        NSRange typRange = [theMainJRFView getTypingRange];
        if( (typRange.location <= selRange.location) && (selRange.location <= (typRange.location + typRange.length)) ) {
            ;   // We can move the cursor around, or delete, as long as we stay inside the typing range.
            }
        else {
            [theMainJRFView setTypingRangeIsActive:NO];
            [self dealWithTextRange:NO];
            }
        }
    else
        [self dealWithTextRange:NO];


    if( [theMainJRFView selectedRange].location == 2147483647 )  // Hack to adjust typing position when keyboard reappears
        [theMainJRFView setSelectedRange:rangeForNextInsertion]; //   after output from Scheme engine.

    [theMainJRFView showSelectedRange];

    NSRange selRange = [theMainJRFView getSelectedRange];
    if( ([theMainJRFView text] != nil) && (selRange.length == 0) && (selRange.location > 0) )
        [theMainJRFView displayParenMatchAt:(selRange.location - 1)];

//    [theMainJRFView logTypingRangeDescription];  // DEBUG
//    NSLog(@"textViewDidChangeSelection ending");  // DEBUG entry/exit
    }

- (void) textViewDidBeginEditing:(UITextView *)textView {

//    NSLog(@"textViewDidBeginEditing starting");  // DEBUG entry/exit
//    [theMainJRFView logTypingRangeDescription];  // DEBUG

    [theMainJRFView showSelectedRange];
//    [theMainJRFView logTypingRangeDescription];  // DEBUG
//    NSLog(@"textViewDidBeginEditing ending");  // DEBUG entry/exit
    }

- (void) textViewDidEndEditing:(UITextView *)textView {

//    NSLog(@"textViewDidEndEditing starting");  // DEBUG entry/exit
//    [theMainJRFView logTypingRangeDescription];  // DEBUG

    [self dealWithTextRange:[theMainJRFView typingRangeIsActive]];
    [theMainJRFView showSelectedRange];
//    [theMainJRFView logTypingRangeDescription];  // DEBUG
//    NSLog(@"textViewDidEndEditing ending");  // DEBUG entry/exit
    }

- (BOOL) textView:(UITextView *)textView shouldChangeTextInRange:(NSRange)range replacementText:(NSString *)text {

//    NSLog(@"textView:shouldChangeTextInRange:replacementText starting");  // DEBUG entry/exit
//    NSLog(@"selectionRange %u %u", [theMainJRFView getSelectedRange].location, [theMainJRFView getSelectedRange].length ); // DEBUG
//    [theMainJRFView logTypingRangeDescription];  // DEBUG

    rangeBeingChanged = range;
    if( stringOfChange != nil )
        [stringOfChange release];
    stringOfChange = text;
    if( stringOfChange != nil )
        [stringOfChange retain];

    // The next two lines automagically disable the "." shortcut.

    if([text isEqualToString:@". "])
        return NO;

    if( [text isEqualToString:@"æ"] ) {                       // "Select That?"/"Evaluate" keyboard command.
        [theMainJRFView markOrTransmitSelection:self];
        return NO;
        }

    if(   [text isEqualToString:@"ß"]
        ||[text isEqualToString:@"Í"]
        ) {                                                   // "Save World" keyboard command.
        [self saveWorld:self];
        return NO;
        }

    // As of iPhone simulator 3.2.2, whenever I type certain characters at the end of a line (' ' and ')' at least), this
    // method gets called *twice* with the same range and that same character, but only one character actually gets inserted
    // (or maybe they both do, and one overrides the other).  The next few lines are an attempt to hack around the problem.

    if(    (range.location == textLastRange.location)
        && (range.length == textLastRange.length)
        && ([text length] == 1)
        && ([[theMainJRFView text] length] > range.location)
        && ([[theMainJRFView text] characterAtIndex:range.location] == 0x0a)
        ) {
        return YES;
        }
    
    // It appears that this code is *only* called with printable characters and "delete".

    bool isDelete = (([text length] == 0) && (range.length == 1));

    if( (! [theMainJRFView typingRangeIsActive]) && (! isDelete) )
        [theMainJRFView startTypingRange];
    [theMainJRFView adjustTypingRange:range forLength:[text length] isDelete:isDelete];
    
    [self dealWithTextRange:[theMainJRFView typingRangeIsActive]];

    textLastRange.location = range.location;   // Part of fix above.
    textLastRange.length   = range.length;     // Part of fix above.

//    NSLog(@"textView:shouldChangeTextInRange:replacementText ending");  // DEBUG entry/exit
    return YES;
    }

- (BOOL) textViewShouldEndEditing:(UITextView *)textView { return YES; }

- (void) touchesBeganInView:(UIEvent *)theEvent {

//    NSLog(@"touchesBeganInView starting");  // DEBUG entry/exit
//    [theMainJRFView logTypingRangeDescription];  // DEBUG

    if( [theMainJRFView hasSelectedText] ) {
        [self dealWithSelection:YES];
        [theMainJRFView setTypingRangeIsActive:NO];
        }
    else if( [theMainJRFView typingRangeIsActive] ) {
        [self dealWithTextRange:YES];
        [theMainJRFView setTypingRangeIsActive:NO];
        }
    else
        [self dealWithTextRange:NO];

//    [theMainJRFView logTypingRangeDescription];  // DEBUG
//    NSLog(@"touchesBeganInView ending");  // DEBUG entry/exit
    }


//################################################################################
//
//########  Mode changes:  Toggling this and that.

//########  Toggle compile-defines flag.

- (IBAction) toggleCompileDefines:(id)sender {
    [self tellEvaluator:([sender isOn] ? C_PLEASE_COMPILE_DEFINES : C_PLEASE_DO_NOT_COMPILE_DEFINES ) withLong:0 withString:""];
    }
    
//########  Toggle use-full-numeric-precision flag.

- (IBAction) toggleFullNumericPrecision:(id)sender {
    [self tellEvaluator:([sender isOn] ? C_PLEASE_USE_HIGH_PRECISION : C_PLEASE_USE_LOW_PRECISION ) withLong:0 withString:""];
    }

- (void) updateLevel:( float )newValue {

    [myLevelIndicator setProgress:newValue];
    [myLevelIndicator setHidden:NO];
    [myLevelIndicator setNeedsDisplay];
    }


// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void) viewDidLoad {
/*
    [super viewDidLoad];
    UIView *inputAccessoryView = [[UIView alloc] initWithFrame:CGRectMake( 0.0F, 0.0F, 768.0F, 71.0F )];
    inputAccessoryView.backgroundColor = [UIColor darkGrayColor];
    UIButton *testButton1 = [UIButton buttonWithType:UIButtonTypeRoundedRect];
    UIButton *testButton2 = [UIButton buttonWithType:UIButtonTypeRoundedRect];
    testButton1.frame = CGRectMake( 6.0F, 7.0F, 57.0F, 57.0F );
    testButton2.frame = CGRectMake( 76.0F, 7.0F, 57.0F, 57.0F );
    [testButton1 setTitle:@"(" forState:UIControlStateNormal];
    [testButton2 setTitle:@")" forState:UIControlStateNormal];
    [testButton1 addTarget:self action:@selector(takeInputFromTitle:) forControlEvents:UIControlEventTouchUpInside];
    [testButton2 addTarget:self action:@selector(takeInputFromTitle:) forControlEvents:UIControlEventTouchUpInside];
    [inputAccessoryView addSubview:testButton1];
    [inputAccessoryView addSubview:testButton2];
    theMainJRFView.inputAccessoryView = inputAccessoryView;
*/
    }

- (void) takeInputFromTitle:(id)sender {
    
    [self               textView:theMainJRFView 
         shouldChangeTextInRange:theMainJRFView.selectedRange
                 replacementText:((UIButton *)sender).currentTitle];
    theMainJRFView.text = [theMainJRFView.text stringByReplacingCharactersInRange:theMainJRFView.selectedRange
                                                                       withString:((UIButton *)sender).currentTitle];
    }

- (void) viewDidUnload {
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
    }

- (void) viewWillAppear:(BOOL)animated {
/*
    [[NSNotificationCenter defaultCenter]
        addObserver: self
           selector: @selector(keyboardDidShow:)
               name: UIKeyboardDidShowNotification
             object: nil];

    [[NSNotificationCenter defaultCenter]
        addObserver: self
           selector: @selector(keyboardWillHide:)
               name: UIKeyboardWillHideNotification
             object: nil];

    [[NSNotificationCenter defaultCenter]
        addObserver: self
           selector: @selector(applicationWillTerminate:)
               name: UIApplicationWillTerminateNotification
             object: nil];

    [[NSNotificationCenter defaultCenter]
        addObserver: self
           selector: @selector(applicationWillResignActive:)
               name: UIApplicationWillResignActiveNotification
             object: nil];

    [[NSNotificationCenter defaultCenter]
        addObserver: self
           selector: @selector(applicationDidEnterBackground:)
               name: UIApplicationDidEnterBackgroundNotification
             object: nil];
*/
    }

//######## Loading a world: Succeeded

- (void) worldLoadSucceeded:(const char *)path {
    }

- (NSArray *)createCompletionArray {

    return [NSArray arrayWithObjects:
        @"*",
        @"+",
        @"-",
        @"/",
        @"<",
        @"<=",
        @"=",
        @">",
        @">***<",
        @">**<",
        @">*<",
        @">+++<",
        @">++<",
        @">+<",
        @">-<",
        @">=",
        @"abs",
        @"acos",
        @"and",
        @"append",
        @"apply",
        @"asin",
        @"assoc",
        @"assq",
        @"assv",
        @"atan",
        @"begin",
        @"boolean?",
        @"call-with-current-continuation",
        @"call-with-values",
        @"car",
        @"cdr",
        @"caar",
        @"cadr",
        @"cdar",
        @"cddr",
        @"caaar",
        @"caadr",
        @"cadar",
        @"caddr",
        @"cdaar",
        @"cdadr",
        @"cddar",
        @"cdddr",
        @"caaaar",
        @"caaadr",
        @"caadar",
        @"caaddr",
        @"cadaar",
        @"cadadr",
        @"caddar",
        @"cadddr",
        @"cdaaar",
        @"cdaadr",
        @"cdadar",
        @"cdaddr",
        @"cddaar",
        @"cddadr",
        @"cdddar",
        @"cddddr",
        @"case",
        @"ceiling",
        @"char->integer",
        @"char-alphabetic?",
        @"char-ci<=?",
        @"char-ci&lt;?",
        @"char-ci=?",
        @"char-ci>=?",
        @"char-ci>?",
        @"char-downcase",
        @"char-lower-case?",
        @"char-numeric?",
        @"char-ready?",
        @"char-upcase",
        @"char-upper-case?",
        @"char-whitespace?",
        @"char<=?",
        @"char&lt;?",
        @"char=?",
        @"char>=?",
        @"char>?",
        @"char?",
        @"complex?",
        @"cond",
        @"cons",
        @"cos",
        @"current-input-port",
        @"current-output-port",
        @"define",
        @"define-syntax",
        @"delay",
        @"display",
        @"do",
        @"dynamic-wind",
        @"else",
        @"eof-object?",
        @"eq?",
        @"equal?",
        @"eqv?",
        @"eval",
        @"even?",
        @"exact->inexact",
        @"exact?",
        @"exp",
        @"expt",
        @"floor",
        @"for-each",
        @"force",
        @"gcd",
        @"if",
        @"inexact->exact",
        @"inexact?",
        @"input-port?",
        @"integer->char",
        @"integer?",
        @"interaction-environment",
        @"lambda",
        @"lcm",
        @"length",
        @"let",
        @"let-syntax",
        @"let*",
        @"letrec",
        @"letrec-syntax",
        @"list",
        @"list->string",
        @"list->vector",
        @"list-ref",
        @"list-tail",
        @"list?",
        @"log",
        @"make-string",
        @"make-vector",
        @"map",
        @"max",
        @"member",
        @"memq",
        @"memv",
        @"min",
        @"modulo",
        @"negative?",
        @"newline",
        @"not",
        @"null-environment",
        @"null?",
        @"number->string",
        @"number?",
        @"odd?",
        @"or",
        @"output-port?",
        @"pair?",
        @"peek-char",
        @"port?",
        @"positive?",
        @"procedure?",
        @"quasiquote",
        @"quote",
        @"quotient",
        @"rational?",
        @"read",
        @"read-char",
        @"real?",
        @"remainder",
        @"reverse",
        @"round",
        @"scheme-report-environment",
        @"set!",
        @"set-car!",
        @"set-cdr!",
        @"sin",
        @"sqrt",
        @"string->list",
        @"string->number",
        @"string->symbol",
        @"string-append",
        @"string-ci<=?",
        @"string-ci&lt;?",
        @"string-ci=?",
        @"string-ci>=?",
        @"string-ci>?",
        @"string-copy",
        @"string-fill!",
        @"string-length",
        @"string-ref",
        @"string-set!",
        @"string<=?",
        @"string<?",
        @"string=?",
        @"string>=?",
        @"string>?",
        @"string?",
        @"substring",
        @"symbol->string",
        @"symbol?",
        @"syntax-rules",
        @"tan",
        @"truncate",
        @"unquote",
        @"unquote-splicing",
        @"values",
        @"vector",
        @"vector->list",
        @"vector-fill!",
        @"vector-length",
        @"vector-ref",
        @"vector-set!",
        @"vector?",
        @"write",
        @"write-char",
        @"zero?",
        @"e::active-room",
        @"e::active-store-size",
        @"e::aging-room",
        @"e::all?",
        @"e::any?",
        @"e::atom?",
        @"e::bit-and",
        @"e::bit-not",
        @"e::bit-or",
        @"e::bit-shift-left",
        @"e::bit-shift-right-arithmetic",
        @"e::bit-shift-right-logical",
        @"e::bit-xor",
        @"e::block-any-binding",
        @"e::block-symbol-binding",
        @"e::bound-instance?",
        @"e::c-boolean->scheme-boolean?",
        @"e::clear-compiler-on!",
        @"e::clear-permanent!",
        @"e::clear-show-full-precision!",
        @"e::closed-port?",
        @"e::compiler-on?",
        @"e::compile-form",
        @"e::cons-with-continuation",
        @"e::debug",
        @"e::deep-copy",
        @"e::define-no-compile",
        @"e::entropy-death",
        @"e::error",
        @"e::error-port",
        @"e::exit",
        @"e::expand-macro",
        @"e::expand-macros-recursively",
        @"e::fixnum?",
        @"e::float?",
        @"e::forced?",
        @"e::forgettable?",
        @"e::forgettable-expiration-time",
        @"e::forgettable-forgotten?",
        @"e::forgettable-object",
        @"e::forgettable-remembered?",
        @"e::forgettable-remember-probability",
        @"e::forgettable-unexpired?",
        @"e::full-gc",
        @"e::gensym",
        @"e::get-tag",
        @"e::hide-sense-lights!",
        @"e::inf?",
        @"e::inspect",
        @"e::list-print-depth",
        @"e::list-print-length",
        @"e::set-list-print-depth!",
        @"e::set-list-print-length!",
        @"e::set-vector-print-depth!",
        @"e::set-vector-print-length!",
        @"e::vector-print-depth",
        @"e::vector-print-length",
        @"e::logic-constant?",
        @"e::macro",
        @"e::macro-body",
        @"e::macro?",
        @"e::main",
        @"e::make-forgettable",
        @"e::make-simple-class",
        @"e::money->string",
        @"e::multiple-values?",
        @"e::nan?",
        @"e::ninety-percent-used?",
        @"e::non-printing-object?",
        @"e::not-null?",
        @"e::number->string-with-n-decimals",
        @"e::original-cwcc",
        @"e::perform-applicative",
        @"e::permanent?",
        @"e::promise?",
        @"e::random",
        @"e::read-string-with-prompt",
        @"e::reset",
        @"e::room",
        @"e::rotate-sense-lights!",
        @"e::sense-light-number?",
        @"e::sense-light-illumination-number?",
        @"e::set-sense-light-illuminations!",
        @"e::set-sense-light-n-illumination!",
        @"e::set-sense-light-n-visibility!",
        @"e::set-sense-lights-off!",
        @"e::set-compiler-on!",
        @"e::set-forgettable-expiration-time!",
        @"e::set-forgettable-remember-probability!",
        @"e::set-permanent!",
        @"e::set-show-full-precision!",
        @"e::set-tag!",
        @"e::show-active-memory",
        @"e::show-aging-memory",
        @"e::show-dynamic-environment-list",
        @"e::show-environment",
        @"e::show-environment-list",
        @"e::show-full-precision?",
        @"e::show-locks",
        @"e::show-memory",
        @"e::show-room",
        @"e::show-sense-lights!",
        @"e::srandom",
        @"e::stack-depth",
        @"e::store-size",
        @"e::substitute-new-permanent-symbols",
        @"e::time",
        @"e::top-loop-code",
        @"e::usleep",
        @"e::version",
        @"e::warning",
        @"e::write-continuation",
        @"e::write-stack",
        @"c::block",
        @"c::compiling?",
        @"c::define",
        @"c::down-in-flames!!!",
        @"c::environment",
        @"c::hash-string",
        @"c::let-deadlock-time-out",
        @"c::load-from-string",
        @"c::number->string-with-c-format",
        @"c::release-block",
        @"c::release-locks",
        @"c::set-blocked-binding!",
        @"c::symeval",
        nil
        ];
    }

@end

