;; Copyright (c) 2009 by Jay Reynolds Freeman

;;    This program is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    This program is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;
;;    Contact information:
;;
;;        EMail:       Jay_Reynolds_Freeman@mac.com
;;        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman
;;
;;        Paper Mail:  Jay Reynolds Freeman
;;                     Post Office Box 60628
;;                     Palo Alto, CA, 94306-0628

(e::set-list-print-length! 100000)
(e::set-list-print-depth! 100000)
(e::set-vector-print-length! 100000)
(e::set-vector-print-depth! 100000)

(c::define macroexpand-expressions
  (lambda (inport outport)
    (let ((next-sexp (read inport)))
      (if (eof-object? next-sexp)
        (begin
          (close-input-port inport)
          (close-output-port outport))
        (begin
          (write (expand-and-redefine-if-cde next-sexp) outport)
          (newline outport)
          (newline outport)
          (macroexpand-expressions inport outport))))))

(c::define macroexpand-file
  (lambda (infile outfile)
    (display "Macroexpanding file \"") (display infile) (display "\"\n")
    (let
      ((inport (open-input-file infile))
       (outport (open-output-file outfile)))
      (macroexpand-expressions inport outport))))

(c::define expand-and-redefine-if-cde
  (lambda (sexp)  ;; "CDE" == c::define
    (if (and (list? sexp)
             (>= (length sexp) 3)
             (equal? (car sexp) 'c::define)
             (symbol? (cadr sexp)))
      (expand-and-redefine sexp)
      sexp)))

(c::define error-check
  (lambda (cde)
    (if (not (list? cde))
      (begin
        (display cde (e::error-port))
        (display " is not a list"." (e::error-port))
        (e::error "Unknown expression type.")))
    (if (< (length cde) 3)
      (begin
        (display cde (e::error-port))
        (display " is not a list of three or more items"." (e::error-port))
        (e::error "Unknown expression type.")))
    (if (not (equal? (car cde) 'c::define))
      (begin
        (display "Car of expression " (e::error-port))
        (display cde (e::error-port))
        (display " is not the symbol \"c::define\"." (e::error-port))
        (e::error "Unknown expression type.")))
    (if (not (symbol? (cadr cde)))
      (begin
        (display "Cadr of expression " (e::error-port))
        (display cde (e::error-port))
        (display " is not a symbol." (e::error-port))
        (e::error "Unknown expression type.")))
    #t))

(c::define expand-and-redefine
  (lambda (cde)
;    (display (cadr cde)) (newline)  ;; DEBUG
    (let*
      ((recde (e::expand-macros-recursively (cddr cde)))
       (new-recde
         (subst-many-in-sexp
           (l-gensym-pairs
             (uniquify-symbols-list
               (list-gensyms recde '())
               '())
             0)
           recde))
       )
      (cons (car cde) (cons (cadr cde) new-recde)))))

(c::define all-numbers?
  (lambda (list-of-letters)
    (cond
      ((null? list-of-letters) #t)
      ((and (member (car list-of-letters) '(#\0 #\1 #\2 #\3 #\4 #\5 #\6 #\7 #\8 #\9)) (all-numbers? (cdr list-of-letters))) #t)
      (#t #f))))
        
(c::define list-gensyms
  (lambda (sexp answer)
    (cond ((null? sexp) answer)
          ((pair? sexp) (list-gensyms (cdr sexp) (list-gensyms (car sexp) answer)))
          ((vector? sexp) (list-gensyms (vector->list sexp) answer))
          ((symbol? sexp)
            (let ((letters (string->list (symbol->string sexp))))
              (if (and (equal? (car letters) #\g)
                       (>= (length letters) 5)
                       (member (cadr letters) '(#\0 #\1 #\2 #\3 #\4 #\5 #\6 #\7 #\8 #\9))
                       (all-numbers? (cdr letters)))
                (cons sexp answer)
                answer)))
        (#t answer))))

(c::define uniquify-symbols-list
  (lambda (lyst answer)
    (cond ((null? lyst) answer)
          ((member (car lyst) answer) (uniquify-symbols-list (cdr lyst) answer))
          (#t (uniquify-symbols-list (cdr lyst) (cons (car lyst) answer))))))

(c::define l-gensym-pairs
  (lambda (lyst n)
    (if (null? lyst)
      '()
      (cons (list (car lyst) (string->symbol (string-append "lambda-var-" (number->string n))))
            (l-gensym-pairs (cdr lyst) (+ n 1))))))

(c::define subst-in-sexp
  (lambda (new old sexp)
    ;;
    ;; Substitute the variable "new" for the variable "old" in "sexp"
    ;;
    (cond ((list? sexp) (map (lambda (x) (subst-in-sexp new old x)) sexp))
          ((pair? sexp) (cons (subst-in-sexp new old (car sexp)) (subst-in-sexp new old (cdr sexp))))
          ((vector? sexp) (list->vector (map (lambda (x) (subst-in-sexp new old x)) (vector->list sexp))))
          ((equal? sexp old) new)
          (#t sexp))))

(c::define subst-many-in-sexp
  (lambda (old-new-pair-lists sexp)
    ;;
    ;; Cdr down the two lists of variables, "news" and "olds", in parallel,
    ;; using "subst-in-sexp" to do variable substitution with each
    ;; pair of items from the two lists just mentioned, in turn, into the body.
    ;;
    (if (null? old-new-pair-lists)
       sexp 
       (subst-many-in-sexp (cdr old-new-pair-lists) (subst-in-sexp (cadar old-new-pair-lists) (caar old-new-pair-lists) sexp)))))

(display "\nCompiling ...\n\n")

(c::define macroexpand-expressions (e::compile-form macroexpand-expressions))

(c::define macroexpand-file (e::compile-form macroexpand-file))

(c::define expand-and-redefine-if-cde (e::compile-form expand-and-redefine-if-cde))

(c::define error-check (e::compile-form error-check))

(c::define expand-and-redefine (e::compile-form expand-and-redefine))

(c::define all-numbers? (e::compile-form all-numbers?))

(c::define list-gensyms (e::compile-form list-gensyms))

(c::define uniquify-symbols-list (e::compile-form uniquify-symbols-list))

(c::define l-gensym-pairs (e::compile-form l-gensym-pairs))

(c::define subst-in-sexp (e::compile-form subst-in-sexp))

(c::define subst-many-in-sexp (e::compile-form subst-many-in-sexp))

(macroexpand-file
  "/Users/JayFreeman/developer/Scheme/WraithScheme.64/SchemeSource/LowLevelMacros.s"
  "/Users/JayFreeman/developer/Scheme/WraithScheme.64/SchemeSource/LowLevelMacros.Expanded.s")

(macroexpand-file
  "/Users/JayFreeman/developer/Scheme/WraithScheme.64/SchemeSource/CompileForms.s"
  "/Users/JayFreeman/developer/Scheme/WraithScheme.64/SchemeSource/CompileForms.Expanded.s")

(macroexpand-file
  "/Users/JayFreeman/developer/Scheme/WraithScheme.64/SchemeSource/RecursiveMacroExpand.s"
  "/Users/JayFreeman/developer/Scheme/WraithScheme.64/SchemeSource/RecursiveMacroExpand.Expanded.s")

(macroexpand-file
  "/Users/JayFreeman/developer/Scheme/WraithScheme.64/SchemeSource/R5Macros.s"
  "/Users/JayFreeman/developer/Scheme/WraithScheme.64/SchemeSource/R5Macros.Expanded.s")

(macroexpand-file
  "/Users/JayFreeman/developer/Scheme/WraithScheme.64/SchemeSource/PrimitiveLibrary.s"
  "/Users/JayFreeman/developer/Scheme/WraithScheme.64/SchemeSource/PrimitiveLibrary.Expanded.s")

(macroexpand-file
  "/Users/JayFreeman/developer/Scheme/WraithScheme.64/SchemeSource/HygienicMacros.s"
  "/Users/JayFreeman/developer/Scheme/WraithScheme.64/SchemeSource/HygienicMacros.Expanded.s")

(macroexpand-file
  "/Users/JayFreeman/developer/Scheme/WraithScheme.64/SchemeSource/DynamicWind.s"
  "/Users/JayFreeman/developer/Scheme/WraithScheme.64/SchemeSource/DynamicWind.Expanded.s")

(macroexpand-file
  "/Users/JayFreeman/developer/Scheme/WraithScheme.64/SchemeSource/Compiler/ConstantsAndPrimitives.s"
  "/Users/JayFreeman/developer/Scheme/WraithScheme.64/SchemeSource/Compiler/ConstantsAndPrimitives.Expanded.s")

(macroexpand-file
  "/Users/JayFreeman/developer/Scheme/WraithScheme.64/SchemeSource/Compiler/ReleasedCompilerCode.s"
  "/Users/JayFreeman/developer/Scheme/WraithScheme.64/SchemeSource/Compiler/ReleasedCompilerCode.Expanded.s")

(macroexpand-file
  "/Users/JayFreeman/developer/Scheme/WraithScheme.64/SchemeSource/SimpleClass.s"
  "/Users/JayFreeman/developer/Scheme/WraithScheme.64/SchemeSource/SimpleClass.Expanded.s")

(e::exit)
