;; Copyright (c) 2009 by Jay Reynolds Freeman

;;    This program is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    This program is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;
;;    Contact information:
;;
;;        EMail:       Jay_Reynolds_Freeman@mac.com
;;        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman
;;
;;        Paper Mail:  Jay Reynolds Freeman
;;                     Post Office Box 60628
;;                     Palo Alto, CA, 94306-0628

(display "CompileForms.s:\n")

(c::define c::compile-defines-list
  (list
    'c::define-code
    'define
    'c::define-no-compile-code
    'e::define-no-compile
  ))

;;; DEFINE

;    Redefined.

(c::define c::define-code
  (c::lambda (form)
    (if (not (list? form))
      (c::begin 
        (display form (e::error-port)) (newline (e::error-port))
        (e::error e::cm54)))
    (if (< (length form) 3)
       (c::begin (display form (e::error-port)) (e::error e::cm11)))
    (let*
      ((sugar (pair? (cadr form)))                 ;; Syntactic sugar:  (define (foo ...) ...)
       (name (if sugar (caadr form) (cadr form)))
       (vars (if sugar (cdadr form) '()))
       (stuff (cddr form))
       )
      (if sugar
        (list 'c::define
              name
              (if (c::compiling?)
                (list 'e::compile-form (append (list 'lambda vars) stuff))
                (if (list? stuff)
                  (e::expand-macros-recursively (append (list 'lambda vars) stuff))
                  (c::begin (display form (e::error-port)) (newline) (e::error e::cm58)))))
        (if (> (length form) 3)
          (c::begin (display form (e::error-port)) (e::error e::cm12))
          (list 'c::define
                name
                (e::expand-macros-recursively (car stuff))))))))

(e::macro define c::define-code)

;;; DEFINE-NO-COMPILE

(c::define c::define-no-compile-code
  (c::lambda (form)
    (if (not (list? form))
      (c::begin 
        (display form (e::error-port)) (newline (e::error-port))
        (e::error e::cm55)))
    (if (< (length form) 3)
       (c::begin (display form (e::error-port)) (e::error e::cm11)))
    (let*
      ((sugar (pair? (cadr form)))                 ;; Syntactic sugar:  (define (foo ...) ...)
       (name (if sugar (caadr form) (cadr form)))
       (vars (if sugar (cdadr form) '()))
       (stuff (cddr form))
       )
      (if sugar
        (list 'c::define
              name
              (if (list? stuff)
                (e::expand-macros-recursively (append (list 'lambda vars) stuff))
                (c::begin (display form (e::error-port)) (newline) (e::error e::cm58))))
        (if (> (length form) 3)
          (c::begin (display form (e::error-port)) (e::error e::cm56))
          (list 'c::define
                name
                (e::expand-macros-recursively (car stuff))))))))
          
(e::macro e::define-no-compile c::define-no-compile-code)

;;; e::COMPILE-FORM
;
;     Temporary, to ensure that forms are macroexpanded, until the real
; "e::compile-form" is loaded later on.  Note, that this definition is not
; made permanent.

(e::clear-permanent! 'e::compile-form)

(c::define e::compile-form
  (c::lambda( form)
    (c::if (= (c::get-pure-tag form) c::LAMBDA_PAIR_TAG)
      ((c::lambda (original-tag)
        ((c::lambda (lam-as-pair)
           (c::set-tag!
             (cons
               (car lam-as-pair)                                                           ;;  formal parameters
               (cons
                 (cdr (e::expand-macros-recursively (cons '() (cadr lam-as-pair))))        ;;  lambda body
                 (cddr lam-as-pair)                                                        ;;  environment list
                 )
               )
             original-tag))
         (c::set-tag! form c::PAIR_TAG)))
       (e::get-tag form))
      form)))   
