;;  Copyright (c) 2009 by Jay Reynolds Freeman

;;    This program is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    This program is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;
;;    Contact information:
;;
;;        EMail:       Jay_Reynolds_Freeman@mac.com
;;        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman
;;
;;        Paper Mail:  Jay Reynolds Freeman
;;                     Post Office Box 60628
;;                     Palo Alto, CA, 94306-0628

(write "ConstantsAndPrimitives.s:") (newline)

(c::define c::constants-and-primitives-list
  (list
        'c::self-evals-list
        'c::lambda-pair?
        'c::lambda-pair-interpreted?
        'c::list-up-numbers
        'c::find-variables
        'c::make-alist-from-envlist
        'c::make-new-variables-alist-aux
        'c::make-new-variables-alist
        'c::rassv-count
        ))

(define c::self-evals-list
  (list c::EMPTY_LIST_TAG
        c::BUILT_IN_FUNCTION_TAG
        c::CHARACTER_TAG
        c::FALSE_BOOLEAN_TAG
        c::LAMBDA_PAIR_TAG
        c::MACRO_TAG
        c::SPECIAL_FORM_TAG
        c::STRING_TAG
        c::TRUE_BOOLEAN_TAG
	))

;########################################################
;
;     Handy primitives:

;;; c::LAMBDA-PAIR?

(define c::lambda-pair?
  (c::lambda (thing)
    (= (c::get-pure-tag thing) c::LAMBDA_PAIR_TAG)))

;;; c::LAMBDA-PAIR-INTERPRETED?

(define c::lambda-pair-interpreted?
  (c::lambda (thing)
    (not (number? (car (c::set-tag! thing c::PAIR_TAG))))))

;;; c::LIST-UP-NUMBERS

;     Returns a sequence of integers, starting with N, one
; integer for each item in the list "lyst".

(define (c::list-up-numbers lyst n)
  (c::if (null? lyst)
      '()
      (c::if (pair? lyst)
        (cons n (c::list-up-numbers (cdr lyst) (+ n 1)))
        (list n))))

;;; c::FIND-VARIABLES

;     Extract the list of formal parameters from an interpreted lambda expression.

(define c::find-variables car)

;;; c::MAKE-ALIST-FROM-ENVLIST

(define (c::make-alist-from-envlist envlist)
  (c::map-without-type-checking cons
       (c::list-up-numbers envlist 0)
       (c::map-without-type-checking cdr envlist))) 

;;; c::MAKE-NEW-VARIABLES-ALIST-AUX

(define (c::make-new-variables-alist-aux vars numbers)
  (cond ((null? vars) '())
        ((pair? vars)
         (cons (cons (car vars) (car numbers))
               (c::make-new-variables-alist-aux (cdr vars) (cdr numbers))))
        (else
         (list (cons vars (car numbers))))))

;;; c::MAKE-NEW-VARIABLES-ALIST

(define (c::make-new-variables-alist vars)
  (c::make-new-variables-alist-aux vars (reverse (c::list-up-numbers vars 0))) )

;;; c::RASSV-COUNT

(define (c::rassv-count item ralist count)
  (cond ((null? ralist)
          (display item) (newline) (e::error "c::rassv-count -- Cannot find item in ralist."))
        ((and (pair? (car ralist)) (eqv? item (cdar ralist)))
         count)
        (else 
         (c::rassv-count item (cdr ralist) (+ count 1)))))
