;;  Copyright (c) 2009 by Jay Reynolds Freeman

;;    This program is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    This program is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;
;;    Contact information:
;;
;;        EMail:       Jay_Reynolds_Freeman@mac.com
;;        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman
;;
;;        Paper Mail:  Jay Reynolds Freeman
;;                     Post Office Box 60628
;;                     Palo Alto, CA, 94306-0628

(display "ReleasedCompilerCode.s:\n")

(c::define c::released-compiler-code-list
  (list 
    'c::BUCKET_NUMBER_OFFSET
    'c::ENVIRONMENT_NUMBER_OFFSET
    'c::LOWER_WORD_MASK
    'c::evaluates-to-self-list
    'c::evaluates-to-self?
    'c::lambda-evaled-form?
    'c::quoted-evaled-form?
    'c::set!-evaled-form?
    'c::c::set!-evaled-form?
    'c::quoted-form?
	'c::set!-form?
	'c::c::set!-form?
    'c::make-symbols-and-positions-list-aux
    'c::make-symbols-and-positions-add-numbers
    'c::make-symbols-and-positions-list
    'c::find-lambda-pair-and-beta-reduce
    'e::beta-reduce-lambdas-recursively
    'c::substitute-permanent-symbols-aux
    'e::substitute-permanent-symbols
    'c::top-level-environment-ref?
    'c::substitute-new-permanent-symbols-aux
    'e::substitute-new-permanent-symbols
    'c::list-index-for-binding-aux
    'c::list-index-for-binding
    'c::symbol-ref-in-top-level-environment
    'c::substitute-outer-environment-references-aux
    'e::substitute-outer-environment-references
    'c::compile-interpreted-lambda
    'c::compile-interpreted-lambda-simple
    'c::compile-lambda-pair
    'e::compile-form
    'e::compile-form-simple
  ) )

;########################################################
;
;     Stuff that deals with generation of code:

(c::define c::BUCKET_NUMBER_OFFSET 32)
(c::define c::ENVIRONMENT_NUMBER_OFFSET 32)
(c::define c::LOWER_WORD_MASK #xffffffff)

(c::define c::evaluates-to-self-list
  (list
    c::EMPTY_LIST_TAG
    c::BUILT_IN_FUNCTION_TAG
    c::CHARACTER_TAG
    c::EXACT_SHORT_FIXNUM_TAG
    c::EXACT_SHORT_FLOAT_TAG
    c::FAIL_TAG
    c::FALSE_BOOLEAN_TAG
    c::INEXACT_SHORT_FIXNUM_TAG
    c::INEXACT_SHORT_FLOAT_TAG
    c::LAMBDA_PAIR_TAG
    c::MACRO_TAG
    c::PORT_TAG
    c::SPECIAL_FORM_TAG
    c::STRING_TAG
    c::SUCCEED_TAG
    c::TRUE_BOOLEAN_TAG
    c::PECULIAR_NON_LEAF_TAG))

(c::define c::evaluates-to-self?
  (c::lambda(form) (memq (e::get-tag form) c::evaluates-to-self-list)))

(c::define c::lambda-evaled-form?
  (c::lambda (form)
    (and (list? form) (equal? (car form) c::lambda) (>= (length form) 3))))

(c::define c::quoted-form?
  (c::lambda (form)
    (and (list? form) (equal? (car form) 'quote))))

(c::define c::quoted-evaled-form?
  (c::lambda (form)
    (and (list? form) (equal? (car form) quote))))

(c::define c::set!-form?
  (c::lambda (form)
    (and (list? form) (equal? (car form) 'set!) (= (length form) 3))))

(c::define c::set!-evaled-form?
  (c::lambda (form)
    (and (list? form) (equal? (car form) set!) (= (length form) 3))))

(c::define c::c::set!-form?
  (c::lambda (form)
    (and (list? form) (equal? (car form) 'c::set!) (= (length form) 3))))

(c::define c::c::set!-evaled-form?
  (c::lambda (form)
    (and (list? form) (equal? (car form) c::set!) (= (length form) 3))))

(c::define c::make-symbols-and-positions-list-aux
  (c::lambda (symbols)
    (cond ((null? symbols) '())
          ((pair? symbols)
           (cons (car symbols) (c::make-symbols-and-positions-list-aux (cdr symbols))))
          (#t (list symbols)))))

(c::define c::make-symbols-and-positions-add-numbers
  (c::lambda (symbols n)
    (cond ((null? symbols) '())
          (#t
           (cons (list (car symbols) n)
                 (c::make-symbols-and-positions-add-numbers (cdr symbols) (+ n 1)))))))

(c::define c::make-symbols-and-positions-list
  (c::lambda (symbols)
    (c::make-symbols-and-positions-add-numbers
      (reverse (c::make-symbols-and-positions-list-aux symbols))
      0)))

(c::define c::find-lambda-pair-and-beta-reduce
  (c::lambda (form env-n-down symbols-and-positions check-function?)
;    (display (list 'fleabe form env-n-down symbols-and-positions check-function?)) (newline) ;; DEBUG
    (cond
      ((null? form)                                     #t)
      ((and check-function? (c::quoted-form? form))     #t)
      ((and check-function? (c::quoted-evaled-form? form))     #t)
      ((and check-function? (c::set!-form? form))
          (c::find-lambda-pair-and-beta-reduce (cddr form) env-n-down symbols-and-positions #f))
      ((and check-function? (c::c::set!-form? form))
          (c::find-lambda-pair-and-beta-reduce (cddr form) env-n-down symbols-and-positions #f))
      ((c::lambda-form? form)
       (c::find-lambda-pair-and-beta-reduce (cddr form) 0 (c::make-symbols-and-positions-list (cadr form)) #f)
       (c::find-lambda-pair-and-beta-reduce (cddr form) (+ env-n-down 1) symbols-and-positions #f))
      ((pair? form)
       (cond
         ((symbol? (car form))
          (let ((found (assv (car form) symbols-and-positions)))
;            (display "Found a symbol: ") (display (car form)) (newline)
            (if found
              (set-car! form 
                (c::set-tag!
                  (e::bit-or (e::bit-shift-left env-n-down c::ENVIRONMENT_NUMBER_OFFSET) (cadr found))
                  c::ENVIRONMENT_REFERENCE_TAG)))
            (c::find-lambda-pair-and-beta-reduce (cdr form) env-n-down symbols-and-positions #f)))
         (else
          (c::find-lambda-pair-and-beta-reduce (car form) env-n-down symbols-and-positions #t)
          (c::find-lambda-pair-and-beta-reduce (cdr form) env-n-down symbols-and-positions #f))))
      ((vector? form)
       (c::find-lambda-pair-and-beta-reduce (c::set-tag! form c::PAIR_TAG) env-n-down symbols-and-positions #f))
      (else #t))))

(c::define e::beta-reduce-lambdas-recursively
  (c::lambda (form)
    (let ((listed-form (cons form '())))
      (c::find-lambda-pair-and-beta-reduce listed-form 0 '() #t)
      (car listed-form))))

(c::define c::substitute-permanent-symbols-aux
  (c::lambda (form check-function?)
;    (display (list 'spsa form check-function?)) (newline) ;; DEBUG
    (cond
      ((null? form)                                   #t)
      ((and check-function? (c::quoted-form? form))
          (set-car! form (c::symeval (car form))))
      ((and check-function? (c::quoted-evaled-form? form))
          #t)
      ((and check-function? (c::set!-form? form))
          (set-car! form (c::symeval (car form)))
          (c::substitute-permanent-symbols-aux (cddr form) #f))
      ((and check-function? (c::c::set!-form? form))
          (set-car! form (c::symeval (car form)))
          (c::substitute-permanent-symbols-aux (cddr form) #f))
      ((c::lambda-form? form)   (c::substitute-permanent-symbols-aux (cddr form) #f))
      ((pair? form)
       (cond
         ((and (symbol? (car form)) (e::permanent? (car form)) (c::evaluates-to-self? (c::symeval (car form))))
;;          (display (string-append (symbol->string (car form)) " position 1\n")) ;; DEBUG
          (set-car! form (c::symeval (car form)))
          (c::substitute-permanent-symbols-aux (cdr form) #f))
         (else
          (c::substitute-permanent-symbols-aux (car form) #t)
          (c::substitute-permanent-symbols-aux (cdr form) #f))))
      ((vector? form)           (c::substitute-permanent-symbols-aux (c::set-tag! form c::PAIR_TAG) #f))
      (else #t))))

(c::define e::substitute-permanent-symbols
  (c::lambda (form)
    (let ((listed-form (cons (cons form '()) '())))
      (c::substitute-permanent-symbols-aux listed-form #t)
      (caar listed-form))))

(c::define c::top-level-environment-ref?
  (c::lambda (x)
    (equal? (c::get-pure-tag x) c::TOP_LEVEL_ENVIRONMENT_REFERENCE_TAG)))

(c::define c::substitute-new-permanent-symbols-aux
  (c::lambda (form check-function?)
;    (display (list 'snpsa form check-function?)) (newline) ;; DEBUG
    (cond
      ((null? form)                                                                       #t)
      ((and check-function? (or (c::quoted-form? form) (c::quoted-evaled-form? form)))    #t)
      ((and check-function? (or (c::set!-form? form ) (c::set!-evaled-form? form)))
       (c::substitute-new-permanent-symbols-aux (cddr form) #f))
      ((and check-function? (or (c::c::set!-form? form ) (c::c::set!-evaled-form? form)))
       (c::substitute-new-permanent-symbols-aux (cddr form) #f))
      ((or (c::lambda-form? form) (c::lambda-evaled-form? form))
       (c::substitute-new-permanent-symbols-aux (cddr form) #f))
      ((pair? form)
       (cond
         ((and (symbol? (car form)) (e::permanent? (car form)) (c::evaluates-to-self? (c::symeval (car form))))
          (set-car! form (c::symeval (car form)))
          (c::substitute-new-permanent-symbols-aux (cdr form) #f))
         ((c::top-level-environment-ref? (car form))
;          (display "Top-level-environment-reference\n")      ;  DEBUG
          (let ((sym (c::top-level-environment-ref-symbol (car form))))
;            (display "Symbol: ") (display sym) (newline) ; DEBUG
            (if (and (symbol? sym) (e::permanent? sym) (c::evaluates-to-self? (c::symeval sym)))
              (set-car! form (c::symeval sym)))
          (c::substitute-new-permanent-symbols-aux (cdr form) #f)))
         (else
          (c::substitute-new-permanent-symbols-aux (car form) #t)
          (c::substitute-new-permanent-symbols-aux (cdr form) #f))))
      ((vector? form) 
       (c::substitute-new-permanent-symbols-aux (c::set-tag! form c::PAIR_TAG) #f))
      (else #t))))

(c::define e::substitute-new-permanent-symbols
  (c::lambda (form)
    (if (and (c::lambda-pair? form) (c::lambda-pair-interpreted? form))  ;; Commented out 2010.05.24
      (let* ((lam-as-pair (c::set-tag! form c::PAIR_TAG)))
        (c::substitute-new-permanent-symbols-aux (cons (cons 'c::lambda (cons (car lam-as-pair) (cadr lam-as-pair))) '()) #t)
        form)
      form)))

(c::define c::list-index-for-binding-aux
  (c::lambda (lyst sym lnth n)
    (if (>= n lnth)
      #f
      (if (equal? sym (cdar lyst))
        n
        (c::list-index-for-binding-aux (cdr lyst) sym lnth (+ n 1))))))

(c::define c::list-index-for-binding
  (c::lambda (lyst sym lnth)
    (c::list-index-for-binding-aux lyst sym lnth 0)))    

(c::define c::symbol-ref-in-top-level-environment
  (c::lambda (sym)
    (let*
      ((hashval (cadr (c::set-tag! sym c::PAIR_TAG)))
       (bucket (vector-ref (car (c::top-level-environment)) hashval))
       (index (c::list-index-for-binding bucket sym (length bucket)))
       )
      (if index 
        (c::set-tag! (e::bit-or (e::bit-shift-left hashval c::BUCKET_NUMBER_OFFSET) index) c::TOP_LEVEL_ENVIRONMENT_REFERENCE_TAG)
        #f))))

(c::define c::substitute-outer-environment-references-aux
  (c::lambda (form check-function?)
;    (display (list 'soera form check-function?)) (newline) ;; DEBUG
    (cond
      ((null? form)                                             #t)
      ((and check-function?  (c::quoted-evaled-form? form))     #t)
      ((and check-function? (c::set!-evaled-form? form))
          (c::substitute-outer-environment-references-aux (cddr form) #f))
      ((and check-function? (c::c::set!-evaled-form? form))
          (c::substitute-outer-environment-references-aux (cddr form) #f))
      ((c::lambda-form? form)   (c::substitute-outer-environment-references-aux (cddr form) #f))
      ((pair? form)
       (let ((ref #f))
         (cond
           ((and (symbol? (car form)) (c::begin (c::set! ref (c::symbol-ref-in-top-level-environment (car form))) ref))
            (set-car! form ref)
            (c::substitute-outer-environment-references-aux (cdr form) #f))
           (else
            (c::substitute-outer-environment-references-aux (car form) #t)
            (c::substitute-outer-environment-references-aux (cdr form) #f)))))
      ((vector? form)           (c::substitute-outer-environment-references-aux (c::set-tag! form c::PAIR_TAG) #f))
      (else #t))))

(c::define e::substitute-outer-environment-references
  (c::lambda (form)
    (let ((listed-form (cons (cons form '()) '())))
      (c::substitute-outer-environment-references-aux listed-form #t)
      (caar listed-form))))

;;; c::COMPILE-INTERPRETED-LAMBDA

(c::define c::compile-interpreted-lambda
  (c::lambda (lam)
    (let*
      ((original-tag (e::get-tag lam))
       (lam-as-pair (c::set-tag! lam c::PAIR_TAG))
       (vars (car lam-as-pair))
       (body (cadr lam-as-pair))
       (fake-lambda-form (cons 'c::lambda (cons vars (e::expand-macros-recursively body))))
       (fake-form-fussed-with
         (e::substitute-outer-environment-references
           (e::substitute-permanent-symbols
             (e::beta-reduce-lambdas-recursively
               fake-lambda-form))))
       )
      (c::set-tag!
        (cons 
          (cadr fake-form-fussed-with)
          (cons
            (cddr fake-form-fussed-with)
            (cddr lam-as-pair)))
        original-tag))))

(c::define c::compile-interpreted-lambda-simple
  (c::lambda (lam)
    (let*
      ((original-tag (e::get-tag lam))
       (lam-as-pair (c::set-tag! lam c::PAIR_TAG))
       (vars (car lam-as-pair))
       (body (cadr lam-as-pair))
       (fake-lambda-form (cons 'c::lambda (cons vars (e::expand-macros-recursively body))))
       (fake-form-fussed-with fake-lambda-form)
       )
      (c::set-tag!
        (cons 
          (cadr fake-form-fussed-with)
          (cons
            (cddr fake-form-fussed-with)
            (cddr lam-as-pair)))
        original-tag))))

;     Splice in references, then call an auxilary to finish the job.

;;; c::COMPILE-LAMBDA-PAIR

;     If the lambda pair has not been compiled, compile it.
; Otherwise there isn't much to do.

(c::define c::compile-lambda-pair
  (c::lambda (lam)
    (c::if (c::lambda-pair-interpreted? lam)  ;; Commented out 2010.05.24
      (c::compile-interpreted-lambda lam)
      lam)))

;;; c::COMPILE-FORM

(c::define e::compile-form
  (c::lambda (form)
    (c::if (c::lambda-pair? form)
      (c::compile-lambda-pair form)
      form)))

(c::define e::compile-form-simple
  (c::lambda (form)
    (c::if (and (c::lambda-pair? form) (c::lambda-pair-interpreted? form))
      (c::compile-interpreted-lambda-simple form)
      form)))


