;;    This program is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    This program is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;
;;    Contact information:
;;
;;        EMail:       Jay_Reynolds_Freeman@mac.com
;;        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman
;;
;;        Paper Mail:  Jay Reynolds Freeman
;;                     Post Office Box 60628
;;                     Palo Alto, CA, 94306-0628

;; Jay Freeman makes NO CLAIM to copyright this material.

;;  What follows first hereafter is (but for reformatting) the
;;   implementation of "dynamic-wind" published in Appendix A of
;;
;;     Rees, Jonathan, 1992.  "The Scheme of Things: The June
;;       Meeting", in "Lisp Pointers V(4)", October-December 1992.
;;
;;   but which I found on the Web.
;    
;        (define *here* (list #f))
;        
;        (define original-cwcc call-with-current-continuation)
;        
;        (define (call-with-current-continuation proc)
;          (let ((here *here*))
;            (original-cwcc
;              (lambda (cont)
;                (proc
;                  (lambda results
;                    (reroot! here)
;                    (apply cont results)))))))
;        
;        (define (dynamic-wind before during after)
;          (let ((here *here*))
;            (reroot! (cons (cons before after) here))
;            (call-with-values
;              during
;              (lambda results
;                (reroot! here)
;                (apply values results)))))
;        
;        (define (reroot! there)
;          (if (not (eq? *here* there))
;            (begin
;              (reroot! (cdr there))
;              (let
;                ((before (caar there))
;                 (after  (cdar there)))
;                (set-car! *here* (cons after before))
;                (set-cdr! *here* there)
;                (set-car! there #f)
;                (set-cdr! there '())
;                (set! *here* there)
;                (before)))))



(c::define c::dynamic-wind-list
  (list
    'c::reroot!
    'call-with-current-continuation
    'dynamic-wind
    ))

(define c::here (list #f))  ;; *NOT* to be made permanent.

;; Next step (commented out) accomplished in LowLevelMacros.s
;;
;; (define e::original-cwcc call-with-current-continuation)

(e::clear-permanent! 'call-with-current-continuation)

(define (call-with-current-continuation proc)
  (let ((here c::here))
    (e::original-cwcc
      (lambda (cont)
        (proc
          (lambda results
            (c::reroot! here)
            (apply cont results)))))))

(define (dynamic-wind before during after)
  (if (not (procedure? before))
    (begin
      (display before (e::error-port)) (newline (e::error-port))
      (e::error "dynamic-wind called with a first argument (before) that is not a procedure.")))
  (if (not (procedure? during))
    (begin
      (display during (e::error-port)) (newline (e::error-port))
      (e::error "dynamic-wind called with a second argument (during) that is not a procedure.")))
  (if (not (procedure? after))
    (begin
      (display after (e::error-port)) (newline (e::error-port))
      (e::error "dynamic-wind called with a third argument (after) that is not a procedure.")))
  (let ((here c::here))
    (c::reroot! (cons (cons before after) here))
    (call-with-values
      during
      (lambda results
        (c::reroot! here)
        (apply values results)))))

(define (c::reroot! there)
  (if (not (eq? c::here there))
    (begin
      (c::reroot! (cdr there))
      (let
        ((before (caar there))
         (after  (cdar there)))
        (set-car! c::here (cons after before))
        (set-cdr! c::here there)
        (set-car! there #f)
        (set-cdr! there '())
        (c::set! c::here there)
        (before)))))

