;; Copyright (c) 2009 by Jay Reynolds Freeman

;;    This program is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    This program is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;
;;    Contact information:
;;
;;        EMail:       Jay_Reynolds_Freeman@mac.com
;;        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman
;;
;;        Paper Mail:  Jay Reynolds Freeman
;;                     Post Office Box 60628
;;                     Palo Alto, CA, 94306-0628

(write "EnvironmentManipulation.s:\n")

(c::define c::syntactic-keywords
  (list
    'do
    'delay
    'case
    '=>
    'else
    'cond
    'set!
    'define
    'or
    'and
    'letrec
    'let*
    'let
    'quasiquote
    'unquote
    'unquote-splicing
    'quote
    'begin
    'c::begin
    'c::if
    'if
    'lambda
    'c::lambda
    'c::set!
   ))

(c::define c::scheme-report-names
  (list
    'abs
    'acos
    'asin
    'atan
    'call-with-current-continuation
    'call-with-input-file
    'call-with-output-file
    'call-with-values
    'ceiling
    'char->integer
    'char-alphabetic?
    'char-ci<=?
    'char-ci<?
    'char-ci=?
    'char-ci>=?
    'char-ci>?
    'char-downcase
    'char-lower-case?
    'char-numeric?
    'char-ready?
    'char-upcase
    'char-upper-case?
    'char-whitespace?
    'char<=?
    'char<?
    'char=?
    'char>=?
    'char>?
    'char?
    'close-input-port
    'close-output-port
    'complex?
    'cos
    'current-input-port
    'current-output-port
    'dynamic-wind
    'eval
    'even?
    'exact->inexact
    'exact?
    'exp
    'expt
    'floor
    'force
    'gcd
    'inexact->exact
    'inexact?
    'input-port?
    'integer->char
    'integer?
    'interaction-environment
    'lcm
    'load
    'log
    'make-string
    'make-vector
    'max
    'min
    'modulo
    'negative?
    'null-environment
    'number->string
    'number?
    'odd?
    'open-input-file
    'open-output-file
    'output-port?
    'peek-char
    'port?
    'positive?
    'quotient
    'rational?
    'read
    'read-char
    'real?
    'remainder
    'round
    'scheme-report-environment
    'sin
    'sqrt
    'string->list
    'string->number
    'string->symbol
    'string-append
    'string-ci<=?
    'string-ci<?
    'string-ci=?
    'string-ci>=?
    'string-ci>?
    'string-copy
    'string-fill!
    'string-length
    'string-ref
    'string-set!
    'string<=?
    'string<?
    'string=?
    'string>=?
    'string>?
    'string?
    'substring
    'tan
    'transcript-off
    'transcript-on
    'truncate
    'vector-fill!
    'vector?
    'with-input-from-file
    'with-output-to-file
    'write
    'write-char
    'zero?
    'assoc
    'assq
    'assv
    'boolean?
    'define-syntax
    'display
    'eof-object?
    'for-each
    'let-syntax
    'letrec-syntax
    'list->string
    'list->vector
    'list-ref
    'list-tail
    'map
    'member
    'memq
    'memv
    'newline
    'not
    'null?
    'pair?
    'procedure?
    'reverse
    'set-car!
    'set-cdr!
    'symbol?
    'syntax-rules
    'values
    'vector
    'vector->list
    'vector-length
    'vector-ref
    'vector-set!
    '+
    '-
    '/
    '<
    '<=
    '=
    '>
    '>=
    'append
    'apply
    'caaaar
    'caaadr
    'caaar
    'caadar
    'caaddr
    'caadr
    'caar
    'cadaar
    'cadadr
    'cadar
    'caddar
    'cadddr
    'caddr
    'cadr
    'cdaaar
    'cdaadr
    'cdaar
    'cdadar
    'cdaddr
    'cdadr
    'cdar
    'cddaar
    'cddadr
    'cddar
    'cdddar
    'cddddr
    'cdddr
    'cddr
    'eq?
    'equal?
    'eqv?
    'length
    'list?
    'symbol->string
    'list
    'car
    'cdr
    'cons
    ))

(c::define c::find-cdr-whose-cadr-is-binding
  (c::lambda (symbol lyst)
    (cond
      ((null? lyst) '())
      ((equal? (cdadr lyst) symbol) lyst)
      (else (c::find-cdr-whose-cadr-is-binding symbol (cdr lyst))))))

(c::define c::excise-from-top-level-environment
  (c::lambda (symbol)
    (let*
      ((hashval (c::hash-string (symbol->string symbol)))
       (tle-vector (car (c::top-level-environment)))
       (bucket (vector-ref tle-vector hashval)))
      (if (equal? (cdar bucket) symbol)
        (vector-set! tle-vector hashval (cdr bucket))
        (let ((pair-sought (c::find-cdr-whose-cadr-is-binding symbol bucket)))
          (if (null? pair-sought)
            (e::error (string-append "Cannot find ") (symbol->string symbol) " in top-level environment.")
            (set-cdr! pair-sought (cddr pair-sought))))))))

(c::define c::insert-with-value-in-null-environment
  (c::lambda (symbol)
    (let
      ((null-env (c::null-environment))
       (value (c::symeval symbol)))
      (set-car! null-env (cons (cons value symbol) (car null-env))))))

;;(e::inspect c::insert-with-value-in-null-environment)  ;; DEBUG

;;(e::expand-macros-recursively (e::deep-copy    ;; DEBUG
;;(append (list 'lambda (list 'symbol) (cadr (c::set-tag! c::insert-with-value-in-null-environment 2))))))    ;; DEBUG


(c::define c::insert-with-value-in-scheme-report-environment
  (c::lambda (symbol)
    (let
      ((scheme-report-env (c::scheme-report-environment))
       (value (c::symeval symbol)))
      (set-car! scheme-report-env (cons (cons value symbol) (car scheme-report-env))))))

(map
  (c::lambda (symbol)
    (c::insert-with-value-in-null-environment symbol)
    (c::excise-from-top-level-environment symbol))
  c::syntactic-keywords)

(map
  (c::lambda (symbol)
    (c::insert-with-value-in-scheme-report-environment symbol)
    (c::excise-from-top-level-environment symbol))
  c::scheme-report-names)

(map 
  c::excise-from-top-level-environment
  '(
     c::syntactic-keywords
     c::scheme-report-names
     c::insert-with-value-in-null-environment
     c::insert-with-value-in-scheme-report-environment
     )
  )

(c::set! c::find-cdr-whose-cadr-is-binding '())
(c::set! c::excise-from-top-level-environment '())
