;; Copyright (c) 2009 by Jay Reynolds Freeman

;;    This program is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    This program is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;
;;    Contact information:
;;
;;        EMail:       Jay_Reynolds_Freeman@mac.com
;;        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman
;;
;;        Paper Mail:  Jay Reynolds Freeman
;;                     Post Office Box 60628
;;                     Palo Alto, CA, 94306-0628

(display "HygienicMacros.s:\n")

(c::define c::hygienic-macros-list
  (list
    'define-syntax
    'if
    'let*
    'let
    'letrec
    'and
    'or
    'cond
    'case
    'do
    'let-syntax
    'letrec-syntax
    )
  )

(e::macro define-syntax c::define-syntax-code)

(define-syntax if
  (syntax-rules ()
    ((if predicate consequent) (c::if predicate consequent #f))
    ((if predicate consequent alternative) (c::if predicate consequent alternative))))

(define-syntax let*
  (syntax-rules ()
    ((let* () body1 body2 ...)
     (let () body1 body2 ...))
    ((let* ((name1 val1) (name2 val2) ...)
       body1 body2 ...)
     (let ((name1 val1))
       (let* ((name2 val2) ...)
         body1 body2 ...)))
  ))

(define-syntax let
  (syntax-rules ()
    ((let ((name val) ...) body1 body2 ...)
     ((lambda (name ...) body1 body2 ...)
      val ...))
    ((let tag ((name val) ...) body1 body2 ...)
     ((letrec ((tag (lambda (name ...)
                      body1 body2 ...)))
        tag)
      val ...))
    ))

(define-syntax letrec
  (syntax-rules ()
    ((letrec ((name val) ...) body1 body2 ...)
     ((lambda (name ...)
       (begin (c::set! name val) ...)
       ((lambda () body1 body2 ...)))
      (quote name) ...))
   ))

(define-syntax and
  (syntax-rules ()
    ((and) #t)
    ((and e) e)
    ((and e1 e2 ...)
     (if e1
         (and e2 ...)
         #f))))

(define-syntax or
  (syntax-rules ()
    ((or) #f)
    ((or e) e)
    ((or e1 e2 ...)
     (let ((temp e1))
       (if temp
           temp
           (or e2 ...))))))

(define-syntax cond
  (syntax-rules (else =>)
    ((cond (else result1 result2 ...))
     (begin result1 result2 ...))
    ((cond (test => result))
     (let ((temp test))
       (if temp (result temp))))
    ((cond (test => result) clause1 clause2 ...)
     (let ((temp test))
       (if temp
           (result temp)
           (cond clause1 clause2 ...))))
    ((cond (test)) test)
    ((cond (test) clause1 clause2 ...)
     (let ((temp test))
       (if temp
           temp
           (cond clause1 clause2 ...))))
    ((cond (test result1 result2 ...))
     (if test (begin result1 result2 ...) #t))
    ((cond (test result1 result2 ...)
           clause1 clause2 ...)
     (if test
         (begin result1 result2 ...)
         (cond clause1 clause2 ...)))
    ))

(define-syntax case
  (syntax-rules (else)
    ((case (key ...)
       clauses ...)
     (let ((atom-key (key ...)))
       (case atom-key clauses ...)))
    ((case key
       (else result1 result2 ...))
     (begin result1 result2 ...))
    ((case key
       ((atoms ...) result1 result2 ...))
     (if (memv key '(atoms ...))
       (begin result1 result2 ...)
       #t))
    ((case key
       ((atoms ...) result1 result2 ...)
       clause clauses ...)
     (if (memv key '(atoms ...))
       (begin result1 result2 ...)
       (case key clause clauses ...)))))

(define-syntax do
  (syntax-rules ()
    ((do ((var init step ...) ...)
         (test expr ...)
         command ...)
     (letrec
       ((loop
         (lambda (var ...)
           (if test
               (begin
                 #t
                 expr ...)
               (begin
                 #t
                 command ...
                 (loop (do "step" var step ...)
                       ...))))))
       (loop init ...)))
    ((do "step" x) x)
    ((do "step" x y) y)
    ))

(define-syntax let-syntax
  (syntax-rules ()
    ((let-syntax () body ...)
     (begin body ...))
    ((let-syntax ((kwd tspec) ...) body ...)
     ((lambda (kwd ...) body ...)
      (e::syntax-macro-body kwd tspec) ... ))))
       	
(define-syntax letrec-syntax
  (syntax-rules ()
    ((letrec-syntax () body ...)
     (begin body ...))
    ((letrec-syntax ((kwd tspec) ...) body ...)
     (letrec ((kwd (e::syntax-macro-body kwd tspec)) ...)
        body ...))))

