;; Copyright (c) 2009 by Jay Reynolds Freeman

;;    This program is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    This program is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;
;;    Contact information:
;;
;;        EMail:       Jay_Reynolds_Freeman@mac.com
;;        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman
;;
;;        Paper Mail:  Jay Reynolds Freeman
;;                     Post Office Box 60628
;;                     Palo Alto, CA, 94306-0628

(write "LowLevelMacros.s:\n")

(c::define c::low-level-macros-list
  (list
    'e::error
    'e::warning
    'c::get-pure-tag
    'c::immutable?
    'c::interpreted-lambda?
    'c::lambda-form?
    'c::lambda-form-possible?
    'c::lambda-syntax-check
    'c::listify
    'c::make-immutable!
    'c::make-proper-list

    'c::qq
    'c::qq-pair
    'c::qq-syntax-check
    'c::quasiquote-code
    'c::unquote-code
    'c::unquote-splicing-code
    'c::delay-code

    'quasiquote
;;    'quote
    'unquote
    'unquote-splicing
    'delay

    'c::list-lambda-formals
    'c::check-lambda-syntax-details
    'lambda
    'begin
    'e::original-cwcc
    )
  )

;########################################################
;
;    debugging

(c::define c::*display-macro-expansion* #f)


;########################################################
;
;    Tags and parameters

(c::define c::FORWARDING_POINTER_TAG                    #x00)
(c::define c::STRING_TAG                                #x01)
(c::define c::PAIR_TAG                                  #x02)
(c::define c::SYMBOL_TAG                                #x03)
(c::define c::LAMBDA_PAIR_TAG                           #x04)
(c::define c::BUILT_IN_FUNCTION_TAG                     #x05)
(c::define c::SPECIAL_FORM_TAG                          #x06)
(c::define c::VECTOR_TAG                                #x07)
(c::define c::PECULIAR_NON_LEAF_TAG                     #x08)
(c::define c::SYNTAX_BINDING_TAG                        #x09)
(c::define c::MACRO_TAG                                 #x0A)
(c::define c::PROMISE_TAG                               #x0B)
(c::define c::MULTIPLE_VALUES_TAG                       #x0C)
(c::define c::POINTER_TO_BYTE_BLOCK_TAG                 #x0D)
(c::define c::FORGETTABLE_UNEXPIRED_TAG                 #x0E)
(c::define c::FORGETTABLE_REMEMBERED_TAG                #x0F)
(c::define c::MMAPPED_BLOCK_TAG                         #x10)
(c::define c::GGC_FORWARDING_POINTER_TAG                #x11)

(c::define c::MAX_NON_LEAF_TAG                          #x11)


(c::define c::BYTE_BLOCK_TAG                            #x20)
(c::define c::FALSE_BOOLEAN_TAG                         #x21)
(c::define c::TRUE_BOOLEAN_TAG                          #x22)
(c::define c::CHARACTER_TAG                             #x23)
(c::define c::SCHEME_MACHINE_PC_TAG                     #x24)
(c::define c::PECULIAR_LEAF_TAG                         #x25)
(c::define c::STACK_REFERENCE_TAG                       #x26)
(c::define c::ENVIRONMENT_REFERENCE_TAG                 #x27)
(c::define c::TOP_LEVEL_ENVIRONMENT_REFERENCE_TAG       #x28)
(c::define c::EMPTY_LIST_TAG                            #x29)
(c::define c::FAIL_TAG                                  #x2A)
(c::define c::SUCCEED_TAG                               #x2B)
(c::define c::SCHEME_MACHINE_PC_INDEX_TAG               #x2C)
(c::define c::PORT_TAG                                  #x2D)
(c::define c::FORGETTABLE_FORGOTTEN_TAG                 #x2E)

(c::define c::INEXACT_SHORT_FIXNUM_TAG                  #x38)
(c::define c::EXACT_SHORT_FIXNUM_TAG                    #x39)
(c::define c::INEXACT_SHORT_FLOAT_TAG                   #x3A)
(c::define c::EXACT_SHORT_FLOAT_TAG                     #x3B)

(c::define c::MAX_LEAF_TAG                              #x3B)


(c::define c::IMMUTABLE_BIT                             #x0200)
(c::define c::PURE_TYPE_CODE_BITS                       #x3f)

(c::define e::cm11 "Wrong number of arguments to \"define\".")
(c::define e::cm12 "\"Define\" with more than two arguments must have a list as its first one.")
(c::define e::cm13 "Wrong number of arguments to \"")
(c::define e::cm14 "\" -- should be 1.")
(c::define e::cm15 "Malformed instance of \"")
(c::define e::cm16 "Encountered \"")
(c::define e::cm17 "\" when not quoting.")
(c::define e::cm18 "Encountered \"unquote\" when not quoting.")
(c::define e::cm19 "Encountered \"unquote-splicing\" when not quoting.")
(c::define e::cm46 "Malformed argument list to \"delay\".")
(c::define e::cm47 "Wrong number of arguments to \"delay\" -- should be 1.")
(c::define e::cm48 "\"Delay\" with no expression.")
(c::define e::cm54 "\"Define\" is not a proper list.")
(c::define e::cm55 "\"e::define-no-compile\" is not a proper list.")
(c::define e::cm56 "\"e::define-no-compile\" with more than two arguments must have a list as its first one.")
(c::define e::cm58 "Lambda expression with malformed body.")
(c::define e::cm59 "Lambda expression with empty body.")
(c::define e::cmset! c::set!)

(for-each e::set-permanent!
  (list
   'c::FORWARDING_POINTER_TAG
   'c::STRING_TAG
   'c::PAIR_TAG
   'c::SYMBOL_TAG
   'c::LAMBDA_PAIR_TAG
   'c::SPECIAL_FORM_TAG
   'c::BUILT_IN_FUNCTION_TAG
   'c::VECTOR_TAG
   'c::PECULIAR_NON_LEAF_TAG
   'c::SYNTAX_BINDING_TAG
   'c::MACRO_TAG
   'c::PROMISE_TAG
   'c::MULTIPLE_VALUES_TAG
   'c::POINTER_TO_BYTE_BLOCK_TAG
   'c::FORGETTABLE_UNEXPIRED_TAG
   'c::FORGETTABLE_REMEMBERED_TAG
   'c::GGC_FORWARDING_POINTER_TAG
   'c::MAX_NON_LEAF_TAG
   'c::BYTE_BLOCK_TAG
   'c::FALSE_BOOLEAN_TAG
   'c::TRUE_BOOLEAN_TAG
   'c::CHARACTER_TAG
   'c::SCHEME_MACHINE_PC_TAG
   'c::PECULIAR_LEAF_TAG
   'c::STACK_REFERENCE_TAG
   'c::ENVIRONMENT_REFERENCE_TAG
   'c::TOP_LEVEL_ENVIRONMENT_REFERENCE_TAG
   'c::EMPTY_LIST_TAG
   'c::FAIL_TAG
   'c::SUCCEED_TAG
   'c::SCHEME_MACHINE_PC_INDEX_TAG
   'c::PORT_TAG
   'c::FORGETTABLE_FORGOTTEN_TAG
   'c::INEXACT_SHORT_FIXNUM_TAG
   'c::EXACT_SHORT_FIXNUM_TAG
   'c::INEXACT_SHORT_FLOAT_TAG
   'c::EXACT_SHORT_FLOAT_TAG
   'c::MAX_LEAF_TAG
        
   'c::IMMUTABLE_BIT
   'c::PURE_TYPE_CODE_BITS
        
   'e::cm11
   'e::cm12
   'e::cm13
   'e::cm14
   'e::cm15
   'e::cm16
   'e::cm17
   'e::cm18
   'e::cm19
   'e::cm46
   'e::cm47
   'e::cm48
   'e::cm54
   'e::cm55
   'e::cm56
   'e::cm58
   'e::cm59
   'e::cmset!
    ))

;########################################################
;
;    Procedures:

;;; e::ERROR

;     Describe the problem, then reset.

(c::define e::error
  (c::lambda (message)
    (newline (e::error-port)) 
    (display "Problem: " (e::error-port))
    (display message (e::error-port))
    (display "  (Resetting)" (e::error-port))
    (newline (e::error-port))
    (e::reset)))
        
;;; e::WARNING

;     Describe the problem, then go on.

(c::define e::warning
  (c::lambda (message)
    (newline (e::error-port)) 
    (display "Warning: " (e::error-port))
    (display message (e::error-port))
    (display "  (Continuing) ..." (e::error-port))
    (newline (e::error-port))
    #t))
        
;;; c::GET-PURE-TAG

(c::define c::get-pure-tag
  (c::lambda (thing)
    (e::bit-and (e::get-tag thing) c::PURE_TYPE_CODE_BITS)))

;;; c::IMMUTABLE?

(c::define c::immutable?
  (c::lambda (thing)
    (not (zero? (e::bit-and (e::get-tag thing) c::IMMUTABLE_BIT)))))

;;; c::INTERPRETED-LAMBDA?

(c::define c::interpreted-lambda?
  (c::lambda (thing)
    (and (= (c::get-pure-tag thing) c::LAMBDA_PAIR_TAG)
         (not (null? (caddr (c::set-tag! thing c::PAIR_TAG)))))))

;;; c::LAMBDA-FORM?

(c::define c::lambda-form?
  (c::lambda (form)
    (and (pair? form)
         (or (eqv? (car form) 'lambda)
             (eqv? (car form) 'c::lambda))
         (list? form)
         (>= (length form) 3))))
                
;;; c::LAMBDA-FORM-POSSIBLE?

(c::define c::lambda-form-possible?
  (c::lambda (form)
    (and (pair? form)
         (or (eqv? (car form) 'lambda)
             (eqv? (car form) 'c::lambda)))))

;;; c::LAMBDA-SYNTAX-CHECK

(c::define c::lambda-syntax-check
  (c::lambda (form)
    (c::if (list? form)
      (c::if (>= (length form) 2)
        #t
        (e::error e::cm59))
      (c::begin (newline) (display (cddr form)) (e::error e::cm58)))))
                
;;; c::LISTIFY

;     Makes a pair out of an object that isn't.

(c::define c::listify
  (c::lambda ( thing )
    (c::if (pair? thing)
      thing
      (cons thing '()))))
          
;;; c::MAKE-IMMUTABLE!

;(c::define c::make-immutable! (c::lambda ( thing ) thing) )

(c::define c::make-immutable!
  (c::lambda ( object )
    (c::if (c::immutable? object)
      object
      (c::if (pair? object)
        (c::set-tag!
          (cons (c::make-immutable! (car object)) (c::make-immutable! (cdr object)))
          (e::bit-or c::PAIR_TAG c::IMMUTABLE_BIT))
          (c::if (vector? object)
            (c::set-tag!
              ((c::lambda (lyst)
                 (list->vector
                    (c::if (null? lyst)
                      '()
                       (cons (c::make-immutable! (car lyst)) (c::make-immutable! (cdr lyst))))))
               (vector->list object))
              (e::bit-or c::VECTOR_TAG c::IMMUTABLE_BIT))
            (c::if (string? object)
              (c::set-tag! object (e::bit-or (c::get-pure-tag object) c::IMMUTABLE_BIT))
              object))))))
          
;;; c::MAKE-PROPER-LIST

(c::define c::make-proper-list
  (c::lambda (object)
    (c::if (null? object)
      '()
      (c::if (pair? object)
        (cons (car object) (c::make-proper-list (cdr object)))
        (list object)))))

;########################################################
;
;    Macros:

;;; c::QQ-SYNTAX-CHECK

(c::define c::qq-syntax-check
  (c::lambda (form)
    (c::if (list? form)
      (c::if (= (length form) 2)
        #t
        (c::begin 
          (newline) (display form)
          (e::error (string-append e::cm13 (symbol->string (car form)) e::cm14))))
      (c::begin
        (newline) (display form)
        (e::error (string-append e::cm15 (symbol->string (car form)) "\""))))))

;;; c::QQ-PAIR

(c::define c::qq-pair
  (c::lambda (form car-form quotation-level)
    (c::if (memv car-form '(quasiquote unquote unquote-splicing))
      (c::begin
      (c::qq-syntax-check form)
      (c::if (eqv? car-form 'quasiquote)
         (c::if (= quotation-level 0)
           (c::qq (cadr form) (+ quotation-level 1))
           (cons 'cons (cons ''quasiquote (list (c::qq (cdr form) (+ quotation-level 1))))))
          (c::if (memv car-form '(unquote unquote-splicing))
            (c::if (<= quotation-level 0)
              (c::begin
                (newline) (display form)
                (e::error (string-append e::cm16 (symbol->string (car form)) e::cm17)))
              (c::if (= quotation-level 1)
                (c::qq (cadr form) (- quotation-level 1))
                (cons 
                  'cons 
                  (cons 
                    (list 'quote car-form) 
                    (list (cons 'list (list (c::qq (cadr form) (- quotation-level 1)))))))))
        #t)))
    (c::if (eqv? car-form 'quote)
      (cons 'cons (cons ''quote (list (c::qq (cdr form) quotation-level))))
      (c::if (= quotation-level 0)
        form
        (list
          (c::if (pair? car-form)
            (c::if (eqv? (caar form) 'unquote-splicing)
              'append
              'cons)
          'cons)
        (c::qq car-form quotation-level)
        (c::qq (cdr form) quotation-level))
 )))))

;;; c::QQ

;     Auxiliary for quasiquote and friends.

(c::define c::qq
  (c::lambda (form quotation-level)
    (c::if (null? form) 
      (list 'quote form)
    (c::if (vector? form)
      (c::if (= quotation-level 0)
        form
            (cons 'list->vector (list (c::qq (cdr (c::set-tag! form c::PAIR_TAG)) quotation-level))))
    (c::if (pair? form)
      (c::qq-pair form (car form) quotation-level)
    (c::if (= quotation-level 0)
      form
    (list 'quote form)))))))

;;; QUASIQUOTE

(c::define c::quasiquote-code
  (c::lambda (form) 
    (cons 'c::make-immutable! (list (c::qq form 0)))))

(e::macro quasiquote c::quasiquote-code)

;;; UNQUOTE

(c::define c::unquote-code
  (c::lambda (form) 
    (newline) (display form)
    (e::error e::cm18)))

(e::macro unquote c::unquote-code)

;;; UNQUOTE-SPLICING

(c::define c::unquote-splicing-code
  (c::lambda (form) 
    (newline) (display form)
    (e::error e::cm19)))

(e::macro unquote-splicing c::unquote-splicing-code)

;;; DELAY

(c::define c::delay-code
  (c::lambda (form)
    ((c::lambda (body)
       (cond ((null? body) (e::error e::cm48))
             ((not (= (length body) 1)) (newline) (display form) (e::error e::cm47))
             ((not (list? body)) (newline) (display form) (e::error e::cm46))
             (else
               (cons 
                 'c::set-tag!
                   (cons 
                     (cons 
                       'cons
                       (cons 
                         (cons 'lambda (cons '() (append body '())))
                         (cons #t '())))
                     (cons 'c::promise_tag '())))
	       )))
     (cdr form))))

(e::macro delay c::delay-code)

(c::define c::list-lambda-formals
  ;;
  ;; Suppose "sexp" is known to be the lambda-formals part of a
  ;; lambda expression.  List the variables which it contains.
  ;;
  (c::lambda( sexp )
    (if (null? sexp)
      '()
      (if (pair? sexp)
        (append (c::list-lambda-formals (car sexp))
                (c::list-lambda-formals (cdr sexp)))
        (list sexp)))))

(c::define c::check-lambda-syntax-details
  (lambda (lambda-expression)
    (let ((listed-lambda-formals (c::list-lambda-formals (cadr lambda-expression))))
      (if (null? listed-lambda-formals)
        #t
        (c::map-without-type-checking
          (c::lambda (formal-parameter)
            (if (not (symbol? formal-parameter))
              (c::begin
                (display "\nIn the lambda expression:\n\n" (e::error-port))
                (display lambda-expression (e::error-port))
                (display "\n\nthe formal parameter \"" (e::error-port))
                (display formal-parameter (e::error-port))
                (display  "\" is not a symbol." (e::error-port))
                (e::error "Incorrect syntax for lambda expression."))
            (if (e::permanent? formal-parameter)
              (c::begin
                (display "\nIn the lambda expression:\n\n" (e::error-port))
                (display lambda-expression (e::error-port))
                (display "\n\nthe formal parameter \"" (e::error-port))
                (display formal-parameter (e::error-port))
                (display  "\" is a permanent symbol." (e::error-port))
                (e::error "Incorrect syntax for lambda expression.")))))
            listed-lambda-formals)))))

(e::macro lambda
  (c::lambda (form)
    (c::check-lambda-syntax-details form)
    (cons 'c::lambda (cons (cadr form) (c::letrecify-defines (cddr form))))))

(e::macro begin
  (c::lambda (form)
    (if (and (list? form) (= (length form) 2))
      (cadr form)
      (cons 'c::begin (cdr form)))))

(c::define e::original-cwcc call-with-current-continuation)  
