;; Copyright (c) 2009 by Jay Reynolds Freeman

;; Library containing Wraith Scheme built-ins that are
;; provided as Scheme source code.

;; This file loads the functions but does not compile
;; them; that is done in "CompilePrimitiveLibrary.s".	

;;    This program is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    This program is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;
;;    Contact information:
;;
;;        EMail:       Jay_Reynolds_Freeman@mac.com
;;        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman
;;
;;        Paper Mail:  Jay Reynolds Freeman
;;                     Post Office Box 60628
;;                     Palo Alto, CA, 94306-0628

(display "PrimitiveLibrary.s:\n")
	
(c::define c::primitive-library-functions-list
  (list
    'call-with-values
    'port?
    'values
    'e::syntax-binding-binding
    'e::syntax-binding-bound?
    'e::syntax-binding-symbol
    'e::syntax-binding-properties
    'e::deep-copy
    'null-environment
    'scheme-report-environment
    'interaction-environment
    'eval
    'e::number->string-with-n-decimals
    'e::money->string
    'e::logic-constant?
    'e::non-printing-object?
    'c::pi
    'c::make-blocking-queue
    'c::last-pair
    'c::dequeue
    'c::enqueue
    'c::empty-queue?
    'c::purge-queue
    'c::make-lambda-immutable!
    'e::entropy-death
    'e::all?-aux
    'e::all?
    'e::any?-aux
    'e::any?
    'e::umber-nay->ing-stray
    'e::make-offsets
    'c::get-n-thousands
    'c::n-thousands-characters
    'c::get-n-hundreds
    'c::n-hundreds-characters
    'c::get-n-tens
    'c::n-tens-characters
    'c::get-n-ones
    'c::n-ones-characters
    'e::ing-stray->umber-nay
    'e::perform-applicative
    )
  )

(c::map-without-type-checking e::clear-permanent! c::primitive-library-functions-list)

;;(c::define values (c::lambda x (c::set-tag! x c::MULTIPLE_VALUES_TAG)))

(c::define values
   (c::lambda x 
     (if (> (length x) 1)
       (c::set-tag! x c::MULTIPLE_VALUES_TAG)
       (if (= (length x) 1)
         (car x)
         (e::error "\"values\" called with no arguments.")))))

(c::define port? (c::lambda (x) (or (input-port? x) (output-port? x))))

(c::define call-with-values 
  (c::lambda (producer consumer)
    (if (and (procedure? consumer) (procedure? producer))
      (let ((x (producer)))
        (if (e::multiple-values? x)
          (apply consumer (c::set-tag! x c::PAIR_TAG))
          (apply consumer (list x))))
      (c::begin
        (newline)
        (display "Procedure \"call-with-values\" applied to " (e::error-port))
        (display (list producer consumer) (e::error-port))
        (display ":" (e::error-port))
        (if (not (procedure? producer))
           (c::begin
             (newline) (display " -> " (e::error-port)) (display producer (e::error-port))
                       (display "\n    Expected a procedure (either a built-in or a lambda expression) for argument list item 0 (zero-based)." (e::error-port))
             ))
        (if (not (procedure? consumer))
           (c::begin
             (newline) (display " -> " (e::error-port)) (display consumer (e::error-port))
                       (display "\n    Expected a procedure (either a built-in or a lambda expression) for argument list item 1 (zero-based)." (e::error-port))
             ))
        (e::error
          (string-append
            "Marked argument"
            (if (and (not (procedure? consumer)) (not (procedure? producer)))
              "s have improper types"
              " has improper type")
            " for this operation."))))))

(c::define e::syntax-binding-binding
  (c::lambda (sb)
    (if (e::syntax-binding? sb)
      (if (not (cadr (c::set-tag! sb c::PAIR_TAG)))
        (c::begin
          (newline)
          (display "Procedure \"e::syntax-binding-binding\" applied to\n    " (e::error-port))
          (display sb (e::error-port))
          (display ":" (e::error-port))
          (e::error "Variable is not bound in this syntax binding."))
        (caddr (c::set-tag! sb 2)))
      (c::begin
        (newline)
        (display "Procedure \"e::syntax-binding-binding\" applied to\n    " (e::error-port))
        (display sb (e::error-port))
        (display ":" (e::error-port))
        (e::error "Argument has improper type for this operation.")))))

(c::define e::syntax-binding-bound?
  (c::lambda (sb)
    (if (e::syntax-binding? sb)
      (cadr (c::set-tag! sb c::PAIR_TAG))
      (c::begin
        (newline)
        (display "Procedure \"e::syntax-binding-bound?\" applied to\n    " (e::error-port))
        (display sb (e::error-port))
        (display ":" (e::error-port))
        (e::error "Argument has improper type for this operation.")))))

(c::define e::syntax-binding-symbol
  (c::lambda (sb)
    (if (e::syntax-binding? sb)
      (car (c::set-tag! sb c::PAIR_TAG))
      (c::begin
        (newline)
        (display "Procedure \"e::syntax-binding-symbol\" applied to\n    " (e::error-port))
        (display sb (e::error-port))
        (display ":" (e::error-port))
        (e::error "Argument has improper type for this operation.")))))

(c::define e::syntax-binding-properties
  (c::lambda (sb)
    (if (e::syntax-binding? sb)
      (cdddr (c::set-tag! sb c::PAIR_TAG))
      (c::begin
        (newline)
        (display "Procedure \"e::syntax-binding-properties\" applied to\n    " (e::error-port))
        (display sb (e::error-port))
        (display ":" (e::error-port))
        (e::error "Argument has improper type for this operation.")))))

(c::define e::deep-copy
  (c::lambda (x)
    (cond
      ((pair? x)   (cons (e::deep-copy (car x)) (e::deep-copy (cdr x))))
      ((vector? x) (list->vector (map e::deep-copy (vector->list x))))
      ((string? x) (string-copy x))
      (#t x))))

(c::define null-environment
  (c::lambda (n)
    (if (and (= n 5) (exact? n))
      (c::null-environment)
      (c::begin
        (display "Procedure \"null-environment\" called with version: " (e::error-port))
        (display n (e::error-port))
        (e::error "The only value of \"version\" allowed by Wraith Scheme is the exact integer 5.")))))

(c::define scheme-report-environment
  (c::lambda (n)
    (if (and (= n 5) (exact? n))
      (c::scheme-report-environment)
      (c::begin
        (display "Procedure \"scheme-report-environment\" called with version: " (e::error-port))
        (display n (e::error-port))
        (e::error "The only value of \"version\" allowed by Wraith Scheme is the exact integer 5.")))))

(c::define interaction-environment
  (c::lambda ()
    (c::top-level-environment)))

(c::define eval
  (c::lambda (quoted-thunk env)
    (c::eval-in-environment quoted-thunk env)))

(c::define e::number->string-with-n-decimals
  (lambda (x n)
    (if (not (number? x))
      (begin
        (newline (e::error-port))
        (display x (e::error-port))
        (newline (e::error-port))
        (e::error "\"e::number->string-with-n-decmals\" called with first argument that is not a number.")))
    (if (not (and (integer? n) (not (negative? n)) (< n 11) (exact? n)))
      (begin
        (newline (e::error-port))
        (display n (e::error-port))
        (newline (e::error-port))
        (e::error "\"e::number->string-with-n-decmals\" called with second argument that is not an exact  positive integer less than 11.")))
    (c::number->string-with-c-format x (string-append "%." (number->string n) "f"))))

(c::define e::money->string
  (lambda (x)
    (If (not (number? x))
      (begin
        (newline (e::error-port))
        (display x (e::error-port))
        (newline (e::error-port))
        (e::error "\"e::money->string\" called with argument that is not a number.")))
    (e::number->string-with-n-decimals x 2)))

(c::define e::logic-constant?
  (lambda (x)
    (or (eq? x #u) (eq? x #s))))

(c::define c::pi 3.1415926535897932384626433)
(c::define e::entropy-death #e9223372036854775807)

;; Stuff for kitten input queues:

(c::define (c::make-blocking-queue) (list '() 42))

(c::define c::last-pair
  (lambda (l)
    (if (null? (cdr l))
      l
      (c::last-pair (cdr l)))))

(c::define c::dequeue
  (lambda (queue)
    (let
      ((blockedItem (c::block (cdr queue))))
      (if (eq? (car queue) '())
        (begin
          (e::error "Attempt to dequeue an empty blocking queue")
          (c::release-block blockedItem)
          #t)
        (let*
          ((out (caar queue)))
          (set-car! queue (cdar queue))
          (c::release-block blockedItem)
          out)))))

(c::define c::enqueue
  (lambda (x queue)
    (let
      ((blockedItem (c::block (cdr queue))))
      (if (null? (car queue))
        (set-car! queue (cons x '()))
        (set-cdr! (c::last-pair (car queue)) (cons x '())))
      (c::release-block blockedItem)
      #t
      )))

(c::define c::empty-queue?
  (lambda( queue )
    (let*
      ((blockedItem (c::block (cdr queue)))
       (out (null? (car queue))))
      (c::release-block blockedItem)
      out
      )))

(c::define c::purge-queue
  (lambda (queue)
    (let
      ((blockedItem (c::block (cdr queue))))
      (set-car! queue '())
      (c::release-block blockedItem)
      #t
      )))

(c::define c::make-lambda-immutable!
  (lambda( lam )
    (if (procedure? lam)
      (let*
        ((lam-pair (c::set-tag! lam c::PAIR_TAG))
         (cadr-lambda (cadr lam-pair))
         (rest-of-lambda (cddr lam-pair)))
        (set-cdr! lam-pair (cons (c::make-immutable! cadr-lambda) rest-of-lambda))))
        #t))

(c::define e::all?-aux
  (lambda (lyst)
    (cond 
      ((null? lyst) #t)
      ((car lyst) (e::all?-aux (cdr lyst)))
      (#t #f))))

(c::define e::all?
  (lambda (lyst)
    (if (list? lyst)
      (e::all?-aux lyst)
      (c::begin
        (newline)
        (display "Procedure \"e::all?\" applied to " (e::error-port))
        (display (list lyst) (e::error-port))
        (display ":" (e::error-port))
        (display "\n    Expected a list for the argument." (e::error-port))
        (e::error "Argument has improper type for this operation.")))))

(c::define e::any?-aux
  (lambda( lyst )
    (cond 
      ((null? lyst) #f)
      ((car lyst) #t)
      (#t (e::any?-aux (cdr lyst))))))

(c::define e::any?
  (lambda (lyst)
    (if (list? lyst)
      (e::any?-aux lyst)
      (c::begin
        (newline)
        (display "Procedure \"e::any?\" applied to " (e::error-port))
        (display (list lyst) (e::error-port))
        (display ":" (e::error-port))
        (display "\n    Expected a list for the argument." (e::error-port))
        (e::error "Argument has improper type for this operation.")))))

;; Decimal to Roman number conversion ...

(c::define e::umber-nay->ing-stray
  (lambda (x) 
    (if (not (number? x))
      (begin
        (newline)
        (display "Procedure \"e::umber-nay->ing-stray\" applied to\n    " (e::error-port))
        (display x (e::error-port))
        (display ":" (e::error-port))
        (e::error "Argument has improper type for this operation.")))
    (if (and (number? x) (integer? x) (exact? x) (>= x 0) (<= x 3999))
      (if (= x 0)
        "N"                ;; For "Nullus" -- occasionally used for zero.
        (let
          ((mval (* (quotient x 1000) 1000))
           (hval (* (modulo (quotient x 100) 10) 100))
           (tval (* (modulo (quotient x 10) 10) 10))
           (uval (* (modulo (quotient x 1) 10) 1))
           (answer "")
           )
          (begin
            (cond ((= mval    0) #t)
                  ((= mval 1000) (c::set! answer "M"))
                  ((= mval 2000) (c::set! answer "MM"))
                  (#t            (c::set! answer "MMM")))
             
            (cond ((= hval   0) #t)
                  ((= hval 100) (c::set! answer (string-append answer "C")))
                  ((= hval 200) (c::set! answer (string-append answer "CC")))
                  ((= hval 300) (c::set! answer (string-append answer "CCC")))
                  ((= hval 400) (c::set! answer (string-append answer "CD")))
                  ((= hval 500) (c::set! answer (string-append answer "D")))
                  ((= hval 600) (c::set! answer (string-append answer "DC")))
                  ((= hval 700) (c::set! answer (string-append answer "DCC")))
                  ((= hval 800) (c::set! answer (string-append answer "DCCC")))
                  (#t           (c::set! answer (string-append answer "CM"))))
      
            (cond ((= tval  0) #t)
                  ((= tval 10) (c::set! answer (string-append answer "X")))
                  ((= tval 20) (c::set! answer (string-append answer "XX")))
                  ((= tval 30) (c::set! answer (string-append answer "XXX")))
                  ((= tval 40) (c::set! answer (string-append answer "XL")))
                  ((= tval 50) (c::set! answer (string-append answer "L")))
                  ((= tval 60) (c::set! answer (string-append answer "LX")))
                  ((= tval 70) (c::set! answer (string-append answer "LXX")))
                  ((= tval 80) (c::set! answer (string-append answer "LXXX")))
                  (#t          (c::set! answer (string-append answer "XC"))))
      
            (cond ((= uval 0) #t)
                  ((= uval 1) (c::set! answer (string-append answer "I")))
                  ((= uval 2) (c::set! answer (string-append answer "II")))
                  ((= uval 3) (c::set! answer (string-append answer "III")))
                  ((= uval 4) (c::set! answer (string-append answer "IV")))
                  ((= uval 5) (c::set! answer (string-append answer "V")))
                  ((= uval 6) (c::set! answer (string-append answer "VI")))
                  ((= uval 7) (c::set! answer (string-append answer "VII")))
                  ((= uval 8) (c::set! answer (string-append answer "VIII")))
                  (#t         (c::set! answer (string-append answer "IX"))))
      
            answer)))
    
      (number->string x))))
  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; e::make-offsets
;;
;; (make-offsets start-offset `((name0 ,size0) (name1 ,size1) (name2 ,size2) ...))
;;
;;   acts as if you had typed
;;
;; (define name0-offset start-offset)
;; (define name1-offset (+ name0-offset size0))
;; (define name2-offset (+ name1-offset size1))
;;
;;    ...                                                                                                                             

(define (e::make-offsets initial-offset pairs)
  (if (not (integer? initial-offset))
    (c::begin
      (newline)
      (display "Procedure \"e::make-offsets\" applied to " (e::error-port))
      (display (list initial-offset pairs) (e::error-port))
      (display ":\n -> " (e::error-port))
      (display initial-offset (e::error-port))
      (display "\n Expected a number for argument list item 0 (zero-based)." (e::error-port))
      (e::error "Argument has improper type for this operation.")))
  (if (not (pair? pairs))
    (c::begin
      (newline)
      (display "Procedure \"e::make-offsets\" applied to " (e::error-port))
      (display (list initial-offset pairs) (e::error-port))
      (display ":\n -> " (e::error-port))
      (display pairs (e::error-port))
      (display "\n Expected a non-empty list for argument list item 1 (zero-based)." (e::error-port))
      (e::error "Argument has improper type for this operation.")))
  (if (not (pair? (car pairs)))
    (c::begin
      (newline)
      (display "Procedure \"e::make-offsets\" applied to " (e::error-port))
      (display (list initial-offset pairs) (e::error-port))
      (display ":\n -> " (e::error-port))
      (display pairs (e::error-port))
      (display "\n Expected a non-empty list for the car of argument list item 1 (zero-based)." (e::error-port))
      (e::error "Argument has improper type for this operation.")))
  (letrec
    ((last-value '())
     (do-one-form
       (lambda (previous-offset previous-size name current-size)
;;         (display (list previous-offset previous-size name current-size)) (newline)  ;; DEBUG
         (if (not (integer? previous-offset))
           (c::begin
             (newline)
             (display "Auxiliary procedure \"do-one-form\" for \"e::make-offsets\" applied to " (e::error-port))
             (display (list previous-offset previous-size name current-size) (e::error-port))
             (display ":\n -> " (e::error-port))
             (display previous-offset (e::error-port))
             (display "\n Expected a number for argument list item 0 (zero-based)." (e::error-port))
             (e::error "Argument has improper type for this operation.")))
         (if (not (integer? previous-size))
           (c::begin
             (newline)
             (display "Auxiliary procedure \"do-one-form\" for \"e::make-offsets\" applied to " (e::error-port))
             (display (list previous-offset previous-size name current-size) (e::error-port))
             (display ":\n -> " (e::error-port))
             (display previous-size (e::error-port))
             (display "\n Expected a number for argument list item 1 (zero-based)." (e::error-port))
             (e::error "Argument has improper type for this operation.")))
         (if (not (symbol? name))
           (c::begin
             (newline)
             (display "Auxiliary procedure \"do-one-form\" for \"e::make-offsets\" applied to " (e::error-port))
             (display (list previous-offset previous-size name current-size) (e::error-port))
             (display ":\n -> " (e::error-port))
             (display name (e::error-port))
             (display "\n Expected a symbol for argument list item 2 (zero-based)." (e::error-port))
             (e::error "Argument has improper type for this operation.")))
         (if (not (integer? current-size))
           (c::begin
             (newline)
             (display "Auxiliary procedure \"do-one-form\" for \"e::make-offsets\" applied to " (e::error-port))
             (display (list previous-offset previous-size name current-size) (e::error-port))
             (display ":\n -> " (e::error-port))
             (display current-size (e::error-port))
             (display "\n Expected a number for argument list item 3 (zero-based)." (e::error-port))
             (e::error "Argument has improper type for this operation.")))
         (c::set! last-value (+ previous-offset previous-size current-size))
         `(define
            ,(string->symbol (string-append (symbol->string name) "-offset"))
            ,(+ previous-offset previous-size))
          ))
     (do-forms
       (lambda (previous-offset previous-size pairs)
         (if (not (integer? previous-offset))
           (c::begin
             (newline)
             (display "Auxiliary procedure \"do-forms\" for \"e::make-offsets\" applied to " (e::error-port))
             (display (list previous-offset previous-size pairs) (e::error-port))
             (display ":\n -> " (e::error-port))
             (display previous-offset (e::error-port))
             (display "\n Expected a number for argument list item 0 (zero-based)." (e::error-port))
             (e::error "Argument has improper type for this operation.")))
         (if (not (integer? previous-size))
           (c::begin
             (newline)
             (display "Auxiliary procedure \"do-forms\" for \"e::make-offsets\" applied to " (e::error-port))
             (display (list previous-offset previous-size pairs) (e::error-port))
             (display ":\n -> " (e::error-port))
             (display previous-size (e::error-port))
             (display "\n Expected a number for argument list item 1 (zero-based)." (e::error-port))
             (e::error "Argument has improper type for this operation.")))
         (if (not (list? pairs))
           (c::begin
             (newline)
             (display "Auxiliary procedure \"do-forms\" for \"e::make-offsets\" applied to " (e::error-port))
             (display (list previous-offset previous-size pairs) (e::error-port))
             (display ":\n -> " (e::error-port))
             (display pairs (e::error-port))
             (display "\nExpected a list for argument list item 2 (zero-based)." (e::error-port))
             (e::error "Argument has improper type for this operation.")))
         (cond ((null? pairs) '())
               (#t
                 (cons (do-one-form previous-offset previous-size (caar pairs) (cadar pairs))
                       (do-forms (+ previous-offset previous-size) (cadar pairs) (cdr pairs)))))))
     )
    (display (list initial-offset pairs (caar pairs))) (newline)
    (eval
      `(begin ,@(do-forms 0 initial-offset pairs) ,(- last-value initial-offset))
      (interaction-environment))))

(c::define c::get-n-thousands
  (lambda (s)
    (let ((l (string-length s)))
      (cond
        ((and (>= l 3) (string=? (substring s 0 3) "MMM")) 3)
        ((and (>= l 2) (string=? (substring s 0 2) "MM")) 2)
        ((and (>= l 1) (string=? (substring s 0 1) "M")) 1)
        (else 0)))))

(c::define c::n-thousands-characters
  (lambda (s)
    (let ((l (string-length s)))
      (cond
        ((and (>= l 3) (string=? (substring s 0 3) "MMM")) 3)
        ((and (>= l 2) (string=? (substring s 0 2) "MM")) 2)
        ((and (>= l 1) (string=? (substring s 0 1) "M")) 1)
        (else 0)))))

(c::define c::get-n-hundreds
  (lambda (s)
    (let ((l (string-length s)))
      (cond
        ((and (>= l 2) (string=? (substring s 0 2) "CM"))   9)
        ((and (>= l 4) (string=? (substring s 0 4) "DCCC")) 8)
        ((and (>= l 3) (string=? (substring s 0 3) "DCC"))  7)
        ((and (>= l 2) (string=? (substring s 0 2) "DC"))   6)
        ((and (>= l 1) (string=? (substring s 0 1) "D"))    5)
        ((and (>= l 2) (string=? (substring s 0 2) "CD"))   4)
        ((and (>= l 3) (string=? (substring s 0 3) "CCC"))  3)
        ((and (>= l 2) (string=? (substring s 0 2) "CC"))   2)
        ((and (>= l 1) (string=? (substring s 0 1) "C"))    1)
        (else 0)))))

(c::define c::n-hundreds-characters
  (lambda (s)
    (let ((l (string-length s)))
      (cond
        ((and (>= l 2) (string=? (substring s 0 2) "CM"))   2)
        ((and (>= l 4) (string=? (substring s 0 4) "DCCC")) 4)
        ((and (>= l 3) (string=? (substring s 0 3) "DCC"))  3)
        ((and (>= l 2) (string=? (substring s 0 2) "DC"))   2)
        ((and (>= l 1) (string=? (substring s 0 1) "D"))    1)
        ((and (>= l 2) (string=? (substring s 0 2) "CD"))   2)
        ((and (>= l 3) (string=? (substring s 0 3) "CCC"))  3)
        ((and (>= l 2) (string=? (substring s 0 2) "CC"))   2)
        ((and (>= l 1) (string=? (substring s 0 1) "C"))    1)
        (else 0)))))

(c::define c::get-n-tens
  (lambda (s)
    (let ((l (string-length s)))
      (cond
        ((and (>= l 2) (string=? (substring s 0 2) "XC"))   9)
        ((and (>= l 4) (string=? (substring s 0 4) "LXXX")) 8)
        ((and (>= l 3) (string=? (substring s 0 3) "LXX"))  7)
        ((and (>= l 2) (string=? (substring s 0 2) "LX"))   6)
        ((and (>= l 1) (string=? (substring s 0 1) "L"))    5)
        ((and (>= l 2) (string=? (substring s 0 2) "XL"))   4)
        ((and (>= l 3) (string=? (substring s 0 3) "XXX"))  3)
        ((and (>= l 2) (string=? (substring s 0 2) "XX"))   2)
        ((and (>= l 1) (string=? (substring s 0 1) "X"))    1)
        (else 0)))))

(c::define c::n-tens-characters
  (lambda (s)
    (let ((l (string-length s)))
      (cond
        ((and (>= l 2) (string=? (substring s 0 2) "XC"))   2)
        ((and (>= l 4) (string=? (substring s 0 4) "LXXX")) 4)
        ((and (>= l 3) (string=? (substring s 0 3) "LXX"))  3)
        ((and (>= l 2) (string=? (substring s 0 2) "LX"))   2)
        ((and (>= l 1) (string=? (substring s 0 1) "L"))    1)
        ((and (>= l 2) (string=? (substring s 0 2) "XL"))   2)
        ((and (>= l 3) (string=? (substring s 0 3) "XXX"))  3)
        ((and (>= l 2) (string=? (substring s 0 2) "XX"))   2)
        ((and (>= l 1) (string=? (substring s 0 1) "X"))    1)
        (else 0)))))

(c::define c::get-n-ones
  (lambda (s)
    (let ((l (string-length s)))
      (cond
        ((and (>= l 2) (string=? (substring s 0 2) "IX"))   9)
        ((and (>= l 4) (string=? (substring s 0 4) "VIII")) 8)
        ((and (>= l 3) (string=? (substring s 0 3) "VII"))  7)
        ((and (>= l 2) (string=? (substring s 0 2) "VI"))   6)
        ((and (>= l 1) (string=? (substring s 0 1) "V"))    5)
        ((and (>= l 2) (string=? (substring s 0 2) "IV"))   4)
        ((and (>= l 3) (string=? (substring s 0 3) "III"))  3)
        ((and (>= l 2) (string=? (substring s 0 2) "II"))   2)
        ((and (>= l 1) (string=? (substring s 0 1) "I"))    1)
        (else 0)))))

(c::define c::n-ones-characters
  (lambda (s)
    (let ((l (string-length s)))
      (cond
        ((and (>= l 2) (string=? (substring s 0 2) "IX"))   2)
        ((and (>= l 4) (string=? (substring s 0 4) "VIII")) 4)
        ((and (>= l 3) (string=? (substring s 0 3) "VII"))  3)
        ((and (>= l 2) (string=? (substring s 0 2) "VI"))   2)
        ((and (>= l 1) (string=? (substring s 0 1) "V"))    1)
        ((and (>= l 2) (string=? (substring s 0 2) "IV"))   2)
        ((and (>= l 3) (string=? (substring s 0 3) "III"))  3)
        ((and (>= l 2) (string=? (substring s 0 2) "II"))   2)
        ((and (>= l 1) (string=? (substring s 0 1) "I"))    1)
        (else 0)))))

(c::define e::ing-stray->umber-nay
  (lambda (given-string)
    (if (= (string-length given-string) 0)
      #f
      (let
        ((s1000 (list->string (map char-upcase (string->list given-string))))
         (l (string-length given-string))
         )         
        (if (string=? s1000 "N")
          0
          (let*
            ((n-thousands (c::get-n-thousands s1000))
             (past-thousands (c::n-thousands-characters s1000))
             (s100 (if (< past-thousands l) (substring s1000 past-thousands l) ""))
             (n-hundreds (c::get-n-hundreds s100))
             (past-hundreds (c::n-hundreds-characters s100))
             (s10 (if (< past-hundreds (string-length s100)) (substring s100 past-hundreds (string-length s100)) ""))
             (n-tens (c::get-n-tens s10))
             (past-tens (c::n-tens-characters s10))
             (s1 (if (< past-tens (string-length s10)) (substring s10 past-tens (string-length s10)) ""))
             (s1 (substring s10 past-tens (string-length s10)))
             (n-ones (c::get-n-ones s1))
             (past-ones (c::n-ones-characters s1))
             )
            (if (= (+ past-thousands past-hundreds past-tens past-ones) l)
              (+
                (* 1000 n-thousands)
                (*  100 n-hundreds)
                (*   10 n-tens)
                n-ones)
              #f)))))))

(c::define e::perform-applicative
  (lambda (thunk)
    (c::set-applicative-bit-n-down! 0)
    (thunk)))
