;; Copyright (c) 2009 by Jay Reynolds Freeman

;;    This program is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    This program is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;
;;    Contact information:
;;
;;        EMail:       Jay_Reynolds_Freeman@mac.com
;;        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman
;;
;;        Paper Mail:  Jay Reynolds Freeman
;;                     Post Office Box 60628
;;                     Palo Alto, CA, 94306-0628

(display "R5Macros.s:\n")

(c::define c::r5-macros-list
  (list
    'c::identifier?
    'c::keyword?
    'c::ellipsis?
    'c::identifier-zero-or-more?
    'c::pattern-datum?
    'c::pattern-identifier?
    'c::template-datum?
    'c::pattern-keyword
    'c::pattern-variable?
    'c::variable-identifier
    'c::n-ellipses?
    'c::how-I-got-here
    'c::how-I-got-to-list
    'c::how-I-got-here-from-list
    'c::list-variable?
    'c::value-grabber
    'c::success-checker
    'c::follow-directions-and-return-result
    'c::tack-on-directions
    'c::tack-on
    'c::pattern-one-or-more-ellipsis?
    'c::pattern-one-or-more-dot-pattern?
    'c::pattern-zero-or-more?
    'c::pattern?
    'c::ellipsis-follows-keyword?
    'c::template-and-ellipsis?
    'c::template-element-one-or-more-template?
    'c::template-element-zero-or-more?
    'c::template?
    'c::template-element?
    'c::syntax-rule?
    'c::syntax-rule-zero-or-more?
    'c::transformer-spec?
    'c::pattern-has-correct-keyword?
    'c::patterns-have-correct-keyword?
    'c::transformer-spec-with-keyword?
    'c::syntax-spec?
    'c::make-pattern-variable-structure

    'c::make-syntax-binding
    'c::make-syntax-binding-with-variable
    'c::traverse-pattern-vector
    'c::pattern-variable-structures-aux
    'c::transform-raw-variable-discovery
    'c::pattern-variable-structures
    'c::insert-define-time-syntax-bindings-aux
    'c::insert-define-time-syntax-bindings
    'c::cdr-contains-only-ellipsis?
    'c::match-aux
    'c::match-with-patvars?
    'c::list-lambda-formals-sbs
    'c::subst-sbs-in-sexp
    'c::subst-many-sbs-in-sexp
    'c::match-sb?-with-type-with-patvar-list
    'c::list-patvars-within
    'c::cons-up-deep-copies
    'c::substitute-sb-vars-in-body
    'c::strip-let-list-in-environment
    'c::remove-pattern-variables
    'c::collect-let-list-in-environment
    'c::insert-syntax-bindings-of-type-from-environment
    'c::syntax-bound-entity?
    'c::quotation-level-change
    'c::quotation-level-change-b
    'c::quoted-sbs
    'c::quasiquoted-sbs
    'c::substitute-gensymed-lambda-formals
    'c::gensym-lambdas
    'c::remove-syntax-bindings
    'c::expand-partial-transformer
    'c::make-partial-transformer 
    'c::find-matching-ppt
    'c::check-uniqueness-with-sexp-and-pattern
    'c::check-for-same-keyword
    'c::extract-simple-variable-names
    'c::foo-syntax-syntax-check
    'c::define-syntax-code
    'c::macro-body-code
    'c::substitutions-from-finders
    'c::syntax-rules-code
    'syntax-rules
    'e::syntax-macro-body
    ))

;; What follows is Scheme procedures to implement pattern-matching.

(c::define c::identifier? symbol?)

(c::define c::keyword? symbol?)

(c::define c::ellipsis?
  (c::lambda (x)
    (equal? x '...)))

(c::define c::identifier-zero-or-more?
  (c::lambda (x)
    (or (null? x)
        (and (pair? x)
             (c::identifier? (car x))
             (c::identifier-zero-or-more? (cdr x))))))

(c::define c::pattern-datum?
  (c::lambda (x)
    (or (vector? x)
        (string? x)
        (char? x)
        (boolean? x)
        (number? x))))

(c::define c::pattern-identifier?
  (c::lambda (x)
    (and (symbol? x) (not (c::ellipsis? x)))))

(c::define c::template-datum? c::pattern-datum?)

(c::define c::pattern-keyword
  (c::lambda (pat)
    (if (pair? pat)
      (car pat)
      pat)))

(c::define c::pattern-variable?
  (c::lambda (x pat-kwd lits)
    (and (c::identifier? x)
       (not (c::ellipsis? x))
       (not (equal? x pat-kwd))
         (not (memv x lits)))))

(c::define c::pattern-one-or-more-ellipsis?
  (c::lambda (x)
     (and
       (pair? x)
       (c::pattern? (car x))
       (or (equal? (cdr x) (list '...))
           (c::pattern-one-or-more-ellipsis? (cdr x))))))

(c::define c::pattern-one-or-more-dot-pattern?
  (c::lambda (x)
    (and
      (pair? x)
      (c::pattern? (car x))
      (or (c::pattern? (cdr x))
          (pattern-one-or-more-dot-pattern (cdr x))))))

(c::define c::pattern-zero-or-more?
  (c::lambda (x)
    (or (null? x)
        (and (pair? x)
             (c::pattern? (car x))
             (c::pattern-zero-or-more? (cdr x))))))

(c::define c::pattern-starts-with-literal?
  (c::lambda (pat lits)
    (and (not (null? lits))
         (not (null? pat))
         (list? pat)
         (member (car pat) lits))))

(c::define c::pattern?
  (c::lambda (x)
    (cond ((c::pattern-identifier? x) #t)
          ((c::pattern-datum? x)      #t)
          ((list? x)
           (or
             (c::pattern-zero-or-more? x)
             (c::pattern-one-or-more-ellipsis? x)))
          (#t
            (c::pattern-one-or-more-dot-pattern? x)))))

(c::define c::ellipsis-follows-keyword?
  (c::lambda (x)
    (and (list? x)
         (>= (length x) 2)
         (c::ellipsis? (cadr x)))))

(c::define c::template-and-ellipsis?
  (c::lambda (x)
    (and (list? x)
        (= (length x) 2)
        (c::ellipsis? (cadr x))
        (c::template? (car x)))))

(c::define c::template-element?
  (c::lambda (x)
    (or (c::template-and-ellipsis? x)
        (c::template? x))))

(c::define c::template-element-one-or-more-template?
  (c::lambda (x)
    (and
      (pair? x)
      (c::template-element? (car x))
      (or (c::template? (cdr x))       ;; CHECK -- should this be "template-element?"?
          (c::template-element-one-or-more-template? (cdr x))))))

(c::define c::template-element-zero-or-more?
  (c::lambda (x)
    (cond ((null? x) #t)                                               ;; "zero ..."
          ((list? x)                                                   ;; Safe for car ...
           (if (c::template? (car x))                                  ;; Gotta have a template first, in which case ...
             (cond ((and (>= (length x) 2) (c::ellipsis? (cadr x)))    ;; <template> ... <maybe more templates>
                    (c::template-element-zero-or-more? (cddr x)))
                   (#t (c::template-element-zero-or-more? (cdr x))))   ;; <template> <maybe more templates>
             #f))                                                      ;; Did *not* have a template first.
          (#t #f))))                                                   ;; Structure doesn't fit.

(c::define c::template?
  (c::lambda (x)
    (or (c::pattern-identifier? x)
        (c::template-element-zero-or-more? x)
        (c::template-element-one-or-more-template? x)
        (c::template-datum? x))))

(c::define c::syntax-rule?
  (c::lambda (x)
    (and
      (list? x)
      (= (length x) 2)
      (and (pair? (car x)) (c::identifier? (caar x)))
      (c::pattern? (car x))
      (and
        (c::template? (cadr x))
        (not (c::ellipsis-follows-keyword? (cadr x)))))))

(c::define c::syntax-rule-zero-or-more?
  (c::lambda (x)
    (or (null? x)
        (and (pair? x)
             (c::syntax-rule? (car x))
             (c::syntax-rule-zero-or-more? (cdr x))))))

(c::define c::transformer-spec?
  (c::lambda (x)
    (and
      (list? x)
      (>= (length x) 3)
      (equal? (car x) 'syntax-rules)
      (c::identifier-zero-or-more? (cadr x))
      (c::syntax-rule-zero-or-more? (cddr x)))))

(c::define c::pattern-has-correct-keyword?
  (c::lambda (pat kwd)
     (equal? (car pat) kwd)))

(c::define c::patterns-have-correct-keyword?
  (c::lambda (pats kwd)
    (if (null? pats)
      #t
      (if (c::pattern-has-correct-keyword? (car pats) kwd)
        (c::patterns-have-correct-keyword? (cdr pats) kwd)
        (c::begin (c::patterns-have-correct-keyword? (cdr pats) kwd) #f)))))

(c::define c::transformer-spec-with-keyword?
  (c::lambda (kwd x)
    (and
      (list? x)
      (>= (length x) 2)
      (equal? (car x) 'syntax-rules)
      (c::identifier-zero-or-more? (cadr x))
      (c::syntax-rule-zero-or-more? (cddr x))
      (c::patterns-have-correct-keyword? (map car (cddr x)) kwd))))
        
(c::define c::syntax-spec?
  (c::lambda (x)
    (and
      (list? x)
      (= (length x) 2)
      (c::keyword? (car x))
      (c::transformer-spec-with-keyword? (car x) (cadr x)))))

(c::define c::variable-identifier car)

(c::define c::n-ellipses? cadr)

(c::define c::how-I-got-here caddr)

(c::define c::how-I-got-to-list cadddr)

(c::define c::how-I-got-here-from-list
  (c::lambda (lyst)
    (car (cddddr lyst))))

(c::define c::list-variable? cadr)

(c::define c::value-grabber caddr)

(c::define c::success-checker cadddr)

(c::define c::follow-directions-and-return-result
  (c::lambda (start directions)
    (if (null? directions)
      (values #t start directions)
      (case (car directions)
        ((car)
         (if (pair? start)
           (c::follow-directions-and-return-result (car start) (cdr directions))
           (values #f start directions)))
        ((cdr)
         (if (pair? start)
           (c::follow-directions-and-return-result (cdr start) (cdr directions))
           (values #f start directions)))
        ((vector-ref)
         (if (and (vector? start)
                  (pair? (cdr directions))
                  (integer? (cadr directions))
                  (> (vector-length start) (cadr directions)))
           (c::follow-directions-and-return-result (vector-ref start (cadr directions)) (cddr directions))
           (values #f start directions)))
        (else
           (display "\nIn c::follow-directions-and-return-result, the directions are:\n  " (e::error-port))
           (display directions (e::error-port))
           (e::error "Directions steps must be \"... car ...\", \"... cdr ...\", or \"... vector-ref n ...\"."))))))

(c::define c::tack-on-directions
  (c::lambda (next finder)
    (cons next finder)))

(c::define c::tack-on
  (c::lambda (stuff finder)
    (let ((next (car stuff)) (after (cadr stuff)))
      (if after
        (list next finder after )    ;; e.g., (vector-ref (...) 42)
        (if next
          (if (pair? finder)
            (let ((current (car finder)))
             (if (equal? next 'car)
               (let
                 ((apair
                    (assoc current
                      '((car     caar) (cdr     cadr) (caar   caaar) (cadr   caadr)
                        (cdar   cadar) (cddr   caddr) (caaar caaaar) (caadr caaadr)
                        (cadar caadar) (caddr caaddr) (cdaar cadaar) (cdadr cadadr)
                        (cddar caddar) (cdddr cadddr)
                        ))))
                 (if apair
                   (cons (cadr apair) (cdr finder))
                   (list 'car finder)))
               (if (equal? next 'cdr)
                 (let
                   ((apair
                      (assoc current
                        '((car     cdar) (cdr     cddr) (caar   cdaar) (cadr   cdadr)
                          (cdar   cddar) (cddr   cdddr) (caaar cdaaar) (caadr cdaadr)
                          (cadar cdadar) (caddr cdaddr) (cdaar cddaar) (cdadr cddadr)
                          (cddar cdddar) (cdddr cddddr)
                          ))))
                   (if apair
                     (cons (cadr apair) (cdr finder))
                     (list 'cdr finder)))
                 (list next finder))))
            (list next finder))
          finder)))))

;; MATERIAL DEALING WITH PATTERN-MATCHING STOPS HERE ...

;; This next returns a list of items, each of the form:
;; 
;; (
;;  variable-identifier       ;; symbol
;;  n-ellipses                ;; number of ellipses governing this variable
;;                            ;; 0: Variable that can be bound to at most one thing
;;                            ;; 1: E.g., "z" in (x y z ...)
;;                            ;; 2: E.g., "z" in ((x y z ...) ...)
;;  how-I-got-here            ;; function to pull the symbol out of the pattern
;;  how-I-got-to-list         ;; if list, how to get to the "..." list
;;  how-I-got-here-from-list  ;; if list, how to get from an element of the "..." list to the variable
;;  syntax-binding            ;; syntax binding for the variable
;;  )

(c::define c::make-pattern-variable-structure
  (c::lambda (vid ne high higtl highfl)
      (list vid ne high higtl highfl)))

(c::define c::make-syntax-binding
  (c::lambda (variable bound-p value kind)
    (c::set-tag!
      (list variable bound-p value kind)
      c::SYNTAX_BINDING_TAG)
    )) 

(c::define c::make-syntax-binding-with-variable
  (c::lambda (variable environment kind)
  ;;
  ;; Return a syntax binding of the given variable in the given environment.
  ;;
  (if (c::bound-instance-in-environment? variable environment)
    (c::set-tag! (list variable #t (c::symeval-in-environment variable environment) kind) c::SYNTAX_BINDING_TAG)
    (c::set-tag! (list variable #f 'unbound-variable                                kind ) c::SYNTAX_BINDING_TAG))))
    
(c::define c::pattern-variable-structures-aux
  (c::lambda (full-pattern partial-pattern pat-kwd lits variables-so-far how-many-ellipses
           constructor-so-far constructor-to-list constructor-since-list tacked-on-last environment last-constructor-so-far)
    (cond
      ((null? partial-pattern) variables-so-far)
      ((c::pattern-variable? partial-pattern pat-kwd lits) 
        (case how-many-ellipses
          ((2) (cons (c::make-pattern-variable-structure partial-pattern 2
                       (e::cons-with-continuation `(c::lambda (x) ,constructor-so-far))
                       (e::cons-with-continuation `(c::lambda (x) ,last-constructor-so-far))
                       (reverse constructor-since-list))
                     variables-so-far))
          ((1) (cons (c::make-pattern-variable-structure partial-pattern 1
                       (e::cons-with-continuation `(c::lambda (x) ,constructor-so-far))
                       (e::cons-with-continuation `(c::lambda (x) ,constructor-to-list))
                       (reverse constructor-since-list))
                     variables-so-far))
          ((0) (cons (c::make-pattern-variable-structure partial-pattern 0
                       (e::cons-with-continuation `(c::lambda (x) ,constructor-so-far)) #f #f)
                     variables-so-far))
          (else
            (newline (e::error-port)) (display full-pattern (e::error-port)) (newline (e::error-port))
            (display "In the pattern shown, pattern variable \"" (e::error-port)) (display partial-pattern (e::error-port))
            (display "\" appears within the scope of " (e::error-port)) (display how-many-ellipses (e::error-port))
            (display " ellipses.\n" (e::error-port))
            (e::error "That's too many ellipses for Wraith Scheme's hygienic macro system."))
          ))
      ((pair? partial-pattern)
        (if (and (pair? (cdr partial-pattern)) (c::ellipsis? (cadr partial-pattern))) 

          ;; An ellipsis is coming up -- which means, among other things, no need to recurse on (cdr partial-pattern) 

          (c::pattern-variable-structures-aux full-pattern (car partial-pattern) pat-kwd lits variables-so-far
             (+ how-many-ellipses 1)
             (c::tack-on '(car #f) constructor-so-far)
             (c::tack-on tacked-on-last constructor-to-list)
             (if (> how-many-ellipses 0)
               (if constructor-since-list (c::tack-on-directions 'car constructor-since-list) #f)
               '())
             '(car #f)
             environment
             (if (= how-many-ellipses 0) constructor-so-far last-constructor-so-far))

          ;; There is *not* an ellipsis coming up; recurse on both car and cdr.

          (c::pattern-variable-structures-aux   full-pattern (cdr partial-pattern) pat-kwd lits
            (c::pattern-variable-structures-aux full-pattern (car partial-pattern) pat-kwd lits variables-so-far
               how-many-ellipses 
               (c::tack-on '(car #f)          constructor-so-far )
               (if (> how-many-ellipses 0) constructor-to-list (c::tack-on tacked-on-last constructor-to-list ))
               (if constructor-since-list (c::tack-on-directions 'car constructor-since-list) #f)
               '(car #f)
               environment
               (if (= how-many-ellipses 0) constructor-so-far last-constructor-so-far))
            how-many-ellipses
            (c::tack-on '(cdr #f)           constructor-so-far )
            (if (> how-many-ellipses 0) constructor-to-list (c::tack-on tacked-on-last constructor-to-list ))
            (if constructor-since-list (c::tack-on-directions 'cdr constructor-since-list) #f)
            '(cdr #f)
            environment
            (if (= how-many-ellipses 0) constructor-so-far last-constructor-so-far))))
      ((vector? partial-pattern)
         (c::traverse-pattern-vector full-pattern partial-pattern pat-kwd lits variables-so-far how-many-ellipses
           constructor-so-far constructor-to-list constructor-since-list tacked-on-last environment constructor-so-far))
      (#t variables-so-far))))

(c::define c::traverse-pattern-vector
  (c::lambda (full-pattern a-vector
           pat-kwd lits variables-so-far how-many-ellipses
           constructor-so-far constructor-to-list constructor-since-list
           tacked-on-last environment last-constructor-so-far)
    (let*
      ((vlist (vector->list a-vector))
       (lnth (vector-length a-vector))
       (nums '())
       )
      (if (not (c::ellipsis? (car (reverse vlist))))
        (c::begin
           (do ((i 0 (+ i 1)))
              ((>= i lnth) nums)
              (c::set! nums (cons i nums)))
           (append variables-so-far
             (apply append
                    (c::map-without-type-checking (c::lambda (element i)
                           (c::pattern-variable-structures-aux full-pattern element pat-kwd lits '()
                             how-many-ellipses
                             (c::tack-on (list 'vector-ref i) constructor-so-far)
                             (if (> how-many-ellipses 0) constructor-to-list (c::tack-on tacked-on-last constructor-to-list))
                             (if constructor-since-list  (cons i (cons 'vector-ref constructor-since-list)) #f)
                             (list 'vector-ref i)
                             environment last-constructor-so-far)
                           )
                         vlist
                         (reverse nums)))))
        (c::begin
           (do ((i 0 (+ i 1)))
              ((>= i (- lnth 2)) nums)
              (c::set! nums (cons i nums)))
           (append variables-so-far
             (apply append
                    (c::map-without-type-checking (c::lambda (element i)
                           (c::pattern-variable-structures-aux full-pattern element pat-kwd lits '()
                             how-many-ellipses
                             (c::tack-on (list 'vector-ref i) constructor-so-far)
                             (if (> how-many-ellipses 0) constructor-to-list (c::tack-on tacked-on-last constructor-to-list))
                             (if constructor-since-list (cons i (cons 'vector-ref constructor-since-list)) #f)
                             (list 'vector-ref i)
                             environment last-constructor-so-far)
                           )
                         (reverse (cddr (reverse vlist)))
                         (reverse nums)))
             (c::pattern-variable-structures-aux full-pattern (cadr (reverse vlist)) pat-kwd lits '()
               (+ how-many-ellipses 1)
               (c::tack-on (list 'vector-ref (- lnth 2)) constructor-so-far)
               (c::tack-on (list 'vector->list #f) (c::tack-on tacked-on-last constructor-to-list))
               '()
               (list 'vector-ref (- lnth 2))
               environment last-constructor-so-far)))))))

(c::define c::transform-raw-variable-discovery
  (c::lambda (rvd sexp)
    (let*
      ((n-ellipses (c::n-ellipses? rvd))
       (islist (positive? n-ellipses))
       (discoverer (c::how-I-got-here rvd))
       (directions (if islist (c::how-I-got-here-from-list rvd)) #f)
       (list-finder (if islist (c::how-I-got-to-list rvd) #f))
       (local-value-grabber
         (if islist
           (if (= n-ellipses 1)
             (let*
               ((here-from-list 
                  (c::lambda (form) 
                    (call-with-values         
                      (c::lambda () (c::follow-directions-and-return-result form directions))
                      (c::lambda (succeeded? result directions)
                        (if succeeded?
                          (list result)
                          (c::begin
                            (newline (e::error-port))
                            (display "Macro definition: " (e::error-port))(display sexp (e::error-port)) (newline (e::error-port))
                            (display "Could not match definition at this point in a macro call: " (e::error-port))
                            (display form (e::error-port)) (newline (e::error-port))
                            (e::error "Mismatch occurred after an ellipsis."))))))))
               (c::lambda (pat) (c::map-without-type-checking here-from-list (list-finder pat))))

             (let*                       ;; Case for n-ellipses == 2 starts here:
               ((here-from-list 
                  (c::lambda (form) 
                    (call-with-values         
                      (c::lambda () (c::follow-directions-and-return-result form directions))
                      (c::lambda (succeeded? result directions)
                        (if succeeded? (list result) '())
                        )))))           
               (c::lambda (pat) (c::map-without-type-checking here-from-list (list-finder pat)))))
           (c::lambda (pat) (list (discoverer pat)))
           ))
       (any-values
         (if (and islist (= n-ellipses 2))
           (let*
             ((success-once
               (lambda (form)
                  (call-with-values         
                    (c::lambda () (c::follow-directions-and-return-result form directions))
                    (c::lambda (succeeded? result directions)
                      (if succeeded?
                        (list #t)
                        '())))))
             )
             (lambda (pat) (not (null? (apply append (c::map-without-type-checking success-once (list-finder pat))))))
             )
           (lambda (pat) #t)))
        )
      (list (c::variable-identifier rvd) islist local-value-grabber any-values)
      )
    )
  )

(c::define c::pattern-variable-structures
  (c::lambda ( pat lits environment sexp)
      (c::map-without-type-checking
         (c::lambda (x) (c::transform-raw-variable-discovery x sexp))
         (c::pattern-variable-structures-aux pat pat (c::pattern-keyword pat) lits '() 0 'x 'x #f '(#f #f) environment 'x)
         )
      ))

(c::define c::insert-define-time-syntax-bindings-aux 
  (c::lambda (partial-template pat-kwd lits patvars environment)
    (cond
      ((null? partial-template) '())
      ((equal? partial-template pat-kwd)
        (c::make-syntax-binding partial-template #f #f 'pattern-keyword))
      ((member partial-template lits)
        (c::make-syntax-binding partial-template #f #f 'pattern-literal))
      ((member partial-template patvars)
        (c::make-syntax-binding partial-template #f #f 'pattern-variable))
      ((and (c::identifier? partial-template)
            (not (c::ellipsis? partial-template)))
       (c::make-syntax-binding-with-variable partial-template environment 'define-time))
      ((pair? partial-template)
       (cons (c::insert-define-time-syntax-bindings-aux (car partial-template) pat-kwd lits patvars environment)
             (c::insert-define-time-syntax-bindings-aux (cdr partial-template) pat-kwd lits patvars environment)))
      ((vector? partial-template)
       (list->vector
         (c::map-without-type-checking (c::lambda (vector-element) (c::insert-define-time-syntax-bindings-aux vector-element pat-kwd lits patvars environment))
              (vector->list partial-template))))
      (#t partial-template))))

(c::define c::insert-define-time-syntax-bindings
  (c::lambda ( pat template lits patvars environment)
    (c::insert-define-time-syntax-bindings-aux template (c::pattern-keyword pat) lits patvars environment)))

(c::define c::cdr-contains-only-ellipsis?
  (c::lambda ( x)
    ;;
    ;; Is x of form "(foo ...)"?  No test done here of what "foo" is.
    ;;
    (and (pair? x)
         (pair? (cdr x))
         (null? (cddr x))
         (c::ellipsis? (cadr x)))))

(c::define c::match-aux
  (c::lambda (x patvars lits form)
    ;;
    ;; The second argument is assumed known to be a pattern, in the sense
    ;; that (pattern? full-pattern) ==> #t.  The third argument is assumed known
    ;; to be a list of symbols, possibly null.
    ;;
    (cond
      ((and (null? x) (null? form))                                  #t)     ;; End of line ...
      ((c::ellipsis? x)                                              #t)     ;; Ellipsis matches anything.
      ((and (c::identifier? x) (memv x lits) (equal? x form))        #t)     ;; Literal matches itself.
      ((and (c::identifier? x) (memv x patvars))                     #t)     ;; Pattern variable matches anything
      ((and (pair? x) (c::cdr-contains-only-ellipsis? x))  (list? form))     ;; (<pattern> ...) matches any list, and nothing else.
      ((and (pair? x) (pair? form))                                          ;; Pattern and form are of form (foo . bar)
       (and (c::match-aux (car x) patvars lits (car form))
            (c::match-aux (cdr x) patvars lits (cdr form))))
      ((and (vector? x) (vector? form))                               ;; Recurse on vectors.
         (c::match-aux (vector->list x) patvars lits (vector->list form)))
      (#t (equal? x form)))))                                         ;; Simple match.

(c::define c::match-with-patvars?
  (c::lambda (pattern lits form simple-patvars)
    (cond ((c::cdr-contains-only-ellipsis? pattern) (list form))
          ((c::match-aux (cdr pattern) simple-patvars lits (cdr form))    ;; Omit checking on macro name, so that we may
           form)                                                           ;;   pass an "unnamed" macro into "let-syntax".
          (else #f))
    ))

(c::define c::substitutions-from-finders
  (c::lambda (patvars form sexp)
      (c::map-without-type-checking
        (c::lambda (x)
          (if (and (not (list? form)) (c::list-variable? x))
            (c::begin
              (newline (e::error-port))
              (display "Macro call: " (e::error-port))
              (display form (e::error-port))
              (newline (e::error-port))
              (e::error "Poorly-formed macro call -- expected a list.  (The macro definition contains\n  a variable name followed by \"...\"; that pattern can only be matched by a list.)")))

          (if (not (procedure? (c::value-grabber x)))
            (c::begin
              (newline (e::error-port))
              (display x (e::error-port))
              (e::error "Impementation -- c::value-grabber does not return a procedure when applied to the preceding patvar.")))
          (if (not (procedure? (c::success-checker x)))
            (c::begin
              (newline (e::error-port))
              (display x (e::error-port))
              (e::error "Impementation -- c::success-checker does not return a procedure when applied to the preceding patvar.")))
            ((c::value-grabber x) form)
          )
        patvars
        )
      )
    )

(c::define c::list-lambda-formals-sbs
  ;;
  ;; For use when every variable in an s-expression (including lambda formals)
  ;; has been replaced by a syntax-binding for it.
  ;;
  ;; Suppose "sexp" is known to be the lambda-formals part of a
  ;; lambda expression.  List the variables which it contains.
  ;;
  (c::lambda( sexp )
    (if (null? sexp)
      '()
      (if (e::syntax-binding? sexp)
        (list sexp)
        (append (c::list-lambda-formals-sbs (car sexp))
                (c::list-lambda-formals-sbs (cdr sexp)))))))

;;  Substitute renamed lambda variables only for things that are going to
;;  to be evaluated.  Here is a decision table for whether a thing is going
;;  to be evaluated or not:
;;  
;;    Let QQ be (Sum of ` enclosing the item).
;;  
;;    Let S be (Sum of ` enclosing the item) - (Sum of , enclosing the item)
;;      - (Sum of '@ enclosing the item).
;;  
;;    Z means "value must be zero -- error if not".
;;  
;;    X means "don't care".
;;  
;;    QQ      S     In scope of '    Item will be evaluated?
;;    
;;     0      Z        Yes                    NO
;;     0      Z         No                   YES
;;    > 0    > 0         X                    NO
;;    > 0     0          X                   YES
;;  
;;  Psuedocode for using the table to compute boolean "will be evaluated":
;;  
;;    if( QQ == 0 )
;;        not( in scope of ')
;;    else
;;        (S == 0)
    
(c::define c::subst-sbs-in-sexp
  ;;
  ;; Substitute the variable "new" for the variable "old" in "sexp"
  ;;
  (c::lambda (new old sexp qq quoted? s)

    (cond

      ((null? sexp) '())

      ((list? sexp)

       (let*
         ((the-car (car sexp)))

         (c::map-without-type-checking
           (c::lambda (x) 
             (c::subst-sbs-in-sexp new old x
               (+ qq (if (c::quasiquoted-sbs the-car) 1 0))
               (or quoted? (c::quoted-sbs the-car))
               (+ (c::quotation-level-change-b the-car) s))) sexp)))

      ((pair? sexp)
       (cons (c::subst-sbs-in-sexp new old (car sexp) qq quoted? s)
             (c::subst-sbs-in-sexp new old (cdr sexp) qq quoted? s)))

      ((vector? sexp)
       (list->vector (c::map-without-type-checking (c::lambda (x) (c::subst-sbs-in-sexp new old x qq quoted? s)) (vector->list sexp))))

      ((equal? sexp old)
       (cond ((zero? qq)    (if quoted? old new))
             ((zero? s)     new)
             (else          old)))

      (else sexp)
      )))

(c::define c::subst-many-sbs-in-sexp
  ;;
  ;; Cdr down the two lists of variables, "news" and "olds", in parallel,
  ;; using "subst-sbs-in-sexp" to do variable substitution with each
  ;; pair of items from the two lists just mentioned, in turn, into the body.
  ;;
  (c::lambda (news olds body)
    (if (null? news)
       body 
       (c::subst-many-sbs-in-sexp (cdr news) (cdr olds) (c::subst-sbs-in-sexp (car news) (car olds) body 0 #f 0)))))

(c::define c::match-sb?-with-type-with-patvar-list
  (c::lambda (sb? type lyst)
    (and (e::syntax-binding? sb?) 
         (not (null? (e::syntax-binding-properties sb?)))
         (equal? (car (e::syntax-binding-properties sb?)) type)
         (assoc (e::syntax-binding-symbol sb?) lyst))))

(c::define c::list-patvars-within
  (c::lambda (sexp pairs answer)
    (let ((matcher #f))
      (cond
        ((null? sexp) answer)
        ((member sexp answer) answer)
        ((c::begin (c::set! matcher (c::match-sb?-with-type-with-patvar-list sexp 'pattern-variable pairs)) matcher)
         (cons matcher answer))
        ((pair? sexp) (c::list-patvars-within (car sexp) pairs (c::list-patvars-within (cdr sexp) pairs answer)))
        ((vector? sexp) (c::list-patvars-within (vector->list sexp) pairs answer))
        (#t answer)))))

(c::define c::cons-up-deep-copies
  (c::lambda (sexp patvars-within data-lists)
    (if (null? (car data-lists))
      '()
      (let*
        ((deep-copy-of-sexp (e::deep-copy sexp))
         (modified-patvars
           (c::map-without-type-checking
             (c::lambda (pvw dl)
               (if (null? dl)
                 (set-cdr! (cdr pvw) (list (list)))
                 (set-cdr! (cdr pvw) (list (list (car dl)))))
               pvw
               )
             (c::map-without-type-checking e::deep-copy patvars-within)
             data-lists))
         (action-on-cdr
           (c::cons-up-deep-copies
             sexp
             patvars-within
             (c::map-without-type-checking
               (lambda (dl) (if (null? dl) '() (cdr dl)))
               data-lists))
           )
         (action-on-car
           (c::substitute-sb-vars-in-body modified-patvars deep-copy-of-sexp)
           )
         )
        (cons action-on-car action-on-cdr)))))

(c::define c::substitute-sb-vars-in-body
  (c::lambda (pairs body)    ;; "body" must have syntax-bindings substituted for identifiers

    (cond 
      ((null? body) '())

      ((and 
         (list? body)
         (>= (length body) 2)
         (c::ellipsis? (cadr body))
         (not (c::match-sb?-with-type-with-patvar-list (car body) 'pattern-variable pairs)))
       (let*
         ((patvars-within (c::list-patvars-within (car body) pairs '()))
          (data-lists (c::map-without-type-checking caddr patvars-within))
          (arities (c::map-without-type-checking length data-lists))
          )
         (if (null? patvars-within)
           (c::begin 
             (newline (e::error-port))
             (display (car body) (e::error-port)) (display '... (e::error-port))
             (e::error "No pattern variables in the preceding expression -- ellipsis makes no sense.")))
         (if (and (> (length arities) 1) (not (apply = arities)))
           (c::begin
             (newline (e::error-port))
             (display "Pattern variables: " (e::error-port))
             (display (c::map-without-type-checking car patvars-within) (e::error-port)) (newline (e::error-port))
             (display "Data lists, respectively: " (e::error-port)) (display data-lists (e::error-port)) (newline (e::error-port))
             (display "Data list lengths, respectively: " (e::error-port)) (display arities (e::error-port)) (newline (e::error-port))
             (e::error "The lists of data to substitute for the given pattern variables are not all the same length.")))
         (c::cons-up-deep-copies (car body) patvars-within data-lists)
         ))
      ((list? body)
       (if (c::ellipsis? (car body))                                 ;; Args for ellipsis will already be spliced in ...
         (c::substitute-sb-vars-in-body pairs (cdr body))            ;;   so ignore ellipsis and work on cdr.
         (let                                                        ;; Regular list -- see if the car matches anything ...
           ((matcher (c::match-sb?-with-type-with-patvar-list (car body) 'pattern-variable pairs)))
           (if matcher                                               ;; Found a match
              (if (c::list-variable? (cadr matcher))                 ;; Splice in a list ...
                 (append                                             ;;    ... by appending.
                   (apply append (caddr matcher))
                   (c::substitute-sb-vars-in-body pairs (cdr body)))
                 (cons 
                   (car (caddr matcher))                               ;; Just one item that matches (is this an error case?)
                   (c::substitute-sb-vars-in-body pairs (cdr body)))
                 )
              (cons (c::substitute-sb-vars-in-body pairs (car body)) ;; No match -- recurse and cons.
                    (c::substitute-sb-vars-in-body pairs (cdr body))))))
        )
      ((pair? body)
       (let                               ;; similar to list case, preceding, but ellipsis is not expected.
         ((matcher (c::match-sb?-with-type-with-patvar-list (car body) 'pattern-variable pairs)))
         (if matcher                      ;; Found a match
            (if (c::list-variable? (cadr matcher))                 ;; Splice in a list ...
               (append                                             ;;    ... by appending.
                 (apply append (caddr matcher))
                 (c::map-without-type-checking car (caddr matcher))
                 (c::substitute-sb-vars-in-body pairs (cdr body)))
               (cons
                 (car (caddr matcher))                               ;; Just one item that matches (is this an error case?)
                 (c::substitute-sb-vars-in-body pairs (cdr body)))
               )
            (cons (c::substitute-sb-vars-in-body pairs (car body)) ;; No match -- recurse and cons.
                  (c::substitute-sb-vars-in-body pairs (cdr body))))))
      ((vector? body)                ;; Vector -- listify, recurse on elements, re-vectorize.
       (list->vector (c::map-without-type-checking (c::lambda (x) (c::substitute-sb-vars-in-body pairs x)) (vector->list body))))
      (#t
       (let ((matcher (c::match-sb?-with-type-with-patvar-list body 'pattern-variable pairs)))
         (if matcher
           (car (caddr matcher))
           body))))))     

(c::define c::collect-let-list-in-environment
  (c::lambda (sexp env answer)
    (cond 
      ((null? sexp) answer)
      ((pair? sexp)
       (c::collect-let-list-in-environment (cdr sexp) env (c::collect-let-list-in-environment (car sexp) env answer)))
      ((vector? sexp)
       (c::collect-let-list-in-environment (vector->list sexp) env answer))
      ((c::identifier? sexp)
       (if (c::bound-instance-in-environment? sexp env)
         (if (assoc sexp answer)
           answer
           (cons (list sexp (c::symeval-in-environment sexp env)) answer))
         answer))
      (#t answer))))

(c::define c::strip-let-list-in-environment
  (c::lambda (let-list env answer)
    (if (null? let-list)
      answer
      (let
        ((the-symbol (caar let-list))
         )
        (if (and (c::bound-instance-in-environment? the-symbol env)
                 (equal? (cadar let-list) (c::symeval-in-environment the-symbol env)))
            (c::strip-let-list-in-environment (cdr let-list) env answer)
            (cons (car let-list) (c::strip-let-list-in-environment (cdr let-list) env answer)))))))

(c::define c::remove-pattern-variables
  (c::lambda (let-list var-list answer)
    (if (or (null? var-list) (null? let-list))
      answer
        (if (member (caar let-list) var-list)
          (c::remove-pattern-variables (cdr let-list) var-list answer)
          (c::remove-pattern-variables (cdr let-list) var-list (cons (car let-list) answer))))))

(c::define c::insert-syntax-bindings-of-type-from-environment
  (c::lambda (sexp type env)
    (cond 
      ((null? sexp) '())
      ((pair? sexp)
       (cons (c::insert-syntax-bindings-of-type-from-environment (car sexp) type env)
             (c::insert-syntax-bindings-of-type-from-environment (cdr sexp) type env)))
      ((vector? sexp)
       (list->vector
         (c::map-without-type-checking (c::lambda (x) (c::insert-syntax-bindings-of-type-from-environment x type env))
              (vector->list sexp))))
      ((c::identifier? sexp) (c::make-syntax-binding-with-variable sexp env type))
      (#t sexp))))

(c::define c::syntax-bound-entity?
  (c::lambda (x test)
    (and (e::syntax-binding? x)
         (e::syntax-binding-bound? x)
         (test (e::syntax-binding-binding x)))))

(c::define c::quotation-level-change
  (c::lambda (x)
    (if (and (e::syntax-binding? x) (e::syntax-binding-bound? x))
      (let ((val (e::syntax-binding-binding x)))
        (cond ((equal? val quote)             1)
              ((equal? val quasiquote)        1)
              ((equal? val unquote)          -1)
              ((equal? val unquote-splicing) -1)
              (#t                             0)))
      0)))

(c::define c::quotation-level-change-b
  (c::lambda (x)
    (if (and (e::syntax-binding? x) (e::syntax-binding-bound? x))
      (let ((val (e::syntax-binding-binding x)))
        (cond ((equal? val quasiquote)        1)
              ((equal? val unquote)          -1)
              ((equal? val unquote-splicing) -1)
              (#t                             0)))
      0)))

(c::define c::quoted-sbs
  (c::lambda (x)
    (and
      (e::syntax-binding? x)
      (e::syntax-binding-bound? x)
      (equal? (e::syntax-binding-binding x) quote))))

(c::define c::quasiquoted-sbs
  (c::lambda (x)
    (and
      (e::syntax-binding? x)
      (e::syntax-binding-bound? x)
      (equal? (e::syntax-binding-binding x) quasiquote))))

(c::define c::substitute-gensymed-lambda-formals
  (c::lambda (body)
    (let*
      ((lambda-formals (c::list-lambda-formals-sbs (cadr body))))
       (c::subst-many-sbs-in-sexp
         (c::map-without-type-checking
           (c::lambda (x)
             (c::make-syntax-binding (e::gensym) #f '() 'lambda-formal))
           lambda-formals)
         lambda-formals body))))

(c::define c::gensym-lambdas
  (c::lambda (body environment quotation-level)
    (cond
      ((null? body) '())
      ((list? body)
       (let*
         ((the-car (car body))
          (new-quotation-level (+ (c::quotation-level-change-b the-car) quotation-level)))
         (if (> new-quotation-level 0)
            (cons the-car (c::map-without-type-checking (c::lambda (x) (c::gensym-lambdas x environment new-quotation-level)) (cdr body)))
            (cond
              ((c::syntax-bound-entity? the-car (c::lambda (x) (equal? x lambda)))
               (c::map-without-type-checking (c::lambda (x) (c::gensym-lambdas x environment 0))
                    (c::substitute-gensymed-lambda-formals body)))
              (#t (c::map-without-type-checking (c::lambda (x) (c::gensym-lambdas x environment 0)) body)))
            )))
      ((pair? body) 
       (cons (c::gensym-lambdas (car body) environment quotation-level)
             (c::gensym-lambdas (cdr body) environment quotation-level)))
      ((vector? body) (list->vector 
                        (c::map-without-type-checking (c::lambda (x) (c::gensym-lambdas x environment quotation-level))
                        (vector->list body))))
      (#t body))))
                 
(c::define c::remove-syntax-bindings
  (c::lambda (body)
    (cond
      ((pair? body)
       (cons (c::remove-syntax-bindings (car body))
             (c::remove-syntax-bindings (cdr body))))
      ((vector? body)
       (list->vector (c::map-without-type-checking c::remove-syntax-bindings (vector->list body))))
      ((e::syntax-binding? body)
       (e::syntax-binding-symbol body))
      (#t body))))

(c::define c::expand-partial-transformer
  (c::lambda (vars patvar-structures body environment let-list)    ;; "body" must have syntax-bindings substituted for identifier.
    (let*
      ((syntax-bound-vars
         (if (null? vars)
           '()
           (c::map-without-type-checking (c::lambda (x) (c::insert-syntax-bindings-of-type-from-environment x 'expand-time environment)) vars)))
       (pairs
         (if (null? vars)
       '()
       (c::map-without-type-checking (c::lambda (new old) (list (c::variable-identifier old) old new)) syntax-bound-vars patvar-structures)))
        (first-expansion 
      (if (null? vars)
        body
        (c::substitute-sb-vars-in-body pairs body)))
        (third-expansion (c::remove-syntax-bindings (c::gensym-lambdas first-expansion environment 0)))
        (stripped-let-list (c::strip-let-list-in-environment let-list environment '()))
        )
      (if (null? stripped-let-list)
        third-expansion
        (list 'let stripped-let-list third-expansion))
      )))
   
(c::define c::make-partial-transformer 
  ;;
  ;; Create a "partial transformer" (my term) from a template, by syntax-binding all variables in the template
  ;; to their values in the macro-definition environment, and returning a lamda expression made from the template
  ;; thus modified, that will further expand the template when it is run, in the macro-expansion environment.
  ;;
  (c::lambda (lits pattern template define-time-env patvar-structures)
    (let*
      ((simple-pattern-variables (c::map-without-type-checking c::variable-identifier patvar-structures))
       (let-list (c::collect-let-list-in-environment template define-time-env '()))
       ) 

      ;; Note that the following lambda expression is the return-value from "c::make-partial-transformer".

      (c::lambda vars 
        (let ((run-time-env (c::environment-n-down 1)))
          (c::expand-partial-transformer vars patvar-structures 
            (c::insert-define-time-syntax-bindings pattern template lits simple-pattern-variables define-time-env)
            run-time-env
            (c::remove-pattern-variables let-list simple-pattern-variables '())))))))

(c::define c::find-matching-ppt
   (c::lambda ( ppts lits form )
     (cond
       ((null? ppts) #f)
       ((c::match-with-patvars?  (caar ppts) lits form (c::map-without-type-checking c::variable-identifier (caddar ppts))) (car ppts))
       (#t (c::find-matching-ppt (cdr ppts) lits form)))))

(c::define c::check-uniqueness-with-sexp-and-pattern
  (c::lambda (raw-variables sexp pattern)
    (cond
      ((null? raw-variables) #t)
      ((null? (cdr raw-variables)) #t)
      ((member (car raw-variables) (cdr raw-variables))
        (c::begin
          (newline (e::error-port))
          (display "Pattern: " (e::error-port)) (display pattern (e::error-port)) (newline (e::error-port))
          (display "\"syntax-rules\" used as: " (e::error-port)) (display sexp (e::error-port))
          (e::error (string-append "Variable " (symbol->string (car raw-variables)) " appears more than once in the pattern given."))))
      (#t (c::check-uniqueness-with-sexp-and-pattern (cdr raw-variables) sexp pattern)))))

(c::define c::check-for-same-keyword
  (c::lambda (keyword pattern sexp)
    (if (equal? keyword (car pattern))
      #t
      (c::begin
        (newline (e::error-port))
        (display sexp (e::error-port)) (newline (e::error-port))
        (e::error (string-append "\"syntax-rules\" call uses different pattern identifiers\n  -- \"" 
                                 (symbol->string keyword) "\" and \"" (symbol->string (car pattern))
                                 "\" -- in different patterns."))))))

(c::define c::extract-simple-variable-names
  (c::lambda (patvars)
    (cond
      ((null? patvars) '())
      ((symbol? (car patvars)) (cons (car patvars) (c::extract-simple-variable-names (cdr patvars))))
      (#t (c::extract-simple-variable-names (cdr patvars))))))

(c::define c::foo-syntax-syntax-check
  (c::lambda (sexp form-name)
    (if (not (= (length sexp) 3))
      (c::begin
        (newline (e::error-port))
        (display sexp (e::error-port)) (newline (e::error-port))
        (e::error (string-append "\"" (symbol->string form-name)"\" used as shown, but two arguments are required."))))
    (if (not (symbol? (cadr sexp)))
      (c::begin
        (newline (e::error-port))
        (display sexp (e::error-port)) (newline (e::error-port))
        (e::error "\"syntax-rules\" used as shown, but the first argument must be a symbol.")))
    (if (not (list? (cddr sexp)))
      (c::begin
        (newline (e::error-port))
        (display sexp (e::error-port)) (newline (e::error-port))
        (e::error "\"syntax-rules\" used as shown, but the second argument must be a list.")))
    (if (not (c::transformer-spec? (caddr sexp)))
      (c::begin
        (newline (e::error-port))
        (display sexp (e::error-port)) (newline (e::error-port))
        (display "\"syntax-rules\" used as shown, but the second argument is not a transformer specification.\n" (e::error-port))
        (display "  The problems include:\n" (e::error-port))
        (if (not (list? (caddr sexp)))
          (e::error  "That argument is not a list."))
        (if (not (>= (length (caddr sexp)) 3))
          (e::error  "That argument is not a list of three or more arguments."))
        (if (not (equal? (car (caddr sexp)) 'syntax-rules))
          (e::error  "The first item in that argument is not the symbol \"syntax-rules\"."))
        (if (not (c::identifier-zero-or-more? (cadr (caddr sexp))))
          (e::error  "The second item in that argument is not a list of identifiers."))
        (c::map-without-type-checking
          (c::lambda (x)
            (if (c::syntax-rule? x)
               #t
               (c::begin
                 (display "    \"" (e::error-port)) (display x (e::error-port)) (display "\" is not a syntax rule:\n" (e::error-port))
                 (if (not (and (list? x) (= (length x) 2)))
                   (display "    It is not a list of two items\n" (e::error-port))
                   (c::begin
                     (if (not (pair? (car x)))
                       (c::begin
                         (display "      \""               (e::error-port))
                         (display (car x)                  (e::error-port))
                         (display "\" is not a pair:\n" (e::error-port)))
                       (if (not (c::pattern? (car x)))
                         (c::begin
                           (display "      \""               (e::error-port))
                           (display (car x)                  (e::error-port))
                           (display "\" is not a pattern:\n" (e::error-port)))
                         (c::begin
                           (if (not (equal? (caar x) (cadr sexp)))
                             (c::begin
                               (display "      Pattern \""                           (e::error-port))
                               (display (car x)                                      (e::error-port))
                               (display "\" does not start with expected keyword \"" (e::error-port))
                               (display (cadr sexp)                                  (e::error-port))
                               (display "\".\n")                                     (e::error-port))
                             '()))))
                     (if (not (c::template? (cadr x)))
                       (c::begin
                         (display "      \""                (e::error-port))
                         (display (cadr x)                  (e::error-port))
                         (display "\" is not a template:\n" (e::error-port)))
                       '()))))))
          (cddr (caddr sexp)))
        (display "No further problems discovered.  (Resetting)\n\n" (e::error-port)) (e::reset)
        ))
;;    (let
;;      ((symbol-being-defined (cadr sexp))
;;       (first-pattern-keyword (caaadr (cdaddr sexp))))
;;      (if (not (equal? symbol-being-defined first-pattern-keyword))
;;        (c::begin
;;          (newline (e::error-port))
;;          (display sexp (e::error-port)) (newline (e::error-port))
;;          (display
;;            (string-append 
;;              "\"syntax-rules\" used as shown, but the symbol being defined,\n  \""
;;              (symbol->string symbol-being-defined)
;;              "\", differs from the keyword,\n  \""
;;              (symbol->string first-pattern-keyword)
;;              "\", of the first pattern.") 
;;            (e::error-port))
;;          (e::error "To use this syntax, all pattern keywords should match the symbol defined."))))
    #t))

(c::define c::define-syntax-code
  (c::lambda (sexp)
    (c::foo-syntax-syntax-check sexp 'define-syntax)
    (list define (cadr sexp) (list e::macro-body (caddr sexp)))
    ))

(c::define c::macro-body-code
  (c::lambda (sexp)
    (c::foo-syntax-syntax-check sexp 'e::syntax-macro-body)
    (list e::macro-body (caddr sexp))
    ))

(c::define c::syntax-rules-code
  (c::lambda (sexp define-time-env)

    ;; Start with lots of local definitions and error-checking ...

    (letrec
      ((lits (cadr sexp))
       (pattern-templates (cddr sexp))
       (first-keyword
         (if pattern-templates
           (caaar pattern-templates)
           '()))
       (pattern-partial-transformers
         (c::map-without-type-checking
           (c::lambda( pattern-template ) 
             (c::check-for-same-keyword first-keyword (car pattern-template) sexp)
             (let 
               ((pattern-variable-structures
                  (c::pattern-variable-structures (car pattern-template) lits define-time-env sexp))
                )
               (c::check-uniqueness-with-sexp-and-pattern
                 (c::extract-simple-variable-names pattern-variable-structures) sexp (car pattern-template))
               (list (car pattern-template)
                     (c::make-partial-transformer
                       lits
                       (car pattern-template)
                       (cadr pattern-template) 
                       define-time-env
                       pattern-variable-structures)
                     pattern-variable-structures)
               )
             )
           pattern-templates))
         )

      ;; Note that the following lambda-expression is the return value from "syntax-rules".

      (c::lambda (form)
        (let
          ((matched-ppt (c::find-matching-ppt pattern-partial-transformers lits form)))
          (if matched-ppt
            (apply (cadr matched-ppt) (c::substitutions-from-finders (caddr matched-ppt) form sexp))
            (c::begin
              (newline (e::error-port))
              (display "Form: " (e::error-port)) (display form (e::error-port)) (newline (e::error-port))
              (e::error 
                (string-append "That use of macro \"" (symbol->string (car form)) "\", matches no pattern for the macro.\n"
                               "  (That is, that's not how \"" (symbol->string (car form)) "\" is supposed to be called.)")))
            )))
      )
    ))

(e::macro syntax-rules
  (c::lambda (sexp)
   ((c::lambda (env)
      (c::syntax-rules-code sexp env))
    (c::environment-n-down 0))))

(e::macro e::syntax-macro-body c::macro-body-code)

