;; Copyright (c) 2009 by Jay Reynolds Freeman

;;    This program is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    This program is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;
;;    Contact information:
;;
;;        EMail:       Jay_Reynolds_Freeman@mac.com
;;        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman
;;
;;        Paper Mail:  Jay Reynolds Freeman
;;                     Post Office Box 60628
;;                     Palo Alto, CA, 94306-0628

(display "RecursveMacroExpand.s:\n")

(c::define c::recursive-list
  (list
    'e::macro-call?
    'c::check-name-for-lambda-formal
    'c::list-lambda-variable-names
    'c::get-lambda-variables-list
    'c::check-lambda-names
    'c::expand-macro-calls-till-done
    'c::find-macro-expansion-and-expand
    'e::expand-macros-recursively
    'c::collect-initial-defines
    'c::find-where-body-starts
    'c::show-internal-define-and-form-error
    'c::convert-define-to-letrec-list-item
    'c::letrecify-defines
    ))

(c::define e::macro-call?
  (c::lambda (form)
    (and (pair? form) (symbol? (car form)) (e::bound-instance? (car form)) (e::macro? (c::symeval (car form))))))

(c::define c::check-name-for-lambda-formal
  (c::lambda (name lambda-expression)
    (if (e::permanent? name)
      (c::begin
        (display lambda-expression (e::error-port))
        (e::error (string-append
                    "The given lambda expression uses the permanent symbol \""
                    (symbol->string name)
                    "\" as a formal parameter."))))
    (if (and (e::bound-instance? name) (e::macro? (c::symeval name)))
      (c::begin
        (display lambda-expression (e::error-port))
        (e::error (string-append
                    "The given lambda expression uses the macro name \""
                    (symbol->string name)
                    "\" as a formal parameter."))))
    #t))

(c::define c::list-lambda-variable-names 
  (c::lambda (varlist whole-varlist)
    (cond ((null? varlist)    '())
          ((symbol? varlist) (cons varlist '()))
          ((not (pair? varlist))
           (display whole-varlist (e::error-port))
           (e::error "Ill-formed formal parameter list for a lambda expression."))
          ((symbol? (car varlist))
           (cons (car varlist) (c::list-lambda-variable-names (cdr varlist) whole-varlist)))
          (else
           (display whole-varlist (e::error-port))
           (e::error "Ill-formed formal parameter list for a lambda expression.")))))

(c::define c::get-lambda-variables-list
  (c::lambda (form)
    (if (not (pair? form))
      (c::begin
        (display form (e::error-port))
        (e::error "Implementation: \"c::get-lambda-variables-list\" called with a form that is not a pair.")))
    (if (not (or (equal? (car form) 'lambda) (equal? (car form) 'c::lambda)))
      (c::begin
        (display form (e::error-port))
        (e::error "Implementation: \"c::get-lambda-variables-list\" called with a form that is not a lambda expression.")))
    (if (not (pair? (cdr form)))
      (c::begin
        (display form (e::error-port))
        (e::error "\"c::get-lambda-variables-list\" called with an ill-formed lambda expression.")))
    (c::list-lambda-variable-names (cadr form) (cadr form))))

(c::define c::check-lambda-names
  (c::lambda (form)
    (c::map-without-type-checking
      (c::lambda (name)
        (c::check-name-for-lambda-formal name form))
      (c::get-lambda-variables-list form))
  #t))

(c::define c::expand-macro-calls-till-done
  (c::lambda (form quotation-level)	
;    (display (list 'emctd form quotation-level)) (newline) ;; DEBUG
    (cond ((null? (car form)) quotation-level)
          ((and (list? (car form)) 
                (or (equal? (caar form) 'quote) (equal? (caar form) quote)))
;           (display "Quotation -- let it stand\n")  ;; DEBUG
           (+ quotation-level 1))
          ((and (= quotation-level 0) 
                (list? (car form))
                (equal? (caar form) 'c::lambda))
           (c::check-lambda-names (car form))
           quotation-level)
          ((and (= quotation-level 0) (e::macro-call? (car form)))
;           (display "Macro call -- expanding\n")  ;; DEBUG
           (case (caar form)
             ((quasiquote)
              (set-car! form (e::expand-macro (car form)))
              0)
             (else
              (set-car! form (e::expand-macro (car form)))
              (c::expand-macro-calls-till-done form 0))
             ))
          (else
;            (display "Not a macro call to be expanded -- returning\n")  ;; DEBUG
            quotation-level
            ))
    ))

(c::define c::find-macro-expansion-and-expand
  (c::lambda (form quotation-level)
;    (display (list 'fmeae form quotation-level)) (newline) ;; DEBUG
    (if (not (or (pair? form) (vector? form)))
       (c::begin (display form (e::error-port))
              (e::error "c::find-macro-expansion-and-expand called with the form shown, which is not a pair.")))
    (cond
      ((null? form) #t)
      ((pair? form)         ;; FIXME -- Move error-test down here ...
       (let ((new-quotation-level (c::expand-macro-calls-till-done form quotation-level)))
         (if (or (pair? (car form)) (vector? (car form)))
           (c::begin 
             (c::find-macro-expansion-and-expand (car form) new-quotation-level)
             ))
         (if (or (pair? (cdr form)) (vector? (cdr form)))
           (c::begin 
             (c::find-macro-expansion-and-expand (cdr form) quotation-level)))
           ))
      ((vector? form)
       (c::find-macro-expansion-and-expand (c::set-tag! form c::PAIR_TAG) quotation-level))
      (else #t))))

;;####################

(c::define e::expand-macros-recursively
  (c::lambda (form)
    (let ((listed-form (cons form '())))
      (c::find-macro-expansion-and-expand listed-form 0)
      (car listed-form))))

(c::define c::collect-initial-defines
  (c::lambda (form)
    (cond
      ((null? form) '())
      ((and (pair? form) (list? (car form)) (equal? (caar form) 'define))
       (cons (car form) (c::collect-initial-defines (cdr form))))
      (else '()))))

(c::define c::find-where-body-starts
  (c::lambda (form)
    (cond
      ((and (pair? form) (list? (car form)) (equal? (caar form) 'define))
       (c::find-where-body-starts (cdr form)))
      (else form))))

(c::define c::show-internal-define-and-form-error
  (c::lambda (internal-deine form msg)
    (display "\nInternal define: " (e::error-port))
    (display internal-define (e::error-port))
    (display "\nIn form: " (e::error-port))
    (display form (e::error-port))
    (e::error msg)))

(c::define c::convert-define-to-letrec-list-item
  (c::lambda (internal-define form)
    (if (or (not (list? internal-define)) (< (length internal-define) 3) (not (equal? (car internal-define) 'define)))
      (c::show-internal-define-and-form-error internal-define form "Badly-formed internal define in the form given."))
    (if (pair? (cadr internal-define))
      (let
        ((name (caadr internal-define))
         (var-stuff (cdadr internal-define))
         )
        (if (not (symbol? name))
          (c::show-internal-define-and-form-error internal-define form "Second item of internal define does not name a symbol.")
          `(,name (c::lambda ,var-stuff ,@(cddr internal-define))))
        )
      (if (not (symbol? (cadr internal-define)))
        (c::show-internal-define-and-form-error internal-define form "Second item of internal define is not a symbol.")
        (cdr internal-define)))))

(c::define c::letrecify-defines
  (c::lambda (form)
    (if (and (pair? form) (list? (car form)) (equal? (caar form) 'define))
      (let*
        ((initial-defines (c::collect-initial-defines form))
         (body (c::find-where-body-starts form))
         (letrec-list 
           (c::map-without-type-checking 
             (c::lambda (internal-define) 
               (c::convert-define-to-letrec-list-item internal-define form))
           initial-defines))
         )
        (if (not (null? letrec-list))
          (c::begin
            (if (null? body)
              (c::set! body (list #t)))
            (let ((result (list 'letrec letrec-list)))
              (set-cdr! (cdr result) body)
              (list result)))
         form))
      form)))
 