;; Copyright (c) 2009 by Jay Reynolds Freeman

;; Library containing Wraith Scheme code for a very
;; simple single-inheritance class system.

;; This file loads the functions but does not compile
;; them; that is done in "CompilePrimitiveLibrary.s".	

;;    This program is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    This program is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;
;;    Contact information:
;;
;;        EMail:       Jay_Reynolds_Freeman@mac.com
;;        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman
;;
;;        Paper Mail:  Jay Reynolds Freeman
;;                     Post Office Box 60628
;;                     Palo Alto, CA, 94306-0628

(display "SimpleClass.s:\n")
	
(c::define c::simple-class-list
  (list
    'c::simple-class-format-call
    'c::simple-class-last-pair
    'c::check-simple-class-name-syntax    
    'c::check-simple-class-ivar-syntax
    'c::check-simple-class-binding-pair-is-pair
    'c::check-simple-class-list-is-list
    'c::check-simple-class-has-ivar
    'c::check-make-simple-class-syntax
    'c::check-make-simple-class-instance-syntax
    'c::set!-new-bindings
    'c::duplicate-simple-all-variable-names
    'c::duplicate-simple-class-variable-names
    'c::duplicate-simple-instance-bindings-aux
    'c::duplicate-simple-instance-bindings
    'c::duplicate-simple-class-bindings-aux
    'c::duplicate-simple-class-bindings
    'c::report-duplicate-simple-class-variable-names-aux
    'c::report-duplicate-simple-class-variable-names
    'e::make-simple-class
    )
  )

(c::map-without-type-checking e::clear-permanent! c::simple-class-list)

;; Format a sane representation of what the call looked like

(c::define c::simple-class-format-call
  (lambda (name lyst)
    (newline (e::error-port))
    (display (cons (string-append "<instance of \"" (symbol->string name) "\">") lyst) (e::error-port))))

;; Get the last pair of a proper, non-null list.

(c::define c::simple-class-last-pair
  (lambda (lyst)
    (if (null? (cdr lyst  ))
      lyst
      (c::simple-class-last-pair (cdr lyst)))))


;; Straighforward check for whether the proposed class name is a symbol:
;;
;; Most of the work here is to produce a useful error message.

(c::define c::check-simple-class-name-syntax
  (lambda (name)
    (if (not (symbol? name))
      (begin
        (newline (e::error-port))
        (display " -> " (e::error-port))      
        (display name (e::error-port))
        (e::error
          (string-append
            "While creating a new simple class, this object occurred instead of a name for the class."))))
    #t))


;; Straighforward check for whether the proposed instance-variable name is a symbol:
;;
;; Most of the work here is to produce a useful error message.

(c::define c::check-simple-class-ivar-syntax
  (lambda (name class-name message1 message2)
    (if (not (symbol? name))
      (begin
        (newline (e::error-port))
        (display " -> " (e::error-port))      
        (display name (e::error-port))
        (e::error
          (string-append
            message1
            " \""
            (symbol->string class-name)
            "\", this object occurred instead of "
            message2
            "."
            ))))
    #t))


;; Straighforward check for whether the proposed binding pair is in 
;;   fact a list of length 2.
;;
;; Most of the work here is to produce a useful error message.

(c::define c::check-simple-class-binding-pair-is-pair
  (lambda (element class-name message)
    (if (not (and (list? element) (= (length element) 2)))
      (begin
        (newline (e::error-port))
        (display " -> " (e::error-port))      
        (display element (e::error-port))
        (e::error
          (string-append
            message
            " \""
            (symbol->string class-name)
            "\", this object occurred instead of a list of the form (name value)."))))
    #t))


;; Straighforward check for whether the proposed list of binding pairs
;;   is in fact a proper list.
;;
;; Most of the work here is to produce a useful error message.

(c::define c::check-simple-class-list-is-list
  (lambda (lyst class-name message1 message2)
    (if (not (list? lyst))
      (begin
        (newline (e::error-port))
        (display " -> " (e::error-port))      
        (display lyst (e::error-port))
        (e::error
          (string-append
            message1
            " \""
            (symbol->string class-name)
            "\", this object occurred instead of a list of (name value) sublists"
            " "
            message2
            "."
            ))))
    #t))


;; Straighforward check for whether the given class has an instance variable
;;   whose name is the symbol 'x'.
;;
;; Most of the work here is to produce a useful error message.

(c::define c::check-simple-class-has-ivar
  (lambda (x all-variable-names class-name message lyst)
    (if (not (symbol? x))
      (begin
        (if lyst
          (c::simple-class-format-call class-name lyst))
        (newline (e::error-port))
        (display " -> " (e::error-port))      
        (display x (e::error-port))
        (e::error
          (string-append
            "In a call to an instance of simple class \""
            (symbol->string class-name)
             "\" "
            message
            ", this object occurred instead of a variable name."))))
    (if (not (member x all-variable-names))
      (begin
        (if lyst
          (c::simple-class-format-call class-name lyst))
        (e::error
            (string-append
              "In a call to an instance of simple class \""
              (symbol->string class-name)
               "\" "
               message
               ", there is no variable named \""
               (symbol->string x)
               "\"."))))
    #t))


;; Portmanteau for a bunch of syntax checking to be done 
;;   when creating a new class.

(c::define c::check-make-simple-class-syntax
  (lambda (name class-binding-pairs binding-pairs)
    (c::check-simple-class-name-syntax name)
    (c::check-simple-class-list-is-list
      class-binding-pairs
      name
      "While creating a new simple class named"
      "for the class variable bindings"
      )
    (c::check-simple-class-list-is-list binding-pairs name "While creating a new simple class named" "for the instance bindings")
    (map
      (lambda (pair) (c::check-simple-class-binding-pair-is-pair pair name "While creating a new simple class named"))
       class-binding-pairs)
    (map
      (lambda (pair) (c::check-simple-class-binding-pair-is-pair pair name "While creating a new simple class named"))
       binding-pairs)
    (map 
      (lambda (pair)
        (c::check-simple-class-ivar-syntax (car pair) name "While creating a new simple class named" "a class variable name"))
      class-binding-pairs)
    (map 
      (lambda (pair)
        (c::check-simple-class-ivar-syntax (car pair) name "While creating a new simple class named" "an instance variable name"))
      binding-pairs)
    ))  


;; Portmanteau for a bunch of syntax checking to be done 
;;   when creating a new instance of an existing class.

(c::define c::check-make-simple-class-instance-syntax
  (lambda (name more-binding-pairs)
    (c::check-simple-class-list-is-list
      more-binding-pairs
      name 
      "While creating an instance of simple class"
      "for the instance bindings"
      )
    (map
      (lambda (pair) (c::check-simple-class-binding-pair-is-pair pair name "While creating an instance of simple class"))
      more-binding-pairs)
    (map
      (lambda (pair)
        (c::check-simple-class-ivar-syntax (car pair) name "While creating an instance of simple class" "a variable name"))
      more-binding-pairs)
    ))


;; This procedure is called when a new instance of a class is made with a non-null list of
;;   bindings, that override some of the default bindings of the class.  
;;
;; It first checks that the given class actually has all the instance variables mentioned
;;   in the list of new bindings, then goes ahead and changes the list of bindings.

(c::define c::set!-new-bindings
  (lambda (class-name all-variable-names original-bindings more-bindings)
    (map
      (lambda (new-binding)
        (c::check-simple-class-has-ivar (car new-binding) all-variable-names class-name "that initializes that instance" #f))
        more-bindings)
    (map
      (lambda (new-binding)
        (set-car! (member (assoc (car new-binding) original-bindings) original-bindings)
                  (list (car new-binding) (cadr new-binding))))
      more-bindings)))


;; This procedure constructs a duplicate of the list of instance variable names given (which is
;;   assumed to be gotten from a class, EXCEPT for the ones that are members of every
;;   class -- that is, "all-variable-names" and so on.
;;
;; The proper working of this procedure depends on the fact that "all-variable-names" is
;;   the first of the instance names to be omitted -- recursion simply stops when it
;;   sees that value.

(c::define c::duplicate-simple-all-variable-names
  (lambda (superinstance-all-variable-names)
    (cond ((null? superinstance-all-variable-names)     ;; Defensive programming.
           (error: "Badly-formed list of instance names passed to \"c::duplicate-simple-all-variable-names\".")         )
          ((equal? (car superinstance-all-variable-names) 'all-variable-names) '())
          ((cons (car superinstance-all-variable-names)
                 (c::duplicate-simple-all-variable-names (cdr superinstance-all-variable-names)))))))
  

;; This procedure constructs a duplicate of the list of class variable names given (which is
;;   assumed to be gotten from a class;;

(c::define c::duplicate-simple-class-variable-names
  (lambda (super-class-variable-names)
    (cond ((null? super-class-variable-names) '())
          ((cons (car super-class-variable-names)
                 (c::duplicate-simple-class-variable-names (cdr super-class-variable-names)))))))
  

;; This procedure is an auxiliary for the procedure immediately below it.
;;
;; The proper working of this procedure depends on the fact that "all-variable-names" is
;;   the first of the instance names to be omitted -- recursion simply stops when it
;;   sees that value.  

(c::define c::duplicate-simple-instance-bindings-aux
  (lambda (superinstance superinstance-all-variable-names)
    (cond ((null? superinstance-all-variable-names)     ;; Defensive programming.
           (error: "Badly-formed list of instance names passed to \"c::duplicate-simple-instance-bindings-aux\"."))
          ((equal? (car superinstance-all-variable-names) 'all-variable-names) '())
          ((cons (list (car superinstance-all-variable-names) (superinstance (car superinstance-all-variable-names)))
                 (c::duplicate-simple-instance-bindings-aux superinstance (cdr superinstance-all-variable-names)))))))
  

;; This procedure constructs a duplicate of the list of names/value bindings for the
;;   given class, EXCEPT for the bindings to instance variables that are in every
;;   class -- that is, "all-variable-names" and so on.
;;
;; The proper working of this procedure depends on the fact that "all-variable-names" is
;;   the first of the instance names to be omitted -- recursion simply stops when it
;;   sees that value.  

(c::define c::duplicate-simple-instance-bindings
  (lambda (the-simple-class)
    (let ((instance (the-simple-class '())))
      (c::duplicate-simple-instance-bindings-aux instance (instance 'all-variable-names)))))    


;; This procedure is an auxiliary for the procedure immediately below it.

(c::define c::duplicate-simple-class-bindings-aux
  (lambda (superinstance superinstance-class-variable-names)
    (cond ((null? superinstance-class-variable-names) '())
          ((cons (list (car superinstance-class-variable-names) (superinstance (car superinstance-class-variable-names)))
                 (c::duplicate-simple-class-bindings-aux superinstance (cdr superinstance-class-variable-names)))))))
  

;; This procedure constructs a duplicate of the list of names/value bindings for the
;;   given class, EXCEPT for the bindings to instance variables that are in every
;;   class -- that is, "all-variable-names" and so on.
;;
;; The proper working of this procedure depends on the fact that "all-variable-names" is
;;   the first of the instance names to be omitted -- recursion simply stops when it
;;   sees that value.  

(c::define c::duplicate-simple-class-bindings
  (lambda (the-simple-class)
    (let ((instance (the-simple-class '())))
      (c::duplicate-simple-class-bindings-aux instance (instance 'class-variable-names)))))    


;; Auxiliary for the function immediately following.

(c::define c::report-duplicate-simple-class-variable-names-aux
  (lambda (variable-names duplicates)
    (cond
      ((null? variable-names) duplicates)
      ((and (not (member (car variable-names) duplicates))
            (member (car variable-names) (cdr variable-names)))
       (c::report-duplicate-simple-class-variable-names-aux
         (cdr variable-names)
         (cons (car variable-names) duplicates)))
      (#t
        (c::report-duplicate-simple-class-variable-names-aux
          (cdr variable-names)
          duplicates)))))


;; Warn of any duplicate variable names.

(c::define c::report-duplicate-simple-class-variable-names
  (lambda (class-name variable-names)
    (let
      ((duplicates
       (c::report-duplicate-simple-class-variable-names-aux
         variable-names
         '())))
      (if (not (null? duplicates))
        (e::warning
          (string-append
            "Class \""
            (symbol->string class-name)
            "\" has duplicates of the following variable names:\n"
            (apply
              string-append
              (map
                (lambda (x)
                  (string-append "    " (symbol->string x) "\n"))
              (reverse duplicates))))))
      #t)))




;; Here is the procedure for making a new class.  It returns a lambda expression
;;   that makes instances of the new class.  That lambda expression in turn
;;   returns *another* lambda expression, which is in essence a big case statement
;;   that handles all the things you can do to an instance -- set, get, and apply
;;   instance variables.
;;
;; There are lots of syntax-checking functions.
;;
;; It is important to copy various lists, to avoid inadvertantly sharing structure
;;   between instances, or between different subclasses of the same superclass.
;;
;; The calling form here is
;;
;;    (e::make-simple-class
;;      <a symbol naming the class>
;;      <a list of superclasses, or the empty list if there is none>
;;      <list of symbol/value lists for class variables and their initial bindings>
;;      <list of symbol/value lists for instance variables and their default bindings>
;;      )

(c::define e::make-simple-class
  (lambda (name superclasses class-binding-pairs binding-pairs)
    (c::check-make-simple-class-syntax name class-binding-pairs binding-pairs)
  
    ;; The following let takes note of whether a superclass was
    ;;   provided (and has no way to type-check the passed argument).
    ;;   If so, it constructs an instance of the superclass, which
    ;;   is to be used to get the instance variables and so on of
    ;;   the superclass.
  
    (let ((superinstances (if (null? superclasses) '() (map (lambda( superclass ) (superclass '())) superclasses))))
  
      ;; The following let creates local COPIES of all the relevant
      ;;   instance names, bindings, and of the inheritance.  It
      ;;   appends in the stuff from the superclass, if there is one.
  
      (let ((class-variable-names-local
              (if (null? superclasses)
                (map car class-binding-pairs)
                (append
                  (map car class-binding-pairs)
                  (apply
                    append
                    (map
                      (lambda (superinstance)
                        (c::duplicate-simple-class-variable-names (superinstance 'class-variable-names)))
                      superinstances)))))
            (all-variable-names-local
              (if (null? superclasses)
                (map car binding-pairs)
                (append
                  (map car binding-pairs)
                  (apply
                    append
                    (map
                      (lambda (superinstance)
                        (c::duplicate-simple-all-variable-names (superinstance 'all-variable-names)))
                      superinstances)))))
            (class-bindings-local
              (if (null? superclasses)
                (map (lambda (pair) (list (car pair) (cadr pair))) class-binding-pairs)
                (append
                  (map (lambda (pair) (list (car pair) (cadr pair))) class-binding-pairs)
                  (apply
                    append
                    (map
                      (lambda( superclass )
                        (c::duplicate-simple-class-bindings superclass))
                      superclasses)))))
            (instance-bindings-local
              (if (null? superclasses)
                (map (lambda (pair) (list (car pair) (cadr pair))) binding-pairs)
                (append
                  (map (lambda (pair) (list (car pair) (cadr pair))) binding-pairs)
                  (apply
                    append
                    (map
                      (lambda( superclass )
                        (c::duplicate-simple-instance-bindings superclass))
                      superclasses)))))
            (inheritance-local
              (if (null? superclasses)
                (list name)
                (cons name (map (lambda (superinstance) (superinstance 'inheritance)) superinstances))))
            )
  
       ;; This is the start of the lambda expression that creates a new
       ;;   instance of the given class.  It is called with a possibly null
       ;;   list of overrides for none, some, or all of the default bindings
       ;;   for the class -- instance variables or class variables, or both.
  
       (lambda (more-binding-pairs)
         (c::check-make-simple-class-instance-syntax name more-binding-pairs)
  
         ;; This let* defines the name, instance names, and bindings for the new instance.
  
         (let* ((class-name name)
                (all-variable-names
                  (append
                    all-variable-names-local
                    (list 'all-variable-names 'inheritance 'class-variable-names 'class-name 'this)
                    class-variable-names-local
                    ))
                (bindings
                  ((lambda (lyst) (set-cdr! (c::simple-class-last-pair lyst) class-bindings-local) lyst)
                    (append
                      (map (lambda (x) x) instance-bindings-local)
                      (list (list 'all-variable-names all-variable-names)
                            (list 'inheritance inheritance-local) 
                            (list 'class-variable-names class-variable-names-local)
                            (list 'class-name class-name)
                            (list 'this '())
                        )
                    ))))
  
               ;; The next lines warns of duplicate variable names.

               (c::report-duplicate-simple-class-variable-names class-name all-variable-names)

               ;; The next line sets any bindings that were overridden in the procedure call that
               ;;   created the new instance.
  
               (c::set!-new-bindings class-name all-variable-names bindings more-binding-pairs)
  
               ;; This is the start of the lambda expression that constitutes the new instance
               ;;   new instance of the given class. 
  
               (let 
                ((this
                   (lambda lyst
                     (case (length lyst)
    
                       ;; There is only one valid cases of length one, a getter for an instance variable, such as
                       ;;
                       ;;   (class-instance 'my-instance-variable)
                       ;;
    
                       ((1)
    
                            (c::check-simple-class-has-ivar
                              (car lyst)
                              all-variable-names
                              class-name
                              "that reads a variable"
                              lyst
                              )
                            (cadr (assoc (car lyst) bindings))
                         )
    
    
                       ;; There are two valid cases of length two.  The first is a request to block a binding, such as
                       ;;
                       ;;   (class-instance 'block-binding 'my-instance-variable)
                       ;;
                       ;;   The second is running a method with no arguments, such as
                       ;;
                       ;;   (class-instance 'method 'my-method-with-no-arguments)

                       ((2)

                            (if (and (symbol? (car lyst)) (equal? (car lyst) 'block-binding))
                              (begin 
                                (c::check-simple-class-has-ivar
                                  (cadr lyst)
                                  all-variable-names
                                  class-name
                                  "that blocks the binding of a variable"
                                 lyst
                                  )
                                (e::block-any-binding 
                                  (cdr (assoc (cadr lyst) bindings))))
                            (if (and (symbol? (car lyst)) (equal? (car lyst) 'method))
                              (begin 
                                (c::check-simple-class-has-ivar
                                  (cadr lyst)
                                  all-variable-names
                                  class-name
                                  "that uses a variable as a method"
                                  lyst
                                  )
                                (if (not (procedure? (cadr (assoc (cadr lyst) bindings))))
                                  (e::error
                                    (string-append
                                      "Trying to use variable "
                                      (symbol->string (cadr lyst))
                                      " as a method of class "
                                      (symbol->string class-name)
                                      ", but that variable is not bound to a procedure."
                                      )))
                                  (apply (cadr (assoc (cadr lyst) bindings)) (list (cadr (assoc 'this bindings)))))

                            ;; else

                            (begin
                              (c::simple-class-format-call name lyst)
                              (e::error
                                (string-append
                                  "Unknown syntax of a call to an instance of simple class \""
                                  (symbol->string class-name)
                                  "\"."
                                  )))))
                          )


                       ;; There are three valid cases of length three.  The first is a setter, such as
                       ;;
                       ;;   (class-instance 'set! 'my-instance-variable new-value)
                       ;;
                       ;;   The second is running a method with one argument, such as
                       ;;
                       ;;   (class-instance 'method 'my-method-with-one-argument argument)
                       ;;
                       ;;   The third is a set of a blocked binding (obtained from 'block-binding), such as
                       ;;
                       ;;   (class-instance 'set-blocked-binding a-binding new-value)

                       ((3) 
    
                            (if (and (symbol? (car lyst)) (equal? (car lyst) 'set!))
                              (begin 
                                (c::check-simple-class-has-ivar
                                  (cadr lyst)
                                  all-variable-names
                                  class-name
                                  "that set!s a variable"
                                  lyst
                                  )
                                (set-car! (member (assoc (cadr lyst) bindings) bindings)
                                          (list (cadr lyst) (caddr lyst))))

                            ;; else

                            (if (and (symbol? (car lyst)) (equal? (car lyst) 'method))
                              (begin 
                                (c::check-simple-class-has-ivar
                                  (cadr lyst)
                                  all-variable-names
                                  class-name
                                  "that uses a variable as a method"
                                  lyst
                                  )
                                (if (not (procedure? (cadr (assoc (cadr lyst) bindings))))
                                  (e::error
                                    (string-append
                                      "Trying to use variable "
                                      (symbol->string (cadr lyst))
                                      " as a method of class "
                                      (symbol->string class-name)
                                      ", but that variable is not bound to a procedure."
                                      )))
                                  (apply (cadr (assoc (cadr lyst) bindings)) (cons (cadr (assoc 'this bindings)) (cddr lyst)))
                                )

                            ;; else

                            (if (and (symbol? (car lyst)) (equal? (car lyst) 'set-blocked-binding!) (pair? (cadr lyst)))
                              (c::set-blocked-binding! (cadr lyst) (caddr lyst))
                              (begin
                                (c::simple-class-format-call name lyst)
                                (if (and (symbol? (car lyst)) (equal? (car lyst) 'set-blocked-binding!) (not (pair? (cadr lyst))))
                                  (begin
                                    (display "\n -> " (e::error-port))
                                    (display (cadr lyst) (e::error-port))
                                    (display "\n    That second argument is certainly not a lock." (e::error-port))
                                    (e::error
                                      (string-append
                                        "Incorrect syntax in an attempt to set-blocked-binding! in an instance of simple class \""
                                        (symbol->string class-name)
                                        "\".")))
                                  (e::error
                                    (string-append
                                      "Unknown syntax of a call to an instance of simple class \""
                                      (symbol->string class-name)
                                     "\".")))
                                  )
                                )
                              )
                            )
                          )
    
    
                       ;;  No arguments is a mistake.
    
                       ((0)
    
                         (e::error
                           (string-append
                             "No arguments passed in a call to an instance of simple class \""
                             (symbol->string class-name)
                             "\"."))
                         )
    
    
                       ;; All other cases can only be applications with some number of arguments, such as
                       ;;
                       ;;  (class-instance 'my-method-with-three-or-more-arguments arg1 arg2 arg3 ...)
     
                       (else
    
                         (if (and (symbol? (car lyst)) (equal? (car lyst) 'method))
                           (begin 
                             (c::check-simple-class-has-ivar
                               (cadr lyst)
                               all-variable-names
                               class-name
                               "that uses a variable as a method"
                               lyst
                               )
                             (if (not (procedure? (cadr (assoc (cadr lyst) bindings))))
                               (e::error
                                 (string-append
                                   "Trying to use variable "
                                   (symbol->string (cadr lyst))
                                   " as a method of class "
                                   (symbol->string class-name)
                                   ", but that variable is not bound to a procedure."
                                   )))
                                (apply (cadr (assoc (cadr lyst) bindings)) (cons (cadr (assoc 'this bindings)) (cddr lyst)))
                             )
                           (begin
                             (c::simple-class-format-call name lyst)
                             (e::error
                               (string-append
                                 "Unknown syntax of a call to an instance of simple class \""
                                 (symbol->string class-name)
                                 "\".")))
                            )
                         )
                       )
                     )
                   ))
               (this 'set! 'this this)
               this
               )))))))
