//
//  BadgedIconView.m
//
//  Put simple badges on program icons.  Copyright (c) 2009 by Jay Reynolds Freeman

/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628
*/

#import "BadgedIconView.h"

@implementation BadgedIconView

- (id) initWithImage:(NSImage *)anImage andString:(NSString *)aString { 

    // Retain the image and string that were passed in.

    theImage  = [anImage retain];
    theString = [aString retain];

    // Define the color for the background of the badge.  This is kind of a shiny gray ...

    NSColor *badgeColor = [NSColor colorWithCalibratedRed:0.922 green:0.922 blue:.922 alpha:1.00];

    // Set up this view to match the size of the image.

    NSRect imageFrame;

    imageFrame.origin = NSZeroPoint;
    imageFrame.size   = [theImage size];

    self = [super initWithFrame:imageFrame];

    // The size and location of the badge is based on an aesthetic judgment of what
    // looks good for the particular image I have in mind, the particular font, and so on.
    // It would be easy to generalize the init method to provide this information at call time,
    // but for use with a single image and badge type, I chose to hard-code it here.

    // The intent is to create a frame the right size for a string of one, two, or
    // three characters.  We trust the developer -- me --  to pass in the right sort of string.

    if( [theString length] <= 1 ) {
        badgeFrame.origin.x    = 48.0;
        badgeFrame.size.width  = 32.0;
        }
    else if( [theString length] <= 2 ) {
        badgeFrame.origin.x    = 40.0;
        badgeFrame.size.width  = 48.0;
        }
    else {
        badgeFrame.origin.x    = 32.0;
        badgeFrame.size.width  = 64.0;
        }

    badgeFrame.origin.y    = 4.0;
    badgeFrame.size.height = 28.0;

    // Make an instance of NSText to contain the text, and add it to the view hierarchy of this view.

    badgeNSText = [[[NSText alloc] initWithFrame:badgeFrame] retain];
    [self addSubview:badgeNSText];

    // Set assorted text properties, and load in the string.

    [badgeNSText alignCenter:self];
    [badgeNSText setBackgroundColor:badgeColor];
    [badgeNSText setDrawsBackground:YES];
    [badgeNSText setFont:[NSFont boldSystemFontOfSize:24.0]];
    [badgeNSText setString:theString];

    // Back to the caller ...

    return [self autorelease];

    }

- (void)drawRect:(NSRect)dirtyRect {

    // Most of the setup for drawing was done in the initialization method.

    [theImage drawAtPoint:NSZeroPoint fromRect:NSZeroRect operation:NSCompositeCopy fraction:1.0];
    [badgeNSText display];

    }


@end
