/****************************************************************************************

SocketClient.c++ -- Copyright (c) by Jay Reynolds Freeman 2009.

Demonstration program for memory-mapping and interrupts in connection with Wraith Scheme.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

/****************************************************************************************

    USE:  This is one of two C++ programs, which in connection with Wraith Scheme,
          demonstrates the use of Unix sockets,  memory-mapping and interrupts in
          the Wraith Scheme foreign-function interface.  The two programs' source
          files are

              SocketServer.c++
              SocketClient.c++

          See the section in the Wraith Scheme help file titled

            "Demonstrating the Foreign-Function Interface with Other Programs"

          Copy the two files somewhere handy, and compile them with

            g++ -o SocketServer SocketServer.c++
            g++ -o SocketClient SocketClient.c++

          Then follow the instructions in the help file.

****************************************************************************************/

/****************************************************************

;; For reference, here is the Scheme code required for the demonstration.

(define server-mmapped-block (e::make-memory-mapped-block 1000 "/tmp/serverMMap"))
(c::set-interrupt-handler! 42 0
  '(begin (display (e::peek-c-string server-mmapped-block 0)) (newline)))

(define client-mmapped-block (e::make-memory-mapped-block 1000 "/tmp/clientMMap"))

(e::memory-map-block-in-current-kitten server-mmapped-block)  ;; In the MomCat

(e::poke-c-string client-mmapped-block 0 "Your favorite string.")

(e::system "kill -2 <Unix process identification of the SocketClient program>")

****************************************************************/

#include <sys/un.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <signal.h>

#define SOCKET_ERROR        -1
#define BUFFER_SIZE         100
#define SOCKET_NAME    "/tmp/testSocket"
#define CLIENT_MMAP_AREA_NAME "/tmp/clientMMap"
#define TRUE 1
#define FALSE 0
#define MMAP_FLAGS (MAP_SHARED | MAP_FILE | MAP_HASSEMAPHORE)
#define MMAP_FILE_FLAGS (O_CREAT | O_RDWR)

extern int errno;
typedef struct socketaddr_un *pSAun;

static int clientSocketID = 0;
static char *messageBuffer = 0;

void dealWithMessage( int ignore )
{
    // Write what we received. 

    write( clientSocketID, messageBuffer, strlen( messageBuffer ) + 1 );
    fprintf( stderr, "Client wrote \"%s\" to server.\n", messageBuffer );
    fflush( stderr );

    *messageBuffer = 0;
    }

int  main(int argc, char* argv[])
{
    struct sockaddr_un clientSocketAddress;
    unsigned nReadAmount = 0;
    sigset_t sa_mask;

    if( argc != 1 ) {
        fprintf( stderr, "\nUsage: SocketClient\n");
        fflush( stderr );
	    exit( 1 );
        }

    int mmappedFile = open( CLIENT_MMAP_AREA_NAME, MMAP_FILE_FLAGS, 0666 );

    if( mmappedFile == -1 ) {
        fprintf( stderr, "Client mmapped file open failed with error %d, \"%s\"\n", errno, strerror( errno ));
        fflush( stderr );
        exit( errno );
        }

    char *sharedSpace = (char *)mmap( 0, 1000, PROT_READ | PROT_WRITE, MMAP_FLAGS, mmappedFile, 0 );
    messageBuffer = sharedSpace;
 
    if( sharedSpace == MAP_FAILED ) {
        fprintf( stderr, "Client mmap of private file failed with error %d, \"%s\"\n", errno, strerror( errno ));
        fflush( stderr );
        exit( errno );
        }

    fprintf( stderr, "Making client socket:\n");

    if( (clientSocketID = socket( AF_UNIX, SOCK_STREAM, 0)) == SOCKET_ERROR ) {
        fprintf( stderr, "Call to \"socket\" failed.\n");
        fflush( stderr );
        exit( 1 );
        }

    /* fill address struct */
    strcpy( clientSocketAddress.sun_path, SOCKET_NAME );
    clientSocketAddress.sun_family = AF_UNIX;

    fprintf( stderr, "Client is connecting ...\n");
    fflush( stderr );
    
    if(     connect( clientSocketID,
                     (struct sockaddr*)&clientSocketAddress,
                     sizeof( clientSocketAddress )) 
         == SOCKET_ERROR)
        {
        fprintf( stderr, "Client could not connect to server.\n");
        fflush( stderr );
        exit( 1 );
        }
    
    // Set up to handle interrupt to be sent by Wraith Scheme
    // when it has provided us with a string to send to the server.

    sigemptyset( &sa_mask );
    signal( SIGINT, dealWithMessage );

    fprintf( stderr, "Client has a connection, client Unix process identification is %d\n", getpid() );
    fflush( stderr );
    
    while( TRUE )
        sigsuspend( &sa_mask );
    }
	