//
//  JRF2NSTextView.m
//  WraithCocoa
//
//  Created by Jay Freeman on 12/29/06.
//  Copyright 2006 Jay Reynolds Freeman. All rights reserved.
//

// Specialized view for the response area of the dialog.
/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628
*/

#import "JRF2NSTextView.h"
#import "WraithSchemeOuterController.h"

@implementation JRF2NSTextView

- (void) setDealWithText:(SEL)aMethod { dealWithText = aMethod; }

- (void) transmitLastLine {

    // This only works if the InputView's text ends in a newline.

    NSString *allText = [self string];
    NSString *textToSend;
    NSInteger lnth = [allText length];
    NSInteger counter = 0;
    NSRange rangeToSend;
    NSRange rangeOfAll;

    rangeOfAll.location = 0;
    rangeOfAll.length = lnth;
    for( counter = lnth - 1; counter > 0; --counter ) {
        if( [allText characterAtIndex:(counter - 1)] == NEWLINE )
            break;
        }
    rangeToSend.location = counter;
    rangeToSend.length = lnth - counter;
    if( rangeToSend.length > 0 ) {
        textToSend = [NSString stringWithString:[allText substringWithRange:rangeToSend]];
        [self replaceCharactersInRange:rangeOfAll withString:@""];
        [theController performSelector:dealWithText withObject:textToSend];
        }
    }

- (void) appendNewline {

    NSRange endRange;

    endRange.location = [[self string] length];
    endRange.length = 0;
    [self replaceCharactersInRange:endRange withString:@"\n"];
    }

- (bool) atFirstCharacterInLine {

    NSArray *selRanges = [self selectedRanges];
    NSRange selectedRange;
    NSString *allText = [self string];

    if( [selRanges count] != 0 ) {
        selectedRange = [[selRanges objectAtIndex:0] rangeValue];
        if( selectedRange.location == 0 )
            return YES;
        if( [allText characterAtIndex:(selectedRange.location - 1)] == NEWLINE )
            return YES;
        }
    return NO;
    }

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender {

    NSPasteboard *pb = [sender draggingPasteboard];
    NSArray *allowedTypes = [NSArray arrayWithObjects: NSStringPboardType, nil];
    NSString *bestType = [pb availableTypeFromArray:allowedTypes];    

    if( bestType == nil )
        return NSDragOperationNone;
    [super performDragOperation:sender];
    [self dealWithTextChunk];
    return NSDragOperationGeneric;
    }

- (void) paste:(id)sender {
    NSArray *allowedTypes = [NSArray arrayWithObjects: NSStringPboardType, nil];
    NSPasteboard *pb = [NSPasteboard generalPasteboard];
    NSString *bestType = [pb availableTypeFromArray:allowedTypes];    

    if( bestType == nil ) {
        NSBeep();
        return;
        }
    NSArray *selRanges = [self selectedRanges];
    if( [selRanges count] != 0 ) {
        [super paste:sender];
        [self dealWithTextChunk];
        }
    else
        NSBeep();
    }

- (void) keyDown:(NSEvent *)theEvent {
    NSString *theCharacters = [theEvent characters];
    NSInteger c;
    
    c = [theCharacters characterAtIndex:0];

    if( c == CONTROL_M ) {
        [self appendNewline];
        [self transmitLastLine];
        }
    else if( (c == NSLeftArrowFunctionKey) || (c == KEYBOARD_DELETE) || (c == CONTROL_B) ) {
        if( ! [self atFirstCharacterInLine ] )
            [super keyDown:theEvent];
        }
    else {
        [super keyDown:theEvent];
        }
    }

- (void) dealWithTextChunk {

    // Complain about multi-line field; transmit if field ends in newline.  Otherwise, let it sit there.

    NSString *allText = [self string];
    NSInteger lnth = [allText length];
    NSInteger counter = 0;
    NSRange rangeOfAll;

    rangeOfAll.location = 0;
    rangeOfAll.length = lnth;

    for( counter = 0; counter < lnth - 1; ++counter )
        if( [allText characterAtIndex:counter] == NEWLINE ) {
            [self replaceCharactersInRange:rangeOfAll withString:@""];
            [theController localError:@"Your response to a dialog should contain only one line."];
            return;
            }
    if( [allText characterAtIndex:(lnth - 1)] == NEWLINE )
        [self transmitLastLine];
    }

- (BOOL) isOpaque { return YES; }

@end
