//
//  JRF4NSTextView.m
//  WraithCocoa
//
//  Created by Jay Freeman on 01/04/07
//  Copyright 2007 Jay Reynolds Freeman. All rights reserved.
//

//  Specialized for the input view.
/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628
*/

#import "JRF4NSTextView.h"
#import "WraithSchemeOuterController.h"
#import "ASCIITypes.h"
#import "SchemeCType.h"

@implementation JRF4NSTextView

- (void) awakeFromNib {
    allowedTypes = [[NSArray arrayWithObjects: NSStringPboardType, nil] retain];
    }

- (void) applicationDidFinishLaunching:(NSNotification *)aNotification {
    [pCButton setEnabled:NO];
    [sCButton setEnabled:NO];
    }

- (void) setDealWithText:(SEL)aMethod { dealWithText = aMethod; }

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender {

    NSPasteboard *pb = [sender draggingPasteboard];
    NSString *bestType = [pb availableTypeFromArray:allowedTypes];    
    NSString *toBePasted;
    NSString *toBeSent;
    NSRange lineGrabRange;
    NSRange selectionRange;
    NSRange lastLineBefore;
    NSRange lastLineAfter;
    NSRange finalRange;
    NSRange rangeToBeSent;
    NSInteger lnth = [[self string] length];
    NSInteger lineGrabOffset;

    if( bestType == nil )
        return NSDragOperationNone;

    NSArray *selRanges = [self selectedRanges];
    if( [selRanges count] == 0 ) {
        NSBeep();
        return NSDragOperationNone;
        }

    toBePasted = [pb stringForType:bestType];
    [self lastLineStartsAt:&(lastLineBefore.location) withLength:&(lastLineBefore.length)];
    if( [self inLastLine] )
        selectionRange = [[selRanges objectAtIndex:0] rangeValue];
    else {
        [self lineStartsAt:&(lineGrabRange.location) withLength:&(lineGrabRange.length) withSelectionOffset:&lineGrabOffset];
        if( lineGrabRange.length > 0 )
            [self appendString:[[self string] substringWithRange:lineGrabRange]];
        [self showEndRange];
        selectionRange.location = lnth + lineGrabOffset;
        selectionRange.length = 0;
        }
    [self replaceCharactersInRange:selectionRange withString:toBePasted];

    // We have now pasted the text into an appropriate part of the last line, but not sent anything about it
    // to the Scheme processor.  What we wish to send is everything from the start of the previous last line
    // up to the start of the present last line.

    [self lastLineStartsAt:&(lastLineAfter.location) withLength:&(lastLineAfter.length)];
    rangeToBeSent.location = lastLineBefore.location;
    rangeToBeSent.length = lastLineAfter.location - lastLineBefore.location;
    toBeSent = [[self string] substringWithRange:rangeToBeSent];
    [self transmitAll:toBeSent];

    finalRange.location = selectionRange.location - selectionRange.length + [toBePasted length];
    finalRange.length = 0;

    [self setSelectedRange:finalRange];
    [self scrollRangeToVisible:finalRange];

    if( [self inFirstLine] )
        [pCButton setEnabled:NO];
    else
        [pCButton setEnabled:YES];

    if( [self inLastLine] )
        [sCButton setEnabled:NO];
    else
        [sCButton setEnabled:YES];

    return NSDragOperationGeneric;
    }

- (void) paste:(id)sender {

    NSPasteboard *pb = [NSPasteboard generalPasteboard];
    NSString *bestType = [pb availableTypeFromArray:allowedTypes];    
    NSString *toBePasted;
    NSString *toBeSent;
    NSRange lineGrabRange;
    NSRange selectionRange;
    NSRange lastLineBefore;
    NSRange lastLineAfter;
    NSRange finalRange;
    NSRange rangeToBeSent;
    NSInteger lnth = [[self string] length];
    NSInteger lineGrabOffset;

    if( bestType == nil ) {
        NSBeep();
        return;
        }

    NSArray *selRanges = [self selectedRanges];
    if( [selRanges count] == 0 ) {
        NSBeep();
        return;
        }
    toBePasted = [pb stringForType:bestType];
    [self lastLineStartsAt:&(lastLineBefore.location) withLength:&(lastLineBefore.length)];
    if( [self inLastLine] )
        selectionRange = [[selRanges objectAtIndex:0] rangeValue];
    else {
        [self lineStartsAt:&(lineGrabRange.location) withLength:&(lineGrabRange.length) withSelectionOffset:&lineGrabOffset];
        if( lineGrabRange.length > 0 )
            [self appendString:[[self string] substringWithRange:lineGrabRange]];
        [self showEndRange];
        selectionRange.location = lnth + lineGrabOffset;
        selectionRange.length = 0;
        }
    [self replaceCharactersInRange:selectionRange withString:toBePasted];

    // We have now pasted the text into an appropriate part of the last line, but not sent anything about it
    // to the Scheme processor.  What we wish to send is everything from the start of the previous last line
    // up to the start of the present last line.

    [self lastLineStartsAt:&(lastLineAfter.location) withLength:&(lastLineAfter.length)];
    rangeToBeSent.location = lastLineBefore.location;
    rangeToBeSent.length = lastLineAfter.location - lastLineBefore.location;
    toBeSent = [[self string] substringWithRange:rangeToBeSent];
    [self transmitAll:toBeSent];

    finalRange.location = selectionRange.location - selectionRange.length + [toBePasted length];
    finalRange.length = 0;

    [self setSelectedRange:finalRange];
    [self scrollRangeToVisible:finalRange];

    if( [self inFirstLine] )
        [pCButton setEnabled:NO];
    else
        [pCButton setEnabled:YES];

    if( [self inLastLine] )
        [sCButton setEnabled:NO];
    else
        [sCButton setEnabled:YES];
    }

- (void) transmitLastLine {

    // This only works if the InputView's text ends in a newline.

    NSString *allText = [self string];
    NSString *textToSend;
    NSInteger lnth = [allText length];
    NSInteger counter = 0;
    NSRange rangeToSend;

    for( counter = lnth - 1; counter > 0; --counter )
        if( [allText characterAtIndex:(counter - 1)] == NEWLINE )
            break;

    rangeToSend.location = counter;
    rangeToSend.length = lnth - counter;
    if( rangeToSend.length > 0 ) {
        textToSend = [NSString stringWithString:[allText substringWithRange:rangeToSend]];
        [theController performSelector:dealWithText withObject:textToSend];
        }
    }

- (void) appendNewline { [self appendString:@"\n"]; }

- (void) appendString:(NSString *)aString {

    NSRange endRange;

    endRange.location = [[self string] length];
    endRange.length = 0;
    [self replaceCharactersInRange:endRange withString:aString];

    }

- (void) insertString:(NSString *)aString {

    NSArray *selRanges = [self selectedRanges];
    NSRange selectionRange;;

    if( [selRanges count] != 0 ) {
        selectionRange = [[selRanges objectAtIndex:0] rangeValue];
        [self replaceCharactersInRange:selectionRange withString:aString];
        }
    }

- (void) showEndRange {

    NSRange endRange;

    endRange.location = [[self string] length];
    endRange.length = 0;
    [self setSelectedRange:endRange];
    [self scrollRangeToVisible:endRange];

    }

- (void) showSelectedRange {

    NSArray *selRanges = [self selectedRanges];
    NSRange theRange;
    NSUInteger theLength = 0;

    if( [selRanges count] != 0 ) {
        theRange = [[selRanges objectAtIndex:0] rangeValue];
        if( theRange.length == 0 ) {
            if( ([self string] != nil) && ((theLength = [[self string] length]) > 0 ) ) {
                if( theRange.location < theLength )
                    theRange.length = 1;
                else {
                    theRange.location = theRange.location - 1;
                    theRange.length = 1;
                    }
                [self scrollRangeToVisible:theRange];
                }
            }

        // This next if statement fixes slight glitchiness in where in the
        // window the text is displayed, depending on whether we got to it by
        // scrolling up or down.  There is an off-by-one.  Best aesthetics
        // turns out to be by scrolling down, so we force that when possible.

//        if( ![self inFirstLine] ) {
//            [super moveUp:self];
//            [super moveDown:self];
//            }
        }
    }

/****

- (void) showSelectedRange {
    NSArray *selRanges = [self selectedRanges];
    if( [selRanges count] != 0 ) {
        [self scrollRangeToVisible:[[selRanges objectAtIndex:0] rangeValue]];

        // This next if statement fixes slight glitchiness in where in the
        // window the text is displayed, depending on whether we got to it by
        // scrolling up or down.  There is an off-by-one.  Best aesthetics
        // turns out to be by scrolling down, so we force that when possible.

        if( ![self inFirstLine] ) {
            [super moveUp:self];
            [super moveDown:self];
            }
        }
    }

****/

- (bool) inFirstLine {

    NSArray *selRanges = [self selectedRanges];
    NSRange selectedRange;
    NSInteger counter;
    NSString *allText = [self string];
    NSInteger lnth = [allText length];

    if( [selRanges count] != 0 ) {
        selectedRange = [[selRanges objectAtIndex:0] rangeValue];
        if( lnth > 0 )
            for( counter = selectedRange.location - 1; counter >= 0; --counter )
                if( [allText characterAtIndex:counter ] == NEWLINE )
                    return NO;
        return YES;
        }
    return NO;
    }

- (bool) inLastLine {

    NSArray *selRanges = [self selectedRanges];
    NSRange selectedRange;
    NSInteger counter;
    NSString *allText = [self string];
    NSInteger lnth = [allText length];

    if( [selRanges count] != 0 ) {
        selectedRange = [[selRanges objectAtIndex:0] rangeValue];
        for( counter = selectedRange.location; counter < lnth; ++counter )
            if( [allText characterAtIndex:counter ] == NEWLINE )
                return NO;
        return YES;
        }
    return NO;
    }

- (bool) atFirstCharacterInLine {

    NSArray *selRanges = [self selectedRanges];
    NSRange selectedRange;
    NSString *allText = [self string];

    if( [selRanges count] != 0 ) {
        selectedRange = [[selRanges objectAtIndex:0] rangeValue];
        if( selectedRange.location == 0 )
            return YES;
        if( [allText characterAtIndex:(selectedRange.location - 1)] == NEWLINE )
            return YES;
        }
    return NO;
    }

- (bool) atLastCharacterInLine {

    NSArray *selRanges = [self selectedRanges];
    NSRange selectedRange;
    NSString *allText = [self string];
    NSUInteger lnth = [allText length];

    if( [selRanges count] != 0 ) {
        selectedRange = [[selRanges objectAtIndex:0] rangeValue];
        if( selectedRange.location == lnth )
            return YES;
        if( [allText characterAtIndex:selectedRange.location] == NEWLINE )
            return YES;
        }
    return NO;
    }

- (void) lineStartsAt:(NSUInteger *)lineLocation withLength:(NSUInteger *)lineLength withSelectionOffset:(NSInteger *)selOffset {

    NSArray *selRanges = [self selectedRanges];
    NSRange selectedRange;
    NSString *allText = [self string];
    NSInteger lnth = [allText length];
    NSInteger pastChar;

    if( [selRanges count] != 0 ) {
        selectedRange = [[selRanges objectAtIndex:0] rangeValue];
        for( *lineLocation = selectedRange.location; *lineLocation >= 1; --(*lineLocation) )
            if( [allText characterAtIndex:((*lineLocation) - 1)] == NEWLINE )
                break;
        *selOffset = selectedRange.location - *lineLocation;
        for( pastChar = selectedRange.location; pastChar < lnth; ++pastChar )
            if( [allText characterAtIndex:pastChar] == NEWLINE )
                break;
        *lineLength = pastChar - *lineLocation;
        }
    }

- (void) lastLineStartsAt:(NSUInteger *)lineLocation withLength:(NSUInteger *)lineLength {

    NSRange selectedRange;
    NSString *allText = [self string];
    NSUInteger lnth = [allText length];
    NSUInteger pastChar;

    selectedRange.location = lnth;
    selectedRange.length = 0;
    for( *lineLocation = selectedRange.location; *lineLocation >= 1; --(*lineLocation) )
        if( [allText characterAtIndex:((*lineLocation) - 1)] == NEWLINE )
            break;
    for( pastChar = selectedRange.location; pastChar < lnth; ++pastChar )
        if( [allText characterAtIndex:pastChar] == NEWLINE )
            break;
    *lineLength = pastChar - *lineLocation;
    }

- (void) scrollToBottom {

    NSClipView *theClipView = nil;
    NSRange endRange;
    NSPoint newPoint;
    NSRect clipViewBounds;
    NSRect mainBounds;
    NSInteger lnth = [[self string] length];

    endRange.location = lnth;
    if( endRange.location < 0 )
        endRange.location = 0;
    endRange.length = 0;

    [self scrollRangeToVisible:endRange];   // Doesn't work if last two characters are both 0x0a; erratic.  Fix-up follows:

    mainBounds = [self bounds];
    theClipView = (NSClipView *)[self superview];
    clipViewBounds = [theClipView bounds];

    newPoint.x = 0;
    newPoint.y = mainBounds.size.height - clipViewBounds.size.height;

    if( newPoint.y < 0 )
        newPoint.y = 0;

    [theClipView scrollToPoint:newPoint];
    }

- (IBAction) selectAll:(id)sender {
    NSRange lineGrabRange;
    NSInteger selectionOffset;

    [self lineStartsAt:&(lineGrabRange.location) withLength:&(lineGrabRange.length) withSelectionOffset:&selectionOffset];
    [self setSelectedRange:lineGrabRange];
    [self showSelectedRange];
    [self setSelectedRange:lineGrabRange];
    }

- (void) keyDown:(NSEvent *)theEvent {
    NSString *theCharacters = [theEvent characters];
    NSRange lineGrabRange;
    NSRange selectionRange;
    NSRange lastLineRange;
    NSInteger c;
    NSInteger selectionOffset;
    NSArray *selRanges = [self selectedRanges];
    
    c = [theCharacters characterAtIndex:0];

    if( [self inLastLine] ) {
        if( c == CONTROL_M ) {
            [self appendNewline];
            [self transmitLastLine];
            [self showEndRange];
            }
        else if( (c == NSLeftArrowFunctionKey) || (c == CONTROL_B) ) {
            if( ! [self atFirstCharacterInLine ] )
                [super keyDown:theEvent];
            }
        else if( c == KEYBOARD_DELETE ) {
            if( (! [self atFirstCharacterInLine ]) )
                [super keyDown:theEvent];
            else if( [selRanges count] != 0 ) {
                selectionRange = [[selRanges objectAtIndex:0] rangeValue];
                if( selectionRange.length > 0 )
                    [super keyDown:theEvent];
                }
            }
        else {
            [super keyDown:theEvent];
            [self showSelectedRange];
            }
        }
    else {  // Not in last line
        if(    (c == NSUpArrowFunctionKey) || (c == NSDownArrowFunctionKey)
            || (c == CONTROL_P)            || (c == CONTROL_N) ) {
            [super keyDown:theEvent];
            [self showSelectedRange];
            }
        else if( (c == NSRightArrowFunctionKey) || (c == CONTROL_F) ) {
            if( ! [self atLastCharacterInLine ] ) {
                [super keyDown:theEvent];
                [self showSelectedRange];
                }
            }
        else if( (c == NSLeftArrowFunctionKey) || (c == CONTROL_B) ) {
            if( ! [self atFirstCharacterInLine ] ) {
                [super keyDown:theEvent];
                [self showSelectedRange];
                }
            }
        else if( (c == CONTROL_A) || (c == CONTROL_E) ) {
            [super keyDown:theEvent];
            [self showSelectedRange];
            }
        else if( c == CONTROL_M ) {
            [self lineStartsAt:&(lineGrabRange.location) withLength:&(lineGrabRange.length) withSelectionOffset:&selectionOffset];
            [self lastLineStartsAt:&(lastLineRange.location) withLength:&(lastLineRange.length)];
            [self replaceCharactersInRange:lastLineRange withString:[[self string] substringWithRange:lineGrabRange]];
            [self appendNewline];
            [self transmitLastLine];
            [self showEndRange];
            }
    	else {
            [self lineStartsAt:&(lineGrabRange.location) withLength:&(lineGrabRange.length) withSelectionOffset:&selectionOffset];
            [self lastLineStartsAt:&(lastLineRange.location) withLength:&(lastLineRange.length)];
            [self replaceCharactersInRange:lastLineRange withString:[[self string] substringWithRange:lineGrabRange]];
            selectionRange.location = lastLineRange.location + selectionOffset;
            selectionRange.length = 0;
            [self setSelectedRange:selectionRange];
            [super keyDown:theEvent];
            [self showSelectedRange];
            }
        }

    if( [self inFirstLine] )
        [pCButton setEnabled:NO];
    else
        [pCButton setEnabled:YES];

    if( [self inLastLine] ) {
        [sCButton setEnabled:NO];
        [self scrollToBottom];
        }
    else
        [sCButton setEnabled:YES];
    }

- (void) moveDown:(id)sender {
    [super moveDown:sender];
    if( sender == sCButton ) {
        if( [self inFirstLine] )
            [pCButton setEnabled:NO];
        else
            [pCButton setEnabled:YES];

        if( [self inLastLine] )
            [sCButton setEnabled:NO];
        else
            [sCButton setEnabled:YES];
        }
    }

- (void) moveUp:(id)sender {
    [super moveUp:sender];
    if( sender == pCButton ) {
        if( [self inFirstLine] )
            [pCButton setEnabled:NO];
        else
            [pCButton setEnabled:YES];

        if( [self inLastLine] )
            [sCButton setEnabled:NO];
        else
            [sCButton setEnabled:YES];
        }
    }

- (void) disableShortcuts {

    [iMPMenuItem setEnabled:NO];
    [sSWPMenuItem setEnabled:NO];
    [iPOQMMenuItem setEnabled:NO];
    [sSWQMMenuItem setEnabled:NO];
    }

- (void) enableShortcuts {

    [iMPMenuItem setEnabled:YES];
    [sSWPMenuItem setEnabled:YES];
    [iPOQMMenuItem setEnabled:YES];
    [sSWQMMenuItem setEnabled:YES];
    }

- (void)mouseDown:(NSEvent *)theEvent {

//    [self enableShortcuts];
    [super mouseDown:theEvent];
    }

- (IBAction) moveUpAndShow:(id)sender {

    [self moveUp:sender];
    [self showSelectedRange];
    }

- (IBAction) moveDownAndShow:(id)sender {

    [self moveDown:sender];
    [self showSelectedRange];
    }


- (void) transmitAll:(NSString *)theString {

    if( [theString length] > 0 ) {
        [theController performSelector:dealWithText withObject:theString];
        }
    }

- (IBAction) insertMatchedParentheses:(id)sender {

    if( [[theWindowController getMainWindow] firstResponder] == self ) {
        if( [self inLastLine] ) {
            [self insertString:@"()"];
            NSArray *selRanges = [self selectedRanges];
            NSRange selectionRange = [[selRanges objectAtIndex:0] rangeValue];
            selectionRange.location = selectionRange.location - 1;
            [self setSelectedRange:selectionRange];
            }
        else
            NSBeep();
        }
    else
        NSBeep();
    }

- (IBAction) surroundSelectionWithParentheses:(id)sender {

    NSArray *selRanges = [self selectedRanges];
    NSRange selectionRange = [[selRanges objectAtIndex:0] rangeValue];
    NSRange newRange;
    NSInteger savedLength = 0;    

    if( selectionRange.length == 0 )
        [self insertMatchedParentheses:sender];
    else {
        if( [self inLastLine] ) {
            savedLength = selectionRange.length;
            newRange.location = selectionRange.location;
            newRange.length = 0;
            [self setSelectedRange:newRange];
            [self insertString:@"("];
            newRange.location = newRange.location + savedLength + 1;
            [self setSelectedRange:newRange];
            [self insertString:@")"];
            [self setSelectedRange:newRange];
            }
        else
            NSBeep();
        }
    }

- (IBAction) insertPairOfQuotationMarks:(id)sender {

    if( [[theWindowController getMainWindow] firstResponder] == self ) {
        if( [self inLastLine] ) {
            [self insertString:@"\"\""];
            NSArray *selRanges = [self selectedRanges];
            NSRange selectionRange = [[selRanges objectAtIndex:0] rangeValue];
            selectionRange.location = selectionRange.location - 1;
            [self setSelectedRange:selectionRange];
            }
        else
            NSBeep();
        }
    else
        NSBeep();
    }

- (IBAction) surroundSelectionWithQuotationMarks:(id)sender {

    NSArray *selRanges = [self selectedRanges];
    NSRange selectionRange = [[selRanges objectAtIndex:0] rangeValue];
    NSRange newRange;
    NSInteger savedLength = 0;    

    if( selectionRange.length == 0 )
        [self insertPairOfQuotationMarks:sender];
    else {
        if( [self inLastLine] ) {
            savedLength = selectionRange.length;
            newRange.location = selectionRange.location;
            newRange.length = 0;
            [self setSelectedRange:newRange];
            [self insertString:@"\""];
            newRange.location = newRange.location + savedLength + 1;
            [self setSelectedRange:newRange];
            [self insertString:@"\""];
            [self setSelectedRange:newRange];
            }
        else
            NSBeep();
        }
    }



// Overridden to allow checking for built-in Scheme names.

- (NSArray *)completionsForPartialWordRange:(NSRange)charRange indexOfSelectedItem:(NSInteger *)i {

    NSArray *answer = [theController spellingCompletionsForPartialWord:[[self string] substringWithRange:charRange]];

    if( [answer count] == 1 )
        *i = 0;
    else
        *i = -1;

    return answer;

    }

- (NSRange)rangeForUserCompletion {

    NSArray *selRanges = [self selectedRanges];
    NSRange selectionRange = [[selRanges objectAtIndex:0] rangeValue];
    NSString *theString = [self string];
    NSRange destinationRange = selectionRange;
    SInt32 c = 0;    
    SInt64 idx = 0;

    for( idx = selectionRange.location - 1; idx >= 0; --idx ) {
        c = [theString characterAtIndex:idx];
        if( isSubsequent_M( c ) || isInitial_M( c ) ) {
            destinationRange.location = idx;
            (destinationRange.length)++;
            }
        else
           break;
        }
    return destinationRange;
    }

- (BOOL) isOpaque { return YES; }

@end
