//
//  MainWindowController.h
//  WraithCocoa
//
//  Created by Jay Freeman on 12/16/06.
//  Copyright 2006 Jay Reynolds Freeman. All rights reserved.
//
/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628
*/

#import <Cocoa/Cocoa.h>
#import <AppKit/NSColor.h>

@class JRFNSTextView;
@class JRF2NSTextView;
@class JRF4NSTextView;

@interface MainWindowController : NSObject {

    NSSound *buttonMeow;
    NSSound *disasterMeow;
    NSSound *kittenMeow;
    CGFloat currentFontSize;
    CGFloat currentLineSize;
    NSColor *normalBackgroundColor;
    NSColor *normalTextColor;
    NSColor *black;
    NSColor *currentBrightColor;
    NSColor *transparent;
    NSInteger colorCycleCount;
    NSColor *green;
    NSColor *currentBackgroundColor;
    NSColor *currentTextColor;
    bool promptIsBright;

    IBOutlet NSTextView *cosmeticArea1;     //  Little patch left of ResponseView, that has to be the right color.
    IBOutlet NSTextField *cosmeticArea2;    //  Box around MessageView, PromptView, and ResponseView, that doesn't draw
                                            //  its background:  Effectively, a border for all three of those views.

    IBOutlet NSTextField *expectView;
    IBOutlet JRF4NSTextView *inputView;
    IBOutlet JRFNSTextView *mainView;
    IBOutlet NSTextView *messageView;
    IBOutlet NSTextView *promptView;
    IBOutlet JRF2NSTextView *responseView;

    IBOutlet NSScrollView *mainScrollView;

    IBOutlet NSWindow *mainWindow;

    IBOutlet NSTextField *compileStringField;

    IBOutlet NSButton *meowButton;
    IBOutlet NSButton *meowButtonBackdrop;

    IBOutlet NSMenu *fontMenu;
    IBOutlet NSMenuItem *fontBiggerItem;
    IBOutlet NSMenuItem *fontSmallerItem;
    IBOutlet NSMenuItem *makeInputPanelFirstResponderItem;

    IBOutlet NSProgressIndicator *progressIndicator;

    IBOutlet NSImageView *wraithIconView;
             NSImage *wraithIconImage;
             NSImage *wraithIconImageOM;
             NSImage *wraithIconImageClosedEye;

    IBOutlet NSTextField *authorField;

             NSMutableString *mainDisplayBuffer;
    bool localCopyOfNostalgiaMode;
    }

- (void) awakeFromNib;
- (void) displayAll;

- (void) setInputViewText:(NSString *)msg;

- (void) displayStringInMainView:(NSString *)str;
- (NSColor *) backgroundColor;
- (NSColor *) textColor;
- (void) setBackgroundColor:(NSColor *)aColor;
- (void) setTextColor:(NSColor *)aColor;
- (void) setMainViewText:(NSString *)msg;

- (void) setMessageViewText:(NSString *)msg;

- (void) setPrompt:(NSString *)msg;
- (void) cycleBrightColor:(NSTimer *)theTimer;
- (void) brightPrompt;
- (void) normalPrompt;

- (void) setResponse:(NSString *)msg;

- (void) setCompileString;

- (void) setColors;
- (void) setFontsToSize:(CGFloat)fontSize;
- (void) setInitialFields;
- (void) setNostalgiaMode:(bool)modeIsOn;

- (IBAction) makeFontBigger:(id)sender;
- (IBAction) makeFontSmaller:(id)sender;

CGFloat nextLargestFontSize( CGFloat fontSize );
CGFloat nextSmallestFontSize( CGFloat fontSize );
CGFloat matchingLineHeight( CGFloat fontSize );
void resizeField( NSTextField *aField, CGFloat newY, CGFloat newHeight, CGFloat newX, CGFloat newWidth );
void resizeView( NSTextView *aView, CGFloat newY, CGFloat newHeight );

- (void) setFramesForFontChange:(CGFloat)newLineSize;

- (CGFloat) currentFontSize;

- (IBAction) trimScrollback:(id)sender;

- (IBAction) playButtonMeow:(id)sender;
- (IBAction) playButtonMeowAndWait:(id)sender;
- (IBAction) playDisasterMeow:(id)sender;
- (IBAction) playDisasterMeowAndWait:(id)sender;
- (void) resizeProgressIndicator:(CGFloat)newY;
- (IBAction) makeInputPanelFirstResponder:(id)sender;
- (IBAction) enableMakeInputPanelFirstResponderItem;
- (IBAction) disableMakeInputPanelFirstResponderItem;

- (void) showIconView:(bool)visible;
- (void) winkIconView;
- (void) showIconViewAbruptly:(bool)visible;
- (void) showAuthorField:(bool)visible;
- (double) currentLineHeight;
- (NSWindow *)getMainWindow;

@end

void displayRect( char *msg, NSRect aRect );
