
//
//  MainWindowController.m
//  WraithCocoa
//
//  Created by Jay Freeman on 12/16/06.
//  Copyright 2006 Jay Reynolds Freeman. All rights reserved.
//
/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628
*/

#import "MainWindowController.h"
#import "JRFNSTextView.h"
#import "JRF2NSTextView.h"
#import "JRF4NSTextView.h"
#define WRAITH_SCHEME_FONT @"Monaco"
#define WRAITH_SCHEME_BOLD_FONT @"Monaco"
#define WRAITH_SCHEME_INITIAL_FONT_SIZE 12  // Points
#define INITIAL_MAIN_BUFFER_SIZE 4096

extern char releaseDate[];
extern int usleep(useconds_t useconds);

@implementation MainWindowController

- (void)awakeFromNib {

    normalBackgroundColor = [[mainView backgroundColor] retain];
    normalTextColor = [[mainView textColor] retain];
    green = [NSColor greenColor];
    black = [NSColor blackColor];
    transparent = [[NSColor colorWithCalibratedRed:0.00 green:0.00 blue:0.00 alpha:0.00] retain];

    currentBackgroundColor = normalBackgroundColor;
    currentTextColor = normalTextColor;
    promptIsBright = false;

    currentFontSize = WRAITH_SCHEME_INITIAL_FONT_SIZE;
    currentLineSize = matchingLineHeight( currentFontSize );

    buttonMeow = [NSSound soundNamed:@"ButtonMeow"];
    disasterMeow = [NSSound soundNamed:@"DisasterMeow"];
    kittenMeow = [NSSound soundNamed:@"KittenMeow"];
    
    [inputView setSmartInsertDeleteEnabled:NO];
    [responseView setSmartInsertDeleteEnabled:NO];

    [mainWindow center];
    [mainView scrollToBottom];
    [self setFramesForFontChange:matchingLineHeight( currentFontSize )];
    
    // The next line is a hack:  I cannot get the interface builder to set the autosize
    //   properties of just this one field to be the way I want them.  All the other fields
    //   behave as I expect, but this one is the odd entity out ... ?
    
    [messageView setAutoresizingMask:([messageView autoresizingMask] | NSViewWidthSizable )];

    [fontMenu      setAutoenablesItems:    NO];
    [fontBiggerItem   setEnabled:YES];
    [fontSmallerItem  setEnabled:YES];

    [self setInitialFields];

    wraithIconImage = [[NSImage imageNamed:@"WraithFace.png"] retain];
    wraithIconImageOM = [[NSImage imageNamed:@"WraithFaceOpenMouth.png"] retain];
    wraithIconImageClosedEye = [[NSImage imageNamed:@"WraithFace.Shut.png"] retain];
    [wraithIconView setImage:wraithIconImage];

    localCopyOfNostalgiaMode = false;

    mainDisplayBuffer = [[[NSMutableString alloc] initWithCapacity:INITIAL_MAIN_BUFFER_SIZE] retain];
    }
    
- (void)displayAll {
    [messageView display];
    [promptView display];
    [responseView display];
    [cosmeticArea1 display];
    [cosmeticArea2 display];
    [mainView display];
    [inputView display];
    }

// InputView

- (void)setInputViewText:(NSString *)msg {
 
    [inputView setString:msg];
    [inputView setFont:[NSFont fontWithName:WRAITH_SCHEME_BOLD_FONT size:currentFontSize]];
    [inputView setNeedsDisplay:YES];
    }

// MainView

- (void)displayStringInMainView:(NSString *)str {

    [mainView setString:[[mainView string] stringByAppendingString:str]];
    [mainView scrollToBottom];
    }

- (void)setMainViewText:(NSString *)msg {
 
    [mainView setString:msg];
    [mainView setFont:[NSFont fontWithName:WRAITH_SCHEME_FONT size:currentFontSize]];
    [mainView scrollToBottom];
    }

- (NSColor *)backgroundColor { return normalBackgroundColor; }

- (NSColor *)textColor { return normalTextColor; }

- (void)setBackgroundColor:(NSColor *)aColor { 

    [normalBackgroundColor release];
    normalBackgroundColor = [aColor retain];
    }

- (void)setTextColor:(NSColor *)aColor { 

    [normalTextColor release];
    normalTextColor = [aColor retain];
    }

// MessageView

- (void)setMessageViewText:(NSString *)msg {
    
    [messageView setString:@"     "];
    if( [msg length] > 0 )
        [messageView setString:[@"NOTE: " stringByAppendingString:msg]];
    else
        [messageView setString:@"     "];

    [messageView setFont:[NSFont fontWithName:WRAITH_SCHEME_FONT size:currentFontSize]];
    [messageView setNeedsDisplay:YES];
    }

// PromptView

- (void)setPrompt:(NSString *)msg {
    [promptView setString:msg];
    [promptView setFont:[NSFont fontWithName:WRAITH_SCHEME_FONT size:currentFontSize]];
    [promptView setNeedsDisplay:YES];
    }

#define COLOR_CYCLE_INTERVAL 0.5
static const int colorLoopSize = 10;

- (void)cycleBrightColor:(NSTimer *)theTimer {

    if( promptIsBright ) {
        if( ! localCopyOfNostalgiaMode ) {
            if( (colorCycleCount % colorLoopSize) != 0 )
                [promptView setTextColor:currentTextColor];
            else
                [promptView setTextColor:currentBackgroundColor];
            }
        ++colorCycleCount;

        [NSTimer scheduledTimerWithTimeInterval  :  COLOR_CYCLE_INTERVAL
                                         target  :  self
                                       selector  :  @selector(cycleBrightColor:)
                                       userInfo  :  nil
                                        repeats  :  NO];
        }
    else {
        [promptView setTextColor:currentTextColor];
        }

    }

- (void)brightPrompt { 

    promptIsBright = true;

    colorCycleCount = 1;
    [NSTimer scheduledTimerWithTimeInterval  :  COLOR_CYCLE_INTERVAL
                                     target  :  self
                                   selector  :  @selector(cycleBrightColor:)
                                   userInfo  :  nil
                                    repeats  :  NO];
    }

- (void)normalPrompt { 

    promptIsBright = false;
    [promptView setTextColor:currentTextColor];
    }


// ResponseView

- (void)setResponse:(NSString *)msg {
    
    NSRange aRange;

    [responseView setString:msg];
 
    aRange.location = 0;
    aRange.length = 0;

    [responseView scrollRangeToVisible:aRange];
    [responseView setFont:[NSFont fontWithName:WRAITH_SCHEME_FONT size:currentFontSize]];
    [responseView setNeedsDisplay:YES];
    }

// Miscellany

- (void)setCompileString {
    [compileStringField setStringValue:[NSString stringWithFormat:@"Compiled %s: 64-bit application.", releaseDate]];
    }
    

- (void)setInitialFields {
    [self setInputViewText:@""];
    [self setMainViewText:@""];
    [self setMessageViewText:@""];
    [self setCompileString];
    }

- (void)setColors {
    [inputView         setBackgroundColor:        currentBackgroundColor  ];
    [inputView         setTextColor:              currentTextColor        ];
    [inputView         setInsertionPointColor:    currentTextColor        ];

    [mainView          setBackgroundColor:        currentBackgroundColor  ];
    [mainView          setTextColor:              currentTextColor        ];

    [messageView       setBackgroundColor:        currentTextColor        ];
    [messageView       setTextColor:              currentBackgroundColor  ];

    [promptView        setBackgroundColor:        currentBackgroundColor  ];
    [promptView        setTextColor:              currentTextColor        ];

    [responseView      setBackgroundColor:        currentBackgroundColor  ];
    [responseView      setTextColor:              currentTextColor        ];
    [responseView      setInsertionPointColor:    currentTextColor        ];

    [cosmeticArea1     setBackgroundColor:        currentBackgroundColor  ];
    }
    
- (void)setFontsToSize:(CGFloat)fontSize {
    [inputView    setFont:[NSFont fontWithName:WRAITH_SCHEME_BOLD_FONT size:fontSize]];
    [mainView     setFont:[NSFont fontWithName:WRAITH_SCHEME_FONT      size:fontSize]];
    [messageView  setFont:[NSFont fontWithName:WRAITH_SCHEME_FONT      size:fontSize]];
    [promptView   setFont:[NSFont fontWithName:WRAITH_SCHEME_FONT      size:fontSize]];
    [responseView setFont:[NSFont fontWithName:WRAITH_SCHEME_FONT      size:fontSize]];
    }

- (CGFloat) currentFontSize { return currentFontSize; }

- (void)setNostalgiaMode:(bool)modeIsOn {
    localCopyOfNostalgiaMode = modeIsOn;
    if( modeIsOn ) {
        currentBackgroundColor = black;
        currentTextColor = green;
        }
    else {
        currentBackgroundColor = normalBackgroundColor;
        currentTextColor = normalTextColor;
        }
    [self setColors];
    }
    
#define MAIN_VIEW_REMAINING_AFTER_TRIM 6000
#define INPUT_VIEW_REMAINING_AFTER_TRIM 6000

- (IBAction)trimScrollback:(id)sender {
    NSRange aRange;
    NSInteger mvLength = [[mainView string] length];
    NSInteger ivLength = [[inputView string] length];

    if( mvLength > MAIN_VIEW_REMAINING_AFTER_TRIM ) {
        aRange.location = 0;
        if( mvLength > (2 * MAIN_VIEW_REMAINING_AFTER_TRIM)) 
            aRange.length =  mvLength / 2;
        else 
            aRange.length =  mvLength - MAIN_VIEW_REMAINING_AFTER_TRIM;
        [mainView replaceCharactersInRange:aRange withString:@""];
        [mainView scrollToBottom];
        }    

    if( ivLength > INPUT_VIEW_REMAINING_AFTER_TRIM ) {
        aRange.location = 0;
        if( ivLength > (2 * INPUT_VIEW_REMAINING_AFTER_TRIM)) 
            aRange.length =  ivLength / 2;
        else 
            aRange.length =  ivLength - INPUT_VIEW_REMAINING_AFTER_TRIM;
        [inputView replaceCharactersInRange:aRange withString:@""];
        [mainView scrollToBottom];
        }
    }

- (IBAction)playButtonMeow:(id)sender {

    NSSound *toPlay = nil;

    if( localCopyOfNostalgiaMode )
        toPlay = kittenMeow;
    else
        toPlay = buttonMeow;

    if( ! [toPlay isPlaying] ) {
        [self showIconViewAbruptly:YES];
        usleep( 200000 );
        [wraithIconView setImage:wraithIconImageOM];
        [self showIconViewAbruptly:YES];
        [toPlay play];
        while( [toPlay isPlaying] )
            ;
        [wraithIconView setImage:wraithIconImage];
        [self showIconViewAbruptly:YES];
        usleep( 200000 );

        [NSAnimationContext beginGrouping];
    
        [[NSAnimationContext currentContext] setDuration:0.55]; 
        [[wraithIconView animator] setAlphaValue:0.0];        

        [NSAnimationContext endGrouping];
        }
    }

- (IBAction)playButtonMeowAndWait:(id)sender {

    if( ! [buttonMeow isPlaying] )
        [buttonMeow play];
    while( [buttonMeow isPlaying] )
        ;
    }

- (IBAction)playDisasterMeow:(id)sender {

  if( ! [disasterMeow isPlaying] ) {
        [self showIconViewAbruptly:YES];
        usleep( 200000 );
        [wraithIconView setImage:wraithIconImageOM];
        [self showIconViewAbruptly:YES];
        [disasterMeow play];
        while( [disasterMeow isPlaying] )
            ;
        [wraithIconView setImage:wraithIconImage];
        [self showIconViewAbruptly:YES];
        }
    }

- (IBAction)playDisasterMeowAndWait:(id)sender {

    if( ! [disasterMeow isPlaying] )
        [disasterMeow play];
    while( [disasterMeow isPlaying] )
	;
    }

- (void) showIconView:(bool)visible {

    [NSAnimationContext beginGrouping];

    if( visible ) {
        [[NSAnimationContext currentContext] setDuration:0.2]; 
        [[wraithIconView animator] setAlphaValue:1.0];        
        }
    else {
        [[NSAnimationContext currentContext] setDuration:1.0]; 
        [[wraithIconView animator] setAlphaValue:0.0];        
        }

    [NSAnimationContext endGrouping];

    }

- (void) winkIconView {

    [NSAnimationContext beginGrouping];
    [[NSAnimationContext currentContext] setDuration:0.8]; 
    [[wraithIconView animator] setImage:wraithIconImageClosedEye];
    [[wraithIconView animator] setAlphaValue:0.0];        
    [NSAnimationContext endGrouping];
/*
    [NSAnimationContext beginGrouping];
    [[NSAnimationContext currentContext] setDuration:0.5]; 
    [[wraithIconView animator] setAlphaValue:0.0];        
    [NSAnimationContext endGrouping];
*/
    }

- (void) showIconViewAbruptly:(bool)visible {

    if( visible )
        [wraithIconView setAlphaValue:1.0];        
    else
        [wraithIconView setAlphaValue:0.0];        
    [wraithIconView display];
    }


- (void) showAuthorField:(bool)visible {

    [NSAnimationContext beginGrouping];

    if( visible ) {
        [[NSAnimationContext currentContext] setDuration:0.2]; 
        [[authorField animator] setAlphaValue:1.0];        
        }
    else {
        [[NSAnimationContext currentContext] setDuration:1.0]; 
        [[authorField animator] setAlphaValue:0.0];        
        }

    [NSAnimationContext endGrouping];

    }

// Lots of stuff for changing font size and modifying the height of the various
// panels of the Wraith Scheme window in consequence.  Here is where a lot of
// work would need to be done to make things behave well if resolution independence
// goes through in the manner described in the pre-release days of Leopard.

- (IBAction)makeFontBigger:(id)sender {

    CGFloat newFontSize = nextLargestFontSize( currentFontSize );
    CGFloat newLineSize = matchingLineHeight( newFontSize );
    
    currentFontSize = newFontSize;
    currentLineSize = newLineSize;

    [self setFontsToSize:newFontSize ];
    [self setFramesForFontChange:newLineSize ];
    [mainView scrollToBottom];
    [inputView showSelectedRange];
    if( currentFontSize >= 30. )
        [fontBiggerItem setEnabled:NO];
    else
        [fontBiggerItem setEnabled:YES];
    if( currentFontSize >= 9.5 )
        [fontSmallerItem setEnabled:YES];
    else
        [fontSmallerItem setEnabled:NO];
    }

- (IBAction)makeFontSmaller:(id)sender {

    CGFloat newFontSize = nextSmallestFontSize( currentFontSize );
    CGFloat newLineSize = matchingLineHeight( newFontSize );

    currentFontSize = newFontSize;
    currentLineSize = newLineSize;

    [self setFontsToSize:newFontSize ];
    [self setFramesForFontChange:newLineSize ];
    [mainView scrollToBottom];
    [inputView showSelectedRange];
    if( currentFontSize <= 30. )
        [fontBiggerItem setEnabled:YES];
    else
        [fontBiggerItem setEnabled:NO];
    if( currentFontSize <= 9.5 )
        [fontSmallerItem setEnabled:NO];
    else
       [fontSmallerItem setEnabled:YES];
    }

CGFloat nextLargestFontSize( CGFloat fontSize ) {
    if( fontSize < 9.5 )
        return 10.;
    else if( fontSize < 11. )
        return 12.;
    else if( fontSize < 13. )
        return 14.;
    else if( fontSize < 16. )
        return 18.;
    else if( fontSize < 21. )
        return 24.;
    else
        return 36.;
    }
    
CGFloat nextSmallestFontSize( CGFloat fontSize ) {
    if( fontSize > 30 )
        return 24.;
    else if( fontSize > 21. )
        return 18.;
    else if( fontSize > 16. )
        return 14.;
    else if( fontSize > 13. )
        return 12.;
    else if( fontSize > 11. )
        return 10.;
    else
        return 9.;
    }

CGFloat matchingLineHeight( CGFloat fontSize ) {
    if( fontSize < 9.5 )
        return 12.;
    else if( fontSize < 11. )
        return 14.;
    else if( fontSize < 13. )
        return 16.;
    else if( fontSize < 16. )
        return 19.;
    else if( fontSize < 21. )
        return 25.;
    else if( fontSize < 30. )
        return 32.;
    else
        return 48.;
    }

void resizeField( NSTextField *aField, CGFloat newY, CGFloat newHeight, CGFloat newX, CGFloat newWidth ) {

    NSRect newFrame = [[aField superview] frame];
    
    newFrame.origin.x = newX;
    newFrame.size.width = newWidth;  //  The one use of "resizeField" herein seems not to need "newWidth.
    newFrame.origin.y = newY;
    newFrame.size.height = newHeight;
    [[aField superview] setNeedsDisplayInRect:[[[aField superview] superview] frame]];
    [aField setFrame:newFrame];
    [aField setNeedsDisplay:YES];
   
    }

void resizeView( NSTextView *aView, CGFloat newY, CGFloat newHeight ) {

    NSRect newFrame = [[[aView superview] superview] frame];
    
    newFrame.origin.y = newY;
    newFrame.size.height = newHeight;
    [[[[aView superview] superview] superview] setNeedsDisplayInRect:[[[aView superview] superview] frame]];
    [[[aView superview] superview] setFrame:newFrame];
    [[[aView superview] superview] setNeedsDisplay:YES];
   
    }

- (void)setFramesForFontChange:(CGFloat)newLineSize {

    NSRect meowButtonFrame =  [meowButton frame];
    NSRect meowButtonBackdropFrame =  [meowButtonBackdrop frame];
    CGFloat heightAvailable  =  [[mainWindow contentView] frame].size.height;
    CGFloat widthAvailable   =  [[mainWindow contentView] frame].size.width;

    [[mainWindow contentView] setAutoresizesSubviews:NO];
    [[mainWindow contentView] setNeedsDisplay:YES];

    [self resizeProgressIndicator:(heightAvailable - newLineSize - 29)];

    resizeView(  cosmeticArea1,              heightAvailable  -  3 * newLineSize - 1  ,  newLineSize + 1                           );

    resizeField( cosmeticArea2,              heightAvailable  -  3 * newLineSize - 1  ,  3 * newLineSize + 1, 
                                             2,                                          widthAvailable - 17                       );

    resizeView(  messageView,                heightAvailable  -  1 * newLineSize - 1  ,  newLineSize + 1                           );
    resizeView(  promptView,                 heightAvailable  -  2 * newLineSize - 1  ,  newLineSize                               );
    resizeView(  (NSTextView *)responseView, heightAvailable  -  3 * newLineSize - 1  ,  newLineSize                               );

    // The height of the inputView must put the lower end of the mainView adjacent the top of the 
    // widget which controls resizing of the Wraith Scheme window.  The "14", etc, have to do with that size.

    if( newLineSize < 14.0) {
        resizeView(  (NSTextView *)mainView,     15                                   ,  heightAvailable  -  3 * newLineSize  - 16 );
        resizeView(  (NSTextView *)inputView,    2                                    ,  14.0                                      );
        }
    else {
        resizeView(  (NSTextView *)mainView,     newLineSize  +  2                    ,  heightAvailable  -  4 * newLineSize  -  3 );
        resizeView(  (NSTextView *)inputView,    2                                    ,  newLineSize + 1                           );
        }

    meowButtonFrame.origin.y = heightAvailable - 3 * newLineSize + 1;
    meowButtonFrame.size.height = 3 * newLineSize - 1;

    meowButtonBackdropFrame.origin.y = heightAvailable - 3 * newLineSize - 1;
    meowButtonBackdropFrame.size.height = 3 * newLineSize + 1;

    [[meowButton superview] setNeedsDisplayInRect:[meowButton frame]];
    [meowButton setFrame:meowButtonFrame];

    [[meowButtonBackdrop superview] setNeedsDisplayInRect:[meowButtonBackdrop frame]];
    [meowButtonBackdrop setFrame:meowButtonBackdropFrame];

    if( newLineSize < 16 )
        [meowButton setTitle:@"m\ne\nw"];
    else if( newLineSize < 48 )
        [meowButton setTitle:@"M\nE\nO\nW"];
    else
        [meowButton setTitle:@"M\n\nE\n\nO\n\nW"];

    [meowButton setNeedsDisplay:YES];
	
    [[mainView enclosingScrollView] setVerticalLineScroll:newLineSize];  //  New 2007.03.04

    [[mainWindow contentView] setAutoresizesSubviews:YES];

    }
    
- (void) resizeProgressIndicator:(CGFloat)newY {

    NSRect pFrame = [progressIndicator frame];
    NSRect rFrame = [[[progressIndicator superview] superview] frame];
 
    pFrame.origin.y = newY;

    [[progressIndicator superview] setNeedsDisplayInRect:rFrame];
    [progressIndicator setFrame:pFrame];
    [progressIndicator setNeedsDisplay:YES];
   
    }

- (IBAction) makeInputPanelFirstResponder:(id)sender {
    [mainWindow makeFirstResponder:inputView];
    }

- (IBAction) disableMakeInputPanelFirstResponderItem { [makeInputPanelFirstResponderItem setEnabled:NO]; }

- (IBAction) enableMakeInputPanelFirstResponderItem { [makeInputPanelFirstResponderItem setEnabled:YES]; }

- (double)currentLineHeight { return currentLineSize; }

- (NSWindow *)getMainWindow { return mainWindow; }

@end

void displayRect( char *msg, NSRect aRect )
{
    printf( "Rect \"%s\": origin %f %f size %f %f\n",
            msg, aRect.origin.x, aRect.origin.y, aRect.size.width, aRect.size.height );
    }

