
//
//  WraithSchemeControllerAndModel.h
//  WraithCocoa
//
//  Created by Jay Freeman on 12/15/06.
//  Copyright 2006 Jay Reynolds Freeman. All rights reserved.
//
/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628
*/

#import <sys/types.h>

#import "LocalTypes.h"
#import <Cocoa/Cocoa.h>
#import "CocoaInterface.h"
#import "CocoaIO.h"
#import "UserSetup.h"
#import "MainAux.e"

@interface WraithSchemeControllerAndModel : NSObject {

    bool hasWorldToOpenAtLaunch;
    NSInteger generationalMemorySizeInUse;
    NSInteger mainMemorySizeInUse;
    NSString *pathToLaunchWorld;

    }

- (void) applicationWillTerminate:(NSNotification *)notification;
- (void) launchScheme;
- (void) runEvaluator:(id)ignored;
- (void) setUpSchemeWithNumberOfKittens:(UnsignedWord)nKittens
                     withMainMemorySize:(NSInteger)mmSize
             withGenerationalMemorySize:(NSInteger)gmSize
                withGenerationalGCInUse:(Boolean)useGGC
                   withHasWorldToLaunch:(Boolean)hasWorld
                        withPathToWorld:(NSString *)pathToWorld
                          withIsMomCatP:(Boolean)momCatP
                       withKittenNumber:(unsigned int)kittenNumber;
- (void) tellEvaluator:(int)mID withLong:(LongWord)mLong withString:(const char *)mString;

- (void) runInterruptLocally:(NSUInteger)interruptNumber;
- (void) runInterruptLocallyForPushbutton:(NSUInteger)pushButtonNumber;
- (void) runInterruptLocallyForBigRedButton;

//######## Do-something methods that might possibly be overwritten by subclasses of WraithSchemeControllerAndModel:

- (NSArray *) createCompletionArray;

//######## Do-nothing methods to be overwritten by subclasses of WraithSchemeControllerAndModel:

- (void) incrementProgressWithLong:(long)theLong;

- (void) evaluatorReportsCompletedInitialization;
- (void) evaluatorReportsFatalProblemWithLong:(long)theLong withString:(NSString *)theString;
- (void) evaluatorRequestsMakeFront;
- (void) evaluatorReportsNewOblistSymbolWithLong:(long)theLong withString:(NSString *)theString;
- (void) evaluatorRequestsClearOblistSymbolsWithLong:(long)theLong withString:(NSString *)theString;
- (void) evaluatorReportsEvaluating;
- (void) evaluatorReportsNotEvaluating;
- (void) evaluatorReportsAcknowledgeReset;
- (void) evaluatorReportsGarbageCollecting;
- (void) evaluatorReportsNotGarbageCollecting;
- (void) evaluatorReportsSleeping;
- (void) evaluatorReportsWakedUp;
- (void) evaluatorReportsCompilingDefines;
- (void) evaluatorReportsNotCompilingDefines;
- (void) evaluatorReportsUsingLowPrecision;
- (void) evaluatorReportsUsingHighPrecision;
- (void) evaluatorReportsOutputReadyWithLong:(long)theLong withString:(NSString *)theString;
- (void) evaluatorReportsMessageToDisplayWithLong:(long)theLong withString:(NSString *)theString;
- (void) evaluatorReportsDialogToStartWithLong:(long)theLong withString:(NSString *)theString;
- (void) evaluatorReportsFileLoadBeginningWithLong:(long)theLong withString:(NSString *)theString;
- (void) evaluatorReportsFileLoadSucceeded;
- (void) evaluatorReportsFileLoadFailed;
- (void) evaluatorReportsWorldLoadSucceededWithLong:(long)theLong withString:(NSString *)theString;
- (void) evaluatorReportsWorldLoadFailed;
- (void) evaluatorReportsWorldSaveSucceeded;
- (void) evaluatorReportsWorldSaveFailed;
- (void) evaluatorReportsExpectMoreInput;
- (void) evaluatorReportsDoNotExpectMoreInput;
- (void) evaluatorReportsExpectParensWithLong:(long)theLong;
- (void) evaluatorReportsExpectDoubleQuote;
- (void) evaluatorReportsDiscardedInput;
- (void) evaluatorReportsNewGCCountWithLong:(long)theLong;
- (void) evaluatorReportsNewGenerationalGCCountWithLong:(long)theLong;
- (void) evaluatorReportsMemorySizeWithLong:(long)theLong;
- (void) evaluatorReportsMemoryInUseWithLong:(long)theLong;
- (void) evaluatorReportsGenerationalMemorySizeWithLong:(long)theLong;
- (void) evaluatorReportsGenerationalMemoryInUseWithLong:(long)theLong;
- (void) evaluatorReportsListPrintLengthWithLong:(long)theLong;
- (void) evaluatorReportsListPrintDepthWithLong:(long)theLong;
- (void) evaluatorReportsVectorPrintLengthWithLong:(long)theLong;
- (void) evaluatorReportsVectorPrintDepthWithLong:(long)theLong;
- (void) evaluatorReportsSenseLightCommandWithLong:(long)theLong;
- (void) evaluatorReportsSenseLightRotateWithLong:(long)theLong;
- (void) evaluatorReportsInitProgress;
- (void) evaluatorReportsIncrementProgressWithLong:(long)theLong;
- (void) evaluatorReportsWrapupProgress;
- (void) evaluatorReportsDisasterWithString:(NSString *)theString;
- (void) evaluatorReportsLevelIndicator0NewValueWithLong:(long)theLong;
- (void) evaluatorReportsLevelIndicator1NewValueWithLong:(long)theLong;
- (void) evaluatorReportsLevelIndicator2NewValueWithLong:(long)theLong;
- (void) evaluatorReportsLevelIndicator3NewValueWithLong:(long)theLong;

@end
