//
//  WraithSchemeControllerAndModel.mm
//  WraithCocoa
//
//  Created by Jay Freeman on 12/15/06.
//  Copyright 2006 Jay Reynolds Freeman.
//
/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628
*/

#import "WraithSchemeControllerAndModel.h"

extern Boolean isMomCatP;                // In MainAux.c++
extern UnsignedWord theNumberOfKittens;  // In MainAux.c++
extern UnsignedWord theKittenNumber;     // In MainAux.c++
extern char **globalArgv;                // In main.mm

// #defines for initialization waits:

#define WAIT_FOR_EVALUATOR_DELAY 1000

// #defines for debugging:

#define SHOW_THREAD_INTERACTIONS FALSE      // Make TRUE to get log of thread-interaction details.

// The size of string passed from the controller to the evaluator is variable; the
// next #define is the length started out with.  (Old string is freed and new calloced
// when necessary.)

#define START_STRING_SIZE 512

extern int runSchemeWithPathAndMemory( const char *argv0, long memorySizeProvided );
extern void cleanUpForExit();
extern void makeKittensExit();
extern char *messagesFromControllerNames[];
extern Boolean fatalErrorOccurred;
extern LongWord progressFractionInThousandths;
extern Boolean usingGenerationalGC;

threadMessage toEvaluator;
threadMessage toController;

SInt32 worldIsLoaded = 0;
SInt32 controllerReadyForMessages = 0;
SInt32 clowderIsOperating = 0;
SInt32 evaluatorHasInitialized = 0;
SInt32 GUIInitialized = 0;
SInt32 readyToRead = 0;

int worldFileID;
Boolean worldFileIDIsGood = FALSE;

#define BUFSIZE_FOR_PATHNAME 1000

Text worldFilePathName[ BUFSIZE_FOR_PATHNAME + 1 ];
extern volatile int controllerOutboundMailboxFlag;
extern volatile int evaluatorOutboundMailboxFlag;

NSLock *controllersMailboxLock = nil;
NSLock *evaluatorsMailboxLock = nil;

WraithSchemeControllerAndModel *theWraithSchemeControllerAndModel;

//########  Stuff to set metadata for saved worlds.

//const OSType WraithSchemeCreatorCode      = 'cld4';  // FIXME -- reenable later?
//const OSType WraithSchemeDocumentTypeCode = 'kdl4';  // FIXME -- reenable later?

/*  Reenable later?  --  FIXME

void setWorldAttributesForPath( const char *path ) 
{
    NSMutableDictionary *fileAttributes = [NSMutableDictionary dictionaryWithCapacity: 2];

    [fileAttributes
        setObject: [NSNumber numberWithUnsignedInt:WraithSchemeCreatorCode]
           forKey: NSFileHFSCreatorCode];

    [fileAttributes
        setObject: [NSNumber numberWithUnsignedInt:WraithSchemeDocumentTypeCode]
           forKey: NSFileHFSTypeCode];

    [[NSFileManager defaultManager]
       setAttributes: fileAttributes
        ofItemAtPath: [NSString stringWithCString:path encoding:NSASCIIStringEncoding]
               error: 0];

    }
*/

@implementation WraithSchemeControllerAndModel


//########  Save defaults and clean up before application termination.

- (void) applicationWillTerminate:(NSNotification *)notification {

    if( isMomCatP )
        makeKittensExit();
    cleanUpForExit();
    }
    

//########  This code is merely a convenient place to store and provide the list of Wraith Scheme 
//########  built-in procedures, special forms, and the like.

- (NSArray *)createCompletionArray {

    return [NSArray arrayWithObjects:
        @"*",
        @"+",
        @"-",
        @"/",
        @"<",
        @"<=",
        @"=",
        @">",
        @">***<",
        @">**<",
        @">*<",
        @">+++<",
        @">++<",
        @">+<",
        @">-<",
        @">=",
        @"abs",
        @"acos",
        @"and",
        @"append",
        @"apply",
        @"asin",
        @"assoc",
        @"assq",
        @"assv",
        @"atan",
        @"begin",
        @"boolean?",
        @"call-with-current-continuation",
        @"call-with-input-file",
        @"call-with-output-file",
        @"call-with-values",
        @"car",
        @"cdr",
        @"caar",
        @"cadr",
        @"cdar",
        @"cddr",
        @"caaar",
        @"caadr",
        @"cadar",
        @"caddr",
        @"cdaar",
        @"cdadr",
        @"cddar",
        @"cdddr",
        @"caaaar",
        @"caaadr",
        @"caadar",
        @"caaddr",
        @"cadaar",
        @"cadadr",
        @"caddar",
        @"cadddr",
        @"cdaaar",
        @"cdaadr",
        @"cdadar",
        @"cdaddr",
        @"cddaar",
        @"cddadr",
        @"cdddar",
        @"cddddr",
        @"case",
        @"ceiling",
        @"char->integer",
        @"char-alphabetic?",
        @"char-ci<=?",
        @"char-ci&lt;?",
        @"char-ci=?",
        @"char-ci>=?",
        @"char-ci>?",
        @"char-downcase",
        @"char-lower-case?",
        @"char-numeric?",
        @"char-ready?",
        @"char-upcase",
        @"char-upper-case?",
        @"char-whitespace?",
        @"char<=?",
        @"char&lt;?",
        @"char=?",
        @"char>=?",
        @"char>?",
        @"char?",
        @"close-input-port",
        @"close-output-port",
        @"complex?",
        @"cond",
        @"cons",
        @"cos",
        @"current-input-port",
        @"current-output-port",
        @"define",
        @"define-syntax",
        @"delay",
        @"display",
        @"do",
        @"dynamic-wind",
        @"else",
        @"eof-object?",
        @"eq?",
        @"equal?",
        @"eqv?",
        @"eval",
        @"even?",
        @"exact->inexact",
        @"exact?",
        @"exp",
        @"expt",
        @"floor",
        @"for-each",
        @"force",
        @"gcd",
        @"if",
        @"inexact->exact",
        @"inexact?",
        @"input-port?",
        @"integer->char",
        @"integer?",
        @"interaction-environment",
        @"lambda",
        @"lcm",
        @"length",
        @"let",
        @"let-syntax",
        @"let*",
        @"letrec",
        @"letrec-syntax",
        @"list",
        @"list->string",
        @"list->vector",
        @"list-ref",
        @"list-tail",
        @"list?",
        @"load",
        @"log",
        @"make-string",
        @"make-vector",
        @"map",
        @"max",
        @"member",
        @"memq",
        @"memv",
        @"min",
        @"modulo",
        @"negative?",
        @"newline",
        @"not",
        @"null-environment",
        @"null?",
        @"number->string",
        @"number?",
        @"odd?",
        @"open-input-file",
        @"open-output-file",
        @"or",
        @"output-port?",
        @"pair?",
        @"peek-char",
        @"port?",
        @"positive?",
        @"procedure?",
        @"quasiquote",
        @"quote",
        @"quotient",
        @"rational?",
        @"read",
        @"read-char",
        @"real?",
        @"remainder",
        @"reverse",
        @"round",
        @"scheme-report-environment",
        @"set!",
        @"set-car!",
        @"set-cdr!",
        @"sin",
        @"sqrt",
        @"string->list",
        @"string->number",
        @"string->symbol",
        @"string-append",
        @"string-ci<=?",
        @"string-ci&lt;?",
        @"string-ci=?",
        @"string-ci>=?",
        @"string-ci>?",
        @"string-copy",
        @"string-fill!",
        @"string-length",
        @"string-ref",
        @"string-set!",
        @"string<=?",
        @"string<?",
        @"string=?",
        @"string>=?",
        @"string>?",
        @"string?",
        @"substring",
        @"symbol->string",
        @"symbol?",
        @"syntax-rules",
        @"tan",
        @"transcript-off",
        @"transcript-on",
        @"truncate",
        @"unquote",
        @"unquote-splicing",
        @"values",
        @"vector",
        @"vector->list",
        @"vector-fill!",
        @"vector-length",
        @"vector-ref",
        @"vector-set!",
        @"vector?",
        @"with-input-from-file",
        @"with-output-to-file",
        @"write",
        @"write-char",
        @"zero?",
        @"e::active-room",
        @"e::active-store-size",
        @"e::aging-room",
        @"e::alarm",
        @"e::alarm?",
        @"e::alarm-time",
        @"e::all?",
        @"e::any?",
        @"e::atom?",
        @"e::big-red-button-enabled?",
        @"e::big-red-button-interrupt-number",
        @"e::bit-and",
        @"e::bit-not",
        @"e::bit-or",
        @"e::bit-shift-left",
        @"e::bit-shift-right-arithmetic",
        @"e::bit-shift-right-logical",
        @"e::bit-xor",
        @"e::block-any-binding",
        @"e::block-symbol-binding",
        @"e::bound-instance?",
        @"e::c-boolean->scheme-boolean?",
        @"e::clear-compiler-on!",
        @"e::clear-permanent!",
        @"e::clear-show-full-precision!",
        @"e::closed-port?",
        @"e::compiler-on?",
        @"e::compile-form",
        @"e::cons-with-continuation",
        @"e::current-directory",
        @"e::debug",
        @"e::deep-copy",
        @"e::define-no-compile",
        @"e::disable-big-red-button!",
        @"e::disable-pushbutton!",
        @"e::enable-big-red-button!",
        @"e::enable-pushbutton!",
        @"e::entropy-death",
        @"e::error",
        @"e::error-port",
        @"e::exit",
        @"e::expand-macro",
        @"e::expand-macros-recursively",
        @"e::file-exists?",
        @"e::fixnum?",
        @"e::float?",
        @"e::forced?",
        @"e::forgettable?",
        @"e::forgettable-expiration-time",
        @"e::forgettable-forgotten?",
        @"e::forgettable-object",
        @"e::forgettable-remembered?",
        @"e::forgettable-remember-probability",
        @"e::forgettable-unexpired?",
        @"e::full-gc",
        @"e::gensym",
        @"e::get-tag",
        @"e::hide-sense-lights!",
        @"e::inf?",
        @"e::ing-stray->umber-nay",
        @"e::inspect",
        @"e::inspect-world",
        @"e::interrupt-file-information",
        @"e::kitten-number",
        @"e::list-print-depth",
        @"e::list-print-length",
        @"e::set-list-print-depth!",
        @"e::set-list-print-length!",
        @"e::set-vector-print-depth!",
        @"e::set-vector-print-length!",
        @"e::vector-print-depth",
        @"e::vector-print-length",
        @"e::load-world!",
        @"e::logic-constant?",
        @"e::macro",
        @"e::macro-body",
        @"e::macro?",
        @"e::main",
        @"e::make-forgettable",
        @"e::make-memory-mapped-block",
        @"e::make-offsets",
        @"e::make-simple-class",
        @"e::memory-map-block-in-current-kitten",
        @"e::memory-mappable-space-size",
        @"e::memory-mapped-block?",
        @"e::momcat?",
        @"e::money->string",
        @"e::multiple-values?",
        @"e::nan?",
        @"e::ninety-percent-used?",
        @"e::non-printing-object?",
        @"e::not-null?",
        @"e::number-of-kittens",
        @"e::number->string-with-n-decimals",
        @"e::original-cwcc",
        @"e::peek-poke-atomic",
        @"e::peek8",
        @"e::peek16",
        @"e::peek32",
        @"e::peek64",
        @"e::peek64-float",
        @"e::peek64-float-set-vector!",
        @"e::peek8-set-vector!",
        @"e::peek16-set-vector!",
        @"e::peek32-set-vector!",
        @"e::peek64-set-vector!",
        @"e::peek8-set-vector-sign-extend!",
        @"e::peek16-set-vector-sign-extend!",
        @"e::peek32-set-vector-sign-extend!",
        @"e::peek8-sign-extend",
        @"e::peek16-sign-extend",
        @"e::peek32-sign-extend",
        @"e::peek-c-string",
        @"e::perform-applicative",
        @"e::permanent?",
        @"e::poke8",
        @"e::poke16",
        @"e::poke32",
        @"e::poke64",
        @"e::poke64-float",
        @"e::poke64-float-vector",
        @"e::poke8-vector",
        @"e::poke16-vector",
        @"e::poke32-vector",
        @"e::poke64-vector",
        @"e::poke-c-string",
        @"e::process-id",
        @"e::program-directory",
        @"e::promise?",
        @"e::pushbutton-enabled?",
        @"e::pushbutton-interrupt-number",
        @"e::random",
        @"e::read-string-with-prompt",
        @"e::reset",
        @"e::room",
        @"e::rotate-sense-lights!",
        @"e::save-world",
        @"e::scheme-boolean->c-boolean",
        @"e::sense-light-number?",
        @"e::sense-light-illumination-number?",
        @"e::sense-switches",
        @"e::set-sense-light-illuminations!",
        @"e::set-sense-light-n-illumination!",
        @"e::set-sense-light-n-visibility!",
        @"e::set-sense-lights-off!",
        @"e::set-alarm!",
        @"e::set-compiler-on!",
        @"e::set-current-directory!",
        @"e::set-forgettable-expiration-time!",
        @"e::set-forgettable-remember-probability!",
        @"e::set-level-indicator!",
        @"e::set-input-port!",
        @"e::set-permanent!",
        @"e::set-show-full-precision!",
        @"e::set-tag!",
        @"e::show-active-memory",
        @"e::show-aging-memory",
        @"e::show-dynamic-environment-list",
        @"e::show-environment",
        @"e::show-environment-list",
        @"e::show-full-precision?",
        @"e::show-kittens",
        @"e::show-locks",
        @"e::show-memory",
        @"e::show-room",
        @"e::show-sense-lights!",
        @"e::slider",
        @"e::srandom",
        @"e::stack-depth",
        @"e::startup-directory",
        @"e::store-size",
        @"e::substitute-new-permanent-symbols",
        @"e::system",
        @"e::time",
        @"e::tell-kitten",
        @"e::top-loop-code",
        @"e::umber-nay->ing-stray",
        @"e::usleep",
        @"e::version",
        @"e::warning",
        @"e::write-continuation",
        @"e::write-stack",
        @"c::block",
        @"c::clear-interrupt-handler!",
        @"c::clear-interrupt-requested!",
        @"c::compiling?",
        @"c::define",
        @"c::disable-window-output",
        @"c::down-in-flames!!!",
        @"c::enable-window-output",
        @"c::environment",
        @"c::hash-string",
        @"c::interrupt",
        @"c::interrupt-handler-vector",
        @"c::kitten-busy",
        @"c::kitten-empty-queue?",
        @"c::kitten-input-queue",
        @"c::kitten-purge-queue",
        @"c::kitten-reset",
        @"c::let-deadlock-time-out",
        @"c::load-from-string",
        @"c::number->string-with-c-format",
        @"c::open-input-socket",
        @"c::pi",
        @"c::release-block",
        @"c::release-locks",
        @"c::set-blocked-binding!",
        @"c::set-interrupt-handler!",
        @"c::socket-accept",
        @"c::socket-create-and-listen",
        @"c::socket-fgets",
        @"c::socket-write",
        @"c::symeval",
        @"c::window-output-enabled?",
        nil
        ];
    }

//########  Function called by the Scheme Engine to wake up the controller and get it to listen to the evaluator.

void doPollEvaluatorOnce()
{
    [theWraithSchemeControllerAndModel performSelectorOnMainThread:@selector(pollEvaluatorOnce) withObject:nil waitUntilDone:FALSE];
    }

//########  "init" mostly sets up controller state variables.

- (id)init {

    self = [super  init];

    hasWorldToOpenAtLaunch = false;
    pathToLaunchWorld = nil;
    worldFileID = 0;

    toEvaluator.messageID = C_NO_MESSAGE_YET;
    toEvaluator.theString = 0;
    toEvaluator.theLong = 0;

    toController.messageID = E_NO_MESSAGE_YET;
    toController.theString = 0;
    toController.theLong = 0;

    controllersMailboxLock = [[[NSLock alloc] init] retain];
    evaluatorsMailboxLock = [[[NSLock alloc] init] retain];

    srandom( (SInt32)(time( 0 ) & WORD_MASK));

    return self;
    }


//########   Launch the Scheme machine, and wait for the world to load.

- (void) launchScheme {

    NSString *earlyFatalErrorString = @"Wraith Scheme must quit because of an implementation error while starting up:  This Error is not your fault.  Please report it to Jay_Reynolds_Freeman@mac.com.  For details, see the end of the console log file, somewhere in the \"Console\" folder of the \"Logs\" folder of the \"Library\" folder of your main disk drive.  Thank you.  (Wraith Scheme's normal error-handling mechanism isn't running yet.)";

    //  And away we go ... (this next dispatches a thread that runs the evaluator).

    [NSThread detachNewThreadSelector:@selector(runEvaluator:) toTarget:self withObject:nil];

    while( ! worldIsLoaded ) {

        [self incrementProgressWithLong:progressFractionInThousandths];

        usleep( WAIT_FOR_EVALUATOR_DELAY );
//        if( true )                               // Change this line to test early errors.
        if( fatalErrorOccurred )
            [self evaluatorReportsDisasterWithString:earlyFatalErrorString];
        }

    if( fatalErrorOccurred )
        [self evaluatorReportsDisasterWithString:earlyFatalErrorString];

    //  On your mark ... get set ... (this starts a never-ending timer series running).

//    [self Test1:self];    // For testing fatal error handling.

    }


//########  Get just one request from the evaluator, and act on it:  Critical-section-access and lock slime, and a big switch.

- (bool) pollEvaluatorOnce {

    SInt32 newMessage;
    long newLong;
    NSString *newNSString;
    bool returnValue;

    OSMemoryBarrier();     // Defensive programming

    if( evaluatorOutboundMailboxFlag == MAILBOX_FULL ) {

        newMessage  = toController.messageID;
        newLong     = toController.theLong;
        newNSString = [[NSString stringWithCString:toController.theString encoding:NSASCIIStringEncoding] copy];
     
        switch( newMessage ) {

            // System administration.

            case E_NO_MESSAGE_YET: break;                   // Initialization state; evaluator has not sent any messages.
            case E_COMPLETED_INITIALIZATION:
                [self evaluatorReportsCompletedInitialization];
                break;
            case E_HELLO_THERE: break;                      // No action required.
            case E_FATAL_PROBLEM:
                [self evaluatorReportsFatalProblemWithLong:newLong withString:newNSString];
                break;
            case E_PLEASE_TIDY_UP_FOR_EXIT: break;
            case E_TIDIED_UP: break;
            case E_COGNITO_ERGO_SUM: break;
            case E_MAKE_FRONT:
                [self evaluatorRequestsMakeFront];
                break;
            case E_NEW_OBLIST_SYMBOL:
//                [self evaluatorReportsNewOblistSymbolWithLong:newLong withString:newNSString];
                break;
            case E_CLEAR_OBLIST_SYMBOLS:
//                [self evaluatorRequestsClearOblistSymbolsWithLong:newLong withString:newNSString];
                break;

            // Simple state transitions.

            case E_EVALUATING:                 
                [self evaluatorReportsEvaluating];
                break;
            case E_NOT_EVALUATING:
                [self evaluatorReportsNotEvaluating];
                break;
            case E_ACKNOWLEDGE_RESET:
                [self evaluatorReportsAcknowledgeReset];
                break;
            case E_GARBAGE_COLLECTING:
                [self evaluatorReportsGarbageCollecting];
                 break;
            case E_NOT_GARBAGE_COLLECTING:
                [self evaluatorReportsNotGarbageCollecting];
                break;

            case E_SLEEPING:
                [self evaluatorReportsSleeping];
                break;
            case E_WAKED_UP:
                [self evaluatorReportsWakedUp];
                break;
            case E_COMPILING_DEFINES:
                [self evaluatorReportsCompilingDefines];
                break;
            case E_NOT_COMPILING_DEFINES:
                [self evaluatorReportsNotCompilingDefines];
                break;
            case E_USING_LOW_PRECISION:
                [self evaluatorReportsUsingLowPrecision];
                break;
            case E_USING_HIGH_PRECISION:
                [self evaluatorReportsUsingHighPrecision];
                break;

            // Routine I/O.

            case E_OUTPUT_READY: 
                [self evaluatorReportsOutputReadyWithLong:newLong withString:newNSString];
                break;
            case E_MESSAGE_TO_DISPLAY:
                [self evaluatorReportsMessageToDisplayWithLong:newLong withString:newNSString];
                break;
            case E_DIALOG_TO_START:
                [self evaluatorReportsDialogToStartWithLong:newLong withString:newNSString];
                break;
            case E_FILE_LOAD_BEGINNING:
                [self evaluatorReportsFileLoadBeginningWithLong:newLong withString:newNSString];
                break;
            case E_FILE_LOAD_SUCCEEDED:
                [self evaluatorReportsFileLoadSucceeded];
                break;
            case E_FILE_LOAD_FAILED:
                [self evaluatorReportsFileLoadFailed];
                break;
            case E_WORLD_LOAD_SUCCEEDED:
                [self evaluatorReportsWorldLoadSucceededWithLong:newLong withString:newNSString];
                break;
            case E_WORLD_LOAD_FAILED:
                [self evaluatorReportsWorldLoadFailed];
                break;    // Nothing to do.
            case E_WORLD_SAVE_SUCCEEDED:
                [self evaluatorReportsWorldSaveSucceeded];
                break;
            case E_WORLD_SAVE_FAILED:
                [self evaluatorReportsWorldSaveFailed];
                break;

            // Input parsing.

            case E_EXPECT_MORE_INPUT:
                [self evaluatorReportsExpectMoreInput];
                break;

            case E_DO_NOT_EXPECT_MORE_INPUT:
                [self evaluatorReportsDoNotExpectMoreInput];
                break;

            case E_EXPECT_PARENS:
                [self evaluatorReportsExpectParensWithLong:newLong];
                break;

            case E_EXPECT_DOUBLE_QUOTE:
                [self evaluatorReportsExpectDoubleQuote];


            case E_DISCARDED_INPUT:
                [self evaluatorReportsDiscardedInput];
                break;

            // Status information, possibly for display.

            case E_NEW_GC_COUNT:
                [self evaluatorReportsNewGCCountWithLong:newLong];
                break;
            case E_NEW_GENERATIONAL_GC_COUNT:
                [self evaluatorReportsNewGenerationalGCCountWithLong:newLong];
                break;
            case E_MEMORY_SIZE:
                [self evaluatorReportsMemorySizeWithLong:newLong];
                break;
            case E_MEMORY_IN_USE:
                [self evaluatorReportsMemoryInUseWithLong:newLong];
                break;
            case E_GENERATIONAL_MEMORY_SIZE:
                [self evaluatorReportsGenerationalMemorySizeWithLong:newLong];
                break;
            case E_GENERATIONAL_MEMORY_IN_USE:
                [self evaluatorReportsGenerationalMemoryInUseWithLong:newLong];
                break;
            case E_LIST_PRINT_LENGTH:
                [self evaluatorReportsListPrintLengthWithLong:newLong];
                break;
            case E_LIST_PRINT_DEPTH:
                [self evaluatorReportsListPrintDepthWithLong:newLong];
                break;
            case E_VECTOR_PRINT_LENGTH:
                [self evaluatorReportsVectorPrintLengthWithLong:newLong];
                break;
            case E_VECTOR_PRINT_DEPTH:
                [self evaluatorReportsVectorPrintDepthWithLong:newLong];
                break;
            case E_SENSE_LIGHT_COMMAND:
                [self evaluatorReportsSenseLightCommandWithLong:newLong];
                break;
            case E_SENSE_LIGHT_ROTATE:
                [self evaluatorReportsSenseLightRotateWithLong:newLong];
                break;
            case E_TOTAL_MEMORY_IN_USE:
                // Nothing to do.
                break;
            case E_LEVEL_INDICATOR_0_VALUE:
                [self evaluatorReportsLevelIndicator0NewValueWithLong:newLong];
                break;
            case E_LEVEL_INDICATOR_1_VALUE:
                [self evaluatorReportsLevelIndicator1NewValueWithLong:newLong];
                break;
            case E_LEVEL_INDICATOR_2_VALUE:
                [self evaluatorReportsLevelIndicator2NewValueWithLong:newLong];
                break;
            case E_LEVEL_INDICATOR_3_VALUE:
                [self evaluatorReportsLevelIndicator3NewValueWithLong:newLong];
                break;

            // Progress bar manipulation.

            case E_INIT_PROGRESS:
                [self evaluatorReportsInitProgress];
                break;
            case E_INCREMENT_PROGRESS:
                [self evaluatorReportsIncrementProgressWithLong:newLong];
                break;
            case E_WRAPUP_PROGRESS:
                [self evaluatorReportsWrapupProgress];
                break;

            default:
                [self evaluatorReportsDisasterWithString:
                    [NSString stringWithFormat:@"Controller got unknown message, number %d.", newMessage]];
                break;
            }
        returnValue = TRUE;

        [newNSString release];     // TEST

        OSMemoryBarrier();     // Defensive programming
        evaluatorOutboundMailboxFlag = MAILBOX_EMPTY;
        }
    else
        returnValue = FALSE;

    return returnValue;
    }

//########  Run the evaluator, in a thread dispatched for that purpose:

- (void) runEvaluator:(id)ignored {

    NSAutoreleasePool *thePool = [[NSAutoreleasePool alloc] init];

    NSArray *args = [[NSProcessInfo processInfo] arguments];
    NSString *argv0String = (NSString *)[args objectAtIndex:0];
    const char *argv0 = [argv0String cStringUsingEncoding:NSASCIIStringEncoding];

    runSchemeWithPathAndMemory( argv0, mainMemorySizeInUse, generationalMemorySizeInUse );  // Never returns ...

    [thePool release];  // It never returns, but just in case ...

    }


//########   Set up some necessary stuff early in the launch of the Scheme machine.

- (void) setUpSchemeWithNumberOfKittens:(UnsignedWord)nKittens
                     withMainMemorySize:(NSInteger)mmSize
             withGenerationalMemorySize:(NSInteger)gmSize
                withGenerationalGCInUse:(Boolean)useGGC
                   withHasWorldToLaunch:(Boolean)hasWorld
                        withPathToWorld:(NSString *)pathToWorld
                          withIsMomCatP:(Boolean)momCatP
                       withKittenNumber:(unsigned int)kittenNumber
{
    theNumberOfKittens = nKittens;
    mainMemorySizeInUse = mmSize;
    generationalMemorySizeInUse = gmSize;
    usingGenerationalGC = useGGC;
    hasWorldToOpenAtLaunch = hasWorld;
    pathToLaunchWorld = pathToWorld;
    isMomCatP = momCatP;
    theKittenNumber = kittenNumber;

    theWraithSchemeControllerAndModel = self;

    // If we are the MomCat and the user has told us to open a particular world, see
    // if it exists and can be opened, and squirrel away the file ID for it.

    if( isMomCatP && hasWorldToOpenAtLaunch ) {

        NSFileHandle *userWorldHandle = [NSFileHandle fileHandleForReadingAtPath:pathToLaunchWorld];

        if( userWorldHandle != nil ) {
            @try { worldFileID = [userWorldHandle fileDescriptor]; }
            @catch( NSException *exception ) {
                [self evaluatorReportsDisasterWithString:@"Could not get a file descriptor for the world file requested."];
                }
            worldFileIDIsGood = TRUE;
            strncpy( worldFilePathName, [pathToLaunchWorld cStringUsingEncoding:NSASCIIStringEncoding], BUFSIZE_FOR_PATHNAME );
            }
        else
            [self evaluatorReportsDisasterWithString:@"Could not open the world file requested."];
        }

    }


//########  Send a request to the evaluator.

- (void) tellEvaluator:(SInt32)mID withLong:(LongWord)mLong withString:(char *)mString; {

    static char *stringToPass = 0;
    static SInt32 currentLength = 0;
    SInt32 newBufferSize = 0;

    OSMemoryBarrier();     // Defensive programming

    LongWord lengthNeeded = strlen( mString ) + 1;

    if( lengthNeeded > (currentLength - 1) ) {  //  Need a bigger hammer ... er, buffer.
        if( currentLength != 0 )
            free( stringToPass );
        for( newBufferSize = START_STRING_SIZE; newBufferSize < lengthNeeded; newBufferSize = newBufferSize * 2 )
             ;
        stringToPass = (Text *)calloc( newBufferSize, sizeof( char ) );
        if( stringToPass == 0 )
            [self evaluatorReportsDisasterWithString:@"Cannot allocate memory for a communication buffer."];
        currentLength = newBufferSize;
        }

#if SHOW_THREAD_INTERACTIONS
    printf("Kitten %d: C: tellEvaluator called with message \"%s\".\n",
           theKittenNumber, messagesFromControllerNames[ mID ] );
#endif

    if( ! evaluatorHasInitialized )
        return;

#if SHOW_THREAD_INTERACTIONS
    printf("Kitten %d: C: tellEvaluator attempting critical section with message \"%s\".\n",
           theKittenNumber, messagesFromControllerNames[ mID ] );
#endif

    while( controllerOutboundMailboxFlag == MAILBOX_FULL )
        ;

#if SHOW_THREAD_INTERACTIONS
    printf("Kitten %d: C: tellEvaluator has the evaluator's mailbox with message \"%s\".\n",
           theKittenNumber, messagesFromControllerNames[ mID ] );
#endif

    if( mString != 0 )
        strncpy( stringToPass, mString, currentLength - 1 );
    else
        stringToPass[0] = 0;
    toEvaluator.messageID = mID;
    toEvaluator.theLong   = mLong;
    toEvaluator.theString = stringToPass;
 
    controllerOutboundMailboxFlag = MAILBOX_FULL;

    OSMemoryBarrier();     // Defensive programming

#if SHOW_THREAD_INTERACTIONS
        printf("Kitten %d: C: tellEvaluator left critical section with message  \"%s\".\n",
               theKittenNumber, messagesFromControllerNames[ mID ] );
#endif
    }


- (void) runInterruptLocally:(NSUInteger)interruptNumber { runLocalInterrupt( interruptNumber ); }

- (void) runInterruptLocallyForPushbutton:(NSUInteger)pushButtonNumber { runLocalInterruptForPushbutton( pushButtonNumber ); }

- (void) runInterruptLocallyForBigRedButton { runLocalInterruptForBigRedButton(); }

//######## Do-nothing methods to be overwritten by subclasses of WraithSchemeControllerAndModel.

- (void) incrementProgressWithLong:(long)theLong {}

- (void) evaluatorReportsCompletedInitialization {}
- (void) evaluatorReportsFatalProblemWithLong:(long)theLong withString:(NSString *)theString {}
- (void) evaluatorRequestsMakeFront {}
- (void) evaluatorReportsNewOblistSymbolWithLong:(long)theLong withString:(NSString *)theString {}
- (void) evaluatorRequestsClearOblistSymbolsWithLong:(long)theLong withString:(NSString *)theString {}
- (void) evaluatorReportsEvaluating {}
- (void) evaluatorReportsNotEvaluating {}
- (void) evaluatorReportsAcknowledgeReset {}
- (void) evaluatorReportsGarbageCollecting {}
- (void) evaluatorReportsNotGarbageCollecting {}
- (void) evaluatorReportsSleeping {}
- (void) evaluatorReportsWakedUp {}
- (void) evaluatorReportsCompilingDefines {}
- (void) evaluatorReportsNotCompilingDefines {}
- (void) evaluatorReportsUsingLowPrecision {}
- (void) evaluatorReportsUsingHighPrecision {}
- (void) evaluatorReportsOutputReadyWithLong:(long)theLong withString:(NSString *)theString {}
- (void) evaluatorReportsMessageToDisplayWithLong:(long)theLong withString:(NSString *)theString {}
- (void) evaluatorReportsDialogToStartWithLong:(long)theLong withString:(NSString *)theString {}
- (void) evaluatorReportsFileLoadBeginningWithLong:(long)theLong withString:(NSString *)theString {}
- (void) evaluatorReportsFileLoadSucceeded {}
- (void) evaluatorReportsFileLoadFailed {}
- (void) evaluatorReportsWorldLoadSucceededWithLong:(long)theLong withString:(NSString *)theString {}
- (void) evaluatorReportsWorldLoadFailed {}
- (void) evaluatorReportsWorldSaveSucceeded {}
- (void) evaluatorReportsWorldSaveFailed {}
- (void) evaluatorReportsExpectMoreInput {}
- (void) evaluatorReportsDoNotExpectMoreInput {}
- (void) evaluatorReportsExpectParensWithLong:(long)theLong {}
- (void) evaluatorReportsExpectDoubleQuote {}
- (void) evaluatorReportsDiscardedInput {}
- (void) evaluatorReportsNewGCCountWithLong:(long)theLong {}
- (void) evaluatorReportsNewGenerationalGCCountWithLong:(long)theLong {}
- (void) evaluatorReportsMemorySizeWithLong:(long)theLong {}
- (void) evaluatorReportsMemoryInUseWithLong:(long)theLong {}
- (void) evaluatorReportsGenerationalMemorySizeWithLong:(long)theLong {}
- (void) evaluatorReportsGenerationalMemoryInUseWithLong:(long)theLong {}
- (void) evaluatorReportsListPrintLengthWithLong:(long)theLong {}
- (void) evaluatorReportsListPrintDepthWithLong:(long)theLong {}
- (void) evaluatorReportsVectorPrintLengthWithLong:(long)theLong {}
- (void) evaluatorReportsVectorPrintDepthWithLong:(long)theLong {}
- (void) evaluatorReportsSenseLightCommandWithLong:(long)theLong {}
- (void) evaluatorReportsSenseLightRotateWithLong:(long)theLong {}
- (void) evaluatorReportsInitProgress {}
- (void) evaluatorReportsIncrementProgressWithLong:(long)theLong {}
- (void) evaluatorReportsWrapupProgress {}
- (void) evaluatorReportsDisasterWithString:(NSString *)theString {}
- (void) evaluatorReportsLevelIndicator0NewValueWithLong:(long)theLong {}
- (void) evaluatorReportsLevelIndicator1NewValueWithLong:(long)theLong {}
- (void) evaluatorReportsLevelIndicator2NewValueWithLong:(long)theLong {}
- (void) evaluatorReportsLevelIndicator3NewValueWithLong:(long)theLong {}


@end

