//
//  WraithSchemeOuterController.mm
//  WraithCocoa
//
//  Created by Jay Freeman on 12/15/06.
//  Copyright 2006 Jay Reynolds Freeman.
//
/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628
*/

#import "WraithSchemeOuterController.h"
#import "BadgedIconView.h"
#import "ColorChooserWindow.h"
#import "DrawerController.h"
#import "MainWindowController.h"
#import "SenseLightArrayController.h"
#import "StatusLightController.h"
#import "JRFNSTextView.h"
#import "JRF2NSTextView.h"
#import "JRF4NSTextView.h"
#import "Preferences.h"
#import "SneakyWindowController.h"
#import "MemoryManager.h"

#import "MainAux.e"

//#import "Globals.e"

extern Boolean isMomCatP;                            // In MainAux.c++
extern Boolean useStdinP;                            // In MainAux.c++
extern UnsignedWord theNumberOfKittens;              // In MainAux.c++
extern UnsignedWord theKittenNumber;                 // In MainAux.c++
extern int evaluatorHasInitialized;                  // In MainAux.c++
extern Boolean setCompileDefinesFromCommandLine;     // In main.mm
extern Boolean commandLineCompileDefinesFlag;        // In main.mm
extern Text InitFilePath[];                          // In SECDIO.c++
extern Boolean	GCInProgressFlag;                    // In Events.c++ and EventsForCocoa.c++
extern Word readyToRead;                             // In WraithSchemeControllerAndModel.mm
extern MemoryManager *theHeap;                       // In MainAux.c++

#define N_SLIDERS 4                // Must match the definition in "Registers.h".
#define N_LEVEL_INDICATORS 4       // Must match the definition in "Registers.h".
#define N_PUSHBUTTONS 4            // Must match the definition in "Registers.h".

extern UnsignedLongWord senseSwitchPositions;
extern UnsignedLongWord sliderValues[ N_SLIDERS ];

// #defines for initialization waits:

#define WAIT_FOR_EVALUATOR_DELAY 1000
#define WAIT_FOR_MAILBOX_DELAY      1

// #defines for debugging:

#define SHOW_THREAD_INTERACTIONS FALSE      // Make TRUE to get log of thread-interaction details.

// #defines for cuteness:

#define ASK_FOR_TREAT_HOW_OFTEN 250         // Ask for kitty treat once in this many startups.
#define SHOW_SNEAKY_WINDOW_HOW_OFTEN 100    // Show classic Apple icon in menu bar once in this many become-actives.
#define PICK_A_FORTUNE_HOW_OFTEN 100        // Display a fortune once in this many exits.
#define SHOW_COLORED_KITTY_HOW_OFTEN 5      // Color the color-chooser window kitty icon the chosen color once in this many choices.
#define SHOW_AUTHOR_HOW_LONG_COUNT 10       // Controls how long the kitty icon shows in Basic Buttons Drawer on startup.
#define SHOW_ICON_HOW_LONG_COUNT 17         // Controls how long the kitty icon shows in Basic Buttons Drawer on startup.

// During initialization, every so often a timer fires to attempt a controller/evaluator
// thread interaction that requires their cooperation to get it done.  This next #define 
// is the delay between such timer firings.  Changing this probably requires a 
// change to SHOW_ICON_HOW_LONG_COUNT in the opposite direction.

#define STANDARD_INITIALIZATION_TIMER_INTERVAL 0.1

// The size of string passed from the controller to the evaluator is variable; the
// next #define is the length started out with.  (Old string is freed and new calloced
// when necessary.)

#define START_STRING_SIZE 512

// The "CAC_" #defines control how often periodic tasks are performed by the controller.
// "CAC" stands for "Controller Action Counter".  Certain things are done once in N
// instances of "singleControllerAction"; these are the N's and the stuff to control
// the relevant counter.  The slight differences in "N"s are for cosmetics -- they
// slightly vary the pattern of blinking lights in the Instrument Panel.

#define CAC_ROLLOVER                                100000

#define CAC_COUNT_FOR_LOAD_LAST_FILE                   20
#define CAC_COUNT_FOR_MOM_SAYS_QUIT                    21
#define CAC_COUNT_FOR_TOGGLE_WHICH_GC_COUNT_WINDOW    100
#define CAC_COUNT_FOR_FADE_EXPECT_VIEW                 21
#define CAC_COUNT_FOR_TOGGLE_EVALUATING_LIGHT           2
#define CAC_COUNT_FOR_TOGGLE_GC_LIGHT                   3
#define CAC_COUNT_FOR_TOGGLE_CD_LIGHT                  30
#define CAC_COUNT_FOR_TOGGLE_LOU_LIGHT                 31
#define CAC_COUNT_FOR_TOGGLE_MLI_LIGHT                 32
#define CAC_COUNT_FOR_TOGGLE_SFP_LIGHT                 33
#define CAC_COUNT_FOR_TOGGLE_EVALUATING_LIGHT_SLOW     34

#define MESSAGE_DISPLAY_COUNT                         1000      // Separate counter, but used similarly to "CAC".

// Defines for placing windows offscreen:

#define OFFSCREEN_X_DELTA 300.0
#define OFFSCREEN_Y_DELTA -300.0

extern Boolean momSaysQuit();
extern void pollControllerOnce();
extern void cleanUpForExit();
extern void makeKittensExit();
extern char *messagesFromControllerNames[];
extern char *messagesFromEvaluatorNames[];
extern void reportGUIInitialized();
extern void increaseBlocking();
extern void setFrontKitten( int aKitten, int aValue );
extern int getFrontKitten( int aKitten );
extern Boolean fatalErrorOccurred;
//extern int evaluatorHasInitialized;
extern LongWord progressFractionInThousandths;
extern Boolean usingGenerationalGC;

SInt32 waitingToExit = 0;
SInt32 outputEnabled = 1;

WraithSchemeOuterController *theWraithSchemeOuterController;

static NSString *fortunes[] = {
    @"\n\nRemember, the world you save may be your own.",
    @"\n\nI wish we didn't need artificial intelligence.",
    @"\n\nRemember to think.",
    @"\n\nA programming language is a means of communication whereby humans and computers each seek to improve the other's thinking.",
    @"\n\nWraith:  With Real Artificial Intelligence, There's Hope.",
    @"\n\nA sign of intelligence is an awareness of one's own ignorance.",
    };

//########  Stuff to set metadata for saved worlds.

const OSType WraithSchemeCreatorCode      = 'cld4';
const OSType WraithSchemeDocumentTypeCode = 'kdl4';

// Must use inner controller's  -- FIXME ... Maybe ...

void setWorldAttributesForPath( const char *path ) 
{
    NSMutableDictionary *fileAttributes = [NSMutableDictionary dictionaryWithCapacity: 2];

    [fileAttributes
        setObject: [NSNumber numberWithUnsignedInt:WraithSchemeCreatorCode]
           forKey: NSFileHFSCreatorCode];

    [fileAttributes
        setObject: [NSNumber numberWithUnsignedInt:WraithSchemeDocumentTypeCode]
           forKey: NSFileHFSTypeCode];

    [[NSFileManager defaultManager]
       setAttributes: fileAttributes
        ofItemAtPath: [NSString stringWithCString:path encoding:NSASCIIStringEncoding]
               error: 0];

    }


//################################################################################
//
//########  Stuff used during initialization.

SInt32 localArgc = 0;
char *localArgv[ 12 ] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };

@implementation WraithSchemeOuterController

//######## Accept the color chosen for the background, and ask if it should be made a preference.

- (void) acceptBackgroundColorChooser {

    static bool coloredKittyShownYet = NO;

    NSImage *scratchKitty = [NSImage imageNamed:@"WraithFace.png"];

    [mWController setBackgroundColor:[myBackgroundColorChooserColorWell color]];
    [mWController setNostalgiaMode:[nBController isOn]];

    [theSharedColorPanel orderOut:nil];

    if( coloredKitty != nil )
        [coloredKitty release];

    if( (! coloredKittyShownYet) && ((random() % SHOW_COLORED_KITTY_HOW_OFTEN) == 0) ) {
        coloredKittyShownYet = YES;
        coloredKitty = [self replaceColorAtPixelX:31 y:63 inImage:scratchKitty withColor:[myBackgroundColorChooserColorWell color]];
        }
    else
        coloredKitty = [scratchKitty retain];

    [backgroundColorPreferenceSheetImageView setImage:coloredKitty];

    if( ! [[myBackgroundColorChooserColorWell color] isEqual:[thePreferences preferredBackgroundColor]] )
        if( isMomCatP )
            [NSApp beginSheet: backgroundColorPreferenceSheet
               modalForWindow: myParentWindow
                modalDelegate: self
               didEndSelector: @selector(colorSheetDidEnd:returnCode:contextInfo:)
                  contextInfo: nil];
    }

//######## Make the selected background color a preference.

- (IBAction) acceptBackgroundColorPreference:(id)sender {

    [thePreferences setPreferredBackgroundColor:[myBackgroundColorChooserColorWell color]];
    [NSApp endSheet:backgroundColorPreferenceSheet];
    }

//######## Accept the chosen color; what to do with it depends on the sender.

- (void) acceptColorChooser:(id)sender {

    if( [sender isEqual:myBackgroundColorChooser] )
        [self acceptBackgroundColorChooser];
    else
        [self acceptTextColorChooser];
    }

//######## Accept the color chosen for the text, and ask if it should be made a preference.

- (void) acceptTextColorChooser {

    NSImage *scratchKitty = [NSImage imageNamed:@"WraithFace.png"];

    [textColorPreferenceSheetImageView setImage:scratchKitty];
    [mWController setTextColor:[myTextColorChooserColorWell color]];
    [mWController setNostalgiaMode:[nBController isOn]];

    [theSharedColorPanel orderOut:nil];

    if( ! [[myTextColorChooserColorWell color] isEqual:[thePreferences preferredTextColor]] )
        if( isMomCatP )
            [NSApp beginSheet: textColorPreferenceSheet
               modalForWindow: myParentWindow
                modalDelegate: self
               didEndSelector: @selector(colorSheetDidEnd:returnCode:contextInfo:)
                  contextInfo: nil];
    }

//######## Make the selected text color a preference.

- (IBAction) acceptTextColorPreference:(id)sender {

    [thePreferences setPreferredTextColor:[myTextColorChooserColorWell color]];
    [NSApp endSheet:textColorPreferenceSheet];
    }

//######## Make the status lights appearances match the states indicated in their controllers.

- (void) adjustLights {
    if( [cDBController statusControllerState] )
        [cDBController turnOn];
    else
        [cDBController turnOff];
    if( [eBController statusControllerState] )
        [eBController turnOn];
    else
        [eBController turnOff];
    if( [gCBController statusControllerState] )
        [gCBController turnOn];
    else
        [gCBController turnOff];
    if( [lOUBController statusControllerState] )
        [lOUBController turnOn];
    else
        [lOUBController turnOff];
    if( [mLIBController statusControllerState] )
        [mLIBController turnOn];
    else
        [mLIBController turnOff];
    if( [nBController statusControllerState] )
        [nBController turnOn];
    else
        [nBController turnOff];
    if( [sFPBController statusControllerState] )
        [sFPBController turnOn];
    else
        [sFPBController turnOff];
    if( [sBController statusControllerState] )
        [sBController turnOn];
    else
        [sBController turnOff];
    }

//########  Every so often, paste a classic multi-color striped-apple Apple icon on top of the
//########  icon used in the menu bar.  I intend this code to honor the, er, Spirit of Apple Past,
//########  and via the associated tooltip, which is:
//########  
//########      "It's a poor sort of memory that only works backwards."  --  Lewis Carroll
//########  
//########  I intend to suggest that that spirit continues.

- (void) applicationDidBecomeActive:(NSNotification *)aNotification {

    static bool sneakyWindowShownYet = FALSE;

    if( (! wantsToQuit) && (! sneakyWindowShownYet) && ((random() % SHOW_SNEAKY_WINDOW_HOW_OFTEN) == 0 ) ) {
        sneakyWindowShownYet = TRUE;
        [mySneakyWindow reveal];
        }
    }

//########   "applicationDidFinishLaunching" is lengthy but straightforward, up till the final
//########   call, to "arrangeForcontrollerInitializationTasksForEvaluator", which see ...

- (void) applicationLaunchWithNumberOfKittens:(UnsignedWord)nKittens
                           withMainMemorySize:(NSInteger)mmSize
                   withGenerationalMemorySize:(NSInteger)gmSize
                      withGenerationalGCInUse:(Boolean)useGGC
                         withHasWorldToLaunch:(Boolean)hasWorld
                              withPathToWorld:(NSString *)pathToWorld
                                withIsMomCatP:(Boolean)momCatP
                             withKittenNumber:(unsigned int)kittenNumber
{
    [super setUpSchemeWithNumberOfKittens:nKittens
                       withMainMemorySize:mmSize
               withGenerationalMemorySize:gmSize
                  withGenerationalGCInUse:useGGC
                     withHasWorldToLaunch:hasWorld
                          withPathToWorld:pathToWorld
                            withIsMomCatP:momCatP
                         withKittenNumber:kittenNumber];

    theWraithSchemeOuterController = self;

    double progress = 0.0;

    // Set up the submenu to load recent files ...

    lRecentFSubMenu = [NSMenu alloc];
    [lRecentFMenuItem setSubmenu:lRecentFSubMenu];
    recentFiles = 0;
//    symbolsInObList = [[NSMutableArray arrayWithCapacity:0] retain];
//    [self loadMutableArray:symbolsInObList fromCStringArrayEndingInZero:DictionaryNames startingWith:0];
    symbolsInWraithSchemeDictionary = [[self createCompletionArray] retain];

    // Set up the submenu to bring kitten windows front and center ...

    kittenFrontSubMenu = [NSMenu alloc];
    [kittenFrontMenuItem setSubmenu:kittenFrontSubMenu];

    // Set up background color, text color, and color-chooser mechanism.

    [mWController setBackgroundColor:[thePreferences preferredBackgroundColor]];
    [myBackgroundColorChooserColorWell setColor:[thePreferences preferredBackgroundColor]];
    [mWController setTextColor:[thePreferences preferredTextColor]];
    [myTextColorChooserColorWell setColor:[thePreferences preferredTextColor]];
    theSharedColorPanel = [NSColorPanel sharedColorPanel];

    // Set up the drawer controller

    [theDrawerController applicationDidFinishLaunchingContent];

    // Show the kitty face in the Basic Buttons Drawer:

    [mWController showIconView:YES];

    // Set up the progress indicator for initialization:

    [myProgressIndicator                setHidden:    YES];
    [myProgressIndicator           setDoubleValue:    progress];

    // Enable or disable assorted menu items and buttons:

    [wraithSchemeMenu         setAutoenablesItems:    NO];
    [aboutMenuItem                     setEnabled:    YES];
    [preferencesMenuItem               setEnabled:    isMomCatP];
    [servicesMenuItem                  setEnabled:    YES];
    [hideWraithSchemeMenuItem          setEnabled:    YES];
    [hideOthersMenuItem                setEnabled:    YES];
    [showAllMenuItem                   setEnabled:    YES];
    [considerSharewareDonationMenuItem setEnabled:    YES];
    [quitMenuItem                      setEnabled:    YES];
    [interpreterMenu          setAutoenablesItems:    NO];
    [lLFWUButton                         setState:    NSOffState];
    [lLFWUMenuItem                       setState:    NSOffState];
    [shortcutsMenu            setAutoenablesItems:    NO];

    [wraithSchemeHelpMenuItem                setKeyEquivalent:    @"?"];
    [wraithSchemeHelpMenuItem    setKeyEquivalentModifierMask:    NSCommandKeyMask];

    // Load some images:

    sadKitty = [[NSImage imageNamed:@"WraithFaceSadKitty.png"] retain];
    wraithFace = [[NSImage imageNamed:@"WraithFace.png"] retain];
    [mySneakyWindow setImage:[NSImage imageNamed:@"StripedApple.png"]];

    // Toggle switches:

    [toggleSwitch0  setEnabled: YES];
    [toggleSwitch1  setEnabled: YES];
    [toggleSwitch2  setEnabled: YES];
    [toggleSwitch3  setEnabled: YES];
    [toggleSwitch4  setEnabled: YES];
    [toggleSwitch5  setEnabled: YES];
    [toggleSwitch6  setEnabled: YES];
    [toggleSwitch7  setEnabled: YES];

    [toggleSwitch0  setState: NSOffState];
    [toggleSwitch1  setState: NSOffState];
    [toggleSwitch2  setState: NSOffState];
    [toggleSwitch3  setState: NSOffState];
    [toggleSwitch4  setState: NSOffState];
    [toggleSwitch5  setState: NSOffState];
    [toggleSwitch6  setState: NSOffState];
    [toggleSwitch7  setState: NSOffState];

    [toggleSwitch0  setTarget: self];
    [toggleSwitch1  setTarget: self];
    [toggleSwitch2  setTarget: self];
    [toggleSwitch3  setTarget: self];
    [toggleSwitch4  setTarget: self];
    [toggleSwitch5  setTarget: self];
    [toggleSwitch6  setTarget: self];
    [toggleSwitch7  setTarget: self];

    // Push buttons:

    [pushButton0  setEnabled: YES];
    [pushButton1  setEnabled: YES];
    [pushButton2  setEnabled: YES];
    [pushButton3  setEnabled: YES];

    [pushButton0  setState: NSOffState];
    [pushButton1  setState: NSOffState];
    [pushButton2  setState: NSOffState];
    [pushButton3  setState: NSOffState];

    [pushButton0  setTarget: self];
    [pushButton1  setTarget: self];
    [pushButton2  setTarget: self];
    [pushButton3  setTarget: self];

    // The Big Red Button:

    [bigRedButton setEnabled: YES];
    [bigRedButton setState: NSOffState];
    [bigRedButton setTarget: self];

    // Sliders:

    [slider0  setEnabled: YES];
    [slider1  setEnabled: YES];
    [slider2  setEnabled: YES];
    [slider3  setEnabled: YES];

    [slider0  setTarget: self];
    [slider1  setTarget: self];
    [slider2  setTarget: self];
    [slider3  setTarget: self];

    [slider0Value  setTarget: self];
    [slider1Value  setTarget: self];
    [slider2Value  setTarget: self];
    [slider3Value  setTarget: self];

    [slider0  setIntValue: 0];
    [slider1  setIntValue: 0];
    [slider2  setIntValue: 0];
    [slider3  setIntValue: 0];

    [slider0Value  setIntValue: 0];
    [slider1Value  setIntValue: 0];
    [slider2Value  setIntValue: 0];
    [slider3Value  setIntValue: 0];

    sliderValues[ 0 ] = 0;
    sliderValues[ 1 ] = 0;
    sliderValues[ 2 ] = 0;
    sliderValues[ 3 ] = 0;

    sliderFormatter = [[NSNumberFormatter alloc] init];

    [sliderFormatter setAllowsFloats:NO];
    [sliderFormatter setMaximum:[NSNumber numberWithInt:255]];
    [sliderFormatter setMinimum:[NSNumber numberWithInt:0]];

    [slider0Value  setFormatter: sliderFormatter];
    [slider1Value  setFormatter: sliderFormatter];
    [slider2Value  setFormatter: sliderFormatter];
    [slider3Value  setFormatter: sliderFormatter];

    // LevelIndicators:

    [levelIndicator0  setEnabled: YES];
    [levelIndicator1  setEnabled: YES];
    [levelIndicator2  setEnabled: YES];
    [levelIndicator3  setEnabled: YES];

    [levelIndicator0  setTarget: self];
    [levelIndicator1  setTarget: self];
    [levelIndicator2  setTarget: self];
    [levelIndicator3  setTarget: self];

    [levelIndicator0Value  setTarget: self];
    [levelIndicator1Value  setTarget: self];
    [levelIndicator2Value  setTarget: self];
    [levelIndicator3Value  setTarget: self];

    // Set menu preferences and all for non-evaluating:

    [self setNonEvaluatorPreferences];

    // Set up lots of view fields:

    [numberOfGCsView                    setHidden:  NO];
    [numberOfGCsLabelView               setHidden:  NO];

    [numberOfGenerationalGCsView        setHidden:  NO];
    [numberOfGenerationalGCsView        setAlphaValue: 0.0];
    [generationalMemorySizeView         setAlphaValue: 0.0];
    [generationalMemoryInUseView        setAlphaValue: 0.0];

    showingGenerationalInformation = false;
    [numberOfGenerationalGCsLabelView   setHidden:  NO];
    [numberOfGenerationalGCsLabelView   setAlphaValue: 0.0];

    [numberOfGCsView               setStringValue:    @"0"];
    [numberOfGenerationalGCsView   setStringValue:    @"0"];
    [lastFileLoadedView            setStringValue:    @""];
    [lastWorldLoadedView           setStringValue:    @""];
    [memorySizeView                setStringValue:    @""];
    [memoryInUseView               setStringValue:    @""];
    [generationalMemorySizeView    setStringValue:    @""];
    [generationalMemoryInUseView   setStringValue:    @""];
    [listPrintLengthView           setStringValue:    @""];
    [listPrintDepthView            setStringValue:    @""];
    [vectorPrintLengthView         setStringValue:    @""];
    [vectorPrintDepthView          setStringValue:    @""];

    showingExpectView = false;
    expectOnCount = 0;
    [expectView                    setStringValue:    @""];

    [mWController              setMessageViewText:    @"Wraith Scheme is initializing.  (Initializing large main memory takes a while.)"];
    [mWController                       setPrompt:    @"Wraith Scheme may ask questions here ..."];
    [mWController                     setResponse:    @"... and will expect you to enter your answers here."];
    [inputView                    setDealWithText:    @selector(dealWithInputText:)];
    [inputView                        setEditable:    YES];
    [responseView                 setDealWithText:    @selector(dealWithResponseText:)];
    [responseView                     setEditable:    NO];
    [mWController displayAll];
    [self setNormalLights];

    [sLAController initialize];

    [self updateLoadRecentMenu];

    [self setForEvaluating];                    //  To protect world load.
    [eBController turnOff];                     //  Don't confuse the user by having the "evaluating" light on in initial world load.

    // Cuteness:  If preferences show that kitty wants a treat -- presumably the user killed the last
    //            process rather than provide one -- be very stubborn, and ask for that treat again!

    if( (! useStdinP) && isMomCatP && (! hasWorldToOpenAtLaunch) ) {
        if( [thePreferences treatPreference] )
            [self runTreatSheet:@"I STILL want a kitty treat!"];
        else if( (random() % ASK_FOR_TREAT_HOW_OFTEN) == 0 )
            [self runTreatSheet:@"May I please have a kitty treat?"];
        }

    // Set up front-kitten mechanism.  Couldn't do this till "theNumberOfKittens" was set.

    [self setUpKittenFrontSubMenu:theNumberOfKittens];

    // Deal with interface appearance and window position on a per-kitten basis:

    [self colorIconForKittenNumber];

    if( theKittenNumber > 0 )
        [myParentWindow setTitle:[NSString stringWithFormat:@"Wraith Scheme -- Kitten %ld", theKittenNumber]];
    else if( theNumberOfKittens > 1 )
        [myParentWindow setTitle:@"Wraith Scheme -- MomCat"];
    else
        [myParentWindow setTitle:@"Wraith Scheme"];

    switch( theKittenNumber ) {
        case  0:                                                                                                    break;
        case  1: [mWController setBackgroundColor:kittenColor1];  [mWController setTextColor:[NSColor blackColor]]; break;
        case  2: [mWController setBackgroundColor:kittenColor2];  [mWController setTextColor:[NSColor blackColor]]; break;
        case  3: [mWController setBackgroundColor:kittenColor3];  [mWController setTextColor:[NSColor blackColor]]; break;
        case  4: [mWController setBackgroundColor:kittenColor4];  [mWController setTextColor:[NSColor blackColor]]; break;
        case  5: [mWController setBackgroundColor:kittenColor5];  [mWController setTextColor:[NSColor blackColor]]; break;
        case  6: [mWController setBackgroundColor:kittenColor6];  [mWController setTextColor:[NSColor blackColor]]; break;
        case  7: [mWController setBackgroundColor:kittenColor7];  [mWController setTextColor:[NSColor blackColor]]; break;
        case  8: [mWController setBackgroundColor:kittenColor8];  [mWController setTextColor:[NSColor blackColor]]; break;
        case  9: [mWController setBackgroundColor:kittenColor9];  [mWController setTextColor:[NSColor blackColor]]; break;
        case 10: [mWController setBackgroundColor:kittenColor10]; [mWController setTextColor:[NSColor blackColor]]; break;
        case 11: [mWController setBackgroundColor:kittenColor11]; [mWController setTextColor:[NSColor blackColor]]; break;
        case 12: [mWController setBackgroundColor:kittenColor12]; [mWController setTextColor:[NSColor blackColor]]; break;
        case 13: [mWController setBackgroundColor:kittenColor13]; [mWController setTextColor:[NSColor blackColor]]; break;
        case 14: [mWController setBackgroundColor:kittenColor14]; [mWController setTextColor:[NSColor blackColor]]; break;
        case 15: [mWController setBackgroundColor:kittenColor15]; [mWController setTextColor:[NSColor blackColor]]; break;
        default:                                                                                                    break;
        }

        NSRect theWindowFrame = [myParentWindow frame];

        CGFloat slightOffsetX =  36.0;
        CGFloat slightOffsetY = -50.0;
        CGFloat biggerOffsetX =  50.0;

        switch( theKittenNumber ) {
            case 1: 
                theWindowFrame.origin.x += 1.0 * slightOffsetX + biggerOffsetX;
                theWindowFrame.origin.y += 1.0 * slightOffsetY;
                break;
            case 2: 
                theWindowFrame.origin.x += 2.0 * slightOffsetX + biggerOffsetX;
                theWindowFrame.origin.y += 2.0 * slightOffsetY;
                break;
            case 3: 
                theWindowFrame.origin.x += 3.0 * slightOffsetX + biggerOffsetX;
                theWindowFrame.origin.y += 3.0 * slightOffsetY;
                break;
            case 4: 
                theWindowFrame.origin.x += 4.0 * slightOffsetX + biggerOffsetX;
                theWindowFrame.origin.y += 4.0 * slightOffsetY;
                break;
            case 5: 
                theWindowFrame.origin.x += 5.0 * slightOffsetX + biggerOffsetX;
                theWindowFrame.origin.y += 5.0 * slightOffsetY;
                break;
            case 6: 
                theWindowFrame.origin.x += 6.0 * slightOffsetX + biggerOffsetX;
                theWindowFrame.origin.y += 6.0 * slightOffsetY;
                break;
            case 7: 
                theWindowFrame.origin.x += 7.0 * slightOffsetX + biggerOffsetX;
                theWindowFrame.origin.y += 7.0 * slightOffsetY;
                break;
            case 8: 
                theWindowFrame.origin.x += biggerOffsetX;
                // theWindowFrame.origin.y is unchanged
                break;
            case 9: 
                theWindowFrame.origin.x += 1.0 * slightOffsetX;
                theWindowFrame.origin.y += 1.0 * slightOffsetY;
                break;
            case 10: 
                theWindowFrame.origin.x += 2.0 * slightOffsetX;
                theWindowFrame.origin.y += 2.0 * slightOffsetY;
                break;
            case 11: 
                theWindowFrame.origin.x += 3.0 * slightOffsetX;
                theWindowFrame.origin.y += 3.0 * slightOffsetY;
                break;
            case 12: 
                theWindowFrame.origin.x += 4.0 * slightOffsetX;
                theWindowFrame.origin.y += 4.0 * slightOffsetY;
                break;
            case 13: 
                theWindowFrame.origin.x += 5.0 * slightOffsetX;
                theWindowFrame.origin.y += 5.0 * slightOffsetY;
                break;
            case 14: 
                theWindowFrame.origin.x += 6.0 * slightOffsetX;
                theWindowFrame.origin.y += 6.0 * slightOffsetY;
                break;
            case 15: 
                theWindowFrame.origin.x += 7.0 * slightOffsetX;
                theWindowFrame.origin.y += 7.0 * slightOffsetY;
                break;
            default: break;
            }

        [myParentWindow setFrame:theWindowFrame display:YES];

    // Cuteness:  Deal with possible preference for nostalgia mode.

    if( [thePreferences objectForKey:NostalgiaMode] && ([[thePreferences objectForKey:NostalgiaMode] boolValue] != 0) )
        [self setForNostalgiaMode];
    else
        [self setForNotNostalgiaMode];

    [self beginInitProgress];

    [self launchScheme];

    [self endInitProgress];

    if( fatalErrorOccurred )
        [self reportEarlyDisasterAndQuit];

    //  On your mark ... get set ... (this starts a never-ending timer series running).

    [self startController];

    //  On your mark ... get set ... (this starts a series of events that must synchronize with evaluator initialization).

    // To avoid a "while( TRUE )" loop in "applicationDidFinishLaunching", we arrange to
    // fire a timer which calls a function that checks whether the evaluator has initialized, and does some tasks
    // for it if so; if not, it fires another timer to do the same thing,

    [NSTimer scheduledTimerWithTimeInterval  :  STANDARD_INITIALIZATION_TIMER_INTERVAL
                                     target  :  self
                                   selector  :  @selector(doControllerInitializationTask:)
                                   userInfo  :  nil
                                    repeats  :  NO];

//    [self Test1:self];    // For testing fatal error handling.

    }

//########  Make the given kitten become the front window on the desktop.

- (IBAction) arrangeKittenInFront:(id)sender { setFrontKitten( (int)([sender tag] & WORD_MASK), 1 ); }


//########  Make all kittens' windows come to the front, with this kitten's window foremost, and the MomCat's just under it.

#define KITTEN_DISPLAY_DELAY 100000  // uS

- (IBAction) arrangeAllKittensInFront:(id)sender {

    UnsignedWord kittenCounter = 0;
    
    for( kittenCounter = 1; kittenCounter < theNumberOfKittens; ++kittenCounter ) {
        if( kittenCounter != theKittenNumber ) {
            setFrontKitten( kittenCounter, 1 );
            usleep( KITTEN_DISPLAY_DELAY );
            }
        }

    if( theKittenNumber != 0 ) {
        setFrontKitten( 0, 1 );
        usleep( KITTEN_DISPLAY_DELAY );
        }
    setFrontKitten( theKittenNumber, 1 );
    }

//########  Start the progress bar at zero.

- (void)beginInitProgress {

    [myProgressIndicator setDoubleValue:0.0];
    [myProgressIndicator setHidden:NO];
    [myProgressIndicator display];
    }

//########  Take action when the Big Red Button gets pushed.

- (IBAction) bigRedButtonPushed:(id)sender { [self runInterruptLocallyForBigRedButton]; }

//########  Forget about changing the background color.

- (void) cancelBackgroundColorChooser:(id)sender {

    [mWController setBackgroundColor:savedBackgroundColor];
    [mWController setNostalgiaMode:[nBController isOn]];
    [theSharedColorPanel orderOut:nil];
    [myBackgroundColorChooser close];
    }

//########  Forget about changing the text color.

- (void) cancelTextColorChooser:(id)sender {

    [mWController setTextColor:savedTextColor];
    [mWController setNostalgiaMode:[nBController isOn]];
    [theSharedColorPanel orderOut:nil];
    [myTextColorChooser close];
    }

//########  Clean up the color-choosing stuff.

- (void) cleanUpColorChooser { [theSharedColorPanel orderOut:nil]; }

//########  Clear the list of recently-loaded files.

- (IBAction) clearRecentFilesWrapper:(id)sender {

    [thePreferences clearRecentFiles:self];
    [self updateLoadRecentMenu];
    }

//########  Close the sheet that asks for a shareware donation.

- (IBAction) closeDonationSheet:(id)sender {

    [NSApp endSheet:myDonationSheet];
    }

//########  Main method that recolors the Wraith Scheme icon on a per-kitten basis.

- (void) colorIconForKittenNumber {

//    NSImage *scratchKitty = [[NSImage imageNamed:@"WraithFace.png"] retain];   // Out 2010.09.02
    NSImage *scratchKitty = [NSImage imageNamed:@"WraithFace.png"];
    NSImage *kittyIcon = NULL;

    switch( theKittenNumber ) {
        case  0: kittyIcon = [NSImage imageNamed:@"WraithSchemeIcon.128.png"];                                 break;
        case  1: kittyIcon = [self replaceColorAtPixelX:31 y:63 inImage:scratchKitty withColor:kittenColor1];  break;
        case  2: kittyIcon = [self replaceColorAtPixelX:31 y:63 inImage:scratchKitty withColor:kittenColor2];  break;
        case  3: kittyIcon = [self replaceColorAtPixelX:31 y:63 inImage:scratchKitty withColor:kittenColor3];  break;
        case  4: kittyIcon = [self replaceColorAtPixelX:31 y:63 inImage:scratchKitty withColor:kittenColor4];  break;
        case  5: kittyIcon = [self replaceColorAtPixelX:31 y:63 inImage:scratchKitty withColor:kittenColor5];  break;
        case  6: kittyIcon = [self replaceColorAtPixelX:31 y:63 inImage:scratchKitty withColor:kittenColor6];  break;
        case  7: kittyIcon = [self replaceColorAtPixelX:31 y:63 inImage:scratchKitty withColor:kittenColor7];  break;
        case  8: kittyIcon = [self replaceColorAtPixelX:31 y:63 inImage:scratchKitty withColor:kittenColor8];  break;
        case  9: kittyIcon = [self replaceColorAtPixelX:31 y:63 inImage:scratchKitty withColor:kittenColor9];  break;
        case 10: kittyIcon = [self replaceColorAtPixelX:31 y:63 inImage:scratchKitty withColor:kittenColor10]; break;
        case 11: kittyIcon = [self replaceColorAtPixelX:31 y:63 inImage:scratchKitty withColor:kittenColor11]; break;
        case 12: kittyIcon = [self replaceColorAtPixelX:31 y:63 inImage:scratchKitty withColor:kittenColor12]; break;
        case 13: kittyIcon = [self replaceColorAtPixelX:31 y:63 inImage:scratchKitty withColor:kittenColor13]; break;
        case 14: kittyIcon = [self replaceColorAtPixelX:31 y:63 inImage:scratchKitty withColor:kittenColor14]; break;
        case 15: kittyIcon = [self replaceColorAtPixelX:31 y:63 inImage:scratchKitty withColor:kittenColor15]; break;
        default: kittyIcon = scratchKitty;
        }

    if( theKittenNumber >= 1 ) {
        [[[NSApplication sharedApplication] dockTile]
            setContentView:
                [[BadgedIconView alloc] initWithImage:kittyIcon andString:[NSString stringWithFormat:@"%d", theKittenNumber]]];
        [[[NSApplication sharedApplication] dockTile] display];
        }

    }

//######## The "did-end" handler is used by both the text and background color-choice stuff.

- (void) colorSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo {
 
    [sheet orderOut:self];
    }
        
//########  The problem here is potential deadlock if we do two "tellEvaluators" in the same timer-firing.  This is the
//########  thread that does GUI I/O, and the evaluator must call the thread that does GUI I/O in order to respond to
//########  some of the "tellEvaluator" commands.  But if there are two "tellEvaluators" in a row, the GUI I/O thread
//########  will be waiting for the second "tellEvaluator" to respond while the evaluator thread is waiting for the
//########  GUI I/O thread to perform some I/O that was consequent to the first "tellEvaluator".  The fix is to
//########  (1) *DO* *AT* *MOST* *ONE* "tellEvaluator" per timer firing -- the code following is so written -- and
//########  (2) Make sure not even to call "controllerInitializationTask" unless it is the controller's turn to use
//########  the controller/evaluator interface (guaranteed by test in "doControllerInitializationTask").

- (bool)controllerInitializationTask {

    static int  task8TryCount = 0;
    static int  task9TryCount = 0;

    // Rule here -- at most one "tellEvaluator" per task.
    // This code allows for as many as nine tasks, and it is easy to add more.
    // Not all task opportunities are presently used for anything.

    const char *initialFileName = 0;

    // Synchronize threads  --  getting past the first "if" means that the evaluator is set up for
    // kitten-to-kitten communication (the Clowder is running), and also means that the
    // GUI thread has advised the evaluator thread that the GUI is ready for messages from the 
    // evaluator.

    switch( nextTask ) {
        case 1:
            reportGUIInitialized();
            nextTask = 2;
            break;
        case 2:
            if( evaluatorHasInitialized )
                nextTask = 3;
            break;
        case 3:
            if(    [thePreferences objectForKey:DisplayFullPrecision]
                && ([[thePreferences objectForKey:DisplayFullPrecision] boolValue] != 0) )
                [self tellEvaluator:C_PLEASE_USE_HIGH_PRECISION withLong:0 withString:""];
            else
                [self tellEvaluator:C_PLEASE_USE_LOW_PRECISION withLong:0 withString:""];
            nextTask = 4;
            break;
        case 4:
            if( setCompileDefinesFromCommandLine ) {
                if( commandLineCompileDefinesFlag )
                    [self tellEvaluator:C_PLEASE_COMPILE_DEFINES withLong:0 withString:""];
                else
                    [self tellEvaluator:C_PLEASE_DO_NOT_COMPILE_DEFINES withLong:0 withString:""];
                }
            else {
                if( [thePreferences objectForKey:CompileDefines] && ([[thePreferences objectForKey:CompileDefines] boolValue] != 0) )
                    [self tellEvaluator:C_PLEASE_COMPILE_DEFINES withLong:0 withString:""];
                else
                    [self tellEvaluator:C_PLEASE_DO_NOT_COMPILE_DEFINES withLong:0 withString:""];
                }
            nextTask = 5;
            break;
        case 5:
            if( allKittensAreActive() )
                nextTask = 6;
            break;
        case 6:
            readyToRead = 1;
            if( isMomCatP && ([thePreferences objectForKey:PathToInitialLoad] != nil) ) {
              initialFileName = [[thePreferences objectForKey:PathToInitialLoad] cStringUsingEncoding:NSASCIIStringEncoding];
                if( initialFileName ) {
                    if( strlen( initialFileName ) > 0 ) {
                       [self setForEvaluating];
                       if( initialFileName[0] == '/' )    // Absolute path.
                           [self    tellEvaluator:    C_INPUT_READY
                                          withLong:    0
                                        withString:    [[NSString stringWithFormat:@"(load \"%s\")\n", initialFileName]
                                                          cStringUsingEncoding:NSASCIIStringEncoding]];
                       else                              // Relative path.
                           [self    tellEvaluator:    C_INPUT_READY
                                          withLong:    0
                                        withString:    [[NSString stringWithFormat:
                                                                     @"(load (string-append (e::program-directory) \"%s\"))\n", 
                                                                     initialFileName]
                                                          cStringUsingEncoding:NSASCIIStringEncoding]];
                        }
                    }
                else
                    [self localError:@"\nThe path (including the file name) to the initial file to load -- as shown in your preferences -- includes non-ASCII characters.\n\nScheme is supposed to use only ASCII characters.  They are pretty much the \"regular\" keyboard characters, entered without the \"command-\" and \"option-\" modifier keys.\n\nPlease use the \"Preferences\" item in the \"Wraith Scheme\" menu to select a path containing only ASCII characters."];
                }
            // else there is no initial-load file
            nextTask = 7;
            break;
        case 7:
            [mWController setMessageViewText:@"Wraith Scheme puts brief messages here."];
            nextTask = 8;
            break;
        case 8:
            if( task8TryCount++ < SHOW_AUTHOR_HOW_LONG_COUNT )  // Leave the author named for a while.
                break;
            [mWController showAuthorField:NO];
            nextTask = 9;
            break;
        case 9:
            if( (task8TryCount + task9TryCount++) < SHOW_ICON_HOW_LONG_COUNT )  // Leave icon on a while longer, even.
                break;
            [mWController showIconView:NO];
            nextTask = 10;
            break;
        default:
            return YES;       // Done with initialization tasks.
        }
    return NO;                // Not done with initialization tasks.
    }    

//########  Process text from the Input Panel:

- (bool) dealWithInputText:(NSString *)theText {

    const char *theString = 0;

    if( [theText canBeConvertedToEncoding:NSASCIIStringEncoding] ) {
        theString = [theText cStringUsingEncoding:NSASCIIStringEncoding];
        if( theString && (strlen( theString ) > 0) ) {
            [mWController displayStringInMainView:theText];
            [self tellEvaluator:C_INPUT_READY withLong:0 withString:theString];
            }
        // else nothing to do
        }
    else {
        [self localError:@"\nYou have entered some non-ASCII characters.\n\nScheme is supposed to use only ASCII characters.  They are pretty much the \"regular\" keyboard characters, entered without the \"command-\" and \"option-\" modifier keys.\n\nPlease retype the previous line of input.\n\n(To retrieve that line for modification, use the \"Previous Command\" button, or put the cursor in the input panel and use the \"up-arrow\" key.)"];
        }
    return true;
    }

//########  Deal with the response to a dialog:

- (bool) dealWithResponseText:(NSString *)theText {

    const char *theString = 0;
    const char *theCString = "";
    NSInteger theLength = 0;

    if( [theText canBeConvertedToEncoding:NSASCIIStringEncoding] ) {
        theString = [theText cStringUsingEncoding:NSASCIIStringEncoding];
        if( theString && (strlen( theString ) > 0 ) ) {
            [self startNormalInput];
            if( (theLength = [theText length]) > 0 )
                theCString = [[theText substringToIndex:(theLength - 1)] cStringUsingEncoding:NSASCIIStringEncoding];
            [self tellEvaluator:C_DIALOG_RESPONSE_READY withLong:0 withString:theCString];
            }
        // else nothing to do
        }
    else {
        [self localError:@"\nYour response includes some non-ASCII characters.\n\nScheme is supposed to use only ASCII characters.  They are pretty much the \"regular\" keyboard characters, entered without the \"command-\" and \"option-\" modifier keys.\n\nPlease retype your response to the dialog."];
        [responseView insertText:[theText substringToIndex:([theText length] - 1)]];
        return false;
        }

    [mWController      setResponse:theText];

    [mWController      enableMakeInputPanelFirstResponderItem];
    [inputView         setEditable:YES];
    [inputView         enableShortcuts];
    [responseView      setEditable:NO];
    [responseView      setSelectable:YES];
    [mWController      showIconView:NO];

    return true;
    }

//########  Decline to make the recently chosen background color a preference.

- (IBAction) declineBackgroundColorPreference:(id)sender {

    [NSApp endSheet:backgroundColorPreferenceSheet];
    }

//########  Decline to make the recently chosen text color a preference.

- (IBAction) declineTextColorPreference:(id)sender {

    [NSApp endSheet:textColorPreferenceSheet];
    }

//########  Sheet for verifying the user wants to disable console output: Did-end handler.

- (void) disableOutputSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo {
    outputEnabled = TRUE;
    [mWController showIconView:FALSE];
    [sheet orderOut:nil];
    }

//########  This is the code that runs during the initialization-timer events.
//########  It runs first on the basis of "arrangeForControllerInitializationTasksForEvaluator"'s timer,
//########  then on the basis of a timer which it itself sets.

- (void) doControllerInitializationTask:(NSTimer *)theTimer {

    if( [self controllerInitializationTask] )
        ;
    else
        [NSTimer scheduledTimerWithTimeInterval  :  STANDARD_INITIALIZATION_TIMER_INTERVAL
                                         target  :  self
                                       selector  :  @selector(doControllerInitializationTask:)
                                       userInfo  :  nil
                                        repeats  :  NO];
    }

//########  Sheet for reminding the user how to make a shareware donation: Did-end handler.

- (void) donationSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo {

    [donationSheetImageView setImage:sadKitty];
    [donationSheetImageView display];
    [boldField setHidden:TRUE];
    [myDonationSheet display];
    usleep( 400000  );
    [normalField setHidden:TRUE];
    [myDonationSheet display];
    usleep( 600000 );
    [closeButton setHidden:TRUE];
    [myDonationSheet display];

    usleep( 600000 );

    [sheet orderOut:self];
    }

//########  Wrap up use of the progress indicator.

- (void)endInitProgress {

    [myProgressIndicator setHidden:YES];
    [myProgressIndicator display];
    }

//######## Do-nothing methods to be overwritten by subclasses of WraithSchemeControllerAndModel.

- (void) incrementProgressWithLong:(long)theLong { [self updateInitProgress:((double)theLong)]; }

- (void) evaluatorReportsCompletedInitialization { [myParentWindow display]; }

- (void) evaluatorReportsFatalProblemWithLong:(long)theLong withString:(NSString *)theString {
    [self reportDisasterAndQuit:[theString cStringUsingEncoding:NSASCIIStringEncoding]];
    }

- (void) evaluatorRequestsMakeFront {
    if( [myParentWindow isMiniaturized] )
        [myParentWindow deminiaturize:self];
    [NSApp activateIgnoringOtherApps:YES];
    }

- (void) evaluatorReportsNewOblistSymbolWithLong:(long)theLong withString:(NSString *)theString {}
- (void) evaluatorRequestsClearOblistSymbolsWithLong:(long)theLong withString:(NSString *)theString {}

- (void) evaluatorReportsEvaluating { [self setForEvaluating]; }

- (void) evaluatorReportsNotEvaluating { [self setForNotEvaluating]; }

- (void) evaluatorReportsAcknowledgeReset {
    [self newViewNumber:theHeap->garbageCollectorGCCount()              forView:    numberOfGCsView             ];
    [self newViewNumber:theHeap->garbageCollectorGenerationalGCCount()  forView:    numberOfGenerationalGCsView ];
    [self setForNotEvaluating];
    [self setForNotGCing];
    }

- (void) evaluatorReportsGarbageCollecting { [self setForGCing]; }

- (void) evaluatorReportsNotGarbageCollecting { [self setForNotGCing]; }

- (void) evaluatorReportsSleeping { [sBController turnOn]; }

- (void) evaluatorReportsWakedUp { [sBController turnOff]; }

- (void) evaluatorReportsCompilingDefines {
    [cDBController turnOn];
    [cDMenuItem setState:NSOnState];
    }

- (void) evaluatorReportsNotCompilingDefines {
    [cDBController turnOff];
    [cDMenuItem setState:NSOffState];
    }

- (void) evaluatorReportsUsingLowPrecision {
    [sFPBController turnOff];
    [sFPMenuItem setState:NSOffState];
    }

- (void) evaluatorReportsUsingHighPrecision {
    [sFPBController turnOn];
    [sFPMenuItem setState:NSOnState];
    }

- (void) evaluatorReportsOutputReadyWithLong:(long)theLong withString:(NSString *)theString {
    if( outputEnabled )
        [mWController displayStringInMainView:theString];
    }

- (void) evaluatorReportsMessageToDisplayWithLong:(long)theLong withString:(NSString *)theString {
    messageDisplayCounter = 0;
    displayingMessage = true;
    [mWController setMessageViewText:theString];
    }

- (void) evaluatorReportsDialogToStartWithLong:(long)theLong withString:(NSString *)theString {
    [NSApp requestUserAttention:NSInformationalRequest];
    if( [theString length] > 0 )
        [self startDialog:theString];
    else {
        [mWController setPrompt:@""];
        [mWController setResponse:@""];
        }
    [mWController showIconView:YES];
    }

- (void) evaluatorReportsFileLoadBeginningWithLong:(long)theLong withString:(NSString *)theString {
    [self fileLoadBeginning:[theString cStringUsingEncoding:NSASCIIStringEncoding]];
    }

- (void) evaluatorReportsFileLoadSucceeded { [self setForFileLoadBegan]; }

- (void) evaluatorReportsFileLoadFailed {
    if( lastFileLoadBegan )
        [self setForFileLoadBegan];
    else
        [self fileLoadFailed]; 
    }

- (void) evaluatorReportsWorldLoadSucceededWithLong:(long)theLong withString:(NSString *)theString {
    [self worldLoadSucceeded:[theString cStringUsingEncoding:NSASCIIStringEncoding]];
    }

- (void) evaluatorReportsWorldLoadFailed {}

- (void) evaluatorReportsWorldSaveSucceeded {
    if( waitingToExit ) {
        [self turnLightsOff];
        makeKittensExit();
        cleanUpForExit();
        [NSApp terminate:self];
        }
    }

- (void) evaluatorReportsWorldSaveFailed { waitingToExit = 0; }

- (void) evaluatorReportsExpectMoreInput {
    expectMoreInput = true;
    [mLIBController turnOn];
    }

- (void) evaluatorReportsDoNotExpectMoreInput {
    expectMoreInput = false;
    [mLIBController turnOff];
    [self newViewString:"" forView:expectView];
    }

- (void) evaluatorReportsExpectParensWithLong:(long)theLong {
    switch( theLong ) {
        case 0:  [self newViewString:""           forView:expectView]; break;
        case 1:  [self newViewString:"Expect )"   forView:expectView]; break;
        case 2:  [self newViewString:"Expect ))"  forView:expectView]; break;
        case 3:  [self newViewString:"Expect )))" forView:expectView]; break;
        default: [self newViewString:"Expect ))s" forView:expectView]; break;
        }
    }

- (void) evaluatorReportsExpectDoubleQuote { [self newViewString:"Expect \""   forView:expectView]; }

- (void) evaluatorReportsDiscardedInput {}

- (void) evaluatorReportsNewGCCountWithLong:(long)theLong { [self newViewNumber:theLong forView:numberOfGCsView]; }

- (void) evaluatorReportsNewGenerationalGCCountWithLong:(long)theLong {
    [self newViewNumber:theLong forView: numberOfGenerationalGCsView ];
    }

- (void) evaluatorReportsMemorySizeWithLong:(long)theLong { [self newViewNumber:theLong forView:memorySizeView]; }

- (void) evaluatorReportsMemoryInUseWithLong:(long)theLong { [self newViewNumber:theLong forView:memoryInUseView]; }

- (void) evaluatorReportsGenerationalMemorySizeWithLong:(long)theLong {
    [self newViewParenthesizedNumber:theLong forView:generationalMemorySizeView];
    }

- (void) evaluatorReportsGenerationalMemoryInUseWithLong:(long)theLong {
    [self newViewParenthesizedNumber:theLong forView:generationalMemoryInUseView];
    }

- (void) evaluatorReportsListPrintLengthWithLong:(long)theLong { [self newViewNumber:theLong forView:listPrintLengthView]; }

- (void) evaluatorReportsListPrintDepthWithLong:(long)theLong { [self newViewNumber:theLong forView:listPrintDepthView]; }

- (void) evaluatorReportsVectorPrintLengthWithLong:(long)theLong { [self newViewNumber:theLong forView:vectorPrintLengthView]; }

- (void) evaluatorReportsVectorPrintDepthWithLong:(long)theLong { [self newViewNumber:theLong forView:vectorPrintDepthView]; }

- (void) evaluatorReportsSenseLightCommandWithLong:(long)theLong { [sLAController interpretLongCommand:theLong]; }

- (void) evaluatorReportsSenseLightRotateWithLong:(long)theLong { [sLAController rotateLights:theLong]; }

- (void) evaluatorReportsInitProgress { [self beginInitProgress]; }

- (void) evaluatorReportsIncrementProgressWithLong:(long)theLong { [self updateInitProgress:((double)theLong)]; }

- (void) evaluatorReportsWrapupProgress { [self endInitProgress]; }

- (void) evaluatorReportsLevelIndicator0NewValueWithLong:(long)theLong {

    [levelIndicator0 setIntValue:(int)theLong];
    [levelIndicator0Value setIntValue:(int)theLong];
    }

- (void) evaluatorReportsLevelIndicator1NewValueWithLong:(long)theLong {

    [levelIndicator1 setIntValue:(int)theLong];
    [levelIndicator1Value setIntValue:(int)theLong];
    }

- (void) evaluatorReportsLevelIndicator2NewValueWithLong:(long)theLong {

    [levelIndicator2 setIntValue:(int)theLong];
    [levelIndicator2Value setIntValue:(int)theLong];
    }

- (void) evaluatorReportsLevelIndicator3NewValueWithLong:(long)theLong {

    [levelIndicator3 setIntValue:(int)theLong];
    [levelIndicator3Value setIntValue:(int)theLong];
    }


//########  Code to run when a file load is to begin.

- (void) fileLoadBeginning:(const char *)givenPath {

    char path[ MAX_PATHNAME_LENGTH ];

    if( givenPath[ 0 ] == '/' )  // givenPath is absolute.
        strncpy( path, givenPath, MAX_PATHNAME_LENGTH );
    else {                  // givenPath is relative, must prepend current working directory
        if( getcwd( path, MAX_PATHNAME_LENGTH ) == NULL )
            [self reportDisasterAndQuit:FatalErrorStrings[ 182 ]];
        strncpy( path + strlen( path ), "/", MAX_PATHNAME_LENGTH - strlen( path ) );
        strncpy( path + strlen( path ), givenPath, MAX_PATHNAME_LENGTH - strlen( path ) );
        }

    [self newViewString:[[[NSString stringWithFormat:@"%s", path] lastPathComponent] cStringUsingEncoding:NSASCIIStringEncoding]
                forView:lastFileLoadedView];
    lastFileLoadBegan = TRUE;

    strncpy( nextToLastFileLoadedPathAsCString, lastFileLoadedPathAsCString, sizeof( nextToLastFileLoadedPathAsCString ) - 1 );
    strncpy( lastFileLoadedPathAsCString, path, sizeof( lastFileLoadedPathAsCString ) - 1 );

    [thePreferences addLoadRecentPreference:[NSString stringWithFormat:@"%s", path]];
    [self updateLoadRecentMenu];

    if( nextToLastFileLoadedPath != nil)
        [nextToLastFileLoadedPath release];

    nextToLastFileLoadedPath = [lastFileLoadedPath retain];

    if( lastFileLoadedPath != nil)
        [lastFileLoadedPath release];

    lastFileLoadedPath = [[NSString stringWithFormat:@"%s", path] retain];

    [lastFileLoadedView setToolTip:lastFileLoadedPath];
    [lLFButton    setToolTip:[NSString stringWithFormat:@"Reload \"%s\"", path]];
    [lLFMenuItem  setToolTip:[NSString stringWithFormat:@"Reload \"%s\"", path]];
    if( nextToLastFileLoadedPathAsCString[0] != 0 )
        [lNLFMenuItem setToolTip:[NSString stringWithFormat:@"Reload \"%s\"", nextToLastFileLoadedPathAsCString]];


    if( stat( path, &lastLoadStatBuffer ) == -1 )
        lastLoadStatBufferIsGood = NO;
    else
        lastLoadStatBufferIsGood = YES;

    [self setForFileLoadBegan];
    }

//######## Loading a file: Failed.

- (void) fileLoadFailed {

    lastFileLoadBegan = FALSE;

    [self newViewString:"" forView:lastFileLoadedView];
    [self setForFileLoadFailed];
    }

//########  Find a given string in the Wraith Scheme Dictionary, if possible.

- (IBAction)findInWraithSchemeDictionary:(id)sender {

//  Format string for the lookup.  This *is* part of the method, just formatted in a compromising manner.

const char *scriptTemplate =
"echo $'\
tell application \"Safari\"\n\
     make new document with properties {URL:\"file://%sWraith%%20Scheme.app/Contents/Resources/English.lproj/Wraith%%20Scheme%%20Help/Wraith%%20Scheme%%20Dictionary.html#%s\"}\n\
     get source of front document\n\
     if source of front document contains \"<a name=\\\\\"%s\" then\n\
        activate\n\
    else\n\
        close front window\n\
        beep\n\
    end if\n\
end tell\
' | osascript";

    NSResponder *currentFocusedView = [myParentWindow firstResponder];

    if( [currentFocusedView isEqual:inputView] || [currentFocusedView isEqual:mainView] ) {

        NSArray *selRanges = [((NSTextView *)currentFocusedView) selectedRanges];
        NSRange selectionRange = [[selRanges objectAtIndex:0] rangeValue];

        if( selectionRange.length > 0 ) {

            NSString *theString = [[((NSTextView *)currentFocusedView) string] substringWithRange:selectionRange];
            char buffer[ MAXPATHLEN + 3000 ];

            snprintf( buffer,
                      MAXPATHLEN + 3000,
                      scriptTemplate,
                      InitFilePath,
                      [theString UTF8String],
                      [theString UTF8String]
                      );

            if( system( buffer ) != 0 ) {
                [self localError:@"Wraith Scheme could not find such an entry in the dictionary."];
                return;
                }

            }
        else
            NSBeep();
        }
    else
        NSBeep();
    }

//########  "init" mostly sets up controller state variables.

- (id) init {

    self = [super  init];

    evaluating = false;
    displayingMessage = false;
    stopEvaluatingPending = false;
    gcing = false;
    wantsToQuit = false;
    lastFileLoadBegan = false;
    loadingLastFileWhenUpdated = false;

    outputEnabled = 1;

    expectMoreInput = false;

    controllerActionCounter = 0;
    nextTask = 1;
    
    savedBackgroundColor = nil;
    savedTextColor = nil;
    lastFileLoadedPath = nil;
    nextToLastFileLoadedPath = nil;
    lastFileLoadedPathAsCString[0] = 0;
    nextToLastFileLoadedPathAsCString[0] = 0;

    kittenColor1  = [[NSColor colorWithCalibratedRed:1.00 green:0.75 blue:0.75 alpha:1.0] retain];
    kittenColor2  = [[NSColor colorWithCalibratedRed:0.75 green:1.00 blue:0.75 alpha:1.0] retain];
    kittenColor3  = [[NSColor colorWithCalibratedRed:0.75 green:0.75 blue:1.00 alpha:1.0] retain];
    kittenColor4  = [[NSColor colorWithCalibratedRed:1.00 green:0.75 blue:1.00 alpha:1.0] retain];
    kittenColor5  = [[NSColor colorWithCalibratedRed:0.75 green:1.00 blue:1.00 alpha:1.0] retain];
    kittenColor6  = [[NSColor colorWithCalibratedRed:0.75 green:0.75 blue:0.75 alpha:1.0] retain];
    kittenColor7  = [[NSColor colorWithCalibratedRed:0.95 green:0.95 blue:0.95 alpha:1.0] retain];
    kittenColor8  = [[NSColor colorWithCalibratedRed:1.00 green:0.87 blue:0.75 alpha:1.0] retain];
    kittenColor9  = [[NSColor colorWithCalibratedRed:1.00 green:0.75 blue:0.87 alpha:1.0] retain];
    kittenColor10 = [[NSColor colorWithCalibratedRed:0.87 green:1.00 blue:0.75 alpha:1.0] retain];
    kittenColor11 = [[NSColor colorWithCalibratedRed:0.75 green:1.00 blue:0.87 alpha:1.0] retain];
    kittenColor12 = [[NSColor colorWithCalibratedRed:0.87 green:0.75 blue:1.00 alpha:1.0] retain];
    kittenColor13 = [[NSColor colorWithCalibratedRed:0.75 green:0.87 blue:1.00 alpha:1.0] retain];
    kittenColor14 = [[NSColor colorWithCalibratedRed:1.00 green:0.87 blue:0.87 alpha:1.0] retain];
    kittenColor15 = [[NSColor colorWithCalibratedRed:0.87 green:1.00 blue:0.87 alpha:1.0] retain];

    srandom( (SInt32)(time( 0 ) & WORD_MASK));

    return self;
    }

//######## Loading the last file loaded:

- (IBAction) loadLastFile:(id)sender {

    if( strlen( lastFileLoadedPathAsCString ) > 0 ) {
        [self setForEvaluating];
        [self tellEvaluator:C_INPUT_READY
                   withLong:0
                 withString:[[NSString stringWithFormat: @"(load \"%s\")\n", lastFileLoadedPathAsCString]
                             cStringUsingEncoding:NSASCIIStringEncoding]];
        }
    }
        
//- (void)loadMutableArray:(NSMutableArray *)nsa fromCStringArrayEndingInZero:(char **)csa startingWith:(int)start {
//
//    for( int counter = start; csa[ counter ] != 0; ++counter )
//        [nsa addObject:[NSString stringWithCString:csa[ counter ] encoding:NSASCIIStringEncoding]];
//    }

//######## Loading the next-to-last file loaded:

- (IBAction) loadNextToLastFile:(id)sender {

    if( strlen( nextToLastFileLoadedPathAsCString ) > 0 ) {
        [self setForEvaluating];
        [self tellEvaluator:C_INPUT_READY
                   withLong:0
                 withString:[[NSString stringWithFormat: @"(load \"%s\")\n", nextToLastFileLoadedPathAsCString]
                             cStringUsingEncoding:NSASCIIStringEncoding]];
        }
    }
        
//########  Load a recently-loaded file.

- (void) loadRecentFileNo:(id)sender {

    NSInteger i = [sender tag];

    [self setForEvaluating];
    [self tellEvaluator:C_INPUT_READY
               withLong:0
             withString:[[NSString stringWithFormat: @"(load \"%s\")\n",
                                                     [[recentFiles objectAtIndex:i] cStringUsingEncoding:NSASCIIStringEncoding]]
                          cStringUsingEncoding:NSASCIIStringEncoding]];
    }

//######## Sheet for handling non-local errors:

- (void) localError:(NSString *)message {
    NSBeginAlertSheet(
        message,
        @"Okay", NULL, NULL,
        myParentWindow,
        self,
        NULL,
        NULL,
        NULL,
        @"",
        NULL
        );
    }

//########  Wrapper to set a new numeric value in the given view.

- (void) newViewNumber:(long)newNumber forView:(NSTextField *)theView {
    [theView setStringValue:[NSString stringWithFormat:@"%ld", newNumber]];
    }

//########  Wrapper to set a new numeric value in the given view, in parentheses.

- (void) newViewParenthesizedNumber:(long)newNumber forView:(NSTextField *)theView {
    [theView setStringValue:[NSString stringWithFormat:@"(%ld)", newNumber]];
    }

//########  Wrapper to set a new string value in the given view.

- (void) newViewString:(const char *)newString forView:(NSTextField *)theView {
    [theView setStringValue:[NSString stringWithCString:newString encoding:NSASCIIStringEncoding]];
    }

//######## For the background color-chooser:  open, cancel, try, accept, accept preference, decline preference.

- (void) openBackgroundColorChooser:(id)sender {

    if( savedBackgroundColor )
        [savedBackgroundColor release];
    savedBackgroundColor = [[mWController backgroundColor] retain];
    [theSharedColorPanel setShowsAlpha:FALSE];
    [myBackgroundColorChooserColorWell activate:TRUE];
    [myBackgroundColorChooser makeKeyAndOrderFront:sender];
    }

//######## Opening the Wraith Scheme dictionary.

- (IBAction) openDictionary:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sWraith Scheme.app/Contents/Resources/English.lproj/Wraith Scheme Help/Wraith Scheme Dictionary.html\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Wraith Scheme could not open the Wraith Scheme Dictionary with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }


//######## Loading a file: Open and try to do it.

- (IBAction) openForLoad:(id)sender {
    static NSString *fileToLoad = nil;
    NSInteger result;
    const char *theString = 0;
    NSOpenPanel *op = [NSOpenPanel openPanel];

    [op setTitle:@"Load What File?"];
    [op setMessage:@"Pick a file for Wraith Scheme to load and process."];
    result = [op runModalForDirectory:nil file:nil types:nil];
    if (result == NSOKButton) {
        fileToLoad = [[op filenames] objectAtIndex:0];
        theString = [fileToLoad cStringUsingEncoding:NSASCIIStringEncoding];
        if( theString && (strlen( theString ) > 0 ) ) {
            [self setForEvaluating];
            [self tellEvaluator:C_INPUT_READY
                       withLong:0
                     withString:[[NSString stringWithFormat: @"(load \"%s\")\n", theString]
                                 cStringUsingEncoding:NSASCIIStringEncoding]];
            }
        else
            [self localError:@"\nThe path (including the file name) to the file to load includes some non-ASCII characters.\n\nScheme is supposed to use only ASCII characters.  They are pretty much the \"regular\" keyboard characters, entered without the \"command-\" and \"option-\" modifier keys.\n\nTo load the file you selected, you must change its name, or the names of the directories in the path to the file, or both, to contain only ASCII characters."];
        }
    }
    
//######## Loading a world:  Try it.

- (IBAction) openForWorldLoad:(id)sender {
    static NSString *worldToLoad = nil;
    NSInteger result;
    const char *theString = 0;
    NSOpenPanel *op = [NSOpenPanel openPanel];

    [op setTitle:@"Load What World?"];
    [op setMessage:@"Choose a saved world for Wraith Scheme to use.\n\nCAUTION: World loads overwrite all of Wraith Scheme's memory:\nThat is, you are about to lose any work that you have not saved."];
    result = [op runModalForDirectory:nil file:nil types:nil];
    if (result == NSOKButton) {
        worldToLoad = [[op filenames] objectAtIndex:0];
        theString = [worldToLoad cStringUsingEncoding:NSASCIIStringEncoding];
        if( theString && (strlen( theString ) > 0 ) ) {
            [self setForEvaluating];
            [self tellEvaluator:C_INPUT_READY
                       withLong:0
                     withString:[[NSString stringWithFormat: @"(e::load-world! \"%s\")\n", theString]
                                 cStringUsingEncoding:NSASCIIStringEncoding]];
            }
        else
            [self localError:@"\nThe path (including the file name) to the world file to load includes some non-ASCII characters.\n\nScheme is supposed to use only ASCII characters.  They are pretty much the \"regular\" keyboard characters, entered without the \"command-\" and \"option-\" modifier keys.\n\nTo load the world file you selected, you must change its name, or the names of the directories in the path to the file, or both, to contain only ASCII characters."];
        }
    }
    
//######## Saving a world: Try it

- (IBAction) openForWorldSave:(id)sender {
    static NSString *worldToSave = nil;
    NSInteger result;
    const char *theString = 0;
    NSSavePanel *sp = [NSSavePanel savePanel];

    [sp setTitle:@"Save as What World File?"];
    [sp setMessage:@"Provide the name of a world file to contain all of Wraith Scheme's memory."];
    result = [sp runModalForDirectory:nil file:nil];
    if (result == NSOKButton) {
        worldToSave = [sp filename];
        theString = [worldToSave cStringUsingEncoding:NSASCIIStringEncoding];
        if( theString && (strlen( theString ) > 0 ) ) {
            [self setForEvaluating];
            [self tellEvaluator:C_INPUT_READY
                       withLong:0
                     withString:[[NSString stringWithFormat: @"(e::save-world \"%s\")\n", theString]
                                 cStringUsingEncoding:NSASCIIStringEncoding]];
            }
        else
            [self localError:@"\nThe path (including the file name) to the file in which to save the world includes some non-ASCII characters.\n\nScheme is supposed to use only ASCII characters.  They are pretty much the \"regular\" keyboard characters, entered without the \"command-\" and \"option-\" modifier keys.\n\nPlease use the \"Save World...\" menu item again, and choose a path and enter a file name that contain only ASCII characters."];
        }
    }

//######## Opening GNU General Public License information.

- (IBAction) openGNU:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sWraith Scheme.app/Contents/Resources/English.lproj/GNUEtcetera.txt\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Wraith Scheme could not open the GNU warranty and license file with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

//######## Opening the Wraith Scheme help file in a browser.

- (IBAction) openHelpFile:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sWraith Scheme.app/Contents/Resources/English.lproj/Wraith Scheme Help/Wraith Scheme Help.html\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Wraith Scheme could not open the Wraith Scheme Help File with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

//######## Opening the Wraith Scheme Internals file.

- (IBAction) openInternalsFile:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sWraith Scheme.app/Contents/Resources/English.lproj/Wraith Scheme Help/Wraith Scheme Internals.html\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Wraith Scheme could not open the Wraith Scheme Internals file with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

//######## Opening the Wraith Scheme README file.

- (IBAction) openReadMe:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sWraith Scheme.app/Contents/Resources/English.lproj/README.txt\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Wraith Scheme could not open the Wraith Scheme \"README.txt\" file with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

//########  Opening various files of source code that are built-in to Wraith Scheme.

//########  Easy to add more.  I could probably store the file name to be loaded in connection
//########  with the menu item that accesses code, and write one procedure to do it all.

- (IBAction) openSourceCode1:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sWraith Scheme.app/Contents/Resources/English.lproj/SampleSourceCode/Checkbook.txt\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Wraith Scheme could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

- (IBAction) openSourceCode2:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sWraith Scheme.app/Contents/Resources/English.lproj/SampleSourceCode/FunctionalProgramming.txt\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Wraith Scheme could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

- (IBAction) openSourceCode3:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sWraith Scheme.app/Contents/Resources/English.lproj/SampleSourceCode/ByAnyOtherName.txt\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Wraith Scheme could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

- (IBAction) openSourceCode4:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sWraith Scheme.app/Contents/Resources/English.lproj/SampleSourceCode/ShellEmulator.txt\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Wraith Scheme could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

- (IBAction) openSourceCode5:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sWraith Scheme.app/Contents/Resources/Demo.txt\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Wraith Scheme could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

- (IBAction) openSourceCode6:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sWraith Scheme.app/Contents/Resources/English.lproj/SampleSourceCode/MMapDemo.c++\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Wraith Scheme could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

- (IBAction) openSourceCode7:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sWraith Scheme.app/Contents/Resources/English.lproj/SampleSourceCode/MMapInterruptDemo.c++\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Wraith Scheme could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }


- (IBAction) openSourceCode8:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sWraith Scheme.app/Contents/Resources/English.lproj/SampleSourceCode/SocketClient.c++\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Wraith Scheme could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }


- (IBAction) openSourceCode9:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sWraith Scheme.app/Contents/Resources/English.lproj/SampleSourceCode/SocketServer.c++\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Wraith Scheme could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }


- (IBAction) openSourceCode10:(id)sender {
    char buffer[ MAXPATHLEN + 300 ];

    sprintf( buffer, "open -a \"Safari\" \"%sWraith Scheme.app/Contents/Resources/English.lproj/SampleSourceCode/SenseLightDemonstrations.txt\"", InitFilePath );
    if( system( buffer ) != 0 ) {
        [self localError:@"Wraith Scheme could not open this file of Scheme source code examples with Safari, and hasn't a clue what the problem was ... ?"];
        return;
        }
    system("osascript -e 'tell app \"Safari\" to activate';");
    }

//######## For the text color-chooser:  open, cancel, try, accept, accept preference, decline preference.

- (void) openTextColorChooser:(id)sender {

    if( savedTextColor )
        [savedTextColor release];
    savedTextColor = [[mWController textColor] retain];
    [theSharedColorPanel setShowsAlpha:FALSE];
    [myTextColorChooserColorWell activate:TRUE];
    [myTextColorChooser makeKeyAndOrderFront:sender];
    }

//########  Take action when a push button gets pushed (but not the Big Red Button).

- (IBAction) pushButtonPushed:(id)sender { [self runInterruptLocallyForPushbutton:[sender tag ]]; }

//######## quitFinal -- to be called from quitSheetDidEnd, with a timing delay ...

- (void) quitFinal { [NSApp terminate:self]; }

//######## Canonical quit sheet did end handler.

- (void) quitSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo {
    const char *theString = 0;

    if( returnCode == NSAlertDefaultReturn ) {    // Quit.
        waitingToExit = 1;
        [mWController showIconView:FALSE];
        [self turnLightsOff];
        makeKittensExit();
        cleanUpForExit();
        increaseBlocking();
        if( ! isMomCatP )
            while( GCInProgressFlag )
                usleep( 1000 );
        [mWController winkIconView];
        [self performSelector:@selector( quitFinal ) withObject:nil afterDelay:0.3];
        }
    else if( returnCode == NSAlertOtherReturn ) { // Cancel.
        waitingToExit = 0;
        [mWController showIconView:FALSE];
        [sheet orderOut:nil];
        }
    else { // NSAlertAlternateReturn              // Save World.
        NSInteger result;
        NSSavePanel *sp = [NSSavePanel savePanel];
        
        [sp setTitle:@"Save as What World File?"];
        [sp setMessage:@"Provide the name of a world file to contain all of Wraith Scheme's memory."];
        result = [sp runModalForDirectory:nil file:nil];
        if( result == NSOKButton ) {
            if( [[sp filename] canBeConvertedToEncoding:NSASCIIStringEncoding] ) {
                theString = [[sp filename] cStringUsingEncoding:NSASCIIStringEncoding];
                if( theString && (strlen( theString ) > 0 ) ) {
                    waitingToExit = 1;    // Tested when evaluator reports "E_WORLD_SAVE_SUCCEEDED"
                    [sheet orderOut:nil];
                    [self setForEvaluating];
                    [self tellEvaluator:C_INPUT_READY
                               withLong:0
                             withString:[[NSString stringWithFormat: @"(e::save-world \"%s\")\n", theString]
                                         cStringUsingEncoding:NSASCIIStringEncoding]];
                    }
                // else nothing to do
                }
            else {
                waitingToExit = 0;
                [sheet orderOut:nil];
                [self localError:@"\nThe path (including the file name) to the file in which to save the world includes some non-ASCII characters.\n\nScheme is supposed to use only ASCII characters.  They are pretty much the \"regular\" keyboard characters, entered without the \"command-\" and \"option-\" modifier keys.\n\nPlease try again, and choose a path and enter a file name that contain only ASCII characters."];
                }
            }
        [mWController showIconView:FALSE];
        }
    }
        
//########  Low-level method for changing the color of a particular pixel of an image.

- (NSImage *) replaceColorAtPixelX:(SInt32)i y:(SInt32)j inImage:(NSImage *)anImage withColor:(NSColor *)newColor {

    //  I should really make this method a function, --it has nothing to do with the class it belongs to --
    //  but this morning I am feeling lazy, so ...
    //
    //  Anyhow, the idea here is to change the color of large portions of an image that all happen to be
    //  the same color to begin with; in particular, the icon for Wraith Scheme is a gray cat face, and
    //  there are times when, for the sake of whimsy, I would like to make a temporary image in which
    //  the cat's face is some other color.
    //
    //  In the calling interface, "anImage" is the image to be so treated, "newColor" is the "other"
    //  color in the discussion above, and "i" and "j" are pixel coordinates within "anImage" of a
    //  pixel that happens to be the color you would like to replace.  (There will be many such pixels;
    //  any one will do to identify the color in question.)  Thus if the goal is to make a pink cat face
    //  instead of a gray one, "anImage" would be the original gray-faced cat, "newColor" would be
    //  some variety of pink, and "i" and "j" would be the pixel coordinates in "anImage" of a pixel
    //  that happened to be gray.
    //
    //  The algorithm works like this:
    //
    //    (1) Get hold of a bit-map image representation for the image in question.
    //    (2) Copy it.
    //    (3) Using the color at the ( i, j ) pixel in the original image as a reference, loop
    //        through the copy and change the colors of all of its pixels that need changing.
    //    (4) Alloc and init-with-size a brand new image, of the same size as the original image.
    //    (5) Add the modified, copied representation to the new image.
    //    (6) Return the new image.
    //  
    //  This method does not release the old image, but it does retain the new one before returning it.
    //
    //  There is one gotcha; I have never known for sure whether I should automagically expect any random
    //  image to have a bit-map image representation attached to it by default, and I have no idea how
    //  to get an image to create a bit-map image representation of its contents if it doesn't have one. 
    //  I haven't encountered an image which did not have a bit-map image representation of its contents
    //  already, but I am way too paranoid to count on one being there.  So what I do herein is look for a 
    //  bit-map image representation among the image representations of the given image, and copy it,
    //  et cetera, if I find one.  If I do not find one, I return the original image unaltered, and in
    //  that case, e.g., the cat face remains gray instead of turning pink.  That will suffice to escape
    //  from the error condition of "no bit-map image representation to be found", for my application.

    NSArray *theRepresentations = nil;
    NSBitmapImageRep *theBMIR = nil;              // "BMIR" stands for "bit-map image representation.
    NSImage *newImage = nil;
    NSBitmapImageRep *newBMIR = nil;
    NSColor *oldColor = nil;
    NSInteger width = 0;
    NSInteger height = 0;
    UInt32 k = 0; 
    SInt32 l = 0;
    SInt32 m = 0;
    NSSize aSize;

    theRepresentations = [anImage representations];

    for( k = 0; k < [theRepresentations count]; ++k )
        if( strcmp( [[[theRepresentations objectAtIndex:k] className] cStringUsingEncoding:NSASCIIStringEncoding],
                    "NSBitmapImageRep")          // There's a better way to do this but I can never remember it.
               == 0 ) {
            theBMIR = [[theRepresentations objectAtIndex:k] retain];
            break;
            }
    if( theBMIR == nil )                         // Error condition ...
        return [anImage retain];                 //   ... this will do in a pinch

    height = [theBMIR pixelsHigh];
    width  = [theBMIR pixelsWide];

    aSize.height = height;
    aSize.width  = width;

    newBMIR = [theBMIR copyWithZone:nil];                         // "copyWithZone" does an implicit retain.
    newImage = [[[NSImage alloc] initWithSize:aSize] retain];     // Note: newImage retained here.

    oldColor = [theBMIR colorAtX:i y:j];
        
    for( l = 0; l < width; ++l )
        for( m = 0; m < height; ++m )
            if( [[theBMIR colorAtX:l y:m] isEqual:oldColor] )
                [newBMIR setColor:newColor atX:l y:m];

    [newImage addRepresentation:newBMIR];
    [newBMIR release];                                            // Balances the retain done by "copyWithZone".
    [newImage setScalesWhenResized:YES];
 
    return newImage;                                              // newImage is already retained.
    }

//######## Sheet display for reporting fatal error.

- (void) reportDisasterAndQuit:(const char *)errorMessage {

    wantsToQuit = true;
    [mWController showIconView:TRUE];
    [self turnLightsOn];
    [mWController playDisasterMeow:self];
    NSBeginAlertSheet(
        @"Wraith Scheme must quit because:",
        @"Sorry!", NULL, NULL,
        myParentWindow,
        self,
        @selector( reportDisasterAndQuitDidEnd:returnCode:contextInfo: ),
        NULL,
        NULL,
        [NSString stringWithCString:errorMessage encoding:NSASCIIStringEncoding],
        NULL
        );
    }

//######## Sheet did end handler for reporting fatal error.

- (void) reportDisasterAndQuitDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo {
    [mWController showIconView:FALSE];
    [self turnLightsOff];
    [sheet orderOut:nil];
    makeKittensExit();
    cleanUpForExit();
    [NSApp terminate:self];
    }

//######## Sheet display for reporting fatal error in early initialization.

- (void) reportEarlyDisasterAndQuit {

    if( wantsToQuit ) {
        usleep( 15000000 );
        makeKittensExit();
        exit( 1 );
        }

    wantsToQuit = true;
    [mWController showIconView:TRUE];
    [mWController playDisasterMeow:self];
    NSBeginAlertSheet(
        @"Wraith Scheme must quit because of an implementation error while starting up:",
        @"Sorry!", NULL, NULL,
        myParentWindow,
        self,
        @selector( reportEarlyDisasterAndQuitDidEnd:returnCode:contextInfo: ),
        NULL,
        NULL,
        @"This error is not your fault.  Please report it to Jay_Reynolds_Freeman@mac.com.  For details, see the end of the console log file, somewhere in the \"Console\" folder of the \"Logs\" folder of the \"Library\" folder of your main disk drive.  Thank you.\n\nWraith Scheme's normal error-handling mechanism isn't running yet.",
        NULL
        );
    }

//######## Sheet did end handler for reporting fatal error.

- (void) reportEarlyDisasterAndQuitDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo {
    [mWController showIconView:FALSE];
    [sheet orderOut:nil];
    makeKittensExit();
    exit( 1 );
    }

//########  Resetting:

- (IBAction) requestReset:(id)sender {

    if( evaluatorHasInitialized && (! stopEvaluatingPending ) ) {
        [resetMenuItem     setEnabled:NO];
        stopEvaluatingPending = true;
        [self tellEvaluator:C_PLEASE_STOP_EVALUATING withLong:0 withString:""];
        [inputView         setEditable:YES];
        [responseView      setEditable:NO];
        [inputView appendNewline];
        [inputView showEndRange];
        [self startNormalInput];
        }
    }

//########  Sheet for verifying the user wants to disable console output:

- (void) runDisableOutputSheet:(id)sender {

    NSString *mainAlert = nil;

    if( isMomCatP )
        mainAlert = @"Text output to the Wraith Scheme main window is disabled.";
    else
        mainAlert = @"Text output to this kitten's Wraith Scheme window is disabled.";

    outputEnabled = FALSE;
    [mWController showIconView:TRUE];

    NSBeginAlertSheet(
        mainAlert,
        @"Provide Normal Output", NULL, NULL,
        myParentWindow,
        self,
        @selector( disableOutputSheetDidEnd:returnCode:contextInfo: ),
        NULL,
        NULL,
        @"Text that would have been printed is lost beyond recovery.  Output to files, including transcript files, is not affected.",
        NULL
        );
    }

//########  Sheet for reminding the user how to make a shareware donation.

- (IBAction) runDonationSheet:(id)sender {

    [boldField setHidden:FALSE];
    [normalField setHidden:FALSE];
    [closeButton setHidden:FALSE];
    [donationSheetImageView setImage:wraithFace];
    [donationSheetImageView setAlphaValue:1.0];        

    [NSApp beginSheet: myDonationSheet
       modalForWindow: myParentWindow
        modalDelegate: self
       didEndSelector: @selector(donationSheetDidEnd:returnCode:contextInfo:)
          contextInfo: nil];
    }

//######## Canonical quit sheet.  Details vary depending on which kitten and whether evaluating.

- (IBAction) runQuitSheet:(id)sender {

    SInt32 evaluatingCopy = evaluating;
    NSString *aFortune;
    SInt32 fortuneNumber = (SInt32)((random() % (sizeof( fortunes ) / sizeof( NSString * ))) & WORD_MASK);

    if( (random() % PICK_A_FORTUNE_HOW_OFTEN) == 0 )
        aFortune = fortunes[ fortuneNumber ];
    else
        aFortune = @"";

    if( waitingToExit )
        return;

    [mWController showIconView:TRUE];
    NSBeginAlertSheet(
        ((isMomCatP && theNumberOfKittens > 1)
          ?
          @"Really Quit Wraith Scheme MomCat and Kittens?"
          :
          @"Really Quit Wraith Scheme?"
          ),
        ((evaluatingCopy && isMomCatP) ? @"Just Quit" : @"Quit"), 
        (evaluatingCopy ? NULL : ((! isMomCatP) ? NULL : @"Save World, Then Quit") ),
        @"Cancel",
        myParentWindow,
        self,
        @selector( quitSheetDidEnd:returnCode:contextInfo: ),
        NULL,
        NULL,
        (evaluatingCopy ?
          (isMomCatP ?
            @"\nTo save a world first, press \"cancel\", wait till Wraith Scheme stops evaluating (or reset it), and try again."
            :
            @"")
          :
          (isMomCatP ?
            aFortune
            :
            @"\nIf garbage collection is in progress, the kitten will wait till it has finished before quitting.")),
        NULL
        );
    }
    
//########  Code to deal with a sheet that occasionally appears, wherein the kitty asks for a kitty treat ...

- (IBAction) runTreatSheet:(NSString *)prompt {

    [thePreferences setTreatPreference:YES];
    [thePreferences saveDefaults];

    NSBeginAlertSheet(
        prompt,
        @"Treat",
        @"No Treat",
        NULL,
        myParentWindow,
        self,
        @selector( treatSheetDidEnd:returnCode:contextInfo: ),
        NULL,
        NULL,
        @"",
        NULL
        );
    }
    
- (void) setForEvaluating {

    evaluating = TRUE;
    notEvaluatingHowLongCounter = 0;

    [promptView display];
    [responseView display];

    [resetMenuItem     setEnabled:YES];
    [dIMenuItem        setEnabled:NO];
    [tSMenuItem        setEnabled:YES];
    [lFMenuItem        setEnabled:NO];
    [lRecentFMenuItem  setEnabled:NO];
    [lLFMenuItem       setEnabled:NO];
    [lNLFMenuItem      setEnabled:NO];
    [lLFWUMenuItem     setEnabled:NO];
    [lWMenuItem        setEnabled:NO];
    [sWMenuItem        setEnabled:NO];
    [cDMenuItem        setEnabled:NO];
    [sFPMenuItem       setEnabled:NO];
    [nMMenuItem        setEnabled:YES];
    
    [lLFButton         setEnabled:NO];
    [lLFWUButton       setEnabled:NO];
    [dIButton          setEnabled:NO];
    
    [eBController       turnOn];     

    }

- (void) setForFileLoadBegan {
    if( (! evaluating) && (! gcing) ) {
        if( recentFiles && ([recentFiles count] > 0) )
            [lRecentFMenuItem  setEnabled:YES];
        else
            [lRecentFMenuItem  setEnabled:NO];
        [lLFMenuItem setEnabled:YES];
        [lNLFMenuItem setEnabled:(nextToLastFileLoadedPath != nil)];
        [lLFButton setEnabled:YES];
        [lLFWUMenuItem setEnabled:YES];
        [lLFWUButton setEnabled:YES];
        }
    }
    
- (void) setForFileLoadFailed {
    if( recentFiles && ([recentFiles count] > 0) )
        [lRecentFMenuItem  setEnabled:YES];
    else
        [lRecentFMenuItem  setEnabled:NO];
    [lLFMenuItem setEnabled:NO];
    [lNLFMenuItem setEnabled:(nextToLastFileLoadedPath != nil)];
    [lLFButton setEnabled:NO];
    [lLFWUMenuItem setEnabled:NO];
    [lLFWUButton setEnabled:NO];
    }
    
- (void) setForGCing {

    gcing = TRUE;
    notEvaluatingHowLongCounter = 0;

    [resetMenuItem     setEnabled:YES];
    [dIMenuItem        setEnabled:NO];
    [tSMenuItem        setEnabled:NO];
    [lFMenuItem        setEnabled:NO];
    [lRecentFMenuItem  setEnabled:NO];
    [lLFMenuItem       setEnabled:NO];
    [lNLFMenuItem      setEnabled:NO];
    [lLFWUMenuItem     setEnabled:NO];
    [lWMenuItem        setEnabled:NO];
    [sWMenuItem        setEnabled:NO];
    [cDMenuItem        setEnabled:NO];
    [sFPMenuItem       setEnabled:NO];
    [nMMenuItem        setEnabled:NO];
    
    [lLFButton         setEnabled:NO];
    [lLFWUButton       setEnabled:NO];
    [dIButton          setEnabled:NO];
    
    [gCBController     turnOn];     
    }

- (void) setForNostalgiaMode {
    [self setNostalgiaLights];
//    [mySneakyWindow reveal];
    [mWController setNostalgiaMode:TRUE];
    [nBController turnOn];
    [nMMenuItem setState:NSOnState];
    [self adjustLights];
    nostalgiaMode = TRUE;
    }

- (void) setForNotEvaluating {

    evaluating = FALSE;

    stopEvaluatingPending = false;

    [promptView          display];
    [responseView        display];

    [resetMenuItem     setEnabled:YES];
    [dIMenuItem        setEnabled:YES];
    [tSMenuItem        setEnabled:YES];
    [lFMenuItem        setEnabled:YES];
    if( recentFiles && ([recentFiles count] > 0) )
        [lRecentFMenuItem  setEnabled:YES];
    else
        [lRecentFMenuItem  setEnabled:NO];
    [lLFMenuItem       setEnabled:YES];
    [lNLFMenuItem      setEnabled:(nextToLastFileLoadedPath != nil)];
    [lLFWUMenuItem     setEnabled:YES];

    if( ! isMomCatP ) {
        [lWMenuItem        setEnabled:NO];
        [sWMenuItem        setEnabled:NO];
        }
    else {
        [lWMenuItem        setEnabled:YES];
        [sWMenuItem        setEnabled:YES];
        }

    [cDMenuItem        setEnabled:YES];
    [sFPMenuItem       setEnabled:YES];
    [nMMenuItem        setEnabled:YES];
    
    [lLFButton         setEnabled:YES];
    [lLFWUButton       setEnabled:YES];
    [dIButton          setEnabled:YES];
    
    // The next "if" overwrites some of the above -- just defensive programming.

    if( lastFileLoadBegan )
        [self setForFileLoadBegan];
    else
        [self setForFileLoadFailed];

    [eBController      turnOff];     
    }

- (void) setForNotGCing {

    gcing = FALSE;
    
    [promptView display];
    [responseView display];

    [gCBController       turnOff];     

    if( evaluating )
        [self setForEvaluating];
    else
        [self setForNotEvaluating];
    }

- (void) setForNotNostalgiaMode {
    [self setNormalLights];
//    [mySneakyWindow conceal];
    [mWController setNostalgiaMode:FALSE];
    [nBController turnOff];
    [nMMenuItem setState:NSOffState];
    [self adjustLights];
    nostalgiaMode = FALSE;
    }

//########  Set preferences that don't involve using the evaluator.  Called during initialization.
//########  Preferences involving the evaluator are set one-at-a-time, during initialization timer events.

- (void) setNonEvaluatorPreferences {

    if( [thePreferences objectForKey:NostalgiaMode] && ([[thePreferences objectForKey:NostalgiaMode] boolValue] != 0) )
        [self setForNostalgiaMode];
    else
        [self setForNotNostalgiaMode];

    if( [thePreferences objectForKey:LoadLastFile] && ([[thePreferences objectForKey:LoadLastFile] boolValue] != 0) ) {
        [lOUBController turnOn];
        [lLFWUMenuItem setState:NSOnState];
        [lLFWUButton setState:NSOnState];
        loadingLastFileWhenUpdated = true;
        }
    else {
        [lOUBController turnOff];
        [lLFWUMenuItem setState:NSOffState];
        [lLFWUButton setState:NSOffState];
        loadingLastFileWhenUpdated = false;
        }

    if( [thePreferences objectForKey:ShowBasicButtons] && ([[thePreferences objectForKey:ShowBasicButtons] boolValue] != 0) )
        [theDrawerController presentBasicButtonsDrawer];
    else
        [theDrawerController concealBasicButtonsDrawer];
        
    if( [thePreferences objectForKey:ShowInstrumentPanel] && ([[thePreferences objectForKey:ShowInstrumentPanel] boolValue] != 0) )
        [theDrawerController presentInstrumentPanelDrawer];
    else
        [theDrawerController concealInstrumentPanelDrawer];

    if( [thePreferences objectForKey:PathToInitialLoad] != nil )
        [lastFileLoadedView setStringValue:[thePreferences objectForKey:PathToInitialLoad]];
    else
        [lastFileLoadedView setStringValue:@""];

    }

- (void) setNormalLights {

    [cDBController setOnImage:[NSImage imageNamed:@"OrangeLight.png"]];
    [cDBController setAlternateOnImage:[NSImage imageNamed:@"OrangeLightAlternate.png"]];
    [cDBController setOffImage:[NSImage imageNamed:@"OffLight.png"]];
    [eBController setOnImage:[NSImage imageNamed:@"MagentaLight.png"]];
    [eBController setAlternateOnImage:[NSImage imageNamed:@"MagentaLightAlternate.png"]];
    [eBController setOffImage:[NSImage imageNamed:@"OffLight.png"]];
    [gCBController setOnImage:[NSImage imageNamed:@"RedLight.png"]];
    [gCBController setAlternateOnImage:[NSImage imageNamed:@"RedLightAlternate.png"]];
    [gCBController setOffImage:[NSImage imageNamed:@"OffLight.png"]];
    [lOUBController setOnImage:[NSImage imageNamed:@"YellowLight.png"]];
    [lOUBController setAlternateOnImage:[NSImage imageNamed:@"YellowLightAlternate.png"]];
    [lOUBController setOffImage:[NSImage imageNamed:@"OffLight.png"]];
    [mLIBController setOnImage:[NSImage imageNamed:@"YellowLight.png"]];
    [mLIBController setAlternateOnImage:[NSImage imageNamed:@"YellowLightAlternate.png"]];
    [mLIBController setOffImage:[NSImage imageNamed:@"OffLight.png"]];
    [nBController setOnImage:[NSImage imageNamed:@"GreenLight.png"]];
    [nBController setOffImage:[NSImage imageNamed:@"OffLight.png"]];
    [sFPBController setOnImage:[NSImage imageNamed:@"CyanLight.png"]];
    [sFPBController setAlternateOnImage:[NSImage imageNamed:@"CyanLightAlternate.png"]];
    [sFPBController setOffImage:[NSImage imageNamed:@"OffLight.png"]];
    [sBController setOnImage:[NSImage imageNamed:@"CyanLight.png"]];
    [sBController setOffImage:[NSImage imageNamed:@"OffLight.png"]];
    }

- (void) setNostalgiaLights {

    [cDBController setOnImage:[NSImage imageNamed:@"SimpleOnLight.png"]];
    [cDBController setAlternateOnImage:[NSImage imageNamed:@"SimpleOnAlternateLight.png"]];
    [cDBController setOffImage:[NSImage imageNamed:@"SimpleOffLight.png"]];
    [eBController setOnImage:[NSImage imageNamed:@"SimpleOnLight.png"]];
    [eBController setAlternateOnImage:[NSImage imageNamed:@"SimpleOnAlternateLight.png"]];
    [eBController setOffImage:[NSImage imageNamed:@"SimpleOffLight.png"]];
    [gCBController setOnImage:[NSImage imageNamed:@"SimpleOnLight.png"]];
    [gCBController setAlternateOnImage:[NSImage imageNamed:@"SimpleOnAlternateLight.png"]];
    [gCBController setOffImage:[NSImage imageNamed:@"SimpleOffLight.png"]];
    [lOUBController setOnImage:[NSImage imageNamed:@"SimpleOnLight.png"]];
    [lOUBController setAlternateOnImage:[NSImage imageNamed:@"SimpleOnAlternateLight.png"]];
    [lOUBController setOffImage:[NSImage imageNamed:@"SimpleOffLight.png"]];
    [mLIBController setOnImage:[NSImage imageNamed:@"SimpleOnLight.png"]];
    [mLIBController setAlternateOnImage:[NSImage imageNamed:@"SimpleOnAlternateLight.png"]];
    [mLIBController setOffImage:[NSImage imageNamed:@"SimpleOffLight.png"]];
    [nBController setOnImage:[NSImage imageNamed:@"SimpleOnLight.png"]];
    [nBController setOffImage:[NSImage imageNamed:@"SimpleOffLight.png"]];
    [sFPBController setOnImage:[NSImage imageNamed:@"SimpleOnLight.png"]];
    [sFPBController setAlternateOnImage:[NSImage imageNamed:@"SimpleOnAlternateLight.png"]];
    [sFPBController setOffImage:[NSImage imageNamed:@"SimpleOffLight.png"]];
    [sBController setOnImage:[NSImage imageNamed:@"SimpleOnLight.png"]];
    [sBController setOffImage:[NSImage imageNamed:@"SimpleOffLight.png"]];
    }

//########  Set up the menu that brings this or that kitten's window to the front.

- (void) setUpKittenFrontSubMenu:(SInt32)nKittens {
    
    NSMenuItem *aMenuItem = 0;
    SInt32 i = 0;

    if( nKittens <= 1 ) {
        [kittenFrontMenuItem setEnabled:NO];
        [kittensAllFrontMenuItem setEnabled:NO];
        return;
        }

    [kittenFrontMenuItem setEnabled:YES];
    [kittensAllFrontMenuItem setEnabled:YES];

    // Install new items.  Zero is a special case -- the MomCat.

    for( i = 0; i < nKittens; ++i ) {
        if( i == 0 ) {
            aMenuItem = [[NSMenuItem alloc]
                             initWithTitle:@"Momcat"
                             action:@selector(arrangeKittenInFront:)
                             keyEquivalent:@""];
            [aMenuItem setTarget:self];
            [aMenuItem setTag:i];
            [aMenuItem setToolTip:@"Bring the MomCat's Wraith Scheme window to the front."];
            [kittenFrontSubMenu insertItem:aMenuItem atIndex:i];
            }
        else {
            aMenuItem = [[NSMenuItem alloc]
                             initWithTitle:[NSString stringWithFormat:@"Kitten %d", i]
                             action:@selector(arrangeKittenInFront:)
                             keyEquivalent:@""];
            [aMenuItem setTarget:self];
            [aMenuItem setTag:i];
            [aMenuItem setToolTip:[NSString stringWithFormat:@"Bring Kitten %d's Wraith Scheme window to the front.", i]];
            [kittenFrontSubMenu insertItem:aMenuItem atIndex:i];
            }
        }
    }

//########  This next is called by a timer that asks the controller to do something.
//########  It does something, then fires another timer -- unless the application is quitting.

- (void) singleControllerAction:(NSTimer*)theTimer {

    [self performSelectorOnMainThread:@selector(singleControllerActionGuts) withObject:nil waitUntilDone:TRUE];

    if( ! wantsToQuit )
        [NSTimer scheduledTimerWithTimeInterval  :  STANDARD_TIMER_INTERVAL
                                         target  :  self
                                       selector  :  @selector(singleControllerAction:)
                                       userInfo  :  nil
                                        repeats  :  NO];
    }

//########  This is the guts of what the controller does when a timer fires to ask it to do something.

#define POLL_COUNTER_MAXIMUM 25
#define POLL_DELAY 200                 // microseconds

- (void) singleControllerActionGuts {

    // First, bump some counters:

    if( controllerActionCounter++ == CAC_ROLLOVER )
        controllerActionCounter = 0;

    // As of 2010.06.02, "notEvaluatingHowLongCounter" is not used for anything.

    if( ! evaluating )
        ++notEvaluatingHowLongCounter;
    else
        notEvaluatingHowLongCounter = 0;

    // Next, do a whole lot of things based on those counters.
    // These things are in essence GUI administration or cosmetics.

    if( displayingMessage && (++messageDisplayCounter >= MESSAGE_DISPLAY_COUNT) ) {
        [self tellEvaluator:C_PLEASE_CLEAR_STATUS withLong:0 withString:""];
        displayingMessage = false;
        }

    if( (controllerActionCounter % CAC_COUNT_FOR_MOM_SAYS_QUIT) == 0 )
        if( (theKittenNumber > 0) && momSaysQuit() )
            exit( 0 );

    if( (controllerActionCounter % CAC_COUNT_FOR_LOAD_LAST_FILE) == 0 ) {
        if( loadingLastFileWhenUpdated && (! evaluating) && (! gcing) && (! expectMoreInput) ) {
            if( lastLoadStatBufferIsGood && (stat( lastFileLoadedPathAsCString, &currentStatBuffer ) != -1) ) {
                if( currentStatBuffer.st_mtime > lastLoadStatBuffer.st_mtime ) {
                    lastLoadStatBuffer = currentStatBuffer;
                    [self setForEvaluating];
                    [self tellEvaluator:C_INPUT_READY
                               withLong:0
                             withString:[[NSString stringWithFormat: 
                                         @"(load \"%s\")\n", lastFileLoadedPathAsCString]
                                         cStringUsingEncoding:NSASCIIStringEncoding]];
                    }
                }
            }
        }
    if( evaluating && (! gcing) && ((controllerActionCounter % CAC_COUNT_FOR_TOGGLE_EVALUATING_LIGHT) == 0 ) )
        [eBController toggleBrightness];
    if( evaluating && (gcing) && ((controllerActionCounter % CAC_COUNT_FOR_TOGGLE_EVALUATING_LIGHT_SLOW) == 0 ) )
        [eBController toggleBrightness];
    if( gcing && ((controllerActionCounter % CAC_COUNT_FOR_TOGGLE_GC_LIGHT) == 0 ) )
        [gCBController toggleBrightness];
    if( (controllerActionCounter % CAC_COUNT_FOR_TOGGLE_CD_LIGHT) == 0 )
        [cDBController toggleBrightness];
    if( (controllerActionCounter % CAC_COUNT_FOR_TOGGLE_LOU_LIGHT) == 0 )
        [lOUBController toggleBrightness];
    if( (controllerActionCounter % CAC_COUNT_FOR_TOGGLE_MLI_LIGHT) == 0 )
        [mLIBController toggleBrightness];
    if( (controllerActionCounter % CAC_COUNT_FOR_TOGGLE_SFP_LIGHT) == 0 )
        [sFPBController toggleBrightness];
    if( (controllerActionCounter % CAC_COUNT_FOR_FADE_EXPECT_VIEW) == 0 ) {
        if( showingExpectView ) {
            if( ++expectOnCount >= 2) {
                [NSAnimationContext beginGrouping];
                [[NSAnimationContext currentContext] setDuration:0.7]; 
                [[expectView animator] setAlphaValue:0.0];
                [NSAnimationContext endGrouping];
                expectOnCount = 0;
                showingExpectView = false;
                }
            }
        else {
            [NSAnimationContext beginGrouping];
            [[NSAnimationContext currentContext] setDuration:0.7]; 
            [[expectView animator] setAlphaValue:1.0];
            [NSAnimationContext endGrouping];
            showingExpectView = true;
            }
        }
    if( (controllerActionCounter % CAC_COUNT_FOR_TOGGLE_WHICH_GC_COUNT_WINDOW) == 0 ) {
        if( (showingGenerationalInformation == false) && usingGenerationalGC ) {
            [NSAnimationContext beginGrouping];
            [[NSAnimationContext currentContext] setDuration:0.7]; 
            [[numberOfGCsView animator] setAlphaValue:0.0];
            [[numberOfGenerationalGCsLabelView animator] setAlphaValue:1.0];
            [[numberOfGenerationalGCsView animator] setAlphaValue:1.0];
            [[memorySizeView animator] setAlphaValue:0.0];
            [[generationalMemorySizeView animator] setAlphaValue:1.0];
            [[memoryInUseView animator] setAlphaValue:0.0];
            [[generationalMemoryInUseView animator] setAlphaValue:1.0];
            [NSAnimationContext endGrouping];
            showingGenerationalInformation = true;
            }
        else {
            [NSAnimationContext beginGrouping];
            [[NSAnimationContext currentContext] setDuration:0.7]; 
            [[numberOfGenerationalGCsView animator] setAlphaValue:0.0];
            [[numberOfGenerationalGCsLabelView animator] setAlphaValue:0.0];
            [[numberOfGCsView animator] setAlphaValue:1.0];
            [[generationalMemorySizeView animator] setAlphaValue:0.0];
            [[memorySizeView animator] setAlphaValue:1.0];
            [[generationalMemoryInUseView animator] setAlphaValue:0.0];
            [[memoryInUseView animator] setAlphaValue:1.0];
            [NSAnimationContext endGrouping];
            showingGenerationalInformation = false;
            }
        }
    if( getFrontKitten( theKittenNumber ) ) {
        setFrontKitten( theKittenNumber, 0 );
        if( [myParentWindow isMiniaturized] )
            [myParentWindow deminiaturize:self];
        [NSApp activateIgnoringOtherApps:YES];
        }
    }

- (NSArray *)spellingCompletionsForPartialWord:(NSString *)partWord {

    NSPredicate *aPredicate = [NSPredicate predicateWithFormat:@"SELF beginswith[c] %@", partWord ];

//    return [symbolsInObList filteredArrayUsingPredicate:aPredicate];
    return [symbolsInWraithSchemeDictionary filteredArrayUsingPredicate:aPredicate];
    }

//########  Take action when a slider's position has changed:

- (IBAction) sliderDidChange:(id)sender {
    
    sliderValues[ [sender tag ] ] = [sender intValue];
    [self sliderValueUpdateForTag:[sender tag] toValue:[sender intValue]];
    }

- (void) sliderUpdateForTag:(NSInteger)tag toValue:(int)value {

    switch( tag ) {
        case 0: [slider0 setIntValue:value]; break;
        case 1: [slider1 setIntValue:value]; break;
        case 2: [slider2 setIntValue:value]; break;
        case 3: [slider3 setIntValue:value]; break;
        }
    }

//########  Take action when a slider's value has changed:

- (IBAction) sliderValueDidChange:(id)sender {

    sliderValues[ [sender tag ] ] = [sender intValue];
    [self sliderUpdateForTag:[sender tag] toValue:[sender intValue]];
    }

- (void) sliderValueUpdateForTag:(NSInteger)tag toValue:(int)value {

    switch( tag ) {
        case 0: [slider0Value setIntValue:value]; break;
        case 1: [slider1Value setIntValue:value]; break;
        case 2: [slider2Value setIntValue:value]; break;
        case 3: [slider3Value setIntValue:value]; break;
        }
    }

//########  Start the controller -- fire the first of a never-ending series of timers:

- (void) startController {

    notEvaluatingHowLongCounter = 0;
    messageDisplayCounter = 0;
    [NSTimer scheduledTimerWithTimeInterval  :  STANDARD_TIMER_INTERVAL 
                                     target  :  self
                                   selector  :  @selector(singleControllerAction:) 
                                   userInfo  :  nil
                                    repeats  :  NO];
    }

//########  Get a dialog going:

- (void) startDialog:(NSString *)thePrompt {

    [mWController      setPrompt:thePrompt];
    [mWController      brightPrompt];
    [mWController      setResponse:@""];
    [mWController      disableMakeInputPanelFirstResponderItem];
    [myParentWindow    makeFirstResponder:responseView];

    [inputView         setEditable:NO];
    [inputView         disableShortcuts];
    [responseView      setEditable:YES];
    [responseView      setSelectable:YES];
    }

//########  Return to getting input after a dialog.  (Also used in resetting.)

- (void) startNormalInput {
    [mWController      enableMakeInputPanelFirstResponderItem];
    [mWController      normalPrompt];
    [inputView         enableShortcuts];
    [myParentWindow    makeFirstResponder:inputView];
    }

//########  Toggle compile-defines flag.

- (IBAction) toggleCompileDefines:(id)sender {
    [self tellEvaluator:([sender state] ? C_PLEASE_DO_NOT_COMPILE_DEFINES : C_PLEASE_COMPILE_DEFINES ) withLong:0 withString:""];
    }

//########  Toggle use-full-numeric-precision flag.

- (IBAction) toggleFullNumericPrecision:(id)sender {
    [self tellEvaluator:([sender state] ? C_PLEASE_USE_LOW_PRECISION : C_PLEASE_USE_HIGH_PRECISION ) withLong:0 withString:""];
    }

//########  Toggle load-last-file-when-updated state.  (Which is not a flag

- (IBAction) toggleLoadLastFileWhenUpdated:(id)sender {

    NSInteger valueBeforeAction = [sender state];
    
    if( [sender classForCoder] == [NSButton class] )   // The button's state changes before this message gets sent.
        if( valueBeforeAction == NSOnState )
            valueBeforeAction = NSOffState;
        else
            valueBeforeAction = NSOnState;

    if( valueBeforeAction == NSOnState ) {
        [lOUBController turnOff];
        [lLFWUMenuItem setState:NSOffState];
        [lLFWUButton setState:NSOffState];
        loadingLastFileWhenUpdated = false;
        }
    else {
        [lOUBController turnOn];
        [lLFWUMenuItem setState:NSOnState];
        [lLFWUButton setState:NSOnState];
        loadingLastFileWhenUpdated = true;
        }
    }

//########  Toggle nostalgia-mode flag.

- (IBAction) toggleNostalgiaMode:(id)sender {
    if( [sender state] == NSOnState )
        [self setForNotNostalgiaMode];
    else
        [self setForNostalgiaMode];
    }

//########  Take action when a toggle switch has changed state.

- (IBAction) toggleSwitchDidChange:(id)sender {

    if( [sender state] == NSOnState )
        senseSwitchPositions |= (1 << [sender tag]);
    else
        senseSwitchPositions &= ~(1 << [sender tag]);
    }

//########  Clear out the treat sheet.

- (void) treatSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo {
    [sheet orderOut:nil];
    if( returnCode != NSAlertDefaultReturn )
        [self runTreatSheet:@"I want a kitty treat!"];
    else {
        [thePreferences setTreatPreference:NO];
        [thePreferences saveDefaults];
        NSBeginAlertSheet(
            @"Oh, wow!  Thank you!",
            @"You're welcome ...",
            NULL,
            NULL,
            myParentWindow,
            self,
            NULL,
            NULL,
            NULL,
            @"",
            NULL
            );
        }
    }

//########  Try the new background color.

- (void) tryBackgroundColorChooser:(id)sender {

    [mWController setBackgroundColor:[myBackgroundColorChooserColorWell color]];
    [mWController setNostalgiaMode:[nBController isOn]];
    }

//########  Try the new text color.

- (void) tryTextColorChooser:(id)sender {

    [mWController setTextColor:[myTextColorChooserColorWell color]];
    [mWController setNostalgiaMode:[nBController isOn]];
    }

//######## Turn on all the status lights.

- (void) turnLightsOn {

    [cDBController turnOn];
    [eBController turnOn];
    [gCBController turnOn];
    [lOUBController turnOn];
    [mLIBController turnOn];
    [nBController turnOn];
    [sFPBController turnOn];
    [sBController turnOn];
    }

//######## Turn off all the status lights.

- (void) turnLightsOff {

    [cDBController turnOff];
    [eBController turnOff];
    [gCBController turnOff];
    [lOUBController turnOff];
    [mLIBController turnOff];
    [nBController turnOff];
    [sFPBController turnOff];
    [sBController turnOff];
    }

//########  Update the progress bar.

- (void)updateInitProgress:( double )newValue {

    [myProgressIndicator setDoubleValue:newValue];
    [myProgressIndicator setHidden:NO];
    [myProgressIndicator display];
    }

//########  Update the list of recently-loaded files, on the basis of a recent load.

- (void) updateLoadRecentMenu {

    if( recentFiles != 0 )
        [recentFiles release];

    recentFiles = [[thePreferences loadRecentPreferences] retain];

    NSInteger nItems = [recentFiles count];
    NSInteger i = 0;
    NSMenuItem *aMenuItem = 0;

    if( nItems == 0 ) {
        [lRecentFMenuItem  setEnabled:NO];
        return;
        }

    // First, remove the old items:

    NSInteger nCurrentItems = [lRecentFSubMenu numberOfItems];
    for( i = nCurrentItems - 1; i >= 0; --i )
        [lRecentFSubMenu removeItemAtIndex:i];

    // Next, install the new items:

    for( i = 0; i < nItems; ++i ) {
        if( [[recentFiles objectAtIndex:i] isEqual:@""] )
            break;
        aMenuItem = [[NSMenuItem alloc]
                         initWithTitle:[[recentFiles objectAtIndex:i] lastPathComponent]
                         action:@selector(loadRecentFileNo:)
                         keyEquivalent:@""];
        [aMenuItem setTarget:self];
        [aMenuItem setTag:i];
        [aMenuItem setToolTip:[recentFiles objectAtIndex:i]];
        [lRecentFSubMenu insertItem:aMenuItem atIndex:i];
        }

    // Finally, if there are any items in the menu, install an item to clear the menu, preceded by a separatorItem.

    if( nItems > 0 ) {

        aMenuItem = [NSMenuItem separatorItem];
        [aMenuItem setTag:i];
        [lRecentFSubMenu insertItem:aMenuItem atIndex:i];
        ++i;

        aMenuItem = [[NSMenuItem alloc]
                         initWithTitle:@"Clear This Submenu"
                         action:@selector(clearRecentFilesWrapper:)
                         keyEquivalent:@""];
        [aMenuItem setTarget:self];
        [aMenuItem setTag:i];
        [aMenuItem setToolTip:@"Remove all file names from this submenu.  (Does nothing to the files themselves.)"];
        [lRecentFSubMenu insertItem:aMenuItem atIndex:i];
        ++i;

        }
    }

//######## Loading a world: Succeeded

- (void) worldLoadSucceeded:(const char *)path {
    [self newViewString:[[[NSString stringWithFormat:@"%s", path] lastPathComponent] cStringUsingEncoding:NSASCIIStringEncoding]
                forView:lastWorldLoadedView];
    [lastWorldLoadedView setToolTip:[NSString stringWithFormat:@"%s", path]];
    }

//################################################################################
//
//######## Historical code:  Left in place for use in a "Test" menu (not now installed) in the menu bar.

- (IBAction)Test1:(id)sender {
    [self reportDisasterAndQuit:"This is a test error message.  It is only a test.  Wraith Scheme will quit anyway, though."];
    }

- (IBAction)Test2:(id)sender {
    [self tellEvaluator: (SInt32)((random() % C_NUMBER_OF_MESSAGES) & WORD_MASK)
               withLong: 137
             withString: "Test string for evaluator."];
    }

- (IBAction)Test3:(id)sender {
    }

- (IBAction)Test4:(id)sender {
    }

- (IBAction)Test5:(id)sender {
    }

- (IBAction)Test6:(id)sender {
    [self toggleCompileDefines:cDMenuItem];
    }

- (IBAction)Test7:(id)sender {
    [numberOfGCsView setStringValue:[NSString stringWithFormat:@"%d",(random() % 1000 + 1)]];
    }

- (IBAction)Test8:(id)sender {
    }

@end

