/****************************************************************************************

Clowder.h -- Copyright (c) by Jay Reynolds Freeman 2007.

A clowder is a group of cats.

This file is the header for the object that coordinates a group of parallel
Wraith Scheme processes.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#ifndef __CLOWDER_H__
#define __CLOWDER_H__

#import "WraithHeader.h"
#import "DataTypes.h"
#import "MemoryManager.h"
#import "MMapStuff.e"
#import "SECD.h"

#define SPIN_ON_GC_DELAY       1     // microseconds
#define SPIN_ON_MESSAGE_DELAY  1000  // microseconds
#define KITTEN_BLOCK_DELAY     1     // microseconds
#define ALLOCATE_BLOCK_DELAY   1     // microseconds
#define TAG_BLOCK_DELAY        1     // microseconds
#define OBLIST_BLOCK_DELAY     1     // microseconds
#define COLOR_BLOCK_DELAY      1     // microseconds
#define GENSYM_BLOCK_DELAY     1     // microseconds
#define TLE_BLOCK_DELAY        1     // microseconds
#define RESET_BLOCK_DELAY      100   // microseconds
#define INTERRUPT_BLOCK_DELAY  1     // microseconds
#define NO_BLOCK_VALUE (-1)

#define LOOPS_BEFORE_GC (1000)
#define LOOP_MASK 0x3ff
#define WORRY_VALUE (20000)
#define BIN_WORRY_VALUE (10000000)

#define N_CACHED_ADDRESSES_PER_KITTEN 16384

typedef enum {
/*  0 */    MOM_SAYS_NOTHING,
/*  1 */    MOM_SAYS_EXIT,
/*  2 */    MOM_SAYS_WRITE_STACK_TO_SCRATCH,
/*  3 */    MOM_SAYS_COPY_STACK_FROM_SCRATCH,
/*  4 */    MOM_SAYS_WRITE_REGISTERS_TO_SCRATCH,
/*  5 */    MOM_SAYS_COPY_REGISTERS_FROM_SCRATCH,
/*  6 */    MOM_SAYS_SWAP_SPACES,
/*  7 */    MOM_SAYS_CLEAN_UP_PORTS,
/*  8 */    MOM_SAYS_NO_MORE_MESSAGES,
/*  9 */    MOM_SAYS_MOM_ACKNOWLEDGES,
/* 10 */    MOM_SAYS_RESET,
/* 11 */    MOM_SAYS_WORLD_OPERATION,
/* 12 */    MOM_SAYS_CLEAR_COLOR_BITS,
/* 13 */    MOM_SAYS_WRITE_CONTINUATION_TO_SCRATCH,
/* 14 */    MOM_SAYS_COPY_CONTINUATION_FROM_SCRATCH,
/* 15 */    MOM_SAYS_SWAP_GENERATION_SPACES,
/* 16 */    MOM_SAYS_GC_SEQUENCE_STARTING,
/* 17 */    MOM_SAYS_FULL_GC,
/* 18 */    MOM_SAYS_GENERATIONAL_GC,
/* 19 */    MOM_SAYS_GC_HAS_FAILED,
/* 20 */    MOM_SAYS_GC_SEQUENCE_ENDING,
/* 21 */    MOM_SAYS_RESET_FOR_LOW_MEMORY,
/* 22 */    MOM_SAYS_MAX_NUMBER
    } messageFromMom;

typedef enum {
/* 0 */    MOM_SAYS_NOTHING_TO_CONTROLLER,
/* 1 */    MOM_SAYS_EXIT_TO_CONTROLLER,
/* 2 */    MOM_SAYS_FRONT_AND_CENTER_TO_CONTROLLER,
/* 3 */    MOM_SAYS_MAX_NUMBER_TO_CONTROLLER
    } messageFromMomToController;

typedef enum {
/*  0 */    KITTEN_SAYS_NOTHING,
/*  1 */    KITTEN_STARTING_GC_SEQUENCE,
/*  2 */    KITTEN_HAS_WRITTEN_STACK_TO_SCRATCH,
/*  3 */    KITTEN_HAS_COPIED_STACK_FROM_SCRATCH,
/*  4 */    KITTEN_HAS_WRITTEN_REGISTERS_TO_SCRATCH,
/*  5 */    KITTEN_HAS_COPIED_REGISTERS_FROM_SCRATCH,
/*  6 */    KITTEN_HAS_SWAPPED_SPACES,
/*  7 */    KITTEN_HAS_CLEANED_UP_PORTS,
/*  8 */    KITTEN_ACKNOWLEDGES_NO_MORE_MESSAGES,
/*  9 */    KITTEN_ACKNOWLEDGES_RESET,
/* 10 */    KITTEN_ACKNOWLEDGES_WORLD_OPERATION,
/* 11 */    KITTEN_HAS_CLEARED_COLOR_BITS,
/* 12 */    KITTEN_HAS_WRITTEN_CONTINUATION_TO_SCRATCH,
/* 13 */    KITTEN_HAS_COPIED_CONTINUATION_FROM_SCRATCH,
/* 14 */    KITTEN_HAS_SWAPPED_GENERATION_SPACES,
/* 15 */    KITTEN_ACKNOWLEDGES_GC_SEQUENCE_STARTING,
/* 16 */    KITTEN_ACKNOWLEDGES_FULL_GC_STARTING,
/* 17 */    KITTEN_ACKNOWLEDGES_FULL_GC_ENDING,
/* 18 */    KITTEN_ACKNOWLEDGES_GENERATIONAL_GC_STARTING,
/* 19 */    KITTEN_ACKNOWLEDGES_GENERATIONAL_GC_ENDING,
/* 20 */    KITTEN_ACKNOWLEDGES_GC_HAS_FAILED,
/* 21 */    KITTEN_ACKNOWLEDGES_GC_SEQUENCE_ENDING,
/* 22 */    KITTEN_ACKNOWLEDGES_RESET_FOR_LOW_MEMORY,
/* 23 */    KITTEN_SAYS_MAX_NUMBER
    } messageFromKitten;

typedef enum {
/* 0 */    KITTEN_ACTIVITY_NOT_STARTED,
/* 1 */    KITTEN_ACTIVITY_INITIALIZED_MEMORY,
/* 2 */    KITTEN_ACTIVITY_INITIALIZED_GLOBALS,
/* 3 */    KITTEN_ACTIVITY_LOADED_WORLD,
/* 4 */    KITTEN_ACTIVITY_READY_TO_EVAL,
/* 5 */    KITTEN_ACTIVITY_MAX_NUMBER,
    } kittenEvaluatorActivityLevel;

typedef enum {
/* 0 */    KITTEN_GUI_ACTIVITY_NOT_STARTED,
/* 1 */    KITTEN_GUI_ACTIVITY_COMPLETED_INITIALIZATION,
/* 2 */    KITTEN_GUI_ACTIVITY_MAX_NUMBER,
    } kittenGUIActivityLevel;

// Instrumentation and display in human-readable form:

typedef struct localRegisterSet_struct {
    taggedAval localE;
    taggedAval localR;
    taggedAval localA;
    taggedAval localCurrentInputPort;
    taggedAval localCurrentOutputPort;
    taggedAval localErrorPort;
    taggedAval localDefaultInputPort;
    taggedAval localMasterDefaultInputPort;
    taggedAval localTranscriptPort;
    taggedAval localSavedE;
    taggedAval localL;
    taggedAval *localS;
    taggedAval *localF;
    taggedAval *localCS;
    taggedAval *localCF;
    } localRegisterSet;

//  This next typedef, and the array of it that is used in the clowder
//  file content, are set up so as to avoid possible false cache-line
//  sharing of kitten-specific data, when the kitten processes are
//  running on different processor cores.

#define CACHE_LINE_SIZE 64

typedef struct kittenSpecificClowderStuff_struct {

    volatile int breakKittenDeadlockFlag;
    volatile UnsignedWord currentActiveBlock;
    volatile taggedAval *currentActiveTaggedAval;

    volatile int kittenBlock;
    localRegisterSet localRegisters;

    volatile messageFromMom momSays;
    volatile messageFromMomToController momSaysToController;
    volatile messageFromKitten kittenSays;

    volatile Word ObListInRegister;
    volatile Word kittenHasLockedObListBin;
    volatile Word kittenHasLockedTLEBin;
    UnsignedLongWord genSymNumberForKitten;

    volatile Word kittenPID;

    volatile Word KittenEvaluating;
    volatile kittenEvaluatorActivityLevel KittenEvaluatorActivity;
    volatile kittenGUIActivityLevel KittenGUIActivity;
    volatile Word KittenFailedOnInit;

//    volatile Word KittenAcknowledgesGCSequenceStart;
//    volatile Word KittenAcknowledgesMoreOrNoMore;
//    volatile Word KittenAcknowledgesEndOfOneGC;
//    volatile Word KittenAcknowledgesGCSequenceEnd;

    volatile Word KittenFront;
    taggedAval clowderKIQ;
    volatile UnsignedLongWord QueueAccessFlag;

    Byte cacheLinePad[ CACHE_LINE_SIZE ];

    } kittenSpecificClowderStuff;

typedef struct clowderFileContent_struct {

    taggedAval clowderTopLevelE;
    taggedAval clowderIVR;
    taggedAval clowderObList;
    taggedAval clowderTLC;

    kittenSpecificClowderStuff kittenData[ ABSOLUTE_MAX_KITTENS ];

    volatile Boolean GCRequired;
    volatile Boolean FullGCRequired;
    UnsignedLongWord GCCount;                      // How many full garbage collections to date?
    UnsignedLongWord GenerationalGCCount;          // How many generational garbage collections to date?

    Word allocateLock;
    Word gensymLock;
    Word oblistCountLock;

    Word interruptsDisabledGlobally;

    volatile UnsignedLongWord oblistCount;
    volatile UnsignedLongWord oblistSize;

    volatile Byte TLEBinLockBits[ SYMBOL_HASH_CONSTANT / 8  ];
    volatile Byte ObListBinLockBits[ SYMBOL_HASH_CONSTANT / 8 ];
    volatile UnsignedWord interruptBlock[ NUMBER_OF_INTERRUPTS ];    

    taggedAval scratch1;                // E.g., for passing parameters between MomCat and kittens.
    taggedAval scratch2;                // E.g., for passing parameters between MomCat and kittens.

    volatile Byte *nextFreeMMappableSpace;  // For Wraith Scheme MMap System.
    Word nextFreeMMappableSpaceLock;        // For Wraith Scheme MMap System.

    taggedAval **gGCAddressStackAPointer;
    taggedAval **gGCAddressStackAEnd;
    UnsignedLongWord gGCAddressStackALock;

    Boolean gCBytesLock;
    Boolean gCBytesValid;
    UnsignedLongWord gCBytes;

    taggedAval **gGCAddressStackBPointer;
    taggedAval **gGCAddressStackBEnd;
    UnsignedLongWord gGCAddressStackBLock;

    Boolean liveStackIsA;    

    volatile Boolean gcSequenceReturnValue;
    volatile Word kittenRequestingGC;
    volatile UnsignedLongWord gCErrorNumber;

    Boolean rebuildObList;

    } clowderFileContent;

typedef struct interruptInformationFileContent_struct {

    volatile Word KittenPID[ ABSOLUTE_MAX_KITTENS ];
    volatile Boolean interruptRequested[ NUMBER_OF_INTERRUPTS ];
    volatile Boolean interruptInstalled[ NUMBER_OF_INTERRUPTS ];

    } interruptInformationFileContent;

extern void breakDeadlock( taggedAval *tp, Tag thePresentTag, UnsignedWord location );
extern Boolean lockedS0();
extern Boolean blockOnTagLock1( taggedAval *tp, UnsignedLongWord location );
extern Boolean blockOnTagLock1Force( taggedAval *tp, UnsignedLongWord location );
extern void releaseQueue( UnsignedWord which, UnsignedLongWord location );
extern Boolean blockQOnTagLock1( UnsignedWord which, UnsignedLongWord location );
extern void releaseTagLock1( volatile taggedAval *tp, UnsignedLongWord location );
extern void releaseTagLock1Barrier( volatile taggedAval *tp, UnsignedLongWord location );
extern void releaseTagLock1Force( volatile taggedAval *tp, UnsignedLongWord location );
extern Boolean lockedP( volatile Tag *tp );
extern Boolean lockBitSetP( volatile Tag *tp );
extern void assertLocked( volatile Tag *tp, UnsignedWord location, Text *message );
extern void assertNotLocked( volatile Tag *tp, UnsignedWord location, Text *message, UnsignedWord aKitten );
extern void assertCreatedByKitten( volatile Tag *tp, UnsignedWord location, Text *message, UnsignedWord aKitten );

extern void setFrontKitten( int aKitten, int aValue );
extern int getFrontKitten( int aKitten );


class Clowder {

  public:

    // Initialization and allocation of heap:

    void initialize( Byte *clowderSpaceGivenTargetAddress, Byte *interruptInformationSpaceGivenTargetAddress );
    void initializeByMalloc();
    void allocateBitsForProper( UnsignedLongWord memorySize );
    void setColoringBitN( UnsignedLongWord n );
    void clearColoringBitN( UnsignedLongWord n );
    UnsignedLongWord getColoringBitN( UnsignedLongWord n );
    void clearColoringBits();

    // Release of memory-mapped clowder files.

    void releaseClowderFiles();
    void releaseMyStuff();
    void resetMyStuff();

    // Scratch space for use as personal coloring bits for taggedAvals in the heap.

    Byte *coloringBits;
    size_t nColoringBits;

    // Information, Instrumentation and display in human-readable form:

//    volatile UnsignedWord kittenNumber();

    // Global registers:

    taggedAval *globalTopLevelE;
    taggedAval *globalIVR;
    taggedAval *globalObList;
    taggedAval *globalTLC;
    taggedAval *globalKIQ;

    taggedAval *getKIQPointer( UnsignedLongWord n );
    void spliceOutKIQPointer( UnsignedLongWord n );

    // Stuff for critical-section access:

    clowderFileContent *thisClowderFileContent;
    interruptInformationFileContent *thisInterruptInformationFileContent;

    void setRebuildObList();
    void clearRebuildObList();
    Boolean getRebuildObList();

    void blockOnAllocateLock();
    void releaseAllocateLock();

    UnsignedLongWord nextGenSymNumberForKitten( UnsignedWord aKitten );
    void blockOnInterruptsDisabledGlobally();
    void releaseInterruptsDisabledGlobally();
    Boolean getInterruptsDisabledGlobally();

    void clearInterruptRequested( UnsignedLongWord n );
    Boolean getInterruptRequested( UnsignedLongWord n );
    void setInterruptRequested( UnsignedLongWord n );

    void clearInterruptInstalled( UnsignedLongWord n );
    Boolean getInterruptInstalled( UnsignedLongWord n );
    void setInterruptInstalled( UnsignedLongWord n );

    Boolean lockTLEBin( uint32_t n );
    void unlockTLEBin( uint32_t n );
    Boolean TLEBinLockBit( uint32_t n );

    Boolean lockObListBin( uint32_t n );
    void unlockObListBin( uint32_t n );
    Boolean obListBinLockBit( uint32_t n );

    Boolean lockObListCount();
    void unlockObListCount();
    Boolean obListCountLock();
    void incrementObListCount();
    void setObListCount( UnsignedLongWord n );
    UnsignedLongWord getObListCount();

    UnsignedLongWord getObListSize();

    Boolean lockGCBytes();
    void unlockGCBytes();
    Boolean gCBytesLock();
    void setGCBytesValid();
    void clearGCBytesValid();
    Boolean gCBytesValid();
    void setGCBytes( UnsignedLongWord nBytes );
    UnsignedLongWord getGCBytes();

    Boolean lockInterrupt( UnsignedLongWord n );
    void unlockInterrupt( UnsignedLongWord n );
    Boolean interruptLockBit( UnsignedLongWord n );

    void setMomSaysToKitten( messageFromMom aMessage, Word aKitten );
    void setMomSaysToEverybody( messageFromMom aMessage );
    volatile messageFromMom getMomSays();
    volatile messageFromMom getMomSays( Word aKitten );

    void setMomSaysToKittenController( messageFromMomToController aMessage, Word aKitten );
    void setMomSaysToAllKittenControllers( messageFromMomToController aMessage );
    volatile messageFromMomToController getMomSaysToController();
    volatile messageFromMomToController getMomSaysToController( Word aKitten );

    void setKittenSays( messageFromKitten aMessage );
    volatile messageFromKitten getKittenSays( Word aKitten );

    void setKittenRequestingGC( Word aKitten );
    Word getKittenRequestingGC();

    void setGCErrorNumber( UnsignedLongWord errorNumber );
    UnsignedLongWord getGCErrorNumber();

    volatile UnsignedLongWord getKittenBlock( Word aKitten );
    void increaseKittenBlock();
    void reduceKittenBlock();

    volatile UnsignedLongWord getBreakKittenDeadlockFlag( Word aKitten );
    void setBreakKittenDeadlockFlag( Word aKitten );
    void clearBreakKittenDeadlockFlag( Word aKitten );
    void clearBreakKittenDeadlockFlag();

    void setGCSequenceReturnValue( Boolean toThis );
    Boolean getGCSequenceReturnValue();

    void setKittenFailedOnInit();
    void clearKittenFailedOnInit();
    volatile Boolean getKittenFailedOnInit( Word aKitten);

    void setKittenEvaluating();
    void clearKittenEvaluating();
    volatile Boolean getKittenEvaluating( UnsignedLongWord aKitten);
    volatile Boolean anyKittenEvaluating();

    void setKittenHasLockedObListBin( Word aKitten, Word aBin );
    volatile Word getKittenHasLockedObListBin( Word aKitten);
    volatile Boolean anyKittenHasLockedObListBin();

    void setKittenHasLockedTLEBin( Word aKitten, Word aBin );
    volatile Word getKittenHasLockedTLEBin( Word aKitten);
    volatile Boolean anyKittenHasLockedTLEBin();

    void setKittenEvaluatorActivity( kittenEvaluatorActivityLevel activityLevel );
    volatile kittenEvaluatorActivityLevel getKittenEvaluatorActivity( Word aKitten );

    void setKittenGUIActivity( kittenGUIActivityLevel activityLevel );
    volatile kittenGUIActivityLevel getKittenGUIActivity( Word aKitten);

    void setObListInRegister();
    void clearObListInRegister();
    volatile Boolean getObListInRegister( Word aKitten);

    void setGCIsRequired();
    void clearGCIsRequired();
    volatile Boolean GCIsRequired();

    void setFullGCIsRequired();
    void clearFullGCIsRequired();
    volatile Boolean FullGCIsRequired();

    volatile UnsignedLongWord getQueueBlock( Word aKitten );
    volatile UnsignedLongWord *getQueueBlockPointer( Word aKitten );

    void setCurrentActiveBlock( Word aKitten, UnsignedWord location );
    void setCurrentActiveBlock( UnsignedWord location );
    volatile UnsignedWord getCurrentActiveBlock( Word aKitten);
    volatile Boolean anyCurrentActiveBlock();

    void setCurrentActiveTaggedAval( Word aKitten, volatile taggedAval *tp );
    void setCurrentActiveTaggedAval( volatile taggedAval *tp );
    volatile taggedAval *getCurrentActiveTaggedAval( Word aKitten);
    volatile Boolean anyCurrentActiveTaggedAval();

    Word getKittenPID( Word aKitten );

    void setScratch1( taggedAval *aTaggedAval );
    void setScratch2( taggedAval *aTaggedAval );
    taggedAval getScratch1();
    taggedAval getScratch2();

    // Garbage Collection Interface

    volatile messageFromMom kittenWaitForMessage( messageFromMom aMessage );
    volatile messageFromMom kittenWaitForNewMessage( messageFromMom oldMessage );
    volatile Boolean momWaitForMessage( messageFromKitten aMessage, UnsignedWord aKitten );
    void kittenGCSequence( Boolean doingGenerational );

    // Memory Manager Interface

    void setMemoryManager( MemoryManager *aMM );

    // Utilities

    volatile Boolean allKittensActiveAt( kittenEvaluatorActivityLevel theLevel );
    volatile Boolean momActiveAt( kittenEvaluatorActivityLevel theLevel );
    void momDialogWithKitten( UnsignedWord aKitten, messageFromMom momSays, messageFromKitten kittenReplies );
    void momDialogWithEverybody( messageFromMom momSays, messageFromKitten kittenReplies );
    void momReceiveFromEverybody( messageFromKitten kittenReplies );
    Boolean kittenHandleGCSequenceMessages();
    void kittenHandleMessages();
    volatile Boolean anyObListInRegister();
    void setKittenFrontRequest( Word aKitten, Word aValue );
    volatile Word getKittenFrontRequest( Word aKitten);
    Boolean allKittensSay( messageFromKitten aMessage );
    Text *getInterruptInformationFileName();
    UnsignedLongWord getInterruptInformationFileSize();

    // Wraith Scheme MMap System;

    volatile UnsignedLongWord getMMappableSpaceSize();               // For Wraith Scheme MMap System.
    void *incrementNextFreeMMappableSpace( UnsignedLongWord size);   // For Wraith Scheme MMap System.

    // Generational GC interesting address stack system:

    void initializeGGCAddressStacks( volatile memorySpace *inThisSpace );
    void swapGGCAddressStacks();

    taggedAval **getGGCLiveAddressStackPointer();
    taggedAval **getGGCLiveAddressStackEnd();

    taggedAval **getGGCDeadAddressStackPointer();
    taggedAval **getGGCDeadAddressStackEnd();

    void addToLiveAddressStack( taggedAval *addressOfInterest );
    void flushCachedAddresses();
    void clearDeadAddressStack();
    void addToDeadAddressStack( taggedAval *addressOfInterest );

    void setLiveStackIsA();
    void checkLiveStack( const Text *message );
    void seeIfStacked( taggedAval *anAddress, const Text *message );
    void seeIfOn( taggedAval *anAddress, const Text *message );
    UnsignedLongWord activeStackLength();
    void runInterruptLocally( UnsignedLongWord interruptNumber );

  private:

    Text scratch[ MAX_PATHNAME_LENGTH + 1 ];
    Text clowderFileName[ MAX_PATHNAME_LENGTH + 1 ];
    Text interruptInformationFileName[ MAX_PATHNAME_LENGTH + 1 ];

    void writeRegistersToScratch();
    void copyRegistersFromScratch();

    void writeStackToScratch();
    void copyStackFromScratch();

    void writeContinuationToScratch();
    void copyContinuationFromScratch();

    taggedAval *cachedLiveAddresses[ N_CACHED_ADDRESSES_PER_KITTEN ];
    UnsignedLongWord cachedLiveAddressStackIndex;

    };

#endif

