/*
 *  CocoaMachineMesssageNames.h
 *  WraithCocoa
 *
 *  Created by Jay Freeman on 12/20/06.
 *  Copyright 2006 Jay Reynolds Freeman. All rights reserved.
 *
 */
/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

*/

#ifndef COCOA_MESSAGE_NAMES

#define COCOA_MESSAGE_NAMES

static char *messagesFromEvaluatorNames[] = {

    // System administration.

    "E_NO_MESSAGE_YET",                 // Initialization state -- no messages sent yet.
    "E_COMPLETED_INITIALIZATION",       // And ready to handle messages.
    "E_HELLO_THERE",                    // Please acknowledge your existence.
    "E_FATAL_PROBLEM",                  // Display the string in a modal dialog, and quit.
    "E_PLEASE_TIDY_UP_FOR_EXIT",        // Please flush and close files, et cetera.
    "E_TIDIED_UP",                      // Evaluator has tidied up as necessary for exit -- close ports, perhaps.
    "E_COGNITO_ERGO_SUM",               // I think, therefore ... I AM!!

    // Simple state transitions.

    "E_EVALUATING",                     // Evaluator is evaluating; some things are not allowed now.
    "E_NOT_EVALUATING",                 // Evaluator is not evaluating.
    "E_GARBAGE_COLLECTING",             // Evaluator is garbage-collecting.
    "E_NOT_GARBAGE_COLLECTING",         // Evaluator is not garbage-collecting.
    "E_SLEEPING",                       // Evaluator is sleeping.
    "E_WAKED_UP",                       // Evaluator has waked up.
    "E_COMPILING_DEFINES",              // Evaluator will compile defines.
    "E_NOT_COMPILING_DEFINES",          // Evaluator will not compile defines.
    "E_USING_LOW_PRECISION",            // Evaluator will display numbers with low precision in top-level loop.
    "E_USING_HIGH_PRECISION",           // Evaluator will display numbers with high precision in top-level loop.

    // Routine I/O.

    "E_OUTPUT_READY",                   // Display the string in the main window.
    "E_MESSAGE_TO_DISPLAY",             // Message to display is in the string.
    "E_DIALOG_TO_START",				// Dialog starting, prompt is in the string.
    "E_FILE_LOAD_BEGINNING",            // Opened the in file in the string.
    "E_FILE_LOAD_SUCCEEDED",            // Read in file in the string.
    "E_FILE_LOAD_FAILED",               // Couldn't find, or open, or something, the file in the string.
    "E_WORLD_LOAD_SUCCEEDED",           // Loaded the world in the string.  (Failure of a world load might be fatal, and is so reported if so.)
    "E_WORLD_LOAD_FAILED",              // Couldn't find, or open, or something, the file in the string.  (Might also be fatal!)
    "E_WORLD_SAVE_SUCCEEDED",           // Evaluator has saved a world to the file in string.
    "E_WORLD_SAVE_FAILED",              // Attempt to save world to the file in string has failed.

    // Input parsing.

    "E_EXPECT_MORE_INPUT",              // Processing multi-line input, need more (expression is not finished).
    "E_DO_NOT_EXPECT_MORE_INPUT",       // Parsing finished; interpreter is doing something else.
    "E_EXPECT_PARENS",                  // Processing multi-line input, need N (in the long right parens to finish).
    "E_EXPECT_DOUBLE_QUOTE",            // Processing multi-line input, need a double-quote (e.g., in a string).
    "E_DISCARDED_INPUT",                // Evaluator has abandoned processing of the current S-expression.

    // Status information, possibly for display.

    "E_NEW_GC_COUNT",                   // New total of GCs is in the long.
    "E_MEMORY_SIZE",                    // The memory size is in the long.
    "E_MEMORY_IN_USE",                  // The memory in use is in the long.
    "E_LIST_PRINT_LENGTH",              // The list print length is in the long.
    "E_LIST_PRINT_DEPTH",               // The list print depth is in the long.
    "E_VECTOR_PRINT_LENGTH",            // The vector print length is in the long.
    "E_VECTOR_PRINT_DEPTH",             // The vector print depth is in the long.
    "E_GENERATIONAL_MEMORY_SIZE",       // The generational memory size is in the long.
    "E_GENERATIONAL_MEMORY_IN_USE",     // The generational memory in use is in the long.
    "E_SENSE_LIGHT_COMMAND",            // A command for zero or more sense lights is in the long.
    "E_SENSE_LIGHT_ROTATE",             // A command for how far to rotate the sense light array is in the long.
    "E_TOTAL_MEMORY_IN_USE",            // The total memory size in use in in the long.
    "E_LEVEL_INDICATOR_0_VALUE",        // New value for level indicator 0 is in the long.
    "E_LEVEL_INDICATOR_1_VALUE",        // New value for level indicator 1 is in the long.
    "E_LEVEL_INDICATOR_2_VALUE",        // New value for level indicator 2 is in the long.
    "E_LEVEL_INDICATOR_3_VALUE",        // New value for level indicator 3 is in the long.

    // Progress bar manipulation.

    "E_INIT_PROGRESS",                  // Make the progress bar visible with an initial setting of zero.
    "E_INCREMENT_PROGRESS",             // Set the progress to the number of tenths of percents that is the long.
    "E_WRAPUP_PROGRESS",                // Make the progress bar invisible.

    "E_NUMBER_OF_MESSAGES"              // Handy for counting.
    };

static char *messagesFromControllerNames[] = {

    // System administration.

    "C_NO_MESSAGE_YET",                 // Initialization state -- no messages sent yet.
    "C_PLEASE_CLEAR_STATUS",            // Clear the status-message window.
    "C_COMPLETED_INITIALIZATION",       // And ready to handle messages.
    "C_HELLO_THERE",                    // Please acknowledge your existence.
    "C_PLEASE_TIDY_UP_FOR_EXIT",        // Please flush and close files, et cetera.
    "C_TIDIED_UP",                      // Evaluator has tidied up as necessary for exit -- close ports, perhaps.
    "C_COGNITO_ERGO_SUM",               // I think, therefore ... I AM!!

    // Simple state transitions.

    "C_PLEASE_STOP_EVALUATING",         // Evaluator please stop evaluating and return control to top-level loop.
    "C_PLEASE_COMPILE_DEFINES",         // Evaluator please compile defines.
    "C_PLEASE_DO_NOT_COMPILE_DEFINES",  // Evaluator please do not compile defines.
    "C_PLEASE_USE_LOW_PRECISION",       // Evaluator please display numbers with low precision in top-level loop.
    "C_PLEASE_USE_HIGH_PRECISION",      // Evaluator please display numbers with high precision in top-level loop.

    // Routine I/O.

    "C_INPUT_READY",                    // This input is ready for parsing.
    "C_DIALOG_RESPONSE_READY",          // Dialog ending, response is in the string.

    "C_PLEASE_LOAD_FILE",               // Please load the file in the string.
    "C_PLEASE_LOAD_WORLD",              // Please load the world in the string.
    "C_PLEASE_SAVE_WORLD",              // Please save a world to the file in string.

    // Input parsing.

    "C_PLEASE_DISCARD_INPUT",           // Please discard what has thus far been read of a multi-line S-expression.

    // Status information, possibly for display.

    "C_NUMBER_OF_MESSAGES"              // Handy for counting.
    };

#endif // COCOA_MESSAGE_NAMES
