/****************************************************************************************

GarbageCollector.h -- Copyright (c) by Jay Reynolds Freeman 2007.

Garbabe collector class, for Wraith Scheme.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#ifndef __GARBAGECOLLECTOR_H__
#define __GARBAGECOLLECTOR_H__

#import <time.h>
#import "WraithHeader.h"
#import "DataTypes.h"

typedef enum {
    COPY_NON_CAR,
    COPY_CC_NIL,
    COPY_CC_NEXT_SECTION_CAR_CDR_NIL,
    COPY_CC_NEXT_SECTION_NOT_CAR_POSITION_ONE,
    COPY_CC_NEXT_SECTION_NOT_CAR_POSITION_TWO,
    COPY_CC_NEXT_SECTION_NOT_CAR_POSITION_THREE,
    COPY_CC_NEXT_SECTION_NOT_CAR_POSITION_FOUR,
    COPY_CC_NEXT_SECTION_NOT_CAR_POSITION_FIVE,
    COPY_ACROSS_LOW
    } calledFromWhere;

class MemoryManager;
class WorldHandler;
class Clowder;

extern void clearAllGCAndRecursionBits( volatile memorySpace *m );

class GarbageCollector {

  friend class MemoryManager;
  friend class Clowder;

  public:

    void initializeGC( MemoryManager *myMemoryManager );
    UnsignedLongWord garbageCollectorGCCount();
    UnsignedLongWord garbageCollectorGenerationalGCCount();
    void fullGCType2( volatile memorySpace *fromSpace, volatile memorySpace *toSpace, Boolean dealWithObList );
    void standardFullGC( volatile memorySpace *currentSpace, volatile memorySpace *alternateSpace, Boolean *didRebuildObList );
    void standardGenerationalGC( volatile memorySpace *currentSpace, volatile memorySpace *alternateSpace );

 private:

    MemoryManager *myMemoryManager;
    Clowder *myClowder;
    Boolean isMomCatLocalP;
    volatile Boolean garbageCollectorRunning;               // Checked to avoid reentrant code to garbage collector inner routines.
    volatile Boolean garbageCollectorEntryRoutineRunning;   // Checked to avoid reentrant code to garbage collector entry points.

#ifdef USE_COCOA_IO

    double gcProgress;                 // Handy static for progress-bar use.
    Boolean useProgressBar;            // Handy static for progress-bar use.

#endif

    UnsignedLongWord gcRandomNumber;
    UnsignedLongWord scratchRandomNumber;

    void setIsMomCat( Boolean trueOrFalse );
    void setClowder( Clowder *aClowder );
    void postGCAuxiliary( volatile memorySpace *currentSpace, volatile memorySpace *alternateSpace );
    void checkForgettables( volatile memorySpace *m, const Text *message );
    UnsignedLongWord expireForgettables( volatile memorySpace *m, volatile memorySpace *scratch );
    void reportCheckOneRegisterError( taggedAval *aReg, const Text* what, Word messageIndex, Boolean quitOnErrorP );
    Boolean checkOneRegister( taggedAval *aReg, volatile memorySpace *tenured,
                              volatile memorySpace *active, volatile memorySpace *aging, const Text *message,
                              Boolean quitOnErrorP ); 
    void reportCheckOneRegisterError( taggedAval *aReg, const Text* what, Word messageIndex );
    Boolean checkStackRegisters( volatile memorySpace *aStack, volatile memorySpace *tenured,
                                 volatile memorySpace *active, volatile memorySpace *aging, const Text *what,
                                 taggedAval *aStackPointer, Boolean quitOnErrorP );
    Boolean checkRegistersForKitten( UnsignedWord aKittenNumber, volatile memorySpace *tenured,
                                     volatile memorySpace *active, volatile memorySpace *aging, const Text *message,
                                     Boolean quitOnErrorP );
    void checkRegisters( volatile memorySpace *tenured,
                         volatile memorySpace *active, volatile memorySpace *aging, const Text *message,
                         Boolean quitOnErrorP );
    void checkForPointsToActiveBits( volatile memorySpace *m, const Text *message, Boolean quitOnError );
    void checkSpace( volatile memorySpace *check, volatile memorySpace *pointInto, taggedAval *stackPointer,
                     const Text *message, Boolean quitOnError );
//    void checkPorts( volatile memorySpace *m, Text *message, Boolean quitOnError );
    void cleanUpPorts( volatile memorySpace *m, volatile memorySpace *also );
    void clearAllGCBits( volatile memorySpace *m );
    void clearAllRecursionBits( volatile memorySpace *m );
    void clearAllPointsToActiveBits( volatile memorySpace *m );
    void clearAllGCAndRecursionBits( volatile memorySpace *m );
    UnsignedLongWord copyAcrossLowAUX2( volatile memorySpace *m, volatile memorySpace *to, UnsignedLongWord *nNonGarbageObjects,
                                        Boolean useRandomizerP, UnsignedLongWord *nInSpaceFPsSeen,
                                        UnsignedWord whichKitten );
    UnsignedLongWord copyAcrossLowAUX3( volatile memorySpace *m, volatile memorySpace *to, UnsignedLongWord *nNonGarbageObjects,
                                        UnsignedLongWord *nInSpaceFPsSeen,
                                        UnsignedWord whichKitten );
    UnsignedLongWord copyAcrossLow( volatile memorySpace *m, volatile memorySpace *to, UnsignedLongWord expectedObjectCount );
    Byte *copyBlockLow( taggedAval *blockPointer, memoryBlockSize blockLength, UnsignedLongWord nObjects,
                        volatile memorySpace *to, calledFromWhere whence );
    Byte *copyCCNextSection( taggedAval *start, volatile memorySpace *m, volatile memorySpace *to,
                             UnsignedLongWord *nObjectsCopied, UnsignedLongWord *nNonGarbageObjects,
                             UnsignedLongWord *nInSpaceFPsSeen );
    Byte *copyCCNextSectionOLD( taggedAval *start, volatile memorySpace *m, volatile memorySpace *to,
                                UnsignedLongWord *nObjectsCopied, UnsignedLongWord *nNonGarbageObjects );
    Byte *copyCCNil( taggedAval *start, volatile memorySpace *m, volatile memorySpace *to );
    Byte *copyNonCar( taggedAval *start, volatile memorySpace *m, volatile memorySpace *to );
    void fussProgress( double increment );
    void addIfPointsToSymbol( taggedAval *tp );
    void addIfPointsToSymbolStack( volatile memorySpace *aStack, taggedAval *aStackPointer );
    Boolean anyBlockedSymbols( volatile memorySpace *fromSpace );
    void rebuildObList( volatile memorySpace *fromSpace );
    void GCObList();
    UnsignedLongWord getRandomBit();
    UnsignedLongWord markGarbageFromReg( taggedAval *aReg, volatile memorySpace *m, volatile memorySpace *scratchSpace );
    UnsignedLongWord markGarbageFromStack( volatile memorySpace *aStack, volatile memorySpace *m, volatile memorySpace *scratchSpace,
                                           taggedAval *aStackPointer );
    UnsignedLongWord markGarbageFromRegGGC( taggedAval *aReg,
                                            volatile memorySpace *tenured,
                                            volatile memorySpace *aging,
                                            volatile memorySpace *active,
                                            volatile memorySpace *scratchSpace );
    UnsignedLongWord markGarbageFromStackGGC( volatile memorySpace *aStack,
                                              volatile memorySpace *tenured,
                                              volatile memorySpace *aging,
                                              volatile memorySpace *active,
                                              volatile memorySpace *scratchSpace,
                                              taggedAval *aStackPointer );
    UnsignedLongWord markGarbageFromAddressStackReferences( taggedAval **start, taggedAval **end,
                                                            volatile memorySpace *tenured,
                                                            volatile memorySpace *aging,
                                                            volatile memorySpace *active,
                                                            volatile memorySpace *scratchSpace,
                                                            const Text *message );
    void markNonGarbageCCNBlocksInSpace( volatile memorySpace *m );
    Boolean notCarFPDereferencesInSpace( taggedAval *xp, volatile memorySpace *spaceToBeIn, taggedAval **whither,
                                         UnsignedLongWord *nInSpaceFPsSeen );
    void replaceRegisterPointer( taggedAval *aReg, LongWord bumpDelta, const Text *name,
                                        volatile memorySpace *from, volatile memorySpace *to );
    void replaceCarCdrNextFollowedByNilWithCarCdrNil( volatile memorySpace *m );
    void replaceCarCdrNilPointersToNilWithNil( volatile memorySpace *m );
    void replaceStackPointers( volatile memorySpace *aStack, taggedAval *aStackPointer, LongWord bumpDelta, UnsignedWord kitten,
                               volatile memorySpace *from, volatile memorySpace *to );
    void replaceContinuationPointers( volatile memorySpace *aContinuation, taggedAval *aContinuationPointer,
                                      LongWord bumpDelta, UnsignedWord kitten,
                                      volatile memorySpace *from, volatile memorySpace *to );
    void reportCheckSpaceError( volatile memorySpace *m, volatile Byte *bp, volatile Byte *lastbp, taggedAval *tap,
                                taggedAval lastTag,
                                Boolean lastValid, const Text *message, Word messageIndex, Boolean quitOnError );
    void reportProgress( double newValue );
    void markGarbageFromRegForKitten( UnsignedWord aKittenNumber, volatile memorySpace *fromSpace, volatile memorySpace *toSpace,
                                      UnsignedLongWord *nonGarbageCount );
    void markGarbageFromRegForKittenGGC( UnsignedWord aKittenNumber,
                                         volatile memorySpace *tenured,
                                         volatile memorySpace *aging,
                                         volatile memorySpace *active,
                                         volatile memorySpace *scratchSpace,
                                         UnsignedLongWord *nonGarbageCount );
    void replaceRegisterPointersForKitten( UnsignedWord aKittenNumber,
                                           volatile memorySpace *fromSpace, volatile memorySpace *toSpace,
                                           LongWord bumpDelta );
    UnsignedLongWord fullGCMarkNonGarbage( volatile memorySpace *fromSpace, volatile memorySpace *toSpace, Boolean dealWithObList );
    void fullGCReplaceRegisterPointers( volatile memorySpace *fromSpace, volatile memorySpace *toSpace,
                                        Boolean dealWithObList, LongWord bumpDelta );
    UnsignedLongWord gGCMarkNonGarbage( volatile memorySpace *fromSpace );
    void gGCReplaceRegisterPointers( volatile memorySpace *agingSpace );
    void setGCRandomNumbers();
    Byte *skipCCNextSection( taggedAval *start, volatile memorySpace *m );
    UnsignedLongWord slideLowToHigh( volatile memorySpace *m, UnsignedLongWord expectedObjectCount );
    void spliceOutAllForwardingPointers( volatile memorySpace *m, taggedAval *stackPointer );
    taggedAval *whereDoesNotCarFPGoUltimately( taggedAval *xp );
    UnsignedLongWord countNonGarbageLocks( volatile memorySpace *m );
    UnsignedLongWord countAllLocks( volatile memorySpace *m );
    void mergeGarbage( volatile memorySpace *m );
    void markNonGarbageCCNBlocksInSpaceAndMergeGarbage( volatile memorySpace *m );
    UnsignedLongWord countNonGarbageLocksAndMergeGarbage( volatile memorySpace *m );
    UnsignedLongWord countNonGarbageBytesInSpace( volatile memorySpace *m );
    void replaceStackPointersGenGCB( volatile memorySpace *aStack, taggedAval *aStackPointer, UnsignedWord kitten,
                                     volatile memorySpace *aging );
    void replaceContinuationPointersGenGCB( volatile memorySpace *aContinuation, taggedAval *aContinuationPointer,
                                            UnsignedWord kitten, volatile memorySpace *aging );
    void replaceRegisterPointersForKittenGenGCB( UnsignedWord aKittenNumber, volatile memorySpace *aging );
    void replacePointersInAddressStackB( taggedAval **start, taggedAval **stop, volatile memorySpace *aging );
    void replaceRegisterPointerGenGCB( taggedAval *aReg, volatile memorySpace *aging );
    UnsignedLongWord copyDownB( volatile memorySpace *m, volatile memorySpace *to );
    UnsignedLongWord countNonGarbageBytesAndMergeGarbage( volatile memorySpace *m );
    void updatePointersInCopiedDownArea( Byte *start, Byte *stop, volatile memorySpace *lookForPointersToHere );
    void updatePointersFromActiveToAging( volatile memorySpace *active, volatile memorySpace *aging );
    void stackPointersToActiveInCopiedDownAreaB( Byte *start, Byte *stop, volatile memorySpace *activeSpace );
    void primeDeadStackB( taggedAval **start, taggedAval **stop, volatile memorySpace *tenured,
                          volatile memorySpace *active, volatile memorySpace *aging );
    void checkActiveForPointersToAging( volatile memorySpace *active, volatile memorySpace *aging );
    void checkAgingCdrCodedBlocks( volatile memorySpace *aging );
    void checkTenuredForMissedNonGarbage( volatile memorySpace *tenured, 
                                          volatile memorySpace *active,
                                          volatile memorySpace *aging );
    void generationalGCB( volatile memorySpace *currentSpace, volatile memorySpace *alternateSpace );

	void startEvents( Boolean isFullGC );
	void endEvents( Boolean isFullGC );
    };

#endif
