/****************************************************************************************

Globals.e -- Copyright (c) by Jay Reynolds Freeman 2007

Main globals header file for Wraith Scheme

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#ifndef __GLOBALS__
#define __GLOBALS__

#import "LocalTypes.h"

//extern UnsignedLongWord debugCount;  // DEBUG, in SECDEval.c++

#import "Clowder.h"
#import "MemoryManager.h"
#import "WorldHandler.h"

extern int vRefNumber;  // DEBUG

#ifdef USE_COCOA_IO

extern int globalArgc;                               // In main.mm
extern char **globalArgv;                            // In main.mm

extern Boolean setCompileDefinesFromCommandLine;     // In main.mm
extern Boolean commandLineCompileDefinesFlag;        // In main.mm

#endif

// Next big batch of externs are all from "WraithConsts.c++".

#import "WraithConsts.h"

// End of stuff defined in "WraithConsts.c++"

class MemoryManager;
extern MemoryManager *theHeap;           // In MainAux.c++

class Clowder;
extern Clowder *theClowder;              // In MainAux.c++

class WorldHandler;
extern WorldHandler *theWorldHandler;    // in MainAux.c++
extern UnsignedLongWord *theSimpleTable; // in MainAux.c++

extern UnsignedWord theKittenNumber;     // In MainAux.c++
extern UnsignedWord theNumberOfKittens;  // In MainAux.c++

extern Boolean bigHeap;                  // In MainAux.c++  
extern Boolean BigEndianFlag;            // In MainAux.c++  
extern long long hardwareMemorySize;     // In MainAux.c++  
extern Text lastWorldLoadedName[];       // In MainAux.c++  
extern Text InitFileName[];              // In MainAux.c++
extern Boolean useStdinP;                // In MainAux.c++
extern Boolean isMomCatP;                // In MainAux.c++

extern Boolean debugBufferInitialized;   // In MainAux.c++
extern Text theKittenString[];           // In MainAux.c++

extern Byte *globalSpaceATargetAddress;        // In MainAux.c++
extern Byte *globalSpaceBTargetAddress;        // In MainAux.c++
extern Byte *globalScratchStackTargetAddress;  // In MainAux.c++
extern Byte *globalClowderSpaceTargetAddress;  // In MainAux.c++

extern Boolean aWorldHasJustBeenLoaded;        // In MainAux.c++
extern Boolean usingGenerationalGC;            // In MainAux.c++


extern Text appSupDirName[];             // In MainAux.c++

extern Boolean isAlarmSet;               // In MainAux.c++
extern LongWord alarmTime;       // In MainAux.c++

extern FILE *kittenLogFile;              // In MainAux.c++

extern LongWord progressFractionInThousandths;     // In MainAux.c++

extern dataDescriptor **dataTypeTable;                                      // In DataTypes.c++
extern UnsignedLongWord depthSoFar, vDepthSoFar;                            // In DataTypes.c++
extern UnsignedLongWord printDepth, printLength, vPrintDepth, vPrintLength; // In DataTypes.c++

extern Boolean	CompileDefinesFlag;             // In Events.c++ and EventsForCocoa.c++
extern Boolean	GCInProgressFlag;               // In Events.c++ and EventsForCocoa.c++
extern Boolean	EmptyListIsFalseFlag;           // In Events.c++ and EventsForCocoa.c++

extern LongWord evalStartCounter;               // In EventsForCocoa.c++

#ifdef USE_COCOA_IO
extern Boolean interruptRequested[];            //  TEMPORARY
#endif

extern Boolean  consoleOutputEnabledFlag;  // In IO.c++

extern Boolean BigEndianFlag;         // In MainAux.c++
extern jmp_buf ExitBuffer;            // In MainAux.c++
extern jmp_buf ResetErrorBuffer;      // In MainAux.c++
extern jmp_buf debugBuffer;           // In MainAux.c++
extern int evaluatorHasInitialized;   // In MainAux.c++

extern UnsignedLongWord NumberOfErrorStrings;       // In ManyStrings.c++
extern UnsignedLongWord NumberOfFatalErrorStrings;  // In ManyStrings.c++

extern UnsignedLongWord schemeMMapReserveSize;      // In MMapStuff.c++
extern unsigned char *schemeMMapReserve;            // In MMapStuff.c++

extern Boolean useFullNumericPrecision;  // In NumberFormats.c++

extern Boolean ParserActive;             // In Reader.c++
extern Boolean ReadingString;            // In Reader.c++
extern UnsignedLongWord RParensNeeded;   // In Reader.c++

extern taggedAval *stackLowLimit;         // In SECDEval.c++
extern taggedAval *continuationLowLimit;  // In SECDEval.c++

extern Text InitFilePath[];  // In SECDIO.c++
extern Text SWFBuffer[];     // In SECDIO.c++

extern taggedAval zero, one, two, three, four, five, six, seven, eight, nine, ten,  // In SECDMachine.c++
                  maxChar,                                             // In SECDMachine.c++
                  numberOfInterrupts,                                  // In SECDMachine.c++
		          symbolHashSize;                                      // In SECDMachine.c++
extern taggedAval empty;                                               // In SECDMachine.c++
extern taggedAval blankChar, newlineChar;                              // In SECDMachine.c++
extern taggedAval sharpF;                                              // In SECDMachine.c++
extern taggedAval taggedAvalLength;                                    // In SECDMachine.c++
extern taggedAval dummyEntry;                                          // In SECDMachine.c++

extern Text versionString[];   // In SchemeGlobals.c++

extern fileDescriptor *conErr;                 // In StreamHandling.c++
extern fileDescriptor *conIn;                  // In StreamHandling.c++
extern fileDescriptor *conOut;                 // In StreamHandling.c++
extern fileDescriptor *masterDefaultInputFD;   // In StreamHandling.c++

extern fileDescriptor *TranscriptFD;  // In StreamHandling.c++
extern fileDescriptor *WFile;  // In StreamHandling.c++

extern Boolean descriptorsInitializedP;         // In StreamHandling.c++
extern fileDescriptor AllFileDescriptors[];     // In StreamHandling.c++

extern fileDescriptor *transcriptFD;            // In StreamHandling.c++
extern fileDescriptor *worldFD;                 // In StreamHandling.c++
	
extern Text *theWorldString;                    // In WorldHandler.c++

#ifdef USE_COCOA_IO

extern Boolean useStdinP;               // In MainAux.c++  // Just for Cocoa I/O.

#endif

extern Word outputEnabled;               // In WraithSchemeOuterController.mm for Cocoa I/O, also in MainAux.c++ for other I/O.
extern Word clowderIsOperating;          // In WraithSchemeControllerAndModel.mm for Cocoa I/O, also in MainAux.c++ for other I/O.
extern Word worldIsLoaded;               // In WraithSchemeControllerAndModel.mm for Cocoa I/O, also in MainAux.c++ for other I/O.
extern Word controllerReadyForMessages;  // In WraithSchemeControllerAndModel.mm for Cocoa I/O, also in MainAux.c++ for other I/O.
extern Word GUIInitialized;              // In WraithSchemeControllerAndModel.mm for Cocoa I/O, also in MainAux.c++ for other I/O.
extern Word readyToRead;                 // In WraithSchemeControllerAndModel.mm for Cocoa I/O, also in MainAux.c++ for other I/O.

// Portmanteau of table indices initialized at startup:

extern UnsignedLongWord CPushFunction_index;
extern UnsignedLongWord CPushFunction_index;
extern UnsignedLongWord ClearLFunction_index;
extern UnsignedLongWord closeStackedPortRestoreOutput_index;
extern UnsignedLongWord MakeRTrueFunction_index;
extern UnsignedLongWord S0toC_index;
extern UnsignedLongWord SPopRFunction_index;
extern UnsignedLongWord SPushFunction_index;
extern UnsignedLongWord SwapCons_index;
extern UnsignedLongWord apply_index;
extern UnsignedLongWord applyEnd_index;
extern UnsignedLongWord assertEvaluating_index;
extern UnsignedLongWord assertFileLoadSucceeded_index;
extern UnsignedLongWord assertNotEvaluating_index;
extern UnsignedLongWord assign_index;
extern UnsignedLongWord assign_with_block_index;
extern UnsignedLongWord bindOrAssign_index;
extern UnsignedLongWord bindScheme_index;
extern UnsignedLongWord bindToOptionMinus_index;
extern UnsignedLongWord closePort_index;
extern UnsignedLongWord closeStackedPort_index;
extern UnsignedLongWord closeStackedPortRestoreInput_index;
extern UnsignedLongWord cwCCode_index;
extern UnsignedLongWord defineEvalCode_index;
extern UnsignedLongWord dereferenceStackReferenceMC_index;
extern UnsignedLongWord displayObject_index;
extern UnsignedLongWord dropNFromStack_index;
extern UnsignedLongWord expandMacroAux_index;
extern UnsignedLongWord ifAuxCode_index;
extern UnsignedLongWord loadCode_index;
extern UnsignedLongWord macroBodyAux_index;
extern UnsignedLongWord macroEvalAux_index;
extern UnsignedLongWord macroPartTwo_index;
extern UnsignedLongWord newlineIfPrinting_index;
extern UnsignedLongWord noop_index;
extern UnsignedLongWord oneForEach_index;
extern UnsignedLongWord oneMap_index;
extern UnsignedLongWord oneMapFirst_index;
extern UnsignedLongWord peekLoop_index;
extern UnsignedLongWord popToEFunction_index;
extern UnsignedLongWord readSexpFromPort_index;
extern UnsignedLongWord rebindForce_index;
extern UnsignedLongWord rebindTopLoopVariables_index;
extern UnsignedLongWord resetScheme_index;
extern UnsignedLongWord restoreStackAndFrame_index;
extern UnsignedLongWord reverseListCode_index;
extern UnsignedLongWord tfflushConsoleOutput_index;
extern UnsignedLongWord topLoopCode_index;
extern UnsignedLongWord writeObject_index;
extern UnsignedLongWord readSexpOrQueueTopLevel_index;
extern UnsignedLongWord kittenDequeueCode_index;
extern UnsignedLongWord restoreInputPort_index;
extern UnsignedLongWord restoreForCWCC_index;
extern UnsignedLongWord setApplicativeBitNDownBang_index;

#endif
