/****************************************************************************************

MMapStuff.e -- Copyright (c) by Jay Reynolds Freeman 2007, 2008

For dealing with malloced and mmapped stuff in "main.m".

This is actually a combination ".e" and ".h" file ...

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/


#import <sys/mman.h>
#import <fcntl.h>
#import "WraithHeader.h"

const UnsignedLongWord GIGABYTE = ((UnsignedLongWord)1073741824);
const UnsignedLongWord MEGABYTE = ((UnsignedLongWord)1048576);
const UnsignedLongWord KILOBYTE = ((UnsignedLongWord)1024);

#ifdef PIXIE_SCHEMES

const UnsignedLongWord PIXIE_SCHEMES_MEM_SIZE_MBYTES     =  40;   // Also check text in the UI and in the help files.
const UnsignedLongWord PIXIE_SCHEMES_STACK_SIZE_MBYTES   =  4;

const UnsignedLongWord MAX_GENERATION_SPACE_SIZE           =  (1 * MEGABYTE + 2 * PAGE_SIZE);

const UnsignedLongWord INTERRUPT_INFORMATION_RESERVE_SIZE  =  (1 * MEGABYTE);
const UnsignedLongWord SPACE_RESERVE_SIZE                  =  ((size_t)(PIXIE_SCHEMES_MEM_SIZE_MBYTES * MEGABYTE + 2 * PAGE_SIZE));
const UnsignedLongWord STACK_RESERVE_SIZE                  =  ((size_t)(PIXIE_SCHEMES_STACK_SIZE_MBYTES * MEGABYTE + 2 * PAGE_SIZE));
const UnsignedLongWord CLOWDER_RESERVE_SIZE                =  ((size_t)(STACK_RESERVE_SIZE + 1 * MEGABYTE));
const UnsignedLongWord GGC_SCRATCH_RESERVE_SIZE            =  ((size_t)(2 * MAX_GENERATION_SPACE_SIZE));

const UnsignedLongWord MAXIMUM_MEMORY_SIZE = (1 * GIGABYTE);

#else

const UnsignedLongWord MAX_GENERATION_SPACE_SIZE           =  (101 * MEGABYTE);

const UnsignedLongWord INTERRUPT_INFORMATION_RESERVE_SIZE  =  (1 * MEGABYTE);
const UnsignedLongWord SPACE_RESERVE_SIZE                  =  ((size_t)(65 * GIGABYTE));
const UnsignedLongWord STACK_RESERVE_SIZE                  =  ((size_t)(7 * GIGABYTE));
const UnsignedLongWord CLOWDER_RESERVE_SIZE                =  ((size_t)(STACK_RESERVE_SIZE + 1 * MEGABYTE));  //  FIXME
const UnsignedLongWord GGC_SCRATCH_RESERVE_SIZE            =  ((size_t)(2 * MAX_GENERATION_SPACE_SIZE));

const UnsignedLongWord MAXIMUM_MEMORY_SIZE = (64 * GIGABYTE);

#endif

//const UnsignedWord MOMCAT_MMAP_FLAGS = (MAP_SHARED | MAP_FILE | MAP_FIXED | MAP_HASSEMAPHORE);  // OUT FOR TEST
//const UnsignedWord KITTEN_MMAP_FLAGS = (MAP_SHARED | MAP_FILE | MAP_FIXED | MAP_HASSEMAPHORE);  // OUT FOR TEST

const UnsignedWord MOMCAT_MMAP_FLAGS = (MAP_SHARED | MAP_FILE | MAP_FIXED);  // TEST
const UnsignedWord KITTEN_MMAP_FLAGS = (MAP_SHARED | MAP_FILE | MAP_FIXED);  // TEST

const UnsignedWord MMAP_PROT =         (PROT_READ | PROT_WRITE);

const UnsignedWord MOMCAT_MMAP_CREATE_FLAGS = (O_CREAT | O_RDWR | O_EXCL);
const UnsignedWord KITTEN_MMAP_CREATE_FLAGS = (O_RDWR);

extern void mmapForMomCat();
extern void mmapForKittens();

extern unsigned char *interruptInformationReserve;  // In MMapStuff.c++
extern unsigned char *spaceAReserve;                // In MMapStuff.c++
extern unsigned char *spaceBReserve;                // In MMapStuff.c++
extern unsigned char *scratchStackReserve;          // In MMapStuff.c++
extern unsigned char *gGCScratchReserve;            // In MMapStuff.c++
extern unsigned char *clowderReserve;               // In MMapStuff.c++

