/****************************************************************************************

Numbers.h -- Copyright (c) by Jay Reynolds Freeman 1988, 1989, 1990, 1991, 2006

Header file for stuff used in Wraith's numeric format routines.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import "SchemeCType.h"
#import "SECDIO.e"

#define DELIMITERS                          MiscStrings[ 162 ]
#define EXACTNESSES                         MiscStrings[ 163 ]
#define EXPONENT                            MiscChars[ 20 ]
#define SHORT_FLOAT_EXPONENT                MiscChars[ 39 ]
#define PRECISIONS                          MiscStrings[ 165 ]
#define RADICES                             MiscStrings[ 166 ]
#define SHARP_NS                            MiscChars[ 19 ]

#define BINARY_BASE                         MiscChars[ 21 ]
#define OCTAL_BASE                          MiscChars[ 22 ]
#define DECIMAL_BASE                        MiscChars[ 23 ]
#define HEX_BASE                            MiscChars[ 24 ]

#define PLUS_SIGN                           MiscChars[ 25 ]
#define MINUS_SIGN                          MiscChars[ 26 ]
#define RADIX_POINT                         MiscChars[ 27 ]

#define EXACT                               MiscChars[28]
#define INEXACT                             MiscChars[29]
// #define SHORT                               MiscChars[30]
// #define LONG                                MiscChars[31]

#define SLASH                               MiscChars[ 32 ]

#define EXPRESS                             MiscChars[ 33 ]
#define SURPRESS                            MiscChars[ 34 ]
#define HEURISTIC_H                         MiscChars[ 35 ]
        
#define MAX_BINARY_DIGITS                   MiscStrings[ 172 ]
#define MAX_OCTAL_DIGITS                    MiscStrings[ 173 ]
#define MAX_DECIMAL_DIGITS                  MiscStrings[ 174 ]
#define MAX_HEX_DIGITS                      MiscStrings[ 175 ]

#define MAX_BINARY_DIGITS_UNSIGNED          MiscStrings[ 420 ]
#define MAX_OCTAL_DIGITS_UNSIGNED           MiscStrings[ 421 ]
#define MAX_DECIMAL_DIGITS_UNSIGNED         MiscStrings[ 422 ]
#define MAX_HEX_DIGITS_UNSIGNED             MiscStrings[ 423 ]

//#define MAX_FIXNUM_AS_FLOAT	            (2147483000.0)
#define MAX_FIXNUM_AS_FLOAT	            ( 9223372036854775807.0)
//#define MIN_FIXNUM_AS_FLOAT	            (-2147483000.0)
#define MIN_FIXNUM_AS_FLOAT	            (-9223372036854775807.0)

//#define MAX_DECIMAL                         (2147483647)

#define MAX_FIXNUM_DECIMAL                          (9223372036854775807LL)
#define MIN_FIXNUM_DECIMAL                         (-9223372036854775807LL)

#define FENCE_SPLITTER                          (0x8000000000000000LL)

#define MAX_FIXNUM_AS_64_BIT_FLOAT	            ( 9007199254740991.0)
#define MIN_FIXNUM_AS_64_BIT_FLOAT	            (-9007199254740991.0)

#define MAX_FIXNUM_AS_64_BIT_FLOAT_DECIMAL                         ( 9007199254740991LL)
#define MIN_FIXNUM_AS_64_BIT_FLOAT_DECIMAL                         (-9007199254740991LL)

#define MAX_INT_ACCURATELY_REPRESENTABLE_AS_SHORT_FLOAT	            ( 9007199254740991LL)
#define MIN_INT_ACCURATELY_REPRESENTABLE_AS_SHORT_FLOAT	            (-9007199254740991LL)

#define MAX_OUTPUT_NUMBER_SIZE              ((UnsignedLongWord)5000)
    
#define translate(c)            (15 - ((Text *)strchr( ALL_DIGITS, (c) )  - ALL_DIGITS ))
#define getDigitChar( n )       (ALL_DIGITS[ 15 - (n) ])

#define absl( n )               ((LongWord)abs( (int)(n) ))

#define isExponentMarker( c )   ( strchr( MiscStrings[ 313 ], ( c ) ) != NULL )


#ifdef FLOATS_ARE_64_BIT

#define MAX_SHORT_FLOAT_EXACT_INTEGER        ((ShortFloat)9.007199254740991e15L)           // For IEEE 64-bit.

#define acosShortFloat( x ) ((ShortFloat)acos( x ))
#define asinShortFloat( x ) ((ShortFloat)asin( x ))
#define atanShortFloat( x ) ((ShortFloat)atan( x ))
#define atan2ShortFloat( x, y ) ((ShortFloat)atan2( (x), (y) ))
#define ceilShortFloat( x ) ((ShortFloat)ceil( x ))
#define cosShortFloat( x ) ((ShortFloat)cos( x ))
#define expShortFloat( x ) ((ShortFloat)exp( x ))
#define fabsShortFloat( x ) ((ShortFloat)fabs( x ))
#define floorShortFloat( x ) ((ShortFloat)floor( x ))
#define fmodShortFloat( x, y ) ((ShortFloat)fmod( (x), (y) ))
#define logShortFloat( x ) ((ShortFloat)log( x ))
#define modfShortFloat( x, y ) ((ShortFloat)modf( (x), (y) ))
#define powShortFloat( x, y ) ((ShortFloat)pow( (x), (y) ))
#define roundShortFloat( x ) ((ShortFloat)round( x ))
#define sinShortFloat( x ) ((ShortFloat)sin( x ))
#define sqrtShortFloat( x ) ((ShortFloat)sqrt( x ))
#define strtodShortFloat( x, y ) ((ShortFloat)strtod( (x), (y)))
#define tanShortFloat( x ) ((ShortFloat)tan( x ))

#else  // *not* FLOATS_ARE_64_BIT

#define MAX_SHORT_FLOAT_EXACT_INTEGER        ((ShortFloat)1.8446744073709551615e19L)           // For IEEE 80-bit.
// #define MAX_SHORT_FLOAT_EXACT_INTEGER        ((ShortFloat) 1.01412048018258e31L)  // For IEEE long double.

#define acosShortFloat( x ) ((ShortFloat)acosl( x ))
#define asinShortFloat( x ) ((ShortFloat)asinl( x ))
#define atanShortFloat( x ) ((ShortFloat)atanl( x ))
#define atan2ShortFloat( x, y ) ((ShortFloat)atan2l( (x), (y) ))
#define ceilShortFloat( x ) ((ShortFloat)ceill( x ))
#define cosShortFloat( x ) ((ShortFloat)cosl( x ))
#define expShortFloat( x ) ((ShortFloat)expl( x ))
#define fabsShortFloat( x ) ((ShortFloat)fabsl( x ))
#define floorShortFloat( x ) ((ShortFloat)floorl( x ))
#define fmodShortFloat( x, y ) ((ShortFloat)fmodl( (x), (y) ))
#define logShortFloat( x ) ((ShortFloat)logl( x ))
#define modfShortFloat( x, y ) ((ShortFloat)modfl( (x), (y) ))
#define powShortFloat( x, y ) ((ShortFloat)powl( (x), (y) ))
#define roundShortFloat( x ) ((ShortFloat)roundl( x ))
#define sinShortFloat( x ) ((ShortFloat)sinl( x ))
#define sqrtShortFloat( x ) ((ShortFloat)sqrtl( x ))
#define strtodShortFloat( x, y ) ((ShortFloat)strtold( (x), (y)))
#define tanShortFloat( x ) ((ShortFloat)tanl( x ))

#endif  // FLOATS_ARE_64_BIT

// ShortFloat <--> 128-bit long double -- this would likely work in such an implementation.
// LongWord maxShortFloatDigits[17] = {  0, 0, 104, 0, 0, 0, 0, 0, 35, 0, 32, 0, 0, 0, 0, 0, 27 };

