/****************************************************************************************

OpCodes.h -- Copyright (c) by Jay Reynolds Freeman 1988, 1989, 1990, 1991, 2006

Header file for SECD Machine "OpCode", macros widely used in Wraith.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#ifndef __OPCODESHEADER__
#define __OPCODESHEADER__

#import "Registers.h"
#import "Apply.e"
#import "SECDEval.e"
#import "SECDFloatStuff.e"
#import "SECDIO.e"
#import "SECDMachine.e"
#import "SECDRegisters.e"
#import "Registers.e"

/******** Syntactic sugar: ********/

#define END_PROC                            }
#define PROC( name )                    void name() {

/******** "Real" op codes: ********/

#define ACOS                                 acosMC();
#define ASIN                                 asinMC();
#define ATAN                                 atanMC();
#define ATAN2                                atan2MC();
#define ADD                                  add();
#define APPLY                                apply();
#define ARITY_CHECK                          arityCheck();
#define BIG_ARRAY_NUMBER                     getBigArrayNumberMC();
#define BIT_AND                              bitAnd();
#define BIT_NOT                              bitNot();
#define BIT_OR                               bitOr();
#define BIT_XOR                              bitXor();
#define BIT_SHIFT_LEFT                       bitShiftLeft();
#define BIT_SHIFT_RIGHT_ARITHMETIC           bitShiftRightArithmetic();
#define BIT_SHIFT_RIGHT_LOGICAL              bitShiftRightLogical();
#define BUMP( object )                       ( (object).u.a = (object).u.a + 1 );
#define BUMP_N( object, num )                ( (object).u.a = (object).u.a + (LongWord)((num).u.c) );
#define CALL( foo )                          (foo)();
#define CAR                                  car();
#define CDR                                  cdr();
#define CEILING                              ceilingMC();
#define CHAR_TO_LOWER                        charToLower();
#define CHAR_TO_UPPER                        charToUpper();
#define CLEAR_B                              (B = FALSE);
#define CLEAR_R_FROM_B                       ( R.t = (B ? falseBooleanTag : trueBooleanTag) );
#define CLOSE_PORT                           closePort();
#define CLOSE_TRANSCRIPT                     closeTranscript();
#define COERCE_R_SHORT_FIXNUM                coerceRToShortFixnum();
#define COERCE_S0_SHORT_FIXNUM               coerceS0ToShortFixnum();
#define COERCE_SHORT_FIXNUM                  coerceRToShortFixnum();
#define COERCE_SHORT_FLOAT                   coerceRToShortFLoat();
#define COMPARE                              (B = ( R.t == (STACK( 0 )).t) && ( R.u.c == (STACK( 0 )).u.c));
#define CONS                                 cons();
#define CONTINUE                             return;
#define COS                                  cosMC();
#define DEBUG_ENTRY                          debugEntry();
#define DECREMENT_R                          (R.u.c = R.u.c - 1);
#define DESCRIBE_ROOM                        describeRoomOp();
#define DESCRIBE_SPACE                       describeCurrentMainSpace();
#define DESCRIBE_STACK                       writeStack();
#define DISPLAY                              displayObject();
#define DISPLAY_ON_PORT                      displayObjectOnPort();
#define DIVIDE                               divide();
#define DROP                                 SDrop();
#define DROP_N                               dropNFromStack();
#define DUP                                  SDupR();
#define ENV_REF                              dereferenceEnvironmentReferenceMC();
#define EVAL_SYMBOL                          lookUp();
#define EXP                                  expMC();
#define EXPT                                 exptMC();
//#define EXCHANGE( foo, bar)                  { taggedAval dummy; (dummy = foo, foo = bar, bar = dummy); }
//#define EXCHANGE( foo, bar)                  ( A = (foo), (foo) = (bar), (bar) = A );
#define EXCHANGE( foo, bar)                  exchangeFunction( &(foo), &(bar) );
#define FLOOR                                floorMC();
#define GCD                                  gcdMC();
#define GENSYM                               genSymMC();
#define GETC                                 getNextChar();
#define GETC_LOWER                           getNextCharLower();
#define GETC_RAW                             getNextCharRaw();
#define GET_HASH_VALUE                       getHashValue();
//#define GET_INPUT_DESCRIPTOR                 getInputDescriptor();
//#define GET_OUTPUT_DESCRIPTOR                getOutputDescriptor();
#define GET_PNAME                            getPName();
#define GET_SYMBOL_BINDING_VALUE             getSymbolBindingValue();
#define GET_STARTUP_WORKING_FOLDER           getStartupWorkingFolder();
#define GET_WORKING_FOLDER                   getWorkingFolder();
#define GET_INIT_FILE_PATH                   getInitFilePath();
#define HASH_STRING                          hashString();
#define HASH_TEXT                            hashText();
#define INCREMENT_R                          (R.u.c = R.u.c + 1);
#define INSPECT                              inspectObject();
#define INSPECT_ON_PORT                      inspectObjectOnPort();
#define JUMP_FUNCTION( foo )                 { ((pFVoid)(foo))(); return; }
#define JUMP_FALSE( L )                      if( ! B ) goto L;
#define JUMP_LABEL( L )                      goto L;
#define JUMP_TRUE( L  )                      if( B ) goto L;
#define LCM                                  lcmMC();
#define LENGTH                               lengthOp();
#define LIST_TOP_N                           listTopN();
//#define LOAD_CONTINUATION                    loadContinuation();
#define LOAD_CONTINUATION                    CSPushProperList();
#define LOAD_FRAME( S, M )                   LoadFrame( S, M );
#define LOAD_OBJECT( VALUE, X )              (X).u.c = (LongWord)(VALUE);
#define LOAD_CHARACTER_OBJECT( VALUE, X )    (X).u.c = (VALUE);
#define LOAD_STACK                           pushListOnStack();
#define LOAD_TAG( TAG, X )                   (X).t = (Tag)(TAG);
#define LOAD_R_TAG( TAG )                    R.t = (TAG);
#define LOAD_WORLD                           theHeap->loadWorld();
#define LOOK_UP_BINDING_PAIR                 lookUpBindingPair();
#define EXAMINE_WORLD                        theHeap->examineWorld( false, 0 );
#define LOG                                  logMC();
#define MAKE_EMPTY_STRING                    makeStringNEmpty( (LongWord)(R.u.c) );
#define MAKE_FRAME                           MakeFrame();
#define MAKE_STRING                          makeString( (Text *)(R.u.c) );
#define MOVE( foo, bar )                     (bar) = (foo);
#define MULTIPLY                             multiply();
#define NILCONS                              nilCons();
#define NUMBER_TO_STRING                     numberToStringMC();
#define OBJECT( X )                          ((X).u.c)
#define OPEN_INPUT_NAMED                     openInputNamed();
#define OPEN_OUTPUT_NAMED                    openOutputNamed();
#define OPEN_READ_SOCKET_GIVEN               openReadSocketGiven();
#define OPEN_TRANSCRIPT_NAMED                openTranscriptNamed();
#define OPEN_DIRECTORY_SF                    openDirectorySF();
#define OPEN_INPUT_SF                        openInputSF();
#define OPEN_OUTPUT_SF                       openOutputSF();
#define OPEN_TRANSCRIPT_SF                   openTranscriptSF();
#define OUTPUT_TO_ERROR                      useErrorOutput();
#define PEEKC                                peek();
#define PECULIAR_NON_LEAF_NUMBER             getPeculiarNonLeafNumberMC();
#define POP                                  SPopR();
#define POP_A                                SPopA();
#define PORT_NUMBER                          getPortBigArrayNumberMC();
#define PURE_TYPE_CODE( X )                  pure_type_code( (X).t )
#define PUSH                                 SPushR();
#define PUSH_CONTINUATION                    CSPush( R );
#define QUOTIENT                             quotient2();
#define REMAINDER                            remainder2();
#define RETURN_DEFAULT                       LOAD_R_TAG( trueBooleanTag )
#define REVERSE_TOP_N                        reverseTopN();
#define ROUND                                roundMC();
#define SAVE_WORLD                           theHeap->saveWorld();
#define STACK_DEPTH                          stackDepth();
#define SETCAR                               setCar();
#define SETCDR                               setCdr();
#define SET_B                                (B = TRUE );
#define SET_B_FROM( X )                      ( B = (X) ? TRUE : FALSE );
#define SET_EXACT_FROM_B                     ( B ? (R.t |= (exactBitMask)) : (R.t &= (~(exactBitMask))));
#define SET_R_FROM( X )                      ( R.t = ((X) ? trueBooleanTag : falseBooleanTag) );
#define SET_R_FROM_B                         ( R.t = (B ? trueBooleanTag : falseBooleanTag) );
#define SET_WORKING_FOLDER                   setWorkingFolder();
#define SET_WORKING_FOLDER_INTERACTIVELY     setWorkingFolderInteractively();
#define SIN                                  sinMC();
#define SQRT                                 sqrtMC();
#define STACK( N )                           (S[N])
#define STORE_U_A( DATA, ADDRESS)            ((((taggedAval *)((ADDRESS).u.a))->u).a = (DATA).u.a);
#define STRING_APPEND                        stringAppend();
#define STRING_COPY                          stringCopy();
#define STRING_INSERT                        stringInsert();
#define STRING_LENGTH                        stringLength();
#define STRING_LENGTH_MANY                   stringLengthMany();
#define STRING_REF                           stringRef();
#define STRING_SET                           stringSet();
#define STRING_TO_LOWER                      stringToLower();
#define STRING_TO_NUMBER                     stringToNumber();
#define STRING_TO_UPPER                      stringToUpper();
#define SUBSTRING                            substring();
#define SUBTRACT                             subtract();
#define SWAP                                 Swap();
#define TAG( X )                             ((X).t)
#define TAN                                  tanMC();
#define TOP_LEVEL_ENV_REF                    dereferenceTopLevelEnvironmentReferenceMC();
#define TOP_LEVEL_ENV_REF_SYMBOL             dereferenceTopLevelEnvironmentReferenceForSymbolMC();
#define TRUNCATE                             truncateMC();
#define UNBUMP( object )                     ( (object).u.a = (object).u.a - 1 );

#ifndef USE_COCOA_IO

#define USLEEP                               usleep( (Word)((R.u.c) & WORD_MASK) );

#else

#import "CocoaInterface.h"

#define USLEEP                               { tellController( E_EVALUATING, 0, 0 ); tellController( E_SLEEPING, 0, 0 ); usleep( (Word)((R.u.c) & WORD_MASK) ); reportEvent( WAKED_UP_EVENT ); }

#endif

#define VLENGTH                              vectorLength();
#define VREF                                 vectorRef();
#define VSET                                 vectorSet();
#define WRITE                                writeObject();
#define WRITE_ON_PORT                        writeObjectOnPort();
#define WRITE_CHAR                           writeChar();
#define WRITE_CHAR_ON_PORT                   writeCharOnPort();
#define WRITE_NEWLINE_ERROR                  writeNewlineError();

/**** Booleans -- leave result in B *****/

#define ATOM_P                               atomPMC();
#define BOUND_SYMBOL_P                       { PUSH boundSymbolP(); POP }
#define BOOLEAN_P                            booleanPMC();
#define BUILT_IN_FUNCTION_P                  builtInFunctionPMC();
#define BYTE_BLOCK_P                         byteBlockPMC();
#define CHARACTER_P                          characterPMC();
#define CHAR_ALPHA_P                         charIsAlphaPMC();
#define CHAR_LOWER_P                         charIsLowerPMC();
#define CHAR_NUMERIC_P                       charIsNumericPMC();
#define CHAR_READY_P                         charReadyPMC();
#define CHAR_UPPER_P                         charIsUpperPMC();
#define CHAR_WHITESPACE_P                    charIsWhitespacePMC();
#define COMPLEX_P                            complexPMC();
#define EMPTY_LIST_IS_FALSE_P                B = EmptyListIsFalseFlag;
#define EMPTY_LIST_P                         emptyListPMC();
#define ENVIRONMENT_REFERENCE_P              environmentReferencePMC();
#define EOF_P                                B = isEOF( R.u.c );
#define EQ_P                                 eqPMC();
#define EQUAL_P                              equalPMC();
#define EQV_P                                eqvPMC();
#define EVEN_P                               evenPMC();
#define EXACT_P                              exactPMC();
#define EXACT_INTEGER_P                      exactIntegerPMC();
#define EXACT_SHORT_FLOAT_P                  exactShortFloatPMC();
#define EXACT_SHORT_FIXNUM_P                 exactShortFixnumPMC();
#define FAIL_P                               failPMC();
#define FALSE_P                              falsePMC();
#define FILE_EXISTS_P                        fileExistsPMC();
#define FIXNUM_P                             fixnumPMC();
#define FLOAT_P                              floatPMC();
#define FORGETTABLE_P                        forgettablePMC();
#define FORGETTABLE_FORGOTTEN_P              forgettableForgottenPMC();
#define FORGETTABLE_REMEMBERED_P             forgettableRememberedPMC();
#define FORGETTABLE_UNEXPIRED_P              forgettableUnexpiredPMC();
#define FORWARDING_POINTER_P                 forwardingPointerPMC();
#define GGC_FORWARDING_POINTER_P             ggcForwardingPointerPMC();
#define IMMUTABLE_P                          immutablePMC();
#define IMMUTABLE_P_S0                       B = (immutable_p( (S[0]).t ) != 0);
#define IMMUTABLE_P_S1                       B = (immutable_p( (S[1]).t ) != 0);
#define IMMUTABLE_STRING_P                   B = (immutable_p( ((taggedAval *)(R.u.a))->t ) != 0 );
#define INEXACT_P                            inexactPMC();
#define INEXACT_SHORT_FLOAT_P                inexactShortFloatPMC();
#define INEXACT_SHORT_FIXNUM_P               inexactShortFixnumPMC();
#define INF_P                                infPMC();
#define INPUT_PORT_P                         inputPortPMC();
#define INTEGER_P                            integerPMC();
#define INTERRUPT_RANGE_CHECK_P              interruptRangeCheckPMC();
#define KITTEN_RANGE_CHECK_P                 kittenRangeCheckPMC();
#define LAMBDA_PAIR_P                        lambdaPairPMC();
#define LETTER_P                             letterPMC();
#define LEAF_P                               B = (leaf_p( R.t ));
#define NON_LEAF_P                           nonLeafPMC();
#define LEVEL_INDICATOR_NUMBER_RANGE_CHECK_P levelIndicatorNumberRangeCheckPMC();
#define LEVEL_INDICATOR_VALUE_RANGE_CHECK_P  levelIndicatorValueRangeCheckPMC();
#define LIST_P                               listPMC();
#define LONG_P                               longPMC();
#define MACRO_P                              macroPMC();
#define MMAPPED_BLOCK_P                      mmappedBlockPMC();
#define MULTIPLE_VALUES_P                    multipleValuesPMC();
#define MINUS_ONE_FIXNUM_P                   B = ( (LongWord)R.u.c == -1 );
#define NAN_P                                nanPMC();
#define NAN_OR_INF_P                         nanOrInfPMC();
#define NON_NEGATIVE_INTEGER_P               nonNegativeIntegerPMC();
#define NON_PRINTING_OBJECT_P                nonPrintingObjectPMC();
#define NUMBER_P                             numberPMC();
#define NUM_EQ_P                             numEqualPMC();
#define NUM_GE_P                             numGreaterEqualPMC();
#define NUM_GT_P                             numGreaterPMC();
#define NUM_LE_P                             numLessEqualPMC();
#define NUM_LT_P                             numLessPMC();
#define OBJECT_EQ_P( X, Y )                  B = ((X).u.c == (Y).u.c);
#define OBJECT_GE_P( X, Y )                  B = ((X).u.c >= (Y).u.c);
#define OBJECT_GT_P( X, Y )                  B = ((X).u.c > (Y).u.c);
#define OBJECT_LE_P( X, Y )                  B = ((X).u.c <= (Y).u.c);
#define OBJECT_LT_P( X, Y )                  B = ((X).u.c < (Y).u.c);
#define ODD_P                                oddPMC();
#define ONE_FIXNUM_P                         B = ( (LongWord)R.u.c == 1 );
#define OUTPUT_PORT_P                        outputPortPMC();
#define PAIR_P                               pairPMC();
#define PECULIAR_LEAF_P                      peculiarLeafPMC();
#define PECULIAR_NON_LEAF_P                  peculiarNonLeafPMC();
#define POINTER_EQUAL_P                      B = ((R.u.a) == (S[0]).u.a);
#define POINTER_TO_BYTE_BLOCK_P              pointerToByteBlockPMC();
#define PORT_P                               portPMC();
#define POSITIVE_INTEGER_P                   positiveIntegerPMC();
#define PROCEDURE_P                          procedurePMC();
#define PROMISE_P                            promisePMC();
#define PROPER_LIST_P                        properListPMC();
#define PUSHBUTTON_NUMBER_RANGE_CHECK_P      pushbuttonNumberRangeCheckPMC();
#define RADIX_P                              radixPMC();
#define RANGE_CHECK_SIGNED_FIXNUM            rangeCheckAsSignedFixnumMC();
#define RANGE_CHECK_UNSIGNED_FIXNUM          rangeCheckAsUnsignedFixnumMC();
#define RATIONAL_P                           rationalPMC();
#define REAL_P                               realPMC();
#define SCHEME_MACHINE_PC_P                  schemeMachinePCPMC();
#define SCHEME_MACHINE_PC_INDEX_P            schemeMachinePCIndexPMC();
#define SHORT_FIXNUM_P                       shortFixnumPMC();
#define SLIDER_NUMBER_RANGE_CHECK_P          sliderNumberRangeCheckPMC();
#define SPECIAL_FORM_P                       specialFormPMC();
#define STACK_REFERENCE_P                    stackReferencePMC();
#define STRING_P                             stringPMC();
#define STRING_EQUALS_TEXT_P                 stringEqualsTextPMC();
#define STRING_EQUAL_P                       stringEqual();
#define STRING_GE_P                          stringGreaterEqualPMC();
#define STRING_GT_P                          stringGreaterPMC();
#define STRING_LE_P                          stringLessEqualPMC();
#define STRING_LT_P                          stringLessPMC();
#define STRING_EQUAL_CI_P                    stringEqualCI();
#define STRING_GE_CI_P                       stringGreaterEqualCIPMC();
#define STRING_GT_CI_P                       stringGreaterCIPMC();
#define STRING_LE_CI_P                       stringLessEqualCIPMC();
#define STRING_LT_CI_P                       stringLessCIPMC();
#define SUCCEED_P                            succeedPMC();
#define SYMBOL_P                             symbolPMC();
#define SYNTAX_BINDING_P                     syntaxBindingPMC();
#define TOP_LEVEL_ENVIRONMENT_REFERENCE_P    topLevelEnvironmentReferencePMC();
#define TOP_N_SAME_LENGTH_P                  topNSameLength();
#define TRUE_P                               truePMC();
#define TWO_FIXNUM_P                         B = ( (LongWord)R.u.c == 2 );
#define VECTOR_P                             vectorPMC();
#define ZERO_FIXNUM_P                        B = ( (LongWord)R.u.c == 0 );

#endif
