/****************************************************************************************

SchemeCType.h -- Copyright (c) by Jay Reynolds Freeman 1988, 1989, 1990, 1991, 2006

Header file for type definitions that deal with Scheme's notion of the character set.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#ifndef CHARTYPES_HERE

extern UnsignedWord *pCharTypes;

#endif

#import <CType.h>

#define BACKSLASH                               MiscChars[ 6 ]
#define COMMENT_PREFIX                          MiscChars[ 8 ]
#define CONS_DOTTER                             MiscChars[ 0 ]
#define DOUBLE_QUOTE                            MiscChars[ 5 ]
#define FALSE_F                                 MiscChars[ 17 ]
#define LIST_CLOSER                             MiscChars[ 4 ]
#define LIST_OPENER                             MiscChars[ 3 ]
#define OPTION_PLUS                             MiscChars[ 36 ]
#define OPTION_MINUS                            MiscChars[ 37 ]
#define OPTION_STAR                             MiscChars[ 38 ]
#define QUASIQUOTER                             MiscChars[ 9 ]
#define QUOTER                                  MiscChars[ 13 ]
#define RETURN                                  MiscChars[ 14 ]
#define SHARP                                   MiscChars[ 2 ]
#define TRUTH_T                                 MiscChars[ 16 ]
#define UNQUOTER                                MiscChars[ 10 ]
#define UNQUOTE_SPLICING_SECOND_CHARACTER       MiscChars[ 12 ]
#define PECULIAR_SUBSEQUENT_DOT                 MiscChars[ 40 ]
#define PECULIAR_INITIAL_DOT                    MiscChars[ 41 ]
#define PECULIAR_INITIAL_PLUS                   MiscChars[ 42 ]
#define PECULIAR_INITIAL_MINUS                  MiscChars[ 43 ]
#define FAIL_U                                  MiscChars[ 44 ]
#define SUCCEED_S                               MiscChars[ 45 ]
#define FRONTSLASH                              MiscChars[ 46 ]
#define NON_PRINT_N                             MiscChars[ 47 ]

#define ALL_DIGITS                              MiscStrings[ 161 ]
#define HEX_DIGITS                              (ALL_DIGITS)
#define DECIMAL_DIGITS                          (ALL_DIGITS + 6)
#define OCTAL_DIGITS                            (ALL_DIGITS + 8)
#define BINARY_DIGITS                           (ALL_DIGITS + 14)

#define ALPHABET                                MiscStrings[ 176 ]
#define ALPHABET_LOWER                          MiscStrings[ 178 ]
#define ALPHABET_UPPER                          MiscStrings[ 177 ]

#define extraDelimiters                         MiscStrings[ 168 ]
#define miscellaneousChars                      MiscStrings[ 202 ]
#define peculiarInitials                        MiscStrings[ 167 ]
#define peculiarSubsequents                     MiscStrings[ 329 ]
#define specialInitials                         MiscStrings[ 169 ]
#define specialSubsequents                      MiscStrings[ 170 ]
#define whiteSpace                              MiscStrings[ 171 ]
#define weirdChars                              MiscStrings[ 251 ]

Text *strchr();

#define isDelimiter_M( c )                   ( isWhiteSpace_M( c ) || (strchr( extraDelimiters, ( c ) ) != (Text *)NULL) )
#define isEOF( n )                           ( (LongWord)(n) == -1 )
#define isInitial_M( c )                     ( isSchemeAlpha_M( c ) || (strchr( specialInitials, ( c ) ) != (Text *)NULL ) )
#define isMisc_M( c )                        ( strchr( miscellaneousChars, ( c ) ) != (Text *)NULL )
#define isPeculiarInitial_M( c )             ( strchr( peculiarInitials, ( c ) ) != (Text *)NULL )
#define isPeculiarSubsequent_M( c )          ( strchr( peculiarSubsequents, ( c ) ) != (Text *)NULL )
#define isSchemeAlpha_M( c )                 ( strchr( ALPHABET, ( c ) ) != (Text *)NULL )
#define isSchemeDigit_M( c )                 ( strchr( DECIMAL_DIGITS, ( c ) ) != (Text *)NULL )
#define isSchemeLegalCharacter_M( c )        (     isInitial_M( c )   || isSubsequent_M( c ) \
                                                || isDelimiter_M( c ) || isWhiteSpace_M( c ) \
                                                || isMisc_M( c )      || isWeird_M( c ) )
#define isSchemeLower_M( c )                 ( strchr( ALPHABET_LOWER, ( c ) ) != (Text *)NULL )
#define isSchemeUpper_M( c )                 ( strchr( ALPHABET_UPPER, ( c ) ) != (Text *)NULL )
#define isSubsequent_M( c )                  (    isInitial_M( c ) || isSchemeDigit_M( c ) \
                                               || (strchr( specialSubsequents, ( c ) ) != (Text *)NULL ) )
#define isWhiteSpace_M( c )                  ( strchr( whiteSpace, ( c ) ) != (Text *)NULL )
#define isWeird_M( c )                       ( strchr( weirdChars, ( c ) ) != (Text *)NULL )

#define DELIMITER_BIT               0x1
#define INITIAL_BIT                 0x2
#define MISC_BIT                    0x4
#define PECULIAR_INITIAL_BIT        0x8
#define SCHEME_ALPHA_BIT            0x10
#define SCHEME_DIGIT_BIT            0x20
#define SCHEME_LEGAL_BIT            0x40
#define SCHEME_LOWER_BIT            0x80
#define SCHEME_UPPER_BIT            0x100
#define SUBSEQUENT_BIT              0x200
#define WHITESPACE_BIT              0x400
#define WEIRDNESS_BIT               0x800
#define PECULIAR_SUBSEQUENT_BIT     0x1000

#define FF_MASK                     0xFF

#define isDelimiter( c )                        (( (pCharTypes)[ ((c) & FF_MASK) ] & DELIMITER_BIT )            != 0)
#define isInitial( c )                          (( (pCharTypes)[ ((c) & FF_MASK) ] & INITIAL_BIT )              != 0)
#define isMisc( c )                             (( (pCharTypes)[ ((c) & FF_MASK) ] & MISC_BIT )                 != 0)
#define isPeculiarInitial( c )                  (( (pCharTypes)[ ((c) & FF_MASK) ] & PECULIAR_INITIAL_BIT )     != 0)
#define isPeculiarSubseauent( c )               (( (pCharTypes)[ ((c) & FF_MASK) ] & PECULIAR_SUBSEQUENT_BIT )  != 0)
#define isSchemeAlpha( c )                      (( (pCharTypes)[ ((c) & FF_MASK) ] & SCHEME_ALPHA_BIT )         != 0)
#define isSchemeDigit( c )                      (( (pCharTypes)[ ((c) & FF_MASK) ] & SCHEME_DIGIT_BIT )         != 0)
#define isSchemeLegalCharacter( c )             (( (pCharTypes)[ ((c) & FF_MASK) ] & SCHEME_LEGAL_BIT )         != 0)
#define isSchemeLower( c )                      (( (pCharTypes)[ ((c) & FF_MASK) ] & SCHEME_LOWER_BIT )         != 0)
#define isSchemeUpper( c )                      (( (pCharTypes)[ ((c) & FF_MASK) ] & SCHEME_UPPER_BIT )         != 0)
#define isSubsequent( c )                       (( (pCharTypes)[ ((c) & FF_MASK) ] & SUBSEQUENT_BIT )           != 0)
#define isWhiteSpace( c )                       (( (pCharTypes)[ ((c) & FF_MASK) ] & WHITESPACE_BIT )           != 0)
#define isWeird( c )                            (( (pCharTypes)[ ((c) & FF_MASK) ] & WEIRDNESS_BIT )            != 0)

