/****************************************************************************************

SchemeFunctions.h -- Copyright (c) by Jay Reynolds Freeman 1988, 1989, 1990, 1991, 2006

Header file for the Scheme built-in functions used in Wraith Scheme.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import "WraithHeader.h"

/**** functionDescriptorTypeOne ****/

typedef struct {
	void	 		  (*code)();		         /* code to do the work */
	LongWord 		  pNameNumber;		         /* index of the print name in a table */
	Boolean           returnsValueP;    	     /* does function return a meaningful value? */
	void       	  	  (*resultTypeChecker)();    /* function which could type-check the result */
	Tag 			  resultTag;		         /* the least special tag that can be returned (?) */
	UnsignedLongWord  nArgs;			         /* number of args pushed on the stack for the
						                		    code to work on; also the length of the
								                    array immediately following */
	void       	  	  (**argTypeChecker)();      /* functions to use to type-check the pushed args */
    const Text        **argDescription;          /* to name the args in error messages */
	} functionDescriptorTypeOne;

/**** functionDescriptorTypeFive ****/

typedef struct {
	void	 		  (*code)();		         /* code to do the work */
	LongWord 		  pNameNumber;		         /* index of the print name in a table */
	Boolean           returnsValueP;             /* does function return a meaningful value? */
	void       	  	  (*resultTypeChecker)();    /* function which could type-check the result */
	Tag 			  resultTag;		         /* the least special tag that can be returned (?) */
	LongWord		  minArgs;		             /* minimum number of arguments */
	void       	  	  (*argTypeChecker)();	     /* function to use to type-check the args */
    const Text        *argDescription;           /* to name the args in error messages */
	} functionDescriptorTypeFive;

/**** functionDescriptorTypeSix ****/

typedef struct {
	void	 		  (*code)();		         /* code to do the work */
	LongWord 		  pNameNumber;		         /* index of the print name in a table */
	Boolean           returnsValueP;             /* does function return a meaningful value? */
	void       	  	  (*resultTypeChecker)();    /* function which could type-check the result */
	Tag 			  resultTag;		         /* the least special tag that can be returned (?) */
	LongWord		  maxArgs;		             /* maximum number of arguments; -1 means infinity */
	void       	  	  (*firstArgTypeChecker)();  /* function to use to type-check the first arg */
	void       	  	  (*restArgTypeChecker)();   /* function to use to type-check the other args */
	const Text        *firstArgDescription;      /* to name the args in error messages */
	const Text        *restArgDescription;       /* to name the args in error messages */
	} functionDescriptorTypeSix;
	
/**** functionDescriptorTypeSeven ****/

typedef struct {
	void	 		  (*code)();		         /* code to do the work */
	LongWord 		  pNameNumber;		         /* index of the print name in a table */
	Boolean           returnsValueP;             /* does function return a meaningful value? */
	void       	  	  (*resultTypeChecker)();    /* function which could type-check the result */
	Tag 			  resultTag;		         /* the least special tag that can be returned (?) */
	LongWord		  maxArgs;		             /* maximum number of arguments; -1 means infinity */
	void       	  	  (*firstArgTypeChecker)();  /* function to use to type-check the first arg */
	void       	  	  (*restArgTypeChecker)();   /* function to use to type-check the other args */
	LongWord		  minArgs;		             /* minimum number of arguments */
	const Text        *firstArgDescription;      /* to name the args in error messages */
	const Text        *restArgDescription;       /* to name the args in error messages */
	} functionDescriptorTypeSeven;
	
/**** functionDescriptorTypeEight ****/

typedef struct {
	void	 		  (*code)();		         /* code to do the work */
	LongWord 		  pNameNumber;		         /* index of the print name in a table */
    Boolean           returnsValueP;             /* does function return a meaningful value? */
	void       	  	  (*resultTypeChecker)();    /* function which could type-check the result */
	Tag 			  resultTag;    		     /* the least special tag that can be returned (?) */
	LongWord		  maxArgs;	        	     /* maximum number of arguments; -1 means infinity */
	void       	  	  (*firstArgTypeChecker)();  /* function to use to type-check the other args */
	void       	  	  (*restArgTypeChecker)();   /* function to use to type-check the last arg */
	LongWord		  minArgs;	        	     /* minimum number of arguments */
	const Text        *firstArgDescription;      /* to name the args in error messages */
	const Text        *restArgDescription;       /* to name the args in error messages */
	} functionDescriptorTypeEight;
	
/**** functionDescriptorTypeNine ****/           // Just like type six except that zero arguments is permitted.

typedef struct {
	void	 		  (*code)();		         /* code to do the work */
	LongWord 		  pNameNumber;		         /* index of the print name in a table */
	Boolean           returnsValueP;             /* does function return a meaningful value? */
	void       	  	  (*resultTypeChecker)();    /* function which could type-check the result */
	Tag 			  resultTag;		         /* the least special tag that can be returned (?) */
	LongWord		  maxArgs;		             /* maximum number of arguments; -1 means infinity */
	void       	  	  (*firstArgTypeChecker)();  /* function to use to type-check the first arg */
	void       	  	  (*restArgTypeChecker)();   /* function to use to type-check the other args */
	const Text        *firstArgDescription;      /* to name the args in error messages */
	const Text        *restArgDescription;       /* to name the args in error messages */
	} functionDescriptorTypeNine;
	
