/****************************************************************************************

WorldHandler.h -- Copyright (c) by Jay Reynolds Freeman 2007.

World-handling class (load, save, et cetera), for Wraith Scheme.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#ifndef __WORLDHANDLER_H__
#define __WORLDHANDLER_H__

#import "WraithHeader.h"
#import "DataTypes.h"
#import "UserSetup.h"

//  For correct interpretations of old worlds, add new entries only at the end of this struct:

struct tableHeader {
    Text versionString[ 256 ];
    UnsignedLongWord topECursor;
    UnsignedLongWord heapSizeWanted;
    UnsignedLongWord tableSizeExpected;
    UnsignedLongWord nKittensExpected;
    LongWord prDepth; 
    LongWord prLength; 
    LongWord vPrDepth;
    LongWord vPrLength;
    UnsignedLongWord worldType;
    UnsignedLongWord shortFloatSize;
    UnsignedLongWord ObListCursor;
    UnsignedLongWord TLCCursor;
    UnsignedLongWord IVRCursor;
    UnsignedLongWord extra1;         //  A few unallocated slots to save fuss in case one is needed for development.
    UnsignedLongWord extra2;
    UnsignedLongWord extra3;
    UnsignedLongWord stringLength;   //  Null-terminated string starts after the heap image, this long not including the null.
    userSetup savedUserSetup;
    };      

typedef struct {
    UnsignedLongWord tableCursor;
    UnsignedLongWord codePointer;
    } cursorAndCode;

class MemoryManager;
class GarbageCollector;

#define BIG_ENOUGH_TABLE_SIZE  1024 //  As of about Xcode 3.2.3, there is something very odd about the compiler's handling
                                    //  of dynamically allocated complicated tables; this is part of a hacked-up work around.

class WorldHandler {

  public:

    void initializeWH( Boolean bEFlag, Text *externalNameString, size_t externalNameStringSize );
    UnsignedLongWord *theSimpleTable;
    UnsignedLongWord theUnsortedTableSize;

    void loadWorld( volatile memorySpace *where, volatile memorySpace *alternate );
    void loadWorldGuts( fileDescriptor *wrldFile, volatile memorySpace *where, volatile memorySpace *alternate, Boolean describeP );
    Boolean worldIsUseable( Text *pathToWorld, volatile memorySpace *where );
    void saveUserSetupToFileNamed( Text *setupFileName );
    void saveWorld( volatile memorySpace *where, volatile memorySpace *alternate );
    void examineWorld( Boolean compareP, Text *fileName, volatile memorySpace *where, volatile memorySpace *alternate );
    Text *lastWorldLoadedName;
    size_t lastWorldLoadedNameSize;
    void createUnsortedTable();
    UnsignedLongWord pcForIndex( UnsignedLongWord anIndex );
    UnsignedLongWord indexForPC( UnsignedLongWord aPC );

    void setUserSetup( userSetup *aUserSetup );
    void readUserSetupFromFileNamed( Text *fileName );
    void retrieveUserSetup( userSetup *aUserSetup );

    void setTheString( Text *aString );
    void freeTheString();
    Text *getTheString();

 private:

    Boolean BigEndianFlag;
    userSetup theUserSetup;
    Text *theString;

    UnsignedLongWord codePointerTable[ BIG_ENOUGH_TABLE_SIZE ];
    cursorAndCode theSortedTable[ BIG_ENOUGH_TABLE_SIZE ];
    cursorAndCode theUnsortedTable[ BIG_ENOUGH_TABLE_SIZE ];

    void describeTableHeader( struct tableHeader *th );
    void describeTable( UnsignedLongWord *tab );
    Boolean badPointerAfterLoad( volatile memorySpace *where, taggedAval *tp );
    void fixPointerForLoad( volatile memorySpace *where, taggedAval *tp );
    void fixPointerForSave( volatile memorySpace *where, taggedAval *tp );
    void saveWorldGuts( fileDescriptor *wrldFile, volatile memorySpace *where, volatile memorySpace *alternate );
    void printfCodePointerTable( cursorAndCode *table, UnsignedLongWord tableSize );
    };   

#endif
