/****************************************************************************************

WraithConsts.h -- Copyright (c) by Jay Reynolds Freeman 2009

Consts and related stuff for Wraith Scheme.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import "LocalTypes.h"
#import "TaggedData.h"
#import "Registers.e"

/**** Masks ****/

const UnsignedLongWord exactBitMask          = 0x0000000000000001LL;    // Meaningful only when the rest of the bits encode a number.
const UnsignedLongWord floatBitMask          = 0x0000000000000002LL;    // Meaningful only when the rest of the bits encode a number.
const UnsignedLongWord weirdBitMask          = 0x0000000000000004LL;    // Meaningful only when the rest of the bits encode a number.
                                                                        //   To be used for "weird numbers", but as of 2009.06.30,
                                                                        //   there are none yet.
const UnsignedLongWord numberBitsMask        = 0x0000000000000018LL;
const UnsignedLongWord leafBitMask           = 0x0000000000000020LL;
const UnsignedLongWord pointsToActiveBitMask = 0x0000000000000040LL;    // To be set for non-leaves pointing to the active generation.
//const UnsignedLongWord lockBit2Mask          = 0x0000000000000080LL;    // NOT USED.
const UnsignedLongWord colorBitMask          = 0x0000000000000100LL;    // Spare bit used for coloring, occasionally ...
const UnsignedLongWord immutableBitMask      = 0x0000000000000200LL;
const UnsignedLongWord startCCNBlockBitMask  = 0x0000000000000400LL;    // Duplicate use of following bit.
const UnsignedLongWord recursionBitCdrMask   = 0x0000000000000400LL;    // Duplicate use of preceding bit.
const UnsignedLongWord recursionBitCarMask   = 0x0000000000000800LL;
const UnsignedLongWord recursionBitsMask     = 0x0000000000000c00LL;
const UnsignedLongWord gcBitMask             = 0x0000000000008000LL;
const UnsignedLongWord gcBit2Mask            = 0x0000000000004000LL;
const UnsignedLongWord gcBitsMask            = 0x000000000000c000LL;
const UnsignedLongWord cdrCodingBitsMask     = 0x0000000000003000LL;
const UnsignedLongWord typeCodeBitsMask      = 0x000000000000303fLL;

const UnsignedLongWord u0ByteBitsMask        = 0x00000000ff000000LL;    // "u" as in "ultimate", for the most significant byte ...
const UnsignedLongWord p0ByteBitsMask        = 0x0000000000ff0000LL;    // "p" as in "penultimate" ...
const UnsignedLongWord s0ByteBitsMask        = 0x000000000000ff00LL;    // "s" as in "second" ...
const UnsignedLongWord l0ByteBitsMask        = 0x00000000000000ffLL;    // "l" as in "least-significant" ...

const UnsignedLongWord u1ByteBitsMask        = 0xff00000000000000LL;    // "u" as in "ultimate", for the most significant byte ...
const UnsignedLongWord p1ByteBitsMask        = 0x00ff000000000000LL;    // "p" as in "penultimate" ...
const UnsignedLongWord s1ByteBitsMask        = 0x0000ff0000000000LL;    // "s" as in "second" ...
const UnsignedLongWord l1ByteBitsMask        = 0x000000ff00000000LL;    // "l" as in "least-significant" ...

const UnsignedLongWord lowerWordMask         = 0x00000000ffffffffLL;    // The entire lower word.
const UnsignedLongWord upperWordMask         = 0xffffffff00000000LL;    // The entire upper word.
const UnsignedLongWord justOneByteMask       = 0x00000000000000ffLL;

const UnsignedLongWord shortMask             = 0x000000000000ffffLL;

const UnsignedLongWord MSB_8                 = 0x80LL;
const UnsignedLongWord MSB_16                = 0x8000LL;
const UnsignedLongWord MSB_32                = 0x80000000LL;
const UnsignedLongWord BYTE_SIGN_EXTENDER    = 0xffffffffffffff00LL;
const UnsignedLongWord SHORT_SIGN_EXTENDER   = 0xffffffffffff0000LL;
const UnsignedLongWord WORD_SIGN_EXTENDER    = 0xffffffff00000000LL;


/**** Shift Values ****/

const UnsignedLongWord u0ByteOffset             = 24LL;
const UnsignedLongWord p0ByteOffset             = 16LL;
const UnsignedLongWord s0ByteOffset             =  8LL;
const UnsignedLongWord l0ByteOffset             =  0LL;

const UnsignedLongWord u1ByteOffset             = 56LL;
const UnsignedLongWord p1ByteOffset             = 48LL;
const UnsignedLongWord s1ByteOffset             = 40LL;
const UnsignedLongWord l1ByteOffset             = 32LL;

const UnsignedLongWord environmentNumberOffset  = 32LL;
const UnsignedLongWord bucketNumberOffset       = 32LL;


/**** Tag Bit/Field Values -- for cdr-coding bits ****/

const UnsignedLongWord carCdrNext          = 0x0000000000002000LL;
const UnsignedLongWord carCdrNil           = 0x0000000000001000LL;
const UnsignedLongWord extraCdrCode        = 0x0000000000003000LL;
const UnsignedLongWord notCar              = 0x0000000000000000LL;

/**** Tag Bit/Field Values -- for type-code bits

      Note:  Tags 0x18 through 0x1f, and 0x38 through 0x3f, reserved for numbers.                ****/

const UnsignedLongWord forwardingPointerTag              = 0x00LL;
const UnsignedLongWord stringTag                         = 0x01LL;
const UnsignedLongWord pairTag                           = 0x02LL;
const UnsignedLongWord symbolTag                         = 0x03LL;
const UnsignedLongWord lambdaPairTag                     = 0x04LL;
const UnsignedLongWord builtInFunctionTag                = 0x05LL;
const UnsignedLongWord specialFormTag                    = 0x06LL;
const UnsignedLongWord vectorTag                         = 0x07LL;
const UnsignedLongWord peculiarNonLeafTag                = 0x08LL;
const UnsignedLongWord syntaxBindingTag                  = 0x09LL;
const UnsignedLongWord macroTag                          = 0x0ALL;
const UnsignedLongWord promiseTag                        = 0x0BLL;
const UnsignedLongWord multipleValuesTag                 = 0x0CLL;
const UnsignedLongWord pointerToByteBlockTag             = 0x0DLL;
const UnsignedLongWord forgettableUnexpiredTag           = 0x0ELL;
const UnsignedLongWord forgettableRememberedTag          = 0x0FLL;
const UnsignedLongWord mmappedBlockTag                   = 0x10LL;
const UnsignedLongWord ggcForwardingPointerTag           = 0x11LL;

const UnsignedLongWord maxNonLeafTag                     = 0x11LL;

const UnsignedLongWord byteBlockTag                      = 0x20LL;
const UnsignedLongWord falseBooleanTag                   = 0x21LL;
const UnsignedLongWord trueBooleanTag                    = 0x22LL;
const UnsignedLongWord characterTag                      = 0x23LL;
const UnsignedLongWord schemeMachinePCTag                = 0x24LL;
const UnsignedLongWord peculiarLeafTag                   = 0x25LL;
const UnsignedLongWord stackReferenceTag                 = 0x26LL;
const UnsignedLongWord environmentReferenceTag           = 0x27LL;
const UnsignedLongWord topLevelEnvironmentReferenceTag   = 0x28LL;
const UnsignedLongWord emptyListTag                      = 0x29LL;
const UnsignedLongWord failTag                           = 0x2ALL;
const UnsignedLongWord succeedTag                        = 0x2BLL;
const UnsignedLongWord schemeMachinePCIndexTag           = 0x2CLL;
const UnsignedLongWord portTag                           = 0x2DLL;
const UnsignedLongWord forgettableForgottenTag           = 0x2ELL;

const UnsignedLongWord maxLeafTag                        = 0x2ELL;

/**** Tag Bit/Field Values -- miscellaneous and combinations ****/

const UnsignedLongWord numberBits                   =  (numberBitsMask);                   /* both on iff. number */
const UnsignedLongWord pureTypeCodeBitsMask         =  (typeCodeBitsMask & ~cdrCodingBitsMask);
const UnsignedLongWord pureTypeCodeNoExactBitsMask  =  (pureTypeCodeBitsMask & (~ exactBitMask));
const UnsignedLongWord shortFloatBitsMask           =  (leafBitMask | numberBits | floatBitMask);
const UnsignedLongWord shortFixnumBitsMask          =  (leafBitMask | numberBits);
const UnsignedLongWord exactShortFloatBitsMask      =  (shortFloatBitsMask | exactBitMask);
const UnsignedLongWord exactShortFixnumBitsMask     =  (shortFixnumBitsMask | exactBitMask);
const UnsignedLongWord inexactShortFloatBitsMask    =  shortFloatBitsMask;
const UnsignedLongWord inexactShortFixnumBitsMask   =  shortFixnumBitsMask;
const UnsignedLongWord checkForNumberBitsMask       =     (pureTypeCodeBitsMask
                                                        & (~(leafBitMask | weirdBitMask | floatBitMask | exactBitMask)));
const UnsignedLongWord numberMask                   =  numberBits;
const UnsignedLongWord fixnumOrFloatMask            =  (numberBits | weirdBitMask | floatBitMask);
const UnsignedLongWord immutableLeafMask            =  (immutableBitMask | leafBitMask);

const UnsignedLongWord gcPreservedBitsMask          =  (pureTypeCodeBitsMask | colorBitMask | u0ByteBitsMask | p0ByteBitsMask);

const UnsignedLongWord administrationBitsMask       =  (upperWordMask | u0ByteBitsMask | p0ByteBitsMask);

const UnsignedLongWord maxTag                       =  (pureTypeCodeBitsMask + 1);

/**** Peculiar leaf identifications: ****/

const UnsignedLongWord consDotterPLTag           =    0;
const UnsignedLongWord eofObjectPLTag            =    1;

const UnsignedLongWord whereTLEPointsPLTag       =    2;
const UnsignedLongWord whereObListPointsPLTag    =    3;

const UnsignedLongWord nonPrintingObjectTag      =    4;

/**** Peculiar non-leaf identifications: ****/

const UnsignedLongWord nInputFileDescriptors    =    8;
const UnsignedLongWord nOutputFileDescriptors   =    8;

/**** Inline functions that mostly used to be macros. ****/

const UnsignedLongWord kittenByteOffset = p0ByteOffset;
const UnsignedLongWord kittenByteBitsMask = p0ByteBitsMask;
inline void orIntoKittenByteNonAtomic( UnsignedLongWord &thing, UnsignedLongWord value )
{
    thing = ((volatile Tag)(thing)) | ((((UnsignedLongWord)(value)) & justOneByteMask) << kittenByteOffset);
    }
inline void clearKittenByteNonAtomic( UnsignedLongWord &thing ) { (thing) = (((volatile Tag)(thing)) & ~(kittenByteBitsMask)); }
inline Byte getKittenByte( UnsignedLongWord thing ) { return (Byte)(((thing) >> kittenByteOffset) & justOneByteMask); }


const UnsignedLongWord leafByteOffset = l0ByteOffset;
inline void orIntoLeafByteNonAtomic( LongWord &thing, UnsignedLongWord value )
{
    thing = ((volatile Tag)(thing)) | ((((LongWord)(value)) & (LongWord)justOneByteMask) << leafByteOffset);
    }
inline Byte getLeafByte( UnsignedLongWord thing ) { return (Byte)(((thing) >> leafByteOffset) & justOneByteMask); }


const UnsignedLongWord locationByteOffset = u0ByteOffset;
const UnsignedLongWord locationByteBitsMask = u0ByteBitsMask;
inline void orIntoLocationByteNonAtomic( UnsignedLongWord &thing, UnsignedLongWord value )
{
    thing = ((volatile Tag)(thing)) | ((((UnsignedLongWord)(value)) & justOneByteMask) << locationByteOffset);
    }
inline void clearLocationByteNonAtomic( UnsignedLongWord &thing ) { (thing) = (((volatile Tag)(thing)) & ~(locationByteBitsMask)); }
inline Byte getLocationByte( UnsignedLongWord thing ) { return (Byte)(((thing) >> locationByteOffset) & justOneByteMask); }
inline Boolean hasLocationByte( UnsignedLongWord thing ) { return( (thing & (locationByteBitsMask)) != 0 ); }


const UnsignedLongWord peculiarLeafByteOffset = l0ByteOffset;
inline void orIntoPeculiarLeafByteNonAtomic( UnsignedLongWord &thing, UnsignedLongWord value )
{
    thing = ((volatile Tag)(thing)) | ((((UnsignedLongWord)(value)) & justOneByteMask) << peculiarLeafByteOffset);
    }
inline Byte getPeculiarLeafByte( UnsignedLongWord thing ) { return (Byte)(((thing) >> peculiarLeafByteOffset) & justOneByteMask); }


const UnsignedLongWord peculiarNonLeafByteOffset = l0ByteOffset;
inline Byte getPeculiarNonLeafByte( UnsignedLongWord thing ) { return (Byte)(((thing) >> peculiarNonLeafByteOffset) & justOneByteMask); }


const UnsignedLongWord portKittenByteOffset = s0ByteOffset;
inline void orIntoPortKittenByteNonAtomic( UnsignedLongWord &thing, UnsignedLongWord value )
{
    thing = ((volatile Tag)(thing)) | ((((UnsignedLongWord)(value)) & justOneByteMask) << portKittenByteOffset);
    }
inline Byte getPortKittenByte( UnsignedLongWord thing ) { return (Byte)(((thing) >> portKittenByteOffset) & justOneByteMask); }


const UnsignedLongWord portNumberByteOffset = l0ByteOffset;
inline void orIntoPortNumberByteNonAtomic( UnsignedLongWord &thing, UnsignedLongWord value )
{
    thing = ((volatile Tag)(thing)) | ((((UnsignedLongWord)(value)) & justOneByteMask) << portNumberByteOffset);
    }
inline Byte getPortNumberByte( UnsignedLongWord thing ) { return (Byte)(((thing) >> portNumberByteOffset) & justOneByteMask); }




/**** Bit/Field-Fetching Macros ****/

inline UnsignedLongWord cdr_code( Tag tag ) { return (UnsignedLongWord)((tag) & cdrCodingBitsMask); }
inline UnsignedLongWord exact_bit( Tag tag ) { return (UnsignedLongWord)((tag) & exactBitMask); }
inline UnsignedLongWord float_bit( Tag tag ) { return (UnsignedLongWord)((tag) & floatBitMask); }
inline UnsignedLongWord gc_bit( Tag tag ) { return (UnsignedLongWord)((tag) & gcBitMask); }
inline UnsignedLongWord gc_bit_2( Tag tag ) { return (UnsignedLongWord)((tag) & gcBit2Mask); }
inline UnsignedLongWord color_bit( Tag tag ) { return (UnsignedLongWord)((tag) & colorBitMask); }
inline UnsignedLongWord start_ccn_block_bit( Tag tag ) { return (UnsignedLongWord)((tag) & startCCNBlockBitMask); }
inline UnsignedLongWord recursion_bit_car( Tag tag ) { return (UnsignedLongWord)((tag) & recursionBitCarMask); }
inline UnsignedLongWord recursion_bit_cdr( Tag tag ) { return (UnsignedLongWord)((tag) & recursionBitCdrMask); }
inline UnsignedLongWord immutable_bit( Tag tag ) { return (UnsignedLongWord)((tag) & immutableBitMask); }
inline UnsignedLongWord leaf_bit( Tag tag ) { return (UnsignedLongWord)((tag) & leafBitMask); }
inline UnsignedLongWord mutable_type_code( Tag tag ) { return (UnsignedLongWord)((tag) & (pureTypeCodeBitsMask)); }
inline UnsignedLongWord pure_type_code( Tag tag ) { return (UnsignedLongWord)((tag) & pureTypeCodeBitsMask); }
inline UnsignedLongWord type_code( Tag tag ) { return (UnsignedLongWord)((tag) & typeCodeBitsMask); }
inline UnsignedLongWord gc_preserved_bits( Tag tag ) { return (UnsignedLongWord)((tag) & gcPreservedBitsMask); }
inline UnsignedLongWord points_to_active_bit( Tag tag ) { return (UnsignedLongWord)((tag) & pointsToActiveBitMask); }
inline UnsignedLongWord leaf_number( taggedAval x ) { return ((UnsignedLongWord)(getLeafByte( (UnsignedLongWord)((x).u.a) ))); }
inline UnsignedLongWord non_leaf_number( taggedAval x ) { return ((UnsignedLongWord)(getLeafByte( (UnsignedLongWord)((x).u.a) ))); }
inline UnsignedLongWord leaf_number_indirect( taggedAval *x ) { return ((UnsignedLongWord)(getLeafByte( (UnsignedLongWord)(((x)->u).a) ))); }

inline Boolean leaf_or_points_to_active_p( Tag tag ) { return (UnsignedLongWord)((tag) & (pointsToActiveBitMask | leafBitMask)); }

/**** Predicate inline functions -- note distinction between ones that  operate on tags, and on taggedAvals ****/

inline Boolean gc_bit_off( Tag tag) { return (((tag) & gcBitMask ) == 0); }
inline Boolean both_gc_bits_off( Tag tag ) { return (((tag) & gcBitsMask) == 0); }
inline Boolean both_gc_bits_on( Tag tag ) { return (((tag) & gcBitsMask) == gcBitsMask); }
inline Boolean any_gc_bit_on( Tag tag ) { return (((tag) & gcBitsMask) != 0); }
inline Boolean both_recursion_bits_off( Tag tag ) { return (((tag) & recursionBitsMask) == 0); }
inline Boolean both_recursion_bits_on( Tag tag ) { return (((tag) & recursionBitsMask) == recursionBitsMask); }
inline Boolean any_recursion_bit_on( Tag tag ) { return (((tag) & recursionBitsMask) != 0); }
inline Boolean gc_and_both_recursion_bits_off( Tag tag ) { return (((tag) & (gcBitMask | recursionBitsMask)) == 0); }

inline Boolean garbage_p( Tag tag ) { return( gc_bit(tag) == 0 ); }
inline Boolean not_garbage_p( Tag tag ) { return( gc_bit(tag) != 0); }

inline Boolean points_to_active_p( Tag tag ) { return (points_to_active_bit( tag ) != 0); }

inline Boolean not_car_p( Tag tag)   { return( cdr_code(tag) == 0 ); }
inline Boolean car_p( Tag tag)   { return( cdr_code(tag) != 0); }

inline Boolean car_cdr_nil_p( Tag tag ) { return (((tag) & cdrCodingBitsMask) == carCdrNil); }
inline Boolean car_cdr_next_p( Tag tag ) { return (((tag) & cdrCodingBitsMask) == carCdrNext); }
inline Boolean extra_cdr_code_p( Tag tag ) { return (((tag) & cdrCodingBitsMask) == extraCdrCode); }

inline Boolean byte_block_p( Tag tag ) { return (((tag) & typeCodeBitsMask) == byteBlockTag); }
inline Boolean complex_p( Tag tag ) { return (((tag) & checkForNumberBitsMask) == numberBitsMask); }
inline Boolean exact_short_float_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == exactShortFloatBitsMask); }
inline Boolean exact_short_fixnum_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == exactShortFixnumBitsMask); }
inline Boolean float_p( Tag tag ) { return (((tag) & fixnumOrFloatMask) == (numberBitsMask | floatBitMask)); }
inline Boolean short_float_p( Tag tag ) { return (((tag) & pureTypeCodeNoExactBitsMask) == shortFloatBitsMask); }
inline Boolean inexact_short_float_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == inexactShortFloatBitsMask); }
inline Boolean inexact_short_fixnum_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == inexactShortFixnumBitsMask); }
inline Boolean fixnum_p( Tag tag ) { return (((tag) & fixnumOrFloatMask) == numberBitsMask); }
inline Boolean number_p( Tag tag ) { return (((tag) & checkForNumberBitsMask) == numberBitsMask); }
inline Boolean pair_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == pairTag); }
inline Boolean rational_p( Tag tag ) { return (((tag) & checkForNumberBitsMask) == numberBitsMask); }
inline Boolean real_p( Tag tag ) { return (((tag) & checkForNumberBitsMask) == numberBitsMask); }
inline Boolean immutable_p( Tag tag ) { return (immutable_bit(tag) == immutableBitMask); }
inline Boolean mutable_p( Tag tag ) { return (immutable_bit(tag) == 0); }
inline Boolean immutable_leaf_p( Tag tag ) { return (((tag) & immutableLeafMask) == immutableLeafMask); }
inline Boolean immutable_non_leaf_p( Tag tag ) { return (((tag) & immutableLeafMask) == immutableBitMask); }
inline Boolean string_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == stringTag); }
inline Boolean false_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == falseBooleanTag); }
inline Boolean empty_list_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == emptyListTag); }
inline Boolean true_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == trueBooleanTag); }
inline Boolean character_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == characterTag); }
inline Boolean symbol_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == symbolTag); }
inline Boolean scheme_machine_pc_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == schemeMachinePCTag); }
inline Boolean port_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == portTag); }
inline Boolean lambda_pair_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == lambdaPairTag); }
inline Boolean built_in_function_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == builtInFunctionTag); }
inline Boolean special_form_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == specialFormTag); }
inline Boolean vector_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == vectorTag); }
inline Boolean promise_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == promiseTag); }
inline Boolean peculiar_leaf_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == peculiarLeafTag); }
inline Boolean peculiar_non_leaf_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == peculiarNonLeafTag); }
inline Boolean syntax_binding_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == syntaxBindingTag); }
inline Boolean multiple_values_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == multipleValuesTag); }
inline Boolean pointer_to_byte_block_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == pointerToByteBlockTag); }
inline Boolean forgettable_unexpired_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == forgettableUnexpiredTag); }
inline Boolean forgettable_remembered_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == forgettableRememberedTag); }
inline Boolean forgettable_forgotten_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == forgettableForgottenTag); }
inline Boolean mmapped_block_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == mmappedBlockTag); }
inline Boolean macro_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == macroTag); }
inline Boolean stack_reference_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == stackReferenceTag); }
inline Boolean environment_reference_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == environmentReferenceTag); }
inline Boolean top_level_environment_reference_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == topLevelEnvironmentReferenceTag); }
inline Boolean forwarding_pointer_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == forwardingPointerTag); }
inline Boolean ggc_forwarding_pointer_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == ggcForwardingPointerTag); }
inline Boolean not_car_forwarding_pointer_p( Tag tag ) { return (((tag) & (pureTypeCodeBitsMask | cdrCodingBitsMask)) == forwardingPointerTag); }
inline Boolean succeed_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == succeedTag); }
inline Boolean fail_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == failTag); }
inline Boolean scheme_machine_pc_index_p( Tag tag ) { return (((tag) & pureTypeCodeBitsMask) == schemeMachinePCIndexTag); }

inline Boolean atom_with_no_cdr_p( Tag tag ) { return( (tag & (leafBitMask | carCdrNext)) == leafBitMask ); }

inline Boolean leaf_p( Tag tag ) { return (leaf_bit(tag)); }
inline Boolean exact_p( Tag tag ) { return (exact_bit(tag)); }
inline Boolean long_p( Tag tag ) { return (!((tag) & leafBitMask)); }

inline Boolean short_fixnum_p( Tag tag ) { return (((tag) & pureTypeCodeNoExactBitsMask) == shortFixnumBitsMask); }

inline Boolean known_tag_p( Tag tag )   
{
    return (   (((tag) & pureTypeCodeBitsMask) <= maxNonLeafTag)
            || number_p( tag ) 
            || (    (((tag) & pureTypeCodeBitsMask) >= leafBitMask) 
                 && (((tag) & pureTypeCodeBitsMask) <= maxLeafTag) ) );
    }

inline Boolean peculiar_leaf_n_p( taggedAval x, UnsignedLongWord n ) { return (peculiar_leaf_p((x).t) && (leaf_number(x) == (n))); }
inline Boolean peculiar_leaf_p_n_p( taggedAval *x, UnsignedLongWord n ) { return (peculiar_leaf_p((x)->t) && (leaf_number_indirect(x) ==(n))); }

/**** Stuff for handling file descriptors ****/

const UnsignedLongWord coninIndex = nInputFileDescriptors + nOutputFileDescriptors + 0;
const UnsignedLongWord conoutIndex = nInputFileDescriptors + nOutputFileDescriptors + 1;
const UnsignedLongWord conerrIndex = nInputFileDescriptors + nOutputFileDescriptors + 2;
const UnsignedLongWord transcriptIndex = nInputFileDescriptors + nOutputFileDescriptors + 3;
const UnsignedLongWord worldIndex = nInputFileDescriptors + nOutputFileDescriptors + 4;
const UnsignedLongWord closedFileIndex = nInputFileDescriptors + nOutputFileDescriptors + 5;

const UnsignedLongWord nBigArray =
    + nInputFileDescriptors
    + nOutputFileDescriptors
    + 1     /* conIn */ 
    + 1     /* conOut */
    + 1     /* conErr */
    + 1     /* transcript */
    + 1     /* world */
    + 1;     /* closed file */ 

/**** Bit Control Macros -- all operate on tags, not taggedAvals ****/

inline void clearGCBitNonAtomic( Tag &tag ) { (tag) = (((volatile Tag)(tag)) & ~gcBitMask); }
inline void setGCBitNonAtomic( Tag &tag ) { (tag) = (((volatile Tag)(tag)) |  gcBitMask); }
inline void clearGCBit2NonAtomic( Tag &tag ) { (tag) = (((volatile Tag)(tag)) & ~gcBit2Mask); }
inline void setGCBit2NonAtomic( Tag &tag ) { (tag) = (((volatile Tag)(tag)) |  gcBit2Mask); }
inline void clearBothGCBitsNonAtomic( Tag &tag ) { (tag) = (((volatile Tag)(tag)) & ~(gcBitMask | gcBit2Mask)); }
inline void setPointsToActiveBitNonAtomic( Tag &tag ) { (tag) = (((volatile Tag)(tag)) |  pointsToActiveBitMask); }
inline void clearPointsToActiveBitNonAtomic( Tag &tag ) { (tag) = (((volatile Tag)(tag)) & ~pointsToActiveBitMask); }
inline void clearStartCCNBlockBitNonAtomic( Tag &tag ) { (tag) = (((volatile Tag)(tag)) & ~startCCNBlockBitMask); }
inline void setStartCCNBlockBitNonAtomic( Tag &tag ) { (tag) = (((volatile Tag)(tag)) |  startCCNBlockBitMask); }
inline void clearRecursionBitCarNonAtomic( Tag &tag ) { (tag) = (((volatile Tag)(tag)) & ~recursionBitCarMask); }
inline void setRecursionBitCarNonAtomic( Tag &tag ) { (tag) = (((volatile Tag)(tag)) |  recursionBitCarMask); }
inline void clearRecursionBitCdrNonAtomic( Tag &tag ) { (tag) = (((volatile Tag)(tag)) & ~recursionBitCdrMask); }
inline void setRecursionBitCdrNonAtomic( Tag &tag ) { (tag) = (((volatile Tag)(tag)) |  recursionBitCdrMask); }
inline void clearBothRecursionBitsNonAtomic( Tag &tag ) { (tag) = (((volatile Tag)(tag)) & ~(recursionBitCarMask | recursionBitCdrMask)); }
inline void setImmutableBitNonAtomic( Tag &tag ) { (tag) = (((volatile Tag)(tag)) | immutableBitMask); }
inline void makeImmutableObjectNonAtomic( Tag &tag ) { (tag) = (((volatile Tag)(tag)) | immutableBitMask); }
inline void makeInexact( Tag &tag ) { (tag) = (((volatile Tag)(tag)) & ~exactBitMask); }
inline void makeExact( Tag &tag ) { (tag) = (((volatile Tag)(tag)) | exactBitMask); }
inline void clearCdrCodingBitsNonAtomic( Tag &tag)  { (tag) = (((volatile Tag)(tag)) & ~(cdrCodingBitsMask)); }
inline void clearColorBitNonAtomic( Tag &tag ) { (tag) = (((volatile Tag)(tag)) & ~colorBitMask); }
inline void setColorBitNonAtomic( Tag &tag ) { (tag) = (((volatile Tag)(tag)) |  colorBitMask); }

inline void setGCBitAndClearRecursionBitsNonAtomic( Tag &tag ) { (tag) = ((((volatile Tag)(tag)) | gcBitMask) & ~(recursionBitCarMask | recursionBitCdrMask)); }

/**** enum for procedure arity ****/

typedef enum {
        FIXED_ARITY,                          // functionDescriptorTypeOne
        ANY_ARITY_ONE_TYPE_STACK,             // functionDescriptorTypeFive
        POSITIVE_ARITY_TWO_TYPES_STACK,       // functionDescriptorTypeSix
        ANY_ARITY_TWO_TYPES_STACK,            // functionDescriptorTypeSeven
        ANY_ARITY_TWO_TYPES_STACK_2,          // functionDescriptorTypeEight
        NONNEGATIVE_ARITY_TWO_TYPES_STACK     // functionDescriptorTypeNine
        } BuiltInProcedureType;
        
/**** Simple predicates, that operate on R and return values in the B register: ****/

inline void alwaysFalseMC()                    { B = (Boolean)FALSE; }
inline void alwaysTrueMC()                     { B = (Boolean)TRUE; }
inline void booleanPMC()                       { B = (Boolean)(false_p(R.t) || true_p(R.t)); }
inline void builtInFunctionPMC()               { B = (Boolean)built_in_function_p(R.t); }
inline void byteBlockPMC()                     { B = (Boolean)byte_block_p(R.t); }
inline void characterPMC()                     { B = (Boolean)character_p(R.t); }
inline void complexPMC()                       { B = (Boolean)complex_p(R.t); }
inline void emptyListPMC()                     { B = (Boolean)empty_list_p(R.t); }
inline void environmentReferencePMC()          { B = (Boolean)environment_reference_p(R.t); }
inline void exactPMC()                         { B = (Boolean)exact_p(R.t); }
inline void exactShortFloatPMC()               { B = (Boolean)exact_short_float_p(R.t); }
inline void exactShortFixnumPMC()              { B = (Boolean)exact_short_fixnum_p(R.t); }
inline void falsePMC()                         { B = (Boolean)false_p(R.t); }
inline void fixnumPMC()                        { B = (Boolean)fixnum_p(R.t); }
inline void floatPMC()                         { B = (Boolean)float_p(R.t); }
inline void forgettablePMC()
  { B = ((Boolean)forgettable_remembered_p(R.t) || (Boolean)forgettable_forgotten_p(R.t) || (Boolean)forgettable_unexpired_p(R.t)); }
inline void forgettableForgottenPMC()          { B = (Boolean)forgettable_forgotten_p(R.t); }
inline void forgettableRememberedPMC()         { B = (Boolean)forgettable_remembered_p(R.t); }
inline void forgettableUnexpiredPMC()          { B = (Boolean)forgettable_unexpired_p(R.t); }
inline void forwardingPointerPMC()             { B = (Boolean)forwarding_pointer_p(R.t); }
inline void ggcForwardingPointerPMC()          { B = (Boolean)ggc_forwarding_pointer_p(R.t); }
inline void immutablePMC()                     { B = (Boolean)(immutable_p(R.t) != 0); }
inline void inexactShortFloatPMC()             { B = (Boolean)inexact_short_float_p(R.t); }
inline void inexactShortFixnumPMC()            { B = (Boolean)inexact_short_fixnum_p(R.t); }
inline void inexactPMC()                       { B = (Boolean)! exact_p(R.t); }
inline void lambdaPairPMC()                    { B = (Boolean)lambda_pair_p(R.t); }
inline void listPMC()                          { B = (Boolean)(pair_p(R.t) || empty_list_p(R.t)); }
inline void longPMC()                          { B = (Boolean)long_p(R.t); }
inline void macroPMC()                         { B = (Boolean)macro_p(R.t); }
inline void mmappedBlockPMC()                  { B = (Boolean)mmapped_block_p( R.t ); }
inline void multipleValuesPMC()                { B = (Boolean)multiple_values_p(R.t); }
inline void nonLeafPMC()                       { B = (Boolean)(! leaf_p( R.t )); }
inline void numberPMC()                        { B = (Boolean)number_p( R.t ); }
inline void pairPMC()                          { B = (Boolean)pair_p(R.t); }
inline void peculiarLeafPMC()                  { B = (Boolean)peculiar_leaf_p(R.t); }
inline void peculiarNonLeafPMC()               { B = (Boolean)peculiar_non_leaf_p(R.t); }
inline void pointerToByteBlockPMC()            { B = (Boolean)pointer_to_byte_block_p(R.t); }
inline void portPMC()                          { B = (Boolean)port_p(R.t); }
inline void promisePMC()                       { B = (Boolean)promise_p(R.t); }
inline void rationalPMC()                      { B = (Boolean)rational_p(R.t); }
inline void realPMC()                          { B = (Boolean)real_p( R.t ); }
inline void schemeMachinePCPMC()               { B = (Boolean)scheme_machine_pc_p(R.t); }
inline void schemeMachinePCIndexPMC()          { B = (Boolean)scheme_machine_pc_index_p(R.t); }
inline void shortFixnumPMC()                   { B = (Boolean)short_fixnum_p(R.t); }
inline void specialFormPMC()                   { B = (Boolean)special_form_p(R.t); }
inline void stackReferencePMC()                { B = (Boolean)stack_reference_p(R.t); }
inline void stringPMC()                        { B = (Boolean)string_p(R.t); }
inline void symbolPMC()                        { B = (Boolean)symbol_p(R.t); }
inline void syntaxBindingPMC()                 { B = (Boolean)syntax_binding_p(R.t); }
inline void topLevelEnvironmentReferencePMC()  { B = (Boolean)top_level_environment_reference_p(R.t); }
inline void truePMC()                          { B = (Boolean)true_p(R.t); }
inline void vectorPMC()                        { B = (Boolean)vector_p(R.t); }
inline void failPMC()                          { B = (Boolean)fail_p(R.t); }
inline void succeedPMC()                       { B = (Boolean)succeed_p(R.t); }

