/****************************************************************************************

WraithHeader.h -- Copyright (c) by Jay Reynolds Freeman 1987, 1988, 1989, 1990, 1991, 2006

Header file for miscellaneous stuff widely used in Wraith Scheme.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import <stdio.h>
#import <string.h>
#import <stdlib.h>
#import <setjmp.h>
#import <signal.h>
#import <ctype.h>
#import <math.h>
#import <fenv.h>
#import <time.h>
#import <sys/param.h>
#import <unistd.h>
#import <libgen.h>
#import <sys/types.h>
#import <sys/stat.h>
#import <libkern/OSAtomic.h>
#import <mach-o/arch.h>

#import "LocalTypes.h"
#import "ASCIITypes.h"
#import "Events.h"
#import "DataTypes.h"
#import "Events.e"
#import "LowLevelIO.e"

#define SYgetc() fgetc(stdin)

#define SYprintf1( s0 ) (fputs( (s0), stderr ))
#define SYprintf2( s0, s1 ) (fprintf( stderr, (s0), (s1) ))
#define SYprintf3( s0, s1, s2 ) (fprintf( stderr, (s0), (s1), (s2) ))
#define SYprintf4( s0, s1, s2, s3 ) (fprintf( stderr, (s0), (s1), (s2), (s3) ))
#define SYprintf5( s0, s1, s2, s3, s4 ) (fprintf( stderr, (s0), (s1), (s2), (s3), (s4) ))
#define SYprintf6( s0, s1, s2, s3, s4, s5 ) (fprintf( stderr, (s0), (s1), (s2), (s3), (s4), (s5)))
#define SYprintf7( s0, s1, s2, s3, s4, s5, s6 ) (fprintf( stderr, (s0), (s1), (s2), (s3), (s4), (s5), (s6)))
#define SYprintf8( s0, s1, s2, s3, s4, s5, s6, s7 ) (fprintf( stderr, (s0), (s1), (s2), (s3), (s4), (s5), (s6), (s7)))

#define debug( x ) { tprintf3( stdout, "%s\n", x ); fflush( stdout ); }

extern Text MiscChars[];

extern Text releaseDate[];
extern Text releaseTime[];
extern const Text *CommandHelpStrings[];
extern Text *DataTypeStrings[];
extern Text *DictionaryNames[];
extern Text *EditHelpStrings[];
extern Text *ErrorStrings[];
extern Text *FatalErrorStrings[];
extern Text *FileHelpStrings[];
extern Text *HowToStrings[];
extern Text *MiscStrings[];
extern Text *NameExampleStrings[];
extern Text *NameHelpStrings[];
extern Text *NameStrings[];
extern const Text *ObjectErrorNameStrings[];
extern Text *OptionHelpStrings[];
extern Text *SpecialFormSyntaxStrings[];
extern Text *TypeNameStrings[];
extern Text *WhatIsStrings[];

extern void debugEntry();
extern void exitTopLoop();
extern void resetCleanUp();
extern void topLoopCode();

/****
#define STANDARD_TIMER_INTERVAL     0.005   // In seconds: Used mostly in WraithSchemeOuterController.mm
#define STANDARD_TIMER_INTERVAL_US   5000   // In microseconds -- should be the same interval as the preceding.
****/

#define STANDARD_TIMER_INTERVAL     0.05     // In seconds: Used mostly in WraithSchemeOuterController.mm
#define STANDARD_TIMER_INTERVAL_US   50000   // In microseconds -- should be the same interval as the preceding.

#define MAX_PATHNAME_LENGTH		         (MAXPATHLEN + 42)    // Six times nine, in base thirteen ...
#define MAX_STRING_FROM_CONSOLE_LENGTH   2048
#define MAX_TYPE_LENGTH			         16		

#define HEURISTIC_DIGITS 7

// Stuff for big-endian / little-endian transformations.

#define F_BYTE_MASK 0xff

#define F_BYTE_0( aWord ) ( (Word)(aWord)        & F_BYTE_MASK)
#define F_BYTE_1( aWord ) (((Word)(aWord) >>  8) & F_BYTE_MASK)
#define F_BYTE_2( aWord ) (((Word)(aWord) >> 16) & F_BYTE_MASK)
#define F_BYTE_3( aWord ) (((Word)(aWord) >> 24) & F_BYTE_MASK)

#define FLIP_WORD( aWord ) ( (F_BYTE_0( aWord ) << 24) | (F_BYTE_1( aWord ) << 16) | (F_BYTE_2( aWord) << 8) | (F_BYTE_3( aWord )) )

#define CHECK_SPACE_BEFORE     FALSE       // Turn on to test space before GC.
#define CHECK_SPACE_AFTER      FALSE       // Turn on to test space after GC.
#define CHECK_SPACE_AFTER_ALL  TRUE        // Turn on to test space after GC and possible ObListFussing.
#define CHECK_SPACE_INTERNAL   FALSE       // Left turned on for GC robustness.
#define USE_HIGH_GC FALSE
#define USE_ERS FALSE

#define FINAL_RESET 3
#define FULL_RESET 2
#define RESET_AFTER_INITIAL_WORLD_LOAD 1

#define WORLD_OPERATION_DELAY 100000

#define USE_KITTEN_LOGS 0

#ifndef PAGE_SIZE
#define PAGE_SIZE 4096
#endif

