/****************************************************************************************

Apply.c++ -- Copyright (c) by Jay Reynolds Freeman 1988, 1989, 1990, 1991, 2006

Apply mechanisms for Wraith Scheme.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import "WraithHeader.h"
#import "Registers.h"

#import "DataTypes.e"
#import "FunctionAndSpecialNames.e"
#import "Globals.e"
#import "Intern.e"
#import "IO.e"
#import "Registers.e"
#import "SECDEval.e"
#import "SECDMachine.e"
#import "SECDRegisters.e"

/**** Here come the functions: ****/

void bindFormals(), topNSameLength();                           /* forwards */

/**** applyEnd -- Clean up the stack after a function has run ****/

void applyEnd()         /* S: (proc nArgs arg* environment savedContinuation oldF magicNumber ...) */
{
    while(     (CF == ((F + FRAME_CF_OFFSET)->u).a )                /* empty continuation */
            && (((F + FRAME_NEW_MN_OFFSET)->u).c == 0) )            /* and not stacking arguments */
      {   
        CF = ((F + FRAME_CF_OFFSET)->u).a;                           /* tail recursing -- pop an entire frame */
        F  = ((F + FRAME_FP_OFFSET)->u).a;                           /* tail recursing -- pop an entire frame */
        }
     S = F + FRAME_E_OFFSET;                                         /*  clear stack of (last set of) args*/
     }

/**** applyLambda -- set up environment and continuation for a lambda application ****/

void applyLambda()              /* R: <don't care>; S: (lambda nArgs arg* ...) */
{
    *(F + FRAME_NEW_L_OFFSET) = L = R = S[0];   /* R: lambda; S: (lambda nArgs arg* ...) */

    R.t = pairTag;
    cdr();                              /* R: (lambdaBody envlist . name); S: (lambda nArgs arg* ...) */

    A = R; R = S[0]; S[0] = A;          /* R: lambda; S: ((lambdaBody envlist . name) nArgs arg* ...) */                  
    R.t = pairTag;                     
    car();                              /* R: formals; S: ((lambdaBody envlist . name) nArgs arg* ...) */

    SPushR();       
    R.t = emptyListTag;                   /* a new environment, which of course is empty to begin with */

    /* R: newEnv; S: (formals (lambdaBody envlist . name) nArgs arg* ...) */

    bindFormals();                      /* R: newEnv; S: ((lambdaBody envlist . name) nArgs arg* ...) */
    SPushR();

    R = S[1];
    cdr();         
    car();         
    cons();                             /* R: (newEnv . envList); S: ((lambdaBody envlist . name) nArgs arg* ...) */
    E = R;

    R = S[0];                           /* R: (lambdaBody envlist . name); S: ((lambdaBody envlist . name) nArgs arg* ...) */
    car();                              /* R: lambdaBody */     
    applyEnd();
        
    CPushEmptyFrame();
    CSPushProperList();
    }
 
/**** applyMacroLambda -- set up environment and continuation for a lambda application for a macro ****/

void applyMacroLambda()              /* R: <don't care>; S: (lambda nArgs arg* ...) */
{
    CPushFrame( E );
    R.t = schemeMachinePCIndexTag;
    R.u.c = macroPartTwo_index;
    CSPush( R );

    *(F + FRAME_NEW_L_OFFSET) = L = R = S[0];   /* R: lambda; S: (lambda nArgs arg* ...) */
    R.t = pairTag;
    cdr();                              /* R: (lambdaBody envlist . name); S: (lambda nArgs arg* ...) */

    A = R; R = S[0]; S[0] = A;          /* R: lambda; S: ((lambdaBody envlist . name) nArgs arg* ...) */                  
    R.t = pairTag;                     
    car();                              /* R: formals; S: ((lambdaBody envlist . name) nArgs arg* ...) */

    SPushR();       
    R.t = emptyListTag;                   /* a new environment, which of course is empty to begin with */

    /* R: newEnv; S: (formals (lambdaBody envlist . name) nArgs arg* ...) */

    bindFormals();                      /* R: newEnv; S: ((lambdaBody envlist . name) nArgs arg* ...) */
    SPushR();

    R = S[1];
    cdr();         
    car();         
    cons();                             /* R: (newEnv . envList); S: ((lambdaBody envlist . name) nArgs arg* ...) */
    SavedE = E;                         // NOTE -- use A instead, and get rid of SavedE?
    E = R;

    R = S[0];                           /* R: (lambdaBody envlist . name); S: ((lambdaBody envlist . name) nArgs arg* ...) */
    car();                              /* R: lambdaBody */     
    applyEnd();  
    SPushR();
    CSPushProperList();
    R.t = schemeMachinePCIndexTag;
    R.u.c = macroPartTwo_index;
    SPushR();
    R = SavedE;
    nilCons();
    cons();                             /* R: (macroPartTwo savedE) */
    SPushR();
    R = two;
    appendCode();
    }

/**** bindFormals ****/

void bindFormals()                              /* R: newEnv; S: (formals (lambdaBody envlist . name) nArgs arg* ...);
                                                   binds pairwise conses into newEnv and returns undefined */
{
    LongWord argIndex = S[2].u.c + 2;                   /* location of current arg */
    LongWord nPushed, nRestArgs;

    while( ! empty_list_p( S[0].t ) )
        if( ! pair_p( S[0].t ) ) {              /* final "rest" formal -- must cons up the rest of the actuals */
            SPushR();
            R = S[1];
            SPushR();
            nRestArgs = argIndex - 2;
            for( nPushed = 0; nPushed < nRestArgs; ++nPushed ) {    /* push the args */
                R = S[ argIndex + 2 ];
                SPushR();
                }
            R.t = emptyListTag;
            while( nPushed-- > 0 )              /* cons what we pushed */
                cons();                 
            bindToGrowingEnvironment();
            argIndex = 2;                       /* for final test herein */
            break;
            }
        else {                                  /* another random argument */
            if( argIndex == 2 )                 /* too few actuals */
                resetError( 3 );
            SPushR();                           /* R: newEnv; S: (newEnv formals (lambdaBody envlist . name) nArgs arg* ...) */
            R = S[1];
            cdr();         
            A = R; R = S[1]; S[1] = A;
            car();                  /* R: thisformal; S: (newEnv formalsLeft (lambdaBody envlist . name) nArgs arg* ...) */
            SPushR();               /* R: thisformal; S: (thisformal newEnv formalsLeft (lambdaBody envlist . name) nArgs arg* ...) */
            R = S[ argIndex-- + 2];

            /* R: thisActual; S: (thisformal newEnv formalsLeft (lambdaBody envlist . name) nArgs arg* ...) */

            bindToGrowingEnvironment();

            /* R: newEnv; S: (formalsLeft (lambdaBody envlist . name) nArgs arg* ...) */
                        
            }

    if( argIndex == 2 ) {                       /* ran out of formals and actuals at the same time -- fine */
        SDrop();
        return;
        }
    else {
        nRestArgs = argIndex - 2;
        for( nPushed = 0; nPushed < nRestArgs; ++nPushed ) {    /* push the args */
            R = S[ argIndex ];
            SPushR();
            }
        R.t = emptyListTag;
        while( nPushed-- > 0 )                  /* cons what we pushed */
            cons();                 
        useErrorOutput();
        displayObject();
        resetError( 2 );
        }
    }

/**** oneForEach -- Primitive for doing "for-each".  When called, the arguments of "for-each"
                    should all be stacked, and one push have taken place, so that
                       
                    R: <don't care>;
                    S: (nArgs arglist* procedure ...);
                                   
                    The arglists are assumed to have been checked to see that each one is
                    proper, that all are the same length, and that none is ().  This procedure
                    creates a stack frame appropriate for "apply", replaces each of the
                    arglists by its cdr, and pushes "apply" into the continuation.                       ****/

void oneForEach()
{
    LongWord nArgs = S[0].u.c - 1;
    LongWord argCounter;

    PushFrame( 0 );
        
    for( argCounter = 0; argCounter < nArgs; ++argCounter ) {
        R = S[ nArgs + FrameSize ];
        cdr();         
        A = R; R = S[ nArgs + FrameSize ]; S[ nArgs + FrameSize ] = A;
        car();         
        SPushR();
        }

    R.t = exactShortFixnumBitsMask;
    R.u.c = nArgs;
    SPushR();
    R = S[ 2 * nArgs + (FrameSize + 2) ]; 
    SPushR();

    R.t = schemeMachinePCIndexTag;
    R.u.c = apply_index;
    CPushFrame( R );
    }

/**** oneMap -- Primitive for doing "map".  When called, the arguments of "map"
                should all be stacked, with a reversed list of partial results being
                built up above them, and with the most recent result in R, so that
                                   
                R: a result from the function being mapped;
                S: (reversedResultList nArgs arglist* procedure ...);
                           
                The arglists are assumed to have been checked to see that each one is
                proper, that all are the same length, and that none is ().  This procedure
                creates a stack frame appropriate for "apply", replaces each of the
                arglists by its cdr, and calls "apply".                                                              ****/

void oneMap()
{
    LongWord nArgs = S[1].u.c - 1;
    LongWord argCounter;

    A = R; R = S[0]; S[0] = A;
    cons();
    SPushR();       

    PushFrame( 0 );
                
    for( argCounter = 0; argCounter < nArgs; ++argCounter ) {
        R = S[ nArgs + (FrameSize + 1) ];
        cdr();         
        A = R; R = S[ nArgs + (FrameSize + 1) ]; S[ nArgs + (FrameSize + 1) ] = A;
        car();         
        SPushR();
        }

    R.t = exactShortFixnumBitsMask;
    R.u.c = nArgs;
    SPushR();
    R = S[ 2 * nArgs + (FrameSize + 3) ]; 
    SPushR();

    R.t = schemeMachinePCIndexTag;
    R.u.c = apply_index;
    CPushFrame( R );
    }

/**** oneMapFirst -- Primitive for doing "map".  When called, the arguments of "map"
                     should all be stacked, with a () for partial results being
                     built up above them, and with a don't-care in R, so that
                                                   
                     R: dont-care;
                     S: (() nArgs arglist* procedure ...);
                                           
                     The arglists are assumed to have been checked to see that each one is
                     proper, that all are the same length, and that none is ().  This procedure
                     creates a stack frame appropriate for "apply", replaces each of the
                     arglists by its cdr, and calls "apply".                                                         ****/

void oneMapFirst()
{
    LongWord nArgs = S[1].u.c - 1;
    LongWord argCounter;

    PushFrame( 0 );
        
    for( argCounter = 0; argCounter < nArgs; ++argCounter ) {
        R = S[ nArgs + (FrameSize + 1) ];
        cdr();         
        A = R; R = S[ nArgs + (FrameSize + 1) ]; S[ nArgs + (FrameSize + 1) ] = A;
        car();         
        SPushR();
        }

    R.t = exactShortFixnumBitsMask;
    R.u.c = nArgs;
    SPushR();
    R = S[ 2 * nArgs + (FrameSize + 3) ]; 
    SPushR();

    R.t = schemeMachinePCIndexTag;
    R.u.c = apply_index;
    CPushFrame( R );
    }

/**** topNSameLength -- Verify that the top N stack elements are all
                        lists of the same length.  N is in R.  Returns the
                        length; leaves the number N in S[0].                                 ****/
                                                 
void topNSameLength()
{
    LongWord nLists = R.u.c;
    LongWord listCounter;
    UnsignedLongWord lengthWanted;

    SPushR();
    B = TRUE;
    R = S[ 1 ];
    lengthWanted = lengthFunction();
    if( nLists > 1 ) {
        for( listCounter = 1; listCounter < nLists; ++listCounter ) {
            R = S[ listCounter + 1 ];
            if( lengthFunction() != lengthWanted ) {
                B = FALSE;
                break;
                }
            }
        }
    R.t = exactShortFixnumBitsMask;
    R.u.c = lengthWanted;
    }

