/****************************************************************************************

CommandInterface.c++ -- Copyright (c) by Jay Reynolds Freeman 2006

Command interface for Wraith  --  where menu stuff would be, if there were any menus.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import "WraithHeader.h"
#import "Globals.e"
#import "IO.e"
#import "NumberFormats.e"

const Text *CommandHelpStrings[] = {
/* 0 */    
"\n\n\n\n\
               Wraith Scheme Instrument Panel:\n\
    Terminate all commands with the \"return\" or \"enter\" key.\n\
\n\
C (or c):  %s compiling defines automatically.\n\
E (or e):  Return to Wraith Scheme top-level loop.\n\
N (or n):  %s nostalgia mode.\n\
P (or p):  %s precision for console numeric output.\n\
Q (or q):  Exit from Wraith Scheme.\n\
S (or s):  %s console output.\n\
\n\
: ",
/* 1 */    "Defines will be compiled automatically.",
/* 2 */    "Defines will not be compiled automatically.",
/* 3 */    "Display and write will use full numeric precison.",
/* 4 */    "Display and write will use %d digits of numeric precision.",
/* 5 */    "Really exit from Wraith Scheme?  (Answer: <Y/N> and <return>)\n: ",
/* 6 */    "Unrecognized command",
/* 7 */    "%c",
/* 8 */    "Garbage collection in progress, try again later ...",
/* 9 */    "<Y/N> and <return>): ",
/* 10 */   "Output to Wraith Scheme window is disabled.",
/* 11 */   "Output to Wraith Scheme window is enabled.",
    };

/**** confirm ****/

Boolean confirm()
{
    Word c = 0;

    while( TRUE ) {
        c = tolower( SYgetc() );
        switch( c ) {

            case 'y':      return TRUE;

	    case 'n':      return FALSE;

            case '\n':     SYprintf2( CommandHelpStrings[ 7 ], BELL );
                           SYprintf1( CommandHelpStrings[ 9 ] );
                           break;

            default:       break;
	    }
	}
    }

/**** GCBlocking ****/

Boolean GCBlocking()
{
    if( GCInProgressFlag ) {
        SYprintf1( CommandHelpStrings[ 8 ]);
        SYprintf1("\n");
        SYprintf2( CommandHelpStrings[ 7 ], BELL );
        emitStatusMessage( CommandHelpStrings[ 8 ]);
        }
    return( GCInProgressFlag );
    }

/**** lastBeforeNewline ****/

Text lastBeforeNewline()
{
    Text thisC = 0;
    Text nextC = 0;

    while( (nextC = SYgetc()) != NEWLINE ) {
        thisC = nextC;
        }
    return thisC;
    }

/**** doCommand -- stubbish ****/

void doCommand()
{
    Word c = 0;
    Text buffer[ 256 ];

    switchToSystemIO();

    SYprintf5( CommandHelpStrings[ 0 ],
               CompileDefinesFlag ? "Stop" : "Start",
               nostalgiaMode ? "Clear" : "Set",
               useFullNumericPrecision ? "Decrease" : "Increase",
               consoleOutputEnabledFlag ? "Disable" : "Enable"
               );

    c = tolower( lastBeforeNewline() );
    switch( c ) {

        case 'c':   if( GCBlocking() )
                        break;
                    CompileDefinesFlag = !CompileDefinesFlag;
                    SYprintf1( CommandHelpStrings[ (CompileDefinesFlag ? 1 : 2 ) ] );
                    SYprintf1("\n");
                    emitStatusMessage( CommandHelpStrings[ (CompileDefinesFlag ? 1 : 2 ) ] );
                    break;

        case 'e':   if( GCBlocking() )
                        break;
                    switchFromSystemIO();
                    longjmp( ResetErrorBuffer, FULL_RESET );
                    break;

        case 'n':   if( nostalgiaMode )
                        clearNostalgiaMode();
                    else
                        setNostalgiaMode();
                    break;

        case 'p':   if( GCBlocking() )
                        break;
                    sprintf( buffer, CommandHelpStrings[ 4 ], HEURISTIC_DIGITS );
                    useFullNumericPrecision = !useFullNumericPrecision;
                    SYprintf1( useFullNumericPrecision ? CommandHelpStrings[3] : buffer );
                    SYprintf1("\n");
                    emitStatusMessage( useFullNumericPrecision ? CommandHelpStrings[3] : buffer );
                    break;

        case 's':   consoleOutputEnabledFlag = ! consoleOutputEnabledFlag;
                    SYprintf1( CommandHelpStrings[ (consoleOutputEnabledFlag ? 11 : 10) ] );
                    SYprintf1("\n");
                    break;

        case 'q':   SYprintf1( CommandHelpStrings[ 5 ] );
                    if( confirm() ) {
                        SYprintf1("\n\nMeeow ...\n\n");
                        switchFromSystemIO();
                        longjmp( ExitBuffer, 1 );
                        }
                    break;

        default :   SYprintf1( CommandHelpStrings[ 6 ] );
                    SYprintf1("\n");
                    emitStatusMessage( CommandHelpStrings[ 6 ] );
                    break;
        }        
    switchFromSystemIO();
    }        

