/****************************************************************************************

DataTypes.c++ -- Copyright (c) by Jay Reynolds Freeman 1987, 1988, 1989, 1990, 1991, 2006

Data-management system for Wraith Scheme.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import <time.h>
#import "WraithHeader.h"
#import "WorldHandler.h"
#import "SchemeCType.h"
#import "OpCodes.h"

#import "Globals.e"
#import "Intern.e"
#import "IO.e"
#import "NumberFormats.e"
#import "Registers.e"
#import "SECDEval.e"
#import "SchemeAux.e"
#import "SchemeFunctions.e"
#import "SchemeSpecials.e"
#import "StreamHandling.e"
#import "StringLoader.e"

/******** Here come some of the statics and externals: ********/

/**** The dataTypeTable -- which gets initialized by an initialization routine, from the dataTypeList     ****/
        
dataDescriptor **dataTypeTable = (dataDescriptor **)NULL;

/******** Here come some of the functions: ********/

/**** Forward declarations: ****/

Boolean displayPair( taggedAval *aPort ), displaySymbolAndBinding( taggedAval *aPort );
Boolean displaySymbol( taggedAval *aPort );
memoryBlockSize lengthTaggedAvalPointer( taggedAval *a );
void installDataTypePName( dataDescriptor *d );
FILE *getPortFilePointer( taggedAval *aPort );
fileDescriptor *getPortFD( taggedAval *aPort );
Boolean printfPort( taggedAval *thePort, FILE *where, Text *message );
Boolean printAnyTaggedAval( taggedAval *aPort, taggedAval *a );

/**** End of forward declarations ****/

/**** describeDataDescriptor -- elementary display primitive ****/

void describeDataDescriptor( dataDescriptor *d )
{
    if( d ) {
        tprintf5( &CurrentOutputPort, MiscStrings[ 108 ], d->pName, d->tag, d->pNameNumber );
        if( d->length != 0 )
            tprintf3( &CurrentOutputPort, MiscStrings[ 59 ], d->length );
        else
            tprintf3( &CurrentOutputPort, MiscStrings[ 130 ], d->lengthFinder );
        tprintf5( &CurrentOutputPort, MiscStrings[ 129 ], d->identifier, d->display, d->write );
        }
    else
        tprintf2( &CurrentOutputPort, MiscStrings[ 102 ] );
    }
        
/**** describeDataTypeList -- Describe each data descriptor in the given list.  ****/

void describeDataTypeList( simpleList d )
{
    tprintf2( &CurrentOutputPort, MiscStrings[ 63 ] );
    while( d->flag ) {
        describeDataDescriptor( (dataDescriptor *)(d->pointer) );
        ++d;
        }
    }

#define BYTE_BLOCK_BUFFER_SIZE 200

/**** describeTaggedAvalPointer -- returns length, for caller to use ****/

UnsignedLongWord describeTaggedAvalPointer( taggedAval *a )
{
    static Text byteBlockBuffer[ BYTE_BLOCK_BUFFER_SIZE ];

    UnsignedLongWord bufferCounter = 0;
    Tag t = a->t;
    Tag cc = cdr_code( t );
    dataDescriptor *d = dataTypeTable[ pure_type_code( t ) ];
    Text cdrCodeName[ 256 ], pName[ 256 ];
    UnsignedLongWord i, lb, lnth;
    Byte *bp;
        
    if( d ) {
        strcpy( pName, d->pName);
        lnth =  lengthTaggedAvalPointer( a );
        }
    else {
        strcpy( pName, MiscStrings[ 135 ] );
        lnth = sizeof( taggedAval );
        }
    switch( cc ) {
        case carCdrNext:   strcpy( cdrCodeName, MiscStrings[ 136 ] ); break;
        case carCdrNil:    strcpy( cdrCodeName, MiscStrings[ 137 ] ); break;
        case extraCdrCode: strcpy( cdrCodeName, MiscStrings[ 138 ] ); break;
        default:           strcpy( cdrCodeName, MiscStrings[ 139 ] ); break;
        }
        
    tprintf7( &CurrentOutputPort, MiscStrings[ 56 ], a, t, (a->u).c, pName, cdrCodeName );

    if( byte_block_p( t ) ) {
        lb = (UnsignedLongWord)((a->u).c);
        bp = (Byte *)a + sizeof( taggedAval );
        for( i = 0; i < lb; ++i ) {
            sprintf( &(byteBlockBuffer[ bufferCounter ]), MiscStrings[ 23 ], *(bp + i) );
            bufferCounter += 3;
            if( bufferCounter >= (BYTE_BLOCK_BUFFER_SIZE - 3) ) {
                tprintf2( &CurrentOutputPort, byteBlockBuffer );
                bufferCounter = 0;
                byteBlockBuffer[ bufferCounter ] = 0;
                }
            }
        tprintf2( &CurrentOutputPort, byteBlockBuffer );
        if( i < lb ) {
            tprintf2( &CurrentOutputPort, MiscStrings[ 5 ] );
            }
        tprintf2( &CurrentOutputPort, MiscStrings[ 30 ] );
        bufferCounter = 0;
        for( i = 0; i < lb; ++i ) {
            if( isprint( *(bp + i) ) )
                sprintf( &(byteBlockBuffer[ bufferCounter ]), MiscStrings[ 57 ], *(bp + i) );
            else
                sprintf( &(byteBlockBuffer[ bufferCounter ]), MiscStrings[ 57 ], MiscChars[ 18 ] );
            bufferCounter++;
            if( bufferCounter >= (BYTE_BLOCK_BUFFER_SIZE - 1) ) {
                tprintf2( &CurrentOutputPort, byteBlockBuffer );
                bufferCounter = 0;
                byteBlockBuffer[ bufferCounter ] = 0;
                }
            }
        tprintf2( &CurrentOutputPort, byteBlockBuffer );
        }
    tprintf3( &CurrentOutputPort, MiscStrings[ 57 ], NEWLINE );
        
    return( lnth );
    }
        
/**** printfTaggedAvalPointer -- returns length, for caller to use ****/

UnsignedLongWord printfTaggedAvalPointer( taggedAval *a )
{
    static Text byteBlockBuffer[ BYTE_BLOCK_BUFFER_SIZE ];

    UnsignedLongWord bufferCounter = 0;
    Tag t = a->t;
    Tag cc = cdr_code( t );
    dataDescriptor *d = dataTypeTable[ pure_type_code( t ) ];
    Text cdrCodeName[ 256 ], pName[ 256 ];
    UnsignedLongWord i, lb, lnth;
    Byte *bp;
        
    if( d ) {
        strcpy( pName, d->pName);
        lnth =  lengthTaggedAvalPointer( a );
        }
    else {
        strcpy( pName, MiscStrings[ 135 ] );
        lnth = sizeof( taggedAval );
        }
    switch( cc ) {
        case carCdrNext:   strcpy( cdrCodeName, MiscStrings[ 136 ] ); break;
        case carCdrNil:    strcpy( cdrCodeName, MiscStrings[ 137 ] ); break;
        case extraCdrCode: strcpy( cdrCodeName, MiscStrings[ 138 ] ); break;
        default:           strcpy( cdrCodeName, MiscStrings[ 139 ] ); break;
        }
        
    fprintf( stdout, MiscStrings[ 56 ], a, t, (a->u).c, pName, cdrCodeName );

    if( byte_block_p( t ) ) {
        lb = (UnsignedLongWord)((a->u).c);
        bp = (Byte *)a + sizeof( taggedAval );
        for( i = 0; i < lb; ++i ) {
            sprintf( &(byteBlockBuffer[ bufferCounter ]), MiscStrings[ 23 ], *(bp + i) );
            bufferCounter += 3;
            if( bufferCounter >= (BYTE_BLOCK_BUFFER_SIZE - 3) ) {
                fputs( byteBlockBuffer, stdout );
                bufferCounter = 0;
                byteBlockBuffer[ bufferCounter ] = 0;
                }
            }
        fputs( byteBlockBuffer, stdout );
        if( i < lb ) {
            fputs( MiscStrings[ 5 ], stdout );
            }
        fputs( MiscStrings[ 30 ], stdout );
        bufferCounter = 0;
        for( i = 0; i < lb; ++i ) {
            if( isprint( *(bp + i) ) )
                sprintf( &(byteBlockBuffer[ bufferCounter ]), MiscStrings[ 57 ], *(bp + i) );
            else
                sprintf( &(byteBlockBuffer[ bufferCounter ]), MiscStrings[ 57 ], MiscChars[ 18 ] );
            bufferCounter++;
            if( bufferCounter >= (BYTE_BLOCK_BUFFER_SIZE - 1) ) {
                fputs( byteBlockBuffer, stdout );
                bufferCounter = 0;
                byteBlockBuffer[ bufferCounter ] = 0;
                }
            }
        fputs( byteBlockBuffer, stdout );
        }
    fprintf( stdout, MiscStrings[ 57 ], NEWLINE );
        
    return( lnth );
    }
        
/**** equalByteBlockString -- Assuming that both of the referenced byte blocks are strings, are they equal? ****/

Boolean equalByteBlockString( taggedAval *a, taggedAval *b )
{
    Text *startA = (Text *)((Byte *)a + sizeof(taggedAval));
    Text *startB = (Text *)((Byte *)b + sizeof(taggedAval));
        
    return (Boolean)(!strcmp( startA, startB ));
    }

/**** equalByteBlockToString -- Assuming that the referenced byte block is a string, is it equal to the given string? ****/

Boolean equalByteBlockToString( taggedAval *block, const Text *str )
{
    Text *startBlockStr = (Text *)((Byte *)block + sizeof(taggedAval));

    return (Boolean)(!strcmp( startBlockStr, str ));
    }

/**** extractByteBlockString -- Where are the characters of the given string? ****/

void extractByteBlockString()
{
    R.t = exactShortFixnumBitsMask;
    R.u.a = R.u.a + 1;
    }

/**** getPeculiarNonLeafNumber -- Extract the number of a peculiar non-leaf from the referenced byte block. ****/
                                                                 
LongWord getPeculiarNonLeafNumber( taggedAval *a )
{
    LongWord *start;

    a = spliceOutNotCarForwardingPointersIndirect( a );
    start = (LongWord *)(a + 1);
    return getPeculiarLeafByte( *start );
    }

/**** getBigArrayLocation -- Extract the number of a peculiar non-leaf from the referenced byte block. (For use with ports.) ****/
                                                                 
UnsignedLongWord getBigArrayLocation( taggedAval *a )
{
    UnsignedLongWord *start;

    a = spliceOutNotCarForwardingPointersIndirect( a );
    start = (UnsignedLongWord *)(a + 1);
    return getPeculiarLeafByte( *start );
    }

/**** getPortBigArrayLocation -- Extract the number of a port. ****/
                                                                 
UnsignedLongWord getPortBigArrayLocation( taggedAval *a )
{
    return getPeculiarLeafByte( (UnsignedLongWord)((a->u).c) );
    }

/**** getPeculiarNonLeafKitten -- Extract the kitten number of a peculiar non-leaf from the referenced byte block. ****/
                                                                 
Word getPeculiarNonLeafKitten( taggedAval *a )
{
    LongWord *start;

    a = spliceOutNotCarForwardingPointersIndirect( a );
    start = (LongWord *)(a + 1);
    return getPeculiarNonLeafByte( *start );
    }

/**** getPortKitten -- Extract the kitten number of a port. ****/
                                                                 
Word getPortKitten( taggedAval *a )
{
    return getPortKittenByte( (UnsignedLongWord)((a->u).c) );
    }

/**** installDataTypePName -- Read the print name of a data type in from a resource. ****/

void installDataTypePName( dataDescriptor *d ) { loadAllocateDataTypePNameString( d->pNameNumber, &(d->pName) ); }

/**** lengthString -- Return the number of characters in the (taggedAval) string, not counting the final null.  ****/

UnsignedLongWord lengthString( taggedAval *str ) { return strlen( (Text *)((str->u).a + 1) ); }

/**** lengthTaggedAvalPointer -- Generic length method; branches for any taggedAval. ****/

memoryBlockSize lengthTaggedAvalPointer( taggedAval *a )
{
    dataDescriptor *d = dataTypeTable[pure_type_code(a->t)];

    if( d->length )
        return d->length;
    else
        return (d->lengthFinder)( a );
    }

/**** lengthVector -- Return the number of items in the vector; as by "vector-length".  ****/

UnsignedLongWord lengthVector( taggedAval *vec )
{
    Boolean savedB;
    taggedAval savedA, savedR;
    UnsignedLongWord lnth;

    savedR = R; savedA = A; savedB = B;
    R = *vec;
    R.t = pairTag;
    CAR
    lnth = (UnsignedLongWord)(R.u.c);
    R = savedR; A = savedA; B = savedB;
    return lnth;
    }

/**** startString -- Return a pointer to the first character in the (taggedAval) string.  ****/

Text *startString( taggedAval *str ) { return( (Text *)((str->u).a + 1 ) ); }

/**** refString -- Return the (zero-based) "cursor"th charcter in the (taggedAval) string.
                   This routine performs no range-checking on the string.  ****/

Text refString( taggedAval *str, LongWord cursor ) { return( *(startString( str ) + cursor) ); }

/**** setString -- Set the (zero-based) "cursor"th charcter in the (taggedAval) string.
                   This routine performs no range-checking on the string.  ****/

void setString( taggedAval *str, LongWord cursor, Text character ) { *(startString( str ) + cursor) = character; }

/**** typeName -- Return the name of the type whose tag is passed. ****/

const Text *typeName( Tag tag )
{
    dataDescriptor *d = dataTypeTable[pure_type_code(tag)];
        
    if( pure_type_code(tag) != pairTag )
        return d->pName;
    else
        return MiscStrings[ 271 ];
    }

/**** writeTaggedAvalSafely -- Generic print method -- saves and restores R, A and B.
                               This routine is intended mainly for debugging; it can be
                               used saving those items across its call.                              ****/

Boolean writeTaggedAvalSafely( taggedAval *value )
{
    Boolean savedB;
    taggedAval savedA, savedR;
    taggedAval scratch;

    scratch = *value;
    savedR = R; savedA = A; savedB = B;
    if( forwarding_pointer_p( scratch.t ) ) {       
        scratch = spliceOutNotCarForwardingPointers( scratch );
        scratch = *(scratch.u.a);
        }
    R = scratch;
    (void) ((dataTypeTable[pure_type_code(R.t)])->write)( &CurrentOutputPort );
    R = savedR; A = savedA; B = savedB;
    return TRUE;
    }

/**** writeTaggedAvalSafelyNoScheme ****/

Boolean writeTaggedAvalSafelyNoScheme( taggedAval *value )
{
    (void) ((dataTypeTable[pure_type_code(value->t)])->write)( &CurrentOutputPort );
    return TRUE;
    }

/**** printAnyTaggedAval -- Print a taggedAval without using Scheme primitives.  This routine is intended not to fail
                            even if its argument is garbage or points to garbage.  ****/

Boolean printAnyTaggedAval( taggedAval *aPort, taggedAval *a )
{
    Tag t = 0;
    Tag cc = 0;
    dataDescriptor *d = 0;
    Text cdrCodeName[ 256 ], pName[ 256 ];

    if( a == 0 ) {
        if( theNumberOfKittens > 1 )
            tprintf3( aPort, MiscStrings[ 141 ], theKittenNumber );
        tprintf2( aPort, MiscStrings[ 367 ] );
        return FALSE;
        }

    t = a->t;
    cc = cdr_code( t );
    d = dataTypeTable[ pure_type_code( t ) ];

    if( d )
        strcpy( pName, d->pName);
    else
        strcpy( pName, MiscStrings[ 135 ] );
    switch( cc ) {
        case carCdrNext:   strcpy( cdrCodeName, MiscStrings[ 136 ] ); break;
        case carCdrNil:    strcpy( cdrCodeName, MiscStrings[ 137 ] ); break;
        case extraCdrCode: strcpy( cdrCodeName, MiscStrings[ 138 ] ); break;
        default:           strcpy( cdrCodeName, MiscStrings[ 139 ] ); break;
        }
    fflush( getPortFilePointer( aPort ) );
    tprintf7( aPort, MiscStrings[ 53 ], t, (a->u).c, pName, cdrCodeName, (theNumberOfKittens > 1) ? theKittenString : "" );
    fflush( getPortFilePointer( aPort ) );
    return TRUE;
    }

/**** printfAnyTaggedAval -- Print a taggedAval without using Scheme primitives.  This routine is intended not to fail
                             even if its argument is garbage or points to garbage.  ****/

Boolean printfAnyTaggedAval( FILE *fp, taggedAval *a )
{
    Tag t = 0;
    Tag cc = 0;
    dataDescriptor *d = 0;
    Text cdrCodeName[ 256 ], pName[ 256 ];
        

    if( a == 0 ) {
        if( theNumberOfKittens > 1 )
            fprintf( fp, MiscStrings[ 141 ], theKittenNumber );
        fputs( MiscStrings[ 367 ], fp );
        return FALSE;
        }

    t = a->t;
    cc = cdr_code( t );
    d = dataTypeTable[ pure_type_code( t ) ];

    if( d )
        strcpy( pName, d->pName);
    else
        strcpy( pName, MiscStrings[ 135 ] );
    switch( cc ) {
        case carCdrNext:   strcpy( cdrCodeName, MiscStrings[ 136 ] ); break;
        case carCdrNil:    strcpy( cdrCodeName, MiscStrings[ 137 ] ); break;
        case extraCdrCode: strcpy( cdrCodeName, MiscStrings[ 138 ] ); break;
        default:           strcpy( cdrCodeName, MiscStrings[ 139 ] ); break;
        }
    fflush( fp );
    fprintf( fp, MiscStrings[ 53 ], t, (a->u).c, pName, cdrCodeName, (theNumberOfKittens > 1) ? theKittenString : "" );
    fflush( fp );
    return TRUE;
    }

/**** inspectAnyTaggedAval -- General-purpose "inspect" routine.  ****/

Boolean inspectAnyTaggedAval( taggedAval *aPort )
{
    printAnyTaggedAval( aPort, &R );
    tprintf2( aPort, MiscStrings[ 115 ] );
    return TRUE;
    }

/**** displayAnyNumber -- Generic display routine for any number. ****/

Boolean displayAnyNumber( taggedAval *aPort )
{
     return (Boolean)tprintf2( aPort, displayNumberHeuristically() );
     }

/**** writeAnyNumber -- Generic display routine for any number. ****/

Boolean writeAnyNumber( taggedAval *aPort )
{
     return (Boolean)tprintf2( aPort, displayNumberHeuristically() );
     }

/**** inspectShortFloat -- "inspect" routine for ShortFloat  ****/

Boolean inspectShortFloat( taggedAval *aPort )
{
    printAnyTaggedAval( aPort, &R );
    tprintf2( aPort, MiscStrings[ 264 ] );
    tprintf2( aPort, displayNumberFull() );
    tprintf2( aPort, MiscStrings[ 115 ] );
    return TRUE;
    }

/**** displayBuiltInFunction -- Canonical "display" routine for the indicated data type.  ****/

Boolean displayBuiltInFunction( taggedAval *aPort )
{
    tprintf2( aPort, MiscStrings[ 38 ]);
    CALL( getBuiltInPName )
    PUSH
    MOVE( *aPort, R )
    SWAP
    DISPLAY_ON_PORT
    tprintf2( aPort, MiscStrings[ 109 ]);
    DROP
    return TRUE;
    }
        
/**** displayByteBlockPointer -- Canonical "display" routine for a pointer to a byte block.  ****/

Boolean displayByteBlockPointer( taggedAval *a, taggedAval *aPort )
{
    static Text byteBlockBuffer[ BYTE_BLOCK_BUFFER_SIZE ];

    UnsignedLongWord bufferCounter = 0;
    UnsignedLongWord length = (UnsignedLongWord)((a->u).c);
    Byte *start;
    Boolean containsNull = FALSE;

    a = spliceOutNotCarForwardingPointersIndirect( a );
    start = (Byte*)(a + 1);
    if( byte_block_p( a->t ) ) {
        while( length-- > 0 ) {
            if( *start == 0 )
                containsNull = TRUE;
            sprintf( &( byteBlockBuffer[ bufferCounter ]), MiscStrings[ 54 ], *start );
            bufferCounter += 3;         // "3" is the length added when we use "MiscStrings[ 54 ]" as a format string.
            if( bufferCounter >= (BYTE_BLOCK_BUFFER_SIZE - 3) ) {
                tprintf2( aPort, byteBlockBuffer );
                bufferCounter = 0;
                byteBlockBuffer[ bufferCounter ] = 0;
                }
            ++start;
            }
        tprintf2( aPort, byteBlockBuffer );
        if( containsNull )
            tprintf3( &CurrentOutputPort, MiscStrings[ 366 ], (Byte *)(a + 1) );
        return TRUE;
        }
    else {
        OUTPUT_TO_ERROR
        printAnyTaggedAval(aPort, a );
        resetError( 5 );
        return FALSE;
        }
    }
        
/**** displayPointerToByteBlock  -- Canonical "display" routine for the indicated data type.  ****/

Boolean displayPointerToByteBlock( taggedAval *aPort )
{
    return displayByteBlockPointer( &R, aPort );
    }

/**** displayByteBlock -- Canonical "display" routine for the indicated data type.  ****/

Boolean displayByteBlock( taggedAval *aPort )
{
    tprintf3( aPort, MiscStrings[ 39 ], (UnsignedLongWord)(R.u.c) );
    return TRUE;
    }

/**** displayCharacter -- Canonical "display" routine for the indicated data type.  ****/

Boolean displayCharacter( taggedAval *aPort )
{  
    tprintf3( aPort, MiscStrings[ 57 ], R.u.c );
    return TRUE;
    }

/**** writeCharacter -- Canonical "write" routine for the indicated data type.  ****/

Boolean writeCharacter( taggedAval *aPort )
{
    if( R.u.c == SPACE )
        tprintf2( aPort, MiscStrings[ 49 ]);
    else if( R.u.c == NEWLINE )
        tprintf2( aPort, MiscStrings[ 48 ]);
    else if( R.u.c == TAB )
        tprintf2( aPort, MiscStrings[ 333 ]);
    else if( R.u.c == RETURN )
        tprintf2( aPort, MiscStrings[ 334 ]);
    else
        tprintf3( aPort, MiscStrings[ 47 ], R.u.c );
    return TRUE;
    }

/**** displayLambdaName -- Canonical "display" routine for the name, if any, of a lambda expression.  ****/

Boolean displayLambdaName( taggedAval *aPort )
{
    SPushR();
    R.t = pairTag;
    cdr();
    cdr();
    cdr();
    if( symbol_p( R.t ) )
        (void)((dataTypeTable[pure_type_code(R.t)])->display)(aPort);
    else
        tprintf2( aPort, MiscStrings[ 376 ]);
    if( getPortBigArrayLocation( aPort ) == conoutIndex )
        fflush( conOut->fp );
    SPopR();
    return TRUE;
    }
        
/**** displayLambdaPair -- Canonical "display" routine for the indicated data type.  ****/

Boolean displayLambdaPair( taggedAval *aPort )
{
    if( lambdaIsInterpretedP() ) {
        tprintf2( aPort, MiscStrings[ 40 ] );
        displayLambdaName( aPort );
        tprintf2( aPort, MiscStrings[ 375 ] );
        }
    else {
        LOAD_R_TAG( pairTag )
        CDR
        CAR
        if( PURE_TYPE_CODE( R ) == schemeMachinePCTag )
            tprintf3( aPort, MiscStrings[ 42 ], (UnsignedLongWord)(R.u.c) );
        else
            tprintf2( aPort, MiscStrings[ 241 ] );
        }
    return TRUE;
    }
        
/**** displayLambdaSource ****/

Boolean displayLambdaSource( taggedAval *aPort )
{
    PUSH
    LOAD_R_TAG( pairTag )
    PUSH
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( NameStrings[ 118 ], R )
    CALL( intern )
    SWAP
    PUSH                                    /* S: ( (lambdaList . (lambdaBody env . name)) "lambda" ...) */
    CAR          
    SWAP               /* S: ( lambdaList "lambda" ...) */
    CDR          
    PAIR_P
    JUMP_FALSE( needInterpretedLambda )
    CAR          
    CONS
    CONS
    (void)((dataTypeTable[pure_type_code(R.t)])->display)( aPort);
    if( getPortBigArrayLocation( aPort ) == conoutIndex )
        fflush( conOut->fp );
    POP
    return TRUE;

  needInterpretedLambda:
    resetError( 52 );
    return TRUE;
    }

/**** inspectLambdaPair -- Canonical "inspect" routine for the indicated data type.  ****/

Boolean inspectLambdaPair( taggedAval *aPort )
{
    SPushR();
    SPushR();
    inspectAnyTaggedAval( aPort );
    R = S[0];
    if( lambdaIsInterpretedP() )
        displayLambdaSource( aPort );
    else {
        displayLambdaPair( aPort );
        tprintf3( aPort, MiscStrings[ 57 ], NEWLINE );                
        R.t = pairTag;
        displayPair( aPort );
        }
    tprintf3( aPort, MiscStrings[ 57 ], NEWLINE );
    SPopR();
    R.t = pairTag;
    CDR
    CDR
    CAR
    tprintf2( aPort, MiscStrings[ 348 ] );
    CALL( printDynamicEnvironmentListFromR );
    SPopR();
    return TRUE;
    }
        
/**** displayForwardingPointer -- Canonical "display" routine for the indicated data type.  ****/

Boolean displayForwardingPointer( taggedAval *aPort )
{
    R = spliceOutNotCarForwardingPointers( R );
    R = *(R.u.a);
    PUSH
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( aPort, R )
    SWAP
    DISPLAY_ON_PORT
    DROP
    return TRUE;
    }

/**** displayGGCForwardingPointer -- Canonical "display" routine for the indicated data type.  ****/

Boolean displayGGCForwardingPointer( taggedAval *aPort )
{
    R = spliceOutNotCarForwardingPointers( R );
    R = *(R.u.a);
    PUSH
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( aPort, R )
    SWAP
    DISPLAY_ON_PORT
    DROP
    return TRUE;
    }

/**** displayExactShortFloat ****/

Boolean displayExactShortFloat( taggedAval *aPort )
{
    return (Boolean)tprintf3( aPort, MiscStrings[ 52 ], R.u.f );
    }

/**** displayInexactShortFloat ****/

Boolean displayInexactShortFloat( taggedAval *aPort )
{
    return (Boolean)tprintf3( aPort, MiscStrings[ 52 ], R.u.f );
    }

/**** displayMacro -- Canonical "display" routine for the indicated data type.  ****/

Boolean displayMacro( taggedAval *aPort )
{
    tprintf2( aPort, MiscStrings[ 211 ] );
    return TRUE;
    }
        
/**** displayMacroSource ****/

Boolean displayMacroSource( taggedAval *aPort )
{
    LOAD_R_TAG( lambdaPairTag )
    return displayLambdaSource( aPort );
    }        

/**** inspectMacro -- Canonical "inspect" routine for the indicated data type.  ****/

Boolean inspectMacro( taggedAval *aPort )
{
    SPushR();
    inspectAnyTaggedAval( aPort );
    R = S[0];
    LOAD_R_TAG( lambdaPairTag )
    inspectLambdaPair( aPort );
    tprintf3( aPort, MiscStrings[ 57 ], NEWLINE );
    SPopR();
    return TRUE;
    }
        
/**** displayStackReference -- Canonical "display" routine for the indicated data type.  ****/

Boolean displayStackReference( taggedAval *aPort )
{
    return (Boolean)tprintf3( aPort, MiscStrings[ 242 ], (UnsignedLongWord)(R.u.c) );
    }

/**** inspectStackReference -- Canonical "inspect" routine for the indicated data type.  ****/

Boolean inspectStackReference( taggedAval *aPort )
{
    SPushR();
    inspectAnyTaggedAval( aPort );
    tprintf3( aPort, MiscStrings[ 242 ], (UnsignedLongWord)(R.u.c) );
    tprintf2( aPort, MiscStrings[ 199 ] );
    R = S[ (UnsignedLongWord)(S[0].u.c) + 1 ];                        /* did one extra push herein */
    printAnyTaggedAval( aPort, &R );
    tprintf3( aPort, MiscStrings[ 57 ], NEWLINE );
    SPopR();
    return TRUE;
    }

/**** displayEnvironmentReference -- Canonical "display" routine for the indicated data type.  ****/

Boolean displayEnvironmentReference( taggedAval *aPort )
{
    UnsignedLongWord environmentNumber = ((((UnsignedLongWord)(R.u.c)) >> environmentNumberOffset) & lowerWordMask);
    UnsignedLongWord variableNumber    = (((UnsignedLongWord)(R.u.c)) & lowerWordMask);

    return (Boolean)tprintf4( aPort, MiscStrings[ 275 ], variableNumber, environmentNumber );
    }

/**** inspectEnvironmentReference -- Canonical "inspect" routine for the indicated data type.  ****/

Boolean inspectEnvironmentReference( taggedAval *aPort )
{
    UnsignedLongWord environmentNumber = ((((UnsignedLongWord)(R.u.c)) >> environmentNumberOffset) & lowerWordMask);
    UnsignedLongWord variableNumber    = (((UnsignedLongWord)(R.u.c)) & lowerWordMask);

    SPushR();
    inspectAnyTaggedAval( aPort );
    tprintf4( aPort, MiscStrings[ 275 ], variableNumber, environmentNumber );
    tprintf2( aPort, MiscStrings[ 199 ] );

    R = E;
    while( environmentNumber-- > 0 )
        cdr();
    car();
    R.t = pairTag;
    while( variableNumber-- > 0 )
        cdr();
    car();

    printAnyTaggedAval( aPort, &R );
    tprintf3( aPort, MiscStrings[ 57 ], NEWLINE );
    SPopR();
    return TRUE;
    }

/**** displayTopLevelEnvironmentReference -- Canonical "display" routine for the indicated data type.  ****/

Boolean displayTopLevelEnvironmentReference( taggedAval *aPort )
{
    UnsignedLongWord bucket = ((((UnsignedLongWord)(R.u.c)) >> bucketNumberOffset) & lowerWordMask);
    UnsignedLongWord variableNumber    = (((UnsignedLongWord)(R.u.c)) & lowerWordMask);

    return (Boolean)tprintf4( aPort, MiscStrings[ 276 ], variableNumber, bucket );
    }

/**** inspectTopLevelEnvironmentReference -- Canonical "inspect" routine for the indicated data type.  ****/

Boolean inspectTopLevelEnvironmentReference( taggedAval *aPort )
{
    UnsignedLongWord environmentNumber = ((((UnsignedLongWord)(R.u.c)) >> bucketNumberOffset) & lowerWordMask);
    UnsignedLongWord variableNumber    = (((UnsignedLongWord)(R.u.c)) & lowerWordMask);

    SPushR();
    inspectAnyTaggedAval( aPort );
    tprintf4( aPort, MiscStrings[ 276 ], variableNumber, environmentNumber );
    tprintf2( aPort, MiscStrings[ 199 ] );

    R = E;
    while( environmentNumber-- > 0 )
    cdr();
    car();
    R.t = pairTag;
    while( variableNumber-- > 0 )
    cdr();
    car();

    printAnyTaggedAval( aPort, &R );
    tprintf3( aPort, MiscStrings[ 57 ], NEWLINE );
    SPopR();
    return TRUE;
    }

/**** displayPair -- Canonical "display" routine for the indicated data type.  "If the cdr / Has a cdr / Do not print a dot... " ****/

UnsignedLongWord printDepth  = 10;           /* print no more than this deep into nested lists */
UnsignedLongWord depthSoFar  = 0;            /* reinitialized each time through topLoop */
UnsignedLongWord printLength = 20;           /* print no more than this many elements of a list */

Boolean displayPair( taggedAval *aPort )
{
    UnsignedLongWord noPrinted = 0;

    R = spliceOutNotCarForwardingPointers( R );
    ++depthSoFar;       
    tprintf3( aPort, MiscStrings[ 57 ],  LIST_OPENER );                       /* print the opening "(" */
    PUSH
    MOVE( *aPort, R )
    SWAP
    if( depthSoFar <= printDepth ) {
        PUSH
        CAR
        EXCHANGE( STACK( 0 ), STACK( 1 ) )
        DISPLAY_ON_PORT                                                                 /* print the car */
        EXCHANGE( STACK( 0 ), STACK( 1 ) )
        POP
        CDR
        while( !empty_list_p( R.t ) )
            if( ++noPrinted >= printLength ) {                                          /* enough already!! */
                tprintf2( aPort, MiscStrings[ 3 ]);
                break;
                }
            else if( pair_p( R.t ) ) {                                                  /* if the cdr has a cdr, do not print a dot */
                tprintf3( aPort, MiscStrings[ 57 ], SPACE );
                PUSH
                CAR
                EXCHANGE( STACK( 0 ), STACK( 1 ) )
                DISPLAY_ON_PORT                                                         /*   ... print the cadr ... */
                EXCHANGE( STACK( 0 ), STACK( 1 ) )
                POP
                CDR                                                                     /*   ... and cdr down the list */
                }
            else {                                                                      /* else, DO print a dot ...*/
                tprintf2( aPort, MiscStrings[ 29 ]);
                DISPLAY_ON_PORT                                                         /*   ... and print the cdr ... */
                break;                                                                  /*   ... and break the loop */
                }
        }
    else
        tprintf2( aPort, MiscStrings[ 3 ] );
    tprintf3( aPort, MiscStrings[ 57 ], LIST_CLOSER );                        /* print the final ")" */
    DROP
    --depthSoFar;
    return TRUE;
    }

/**** writePair -- Canonical "display" routine for the indicated data type.  "If the cdr / Has a cdr / Do not print a dot... " ****/

Boolean writePair( taggedAval *aPort )
{
    UnsignedLongWord noPrinted = 0;

    R = spliceOutNotCarForwardingPointers( R );
    ++depthSoFar;       
    tprintf3( aPort, MiscStrings[ 57 ], LIST_OPENER );                              /* print the opening "(" */
    PUSH
    MOVE( *aPort, R )
    SWAP
    if( depthSoFar <= printDepth ) {
        PUSH
        CAR
        EXCHANGE( STACK( 0 ), STACK( 1 ) )
        WRITE_ON_PORT                                                               /* print the car */
        EXCHANGE( STACK( 0 ), STACK( 1 ) )
        POP
        CDR
        while( !empty_list_p( R.t ) )
            if( ++noPrinted >= printLength ) {                                      /* enough already!! */
                tprintf2( aPort, MiscStrings[ 3 ] );
                break;
                }
            else if( pair_p( R.t ) ) {                                              /* if the cdr has a cdr / do not print a dot */
                tprintf3( aPort, MiscStrings[ 57 ], SPACE );
                PUSH
                CAR
                EXCHANGE( STACK( 0 ), STACK( 1 ) )
                WRITE_ON_PORT                                                       /*   ... print the cadr ... */
                EXCHANGE( STACK( 0 ), STACK( 1 ) )
                POP
                CDR                                                                 /*   ... and cdr down the list */
                }
            else {                                                                  /* else, DO print a dot ...*/
                tprintf2( aPort, MiscStrings[ 29 ] );
                WRITE_ON_PORT                                                       /*   ... and print the cdr ... */
                break;                                                              /*   ... and break the loop */
                }
        }
    else
        tprintf2( aPort, MiscStrings[ 3 ] );
    tprintf3( aPort, MiscStrings[ 57 ], LIST_CLOSER );                    /* print the final ")" */
    --depthSoFar;
    DROP
    return TRUE;
    }

/**** inspectPair -- Canonical "inspect" routine for the indicated data type.  ****/

Boolean inspectPair( taggedAval *aPort )
{
    Tag tg;
    taggedAval *p;

    R = spliceOutNotCarForwardingPointers( R );
    SPushR();
    inspectAnyTaggedAval( aPort );
    R = S[0];
    R = spliceOutNotCarForwardingPointers( R );
    p = R.u.a;
    R = *p;
    tg = R.t;
    tprintf2( aPort, MiscStrings[ 17 ] ); 
    inspectAnyTaggedAval( aPort );
    if( car_cdr_next_p( tg ) ) {
        R = *(++p);
        tprintf2( aPort, MiscStrings[ 20 ] ); 
        inspectAnyTaggedAval( aPort );
        }
    R = S[0];
    writePair( aPort );
    tprintf3( aPort, MiscStrings[ 57 ], NEWLINE );
    SPopR();
    return TRUE;
    }

/**** displaySchemeMachinePC -- Canonical "display" routine for the indicated data type.  ****/

Boolean displaySchemeMachinePC( taggedAval *aPort )
{
    tprintf3( aPort, MiscStrings[ 45 ], (UnsignedLongWord)(R.u.c) );
    return TRUE;
    }

/**** displaySchemeMachinePCIndex -- Canonical "display" routine for the indicated data type.  ****/

Boolean displaySchemeMachinePCIndex( taggedAval *aPort )
{
    tprintf4( aPort, MiscStrings[ 374 ], (UnsignedLongWord)(R.u.c),
              theWorldHandler->pcForIndex( (UnsignedLongWord)(R.u.c) ) );
    return TRUE;
    }

/**** displaySpecialForm -- Canonical "display" routine for the indicated data type.  ****/

Boolean displaySpecialForm( taggedAval *aPort )
{
    tprintf2( aPort, MiscStrings[ 37 ] );
    CALL( getSpecialPName )
    PUSH
    MOVE( *aPort, R )
    SWAP
    DISPLAY_ON_PORT
    tprintf2( aPort, MiscStrings[ 109 ] );
    DROP
    return TRUE;
    }

/**** displayString -- Canonical "display" routine for the indicated data type.  ****/

#define STRING_BUFFER_SIZE 200

Boolean displayString( taggedAval *aPort )
{
    static Text stringBuffer[ STRING_BUFFER_SIZE ];

    UnsignedLongWord bufferCounter = 0;
    taggedAval* byteBlockPointer = R.u.a;
    UnsignedLongWord length = lengthString( &R );
    Text *start = (Text *)((Byte *)byteBlockPointer + sizeof(taggedAval));

    while( length-- > 0 ) {
        stringBuffer[ bufferCounter++ ] = *start++;
        if( bufferCounter == (STRING_BUFFER_SIZE - 1) ) {
            stringBuffer[ bufferCounter ] = 0;
            tprintf3( aPort, MiscStrings[ 201 ], stringBuffer );
            bufferCounter = 0;
            }
        }
    stringBuffer[ bufferCounter ] = 0;
    tprintf3( aPort, MiscStrings[ 201 ],stringBuffer );
    return TRUE; 
    }

/**** writeString -- Canonical "write" routine for the indicated data type.  ****/

Boolean writeString( taggedAval *aPort )
{
    static Text stringBuffer[ STRING_BUFFER_SIZE ];

    UnsignedLongWord bufferCounter = 0;
    taggedAval* byteBlockPointer = R.u.a;

    UnsignedLongWord length = lengthString( &R );
    Text c;
    Text *start = (Text *)((Byte *)byteBlockPointer + sizeof(taggedAval));

    stringBuffer[ bufferCounter++ ] = DOUBLE_QUOTE;      
    while( length-- > 0 ) {
        c = *start++;
        if( c == '\n' ) {
            strcpy( stringBuffer + bufferCounter, MiscStrings[ 330 ] );
            bufferCounter += 2;
            }          
        else if( c == '\r' ) {
            strcpy( stringBuffer + bufferCounter, MiscStrings[ 331 ] );
            bufferCounter += 2;
            }          
        else if( c == '\t' ) {
            strcpy( stringBuffer + bufferCounter, MiscStrings[ 332 ] );
            bufferCounter += 2;
            }          
        else {
            if( (c == *MiscStrings[ 106 ]) || (c == *MiscStrings[ 112 ]) ) {
                strcpy( stringBuffer + bufferCounter, MiscStrings[ 330 ] );
                ++bufferCounter;
                }          
            stringBuffer[ bufferCounter++ ] = c;
            }
        if( bufferCounter >= (STRING_BUFFER_SIZE - 3) ) {
            stringBuffer[ bufferCounter ] = 0;
            tprintf3( aPort, MiscStrings[ 201 ], stringBuffer );
            bufferCounter = 0;
            }
        }
    stringBuffer[ bufferCounter++ ] = DOUBLE_QUOTE;      
    stringBuffer[ bufferCounter ] = 0;
    tprintf3( aPort, MiscStrings[ 201 ], stringBuffer );
    return TRUE;
    }

/**** displaySymbol -- Canonical "display" routine for the indicated data type.  ****/

Boolean displaySymbol( taggedAval *aPort )
{
    taggedAval* byteBlockPointer;
    UnsignedLongWord length;
    Text *start;
    static Text stringBuffer[ STRING_BUFFER_SIZE ];
    UnsignedLongWord bufferCounter = 0;

    getBlockedPName();    //  Just in case ...
    byteBlockPointer = R.u.a;
    start = (Text *)((Byte *)byteBlockPointer + sizeof(taggedAval));
    length = lengthString( &R );

    while( length-- > 0 ) {
        stringBuffer[ bufferCounter++ ] = *start++;
        if( bufferCounter == (STRING_BUFFER_SIZE - 1) ) {
            stringBuffer[ bufferCounter ] = 0;
            tprintf2( aPort, stringBuffer );
            bufferCounter = 0;
            }
        }
    stringBuffer[ bufferCounter ] = 0;
    tprintf2( aPort, stringBuffer );
    return TRUE;
    }

Boolean printfSymbol()
{
    taggedAval* byteBlockPointer;
    UnsignedLongWord length;
    Text *start;
    static Text stringBuffer[ STRING_BUFFER_SIZE ];
    UnsignedLongWord bufferCounter = 0;

    GET_PNAME
    byteBlockPointer = R.u.a;
    start = (Text *)((Byte *)byteBlockPointer + sizeof(taggedAval));
    length = lengthString( &R );

    while( length-- > 0 ) {
        stringBuffer[ bufferCounter++ ] = *start++;
        if( bufferCounter == (STRING_BUFFER_SIZE - 1) ) {
            stringBuffer[ bufferCounter ] = 0;
            fputs( stringBuffer, stdout );
            bufferCounter = 0;
            }
        }
    stringBuffer[ bufferCounter ] = 0;
    fputs( stringBuffer, stdout );
    return TRUE;
    }

/**** inspectSymbol -- Canonical "inspect" routine for the indicated data type.  ****/

Boolean inspectSymbol( taggedAval *aPort )
{
    PUSH
    boundSymbolP();
    if( B ) {
        (void)displaySymbolAndBinding( aPort );
        tprintf3( aPort, MiscStrings[ 57 ], NEWLINE );
        }
    else {
        MOVE( STACK( 0 ), R )
        (void)displaySymbol( aPort );
        tprintf2( aPort, MiscStrings[ 277 ] );
        tprintf3( aPort, MiscStrings[ 57 ], NEWLINE );
        }       
    POP
    return TRUE;
    }       

/**** displaySymbolAndBinding -- Canonical "display" routine for a binding pair.
                                                        This routine does not handle unbound symbols reasonably.    ****/

Boolean displaySymbolAndBinding( taggedAval *aPort )
{
    PUSH
    CDR
    displaySymbol( aPort );
    POP
    tprintf2( aPort, MiscStrings[ 15 ] );
    CAR
    PUSH
    MOVE( *aPort, R )
    SWAP
    DISPLAY_ON_PORT
    DROP
    return TRUE;
    }

/**** displayVector -- Canonical "display" routine for the indicated data type.  ****/

UnsignedLongWord vPrintDepth  = 10;             /* print no more than this deep into nested vectors */
UnsignedLongWord vDepthSoFar  = 0;              /* reinitialized each time through topLoop */   
UnsignedLongWord vPrintLength = 20;             /* print no more than this many elements of a vector */

Boolean displayVector( taggedAval *aPort )
{
    UnsignedLongWord noPrinted = 0;
    UnsignedLongWord toGo;

    ++vDepthSoFar;  
    tprintf2( aPort, MiscStrings[ 36 ] );                         /* print the opening "#(" */
    PUSH
    MOVE( *aPort, R )
    SWAP
    if( vDepthSoFar <= vPrintDepth ) {
        LOAD_R_TAG( pairTag )
        PUSH
        CAR
        toGo = (UnsignedLongWord)(R.u.c);
        POP
        CDR
        while( toGo-- > 0 ) {
            if( noPrinted++ >= vPrintLength ) {                             /* enough already!! */
                tprintf2( aPort, MiscStrings[ 107 ] );
                break;
                }
            PUSH
            CAR
            EXCHANGE( STACK( 0 ), STACK( 1 ) )
            DISPLAY_ON_PORT
            EXCHANGE( STACK( 0 ), STACK( 1 ) )
            POP
            CDR
            if( toGo > 0 )
                tprintf3( aPort, MiscStrings[ 57 ], SPACE );
            }
        }
    else
        tprintf2( aPort, MiscStrings[ 3 ] );
    tprintf3( aPort, MiscStrings[ 57 ],  LIST_CLOSER );           /* print the closing ")" */
    DROP
    --vDepthSoFar;
    return TRUE;
    }

/**** writeVector -- Canonical "write" routine for the indicated data type.  ****/

Boolean writeVector( taggedAval *aPort )
{
    UnsignedLongWord noPrinted = 0;
    UnsignedLongWord toGo;

    ++vDepthSoFar;  
    tprintf2( aPort, MiscStrings[ 36 ] );                         /* print the opening "#(" */
    PUSH
    MOVE( *aPort, R )
    SWAP
    if( vDepthSoFar <= vPrintDepth ) {
        LOAD_R_TAG( pairTag )
        PUSH
        CAR
        toGo = (UnsignedLongWord)(R.u.c);
        POP
        CDR
        while( toGo-- > 0 ) {
            if( noPrinted++ >= vPrintLength ) {                             /* enough already!! */
                tprintf2( aPort, MiscStrings[ 107 ] );
                break;
                }
            PUSH
            CAR
            EXCHANGE( STACK( 0 ), STACK( 1 ) )
            WRITE_ON_PORT
            EXCHANGE( STACK( 0 ), STACK( 1 ) )
            POP
            CDR
            if( toGo > 0 )
                tprintf3( aPort, MiscStrings[ 57 ], SPACE );
            }
        }
    else
        tprintf2( aPort, MiscStrings[ 3 ] );
    tprintf3( aPort, MiscStrings[ 57 ], LIST_CLOSER );            /* print the closing ")" */
    --vDepthSoFar;
    DROP
    return TRUE;
    }

/**** inspectVector -- Canonical "inspect" routine for the indicated data type.  ****/

Boolean inspectVector( taggedAval *aPort )
{
    SPushR();
    inspectAnyTaggedAval( aPort );
    R = S[0];
    VLENGTH
    tprintf3( aPort, MiscStrings[ 2 ], R.u.c );
    R = S[0];
    writeVector( aPort );
    tprintf3( aPort, MiscStrings[ 57 ], NEWLINE );
    SPopR();
    return TRUE;
    }

/**** displayPromise -- Canonical "display" routine for the indicated data type.  ****/

Boolean displayPromise( taggedAval *aPort )
{
    tprintf2( aPort, MiscStrings[ 44 ] );
    return TRUE;
    }

/**** inspectPromise -- Canonical "inspect" routine for the indicated data type.  ****/

Boolean inspectPromise( taggedAval *aPort )
{
    SPushR();
    inspectAnyTaggedAval( aPort );
    R = S[0];
    R.t = pairTag;
    SPushR();
    cdr();
    if( R.t == emptyListTag ) {
        tprintf2( aPort, MiscStrings[ 41 ]);
        SPopR();
        car();
        displayObject();
        }
    else {
        tprintf2( aPort, MiscStrings[ 46 ]);
        SPopR();
        car();
        displayLambdaSource( aPort );
        }
    tprintf2( aPort, MiscStrings[ 115 ] );
    SPopR();
    return TRUE;
    }

/**** displayPeculiarLeaf -- Canonical "display" routine for the indicated data type.  ****/

Boolean displayPeculiarLeaf( taggedAval *aPort )
{
    UnsignedLongWord n;

    n = leaf_number( R );
    switch( n ) {
        case consDotterPLTag:           tprintf2( aPort, MiscStrings[ 203 ] );     break;
        case eofObjectPLTag:            tprintf2( aPort, MiscStrings[ 204 ] );     break;
        case whereTLEPointsPLTag:       tprintf2( aPort, MiscStrings[ 466 ] );     break;
        case whereObListPointsPLTag:    tprintf2( aPort, MiscStrings[ 467 ] );     break;
        case nonPrintingObjectTag:                                                 break;  // Doesn't print!
        default:                        tprintf3( aPort, MiscStrings[ 43 ], n );   break;
        }
    return TRUE;
    }

/**** inspectPeculiarLeaf -- Canonical "inspect" routine for the indicated data type.  ****/

Boolean inspectPeculiarLeaf( taggedAval *aPort )
{
    UnsignedLongWord n;

    SPushR();
    inspectAnyTaggedAval( aPort );
    SPopR();
    n = leaf_number( R );
    switch( n ) {
        case consDotterPLTag:           tprintf2( aPort, MiscStrings[ 203 ] );     break;
        case eofObjectPLTag:            tprintf2( aPort, MiscStrings[ 204 ] );     break;
        case whereTLEPointsPLTag:       tprintf2( aPort, MiscStrings[ 466 ] );     break;
        case whereObListPointsPLTag:    tprintf2( aPort, MiscStrings[ 467 ] );     break;
        case nonPrintingObjectTag:      tprintf2( aPort, MiscStrings[ 468 ] );     break;
        default:                        tprintf3( aPort, MiscStrings[ 43 ], n );   break;
        }
    tprintf2( aPort, MiscStrings[ 115 ] );
    return TRUE;
    }

/**** displayPeculiarNonLeaf -- Canonical "display" routine for the indicated data type.  ****/

Boolean displayPeculiarNonLeaf( taggedAval *aPort )
{
    UnsignedLongWord n = 0;
    Word kitten = 0;

    n = getBigArrayLocation( R.u.a );
    kitten = getPeculiarNonLeafKitten( R.u.a );
    tprintf3( aPort, MiscStrings[ 179 ], n );

    return TRUE;
    }

/**** inspectPeculiarNonLeaf -- Canonical "inspect" routine for the indicated data type.  ****/

Boolean inspectPeculiarNonLeaf( taggedAval *aPort )
{
    UnsignedLongWord n = 0;

    SPushR();
    inspectAnyTaggedAval( aPort );
    SPopR();
    n = getBigArrayLocation( R.u.a );
    tprintf3( aPort, MiscStrings[ 180 ], n );

    displayByteBlockPointer( R.u.a, aPort );

    return TRUE;
    }

/**** getPortFD -- get the file descriptor associated with a port ****/

fileDescriptor *getPortFD( taggedAval *aPort )
{
    UnsignedLongWord n = 0;
    UnsignedLongWord kitten = 0;
    
    if( ! port_p( aPort->t ) ) {
        useFatalErrorOutput();
        fatalError( 150 );
        }
    n = getPortBigArrayLocation( aPort );
    kitten = getPortKitten( aPort );
    if( theKittenNumber != kitten ) {
        tprintf5( &CurrentOutputPort, MiscStrings[  232 ], theKittenNumber, n, kitten );
        resetError( 208 );
        }
    return &(AllFileDescriptors[ n ]);
    }

/**** getPortFilePointer -- get the file descriptor associated with a port ****/

FILE *getPortFilePointer( taggedAval *aPort )
{
    UnsignedLongWord n = 0;
    UnsignedLongWord kitten = 0;
    
    if( ! port_p( aPort->t ) ) {
        useFatalErrorOutput();
        printAnyTaggedAval( &CurrentOutputPort, aPort );
        fatalError( 151 );
        }
    n = getPortBigArrayLocation( aPort );
    kitten = getPortKitten( aPort );
    if( theKittenNumber != kitten ) {
        tprintf5( &CurrentOutputPort, MiscStrings[  232 ], theKittenNumber, n, kitten );
        resetError( 209 );
        }
    return (AllFileDescriptors[ n ]).fp;
    }

/**** displayPort -- Canonical "display" routine for the indicated data type.  ****/

Boolean displayPort( taggedAval *aPort )
{
    fileDescriptor *sfd;
    UnsignedLongWord n = 0;
    Word kitten = 0;

    n = getPortBigArrayLocation( &R );
    kitten = getPortKitten( &R );
    sfd = &(AllFileDescriptors[ n ] );
    if( n < nInputFileDescriptors ) {
        tprintf7( aPort, MiscStrings[ 181 ], n, kitten, sfd->pathName,
                  (sfd->fdOpen ? MiscStrings[ 305 ] : MiscStrings[ 306 ]), 
                  ((strlen( sfd->oldPathName ) == 0) ? MiscStrings[ 307 ] : sfd->oldPathName) ); 

        }
    else if( n < nInputFileDescriptors + nOutputFileDescriptors ) {
        tprintf7( aPort, MiscStrings[ 182 ], n - nInputFileDescriptors, kitten, sfd->pathName,
                  (sfd->fdOpen ? MiscStrings[ 305 ] : MiscStrings[ 306 ]), 
                  ((strlen( sfd->oldPathName ) == 0) ? MiscStrings[ 307 ] : sfd->oldPathName) ); 
        }
    else if( n == coninIndex )
        tprintf2( aPort, MiscStrings[ 205 ] );
    else if( n == conoutIndex )
        tprintf2( aPort, MiscStrings[ 206 ] );
    else if( n == conerrIndex )
        tprintf2( aPort, MiscStrings[ 236 ] );
    else if( n == closedFileIndex )
        tprintf2( aPort, MiscStrings[ 184 ] );
    else
        tprintf7( aPort, MiscStrings[ 247 ], n, kitten, sfd->pathName,
                  (sfd->fdOpen ? MiscStrings[ 305 ] : MiscStrings[ 306 ]), 
                  ((strlen( sfd->oldPathName ) == 0) ? MiscStrings[ 307 ] : sfd->oldPathName) ); 

    return TRUE;
    }

Boolean printfPort( taggedAval *thePort, FILE *where, Text *message )
{
    fileDescriptor *sfd;
    UnsignedLongWord n = 0;
    Word kitten = 0;

    fputs( message, where );
    n = getPortBigArrayLocation( thePort );
    kitten = getPortKitten( thePort );
    sfd = &(AllFileDescriptors[ n ] );
    if( n < nInputFileDescriptors )
        fprintf( where, MiscStrings[ 181 ], n, kitten, sfd->pathName,
                 (sfd->fdOpen ? MiscStrings[ 305 ] : MiscStrings[ 306 ]), 
                 ((strlen( sfd->oldPathName ) == 0) ? MiscStrings[ 307 ] : sfd->oldPathName) ); 

    else if( n < nInputFileDescriptors + nOutputFileDescriptors )
        fprintf( where, MiscStrings[ 182 ], n - nInputFileDescriptors, kitten, sfd->pathName,
                 (sfd->fdOpen ? MiscStrings[ 305 ] : MiscStrings[ 306 ]), 
                 ((strlen( sfd->oldPathName ) == 0) ? MiscStrings[ 307 ] : sfd->oldPathName) ); 
    else if( n == coninIndex )
        fputs( MiscStrings[ 205 ], where );
    else if( n == conoutIndex )
        fputs( MiscStrings[ 206 ], where );
    else if( n == conerrIndex )
        fputs( MiscStrings[ 236 ], where );
    else if( n == closedFileIndex )
        fputs( MiscStrings[ 184 ], where );
    else
        fprintf( where, MiscStrings[ 247 ], n, kitten, sfd->pathName,
                  (sfd->fdOpen ? MiscStrings[ 305 ] : MiscStrings[ 306 ]), 
                  ((strlen( sfd->oldPathName ) == 0) ? MiscStrings[ 307 ] : sfd->oldPathName) ); 


    return TRUE;
    }

/**** inspectPort -- Canonical "inspect" routine for the indicated data type.  ****/

Boolean inspectPort( taggedAval *aPort )
{
    fileDescriptor *sfd;
    UnsignedLongWord n = 0;

    SPushR();
    inspectAnyTaggedAval( aPort );
    SPopR();
    n = getPortBigArrayLocation( &R );
    sfd = &(AllFileDescriptors[ n ] );
    if( n < nInputFileDescriptors + nOutputFileDescriptors )
        tprintf5( aPort, MiscStrings[ 183 ], sfd->pathName, sfd->type, sfd->oldPathName );
    else if( n == coninIndex )
        tprintf2( aPort, MiscStrings[ 205 ] );
    else if( n == conoutIndex )
        tprintf2( aPort, MiscStrings[ 206 ] );
    else if( n == conerrIndex )
        tprintf2( aPort, MiscStrings[ 236 ] );
    else if( n == closedFileIndex )
        tprintf2( aPort, MiscStrings[ 184 ] );
    else
        tprintf3( aPort, MiscStrings[ 180 ], n );

    displayByteBlockPointer( R.u.a, aPort );

    return TRUE;
    }

/**** displayExactShortFixnum -- Canonical "display" routine for the indicated data type.  ****/

Boolean displayExactShortFixnum( taggedAval *aPort )
{
    return (Boolean)tprintf3( aPort, MiscStrings[ 60 ], R.u.c );
    }

/**** displayFalse -- Canonical "display" routine for the indicated data type.  ****/

Boolean displayFalse( taggedAval *aPort )
{
    return (Boolean)tprintf2( aPort, MiscStrings[ 50 ] );
    }

/**** displayEmptyList -- Canonical "display" routine for the indicated data type.  ****/

Boolean displayEmptyList( taggedAval *aPort )
{
     return (Boolean)tprintf2( aPort, MiscStrings[ 278 ] ); 
    }

/**** displayInexactShortFixnum -- Canonical "display" routine for the indicated data type.  ****/

Boolean displayInexactShortFixnum( taggedAval *aPort )
{
    return (Boolean)tprintf3( aPort, MiscStrings[ 60 ], R.u.c );
    }

/**** displaySyntaxBinding -- Canonical "display" routine for the indicated data type.  ****/

Boolean displaySyntaxBinding( taggedAval *aPort )
{
    LOAD_R_TAG( pairTag )
    PUSH          // Stack 1 deep
    CAR           // The symbol
    tprintf2( aPort, MiscStrings[ 354 ] );      //  "<"
    tprintf2( aPort, DataTypeStrings[ 22 ] );   //  "Syntax Binding"
    tprintf2( aPort, MiscStrings[ 264 ] );      //  ": "
    displaySymbol( aPort );                         //  the symbol
    POP
    PUSH          // Stack 1 deep
    PUSH          // Stack 2 deep
    CDR
    CAR                                                  //  is it bound?
    FALSE_P
    JUMP_TRUE( unbound )

    MOVE( *aPort, R )
    SWAP
    CDR
    CDR
    CAR           // the value
    tprintf2( aPort, MiscStrings[ 30 ] );       //  " <--> "
    DISPLAY_ON_PORT                                      //  the value
    DROP
    POP
    PUSH
    PUSH          // Stack 2 deep

  wrapUp:
    MOVE( *aPort, R )
    SWAP
    CDR
    CDR
    CDR
    tprintf2( aPort, MiscStrings[ 22 ] );       //  " "
    DISPLAY_ON_PORT                                      //  the rest of the syntax-binding list
    tprintf2( aPort, MiscStrings[ 355 ] );      //  ">"
    DROP
    POP          // Stack cleared
    return TRUE;

  unbound:
    tprintf2( aPort, MiscStrings[ 356 ] );      //  " -- unbound"
    POP
    PUSH         // Stack 2 deep
    JUMP_LABEL( wrapUp )
    }            

/**** writeSyntaxBinding -- Canonical "write" routine for the indicated data type.  ****/

Boolean writeSyntaxBinding( taggedAval *aPort )
{
    LOAD_R_TAG( pairTag )
    PUSH          // Stack 1 deep
    CAR           // The symbol
    tprintf2( aPort, MiscStrings[ 354 ] );      //  "<"
    tprintf2( aPort, DataTypeStrings[ 22 ] );   //  "Syntax Binding"
    tprintf2( aPort, MiscStrings[ 264 ] );      //  ": "
    displaySymbol( aPort );                         //  the symbol
    POP
    PUSH          // Stack 1 deep
    PUSH          // Stack 2 deep
    CDR
    CAR                                                  //  is it bound?
    FALSE_P
    JUMP_TRUE( unbound )

    MOVE( *aPort, R )
    SWAP
    CDR
    CDR
    CAR           // the value
    tprintf2( aPort, MiscStrings[ 30 ] );       //  " <--> "
    WRITE_ON_PORT                                        //  the value
    DROP
    POP
    PUSH
    PUSH          // Stack 2 deep

  wrapUp:
    MOVE( *aPort, R )
    SWAP
    CDR
    CDR
    CDR
    tprintf2( aPort, MiscStrings[ 22 ] );       //  " "
    WRITE_ON_PORT                                        //  the rest of the syntax-binding list
    tprintf2( aPort, MiscStrings[ 355 ] );      //  ">"
    DROP
    POP          // Stack cleared
    return TRUE;

  unbound:
    tprintf2( aPort, MiscStrings[ 356 ] );      //  " -- unbound"
    POP
    PUSH         // Stack 2 deep
    JUMP_LABEL( wrapUp )
    }            

/**** inspectSyntaxBinding -- Canonical "inspect" routine for the indicated data type.  ****/

Boolean inspectSyntaxBinding( taggedAval *aPort )
{
    LOAD_R_TAG( pairTag )
    PUSH          // Stack 1 deep
    CAR           // The symbol
    tprintf2( aPort, DataTypeStrings[ 22 ] );   //  "Syntax Binding"
    tprintf2( aPort, MiscStrings[ 264 ] );      //  ": "
    displaySymbol( aPort );                         //  the symbol
    POP
    PUSH          // Stack 1 deep
    PUSH          // Stack 2 deep
    CDR
    CAR                                                  //  is it bound?
    FALSE_P
    JUMP_TRUE( unbound )

    MOVE( *aPort, R )
    SWAP
    CDR
    CDR
    CAR           // the value
    tprintf2( aPort, MiscStrings[ 30 ] );       //  " <--> "
    INSPECT_ON_PORT                                      //  the value
    DROP
    POP
    PUSH
    PUSH          // Stack 2 deep

  wrapUp:
    MOVE( *aPort, R )
    SWAP
    CDR
    CDR
    CDR
    tprintf2( aPort, MiscStrings[ 362 ] );      //  " -- other syntax binding properties: "
    INSPECT_ON_PORT                                      //  the rest of the syntax-binding list
    DROP
    POP          // Stack cleared
    return TRUE;

  unbound:
    tprintf2( aPort, MiscStrings[ 356 ] );      //  " -- unbound"
    POP
    PUSH         // Stack 2 deep
    JUMP_LABEL( wrapUp )
    }            

/**** displayMultipleValues -- Canonical "display" routine for the indicated data type.  ****/

Boolean displayMultipleValues( taggedAval *aPort )
{
    LOAD_R_TAG( pairTag )    
    PUSH
    LENGTH
    PUSH
    MOVE( one, R )
    NUM_EQ_P
    JUMP_TRUE( justOne )
    POP
    POP
    PUSH
    tprintf2( aPort, MiscStrings[ 359 ] );
    tprintf2( aPort, MiscStrings[ 115 ] );
    tprintf2( aPort, MiscStrings[ 360 ] );
    MOVE( *aPort, R )
    SWAP
    DISPLAY_ON_PORT
    DROP
    tprintf2( aPort, MiscStrings[ 115 ] );
    return TRUE;

  justOne:
    POP
    POP
    CAR
    DISPLAY
    return TRUE;
    }

/**** writeMultipleValues -- Canonical "write" routine for the indicated data type.  ****/

Boolean writeMultipleValues( taggedAval *aPort )
{
    LOAD_R_TAG( pairTag )    
    PUSH
    LENGTH
    PUSH
    MOVE( one, R )
    NUM_EQ_P
    JUMP_TRUE( justOne )
    POP
    POP
    PUSH
    tprintf2( aPort, MiscStrings[ 359 ] );
    tprintf2( aPort, MiscStrings[ 115 ] );
    tprintf2( aPort, MiscStrings[ 360 ] );
    MOVE( *aPort, R )
    SWAP
    WRITE_ON_PORT
    DROP
    tprintf2( aPort, MiscStrings[ 115 ] );
    return TRUE;

  justOne:
    POP
    POP
    CAR
    DISPLAY
    return TRUE;
    }

/**** inspectMultipleValues -- Canonical "display" routine for the indicated data type.  ****/

Boolean inspectMultipleValues( taggedAval *aPort )
{
    printAnyTaggedAval( aPort, &R );
    tprintf2( aPort, MiscStrings[ 115 ] );
    tprintf2( aPort, MiscStrings[ 360 ] );
    PUSH
    LOAD_R_TAG( pairTag )    
    PUSH
    MOVE( *aPort, R )
    SWAP
    DISPLAY_ON_PORT
    DROP
    POP
    tprintf2( aPort, MiscStrings[ 115 ] );
    return TRUE;
    }

/**** displayTrue -- Canonical "display" routine for the indicated data type.  ****/

Boolean displayTrue( taggedAval *aPort )
{
    return (Boolean)tprintf2( aPort, MiscStrings[ 51 ] );
    }

/**** displayFail -- Canonical "display" routine for the indicated data type.  ****/

Boolean displayFail( taggedAval *aPort )
{
    return (Boolean)tprintf2( aPort, MiscStrings[ 390 ] ); 
    }

/**** displaySucceed -- Canonical "display" routine for the indicated data type.  ****/

Boolean displaySucceed( taggedAval *aPort ) 
{
    return (Boolean)tprintf2( aPort, MiscStrings[ 391 ] );
    }

static const LongWord about2100 = 4294967294LL;   // Unix time functions have trouble with larger.  Yes, 4294967294, not 4294967295.

/**** displayForgettableForgotten --  Canonical "display" routine for the indicated data type.  ****/

Boolean displayForgettableForgotten( taggedAval *aPort )
{
    struct tm expirationTimeIn;
    struct tm *expirationTime;
    time_t theTime = 0;

    if( R.u.c < about2100 ) {
        theTime = (time_t)R.u.c;
        expirationTime = gmtime_r( &theTime, &expirationTimeIn );
        (Boolean)tprintf10( aPort, MiscStrings[ 425 ], DataTypeStrings[ 38 ], R.u.c,
                            expirationTime->tm_year + 1900, expirationTime->tm_mon + 1, expirationTime->tm_mday,
                            expirationTime->tm_hour, expirationTime->tm_min, expirationTime->tm_sec );
        }
    else
        tprintf4( aPort, MiscStrings[ 430 ], DataTypeStrings[ 38 ], R.u.c );

    return TRUE;
    }

/**** displayForgettableRemembered --  Canonical "display" routine for the indicated data type.  ****/

Boolean displayForgettableRemembered( taggedAval *aPort )
{
    struct tm expirationTimeIn;
    struct tm *expirationTime;
    time_t theTime = 0;

    PUSH
    MOVE( S[0], R )
    LOAD_R_TAG( vectorTag );
    PUSH
    MOVE( one, R )
    VREF
    
    if( R.u.c < about2100 ) {
        theTime = (time_t)R.u.c;
        expirationTime = gmtime_r( &theTime, &expirationTimeIn );
        (Boolean)tprintf10( aPort, MiscStrings[ 425 ], DataTypeStrings[ 36 ], R.u.c,
                            expirationTime->tm_year + 1900, expirationTime->tm_mon + 1, expirationTime->tm_mday,
                            expirationTime->tm_hour, expirationTime->tm_min, expirationTime->tm_sec );
        }
    else
        tprintf4( aPort, MiscStrings[ 430 ], DataTypeStrings[ 36 ], R.u.c );

    POP

    return TRUE;
    }

/**** displayForgettableUnexpired --  Canonical "display" routine for the indicated data type.  ****/

Boolean displayForgettableUnexpired( taggedAval *aPort )
{
    struct tm expirationTimeIn;
    struct tm *expirationTime;
    time_t theTime = 0;

    PUSH
    MOVE( S[0], R )
    LOAD_R_TAG( vectorTag );
    PUSH
    MOVE( one, R )
    VREF
    
    if( R.u.c < about2100 ) {
        theTime = (time_t)R.u.c;
        expirationTime = gmtime_r( &theTime, &expirationTimeIn );
        (Boolean)tprintf10( aPort, MiscStrings[ 425 ], DataTypeStrings[ 37 ], R.u.c,
                            expirationTime->tm_year + 1900, expirationTime->tm_mon + 1, expirationTime->tm_mday,
                            expirationTime->tm_hour, expirationTime->tm_min, expirationTime->tm_sec );
        }
    else
        tprintf4( aPort, MiscStrings[ 430 ], DataTypeStrings[ 37 ], R.u.c );

    POP

    return TRUE;
    }

/**** inspectFOV -- helper for the next two functions.  ****/

void inspectFOV( taggedAval *aPort )
{
    struct tm expirationTimeIn;
    struct tm *expirationTime;
    time_t theTime = 0;

    PUSH
    inspectAnyTaggedAval( aPort );
    MOVE( S[0], R )
    LOAD_R_TAG( vectorTag );
    PUSH
    MOVE( one, R )
    VREF
    
    if( R.u.c < about2100 ) {
        theTime = (time_t)R.u.c;
        expirationTime = gmtime_r( &theTime, &expirationTimeIn );
        tprintf9( aPort, MiscStrings[ 426 ], R.u.c,
                  expirationTime->tm_year + 1900, expirationTime->tm_mon + 1, expirationTime->tm_mday,
                  expirationTime->tm_hour, expirationTime->tm_min, expirationTime->tm_sec );
        }
    else
        tprintf3( aPort, MiscStrings[ 431 ], R.u.c );

    MOVE( S[0], R )
    LOAD_R_TAG( vectorTag );
    PUSH
    MOVE( two, R )
    VREF
    if( float_p( R.t ) )
        tprintf3( aPort, MiscStrings[ 427 ], R.u.f );
    else
        tprintf3( aPort, MiscStrings[ 429 ], R.u.c );
    tprintf2( aPort, MiscStrings[ 428 ]);
    MOVE( S[0], R )
    LOAD_R_TAG( vectorTag );
    PUSH
    MOVE( zero, R )
    VREF
    PUSH
    MOVE( *aPort, R )
    SWAP
    DISPLAY_ON_PORT
    tprintf2( aPort, MiscStrings[ 115 ]);
    DROP    
    POP    
    }

/**** inspectForgettableRemembered --  Canonical "inspect" routine for the indicated data type.  ****/

Boolean inspectForgettableRemembered( taggedAval *aPort )
{
    inspectFOV( aPort );
    return TRUE;
    }

/**** inspectForgettableUnexpired --  Canonical "inspect" routine for the indicated data type.  ****/

Boolean inspectForgettableUnexpired( taggedAval *aPort )
{
    inspectFOV( aPort );
    return TRUE;
    }


/**** Miscellaneous functions referenced by subsequent statics: ****/

/**** lengthByteBlock -- Return the length of a byte block, which is sizeof( taggedAval )
                        longer than the number of bytes that it can store.                              ****/

memoryBlockSize lengthByteBlock( taggedAval *a )
{
    return( (memoryBlockSize)((a->u).c) + (memoryBlockSize)sizeof( taggedAval ) );
    }

/**** lengthError -- Error routine for a missing "length" function in a data type.  ****/

memoryBlockSize lengthError( taggedAval *a )
{
    useFatalErrorOutput();
    (void)printAnyTaggedAval( &CurrentOutputPort, a );
    tprintf2( &CurrentOutputPort, MiscStrings[ 115 ] );
    fatalError( 3 );
    return (UnsignedLongWord)sizeof(taggedAval);
    }

UnsignedLongWord nothing() { return (UnsignedLongWord)0; }

memoryBlockSize standardLength( taggedAval *a ) { return sizeof( taggedAval ); }

extern Boolean notFlippedYet( taggedAval *aTag, volatile memorySpace *real, volatile memorySpace *scratch );
extern void markFlipped( taggedAval *aTag, volatile memorySpace *real, volatile memorySpace *scratch );

void flipTag( Tag *tP )
{
    *tP = (Tag)FLIP_WORD( *tP );
    }

void flipPointer( LongWord *pP )
{
    *pP = FLIP_WORD( *pP );
    }

/**** flipTaggedAvalOnly  --  for situations when all we need to flip is the tagged-aval part of a datum;
      notably, when the data item is just the tagged aval -- no byte block attached.  ****/

void flipTaggedAvalOnly( taggedAval *a )
{
    flipTag( &(a->t) );
    flipPointer( &((a->u).c) );
    }

memoryBlockSize justFlipTaggedAvalToBigEndian( taggedAval *aTag, void *worldEnd, void *world, void *alternateWorld )
{
    if( notFlippedYet( aTag, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld ) ) {
        markFlipped( aTag, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld );
        flipTaggedAvalOnly( aTag );
        }
    return lengthTaggedAvalPointer( aTag );
    }

memoryBlockSize justFlipTaggedAvalToLittleEndian( taggedAval *aTag, void *worldEnd, void *world, void *alternateWorld )
{
    memoryBlockSize lnth = lengthTaggedAvalPointer( aTag );

    if( notFlippedYet( aTag, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld ) ) {
        markFlipped( aTag, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld );
        lnth = lengthTaggedAvalPointer( aTag );
        flipTaggedAvalOnly( aTag );
        }
    else {
        flipTaggedAvalOnly( aTag );
        lnth = lengthTaggedAvalPointer( aTag );
        flipTaggedAvalOnly( aTag );
        }
    return lnth;
    }

memoryBlockSize noFlipToBigEndian( taggedAval *aTag, void *worldEnd, void *world, void *alternateWorld )
{
    memoryBlockSize lnth = 0;

    if( notFlippedYet( aTag, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld ) ) {
        flipTaggedAvalOnly( aTag );
        lnth = lengthTaggedAvalPointer( aTag );
        flipTaggedAvalOnly( aTag );
        }
    else
        lnth = lengthTaggedAvalPointer( aTag );
    return lnth;
    }

memoryBlockSize noFlipToLittleEndian( taggedAval *aTag, void *worldEnd, void *world, void *alternateWorld )
{
    memoryBlockSize lnth = 0;

    if( notFlippedYet( aTag, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld ) )
        lnth = lengthTaggedAvalPointer( aTag );
    else {
        flipTaggedAvalOnly( aTag );
        lnth = lengthTaggedAvalPointer( aTag );
        flipTaggedAvalOnly( aTag );
        }
    return lnth;
    }

/**** Code for flipping data elements between little-endian and big-endian.  World files are little-endian:
      When the program is running a big-endian machine, their file format must be converted before write, and
      after read.  ****/

void flipLongWord( LongWord *l )
{
    Byte *bP1 = 0;

    bP1 = (Byte *)l;

    Byte *bP2 = bP1 + sizeof( Word );

    Word *w1 = (Word *)bP1;
    Word *w2 = (Word *)bP2;

    *w1 = FLIP_WORD( *w1 );
    if( sizeof( LongWord ) == 8 )
        *w2 = FLIP_WORD( *w2 );
    }

void flipUnsignedLongWord( UnsignedLongWord *l )
{
    Byte *bP1 = 0;

    bP1 = (Byte *)l;

    Byte *bP2 = bP1 + sizeof( UnsignedWord );

    UnsignedWord *w1 = (UnsignedWord *)bP1;
    UnsignedWord *w2 = (UnsignedWord *)bP2;

    *w1 = FLIP_WORD( *w1 );
    if( sizeof( UnsignedLongWord ) == 8 )
        *w2 = FLIP_WORD( *w2 );
    }

/**** flipDataItemToBigEndian -- generic; uses tag information to decide what to do, really. ****/

memoryBlockSize flipDataItemToBigEndian( taggedAval *a, void *worldEnd, void *world, void *alternateWorld )
{
    memoryBlockSize lnth = 0;
    dataDescriptor *d = 0;
        
    if( notFlippedYet( a, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld ) ) {
        flipTaggedAvalOnly( a );    // Item is little-endian, we are running big-endian, so flip to read.
        d = dataTypeTable[ pure_type_code( a->t ) ];
        flipTaggedAvalOnly( a );    // Flip back; it is up to the routine called to decde what to do for real.
        if( d )
            lnth = d->flipToBigEndian( a, worldEnd, world, alternateWorld );
        else 
            lnth = sizeof( taggedAval );
       }
    else 
        lnth = lengthTaggedAvalPointer( a );
    return lnth;
    }

/**** flipDataItemToLittleEndian -- generic; uses tag information to decide what to do, really. ****/

memoryBlockSize flipDataItemToLittleEndian( taggedAval *a, void *worldEnd, void *world, void *alternateWorld )
{
    memoryBlockSize lnth = 0;
    dataDescriptor *d = 0;

    if( notFlippedYet( a, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld ) ) {
        d = dataTypeTable[ pure_type_code( a->t ) ];
        if( d )
            lnth = d->flipToLittleEndian( a, worldEnd, world, alternateWorld );
        else 
            lnth = sizeof( taggedAval );
        }
    else {
        flipTaggedAvalOnly( a );    // Item is little-endian, we are running big-endian, so flip to read.
        lnth = lengthTaggedAvalPointer( a );
        flipTaggedAvalOnly( a );    // Flip back; it is up to the routine called to decde what to do for real.
        }
    return lnth;
    }

/**** flipShortFloatToBigEndian ****/

memoryBlockSize flipShortFloatToBigEndian( taggedAval *aTag, void *worldEnd, void *world, void *alternateWorld )
{
    taggedAval *tag2 = 0;

    LongWord cursor = 0;
    LongWord *wP0 = 0;
    LongWord *wP1 = 0;
    LongWord w0 = 0;
    LongWord w1 = 0;

    if( notFlippedYet( aTag, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld ) ) {
        markFlipped( aTag, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld );
        flipTaggedAvalOnly( aTag );
        cursor = (*aTag).u.c;
        cursor = (LongWord)worldEnd - cursor;
        tag2 = (taggedAval *)cursor;

        if( notFlippedYet( tag2, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld ) ) {

            markFlipped( tag2, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld );
            flipTaggedAvalOnly( tag2 );
 
            if( sizeof( ShortFloat ) == 8 ) {

                cursor = cursor + sizeof( taggedAval );
                wP0    = (LongWord *)cursor;

                cursor = cursor + sizeof( UnsignedLongWord );
                wP1 = (LongWord *)cursor;

                w0 = *wP0;
                w0 = FLIP_WORD( w0 );

                w1 = *wP1;
                w1 = FLIP_WORD( w1 );
    
                *wP0 = w1;
                *wP1 = w0;
                }
            else 
                fatalError( 90 );
            }
        }
    return sizeof( taggedAval );
    }

/**** flipShortFloatToLittleEndian ****/

memoryBlockSize flipShortFloatToLittleEndian( taggedAval *aTag, void *worldEnd, void *world, void *alternateWorld )
{
    taggedAval *tag2 = 0;

    LongWord cursor = 0;
    LongWord *wP0 = 0;
    LongWord *wP1 = 0;
    LongWord w0 = 0;
    LongWord w1 = 0;

    if( notFlippedYet( aTag, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld ) ) {
        markFlipped( aTag, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld );
        cursor = (*aTag).u.c;
        cursor = (LongWord)worldEnd - cursor;
        tag2 = (taggedAval *)cursor;

        if( notFlippedYet( tag2, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld ) ) {
 
            markFlipped( tag2, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld );

            if( sizeof( ShortFloat ) == 8 ) {
 
	        cursor = cursor + sizeof( taggedAval );
	        wP0    = (LongWord *)cursor;

	        cursor = cursor + sizeof( UnsignedWord );
	        wP1 = (LongWord *)cursor;

	        w0 = *wP0;
	        w0 = FLIP_WORD( w0 );

	        w1 = *wP1;
	        w1 = FLIP_WORD( w1 );

	        *wP0 = w1;
	        *wP1 = w0;
                }
	    else 
	        fatalError( 90 );
            flipTaggedAvalOnly( tag2 );
            }
         flipTaggedAvalOnly( aTag );
        }
    return sizeof( taggedAval );
    }

/**** flipPeculiarNonLeafToBigEndian -- Canonical flip routine for the indicated data type.  ****/

memoryBlockSize flipPeculiarNonLeafToBigEndian( taggedAval *a, void *worldEnd, void *world, void *alternateWorld )
{
    UnsignedLongWord n;

    if( notFlippedYet( a, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld ) ) {
        markFlipped( a, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld );
        flipTaggedAvalOnly( a );
        n = getPeculiarNonLeafNumber( a );
        switch( n ) {
            default:
                break;
            }
        }
    return sizeof( taggedAval );
    }

/**** flipPeculiarNonLeafToLittleEndian -- Canonical flip routine for the indicated data type.  ****/

memoryBlockSize flipPeculiarNonLeafToLittleEndian( taggedAval *a, void *worldEnd, void *world, void *alternateWorld )
{
    UnsignedLongWord n;
    LongWord cursor = (*a).u.c;

    if( notFlippedYet( a, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld ) ) {
        markFlipped( a, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld );
        n = getPeculiarNonLeafNumber( (taggedAval *)cursor );
	flipTaggedAvalOnly( a );
        switch( n ) {
	    default:
		break;
	    }
        }
    return sizeof( taggedAval );
    }

/**** flipStringToBigEndian -- Canonical flip routine for the indicated data type.  ****/

memoryBlockSize flipStringToBigEndian( taggedAval *a, void *worldEnd, void *world, void *alternateWorld )
{
    LongWord cursor = 0;
    taggedAval *tag2 = 0;

    if( notFlippedYet( a, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld ) ) {
        markFlipped( a, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld );
        flipTaggedAvalOnly( a );
        cursor = (*a).u.c;
        cursor = (LongWord)worldEnd - cursor;
        tag2 = (taggedAval *)cursor;
        if( notFlippedYet( tag2, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld ) ) {
            markFlipped( tag2, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld );
            flipTaggedAvalOnly( tag2 );
            }
        }
    return sizeof( taggedAval );
    }

/**** flipStringToLittleEndian -- Canonical flip routine for the indicated data type.  ****/

memoryBlockSize flipStringToLittleEndian( taggedAval *a, void *worldEnd, void *world, void *alternateWorld )
{
    LongWord cursor = (*a).u.c;
    cursor = (LongWord)worldEnd - cursor;
    taggedAval *tag2 = (taggedAval *)cursor;

    if( notFlippedYet( a, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld ) ) {
        markFlipped( a, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld );
	flipTaggedAvalOnly( a );
        if( notFlippedYet( tag2, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld ) ) {
            markFlipped( tag2, (volatile memorySpace *)world, (volatile memorySpace *)alternateWorld );
            flipTaggedAvalOnly( tag2 );
	        }
        }
    return sizeof( taggedAval );
    }

Boolean displayMMappedBlock( taggedAval *aPort )
{
    Byte *theByteBlock = (Byte *)(R.u.a);
    mmappedBlockDescriptor *theBlock = (mmappedBlockDescriptor *)(theByteBlock+ sizeof( taggedAval ));
    Text *theName          = (Text *)((Byte *)theBlock + sizeof( mmappedBlockDescriptor ));

    tprintf8( aPort, MiscStrings[ 433 ], DataTypeStrings[ 39 ], theBlock->size, theName,
              theBlock->mmapWasSuccessful[ theKittenNumber ] ? "" : MiscStrings[ 477 ],
              theKittenNumber,
              theBlock->regionHasBeenUnmapped[ theKittenNumber ] ? MiscStrings[ 478 ] : "");
    return TRUE;
    }
            
Boolean inspectMMappedBlock( taggedAval *aPort )
{
    Byte *theByteBlock = (Byte *)(R.u.a);
    mmappedBlockDescriptor *theBlock = (mmappedBlockDescriptor *)(theByteBlock+ sizeof( taggedAval ));
    Text *theName          = (Text *)((Byte *)theBlock + sizeof( mmappedBlockDescriptor ));
    UnsignedLongWord kitten = 0;

    tprintf8( aPort, MiscStrings[ 434 ], DataTypeStrings[ 39 ], theBlock->size, 
              theBlock->absoluteAddress, theBlock->prot, theBlock->flags, theBlock->offset );
    tprintf4( aPort, MiscStrings[ 435 ], theName, theBlock->fileHasBeenWritten ? "" : MiscStrings[ 479 ]);
    for( kitten = 0; kitten< theNumberOfKittens; ++kitten )
        tprintf5( aPort, MiscStrings[ 436 ], kitten,
                 theBlock->mmapWasSuccessful[ kitten ] ? MiscStrings[ 480 ] : MiscStrings[ 481 ],
                 theBlock->regionHasBeenUnmapped[ kitten ] ? MiscStrings[ 478 ] : "");
    return TRUE;
    }
            
/******** Statics go here: ********/

/**** Particular data descriptors: ****/

/**** Unknown data descriptor -- "p" field indeterminate ****/

dataDescriptor unknownDataDescriptor =
  { 1, "", 0, 0,
    lengthError, NULL, inspectAnyTaggedAval, inspectAnyTaggedAval, inspectAnyTaggedAval,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** byteBlock -- "p" field is the (UnsignedLongWord) length of a reserved
                   block of contiguous storage WHICH IMMEDIATELY FOLLOWS                                 ****/

static dataDescriptor byteBlock = 
  { 2, "", byteBlockTag, 0,
    lengthByteBlock, byteBlockP, displayByteBlock, displayByteBlock, inspectAnyTaggedAval,
    noFlipToLittleEndian, noFlipToBigEndian };

/**** exactShortFloat -- "p" field is the literal value of a ShortFloat ****/
/*
static dataDescriptor exactShortFloat = 
  { 3, "", exactShortFloat, sizeof( taggedAval ),
    standardLength, exactShortFloatP, displayAnyNumber, writeAnyNumber, inspectShortFloat,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };
*/
/**** exactShortFixnum -- "p" field is the literal value of a signed LongWord ****/

dataDescriptor exactShortFixnum = 
  { 4, "", exactShortFixnumBitsMask, sizeof( taggedAval ),
    standardLength, exactShortFixnumP, displayAnyNumber, writeAnyNumber, inspectAnyTaggedAval,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** falseBoolean -- "p" field is a don't care -- tag alone suffices to define this taggedAval ****/

static dataDescriptor falseBoolean =
  { 5, "", falseBooleanTag, sizeof( taggedAval ),
    standardLength, falseP, displayFalse, displayFalse, inspectAnyTaggedAval,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** forwardingPointer -- "p" field is a pointer to another taggedAval ****/

dataDescriptor forwardingPointer = 
  { 6, "", forwardingPointerTag, sizeof( taggedAval ),
    standardLength, forwardingPointerP, displayForwardingPointer, displayForwardingPointer, inspectAnyTaggedAval,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** pair -- "p" field is a pointer to the car of a pair ****/

dataDescriptor pair = 
  { 7, "", pairTag, sizeof( taggedAval ),
    standardLength, pairP, displayPair, writePair, inspectPair,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** inexactShortFloat -- "p" field is the literal value of a ShortFloat ****/
/*
static dataDescriptor inexactShortFloat = 
  { 8, "", inexactShortFloat, sizeof( taggedAval ),
    standardLength, inexactShortFloatP, displayAnyNumber, writeAnyNumber, inspectShortFloat,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };
*/
/**** inexactShortFixnum -- "p" field is the literal value of a signed LongWord ****/

static dataDescriptor inexactShortFixnum = 
  { 9, "", inexactShortFixnumBitsMask, sizeof( taggedAval ),
    standardLength, inexactShortFixnumP,  displayAnyNumber, writeAnyNumber, inspectAnyTaggedAval,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** string -- "p" field is a pointer to a byteBlock containing the (null-terminated) string ****/

dataDescriptor string =
  { 10, "", stringTag, sizeof( taggedAval ),
    standardLength, stringP, displayString, writeString, inspectAnyTaggedAval,
    flipStringToLittleEndian, flipStringToBigEndian };

/**** trueBoolean -- "p" field is a don't care -- tag alone suffices to define this taggedAval ****/

static dataDescriptor trueBoolean =
  { 11, "", trueBooleanTag, sizeof( taggedAval ),
    standardLength, trueP, displayTrue, displayTrue, inspectAnyTaggedAval ,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** character -- "p" field is obtained by assigning the character to an UnsignedLongWord ****/

dataDescriptor character =
  { 12, "", characterTag, sizeof( taggedAval ),
    standardLength, characterP, displayCharacter, writeCharacter, inspectAnyTaggedAval ,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** symbol -- "p" field is a pointer to (pname hashValue . permanentP) ****/

dataDescriptor symbol =
  { 13, "", symbolTag, sizeof( taggedAval ),
    standardLength, symbolP, displaySymbol, displaySymbol, inspectSymbol,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** schemeMachinePC -- "p" field is a pointer to the start of some machine code ****/

dataDescriptor schemeMachinePC =
  { 14, "", schemeMachinePCTag, sizeof( taggedAval ),
    standardLength, schemeMachinePCP, displaySchemeMachinePC, displaySchemeMachinePC, inspectAnyTaggedAval,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** lambdaPair -- "p" field is a pointer to a pair:  If the pair is an interpreted lambda,
                    then its car is the lambda list of the lambda expression (strictly, a
                    list of the corresponding symbols), its cadr is the body of the code,
                    its caddr is the environment in which the lambda was defined, and its
                    cdddr is a slot which may contain the symbol to which the lambda was
                    most recently bound, if any.  (This slot is for providing debugging
                    information -- a meaningful stack trace -- on error.)                       ****/

dataDescriptor lambdaPair =
  { 15, "", lambdaPairTag, sizeof( taggedAval ),
     standardLength, lambdaPairP, displayLambdaPair, displayLambdaPair, inspectLambdaPair,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** builtInFunction -- "p" field is a pointer to a functionDescriptor ****/

dataDescriptor builtInFunction =
  { 16, "", builtInFunctionTag, sizeof( taggedAval ),
     standardLength, builtInFunctionP, displayBuiltInFunction, displayBuiltInFunction, inspectAnyTaggedAval ,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** specialForm -- "p" field is a pointer to a specialFormDescriptor ****/

dataDescriptor specialForm =
  { 17, "", specialFormTag, sizeof( taggedAval ),
    standardLength, specialFormP, displaySpecialForm, displaySpecialForm, inspectAnyTaggedAval ,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** vector -- "p" field is a pointer to a cdr-coded list: (length item0 item1 ...) ****/

dataDescriptor vector =
  { 18, "", vectorTag, sizeof( taggedAval ),
    standardLength, vectorP, displayVector, writeVector, inspectVector,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** promise -- "p" field is a pointer to a list:  if the cdr of the list is #t, then
                 the promise has been forced, and the car of the list is the lambda expression
                 whose evaluation will return the desired value; if the cdr of the list is
                 (), then the promise has been forced, and the car of the list the value.
                 that was returned by the force.                                                            ****/

dataDescriptor promise =
  { 19, "", promiseTag, sizeof( taggedAval ),
    standardLength, promiseP, displayPromise, displayPromise, inspectPromise,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** peculiarLeaf -- "p" field is a LongWord that tells what kind of peculiar leaf it is.                  ****/

dataDescriptor peculiarLeaf =
  { 20, "", peculiarLeafTag, sizeof( taggedAval ),
    standardLength, peculiarLeafP, displayPeculiarLeaf, displayPeculiarLeaf, inspectPeculiarLeaf,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** peculiarNonLeaf -- "p" field points to a byte block whose first four bytes (past
                         the length) are a LongWord that tells what kind of peculiar non-leaf it is
                         (lsb) and what kitten it is associated with (next-to-lsb).        ****/

dataDescriptor peculiarNonLeaf =
  { 21, "", peculiarNonLeafTag, sizeof( taggedAval ),
    standardLength, peculiarNonLeafP, displayPeculiarNonLeaf, displayPeculiarNonLeaf, inspectPeculiarNonLeaf,
    flipPeculiarNonLeafToLittleEndian, flipPeculiarNonLeafToBigEndian };

/**** symbolBinding -- "p" field points to a list:

  ( symbol boundP binding-if-any ... )

  The first three elements are respectively the symbol in question, a boolean that is true if and only
  if it has a value, and the value.  The rest are vague, because this data type and its use are still
  in development.

****/

dataDescriptor syntaxBinding =
  { 22, "", syntaxBindingTag, sizeof( taggedAval ),
    standardLength, syntaxBindingP, displaySyntaxBinding, writeSyntaxBinding, inspectSyntaxBinding,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** macro -- "p" field points to code to create that which gets evaluated.  ****/

dataDescriptor macro =
  { 23, "", macroTag, sizeof( taggedAval ),
    standardLength, macroP, displayMacro, displayMacro, inspectMacro,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** stackReference -- "p" field is a cursor for indirect referencing off the stack top.  ****/

dataDescriptor stackReference =
  { 24, "", stackReferenceTag, sizeof( taggedAval ),
    standardLength, stackReferenceP, displayStackReference, displayStackReference, inspectStackReference,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** exactShortFloat -- "p" field is pointer to a byte block containing the literal value of a ShortFloat ****/

static dataDescriptor exactShortFloat = 
  { 25, "", exactShortFloatBitsMask, sizeof( taggedAval ),
    standardLength, exactShortFloatP, displayAnyNumber, writeAnyNumber, inspectShortFloat,
    flipShortFloatToLittleEndian, flipShortFloatToBigEndian };

/**** InexactShortFloat -- "p" field is pointer to a byte block containing the literal value of a ShortFloat ****/

static dataDescriptor inexactShortFloat = 
  { 26, "", inexactShortFloatBitsMask, sizeof( taggedAval ),
    standardLength, inexactShortFloatP, displayAnyNumber, writeAnyNumber, inspectShortFloat,
    flipShortFloatToLittleEndian, flipShortFloatToBigEndian };

/**** environmentReference -- "p" field:  High half-word -- which environment;
                              low half-word  -- which variable therein.  ****/

static dataDescriptor environmentReference =
  { 27, "", environmentReferenceTag, sizeof( taggedAval ),
    standardLength, environmentReferenceP, displayEnvironmentReference, displayEnvironmentReference, inspectEnvironmentReference,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** topLevelEnvironmentReference -- "p" field:  High half-word -- which bucket (hash value);
                                      low half-word  -- which variable therein.  ****/

static dataDescriptor topLevelEnvironmentReference =
  { 28, "", topLevelEnvironmentReferenceTag, sizeof( taggedAval ),
    standardLength, topLevelEnvironmentReferenceP, displayTopLevelEnvironmentReference,
    displayTopLevelEnvironmentReference, inspectTopLevelEnvironmentReference,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** emptyList -- "p" field is a don't care -- tag alone suffices to define this taggedAval ****/

static dataDescriptor emptyList =
  { 29, "", emptyListTag, sizeof( taggedAval ),
    standardLength, emptyListP, displayEmptyList, displayEmptyList, inspectAnyTaggedAval,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** multipleValues -- "p" field is a pointer to a list of values returned by "values" ****/

dataDescriptor multipleValues = 
  { 30, "", multipleValuesTag, sizeof( taggedAval ),
    standardLength, multipleValuesP, displayMultipleValues, writeMultipleValues, inspectMultipleValues,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** pointerToByteBlock -- "p" field is a pointer to a byte block" ****/

dataDescriptor pointerToByteBlock = 
  { 31, "", pointerToByteBlockTag, sizeof( taggedAval ),
    standardLength, pointerToByteBlockP, displayPointerToByteBlock, displayPointerToByteBlock, displayPointerToByteBlock,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** fail -- "p" field is a don't care -- tag alone suffices to define this taggedAval ****/

static dataDescriptor failLogic =
  { 32, "", failTag, sizeof( taggedAval ),
    standardLength, failP, displayFail, displayFail, inspectAnyTaggedAval ,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** succeed -- "p" field is a don't care -- tag alone suffices to define this taggedAval ****/

static dataDescriptor succeedLogic =
  { 33, "", succeedTag, sizeof( taggedAval ),
    standardLength, succeedP, displaySucceed, displaySucceed, inspectAnyTaggedAval ,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

dataDescriptor schemeMachinePCIndex =
  { 34, "", schemeMachinePCIndexTag, sizeof( taggedAval ),
    standardLength, schemeMachinePCIndexP, displaySchemeMachinePCIndex, displaySchemeMachinePCIndex, inspectAnyTaggedAval,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

dataDescriptor Port =
  { 35, "", portTag, sizeof( taggedAval ),
    standardLength, portP, displayPort, displayPort, inspectPort,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** forgettableRemembered -- "p" field is a pointer to a vector.  ****/

dataDescriptor forgettableRemembered =
  { 36, "", forgettableRememberedTag, sizeof( taggedAval ),
    standardLength, forgettableRememberedP, displayForgettableRemembered, displayForgettableRemembered, inspectForgettableRemembered,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** forgettableUnexpired  -- "p" field is a pointer to a vector.  ****/

dataDescriptor forgettableUnexpired = 
  { 37, "", forgettableUnexpiredTag, sizeof( taggedAval ),
    standardLength, forgettableUnexpiredP, displayForgettableUnexpired, displayForgettableUnexpired, inspectForgettableUnexpired,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** forgettableForgotten  ****/

dataDescriptor forgettableForgotten =
  { 38, "", forgettableForgottenTag, sizeof( taggedAval ),
    standardLength, forgettableForgottenP, displayForgettableForgotten, displayForgettableForgotten, inspectAnyTaggedAval,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** mmappedBlock  -- "p" field is a pointer to a byte block
                       NOTE:  Flipping code is NOT set up, since I have no 64-bit big-endian machine to test it on.  ****/

dataDescriptor mmappedBlock =
  { 39, "", mmappedBlockTag, sizeof( taggedAval ),
    standardLength, mmappedBlockP, displayMMappedBlock, displayMMappedBlock, inspectMMappedBlock,
    noFlipToLittleEndian, noFlipToBigEndian };

/**** ggcForwardingPointer -- "p" field is a pointer to another taggedAval ****/

dataDescriptor ggcForwardingPointer = 
  { 40, "", ggcForwardingPointerTag, sizeof( taggedAval ),
    standardLength, ggcForwardingPointerP, displayGGCForwardingPointer, displayGGCForwardingPointer, inspectAnyTaggedAval,
    justFlipTaggedAvalToLittleEndian, justFlipTaggedAvalToBigEndian };

/**** The dataTypeList -- which an initialization routine uses to construct
      a more rapidly accessed structure for actual use at run-time                      ****/

simpleListCell dataTypeList[] = {
/*  1 */    { 1, &byteBlock },
/*  2 */    { 1, &exactShortFloat },
/*  3 */    { 1, &exactShortFixnum },
/*  4 */    { 1, &falseBoolean },
/*  5 */    { 1, &forwardingPointer },
/*  6 */    { 1, &inexactShortFloat },
/*  7 */    { 1, &inexactShortFixnum },
/*  8 */    { 1, &pair },
/*  9 */    { 1, &string },
/* 10 */    { 1, &trueBoolean },
/* 11 */    { 1, &character },
/* 12 */    { 1, &symbol },
/* 13 */    { 1, &schemeMachinePC },
/* 14 */    { 1, &lambdaPair },
/* 15 */    { 1, &builtInFunction },
/* 16 */    { 1, &specialForm },
/* 17 */    { 1, &vector },
/* 18 */    { 1, &promise },
/* 19 */    { 1, &peculiarLeaf },
/* 20 */    { 1, &peculiarNonLeaf },
/* 21 */    { 1, &syntaxBinding },
/* 22 */    { 1, &macro },
/* 23 */    { 1, &stackReference },
/* 24 */    { 1, &environmentReference },
/* 25 */    { 1, &topLevelEnvironmentReference },
/* 26 */    { 1, &emptyList },
/* 27 */    { 1, &multipleValues },
/* 28 */    { 1, &pointerToByteBlock },
/* 29 */    { 1, &failLogic },
/* 20 */    { 1, &succeedLogic },
/* 31 */    { 1, &schemeMachinePCIndex },
/* 32 */    { 1, &Port },
/* 33 */    { 1, &forgettableRemembered },
/* 34 */    { 1, &forgettableUnexpired },
/* 35 */    { 1, &forgettableForgotten },
/* 36 */    { 1, &mmappedBlock },
/* 37 */    { 1, &ggcForwardingPointer },
            { 0, (void *)NULL }
    };

/**** initializeDataTypeList -- Install the print names.  ****/

void initializeDataTypeList()
{
    simpleList d;

    d = dataTypeList;
    while( d->flag ) {
        installDataTypePName( (dataDescriptor *)(d->pointer) );
        ++d;
        }
    }

/**** buildDataTypeTable -- create the table, fill it out from the list, and return it ****/

dataDescriptor **buildDataTypeTable( simpleListCell *dtl )
{
    dataDescriptor **dtt;
    UnsignedLongWord i;

    dtt = (dataDescriptor **)malloc( maxTag * sizeof(dataDescriptor **) );
    if( !dtt )
        fatalError( 4 );
    else {
        for( i = 0; i < maxTag; ++i )
            dtt[i] = &unknownDataDescriptor;
        while( dtl->flag ) {
            dtt[ ((dataDescriptor *)(dtl->pointer))->tag ] = (dataDescriptor *)(dtl->pointer);
            ++dtl;
            }
        }
    return dtt;
    }

/**** describeDataTypeTable -- Describe each known data descriptor in the given table.  ****/

void describeDataTypeTable( dataDescriptor **t )
{
    UnsignedLongWord i;
        
    tprintf2( &CurrentOutputPort, MiscStrings[ 64 ] );
    for( i = 0; i < maxTag; ++i ) {
        if( t[i] != &unknownDataDescriptor )
            describeDataDescriptor( t[i] );
        }
    }
        
/**** initializeDataTypeTable -- Load one global from another.  ****/

void initializeDataTypeTable( Boolean describeListP, Boolean describeTableP )
{
    dataTypeTable = buildDataTypeTable( dataTypeList );
    if( describeListP )
        describeDataTypeList( dataTypeList );
    if( describeTableP )
        describeDataTypeTable( dataTypeTable);
    }

Boolean validDataType( Tag t ) { return( dataTypeTable[ pure_type_code( t ) ] != &unknownDataDescriptor ); }

