/****************************************************************************************

Events.c++ -- Copyright (c) by Jay Reynolds Freeman 2006

Eventloop and state-transition stuff for Wraith Scheme.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import "WraithHeader.h"
#import "CommandInterface.e"
#import "Registers.h"
#import "SECD.h"
#import "FunctionAndSpecialNames.e"
#import "Globals.e"
#import "IO.e"
#import "Intern.e"
#import "Registers.e"
#import "SECDMachine.e"

// Boolean         CompileDefinesFlag = false;
Boolean         CompileDefinesFlag = true;      // TESTING ...
Boolean         GCInProgressFlag = false;
Boolean         EmptyListIsFalseFlag = false;
Boolean         ResetFlag = false;
Boolean         resizeRequired = FALSE;

Boolean         Private_CommandRequestedFlag = false;
Boolean         Private_ResizeRequestedFlag = false;

/**** forward declarations: ****/

void reportEvent( int foo );

/**** dealWithSIGUSR1 ****/

void dealWithSIGUSR1() {

    UnsignedLongWord interruptCounter = 0;

    SPushR();
    for( interruptCounter = 0; interruptCounter < NUMBER_OF_INTERRUPTS; ++interruptCounter ) {
        theClowder->lockInterrupt( interruptCounter );
        if( theClowder->getInterruptRequested( interruptCounter ) ) {
            R = IVR;
            vectorRefN( interruptCounter );
            if( ! empty_list_p( R.t ) ) {
                SPushR();                     // R: kitten/handler; Stack: kitten/handler, saved R.
                car();                        // R: kitten;         Stack: kitten/handler, saved R.
                SPushR();                     // R: kitten;         Stack: kitten, kitten/handler, saved R.
                R = S[1];                     // R: kitten/handler; Stack: kitten, kitten/handler, saved R.
                cdr();                        // R: handler;        Stack: kitten, kitten/handler, saved R.
                tellKittenCode();             // After call --      Stack: kitten/handler, saved R.
                SDrop();                      //                    Stack: saved R.
                }
            theClowder->clearInterruptRequested( interruptCounter );
            }
        theClowder->unlockInterrupt( interruptCounter );
        }
    SPopR();
    }

/**** dealWithSIGALRM ****/

extern void tellKittenCode();

void dealWithSIGALRM() {

    SPushR();
    R.t = exactShortFixnumBitsMask;
    R.u.a = (taggedAval *)NameStrings[ 434 ];
    intern();
    SPushR();
    boundSymbolP();
    SPopR();
    if( B ) {                 // e::alarm is bound...
        lookUp();
        lambdaPairPMC();
        if( B ) {             // ... to a lambda
            nilCons();
            SPushR();
            R.t = exactShortFixnumBitsMask;
            R.u.c = theKittenNumber;
            Swap();
            tellKittenCode();
            }
        }
    SPopR();
    isAlarmSet = FALSE;
    }

/**** reportControlC ****/

void reportControlC( int ignore ) { reportEvent( CONTROL_C_EVENT ); }

/**** reportSIGUSR1 ****/

void reportSIGUSR1( int ignore ) { reportEvent( SIGUSR1_EVENT ); }

/**** reportSIGALRM ****/

void reportSIGALRM( int ignore ) { reportEvent( SIGALRM_EVENT ); }

/**** reportWindowChanged ****/

void reportWindowChanged( int ignore ) { reportEvent( WINDOW_CHANGED_EVENT ); }

/**** ignore ****/

void ignore() { ; }

/**** errorExit ****/

void errorExit( int ignore ) { exit( 1 ); }

/**** initializeSignalHandlers ****/

void initializeSignalHandlers()
{ 
    signal( SIGINT, reportControlC );
    signal( SIGALRM, reportSIGALRM );
//    signal( SIGWINCH, reportWindowChanged );  // Commented out for development, 2006.11.22
    }

/**** reinitializeSignalHandlers ****/

void reinitializeSignalHandlers()
{ 
    signal( SIGINT, errorExit );
    signal( SIGALRM, reportSIGALRM );
    //    signal( SIGWINCH, ignore );  // Commented out for development, 2006.11.22
    }

/**** initializeEvents ****/

void initializeEvents() { ; }     // Nothing to do.

/**** reinitializeEvents ****/

void reinitializeEvents() { ; }     // Nothing to do.

/**** reportEvent ****/

void reportEvent( int foo )
{
    switch( foo ) {

        case CONTROL_C_EVENT:       Private_CommandRequestedFlag = true; break;            

//        case WINDOW_CHANGED_EVENT:  Private_ResizeRequestedFlag  = true; break;

        case GC_START_EVENT:        break;

        case GC_START_EVENT_1:      emitStatusMessage( MiscStrings[ 299 ] );
                                    break;

        case GC_START_EVENT_2:      emitStatusMessage( MiscStrings[ 452 ] );
                                    break;

        case GC_START_EVENT_3:      GCInProgressFlag = true;
                                    break;

        case GC_END_EVENT:          break;

        case GC_END_EVENT_1:        break;     // Nothing to do.

        case GC_END_EVENT_2:        emitStatusMessage( MiscStrings[ 300 ] );
                                    break;

        case GC_END_EVENT_3:        emitStatusMessage( MiscStrings[ 453 ] );
                                    break;

        case GC_END_EVENT_4:                               break;     // Nothing to do.

        case GC_END_EVENT_5:        GCInProgressFlag = false;
                                    theClowder->clearGCBytesValid();
                                    theClowder->clearGCIsRequired();
                                    break;

        case GC_END_EVENT_6:        GCInProgressFlag = false;
                                    theClowder->clearGCBytesValid();
                                    theClowder->clearGCIsRequired();
                                    break;

        case INITIALIZATION_FINISHED_EVENT:                break;     // Nothing to do.
        case COMPILE_DEFINES_EVENT:                        break;     // Nothing to do.
        case DO_NOT_COMPILE_DEFINES_EVENT:                 break;     // Nothing to do.
        case USE_LOW_PRECISION_EVENT:                      break;     // Nothing to do.
        case USE_HIGH_PRECISION_EVENT:                     break;     // Nothing to do.
        case START_EVALUATION_EVENT:                       break;     // Nothing to do.
        case END_EVALUATION_EVENT:                         break;     // Nothing to do.
        case REQUEST_RESET_EVENT:                          break;     // Nothing to do.
        case REQUEST_RESET_EVENT_2:                        break;     // Nothing to do.
        case REQUEST_RESET_EVENT_3:                        break;     // Nothing to do.
        case REQUEST_RESET_EVENT_4:                        break;     // Nothing to do.
        case CURRENT_MEMORY_IN_USE_EVENT:                  break;     // Nothing to do.
        case CURRENT_GENERATIONAL_MEMORY_IN_USE_EVENT:     break;     // Nothing to do.
        case TOTAL_MEMORY_IN_USE_EVENT:                    break;     // Nothing to do.
        case FILE_LOAD_BEGINNING_EVENT:                    break;     // Nothing to do.
        case FILE_LOAD_SUCCEEDED_EVENT:                    break;     // Nothing to do.
        case FILE_LOAD_FAILED_EVENT:                       break;     // Nothing to do.
        case WORLD_LOAD_SUCCEEDED_EVENT:                   break;     // Nothing to do.
        case WORLD_LOAD_FAILED_EVENT:                      break;     // Nothing to do.
        case WORLD_SAVE_SUCCEEDED_EVENT:                   break;     // Nothing to do.
        case PARSER_STATUS_EVENT:                          break;     // Nothing to do.
        case LIST_PRINT_LENGTH_EVENT:                      break;     // Nothing to do.
        case LIST_PRINT_DEPTH_EVENT:                       break;     // Nothing to do.
        case VECTOR_PRINT_LENGTH_EVENT:                    break;     // Nothing to do.
        case VECTOR_PRINT_DEPTH_EVENT:                     break;     // Nothing to do.
        case WAKED_UP_EVENT:                               break;     // Nothing to do.
        case MEMORY_SIZE_EVENT:                            break;     // Nothing to do.
        case GENERATIONAL_MEMORY_SIZE_EVENT:               break;     // Nothing to do.
        case EXPECT_DOUBLE_QUOTE_EVENT:                    break;     // Nothing to do.
        case EXPECT_RPARENS_EVENT:                         break;     // Nothing to do.
        case MAKE_FRONT_EVENT:                             break;     // Nothing to do.
        case INIT_PROGRESS_EVENT:                          break;     // Nothing to do.
        case INCREMENT_PROGRESS_EVENT:                     break;     // Nothing to do.
        case WRAPUP_PROGRESS_EVENT:                        break;     // Nothing to do.
        case REINITIALIZATION_COMPLETE:                    break;     // Nothing to do.
        case SIGUSR1_EVENT:                                
            if( (! GCInProgressFlag) && (! theClowder->GCIsRequired()) ) {
                dealWithSIGUSR1();
                }
            break; 
        case SIGALRM_EVENT:
            if( (! GCInProgressFlag) && (! theClowder->GCIsRequired()) ) {
                dealWithSIGALRM();
                }
            break; 
        default:
            useFatalErrorOutput();
            tprintf3( &ErrorPort, "%d\n", foo );
            fatalError( 50 );
            break;
        }
    }

/**** eventLoopOnce ****/

void eventLoopOnce( Word sleepTimeInMicroseconds )
{
    if( Private_CommandRequestedFlag ) {
        Private_CommandRequestedFlag = false;
        doCommand();
        }
//    if( Private_ResizeRequestedFlag ) {       // Commented out for development, 2006.11.22
//        Private_ResizeRequestedFlag = false;  // Commented out for development, 2006.11.22
//        dealWithResize();                     // Commented out for development, 2006.11.22
//        }                                     // Commented out for development, 2006.11.22
    usleep( sleepTimeInMicroseconds );
    }

