/****************************************************************************************

Intern.c++ -- Copyright (c) by Jay Reynolds Freeman 1988, 1989, 1990, 1991, 2006

Value internment system for Wraith Scheme.

        "In the places I go, there are things that I see
         That I never could spell if I stopped with the Z.
         I'm telling you this 'cause you're one of my friends.
         My alphabet starts where your alphabet ends."

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import "WraithHeader.h"
#import "OpCodes.h"

#import "DataTypes.e"
#import "Globals.e"
#import "IO.e"
#import "SchemeAux.e"

/**** Statics: ****/

/**** Functions ****/

/**** Forward Declarations: ****/

void boundSymbolP(),
     changeLambdaName(),
     getSymbol(),
     intern(),
     nameLambdaIfNotNamed(),
     permanentSymbolFetchPair(),
     permanentSymbolP();

/***** assign ****/

PROC( assign )          /* symbol on stack, value in R; leaves #t in R */

    LAMBDA_PAIR_P
    JUMP_FALSE( notLambda )
    CALL( changeLambdaName )

  notLambda:
    PUSH                                            /* S: (value symbol ...) */
    MOVE( STACK( 1 ), R )                           /* R: symbol; S: (value symbol ...) */
    CALL( permanentSymbolP )
    JUMP_TRUE( permanent )
    CALL( boundSymbolP )
    JUMP_FALSE( unbound )
    PUSH                                            /* S: (bindingPair value symbol ...) */
    MOVE( STACK( 1 ), R )

    SETCAR                                          /* S: (value symbol ...) */
    DROP
    DROP
    LOAD_R_TAG( trueBooleanTag )
    CONTINUE

  permanent:
    OUTPUT_TO_ERROR
    CALL( boundSymbolP )
    displaySymbolAndBinding( &CurrentOutputPort );
    resetError( 8 );
    CONTINUE

  unbound:
    MOVE( STACK( 1 ), R )
    OUTPUT_TO_ERROR
    displaySymbol( &CurrentOutputPort );
    resetError( 157 );
    CONTINUE

END_PROC

/***** assign2 ****/

PROC( assign2 )         /* value and symbol on stack, binding cell in R; leaves the symbol in R */

    PUSH
    MOVE( STACK( 2 ), R )                   /* R: symbol; S: (bindingCell value symbol ...) */
    CALL( permanentSymbolFetchPair )        /* faster than permanentSymbolP, in this context */
    JUMP_TRUE( permanent )
    MOVE( STACK( 1 ), R )                   /* R: value; S: (bindingPair value symbol ...) */

    SETCAR                                  /* S: (value symbol ...) */
    DROP
    POP
    CONTINUE

  permanent:
    POP
    DROP
    DROP
    OUTPUT_TO_ERROR
    displaySymbolAndBinding( &CurrentOutputPort );
    resetError( 8 );
    CONTINUE

END_PROC

/***** bindToGrowingEnvironment ****/

PROC( bindToGrowingEnvironment )        /* symbol on stack, above list which is to be environment,
                                           value in R;
                                           returned R: list with the new symbol consed onto it. */

    LAMBDA_PAIR_P
    JUMP_FALSE( notLambda )
    SWAP 
    CALL( permanentSymbolP )
    JUMP_TRUE( permanent )
    SWAP 
    CALL( nameLambdaIfNotNamed )
    SWAP 
    JUMP_LABEL( consAndGo )

  notLambda:    
    SWAP 
    CALL( permanentSymbolP )
    JUMP_TRUE( permanent )

  consAndGo:
    CONS                                /* R: newBindingPair */
    SWAP
    CONS
    CONTINUE

  permanent:
    OUTPUT_TO_ERROR
    CALL( boundSymbolP )
    displaySymbolAndBinding( &CurrentOutputPort );
    resetError( 9 );
    CONTINUE

END_PROC

/***** bindScheme  --  This would be just "bind", but that conflicts with a Unix function name. ****/

PROC( bindScheme )              // value on stack, symbol in R; leaves symbol in R
 
    PUSH                                            // R: symbol; S: (symbol value ...)
    EXCHANGE( R, STACK( 1 ) )                       // R: value;  S: (symbol symbol ...)

    LAMBDA_PAIR_P
    JUMP_FALSE( notLambda )
    CALL( changeLambdaName )

  notLambda:    
    SWAP 
    CONS                                            // R: newBindingPair S: (symbol ...)
    PUSH
    MOVE( E, R )
    CAR          
    VECTOR_P
    JUMP_TRUE( hashedEnvironment )

//  listEnvironment:
    CONS
    PUSH                                            // S: (newEnvironment symbol ...) 
    MOVE( E, R )
    SWAP
    SETCAR
    POP
    CONTINUE

  hashedEnvironment:
    PUSH
    EXCHANGE( R, STACK( 1 ) )                       // R: bindingPair; S: (table table symbol ...) */      
    PUSH
    CDR          
    LOAD_R_TAG( pairTag )
    CDR          
    CAR                                             // R: hashValue; S: (bindingPair table table symbol ... ) 
    PUSH                                            // R: hashValue; S: (hashValue bindingPair table table symbol ... ) 
    EXCHANGE( STACK( 0 ), STACK( 2 ) )              // R: hashValue; S: (table bindingPair hashValue table symbol ... ) 
    VREF                                            // R: bucket; S: (bindingPair hashvalue table symbol ...) 

    EMPTY_LIST_P
    JUMP_TRUE( emptyBucket )
    CALL( lastPair )
    SWAP                       // R: bindingPair; S: (lastBucketPair hashvalue table symbol ...) 
    NILCONS
    SETCDR
    DROP
    DROP
    POP

    CONTINUE

  emptyBucket:
    POP
    NILCONS                                         // R: newBucket; S: ( hashvalue table ...) 
    VSET

    CONTINUE

END_PROC

/***** bindOrAssign ****/

PROC( bindOrAssign )            /* symbol on stack, value in R; leaves symbol in R */

    LAMBDA_PAIR_P
    JUMP_FALSE( notLambda )
    CALL( changeLambdaName )

  notLambda:   
    PUSH                            /* R: value ; S: (value symbol ...) */
    MOVE( STACK( 1 ), R )           /* R: symbol; S: (value symbol ...) */

    PUSH                            /* R: symbol; S: (symbol value symbol ...) */
    CALL( boundSymbolP )            /*            S: (symbol value symbol ...) */
    JUMP_FALSE( doBinding )
    DROP                            /* R: bindingCell; S: ( value symbol ...) */
    JUMP_FUNCTION( assign2 )        /* R: symbol; */

  doBinding:
    POP
    CALL( bindScheme )
    POP
    CONTINUE

END_PROC

/***** bindOrAssignTopLevel ****/

PROC( bindOrAssignTopLevel )            /* symbol on stack, value in R; leaves symbol in R */

// The following lines of code, up through "  notLambda:", commented out, constitute
//   the only difference between this code and "bindOrAssign".  The point is to avoid
//   associating lambda expressions with top-level-loop variable names, that is just
//   too confusing.

//    LAMBDA_PAIR_P
//    JUMP_FALSE( notLambda )
//    CALL( changeLambdaName )
//
//  notLambda:   
    PUSH                            /* R: value ; S: (value symbol ...) */
    MOVE( STACK( 1 ), R )           /* R: symbol; S: (value symbol ...) */

    PUSH                            /* R: symbol; S: (symbol value symbol ...) */
    CALL( boundSymbolP )            /*            S: (symbol value symbol ...) */
    JUMP_FALSE( doBinding )
    DROP                            /* R: bindingCell; S: ( value symbol ...) */
    JUMP_FUNCTION( assign2 )        /* R: symbol; */

  doBinding:
    POP
    CALL( bindScheme )
    POP
    CONTINUE

END_PROC

/**** boundStringP ****/

PROC( boundStringP )    /* string in R; returned B is #f if there is no symbol with that pname,
                           within the environments lexically reachable by the caller;
                           otherwise, #t.  Leaves the binding cell in R, if there is one. */

    Word theBin = -1;

    PUSH                                            /* S: (string ...) */
    MOVE( E, R )
    while( pair_p( R.t ) ) {
        theBin = -1;
        PUSH                                        /* S: (environmentList string ...) */
        CAR          
        VECTOR_P
        if( B ) {
            PUSH                                    /* S: (table environmentList string ...) */
            MOVE( STACK( 2 ), R )
            HASH_STRING     
            theBin = (Word)(R.u.c);
            if( ! (theClowder->lockTLEBin( theBin )) ) {
                resetError( 303 );
                }
            VREF                                    /* S: (environmentList string ...) */
            }
        while( pair_p( R.t ) ) {
            PUSH                                    /* S: (environment environmentList string ...) */
            CAR          
            PUSH
            CDR          
            GET_PNAME
            if( equalByteBlockString( R.u.a, (S[3]).u.a ) ) {
                POP
                DROP
                DROP
                DROP
                SET_B
                if( theBin != -1 )
                    theClowder->unlockTLEBin( theBin );
                CONTINUE
                }
            else {
                DROP
                POP                                 /* S: (environmentList address ...) */
                CDR          
                }
            }
        if( theBin != -1 )
            theClowder->unlockTLEBin( theBin );
        POP                                         /* S: (address ...) */
        CDR          
        }
    DROP
    CLEAR_B
    if( theBin != -1 )
        theClowder->unlockTLEBin( theBin );
    CONTINUE

END_PROC

/**** boundSymbolP ****/

PROC( boundSymbolP )      /* symbol in R; returned B is #t or #f according 
                             to whether there exists a symbol with that pname,
                             within the environments lexically reachable by
                             the caller.  The binding cell, if any, is in R. */
    Word theBin = -1;

    PUSH                                            /* S: (symbol ...) */
    MOVE( E, R )
    while( pair_p( R.t ) ) {
        theBin = -1;
        PUSH                                        /* S: (environmentList symbol ...) */
        CAR          
        VECTOR_P
        if( B ) {
            PUSH                                    /* S: (table environmentList symbol ...) */
            MOVE( STACK( 2 ), R )
            GET_HASH_VALUE     
            theBin = (Word)(R.u.c);
            if( ! (theClowder->lockTLEBin( theBin )) ) {
                resetError( 303 );
                }
            VREF                                    /* S: (environmentList symbol ...) */
            }
        while( pair_p( R.t ) ) {
            PUSH                                    /* S: (environment environmentList symbol ...) */
            CAR          
            PUSH
            CDR          
//            GET_PNAME
            if( R.u.a == (S[3]).u.a ) {
                POP
                DROP
                DROP
                DROP
                SET_B
                if( theBin != -1 )
                    theClowder->unlockTLEBin( theBin );
                CONTINUE
                }
            else {
                DROP
                POP                                 /* S: (environmentList address ...) */
                CDR          
                }
            }
        if( theBin != -1 )
            theClowder->unlockTLEBin( theBin );
        POP                                         /* S: (address ...) */
        CDR          
        }
    DROP
    CLEAR_B
    if( theBin != -1 )
        theClowder->unlockTLEBin( theBin );
    CONTINUE

END_PROC

/**** boundSymbolInEnvironmentP ****/

PROC( boundSymbolInEnvironmentP )      /* symbol in S(0); environment in R; returned B is #t or #f according 
                                          to whether there exists a symbol with that pname,
                                          within the environments lexically reachable by
                                          the caller, from the given environment.  The binding cell, if any, is in R. */
    Word theBin = -1;

    while( pair_p( R.t ) ) {
        theBin = -1;
        PUSH                                        /* S: (environmentList symbol  ...) */
        CAR          
        VECTOR_P
        if( B ) {
            PUSH                                    /* S: (table environmentList symbol  ...) */
            MOVE( STACK( 2 ), R )
            GET_HASH_VALUE
            theBin = (Word)(R.u.c);
            if( ! (theClowder->lockTLEBin( theBin )) )
                resetError( 304 );
            VREF                                    /* S: (environmentList symbol  ...) */
            }
        while( pair_p( R.t ) ) {
            PUSH                                    /* S: (environment environmentList symbol  ...) */
            CAR                                     // R: Binding pair
            PUSH 
            CDR                                     // R: Symbol
//            GET_PNAME                               /* S: (binding-pair environment environmentList symbol  ...) */
            if( R.u.a == (S[3]).u.a ) {
                POP
                DROP
                DROP
                DROP
                SET_B
                if( theBin != -1 )
                    theClowder->unlockTLEBin( theBin );
                CONTINUE
                }
            else {
                DROP
                POP                                 /* R: environment; S: (environmentList symbol  ...) */
                CDR          
                }
            }
        if( theBin != -1 )
            theClowder->unlockTLEBin( theBin );
        POP                                         /* S: (symbol  ...) */
        CDR          
        }
    DROP
    CLEAR_B
    if( theBin != -1 )
        theClowder->unlockTLEBin( theBin );
    CONTINUE


END_PROC

/**** changeLambdaName ****/

PROC( changeLambdaName )    /* symbol on stack, lambda pair in R; leaves them there. */

    PUSH
    PUSH
    LOAD_R_TAG( pairTag )
    CDR          
    CDR                                     /* R: (env . oldName);  S: (lambda lambda symbol ...) */
    SWAP                                    /* R: lambda;           S: ((env . oldName) lambda symbol ...) */
    MOVE( STACK( 2 ), R )                   /* R: symbol;           S: ((env . oldName) lambda symbol ...) */

    SETCDR                                  /* R: (env . newName);  S: (lambda symbol ...) */
    POP                                     /* R: lambda;           S: (symbol ...) */
    CONTINUE
        
END_PROC

/**** clearPermanentSymbol ****/

PROC( clearPermanentSymbol )    /* R is the symbol to be made un-permanent */

    LOAD_R_TAG( pairTag )
    CDR          
    PUSH
    LOAD_R_TAG( emptyListTag )

    SETCDR
    CONTINUE

END_PROC

/**** createNonPermanentSymbol -- Create a non-permanent symbol named by this string. ****/

PROC( createNonPermanentSymbol )    /* R is exact short fixnum addressing NON-STORE,
                                       text buffer; STACK( 1 ) is the hashvalue.
                                       Returned R is the new symbol */
    MAKE_STRING
    PUSH
    MOVE( STACK( 2 ), R )
    NILCONS
    CONS
    LOAD_R_TAG( symbolTag )
    CONTINUE

END_PROC

/**** genSym -- Return a new unused symbol, interned ****/

PROC( genSym )

  tryAgain:
    GENSYM
    PUSH
    CALL( getSymbol )
    SYMBOL_P
    JUMP_FALSE( success )
    DROP
    JUMP_LABEL( tryAgain )

  success:
    POP
    CALL( intern )
    CONTINUE

END_PROC        

/**** getSymbol ****/

PROC( getSymbol )       /* R is exact short fixnum addressing a NON-STORE text buffer;
                           the returned R is either the interned symbol named by the buffer,
                           or the bucket where it would be if no such symbol exists, for that pname,
                           in the oblist. */ 

    Word theBin = 0;

    theHeap->triggerGC();

    Boolean obListWasInRegister = theClowder->getObListInRegister( theKittenNumber );

    if( ! obListWasInRegister )
        theClowder->setObListInRegister();

    PUSH                                /* S: (address ...) */
    MOVE( ObList, R )
    PUSH                                /* S: (oblist address ...) */
    MOVE( STACK( 1 ), R )
    HASH_TEXT                           /* S: (address ...) */
    theBin = (Word)(R.u.c);
    if( ! (theClowder->lockObListBin( theBin )) ) {
        if( ! obListWasInRegister )
            theClowder->clearObListInRegister();
        resetError( 302 );
        }
    VREF                                /* S: (address ...) */
    PUSH                                /* S: (bucket address ...) */
    while( pair_p( R.t ) ) {
        PUSH                            /* S: (environmentList bucket address ...) */
        CAR          
        LOAD_R_TAG( pairTag )
        CAR          
        if( equalByteBlockToString( R.u.a, (Text *)((S[2]).u.c) ) ) {
            POP
            DROP
            DROP
            CAR
            theClowder->unlockObListBin( theBin );
            if( ! obListWasInRegister )
                theClowder->clearObListInRegister();
            theHeap->triggerOnRebuildObList();
            CONTINUE
            }
        else {
            POP                         /* S: (bucket address ...) */
            CDR          
            }
        }
    POP
    DROP
    theClowder->unlockObListBin( theBin );
    if( ! obListWasInRegister )
        theClowder->clearObListInRegister();
    theHeap->triggerOnRebuildObList();
    CONTINUE

END_PROC

/**** initializeEnvironmentList -- make environment list of one empty environment ****/

void initializeEnvironmentList( Boolean momCatP )
{
    if( momCatP ) {
        MOVE( symbolHashSize, R )
        CALL( makeVectorOfSharpF )
        PUSH
        LOAD_R_TAG( emptyListTag)
        PUSH
        LOAD_R_TAG( emptyListTag)
        NILCONS
        CONS
        CONS
        MOVE( R, TopLevelE )
    }
    MOVE( TopLevelE, E )
    }

/**** countOblist -- how many symbols are in the ObList? ****/

UnsignedLongWord countObList()
{
    UnsignedLongWord answer = 0;
    Boolean obListWasInRegister = theClowder->getObListInRegister( theKittenNumber );

    PUSH
    if( ! obListWasInRegister )
        theClowder->setObListInRegister();
    MOVE( ObList, R )    
    LOAD_R_TAG( pairTag )
    CDR
    while( pair_p( R.t ) ) {
        PUSH
        CAR
        if( pair_p( R.t ) ) {
            LENGTH
            answer += (UnsignedLongWord)(R.u.c);
            }
        POP
        CDR
        }
    POP
    if( ! obListWasInRegister )
        theClowder->clearObListInRegister();
    theHeap->triggerOnRebuildObList();
    return answer;
    }

/**** initializeObList -- make oblist ****/

void initializeObList( Boolean momCatP )
{
    if( momCatP ) {
//        tellController( E_CLEAR_OBLIST_SYMBOLS, 0, 0 );
        MOVE( symbolHashSize, R )
        CALL( makeVectorOfSharpF )
        MOVE( R, ObList )
        theClowder->setObListCount( 0 );
        }
    }

/**** initializeIVR -- make interrupt vector register content ****/

void initializeIVR( Boolean momCatP )
{
    if( momCatP ) {
        MOVE( numberOfInterrupts, R )
        CALL( makeVectorOfSharpF )
        MOVE( R, IVR )
        }
    }

/**** installSymbolInObList ****/

// Put a symbol into the oblist.  We assume it is not already there:
// This function is used only when reinitializing the oblist after GC;
//   and is intended to benefit from the handling of symbols in Wraith
//   Scheme, which is supposed to ensure that there is only one copy of
//   any given symbol present.

void installSymbolInObList( taggedAval *aSymbol )
{
    taggedAval hashVal;
    Boolean obListWasInRegister = theClowder->getObListInRegister( theKittenNumber );

    if( ! obListWasInRegister )
        theClowder->setObListInRegister();
    PUSH                                            // S: saved-R
    MOVE( A, R )
    PUSH                                            // S: saved-A, saved-R
    MOVE( ObList, R )
    PUSH                                            // S: oblist, saved-A, saved-R
    R = *aSymbol;
    R.t &= ~(cdrCodingBitsMask);
    R = spliceOutNotCarForwardingPointers( R );
    PUSH                                            // S: aSymbol, oblist, saved-A, saved-R
    MOVE( ObList, R )
    PUSH                                            // S: oblist, aSymbol, oblist, saved-A, saved-R
    R = *aSymbol;
    LOAD_R_TAG( pairTag )
    cdrRegardlessOfBlocking();
    carRegardlessOfBlocking();                      // R: hashValue; S: oblist, aSymbol, oblist, saved-A, saved-R
    hashVal = R;
    VREF                                            // R: bucket; S: aSymbol, oblist, saved-A, saved-R
    CONS                                            // R: newBucket; S: oblist, saved-A, saved-R
    PUSH                                            // R: newBucket; S: newBucket, oblist, saved-A, saved-R
    MOVE( hashVal, STACK( 0 ) )                     // R: newBucket; S: hashVal, oblist, saved-A, saved-R
    VSET                                            // S: saved-A, saved-R
    POP
    MOVE( R, A )
    POP
    theClowder->incrementObListCount();
    if( ! obListWasInRegister )
        theClowder->clearObListInRegister();
//    theHeap->triggerOnRebuildObList();              // CANNOT leave this in, since "installSymbolInObList is called from within
//                                                    //   the garbage collector.  This call might cause reentrant garbage collection.
//    OSMemoryBarrier();                              // Commented out 2009.08.12
    }    

/**** intern ****/

PROC( intern )  /* R is an exact short fixnum that addresses a NON-STORE text buffer;
                   returned R is (the latest) / (a new) symbol with that pname */

    Word theBin = 0;

    theHeap->triggerGC();                           // TEST (and leave in if it works) 2010.01.15

    Boolean obListWasInRegister = theClowder->getObListInRegister( theKittenNumber );

    if( ! obListWasInRegister )
        theClowder->setObListInRegister();
    PUSH                                            /* S: (address ...) */
    MOVE( ObList, R )
    PUSH                                            /* S: (oblist address ...) */
    MOVE( STACK( 1 ), R )
    HASH_TEXT                                       /* R: hashval; S: (oblist address ...) */
    PUSH                                            /* R: hashval; S: (hashval oblist address ...) */
    EXCHANGE( STACK( 0 ), STACK( 1 ) )      
    theBin = (Word)(R.u.c);
    if( ! (theClowder->lockObListBin( theBin )) ) {
        if( ! obListWasInRegister )
            theClowder->clearObListInRegister();
        resetError( 301 );
        }
    VREF                                            /* R: bucket; S: (hashval address ...) */
    PUSH                                            /* R: bucket; S: (bucket hashval address ...) */
    if( empty_list_p( R.t ) ) {                     /* empty bucket */
        MOVE( STACK( 2 ), R )
//        tellController( E_NEW_OBLIST_SYMBOL, 0, (Text *)((Byte *)(R.u.a)) );
        CALL( createNonPermanentSymbol )            /* R: symbol; S: (bucket hashval address ...) */
        MOVE( R, STACK( 2 ) )                       /* R: symbol; S: (bucket hashval symbol ...) */
        MOVE( STACK( 1 ), STACK( 0 ) )              /* R: symbol; S: (hashval hashval symbol ...) */
        MOVE( ObList, STACK( 1 ) )                  /* R: symbol; S: (hashval oblist symbol ...) */
        NILCONS
        VSET                                        /* S: (symbol ...) */
        POP
        theClowder->incrementObListCount();
        theClowder->unlockObListBin( theBin );
        if( ! obListWasInRegister )
            theClowder->clearObListInRegister();
        theHeap->triggerOnRebuildObList();
        CONTINUE
        }

    while( TRUE ) {
        PUSH                                        /* S: (environmentList bucket hashval address ...) */
        CAR          
        PUSH
        LOAD_R_TAG( pairTag )
        CAR          
        if( equalByteBlockToString( R.u.a, (Text *)((S[4]).u.c) ) ) {
            POP
            DROP
            DROP
            DROP
            DROP
            theClowder->unlockObListBin( theBin );
            if( ! obListWasInRegister )
                theClowder->clearObListInRegister();
            theHeap->triggerOnRebuildObList();
            CONTINUE
            }
        else {
            DROP
            MOVE( STACK( 0 ), R )                   /* R: environmentList; S: (environmentList bucket hashval address ...) */
            CDR          
            if( R.t == emptyListTag )
                break;
            DROP                                    /* R: environmentList; S: (bucket hashval address ...) */
            }
        }
    POP                                             /* R: environmentListLastPair; S: (bucket hashval address ...) */ 
    MOVE( R, STACK( 0 )     )                       /* R: environmentListLastPair; S: (environmentListLastPair hashval address ...) */
    MOVE( STACK( 2 ), R )                           /* R: address; S: (environmentListLastPair hashval address ...) */        
//    tellController( E_NEW_OBLIST_SYMBOL, 0, (Text *)((Byte *)(R.u.a)) );
    CALL( createNonPermanentSymbol )                /* R: symbol; S: (environmentListLastPair hashval address ...) */ 
    MOVE( R, STACK( 1 ) )                           /* R: symbol; S: (environmentListLastPair symbol address ...) */
    NILCONS
    SETCDR
    POP
    DROP
    theClowder->incrementObListCount();
    theClowder->unlockObListBin( theBin );
    if( ! obListWasInRegister )
        theClowder->clearObListInRegister();
    theHeap->triggerOnRebuildObList();
    CONTINUE

END_PROC

/**** lookUpBindingPair ****/

PROC( lookUpBindingPair )       /* R is a symbol; returned R is the instance of that symbol
                                   in the lexically innermost scope in which it is bound. */
                   
    Word theBin = -1;

    PUSH
    MOVE( E, R )
    LENGTH
    PUSH
    MOVE( three, R )
    NUM_GE_P
    POP
    JUMP_FALSE( notEasy )

    MOVE( STACK( 0 ), R )
    CALL( permanentSymbolFetchPair )
    JUMP_FALSE( notEasy )
    DROP
    CONTINUE

  notEasy:      
    MOVE( STACK( 0 ), R )
    GET_HASH_VALUE
    SWAP
    PUSH                                                    /* S: (givenSymbol hashValue ...) */
    MOVE( E, R )

    while( pair_p( R.t ) ) {
        theBin = -1;
        PUSH                                                /* S: (envList givenSymbol hashValue ...) */
        CAR          
        VECTOR_P
        if( B ) {
            EXCHANGE( STACK( 0 ), STACK( 2 ) )
            PUSH                                            /* S: (table hashValue givenSymbol envList ...) */
            MOVE( STACK( 1 ), R )
            theBin = (Word)(R.u.c);
            if( ! (theClowder->lockTLEBin( theBin )) ) {
                resetError( 305 );
                }
            VREF                                            /* S: (hashValue givenSymbol envList ...) */
            EXCHANGE( STACK( 0 ), STACK( 2 ) )
            }
        while( pair_p( R.t ) ) {
            PUSH                                            /* S: (env envList givenSymbol hashValue ...) */
            CAR          
            CDR          
            SWAP
            EXCHANGE( R, STACK( 2 ) )                       /* S: (newSymbol envList env hashValue ...) */
            POINTER_EQUAL_P
            JUMP_TRUE( found )
            EXCHANGE( R, STACK( 2 ) )
            DROP                                            /* S: (envList givenSymbol hashValue ...) */
            CDR          
            }
        if( theBin != -1 )
            theClowder->unlockTLEBin( theBin );
        POP                                                 /* S: (givenSymbol hashValue ...) */
        CDR          
        }
    POP
    PUSH
    OUTPUT_TO_ERROR
    DISPLAY
    if( theBin != -1 ) {
        theClowder->unlockTLEBin( theBin );
        }
    POP
//    GET_HASH_VALUE
//    tprintf3( &CurrentOutputPort, " (hash value %ld)", R.u.c );
    resetError( 10 );

  found:
    DROP
    DROP
    POP                                                     /* R: the env whose car is the binding pair just found */
    CAR          
    DROP
    if( theBin != -1 )
        theClowder->unlockTLEBin( theBin );
    CONTINUE

END_PROC

/**** lookUp ****/

PROC( lookUp )  /* R is a symbol; returned R is the binding of that symbol
                   in the lexically innermost environment in which it is bound. */
                   
    CALL( lookUpBindingPair )
    GET_SYMBOL_BINDING_VALUE
    CONTINUE

END_PROC

/**** lookUpBindingPairInEnvironment ****/

PROC( lookUpBindingPairInEnvironment )       /* R is a symbol; S(0) an environment.  Returned R is the instance of
                                                that symbol in the lexically innermost scope of the given environment. */
    Word theBin = -1;

    PUSH
    GET_HASH_VALUE
    SWAP
////    GET_PNAME
    PUSH                                                    /* S: (givenSymbol hashValue ...) */
    MOVE( STACK( 2), R )                                    /* R: The given environment      */
    while( pair_p( R.t ) ) {
        theBin = -1;
        PUSH                                                /* S: (envList givenSymbol hashValue ...) */
        CAR          
        VECTOR_P
        if( B ) {
            EXCHANGE( STACK( 0 ), STACK( 2 ) )
            PUSH                                            /* S: (table hashValue givenSymbol envList ...) */
            MOVE( STACK( 1 ), R )
            theBin = (Word)(R.u.c);
            if( ! (theClowder->lockTLEBin( theBin )) )
                resetError( 306 );
            VREF                                            /* S: (hashValue givenSymbol envList ...) */
            EXCHANGE( STACK( 0 ), STACK( 2 ) )
            }
        while( pair_p( R.t ) ) {
            PUSH                                            /* S: (env envList givenSymbol hashValue ...) */
            CAR          
            CDR          
////            GET_PNAME
            SWAP
            EXCHANGE( R, STACK( 2 ) )                       /* S: (newSymbol envList env hashValue ...) */
////            STRING_EQUAL_P
            POINTER_EQUAL_P
            JUMP_TRUE( found )
            EXCHANGE( R, STACK( 2 ) )
            DROP                                            /* S: (envList givenSymbol hashValue ...) */
            CDR          
            }
        POP                                                 /* S: (givenSymbol hashValue ...) */
        CDR          
        if( theBin != -1 )
            theClowder->unlockTLEBin( theBin );
        }
    POP
    OUTPUT_TO_ERROR
    DISPLAY
    if( theBin != -1 )
        theClowder->unlockTLEBin( theBin );
    resetError( 202 );

  found:
    DROP
    DROP
    POP                                                     /* R: the env whose car is the binding pair just found */
    CAR          
    DROP
    DROP
    if( theBin != -1 )
        theClowder->unlockTLEBin( theBin );
    CONTINUE

END_PROC

/**** lookUpInEnvironment ****/

PROC( lookUpInEnvironment )  /* S(0) is a symbol; R an environment; returned R is the binding of that symbol
                                in the lexically innermost environment of the given environment in which it is bound. */
                   
    SWAP
    CALL( lookUpBindingPairInEnvironment )
    GET_SYMBOL_BINDING_VALUE
    CONTINUE

END_PROC

/**** makePermanentSymbol ****/

PROC( makePermanentSymbol )             /* R is the symbol to be made permanent */

    PUSH
    CALL( boundSymbolP )

    if( ! B ) {
        POP
        OUTPUT_TO_ERROR
        DISPLAY
        resetError( 11 );               
        }

    DROP
    PUSH                                /* R: bindingPair; S: (bindingPair ...) */
    CDR                                 /* R: symbol; S: (bindingPair ...) */
    LOAD_R_TAG( pairTag )
    CDR                                 /* R: (hashval . <dontCare>); S: (bindingPair ...) */
    SWAP                                /* R: bindingPair; S: ((hashval . <dontCare>) ...) */
    SETCDR

END_PROC

/**** nameLambdaIfNotNamed ****/

PROC( nameLambdaIfNotNamed )     /* symbol on stack, lambda pair in R; leaves them there. */

    PUSH
    PUSH
    LOAD_R_TAG( pairTag )
    CDR          
    CDR                                     /* R: (env . oldName);  S: (lambda lambda symbol ...) */
    PUSH
    CDR          
    EMPTY_LIST_P
    JUMP_FALSE( alreadyNamed )

    POP
    SWAP               /* R: lambda;           S: ((env . oldName) lambda symbol ...) */
    MOVE( STACK( 2 ), R )                   /* R: symbol;           S: ((env . oldName) lambda symbol ...) */
    SETCDR                                  /* R: (env . newName);  S: (lambda symbol ...) */
    POP                                     /* R: lambda;           S: (symbol ...) */
    CONTINUE

  alreadyNamed:
    DROP
    DROP
    POP
    CONTINUE

END_PROC

/**** permanentSymbolFetchPair ****/

PROC( permanentSymbolFetchPair )

    LOAD_R_TAG( pairTag )
    CDR          
    CDR          
    PAIR_P
    CONTINUE

END_PROC

/**** permanentSymbolP ****/

PROC( permanentSymbolP )

    PUSH
    LOAD_R_TAG( pairTag )
    CDR          
    CDR          
    PAIR_P
    POP
    CONTINUE

END_PROC

/**** printEnvironment -- Display the entire contents of the headmost subenvironment
                          of the entire environment, in a form useful to the user.  ****/

PROC( printEnvironment )        /* print current environment only */
 
    MOVE( E, R )
    PUSH
    LENGTH
    tprintf3( &CurrentOutputPort, MiscStrings[ 90 ], R.u.c );
    POP
    if( ! empty_list_p( R.t ) ) {
        CAR          
        VECTOR_P
        if( B ) {
            LOAD_R_TAG( pairTag )
            CDR          
            while( pair_p( R.t ) ) {
                PUSH
                CAR          
                while( pair_p( R.t ) ) {
                    PUSH                                                    /* S: (environment ...) */
                    CAR          
                    tprintf2( &CurrentOutputPort, MiscStrings[ 110 ] );
                    displaySymbolAndBinding( &CurrentOutputPort );
                    tprintf2( &CurrentOutputPort, MiscStrings[ 115 ] );
                    POP                                                     /* S: (...) */
                    CDR          
                    }
                POP
                CDR          
                }
            }
        else
            while( pair_p( R.t ) ) {
                PUSH                                                        /* S: (environment ...) */
                CAR          
                tprintf2( &CurrentOutputPort, MiscStrings[ 110 ] );
                displaySymbolAndBinding(&CurrentOutputPort );
                tprintf2( &CurrentOutputPort, MiscStrings[ 115 ] );
                POP                                                         /* S: (...) */
                CDR
                }
        }
    CONTINUE

END_PROC
 
/**** printEnvironmentList -- Display the entire contents of the entire
                              environment, in a form useful to the user. ****/

PROC( printEnvironmentList )
 
    LongWord levelDown = 0;

    tprintf2( &CurrentOutputPort, MiscStrings[ 115 ] );
    MOVE( E, R )
    while( pair_p( R.t ) ) {
        tprintf3( &CurrentOutputPort, MiscStrings[ 84 ], levelDown );
        ++levelDown;
        PUSH                                                            /* S: (environmentList ...) */
        CAR          
        VECTOR_P
        if( B ) {
            LOAD_R_TAG( pairTag )
            CDR          
            while( pair_p( R.t ) ) {
                PUSH
                CAR          
                while( pair_p( R.t ) ) {
                    PUSH                                                /* S: (environment ...) */
                    CAR          
                    tprintf2( &CurrentOutputPort, MiscStrings[ 110 ] );
                    displaySymbolAndBinding( &CurrentOutputPort );
                    tprintf2( &CurrentOutputPort, MiscStrings[ 115 ] );
                    POP                                                 /* S: (...) */
                    CDR          
                    }
                POP
                CDR          
                }
            }
        else
            while( pair_p( R.t ) ) {
                PUSH                                                    /* S: (environment environmentList ...) */
                CAR          
                tprintf2( &CurrentOutputPort, MiscStrings[ 110 ] );
                displaySymbolAndBinding( &CurrentOutputPort );
                tprintf2( &CurrentOutputPort, MiscStrings[ 115 ] );
                POP                                                     /* S: (environmentList ...) */
                CDR          
                }
        POP                                                             /* S: (...) */
        CDR          
        }
    tprintf2( &CurrentOutputPort, MiscStrings[ 83 ] );
    CONTINUE

END_PROC

/**** printDynamicEnvironmentList -- Display the entire contents of all non-top-level
                                     environments, in a form useful to the user. ****/

PROC( printDynamicEnvironmentList )
 
    LongWord levelDown = 0;

    tprintf2( &CurrentOutputPort, MiscStrings[ 115 ] );
    MOVE( E, R )
    while( pair_p( R.t ) ) {
        tprintf3( &CurrentOutputPort, MiscStrings[ 84 ], levelDown );
        ++levelDown;
        PUSH                                                    /* S: (environmentList ...) */
        CAR          
        VECTOR_P
        if(  B ) {
            tprintf2( &CurrentOutputPort, MiscStrings[ 3 ] );
            tprintf2( &CurrentOutputPort, MiscStrings[ 115 ] );
            break;
            }
        else
            while( pair_p( R.t ) ) {
                PUSH                                            /* S: (environment environmentList ...) */
                CAR          
                tprintf2( &CurrentOutputPort, MiscStrings[ 110 ] );
                displaySymbolAndBinding( &CurrentOutputPort );
                tprintf2( &CurrentOutputPort, MiscStrings[ 115 ] );
                POP                                             /* S: (environmentList ...) */
                CDR          
                }
        POP                                                     /* S: (...) */
        CDR          
        }
    tprintf2( &CurrentOutputPort, MiscStrings[ 83 ] );
    CONTINUE

END_PROC

/**** printDynamicEnvironmentListFromR -- Display the entire contents of all non-top-level
                                          environments, in a form useful to the user. ****/

PROC( printDynamicEnvironmentListFromR )
 
    LongWord levelDown = 0;

    tprintf2( &CurrentOutputPort, MiscStrings[ 115 ] );
    while( pair_p( R.t ) ) {
        PUSH                                                    /* S: (environmentList ...) */
        CAR          
        VECTOR_P
        if(  B ) {
            POP
            break;
            tprintf2( &CurrentOutputPort, MiscStrings[ 3 ] );
            tprintf2( &CurrentOutputPort, MiscStrings[ 115 ] );
             }
        else {
            tprintf3( &CurrentOutputPort, MiscStrings[ 84 ], levelDown );
            ++levelDown;
            while( pair_p( R.t ) ) {
                PUSH                                            /* S: (environment environmentList ...) */
                CAR          
                tprintf2( &CurrentOutputPort, MiscStrings[ 110 ] );
                displaySymbolAndBinding( &CurrentOutputPort );
                tprintf2( &CurrentOutputPort, MiscStrings[ 115 ] );
                POP                                             /* S: (environmentList ...) */
                CDR          
                }
            }
        POP                                                     /* S: (...) */
        CDR          
        }
    tprintf2( &CurrentOutputPort, MiscStrings[ 385 ] );
    CONTINUE

END_PROC

