/****************************************************************************************

MainAux.c++ -- Copyright (c) by Jay Reynolds Freeman 1987, 1988, 1989, 1990, 1991, 2006

Auxiliaries "Main" for Wraith Scheme.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import <sys/param.h>
#import <sys/sysctl.h>
#import <signal.h>
#import "WraithHeader.h"
#import "MemoryManager.h"
#import "Clowder.h"
#import "DataTypes.e"
#import "MemoryManager.h"
#import "Events.e"
#import "Globals.e"
#import "Intern.e"
#import "IO.e"
#import "MMapStuff.e"
#import "OpCodes.h"
#import "Reader.e"
#import "Registers.e"
#import "SECDIO.e"
#import "SECDMachine.e"
#import "SECDRegisters.e"
#import "WorldHandler.h"
#import "SchemeCType.e"
#import "SchemeFunctions.e"
#import "SchemeFunctionExterns.e"
#import "SchemeGlobals.e"
#import "SchemeSpecials.e"
#import "StreamHandling.e"
#import "StringLoader.e"

#define KITTEN_LAUNCH_DELAY 100000
#define MAX_LAUNCH_RETRIES  100
#define MAX_INIT_WAITS      1000

#define LOAD_WORLD_DELAY    1000  // microseconds
#define INIT_GLOBALS_DELAY  1000  // microseconds

#ifdef USE_COCOA_IO

#import "CocoaInterface.h"

#else

// These are only used when a Cocoa-style view is running, and are defined in one of its ".mm" files.
// Duplicates here save a lot of "#ifdef USE_COCOA_IO"s in subsequent code.

int outputEnabled = 1;
int clowderIsOperating = 0;
int worldIsLoaded = 0;
int evaluatorHasInitialized = 0;
int controllerReadyForMessages = 1;
int GUIInitialized = 1;
int readyToRead = 1;

#endif

// Globals

UnsignedWord theKittenNumber = 0;

#ifdef USE_COCOA_IO

UnsignedWord theNumberOfKittens = 0;
Boolean usingGenerationalGC = TRUE;

#else

UnsignedWord theNumberOfKittens = 1;
Boolean usingGenerationalGC = FALSE;

#endif

MemoryManager *theHeap = 0;
Clowder *theClowder = 0;
WorldHandler *theWorldHandler = 0;
UnsignedLongWord *theSimpleTable = 0;
Boolean bigHeap = FALSE;
Boolean BigEndianFlag = FALSE;
long long hardwareMemorySize = 0;
Text lastWorldLoadedName[ MAX_PATHNAME_LENGTH + 1];
Text InitFileName[ MAX_PATHNAME_LENGTH ];
Boolean useStdinP = FALSE;
Boolean isMomCatP = TRUE;
Boolean aWorldHasJustBeenLoaded = FALSE;

LongWord progressFractionInThousandths = 0;

Text theKittenString[ 32 ];
Text appSupDirName[ MAX_PATHNAME_LENGTH + 1 ];

Boolean isAlarmSet = FALSE;
LongWord alarmTime = 0;

// end globals

static Boolean InitFileExists;
static LongWord nFuncs = 0;

// Some targets for longjmp.

jmp_buf debugBuffer;
jmp_buf ExitBuffer;
jmp_buf ResetErrorBuffer;

Boolean debugBufferInitialized = FALSE;

/**** Forwards: ****/

Text *loadWorldIfPossible();
void launchInstance( Word newKittenNumber );
Boolean momSaysReset();
void openKittenLogFile();
Boolean allKittensAreActive();

/**** Here come the functions: ****/

int getProcessorCount()
{

    int     count;
    size_t  size=sizeof(count);

    if( sysctlbyname("hw.ncpu", &count, &size, NULL, 0 ) )
        return 1;

    return count;
    }

/**** failAtOnce ****/

void failAtOnce()
{ 
     kill( getpid(), SIGBUS );     // Doesn't generate kill report, et cetera.
     }

/**** debugEntry -- "longjmp" to restart "eval" ****/

void debugEntry()
{ 
    if( debugBufferInitialized )
        longjmp( debugBuffer, 1 );
    else
        exit( 1 );
    }

/**** exitTopLoop ****/

void exitTopLoop() { longjmp( ExitBuffer, 1 ); }

/**** topLoopCode -- The canonical top-level read-eval-print loop.  Push into the
                     continuation, code that (1) reads an object, (2) pushes the object
                     that was read back into the continuation (so it can in turn be
                     evaluated), (3) writes the result (to "defaultOutputFD"), (4) writes
                     a newline (to "defaultOutputFD"), and finally (5) loads the code
                     for "topLoopCode" itself into the continuation (thus "topLoopCode"
                     tail-recurses).
*/

PROC( topLoopCode )

    depthSoFar = vDepthSoFar = 0;                   // Hackish
    SPushR();
    MOVE( TLC, R );
    CSPushProperList();
    SPopR();
    CONTINUE

END_PROC

/**** runTopLoop ****/

static Boolean InitFileLoadAttempted = FALSE;

void runTopLoop( Boolean InitFilePresent )
{
    Boolean runFirstCode = FALSE;

    depthSoFar = vDepthSoFar = 0;

    tprintf2( &CurrentOutputPort, MiscStrings[ 105 ] );     // "Top-level loop ..."
    tfflushOutput( getPortFilePointer( &CurrentOutputPort ) );

    SPushR();
    MOVE( TLC, R );
    CSPushProperList();
    SPopR();

    if( aWorldHasJustBeenLoaded ) {                         // Deal with a world load.
        aWorldHasJustBeenLoaded = FALSE;
        if( isMomCatP ) {
            LOAD_R_TAG( exactShortFixnumBitsMask )
            LOAD_OBJECT( NameStrings[ 377 ], R )
            CALL( intern )
            BOUND_SYMBOL_P
            JUMP_FALSE( noCode )

            // Here following, we evaluate the function to be run
            // on behalf of the world just loaded, and set up to run it.

            EVAL_SYMBOL
            LAMBDA_PAIR_P
            JUMP_FALSE( noCode )
            NILCONS
            PUSH
			MOVE( zero, R )
            SWAP

extern void tellKittenCode();

            CALL( tellKittenCode )
            runFirstCode = TRUE;                            // The world has a function to run.
          noCode:
            runFirstCode = FALSE;                           // The world does not have a function to run.
            }
        }
    if( runFirstCode )                                      // We are set to run the function.
        ;
    else if( ! InitFileLoadAttempted ) {                    // Check for an "init" file, and load it if applicable.
        InitFileLoadAttempted = TRUE;
        if( InitFilePresent ) {
            LOAD_R_TAG( schemeMachinePCIndexTag )
            LOAD_OBJECT( assertNotEvaluating_index, R )
            PUSH_CONTINUATION
            LOAD_R_TAG( schemeMachinePCIndexTag )
            LOAD_OBJECT( loadCode_index, R )
            PUSH_CONTINUATION
            LOAD_R_TAG( exactShortFixnumBitsMask )
            LOAD_OBJECT( InitFileName, R )
            MAKE_STRING
            PUSH
            MOVE( one, R )
            }
        }       

    setjmp( debugBuffer );                                  // This is the final position of the debug longjmp target.
    debugBufferInitialized = TRUE;                    
    eval();                                                 // ... and it never returned, and its fate is still unlearned ...
    }

/**** initializeApplicationSupportDirectory ****/

extern int errno;

void initializeApplicationSupportDirectory()
{
    int statResult = 0;
    int statError = 0;
    struct stat appDirStatBuffer;

    snprintf( appSupDirName, MAX_PATHNAME_LENGTH, "%s", getenv("HOME") );
    strncat( appSupDirName, "/Library/Application Support/WraithScheme", MAX_PATHNAME_LENGTH - strlen( appSupDirName ) );
    statResult = stat( appSupDirName, &appDirStatBuffer );
    statError = errno;
    if( (statResult == 0) ) {
        if( appDirStatBuffer.st_mode & S_IFDIR )  // Fine!
            return;
        else {
            fprintf( stderr, MiscStrings[ 392 ], appSupDirName, MiscStrings[ 508 ] );
            exit( 1 );
            }
        }
    else {
        if( mkdir( appSupDirName, 0777 ) == 0 ) { // Fine!
             return;
             }
        else {
            fprintf( stderr, MiscStrings[ 393 ], MiscStrings[ 508 ], appSupDirName );
            exit( 1 );
            }
        }
    return;
    }

/**** For developer use -- optional initialization tests to be run on entry, before something else breaks. ****/

void initializationTests()
{

    fprintf( stderr, "INITIALIZATION TESTS:\n");  fflush( stderr );

    }


/**** initialize -- master initialization portmanteau ****/

void initialize( UnsignedLongWord *memorySize, UnsignedLongWord *generationSize ) 
{
    srandom( (Word)time( NULL ) );          // Seed the random number generator.

//    initializationTests();  // Developer option -- for testing.
    initializeEvents();
    initializeLowLevelIO();

    const NXArchInfo *localArchitecture = NXGetLocalArchInfo();

    if( localArchitecture == NULL )
        fatalError( 91 );
    else if( localArchitecture->byteorder == NX_UnknownByteOrder )
        fatalError( 91 );
    else if( localArchitecture->byteorder == NX_BigEndian )
        BigEndianFlag = TRUE;
    else // NX_LittleEndian
        BigEndianFlag = FALSE;

    initializeStartupWorkingFolder();
    initializeSignalHandlers();

    initializeApplicationSupportDirectory();
    theClowder = new Clowder;
    theClowder->initialize( clowderReserve, interruptInformationReserve );
    clowderIsOperating = 1;

    sprintf( theKittenString, MiscStrings[ 399 ], theKittenNumber );

    initializeFileDescriptorsIfNecessary();
    initializeCharTypes();
    initializeSpecialTags();
    initializeDefaultPorts();
    initializeReader();
    initializeMiscChars();

    theHeap = new MemoryManager;
    theHeap->initialize();
    if( isMomCatP ) {
        theHeap->initializeMemoryManager( BigEndianFlag, TRUE, &(lastWorldLoadedName[ 0 ]), MAX_PATHNAME_LENGTH,
                                          *memorySize, *memorySize / 10, *memorySize / 10, *generationSize,
                                          memorySize, 
                                          spaceAReserve, spaceBReserve, scratchStackReserve, gGCScratchReserve );
        }
    else {
        while( ! theClowder->momActiveAt( KITTEN_ACTIVITY_LOADED_WORLD ) )
            usleep( LOAD_WORLD_DELAY );
        theHeap->initializeMemoryManager( BigEndianFlag, TRUE, &(lastWorldLoadedName[ 0 ]), MAX_PATHNAME_LENGTH,
                                          *memorySize, *memorySize / 10, *memorySize / 10, *generationSize,
                                          memorySize,
                                          spaceAReserve, spaceBReserve, scratchStackReserve, gGCScratchReserve );
        }

    theClowder->allocateBitsForProper( *memorySize );
    theClowder->setKittenEvaluatorActivity( KITTEN_ACTIVITY_INITIALIZED_MEMORY );
    theHeap->setClowder( theClowder );
    theWorldHandler = theHeap->getWorldHandler();
    theSimpleTable = theWorldHandler->theSimpleTable;
    initializePortmanteauIndices();

    initializeDataTypeTable( false, false );
    initializeDataTypeList();
    initializeMachine();
    initializeObList( isMomCatP );
    initializeIVR( isMomCatP );
    initializeEnvironmentList( isMomCatP );
    if( isMomCatP ) {
        nFuncs = loadSchemeFunctions( FALSE );
        loadSchemeSpecialForms( FALSE );
        loadSchemeGlobals( FALSE );
        }
    else {
        while( ! theClowder->momActiveAt( KITTEN_ACTIVITY_INITIALIZED_GLOBALS ) )
            usleep( INIT_GLOBALS_DELAY );
        loadKittenizedSchemeGlobals( theKittenNumber );
        }
    theClowder->setKittenEvaluatorActivity( KITTEN_ACTIVITY_INITIALIZED_GLOBALS );

#if USE_KITTEN_LOGS
    openKittenLogFile();
#endif        
    }

/**** errorRecovery ****/

void errorRecovery( Text *reinitializationAction )
{
    reinitializeMachine();
    reinitializeEvents();
    reinitializeLowLevelIO( reinitializationAction );
    initializeDefaultPorts();
    initializeReader();
    theClowder->resetMyStuff();
    assertNotEvaluating();
    theClowder->setMomSaysToEverybody( MOM_SAYS_NOTHING );
    }

/**** resetCleanUp ****/

void resetCleanUp()
{

    }

/**** loadWorldIfPossible ****/

Text *loadWorldIfPossible()
{
    FILE *scratch;
    struct stat sb;
    Text worldFileName[ MAX_PATHNAME_LENGTH ];

#ifdef FLOATS_ARE_64_BIT

#ifdef USE_COCOA_IO

extern Boolean worldFileIDIsGood;
extern int worldFileID;
extern Text worldFilePathName[];

    static Text worldFileTitle[] = "Wraith Scheme.app/Contents/Resources/WraithScheme.64.world";
#else
    static Text worldFileTitle[] = "WraithScheme.64.world";
#endif // USE_COCOA_IO

#else     

    static Text worldFileTitle[] = "WraithScheme.world";

#endif // FLOATS_ARE_64_BIT

#if USE_COCOA_IO
    if( ! worldFileIDIsGood ) {
#else
    if( TRUE ) {
#endif
        strncpy( worldFileName, InitFilePath, sizeof( worldFileName ) );
        strncat( worldFileName, worldFileTitle, sizeof( worldFileName ) - strlen( worldFileName ) );
        if( stat( worldFileName, &sb ) == -1 ) {
            return MiscStrings[ 328 ];
            }
        else  if( (scratch = fopen( worldFileName, "r" )) == NULL ) {
            return MiscStrings[ 327 ];
            }
        else {
            worldFD->fp = scratch;
            strcpy( worldFD->oldPathName, worldFD->pathName );
            strcpy( worldFD->pathName, worldFileName );
            strcpy( worldFD->type, "r" );
            theHeap->loadWorldGuts( worldFD, FALSE );
            return 0;
            }
        }
    else {

#if USE_COCOA_IO

        strncpy( worldFileName, worldFilePathName, sizeof( worldFileName ) );
        strcpy( worldFD->oldPathName, worldFD->pathName );
        strcpy( worldFD->pathName, worldFileName );
        worldFD->fp = fdopen( worldFileID, "r" );
        strcpy( worldFD->type, "r" );
        theHeap->loadWorldGuts( worldFD, FALSE );

#endif

        return 0;
        }
    }

/**** parseArgsAndCheckDefaults ****/

extern char *optarg;
extern int optind;
extern int optopt;
extern int opterr;
extern int optreset;

void parseArgsAndCheckDefaults( int argc, char *argv[], UnsignedLongWord *LocalMemorySize,
                                Boolean *LocalInitFileExists, Text *LocalInitFilePath, Text *LocalInitFileName )
{
    int ch;
    FILE *InitFP;
    Text argv0[ MAX_PATHNAME_LENGTH ];
    UnsignedLongWord originalMemorySize = *LocalMemorySize;

    LocalInitFileName[ 0 ] = 0;

    strcpy( argv0, argv[ 0 ] );

    strcpy( LocalInitFilePath, dirname( argv0 ) );
    if( strcmp( LocalInitFilePath, "." ) == SAME )
        LocalInitFilePath[ 0 ] = 0;
    else
        strcat( LocalInitFilePath, "/" );
    strcat( LocalInitFileName, LocalInitFilePath );
    strcat( LocalInitFileName, MiscStrings[ 210 ] );

    if( (InitFP = fopen( LocalInitFileName, "r" )) != NULL ) {
        fclose( InitFP );
        *LocalInitFileExists = true;
        }
    while( (ch = getopt( argc, argv, "hkm:f:")) != -1 )
        switch( ch ) {
            case 'm':  // a memory size
                if( (*LocalMemorySize = strtoll( optarg, 0, 10 )) <= 250000 ) {
                    fputs( MiscStrings[ 283 ], stderr );
                    exit( 1 );
                    }
                break;
            case 'f':  // path to an initialization file
                if( strcmp( optarg, "" ) == 0 )     // Null file name means "don't use one".
                    *LocalInitFileExists = false;
                else {
#ifndef USE_COCOA_IO
                    if( (InitFP = fopen( optarg, "r" )) == NULL ) {
                        fprintf( stderr, MiscStrings[ 284 ], optarg );
                        exit( 1 );
                        *LocalInitFileExists = false;
                        LocalInitFileName = (Text*)"";
                        }
                    else {
                        fclose( InitFP );
                        *LocalInitFileExists = true;
                        strcpy( LocalInitFileName, optarg );
                        }
#else
                    fatalError( 92 );
#endif
                    }
                break;
            case 'k':  isMomCatP = FALSE;
            case 'h':  // Intentional fall-through.
            case '?':  // Intentional fall-through.
            default:
                fprintf( stderr, MiscStrings[ 281 ], MiscStrings[ 210 ], originalMemorySize  ); 
                fflush( stderr );
                exit( 0 );
                break;
            }
    if( optind != argc ) {
        fputs( MiscStrings[ 282 ], stderr );
        fprintf( stderr, MiscStrings[ 281 ], MiscStrings[ 210 ], originalMemorySize  ); 
        fflush( stderr );
        exit( 0 );
        }
    }

/**** makeKittensExit ****/

void makeKittensExit()
{
    Word counter = 0;
    Text systemBuffer[ABSOLUTE_MAX_KITTENS][ 64 ];

    if( ! isMomCatP )
        theClowder->releaseMyStuff();

    if( isMomCatP )
        for( counter = 1; counter < (Word)theNumberOfKittens; ++counter ) {
            if( theClowder->getKittenPID( counter ) != 0 ) {
                sprintf( systemBuffer[ counter ], "kill -%d %d", SIGKILL, theClowder->getKittenPID( counter ) );
                system( systemBuffer[ counter ] );
                }
            }
    }

/**** cleanUpForExit ****/

void cleanUpForExit()
{
    reinitializeSignalHandlers();
    cleanUpLowLevelIO();
    if( isMomCatP ) {
        theHeap->releaseMemoryFiles();
        theClowder->releaseClowderFiles();
        }        
    }

/**** runSchemeWithArguments ****/

void runSchemeWithArguments( const char *LocalInitFileName, const char *LocalInitFilePath,
                             UnsignedLongWord LocalMemorySize, UnsignedLongWord LocalGenSize, unsigned char LocalInitFileExists )
{
    Word exitValue = 1;
    Word resetState = 0;
    Word nextKitten = 0;
    Text *loadWorldString = 0;

    if( isMomCatP )
        mmapForMomCat();
    else
        mmapForKittens();

    strncpy( InitFileName, LocalInitFileName, sizeof( InitFileName ) - 1 );
    strncpy( InitFilePath, LocalInitFilePath, MAX_PATHNAME_LENGTH - 1 );

    InitFileExists = LocalInitFileExists;

    if( (exitValue = setjmp( ExitBuffer )) != 0 ) {  // Exit from total disaster.
        makeKittensExit();
        if( exitValue == 1 )
            cleanUpForExit();
        exit( --exitValue );
        }
    initialize( &LocalMemorySize, &LocalGenSize );
    if( setjmp( debugBuffer ) )
        fatalError( 133 );
    if( isMomCatP )
        loadWorldString = loadWorldIfPossible();
    worldIsLoaded = 1;
    theClowder->setKittenEvaluatorActivity( KITTEN_ACTIVITY_LOADED_WORLD );

    while( ! GUIInitialized )
        usleep( KITTEN_LAUNCH_DELAY );

    // NOW we can do I/O to the Wraith Scheme window and its panels ...

#ifdef USE_TTY_IO
    tprintf2( &CurrentOutputPort, MiscStrings[ 324 ]);
#endif
#ifdef USE_CURSES_IO
    tprintf2( &CurrentOutputPort, MiscStrings[ 325 ]);
#endif
#ifdef USE_COCOA_IO
    tprintf2( &CurrentOutputPort, MiscStrings[ 337 ]);
#endif

    tprintf2( &CurrentOutputPort, MiscStrings[ 224 ]);
    tprintf2( &CurrentOutputPort, MiscStrings[ 309 ]);
    tprintf2( &CurrentOutputPort, MiscStrings[ 115 ] );
    tprintf2( &CurrentOutputPort, MiscStrings[ 432 ]);
    tprintf2( &CurrentOutputPort, MiscStrings[ 115 ] );
    tprintf3( &CurrentOutputPort, MiscStrings[ 227 ], LocalMemorySize );
    if( loadWorldString != 0 )
        tprintf2( &CurrentOutputPort, loadWorldString );

    if( isMomCatP ) {
        for( nextKitten = 1; nextKitten < (Word)theNumberOfKittens; ++nextKitten ) {
            launchInstance( nextKitten );
            while( theClowder->getKittenGUIActivity( nextKitten ) < KITTEN_GUI_ACTIVITY_COMPLETED_INITIALIZATION ) {
                usleep( KITTEN_LAUNCH_DELAY );
                }
            }
        if( ! useStdinP )
            reportEvent( MAKE_FRONT_EVENT );
        }
#ifdef USE_COCOA_IO

    emitIOInitializationMessages();
    reportInitialEvents();

#endif

    if( (resetState = setjmp( ResetErrorBuffer )) != 0 ) {                 // Recover from mere calamity.
        if( ! isMomCatP ) {
            theClowder->setKittenSays( KITTEN_ACKNOWLEDGES_RESET );
            while( momSaysReset() )
                usleep( RESET_BLOCK_DELAY );
            }
        errorRecovery( 0 );
        emitStatusMessage( MiscStrings[ 384 ] );
        theClowder->setKittenSays( KITTEN_ACKNOWLEDGES_GC_SEQUENCE_ENDING );
        reportEvent( REINITIALIZATION_COMPLETE );
        }
    runTopLoop( InitFileExists );
    }    

/**** runSchemeWithArgcAndArgv -- when they have to be looked up somewhere ...  ****/

int runSchemeWithArgcAndArgv( int argc, char *argv[] )
{

    Boolean LocalInitFileExists = false;
    UnsignedLongWord LocalMemorySize = 20 * MEGABYTE;
    UnsignedLongWord LocalGenerationSize = 1 * MEGABYTE;

    Text LocalInitFileName[ MAX_PATHNAME_LENGTH ];
    Text LocalInitFilePath[ MAX_PATHNAME_LENGTH ];

    parseArgsAndCheckDefaults( argc, argv, &LocalMemorySize, &LocalInitFileExists, LocalInitFilePath, LocalInitFileName );
    runSchemeWithArguments( LocalInitFileName, LocalInitFilePath, LocalMemorySize, LocalGenerationSize, LocalInitFileExists );
    return 0;
    }    

/**** runSchemeWithPathAndMemory ****/

int runSchemeWithPathAndMemory( const char *argv0, UnsignedLongWord memorySizeProvided, UnsignedLongWord generationSizeProvided )
{

    Text LocalInitFilePath[ MAX_PATHNAME_LENGTH ];

#ifdef USE_COCOA_IO
#define SLASHES_TO_BACK_UP 4
#else
#define SLASHES_TO_BACK_UP 4
#endif

    strcpy( LocalInitFilePath, argv0 );   // "<whatever>/Wraith Scheme.app/Contents/MacOS/Wraith Scheme"

    long counter = strlen( LocalInitFilePath );
    int slashesFound = 0;

    while( --counter >= 0 ) {
        if( LocalInitFilePath[ counter ] == '/' )
            ++slashesFound;
        if( slashesFound == SLASHES_TO_BACK_UP )
            break;
        }
    LocalInitFilePath[ counter + 1 ] = 0;

    runSchemeWithArguments( "", LocalInitFilePath, memorySizeProvided, generationSizeProvided, FALSE );
    return 0;
    }

/**** momSaysQuit ****/

Boolean momSaysQuit() { return( theClowder->getMomSays() == MOM_SAYS_EXIT ); }

/**** momSaysReset ****/

Boolean momSaysReset() { return( theClowder->getMomSays() == MOM_SAYS_RESET ); }

/**** momSaysWorldOp ****/

Boolean momSaysWorldOp() { return( theClowder->getMomSays() == MOM_SAYS_WORLD_OPERATION ); }

/**** printfKittens ****/

void printfKittens() {

    UnsignedWord i = 0;

    printf("  Who  On   ObList  Message  Message  Block  Process\n");
    printf("            In Reg   From     From    Number    ID  \n");
    printf("                     Mom      Kitten          Number\n");
    for( i = 0; i < theNumberOfKittens; ++i ) {
        if( i == 0 )
            printf("Mom:  " );
        else
            printf("Kit %d:", i );
        printf("  %d     %d   %6d   %6d   %6lu   %6d\n",
               theClowder->getKittenEvaluatorActivity( i ),
               theClowder->getObListInRegister( i ),
               theClowder->getMomSays( i ),
               theClowder->getKittenSays( i ),
               theClowder->getKittenBlock( i ),
               theClowder->getKittenPID( i )
               );
        }
    }

/**** launchInstance ****/

void launchInstance( Word newKittenNumber ) 
{

#ifdef USE_COCOA_IO

static Text launchScratchBuffer2[ 256 ];
extern char **globalArgv;
extern Boolean setMemoryFromCommandLine;             // In main.mm
extern long commandLineMemorySizeInUse;              // In main.mm

    UnsignedLongWord retryCounter = 0;
    UnsignedLongWord waitCounter = 0;
    Text scratch[MAX_PATHNAME_LENGTH + 1000];

    snprintf( scratch, sizeof( scratch ) - 1, "\"%s\" -k%d ", globalArgv[ 0 ], newKittenNumber );

    if( setCompileDefinesFromCommandLine )
        snprintf( scratch + strlen( scratch ),
                  sizeof( scratch ) - strlen( scratch) - 1,
                  "-c%s ", commandLineCompileDefinesFlag ? "1" : "0");
    if( setMemoryFromCommandLine )
        snprintf( scratch + strlen( scratch ),
                  sizeof( scratch ) - strlen( scratch) - 1,
                  "-m%ld ", commandLineMemorySizeInUse );
    snprintf( scratch + strlen( scratch ),
              sizeof( scratch ) - strlen( scratch) - 1,
              "-z%d ", (Word)(theNumberOfKittens - 1) );

    snprintf( scratch + strlen( scratch ),
              sizeof( scratch ) - strlen( scratch) - 1,
              "&" );

    while( theClowder->getKittenGUIActivity( newKittenNumber - 1 ) < KITTEN_GUI_ACTIVITY_COMPLETED_INITIALIZATION ) {
        usleep( KITTEN_LAUNCH_DELAY );
        }
   while( theClowder->getKittenEvaluatorActivity( newKittenNumber - 1 ) < KITTEN_ACTIVITY_INITIALIZED_GLOBALS ) {
        usleep( KITTEN_LAUNCH_DELAY );
        }
    for( retryCounter = 0; retryCounter < MAX_LAUNCH_RETRIES; ++retryCounter ) {
        system( scratch );
        for( waitCounter = 0; waitCounter < MAX_INIT_WAITS; ++waitCounter ) {
            usleep( KITTEN_LAUNCH_DELAY );
            if( theClowder->getKittenGUIActivity( newKittenNumber ) >= KITTEN_GUI_ACTIVITY_COMPLETED_INITIALIZATION ) {
                while( theClowder->getKittenEvaluatorActivity( newKittenNumber ) < KITTEN_ACTIVITY_INITIALIZED_MEMORY ) {
                    usleep( KITTEN_LAUNCH_DELAY );
                    }
                sprintf( launchScratchBuffer2, MiscStrings[ 402 ], newKittenNumber );
                emitStatusMessage( launchScratchBuffer2 );
                return;
                }
            else if( theClowder->getKittenFailedOnInit( newKittenNumber ) )
                break;
            }
        makeKittensExit();
        useFatalErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 141 ], newKittenNumber );
        fatalError( 153 );
        }
    makeKittensExit();
    useFatalErrorOutput();
    tprintf3( &CurrentOutputPort, MiscStrings[ 141 ], newKittenNumber );
    fatalError( 6 );       

#endif
    }

/**** allKittensAreActive ****/

Boolean allKittensAreActive() { return theClowder->allKittensActiveAt( KITTEN_ACTIVITY_READY_TO_EVAL ); }

/**** reportGUIInitialized ****/

void reportGUIInitialized()
{
    theClowder->setKittenGUIActivity( KITTEN_GUI_ACTIVITY_COMPLETED_INITIALIZATION );
    GUIInitialized = 1;
    }

/**** increaseBlocking ****/

void increaseBlocking() { theClowder->increaseKittenBlock(); }

FILE *kittenLogFile = 0;

/**** openKittenLogFile ****/

void openKittenLogFile() 
{
    Text buffer[ 100 ];

    sprintf( buffer, "/var/tmp/Wraith_Scheme_Kitten_%d_Log.text", theKittenNumber );

    kittenLogFile = fopen( buffer, "a+" );
    fprintf( kittenLogFile,
             "\n################  New run of Wraith Scheme, log file for Kitten %d ################\n\n",
             theKittenNumber
             );
    }

/**** Simple wrapper for fprintf and fflush ... ****/

void markerMessageDEBUG( const Text *message )
{
    printf("Kitten %u: %s\n", theKittenNumber, message );
    }

/**** Wrappers for call to Clowder method.  ****/

void runLocalInterrupt( UnsignedLongWord interruptNo ) { theClowder->runInterruptLocally( interruptNo ); }

void runLocalInterruptForPushbutton( UnsignedLongWord pushButtonNumber )
{ 
    if( pushButtonNumber > N_PUSHBUTTONS ) {
        tprintf3( &CurrentOutputPort, MiscStrings[ 513 ], pushButtonNumber );
        resetError( 343 );
        }
    if( pushbuttonEnabled[ pushButtonNumber ] )
        theClowder->runInterruptLocally( pushbuttonInterruptNumbers[ pushButtonNumber ] );
    }

void runLocalInterruptForBigRedButton()
{
    if( bigRedButtonEnabled )
        theClowder->runInterruptLocally( bigRedButtonInterruptNumber );
    }
