/****************************************************************************************

ManyStrings.c++ -- Copyright (c) by Jay Reynolds Freeman 1990, 1991, 2006

Strings used by Wraith Scheme.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import "LocalTypes.h"

const Text *DataTypeStrings[] =  {
/* 0 */     "Dummy",
/* 1 */     "Unknown Data Type",
/* 2 */     "Byte Block",
/* 3 */     "NOT USED",
/* 4 */     "Exact Short Fixnum",
/* 5 */     "#f",
/* 6 */     "Forwarding Pointer",
/* 7 */     "Pointer to Pair",
/* 8 */     "NOT USED",
/* 9 */     "Inexact Short Fixnum",
/* 10 */    "String",
/* 11 */    "#t",
/* 12 */    "Character",
/* 13 */    "Symbol",
/* 14 */    "Scheme Machine PC",
/* 15 */    "Lambda Pair",
/* 16 */    "Built In Function",
/* 17 */    "Special Form",
/* 18 */    "Vector",
/* 19 */    "Promise",
/* 20 */    "Peculiar Leaf",
/* 21 */    "Peculiar Non-Leaf",
/* 22 */    "Syntax Binding",
/* 23 */    "Macro",
/* 24 */    "Stack Reference",
/* 25 */    "Exact Short Float",
/* 26 */    "Inexact Short Float",
/* 27 */    "Environment Reference",
/* 28 */    "Top-Level Environment Reference",
/* 29 */    "()",
/* 30 */    "Multiple Values",
/* 31 */    "Pointer to Byte Block",
/* 32 */    "Fail",
/* 33 */    "Succeed",
/* 34 */    "Scheme Machine PC Index",
/* 35 */    "Port",
/* 36 */    "Forgettable Object Pointer, remembered",
/* 37 */    "Forgettable Object Pointer, unexpired",
/* 38 */    "Forgettable Object Pointer, forgotten",
/* 39 */    "Memory-Mapped Block",
/* 40 */    "GGC Forwarding Pointer",
    };

const Text *ErrorStrings[] =  {
    /*   0 */     "Dummy",
#ifdef PIXIE_SCHEMES
    /*   1 */     "Even after a full garbage collection, not enough memory:  Try setting some large data structures to '() or to #f, then doing a full garbage collection manually",
#else
    /*   1 */     "Even after a full garbage collection, not enough memory:  Try setting some large data structures to '() or to #f, then doing a full garbage collection manually, or else increase the size of Scheme main memory in the preferences and relaunch, or just quit",
#endif
    /*   2 */     "Too many actual parameters to lambda expression",
    /*   3 */     "Too few actual parameters to lambda expression",
    /*   4 */     "Actual parameters are an improper list",
    /*   5 */     "Implementation: displayByteBlockPointer got pointer to a non byteBlock",
    /*   6 */     "Implementation: Tried to take the cdr of a non-pair, in \"cdrRegardlessOfBlocking\"",
    /*   7 */     "Implementation: In type 2 full garbage collection, to-space is in use.  Attempting reset",
    /*   8 */     "Permanent symbol -- cannot assign",
    /*   9 */     "Permanent symbol -- cannot rebind",
    /*  10 */     "No lexically visible binding of this symbol",
    /*  11 */     "Unbound symbol -- cannot make permanent",
    /*  12 */     "Encountered delimiter \"\\r\" -- expected atom",
    /*  13 */     "Encountered delimiter \"\\n\" -- expected atom",
    /*  14 */     "Encountered delimiter \"\\t\" -- expected atom",
    /*  15 */     "Encountered delimiter -- expected atom",
    /*  16 */     "Expected \")\"",
    /*  17 */     "Read a \".\" not between a car and its cdr",
    /*  18 */     "Read an extra \")\"",
    /*  19 */     "Identifier is too long",
    /*  20 */     "Too many characters in a number",
    /*  21 */     "Bad token -- mistyped boolean?",
    /*  22 */     "Bad token -- mistyped character name?",
    /*  23 */     "String is too long",
    /*  24 */     "Illegal escaped character in string",
    /*  25 */     "Illegal character",
    /*  26 */     "Implementation: \"getToken\"",
    /*  27 */     "Unknown exactness specifier in number",
    /*  28 */     "Unknown radix specifier in number",
    /*  29 */     "Number ends after sign",
    /*  30 */     "Number ends in prefix",
    /*  31 */     "Too many exactnesses in number",
    /*  32 */     "Too many precisions in number",
    /*  33 */     "Too many radices in number",
    /*  34 */     "Unknown radix specifier in number",
    /*  35 */     "Meaningless character after '#', possibly in the prefix of a number",
    /*  36 */     "Number ends prematurely (no digits)",
    /*  37 */     "Digit(s) expected",
    /*  38 */     "Digit(s), '.', or '/' expected",
    /*  39 */     "'#'s before radix point and digits after",
    /*  40 */     "More than one '.' or '/' in a number",
    /*  41 */     "Meaningless character in number: Expected '.', '/' or exponent",
    /*  42 */     "Meaningless character(s) after the end of a number",
    /*  43 */     "Expected exponent",
    /*  44 */     "Number ends in exponent",
    /*  45 */     "No numerator",
    /*  46 */     "No denominator",
    /*  47 */     "Denominator is zero",
    /*  48 */     "Tried to list cars of a list, not all of whose elements are pairs",
    /*  49 */     "Tried to list cars of an improper list of lists",
    /*  50 */     "Tried to list cdrs of a list, not all of whose elements are pairs",
    /*  51 */     "Tried to list cdrs of an improper list of lists",
    /*  52 */     "Tried to display source code for a non-interpreted lambda expression",
    /*  53 */     "Lambda list item is not a symbol",
#ifdef PIXIE_SCHEMES
    /*  54 */     "After a generational garbage collection, even assisted by full garbage collection, insufficient memory to continue:  Try setting some large data structures to '() or to #f, then doing a full garbage collection manually",
#else
    /*  54 */     "After a generational garbage collection, even assisted by full garbage collection, insufficient memory to continue:  Try setting some large data structures to '() or to #f, then doing a full garbage collection manually, or else increase the size of Scheme main memory in the preferences and relaunch, or just quit",
#endif
    /*  55 */     "Cannot write to a file",
    /*  56 */     "Cannot read from a file",
    /*  57 */     "Cannot get information about a file",
    /*  58 */     "Tried to make vector with negative number of elements",
    /*  59 */     "Argument to \"reverse\" is not a proper list",
    /*  60 */     "Second argument to \"assoc\" is not a proper list",
    /*  61 */     "Second argument to \"assoc\" is not an association list",
    /*  62 */     "Second argument to \"assq\" is not a proper list",
    /*  63 */     "Second argument to \"assq\" is not an association list",
    /*  64 */     "Second argument to \"assv\" is not a proper list",
    /*  65 */     "Second argument to \"assv\" is not an association list",
    /*  66 */     "Unknown procedure type in \"call-with-current-continuation\"",
    /*  67 */     "Unknown built-in function type in \"call-with-current-continuation\"",
    /*  68 */      "\"Call-with-current-continuation\" procedure must accept a single argument",
    /*  69 */     "Attempt to raise negative number to non-integer power",
    /*  70 */     "Attempt to divide by zero",
    /*  71 */     "Improper list in \"append\"",
    /*  72 */     "List too short in \"list-ref\"",
    /*  73 */     "Need exact integer in \"list-ref\"",
    /*  74 */     "Need non-negative argument to \"list-tail\"",
    /*  75 */     "Need exact integer in \"list-tail\"",
    /*  76 */     "List too short in \"list-tail\"",
    /*  77 */     "Second argument to \"member\" is not a proper list",
    /*  78 */     "Second argument to \"memq\" is not a proper list",
    /*  79 */     "Second argument to \"memv\" is not a proper list",
    /*  80 */     " -- unknown data type\n",
    /*  81 */     "Integer exceeds length of vector in \"vector-ref\"",
    /*  82 */     "Need exact integer in \"vector-set!\"",
    /*  83 */     "Failed to lock in \"calculatePortableObjectSizeAux\"",
    /*  84 */     "Integer exceeds length of vector in \"vector-set!\"",
    /*  85 */     "Need exact integer in \"vector-set!\"",
    /*  86 */     "Integer out of range for conversion to character",
    /*  87 */     "",    // NOT USED
    /*  88 */     "Integer exceeds length of string in \"string-ref\"",
    /*  89 */     "Need exact integer in \"string-ref\"",
    /*  90 */     "Nonnegative integer required for \"string-set!\"",
    /*  91 */     "Integer exceeds length of string in \"string-set!\"",
    /*  92 */     "Need exact integer in \"string-set!\"",
    /*  93 */     "End exceeds length of string in \"substring\"",
    /*  94 */     "Attempt to take the remainder or modulus of this number when divided by zero",
    /*  95 */     "Start exceeds end in \"substring\"",
    /*  96 */     "Need exact integer for \"substring\" end",
    /*  97 */     "Need exact integer for \"substring\" start",
    /*  98 */     "Unescaped \"newline\" in string",
    /*  99 */     "The last argument to \"apply\" must be a list",
    /* 100 */     "Tried to perform an arithmetic comparison with a nan",
    /* 101 */     "Argument to \"make-string\" must be non-negative integer",
    /* 102 */     "Need exact integer for \"make-string\"",
    /* 103 */     "Tried to perform an arithmetic comparison with two infs of the same sign",
    /* 104 */     "Need exact integer for \"make-vector\"",
    /* 105 */     "Cannot create a file",
    /* 106 */     "\"begin\" starting an empty list",
    /* 107 */     "Cannot flush a file",
    /* 108 */     "Argument is out of range for a kitten number",
    /* 109 */     "Cannot open a file",
    /* 110 */     "Cannot set information about a file",
    /* 111 */     "Cannot get catalog information about a directory",
    /* 112 */     "Cannot close a file",
    /* 113 */     "Cannot flush a volume",
    /* 114 */     "Cannot delete contents of an existing file",
    /* 115 */     "Cannot write saved world",
    /* 116 */     "Cannot open file for loading world",
    /* 117 */     "Cannot load saved world",
    /* 118 */     "World to be loaded is too large",
    /* 119 */     "Cannot use that world in this version of the application (\"wrong table size\")",
    /* 120 */     "\"Cond\" must be followed by a list of clauses",
    /* 121 */     "Wrong number of arguments to \"define\"",
    /* 122 */     "Attempt to use \"define\" when not at top level",
    /* 123 */     "First argument to \"define\" must be a symbol, or a list (<symbol> ...)",
    /* 124 */     "Directory name is too long",
    /* 125 */     "Cannot learn about default directory",
    /* 126 */     "Cannot open file for saving world",
    /* 127 */     "Empty lambda expression",
    /* 128 */     "Cannot get information about a directory",
    /* 129 */     "Cannot get information about a working directory",
    /* 130 */     "Cannot open a working directory",
    /* 131 */     "Cannot close a working directory",
    /* 132 */     "Volume name is too long",
    /* 133 */     "Cannot get information about a volume",
    /* 134 */     "Cannot find startup volume",
    /* 135 */     "Wrong number of arguments to \"c::quote-simple\" -- should be 1",
    /* 136 */     "Wrong number of arguments to \"quote\" -- should be 1",
    /* 137 */     "Wrong number of arguments to \"set!\" -- should be 2",
    /* 138 */     "First argument to \"set!\" must be a symbol",
    /* 139 */     "First argument to \"set!\" is a symbol but is not bound",
    /* 140 */     "o",
    /* 141 */     "o",
    /* 142 */     "Not a procedure or a special form",
    /* 143 */     "Too many arguments for built-in function (in \"apply\")",
    /* 144 */     "Unknown function type",
    /* 145 */     "Attempt to apply a non-procedure",
    /* 146 */     "Reader encountered an unquoted empty list",
    /* 147 */     "Wrong number of arguments",
    /* 148 */     "Not enough arguments",
    /* 149 */     "Too many arguments",
    /* 150 */     "Built-in function with too many arguments",
    /* 151 */     "Argument has improper type for this operation",
    /* 152 */     "Marked argument has improper type for this operation",
    /* 153 */     "Marked arguments have improper types for this operation",
    /* 154 */     "car of \"fmt\" list is not a format-descriptor",
    /* 155 */     "Unknown kind of built-in function",
    /* 156 */     "No way to evaluate this object",
    /* 157 */     "Unbound symbol -- cannot assign a value to it",
    /* 158 */     "Improper list of items to evaluate (last item is shown)",
    /* 159 */     "Argument out of range for \"acos\"",
    /* 160 */     "Argument out of range for \"asin\"",
    /* 161 */     "Number out of range for coercion to a 32-bit signed integer",
    /* 162 */     "Attempt to take the quotient of this number by zero",
    /* 163 */     "Argument out of range for \"log\"",
    /* 164 */     "Argument out of range for \"sqrt\"",
    /* 165 */     "Implementation: Tried to take the car of a non-pair",
    /* 166 */     "Implementation: Tried to take the cdr of a non-pair",
    /* 167 */     "Implementation: Saw 0x3000 (extra cdr code) in \"cdr\"",
    /* 168 */     "Unbound symbol",
    /* 169 */     "Implementation: Argument to \"hashString\" is not a string",
    /* 170 */     "Suspiciously large number of characters",
    /* 171 */     "Tried to take the length of an improper list",
    /* 172 */     "Cannot open file",
    /* 173 */     "Implementation: Argument to \"setCar\" is not a pair",
    /* 174 */     "Implementation: Argument to \"setCdr\" is not a pair",
    /* 175 */     "Implementation: Saw 0x3000 (extra cdr code) in \"setCar\"",
    /* 176 */     "Destination string is too small for a string-insertion",
    /* 177 */     "Implementation: Argument to \"stringRef\" is not a string",
    /* 178 */     "Tried to reference string at negative index",
    /* 179 */     "Tried to reference string beyond its length",
    /* 180 */     "Implementation: Argument to \"stringSet\" is not a string",
    /* 181 */     "Tried to set string at negative index",
    /* 182 */     "Tried to set string beyond its length",
    /* 183 */     "Implementation: Argument to \"vectorRef\" is not a vector",
    /* 184 */     "Tried to reference vector at negative index",
    /* 185 */     "Tried to reference vector beyond its length",
    /* 186 */     "Implementation: Argument to \"vectorSet\" is not a vector",
    /* 187 */     "Tried to set vector at negative index",
    /* 188 */     "Tried to set vector beyond its length",
    /* 189 */     "Too many open files",
    /* 190 */     "Implementation: Bad Scheme file descriptor",
    /* 191 */     "File name is too long",
    /* 192 */     "Cannot evaluate an unquoted empty list",
    /* 193 */     "Unexpected end of input, after a \"#\"",
    /* 194 */     "Unexpected end of input, after \"#\\\"",
    /* 195 */     "Unexpected end of input, within a string",
    /* 196 */     "Unexpected end of input, after a \"\\\" in a string",
    /* 197 */     "Unexpected end of input, in an S-expression",
    /* 198 */     "First argument to \"define\" must be a symbol, or a list (<symbol> )",
    /* 199 */     "Implementation: Saw 0x3000 (extra cdr code) in \"cdrRegardlessOfBlocking\"",
    /* 200 */     "Implementation: Argument to \"setCarWithPairAlreadyBlockedKeepBlock\" is not a pair",
    /* 201 */     "Tried to print too many characters at once.",
    /* 202 */     "No lexically visible binding of this symbol in the given environment",
    /* 203 */     "Failed to lock in \"c::block\"",
    /* 204 */     "Absolute value of exponent is too large",
    /* 205 */     "Exponent has too many digits (at most 10 are allowed)",
    /* 206 */     "Failed to lock in \"e::block-symbol-binding\"",
    /* 207 */     "Implementation: Saw 0x3000 (extra cdr code) in \"speculativeCdr\"",
    /* 208 */     "Seeking a file descriptor associated with a port on another kitten",
    /* 209 */     "Wants to use the stream associated with  a port on another kitten",
    /* 210 */     "Failed to lock in \"make-string\" or \"string-fill!\"",
    /* 211 */     "Failed to lock in \"make-vector\" or \"vector-fill!\"",
    /* 212 */     "Only the MomCat may load a world",
    /* 213 */     "Only the MomCat may save a world",
    /* 214 */     "Implementation: Saw 0x3000 (extra cdr code) in \"setCdr\"",
    /* 215 */     "Implementation: Tried to take the car of a non-pair, in carRegardlessOfBlocking",
    /* 216 */     "Socket creation failed, see console log for details",
    /* 217 */     "Socket bind failed, see console log for details",
    /* 218 */     "Socket listen failed, see console log for details",
    /* 219 */     "Socket accept failed, see console log for details",
    /* 220 */     "Unable to open socket for read",
    /* 221 */     "Number has a fractional part, cannot format as integer",
    /* 222 */     "Number requires too many characters to print",
    /* 223 */     "Failure to unlock",
    /* 224 */     "Calculating lcm, temporary value too large",
    /* 225 */     "Cannot set print lengths and depths to less than three",
    /* 226 */     "Define with more than two arguments must have a list as its first one",
    /* 227 */     "Permanent symbol -- cannot use as formal parameter to lambda expression",
    /* 228 */     "Implementation: Restoring too many bytes from main store to the continuation",
    /* 229 */     "Implementation: Restoring too many bytes from main store to the stack",
    /* 230 */     "Implementation: Bad number of arguments left to load in \"eval\"",
    /* 231 */     "Argument lists to \"for-each\" have different lengths",
    /* 232 */     "Argument lists to \"map\" have different lengths",
    /* 233 */     "Implementation: Tried to take the car of a non-pair, in \"carCdrMixForCSPushProperList\"",
    /* 234 */     "Macro body did not evaluate to a procedure",
    /* 235 */     "First element of list is not a macro",
    /* 236 */     "Incomplete macro definition",
    /* 237 */     "First argument to \"macro\" must be a symbol",
    /* 238 */     "Lambda expression with empty body",
    /* 239 */     "Lambda expression with malformed body",
    /* 240 */     "Failed to lock in \"tellKittenNCode\" (where N is the number of the kitten reporting the error)",  // MAYBE UNUSED
    /* 241 */     "Failed to lock in \"makeImmutable\"",
    /* 242 */     "Failed to lock in \"car\"",
    /* 243 */     "\"begin\" starting an improper list",
    /* 244 */     "Implementation: Wrong number of args for \"quoteSimpleEvalCode\" -- expect 1",
    /* 245 */     "Attempt to apply \"set-car!\" to a constant pair",
    /* 246 */     "Attempt to apply \"set-cdr!\" to a constant pair",
    /* 247 */     "Attempt to apply \"string-fill!\" to a constant string",
    /* 248 */     "Attempt to apply \"string-set!\" to a constant string",
    /* 249 */     "Attempt to apply \"vector-fill!\" to a constant vector",
    /* 250 */     "Attempt to apply \"vector-set!\" to a constant vector",
    /* 251 */     "Encountered (); expected macro, special form or procedure call",
    /* 252 */     "Attempt to change current directory to a directory that does not exist",
    /* 253 */     "One or more open ports must be closed before saving this world",
    /* 254 */     "Implementation:  \"restoreInputDescriptor\" called with something that is not an input port in this kitten",
    /* 255 */     "Implementation:  Unable to open a file or pathname",
    /* 256 */     "Failed to lock in \"e::block-any-binding\"",
    /* 257 */     "Implementation: Bad radix in \"getRequiredDigitsShortFloat\"",
    /* 258 */     "Unable to open file for memory mapping",
    /* 259 */     "Implementation: Bad radix in \"constructNumber\"",
    /* 260 */     "Attempt to display a number that is not an integer with a radix other than ten",
    /* 261 */     "Attempt to display a number using an exponent with a radix other than ten",
    /* 262 */     "Radix point in non-decimal number",
    /* 263 */     "Exponent in non-decimal number",
    /* 264 */     "Meaningless character in non-decimal number (radix points and exponents are not allowed)",
    /* 265 */     "Exponent in number expressed as fraction",
    /* 266 */     "Implementation: Bad radix in \"xecvt\"",
    /* 267 */     "Number has both an explicit \"exact\" prefix and embedded \"#\"s",
    /* 268 */     "\"VERITAS AETERNA -- do not setq T!\" -- MIT MacLisp",
    /* 269 */     "\"NIHIL EX NIHIL -- do not setq nil!\" -- MIT MacLisp",
    /* 270 */     "Bad token -- expected a third '.'",
    /* 271 */     "Premature end of input in token -- expected a third '.'",
    /* 272 */     "Unexpected end of input, after \"'\")",
    /* 273 */     "Unexpected white space, after \"'\"",
    /* 274 */     "Unexpected end of input, after \"`\"",
    /* 275 */     "Unexpected white space, after \"`\"",
    /* 276 */     "Unexpected end of input, after \",\"",
    /* 277 */     "Unexpected white space, after \",\"",
    /* 278 */     "Unexpected end of input, after \",@\"",
    /* 279 */     "Unexpected white space, after \",@\"",
    /* 280 */     "Unexpected newline, after \"#\\\"",
    /* 281 */     "list->string encountered a list element that is not a character",
    /* 282 */     "Cannot open -- does not appear to be a file",
    /* 283 */     "Failed to lock the input queue for this kitten, in \"tellKittenCode\"",
    /* 284 */     "Scheme continuation near overflow, may have overflowed already... attempting reset",
    /* 285 */     "Unable to map memory due to insufficient space",
    /* 286 */     "This kitten tried to unlock an object locked by another kitten",
    /* 287 */     "Trying to change tag of locked object",
    /* 288 */     "Nonnegative integer required for \"c::environment-n-down\"",
    /* 289 */     "Need exact integer in \"c::environment-n-down\"",
    /* 290 */     "Scheme stack near overflow, may have overflowed already:  Perhaps a recursive procedure has recursed too deeply ... attempting reset",
    /* 291 */     "Bad token -- mistyped logic value?",
    /* 292 */     "Evaluator signaled to halt by another kitten",
    /* 293 */     "Failed to lock in \"e::set-tag!\"",
    /* 294 */     "Permanent symbol -- cannot assign (in blocking assign)",
    /* 295 */     "Failed to lock in \"cdr\"",
    /* 296 */     "Assertion failed",
    /* 297 */     "Failed to lock in \"setCar\"",
    /* 298 */     "Failed to lock in \"setCarWithRAlreadyBlockedKeepBlock\"",
    /* 299 */     "Failed to lock in \"setCdr\"",
    /* 300 */     "Failed to lock in \"eval\"",
    /* 301 */     "Failed to lock in \"intern\"",
    /* 302 */     "Failed to lock in \"getSymbol\"",
    /* 303 */     "Failed to lock in \"boundStringP\"",
    /* 304 */     "Cannot write to file to be used for memory mapping",
    /* 305 */     "Failed to lock in \"lookUpBindingPair\"",
    /* 306 */     "Failed to lock in \"lookUpBindingPairInEnvironment\"",
    /* 307 */     "Tried to unlock something that was not locked",
    /* 308 */     "Failed to lock in \"properListPMC\", while coloring list",
    /* 309 */     "Failed to lock in \"properListPMC\", while uncoloring list",
    /* 310 */     "Failed to lock in \"properListPMC\", while coloring list, second location",
    /* 311 */     "Failed to lock in \"properListPMC\", while uncoloring list, second location",
    /* 312 */     "Failed to transfer lock in \"setCarWithRAlreadyBlockedKeepBlock\"",
    /* 313 */     "Miscellaneous Error",  // For development and debugging use, should not be in released code.
    /* 314 */     "Memory mapping (function mmap) failed",
    /* 315 */     "No memory for the scratch buffer used to initialize shared memory",
    /* 316 */     "Memory-mapped block has not yet been mapped for this kitten -- cannot read value",
    /* 317 */     "Memory-mapped block has not yet been mapped for this kitten -- cannot write value",
    /* 318 */     "Range error accessing memory-mapped block",
    /* 319 */     "Alignment error accessing memory-mapped block -- location must be an even multiple of data size (in bytes)",
    /* 320 */     "Integer out of range for 64-bit fixnum",
    /* 321 */     "Memory-mapped block has not yet been mapped for this kitten -- cannot read or write value",
    /* 322 */     "Cannot use that world in this version of the application (\"wrong world type\")",
    /* 323 */     "Cannot use that world in this version of the application (\"wrong float size\")",
    /* 324 */     "Failed to lock in \"duplicateStructureAux\"",
    /* 325 */     "Bad token -- mistyped non-printing object?",
    /* 326 */     "MomCat detected low memory after garbage collection",
    /* 327 */     "That kitten detected low memory after garbage collection",
    /* 328 */     "Attempt to use a non-applicative procedure or special form during applicative programming",
#ifdef PIXIE_SCHEMES
    /* 329 */     "After a generational garbage collection, even assisted by full garbage collection, insufficient memory:  Try setting some large data structures to '() or to #f, then doing a full garbage collection manually",
#else
    /* 329 */     "After a generational garbage collection, even assisted by full garbage collection, insufficient memory:  Try setting some large data structures to '() or to #f, then doing a full garbage collection manually, or else increase the size of Scheme main memory in the preferences and relaunch, or just quit",
#endif
#ifdef PIXIE_SCHEMES
    /* 330 */     "After a generational garbage collection, even assisted by a full garbage collection, not enough memory to continue:  Try setting some large data structures to '() or to #f, then doing a full garbage collection manually",
#else
    /* 330 */     "After a generational garbage collection, even assisted by a full garbage collection, not enough memory to continue:  Try setting some large data structures to '() or to #f, then doing a full garbage collection manually, or else increase the size of Scheme main memory in the preferences and relaunch, or just quit",
#endif
#ifdef PIXIE_SCHEMES
    /* 331 */     "After a generational garbage collection, even assisted by a full garbage collection, not enough memory:  Try setting some large data structures to '() or to #f, then doing a full garbage collection manually",
#else
    /* 331 */     "After a generational garbage collection, even assisted by a full garbage collection, not enough memory:  Try setting some large data structures to '() or to #f, then doing a full garbage collection manually, or else increase the size of Scheme main memory in the preferences and relaunch, or just quit",
#endif
#ifdef PIXIE_SCHEMES
    /* 332 */     "After a generational garbage collection, even assisted by a full garbage collection, insufficient memory to continue:  Try setting some large data structures to '() or to #f, then doing a full garbage collection manually",
#else
    /* 332 */     "After a generational garbage collection, even assisted by a full garbage collection, insufficient memory to continue:  Try setting some large data structures to '() or to #f, then doing a full garbage collection manually, or else increase the size of Scheme main memory in the preferences and relaunch, or just quit",
#endif
    /* 333 */     "Wrong number of arguments to \"c::set!\" -- should be 2",
    /* 334 */     "First argument to \"c::set!\" must be a symbol",
    /* 335 */     "Failed to lock in \"c::set-tag!\"",
    /* 336 */     "Cannot write edit session data",
    /* 337 */     "Cannot open file for writing edit session data",
    /* 338 */     "Cannot read edit session data",
    /* 339 */     "Cannot open file for reading edit session data",
    /* 340 */     "This pushbutton is not enabled, so it does not have an interrupt number",
    /* 341 */     "The big red button is not enabled, so it does not have an interrupt number",
    /* 342 */     "Interrupt number out of range",
    /* 343 */     "Pushbutton number out of range",
    };
UnsignedLongWord NumberOfErrorStrings = 343;


const Text *FatalErrorStrings[] = {
    /*   0 */    "Dummy",
    /*   1 */     "No memory for \"allocateLow\"",
    /*   2 */     "No memory for a memory space",
    /*   3 */     "Implementation: Unknown data descriptor",
    /*   4 */     "No memory for data type table",
    /*   5 */     "Implementation: \"start\" -> ??",
    /*   6 */     "Sorry:  Even after many retries, operating system could not allocate shared memory at necessary address.  Please try again (relaunch)",
    /*   7 */     "Implementation: Saw 0x3000 in \"cdrCodedBlockSize\"",
    /*   8 */     "Implementation: \"cdrCodedBlockSize\"",
    /*   9 */     "Out of range",
    /*  10 */     "Implementation: In \"cdr\" -- bad cdr code",
    /*  11 */     "Implementation: In \"setCdr\" -- bad cdr code",
    /*  12 */     "No memory for array of error strings",
    /*  13 */     "No memory for the error strings",
    /*  14 */     "Implementation: Unused cdr code",
    /*  15 */     "Implementation: \"cdrCodedBlock\"",
    /*  16 */     "No memory for array of miscellaneous strings",
    /*  17 */     "No memory for all the miscellaneous strings",
    /*  18 */     "Implementation: \"constructNumber\"",
    /*  19 */     "No memory for a file name",
    /*  20 */     "No memory for a volume name",
    /*  21 */     "No memory for a file access type",
    /*  22 */     "Cannot open \"Dev:Null\"",
    /*  23 */     "Implementation: Too many args to \"tprint\"",
    /*  24 */     "No memory to store type-checking results",
    /*  25 */     "Implementation: #1 in \"evalBuiltIn\"",
    /*  26 */     "Implementation: #2 in \"evalBuiltIn\"",
    /*  27 */     "No memory for a temporary string",
    /*  28 */     "Implementation: Parsing a \"do\"",
    /*  29 */     "Cannot find enough memory",
    /*  30 */     "No memory for a big dummy buffer",
    /*  31 */     "Implementation: In \"cdrC\" -- bad cdr code",
    /*  32 */     "Cannot read entire world to be loaded",
    /*  33 */     "Implementation: Inconsistent world size",
    /*  34 */     "No memory for a table",
    /*  35 */     "This is only a test -- do not panic",
    /*  36 */     "Scheme stack near overflow, may have overflowed already:  Perhaps a recursive procedure has recursed too deeply..",
    /*  37 */     "Implementation: \"tooManyCharsHandler\"",
    /*  38 */     "Cannot print documents",
    /*  39 */     "Can only load one document at a time",
    /*  40 */     "Cannot get information about startup working directory",
    /*  41 */     "Cannot find startup volume",
    /*  42 */     "Implementation: Cannot find a help string",
    /*  43 */     "Implementation: Misaligned memory request",
    /*  44 */     "Implementation: Menu item not found",
    /*  45 */     "Implementation: \"constructShortFloatWithDigitsAndSharps\"",
    /*  46 */     "Implementation: Pointer in heap points out of heap after world load",
    /*  47 */     "Implementation: Top-level E pointer points out of heap after world load",
    /*  48 */     "Implementation: O pointer points out of heap after world load",
    /*  49 */     "Unable to determine the working directory (\"getcwd\" failed)",
    /*  50 */     "Implementation: Unknown event",
    /*  51 */     "No memory for a text input buffer",
    /*  52 */     "\nTry again with less memory requested",
    /*  53 */     "Implementation: In \"properListPMC\" -- bad cdr code",
    /*  54 */     "Implementation: Fatal error in numeric formatting, position 1",
    /*  55 */     "Implementation: Fatal error in numeric formatting, position 2",
    /*  56 */     "Implementation: Fatal error in numeric formatting, position 3",
    /*  57 */     "Implementation: Fatal error in numeric formatting, position 4",
    /*  58 */     "Implementation: Fatal error in numeric formatting, position 5",
    /*  59 */     "Implementation: Bad combination of recursion bits, in \"markGarbageFromReg\"",
    /*  60 */     "Implementation: Bad switch in \"add\"",
    /*  61 */     "Implementation: IVR pointer points out of heap after world load",
    /*  62 */     "Implementation: The \"space\" portion of a generational memory space is incorrectly aligned",
    /*  63 */     "Implementation: The \"allocateMax\" portion of a memory space is incorrectly aligned",
    /*  64 */     "Implementation: Generational garbage collector found wrong number of bytes",
    /*  65 */     "Implementation: Saw 0x3000 (extra cdr code) in \"calculatePortableObjectSizeAux\"",
    /*  66 */     "Implementation: Error found while checking a register",
    /*  67 */     "Different number of locks before and after generational garbage collection",
    /*  68 */     "Implementation: \"replacePointersInSpaceGenGGC\" did not find a forwarding pointer where expected",
    /*  69 */     "Implementation: Pointer points into non-tenured space but bit saying so is not set.",
    /*  70 */     "Implementation: Bad combination of recursion bits, in \"markGarbageFromRegGGC\"",
    /*  71 */     "Implementation: Generational garbage collector counts of recursion bits set and cleared do not match",
    /*  72 */     "Insufficient memory for a scratch space used in generational garbage collection",
    /*  73 */     "Implementation: In \"calculatePortableObjectSizeAux\" -- bad cdr code",
    /*  74 */     "Implementation: \"updatePointersInCopiedDownArea\" did not find a ggc forwarding pointer where expected",
    /*  75 */     "Implementation: Bad switch in \"divide\"",
    /*  76 */     "Implementation: Bad switch in \"gcd\"",
    /*  77 */     "Implementation: Bad switch in \"lcm\"",
    /*  78 */     "Implementation: Bad switch in \"multiply\"",
    /*  79 */     "Implementation: Bad switch in \"remainder2\"",
    /*  80 */     "Implementation: Bad switch in \"subtract\"",
    /*  81 */     "Implementation: Bad switch in \"numEqualPMC\"",
    /*  82 */     "Implementation: Bad switch in \"numGreaterPMC\"",
    /*  83 */     "Implementation: Bad switch in \"numGreaterEqualPMC\"",
    /*  84 */     "Implementation: Bad switch in \"numLessPMC\"",
    /*  85 */     "Implementation: Bad switch in \"numLessEqualPMC\"",
    /*  86 */     "Try again with a window at least 72 columns wide by 20 lines high",
    /*  87 */     "Implementation: Bad switch in \"quotient2\"",
    /*  88 */     "Implementation: Bad switch in \"pollControllerOnce\"",
    /*  89 */     "Implementation: Generational garbage collection copied the wrong number of bytes",
    /*  90 */     "Unknown long float size", 
    /*  91 */     "Implementation: Could not determine local architecture",
    /*  92 */     "Implementation: \"parseArgsAndCheckDefaults\" called from Cocoa implementation",
    /*  93 */     "Implementation: Failed to allocate \"SpaceC\"",
    /*  94 */     "Implementation: Garbage collector found bad cdr code, location 1",
    /*  95 */     "Implementation: Garbage collector found bad direction",
    /*  96 */     "Implementation: Garbage collector allocated too large a block",
    /*  97 */     "Implementation: Garbage collector found bad cdr code, location 2",
    /*  98 */     "Implementation: Generational garbage collector inner routine called while running",
    /*  99 */     "Implementation: Garbage collector found car-cdr-next non-garbage followed by non-forwarding-pointer garbage",
    /* 100 */     "Implementation: Garbage collector found bad cdr code, location 4",
    /* 101 */     "Implementation: Garbage collector scanned cdr-coded block past the end of Scheme memory",
    /* 102 */     "Implementation: Garbage collector found non-garbage forwarding pointer pointing to garbage",
    /* 103 */     "Implementation: Garbage collector found unexpected non-forwarding-pointer",
    /* 104 */     "Implementation: Garbage collector found wrong number of objects",
    /* 105 */     "Implementation: Garbage collector trying to copy an object already copied, in \"copyCCNextSection\"",
    /* 106 */     "Implementation: Garbage collector scanned cdr-coded block past the end of Scheme memory, second position",
    /* 107 */     "Implementation: Garbage collector encountered an object already copied, in an allegedly uncopied block",
    /* 108 */     "Implementation: Garbage collector found car-cdr-next non-garbage followed by non-forwarding-pointer garbage, second position",
    /* 109 */     "Implementation: Garbage collector found bad cdr code, location 3",
    /* 110 */     "Implementation: Garbage collector counts of recursion bits set and cleared do not match",
    /* 111 */     "Implementation: Garbage collector routine \"spliceOutGarbageFromVector\" called with a non-vector",
    /* 112 */     "Implementation: Error found while checking memory space",
    /* 113 */     "Implementation: Garbage collector trying to copy an object already copied, in \"copyNonCar\"",
    /* 114 */     "Implementation: Garbage collector trying to copy an object already copied, in \"copyCCNextSection\"",
    /* 115 */     "Implementation: Garbage collector trying to copy an object already copied, in \"copyBlockLow\"",
    /* 116 */     "Implementation: Garbage collector found copied forwarding pointer at end of car-cdr-next block",
    /* 117 */     "Implementation: Garbage collector found car-cdr-next block terminated by garbage",
    /* 118 */     "Implementation: \"Can't happen\" error in garbage collector, location 1",
    /* 119 */     "Implementation: \"Can't happen\" error in garbage collector, location 2",
    /* 120 */     "Beings from my planet of origin have at last come to take me home",
    /* 121 */     "Implementation: Too many kittens",
    /* 122 */     "You have requested too many kittens",
    /* 123 */     "No memory for a scratch buffer used to initialize shared memory",
    /* 124 */     "Error writing to shared memory, during initialization.\n\nYou might try using a smaller main memory size.  To do so, you may use the Wraith Scheme preferences window right now, from the Interpreter menu, before pressing the \"Sorry!\" button",
    /* 125 */     "Implementation: Error with memory map (function mmap) of shared memory.",
#ifdef PIXIE_SCHEME_III
    /* 126 */     "Insufficient memory for the Pixie Scheme III stack.",
#else
#ifdef PIXIE_SCHEMES
    /* 126 */     "Insufficient memory for the Pixie Scheme II stack.",
#else
    /* 126 */     "Insufficient memory for the Wraith Scheme stack.\n\nYou might try using a smaller main memory size.  To do so, you may use the Wraith Scheme preferences window right now, from the Interpreter menu, before pressing the \"Sorry!\" button",
#endif
#endif
    /* 127 */     "Implementation: Error opening file for memory map (function mmap) of shared memory.",
    /* 128 */     "Implementation: Unable to create an array for locks",
    /* 129 */     "Implementation: Kitten could not open the clowder file",
    /* 130 */     "Implementation: Error writing to shared memory for clowder file",
    /* 131 */     "Implementation: Error with memory mapping (function mmap) of clowder file",
    /* 132 */     "Implementation: MomCat could not create clowder file",
    /* 133 */     "Implementation: Jumped to debugBuffer before the latter was initialized",
    /* 134 */     "Implementation: MomCat could not create memory space file",
    /* 135 */     "Implementation: In \"speculativeCdr\" -- bad cdr code",
    /* 136 */     "Implementation: Range error for Scheme machine pc index table",
    /* 137 */     "Implementation: This Scheme machine pc is not in the index table",
    /* 138 */     "Implementation: In \"carCdrMixForCSPushProperList\" -- bad cdr code",
    /* 139 */     "Implementation: MomCat could not create scratch stack file",
    /* 140 */     "Implementation: Need longer buffer in \"kittenizeString\"",
    /* 141 */     "Implementation: Attempted to copy one memory space to another of different length",
    /* 142 */     "Implementation: In \"cdrWithoutBlockingAnything\" -- bad cdr code",
    /* 143 */     "Operating system unwilling to initialize shared memory to necessary address -- exit from the MomCat and try again",
    /* 144 */     "Implementation: Unrecognized message from mom, in \"kittenHandleMessages\"",
    /* 145 */     "Implementation: MomCat could not create interrupt information file",
    /* 146 */     "Implementation: Kitten attempted to reduce kitten-block count below zero",
    /* 147 */     "Implementation: In \"cdrRegardlessOfBlocking\" -- bad cdr code",
    /* 148 */     "Implementation: \"getInputDescriptor\" called with something not a port",
    /* 149 */     "Implementation: \"getOutputDescriptor\" called with something not a port",
    /* 150 */     "Implementation: \"getPortFD\" called with something not a port",
    /* 151 */     "Implementation: \"getPortFilePointer\" called with something not a port",
    /* 152 */     "Implementation: \"fatalError\" called recursively",
    /* 153 */     "Sorry:  Even after many retries, kitten did not complete initialization.  Please try again (relaunch)",
    /* 154 */     "Implementation: Memory spaces are too big for reserved areas",
    /* 155 */     "Implementation: Different number of locks before and after garbage collection",
    /* 156 */     "Implementation: Scratch stack is too big for reserved area",
    /* 157 */     "Implementation: Clowder is too big for reserved area",
    /* 158 */     "Atomic operation did not complete",
    /* 159 */     "Assertion failed",
    /* 160 */     "Implementation: Could not allocate memory for kitten's own coloring bits",
    /* 161 */     "Implementation: The \"space\" portion of a memory space is incorrectly aligned",
#ifdef PIXIE_SCHEME_III
    /* 162 */     "Insufficient memory for the Pixie Scheme III continuation.",
#else
#ifdef PIXIE_SCHEMES
    /* 162 */     "Insufficient memory for the Pixie Scheme II continuation.",
#else
    /* 162 */     "Insufficient memory for the Wraith Scheme continuation.\n\nYou might try using a smaller main memory size.  To do so, you may use the Wraith Scheme preferences window right now, from the Interpreter menu, before pressing the \"Sorry!\" button",
#endif
#endif
    /* 163 */     "Implementation: Error writing to shared memory for interrupt information file",
    /* 164 */     "Implementation: Attempted to copy part of one memory space to another memory space that is too small",
    /* 165 */     "Implementation: Interrupt information is too big for reserved area",
    /* 166 */     "Implementation: \"replaceRegisterPointer\" found pointer outside of old space",
    /* 167 */     "Implementation: \"replaceRegisterPointer\" found pointer indirection outside of new space",
    /* 168 */     "Implementation: Error with memory mapping (function mmap) of interrupt information file",
    /* 169 */     "Implementation: \"updateAddressStack1\" found an address that does not point to a generational forwarding pointer",
    /* 170 */     "Implementation: Saw 0x3000 (extra cdr code) in \"carCdrMixForCSPushProperList\"",
    /* 171 */     "Implementation: \"replacePointersInAddresStack\" found an address pointing into the space being collected",
    /* 172 */     "Miscellaneous error",  // DEBUG
    /* 173 */     "Implementation: MomCat could not create generational garbage collection scratch space file",
    /* 174 */     "Implementation: Generational garbage collector found cdr-coded list spanning border into aging space",
    /* 175 */     "Implementation: Generation size, for generational garbage collection, is too large",
    /* 176 */     "Implementation: Generational garbage collector did not find generational forwarding pointer (second location)",
    /* 177 */     "Implementation: Generational garbage collector scratch space is too big for reserved area",
    /* 178 */     "Implementation: Generational garbage collector did not find generational forwarding pointer (third location)",
    /* 179 */     "Implementation: Generational garbage collector did not find generational forwarding pointer (fourth location)",
    /* 180 */     "Implementation: Saw 0x3000 (extra cdr code) in \"calculatePortableObjectSizeAux\"",
    /* 181 */     "Implementation: In \"calculatePortableObjectSizeAux\" -- bad cdr code",
    /* 182 */     "While loading a file, unable to determine the working directory (\"getcwd\" failed)",
    /* 183 */     "Implementation: Full garbage collector entry routine called while garbage collector is already running",
    /* 184 */     "Implementation: Generational garbage collector entry routine called while garbage collector is already running",
    /* 185 */     "Implementation: Full garbage collector inner routine called while running",
    /* 186 */     "Implementation: Generational garbage collector does not have space to do a garbage collection",
    /* 187 */     "Implementation: Unrecognized message from mom, in \"kittenHandleGCSequenceMessages\"",
    /* 188 */     "Implementation: \"standardFullGC\" called by kitten",
    /* 189 */     "Implementation: \"standardGenerationalGC\" called by kitten",
    /* 190 */     "Implementation: \"gcSequence\" called by kitten",
    /* 191 */     "Implementation: Cannot open world for initial load, from application bundle",
    /* 192 */     "Implementation: Cannot find a world to load in either the application bundle or the application support directory",
    /* 193 */     "No memory for a string to use in worlds",
    /* 194 */     "No memory for a table of pointers to void functions of no arguments",
    };

UnsignedLongWord NumberOfFatalErrorStrings = 194;

const Text *MiscStrings[] = {
/* 0 */    "Dummy",
/* 1 */     ".)",
/* 2 */     "Number of elements: %ld\n",
/* 3 */    " ... ",
/* 4 */    "  Last lambda called (which may have returned) was recently named:\n    ",
/* 5 */    " ...",
/* 6 */    ",\n  but exactly %ld arguments are required.",
/* 7 */    "    %ld bytes in use.\n",
#ifndef PIXIE_SCHEME_III
/* 8 */    "    End -> %016lx (should be %016lx).\n",
#else
/* 8 */    "    End -> %016llx (should be %016llx).\n",
#endif
#ifndef PIXIE_SCHEME_III
/* 9 */    "    space --> %016lx, free -> %016lx, lowFree -> %016lx, end -> %016lx, inUse -> %d\n",
#else
/* 9 */    "    space --> %016llx, free -> %016llx, lowFree -> %016llx, end -> %016llx, inUse -> %d\n",
#endif
#ifndef PIXIE_SCHEME_III
/* 10 */    "    Space at %016lx of %016lx(h) bytes, free -> %016lx, lowFree -> %016lx;\n",
#else
/* 10 */    "    Space at %016llx of %016llx(h) bytes, free -> %016llx, lowFree -> %016llx;\n",
#endif
#ifndef PIXIE_SCHEME_III
/* 11 */    "    first type-checker @ %016lx\n",
#else
/* 11 */    "    first type-checker @ %016llx\n",
#endif
/* 12 */    "  %ld embedded cons cars are cdr-coded \"next\"",
/* 13 */    "  %ld embedded cons cars are cdr-coded \"next\", total \"next\" cdr-codes: %ld, vector-lengths %ld:\n",    
/* 14 */    "  (Resetting)\n",
/* 15 */    "  <-  ",
/* 16 */    "  Thus %ld more bytes might be used after a gc.\n",
/* 17 */    "  car:  ",
/* 18 */    "  in application of ",
/* 19 */    "  in the application of ",
/* 20 */    "  next: ",
/* 21 */    "  which is presently: ",
/* 22 */    " ",
/* 23 */    " %02x",
#ifndef PIXIE_SCHEME_III
/* 24 */    " %016lx",
#else
/* 24 */    " %016llx",
#endif
/* 25 */    "    %ld \"next\" cdr codes (%ld in lists), vector lengths %ld\n",
/* 26 */    " %ld of %ld (hex %x) bytes used.\n",
/* 27 */    " (... full GC ...) ",
/* 28 */    " -> ",
/* 29 */    " . ",
/* 30 */    " <--> ",
/* 31 */    " applied to ",
/* 32 */    " applied to zero arguments, but at least one is required.",
/* 33 */    " called with %ld argument",
/* 34 */    " to:\n  ",
/* 35 */    "#### m->free is out of range!!\n",
/* 36 */    "#(" ,
/* 37 */    "#<Built-in expression \"",
/* 38 */    "#<Built-in procedure \"",
#ifndef PIXIE_SCHEME_III
/* 39 */    "#<Byte block, %016lx long>",
#else
/* 39 */    "#<Byte block, %016llx long>",
#endif
/* 40 */    "#<Interpreted lambda expression, possibly named \"",
/* 41 */    "  already forced to: ",
#ifndef PIXIE_SCHEME_III
/* 42 */    "#<Lambda expression with compiled body at %016lx>",
#else
/* 42 */    "#<Lambda expression with compiled body at %016llx>",
#endif
/* 43 */    "#<Peculiar Leaf type %ld>",
/* 44 */    "#<Promise>",
#ifndef PIXIE_SCHEME_III
/* 45 */    "#<Scheme machine code at %016lx>",
#else
/* 45 */    "#<Scheme machine code at %016llx>",
#endif
/* 46 */    "  not yet forced -- forcing will evaluate: ",
/* 47 */    "#\\%c",
/* 48 */    "#\\newline",
/* 49 */    "#\\space",
/* 50 */    "#f",
/* 51 */    "#t",
/* 52 */    "%#.7g",
#ifndef PIXIE_SCHEME_III
/* 53 */    "%016lx-%016lx: %s -- %s%s",
#else
/* 53 */    "%016llx-%016llx: %s -- %s%s",
#endif
/* 54 */    "%02x ",
#ifndef PIXIE_SCHEME_III
/* 55 */    "%016lx : ",
#else
/* 55 */    "%016llx : ",
#endif
#ifndef PIXIE_SCHEME_III
/* 56 */    "%016lx : %016lx-%016lx: %s -- %s",
#else
/* 56 */    "%016llx : %016llx-%016llx: %s -- %s",
#endif
/* 57 */    "%c",
/* 58 */    "%ld bytes free out of %ld (%ld used) in main memory\n",
/* 59 */    "%ld bytes long, ",
/* 60 */    "%ld",
/* 61 */    "(" ,
/* 62 */    ")" ,
/* 63 */    "*** Data type list:\n",
/* 64 */    "*** Data type table:\n",
/* 65 */    "*** Function list:\n",
/* 66 */    "*** Loading Scheme functions (type one) ... ",
/* 67 */    "*** Loading Scheme functions (type three) ... ",
/* 68 */    "*** Loading Scheme functions (type two) ... ",
/* 69 */    "*** Loading Scheme global characters ... ",
/* 70 */    "*** Loading Scheme global strings ... ",
/* 71 */    "*** Loading Scheme global symbols ... ",
/* 72 */    "*** Loading Scheme global taggedAvals ... ",
/* 73 */    "*** Loading Scheme special forms ... ",
#ifndef PIXIE_SCHEME_III
/* 74 */    "*** Memory space descriptor at %016x,",
#else
/* 74 */    "*** Memory space descriptor at %016x,",
#endif
#ifndef PIXIE_SCHEME_III
/* 75 */    "*** Memory space descriptor at %016x;\n",
#else
/* 75 */    "*** Memory space descriptor at %016x;\n",
#endif
/* 76 */    "*** Special form list:\n",
#ifndef PIXIE_SCHEME_III
/* 77 */    ", other type-checker @ %016x\n",
#else
/* 77 */    ", other type-checker @ %016x\n",
#endif
#ifndef PIXIE_SCHEME_III
/* 78 */    ", type-checker @ %016x\n",
#else
/* 78 */    ", type-checker @ %016x\n",
#endif
/* 79 */    ", type-checkers @",
/* 80 */    ",\n  but at least %ld argument is required.",
/* 81 */    ",\n  but at least %ld arguments are required.",
/* 82 */    ",\n  but at most %ld arguments are allowed.",
/* 83 */    "-------- End of Environment List --------\n",
/* 84 */    "-------- Environment %ld of Environment List --------\n",
#ifndef PIXIE_SCHEME_III
/* 85 */    "@ %016x, ",
#else
/* 85 */    "@ %016x, ",
#endif
#ifndef PIXIE_SCHEME_III
/* 86 */    "@ %016x, special form %d\n",
#else
/* 86 */    "@ %016x, special form %d\n",
#endif
/* 87 */    "Argument(s):\n",
/* 88 */    "By measurement,\n",
/* 89 */    "By stored running totals,\n",
/* 90 */    "Current environment -- first of %ld:\n",
/* 91 */    "Environment list:\n",
/* 92 */    "Error while initializing -- exiting ...\n",
/* 93 */    "Fatal problem string number %ld\n",
/* 94 */    "Function type %ld, for function ",
/* 95 */    "In \"hashText\", text being hashed has at least %ld characters",
/* 96 */    "Incorrect types for:\n",
/* 97 */    "Not enough memory for all the fatal error strings.\n",
/* 98 */    "Not enough memory for the array of fatal error strings.\n",
/* 99 */    "Null functionDescriptor (type three)\n",
/* 100 */    "Null functionDescriptor (type one)\n",
/* 101 */    "Null functionDescriptor (type two)\n",
/* 102 */    "Null dataDescriptor\n",
/* 103 */    "Null specialFormDescriptor\n",
/* 104 */    "Problem string number %ld\n",
/* 105 */    "Top-level loop ...\n",
/* 106 */    "\"",
/* 107 */    "... ",
/* 108 */    "\"%s\": Tag %02x, number %d",
/* 109 */    "\">",
/* 110 */    "  ",
/* 111 */    "\nIn \"cdr\", car-cdr-next followed by pair that is not a car.\n",
/* 112 */    "\\",
/* 113 */    "\n    ",
/* 114 */    "\n -> ",
/* 115 */    "\n",
/* 116 */    "\n%ld bytes wanted:",
/* 117 */    "\n******** ... end CurrentMainSpace Description.",
/* 118 */    "\n******** CurrentMainSpace:\n\n",
/* 119 */    "\n%s",
/* 120 */    "\nUnsolvable problem: %s.\n",
/* 121 */    "\nIn \"setCdr\", car-cdr-next followed by pair that is not a car.\n",
/* 122 */    "\nMay the source be with you ...\n\n",
/* 123 */    "\nProblem: %s.",
/* 124 */    "done\n",
/* 125 */    "function %d: ",
/* 126 */    "function %d (%s): %ld args, ",
/* 127 */    " applied to:",
/* 128 */    "g%lu",
#ifndef PIXIE_SCHEME_III
/* 129 */    "identifier @ %016lx, display(er) @ %016lx, write(r) @ %016lx\n",
#else
/* 129 */    "identifier @ %016llx, display(er) @ %016llx, write(r) @ %016llx\n",
#endif
#ifndef PIXIE_SCHEME_III
/* 130 */    "lengthFinder @ %016lx, ",
#else
/* 130 */    "lengthFinder @ %016llx, ",
#endif
#ifndef PIXIE_SCHEME_III
/* 131 */    "resultTag %02x, code @ %016lx",
#else
/* 131 */    "resultTag %02x, code @ %016llx",
#endif
#ifndef PIXIE_SCHEME_III
/* 132 */    "resultTag %02x, code @ %016lx, max args %ld\n",
#else
/* 132 */    "resultTag %02x, code @ %016llx, max args %ld\n",
#endif
#ifndef PIXIE_SCHEME_III
/* 133 */    "resultTag %02x, code @ %016lx, min args %ld",
#else
/* 133 */    "resultTag %02x, code @ %016llx, min args %ld",
#endif
/* 134 */    "s",
/* 135 */    "Unknown type",
/* 136 */    "Cdr Next",
/* 137 */    "Cdr Nil",
/* 138 */    "Extra Cdr Code",
/* 139 */    "Not a Car",
#ifndef PIXIE_SCHEME_III
/* 140 */    "%s -- Object at 0x%016lx has lock bit 1 set and location is zero.",  // WILL BE UNUSED
#else
/* 140 */    "%s -- Object at 0x%016llx has lock bit 1 set and location is zero.",  // WILL BE UNUSED
#endif
/* 141 */    "Kitten %d: ",
#ifndef PIXIE_SCHEME_III
/* 142 */    "\nAtomic test-and-set timed out at memory address 0x%016lx: %016lx-%016lx.\n"\
             "Called by kitten %d, location %d, blocked by kitten %d, location %d.\n"\
             "(values may have changed between timeout and report)\n",
#else
/* 142 */    "\nAtomic test-and-set timed out at memory address 0x%016llx: %016llx-%016llx.\n"\
             "Called by kitten %d, location %d, blocked by kitten %d, location %d.\n"\
             "(values may have changed between timeout and report)\n",
#endif
/* 143 */    "\nAtomic test-and-set timed out while trying to lock top-level environment bin %ld.\n"\
             "Called by kitten %d.\n",  // LIKELY UNUSED
/* 144 */    "\nAtomic test-and-set timed out while trying to lock obList bin %ld.\n"\
             "Called by kitten %d.\n",  // LIKELY UNUSED
#ifndef PIXIE_SCHEME_III
/* 145 */    "\nAttempt to unlock memory address 0x%016lx: %016lx-%016lx, which is not locked.\n"\
             "Called by kitten %d, location %d.  Values may have changed between timeout and report.\n",
#else
/* 145 */    "\nAttempt to unlock memory address 0x%016llx: %016llx-%016llx, which is not locked.\n"\
             "Called by kitten %d, location %d.  Values may have changed between timeout and report.\n",
#endif
/* 146 */    "val",
/* 147 */    "space",
/* 148 */     "newline",
/* 149 */    "\n%c \"%s\"",
/* 150 */    "Not enough memory for a string;.\n",
/* 151 */    "#",
/* 152 */    ";",
/* 153 */    "\'",
/* 154 */    "`",
/* 155 */    ",",
/* 156 */    ",@",
/* 157 */    "\r",
/* 158 */    "\t",
/* 159 */    "t",
/* 160 */    "f",
/* 161 */    "fedcba9876543210",
/* 162 */    "./",
/* 163 */    "ei",
/* 164 */    "e",
/* 165 */    "sl",
/* 166 */    "bodx",
/* 167 */    ".+-",                                 /* peculiar initials */
/* 168 */    "()\";",                               /* extra delimiters */
/* 169 */    "!$%&*/:<=>?~_^",                      /* special initials */
/* 170 */    ".+-@",                                /* special subsequents */
/* 171 */    " \t\n\014\r",                         /* whitespace */
// /* 172 */    "1111111111111111111111111111111",
// /* 173 */    "17777777777",
// /* 174 */    "2147483647",
// /* 175 */    "7fffffff",
/* 172 */    "111111111111111111111111111111111111111111111111111111111111111",
/* 173 */    "777777777777777777777",
/* 174 */    "9223372036854775807",
/* 175 */    "7fffffffffffffff",
/* 176 */    "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ",
/* 177 */    "ABCDEFGHIJKLMNOPQRSTUVWXYZ",
/* 178 */    "abcdefghijklmnopqrstuvwxyz",
/* 179 */    "#<Peculiar Non-Leaf %ld>",
/* 180 */    "#<Peculiar Non-Leaf %ld> -> ",
/* 181 */    "#<Input Port %ld for kitten %d (\"%s\") (%s) -- recently \"%s\")>",
/* 182 */    "#<Output Port %ld for kitten %d (\"%s\") (%s) -- recently \"%s\")>",
/* 183 */    "File \"%s\", access rights \"%s\", recently \"%s\".\n",
/* 184 */    "#<Closed Port>",
/* 185 */    "*** Loading Scheme functions (type four) ... ",
/* 186 */    "*** Loading Scheme functions (type five) ... ",
/* 187 */    "*** Loading Scheme functions (type six) ... ",
#ifndef PIXIE_SCHEME_III
/* 188 */    "#<Pointer to memory location (hex) %016lx>",
#else
/* 188 */    "#<Pointer to memory location (hex) %016llx>",
#endif
/* 189 */    " applied to %ld arguments, but at most %ld are allowed.",
/* 190 */    ",\n  but at least %ld arguments are required.",
/* 191 */    " applied to %ld argument, but at most %ld are allowed.",
/* 192 */    "Null functionDescriptor (type four)\n",
/* 193 */    "Null functionDescriptor (type five)\n",
/* 194 */    "Null functionDescriptor (type six)\n",
/* 195 */    "Null functionDescriptor (type seven)\n",
/* 196 */    "*** Loading Scheme functions (type seven) ... ",
/* 197 */    ",\n  but at most %ld argument is allowed.",
#ifndef PIXIE_SCHEME_III
/* 198 */    "resultTag %02x, code @ %016lx, min args %ld, max args %ld\n",
#else
/* 198 */    "resultTag %02x, code @ %016llx, min args %ld, max args %ld\n",
#endif
/* 199 */    "  ... where there is ...\n",
/* 200 */    ",\n  but at least %ld argument is required.",
/* 201 */    "%s",
/* 202 */    "#'`,@\\",
/* 203 */    "#<Cons Dot>",
/* 204 */    "#<End of File>",
/* 205 */    "#<Console Input Port>",
/* 206 */    "#<Console Output Port>",
/* 207 */    "Releasing locks ...\n",
/* 208 */    "  ... was locked, unlocks to:\n",
/* 209 */    "No locks found in Scheme main memory.\n",
/* 210 */    "WraithScheme.Init.s",
/* 211 */    "#<Macro>",
/* 212 */    "int",
/* 213 */    "rat",
/* 214 */    "fix",
/* 215 */    "flo",
/* 216 */    "sci",
/* 217 */    "Pathname \"%s\" -- for %s",
/* 218 */    "exactness",
/* 219 */    "radix",
/* 220 */    "unknown",
/* 221 */    "h",
/* 222 */    "\n\"%s\" fmt, N: %ld, P: %ld, show-e? %ld, show-r? %ld, radix %ld\n",
/* 223 */    "es",                                          /* express/surpress */
#ifdef PIXIE_SCHEME_III  // Formatted for Palatino-Roman:
/* 224 */    "\n             Pixie Scheme III -- an implementation of the Scheme programming language.\n",
#else
#ifdef PIXIE_SCHEMES
/* 224 */    "\n         Pixie Scheme II -- an implementation of the Scheme programming language.\n",
#else
/* 224 */    "\n          Wraith Scheme -- an implementation of the Scheme programming language.\n",
#endif
#endif
/* 225 */    "                          Compiled %s ",
/* 226 */    "at %s.\n",
/* 227 */    "Allocated two main memories, each %ld bytes long.\n",
/* 228 */    "Free memory: %ld bytes.\n\n",
/* 229 */    "nan",
/* 230 */    "inf",
/* 231 */    "                 Uses MC68020 and MC68881 instructions.\n\n",
/* 232 */    "Kitten %d wants the file descriptor associated with port %ld on kitten %d\n",
/* 233 */    "Select",
/* 234 */    "%s.",
/* 235 */    ",\n  but exactly %ld argument is required.",
/* 236 */    "#<Console Error Port>",
/* 237 */    "Null functionDescriptor (type eight)\n",
/* 238 */    "*** Loading Scheme functions (type eight) ... ",
/* 239 */    "ObList hash bin %ld is locked.\n",
/* 240 */    "o",
/* 241 */    "#<Lambda expression compiled to threaded code>",
/* 242 */    "#<Reference %ld down from stack top>",
#ifndef PIXIE_SCHEME_III
/* 243 */    "ATTEMPT TO LOAD OBSOLETE WORLD: Address 0x%016lx is not in table.\n",
#else
/* 243 */    "ATTEMPT TO LOAD OBSOLETE WORLD: Address 0x%016llx is not in table.\n",
#endif
/* 244 */    "Save world on file:",
/* 245 */    "WraithScheme.world",
/* 246 */    "File to write:",
/* 247 */    "#<Port %ld for kitten %d (\"%s\") (%s) -- recently \"%s\")>",
/* 248 */    "No ObList hash bins locked.\n",
/* 249 */    "Kitten %d has locked ObList hash bin %ld.\n",
/* 250 */    "%c \"%s\"",
/* 251 */    "-",                                         /* weird characters */
/* 252 */    "Too many \"%c\"'s",
/* 253 */    "\"%s\" followed by \"%c\"; the latter character is not legal in this position",
/* 254 */    "Type a string (omit enclosing double-quotes):",
/* 255 */    "*** Loading Scheme functions (type nine) ... ",
/* 256 */    " %04x",
/* 257 */    "Error while initializing -- click mouse to exit ...",
/* 258 */    "Found %ld \'%s\' resource(s); expected %ld.  Virus attack??",
/* 259 */    "",    // NOT USED
#ifndef PIXIE_SCHEME_III
/* 260 */    "Address 0x%016lx: ",
#else
/* 260 */    "Address 0x%016llx: ",
#endif
/* 261 */    "\nMacintosh I/O or file system error %d.",
/* 262 */    "Loaded world \"%s\".\n\n",
/* 263 */    "\n\"%s\"",
/* 264 */    ": ",
/* 265 */    "TLE hash bin %d is locked.\n",
/* 266 */    "    Semantics / Returned Value:  ",
/* 267 */    ".",                                           /* "." as sentence punctuation */
/* 268 */    ",",                                           /* "," as sentence punctuation */
/* 269 */    ";",                                           /* ";" as sentence punctuation */
/* 270 */    "    Syntax:  ",
/* 271 */       "Pair",
#ifndef PIXIE_SCHEME_III
/* 272 */       ",\nresult checker @%016LX",
#else
/* 272 */       ",\nresult checker @%016LLX",
#endif
/* 273 */       "    For example:  ",
/* 274 */       "o"                                                     /* reserved for "c::quote-simple" */,
/* 275 */    "#<Ref to var %ld in environment %ld down>",
/* 276 */    "#<Ref to var %ld in top-level bucket %ld>",
/* 277 */    " ... has no lexically visible binding.",
/* 278 */    "()",
/* 279 */    "  Last lambda called (which may have returned) has never been named.\n",
/* 280 */    "  Recent names of non-tail-recursive stacked lambda expressions:\n",
/* 281 */
"WraithScheme flags and arguments:\n\
\n\
    -h  Print this message.\n\
    -f  Take next argument as initialization file to be loaded\n\
          on startup of WraithScheme -- either an absolute\n\
          pathname or one relative to the directory containing\n\
          WraithScheme.  Default value: \"%s\".\n\
          For no initialization file, use an argument to -f\n\
          of \"\" (with the quotation marks).\n\
    -m  Take next argument as memory size, in bytes, of each\n\
          WraithScheme heap.  (There are two, for copy-across\n\
          garbage collection.)  The argument should be an\n\
          unsigned integer.  Default value: %ld.\n\
\n\
    Note:  WraithScheme accepts no unflagged arguments, but\n\
          Unix I/O redirection, e.g. via pipes, \"<\" and \">\",\n\
          works just fine.  Make sure that any file redirected\n\
          to input this way ends with an instruction to exit\n\
          from Wraith Scheme, or the interpreter will just\n\
          sit there waiting for instructions, and you will\n\
          have to exit the program via Control-\\ or Control-Z\n\
          and \"kill\".\n\
\n",

/* 282 */    "\nSpurious items in WraithScheme command tail.\n\n",
/* 283 */    "Invalid \"-m\" option -- expected a positive integer, at least 250000, for memory-space size.\n",
/* 284 */    "Invalid \"-f\" option: Cannot open \"%s\".  (Does it exist?)\n",
/* 285 */    "Loaded initialization file (\"%s\").\n",
/* 286 */    "World file: \"%s\"\n",
/* 287 */    "Current space and reloaded saved world differ in %s p at  %d (of %d) up from free point of space\n",
/* 288 */    "leaf",
/* 289 */    "non-leaf",
/* 290 */    "Current space and reloaded saved world differ in tag at %d (of %d) up from free point of space\n",
/* 291 */    "Current space and reloaded saved world differ in extra space at %d (of %d) up from free point of space\n",
/* 292 */    "Loading world: \"%s\"\n",
/* 293 */    "...\n",
/* 294 */    "Input port %ld is in use\n",
/* 295 */    "Output port %ld is in use\n",
/* 296 */    "Transcript file is in use\n",
/* 297 */    "Scratch file is in use\n",
/* 298 */    "\"rat\" fmt not implemented yet.\n",
/* 299 */    "Garbage collection ...",
/* 300 */    "Garbage collection ... completed.",
/* 301 */    "NOTE: ",
/* 302 */    "<Uninitialized Input Port>",
/* 303 */    "<Uninitialized Output Port>",
/* 304 */    "<Console Error Port>",
/* 305 */    "open",
/* 306 */    "closed",
/* 307 */    "<not previously used>",
#ifndef PIXIE_SCHEME_III
/* 308 */    "\"%s\" (oldPathName \"%s\"), 0x%016lx 0x%016lx %d, access rights \"%s\", kitten %d\n",
#else
/* 308 */    "\"%s\" (oldPathName \"%s\"), 0x%016llx 0x%016llx %d, access rights \"%s\", kitten %d\n",
#endif
#ifndef PIXIE_SCHEME_III    // Formatted for Palatino-Roman.
/* 309 */    "                       Copyright (c) 2009-2010 by Jay Reynolds Freeman\n",
#else
/* 309 */    "                                 Copyright (c) 2009-2010 by Jay Reynolds Freeman\n",
#endif
#ifndef PIXIE_SCHEME_III
/* 310 */    "Vector has length %ld, attempting to access element %ld (zero-based).",
#else
/* 310 */    "Vector has length %lld, attempting to access element %lld (zero-based).",
#endif
/* 311 */    "               (Personal Web Site: http://web.mac.com/Jay_Reynolds_Freeman)  \n"\
             "\n",
/* 312 */    "Memory size of %ld bytes is too large: The maximum is %ld bytes.\n",
/* 313 */    "deflqs",       // Exponent markers.
/* 314 */    "\"%c\" <ASCII %d>",
/* 315 */    "At the end of command memory.",
/* 316 */    "Pathname of file for input?  (unquoted)",
/* 317 */    "Pathname of file for output?  (unquoted)",
/* 318 */    "Pathname of file for a transcript?  (unquoted)",
/* 319 */    "Pathname of file to insert?  (unquoted)",
/* 320 */    "read",
/* 321 */    "write",
/* 322 */    "Wraith Scheme puts brief messages here.",
/* 323 */    "q",   // Exponent for short floats.
/* 324 */    "\n\n\n\n",
/* 325 */    "\n                           ... peek-a-boo ...\n\n\n",
/* 326 */    "At the end of scroll-back memory.",
/* 327 */    "Implementation: Cannot open world for intial load, for reading.\n",
/* 328 */    "Implementation: Cannot find world for initial load.\n",
/* 329 */    ".",    // peculiar subsequents
/* 330 */    "\\n",
/* 331 */    "\\r",
/* 332 */    "\\t",
/* 333 */    "#\\tab",
/* 334 */    "#\\return",
/* 335 */    "tab",
/* 336 */    "return",
/* 337 */    "",
/* 338 */
"\nThis bug is MY FAULT!\n\nPlease report it to\n\n\
      Jay_Reynolds_Freeman@mac.com\n\n\
so that I can fix it.  Thank you very much.\n",
/* 339 */    "Implementation",
#ifndef PIXIE_SCHEME_III
/* 340 */
"\nThe application needs more\n\
memory than it can find.\n\
Try again when fewer applications\n\
are running, or with a smaller\n\
main memory size.\n",
#else
/* 340 */
"\nThe application needs more\n\
memory than it can find.\n\
Try again when fewer applications\n\
are running\n",
#endif
/* 341 */    "No memory",
/* 342 */    "Null functionDescriptor (type nine)\n",
/* 343 */    "Atomic test-and-set timed out while trying to lock queue %d.\n"\
             "Called by kitten %d, location %d, blocked by kitten %d, location %d.\n"\
             "(values may have changed between timeout and report)\n",
#ifndef PIXIE_SCHEME_III
/* 344 */    "\nJust locked queue access flag %d, but the tag came out wrong -- intended %016lx.\n"\
             "Called by kitten %d at location %d; tag holds %d and %d.\n",  // UNUSED
#else
/* 344 */    "\nJust locked queue access flag %d, but the tag came out wrong -- intended %016llx.\n"\
             "Called by kitten %d at location %d; tag holds %d and %d.\n",  // UNUSED
#endif
/* 345 */    "\nAttempt to unlock queue %d, which is not locked.\n"\
             "Called by kitten %d, location %d.  Values may have changed between timeout and report.\n",
#ifndef PIXIE_SCHEME_III
/* 346 */    "Looking for Scheme PC 0x%016lx\n",
#else
/* 346 */    "Looking for Scheme PC 0x%016llx\n",
#endif
/* 347 */    "Memory space size %d bytes.  Initializing memory ...",
/* 348 */    "Lambda's non-top-level environments are:",
/* 349 */    "Pathname of file for world load?",
/* 350 */    "Pathname of file for world save?",
/* 351 */    " (%d bytes)",
/* 352 */    "Pathname \"%s\"",
/* 353 */    "Pathname of new current directory?",
/* 354 */    "<",     // left pointy bracket
/* 355 */    ">",     // right pointy bracket
/* 356 */    " -- unbound",
/* 357 */    "#<Error Output Port>",
#ifndef PIXIE_SCHEME_III
/* 358 */    "Stack bottom at address 0x%016lx -- grows downward\n",
#else
/* 358 */    "Stack bottom at address 0x%016llx -- grows downward\n",
#endif
/* 359 */    "#<Multiple Values Return>",
/* 360 */    "List of values: ",
/* 361 */    "Memory space size %d bytes.  Initializing memory may take a LONG time ...",    //  WILL BE UNUSED
/* 362 */    "Expected %ld bytes, copied %ld bytes.\n",
#ifndef PIXIE_SCHEME_III
/* 363 */    "Last tag preceding was at 0x%016lx:  ",
#else
/* 363 */    "Last tag preceding was at 0x%016llx:  ",
#endif
#ifndef PIXIE_SCHEME_III
/* 364 */    "%s -- Unknown tag at address 0x%016lx:  ",
#else
/* 364 */    "%s -- Unknown tag at address 0x%016llx:  ",
#endif
/* 365 */    "%s -- Forwarding pointer should have cdr code of zero (not car).",
/* 366 */    " (\"%s\")",
/* 367 */    "Tagged aval pointer is zero",
#ifndef PIXIE_SCHEME_III
/* 368 */    "%s -- Pointer at memory address 0x%016lx points outside of current space.",
#else
/* 368 */    "%s -- Pointer at memory address 0x%016llx points outside of current space.",
#endif
#ifndef PIXIE_SCHEME_III
/* 369 */    "%s -- Byte block at 0x%016lx has cdr code other than not-car.",
#else
/* 369 */    "%s -- Byte block at 0x%016llx has cdr code other than not-car.",
#endif
/* 370 */    "\"%s\" encountered %ld non-garbage objects, expected %ld, number of objects copied was %ld, number of in-space non-garbage forwarding pointers was %ld",
#ifndef PIXIE_SCHEME_III
/* 371 */    "%s -- Non-garbage pointer at 0x%016lx to garbage object.",
#else
/* 371 */    "%s -- Non-garbage pointer at 0x%016llx to garbage object.",
#endif
#ifndef PIXIE_SCHEME_III
/* 372 */    "%s -- Object at 0x%016lx has one or more gc bit or recursion bit set.",
#else
/* 372 */    "%s -- Object at 0x%016llx has one or more gc bit or recursion bit set.",
#endif
/* 373 */    "/",  // Pathname separator
#ifndef PIXIE_SCHEME_III
/* 374 */    "#<Scheme machine PC index #%ld, corresponds to code at 0x%016lx>",
#else
/* 374 */    "#<Scheme machine PC index #%ld, corresponds to code at 0x%016llx>",
#endif
/* 375 */    "\">",
/* 376 */    "<no name>",
/* 377 */    "Compiler has been turned on.",
/* 378 */    "Compiler has been turned off.",
/* 379 */    "Using full precision for output of numbers.",
/* 380 */    "Using reduced precision for output of numbers.",
/* 381 */    "File load started ...",
/* 382 */    "File load completed.",
/* 383 */    "File load did not complete.",
#ifdef PIXIE_SCHEMES
/* 384 */    "Pixie Scheme II has reset to top-level loop.",
#else
/* 384 */    "Wraith Scheme has reset to top-level loop.",
#endif
/* 385 */    "-------- End of Non-Top-Level Environment List --------\n",
/* 386 */    "Pathname of file for world to inspect?",
#ifndef PIXIE_SCHEME_III
/* 387 */    "errno is 0x%016x\n",
#else
/* 387 */    "errno is 0x%016x\n",
#endif
/* 388 */    "u",  // FAIL
/* 389 */    "s",  // SUCCEED
/* 390 */    "#u",
/* 391 */    "#s",
/* 392 */    "Problem using path \"%s\" for the %s application support directory.\n",
/* 393 */    "Problem creating the %s application support directory, \"%s\".\n",
/* 394 */    "Operation at offset %ld will exceed size %ld of memory-mapped block.",
#ifndef PIXIE_SCHEME_III
/* 395 */    "\nAttempt to lock memory address 0x%016lx: %016lx-%016lx, which is %salready locked.\n"\
             "Called by kitten %d, location %d, perhaps once locked by kitten %d, location %d.\n"\
             "(values may have changed between timeout and report)\n",   // PROBABLY UNUSED
#else
/* 395 */    "\nAttempt to lock memory address 0x%016llx: %016llx-%016llx, which is %salready locked.\n"\
             "Called by kitten %d, location %d, perhaps once locked by kitten %d, location %d.\n"\
             "(values may have changed between timeout and report)\n",   // PROBABLY UNUSED
#endif
#ifndef PIXIE_SCHEME_III
/* 396 */    "\nJust locked memory address 0x%016lx: %016lx-%016lx, but the tag came out wrong -- intended %016lx.\n"\
             "Called by kitten %d at location %d; tag holds %d and %d.\n",  // UNUSED
#else
/* 396 */    "\nJust locked memory address 0x%016llx: %016llx-%016llx, but the tag came out wrong -- intended %016llx.\n"\
             "Called by kitten %d at location %d; tag holds %d and %d.\n",  // UNUSED
#endif
/* 397 */    "No TLE hash bins locked.\n",
/* 398 */    "Kitten %d has locked TLE hash bin %ld.\n",
/* 399 */    " (Kitten %d)",
/* 400 */    "Launching kitten %d: Attempt # %d, waits %d",
/* 401 */    "Launching kitten %d: Attempt # %d, waits %d and %d",
/* 402 */    "Kitten %d has initialized memory.",
/* 403 */    "Socket listen of \"%s\" failed with error %d, \"%s\"\n",
#ifndef PIXIE_SCHEME_III
/* 404 */    "Tag was %016lx just before attempt to change.\n",
#else
/* 404 */    "Tag was %016llx just before attempt to change.\n",
#endif
#ifndef PIXIE_SCHEME_III
/* 405 */    "\nJust unlocked memory address 0x%016lx: %016lx-%016lx, but the tag came out wrong -- intended %016lx.\n"\
             "Called by kitten %d at location %d; tag holds %d and %d.\n",
#else
/* 405 */    "\nJust unlocked memory address 0x%016llx: %016llx-%016llx, but the tag came out wrong -- intended %016llx.\n"\
             "Called by kitten %d at location %d; tag holds %d and %d.\n",
#endif
/* 406 */    "Before: %lu -- After: %lu\n",
/* 407 */    "The tag was officially unlocked",
/* 408 */    "The tag was NOT officially unlocked",
/* 409 */    "Kitten %d is waiting for a memory lock at location %u.\n",
/* 410 */    "No kitten is waiting for a memory lock.\n",
#ifndef PIXIE_SCHEME_III
/* 411 */    "Memory location unexpectedly unlocked at position %u ... %s, tag is 0x%016lx, GCInProgressFlag is %d\n",
#else
/* 411 */    "Memory location unexpectedly unlocked at position %u ... %s, tag is 0x%016llx, GCInProgressFlag is %d\n",
#endif
#ifndef PIXIE_SCHEME_III
/* 412 */    "Memory location unexpectedly created by wrong kitten (%d, locked at %u -- expected kitten %d) noted at position %u ... %s, tag is 0x%016lx, GCInProgressFlag is %d\n",
#else
/* 412 */    "Memory location unexpectedly created by wrong kitten (%d, locked at %u -- expected kitten %d) noted at position %u ... %s, tag is 0x%016llx, GCInProgressFlag is %d\n",
#endif
#ifndef PIXIE_SCHEME_III
/* 413 */    "Memory location unexpectedly locked (locked at %u by kitten %d) noted at position %u by kitten %d... %s, tag is 0x%016lx, GCInProgressFlag is %d\n",
#else
/* 413 */    "Memory location unexpectedly locked (locked at %u by kitten %d) noted at position %u by kitten %d... %s, tag is 0x%016llx, GCInProgressFlag is %d\n",
#endif
#ifndef PIXIE_SCHEME_III
/* 414 */    "Memory location lock bit unexpectedly not set (locked at %u by kitten %d) noted at position %u by kitten %d... %s, tag is 0x%016lx, GCInProgressFlag is %d\n",
#else
/* 414 */    "Memory location lock bit unexpectedly not set (locked at %u by kitten %d) noted at position %u by kitten %d... %s, tag is 0x%016llx, GCInProgressFlag is %d\n",
#endif
/* 415 */    "OSAtomic failure at position %d",
/* 416 */    "Kitten %d is waiting a long time for a memory lock at location %u.\n",
/* 417 */    "%s -- Null pointer in non-leaf.",
/* 418 */    "markGarbageFromReg found non-garbage outside of space\n",
/* 419 */    "",    // NOT USED
/* 420 */    "1111111111111111111111111111111111111111111111111111111111111111",
/* 421 */    "1777777777777777777777",
/* 422 */    "9223372036854775808",
/* 423 */    "ffffffffffffffff",
#ifndef PIXIE_SCHEME_III
/* 424 */    "%s -- Object at 0x%016lx is incorrectly aligned.",
#else
/* 424 */    "%s -- Object at 0x%016llx is incorrectly aligned.",
#endif
#ifdef PIXIE_SCHEME_III  // Formatted for Palatino-Roman:
/* 425 */    "#<%s, expiration time %lld (GMT %d.%02d.%02d %02d:%02d:%02d)>",
/* 426 */    "Expiration time %lld (GMT %d.%02d.%02d %02d:%02d:%02d)\n",
/* 427 */    "Probability of remembering is %g.\n",
/* 428 */    "Object is: ",
/* 429 */    "Probability of remembering is %lld.\n",
/* 430 */    "#<%s, expiration time %lld>",
/* 431 */    "Expiration time %lld\n",
#else
/* 425 */    "#<%s, expiration time %ld (GMT %d.%02d.%02d %02d:%02d:%02d)>",
/* 426 */    "Expiration time %ld (GMT %d.%02d.%02d %02d:%02d:%02d)\n",
/* 427 */    "Probability of remembering is %g.\n",
/* 428 */    "Object is: ",
/* 429 */    "Probability of remembering is %ld.\n",
/* 430 */    "#<%s, expiration time %ld>",
/* 431 */    "Expiration time %ld\n",
#endif
#ifdef PIXIE_SCHEME_III  // Formatted for Palatino-Roman:
/* 432 */    "          This program comes with ABSOLUTELY NO WARRANTY.  This is free software\n"\
             "                       and you are welcome to redistribute it under certain conditions.\n"\
             "                                (See \"Warranty\", when you press the \"Help\" button.)\n\n"\
             "         TO GET STARTED, type a Scheme expression, like (+ 2 2), then press \"Select That?\"\n"\
             "                           and \"Evaluate\", which will appear as buttons at top right.\n\n"\
             "   TO SAVE WORK, including typing, use \"Save World\" in the \"Features\" panel at lower left.\n",
#else
#ifdef PIXIE_SCHEMES
/* 432 */    "          This program comes with ABSOLUTELY NO WARRANTY.  This is free software\n"\
             "              and you are welcome to redistribute it under certain conditions.\n"\
             "              (See \"Warranty and Redistribution\" in the Pixie Scheme II menu.)\n",
#else
/* 432 */    "          This program comes with ABSOLUTELY NO WARRANTY.  This is free software\n"\
             "              and you are welcome to redistribute it under certain conditions.\n"\
             "               (See \"Warranty and Redistribution\" in the Wraith Scheme menu.)\n",
#endif
#endif
/* 433 */    "#<%s, size %ld, path \"%s\",%s memory mapped for kitten %d%s>",
#ifndef PIXIE_SCHEME_III
/* 434 */    "%s, size %ld, at 0x%016lx, protection 0x%08x, flags 0x%08x, offset 0x%08x.\n",
#else
/* 434 */    "%s, size %ld, at 0x%016llx, protection 0x%08x, flags 0x%08x, offset 0x%08x.\n",
#endif
/* 435 */    "Block is associated with file at path \"%s\", which has%s been written.\n",
/* 436 */    "  In kitten %d, the block %s memory mapped.%s\n",
/* 437 */    "Not enough memory-mappable space remains to map %lu bytes (only %lu bytes remain).\n",
/* 438 */    "Attempt to memory-map (function mmap) fails with error %d, \"%s\".\n",
/* 439 */    "File \"%s\":\n",
/* 440 */    "Attempt to open this file fails with error %d, \"%s\".\n",
/* 441 */    "#<Built-in procedure \"%s\">:\n",
/* 442 */    "#<Built-in procedure \"%s\">:",
/* 443 */    "Range error at index %ld:",
/* 444 */    "String starting at offset %ld appears to extend past the end of memory-mapped block.",
/* 445 */    "Socket creation of \"%s\" failed with error %d, \"%s\"\n",
/* 446 */    "Socket bind of \"%s\" failed with error %d, \"%s\"\n",
/* 447 */    "\n    Expected ",
/* 448 */    " for argument list item %ld (zero-based).",
/* 449 */    " for the argument.",
/* 450 */    ":",                                           /* ":" as sentence punctuation */
/* 451 */    "\"%s\" expected %ld non-garbage bytes, number of bytes copied was %ld",
/* 452 */    "Generational garbage collection ...",
/* 453 */    "Generational garbage collection ... completed.",
/* 454 */    "%s -- register points to garbage in current main space.",
/* 455 */    "%s -- register points to garbage in (new) aging space.",
/* 456 */    "%s -- register points into (new) active space, which should be empty.",
/* 457 */    "%s -- register does not point into a valid space.",
/* 458 */    "Replacing pointer in %s\n",
#ifndef PIXIE_SCHEME_III
/* 459 */    "%s -- pointer at 0x%016lx to (old) aging space.",
#else
/* 459 */    "%s -- pointer at 0x%016llx to (old) aging space.",
#endif
#ifndef PIXIE_SCHEME_III
/* 460 */    "%s -- Byte block at 0x%016lx has unreasonably long length.",  // DEBUG
#else
/* 460 */    "%s -- Byte block at 0x%016llx has unreasonably long length.",  // DEBUG
#endif
#ifndef PIXIE_SCHEME_III
/* 461 */    "%s -- Object at 0x%016lx is generational forwarding pointer",
#else
/* 461 */    "%s -- Object at 0x%016llx is generational forwarding pointer",
#endif
#ifndef PIXIE_SCHEME_III
/* 462 */    "%s -- Object at 0x%016lx points to generational forwarding pointer",
#else
/* 462 */    "%s -- Object at 0x%016llx points to generational forwarding pointer",
#endif
/* 463 */    "%ld bytes free out of %ld (%ld used) in the active memory generation\n",
/* 464 */    "%ld bytes free out of %ld (%ld used) in the aging memory generation\n",
/* 465 */    "\nUnexpected \"pointsToActiveBit\" on.\n",
/* 466 */    "#<Marker for where Top-Level Environment points>",
/* 467 */    "#<Marker for where ObList points>",
/* 468 */    "#<Non-printing object>",
/* 469 */    "n",  // Non-printing character is represented by #n
/* 470 */    "Address not in tenured space in \"checkLiveStack\" (%s).\n",
/* 471 */    "Missing points_to_active_bit in \"checkLiveStack\" (%s).\n",
/* 472 */    "Ended check of live stack (%s)\n",
/* 473 */    "Stacked and ",
/* 474 */    "not ",
/* 475 */    "Not stacked, bit is ",
/* 476 */    "on (%s).\n",
/* 477 */    " not yet",
/* 478 */    ", and later unmapped",
/* 479 */    " not",
/* 480 */    "has been",
/* 481 */    "has not yet been",
/* 482 */    "slideLowToHigh",
/* 483 */    "copyAcrossLow (fancy)",
/* 484 */    "Bad active pointer to aging.\n",
/* 485 */    "Tenured pointer to garbage active.\n",
/* 486 */    "Tenured pointer to garbage aging.\n",
/* 487 */    "Writing to spaceA file: %s.\n",
/* 488 */    "Writing to spaceB file: %s.\n",
/* 489 */    "Writing to scratchStack file: %s.\n",
/* 490 */    "Writing to gGCScratchSpace file: %s.\n",
/* 491 */    "nSpaceBytes is %ld, SPACE_RESERVE_SIZE is %ld\n",
/* 492 */    "nStackBytes is %ld, STACK_RESERVE_SIZE is %ld\n",
/* 493 */    "nGGCScratchSpaceBytes is %ld, GGC_SCRATCH_RESERVE_SIZE is %ld\n",
/* 494 */    "Mapping space A: %s.\n",
#ifndef PIXIE_SCHEME_III
/* 495 */    "mmap failed for spaceA address of 0x%016lx\n",
#else
/* 495 */    "mmap failed for spaceA address of 0x%016llx\n",
#endif
/* 496 */    "Mapping space B: %s.\n",
#ifndef PIXIE_SCHEME_III
/* 497 */    "mmap failed for spaceB address of 0x%016lx\n",
#else
/* 497 */    "mmap failed for spaceB address of 0x%016llx\n",
#endif
/* 498 */    "Mapping scratch stack: %s.\n",
#ifndef PIXIE_SCHEME_III
/* 499 */    "mmap failed for scratchStack address of 0x%016lx\n",
#else
/* 499 */    "mmap failed for scratchStack address of 0x%016llx\n",
#endif
/* 500 */    "Mapping generational garbage collection scratch space: %s.\n",
#ifndef PIXIE_SCHEME_III
/* 501 */    "mmap failed for gGCScratchSpace address of 0x%016lx\n",
#else
/* 501 */    "mmap failed for gGCScratchSpace address of 0x%016llx\n",
#endif
#ifndef PIXIE_SCHEME_III
/* 502 */    "The tag was 0x%016lx\n",
#else
/* 502 */    "The tag was 0x%016llx\n",
#endif
/* 503 */    "\nGC in progress.",
/* 504 */    "\n%s: Display of top stack frame ...\n",
/* 505 */    "[DIALOG PROMPT] %s\n",
/* 506 */    "",                      // NOT USED
/* 507 */    "[DIALOG ANSWER] %s\n",
#ifdef PIXIE_SCHEME_III
/* 508 */    "Pixie Scheme III",      // Application Name
#else
#ifdef PIXIE_SCHEMES
/* 508 */    "Pixie Scheme II",      // Application Name
#else
/* 508 */    "Wraith Scheme",        // Application Name
#endif
#endif
/* 509 */    "Cannot open initial world from application support directory.  Will try original world instead.",
/* 510 */    "(64-bit version)",
/* 511 */    "In built-in routine \"%s\":",
/* 512 */    "Needed to rebuild obList: %d, did rebuild obList: %d\n",
/* 513 */    "Pushbutton %u:\n",
/* 514 */    "Interrupt %lu:\n",
    };

const Text *NameStrings[] =  {
/* 0 */      "Dummy",
/*   1 */    "*",
/*   2 */    "+",
/*   3 */    "-",
/*   4 */    "/",
/*   5 */    "<",
/*   6 */    "<=",
/*   7 */    "=",
/*   8 */    ">",
/*   9 */    ">=",
/*  10 */    "abs",
/*  11 */    "acos",
/*  12 */    "e::set-current-directory!",
/*  13 */    "append",
/*  14 */    "apply",
/*  15 */    "asin",
/*  16 */    "assoc",
/*  17 */    "assq",
/*  18 */    "assv",
/*  19 */    "atan",
/*  20 */    "begin",
/*  21 */    "boolean?",
/*  22 */    "caaaar",
/*  23 */    "caaadr",
/*  24 */    "caaar",
/*  25 */    "caadar",
/*  26 */    "caaddr",
/*  27 */    "caadr",
/*  28 */    "caar",
/*  29 */    "cadaar",
/*  30 */    "cadadr",
/*  31 */    "cadar",
/*  32 */    "caddar",
/*  33 */    "cadddr",
/*  34 */    "caddr",
/*  35 */    "cadr",
/*  36 */    "call-with-current-continuation",
/*  37 */    "car",
/*  38 */    "e::current-directory",
/*  39 */    "cdaaar",
/*  40 */    "cdaadr",
/*  41 */    "cdaar",
/*  42 */    "cdadar",
/*  43 */    "cdaddr",
/*  44 */    "cdadr",
/*  45 */    "cdar",
/*  46 */    "cddaar",
/*  47 */    "cddadr",
/*  48 */    "cddar",
/*  49 */    "cdddar",
/*  50 */    "cddddr",
/*  51 */    "cdddr",
/*  52 */    "cddr",
/*  53 */    "cdr",
/*  54 */    "ceiling",
/*  55 */    "char->integer",
/*  56 */    "char-alphabetic?",
/*  57 */    "char-ci<=?",
/*  58 */    "char-ci<?",
/*  59 */    "char-ci=?",
/*  60 */    "char-ci>=?",
/*  61 */    "char-ci>?",
/*  62 */    "char-downcase",
/*  63 */    "char-lower-case?",
/*  64 */    "char-numeric?",
/*  65 */    "char-upcase",
/*  66 */    "char-upper-case?",
/*  67 */    "char-whitespace?",
/*  68 */    "char<=?",
/*  69 */    "char<?",
/*  70 */    "char=?",
/*  71 */    "char>=?",
/*  72 */    "char>?",
/*  73 */    "char?",
/*  74 */    "complex?",
/*  75 */    "e::top-loop-code",
/*  76 */    "cons",
/*  77 */    "cos",
/*  78 */    "c::define",
/*  79 */    "c::environment",
/*  80 */    "display",
/*  81 */    "e::startup-directory",
/*  82 */    "e::syntax-binding?",
/*  83 */    "and",
/*  84 */    "e::show-environment",
/*  85 */    "e::show-environment-list",
/*  86 */    "case",
/*  87 */    "e::show-memory",
/*  88 */    "e::exit",
/*  89 */    "e::fixnum?",
/*  90 */    "e::float?",
/*  91 */    "e::forced?",
/*  92 */    "e::full-gc",
/*  93 */    "e::gensym",
/*  94 */    "e::inspect",
/*  95 */    "e::promise?",
/*  96 */    "e::reset",
/*  97 */    "e::show-room",
/*  98 */    "e::version",
/*  99 */    "e::write-continuation",
/* 100 */    "e::write-stack",
/* 101 */    "else",
/* 102 */    "eq?",
/* 103 */    "equal?",
/* 104 */    "eqv?",
/* 105 */    "even?",
/* 106 */    "exact->inexact",
/* 107 */    "exact?",
/* 108 */    "exp",
/* 109 */    "expt",
/* 110 */    "floor",
/* 111 */    "for-each",
/* 112 */    "force",
/* 113 */    "if",
/* 114 */    "inexact->exact",
/* 115 */    "inexact?",
/* 116 */    "integer->char",
/* 117 */    "integer?",
/* 118 */    "lambda",
/* 119 */    "c::lambda",
/* 120 */    "length",
/* 121 */    "e::show-stack",
/* 122 */    "let*",
/* 123 */    "letrec",
/* 124 */    "list",
/* 125 */    "list->string",
/* 126 */    "list->vector",
/* 127 */    "list-ref",
/* 128 */    "list-tail",
/* 129 */    "log",
/* 130 */    "make-string",
/* 131 */    "make-vector",
/* 132 */    "map",
/* 133 */    "max",
/* 134 */    "member",
/* 135 */    "memq",
/* 136 */    "memv",
/* 137 */    "min",
/* 138 */    "modulo",
/* 139 */    "negative?",
/* 140 */    "newline",
/* 141 */    "not",
/* 142 */    "null?",
/* 143 */    "number?",
/* 144 */    "odd?",
/* 145 */    "cond",
/* 146 */    "pair?",
/* 147 */    "positive?",
/* 148 */    "procedure?",
/* 149 */    "quasiquote",
/* 150 */    "quote",
/* 151 */    "quotient",
/* 152 */    "rational?",
/* 153 */    "real?",
/* 154 */    "remainder",
/* 155 */    "reverse",
/* 156 */    "round",
/* 157 */    "set!",
/* 158 */    "set-car!",
/* 159 */    "set-cdr!",
/* 160 */    "sin",
/* 161 */    "sqrt",
/* 162 */    "string->list",
/* 163 */    "string->number",
/* 164 */    "string->symbol",
/* 165 */    "string-append",
/* 166 */    "string-ci<=?",
/* 167 */    "string-ci<?",
/* 168 */    "string-ci=?",
/* 169 */    "string-ci>=?",
/* 170 */    "string-ci>?",
/* 171 */    "string-copy",
/* 172 */    "string-fill!",
/* 173 */    "string-length",
/* 174 */    "string-ref",
/* 175 */    "string-set!",
/* 176 */    "string<=?",
/* 177 */    "string<?",
/* 178 */    "string=?",
/* 179 */    "string>=?",
/* 180 */    "string>?",
/* 181 */    "string?",
/* 182 */    "substring",
/* 183 */    "symbol->string",
/* 184 */    "symbol?",
/* 185 */    "tan",
/* 186 */    "truncate",
/* 187 */    "unquote",
/* 188 */    "unquote-splicing",
/* 189 */    "vector",
/* 190 */    "vector->list",
/* 191 */    "vector-fill!",
/* 192 */    "vector-length",
/* 193 */    "vector-ref",
/* 194 */    "vector-set!",
/* 195 */    "vector?",
/* 196 */    "write",
/* 197 */    "zero?",
/* 198 */    "open-input-file",
/* 199 */    "open-output-file",
/* 200 */    "close-input-port",
/* 201 */    "close-output-port",
/* 202 */    "input-port?",
/* 203 */    "output-port?",
/* 204 */    "load",
/* 205 */    "write-char",
/* 206 */    "call-with-input-file",
/* 207 */    "call-with-output-file",
/* 208 */    "with-input-from-file",
/* 209 */    "with-output-to-file",
/* 210 */    "transcript-on",
/* 211 */    "transcript-off",
/* 212 */    "e::set-input-port!",
/* 213 */    "read",
/* 214 */    "eof-object?",
/* 215 */    "current-input-port",
/* 216 */    "current-output-port",
/* 217 */    "do",
/* 218 */    "read-char",
/* 219 */    "char-ready?",
/* 220 */    "e::closed-port?",
/* 221 */    "e::inf?",
/* 222 */    "e::nan?",
/* 223 */    "number->string",
/* 224 */    "gcd",
/* 225 */    "lcm",
/* 226 */    "e::room",
/* 227 */    "e::store-size",
/* 228 */    "e::debug",
/* 229 */    "e::cons-with-continuation",
/* 230 */    "o",                                           /* placeholder for "e::set-d!" */
/* 231 */    "e::bound-instance?",
/* 232 */    "define",
/* 233 */    "o",                                           /* placeholder for "e::set-default-working-directory!" */
/* 234 */    "e::list-print-depth",
/* 235 */    "e::vector-print-depth",
/* 236 */    "e::list-print-length",
/* 237 */    "e::vector-print-length",
/* 238 */    "e::set-list-print-depth!",
/* 239 */    "e::set-vector-print-depth!",
/* 240 */    "e::set-list-print-length!",
/* 241 */    "e::set-vector-print-length!",
/* 242 */    "e::time",
/* 243 */    "c::begin",
/* 244 */    "e::stack-depth",
/* 245 */    "c::number->string-with-c-format",
/* 246 */    "e::error-port",
/* 247 */    "e::macro",
/* 248 */    "or",
/* 249 */    "e::macro?",
/* 250 */    "e::expand-macro",
/* 251 */    "e::set-tag!",
/* 252 */    "c::push",
/* 253 */    "c::apply",
/* 254 */    "e::permanent?",
/* 255 */    "e::set-permanent!",
/* 256 */    "c::symeval",
/* 257 */    "c::reset",
/* 258 */    "e::get-tag",
/* 259 */    "c::apply-end",
/* 260 */    "c::if",
/* 261 */    "c::kitten-busy?",
/* 262 */    "c::noop",
/* 263 */    "c::s0-to-c",
/* 264 */    "c::stack-ref",
/* 265 */    "c::compiled-block",
/* 266 */    "e::load-world!",
/* 267 */    "e::save-world",
/* 268 */    "e::clear-permanent!",
/* 269 */    "e::long-float?",
/* 270 */    "e::atom?",
/* 271 */    ">-<",
/* 272 */    ">+<",
/* 273 */    ">++<",
/* 274 */    ">+++<",
/* 275 */    ">*<",
/* 276 */    ">**<",
/* 277 */    ">***<",
/* 278 */    "e::show-dynamic-environment-list",
/* 279 */    "let",
/* 280 */    "let (named)",
/* 281 */    "c::quote-simple",
/* 282 */    "e::bit-and",
/* 283 */    "e::bit-not",
/* 284 */    "e::bit-or",
/* 285 */    "e::bit-xor",
/* 286 */    "e::bit-shift-left",
/* 287 */    "e::bit-shift-right-arithmetic",
/* 288 */    "e::bit-shift-right-logical",
/* 289 */    "e::file-exists?",
/* 290 */    "c::environment-ref",
/* 291 */    "c::top-level-environment-ref",
/* 292 */    "c::hash-string",
/* 293 */    "e::compile-form",
/* 294 */    "c::compiling?",
/* 295 */    "e::define-no-compile",
/* 296 */    "list?",
/* 297 */    "c::symeval-in-environment",
/* 298 */    "e::not-null?",
/* 299 */    "e::inspect-world",
/* 300 */    "e::program-directory",
/* 301 */    "e::random",
/* 302 */    "e::srandom",
/* 303 */    "e::usleep",
/* 304 */    "peek-char",
/* 305 */    "e::system",
/* 306 */    "c::bound-instance-in-environment?",
/* 307 */    "e::multiple-values?",
/* 308 */    "c::environment-n-down",
/* 309 */    "c::top-level-environment-ref-symbol",
/* 310 */    "e::macro-body",
/* 311 */    "=>",
/* 312 */    "c::top-level-environment",
/* 313 */    "c::null-environment",
/* 314 */    "c::scheme-report-environment",
/* 315 */    "c::eval-in-environment",
/* 316 */    "c::down-in-flames!!!",
/* 317 */    "e::set-compiler-on!",
/* 318 */    "e::clear-compiler-on!",
/* 319 */    "e::set-show-full-precision!",
/* 320 */    "e::clear-show-full-precision!",
/* 321 */    "e::compiler-on?",
/* 322 */    "e::show-full-precision?",
/* 323 */    "e::momcat?",
/* 324 */    "e::kitten-number",
/* 325 */    "e::number-of-kittens",
/* 326 */    "e::show-kittens",
/* 327 */    "e::show-locks",
/* 328 */    "c::block",
/* 329 */    "c::release-block",
/* 330 */    "e::block-symbol-binding",
/* 331 */    "c::kitten-input-queue",
/* 332 */    "c::kitten-dequeue",
/* 333 */    "c::kitten-purge-queue",
/* 334 */    "c::map-without-type-checking",
/* 335 */    "c::set-blocked-binding!",
/* 336 */    "e::process-id",
/* 337 */    "c::release-locks",
/* 338 */    "c::let-deadlock-time-out",
/* 339 */    "c::kitten-empty-queue?",
/* 340 */    "e::tell-kitten-0",
/* 341 */    "e::tell-kitten-1",
/* 342 */    "e::tell-kitten-2",
/* 343 */    "e::tell-kitten-3",
/* 344 */    "e::tell-kitten-4",
/* 345 */    "e::tell-kitten-5",
/* 346 */    "e::tell-kitten-6",
/* 347 */    "e::tell-kitten-7",
/* 348 */    "e::tell-kitten-8",
/* 349 */    "e::tell-kitten-9",
/* 350 */    "e::tell-kitten-10",
/* 351 */    "e::tell-kitten-11",
/* 352 */    "e::tell-kitten-12",
/* 353 */    "e::tell-kitten-13",
/* 354 */    "e::tell-kitten-14",
/* 355 */    "e::tell-kitten-15",
/* 356 */    "e::tell-kitten",
/* 357 */    "c::socket-create-and-listen",
/* 358 */    "c::socket-accept",
/* 359 */    "c::socket-write",
/* 360 */    "c::socket-fgets",
/* 361 */    "c::open-input-socket",
/* 362 */    "e::ninety-percent-used?",
/* 363 */    "c::disable-window-output",
/* 364 */    "c::enable-window-output",
/* 365 */    "c::window-output-enabled?",
/* 366 */    "c::kitten-reset",
/* 367 */    "e::make-forgettable",
/* 368 */    "e::forgettable?",
/* 369 */    "e::forgettable-forgotten?",
/* 370 */    "e::forgettable-remembered?",
/* 371 */    "e::forgettable-unexpired?",
/* 372 */    "e::forgettable-object",
/* 373 */    "e::forgettable-expiration-time",
/* 374 */    "e::forgettable-remember-probability",
/* 375 */    "e::set-forgettable-expiration-time!",
/* 376 */    "e::set-forgettable-remember-probability!",
/* 377 */    "e::main",                         // To execute after a world load.
/* 378 */    "c::ellipsis?",
/* 379 */    "c::pattern-datum?",
/* 380 */    "c::pattern-identifier?",
/* 381 */    "c::pattern-keyword",
/* 382 */    "c::lambda-pair?",
/* 383 */    "c::tack-on-directions",
/* 384 */    "c::top-level-environment-ref?",
/* 385 */    "c::quoted-form?",
/* 386 */    "e::make-memory-mapped-block",
/* 387 */    "e::memory-map-block-in-current-kitten",
/* 388 */    "e::memory-mappable-space-size",
/* 389 */    "e::peek8",
/* 390 */    "e::peek16",
/* 391 */    "e::peek32",
/* 392 */    "e::peek64",
/* 393 */    "e::poke8",
/* 394 */    "e::poke16",
/* 395 */    "e::poke32",
/* 396 */    "e::poke64",
/* 397 */    "e::memory-mapped-block?",
/* 398 */    "c::interrupt-handler-vector",
/* 399 */    "c::set-interrupt-handler!",
/* 400 */    "c::interrupt",
/* 401 */    "c::clear-interrupt-handler!",
/* 402 */    "e::interrupt-file-information",
/* 403 */    "e::peek8-sign-extend",
/* 404 */    "e::peek16-sign-extend",
/* 405 */    "e::peek32-sign-extend",
/* 406 */    "e::peek-c-string",
/* 407 */    "e::poke-c-string",
/* 408 */    "e::peek64-float",
/* 409 */    "e::poke64-float",
/* 410 */    "e::peek8-set-vector!",
/* 411 */    "e::peek16-set-vector!",
/* 412 */    "e::peek32-set-vector!",
/* 413 */    "e::peek64-set-vector!",
/* 414 */    "e::poke8-vector",
/* 415 */    "e::poke16-vector",
/* 416 */    "e::poke32-vector",
/* 417 */    "e::poke64-vector",
/* 418 */    "e::peek8-set-vector-sign-extend!",
/* 419 */    "e::peek16-set-vector-sign-extend!",
/* 420 */    "e::peek32-set-vector-sign-extend!",
/* 421 */    "e::peek64-float-set-vector!",
/* 422 */    "e::poke64-float-vector",
/* 423 */    "c::load-from-string",
/* 424 */    "e::read-string-with-prompt",
/* 425 */    "e::peek-poke-atomic",
/* 426 */    "e::block-any-binding",
/* 427 */    "e::c-boolean->scheme-boolean",
/* 428 */    "e::scheme-boolean->c-boolean",
/* 429 */    "e::active-room",
/* 430 */    "e::aging-room",
/* 431 */    "e::active-store-size",
/* 432 */    "e::show-active-memory",
/* 433 */    "e::show-aging-memory",
/* 434 */    "e::alarm",
/* 435 */    "e::set-alarm!",
/* 436 */    "e::alarm?",
/* 437 */    "e::alarm-time",
/* 438 */    "c::clear-interrupt-requested!",
/* 439 */    "e::non-printing-object?",
/* 440 */    "e::set-sense-light-n-illumination!",
/* 441 */    "e::set-sense-light-n-visibility!",
/* 442 */    "e::sense-light-number?",
/* 443 */    "e::sense-light-illumination-number?",
/* 444 */    "e::show-sense-lights!",
/* 445 */    "e::hide-sense-lights!",
/* 446 */    "e::set-sense-lights-off!",
/* 447 */    "e::set-sense-light-illuminations!",
/* 448 */    "e::rotate-sense-lights!",
/* 449 */    "c::set!",
/* 450 */    "c::applicative-mode-check-set-bang",
/* 451 */    "c::applicative-mode-check",
/* 452 */    "c::applicative-bit?",
/* 453 */    "c::set-applicative-bit!",
/* 454 */    "c::set-applicative-bit-n-down!",
/* 455 */    "c::clear-applicative-bit!",
/* 456 */    "c::set-tag!",
/* 457 */    "c::clear-applicative-bit-n-down!",
/* 458 */    "e::sense-switches",
/* 459 */    "e::slider",
/* 460 */    "e::enable-pushbutton!",
/* 461 */    "e::pushbutton-interrupt-number",
/* 462 */    "e::disable-pushbutton!",
/* 463 */    "e::enable-big-red-button!",
/* 464 */    "e::big-red-button-interrupt-number",
/* 465 */    "e::disable-big-red-button!",
/* 466 */    "e::pushbutton-enabled?",
/* 467 */    "e::big-red-button-enabled?",
/* 468 */    "e::set-level-indicator!",
/* 469 */    0,
    };

const Text *TypeNameStrings[] = {
/*   0 */    "Dummy",
/*   1 */    "Any Scheme object",
/*   2 */    "List",
/*   3 */    "Number",
/*   4 */    "Character",
/*   5 */    "Integer that will fit into a fixnum",
/*   6 */    "Input Port",
/*   7 */    "Integer",
/*   8 */    "Lambda Procedure",
/*   9 */    "Letter",
/*  10 */    "Syntax Binding",
/*  11 */    "Macro",
/*  12 */    "Output Port",
/*  13 */    "Procedure",
/*  14 */    "Promise",
/*  15 */    "Real Number",
/*  16 */    "String",
/*  17 */    "Symbol",
/*  18 */    "Vector",
/*  19 */    "Takes %d arguments.  ",
/*  20 */    "Takes %d argument: ",
/*  21 */    "Argument %d: %s.  ",
/*  22 */    "Arguments %d and %d: %s.  ",
/*  23 */    "Arguments %d ... : %s.  ",
/*  24 */    "Takes %d or more arguments.  ",
/*  25 */    "Takes %d to %d arguments.  ",
/*  26 */    "Takes %d or %d arguments.  ",
/*  27 */    "Takes %d or %d argument.  ",
/*  28 */     "All arguments: %s.  ",
/*  29 */     "Arguments %d ... %d: %s.  ",
/*  30 */     "%s.  ",
/*  31 */     "Final argument: %s.  ",
/*  32 */     "Preceding argument (if present): %s.  ",
/*  33 */     "Preceding arguments (when present): %s.  ",
/*  34 */     "\n    Returns no useful value (specifically, returns #t).  ",
/*  35 */     "\n    Type of value returned:  %s.  ",
/*  36 */     "Pair",
/*  37 */     "Both arguments: %s.  ",
/*  38 */     "Boolean",
/*  39 */     "Exact integer stored as 32-bit fixnum",
/*  40 */     "Nonnegative integer that will fit into a fixnum",
/*  41 */     "Reference to variable in non-top-level environment",
/*  42 */     "Reference to variable in top-level environment",
/*  43 */     "Proper list",
/*  44 */     "#f or ProperList",
/*  45 */     "\n    This procedure returns no value:  It does not return at all.",
/*  46 */     "Multiple Values",
    };

const Text *ObjectErrorNameStrings[] = {
#ifdef PIXIE_SCHEME_III
/*  0 */  "any Pixie Scheme III Object",
#else
#ifdef PIXIE_SCHEMES
/*  0 */  "any Pixie Scheme II Object",
#else
/*  0 */  "any Wraith Scheme Object",
#endif
#endif
/*  1 */  "an integer",
/*  2 */  "a probability",
/*  3 */  "a list",
/*  4 */  "a proper list",
/*  5 */  "a character",
/*  6 */  "a reference to an binding in a non-top-level environment",
/*  7 */  "an exact integer",
/*  8 */  "a forgettable object",
/*  9 */  "an input port",
/* 10 */  "a nonnegative integer",
/* 11 */  "a string",
/* 12 */  "an integer in the range of interrupt numbers",
/* 13 */  "an integer in the range of kitten numbers",
/* 14 */  "a lambda expression",
/* 15 */  "a character which is a letter of the ASCII alphabet",
/* 16 */  "a macro",
/* 17 */  "a memory-mapped block",
/* 18 */  "a number",
/* 19 */  "a vector",
#ifdef PIXIE_SCHEME_III
/* 20 */  "a Pixie Scheme III object that is not an atom",
#else
#ifdef PIXIE_SCHEMES
/* 20 */  "a Pixie Scheme II object that is not an atom",
#else
/* 20 */  "a Wraith Scheme object that is not an atom",
#endif
#endif
/* 21 */  "a pair",
/* 22 */  "an output port",
/* 23 */  "a procedure (either a built-in or a lambda expression)",
/* 24 */  "a promise",
/* 25 */  "a real number",
/* 26 */  "an integer in the range of 64-bit signed fixnums",
/* 27 */  "an integer in the range of 64-bit signed fixnums, that is nonnegative",
/* 28 */  "a symbol",
/* 29 */  "a reference to an binding in a top-level environment",
/* 30 */  "a radix recognized by Scheme (2, 8, 10 or 16)",
/* 31 */  "a positive integer",
/* 32 */  "an integer in the range valid for sense-light numbers",
/* 33 */  "an integer in the range valid for sense-light illumination numbers",
/* 34 */  "a boolean",
/* 35 */  "an integer in the range valid for slider numbers",
/* 36 */  "an integer in the range valid for pushbutton numbers",
/* 37 */  "an integer in the range valid for level indicator numbers",
/* 38 */  "an integer in the range valid for level indicator values",
    };

