/****************************************************************************************

PutBack.c++ -- Copyright (c) by Jay Reynolds Freeman 2009.

Routines to manage a non-trivial putback buffer for input text.
This material is in a separate file because the same buffer mechanism
will be used in CocoaIO.c++, CursesIO.c++, and TTYIO.c++.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import <stdlib.h>
#import <stdio.h>
#import <string.h>
#import <setjmp.h>
#import <ctype.h>
#import <sys/types.h>
#import <unistd.h>

#import "LocalTypes.h"
#import "WraithHeader.h"
#import "MemoryManager.h"
#import "ASCIITypes.h"
#import "DataTypes.h"

#import "Globals.e"
#import "Events.e"

#define PB_BUFSIZE 2048

static Text *PBStack = 0;
static UnsignedLongWord PBn = 0;

static size_t bufferSize = PB_BUFSIZE;

static Text *newPBStack = 0;

/**** initializePutBack ****/

void initializePutBack()
{
    PBStack = (Text *)calloc( bufferSize, sizeof( Byte ) );
    if( PBStack == 0 )
        fatalError( 51 );

    PBn = 0;
    }

/**** reinitializePutBack ****/

void reinitializePutBack()
{
    PBn = 0;
    }

/**** clearPutBack ****/

void clearPutBack()
{
    PBn = 0;
    }

/**** hasPutBack ****/

Boolean hasPutBack() { return( PBn > 0 ); }

/**** getPutBack ****/

Word getPutBack() { return PBStack[ --PBn ]; }

/**** untgetchar ****/

void untgetchar( Text c )
{
    eventLoopOnce( 0 );

    if( PBn >= bufferSize ) {  // Make larger buffer and copy over.
        newPBStack = (Text *)calloc( 2 * bufferSize, sizeof( Byte ) );
        if( newPBStack == 0 )
            fatalError( 51 );
        memcpy( newPBStack, PBStack, bufferSize * sizeof( Byte ) );
        free( PBStack );
        PBStack = newPBStack;
        bufferSize =  2 * bufferSize;
        }
    PBStack[ PBn++ ] = c;
    }    

