/****************************************************************************************

SECDEval.c -- Copyright (c) by Jay Reynolds Freeman 1988, 1989, 1990, 1991, 2006

The evaluator, and major auxiliaries, for Wraith Scheme.
NOTHING in this file should reference any of the macros in "OpCodes.h".

        "I can summon spirits from the vasty deep."

        "Why, so can I; and so can any man."
        
        "But will they come when you do summon them?"

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import "WraithHeader.h"
#import "MemoryManager.h"
#import "Registers.h"
#import "SECD.h"
#import "Globals.e"
#import "Apply.e"
#import "DataTypes.e"
#import "FunctionAndSpecialNames.e"
#import "Intern.e"
#import "IO.e"
#import "MainAux.e"
#import "Registers.e"
#import "SECDFloatStuff.e"
#import "SECDMachine.e"
#import "SECDRegisters.e"
#import "SchemeFunctions.e"
#import "SchemeGlobals.e"
#import "SchemeSpecials.e"

/**** Globals: ****/

taggedAval *stackLowLimit;
taggedAval *continuationLowLimit;

/**** Statics: ****/

static const int maxArgsToDisplay = 7;

/**** Functions: ****/

/**** Some forward declarations: ****/

Boolean lambdaIsInterpretedP();
void arityCheckBuiltIn(),
     builtInTypeChecker(),
     evalAtom(),
     evalSpecialFormArgs(),
     eval(),
     loadContinuation(),
     macroPartTwo();

/**** apply -- Expects to find:
                              
              R: <don't care>; S:(procedure nArgs arg* ...)
                          
              This function switches on the type of the function,
              arity- and type-checks, then does the application.    ****/

void apply()
{
    pFVoid codeToCall;


    switch( pure_type_code( S[0].t ) ) {
        case lambdaPairTag:
            applyLambda();
            break;
        case builtInFunctionTag:
            R = S[0];
            getBuiltInCodeIndirect();
            codeToCall = (pFVoid)(R.u.c);
            arityCheckBuiltIn();
            builtInTypeChecker();
            codeToCall();
            break;
        default:
            SPopR();
            useErrorOutput();
            displayObject();
            resetError( 145 );
        }
    }

/**** arityCheck -- This function is passed a list and the length the caller thinks it
                    ought to have.  It returns  the actual length in R and sets B to
                    indicate whether or not the length of the list is what it ought to be.
                    It is up to the caller to decide what to do with this information.   ****/

void arityCheck()       /* R: lnthWanted, S: (list ...) -> R: lnth, S: (lnthWanted list ...), sets B */
{ 
    SPushR();
    R = S[1];
    lengthOp();
    B = ( R.u.c == S[0].u.c);
    }

/**** arityCheckTypeOne -- Check that the number of arguments to a type-one 
                           built-in function is correct, and trap if it is not.                  ****/

void arityCheckTypeOne( LongWord nWanted, LongWord nArgs )      /* R: <don't care>; S:(builtIn ...) */
{ 
    LongWord consCounter = nArgs;

    if( nWanted != nArgs ) {
        SPopR();
        useErrorOutput();
        displayObject();
        tprintf2( &CurrentOutputPort, MiscStrings[ 31 ] );
        SDrop();
        R.t = emptyListTag;
        while( consCounter-- > 0 )
            cons();
        writeObject();
        if( nWanted != 1)
            tprintf3( &CurrentOutputPort, MiscStrings[ 6 ], nWanted );
        else
            tprintf3( &CurrentOutputPort, MiscStrings[ 235 ], nWanted );
        resetError( 147 );
        }
    }

/**** arityCheckTypeTwo -- Check that the number of arguments to a type-two, four or
                           five built-in function is correct, and trap if it is not.                     ****/

void arityCheckTypeTwo( LongWord nWantedMin, LongWord nArgs )   /* R: <don't care>; S:(builtIn ...) */
{
    LongWord consCounter = nArgs;

    if( nArgs < nWantedMin ) {
        SPopR();
        useErrorOutput();
        displayObject();
        tprintf2( &CurrentOutputPort, MiscStrings[ 31 ] );
        SDrop();
        R.t = emptyListTag;
        while( consCounter-- > 0 )
            cons();
        writeObject();
        if( nWantedMin == 1 )
            tprintf3( &CurrentOutputPort, MiscStrings[ 80 ], nWantedMin );
        else
            tprintf3( &CurrentOutputPort, MiscStrings[ 81 ], nWantedMin );
        resetError( 148 );
        }
    }

/**** arityCheckTypeSix -- Check that the number of arguments to a type-six
                           built-in function is correct, and trap if it is not.          ****/

void arityCheckTypeSix( LongWord nArgsMax, LongWord nArgs )             /* R: <don't care>; S:(builtIn ...) */
{
    Boolean infiniteArgsP;
    LongWord consCounter = nArgs;

    infiniteArgsP = (nArgsMax == -1);
    if( nArgs == 0 ) {
        SPopR();
        useErrorOutput();
        displayObject();
        tprintf2( &CurrentOutputPort, MiscStrings[ 32 ] );
        resetError( 148 );
        }
    if( (! infiniteArgsP) && (nArgs > nArgsMax) ) {
        SPopR();
        useErrorOutput();
        displayObject();
        tprintf2( &CurrentOutputPort, MiscStrings[ 31 ] );
        SDrop();
        R.t = emptyListTag;
        while( consCounter-- > 0 )
            cons();
        writeObject();
        if( nArgsMax != 1 )
            tprintf3( &CurrentOutputPort, MiscStrings[ 82 ], nArgsMax );
        else
            tprintf3( &CurrentOutputPort, MiscStrings[ 197 ], nArgsMax );
        resetError( 149 );
        }
    }

/**** arityCheckTypeSeven -- Check that the number of arguments to a type-seven or eight
                             built-in function is correct, and trap if it is not.                ****/

void arityCheckTypeSeven( LongWord nArgsMax, LongWord nArgsMin, LongWord nArgs )   /* R: <don't care>; S:(builtIn ...) */
{
    Boolean infiniteArgsP;
    LongWord consCounter = nArgs;

    infiniteArgsP = (nArgsMax == -1);
    if( nArgs < nArgsMin ) {
        SPopR();
        useErrorOutput();
        displayObject();
        tprintf2( &CurrentOutputPort, MiscStrings[ 31 ] );
        SDrop();
        R.t = emptyListTag;
        while( consCounter-- > 0 )
            cons();
        writeObject();
        if( nArgsMin == 1 )
            tprintf3( &CurrentOutputPort, MiscStrings[ 200 ], nArgsMin );
        else
            tprintf3( &CurrentOutputPort, MiscStrings[ 190 ], nArgsMin );
        resetError( 148 );
        }
    if( (! infiniteArgsP) && (nArgs > nArgsMax) ) {
        SPopR();
        useErrorOutput();
        displayObject();
        tprintf2( &CurrentOutputPort, MiscStrings[ 31 ] );
        SDrop();
        R.t = emptyListTag;
        while( consCounter-- > 0 )
            cons();
        writeObject();
        if( nArgsMax != 1 )
            tprintf3( &CurrentOutputPort, MiscStrings[ 82 ], nArgsMax );
        else
            tprintf3( &CurrentOutputPort, MiscStrings[ 197 ], nArgsMax );
        resetError( 149 );
        }
    }

/**** arityCheckTypeNine -- Check that the number of arguments to a type-nine
                            built-in function is correct, and trap if it is not.          ****/

void arityCheckTypeNine( LongWord nArgsMax, LongWord nArgs )             /* R: <don't care>; S:(builtIn ...) */
{
    Boolean infiniteArgsP;
    LongWord consCounter = nArgs;

    infiniteArgsP = (nArgsMax == -1);
    if( (! infiniteArgsP) && (nArgs > nArgsMax) ) {
        SPopR();
        useErrorOutput();
        displayObject();
        tprintf2( &CurrentOutputPort, MiscStrings[ 31 ] );
        SDrop();
        R.t = emptyListTag;
        while( consCounter-- > 0 )
            cons();
        writeObject();
        if( nArgsMax != 1 )
            tprintf3( &CurrentOutputPort, MiscStrings[ 82 ], nArgsMax );
        else
            tprintf3( &CurrentOutputPort, MiscStrings[ 197 ], nArgsMax );
        resetError( 149 );
        }
    }

/**** arityCheckBuiltIn -- Check that the number of arguments passed to a built-in function
                           are correct.  This code is mostly a big switch on the type of
                           the built-in function.  Returns the function type in R.    ****/

void arityCheckBuiltIn()        /* R: <don't care>; S: (builtIn nArgs ...) */
{
    LongWord nArgs, nWanted1, nWanted2, type;

    nArgs = (UnsignedLongWord)(S[1].u.c);
    R = S[0];
    getBuiltInType();
    type = R.u.c;
    R = S[0];
    switch( type ) {                                        /* switch on type of function */
        case FIXED_ARITY:
            getBuiltInNArgs();
            nWanted1 = R.u.c;
            arityCheckTypeOne( nWanted1, nArgs );
            break;
        case ANY_ARITY_ONE_TYPE_STACK:
            getBuiltInMinArgs();
            nWanted1 = R.u.c;
            R = S[0];
            arityCheckTypeTwo( nWanted1, nArgs );
            break;
        case POSITIVE_ARITY_TWO_TYPES_STACK:
            getBuiltInMaxArgs();
            nWanted1 = R.u.c;
            R = S[0];
            arityCheckTypeSix( nWanted1, nArgs );
            break;
        case ANY_ARITY_TWO_TYPES_STACK:
        case ANY_ARITY_TWO_TYPES_STACK_2:
            getBuiltInMinArgsSeven();
            nWanted1 = R.u.c;
            R = S[0];
            getBuiltInMaxArgs();
            nWanted2 = R.u.c;
            R = S[0];
            arityCheckTypeSeven( nWanted2, nWanted1, nArgs );
            break;
        case NONNEGATIVE_ARITY_TWO_TYPES_STACK:
            getBuiltInMaxArgs();
            nWanted1 = R.u.c;
            R = S[0];
            arityCheckTypeNine( nWanted1, nArgs );
            break;
        default:
            useErrorOutput();
            tprintf3( &CurrentOutputPort, MiscStrings[ 94 ], type );
            displayObject();
            resetError( 144 );
            break;
        }
    R.t = exactShortFixnumBitsMask;
    R.u.c = type;
    }

/**** builtInTypeCheckerDisplayAux1 -- Display auxiliary for several type checkers. ****/

void builtInTypeCheckerDisplayAux1( LongWord nArgs )              /* On entry: S: (builtIn arg* ...) */
{
    LongWord consCounter = nArgs;

    SPopR();                                                 // R: builtIn; S: arg* ...
    SPushR();                                                // R: builtIn; S: builtIn, arg* ...
    displayObject();
    SPopR();                                                 // R: builtIn; S: arg* ...
    tprintf2( &CurrentOutputPort, MiscStrings[ 31 ] );
    SPushR();                                                // R: builtIn; S: builtIn, arg* ...
    R.t = emptyListTag;
    while( consCounter-- > 0 ) {
        exchangeFunction( &(S[0]), &(S[1]) );
        cons();
        }
    SPushR();
    writeObject();                                           // R: List of args, S: List of args, builtIn ... */
    tprintf2( &CurrentOutputPort, MiscStrings[ 450 ] );
    SPopR();                                                 // R: builtIn; S: List of args ...
    }       

/**** builtInTypeCheckerDisplayAux3 -- Display auxiliary for several type checkers. ****/

void builtInTypeCheckerDisplayAux3( LongWord nArgs, LongWord nBad )
{
    if( nArgs == 1)
        resetError( 151 );
    else if( nBad == 1 )
        resetError( 152 );
    else
        resetError( 153 );
    }       

/**** builtInTypeCheckerTypeOne -- Perform type-checking for the (evaled) arguments of a
                                   type one built-in function.  All arguments are
                                   type-checked, even after one or more have failed.       ****/

void builtInTypeCheckerTypeOne()                /* on entry, R: builtIn; S:(nArgs arg* ...) */
{
    LongWord nArgs, argumentChecking, argumentPrinting;
    LongWord nBad = 0;
    Boolean badType[ 3 ];
    pFVoid checker; 

    nArgs = S[0].u.c;
    S[0] = R;                                               /* R: builtin, S:(builtIn arg* ...) */
    for( argumentChecking = 0; argumentChecking < nArgs; ++argumentChecking ) {

        /* This loop performs type-checking. */

        R = S[0];
        SPushR();               
        R.t = exactShortFixnumBitsMask;
        R.u.c = argumentChecking;
        getBuiltInNthChecker();
        checker = (pFVoid)(R.u.c);
        R = S[ nArgs - argumentChecking ];
        (*checker)();
        badType[ argumentChecking ] = !B;
        if( ! B )
            ++nBad;
        }
    if( nBad == 0 ) {
        SDrop();
        if( nArgs > 0 )
            SPopR();
        }
    else {

        /* Output a succinct display of all arguments, with the
           ones that have the wrong type marked with an "->".   */

        useErrorOutput();
        builtInTypeCheckerDisplayAux1( nArgs );
        SPushR();
        for( argumentPrinting = 0; argumentPrinting < nArgs; ++argumentPrinting ) {
            car();
            if( badType[ argumentPrinting ] ) {
                if( nArgs != 1) {
                    tprintf2( &CurrentOutputPort, MiscStrings[ 114 ] );
                    writeObject();
                    }
                tprintf2( &CurrentOutputPort, MiscStrings[ 447 ] );
                R = S[ 1 ];
                SPushR();                
                R.t = exactShortFixnumBitsMask;
                R.u.c = argumentPrinting;
                getBuiltInNthArgName();
                displayObject();
                if( nArgs == 1)
                    tprintf2( &CurrentOutputPort, MiscStrings[ 449 ] );
                else
                    tprintf3( &CurrentOutputPort, MiscStrings[ 448 ], argumentPrinting );
                }
            SPopR();
            cdr();
            SPushR();
            }
        SDrop();
        builtInTypeCheckerDisplayAux3( nArgs, nBad );
        }
    }

/**** builtInTypeCheckerTypeFive -- Perform type-checking for the (evaled) arguments of a
                                    type five built-in function.  The arguments are
                                    provided stacked, and are type-checked in turn.  All
                                    are type-checked, even after one or more have failed.
                                    If any type-check fails, the present function traps.

                                    ****/

void builtInTypeCheckerTypeFive()                       /* on entry, R: builtIn; S:(nArgs arg* ...) */
{
    LongWord nArgs, argumentChecking;
    LongWord nBad = 0;
    pFVoid checker; 

    nArgs = S[0].u.c;
    if( nArgs == 0 ) {
        SPopR();
        return;
        }
    SPushR();
    getBuiltInChecker();
    checker = (pFVoid)(R.u.c);                                      /* R: <don't care>, S:(builtIn nArgs arg* ...) */
    for( argumentChecking = 0; argumentChecking < nArgs; ++argumentChecking ) {
        R = S[ nArgs - argumentChecking + 1 ];
        (*checker)();
        if( !B )
            ++nBad;
        }
    if( nBad == 0 ) {
        SDrop();
        SPopR();
        }
    else {
        useErrorOutput();
        SPopR();
        SDrop();
        SPushR();
        builtInTypeCheckerDisplayAux1( nArgs );
        SPushR();
        for( argumentChecking = 0; argumentChecking < nArgs; ++argumentChecking ) {
            car();
            (*checker)();
            if( ! B ) {                                                 /* display the bad object */
                if( nArgs != 1 ) {
                    tprintf2( &CurrentOutputPort, MiscStrings[ 114 ] );
                    writeObject();
                    }
                tprintf2( &CurrentOutputPort, MiscStrings[ 447 ] );
                R = S[ 1 ];
                getBuiltInArgName();
                displayObject();
                if( nArgs == 1)
                    tprintf2( &CurrentOutputPort, MiscStrings[ 449 ] );
                else
                    tprintf3( &CurrentOutputPort, MiscStrings[ 448 ], argumentChecking );
                }
            SPopR();
            cdr();
            SPushR();
            }
        SDrop();
        builtInTypeCheckerDisplayAux3( nArgs, nBad );
        }
    return;
    }

/**** builtInTypeCheckerTypeSix -- Perform type-checking for the (evaled) arguments of a
                                   type six built-in function.  The arguments are
                                   provided stacked, and are type-checked in turn.  All
                                   are type-checked, even after one or more have failed.
                                   If any type-check fails, the present function traps.

                                   Also used for type nine built-in functions.

                                   ****/

void builtInTypeCheckerTypeSix()                /* on entry, R: builtIn; S:(nArgs arg* ...) */
{
    LongWord nArgs, argumentChecking;
    LongWord nBad = 0;
    pFVoid firstChecker, restChecker;       

    nArgs = S[0].u.c;
    if( nArgs == 0 ) {
        SPopR();
        return;
        }
    SPushR();
    getBuiltInFirstChecker();
    firstChecker = (pFVoid)(R.u.c);
    R = S[0];
    getBuiltInRestChecker();
    restChecker = (pFVoid)(R.u.c);          /* R: <don't care>, S:(builtIn nArgs arg* ...) */
    for( argumentChecking = 0; argumentChecking < nArgs; ++argumentChecking ) {
        R = S[ nArgs - argumentChecking + 1 ];
        if( argumentChecking == 0 )
            (*firstChecker)();
        else
            (*restChecker)();
        if( ! B )
            ++nBad;
        }
    if( nBad == 0 ) {
        SDrop();
        SPopR();
        }
    else {
        useErrorOutput();
        SPopR();
        SDrop();
        SPushR();
        builtInTypeCheckerDisplayAux1( nArgs );
        SPushR();
        for( argumentChecking = 0; argumentChecking < nArgs; ++argumentChecking ) {
            car();
            if( argumentChecking == 0 )
                (*firstChecker)();
            else
                (*restChecker)();
            if( ! B ) {                                             /* display the bad object */
                if( nArgs != 1 ) {
                    tprintf2( &CurrentOutputPort, MiscStrings[ 114 ] );
                    writeObject();
                    }
                tprintf2( &CurrentOutputPort, MiscStrings[ 447 ] );
                R = S[ 1 ];
                if( argumentChecking == 0 )
                    getBuiltInFirstArgName69();
                else
                    getBuiltInRestArgName69();
                displayObject();
                if( nArgs == 1)
                    tprintf2( &CurrentOutputPort, MiscStrings[ 449 ] );
                else
                    tprintf3( &CurrentOutputPort, MiscStrings[ 448 ], argumentChecking );
                }
            SPopR();
            cdr();
            SPushR();
            }
        SDrop();
        builtInTypeCheckerDisplayAux3( nArgs, nBad );
        }
    return;
    }

/**** builtInTypeCheckerTypeSeven -- Perform type-checking for the (evaled) arguments of a
                                     type seven built-in function.  The arguments are
                                     provided stacked, and are type-checked in turn.  All
                                     are type-checked, even after one or more have failed.
                                     If any type-check fails, the present function traps.

                                     ****/

void builtInTypeCheckerTypeSeven()              /* on entry, R: builtIn; S:(nArgs arg* ...) */
{
    LongWord nArgs, argumentChecking;
    LongWord nBad = 0;
    pFVoid firstChecker, lastChecker;       

    nArgs = S[0].u.c;
    if( nArgs == 0 ) {
        SPopR();
        return;
        }
    SPushR();
    getBuiltInFirstChecker();
    firstChecker = (pFVoid)(R.u.c);
    R = S[0];
    getBuiltInRestChecker();
    lastChecker = (pFVoid)(R.u.c);          /* R: <don't care>, S:(builtin nArgs arg* ...) */
    for( argumentChecking = 0; argumentChecking < nArgs; ++argumentChecking ) {
        R = S[ nArgs - argumentChecking + 1];
        if( argumentChecking == 0 )
            (*firstChecker)();
        else
            (*lastChecker)();
        if( ! B )
            ++nBad;
        }
    if( nBad == 0 ) {
        SDrop();
        SPopR();
        }
    else  {
        useErrorOutput();
        SPopR();
        SDrop();
        SPushR();
        builtInTypeCheckerDisplayAux1( nArgs );
        SPushR();
        for( argumentChecking = 0; argumentChecking < nArgs; ++argumentChecking ) {
            car();
            if( argumentChecking == 0 )
                (*firstChecker)();
            else
                (*lastChecker)();
            if( ! B ) {                         /* display the bad object */
                if( nArgs != 1 ) {
                    tprintf2( &CurrentOutputPort, MiscStrings[ 114 ] );
                    writeObject();
                    }
                tprintf2( &CurrentOutputPort, MiscStrings[ 447 ] );
                R = S[ 1 ];
                if( argumentChecking == 0 )
                    getBuiltInFirstArgName78();
                else
                    getBuiltInRestArgName78();
                displayObject();
                if( nArgs == 1)
                    tprintf2( &CurrentOutputPort, MiscStrings[ 449 ] );
                else
                    tprintf3( &CurrentOutputPort, MiscStrings[ 448 ], argumentChecking );
                }
            SPopR();
            cdr();
            SPushR();
            }
        SDrop();
        builtInTypeCheckerDisplayAux3( nArgs, nBad );
        }
    return;
    }

/**** builtInTypeCheckerTypeEight -- Perform type-checking for the (evaled) arguments of a
                                     type eight built-in function.  The arguments are
                                     provided stacked, and are type-checked in turn.  All
                                     are type-checked, even after one or more have failed.
                                     If any type-check fails, the present function traps.

                                     ****/

void builtInTypeCheckerTypeEight()              /* on entry, R: builtIn; S:(nArgs arg* ...) */
{
    LongWord nArgs, argumentChecking;
    LongWord nBad = 0;
    pFVoid firstChecker, lastChecker;       

    nArgs = S[0].u.c;
    if( nArgs == 0 ) {
        SPopR();
        return;
        }
    SPushR();
    getBuiltInFirstChecker();
    firstChecker = (pFVoid)(R.u.c);
    R = S[0];
    getBuiltInRestChecker();
    lastChecker = (pFVoid)(R.u.c);          /* R: <don't care>, S:(builtin nArgs arg* ...) */
    for( argumentChecking = 0; argumentChecking < nArgs; ++argumentChecking ) {
        R = S[ nArgs - argumentChecking + 1];
        if( argumentChecking != nArgs - 1 )
            (*firstChecker)();
        else
            (*lastChecker)();
        if( ! B )
            ++nBad;
        }
    if( nBad == 0 ) {
        SDrop();
        SPopR();
        }
    else  {
        useErrorOutput();
        SPopR();
        SDrop();
        SPushR();
        builtInTypeCheckerDisplayAux1( nArgs );
        SPushR();
        for( argumentChecking = 0; argumentChecking < nArgs; ++argumentChecking ) {
            car();
            if( argumentChecking != nArgs - 1 )
                (*firstChecker)();
            else
                (*lastChecker)();
            if( ! B ) {                         /* display the bad object */
                if( nArgs != 1 ) {
                    tprintf2( &CurrentOutputPort, MiscStrings[ 114 ] );
                    writeObject();
                    }
                tprintf2( &CurrentOutputPort, MiscStrings[ 447 ] );
                R = S[ 1 ];
                if( argumentChecking != nArgs - 1 )
                    getBuiltInFirstArgName78();
                else
                    getBuiltInRestArgName78();
                displayObject();
                if( nArgs == 1)
                    tprintf2( &CurrentOutputPort, MiscStrings[ 449 ] );
                else
                    tprintf3( &CurrentOutputPort, MiscStrings[ 448 ], argumentChecking );
                }
            SPopR();
            cdr();
            SPushR();
            }
        SDrop();
        builtInTypeCheckerDisplayAux3( nArgs, nBad );
        }
    return;
    }

/**** builtInTypeChecker -- Dispatch the appropriate type-checker for the function.      ****/

void builtInTypeChecker()               /* on entry, R: type; S:(builtIn nArgs arg* ...) */
{
    LongWord type = R.u.c;

    SPopR();
    switch( type ) {
        case FIXED_ARITY:
            builtInTypeCheckerTypeOne();
            break;
        case ANY_ARITY_ONE_TYPE_STACK:
            builtInTypeCheckerTypeFive();
            break;
        case POSITIVE_ARITY_TWO_TYPES_STACK:
        case NONNEGATIVE_ARITY_TWO_TYPES_STACK:      // Fall-through is intentional.
            builtInTypeCheckerTypeSix();
            break;
        case ANY_ARITY_TWO_TYPES_STACK:
            builtInTypeCheckerTypeSeven();
            break;
        case ANY_ARITY_TWO_TYPES_STACK_2:
            builtInTypeCheckerTypeEight();
            break;
        default:
            useErrorOutput();
            displayObject();
            resetError( 155 );
        }
    }

/**** lambdaIsInterpretedP -- The entity passed should be a lambda expression,
                              though its tag may have been changed to "pair".           ****/
                                                         
Boolean lambdaIsInterpretedP()
{
    Boolean answer;

    SPushR();
    R.t = pairTag;
    car();
    answer = ! number_p( R.t );
    SPopR();
    return answer;
    }

/**** evalAtom -- Eval an atom.  ****/

void evalAtom()
{ 
    switch( pure_type_code(R.t) ) {
        case symbolTag:
            lookUp();                               /* AKA evalSymbol -- save call overhead */
            break;
        case stackReferenceTag:                     /* automagically dereference */
            dereferenceStackReferenceMC();
            break;
        case environmentReferenceTag:               /* automagically dereference */
            dereferenceEnvironmentReferenceMC();
            break;
        case topLevelEnvironmentReferenceTag:       /* automagically dereference */
            dereferenceTopLevelEnvironmentReferenceMC();
            break;
        case emptyListTag:
        case builtInFunctionTag:
        case characterTag:
        case failTag:
        case falseBooleanTag:
        case lambdaPairTag:
        case macroTag:
        case portTag:
        case specialFormTag:
        case stringTag:
        case succeedTag:
        case trueBooleanTag:
        case peculiarNonLeafTag:
        case exactShortFloatBitsMask:
        case exactShortFixnumBitsMask:
        case inexactShortFloatBitsMask:
        case inexactShortFixnumBitsMask:
            /*  the atom evals to itself, and the desired result is already in R */
            break;
        case peculiarLeafTag:
            if( leaf_number( R ) == nonPrintingObjectTag )
                break;
            // Else intentional fall-through to default case.
        default:
            useErrorOutput();
            SPushR();
            printAnyTaggedAval( &CurrentOutputPort, &R );
            tprintf2( &CurrentOutputPort, MiscStrings[ 115 ] );
            SPopR();
            displayObject();
            resetError( 156 );
            break;
        }
    }

/**** eval -- a non-reentrant (and non-recursive!) evaluator, that never returns ****/

#define EVALS_PER_EVENT_LOOP 128
#define EVALS_PER_MEMORY_DISPLAY 1048576
#define EVALS_PER_GC_CHECK 1024
#define FIRST_PASS_EVAL_SLEEP_DELAY 1000 // microseconds

void eval()
{
    LongWord counter = 0;
    LongWord lengthList = 0;
    LongWord stackOpStatus = 0;
    LongWord EPELCounter = EVALS_PER_EVENT_LOOP;
    LongWord MDELCounter = EVALS_PER_MEMORY_DISPLAY;
    LongWord GCCCounter  = EVALS_PER_GC_CHECK;
    static Boolean firstPass = TRUE;

    if( firstPass ) {  // Do one-time-only initializations.
        theClowder->setKittenEvaluatorActivity( KITTEN_ACTIVITY_READY_TO_EVAL );
        while( ! theClowder->allKittensActiveAt( KITTEN_ACTIVITY_READY_TO_EVAL ) )
            usleep( FIRST_PASS_EVAL_SLEEP_DELAY );
        assertNotEvaluating();
        firstPass = FALSE;
        }

extern int evaluatorHasInitialized;

    evaluatorHasInitialized = 1;

#ifdef USE_COCOA_IO

#import "CocoaInterface.h"

    reportEvent( CURRENT_MEMORY_IN_USE_EVENT );
    reportEvent( CURRENT_GENERATIONAL_MEMORY_IN_USE_EVENT );

#endif

    while( TRUE ) {

        // Administration -- heartbeat counters, stack overflow, et cetera:

        if( EPELCounter-- <= 0 ) {
            eventLoopOnce( 0 );
            EPELCounter = EVALS_PER_EVENT_LOOP;
            GCCCounter  -= EVALS_PER_EVENT_LOOP;
            MDELCounter -= EVALS_PER_EVENT_LOOP;
            if( MDELCounter-- <= 0 ) {
                reportEvent( CURRENT_MEMORY_IN_USE_EVENT );
                reportEvent( CURRENT_GENERATIONAL_MEMORY_IN_USE_EVENT );
                reportEvent( TOTAL_MEMORY_IN_USE_EVENT );
                MDELCounter = EVALS_PER_MEMORY_DISPLAY;
                }
            if( GCCCounter-- <= 0 ) {
                theHeap->triggerGC();
                GCCCounter = EVALS_PER_GC_CHECK;
                }
            }
        if( S < stackLowLimit )
            resetError( 290 );
        if( CS < continuationLowLimit )
            resetError( 284 );

        // End of administration.

        if( emptyCSP() ) {                          // All arguments evaluated; either we are about to run code or we just did ...
            if( stackOpStatus == 1 ) {              //   ... stackOpStatus serves as a flag to tell which.  1 means run code.
                stackOpStatus = 0;                  // Zero will tell the evaluator next time through, to pop the stack
                apply();
                continue;
                }
            else if( stackOpStatus == 0 ) {         // Code has run: Must pop the stack.

                S = F - 1;                          // Reload the Scheme machine stack pointer.
                CS = CF;                            // Reload the continuation stack pointer.
                SPopE();                            // Pop E.
                F = ((S++)->u).a;                   // Pop the Scheme machine stack frame pointer.
                CF = ((S++)->u).a;                  // Pop the continuation stack frame pointer.
                stackOpStatus = ((S++)->u).c;       // Pop stackOpStatus.
                ++S;                                // Skip "L" slot of frame.
                ESR = ((S++)->u).c;                 // Pop the evaluation status register.

                // S now points just below L -- to the slot that does contain, or will contain, the code to run.

                if( stackOpStatus < 0 ) {           // Negative stackOpStatus means we were evaluating the
                                                    //   car of a combination, special form or macro.
                    S[0] = R;                       // put evaled function into S[0] -- on top of where the args will go.
                    stackOpStatus = - stackOpStatus;                    // stackOpStatus is 1 plus the number of args to come.
                    }
                else if( stackOpStatus > 1 )  {     // We are evaluating args, this number shows where to put the one just eval'd.
                    S[ stackOpStatus-- ] = R;       // Store value and then decrement stackOpStatus.
                    }
                // else                             // stackOpStatus is one -- all args have been evaluated -- nothing to do
                                                    //   (what's in slot one is the number of args).
                }
            else {
                useErrorOutput();
                tprintf3( &ErrorPort, "%jd\n", (intmax_t)stackOpStatus );
                resetError( 230 );
                }
            }                                       // End of what to do with empty continuation.
        else {                                      // Continuation is not empty.

            // Now put the car of C in A, and replace C by its cdr.

            CSPopA();

            if( scheme_machine_pc_index_p( A.t ) )  // Run some actual code, for some kind of primitive.
                ((pFVoid)(theSimpleTable[ (UnsignedLongWord)(A.u.c) ]))();
            else if( empty_list_p( A.t ) )          // Error: Somehow we have gotten an empty list into the continuation.
                resetError( 192 );
            else {                                  // The car of C is something else; deal with it.
                R = A;
                if( pair_p(R.t) ) {                 // The car of C is pair -- it must be a combination, special form or macro.
                    SPushR();                       // Save a copy.
                    car();                          // Get the "function" part -- the car of the pair.
                    if( symbol_p( R.t ) )           // If it is a symbol, look it up now -- don't bother pushing into C.
                        lookUp();
                    switch( type_code(R.t) ) {      // This switch is based on on what the caar of C pair was, after evaluation.
                        case specialFormTag:        // The car of C must have been an S-expression representing a special form.
                            getSpecialEvalCodeIndex();

                            A = R;                  // A: (code index to run the special form).
                            SPopR();                // R: The form itself.
                            PushFrame( stackOpStatus );
                            stackOpStatus = 0;      // stackOpStatus zero means no arguments to worry about.
                            SPushR();
                            R = A;
                            CSPushProperList();
                            SPopR();                            
                            continue;               // The while.
                            break;
                        case macroTag:              // The car of C must have been an S-expression representing a macro.
                            A.u.c = R.u.c;
                            A.t = lambdaPairTag;    // A: code to expand the macro.
                            SPopR();                // R: the macro "call".
                            PushFrame( stackOpStatus );
                            stackOpStatus = 0;      // stackOpStatus zero means no arguments to worry about.
                            SPushR();
                            R = one;
                            SPushR();
                            R = A;
                            SPushR();
                            applyMacroLambda();
                            continue;               // The while.
                            break;
                        default:                    // The car of C must have been a special form representing a procedure call.
                            R = S[0];
                            lengthList = lengthFunction();  // at least 1 -- no empty pairs in Scheme
                            SPopR();                        // R: (application ...); C has been cdr'd
                            PushFrame( stackOpStatus );
                            stackOpStatus = lengthList;
                            SPushR();
                            CSPushProperList();
                            SPopR();

                            // now build a stack frame with stackOpStatus + 1 slots, initialized to ()

                            counter = stackOpStatus + 1;
                            while( counter-- > 0 )
                                (--S)->t = emptyListTag;

                            // the number of arguments goes in the slot just under the top

                            S[1].t = exactShortFixnumBitsMask;
                            S[1].u.c = (stackOpStatus - 1);      // the number of arguments

                            // switch the sign of "stackOpStatus" as a flag to warn that
                            // evaluation of the application is just beginning; code to
                            // follow will thereby know to put the eval'ed function in S[0]

                            stackOpStatus = - stackOpStatus;
                            continue;                   // the while
                        }
                    }
                else if( empty_list_p( R.t ) ) {        // Error: The car of C is the empty list.
                    useErrorOutput();
                    SPushR();
                    printAnyTaggedAval( &CurrentOutputPort, &R );
                    tprintf2( &CurrentOutputPort, MiscStrings[ 115 ] );
                    SPopR();
                    displayObject();
                    resetError( 251 );
                    }
                else                                    // The car of C is some kind of atom, so we evaluate it.
                    evalAtom();

                if( stackOpStatus > 1 )                 // put argument into S[ stackOpStatus ] (a positive stackOpStatus is a
                                                        // cursor into the stack, showing where to load the next evaled argument)
                    S[ stackOpStatus-- ] = R;           // ... then we decrement stackOpStatus.

                else if( stackOpStatus < 0 ) {          // put the evaled function into S[0].
                    S[0] = R;
                    stackOpStatus = - stackOpStatus;
                    }
                // else stackOpStatus is zero or one and the contents of R need not be saved
                //     ;                     
                }                                       // End of "car of C is something else" action.
            }                                           // End of continuation-not-empty action.
        }
    }

/**** loadContinuation -- with list that is in R; continuation -> append( R, continuation ) ****/

void loadContinuation()
{
    UnsignedLongWord lnth = 0;

    /* first push the elements onto the stack */
    while( mutable_type_code(R.t) != emptyListTag ) {
        if( ! pair_p(R.t) ) {
            useErrorOutput();
            displayObject();
            resetError( 158 );
            }
        ++lnth;
        SPushR();
        car();
        Swap();
        cdr();
        }

    /* then load them */
    while( lnth-- > 0 ) {
        SPopR();
        CSPush( R );
        }
    }

/**** macroPartTwo -- Macro ran, must reevaluate result in caller's environment.   ****/

void macroPartTwo()
{
    CSPopA();
    E = A;
    CSPush( R );
    }
