/****************************************************************************************

SECDIO.c -- Copyright (c) by Jay Reynolds Freeman 1988, 1989, 1990, 1991, 2006

C source for the functions which the "OpCodes.h" file uses for I/O op codes, for Wraith Scheme.
NOTHING in this file should reference any of the macros in "OpCodes.h".

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import "WraithHeader.h"
#import "MemoryManager.h"
#import "Registers.h"
#import "SchemeCType.h"
#import "SECD.h"
#import "DataTypes.e"
#import "Globals.e"
#import "DataTypes.e"
#import "IO.e"
#import "Registers.e"
#import "SECDFloatStuff.e"
#import "SECDMachine.e"
#import "SECDRegisters.e"
#import "StreamHandling.e"

/**** Statics: ****/

/**** Functions: ****/

/**** Some forward declarations: ****/

Boolean openFileGuts( fileDescriptor *sfd, const Text *type, Boolean interactiveP, const Text *prompt );
Text schemeToLower( Text c );
void closePortGuts( fileDescriptor *fd );

fileDescriptor *openFile( const Text *type, Boolean interactiveP );
fileDescriptor *openReadSocket();
Boolean openReadSocketGuts( fileDescriptor *sfd );

/**** charReadyPMC -- Is a character ready on the default port? ****/

void charReadyPMC() { B = tcharready( getPortFilePointer( &DefaultInputPort ) ); }

/**** closePort -- Close the file for the indicated port, and clean up the port's data structure to match. ****/

void closePort()
{
    fileDescriptor *sfd = 0;
    UnsignedLongWord n;

    if( port_p( R.t ) ) {
        n = getPortBigArrayLocation( &R );
        if( n < (nInputFileDescriptors + nOutputFileDescriptors) ) {
            sfd = &(AllFileDescriptors[ n ] );
            closePortGuts( sfd );
            }
        else if( (n == coninIndex) || (n == conoutIndex) )
            ;
        else {
            useErrorOutput();
            writeObject();
            resetError( 190 );
            }
        }
    else {
        useErrorOutput();
        writeObject();
        resetError( 190 );
        }
    R.t = trueBooleanTag;
    }

/**** closeTranscript -- Close the transcript file, if any. ****/

void closeTranscript()
{
    if( transcriptFD->fdOpen )
        closePortGuts( transcriptFD );
    }
        
Text InitFilePath[ MAX_PATHNAME_LENGTH ];
Text SWFBuffer[ MAX_PATHNAME_LENGTH ];

/**** initializeStartupWorkingFolder -- must be called early.  ****/

void initializeStartupWorkingFolder()
{ 
    if( getcwd( SWFBuffer, MAX_PATHNAME_LENGTH ) == NULL )
        fatalError( 49 );
    }

/**** getInitFilePath ****/

void getInitFilePath()
{
    makeString( InitFilePath );
    }

/**** getStartupWorkingFolder ****/

void getStartupWorkingFolder()
{
    makeString( SWFBuffer );
    }

/**** getWorkingFolder ****/

void getWorkingFolder()
{
    Text buffer[ MAX_PATHNAME_LENGTH ];
        
    if( getcwd( buffer, MAX_PATHNAME_LENGTH ) == NULL )
        fatalError( 49 );
    makeString( buffer );
    }

/**** inputPortPMC -- Does R contain an input port? ****/

void inputPortPMC()
{
    UnsignedLongWord n;

    if( port_p( R.t ) ) {
        n = getPortBigArrayLocation( &R );              
        if( (n < nInputFileDescriptors) && (getPortFD( &R ))->fdOpen )
            B = TRUE;
        else if( n == coninIndex )
            B = TRUE;
        else
            B = FALSE;
        }
    else
        B = FALSE;
    }

/**** openInputNamed -- Open a named file for input. ****/

void openInputNamed()
{
    fileDescriptor *sfd;
    UnsignedLongWord theWord = 0;

    sfd = openFile( "r", (Boolean)FALSE );
    if( sfd ) {
        R.t = portTag;
        orIntoPortNumberByteNonAtomic( theWord, sfd->placeInBigArray );
        orIntoPortKittenByteNonAtomic( theWord, theKittenNumber );
        orIntoKittenByteNonAtomic( theWord, theKittenNumber );
        R.u.c = theWord;
        }
    else
        R.t = emptyListTag;
    }

/**** openDirectorySF -- Open a directory, using the standard file package. ****/

void openDirectorySF()
{
    Boolean goodNameP;
    Text buffer[ MAX_PATHNAME_LENGTH + 1 ];
    struct stat sb;

    buffer[ 0 ] = 0;
    eventLoopOnce( 0 );
    goodNameP = performDialogAndReturnString( MiscStrings[ 353 ], buffer, MAX_PATHNAME_LENGTH );
    if( goodNameP ) {
        if( (stat( buffer, &sb ) == -1 ) || (! S_ISDIR( sb.st_mode )) ) {
            useErrorOutput();
            tprintf3( &ErrorPort, MiscStrings[ 352 ], buffer );
            resetError( 252 );
            }
        makeString( buffer );
        }
    else
        R.t = emptyListTag;
    }

/**** openInputSF -- Open a file for input, using the standard file package. ****/

void openInputSF()
{
    fileDescriptor *sfd;
    UnsignedLongWord theWord = 0;

    sfd = openFile( "r", (Boolean)TRUE );
    if( sfd ) {
        R.t = portTag;
        orIntoPortNumberByteNonAtomic( theWord, sfd->placeInBigArray );
        orIntoPortKittenByteNonAtomic( theWord, theKittenNumber );
        orIntoKittenByteNonAtomic( theWord, theKittenNumber );
        R.u.c = theWord;
        }
    else
        R.t = emptyListTag;
    }

/**** openOutputNamed -- Open a named file for output. ****/

void openOutputNamed()
{
    fileDescriptor *sfd;
    UnsignedLongWord theWord = 0;

    sfd = openFile( "w", (Boolean)FALSE );
    if( sfd ) {
        R.t = portTag;
        orIntoPortNumberByteNonAtomic( theWord, sfd->placeInBigArray );
        orIntoPortKittenByteNonAtomic( theWord, theKittenNumber );
        orIntoKittenByteNonAtomic( theWord, theKittenNumber );
        R.u.c = theWord;
        }
    else
        R.t = emptyListTag;
    }

/**** openOutputSF -- Open a file for output, using the standard file package. ****/

void openOutputSF()
{
    fileDescriptor *sfd;
    UnsignedLongWord theWord = 0;

    sfd = openFile( "w", (Boolean)TRUE );
    if( sfd ) {
        R.t = portTag;
        orIntoPortNumberByteNonAtomic( theWord, sfd->placeInBigArray );
        orIntoPortKittenByteNonAtomic( theWord, theKittenNumber );
        orIntoKittenByteNonAtomic( theWord, theKittenNumber );
        R.u.c = theWord;
        }
    else
        R.t = emptyListTag;
    }

/**** openReadSocketGiven -- open the given read socket ****/

void openReadSocketGiven()
{
    fileDescriptor *sfd;
    UnsignedLongWord theWord = 0;

    sfd = openReadSocket();
    if( sfd ) {
        R.t = portTag;
        orIntoPortNumberByteNonAtomic( theWord, sfd->placeInBigArray );
        orIntoPortKittenByteNonAtomic( theWord, theKittenNumber );
        orIntoKittenByteNonAtomic( theWord, theKittenNumber );
        R.u.c = theWord;
        }
    else
        R.t = emptyListTag;
    }

/**** openTranscriptNamed -- Open a file for transcript, NOT using the standard file package. ****/

void openTranscriptNamed()
{
    closeTranscript();
    openFileGuts( transcriptFD, "w", (Boolean)FALSE, NULL );
    }

/**** openTranscriptSF -- Open a file for transcript, using the standard file package. ****/

void openTranscriptSF()
{
    closeTranscript();
    openFileGuts( transcriptFD, "w", (Boolean)TRUE, MiscStrings[ 318 ] );
    }

/**** outputPortPMC -- Does R contain an output port? ****/

void outputPortPMC()
{
    UnsignedLongWord n;

    if( port_p( R.t ) ) {
        n = getPortBigArrayLocation( &R );              
        if(    (n >= nInputFileDescriptors)
            && (n < nInputFileDescriptors + nOutputFileDescriptors)
            && (getPortFD( &R ))->fdOpen )
            B = TRUE;
        else if( (n == conoutIndex) || (n == conerrIndex) )
            B = TRUE;
        else
            B = FALSE;
        }
    else
        B = FALSE;
    }

/**** setWorkingFolder ****/

void setWorkingFolder()
{
    Text *givenName = startString( &R );
    UnsignedLongWord lnth = strlen( givenName );
    Text buffer[ MAX_PATHNAME_LENGTH ];
        
    if( lnth > MAX_PATHNAME_LENGTH - 1 ) {
        tprintf3( &ErrorPort, MiscStrings[ 263 ], givenName );
        resetError( 124 );
        }
     strncpy( buffer, givenName, MAX_PATHNAME_LENGTH - 1 );
     if( buffer[ lnth - 1 ] != FRONTSLASH ) {
        buffer[ lnth++ ] = FRONTSLASH;
        buffer[ lnth ] = 0;
        }
     if( chdir( buffer ) != 0 ) {
        tprintf3( &ErrorPort, MiscStrings[ 263 ], buffer );
        resetError( 252 );
        }
     }

/**** anyOpenPortsP -- Are any ports open? ****/

Boolean anyOpenPortsP()
{
    fileDescriptor *fd;
    UnsignedLongWord i;

    for( i = 0; i < nInputFileDescriptors; ++i ) {
        fd = &(AllFileDescriptors[ i ]);
        if( fd->fdOpen ) {
            tprintf3( &CurrentOutputPort, MiscStrings[ 294 ], i );
            return TRUE;
            }
        }
    for( i = nInputFileDescriptors; i < nOutputFileDescriptors; ++i ) {
        fd = &(AllFileDescriptors[ i ]);
        if( fd->fdOpen ) {
            tprintf3( &CurrentOutputPort, MiscStrings[ 295 ], i );
            return TRUE;
            }
        }
    if( transcriptFD->fdOpen ) {
        tprintf2( &CurrentOutputPort, MiscStrings[ 296 ] );
        return TRUE;
        }
    return FALSE;
    }                               

/**** bogus -- A no-op function, as a bug fixer. ****/

void bogus() {;}

/**** closeAllPorts -- Close every port that is open. ****/

void closeAllPorts()
{
    fileDescriptor *fd;
    UnsignedLongWord i;

    for( i = 0; i < nInputFileDescriptors + nOutputFileDescriptors; ++i ) {
        fd = &(AllFileDescriptors[i]);
        if( fd->fdOpen ) 
            closePortGuts( fd );
        }
    if( transcriptFD->fdOpen ) {
        closePortGuts( transcriptFD );
        }
    }                               

/**** closePortGuts -- Close the file for the given port, and clean up the port's data structure to match. ****/
                                          
void closePortGuts( fileDescriptor *fd )
{
    if( (fd != masterDefaultInputFD) && (fd != conIn) && (fd != conOut) && (fd != conErr) ) {
        if( ! strcmp( fd->type, "w" ) )
            tfflushOutput( fd->fp );
        if( fd->fp != NULL ) {
            fclose( fd->fp );
            fd->fp = NULL;
            }
        reinitializeFileDescriptor( fd );
        }
    }

/**** closeStackedPort -- Close the port in STACK( 0 ) and drop that stack slot. ****/

void closeStackedPort()
{
    UnsignedLongWord n;

    if( port_p( (S[0]).t ) ) {
        n = getPortBigArrayLocation( &(S[0]) );
        if( n < (nInputFileDescriptors + nOutputFileDescriptors) ) {
            SDrop();
            closePortGuts( &(AllFileDescriptors[ n ]) );
            }
        else if( (n == coninIndex) || (n == conoutIndex) || (n == conerrIndex) )
            SDrop();
        else {
            SPopR();
            useErrorOutput();
            writeObject();
            resetError( 190 );
            }
        }
    else {
        SPopR();
        useErrorOutput();
        writeObject();
        resetError( 190 );
        }       
    }       

/**** closeStackedPortRestoreInput -- Close the port in STACK( 0 ), restore the input to the
                                      port in STACK( 1 ), and drop both slots. ****/

void closeStackedPortRestoreInput()
{
    closeStackedPort();
    CurrentInputPort = S[0];
    DefaultInputPort = S[1];
    SDropN( 2 );
    }

/**** closeStackedPortRestoreOutput -- Close the port in STACK( 0 ), restore the output to the
                                       port in STACK( 1 ), and drop both slots. ****/

void closeStackedPortRestoreOutput()
{
    closeStackedPort();
    Swap();
    CurrentOutputPort = R;
    SPopR();
    }

/**** fileExistsPMC -- Does the file named by the string in R exist? ****/

void fileExistsPMC()
{
    FILE *fp;

    B = ((fp = fopen( startString( &R ), "r" )) != NULL);
    if( B )
        fclose( fp );
    }

/**** restoreInputPort -- restore the default input port from R (for use in loading files, afterward) -- don't change R. ****/

void restoreInputPort()
{
    UnsignedLongWord n = 0;

    if( port_p( R.t ) && (getPortKitten( &R ) == theKittenNumber) ) {
        n = getPortBigArrayLocation( &R );
        if( (n < nInputFileDescriptors) || (n == coninIndex) ) {
            DefaultInputPort = R;
            return;
            }
        }
    useErrorOutput();
    printAnyTaggedAval( &CurrentOutputPort, &R );
    resetError( 254 );        
    }

/**** getNextChar-- skip comments */

void getNextChar()
{
    LongWord c;
    FILE *fp = getPortFilePointer( &DefaultInputPort );

    while( (c = tgetc( fp )) == COMMENT_PREFIX )
        while( ((c = tgetc( fp )) != NEWLINE) && (! isEOF( c )) )
                ;
    R.t = characterTag;
    R.u.c = c;        
    }       

/**** getNextCharLower -- skip comments, put in lower case */

void getNextCharLower()
{
    LongWord c;
    FILE *fp = getPortFilePointer( &DefaultInputPort );

    while( (c = tgetc( fp )) == COMMENT_PREFIX )
        while( ((c = tgetc( fp )) != NEWLINE) && (! isEOF( c )) )
                ;
    /* tprintf4( &ErrorPort, "getNextCharLower %c (%02xh)\n", c, c); */
    R.t = characterTag;
    if( ! isEOF( c ) )
        R.u.c = schemeToLower((Text)c);
    else
        R.u.c = (Text)c;
    }       

/**** getNextCharRaw -- don't skip comments */

void getNextCharRaw()
{
    FILE *fp = getPortFilePointer( &DefaultInputPort );

    R.t = characterTag;
    R.u.c = (tgetc( fp ));  
    }       

/**** getNextCharPort -- skip comments */

void getNextCharPort( taggedAval *aPort )
{
    LongWord c;
    FILE *fp = getPortFilePointer( aPort );

    while( (c = tgetc( fp )) == COMMENT_PREFIX )
        while( ((c = tgetc( fp )) != NEWLINE) && (! isEOF( c )) )
                ;
    R.t = characterTag;
    R.u.c = c;        
    }       

/**** getNextCharLowerPort -- skip comments, put in lower case */

void getNextCharLowerPort( taggedAval *aPort )
{
    LongWord c;
    FILE *fp = getPortFilePointer( aPort );

    while( (c = tgetc( fp )) == COMMENT_PREFIX )
        while( ((c = tgetc( fp )) != NEWLINE) && (! isEOF( c )) )
                ;
    /* tprintf4( &ErrorPort, "getNextCharLower %c (%02xh)\n", c, c); */
    R.t = characterTag;
    if( ! isEOF( c ) )
        R.u.c = schemeToLower((Text)c);
    else
        R.u.c = (Text)c;
    }       

/**** getNextCharRawPort -- don't skip comments */

void getNextCharRawPort( taggedAval *aPort )
{
    FILE *fp = getPortFilePointer( aPort );

    R.t = characterTag;
    R.u.c = (tgetcNoHangOnEOF( fp ));  
    }       

/**** getNextCharRawPortNoHangOnEOF -- don't skip comments */

void getNextCharRawPortNoHangOnEOF( taggedAval *aPort )
{
    FILE *fp = getPortFilePointer( aPort );

    R.t = characterTag;
    R.u.c = (tgetcNoHangOnEOF( fp ));  
    }       

/**** loadFromString ****/

void loadFromString( Text *s )
{
    FILE *fp = getPortFilePointer( &DefaultInputPort );
    LongWord counter = 0;

    if( (s != 0) && (*s != 0) )        // Push back the string in reversed order.
        for( counter = strlen( s ); counter > 0; --counter )
            untgetc( s[ counter - 1 ], fp );
    }    

/**** openFile -- Locate an unused file descriptor, figure out what the user wants, open
                  the file, and do what's necessary to the various data structures.              ****/
                                 
fileDescriptor *openFile( const Text *type, Boolean interactiveP )
{
    fileDescriptor *sfd = 0;
    UnsignedLongWord whichFD;
    Boolean triedGarbageCollection = FALSE;
    Boolean foundDescriptor = FALSE;

    // First, see if we have an unused file descriptor of the correct kind.

    while( TRUE ) {
        if( strcmp( type, "r") == SAME ) {
            for( whichFD = 0; whichFD < nInputFileDescriptors; ++whichFD )
                if( ! AllFileDescriptors[ whichFD ].fdOpen )
                    break;
            if( whichFD < nInputFileDescriptors ) {
                sfd = &(AllFileDescriptors[ whichFD ]);
                foundDescriptor = TRUE;
                break;
                }
            }
        else if(  strcmp( type, "w") == SAME ) {
            for( whichFD = nInputFileDescriptors; whichFD < nInputFileDescriptors + nOutputFileDescriptors; ++whichFD )
                if( ! AllFileDescriptors[ whichFD ].fdOpen )
                    break;
            if( whichFD < nInputFileDescriptors + nOutputFileDescriptors ) {
                sfd = &(AllFileDescriptors[ whichFD ]);
                foundDescriptor = TRUE;
                break;
                }
            }

        // At this point we haven't found a descriptor -- if we had, we would have done a "break".

        if( ! triedGarbageCollection ) {       //   If we haven't tried garbage-collection, try it.
            theClowder->setFullGCIsRequired();
            theClowder->setGCIsRequired();
            theHeap->triggerGC();
            triedGarbageCollection = TRUE;       
            } 
        else                                   //   GC was no help, abandon the search.
            resetError( 189 );
        } 

    if( ! foundDescriptor )
        resetError( 189 );

    // Found one; let's try to open it.

    if( openFileGuts( sfd, type, interactiveP, (strcmp( type, "r") == SAME) ? MiscStrings[ 316] : MiscStrings[ 317] ) )
        return sfd;
    else
        return (fileDescriptor *)NULL;
    }

/**** openFileGuts -- Given an unused file descriptor, open the file, and update various data structures.  ****/
                                 
Boolean openFileGuts( fileDescriptor *sfd, const Text *type, Boolean interactiveP, const Text *prompt )
{
    Boolean goodNameP;
    Text *givenName;
    Text buffer[ MAX_PATHNAME_LENGTH + 1 ];
    struct stat sb;

    buffer[ 0 ] = 0;
    if( interactiveP ) {
        eventLoopOnce( 0 );
        goodNameP = performDialogAndReturnString( prompt, buffer, MAX_PATHNAME_LENGTH );
        strcpy( sfd->oldPathName, sfd->pathName );
        strcpy( sfd->pathName, buffer );
	}
    else {
        givenName = startString( &R );
        if( strlen( givenName ) > MAX_PATHNAME_LENGTH - 1 ) {
            goodNameP = FALSE;
            resetError( 191 );
            }
        else {
            goodNameP = TRUE;
            strcpy( sfd->oldPathName, sfd->pathName );
            strcpy( sfd->pathName, givenName );
            }
        }
    if( goodNameP ) {
        if( (strcmp( type, "r" ) == 0) && ((stat( sfd->pathName, &sb ) == -1 ) || S_ISDIR( sb.st_mode )) ) {
            useErrorOutput();
            tprintf4( &ErrorPort, MiscStrings[ 217 ], sfd->pathName,
                      (strcmp( type, "r" ) == 0) ? MiscStrings[ 320 ] : MiscStrings[ 321 ] );
            resetError( 282 );
            }
        strcpy( sfd->type, type );
        if( (sfd->fp = fopen( sfd->pathName, type )) == 0 ) {
            useErrorOutput();
            tprintf4( &ErrorPort, MiscStrings[ 217 ], sfd->pathName,
                      (strcmp( type, "r" ) == 0) ? MiscStrings[ 320 ] : MiscStrings[ 321 ] );
            resetError( 109 );
            }
        else
            sfd->fdOpen = TRUE;
        }
    else
        resetError( 255 );
    return TRUE;
    }

/**** openReadSocket -- Locate an unused file descriptor, figure out what the user wants, open
                        the file, and do what's necessary to the various data structures.              ****/
                                 
fileDescriptor *openReadSocket()
{
    fileDescriptor *sfd = 0;
    UnsignedLongWord whichFD;
    Boolean triedGarbageCollection = FALSE;
    Boolean foundDescriptor = FALSE;

    // First, see if we have an unused file descriptor of the correct kind.

    while( TRUE ) {
        for( whichFD = 0; whichFD < nInputFileDescriptors; ++whichFD )
            if( ! AllFileDescriptors[ whichFD ].fdOpen )
                break;
        if( whichFD < nInputFileDescriptors ) {
            sfd = &(AllFileDescriptors[ whichFD ]);
            foundDescriptor = TRUE;
            break;
            }

        // At this point we haven't found a descriptor -- if we had, we would have done a "break".

        if( ! triedGarbageCollection ) {       //   If we haven't tried garbage-collection, try it.
            theClowder->setFullGCIsRequired();
            theClowder->GCIsRequired();
            theHeap->triggerGC();
            triedGarbageCollection = TRUE;       
            } 
        else                                   //   GC was no help, abandon the search.
            resetError( 189 );
        } 

    if( ! foundDescriptor )
        resetError( 189 );

    // Found one; let's try to open it.

    if( openReadSocketGuts( sfd ) )
        return sfd;
    else
        return (fileDescriptor *)NULL;
    }

/**** openReadSocketGuts -- Given an unused file descriptor, open the port, and update various data structures.  ****/
                                 
Boolean openReadSocketGuts( fileDescriptor *sfd )
{
    if( (sfd->fp = fdopen( (Word)(R.u.c & WORD_MASK), "r")) == 0 ) {
        useErrorOutput();
        tprintf3( &ErrorPort, MiscStrings[ 342 ], R.u.c );
        resetError( 220 );
        }
    strcpy( sfd->oldPathName, sfd->pathName );
    sprintf( sfd->pathName, MiscStrings[ 342 ], R.u.c );
    sfd->fdOpen = TRUE;
    return TRUE;
    }

/**** peek ****/

void peek()
{
    FILE *fp = getPortFilePointer( &DefaultInputPort );
    LongWord c;

    c = tpeek( fp );
    while( c == COMMENT_PREFIX ) {
        while( (c != NEWLINE) && (! isEOF( c )) ) {
            (void)tgetc( fp );
            c = tpeek( fp );
            }
        (void)tgetc( fp );
        c = tpeek( fp );
        }
    R.t = characterTag;
    R.u.c = c;
    }

/**** peekPort ****/

void peekPort( taggedAval *aPort )
{
    FILE *fp = getPortFilePointer( aPort );
    LongWord c;

    c = tpeek( fp );
    while( c == COMMENT_PREFIX ) {
        while( (c != NEWLINE) && (! isEOF( c )) ) {
            (void)tgetc( fp );
            c = tpeek( fp );
            }
        (void)tgetc( fp );
        c = tpeek( fp );
        }
    R.t = characterTag;
    R.u.c = c;
    }

/**** peekPortNoHangOnEOF ****/

void peekPortNoHangOnEOF( taggedAval *aPort )
{
    LongWord c;
    FILE *fp = getPortFilePointer( aPort );
    
    c = tpeekNoHangOnEOF( fp );
    while( c == COMMENT_PREFIX ) {
        while( (c != NEWLINE) && (! isEOF( c )) ) {
            (void)tgetcNoHangOnEOF( fp );
            c = tpeekNoHangOnEOF( fp );
            }
        (void)tgetcNoHangOnEOF( fp );
        c = tpeekNoHangOnEOF( fp );
        }
    R.t = characterTag;
    R.u.c = c;
    }

/**** readDialogString ****/

void readDialogString( const Text *prompt )
{
    Text buffer[ MAX_STRING_FROM_CONSOLE_LENGTH + 1 ];

    buffer[ 0 ] = 0;
    eventLoopOnce( 0 );
    performDialogAndReturnString( prompt, buffer, MAX_STRING_FROM_CONSOLE_LENGTH );
    makeString( buffer );
	}

/**** schemeLegalCharacterP -- Is the given character legal in Scheme? ****/

Boolean schemeLegalCharacterP( Text c ) { return (Boolean)isSchemeLegalCharacter( c ); }

/**** schemeToLower ****/

Text schemeToLower( Text c )
{
    Text *ptr =  strchr( ALPHABET_UPPER, c );

    if( ptr )
        return( *(ALPHABET_LOWER + (ptr - ALPHABET_UPPER)) );   
    else
        return c;
    }

/**** schemeToUpper ****/

Text schemeToUpper( Text c )
{
    Text *ptr =  strchr( ALPHABET_LOWER, c );

    if( ptr )
        return( *(ALPHABET_UPPER + (ptr - ALPHABET_LOWER)) );   
    else
        return c;
    }

/**** writeNewline ****/

void writeNewline( taggedAval *aPort )
{
    tprintf3( aPort, MiscStrings[ 57 ], NEWLINE );
    }

/**** writeNewlineError ****/

void writeNewlineError() { writeNewline( &ErrorPort ); }

