/****************************************************************************************

SECDMachine.c -- Copyright (c) by Jay Reynolds Freeman 1988, 1989, 1990, 1991, 2006, 2008

C source for the functions which the "OpCodes.h" file uses for general op codes, for Wraith Scheme.
NOTHING in this file should reference any of the macros in "OpCodes.h"

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import "WraithHeader.h"
#import "MemoryManager.h"

#import "StreamHandling.e"
#import "Numbers.h"
#import "Registers.h"
#import "SchemeCType.h"
#import "SECD.h"
#import "Globals.e"
#import "DataTypes.e"
#import "Intern.e"
#import "IO.e"
#import "NumberFormats.e"
#import "Reader.e"
#import "ReaderNumbers.e"
#import "Registers.e"
#import "SECDEval.e"
#import "SECDFloatStuff.e"
#import "SECDIO.e"
#import "SECDRegisters.e"

#ifdef PIXIE_SCHEME_III

#define NO_FP_INEXACT 

#endif

#ifdef NO_FP_INEXACT
extern Boolean isFloatingPointOperation;
#endif

// Here are some handy macros, which will possibly make several functions herein more readable.

#define color( tAPtr )                         theClowder->setColoringBitN( theHeap->nOfTaggedAvalInHeap( tAPtr ) )
#define colored_p( tAPtr )                     theClowder->getColoringBitN( theHeap->nOfTaggedAvalInHeap( tAPtr ) )
#define uncolor( tAPtr )                       theClowder->clearColoringBitN( theHeap->nOfTaggedAvalInHeap( tAPtr ) )

/**** Externals for interesting taggedAvals: ****/

taggedAval zero                 = { (Tag)exactShortFixnumBitsMask, { (LongWord)0 } };
taggedAval one                  = { (Tag)exactShortFixnumBitsMask, { (LongWord)1 } };
taggedAval two                  = { (Tag)exactShortFixnumBitsMask, { (LongWord)2 } };
taggedAval three                = { (Tag)exactShortFixnumBitsMask, { (LongWord)3 } };
taggedAval four                 = { (Tag)exactShortFixnumBitsMask, { (LongWord)4 } };
taggedAval five                 = { (Tag)exactShortFixnumBitsMask, { (LongWord)5 } };
taggedAval six                  = { (Tag)exactShortFixnumBitsMask, { (LongWord)6 } };
taggedAval seven                = { (Tag)exactShortFixnumBitsMask, { (LongWord)7 } };
taggedAval eight                = { (Tag)exactShortFixnumBitsMask, { (LongWord)8 } };
taggedAval nine                 = { (Tag)exactShortFixnumBitsMask, { (LongWord)9 } };
taggedAval ten                  = { (Tag)exactShortFixnumBitsMask, { (LongWord)10 } };
taggedAval maxCh                = { (Tag)exactShortFixnumBitsMask, { (LongWord)127 } };
taggedAval symbolHashSize       = { (Tag)exactShortFixnumBitsMask, { (LongWord)SYMBOL_HASH_CONSTANT } };
taggedAval numberOfInterrupts   = { (Tag)exactShortFixnumBitsMask, { (LongWord)NUMBER_OF_INTERRUPTS } };

taggedAval empty                = { (Tag)emptyListTag, { 0 } };

taggedAval blankChar            = { (Tag)characterTag, { 0 } };
taggedAval newlineChar          = { (Tag)characterTag, { 0 } };
taggedAval sharpF               = { (Tag)falseBooleanTag, { 23 } };

taggedAval maxChar              = { (Tag)exactShortFixnumBitsMask, { 0 } };

taggedAval taggedAvalLength     = { (Tag)exactShortFixnumBitsMask, { (sizeof( taggedAval )) } };

/******** End of statics ********/

/**** Functions: ****/

/**** Some forward declarations: ****/

void car();
void cdr();
void displayObject();
void eqPMC(), eqvPMC();
void getPName();
void hashString();
void makeString( const Text *str );
void lengthOp();
void nilCons();
void numEqualPMC();
void setCar();
void setCdr();
void stringEqual();
void useErrorOutput();
void useFatalErrorOutput();
void vectorLength();
void vectorRefN( LongWord n );
void writeObject();
void writeObjectOnPort();

void letterPMC() { B = (Boolean)(character_p(R.t) && isSchemeAlpha((Text)(R.u.c))); }

void nonPrintingObjectPMC() { B = (peculiar_leaf_p( R.t) && (leaf_number( R ) == nonPrintingObjectTag)); }

void radixPMC()
{
    B = (Boolean)(    (fixnum_p(R.t))
                   && ( (R.u.c == (10)) || (R.u.c == (16)) || (R.u.c == (2))  || (R.u.c == (8)) )
                   );
    }

void kittenRangeCheckPMC()
{
    B = (Boolean)(    (fixnum_p(R.t))
                   && (R.u.c >= 0)
                   && ((UnsignedLongWord)(R.u.c) < theNumberOfKittens)
                   && (theClowder->getKittenEvaluatorActivity( (UnsignedWord)(R.u.c) ) >= KITTEN_ACTIVITY_READY_TO_EVAL)
                   );
    }

void interruptRangeCheckPMC()
{
    B = (Boolean)(    (fixnum_p(R.t))
                   && (R.u.c >= 0)
                   && ((UnsignedLongWord)(R.u.c) < NUMBER_OF_INTERRUPTS )
                   );
    }

void sliderNumberRangeCheckPMC()
{
    B = (Boolean)(    (fixnum_p(R.t))
                   && (R.u.c >= 0)
                   && ((UnsignedLongWord)(R.u.c) < N_SLIDERS )
                   );
    }

void pushbuttonNumberRangeCheckPMC()
{
    B = (Boolean)(    (fixnum_p(R.t))
                   && (R.u.c >= 0)
                   && ((UnsignedLongWord)(R.u.c) < N_PUSHBUTTONS )
                   );
    }

void levelIndicatorNumberRangeCheckPMC()
{
    B = (Boolean)(    (fixnum_p(R.t))
                   && (R.u.c >= 0)
                   && ((UnsignedLongWord)(R.u.c) < N_LEVEL_INDICATORS )
                   );
    }

void levelIndicatorValueRangeCheckPMC()
{
    B = (Boolean)(    (fixnum_p(R.t))
                   && (R.u.c >= 0)
                   && ((UnsignedLongWord)(R.u.c) <= LEVEL_INDICATOR_MAX_VALUE )
                   );
    }

/**** initializeSpecialTags ****/

void initializeSpecialTags()
{
    blankChar.u.c = ' ';
    newlineChar.u.c = '\n';
    maxChar.u.c = 127;
    }

/**** bitAnd -- Return the bitwise and of R and S[0]. ****/

void bitAnd()
{
    R.u.c = (R.u.c & (S[0]).u.c);
    if( exact_p( R.t ) && exact_p( S[0].t ) )
        R.t = exactShortFixnumBitsMask;
    else
        R.t = inexactShortFixnumBitsMask;
    }

/**** bitNot -- Return the bitwise inverse of R. ****/

void bitNot()
{
    R.u.c = ~(R.u.c);
    if( exact_p( R.t ) )
        R.t = exactShortFixnumBitsMask;
    else
        R.t = inexactShortFixnumBitsMask;
    }

/**** bitOr -- Return the bitwise or of R and S[0]. ****/

void bitOr()
{
    R.u.c = (R.u.c | (S[0]).u.c);
    if( exact_p( R.t ) && exact_p( S[0].t ) )
        R.t = exactShortFixnumBitsMask;
    else
        R.t = inexactShortFixnumBitsMask;
    }

/**** bitXor -- Return the bitwise exclusive or of R and S[0]. ****/

void bitXor()
{
    R.u.c = (R.u.c ^ (S[0]).u.c);
    if( exact_p( R.t ) && exact_p( S[0].t ) )
        R.t = exactShortFixnumBitsMask;
    else
        R.t = inexactShortFixnumBitsMask;
    }

/**** bitShiftLeft -- Return the bitwise left shift of S[0] by R. ****/

void bitShiftLeft()
{
    if( R.u.c >= (LongWord)(8 * sizeof(UnsignedLongWord)) )
        R.u.c = 0;
    else if( R.u.c > 0 )
        R.u.c = (S[0]).u.c << R.u.c;
    else
        R.u.c = (S[0]).u.c;

    if( exact_p( R.t ) && exact_p( S[0].t ) )
        R.t = exactShortFixnumBitsMask;
    else
        R.t = inexactShortFixnumBitsMask;
    }

/**** bitShiftRightArithmetic -- Return the bitwise right arithmetic shift of S[0] by R. ****/

void bitShiftRightArithmetic()
{
    if( R.u.c >= (LongWord)(8 * sizeof(UnsignedLongWord)) )
        if( (S[0]).u.c >= 0 )
            R.u.c = 0;
        else
            R.u.c = -1;
    else if( R.u.c > 0 )
        R.u.c = (S[0]).u.c >> R.u.c;
    else
        R.u.c = (S[0]).u.c;

    if( exact_p( R.t ) && exact_p( S[0].t ) )
        R.t = exactShortFixnumBitsMask;
    else
        R.t = inexactShortFixnumBitsMask;
    }

/**** bitShiftRightLogical -- Return the bitwise right logical shift of S[0] by R. ****/

void bitShiftRightLogical()
{
    if( R.u.c >= (LongWord)(8 * sizeof(UnsignedLongWord)) )
        R.u.c = 0;
    else if( R.u.c > 0 )
        R.u.c = ((UnsignedLongWord)(S[0]).u.c) >> R.u.c;
    else
        R.u.c = (S[0]).u.c;

    if( exact_p( R.t ) && exact_p( S[0].t ) )
        R.t = exactShortFixnumBitsMask;
    else
        R.t = inexactShortFixnumBitsMask;
    }

/**** carRegardlessOfBlocking ****/

void carRegardlessOfBlocking()
{
    if( ! pair_p( R.t ) ) {
        useErrorOutput();
        (void)printAnyTaggedAval( &CurrentOutputPort, &R );
        resetError( 215 );
        }
    else {
        R = spliceOutNotCarForwardingPointers( R );
        R = *(R.u.a);
        R.t &= ~(cdrCodingBitsMask | administrationBitsMask | pointsToActiveBitMask);
        }
    }

/**** speculativeCar ****/

Boolean speculativeCar()
{
    SPushR();

    //  In the next line, we require R.u.c and not R.u.a, so that in the 32-bit implementations
    //  the entire 64 bit content of the union will be copied.

    if( (R.u.c = (((S[0]).u.a)->u).c) ) {  // Trick to make sure R.u.a gets loaded first.
        R.t = ((S[0]).u.a)->t;
        if( (! forwarding_pointer_p( R.t )) && (! hasLocationByte( R.t )) && (getKittenByte( R.t ) == theKittenNumber) ) {
            R.t &= ~(cdrCodingBitsMask | administrationBitsMask | pointsToActiveBitMask);
            SDrop();
            return TRUE;
            }
        }
    SPopR();
    return FALSE;
    }

/**** car ****/

void car()
{
    if( ! pair_p( R.t ) ) {
        useErrorOutput();
        (void)printAnyTaggedAval( &CurrentOutputPort, &R );
        resetError( 165 );
        }
    if( speculativeCar() )
        return;

    if( ! blockOnTagLock1( &R, 3 ) )
        resetError( 242 );
    else {
        SPushR();
        R = spliceOutNotCarForwardingPointers( R );
        if( R.u.a != (S[0]).u.a ) {
            releaseTagLock1( (volatile taggedAval *)((S[0]).u.a), 8 );
            SDrop();
            if( ! blockOnTagLock1( &R, 4 ) )
                resetError( 242 );
            SPushR();
            }

        // S: possibly indirect R.

        R = *(R.u.a);
        R.t &= ~(cdrCodingBitsMask | administrationBitsMask | pointsToActiveBitMask);
        releaseTagLock1( (volatile taggedAval *)((S[0]).u.a), 9 );
        SDrop();
        }
    }

/**** carCdrMixForCSPushProperList ****/

void carCdrMixForCSPushProperList()
{
    UnsignedLongWord savedImmutableBit = 0;
    Tag theTag = 0;
    taggedAval dummy;

    // Push the car of R, and leave the cdr of R in R.

    savedImmutableBit = R.t & immutableBitMask;

//    carRegardlessOfBlocking();

    if( ! pair_p( R.t ) ) {
        useErrorOutput();
        (void)printAnyTaggedAval( &CurrentOutputPort, &R );
        resetError( 233 );
        }
    R = spliceOutNotCarForwardingPointers( R );
    dummy = R;
    R = *(R.u.a);
    R.t &= ~(cdrCodingBitsMask | administrationBitsMask | pointsToActiveBitMask);

    SPushR();                                // The desired push.

//    cdrRegardlessOfBlocking();

    R = dummy;

    theTag = (R.u.a)->t;
    switch( cdr_code( theTag ) ) {
        case carCdrNext:
            R.u.a = R.u.a + 1;
            R = spliceOutNotCarForwardingPointers( R );
            if( car_p( (R.u.a)->t ) )
                R.t = pairTag | savedImmutableBit;
            else if( empty_list_p( (R.u.a)->t ) )
                R.t = emptyListTag;
            else {
                R = *(R.u.a);                   
                R.t &= ~( administrationBitsMask | pointsToActiveBitMask);
                }
            break;
        case carCdrNil:
            R.t = emptyListTag;
            break;
        case extraCdrCode:
            useFatalErrorOutput();
            (void)printAnyTaggedAval( &CurrentOutputPort, &R );
            tprintf2( &CurrentOutputPort, MiscStrings[ 115 ] );
            (void)printAnyTaggedAval( &CurrentOutputPort, R.u.a );
            fatalError( 170 );
            break;
        default:
            useFatalErrorOutput();
            (void)printAnyTaggedAval( &CurrentOutputPort, &R );
            tprintf2( &CurrentOutputPort, MiscStrings[ 115 ] );
            (void)printAnyTaggedAval( &CurrentOutputPort, R.u.a );
            fatalError( 138 );
            break;
        }
    }

/**** speculativeCdr ****/

Boolean speculativeCdr()
{
    UnsignedLongWord savedImmutableBit = 0;
    Tag theTag       = 0;
    Tag theSecondTag = 0;
    LongWord theValue = 0;

    SPushR();                                      // R: pair; S: pair
    savedImmutableBit = R.t & immutableBitMask;

    theTag = (R.u.a)->t;
    if( forwarding_pointer_p( theTag ) || hasLocationByte( theTag ) || (getKittenByte( theTag ) != theKittenNumber) ) {
        SPopR();
        return FALSE;
        }
    switch( cdr_code( theTag ) ) {
        case carCdrNext:                          // R, S -- possibly-indirected pair.
            if( (R.u.a = R.u.a + 1) && ((theValue = (((R.u.a)->u).c)) || (theValue + 1)) ) {    // Trick to make sure R.u.a gets loaded first.
                theSecondTag = (R.u.a)->t;
                if(    forwarding_pointer_p( theSecondTag )
                    || hasLocationByte( theSecondTag )
                    || (getKittenByte( theSecondTag ) != theKittenNumber)
                    ) {
                    SPopR();
                    return FALSE;
                    }
                }
            if( car_p( theSecondTag ) )
                R.t = pairTag | savedImmutableBit;
            else if( empty_list_p( theSecondTag ) )
                R.t = emptyListTag;
            else {
                R.u.c = theValue;
//                R.t = theSecondTag & (~( administrationBitsMask ));
                R.t = theSecondTag & (~( administrationBitsMask | pointsToActiveBitMask )); // TEST
                }
            SDrop();    // Stack is clear.
            return TRUE;
            break;
        case carCdrNil:                          // R, S -- possibly-indirected pair.
            R.t = emptyListTag;
            SDrop();    // Stack is clear.
            return TRUE;
            break;
        case extraCdrCode:
            useErrorOutput();
            (void)printAnyTaggedAval( &CurrentOutputPort, &(S[0]) );
            tprintf2( &CurrentOutputPort, "\n");
            SDrop();    // Stack is clear.
            resetError( 207 );
            break;
        default:
            useFatalErrorOutput();
            tprintf3( &CurrentOutputPort, MiscStrings[ 502 ], theTag );
            (void)printAnyTaggedAval( &CurrentOutputPort, &(S[0]) );
            tprintf2( &CurrentOutputPort, "\n");
            SDrop();    // Stack is clear.
            fatalError( 135 );
            break;
        }
    return FALSE;
    }

/**** cdr ****/

void cdr()
{
    UnsignedLongWord savedImmutableBit = 0;
    Tag theTag = 0;

    if( ! pair_p( R.t ) ) {
        useErrorOutput();
        (void)printAnyTaggedAval( &CurrentOutputPort, &R );
        resetError( 166 );
        }
    if( speculativeCdr() )
        return;

    if( ! blockOnTagLock1( &R, 5 ) )
        resetError( 295 );

    SPushR();                                      // R: pair; S: pair
    savedImmutableBit = R.t & immutableBitMask;

    R = spliceOutNotCarForwardingPointers( R );
    if( R.u.a != (S[0]).u.a ) {
        releaseTagLock1( (volatile taggedAval *)((S[0]).u.a), 10 );
        SDrop();
        if( ! blockOnTagLock1( &R, 6 ) )
            resetError( 295 );
        SPushR();
        }

    // R, S -- possibly-indirected pair.

    theTag = (R.u.a)->t;
    switch( cdr_code( theTag ) ) {
        case carCdrNext:                          // R, S -- possibly-indirected pair.
            R.u.a = R.u.a + 1;
            releaseTagLock1( (volatile taggedAval *)((S[0]).u.a), 11 );  // Release the possibly-indirected pair
            SDrop();
            if( ! blockOnTagLock1( &R, 7 ) )
                resetError( 295 );
            SPushR();                               // R: next; S: next.
            R = spliceOutNotCarForwardingPointers( R );
            if( R.u.a != (S[0]).u.a ) {
                releaseTagLock1( (volatile taggedAval *)((S[0]).u.a), 12 );
                SDrop();                                // R: possibly-indirected next. 
                if( ! blockOnTagLock1( &R, 8 ) )
                    resetError( 295 );
                SPushR();
                }

            // R, S -- possibly indirected next.

            if( car_p( (R.u.a)->t ) )
                R.t = pairTag | savedImmutableBit;
            else if( empty_list_p( (R.u.a)->t ) )
                R.t = emptyListTag;
            else {
                R = *(R.u.a);                   
                R.t &= ~( administrationBitsMask | pointsToActiveBitMask );
                }
            releaseTagLock1( (volatile taggedAval *)((S[0]).u.a), 13 );
            SDrop();    // Stack is clear.
            break;
        case carCdrNil:                          // R, S -- possibly-indirected pair.
            R.t = emptyListTag;
            releaseTagLock1( (volatile taggedAval *)((S[0]).u.a), 14 );
            SDrop();    // Stack is clear.
            break;
        case extraCdrCode:
            useErrorOutput();
            (void)printAnyTaggedAval( &CurrentOutputPort, &(S[0]) );
            tprintf2( &CurrentOutputPort, "\n");
            (void)printAnyTaggedAval( &CurrentOutputPort, R.u.a );
            if( theClowder->GCIsRequired() )
                tprintf2( &CurrentOutputPort, MiscStrings[ 503 ] );
            releaseTagLock1( (volatile taggedAval *)((S[0]).u.a), 15 );
            SDrop();    // Stack is clear.
            resetError( 167 );
            break;
        default:
            useFatalErrorOutput();
            tprintf3( &CurrentOutputPort, MiscStrings[ 502 ], theTag );
            (void)printAnyTaggedAval( &CurrentOutputPort, &(S[0]) );
            tprintf2( &CurrentOutputPort, "\n");
            (void)printAnyTaggedAval( &CurrentOutputPort, R.u.a );
            if( theClowder->GCIsRequired() )
            releaseTagLock1( (volatile taggedAval *)((S[0]).u.a), 16 );
            SDrop();    // Stack is clear.
            fatalError( 10 );
            break;
        }
    }

/**** cdrRegardlessOfBlocking ****/

void cdrRegardlessOfBlocking()
{
    UnsignedLongWord savedImmutableBit = 0;
    Tag theTag = 0;

    if( ! pair_p( R.t ) ) {
        useErrorOutput();
        (void)printAnyTaggedAval( &CurrentOutputPort, &R );
        resetError( 6 );
        }
    else {
        savedImmutableBit = R.t & immutableBitMask;
        R = spliceOutNotCarForwardingPointers( R );
        theTag = (R.u.a)->t;
        switch( cdr_code( theTag ) ) {
            case carCdrNext:
                R.u.a = R.u.a + 1;
                R = spliceOutNotCarForwardingPointers( R );
                if( car_p( (R.u.a)->t ) )
                    R.t = pairTag | savedImmutableBit;
                else if( empty_list_p( (R.u.a)->t ) )
                    R.t = emptyListTag;
                else {
                    R = *(R.u.a);                   
                    R.t &= ~( administrationBitsMask | pointsToActiveBitMask );
                    }
                break;
            case carCdrNil:
                R.t = emptyListTag;
                break;
            case extraCdrCode:
                useErrorOutput();
                (void)printAnyTaggedAval( &CurrentOutputPort, &R );
                resetError( 199 );
                break;
            default:
                useFatalErrorOutput();
                (void)printAnyTaggedAval( &CurrentOutputPort, &R );
                fatalError( 147 );
                break;
            }
        }
    }

/**** charIsAlphaPMC ****/

void charIsAlphaPMC()
{
    B = (Boolean)isSchemeAlpha( (Text)(R.u.c) );
    }

/**** charIsLowerPMC ****/

void charIsLowerPMC()
{
    B = (Boolean)isSchemeLower( (Text)(R.u.c) );
    }

/**** charIsNumericPMC ****/

void charIsNumericPMC()
{
    B = (Boolean)isSchemeDigit( (Text)(R.u.c) );
    }

/**** charIsUpperPMC ****/

void charIsUpperPMC()
{
    B = (Boolean)isSchemeUpper( (Text)(R.u.c) );
    }

/**** charIsWhitespacePMC ****/

void charIsWhitespacePMC()
{
    B = (Boolean)isWhiteSpace( (Text)(R.u.c) );
    }

/**** charToLower ****/

void charToLower()
{
    R.u.c = schemeToLower( (int)(R.u.c) );
    }

/**** charToUpper ****/

void charToUpper()
{
    R.u.c = schemeToUpper( (int)(R.u.c) );
    }

/**** cons ****/

void cons()             /* first argument on stack, second in R */
{
    LongWord offsetKittenNumber = (LongWord)theKittenNumber << kittenByteOffset;

    if( ! leaf_p( R.t ) )
        R = spliceOutNotCarForwardingPointers( R );

    if( ! leaf_p( (S[0]).t ) )
        S[0] = spliceOutNotCarForwardingPointers( S[0] );

    switch( pure_type_code( R.t ) ) {
        case emptyListTag:
            theHeap->allocateOne();
            (S[0]).t |= carCdrNil;
            break;
        case pairTag:                               /* can we cdr-code? */
            if( theHeap->consLuckOutOneTaggedAval( (Byte *)(R.u.c) ) )               // Yes!!
                (S[0]).t |= carCdrNext;
            else {                                  /* no, and we must install a forwarding pointer in the cdr */
                theHeap->allocateTwo();
                R.t = forwardingPointerTag;
                (S[0]).t |= carCdrNext;
                *(A.u.a + 1) = R;
                (A.u.a + 1)->t &= ~(kittenByteBitsMask);
                (A.u.a + 1)->t &= ~pointsToActiveBitMask;
                (A.u.a + 1)->t |= offsetKittenNumber;
                }
            break;
        default:
            theHeap->allocateTwo();
            (S[0]).t |= carCdrNext;
            *(A.u.a + 1) = R;
            (A.u.a + 1)->t &= ~(kittenByteBitsMask);
            (A.u.a + 1)->t &= ~pointsToActiveBitMask;
            (A.u.a + 1)->t |= offsetKittenNumber;
            break;
        }
    ((A.u.a)->u).c = (S[0]).u.c;
    (A.u.a)->t = (((S[0]).t) & (~(kittenByteBitsMask))) | offsetKittenNumber;
    (A.u.a)->t &= ~pointsToActiveBitMask;
    SDrop();
    R.u.a = A.u.a;
    R.t = pairTag;
    }

/**** dereferenceEnvironmentReferenceMC -- Dereference the passed environment reference.                ****/

extern void printDynamicEnvironmentList();

void dereferenceEnvironmentReferenceMC()
{
    UnsignedLongWord whichEnv = (UnsignedLongWord)(R.u.c) >> environmentNumberOffset;
    UnsignedLongWord whichVar = (UnsignedLongWord)(R.u.c) & lowerWordMask;

    R = E;
    while( whichEnv-- > 0 )
        cdr();         
    car();         
    while( whichVar-- > 0 ) {
        cdr();         
        }
    car();         
    car();         
    }

/**** dereferenceStackReferenceMC -- Dereference the passed stack reference.            ****/

void dereferenceStackReferenceMC()
{
    R = S[ (UnsignedLongWord)(R.u.c) ];
    if( forwarding_pointer_p( R.t ) ) {
        R = spliceOutNotCarForwardingPointers( R );
        R = *(R.u.a);
        }
    R.t &= ~(cdrCodingBitsMask | pointsToActiveBitMask);  
    }

/**** dereferenceTopLevelEnvironmentReferenceMC 
           -- Dereference the passed toplevel environment reference.               ****/

void dereferenceTopLevelEnvironmentReferenceMC()
{
    UnsignedLongWord whichBucket = (UnsignedLongWord)(R.u.c) >> bucketNumberOffset;
    UnsignedLongWord whichVar    = (UnsignedLongWord)(R.u.c) & lowerWordMask;

    R = TopLevelE;
    car();         
    vectorRefN( whichBucket );
    while( whichVar-- > 0 )
        cdr();         
    car();         
    car();         
    }

/**** dereferenceTopLevelEnvironmentReferenceForSymbolMC 
           -- Dereference the passed top-level environment reference, and retrieve the symbol associated therewith.      ****/

void dereferenceTopLevelEnvironmentReferenceForSymbolMC()
{
    UnsignedLongWord whichBucket = (UnsignedLongWord)(R.u.c) >> bucketNumberOffset;
    UnsignedLongWord whichVar    = (UnsignedLongWord)(R.u.c) & lowerWordMask;

    R = TopLevelE;
    car();         
    vectorRefN( whichBucket );
    while( whichVar-- > 0 )
        cdr();         
    car();         
    cdr();         
    }

/**** describeCurrentMainSpace ****/

void describeCurrentMainSpace()
{
    theHeap->describeCurrentMainSpace();
    }

/**** describeRoomOp ****/

void describeRoomOp()
{
    theHeap->describeCurrentRoom();
    }

/**** displayObject ****/

void displayObject()
{
    theHeap->triggerGC();
    (void)((dataTypeTable[pure_type_code(R.t)])->display)( &CurrentOutputPort );
    if( getPortFD( &CurrentOutputPort ) != conOut )
        fflush( conOut->fp );
    }

/**** displayObjectOnPort ****/

void displayObjectOnPort()
{

    theHeap->triggerGC();
    (void)((dataTypeTable[pure_type_code(R.t)])->display)( &(S[0]) );
    if( getPortFD( &CurrentOutputPort ) != conOut )
        tfflushConsoleOutput();
    }

/**** dropNFromStack -- The top of the stack contains a number; below it are that number of
                        things to be dropped.  Drop all, including the number.                               ****/

void dropNFromStack()
{
    LongWord nDrop = S[0].u.c;

    S += nDrop + 1;
    }

/**** equalAux ****/

void equalAux()
{
    eqPMC();
    if( B )
        return;
    switch( pure_type_code( R.t ) ) {
        case stringTag:
            if( string_p( (S[0]).t ) )
                stringEqual();
            else
                B = (Boolean)FALSE;
            break;

        case syntaxBindingTag:
            if( ! syntax_binding_p( (S[0]).t ) ) {
                B = (Boolean)FALSE;
                break;
                }
            R.t = pairTag;
            (S[0]).t = pairTag;
            goto pairFollowUp;

        case multipleValuesTag:
            if( ! multiple_values_p( (S[0]).t ) ) {
                B = (Boolean)FALSE;
                break;
                }
            R.t = pairTag;
            (S[0]).t = pairTag;
            goto pairFollowUp;

        /* NOTA BENE -- no "break" necessary ... */

        case vectorTag:
            if( ! vector_p( (S[0]).t ) ) {
                B = (Boolean)FALSE;
                break;
                }
            R.t = pairTag;
            (S[0]).t = pairTag;
            goto pairFollowUp;

        /* NOTA BENE -- no "break" necessary ... */

        case pairTag:
            if( ! pair_p( (S[0]).t ) ) {
                B = (Boolean)FALSE;
                break;
                }

        /* NOTA BENE -- no "break" necessary ... */

  pairFollowUp:
            SPushR();
            R = S[1];
            car();         
            SPushR();
            R = S[1];
            car();         
            equalAux();
            if( B ) {
                SPopR();
                SPopR();
                SPushR();
                R = S[1];
                cdr();         
                SPushR();
                R = S[1];
                cdr();         
                equalAux();
                }
            SDrop();
            SPopR();
            break;
        default:
            eqvPMC();
        }
    }                       

/**** equalPMC ****/

void equalPMC()
{
    eqPMC();
    if( B )
        return;
    switch( pure_type_code(R.t) ) {
        case stringTag:
        case vectorTag:
        case syntaxBindingTag:
        case multipleValuesTag:
        case pairTag:
            SPushR();
            R = S[1];
            SPushR();
            R = S[1];
            equalAux();
            SDrop();
            SPopR();
            break;
        case pointerToByteBlockTag:
            if( pointer_to_byte_block_p( (S[0].t ) ) )
                B = ( *((UnsignedLongWord *)(R.u.c)) == *((UnsignedLongWord *)((S[0]).u.c)) );
            else
                B = FALSE;
            break;
        default:
            eqvPMC();
        }
    }

/**** eqPMC ****/

void eqPMC()
{ 
    taggedAval dummy;

    dummy = S[0];
    switch( mutable_type_code(R.t) ) {
/*
    To make () operationally equivalent to #f, substitute
    the following two cases for the next two thereafter:

        case emptyListTag:
        case falseBooleanTag:
            B = (Boolean)((mutable_type_code(dummy.t) == emptyListTag) || (mutable_type_code(dummy.t) == falseBooleanTag));
            break;
*/
        case emptyListTag:
        case falseBooleanTag:
        case trueBooleanTag:
        case failTag:
        case succeedTag:
            B = (Boolean)(mutable_type_code(dummy.t) == mutable_type_code(R.t));
            break;
        case stringTag:
        case pairTag:
        case vectorTag:
            B = (Boolean)(mutable_type_code(dummy.t) == mutable_type_code(R.t)) && (dummy.u.c == R.u.c);
            break;
        case symbolTag:
            B = (Boolean)(mutable_type_code(dummy.t) == mutable_type_code(R.t)) && (dummy.u.c == R.u.c);
            break;
        default:
            B = (Boolean)((mutable_type_code(dummy.t) == mutable_type_code(R.t)) && (dummy.u.c == R.u.c));
            break;
        }
    }

/**** eqvPMC ****/

void eqvPMC()
{ 
    eqPMC();

    if( B )
        return;
    else if( number_p(R.t) && number_p((S[0]).t) ) {
        if( exact_bit(R.t) == exact_bit((S[0]).t) )
            numEqualPMC();
        else
            B = (Boolean)FALSE;
        }
    else
        B = (Boolean)FALSE;
    }

/**** evenPMC ****/

void evenPMC() { B = (Boolean)((R.u.c % 2) ? FALSE : TRUE); }

/**** exactIntegerPMC ****/

void exactIntegerPMC()
{
    if( number_p( R.t ) && exact_p( R.t ) ) {
        if( fixnum_p(R.t) )
            B = (Boolean)TRUE;
        else 
            B = (Boolean)( floorShortFloat( R.u.f ) == R.u.f);
        }
    else
        B = (Boolean)FALSE;
    }                       

/**** genSymMC() -- Create a C string of the form "gXXXX", where the "X..." are a number.

               This routine makes no attempt to determine whether or not a symbol
               with that print-name already exists; but it does use a private
               static for the number, and increments it each time 'round.  Thus
               the caller is likely to get a new print name, and probably will not
               not have to iterate too many times to find one.  (The caller
               presumably knows how to check, via functions related to "intern").

               The caller must be careful to do something sensible with this C
               string immediately; the buffer in which it is contained will be
               overwritten by subsequent calls to genSymMC.                                                 ****/

static Text genSymBuffer[ 32 ];

void genSymMC()
{
    sprintf( genSymBuffer, MiscStrings[ 128 ], theClowder->nextGenSymNumberForKitten( theKittenNumber ) );
    R.u.c = (UnsignedLongWord)genSymBuffer;
    R.t = exactShortFixnumBitsMask;
    }       

/**** getHashValue ****/

void getHashValue()      /* symbol in R; returned R is its hash value */
{
    R.t = pairTag;
    cdr();         
    car();         
    }

/**** getPName ****/

void getPName()         /* symbol in R; returned R is its pname */
{
    R.t = pairTag;
    car();         
    }

/**** getBlockedPName ****/

void getBlockedPName()         /* symbol in R; returned R is its pname */
{
    R.t = pairTag;
    carRegardlessOfBlocking();
    }

/**** getPeculiarNonLeafNumberMC ****/

void getPeculiarNonLeafNumberMC()       /* peculiar non-leaf in R; returned R is its number */
{
    R.u.c = getPeculiarNonLeafNumber( R.u.a );
    R.t = exactShortFixnumBitsMask;
    }

/**** getBigArrayNumberMC ****/

void getBigArrayNumberMC()       /* peculiar non-leaf in R; returned R is its number */
{
    R.u.c = getBigArrayLocation( R.u.a );
    R.t = exactShortFixnumBitsMask;
    }

/**** getPortBigArrayNumberMC ****/

void getPortBigArrayNumberMC()       /* port in R; returned R is its number */
{
    R.u.c = getPortBigArrayLocation( &R );
    R.t = exactShortFixnumBitsMask;
    }

/**** getSymbolBindingValue ****/

void getSymbolBindingValue()            /* binding pair in R; returned R is its value */
{
    R.t = pairTag;
    car();         
    }

/**** getSymbolBindingValueNoCheck ****/

void getSymbolBindingValueNoCheck()             /* binding pair in R; returned R is its value */
{
    R.t = pairTag;
    car();         
    }

/**** hashString ****/

void hashString()               /* string in R; returns its hash value (sum of characters) */
{
    UnsignedLongWord hashValue, index, length;
    Text *start;

    if( ! string_p( R.t ) ) {
        useErrorOutput();
        writeTaggedAvalSafely( &R );
        resetError( 169 );
        }
    length = lengthString( &R );
    start  = startString( &R );
    hashValue = 0;
    for( index = 0; index < length; ++index )
        hashValue = HASH_MULTIPLIER * hashValue + *start++;
    hashValue %= SYMBOL_HASH_CONSTANT;
    R.t = exactShortFixnumBitsMask;
    R.u.c = hashValue;
    }

/**** hashText ****/

void hashText()         /* string pointer in R; returns its hash value (sum of characters) */
{
    UnsignedLongWord count, hashValue;
    Text *start;

    start  = (Text *)(R.u.c);
    count = 0;
    hashValue = 0;
    while( *start )
        hashValue = HASH_MULTIPLIER * hashValue + *start++;
    hashValue %= SYMBOL_HASH_CONSTANT;
    R.t = exactShortFixnumBitsMask;
    R.u.c = hashValue;
    }

/**** inspectObject ****/

void inspectObject()
{
    (void)((dataTypeTable[pure_type_code(R.t)])->inspect)( &CurrentOutputPort );
    if( getPortFD( &CurrentOutputPort ) != conOut )
        tfflushConsoleOutput();
    }

/**** inspectObjectOnPort ****/

void inspectObjectOnPort()
{
    (void)((dataTypeTable[pure_type_code(R.t)])->inspect)( &(S[0]) );
    if( getPortFD( &CurrentOutputPort ) != conOut )
        tfflushConsoleOutput();
    }

/**** integerPMC ****/

void integerPMC()
{
    if( number_p( R.t ) ) {
        if( fixnum_p(R.t) )
            B = (Boolean)TRUE;
        else
            B = (Boolean)( floorShortFloat( R.u.f ) == R.u.f);
        }
    else
        B = (Boolean)FALSE;
    }                       

/**** lengthFunction ****/

UnsignedLongWord lengthFunction()
{
    UnsignedLongWord lnth = 0;

    while( ! empty_list_p(R.t) ) {
        if( pair_p(R.t) ) {
            cdr();         
            ++lnth;
            }
        else {
            resetError( 171 );
            }
        }
    return lnth;
    }

/**** lengthOp ****/

void lengthOp()
{
    LongWord lnth;

    lnth = (LongWord)lengthFunction();
    R.t = exactShortFixnumBitsMask;
    R.u.c = lnth;
    }

/**** listTopN -- N things to be listed are stacked, cdr-end on top; N is in R. ****/

void listTopN()
{
    LongWord n;

    if( (n = R.u.c) <= 0 )
        R.t = emptyListTag;
    else {
        SPopR();
        nilCons();
        while( --n > 0 )
            cons();
        }
    }

/**** makeImmutable ****/

void makeImmutable()
{
    LongWord counter, vLength;

    if( immutable_p( R.t ) )
        ;                                           /* All done. */
    else if( pair_p( R.t ) ) {                      /* Traverse, immutabilizing. */
        SPushR();
        SPushR();
        do {
            SDrop();
            if( immutable_p( R.t ) ) {
                SDrop();
                return;
                }
            SPushR();
            
            if( ! blockOnTagLock1Force( &R, 9 ) )
                resetError( 241 );
            (R.u.a)->t |= immutableBitMask;
            releaseTagLock1Force( (volatile taggedAval *)(R.u.a), 17 );

            car();         
            makeImmutable();

            SPopR();
            SPushR();

            cdr();         

            } while( pair_p( R.t ) );

        if( empty_list_p( R.t ) ) {
            SDrop();
            SPopR();
            R.t |= immutableBitMask;
            }
        else {
            makeImmutable();
            setCdr();         // Pops the stack
            SPopR();
            R.t |= immutableBitMask;
            }
        }
    else if( vector_p( R.t ) ) {
        SPushR();
        R.t = pairTag;
        car();         
        vLength = R.u.c;              
        for( counter = 0; counter <= vLength; ++counter ) {
            R = S[0];
            R.u.a = R.u.a + counter;
            R.t = pairTag;
            makeImmutable();
            }
        SPopR();
        R.t |= immutableBitMask;
        }

    // else nothing

    }

/**** makeString ****/

void makeString( const Text *str )
{
    theHeap->allocateStringBlock( str );
    R.u.c = A.u.c;
    R.t = stringTag;
    (A.u.a)->t = byteBlockTag;
    }

/**** makeStringNEmpty ****/

void makeStringNEmpty( UnsignedLongWord n )
{
    theHeap->allocateStringNBlock( (size_t)n );
    R.u.c = A.u.c;
    R.t = stringTag;
    (A.u.a)->t = byteBlockTag;
    }

/**** nilCons ****/

void nilCons()          /* argument in R */
{
    Tag toWrite = 0;

    if( ! leaf_p( R.t ) )
        R = spliceOutNotCarForwardingPointers( R );

    theHeap->allocate( sizeof( taggedAval ) );
    toWrite = R.t | carCdrNil;
    toWrite &= ~(kittenByteBitsMask | pointsToActiveBitMask);
    toWrite |= ((LongWord)theKittenNumber << kittenByteOffset);

    ((A.u.a)->u).c = R.u.c;
    R.t = pairTag;
    R.u.a = A.u.a;

    (A.u.a)->t = toWrite;
    }

/**** numberToStringMC ****/

void numberToStringMC()
{
    static Text buffer[ MAX_OUTPUT_NUMBER_SIZE + 1 ];

    (void)formatNumber( buffer );
    makeString( buffer );
    }

typedef enum { ALL_SHORT_FIXNUMS, ALL_SHORT_FLOATS } comparisonCoercionResult;

/**** coerceForNumericComparison ****/

comparisonCoercionResult coerceForNumericComparison()
{
    if( short_fixnum_p( R.t ) && short_fixnum_p( (S[0]).t ) )
        return ALL_SHORT_FIXNUMS;

    if( ! short_float_p( R.t ) )
        coerceRToShortFloat();
    if( ! short_float_p( (S[0]).t ) )
        coerceS0ToShortFloat();

    if( nanShortFloatP( (S[0]).u.f ) || nanShortFloatP( R.u.f ) )
        resetError( 100 );
    if(    infShortFloatP( (S[0]).u.f )
        && infShortFloatP( R.u.f )
        && ( (S[0]).u.f == R.u.f ) )
        resetError( 103 );
   
    return ALL_SHORT_FLOATS;
    }

/**** nonNegativeIntegerPMC ****/

void nonNegativeIntegerPMC()
{
    if( number_p( R.t ) ) {
        if( fixnum_p(R.t) )
            B = (R.u.c >= 0);
        else
            B = (R.u.f >= 0.0) && (( floorShortFloat( R.u.f ) == R.u.f));
        }
    else
        B = (Boolean)FALSE;
    }                       

/**** numEqualPMC ****/

void numEqualPMC()
{
    SPushR();
    SPushR();
    R = S[1];
    S[0] = S[2];
    switch( coerceForNumericComparison() ) {
        case ALL_SHORT_FIXNUMS:
            B = (Boolean)((S[0]).u.c == R.u.c);
            break;
        case ALL_SHORT_FLOATS:
            B = (Boolean)( (S[0]).u.f == R.u.f );
            break;
        default:
            fatalError( 81 );
            break;
        }
   SDrop();
   SPopR();
    }

/**** numGreaterPMC ****/

void numGreaterPMC()
{
    SPushR();
    SPushR();
    R = S[1];
    S[0] = S[2];
    switch( coerceForNumericComparison() ) {
        case ALL_SHORT_FIXNUMS:
            B = (Boolean)((S[0]).u.c > R.u.c);
            break;
        case ALL_SHORT_FLOATS:
            B = (Boolean)((S[0]).u.f > R.u.f);
            break;
        default:
            fatalError( 82 );
            break;
        }
   SDrop();
   SPopR();
    }

/**** numGreaterEqualPMC ****/

void numGreaterEqualPMC()
{
    SPushR();
    SPushR();
    R = S[1];
    S[0] = S[2];
    switch( coerceForNumericComparison() ) {
        case ALL_SHORT_FIXNUMS:
            B = (Boolean)((S[0]).u.c >= R.u.c);
            break;
        case ALL_SHORT_FLOATS:
            B = (Boolean)((S[0]).u.f >= R.u.f);
            break;
        default:
            fatalError( 83 );
            break;
        }
   SDrop();
   SPopR();
    }

/**** numLessPMC ****/

void numLessPMC()
{
    SPushR();
    SPushR();
    R = S[1];
    S[0] = S[2];
    switch( coerceForNumericComparison() ) {
        case ALL_SHORT_FIXNUMS:
            B = (Boolean)((S[0]).u.c < R.u.c);
            break;
        case ALL_SHORT_FLOATS:
            B = (Boolean)((S[0]).u.f < R.u.f);
            break;
        default:
            fatalError( 84 );
            break;
        }
   SDrop();
   SPopR();
    }

/**** numLessEqualPMC ****/

void numLessEqualPMC()
{
    SPushR();
    SPushR();
    R = S[1];
    S[0] = S[2];
    switch( coerceForNumericComparison() ) {
        case ALL_SHORT_FIXNUMS:
            B = (Boolean)((S[0]).u.c <= R.u.c);
            break;
        case ALL_SHORT_FLOATS:
            B = (Boolean)((S[0]).u.f <= R.u.f);
            break;
        default:
            fatalError( 85 );
            break;
        }
   SDrop();
   SPopR();
    }

/**** oddPMC ****/

void oddPMC()
{
    B = (Boolean)((R.u.c % 2) ? TRUE : FALSE);
    }

/**** peekLoop -- reads from the port in STACK( 0 ), and must not change STACK( 0 ) ****/

void peekLoop()
{
    SDupR();
//    getInputDescriptor();
    SPushR();
    peekPort( &R );

    while( isWhiteSpace( (Text)(R.u.c) ) ) {
        skipWhiteSpacePort( &(S[0]) );
        peekPort( &(S[0]) );
        }
    SDrop();
    if( isEOF( R.u.c ) ) 
        return;

    R.u.c = peekLoop_index;             R.t = schemeMachinePCIndexTag; CSPush( R ); // Push code for another peek loop.
    R.u.c = tfflushConsoleOutput_index; R.t = schemeMachinePCIndexTag; CSPush( R ); // Push code to flush console output.
    R.u.c = displayObject_index;        R.t = schemeMachinePCIndexTag; CSPush( R ); // Push code to display the newline.
    R = newlineChar;                                                   CSPush( R ); // Push a newline.
    R.u.c = writeObject_index;          R.t = schemeMachinePCIndexTag; CSPush( R ); // Push code to write the result.
    R.u.c = CPushFunction_index;        R.t = schemeMachinePCIndexTag; CSPush( R ); // Push in continuation, to be evaluated.
    R.u.c = readSexpFromPort_index;     R.t = schemeMachinePCIndexTag; CSPush( R ); // Read sexp from the port in STACK( 0 )
    }

/**** positiveIntegerPMC ****/

void positiveIntegerPMC()
{
    if( number_p( R.t ) ) {
        if( fixnum_p(R.t) )
            B = (R.u.c > 0);
        else
            B = (R.u.f > 0.0) && (( floorShortFloat( R.u.f ) == R.u.f));
        }
    else
        B = (Boolean)FALSE;
    }                       

/**** procedurePMC ****/

void procedurePMC()
{
    B = (Boolean)( built_in_function_p( R.t ) || lambda_pair_p( R.t ) );
    }

/**** properListPMC  -- Find out whether something is a proper list, with no consing or recursing.

    The idea is to make two passes through the list:

      The first pass colors every cell it sees, and stops when it either finds the end
        of the list -- which it tests for proper or improper -- or encounters a previously-colored cell.

      The second pass uncolors every cell it sees, and stops when it either finds the end
        of the list -- which it tests for proper or improper -- or encounters an uncolored cell.

 ****/                             

void properListPMC()
{
    SPushR();              // Save some copies of R.
    SPushR(); 

   // First pass.

    theClowder->increaseKittenBlock();
    while( (! empty_list_p( R.t )) && pair_p( R.t ) && (! colored_p( R.u.a )) ) {
        color( R.u.a );  
        cdr();
        }

    B = empty_list_p( R.t );

    // Second pass.

    SPopR();
    while( (! empty_list_p( R.t )) && pair_p( R.t ) && (colored_p( R.u.a )) ) {
        uncolor( R.u.a );
        cdr();
        }

    theClowder->reduceKittenBlock();

    SPopR();

    }                                                                               

/**** pushListOnStack -- Actually, leaves last element in R ****/

void pushListOnStack()
{
    if( ! empty_list_p( R.t ) ) {                   /* leave stack balanced with empty list */
        while( ! empty_list_p( R.t ) ) {
            SPushR();
            car();         
            Swap();
            cdr();         
            }
        SPopR();
        }
    }

/**** quotient1 --      R <-- STACK(0) div R    ****/

void quotient1()
{
    if( R.u.c == 0 ) {
        R = S[0];
        useErrorOutput();
        displayObject();
        resetError( 162 );
        }
    R.u.c = ((S[0].u.c) / R.u.c);
    R.t = shortFixnumBitsMask;
    }

/**** rebindForce -- Entered with R: expression; S:(forceLoc ...), returns the expresssion ****/

void rebindForce()
{
    SPushR();
    SPushR();
    S[0] = S[2];                    /* R: expression; S: (forceLoc expression forceLoc ...) */
    setCar();
    SPushR();
    S[0] = S[2];            
    R.t = emptyListTag;             /* R: (); S: (forceLoc expression forceLoc ...) */
    setCdr();
    SPopR();
    SDrop();
    }

/**** remainder1 --     R <-- remainder( STACK(0) / R ) 
                          ("remainder" conflicts with a name in the Apple S.A.N.E. library.) ****/

void remainder1()
{
    if( R.u.c == 0 ) {
        R = S[0];
        useErrorOutput();
        displayObject();
        resetError( 94 );
        }
    R.u.c = ((S[0]).u.c % R.u.c);
    R.t = shortFixnumBitsMask;
    }

/**** resetScheme ****/

void resetScheme() { longjmp( ResetErrorBuffer, FULL_RESET ); }

/**** resetSchemeFinal ****/

void resetSchemeFinal() { longjmp( ResetErrorBuffer, FINAL_RESET ); }

/**** resetEverybody ****/

void resetEverybody()
{ 
    UnsignedWord aKitten = 0;

    for( aKitten = 1; aKitten < theNumberOfKittens; ++aKitten )
        theClowder->momDialogWithKitten( aKitten, MOM_SAYS_RESET, KITTEN_ACKNOWLEDGES_RESET );
    longjmp( ResetErrorBuffer, FULL_RESET );
    }

/**** returnRoomOp ****/

void returnRoomOp()
{
    R.u.c = theHeap->currentRoom();
    R.t = exactShortFixnumBitsMask;
    }       

/**** returnActiveRoomOp ****/

void returnActiveRoomOp()
{
    R.u.c = theHeap->currentActiveRoom();
    R.t = exactShortFixnumBitsMask;
    }       

/**** returnAgingRoomOp ****/

void returnAgingRoomOp()
{
    R.u.c = theHeap->currentAgingRoom();
    R.t = exactShortFixnumBitsMask;
    }       

/**** returnStoreSizeOp ****/

void returnStoreSizeOp()
{
    R.u.c = theHeap->currentStoreSize();
    R.t = exactShortFixnumBitsMask;
    }       

/**** returnActiveStoreSizeOp ****/

void returnActiveStoreSizeOp()
{
    R.u.c = theHeap->currentActiveStoreSize();
    R.t = exactShortFixnumBitsMask;
    }       

/**** return90PercentFlagOp ****/

void return90PercentFlagOp()
{
    if( theHeap->get90PercentFlag() )
        R.t = trueBooleanTag;
    else
        R.t = falseBooleanTag;
    }       

/**** setCar ****/

void setCar()           /* pair on stack, value in R */
{
    Tag toTransfer = 0;

    if( ! blockOnTagLock1( &(S[0]), 10 ) )                  // Lock original pair.
        resetError( 297 );
    SPushR();
    R = S[1];                                        // R: thePair; S: theValue, thePair ...
    if( ! pair_p( pure_type_code( R.t ) ) ) {
        useErrorOutput();
        displayObject();
        releaseTagLock1( (volatile taggedAval *)((S[1]).u.a), 18 );
        SDropN( 2 );
        resetError( 173 );
        }

    R = spliceOutNotCarForwardingPointers( R );

    if( R.u.a != (S[1]).u.a ) {
        releaseTagLock1( (volatile taggedAval *)((S[1]).u.a), 19 );
        if( ! blockOnTagLock1( &R, 11 ) )
            resetError( 297 );
        }

    /* R now points directly to the current car of the pair -- no more forwarding pointers */

    toTransfer = ((S[0]).t) & (~(cdrCodingBitsMask | administrationBitsMask | pointsToActiveBitMask));
    toTransfer |= ((R.u.a)->t & (cdrCodingBitsMask | administrationBitsMask | pointsToActiveBitMask));
    ((R.u.a)->u).c = ((S[0]).u.c);
    (R.u.a)->t = toTransfer;

    theClowder->addToLiveAddressStack( R.u.a );

    if( hasLocationByte( toTransfer ) )
        releaseTagLock1( (volatile taggedAval *)(R.u.a), 21 );

    SDropN( 2 );
    }

/**** setCarWithPairAlreadyBlockedKeepBlock ****/

void setCarWithPairAlreadyBlockedKeepBlock()
{
    Tag toTransfer = 0;

    SPushR();
    R = S[1];                                        // R: thePair; S: theValue, thePair ...
    if( ! pair_p( pure_type_code( R.t ) ) ) {
        useErrorOutput();
        displayObject();
        releaseTagLock1( (volatile taggedAval *)((S[1]).u.a), 22 );
        SDropN( 2 );
        resetError( 200 );
        }

    R = spliceOutNotCarForwardingPointers( R );  // R: theForwardedPair; S: theValue, thePair ...

    /* R now points directly to the current car of the pair -- no more forwarding pointers */

    toTransfer = ((S[0]).t) & (~(cdrCodingBitsMask | administrationBitsMask | pointsToActiveBitMask));
    toTransfer |= ((R.u.a)->t & (cdrCodingBitsMask | administrationBitsMask | pointsToActiveBitMask));
    ((R.u.a)->u).c = ((S[0]).u.c);
    (R.u.a)->t = toTransfer;

    theClowder->addToLiveAddressStack( R.u.a);

    SDropN( 2 );
    }

/**** setCdr ****/

void setCdr()           // pair on stack, value in R
{
    Tag toTransfer = 0;

    if( ! pair_p( ((S[0]).t) ) ) {
        useErrorOutput();
        displayObject();
        resetError( 174 );
        }
    Swap();                                              // R: pair; S: (value ...)
    if( ! blockOnTagLock1( &R, 12 ) )
        resetError( 299 );
    SPushR();                                            // R: pair; S: (pair, value ...)
    R = spliceOutNotCarForwardingPointers( R );          // R points to current car of pair, with no forwarding pointers
                                                         //   That is, R -- finalPair, S: (pair, value ...)
    R.t &= ~pointsToActiveBitMask;  // TEST

    if( R.u.a != (S[0]).u.a ) {
        releaseTagLock1( (volatile taggedAval *)((S[0]).u.a), 24 );
        if( ! blockOnTagLock1( &R, 13 ) )
            resetError( 299 );
        }

    SPushR();                                            // R: finalPair, S: (finalPair, pair, value ...)
    SPushR();                                            // R: finalPair, S: (finalPair, finalPair, pair, value ...)
    S[0] = S[3];                                         // R: finalPair, S: (value, finalPair, pair, value ...)

    if( empty_list_p( (S[0]).t ) ) {                     // Setting cdr to '() -- just modify cdr-coding bits.
        (R.u.a)->t = ((R.u.a)->t) & (~cdrCodingBitsMask);
        (R.u.a)->t = (((R.u.a)->t) | carCdrNil);
        releaseTagLock1( (volatile taggedAval *)((S[1]).u.a), 25 );
        SDropN( 4 );
        }
    else
        switch( cdr_code( (R.u.a)->t ) ) {
            case carCdrNext:
            case carCdrNil:                             // R: finalPair; S: (value finalPair, pair, value ...)
    
                // "Next" is an actual object -- car-cdr-something or not-car;  There may be pointers to it (as (cdr R) or (cadr R))
                // extant already.  To preserve them, cons up a new pair, and replace what "pair" points to with a pointer to it.
    
                carRegardlessOfBlocking();
                SPushR();                               // S: (oldCar value finalPair, pair, value ...)
                R = S[1];
                cons();                                 // R: pair->(old car . value); S: (value finalPair, pair, value ...)
                R.t = forwardingPointerTag;             // R: forwarding pointer to that new cons cell
                R.t |= ((LongWord)theKittenNumber << kittenByteOffset);
                SDrop();                                // S: (finalPair, pair, value ...)

                Swap();                                 // R: finalPair; S: (->newConsCell, pair, value ...)

                // The drill here is that what R points to is already locked; we are going to keep it locked as we
                // bring in the new value, then unlock it.

                toTransfer = ((S[0]).t) & (~(administrationBitsMask));
                toTransfer |= ((R.u.a)->t & (administrationBitsMask));
                ((R.u.a)->u).c = ((S[0]).u.c);
                (R.u.a)->t = toTransfer;

                theClowder->addToLiveAddressStack( R.u.a );

                if( hasLocationByte( toTransfer ) )
                    releaseTagLock1( (volatile taggedAval *)(R.u.a), 27 );

                SDropN( 3 );                                // Stack is clear.
                break;
            case extraCdrCode:
                releaseTagLock1( (volatile taggedAval *)(R.u.a), 28 );

                SDropN( 4 );                                // Stack is clear.
                resetError( 214 );
                break;
            default:
                R = *(R.u.a);
                useFatalErrorOutput();
                (void)printAnyTaggedAval( &CurrentOutputPort, &R );
                releaseTagLock1( (volatile taggedAval *)((S[1]).u.a), 29 );
                fatalError( 11 );
                break;
            }
    }

/**** setUpApply -- auxiliary for "applyCode"; expects R containing the number of
                    arguments thereto, with S[0]... being the arguments.  If the number
                    of arguments is 2, the call was of the form (apply proc list);
                    if more than two it was of the form (apply proc arg* list);
                    if fewer than two it is an error undetected here.                    ****/

void setUpApply()
{
    LongWord nArgsGiven = R.u.c;
    LongWord stackPosition;
    UnsignedLongWord nArgsTotal;
    LongWord listLength = 0;

    R = S[0];                      // R is the list, proc is at S[ nArgsGiven - 1 ].
    listPMC();
    if( ! B ) {
        useErrorOutput();
        displayObject();
        resetError( 99 );
        }
    nArgsTotal = (listLength = lengthFunction()) + nArgsGiven - 2;
    SPopR();                       // R is list, Proc is now at S[ nArgsGiven - 2 ].
    if( listLength > 0 ) {
        pushListOnStack();         // Proc is now at S[ nArgsTotal - 1 ] (last list element is in R).
        SPushR();                  // Proc is now at S[ nArgsTotal ]
        }
    //                             // else Proc is where we left it, at nArgsGiven - 2, which is nArgsTotal.
    R.t = exactShortFixnumBitsMask;
    R.u.c = nArgsTotal;
    SPushR();                      // Proc is now at S[ nArgsTotal + 1 ]
    R = S[ nArgsTotal + 1 ];       // R is proc.
    SPushR();                      // Proc is now at S[ nArgsTotal + 2 ], and is superfluous.
    for( stackPosition = nArgsTotal + 2; stackPosition > 0; --stackPosition )
        S[ stackPosition ] = S[ stackPosition - 1 ];
    SDrop();                        
    R.t = schemeMachinePCIndexTag;
    R.u.c = apply_index;
    CSPush( R );
    }

/**** strcmpci -- case-independent version of strcmp ****/

Word strcmpci( Text *s, Text *t )
{
    for( ; schemeToLower( *s ) == schemeToLower( *t ); s++, t++ )
        if( *s == '\0' )
            return( (Word)0 );
    return( (Word)( schemeToLower( *s ) - schemeToLower( *t ) ) );
    }

/**** Stringappend ****/

void stringAppend()     /* R: secondString; S: (firstString ...) */
{
    UnsignedLongWord firstLength, secondLength;

    firstLength = lengthString( &S[0] );    
    secondLength = lengthString( &R );
    SPushR();
    makeStringNEmpty( (Word)(firstLength + secondLength) );
    strncpy( startString( &R ), startString( &(S[1]) ), (size_t)firstLength );
    strncpy( startString( &R ) + firstLength, startString( &(S[0]) ), (size_t)secondLength );
    SDrop();
    }       

/**** stringCopy ****/

void stringCopy()       /* returns a newly-allocated duplicate of its argument */
{
    UnsignedLongWord lnth;

    lnth = lengthString( &R );
    SPushR();
    makeStringNEmpty( lnth );
    strncpy( startString( &R ), startString( &(S[ 0 ]) ), (size_t)lnth );
    SDrop();
    }       

/**** stringEqual ****/

void stringEqual()      /* strings in R and on stack */
{
    taggedAval *block1 = R.u.a;
    taggedAval *block2 = (S[0]).u.a;
    UnsignedLongWord length1 = lengthString( &R );
    UnsignedLongWord length2 = lengthString( &(S[0]) );
    Text *str1 = 0;
    Text *str2 = 0;

    B = (Boolean)FALSE;

    if( length1 != length2 )
        return;                 /* strings do not match because of unequal length */
    else {
        str1 = (Text *)((Byte *)block1 + sizeof(taggedAval));
        str2 = (Text *)((Byte *)block2 + sizeof(taggedAval));
        while( length1-- > 0 )
            if( *str1++ != *str2++ )
                return;
        }
    B = (Boolean)TRUE;
    }

/**** stringEqualCI ****/

void stringEqualCI()            /* strings in R and on stack */
{
    taggedAval *block1 = R.u.a;
    taggedAval *block2 = (S[0]).u.a;
    UnsignedLongWord length1 = lengthString( &R );
    UnsignedLongWord length2 = lengthString( &(S[0]) );
    Text *str1 = (Text *)((Byte *)block1 + sizeof(taggedAval));
    Text *str2 = (Text *)((Byte *)block2 + sizeof(taggedAval));

    B = (Boolean)FALSE;

    if( length1 != length2 )
        return;                 /* strings do not match because of unequal length */
    else
        while( length1-- > 0 )
            if( schemeToLower( *str1++ ) != schemeToLower( *str2++ ) )
                return;
    B = (Boolean)TRUE;
    }

/**** stringEqualsTextPMC -- R.u.c is assumed to point to a null-terminated string, STACK( 0 )
                             is assumed to be a string taggedAval.  Are they the same?  Set
                             B accordingly, and destroy neither object.                                  ****/

void stringEqualsTextPMC()
{
    B = (Boolean)equalByteBlockToString( (S[0]).u.a, (Text *)(R.u.c) );
    }

/**** stringInsert() -- Destructively insert the string in R into the
                        string in STACK( 0 ), at the position given by STACK( 1 ).   ****/

void stringInsert()
{
    LongWord destinationLength, position, sourceCount, sourceLength;
    Text *insertionPoint, *sourcePoint;

    position = S[1].u.c;
    sourceCount = sourceLength = lengthString( &R );
    destinationLength = lengthString( &(S[0]) );
    insertionPoint = startString( &(S[0]) ) + position;
    sourcePoint = startString( &R );

    if( position + sourceLength > destinationLength )
        resetError( 176 );

#ifndef PIXIE_SCHEME_III           // Broken in -O3 for ARM, 2010.10.08
    while( sourceCount-- > 0 )
        *insertionPoint++ = *sourcePoint++;
#else
    while( TRUE ) {
        if( sourceCount <= 0 )
            break;
        sourceCount = sourceCount - 1;
        *insertionPoint++ = *sourcePoint++;
        }
#endif
    }

/**** stringLength() ****/

void stringLength()
{
    R.u.c = lengthString( &R );
    R.t = exactShortFixnumBitsMask;
    }

/**** stringLengthMany() -- Expects R: <don't-care>; S: (nStrings string* ...) -- returns in R
                            the combined length of all the strings (excluding terminal '\0's). ****/

void stringLengthMany()
{
    LongWord answer = 0;
    LongWord nStrings = S[0].u.c;

    while( nStrings > 0 )
        answer += (LongWord)lengthString( &(S[ nStrings-- ]) );
    R.u.c = answer;
    R.t = exactShortFixnumBitsMask;

    }

/**** stringRef() ****/

void stringRef()
{
    if( ! string_p( (S[0]).t ) ) {
        SPopR();
        useErrorOutput();
        displayObject();
        resetError( 177 );
        }       
    if( R.u.c < 0 ) {
        SPushR();
        R = S[1];
        useErrorOutput();
        displayObject();
        SPopR();
        writeNewlineError();
        displayObject();
        resetError( 178 );
        }
    if( (UnsignedLongWord)(R.u.c) >= lengthString( &(S[0]) ) ) {
        SPushR();
        R = S[1];
        useErrorOutput();
        displayObject();
        SPopR();
        writeNewlineError();
        displayObject();
        resetError( 179 );
        }
    R.u.c = refString( &(S[0]), (LongWord)(R.u.c) );
    R.t = characterTag;
    }

/**** stringSet ****/

void stringSet()
{
    if( ! string_p( (S[1]).t ) ) {
        SDrop();
        SPopR();
        useErrorOutput();
        displayObject();
        resetError( 180 );
        }       
    if( (S[0]).u.c < 0 ) {
        R = S[1];
        useErrorOutput();
        displayObject();
        R = S[0];
        writeNewlineError();
        displayObject();
        resetError( 181 );
        }
    if( (UnsignedLongWord)(S[0].u.c) >= lengthString( &(S[1]) ) ) {
        R = S[1];
        useErrorOutput();
        displayObject();
        R = S[0];
        writeNewlineError();
        displayObject();
        resetError( 182 );
        }
    setString( &(S[1]), (LongWord)((S[0]).u.c), (Text)(R.u.c) );
    }

/**** stringToLower ****/

void stringToLower()
{
    Text *str;
    UnsignedLongWord toGo;

    toGo = lengthString( &R );
    str = startString( &R );

    while( toGo > 0 ) {
        *str = schemeToLower( *str );
        ++str;
        --toGo;
        }
    }

/**** stringToNumber ****/

void stringToNumber()
{
    Text *str;
    LongWord radix = (S[0]).u.c;

    str = startString( &R );
    R = constructNumber( str, radix, FALSE );
    SDrop();
    }

/**** stringToUpper ****/

void stringToUpper()
{
    Text *str;
    UnsignedLongWord toGo;

    toGo = lengthString( &R );
    str = startString( &R );

    while( toGo > 0 ) {
        *str = schemeToUpper( *str );
        ++str;
        --toGo;
        }
    }

/**** stringGreaterEqualPMC ****/

void stringGreaterEqualPMC()   { B = (Boolean)(strcmp( startString( &(S[0]) ), startString( &R ) )     >= 0); }

/**** stringGreaterPMC ****/

void stringGreaterPMC()        { B = (Boolean)(strcmp( startString( &(S[0]) ), startString( &R ) )     > 0); }

/**** stringLessEqualPMC ****/

void stringLessEqualPMC()      { B = (Boolean)(strcmp( startString( &(S[0]) ), startString( &R ) )     <= 0); }

/**** stringLessPMC ****/

void stringLessPMC()           { B = (Boolean)(strcmp( startString( &(S[0]) ), startString( &R ) )     < 0); }

/**** stringGreaterEqualCIPMC ****/

void stringGreaterEqualCIPMC() { B = (Boolean)(strcmpci( startString( &(S[0]) ), startString( &R ) )   >= 0); }

/**** stringGreaterCIPMC ****/

void stringGreaterCIPMC()      { B = (Boolean)(strcmpci( startString( &(S[0]) ), startString( &R ) )   > 0); }

/**** stringLessEqualCIPMC ****/

void stringLessEqualCIPMC()    { B = (Boolean)(strcmpci( startString( &(S[0]) ), startString( &R ) )   <= 0); }

/**** stringLessCIPMC ****/

void stringLessCIPMC()         { B = (Boolean)(strcmpci( startString( &(S[0]) ), startString( &R ) )   < 0); }

/**** substring ****/

void substring()
{
    Text *startOfString;
    UnsignedLongWord begin, end;

    startOfString = startString( &(S[1]) );
    begin = (UnsignedLongWord)((S[0]).u.c);
    end = (UnsignedLongWord)(R.u.c);
    makeStringNEmpty( (end - begin) );
    strncpy( startString( &R ), startOfString + begin, (Word)(end - begin) );
    }       

/**** useErrorOutput ****/

void useErrorOutput()
{
    CurrentOutputPort = ErrorPort;
    writeNewline( &CurrentOutputPort );
    }

/**** vectorLength ****/

void vectorLength()
{
    R.t = pairTag;
    car();         
    }

/**** vectorRefN ****/

void vectorRefN( LongWord n )   /* vector in R */
{
    if( ! vector_p( R.t ) ) {
        useErrorOutput();
        displayObject();
        resetError( 183 );
        }       
    R.t = pairTag;
    if( n < 0 )
        resetError( 184 );
    SPushR();
    car();         
    if( R.u.c <= n++ ) {
        useErrorOutput();
        tprintf4( &CurrentOutputPort, MiscStrings[ 310 ], R.u.c, n - 1 );
        resetError( 185 );
        }
    SPopR();
    R.u.a = R.u.a + n;
    car();         
    }

/**** vectorRef ****/

void vectorRef()        /* number in R, vector on stack */
{
    LongWord n;

    n = R.u.c;
    SPopR();
    vectorRefN( n );
    }

/**** vectorSet ****/

void vectorSet()        /* object in R, index and then vector on stack */
{
    LongWord n;

    if( ! vector_p( S[1].t ) ) {
        SDrop();
        SPopR();
        useErrorOutput();
        displayObject();
        resetError( 186 );
        }       
    (S[1]).t = pairTag;        
    (S[1]).t &= ~cdrCodingBitsMask;  
    if( (n = (S[0]).u.c) < 0 )
        resetError( 187 );
    SDrop();
    SPushR();                                               /* S: (object vector ...) */
    R = S[1];
    car();         
    if( R.u.c <= n++ )
        resetError( 188 );
    SPopR();
    (S[0]).u.a = ((S[0]).u.a) + n;
    setCar();
    }

/**** writeChar ****/

void writeChar()
{
    tprintf3( &CurrentOutputPort, MiscStrings[ 57 ], (Text)(R.u.c) );
    if( getPortFD( &CurrentOutputPort ) != conOut )
        tfflushConsoleOutput();
    }

/**** writeCharOnPort ****/

void writeCharOnPort()
{
    tprintf3( &(S[0]), MiscStrings[ 57 ], (Text)(R.u.c) );
    if( getPortFD( &CurrentOutputPort ) != conOut )
        tfflushConsoleOutput();
    }

/**** writeObject ****/

void writeObject()
{
    theHeap->triggerGC();
    (void)((dataTypeTable[pure_type_code(R.t)])->write)( &CurrentOutputPort );
    if( getPortFD( &CurrentOutputPort ) != conOut )
        tfflushConsoleOutput();
    }

/**** writeObjectOnPort ****/

void writeObjectOnPort()
{
    theHeap->triggerGC();
    (void)((dataTypeTable[pure_type_code(R.t)])->write)( &(S[0]) );
    if( getPortFD( &CurrentOutputPort ) != conOut )
        tfflushConsoleOutput();
    }

/**** writeStack -- Mostly for debugging. ****/

void writeStack()
{
    taggedAval *scratch = S;
    taggedAval *stackEnd = (taggedAval *)(theHeap->schemeStackEnd());

    tprintf3( &CurrentOutputPort, MiscStrings[ 358 ], theHeap->schemeStackAddress());
    while( scratch < stackEnd ) {
        tprintf3( &CurrentOutputPort, MiscStrings[ 260 ], (void *)scratch );
        (void)printAnyTaggedAval( &CurrentOutputPort, scratch++ );
        tprintf2( &CurrentOutputPort, "\n");
        fflush( getPortFilePointer( &CurrentOutputPort ) );
        }
    tprintf2( &CurrentOutputPort, "\n");
    }

/**** reverseTopN -- Reverse the order of the top N elements of the stack; N is in R. ****/

void reverseTopN()
{
    LongWord nFar = R.u.c - 1;
    LongWord nNear = 0;
    taggedAval dummy;

    while( nFar > nNear ) {
        dummy = S[nFar];
        S[nFar--] = S[nNear];
        S[nNear++] = dummy;
        }
    }

/**** executeBoundSymbol ****/

void executeBoundSymbol( const Text *nonHeapSymbolName )
{
    SPushR();
    R.t = exactShortFixnumBitsMask;
    R.u.c = (LongWord)nonHeapSymbolName;
    intern();
    SPushR();
    boundSymbolP();
    SPopR();
    if( B ) {
        SPushR();
        R.u.c = SPopRFunction_index;  R.t = schemeMachinePCIndexTag;              CSPush( R );
        SPopR();
        lookUp();                                                     nilCons();  CSPush( R );
        R.u.c = SPushFunction_index;  R.t = schemeMachinePCIndexTag;              CSPush( R );
        SPopR();
        }
    SPopR();
    }

// Code from here to the end of this file is in development as of 2010.04.29.

/**** portableP -- for use in calculating object sizes, et cetera ****/

Boolean portableP( taggedAval *tp )
{
    switch( (tp->t) & pureTypeCodeBitsMask ) {

        // Note: Intentional fall-through in next block of cases, all return FALSE

        case mmappedBlockTag:
        case portTag:
        case stackReferenceTag:
        case symbolTag:
        case topLevelEnvironmentReferenceTag:
            return FALSE;

        case pairTag:
        case forwardingPointerTag:
            if( (tp->u).a == TopLevelE.u.a )
                return FALSE;
            if( (tp->u).a == ObList.u.a )
                return FALSE;
            return TRUE;

        default:
            return TRUE;

        }
    }

/**** calculatePortableObjectSizeAux -- FIXME -- DANGER WILL ROBINSON -- this version is recursive, with recursive blocks ... ****/

UnsignedLongWord calculatePortableObjectSizeAux( taggedAval *tp, Boolean *failed, UnsignedLongWord *errorCode )
{
    UnsignedLongWord answer = 0;
    taggedAval *nextTP = 0;

     if( colored_p( tp ) )
         return 0;            // We have already been there, so nothing to do.

     if( ! blockOnTagLock1( tp, 54 ) )
         resetError( 83 );

     color( tp );

    if( portableP( tp ) ) {

        answer = lengthTaggedAvalPointer( tp );

        if( ! leaf_p( tp->t ) )
            answer = answer + calculatePortableObjectSizeAux( (tp->u).a, failed, errorCode );

        if( *failed ) {
            releaseTagLock1( tp, 71 );
            return 0;
            }

        switch( cdr_code( tp->t ) ) {
            case carCdrNext:
                nextTP = tp + 1;
                nextTP = spliceOutNotCarForwardingPointersIndirect( nextTP );
                answer = answer + calculatePortableObjectSizeAux( nextTP, failed, errorCode );
                break;
            case carCdrNil:
                break;
            case extraCdrCode:
                useFatalErrorOutput();
                (void)printAnyTaggedAval( &CurrentOutputPort, tp );
                tprintf2( &CurrentOutputPort, MiscStrings[ 115 ] );
                (void)printAnyTaggedAval( &CurrentOutputPort, (tp->u).a );
                *failed = TRUE;
                *errorCode = 65;
                break;
            default:
                useFatalErrorOutput();
                (void)printAnyTaggedAval( &CurrentOutputPort, tp );
                tprintf2( &CurrentOutputPort, MiscStrings[ 115 ] );
                (void)printAnyTaggedAval( &CurrentOutputPort, (tp->u).a );
                *failed = TRUE;
                *errorCode = 73;
                break;
            }
        }
    else {    // Not portableP
        answer = sizeof( taggedAval );
        }

    releaseTagLock1( tp, 70 );

    if( *failed )
        return 0;
    else 
        return answer;
    }

/**** calculatePortableObjectSize ****/

UnsignedLongWord calculatePortableObjectSize()
{
    UnsignedLongWord answer = 0;
    Boolean failed = FALSE;
    UnsignedLongWord errorCode = 0;

    theClowder->increaseKittenBlock();
    theClowder->clearColoringBits();

    answer = calculatePortableObjectSizeAux( R.u.a, &failed, &errorCode );

    theClowder->clearColoringBits();
    theClowder->reduceKittenBlock();

    if( failed )
        fatalError( errorCode );

    return answer;
    }

/****************************************************************

// FIXME -- remember to put this stuff into the .e file.

//////// dupicateStructureAux -- FIXME -- DANGER WILL ROBINSON -- this version is recursive, with recursive blocks ...

// FIXME -- remember to check for sufficient space and do GC if necessary.

taggedAval *duplicateStructureAux( taggedAval *tp, taggedAval *currentConsLocation, taggedAval *pastConsLocations,
                                   Boolean *failed, UnsignedLongWord *errorCode )
{
    taggedAval *answer = 0;
    taggedAval *nextTP = 0;
    taggedAval *nextConsLocation = currentConsLocation;
    UnsignedLongWord thisSize = 0;

     if( colored_p( tp ) )
         return tp;            // We have already been there, so nothing to do.

     if( ! blockOnTagLock1( tp, 55 ) )
         resetError( 324 );

     color( tp );

    // BOOKMARK, have worked down to about here...

    if( portableP( tp ) ) {

        thisSize = lengthTaggedAvalPointer( tp );

        if( ! leaf_p( tp->t ) )
            answer = answer + duplicateStructureAux( (tp->u).a, failed, errorCode );

        if( *failed ) {
            releaseTagLock1( tp, 73 );
            return 0;
            }

        switch( cdr_code( tp->t ) ) {
            case carCdrNext:
                nextTP = tp + 1;
                nextTP = spliceOutNotCarForwardingPointersIndirect( nextTP );
                // Do some consing here
                answer = answer + duplicateStructureAux( nextTP, nextConsLocation, pastConsLocations,failed, errorCode );
                break;
            case carCdrNil:
                break;
            case extraCdrCode:
                useFatalErrorOutput();
                (void)printAnyTaggedAval( &CurrentOutputPort, tp );
                tprintf2( &CurrentOutputPort, MiscStrings[ 115 ] );
                (void)printAnyTaggedAval( &CurrentOutputPort, (tp->u).a );
                *failed = TRUE;
                *errorCode = 180;
                break;
            default:
                useFatalErrorOutput();
                (void)printAnyTaggedAval( &CurrentOutputPort, tp );
                tprintf2( &CurrentOutputPort, MiscStrings[ 115 ] );
                (void)printAnyTaggedAval( &CurrentOutputPort, (tp->u).a );
                *failed = TRUE;
                *errorCode = 181;
                break;
            }
        }
    else {  //  Deal with special cases here:
        thisSize = sizeof( taggedAval );
        }

    releaseTagLock1( tp, 72 );

    if( *failed )
        return 0;
    else 
        return answer;
    }

//////// duplicateStructure -- structure is passed in R.

taggedAval *duplicateStructure()
{
    taggedAval *answer = 0;
    Boolean failed = FALSE;
    UnsignedLongWord errorCode = 0;
    UnsignedLongWord consSize = 0;
    taggedAval *consBlock = 0;

    theHeap->allocate( consSize );

    theClowder->increaseKittenBlock();

    consSize = calculatePortableObjectSize();

    theClowder->clearColoringBits();

    answer = duplicateStructureAux( R.u.a, A.u.a, A.u.a + consSize, &failed, &errorCode );
    R.u.a = answer;

    theClowder->clearColoringBits();
    theClowder->reduceKittenBlock();

    if( failed )
        fatalError( errorCode );

    return answer;
    }

****************************************************************/
