/****************************************************************************************

SchemeAux.c++ -- Copyright (c) by Jay Reynolds Freeman 1988, 1989, 1990, 1991, 2006

Auxiliary code for Scheme built-in functions and special forms, for Pixie.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import "OpCodes.h"
#import "SenseLightCommands.h"

#import "DataTypes.e"
#import "Globals.e"
#import "Intern.e"
#import "IO.e"
#import "SchemeFunctions.e"
#import "SchemeGlobals.e"
#import "StringLoader.e"

/**** forward declarations ****/

void copyList(), forEachCodeAuxAgain(), lastPair(), mapCodeAuxAgain(), properListCheck();

/**** "Boolean"s (leave result in B ) that may be "CALL"ed.
       These routines preserve all other registers.         ****/

PROC( alwaysFalse )                      CLEAR_B                                 CONTINUE END_PROC
PROC( alwaysTrue )                       SET_B                                   CONTINUE END_PROC
PROC( booleanP )                         BOOLEAN_P                               CONTINUE END_PROC
PROC( builtInFunctionP )                 BUILT_IN_FUNCTION_P                     CONTINUE END_PROC
PROC( byteBlockP )                       BYTE_BLOCK_P                            CONTINUE END_PROC
PROC( characterP )                       CHARACTER_P                             CONTINUE END_PROC
PROC( charAlphaP )                       CHAR_ALPHA_P                            CONTINUE END_PROC
PROC( charLowerP )                       CHAR_LOWER_P                            CONTINUE END_PROC
PROC( charNumericP )                     CHAR_NUMERIC_P                          CONTINUE END_PROC
PROC( charUpperP )                       CHAR_UPPER_P                            CONTINUE END_PROC
PROC( charWhitespaceP )                  CHAR_WHITESPACE_P                       CONTINUE END_PROC
PROC( complexP )                         COMPLEX_P                               CONTINUE END_PROC
PROC( emptyListP )                       EMPTY_LIST_P                            CONTINUE END_PROC
PROC( environmentReferenceP )            ENVIRONMENT_REFERENCE_P                 CONTINUE END_PROC
PROC( evenP )                            EVEN_P                                  CONTINUE END_PROC
PROC( exactP )                           EXACT_P                                 CONTINUE END_PROC
PROC( exactIntegerP )                    EXACT_INTEGER_P                         CONTINUE END_PROC
PROC( exactShortFloatP )                 EXACT_SHORT_FLOAT_P                     CONTINUE END_PROC
PROC( exactShortFixnumP )                EXACT_SHORT_FIXNUM_P                    CONTINUE END_PROC
PROC( failP )                            FAIL_P                                  CONTINUE END_PROC
PROC( falseP )                           FALSE_P                                 CONTINUE END_PROC
PROC( fixnumP )                          FIXNUM_P                                CONTINUE END_PROC
PROC( floatP )                           FLOAT_P                                 CONTINUE END_PROC
PROC( forgettableP )                     FORGETTABLE_P                           CONTINUE END_PROC
PROC( forgettableForgottenP )            FORGETTABLE_FORGOTTEN_P                 CONTINUE END_PROC
PROC( forgettableRememberedP )           FORGETTABLE_REMEMBERED_P                CONTINUE END_PROC
PROC( forgettableUnexpiredP )            FORGETTABLE_UNEXPIRED_P                 CONTINUE END_PROC
PROC( forwardingPointerP )               FORWARDING_POINTER_P                    CONTINUE END_PROC
PROC( ggcForwardingPointerP )            GGC_FORWARDING_POINTER_P                CONTINUE END_PROC
PROC( inexactShortFloatP )               INEXACT_SHORT_FLOAT_P                   CONTINUE END_PROC
PROC( inexactShortFixnumP )              INEXACT_SHORT_FIXNUM_P                  CONTINUE END_PROC
PROC( inexactP )                         INEXACT_P                               CONTINUE END_PROC
PROC( infP )                             INF_P                                   CONTINUE END_PROC
PROC( inputPortP )                       INPUT_PORT_P                            CONTINUE END_PROC
PROC( integerP )                         INTEGER_P                               CONTINUE END_PROC
PROC( interruptRangeCheckP )             INTERRUPT_RANGE_CHECK_P                 CONTINUE END_PROC
PROC( kittenRangeCheckP )                KITTEN_RANGE_CHECK_P                    CONTINUE END_PROC
PROC( lambdaPairP )                      LAMBDA_PAIR_P                           CONTINUE END_PROC
PROC( leafP )                            LEAF_P                                  CONTINUE END_PROC
PROC( letterP )                          LETTER_P                                CONTINUE END_PROC
PROC( levelIndicatorNumberRangeCheckP )  LEVEL_INDICATOR_NUMBER_RANGE_CHECK_P    CONTINUE END_PROC
PROC( levelIndicatorValueRangeCheckP )   LEVEL_INDICATOR_VALUE_RANGE_CHECK_P     CONTINUE END_PROC
PROC( listP )                            LIST_P                                  CONTINUE END_PROC
PROC( longP )                            LONG_P                                  CONTINUE END_PROC
PROC( macroP )                           MACRO_P                                 CONTINUE END_PROC
PROC( mmappedBlockP )                    MMAPPED_BLOCK_P                         CONTINUE END_PROC
PROC( multipleValuesP )                  MULTIPLE_VALUES_P                       CONTINUE END_PROC
PROC( nanP )                             NAN_P                                   CONTINUE END_PROC
PROC( nanOrInfP )                        NAN_OR_INF_P                            CONTINUE END_PROC
PROC( nonLeafP )                         NON_LEAF_P                              CONTINUE END_PROC
PROC( nonNegativeIntegerP )              NON_NEGATIVE_INTEGER_P                  CONTINUE END_PROC
PROC( numberP )                          NUMBER_P                                CONTINUE END_PROC
PROC( oddP )                             ODD_P                                   CONTINUE END_PROC
PROC( outputPortP )                      OUTPUT_PORT_P                           CONTINUE END_PROC
PROC( pairP )                            PAIR_P                                  CONTINUE END_PROC
PROC( peculiarLeafP )                    PECULIAR_LEAF_P                         CONTINUE END_PROC
PROC( peculiarNonLeafP )                 PECULIAR_NON_LEAF_P                     CONTINUE END_PROC
PROC( pointerToByteBlockP )              POINTER_TO_BYTE_BLOCK_P                 CONTINUE END_PROC
PROC( portP )                            PORT_P                                  CONTINUE END_PROC
PROC( positiveIntegerP )                 POSITIVE_INTEGER_P                      CONTINUE END_PROC
PROC( procedureP )                       PROCEDURE_P                             CONTINUE END_PROC
PROC( promiseP )                         PROMISE_P                               CONTINUE END_PROC
PROC( properListP )                      PROPER_LIST_P                           CONTINUE END_PROC
PROC( pushbuttonNumberRangeCheckP )      PUSHBUTTON_NUMBER_RANGE_CHECK_P         CONTINUE END_PROC
PROC( rationalP )                        RATIONAL_P                              CONTINUE END_PROC
PROC( radixP )                           RADIX_P                                 CONTINUE END_PROC
PROC( rangeCheckAsSignedFixnumP )        RANGE_CHECK_SIGNED_FIXNUM               CONTINUE END_PROC
PROC( rangeCheckAsUnsignedFixnumP )      RANGE_CHECK_UNSIGNED_FIXNUM             CONTINUE END_PROC
PROC( realP )                            REAL_P                                  CONTINUE END_PROC
PROC( schemeMachinePCP )                 SCHEME_MACHINE_PC_P                     CONTINUE END_PROC
PROC( schemeMachinePCIndexP )            SCHEME_MACHINE_PC_INDEX_P               CONTINUE END_PROC
PROC( shortFixnumP )                     SHORT_FIXNUM_P                          CONTINUE END_PROC
PROC( sliderNumberRangeCheckP )          SLIDER_NUMBER_RANGE_CHECK_P             CONTINUE END_PROC
PROC( specialFormP )                     SPECIAL_FORM_P                          CONTINUE END_PROC
PROC( stackReferenceP )                  STACK_REFERENCE_P                       CONTINUE END_PROC
PROC( stringP )                          STRING_P                                CONTINUE END_PROC
PROC( symbolP )                          SYMBOL_P                                CONTINUE END_PROC
PROC( succeedP )                         SUCCEED_P                               CONTINUE END_PROC
PROC( syntaxBindingP )                   SYNTAX_BINDING_P                        CONTINUE END_PROC
PROC( topLevelEnvironmentReferenceP )    TOP_LEVEL_ENVIRONMENT_REFERENCE_P       CONTINUE END_PROC
PROC( trueP )                            TRUE_P                                  CONTINUE END_PROC
PROC( vectorP )                          VECTOR_P                                CONTINUE END_PROC

/**** probabilityP -- want a number in range [ 0, 1 ] ****/

PROC( probabilityP )

    NUMBER_P
    JUMP_FALSE( returnFalse )
    PUSH
    MOVE( one, R )
    NUM_LE_P
    JUMP_FALSE( popReturnFalse )
    MOVE( zero, R )
    NUM_GE_P
    JUMP_FALSE( popReturnFalse )
    POP
    CONTINUE;      // B was set by the last predicate.

  popReturnFalse:
    POP
  returnFalse:   // B has already been cleared by the predicate that prompted jumping to here.
    CONTINUE

END_PROC    

/******** Miscellaneous functions *******/

/**** carEqP -- Is the car of the list in R eq?
                to the object on the stack?  (Preserves both.) ****/

PROC( carEqP )      /* list in R, object on stack */

    PUSH
    CAR          
    PUSH
    MOVE( STACK( 2 ), R )
    EQ_P
    DROP
    POP
    CONTINUE

END_PROC

/**** carEqualP -- Is the car of the list in R equal?
                   to the object on the stack?  (Preserves both.) ****/

PROC( carEqualP )   /* list in R, object on stack */

    PUSH
    CAR          
    PUSH
    MOVE( STACK( 2 ), R )
    EQUAL_P
    DROP
    POP
    CONTINUE

END_PROC

/**** carEqvP -- Is the car of the list in R eqv? to the
             object on the stack?  (Preserves both.) ****/

PROC( carEqvP )     /* list in R, object on stack */

    PUSH
    CAR          
    PUSH
    MOVE( STACK( 2 ), R )
    EQV_P
    DROP
    POP
    CONTINUE

END_PROC

/**** copyListWithLength -- return a newly-allocated copy of the list that is
                            passed in S[0], whose length is provided in R.  ****/

PROC( copyListWithLength )

    taggedAval *nextCell = 0;
    UnsignedLongWord listLength = (UnsignedLongWord)(R.u.c);
    UnsignedLongWord counter = 0;
    volatile Tag nextTag = 0;

    POP
    PAIR_P
    JUMP_FALSE( exitCopyList )      
    PUSH
    LENGTH
    if( listLength == 0 ) {
        DROP
        R.t = emptyListTag;
        CONTINUE
        }
    PUSH
    MOVE( taggedAvalLength, R )
    MULTIPLY
    theHeap->allocateR();

    // First, convert the new block to a cdr-coded list before anything
    // has a chance to garbage-collect.  A pointer to the list is in R.

    nextCell = R.u.a;
    for( counter = 0; counter < (listLength - 1); ++counter ) {
        nextCell->t = falseBooleanTag | carCdrNext;
        ++nextCell;
        }
    nextCell->t = falseBooleanTag | carCdrNil;

    R.t = pairTag;
    theClowder->addToLiveAddressStack( R.u.a );

    // Next we cdr down the given list, loading the content
    // of each item in turn into the new list.  In so doing,
    // we overwrite the cdr-coding bits in the new list with
    // the same value they already had, but they have to be
    // there so that GC will understand the structure of the
    // new list if it occurs during the opportunity given by 
    // CAR or CDR.

    SWAP

    for( counter = 0; counter < listLength; ++counter ) {
        PUSH
        CAR
        nextCell = (S[1]).u.a + counter;
        nextTag = (R.t & ~(administrationBitsMask));
        if( counter == (listLength - 1) )
            nextTag |= carCdrNil;
        else
            nextTag |= carCdrNext;
        (nextCell->u).c = R.u.c;
        nextCell->t = nextTag;
        POP
        CDR
        }        

  exitCopyList:

    POP
    CONTINUE

END_PROC

/**** copyList -- Return a newly-allocated copy of the list that is passed in R.
                  Note that the list returned is cdr-coded. ****/

PROC( copyList )

    PUSH
    LENGTH
    CALL( copyListWithLength )
    CONTINUE

END_PROC

/**** debugCode -- Seek out the debugger and run it. ****/

PROC( debugCode )

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( NameStrings[ 228 ], R )    /* the name whereby the user knows the debugger */
    CALL( intern )
    BOUND_SYMBOL_P
    JUMP_FALSE( debugCodeExit )
    PUSH
    EVAL_SYMBOL                             /* the value associated with that name */
    PROCEDURE_P
    JUMP_FALSE( noDebugger )

    /* insert here code to fetch the state of the world into R */ /* NEEDS WORK */

    MOVE( E, R )                            /* Gulp! */     

    NILCONS
    PUSH
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( NameStrings[ 281 ], R )    /* the name of "c::quote-simple" */
    CALL( intern )
    SWAP
    CONS
    NILCONS
    CONS
    PUSH_CONTINUATION                       /* C: ((<name-of-debugger> (quote <state-of-world>)) ...) */
#ifdef USE_COCOA_IO
    executeBoundSymbol( kittenizeString( loadNameString( 333 ), theKittenNumber, TRUE ) );
#endif
    DEBUG_ENTRY

  noDebugger:
    DROP
  debugCodeExit:
    CONTINUE

END_PROC

/**** displayLambdaNameOnDefaultPort -- Display the name of the lambda expression
                                        that is passed in R.  Returns the expression.     ****/

PROC( displayLambdaNameOnDefaultPort )

    PUSH
    LOAD_R_TAG( pairTag )
    CDR
    CDR
    CDR
    DISPLAY
    POP
    CONTINUE

END_PROC

/**** displayLambdaSourceOnDefaultPort -- Display the source of the lambda expression
                                          that is passed in R.  Returns the expression.   ****/

PROC( displayLambdaSourceOnDefaultPort )

    PUSH
    LOAD_R_TAG( pairTag )
    PUSH
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( NameStrings[ 118 ], R )
    CALL( intern )
    SWAP
    PUSH                                    /* S: ( (lambdaList . (lambdaBody env . name)) "lambda" ...) */
    CAR          
    SWAP               /* S: ( lambdaList "lambda" ...) */
    CDR          
    PAIR_P
    JUMP_FALSE( needInterpretedLambda )
    CAR          
    CONS
    CONS
    DISPLAY
    POP
    CONTINUE

  needInterpretedLambda:
    resetError( 52 );

END_PROC

/**** displayMacroSourceOnDefaultPort -- Display the source of the macro passed in R.  Returns #t.   ****/

PROC( displayMacroSourceOnDefaultPort )

    LOAD_R_TAG( lambdaPairTag )
    JUMP_FUNCTION( displayLambdaSourceOnDefaultPort )
    CONTINUE
        
END_PROC

/**** lambdaBodyCheck -- Auxiliary for checking the syntax of a lambda body. ****/

PROC( lambdaBodyCheck )

    CALL( properListCheck )
    CONTINUE

END_PROC
        
/**** lambdaListCheck -- Syntax checker for lambda list in R; trap or leave R unchanged. ****/

PROC( lambdaListCheck )

    PUSH                                                    /* R: lambdaList; S:(lambdaList ...) */
        
lambdaListLoop:
    EMPTY_LIST_P
    JUMP_TRUE( lambdaListChecked )
    PAIR_P
    JUMP_FALSE( lastItemIsDotted )
    PUSH
    CAR          
    SYMBOL_P
    JUMP_TRUE( lambdaItemIsSymbol )
    POP
    POP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 53 );

lambdaItemIsSymbol:
    CALL( permanentSymbolP )
    JUMP_TRUE( permanentSymbol )
    POP
    CDR          
    JUMP_LABEL( lambdaListLoop )
        
lastItemIsDotted:
    SYMBOL_P
    JUMP_TRUE( lambdaListChecked )
    POP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 53 );

permanentSymbol:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 227 );

lambdaListChecked:
    POP

    CONTINUE

END_PROC

/**** lastPair -- Return the last pair of the list whose first pair is provided in R.  ****/

PROC( lastPair )

  lastPairLoop:
    PUSH
    CDR          
    PAIR_P
    JUMP_FALSE( foundLastPair )
    DROP
    JUMP_LABEL( lastPairLoop )

  foundLastPair:
    POP
    CONTINUE

END_PROC

/**** lastPairDangerous -- Return the last pair of the list whose first pair is provided in R, using
                           a form of "cdr" that disregards blocks.  (Introduced for working with "Q".)  ****/

PROC( lastPairDangerous )

  lastPairLoop:
    PUSH
    cdrRegardlessOfBlocking();          
    PAIR_P
    JUMP_FALSE( foundLastPair )
    DROP
    JUMP_LABEL( lastPairLoop )

  foundLastPair:
    POP
    CONTINUE

END_PROC

/**** listTail -- Find the (zero-based) nth cdr of a list.      ****/

PROC( listTail )

    EXACT_P
    JUMP_FALSE( needExact )
    COERCE_R_SHORT_FIXNUM
    PUSH
    MOVE( zero, R )
    NUM_LT_P
    JUMP_TRUE( needNonNegative )
    POP

  oneMoreCdr:
    ZERO_FIXNUM_P
    JUMP_TRUE( done )
    PUSH
    MOVE( one, R )
    SUBTRACT
    SWAP
    PAIR_P
    JUMP_FALSE( ranOffTheEnd )
    CDR          
    SWAP
    JUMP_LABEL( oneMoreCdr )

  done:
    POP
    CONTINUE

  needNonNegative:
    POP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 74 );
    CONTINUE

  needExact:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 75 );
    CONTINUE

  ranOffTheEnd:
    resetError( 76 );
    CONTINUE

END_PROC        

/**** makeExactShortFixnum - Unboxed, nothing to allocate. ****/

PROC( makeExactShortFixnum )

    LOAD_R_TAG( exactShortFixnumBitsMask )
    CONTINUE
        
END_PROC

/**** makeVectorAndStuff -- Make a new vector whose length is provided in R, and
                            fill it from the list that is on top of the stack.     ****/

PROC( makeVectorAndStuff )              // length in R, list of contents in STACK( 0 )
 
    UnsignedLongWord vLength = R.u.c;

    SWAP
    CONS                                // R: length . contents
    PUSH

    R.t = exactShortFixnumBitsMask;
    R.u.c = vLength + 1;

    CALL( copyListWithLength );

    R.t = vectorTag;
    theClowder->addToLiveAddressStack( R.u.a );

    CONTINUE

END_PROC

/**** makeVectorAndStuffFromStack -- Make a new vector whose length is provided in R, and
                                     fill it from the items that are individually stacked.     ****/

PROC( makeVectorAndStuffFromStack )             // length in R, contents stacked (item n in STACK( vLength - n - 1))

    UnsignedLongWord vLength = R.u.c;
    UnsignedLongWord counter = 0;
    taggedAval *nextVectorSlot = 0;
    Tag nextTag = 0;

    theHeap->allocateVectorR();                 // R: vectorAddress; S: element-vLength-1 ... element-0
    theClowder->addToLiveAddressStack( R.u.a );

    nextVectorSlot = R.u.a + vLength;           // Counting backwards from the end.
    for( counter = 0; counter < vLength; ++counter ) {
        nextTag = pure_type_code( (S[0]).t );
        nextTag |= ((S[0]).t & kittenByteBitsMask);        
        if( counter == 0 )
            nextTag |= carCdrNil;
        else
            nextTag |= carCdrNext;
        nextVectorSlot->t =  nextTag;
        (nextVectorSlot->u).c = (S[0]).u.c;
        DROP
        --nextVectorSlot;
        }

    CONTINUE
        
END_PROC

/**** makeVectorOfSharpF -- Make a vector (length is provided in R), and fill it with "()"s. ****/

PROC( makeVectorOfSharpF )              /* length in R */

    PUSH
    MOVE( zero, R )
    NUM_LT_P
    JUMP_TRUE( rangeError )
    POP
    theHeap->allocateVectorR();
    theClowder->addToLiveAddressStack( R.u.a );
    CONTINUE

  rangeError:
    POP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 58 );
    CONTINUE
        
END_PROC

/**** noop -- Do nothing:  Used mostly as a place where continuations join together. ****/

PROC( noop )

    CONTINUE

END_PROC

/**** properListCheck -- Auxiliary for checking that a list is non-null and proper. ****/

PROC( properListCheck )

    PUSH                                                    // R: list; S:(list ...)
    PROPER_LIST_P
    POP

    CONTINUE

END_PROC
        
/**** reverseListCode -- Return a newly-allocated reversed copy of the list provided in R.  ****/

PROC( reverseListCode )
 
    UnsignedLongWord listLength = 0;
    Tag nextTag = 0;
    taggedAval *nextLocation = 0;
    UnsignedLongWord counter = 0;

    PAIR_P
    JUMP_FALSE( exitReverseList )   
    LIST_P
    JUMP_FALSE( reverseListError )

    PUSH
    LENGTH
    listLength = R.u.c;

    if( listLength == 0 ) {
        DROP
        R.t = emptyListTag;
        CONTINUE
        }

    R.u.c = R.u.c * sizeof( taggedAval );
    theHeap->allocateR();

    nextLocation = R.u.a;
    for( counter = 0; counter < listLength - 1; ++counter )
        (nextLocation++)->t = falseBooleanTag | carCdrNext;
    nextLocation->t = falseBooleanTag | carCdrNil;

    R.t = pairTag;
    theClowder->addToLiveAddressStack( R.u.a );

    SWAP      // R: OldList; S: NewList ...

    for( counter = 0; counter < listLength; ++counter ) {
        PUSH
        CAR
        nextLocation = (S[1]).u.a + (listLength - 1 - counter);
        nextTag = ((R.t) & ~(administrationBitsMask));
        if( counter == 0 )
            nextTag |= carCdrNil;
        else
            nextTag |= carCdrNext;
        (nextLocation->u).c = R.u.c;
        nextLocation->t = nextTag;
        POP
        CDR
        }

    POP
        
  exitReverseList:
    CONTINUE

  reverseListError:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 59 );
    CONTINUE    

END_PROC

/**** sharpFOrProperListP ****/

PROC( sharpFOrProperListP )

    JUMP_FALSE( returnTrue )

    PUSH
    PROPER_LIST_P
    POP
    CONTINUE

  returnTrue:
    SET_B
    CONTINUE
   
END_PROC

/**** stringFillAux -- Auxiliary for filling a given string with a particular character. ****/

PROC( stringFillAux )           /* R: char; S: (string ...) */

    if( ! blockOnTagLock1( &(S[0]), 14 ) )
        resetError( 210 );
    PUSH
    MOVE( STACK( 1 ), R )
    STRING_LENGTH               /* R: position; S: (char string ...) */

  doneYet:
    ZERO_FIXNUM_P
    JUMP_TRUE( done )
    PUSH
    MOVE( one, R )
    SUBTRACT                    /* R: newPosition; S: (char string ...) */
    SWAP   /* R: char;        S: (newPosition string ...) */
    STRING_SET
    SWAP   /* R: newPosition; S: (char string ...) */
    JUMP_LABEL( doneYet )

  done:
    POP
    releaseTagLock1( (volatile taggedAval *)((S[0]).u.a), 30 );
    CONTINUE

END_PROC

/**** S0toC -- Move from STACK( 0 ) to C, and drop. ****/

PROC( S0toC )

//    MOVE( STACK( 0 ), C )
    CPushFrame( STACK( 0 ) );
    DROP
    CONTINUE
        
END_PROC

/**** vectorFillAux  -- Auxiliary for filling a given vector with a particular object.****/

PROC( vectorFillAux )          /* R: thing; S: (vector ...) */

    UnsignedLongWord index = 0;
    UnsignedLongWord lnth = 0;
    taggedAval *location = 0;
    Tag nextTag = 0;

    PUSH
    MOVE( STACK( 1 ), R )
    VLENGTH                     /* R: position; S: (thing vector ...) */
    lnth = (UnsignedLongWord)(R.u.c);
    if( ! blockOnTagLock1( &(S[1]), 15 ) )   // Can't block sooner, VLENGTH would deadlock ...
        resetError( 211 );

    location = S[1].u.a + 1;
    for( index = 1; index <= lnth; ++index ) {
        nextTag = pure_type_code( (S[0]).t );
        if( index == lnth )
            nextTag |= carCdrNil;
        else
            nextTag |= carCdrNext;
        location->t = nextTag;
        (location->u).c = (S[0]).u.c;
        ++location;
        }
    clearCdrCodingBitsNonAtomic( (S[0]).t );

    POP
    releaseTagLock1( (volatile taggedAval *)((S[0]).u.a), 31 );
    CONTINUE

END_PROC

/**** senseLightP -- is the content of R an integer in the range [ 0, N_SENSE_LIGHTS )? ****/

PROC( senseLightP )

    PUSH
    EXACT_INTEGER_P
    JUMP_FALSE( return_false )
    if( (R.u.c >= 0) && (R.u.c < N_SENSE_LIGHTS ) ) {
        SET_B
        POP
        CONTINUE
        }

  return_false:
    POP
    CLEAR_B
    CONTINUE

END_PROC

/**** senseLightIlluminationP -- is the content of R an integer in the range [ 0, N_SENSE_LIGHTS )? ****/

PROC( senseLightIlluminationP )

    PUSH
    EXACT_INTEGER_P
    JUMP_FALSE( return_false )
    if( (R.u.c >= 0) && (R.u.c < N_SENSE_LIGHT_ILLUMINATIONS ) ) {
        SET_B
        POP
        CONTINUE
        }

  return_false:
    POP
    CLEAR_B
    CONTINUE

END_PROC
