/****************************************************************************************

SchemeFunctionStatics.c -- Copyright (c) by Jay Reynolds Freeman 1988, 1989, 1990, 1991, 2006

File for the Scheme built-in functions' special statics used in Wraith.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import "WraithHeader.h"
#import "OpCodes.h"
#import "Globals.e"
#import "SchemeAux.e"

/******** non-specific statics ********/

void (*anyAnyChecker[2])() = { alwaysTrue, alwaysTrue };
void (*anyChecker[1])() = { alwaysTrue };
void (*anyIntegerProbabilityChecker[3])() = { alwaysTrue, integerP, probabilityP };
void (*anyListChecker[2])() = { alwaysTrue, listP };
void (*anyNumberChecker[2])() = { alwaysTrue , numberP };
void (*anyProperListChecker[2])() = { alwaysTrue, properListP };
void (*anyTypeBinaryChecker[2])() = { alwaysTrue, alwaysTrue };
void (*charBinaryChecker[2])() = { characterP, characterP };
void (*charChecker[1])() = { characterP };
void (*eRefChecker[1])() = { environmentReferenceP };
void (*exactIntegerChecker[1])() = { exactIntegerP };
void (*forgettableChecker[1])() = { forgettableP };
void (*forgettableIntegerChecker[2])() = { forgettableP, integerP };
void (*forgettableProbabilityChecker[2])() = { forgettableP, probabilityP };
void (*inputPortChecker[1])() = { inputPortP };
void (*integerChecker[1])() = { integerP };
void (*integerNonNegativeIntegerChecker[2])() = { integerP, nonNegativeIntegerP };
void (*integerStringChecker[2])() = { integerP, stringP };
void (*interruptChecker[1])() = { interruptRangeCheckP };
void (*interruptKittenAnyChecker[3])() = { interruptRangeCheckP, kittenRangeCheckP, alwaysTrue };
void (*kittenAnyChecker[2])() = { kittenRangeCheckP, alwaysTrue };
void (*kittenChecker[1])() = { kittenRangeCheckP };
void (*kittenInterruptChecker[2])() = { kittenRangeCheckP, interruptRangeCheckP };
void (*lambdaPairChecker[1])() = { lambdaPairP };
void (*letterChecker[1])() = { letterP };
void (*levelIndicatorNumberValueChecker[2])() = { levelIndicatorNumberRangeCheckP, levelIndicatorValueRangeCheckP };
void (*listChecker[1])() = { listP };
void (*listIntegerChecker[2])() = { listP, integerP };
void (*macroChecker[1])() = { macroP };
void (*mmapBlockChecker[1])() = { mmappedBlockP };
void (*mmapBlockUnsigned[2])() = { mmappedBlockP, nonNegativeIntegerP };
void (*mmapBlockUnsignedInteger[3])() = { mmappedBlockP, nonNegativeIntegerP, integerP };
void (*mmapBlockUnsignedNumber[3])() = { mmappedBlockP, nonNegativeIntegerP, numberP };
void (*mmapBlockUnsignedString[3])() = { mmappedBlockP, nonNegativeIntegerP, stringP };
void (*mmapBlockUnsignedVector[3])() = { mmappedBlockP, nonNegativeIntegerP, vectorP };
void (*nonLeafChecker[1])() = { nonLeafP };
void (*nonNegativeIntegerChecker[1])() = { nonNegativeIntegerP };
void (*numberBinaryChecker[2])() = { numberP, numberP };
void (*numberChecker[1])() = { numberP };
void (*numberPairChecker[2])() = { numberP, pairP };
void (*outputPortChecker[1])() = { outputPortP };
void (*pairAnyChecker[2])() = { pairP, alwaysTrue };
void (*pairChecker[1])() = { pairP };
void (*positiveIntegerStringChecker[2])() = { positiveIntegerP, stringP };
void (*probabilityChecker[1])() = { probabilityP };
void (*procedureChecker[1])() = { procedureP };
void (*promiseChecker[1])() = { promiseP };
void (*properListChecker[1])() = { properListP };
void (*pushbuttonChecker[1])() = { pushbuttonNumberRangeCheckP };
void (*pushbuttonInterruptChecker[2])() = { pushbuttonNumberRangeCheckP, interruptRangeCheckP };
void (*realChecker[1])() = { realP };
void (*senseLightBooleanChecker[2])() = { senseLightP, booleanP };
void (*senseLightIlluminationChecker[1])() = { senseLightIlluminationP };
void (*senseLightSenseLightIlluminationChecker[2])() = { senseLightP, senseLightIlluminationP };
void (*sliderChecker[1])() = { sliderNumberRangeCheckP };
void (*signedFixnumBinaryChecker[2])() = { rangeCheckAsSignedFixnumP, rangeCheckAsSignedFixnumP };
void (*signedFixnumChecker[1])() = { rangeCheckAsSignedFixnumP };
void (*signedFixnumUnsignedFixnumChecker[2])() = { rangeCheckAsSignedFixnumP, rangeCheckAsUnsignedFixnumP };
void (*stringBinaryChecker[2])() = { stringP, stringP };
void (*stringCharChecker[2])() = { stringP, characterP };
void (*stringChecker[1])() = { stringP };
void (*stringIntegerCharChecker[3])() = { stringP, integerP, characterP };
void (*stringIntegerChecker[2])() = { stringP, integerP };
void (*stringIntegerIntegerChecker[3])() = { stringP, integerP, integerP };
void (*stringNonNegativeIntegerCharChecker[3])() = { stringP, nonNegativeIntegerP, characterP };
void (*stringNonNegativeIntegerChecker[2])() = { stringP, nonNegativeIntegerP };
void (*stringNonNegativeIntegerNonNegativeIntegerChecker[3])() = { stringP, nonNegativeIntegerP, nonNegativeIntegerP };
void (*stringSymbolSymbolChecker[3])() = { stringP, symbolP, symbolP };
void (*symbolChecker[1])() = { symbolP };
void (*symbolPairChecker[2])() = { symbolP, pairP };
void (*topLevelERefChecker[1])() = { topLevelEnvironmentReferenceP };
void (*unsignedFixnumBinaryChecker[2])() = { rangeCheckAsUnsignedFixnumP, rangeCheckAsUnsignedFixnumP };
void (*unsignedFixnumChecker[1])() = { rangeCheckAsUnsignedFixnumP };
void (*unsignedFixnumSignedFixnumChecker[2])() = { rangeCheckAsUnsignedFixnumP, rangeCheckAsSignedFixnumP };
void (*vectorAnyChecker[2])() = { vectorP, alwaysTrue };
void (*vectorChecker[1])() = { vectorP };
void (*vectorIntegerAnyChecker[3])() = { vectorP, nonNegativeIntegerP, alwaysTrue };
void (*vectorIntegerChecker[2])() = {  vectorP, nonNegativeIntegerP };

const Text *anyAnyCheckerNames[ 2 ] = {
    ObjectErrorNameStrings[ 0 ],
    ObjectErrorNameStrings[ 0 ]
    };
const Text *anyCheckerNames[ 1 ] = {
    ObjectErrorNameStrings[ 0 ]
    };
const Text *anyIntegerProbabilityCheckerNames[3] = {
    ObjectErrorNameStrings[ 0 ],
    ObjectErrorNameStrings[ 1 ],
    ObjectErrorNameStrings[ 2 ]
    };
const Text *anyListCheckerNames[2] = {
    ObjectErrorNameStrings[ 0 ],
    ObjectErrorNameStrings[ 3 ]
    };
const Text *anyNumberCheckerNames[2] = {
    ObjectErrorNameStrings[ 0 ] , "a number"
    };
const Text *anyProperListCheckerNames[2] = {
    ObjectErrorNameStrings[ 0 ],
    ObjectErrorNameStrings[ 4 ]
    };
const Text *anyTypeBinaryCheckerNames[2] = {
    ObjectErrorNameStrings[ 0 ],
    ObjectErrorNameStrings[ 0 ]
    };
const Text *charBinaryCheckerNames[2] = {
    ObjectErrorNameStrings[ 5 ],
    ObjectErrorNameStrings[ 5 ]
    };
const Text *charCheckerNames[1] = {
    ObjectErrorNameStrings[ 5 ]
    };
const Text *eRefCheckerNames[1] = {
    ObjectErrorNameStrings[ 6 ]
    };
const Text *exactIntegerCheckerNames[1] = {
    ObjectErrorNameStrings[ 7 ]
    };
const Text *forgettableCheckerNames[1] = {
    ObjectErrorNameStrings[ 8 ]
    };
const Text *forgettableIntegerCheckerNames[2] = {
    ObjectErrorNameStrings[ 8 ],
    ObjectErrorNameStrings[ 1 ]
    };
const Text *forgettableProbabilityCheckerNames[2] = {
    ObjectErrorNameStrings[ 8 ],
    ObjectErrorNameStrings[ 2 ]
    };
const Text *inputPortCheckerNames[1] = {
    ObjectErrorNameStrings[ 9 ]
    };
const Text *integerCheckerNames[1] = {
    ObjectErrorNameStrings[ 1 ]
    };
const Text *integerNonNegativeIntegerCheckerNames[2] = {
    ObjectErrorNameStrings[ 1 ],
    ObjectErrorNameStrings[ 10 ]
    };
const Text *integerStringCheckerNames[2] = {
    ObjectErrorNameStrings[ 1 ],
    ObjectErrorNameStrings[ 11 ]
    };
const Text *interruptCheckerNames[1] = {
    ObjectErrorNameStrings[ 12 ]
    };
const Text *interruptKittenAnyCheckerNames[3] = {
    ObjectErrorNameStrings[ 12 ],
    ObjectErrorNameStrings[ 13 ],
    ObjectErrorNameStrings[ 0 ]
    };
const Text *kittenAnyCheckerNames[2] = {
    ObjectErrorNameStrings[ 13 ],
    ObjectErrorNameStrings[ 0 ]
    };
const Text *kittenCheckerNames[1] = {
    ObjectErrorNameStrings[ 13 ]
    };
const Text *kittenInterruptCheckerNames[2] = {
    ObjectErrorNameStrings[ 13 ],
    ObjectErrorNameStrings[ 12 ]
    };
const Text *lambdaPairCheckerNames[1] = {
    ObjectErrorNameStrings[ 14 ]
    };
const Text *letterCheckerNames[1] = {
    ObjectErrorNameStrings[ 15 ]
    };
const Text *levelIndicatorNumberValueCheckerNames[2] = {
    ObjectErrorNameStrings[ 37 ],
    ObjectErrorNameStrings[ 38 ]
    };
const Text *listCheckerNames[1] = {
    ObjectErrorNameStrings[ 3 ]
    };
const Text *listIntegerCheckerNames[2] = {
    ObjectErrorNameStrings[ 3 ],
    ObjectErrorNameStrings[ 1 ]
    };
const Text *macroCheckerNames[1] = {
    ObjectErrorNameStrings[ 16 ]
    };
const Text *mmapBlockCheckerNames[1] = {
    ObjectErrorNameStrings[ 17 ]
    };
const Text *mmapBlockUnsignedNames[2] = {
    ObjectErrorNameStrings[ 17 ],
    ObjectErrorNameStrings[ 10 ]
    };
const Text *mmapBlockUnsignedIntegerNames[3] = {
    ObjectErrorNameStrings[ 17 ],
    ObjectErrorNameStrings[ 10 ],
    ObjectErrorNameStrings[ 1 ]
    };
const Text *mmapBlockUnsignedNumberNames[3] = {
    ObjectErrorNameStrings[ 17 ],
    ObjectErrorNameStrings[ 10 ],
    ObjectErrorNameStrings[ 18 ]
    };
const Text *mmapBlockUnsignedStringNames[3] = {
    ObjectErrorNameStrings[ 17 ],
    ObjectErrorNameStrings[ 10 ],
    ObjectErrorNameStrings[ 11 ]
    };
const Text *mmapBlockUnsignedVectorNames[3] = {
    ObjectErrorNameStrings[ 17 ],
    ObjectErrorNameStrings[ 10 ],
    ObjectErrorNameStrings[ 19 ]
    };
const Text *nonLeafCheckerNames[1] = {
    ObjectErrorNameStrings[ 20 ]
    };
const Text *nonNegativeIntegerCheckerNames[1] = {
    ObjectErrorNameStrings[ 10 ]
    };
const Text *numberBinaryCheckerNames[2] = {
    ObjectErrorNameStrings[ 18 ],
    ObjectErrorNameStrings[ 18 ]
    };
const Text *numberCheckerNames[1] = {
    ObjectErrorNameStrings[ 18 ]
    };
const Text *numberPairCheckerNames[2] = {
    ObjectErrorNameStrings[ 18 ],
    ObjectErrorNameStrings[ 21 ]
    };
const Text *outputPortCheckerNames[1] = {
    ObjectErrorNameStrings[ 22 ]
    };
const Text *pairAnyCheckerNames[2] = {
    ObjectErrorNameStrings[ 21 ],
    ObjectErrorNameStrings[ 0 ]
    };
const Text *pairCheckerNames[1] = {
    ObjectErrorNameStrings[ 21 ]
    };
const Text *positiveIntegerStringCheckerNames[2] = {
    ObjectErrorNameStrings[ 31 ],
    ObjectErrorNameStrings[ 11 ]
    };
const Text *probabilityCheckerNames[1] = {
    ObjectErrorNameStrings[ 2 ]
    };
const Text *procedureCheckerNames[1] = {
    ObjectErrorNameStrings[ 23 ]
    };
const Text *promiseCheckerNames[1] = {
    ObjectErrorNameStrings[ 24 ]
    };
const Text *properListCheckerNames[1] = {
    ObjectErrorNameStrings[ 4 ]
    };
const Text *pushbuttonCheckerNames[1] = {
    ObjectErrorNameStrings[ 36 ]
    };
const Text *pushbuttonInterruptCheckerNames[2] = {
    ObjectErrorNameStrings[ 36 ],
    ObjectErrorNameStrings[ 12 ]
    };
const Text *realCheckerNames[1] = {
    ObjectErrorNameStrings[ 25 ]
    };
const Text *senseLightBooleanCheckerNames[2] = {
    ObjectErrorNameStrings[ 32 ],
    ObjectErrorNameStrings[ 34 ]
    };
const Text *senseLightIlluminationCheckerNames[1] = {
    ObjectErrorNameStrings[ 33 ]
    };
const Text *senseLightSenseLightIlluminationCheckerNames[2] = {
    ObjectErrorNameStrings[ 32 ],
    ObjectErrorNameStrings[ 33 ]
    };
const Text *signedFixnumBinaryCheckerNames[2] = {
    ObjectErrorNameStrings[ 26 ],
    ObjectErrorNameStrings[ 26 ]
    };
const Text *signedFixnumCheckerNames[1] = {
    ObjectErrorNameStrings[ 26 ]
    };
const Text *signedFixnumUnsignedFixnumCheckerNames[2] = {
    ObjectErrorNameStrings[ 26 ],
    ObjectErrorNameStrings[ 27 ]
    };
const Text *sliderCheckerNames[1] = {
    ObjectErrorNameStrings[ 35 ]
    };
const Text *stringBinaryCheckerNames[2] = {
    ObjectErrorNameStrings[ 11 ],
    ObjectErrorNameStrings[ 11 ]
    };
const Text *stringCharCheckerNames[2] = {
    ObjectErrorNameStrings[ 11 ],
    ObjectErrorNameStrings[ 5 ]
    };
const Text *stringCheckerNames[1] = {
    ObjectErrorNameStrings[ 11 ]
    };
const Text *stringIntegerCharCheckerNames[3] = {
    ObjectErrorNameStrings[ 11 ],
    ObjectErrorNameStrings[ 1 ],
    ObjectErrorNameStrings[ 5 ]
    };
const Text *stringIntegerCheckerNames[2] = {
    ObjectErrorNameStrings[ 11 ],
    ObjectErrorNameStrings[ 1 ]
    };
const Text *stringIntegerIntegerCheckerNames[3] = {
    ObjectErrorNameStrings[ 11 ],
    ObjectErrorNameStrings[ 1 ],
    ObjectErrorNameStrings[ 1 ]
    };
const Text *stringNonNegativeIntegerCharCheckerNames[3] = {
    ObjectErrorNameStrings[ 11 ],
    ObjectErrorNameStrings[ 10 ],
    ObjectErrorNameStrings[ 5 ]
    };
const Text *stringNonNegativeIntegerCheckerNames[2] = {
    ObjectErrorNameStrings[ 11 ],
    ObjectErrorNameStrings[ 10 ]
    };
const Text *stringNonNegativeIntegerNonNegativeIntegerCheckerNames[3] = {
    ObjectErrorNameStrings[ 11 ],
    ObjectErrorNameStrings[ 10 ],
    ObjectErrorNameStrings[ 10 ]
    };
const Text *stringSymbolSymbolCheckerNames[3] = {
    ObjectErrorNameStrings[ 11 ],
    ObjectErrorNameStrings[ 28 ],
    ObjectErrorNameStrings[ 28 ]
    };
const Text *symbolCheckerNames[1] = {
    ObjectErrorNameStrings[ 28 ]
    };
const Text *symbolPairCheckerNames[2] = {
    ObjectErrorNameStrings[ 28 ],
    ObjectErrorNameStrings[ 21 ]
    };
const Text *topLevelERefCheckerNames[1] = {
    ObjectErrorNameStrings[ 29 ] };
const Text *unsignedFixnumBinaryCheckerNames[2] = {
    ObjectErrorNameStrings[ 27 ],
    ObjectErrorNameStrings[ 27 ]
    };
const Text *unsignedFixnumCheckerNames[1] = {
    ObjectErrorNameStrings[ 27 ]
    };
const Text *unsignedFixnumSignedFixnumCheckerNames[2] = {
    ObjectErrorNameStrings[ 27 ],
    ObjectErrorNameStrings[ 26 ]
    };
const Text *vectorAnyCheckerNames[2] = {
    ObjectErrorNameStrings[ 19 ],
    ObjectErrorNameStrings[ 0 ]
    };
const Text *vectorCheckerNames[1] = {
    ObjectErrorNameStrings[ 19 ]
    };
const Text *vectorIntegerAnyCheckerNames[3] = {
    ObjectErrorNameStrings[ 19 ],
    ObjectErrorNameStrings[ 10 ],
    ObjectErrorNameStrings[ 0 ]
    };
const Text *vectorIntegerCheckerNames[2] = {
    ObjectErrorNameStrings[ 19 ],
    ObjectErrorNameStrings[ 10 ]
    };
