/****************************************************************************************

SchemeFunctions.c++ -- Copyright (c) by Jay Reynolds Freeman 1988, 1989, 1990, 1991, 2006

Scheme built-in functions for Wraith, and all code that needs to know about the
"functionDescriptor" structures.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import "WraithHeader.h"
#import "MemoryManager.h"
#import "OpCodes.h"
#import "SchemeFunctions.h"
#import "DataTypes.e"
#import "FunctionAndSpecialNames.e"
#import "Globals.e"
#import "Intern.e"
#import "IO.e"
#import "Reader.e"
#import "SchemeAux.e"
#import "SchemeGlobals.e"
#import "SchemeSpecials.e"
#import "SchemeFunctions.e"
#import "SchemeFunctionExterns.e"
#import "SchemeFunctionStatics.h"
//#import "SECDWorldSave.e"
#import "StringLoader.e"

/******** Stuff for Scheme built-in functions; code and associated statics ********/

/**** The Function Lists -- which initialization routines use to construct
      more rapidly accessed structures for actual use at run-time                       ****/

static Text scratchBuffer[256];

static simpleListCell *descriptorArray;

typedef struct {
    void *fPointer;
    UnsignedLongWord n;
    } tcnStruct;

static tcnStruct typeCheckNames[] = {
    { (void *)alwaysTrue, 1 },
    { (void *)listP, 2 },
    { (void *)numberP, 3 },
    { (void *)characterP, 4 },
    { (void *)rangeCheckAsSignedFixnumP, 5 },
    { (void *)rangeCheckAsUnsignedFixnumP, 40 },
    { (void *)inputPortP, 6 },
    { (void *)integerP, 7 },
    { (void *)lambdaPairP, 8 },
    { (void *)letterP, 9 },
    { (void *)macroP, 11 },
    { (void *)outputPortP, 12 },
    { (void *)pairP, 36 },
    { (void *)procedureP, 13 },
    { (void *)promiseP, 14 },
    { (void *)realP, 15 },
    { (void *)stringP, 16 },
    { (void *)symbolP, 17 },
    { (void *)syntaxBindingP, 10 },
    { (void *)vectorP, 18 },
    { (void *)booleanP, 38 },
    { (void *)exactShortFixnumP, 39 },
    { (void *)environmentReferenceP, 41 },
    { (void *)topLevelEnvironmentReferenceP, 42 },
    { (void *)properListP, 43 },
    { (void *)sharpFOrProperListP, 44 },
    { (void *)multipleValuesP, 46 },
    { (void *)0, 0 }
    };

// There are two "TypeOne" lists because there seems to be a bug in
// the compiler, or something, with initializing more than 255 or 256
// elements ...

static simpleListCell functionDescriptorTypeOneListB[] = {
    { 1, &tellKittenFD },
    { 1, &inputQueueFD },
    { 1, &kittenEmptyQueueFD },
    { 1, &purgeKittenQueueFD },
    { 1, &socketCreateAndListenFD },
    { 1, &socketAcceptFD },
    { 1, &socketWriteFD },
    { 1, &openInputSocketFD },
    { 1, &socketFGetsFD },
    { 1, &return90PercentFlagFD },
    { 1, &disableConsoleOutputFD },
    { 1, &enableConsoleOutputFD },
    { 1, &consoleOutputEnabledPFD },
    { 1, &kittenResetFD },
    { 1, &kittenBusyFD },
    { 1, &makeForgettableFD },
    { 1, &forgettablePFD },
    { 1, &forgettableForgottenPFD },
    { 1, &forgettableRememberedPFD },
    { 1, &forgettableUnexpiredPFD },
    { 1, &forgettableObjectFD },
    { 1, &forgettableTimeFD },
    { 1, &forgettableRememberProbabilityFD },
    { 1, &setForgettableTimeFD },
    { 1, &setForgettableRememberProbabilityFD },
    { 1, &leafPFD },
    { 1, &makeMMappedBlockFD },
    { 1, &mmapBlockInCurrentKittenFD },
    { 1, &getMMappableSpaceFD },
    { 1, &peek8FD },
    { 1, &peek8SignExtendFD },
    { 1, &peek8VectorFD },
    { 1, &peek8VectorSignExtendFD },
    { 1, &poke8FD },
    { 1, &peek16FD },
    { 1, &peek16SignExtendFD },
    { 1, &peek16VectorFD },
    { 1, &peek16VectorSignExtendFD },
    { 1, &poke16FD },
    { 1, &peek32FD },
    { 1, &peek32SignExtendFD },
    { 1, &peek32VectorFD },
    { 1, &peek32VectorSignExtendFD },
    { 1, &poke32FD },
    { 1, &peek64FD },
    { 1, &peek64VectorFD },
    { 1, &poke64FD },
    { 1, &peekFlonumFD },
    { 1, &pokeFlonumFD },
    { 1, &peekFlonumVectorFD },
    { 1, &pokeFlonumVectorFD },
    { 1, &peekCStringFD },
    { 1, &pokeCStringFD },
    { 1, &poke8VectorFD },
    { 1, &poke16VectorFD },
    { 1, &poke32VectorFD },
    { 1, &poke64VectorFD },
    { 1, &memoryMappedBlockPFD },
    { 1, &interruptVectorRegisterFD },
    { 1, &setInterruptFD },
    { 1, &clearInterruptFD },
    { 1, &interruptFD },
    { 1, &getInterruptFileInformationFD },
    { 1, &loadFromStringFD },
    { 1, &makeStringWithPromptFD },
    { 1, &peekPokeAtomicFD },
    { 1, &blockAnyBindingFD },
    { 1, &cBooleanToSchemeBooleanFD },
    { 1, &schemeBooleanToCBooleanFD },
    { 1, &setAlarmFD },
    { 1, &alarmPFD },
    { 1, &alarmTimeFD },
    { 1, &clearInterruptRequestedFD },
    { 1, &nonPrintingObjectPFD },
    { 1, &senseLightPFD },
    { 1, &senseLightIlluminationPFD },
    { 1, &setSenseLightNIlluminationFD },
    { 1, &setSenseLightNVisibilityFD },
    { 1, &showSenseLightsFD },
    { 1, &hideSenseLightsFD },
    { 1, &setSenseLightsOffFD },
    { 1, &setSenseLightIlluminationsFD },
    { 1, &rotateSenseLightsFD },
    { 1, &applicativeModeCheckSetBangFD },
    { 1, &applicativeModeCheckFD },
    { 1, &applicativeBitPFD },
    { 1, &setApplicativeBitFD },
    { 1, &setApplicativeBitNDownFD },
    { 1, &clearApplicativeBitFD },
    { 1, &cSetTagFD },
    { 1, &clearApplicativeBitNDownFD },
    { 1, &senseSwitchesFD },
    { 1, &sliderFD },
    { 1, &enablePushbuttonFD },
    { 1, &pushbuttonInterruptFD },
    { 1, &disablePushbuttonFD },
    { 1, &enableBigRedButtonFD },
    { 1, &bigRedButtonInterruptFD },
    { 1, &disableBigRedButtonFD },
    { 1, &pushbuttonEnabledFD },
    { 1, &bigRedButtonEnabledFD },
    { 1, &setLevelIndicatorFD },
    { 0, (void *)NULL }
    };

static simpleListCell functionDescriptorTypeOneListA[] = {
    { 1, &carFD },
    { 1, &cdrFD },
    { 1, &consFD },
    { 1, &lengthFD },
    { 1, &nullPFD },
    { 1, &pairPFD },
    { 1, &zeroPFD },
    { 1, &notNullPFD },
    { 1, &acosFD },
    { 1, &asinFD },
    { 1, &assocFD },
    { 1, &assqFD },
    { 1, &assvFD },
    { 1, &bitAndFD },
    { 1, &bitNotFD },
    { 1, &bitOrFD },
    { 1, &bitXorFD },
    { 1, &bitShiftLeftFD },
    { 1, &bitShiftRightArithmeticFD },
    { 1, &bitShiftRightLogicalFD },
    { 1, &blockFD },
    { 1, &releaseFD },
    { 1, &blockSymbolBindingFD },
    { 1, &booleanPFD },
    { 1, &boundInstancePFD },
    { 1, &boundInstanceInEnvironmentPFD },
    { 1, &cadrFD },
    { 1, &cddrFD },
    { 1, &caarFD },
    { 1, &cdarFD },
    { 1, &caddrFD },
    { 1, &cdddrFD },
    { 1, &caadrFD },
    { 1, &cdadrFD },
    { 1, &cadarFD },
    { 1, &cddarFD },
    { 1, &caaarFD },
    { 1, &cdaarFD },
    { 1, &cadddrFD },
    { 1, &cddddrFD },
    { 1, &caaddrFD },
    { 1, &cdaddrFD },
    { 1, &cadadrFD },
    { 1, &cddadrFD },
    { 1, &caaadrFD },
    { 1, &cdaadrFD },
    { 1, &caddarFD },
    { 1, &cdddarFD },
    { 1, &caadarFD },
    { 1, &cdadarFD },
    { 1, &cadaarFD },
    { 1, &cddaarFD },
    { 1, &caaaarFD },
    { 1, &cdaaarFD },
    { 1, &charAlphaPFD },
    { 1, &charDowncaseFD },
    { 1, &charCIEqualPFD },
    { 1, &charCIGreaterPFD },
    { 1, &charCIGreaterEqualPFD },
    { 1, &charCILessPFD },
    { 1, &charCILessEqualPFD },
    { 1, &charEqualPFD },
    { 1, &charGreaterPFD },
    { 1, &charGreaterEqualPFD },
    { 1, &charLessPFD },
    { 1, &charLessEqualPFD },
    { 1, &charLowerPFD },
    { 1, &charNumberPFD },
    { 1, &charPFD },
    { 1, &charToIntegerFD },
    { 1, &charUpcaseFD },
    { 1, &charUpperPFD },
    { 1, &charWhitespacePFD },
    { 1, &ceilingFD },
    { 1, &clearPermanentFD },
    { 1, &closedPortPFD },
    { 1, &closeInputPortFD },
    { 1, &closeOutputPortFD },
    { 1, &complexPFD },
    { 1, &compileCodeFD },
    { 1, &ccompilingPFD },
    { 1, &cosFD },
    { 1, &currentInputPortFD },
    { 1, &currentOutputPortFD },
    { 1, &cwCFD },
    { 1, &eIEFD },
    { 1, &cwccFD },
    { 1, &dEFD },
    { 1, &dELFD },
    { 1, &dDELFD },
    { 1, &describeMemoryFD },       
    { 1, &describeActiveMemoryFD },       
    { 1, &describeAgingMemoryFD },       
    { 1, &describeLocksFD },       
    { 1, &releaseLocksFD },       
    { 1, &letDeadlockTimeOutFD },
    { 1, &describeRoomFD },
    { 1, &describeStackFD },
    { 1, &environmentFD },
    { 1, &topLevelEnvironmentFD },
    { 1, &nullEnvironmentFD },
    { 1, &schemeReportEnvironmentFD },
    { 1, &environmentNDownFD },
    { 1, &environmentRefFD },
    { 1, &eofPFD },
    { 1, &eqPFD },
    { 1, &equalPFD },
    { 1, &eqvPFD },
    { 1, &errorPortFD },
    { 1, &evenPFD },
    { 1, &exactPFD },
    { 1, &exactToInexactFD },
    { 1, &expandMacroFD },
    { 1, &expFD },
    { 1, &exptFD },
    { 1, &fileExistsPFD },
    { 1, &fixnumPFD },
    { 1, &floatPFD },
    { 1, &floorFD },
    { 1, &forceFD },
    { 1, &forcedPFD },
    { 1, &fullGC2FD },
    { 1, &genSymFD },
    { 1, &getStartupFolderFD },
    { 1, &getTagFD },
    { 1, &getWorkingFolderFD },
    { 1, &getInitFilePathFD },
    { 1, &hashStringFD },
    { 1, &inexactPFD },
    { 1, &inexactToExactFD },
    { 1, &infPFD },
    { 1, &inputPortPFD },
    { 1, &integerPFD },
    { 1, &integerToCharFD },
    { 1, &listRefFD },
    { 1, &listTailFD },
    { 1, &listToStringFD },
    { 1, &listToVectorFD },
    { 1, &logFD },
    { 1, &shortFloatPFD },
    { 1, &macroPFD },
    { 1, &multipleValuesPFD },
    { 1, &memberFD },
    { 1, &memqFD },
    { 1, &memvFD },
    { 1, &moduloFD },
    { 1, &nanPFD },
    { 1, &notPFD },
    { 1, &numberPFD },
    { 1, &numAbsFD },
    { 1, &numNegativePFD },
    { 1, &numPositivePFD },
    { 1, &oddPFD },
    { 1, &outputPortPFD },
    { 1, &permanentPFD },
    { 1, &printDepthListGetFD },
    { 1, &printDepthVectorGetFD },
    { 1, &printLengthListGetFD },
    { 1, &printLengthVectorGetFD },
    { 1, &printDepthListSetFD },
    { 1, &printDepthVectorSetFD },
    { 1, &printLengthListSetFD },
    { 1, &printLengthVectorSetFD },
    { 1, &procedurePFD },
    { 1, &promisePFD },
    { 1, &properListPFD },
    { 1, &quotientFD },
    { 1, &quitFD },
    { 1, &randomFD },
    { 1, &rationalPFD },
    { 1, &realPFD },
    { 1, &remainderFD },
    { 1, &resetFD },
    { 1, &returnRoomFD },
    { 1, &returnActiveRoomFD },
    { 1, &returnAgingRoomFD },
    { 1, &returnStoreSizeFD },
    { 1, &returnActiveStoreSizeFD },
    { 1, &reverseFD },
    { 1, &roundFD },
    { 1, &setBlockedBindingFD },
    { 1, &setCarFD },
    { 1, &setCdrFD },
    { 1, &setPermanentFD },
    { 1, &setTagFD },
    { 1, &shortFloatPFD },
    { 1, &sinFD },
    { 1, &sqrtFD },
    { 1, &sRandomFD },
    { 1, &stackDepthFD },
    { 1, &stringCopyFD },
    { 1, &stringFillFD },
    { 1, &stringCIEqualPFD },
    { 1, &stringCIGreaterPFD },
    { 1, &stringCIGreaterEqualPFD },
    { 1, &stringCILessPFD },
    { 1, &stringCILessEqualPFD },
    { 1, &stringEqualPFD },
    { 1, &stringGreaterPFD },
    { 1, &stringGreaterEqualPFD },
    { 1, &stringLessPFD },
    { 1, &stringLessEqualPFD },
    { 1, &downInFlamesFD },
    { 1, &stringLengthFD },
    { 1, &stringPFD },
    { 1, &stringRefFD },
    { 1, &stringSetFD },
    { 1, &stringSymbolFD },
    { 1, &stringToListFD },
    { 1, &substringFD },
    { 1, &syntaxBindingPFD },
    { 1, &symbolPFD },
    { 1, &symbolStringFD },
    { 1, &symevalFD },
    { 1, &symevalInEnvironmentFD },
    { 1, &systemFD },
    { 1, &tanFD },
    { 1, &tickFD },
    { 1, &topLevelEnvironmentRefFD },
    { 1, &topLevelEnvironmentRefForSymbolFD },
    { 1, &transcriptOffFD },
    { 1, &truncateFD },
    { 1, &uSleepFD },
    { 1, &vectorPFD },
    { 1, &vectorFillFD },
    { 1, &vectorToListFD },
    { 1, &versionGetFD },
    { 1, &vLengthFD },
    { 1, &vRefFD },
    { 1, &vSetFD },
    { 1, &writeContinuationFD },
    { 1, &writeStackFD },
    { 1, &compilerOnFD },
    { 1, &compilerOffFD },
    { 1, &fullPrecisionOnFD },
    { 1, &fullPrecisionOffFD },
    { 1, &compilingPFD },
    { 1, &fullPrecisionPFD },
    { 1, &momcatPFD },
    { 1, &kittenNumberFD },
    { 1, &maxKittenNumberFD },
    { 1, &showKittensFD },
    { 1, &getPIDFD },
    { 0, (void *)NULL }
    };

static simpleListCell functionDescriptorTypeFiveList[] = {
    { 1, &listFD },
    { 1, &addFD },
    { 1, &GCDFD },
    { 1, &LCMFD },
    { 1, &multiplyFD },
    { 1, &numEqualPFD },
    { 1, &numGreaterPFD },
    { 1, &numGreaterEqualPFD },
    { 1, &numLessPFD },
    { 1, &numLessEqualPFD },
    { 1, &numMaxFD },
    { 1, &numMinFD },
    { 1, &stringAppendFD },
    { 1, &vectorFD },
    { 0, (void *)NULL }
    };

static simpleListCell functionDescriptorTypeSixList[] = {
    { 1, &atanFD },
    { 1, &displayFD },
    { 1, &divideFD },
    { 1, &inspectFD },
    { 1, &makeVectorFD },
    { 1, &subtractFD },
    { 1, &writeCharFD },
    { 1, &writeFD },
    { 0, (void *)NULL }
    };

static simpleListCell functionDescriptorTypeSevenList[] = {
    { 1, &applyFD },
    { 1, &charReadyPFD },
    { 1, &examineWorldFD },
    { 1, &forEachFD },
    { 1, &loadFD },
    { 1, &loadWorldFD },
    { 1, &mapFD },
    { 1, &mapNoTypeCheckingFD },
    { 1, &newlineFD },
    { 1, &numberToStringFD },
    { 1, &numberToStringWithCFormatFD },
    { 1, &openInputFileFD },
    { 1, &openOutputFileFD },
    { 1, &peekCharFD },
    { 1, &readFD },
    { 1, &readCharFD },
    { 1, &saveWorldFD },
    { 1, &setWorkingFolderFD },
    { 1, &setInputPortFD },
    { 1, &stringToNumberFD },
    { 1, &transcriptOnFD },
    { 0, (void *)NULL }
    };

static simpleListCell functionDescriptorTypeEightList[] = {
    { 1, &appendFD },
    { 1, &callWithInputPortFD },
    { 1, &callWithOutputPortFD },
    { 1, &withInputFileFD },
    { 1, &withOutputFileFD },
    { 0, (void *)NULL }
    };

static simpleListCell functionDescriptorTypeNineList[] = {
    { 1, &makeStringFD },
    { 0, (void *)NULL }
    };

/**** Forward declarations ****/

const Text *procedureTypeStringFetchFunction( void (*typeChecker)() );

/**** More functions: ****/

/**** simpleListLength ****/

UnsignedLongWord simpleListLength( simpleListCell *slist )
{
    UnsignedLongWord answer = 0;

    while( (slist++)->flag )
            answer++;
    return answer;
    }

/**** describeFunctionDescriptorTypeOne -- Output a useful description of a type-one function descriptor.       ****/

void describeFunctionDescriptorTypeOne( functionDescriptorTypeOne *d )
{       
    UnsignedLongWord i;

    if( d ) {
        tprintf3( &CurrentOutputPort, MiscStrings[ 85 ], d );
        tprintf5( &CurrentOutputPort, MiscStrings[ 126 ], d->pNameNumber, NameStrings[ d->pNameNumber ], d->nArgs );
        tprintf4( &CurrentOutputPort, MiscStrings[ 131 ], d->resultTag, d->code );
        tprintf3( &CurrentOutputPort, MiscStrings[ 272 ], d->resultTypeChecker );
        if( d->nArgs > 0 ) {                                       
            tprintf2( &CurrentOutputPort, MiscStrings[ 79 ] );
            for( i = 0; i < d->nArgs; ++i )
                tprintf3( &CurrentOutputPort, MiscStrings[ 24 ], (d->argTypeChecker)[i] );
            }
        tprintf2( &CurrentOutputPort, MiscStrings[ 115 ] );
        }
    else
        tprintf2( &CurrentOutputPort, MiscStrings[ 100 ] );
    }

/**** describeFunctionDescriptorTypeFive -- Output a useful description of a type-five function descriptor.      ****/

void describeFunctionDescriptorTypeFive( functionDescriptorTypeFive *d )
{       
    if( d ) {
        tprintf3( &CurrentOutputPort, MiscStrings[ 85 ], d );
        tprintf3( &CurrentOutputPort, MiscStrings[ 125 ], d->pNameNumber );
        tprintf5( &CurrentOutputPort, MiscStrings[ 133 ], d->resultTag, d->code, d->minArgs );
        tprintf3( &CurrentOutputPort, MiscStrings[ 272 ], d->resultTypeChecker );
        tprintf3( &CurrentOutputPort, MiscStrings[ 78 ], d->argTypeChecker );
        }
    else
        tprintf2( &CurrentOutputPort, MiscStrings[ 193 ] );
    }

/**** describeFunctionDescriptorTypeSix -- Output a useful description of a type-six function descriptor.      ****/

void describeFunctionDescriptorTypeSix( functionDescriptorTypeSix *d )
{       
    if( d ) {
        tprintf3( &CurrentOutputPort, MiscStrings[ 85 ], d );
        tprintf3( &CurrentOutputPort, MiscStrings[ 125 ], d->pNameNumber );
        tprintf5( &CurrentOutputPort, MiscStrings[ 132 ], d->resultTag, d->code, d->maxArgs );
        tprintf3( &CurrentOutputPort, MiscStrings[ 272 ], d->resultTypeChecker );
        tprintf3( &CurrentOutputPort, MiscStrings[ 11 ], d->firstArgTypeChecker );
        tprintf3( &CurrentOutputPort, MiscStrings[ 77 ], d->restArgTypeChecker );
        }
    else
        tprintf2( &CurrentOutputPort, MiscStrings[ 194 ] );
    }

/**** describeFunctionDescriptorTypeSeven -- Output a useful description of a type-seven function descriptor.      ****/

void describeFunctionDescriptorTypeSeven( functionDescriptorTypeSeven *d )
{       
    if( d ) {
        tprintf3( &CurrentOutputPort, MiscStrings[ 85 ], d );
        tprintf3( &CurrentOutputPort, MiscStrings[ 125 ], d->pNameNumber );
        tprintf6( &CurrentOutputPort, MiscStrings[ 198 ], d->resultTag, d->code, d->minArgs, d->maxArgs );
        tprintf3( &CurrentOutputPort, MiscStrings[ 272 ], d->resultTypeChecker );
        tprintf3( &CurrentOutputPort, MiscStrings[ 11 ], d->firstArgTypeChecker );
        tprintf3( &CurrentOutputPort, MiscStrings[ 77 ], d->restArgTypeChecker );
        }
    else
        tprintf2( &CurrentOutputPort, MiscStrings[ 195 ] );
    }

/**** describeFunctionDescriptorTypeEight -- Output a useful description of a type-eight function descriptor.      ****/

void describeFunctionDescriptorTypeEight( functionDescriptorTypeEight *d )
{       
    if( d ) {
        tprintf3( &CurrentOutputPort, MiscStrings[ 85 ], d );
        tprintf3( &CurrentOutputPort, MiscStrings[ 125 ], d->pNameNumber );
        tprintf6( &CurrentOutputPort, MiscStrings[ 198 ], d->resultTag, d->code, d->minArgs, d->maxArgs );
        tprintf3( &CurrentOutputPort, MiscStrings[ 272 ], d->resultTypeChecker );
        tprintf3( &CurrentOutputPort, MiscStrings[ 11 ], d->firstArgTypeChecker );
        tprintf3( &CurrentOutputPort, MiscStrings[ 77 ], d->restArgTypeChecker );
        }
    else
        tprintf2( &CurrentOutputPort, MiscStrings[ 237 ] );
    }

/**** describeFunctionDescriptorTypeNine -- Output a useful description of a type-nine function descriptor.      ****/

void describeFunctionDescriptorTypeNine( functionDescriptorTypeNine *d )
{       
    if( d ) {
        tprintf3( &CurrentOutputPort, MiscStrings[ 85 ], d );
        tprintf3( &CurrentOutputPort, MiscStrings[ 125 ], d->pNameNumber );
        tprintf5( &CurrentOutputPort, MiscStrings[ 132 ], d->resultTag, d->code, d->maxArgs );
        tprintf3( &CurrentOutputPort, MiscStrings[ 272 ], d->resultTypeChecker );
        tprintf3( &CurrentOutputPort, MiscStrings[ 11 ], d->firstArgTypeChecker );
        tprintf3( &CurrentOutputPort, MiscStrings[ 77 ], d->restArgTypeChecker );
        }
    else
        tprintf2( &CurrentOutputPort, MiscStrings[ 342 ] );
    }

/**** describeFunctionDescriptorTypeOneList -- Describe the whole list of type-one function descriptors. ****/

void describeFunctionDescriptorTypeOneList( simpleList m )
{
    tprintf2( &CurrentOutputPort, MiscStrings[ 65 ] );
    while( m->flag ) {
        describeFunctionDescriptorTypeOne( (functionDescriptorTypeOne *)(m->pointer) );
        ++m;
        }
    }

/**** describeFunctionDescriptorTypeFiveList -- Describe the whole list of type-five function descriptors. ****/

void describeFunctionDescriptorTypeFiveList( simpleList m )
{
    tprintf2( &CurrentOutputPort, MiscStrings[ 65 ] );
    while( m->flag ) {
        describeFunctionDescriptorTypeFive( (functionDescriptorTypeFive *)(m->pointer) );
        ++m;
        }
    }

/**** describeFunctionDescriptorTypeSixList -- Describe the whole list of type-six function descriptors. ****/

void describeFunctionDescriptorTypeSixList( simpleList m )
{
    tprintf2( &CurrentOutputPort, MiscStrings[ 65 ] );
    while( m->flag ) {
        describeFunctionDescriptorTypeSix( (functionDescriptorTypeSix *)(m->pointer) );
        ++m;
        }
    }

/**** describeFunctionDescriptorTypeSevenList -- Describe the whole list of type-seven function descriptors. ****/

void describeFunctionDescriptorTypeSevenList( simpleList m )
{
    tprintf2( &CurrentOutputPort, MiscStrings[ 65 ] );
    while( m->flag ) {
        describeFunctionDescriptorTypeSeven( (functionDescriptorTypeSeven *)(m->pointer) );
        ++m;
        }
    }

/**** describeFunctionDescriptorTypeEightList -- Describe the whole list of type-eight function descriptors. ****/

void describeFunctionDescriptorTypeEightList( simpleList m )
{
    tprintf2( &CurrentOutputPort, MiscStrings[ 65 ] );
    while( m->flag ) {
        describeFunctionDescriptorTypeEight( (functionDescriptorTypeEight *)(m->pointer) );
        ++m;
        }
    }

/**** describeFunctionDescriptorTypeNineList -- Describe the whole list of type-nine function descriptors. ****/

void describeFunctionDescriptorTypeNineList( simpleList m )
{
    tprintf2( &CurrentOutputPort, MiscStrings[ 65 ] );
    while( m->flag ) {
        describeFunctionDescriptorTypeNine( (functionDescriptorTypeNine *)(m->pointer) );
        ++m;
        }
    }

/**** getBuiltInChecker -- Obtain the (only) built-in type-checker for a type-five function descriptor.      ****/

void getBuiltInChecker()
{
    LOAD_R_TAG( vectorTag )
    PUSH
    MOVE( five, R )
    VREF
    CALL( getPCForIndexInR )    
    }

/**** getBuiltInArgName -- Obtain the (only) built-in argument name for a type-five function descriptor.      ****/

void getBuiltInArgName()
{
    LOAD_R_TAG( vectorTag )
    PUSH
    MOVE( six, R )
    VREF
    }

/**** getPCForIndexInR ****/

void getPCForIndexInR()
{
    LOAD_R_TAG( schemeMachinePCTag )
    R.u.c = (theSimpleTable[ (UnsignedLongWord)(R.u.c) ]);
    }

/**** getBuiltInCodeIndirect -- Obtain the code for a function descriptor (of any type). ****/

void getBuiltInCodeIndirect()
{
    LOAD_R_TAG( vectorTag )
    PUSH
    MOVE( zero, R )
    VREF
    CALL( getPCForIndexInR )    
    }

/**** getBuiltInFirstChecker -- Obtain the first built-in type-checker for a type-three,
                                -six, -seven, -eight or -nine function descriptor.                      ****/

void getBuiltInFirstChecker()
{
    LOAD_R_TAG( vectorTag )
    PUSH
    MOVE( five, R )
    VREF
    CALL( getPCForIndexInR )    
    }

/**** getBuiltInFirstArgName69 -- Obtain the first built-in arg name for a 
                                  type-six or -nine function descriptor.                      ****/

void getBuiltInFirstArgName69()
{
    LOAD_R_TAG( vectorTag )
    PUSH
    MOVE( seven, R )
    VREF
    }

/**** getBuiltInFirstArgName78 -- Obtain the first built-in arg name for a 
                                  type-seven or -eight function descriptor.                      ****/

void getBuiltInFirstArgName78()
{
    LOAD_R_TAG( vectorTag )
    PUSH
    MOVE( eight, R )
    VREF
    }

/**** getBuiltInMaxArgs -- Obtain the maximum number of arguments that a particular 
                           type-six, -seven, -eight or -nine function may be called with. ****/

void getBuiltInMaxArgs()
{ 
    LOAD_R_TAG( vectorTag )
    PUSH
    MOVE( three, R )
    VREF
    }       

/**** getBuiltInMinArgs -- Obtain the minimum number of arguments that a 
                           particular type-two, -four or -five function may be called with. ****/

void getBuiltInMinArgs()
{ 
    LOAD_R_TAG( vectorTag )
    PUSH
    MOVE( three, R )
    VREF
    }       

/**** getBuiltInMinArgsSeven -- Obtain the minimum number of arguments that a 
                                particular type-seven or -eight function may be called with. ****/

void getBuiltInMinArgsSeven()
{ 
    LOAD_R_TAG( vectorTag )
    PUSH
    MOVE( seven, R )
    VREF
    }       

/**** getBuiltInMinArgsThree -- Obtain the minimum number of arguments that a 
                                particular type-three function may be called with. ****/

void getBuiltInMinArgsThree()
{ 
    LOAD_R_TAG( vectorTag )
    PUSH
    MOVE( seven, R )
    VREF
    }       

/**** getBuiltInNArgs  -- Obtain the exact number of arguments that a particular type-one function may be called with.****/

void getBuiltInNArgs()
{ 
    LOAD_R_TAG( vectorTag )
    PUSH
    MOVE( three, R )
    VREF
    }       

/**** getBuiltInNthChecker  -- Obtain the built-in type-checker for the Nth
                               argument of a particular type-one function descriptor.      ****/

void getBuiltInNthChecker()             /* (argNoWanted) in R; builtIn on stack */
{
    LOAD_TAG( vectorTag, STACK( 0 ) )
    R.u.c = R.u.c + 5;        /*type-checkers start at offset 5 */
    VREF
    CALL( getPCForIndexInR )    
    }       

/**** getBuiltInNthArgName  -- Obtain the built-in name for the Nth
                               argument of a particular type-one function descriptor.      ****/

void getBuiltInNthArgName()             /* (argNoWanted) in R; builtIn on stack */
{
    UnsignedLongWord nameOffset = R.u.c + 5;

    POP
    PUSH
    PUSH                            // R: builtIn; S: builtIn, builtIn
    MOVE( three, R )                // R: location of nArgs in the built-in; S: built-in, built-in
	LOAD_TAG( vectorTag, STACK( 0 ) )
    VREF                            // R: nArgs; S: built-in
    R.u.c += nameOffset;            // R.u.c is now 5 + nArgs + argNoWanted
	LOAD_TAG( vectorTag, STACK( 0 ) )
    VREF                            // R: argNoWanted-th name string
    }       

/**** getBuiltInPname ****/

PROC( getBuiltInPName )

    LOAD_R_TAG( vectorTag )
    PUSH
    MOVE( one, R )
    VREF
    CONTINUE

END_PROC

/**** getBuiltInRestChecker -- Obtain the second built-in type-checker
                               for a type-three, -six, -seven, -eight or -nine function descriptor.   ****/

void getBuiltInRestChecker()
{
    LOAD_R_TAG( vectorTag )
    PUSH
    MOVE( six, R )
    VREF
    CALL( getPCForIndexInR )    
    }

/**** getBuiltInRestArgName69 -- Obtain the second built-in type-checker
                                 for a type-six or -nine function descriptor.   ****/

void getBuiltInRestArgName69()
{
    LOAD_R_TAG( vectorTag )
    PUSH
    MOVE( eight, R )
    VREF
    }

/**** getBuiltInRestArgName78 -- Obtain the second built-in type-checker
                                 for a type-seven or -eight function descriptor.   ****/

void getBuiltInRestArgName78()
{
    LOAD_R_TAG( vectorTag )
    PUSH
    MOVE( nine, R )
    VREF
    }

/**** getBuiltInResultChecker -- Obtain the built-in type-checker for the
                                 result of a built-in function function.                                 ****/

void getBuiltInResultChecker()
{
    LOAD_R_TAG( vectorTag )
    PUSH
    MOVE( four, R )
    VREF
    CALL( getPCForIndexInR )    
    }

/**** getBuiltInType -- Obtain the type of a function (type one, type two ...). ****/

PROC( getBuiltInType )

    LOAD_R_TAG( vectorTag )
    PUSH
    MOVE( two, R )
    VREF
    CONTINUE

END_PROC

/**** loadSchemeFunctionDescriptorTypeOne -- Load a type-one function descriptor into
                                             main storage, bound to the appropriate symbol. ****/

void loadSchemeFunctionDescriptorTypeOne( functionDescriptorTypeOne *f )
{
    LongWord argIndex, noArgs;
    LongWord vectorLnth = (LongWord)(5 + 2 * f->nArgs);
    taggedAval theVector;
    void (**theTypeCheckers)();
    const Text **theArgDescriptions;

    /* cons up list of things to put into a vector */

    R.u.c = theWorldHandler->indexForPC( (UnsignedLongWord)(f->code) );
    R.t = schemeMachinePCIndexTag;
    PUSH                                                    /* push index of code to execute function */
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( loadNameString( f->pNameNumber ), R )
    CALL( intern )                                          /* interning now avoids storing two copies of this string */
    GET_PNAME 
    PUSH                                                    /* push print name */
    R.u.c = FIXED_ARITY;
    R.t = exactShortFixnumBitsMask;
    PUSH                                                    /* push built-in type */
    R.u.c = (f->nArgs);
    R.t = exactShortFixnumBitsMask;
    PUSH                                                    /* push minimum number of arguments required */
    R.u.c = (theWorldHandler->indexForPC( (UnsignedLongWord)(f->resultTypeChecker) ));
    R.t = schemeMachinePCIndexTag;
    PUSH                                                    /* push index of code to check the result */
    noArgs = f->nArgs;
    theTypeCheckers = f->argTypeChecker;
    for( argIndex = 0; argIndex < noArgs; ++argIndex ) {
        R.u.c = (theWorldHandler->indexForPC( (UnsignedLongWord)(*theTypeCheckers++) ));
        R.t = schemeMachinePCIndexTag;
        PUSH;                                               /* push index of code to type-check argument no "argIndex" */
        }
    theArgDescriptions = f->argDescription;
    for( argIndex = 0; argIndex < noArgs; ++argIndex ) {
        makeString( theArgDescriptions[ argIndex ] );
        PUSH;                                               /* push a string describing an argument */
        }

    R.u.c = vectorLnth;
    R.t = exactShortFixnumBitsMask;
    CALL( makeVectorAndStuffFromStack )
    theVector = R;

    /* intern the print name of the function, and assign the vector to it */

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( loadNameString( f->pNameNumber ), R )
    CALL( intern )
    PUSH
    PUSH
    MOVE( theVector, R )
    LOAD_R_TAG( builtInFunctionTag )
    SWAP
    CALL( bindScheme )
    POP
    JUMP_FUNCTION( makePermanentSymbol )
    }

/**** Conventions about the vector into which the stuff from the functionDescriptor 
      (type one) gets stuffed, documented here for want of any obvious other place:
          
          All slots contain Scheme objects (tagged Avals); not C objects.
          
          Slot #                Contents

           0                    scheme-machine pc index to code to execute the function
           1                    print name
           2                    function type (#define'd fixnum)
           3                    minimum number of args (call it N) the function must receive
           4                    scheme-machine pc index to code that could type-check the result
           5...                 N slots with scheme-machine pc indices to code to type-check required arguments
                                0 through N-1 (arg 0 is leftmost in the call, eg (foo arg0 arg1 ...))
           5 + N ...            strings describing the desired arguments in error messages
           5 + 2N ...           more to come ...
           
           ****/

/**** loadSchemeFunctionDescriptorTypeFive -- Load a type-five function descriptor into
                                              main storage, bound to the appropriate symbol. ****/

void loadSchemeFunctionDescriptorTypeFive( functionDescriptorTypeFive *f )
{
    LongWord vectorLnth = (LongWord)7;
    taggedAval theVector;

    /* cons up list of things to put into a vector */

    R.u.c = (theWorldHandler->indexForPC( (UnsignedLongWord)(f->code) ));
    R.t = schemeMachinePCIndexTag;
    PUSH                                                    /* push index of code to execute function */
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( loadNameString( f->pNameNumber ), R )
    CALL( intern )                                          /* interning now avoids storing two copies of this string */
    GET_PNAME 
    PUSH                                                    /* push print name */
    R.u.c = ANY_ARITY_ONE_TYPE_STACK;
    R.t = exactShortFixnumBitsMask;
    PUSH                                                    /* push built-in type */
    R.u.c = (f->minArgs);
    R.t = exactShortFixnumBitsMask;
    PUSH                                                    /* push minimum number of arguments required */
    R.u.c = (theWorldHandler->indexForPC( (UnsignedLongWord)(f->resultTypeChecker) ));
    R.t = schemeMachinePCIndexTag;
    PUSH                                                    /* push index of code that could check the result */
    R.u.c = (theWorldHandler->indexForPC( (UnsignedLongWord)(f->argTypeChecker) ));
    R.t = schemeMachinePCIndexTag;                          /* index of code to check the arguments */
    PUSH
    makeString( f->argDescription );
    PUSH;                                                   /* push a string describing an argument */

    R.u.c = vectorLnth;
    R.t = exactShortFixnumBitsMask;
    CALL( makeVectorAndStuffFromStack )
    theVector = R;

    /* intern the print name of the function, and assign the vector to it */

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( loadNameString( f->pNameNumber ), R )
    CALL( intern )
    PUSH
    PUSH
    MOVE( theVector, R )
    LOAD_R_TAG( builtInFunctionTag )
    SWAP
    CALL( bindScheme )
    POP
    JUMP_FUNCTION( makePermanentSymbol )
    }

/**** Conventions about the vector into which the stuff from the functionDescriptor 
      (type five) gets stuffed, documented here for want of any obvious other place:
          
          All slots contain Scheme objects (tagged Avals); not C objects.
          
          Slot #                Contents

           0                    scheme-machine pc index to code to execute the function
           1                    print name
           2                    function type (#define'd fixnum)
           3                    minimum number of args the function must receive
           4                    scheme-machine pc index to code that could type-check the result
           5                    scheme-machine pc index to code to type-check arguments
           6                    string describing the desired argument in error messages
           7 ...                more to come ...
           
           ****/

/**** loadSchemeFunctionDescriptorTypeSix -- Load a type-six function descriptor into
                                             main storage, bound to the appropriate symbol. ****/

void loadSchemeFunctionDescriptorTypeSix( functionDescriptorTypeSix *f )
{
    LongWord vectorLnth = (LongWord)9;
    taggedAval theVector;

    /* cons up list of things to put into a vector */

    R.u.c = (theWorldHandler->indexForPC( (UnsignedLongWord)(f->code) ));
    R.t = schemeMachinePCIndexTag;
    PUSH                                                    /* push index of code to execute function */
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( loadNameString( f->pNameNumber ), R )
    CALL( intern )                                          /* interning now avoids storing two copies of this string */
    GET_PNAME 
    PUSH                                                    /* push print name */
    R.u.c = POSITIVE_ARITY_TWO_TYPES_STACK;
    R.t = exactShortFixnumBitsMask;
    PUSH                                                    /* push built-in type */
    R.t = exactShortFixnumBitsMask;
    if( f->maxArgs >= 0 )
        R.u.c = (f->maxArgs);
    else
        R.u.c = (-1);
    PUSH                                                    /* push maximum number of args (or -1 if infinite) */
    R.u.c = (theWorldHandler->indexForPC( (UnsignedLongWord)(f->resultTypeChecker) ));
    R.t = schemeMachinePCIndexTag;
    PUSH                                                    /* push index of code to check the result */
    R.u.c = (theWorldHandler->indexForPC( (UnsignedLongWord)(f->firstArgTypeChecker) ));
    R.t = schemeMachinePCIndexTag;
    PUSH                                                    /* push index of type checker for first argument */
    R.u.c = (theWorldHandler->indexForPC( (UnsignedLongWord)(f->restArgTypeChecker) ));
    R.t = schemeMachinePCIndexTag;
    PUSH                                                    /* push index of type checker for remaining arguments */
    makeString( f->firstArgDescription );
    PUSH;                                                   /* push a string describing an argument */
    makeString( f->restArgDescription );
    PUSH;                                                   /* push a string describing an argument */

    R.u.c = vectorLnth;
    R.t = exactShortFixnumBitsMask;
    CALL( makeVectorAndStuffFromStack )
    theVector = R;

    /* intern the print name of the function, and assign the vector to it */

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( loadNameString( f->pNameNumber ), R )
    CALL( intern )
    PUSH
    PUSH
    MOVE( theVector, R )
    LOAD_R_TAG( builtInFunctionTag )
    SWAP
    CALL( bindScheme )
    POP
    JUMP_FUNCTION( makePermanentSymbol )
    }

/**** Conventions about the vector into which the stuff from the functionDescriptor 
      (type six) gets stuffed, documented here for want of any obvious other place:
          
          All slots contain Scheme objects (tagged Avals); not C objects.
          
          Slot #                Contents

           0                    scheme-machine pc index to code to execute the function
           1                    print name
           2                    function type (#define'd fixnum)
           3                    maximum number of arguments (minimum is one); -1 means infinity
           4                    scheme-machine pc index to code that could type-check the result
           5                    scheme-machine pc index to code to type-check the first argument
           6                    scheme-machine pc index to code to type-check the other arguments
           7                    string describing the first desired argument in error messages
           8                    string describing the other desired arguments in error messages
           9 ...                more to come ...
           
           ****/

/**** loadSchemeFunctionDescriptorTypeSeven -- Load a type-seven function descriptor into
                                               main storage, bound to the appropriate symbol. ****/

void loadSchemeFunctionDescriptorTypeSeven( functionDescriptorTypeSeven *f )
{
    LongWord vectorLnth = (LongWord)10;
    taggedAval theVector;

    /* cons up list of things to put into a vector */

    R.u.c = (theWorldHandler->indexForPC( (UnsignedLongWord)(f->code) ));
    R.t = schemeMachinePCIndexTag;
    PUSH                                                    /* push index of code to execute function */
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( loadNameString( f->pNameNumber ), R )
    CALL( intern )                                          /* interning now avoids storing two copies of this string */
    GET_PNAME 
    PUSH                                                    /* push print name */
    R.u.c = ANY_ARITY_TWO_TYPES_STACK;
    R.t = exactShortFixnumBitsMask;
    PUSH                                                    /* push built-in type */
    R.t = exactShortFixnumBitsMask;
    if( f->maxArgs >= 0 )
        R.u.c = (f->maxArgs);
    else
        R.u.c = (-1);
    PUSH                                                    /* push maximum number of args (or -1 if infinite) */
    R.u.c = (theWorldHandler->indexForPC( (UnsignedLongWord)(f->resultTypeChecker) ));
    R.t = schemeMachinePCIndexTag;
    PUSH                                                    /* push index of code to check the result */
    R.u.c = (theWorldHandler->indexForPC( (UnsignedLongWord)(f->firstArgTypeChecker) ));
    R.t = schemeMachinePCIndexTag;
    PUSH                                                    /* push index of type checker for first argument */
    R.u.c = (theWorldHandler->indexForPC( (UnsignedLongWord)(f->restArgTypeChecker) ));
    R.t = schemeMachinePCIndexTag;
    PUSH                                                    /* push index of type checker for last argument */
    R.u.c = (f->minArgs);
    R.t = exactShortFixnumBitsMask;
    PUSH
    makeString( f->firstArgDescription );
    PUSH;                                                   /* push a string describing an argument */
    makeString( f->restArgDescription );
    PUSH;                                                   /* push a string describing an argument */
    R.u.c = vectorLnth;
    R.t = exactShortFixnumBitsMask;
    CALL( makeVectorAndStuffFromStack )
    theVector = R;

    /* intern the print name of the function, and assign the vector to it */

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( loadNameString( f->pNameNumber ), R )
    CALL( intern )
    PUSH
    PUSH
    MOVE( theVector, R )
    LOAD_R_TAG( builtInFunctionTag )
    SWAP
    CALL( bindScheme )
    POP
    JUMP_FUNCTION( makePermanentSymbol )
    }

/**** Conventions about the vector into which the stuff from the functionDescriptor 
      (type seven) gets stuffed, documented here for want of any obvious other place:
          
          All slots contain Scheme objects (tagged Avals); not C objects.
          
          Slot #                Contents

           0                    scheme-machine pc index to code to execute the function
           1                    print name
           2                    function type (#define'd fixnum)
           3                    maximum number of arguments; -1 means infinity; this number must be positive
           4                    scheme-machine pc index to code that could type-check the result
           5                    scheme-machine pc index to code to type-check the first argument
           6                    scheme-machine pc index to code to type-check the last arguments
           7                    minimum number of arguments
           8                    string describing the first desired argument in error messages
           9                    string describing the other desired arguments in error messages
          10 ...                more to come ...
           
           ****/

/**** loadSchemeFunctionDescriptorTypeEight -- Load a type-eight function descriptor into
                                               main storage, bound to the appropriate symbol. ****/

void loadSchemeFunctionDescriptorTypeEight( functionDescriptorTypeEight *f )
{
    LongWord vectorLnth = (LongWord)10;
    taggedAval theVector;

    /* cons up list of things to put into a vector */

    R.u.c = (theWorldHandler->indexForPC( (UnsignedLongWord)(f->code) ));
    R.t = schemeMachinePCIndexTag;
    PUSH                                                    /* push index of code to execute function */
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( loadNameString( f->pNameNumber ), R )
    CALL( intern )                                          /* interning now avoids storing two copies of this string */
    GET_PNAME 
    PUSH                                                    /* push print name */
    R.u.c = ANY_ARITY_TWO_TYPES_STACK_2;
    R.t = exactShortFixnumBitsMask;
    PUSH                                                    /* push built-in type */
    R.t = exactShortFixnumBitsMask;
    if( f->maxArgs >= 0 )
        R.u.c = (f->maxArgs);
    else
        R.u.c = (-1);
    PUSH                                                    /* push maximum number of args (or -1 if infinite) */
    R.u.c = (theWorldHandler->indexForPC( (UnsignedLongWord)(f->resultTypeChecker) ));
    R.t = schemeMachinePCIndexTag;
    PUSH                                                    /* push index of code to check the result */
    R.u.c = (theWorldHandler->indexForPC( (UnsignedLongWord)(f->firstArgTypeChecker) ));
    R.t = schemeMachinePCIndexTag;
    PUSH                                                    /* push index of type checker for first argument */
    R.u.c = (theWorldHandler->indexForPC( (UnsignedLongWord)(f->restArgTypeChecker) ));
    R.t = schemeMachinePCIndexTag;
    PUSH                                                    /* push index of type checker for last argument */
    R.u.c = (f->minArgs);
    R.t = exactShortFixnumBitsMask;
    PUSH
    makeString( f->firstArgDescription );
    PUSH;                                                   /* push a string describing an argument */
    makeString( f->restArgDescription );
    PUSH;                                                   /* push a string describing an argument */
    R.u.c = vectorLnth;
    R.t = exactShortFixnumBitsMask;
    CALL( makeVectorAndStuffFromStack )
    theVector = R;

    /* intern the print name of the function, and assign the vector to it */

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( loadNameString( f->pNameNumber ), R )
    CALL( intern )
    PUSH
    PUSH
    MOVE( theVector, R )
    LOAD_R_TAG( builtInFunctionTag )
    SWAP
    CALL( bindScheme )
    POP
    JUMP_FUNCTION( makePermanentSymbol )
    }

/**** Conventions about the vector into which the stuff from the functionDescriptor 
      (type eight) gets stuffed, documented here for want of any obvious other place:
          
          All slots contain Scheme objects (tagged Avals); not C objects.
          
          Slot #                Contents

           0                    scheme-machine pc index to code to execute the function
           1                    print name
           2                    function type (#define'd fixnum)
           3                    maximum number of arguments; -1 means infinity; this number must be positive
           4                    scheme-machine pc index to code that could type-check the result
           5                    scheme-machine pc index to code to type-check the first arguments
           6                    scheme-machine pc index to code to type-check the last argument
           7                    minimum number of arguments
           8                    string describing the first desired argument in error messages
           9                    string describing the other desired arguments in error messages
          10 ...                more to come ...
           
           ****/

/**** loadSchemeFunctionDescriptorTypeNine -- Load a type-nine function descriptor into
                                             main storage, bound to the appropriate symbol. ****/

void loadSchemeFunctionDescriptorTypeNine( functionDescriptorTypeNine *f )
{
    LongWord vectorLnth = (LongWord)9;
    taggedAval theVector;

    /* cons up list of things to put into a vector */

    R.u.c = (theWorldHandler->indexForPC( (UnsignedLongWord)(f->code) ));
    R.t = schemeMachinePCIndexTag;
    PUSH                                                    /* push index of code to execute function */
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( loadNameString( f->pNameNumber ), R )
    CALL( intern )                                          /* interning now avoids storing two copies of this string */
    GET_PNAME 
    PUSH                                                    /* push print name */
    R.u.c = NONNEGATIVE_ARITY_TWO_TYPES_STACK;
    R.t = exactShortFixnumBitsMask;
    PUSH                                                    /* push built-in type */
    R.t = exactShortFixnumBitsMask;
    if( f->maxArgs >= 0 )
        R.u.c = (f->maxArgs);
    else
        R.u.c = (-1);
    PUSH                                                    /* push maximum number of args (or -1 if infinite) */
    R.u.c = (theWorldHandler->indexForPC( (UnsignedLongWord)(f->resultTypeChecker) ));
    R.t = schemeMachinePCIndexTag;
    PUSH                                                    /* push index of code to check the result */
    R.u.c = (theWorldHandler->indexForPC( (UnsignedLongWord)(f->firstArgTypeChecker) ));
    R.t = schemeMachinePCIndexTag;
    PUSH                                                    /* push index of type checker for first argument */
    R.u.c = (theWorldHandler->indexForPC( (UnsignedLongWord)(f->restArgTypeChecker) ));
    R.t = schemeMachinePCIndexTag;
    PUSH                                                    /* push index of type checker for remaining arguments */
    makeString( f->firstArgDescription );
    PUSH;                                                   /* push a string describing an argument */
    makeString( f->restArgDescription );
    PUSH;                                                   /* push a string describing an argument */
    R.u.c = vectorLnth;
    R.t = exactShortFixnumBitsMask;
    CALL( makeVectorAndStuffFromStack )
    theVector = R;

    /* intern the print name of the function, and assign the vector to it */

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( loadNameString( f->pNameNumber ), R )
    CALL( intern )
    PUSH
    PUSH
    MOVE( theVector, R )
    LOAD_R_TAG( builtInFunctionTag )
    SWAP
    CALL( bindScheme )
    POP
    JUMP_FUNCTION( makePermanentSymbol )
    }

/**** Conventions about the vector into which the stuff from the functionDescriptor 
      (type nine) gets stuffed, documented here for want of any obvious other place:
          
          All slots contain Scheme objects (tagged Avals); not C objects.
          
          Slot #                Contents

           0                    scheme-machine pc index to code to execute the function
           1                    print name
           2                    function type (#define'd fixnum)
           3                    maximum number of arguments (minimum is one); -1 means infinity
           4                    scheme-machine pc index to code that could type-check the result
           5                    scheme-machine pc index to code to type-check the first argument
           6                    scheme-machine pc index to code to type-check the other arguments
           7                    string describing the first desired argument in error messages
           8                    string describing the other desired arguments in error messages
           9 ...                more to come ...
           
           ****/

/**** cellSortAux -- For (q)sorting, in routine immediately following. ****/

#define LOADS_PER_SPIN          16

int cellSortAux( const void *pel1, const void *pel2 )
{
    simpleListCell *el1 = (simpleListCell *)pel1;
    simpleListCell *el2 = (simpleListCell *)pel2;
    Text s1[256], s2[256];

    strcpy( s1, loadNameString( ((functionDescriptorTypeOne *)((*el1).pointer))->pNameNumber ) );
    strcpy( s2, loadNameString( ((functionDescriptorTypeOne *)((*el2).pointer))->pNameNumber ) );
    return strcmp( s1, s2 );
    }

/**** nameFromChecker ****/

const Text *nameFromChecker( void *checker )
{
    LongWord counter = 0;

    while( (typeCheckNames[counter]).n )
        if( (typeCheckNames[counter]).fPointer == checker ) {
            return TypeNameStrings[ (typeCheckNames[counter]).n ];
            }
        else
            ++counter;
    fatalError( 42 );
    return 0;
    }

/**** procedureType ****/

LongWord procedureType( UnsignedLongWord n )
{
    return descriptorArray[n].flag;
    }

/**** procedureArityText ****/

const Text *procedureArityText( UnsignedLongWord n, LongWord fType )
{
    Text controlString[ 256 ];
    Text *scratch = scratchBuffer;
    LongWord minArity, maxArity;

    switch( fType ) {
        case FIXED_ARITY:
            minArity = ((functionDescriptorTypeOne *)(descriptorArray[n].pointer))->nArgs;
            if( minArity == 1 )
                strcpy( controlString, TypeNameStrings[ 20 ] );
            else
                strcpy( controlString, TypeNameStrings[ 19 ] );
            sprintf( scratch, controlString, minArity );
            scratch = scratch + strlen( scratch );
            break;
        case ANY_ARITY_ONE_TYPE_STACK:
            minArity = ((functionDescriptorTypeFive *)(descriptorArray[n].pointer))->minArgs;
            strcpy( controlString, TypeNameStrings[ 24 ] );
            sprintf( scratch, controlString, minArity );
            scratch = scratch + strlen( scratch );
            break;
        case POSITIVE_ARITY_TWO_TYPES_STACK:
            minArity = 1;
            maxArity = ((functionDescriptorTypeSix *)(descriptorArray[n].pointer))->maxArgs;
            if( maxArity == -1 )
                strcpy( controlString, TypeNameStrings[ 24 ] );
            else if( maxArity == minArity + 1 )
                strcpy( controlString, TypeNameStrings[ 26 ] );
            else
                strcpy( controlString, TypeNameStrings[ 25 ] );
            sprintf( scratch, controlString, minArity, maxArity );
            scratch = scratch + strlen( scratch );
            break;
        case ANY_ARITY_TWO_TYPES_STACK:
            minArity = ((functionDescriptorTypeSeven *)(descriptorArray[n].pointer))->minArgs;
            maxArity = ((functionDescriptorTypeSeven *)(descriptorArray[n].pointer))->maxArgs;
            if( maxArity == -1 )
                strcpy( controlString, TypeNameStrings[ 24 ] );
            else if( maxArity == minArity + 1 )
                if( maxArity == 1 )
                    strcpy( controlString, TypeNameStrings[ 27 ] );
                else
                    strcpy( controlString, TypeNameStrings[ 26 ] );
            else
                strcpy( controlString, TypeNameStrings[ 25 ] );
            sprintf( scratch, controlString, minArity, maxArity );
            scratch = scratch + strlen( scratch );
            break;
        case ANY_ARITY_TWO_TYPES_STACK_2:
            minArity = ((functionDescriptorTypeEight *)(descriptorArray[n].pointer))->minArgs;
            maxArity = ((functionDescriptorTypeEight *)(descriptorArray[n].pointer))->maxArgs;
            if( maxArity == -1 )
                strcpy( controlString, TypeNameStrings[ 24 ] );
            else if( maxArity == minArity + 1 )
                if( maxArity == 1 )
                    strcpy( controlString, TypeNameStrings[ 27 ] );
                else
                    strcpy( controlString, TypeNameStrings[ 26 ] );
            else
                    strcpy( controlString, TypeNameStrings[ 25 ] );
            sprintf( scratch, controlString, minArity, maxArity );
            scratch = scratch + strlen( scratch );
            break;
        case NONNEGATIVE_ARITY_TWO_TYPES_STACK:
            minArity = 0;
            maxArity = ((functionDescriptorTypeNine *)(descriptorArray[n].pointer))->maxArgs;
            if( maxArity == -1 )
                strcpy( controlString, TypeNameStrings[ 24 ] );
            else if( maxArity == minArity + 1 )
                strcpy( controlString, TypeNameStrings[ 26 ] );
            else
                strcpy( controlString, TypeNameStrings[ 25 ] );
            sprintf( scratch, controlString, minArity, maxArity );
            scratch = scratch + strlen( scratch );
            break;
        default:
            scratchBuffer[0] = '\0';
        }
    return scratchBuffer;
    }

/**** procedureInputTypeText ****/

Text *procedureInputTypeText( UnsignedLongWord n, LongWord fType )
{
    Boolean allSame = TRUE;
    Text controlString[ 256 ];
    const Text *scratchString;
    Text *scratch = scratchBuffer;
    LongWord counter, minArity, maxArity;
    void (**pChecker)();

    scratchBuffer[0] = '\0';
    switch( fType ) {
        case FIXED_ARITY:
            minArity = ((functionDescriptorTypeOne *)(descriptorArray[n].pointer))->nArgs;
            if( minArity == 1 ) {
                strcpy( controlString, TypeNameStrings[ 30 ] );
                scratchString = procedureTypeStringFetchFunction( 
                                  *(((functionDescriptorTypeOne *)(descriptorArray[n].pointer))->argTypeChecker)
                                  );
                sprintf( scratch, controlString, scratchString );
                scratch[0] = toupper( scratch[0] );
                }
            else if( minArity != 0 ) {
                pChecker = (((functionDescriptorTypeOne *)(descriptorArray[n].pointer))->argTypeChecker);
                for( counter = 1; counter < minArity; ++counter )
                    if( *(pChecker + counter) != *pChecker ) {
                        allSame = FALSE;
                        break;
                        }
                if( allSame ) {
                    if( minArity == 2 )
                        strcpy( controlString, TypeNameStrings[ 37 ] );
                    else
                        strcpy( controlString, TypeNameStrings[ 28 ] );
                    sprintf( scratch, controlString,
                             procedureTypeStringFetchFunction( 
                                 *(((functionDescriptorTypeOne *)(descriptorArray[n].pointer))->argTypeChecker)
                                 ) );
                    }
                else {
                    strcpy( controlString, TypeNameStrings[ 21 ] );
                    pChecker = (((functionDescriptorTypeOne *)(descriptorArray[n].pointer))->argTypeChecker);
                    for( counter = 0; counter < minArity; ++counter ) {
                        sprintf( scratch, controlString, counter + 1, procedureTypeStringFetchFunction( *(pChecker + counter) ) );
                        scratch = scratch + strlen( scratch );
                        }
                    }
                }

           // else nothing

            break;
        case ANY_ARITY_ONE_TYPE_STACK:
            minArity = ((functionDescriptorTypeFive *)(descriptorArray[n].pointer))->minArgs;
            strcpy( controlString, TypeNameStrings[ 28 ] );
            sprintf( scratch, controlString,
                     procedureTypeStringFetchFunction( 
                     *(((functionDescriptorTypeFive *)(descriptorArray[n].pointer))->argTypeChecker)
                     ) );
            break;
        case POSITIVE_ARITY_TWO_TYPES_STACK:
        case NONNEGATIVE_ARITY_TWO_TYPES_STACK:  //  Fall-through is intentional.
            maxArity = ((functionDescriptorTypeSix *)(descriptorArray[n].pointer))->maxArgs;
            if(    *(((functionDescriptorTypeSix *)(descriptorArray[n].pointer))->firstArgTypeChecker)
                ==
                   *(((functionDescriptorTypeSix *)(descriptorArray[n].pointer))->restArgTypeChecker) ) {
                strcpy( controlString, TypeNameStrings[ 28 ] );
                sprintf( scratch, controlString,
                         procedureTypeStringFetchFunction( 
                         *(((functionDescriptorTypeSix *)(descriptorArray[n].pointer))->firstArgTypeChecker)
                         ) );
                }
            else if( maxArity == -1 ) {
                strcpy( controlString, TypeNameStrings[ 21 ] );
                sprintf( scratch, controlString, 1,
                         procedureTypeStringFetchFunction( 
                         *(((functionDescriptorTypeSix *)(descriptorArray[n].pointer))->firstArgTypeChecker)
                         ) );
                scratch = scratch + strlen( scratch );
                strcpy( controlString, TypeNameStrings[ 23 ] );
                sprintf( scratch, controlString, 2,
                         procedureTypeStringFetchFunction( 
                         *(((functionDescriptorTypeSix *)(descriptorArray[n].pointer))->restArgTypeChecker)
                         ) );
                }
            else if( maxArity == 1 ) {
                strcpy( controlString, TypeNameStrings[ 30 ] );
                sprintf( scratch, controlString,
                         procedureTypeStringFetchFunction( 
                         *(((functionDescriptorTypeSix *)(descriptorArray[n].pointer))->firstArgTypeChecker)
                         ) );
                }
           else if( maxArity == 2 ) {
                strcpy( controlString, TypeNameStrings[ 21 ] );
                sprintf( scratch, controlString, 1,
                         procedureTypeStringFetchFunction( 
                         *(((functionDescriptorTypeSix *)(descriptorArray[n].pointer))->firstArgTypeChecker)
                         ) );
                scratch = scratch + strlen( scratch );
                sprintf( scratch, controlString, 2,
                         procedureTypeStringFetchFunction( 
                         *(((functionDescriptorTypeSix *)(descriptorArray[n].pointer))->restArgTypeChecker)
                         ) );
                }
            else if( maxArity == 3 ){
                strcpy( controlString, TypeNameStrings[ 21 ] );
                sprintf( scratch, controlString, 1,
                         procedureTypeStringFetchFunction( 
                         *(((functionDescriptorTypeSix *)(descriptorArray[n].pointer))->firstArgTypeChecker)
                         ) );
                scratch = scratch + strlen( scratch );
                strcpy( controlString, TypeNameStrings[ 22 ] );
                sprintf( scratch, controlString, 2, maxArity,
                         procedureTypeStringFetchFunction( 
                         *(((functionDescriptorTypeSix *)(descriptorArray[n].pointer))->restArgTypeChecker)
                         ) );
                }
            else {
                strcpy( controlString, TypeNameStrings[ 21 ] );
                sprintf( scratch, controlString, 1,
                         procedureTypeStringFetchFunction( 
                         *(((functionDescriptorTypeSix *)(descriptorArray[n].pointer))->firstArgTypeChecker)
                         ) );
                scratch = scratch + strlen( scratch );
                strcpy( controlString, TypeNameStrings[ 29 ] );
                sprintf( scratch, controlString, 2, maxArity,
                         procedureTypeStringFetchFunction( 
                         *(((functionDescriptorTypeSix *)(descriptorArray[n].pointer))->restArgTypeChecker)
                         ) );
                }
            break;
        case ANY_ARITY_TWO_TYPES_STACK:
            minArity = ((functionDescriptorTypeSeven *)(descriptorArray[n].pointer))->minArgs;
            maxArity = ((functionDescriptorTypeSeven *)(descriptorArray[n].pointer))->maxArgs;
            if(    *(((functionDescriptorTypeSeven *)(descriptorArray[n].pointer))->firstArgTypeChecker)
                ==
                   *(((functionDescriptorTypeSeven *)(descriptorArray[n].pointer))->restArgTypeChecker) ) {
                strcpy( controlString, TypeNameStrings[ 28 ] );
                sprintf( scratch, controlString,
                         procedureTypeStringFetchFunction( 
                         *(((functionDescriptorTypeSeven *)(descriptorArray[n].pointer))->firstArgTypeChecker)
                         ) );
                }
            else if( maxArity == -1 ) {
                strcpy( controlString, TypeNameStrings[ 21 ] );
                sprintf( scratch, controlString, 1,
                         procedureTypeStringFetchFunction( 
                         *(((functionDescriptorTypeSeven *)(descriptorArray[n].pointer))->firstArgTypeChecker)
                         ) );
                scratch = scratch + strlen( scratch );
                strcpy( controlString, TypeNameStrings[ 23 ] );
                sprintf( scratch, controlString, 2,
                         procedureTypeStringFetchFunction( 
                         *(((functionDescriptorTypeSeven *)(descriptorArray[n].pointer))->restArgTypeChecker)
                         ) );
                }
            else if( maxArity == 1 ) {
                strcpy( controlString, TypeNameStrings[ 21 ] );
                sprintf( scratch, controlString, 1,
                         procedureTypeStringFetchFunction( 
                         *(((functionDescriptorTypeSeven *)(descriptorArray[n].pointer))->firstArgTypeChecker)
                         ) );
                }
            else if( maxArity == 2 ) {
                strcpy( controlString, TypeNameStrings[ 21 ] );
                sprintf( scratch, controlString, 1,
                         procedureTypeStringFetchFunction( 
                         *(((functionDescriptorTypeSeven *)(descriptorArray[n].pointer))->firstArgTypeChecker)
                         ) );
                scratch = scratch + strlen( scratch );
                sprintf( scratch, controlString, 2,
                         procedureTypeStringFetchFunction( 
                         *(((functionDescriptorTypeSeven *)(descriptorArray[n].pointer))->restArgTypeChecker)
                         ) );
                }
            else if( maxArity == 3 ){
                strcpy( controlString, TypeNameStrings[ 21 ] );
                sprintf( scratch, controlString, 1,
                         procedureTypeStringFetchFunction( 
                         *(((functionDescriptorTypeSeven *)(descriptorArray[n].pointer))->firstArgTypeChecker)
                         ) );
                scratch = scratch + strlen( scratch );
                strcpy( controlString, TypeNameStrings[ 22 ] );
                sprintf( scratch, controlString, 2, maxArity,
                         procedureTypeStringFetchFunction( 
                         *(((functionDescriptorTypeSeven *)(descriptorArray[n].pointer))->restArgTypeChecker)
                         ) );
                }
            else {
                strcpy( controlString, TypeNameStrings[ 21 ] );
                sprintf( scratch, controlString, 1,
                         procedureTypeStringFetchFunction( 
                         *(((functionDescriptorTypeSeven *)(descriptorArray[n].pointer))->firstArgTypeChecker)
                         ) );
                scratch = scratch + strlen( scratch );
                strcpy( controlString, TypeNameStrings[ 29 ] );
                sprintf( scratch, controlString, 2, maxArity,
                         procedureTypeStringFetchFunction( 
                         *(((functionDescriptorTypeSeven *)(descriptorArray[n].pointer))->restArgTypeChecker)
                         ) );
                }
        break;
        case ANY_ARITY_TWO_TYPES_STACK_2:
            minArity = ((functionDescriptorTypeEight *)(descriptorArray[n].pointer))->minArgs;
            maxArity = ((functionDescriptorTypeEight *)(descriptorArray[n].pointer))->maxArgs;
            if(    *(((functionDescriptorTypeEight *)(descriptorArray[n].pointer))->firstArgTypeChecker)
                ==
                   *(((functionDescriptorTypeEight *)(descriptorArray[n].pointer))->restArgTypeChecker) ) {
                strcpy( controlString, TypeNameStrings[ 28 ] );
                sprintf( scratch, controlString,
                         procedureTypeStringFetchFunction( 
                         *(((functionDescriptorTypeEight *)(descriptorArray[n].pointer))->firstArgTypeChecker)
                         ) );
                }
            else if( maxArity == 1 ) {
                strcpy( controlString, TypeNameStrings[ 32 ] );
                sprintf( scratch, controlString,
                         procedureTypeStringFetchFunction( 
                         *(((functionDescriptorTypeEight *)(descriptorArray[n].pointer))->firstArgTypeChecker)
                         ) );
                }
            else {
                strcpy( controlString, TypeNameStrings[ 31 ] );
                sprintf( scratch, controlString,
                         procedureTypeStringFetchFunction( 
                         *(((functionDescriptorTypeEight *)(descriptorArray[n].pointer))->restArgTypeChecker)
                         ) );
                scratch = scratch + strlen( scratch );
                strcpy( controlString, TypeNameStrings[ 33 ] );
                sprintf( scratch, controlString,
                         procedureTypeStringFetchFunction( 
                         *(((functionDescriptorTypeEight *)(descriptorArray[n].pointer))->firstArgTypeChecker)
                         ) );
                }
            break;
        default:
            break;
        }
    return scratchBuffer;
    }

/**** procedureReturnTypeText ****/

const Text *procedureReturnTypeText( UnsignedLongWord n, LongWord fType )
{
    Boolean valueP;
    void (*rChecker)();
    Text controlString[ 256 ];
    Text typeString[ 256 ];

    switch( fType ) {
        case FIXED_ARITY:
            valueP = ((functionDescriptorTypeOne *)(descriptorArray[n].pointer))->returnsValueP;
            rChecker = ((functionDescriptorTypeOne *)(descriptorArray[n].pointer))->resultTypeChecker;
            break;
        case ANY_ARITY_ONE_TYPE_STACK:
            valueP = ((functionDescriptorTypeFive *)(descriptorArray[n].pointer))->returnsValueP;
            rChecker = ((functionDescriptorTypeFive *)(descriptorArray[n].pointer))->resultTypeChecker;
            break;
        case POSITIVE_ARITY_TWO_TYPES_STACK:
            valueP = ((functionDescriptorTypeSix *)(descriptorArray[n].pointer))->returnsValueP;
            rChecker = ((functionDescriptorTypeSix *)(descriptorArray[n].pointer))->resultTypeChecker;
            break;
        case ANY_ARITY_TWO_TYPES_STACK:
            valueP = ((functionDescriptorTypeSeven *)(descriptorArray[n].pointer))->returnsValueP;
            rChecker = ((functionDescriptorTypeSeven *)(descriptorArray[n].pointer))->resultTypeChecker;
            break;
        case ANY_ARITY_TWO_TYPES_STACK_2:
            valueP = ((functionDescriptorTypeEight *)(descriptorArray[n].pointer))->returnsValueP;
            rChecker = ((functionDescriptorTypeSeven *)(descriptorArray[n].pointer))->resultTypeChecker;
            break;
        case NONNEGATIVE_ARITY_TWO_TYPES_STACK:
            valueP = ((functionDescriptorTypeNine *)(descriptorArray[n].pointer))->returnsValueP;
            rChecker = ((functionDescriptorTypeNine *)(descriptorArray[n].pointer))->resultTypeChecker;
            break;
        default:
            scratchBuffer[0] = '\0';
            return scratchBuffer;
        }
    if( valueP ) {
        strcpy( controlString, TypeNameStrings[ 35 ] );
        strcpy( typeString, procedureTypeStringFetchFunction( rChecker ) );
        sprintf( scratchBuffer, controlString, typeString );
        }
    else if(   ((descriptorArray[n].pointer) == (void *)(&quitFD)) 
        || ((descriptorArray[n].pointer) == (void *)(&resetFD)) ) {
        strcpy( controlString, TypeNameStrings[ 43 ] );
        sprintf( scratchBuffer, "%s", controlString );
        }
    else {
    strcpy( controlString, TypeNameStrings[ 34 ] );
    sprintf( scratchBuffer, "%s", controlString );
    }
return scratchBuffer;
}

/**** procedureTypeStringFetchFunction ****/

const Text *procedureTypeStringFetchFunction( void (*typeChecker)() )
{
    LongWord counter = 0;
    Text *answer = 0;

    while( (typeCheckNames[counter]).fPointer != (void *)0 ) {
        if( (typeCheckNames[counter]).fPointer == (void *)typeChecker ) {
            answer = TypeNameStrings[ (typeCheckNames[counter]).n ];
            break;
            }
        else
            ++counter;
        }
    return answer;
    }

/**** loadSchemeFunctions -- Load all types of functions into main storage. ****/

UnsignedLongWord loadSchemeFunctions( Boolean showListP )
{
    simpleListCell *l1A = functionDescriptorTypeOneListA;
    simpleListCell *l1B = functionDescriptorTypeOneListB;
    simpleListCell *l5 = functionDescriptorTypeFiveList;
    simpleListCell *l6 = functionDescriptorTypeSixList;
    simpleListCell *l7 = functionDescriptorTypeSevenList;
    simpleListCell *l8 = functionDescriptorTypeEightList;
    simpleListCell *l9 = functionDescriptorTypeNineList;

    UnsignedLongWord nFuncs =   simpleListLength(l1A) + simpleListLength(l1B) + simpleListLength(l5)
                              + simpleListLength(l6) + simpleListLength(l7)
                              + simpleListLength(l8) + simpleListLength(l9);

    UnsignedLongWord nFuncCounter = 0; 

    descriptorArray = (simpleListCell *)malloc( (size_t)(nFuncs * sizeof( simpleListCell )) );

    if( descriptorArray == (simpleListCell *)0 )
        fatalError( 34 );

    if( showListP ) {
        describeFunctionDescriptorTypeOneList( l1A );
        tprintf2( &CurrentOutputPort, MiscStrings[ 66 ] );
        }
    while( l1A->flag ) {
        loadSchemeFunctionDescriptorTypeOne( (functionDescriptorTypeOne *)(l1A->pointer) );
        descriptorArray[ nFuncCounter ].flag = FIXED_ARITY;
        descriptorArray[ nFuncCounter++ ].pointer = l1A->pointer;
        ++l1A;
        }
    if( showListP )
        tprintf2( &CurrentOutputPort, MiscStrings[ 124 ] );

    if( showListP ) {
        describeFunctionDescriptorTypeOneList( l1B );
        tprintf2( &CurrentOutputPort, MiscStrings[ 66 ] );
        }
    while( l1B->flag ) {
        loadSchemeFunctionDescriptorTypeOne( (functionDescriptorTypeOne *)(l1B->pointer) );
        descriptorArray[ nFuncCounter ].flag = FIXED_ARITY;
        descriptorArray[ nFuncCounter++ ].pointer = l1B->pointer;
        ++l1B;
        }
    if( showListP )
        tprintf2( &CurrentOutputPort, MiscStrings[ 124 ] );

    if( showListP ) {
        describeFunctionDescriptorTypeFiveList( l5 );
        tprintf2( &CurrentOutputPort, MiscStrings[ 186 ] );
        }
    while( l5->flag ) {
        loadSchemeFunctionDescriptorTypeFive( (functionDescriptorTypeFive *)(l5->pointer) );
        descriptorArray[ nFuncCounter ].flag = ANY_ARITY_ONE_TYPE_STACK;
        descriptorArray[ nFuncCounter++ ].pointer = l5->pointer;
        ++l5;
        }
    if( showListP )
        tprintf2( &CurrentOutputPort, MiscStrings[ 124 ] );

    if( showListP ) {
        describeFunctionDescriptorTypeSixList( l6 );
        tprintf2( &CurrentOutputPort, MiscStrings[ 187 ] );
        }
    while( l6->flag ) {
        loadSchemeFunctionDescriptorTypeSix( (functionDescriptorTypeSix *)(l6->pointer) );
        descriptorArray[ nFuncCounter ].flag = POSITIVE_ARITY_TWO_TYPES_STACK;
        descriptorArray[ nFuncCounter++ ].pointer = l6->pointer;
        ++l6;
        }
    if( showListP )
        tprintf2( &CurrentOutputPort, MiscStrings[ 124 ] );

    if( showListP ) {
        describeFunctionDescriptorTypeSevenList( l7 );
        tprintf2( &CurrentOutputPort, MiscStrings[ 196 ] );
        }
    while( l7->flag ) {
        loadSchemeFunctionDescriptorTypeSeven( (functionDescriptorTypeSeven *)(l7->pointer) );
        descriptorArray[ nFuncCounter ].flag = ANY_ARITY_TWO_TYPES_STACK;
        descriptorArray[ nFuncCounter++ ].pointer = l7->pointer;
        ++l7;
        }
    if( showListP )
        tprintf2( &CurrentOutputPort, MiscStrings[ 124 ] );

    if( showListP ) {
        describeFunctionDescriptorTypeEightList( l8 );
        tprintf2( &CurrentOutputPort, MiscStrings[ 238 ] );
        }
    while( l8->flag ) {
        loadSchemeFunctionDescriptorTypeEight( (functionDescriptorTypeEight *)(l8->pointer) );
        descriptorArray[ nFuncCounter ].flag = ANY_ARITY_TWO_TYPES_STACK_2;
        descriptorArray[ nFuncCounter++ ].pointer = l8->pointer;
        ++l8;
        }

    if( showListP ) {
        describeFunctionDescriptorTypeNineList( l9 );
        tprintf2( &CurrentOutputPort, MiscStrings[ 187 ] );
        }
    while( l9->flag ) {
        loadSchemeFunctionDescriptorTypeNine( (functionDescriptorTypeNine *)(l9->pointer) );
        descriptorArray[ nFuncCounter ].flag = POSITIVE_ARITY_TWO_TYPES_STACK;
        descriptorArray[ nFuncCounter++ ].pointer = l9->pointer;
        ++l9;
        }
    if( showListP )
        tprintf2( &CurrentOutputPort, MiscStrings[ 124 ] );

    if( showListP )
        tprintf2( &CurrentOutputPort, MiscStrings[ 124 ] );
    qsort( descriptorArray, (int)nFuncs, sizeof( simpleListCell ), cellSortAux );

    return nFuncCounter;
    }

/**** initializePortmanteauIndices ****/

UnsignedLongWord CPushFunction_index = 0;
UnsignedLongWord ClearLFunction_index = 0;
UnsignedLongWord MakeRTrueFunction_index = 0;
UnsignedLongWord S0toC_index = 0;
UnsignedLongWord SPopRFunction_index = 0;
UnsignedLongWord SPushFunction_index = 0;
UnsignedLongWord SwapCons_index = 0;
UnsignedLongWord apply_index = 0;
UnsignedLongWord applyEnd_index = 0;
UnsignedLongWord assertEvaluating_index = 0;
UnsignedLongWord assertFileLoadSucceeded_index = 0;
UnsignedLongWord assertNotEvaluating_index = 0;
UnsignedLongWord assign_index = 0;
UnsignedLongWord assign_with_block_index = 0;
UnsignedLongWord bindOrAssign_index = 0;
UnsignedLongWord bindScheme_index = 0;
UnsignedLongWord bindToOptionMinus_index = 0;
UnsignedLongWord closePort_index = 0;
UnsignedLongWord closeStackedPort_index = 0;
UnsignedLongWord closeStackedPortRestoreInput_index = 0;
UnsignedLongWord closeStackedPortRestoreOutput_index = 0;
UnsignedLongWord cwCCode_index = 0;
UnsignedLongWord defineEvalCode_index = 0;
UnsignedLongWord dereferenceStackReferenceMC_index = 0;
UnsignedLongWord displayObject_index = 0;
UnsignedLongWord dropNFromStack_index = 0;
UnsignedLongWord expandMacroAux_index = 0;
UnsignedLongWord ifAuxCode_index = 0;
UnsignedLongWord loadCode_index = 0;
UnsignedLongWord macroBodyAux_index = 0;
UnsignedLongWord macroEvalAux_index = 0;
UnsignedLongWord macroPartTwo_index = 0;
UnsignedLongWord newlineIfPrinting_index = 0;
UnsignedLongWord noop_index = 0;
UnsignedLongWord oneForEach_index = 0;
UnsignedLongWord oneMap_index = 0;
UnsignedLongWord oneMapFirst_index = 0;
UnsignedLongWord peekLoop_index = 0;
UnsignedLongWord popToEFunction_index = 0;
UnsignedLongWord readSexpFromPort_index = 0;
UnsignedLongWord rebindForce_index = 0;
UnsignedLongWord rebindTopLoopVariables_index = 0;
UnsignedLongWord resetScheme_index = 0;
UnsignedLongWord restoreInputPort_index = 0;
UnsignedLongWord restoreStackAndFrame_index = 0;
UnsignedLongWord reverseListCode_index = 0;
UnsignedLongWord tfflushConsoleOutput_index = 0;
UnsignedLongWord topLoopCode_index = 0;
UnsignedLongWord writeObject_index = 0;
UnsignedLongWord readSexpOrQueueTopLevel_index = 0;
UnsignedLongWord inputQueueCode_index = 0;
UnsignedLongWord kittenDequeueCode_index = 0;
UnsignedLongWord kittenEmptyQueueCode_index = 0;
UnsignedLongWord restoreForCWCC_index = 0;
UnsignedLongWord setApplicativeBitNDownBang_index = 0;

void initializePortmanteauIndices()
{
    CPushFunction_index = theWorldHandler->indexForPC( (UnsignedLongWord)CPushFunction );
    ClearLFunction_index = theWorldHandler->indexForPC( (UnsignedLongWord)ClearLFunction );
    MakeRTrueFunction_index = theWorldHandler->indexForPC( (UnsignedLongWord)MakeRTrueFunction );
    S0toC_index = theWorldHandler->indexForPC( (UnsignedLongWord)S0toC );
    SPopRFunction_index = theWorldHandler->indexForPC( (UnsignedLongWord)SPopRFunction );
    SPushFunction_index = theWorldHandler->indexForPC( (UnsignedLongWord)SPushFunction );
    SwapCons_index = theWorldHandler->indexForPC( (UnsignedLongWord)SwapCons );
    apply_index = theWorldHandler->indexForPC( (UnsignedLongWord)apply );
    applyEnd_index = theWorldHandler->indexForPC( (UnsignedLongWord)applyEnd );
    assertEvaluating_index = theWorldHandler->indexForPC( (UnsignedLongWord)assertEvaluating );
    assertFileLoadSucceeded_index = theWorldHandler->indexForPC( (UnsignedLongWord)assertFileLoadSucceeded );
    assertNotEvaluating_index = theWorldHandler->indexForPC( (UnsignedLongWord)assertNotEvaluating );
    assign_index = theWorldHandler->indexForPC( (UnsignedLongWord)assign );
    bindOrAssign_index = theWorldHandler->indexForPC( (UnsignedLongWord)bindOrAssign );
    bindScheme_index = theWorldHandler->indexForPC( (UnsignedLongWord)bindScheme );
    bindToOptionMinus_index = theWorldHandler->indexForPC( (UnsignedLongWord)bindToOptionMinus );
    closePort_index = theWorldHandler->indexForPC( (UnsignedLongWord)closePort );
    closeStackedPort_index = theWorldHandler->indexForPC( (UnsignedLongWord)closeStackedPort );
    closeStackedPortRestoreInput_index = theWorldHandler->indexForPC( (UnsignedLongWord)closeStackedPortRestoreInput );
    closeStackedPortRestoreOutput_index = theWorldHandler->indexForPC( (UnsignedLongWord)closeStackedPortRestoreOutput );
    cwCCode_index = theWorldHandler->indexForPC( (UnsignedLongWord)cwCCode );
    defineEvalCode_index = theWorldHandler->indexForPC( (UnsignedLongWord)defineEvalCode );
    dereferenceStackReferenceMC_index = theWorldHandler->indexForPC( (UnsignedLongWord)dereferenceStackReferenceMC );
    displayObject_index = theWorldHandler->indexForPC( (UnsignedLongWord)displayObject );
    dropNFromStack_index = theWorldHandler->indexForPC( (UnsignedLongWord)dropNFromStack );
    expandMacroAux_index = theWorldHandler->indexForPC( (UnsignedLongWord)expandMacroAux );
    ifAuxCode_index = theWorldHandler->indexForPC( (UnsignedLongWord)ifAuxCode );
    loadCode_index = theWorldHandler->indexForPC( (UnsignedLongWord)loadCode );
    macroBodyAux_index = theWorldHandler->indexForPC( (UnsignedLongWord)macroBodyAux );
    macroEvalAux_index = theWorldHandler->indexForPC( (UnsignedLongWord)macroEvalAux );
    macroPartTwo_index = theWorldHandler->indexForPC( (UnsignedLongWord)macroPartTwo );
    newlineIfPrinting_index = theWorldHandler->indexForPC( (UnsignedLongWord)newlineIfPrinting );
    noop_index = theWorldHandler->indexForPC( (UnsignedLongWord)noop );
    oneForEach_index = theWorldHandler->indexForPC( (UnsignedLongWord)oneForEach );
    oneMap_index = theWorldHandler->indexForPC( (UnsignedLongWord)oneMap );
    oneMapFirst_index = theWorldHandler->indexForPC( (UnsignedLongWord)oneMapFirst );
    peekLoop_index = theWorldHandler->indexForPC( (UnsignedLongWord)peekLoop );
    popToEFunction_index = theWorldHandler->indexForPC( (UnsignedLongWord)popToEFunction );
    readSexpFromPort_index = theWorldHandler->indexForPC( (UnsignedLongWord)readSexpFromPort );
    rebindForce_index = theWorldHandler->indexForPC( (UnsignedLongWord)rebindForce );
    rebindTopLoopVariables_index = theWorldHandler->indexForPC( (UnsignedLongWord)rebindTopLoopVariables );
    resetScheme_index = theWorldHandler->indexForPC( (UnsignedLongWord)resetScheme );
    restoreInputPort_index = theWorldHandler->indexForPC( (UnsignedLongWord)restoreInputPort );
    restoreStackAndFrame_index = theWorldHandler->indexForPC( (UnsignedLongWord)restoreStackAndFrame );
    reverseListCode_index = theWorldHandler->indexForPC( (UnsignedLongWord)reverseListCode );
    tfflushConsoleOutput_index = theWorldHandler->indexForPC( (UnsignedLongWord)tfflushConsoleOutput );
    topLoopCode_index = theWorldHandler->indexForPC( (UnsignedLongWord)topLoopCode );
    writeObject_index = theWorldHandler->indexForPC( (UnsignedLongWord)writeObject );
    readSexpOrQueueTopLevel_index = theWorldHandler->indexForPC( (UnsignedLongWord)readSexpOrQueueTopLevel );
    inputQueueCode_index = theWorldHandler->indexForPC( (UnsignedLongWord)inputQueueCode );
    kittenDequeueCode_index = theWorldHandler->indexForPC( (UnsignedLongWord)kittenDequeueCode );
    kittenEmptyQueueCode_index = theWorldHandler->indexForPC( (UnsignedLongWord)kittenEmptyQueueCode );
    restoreForCWCC_index = theWorldHandler->indexForPC( (UnsignedLongWord)restoreForCWCC );
    setApplicativeBitNDownBang_index = theWorldHandler->indexForPC( (UnsignedLongWord)setApplicativeBitNDownCode );
    }
