/****************************************************************************************

SchemeFunctions2.c++ -- Copyright (c) by Jay Reynolds Freeman 1988, 1989, 1990, 1991, 2006

Scheme built-in functions for Pixie, and all code that needs to know about the
"functionDescriptor" structures.  This is file no. 2 of several ...

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import <stdio.h>
#import "WraithHeader.h"
#import "MemoryManager.h"
#import "OpCodes.h"
#import "SchemeFunctions.h"
#import "SchemeFunctions.e"
#import "Globals.e"
#import "DataTypes.e"
#import "Intern.e"
#import "IO.e"
#import "NumberFormats.e"
#import "SchemeAux.e"
#import "SchemeFunctionStatics.h"

/******** Stuff for Scheme built-in functions; code and associated statics ********/

/**** leafPCode -- Is the object in R a leaf? ****/

PROC( leafPCode )

    LEAF_P 
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne leafPFD = { leafPCode, 270, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** assocCode -- Code for "assoc"; R: AList; S: (object ...); returns the "right" pair or (). ****/

PROC( assocCode )

  begin:
    EMPTY_LIST_P
    JUMP_TRUE( notThere )
    PAIR_P
    JUMP_FALSE( improperList )
    PUSH
    CAR                                     /* R: carAList; S: (AList, object ...) */
    EXCHANGE( STACK( 0 ), STACK( 1 ) )      /* R: carAList; S: (object, AList ...) */
    PAIR_P
    JUMP_FALSE( notAList )
    CALL( carEqualP )
    JUMP_TRUE( foundIt )
    POP
    SWAP
    CDR          
    JUMP_LABEL( begin )

  notThere:
    DROP
    LOAD_R_TAG( falseBooleanTag )
    CONTINUE

  foundIt:
    DROP
    DROP
    CONTINUE

  improperList:
    resetError( 60 );
    CONTINUE

  notAList:
    resetError( 61 );
    CONTINUE

END_PROC

functionDescriptorTypeOne assocFD = { assocCode, 16, TRUE, sharpFOrProperListP, pairTag, 2, anyProperListChecker, anyProperListCheckerNames };

/**** assqCode  -- Code for "assq"; R: AList; S: (object ...); returns the "right" pair or ().****/

PROC( assqCode )

begin:
    EMPTY_LIST_P
    JUMP_TRUE( notThere )
    PAIR_P
    JUMP_FALSE( improperList )
    PUSH
    CAR                                     /* R: carAList; S: (AList, object ...) */
    EXCHANGE( STACK( 0 ), STACK( 1 ) )      /* R: carAList; S: (object, AList ...) */
    PAIR_P
    JUMP_FALSE( notAList )
    CALL( carEqP )
    JUMP_TRUE( foundIt )
    POP
    SWAP
    CDR          
    JUMP_LABEL( begin )

  notThere:
    DROP
    LOAD_R_TAG( falseBooleanTag )
    CONTINUE

  foundIt:
    DROP
    DROP
    CONTINUE

  improperList:
    resetError( 62 );
    CONTINUE

  notAList:
    resetError( 63 );
    CONTINUE

END_PROC

functionDescriptorTypeOne assqFD = { assqCode, 17, TRUE, sharpFOrProperListP, pairTag, 2, anyProperListChecker, anyProperListCheckerNames };

/**** assvCode  -- Code for "assv"; R: AList; S: (object ...); returns the "right" pair or ().****/

PROC( assvCode )

begin:
    EMPTY_LIST_P
    JUMP_TRUE( notThere )
    PAIR_P
    JUMP_FALSE( improperList )
    PUSH
    CAR                                     /* R: carAList; S: (AList, object ...) */
    EXCHANGE( STACK( 0 ), STACK( 1 ) )      /* R: carAList; S: (object, AList ...) */
    PAIR_P
    JUMP_FALSE( notAList )
    CALL( carEqvP )
    JUMP_TRUE( foundIt )
    POP
    SWAP
    CDR          
    JUMP_LABEL( begin )

  notThere:
    DROP
    LOAD_R_TAG( falseBooleanTag )
    CONTINUE

  foundIt:
    DROP
    DROP
    CONTINUE

  improperList:
    resetError( 64 );
    CONTINUE

  notAList:
    resetError( 65 );
    CONTINUE

END_PROC

functionDescriptorTypeOne assvFD = { assvCode, 18, TRUE, sharpFOrProperListP, trueBooleanTag, 2, anyProperListChecker, anyProperListCheckerNames };

/**** blockCode -- blocks whatever is in R ****/

PROC( blockCode )

    spliceOutNotCarForwardingPointers( R );
    if( ! blockOnTagLock1( &R, 16 ) )
        resetError( 203 );
    CONTINUE

END_PROC

functionDescriptorTypeOne blockFD = { blockCode, 328, TRUE, alwaysTrue, trueBooleanTag, 1, nonLeafChecker, nonLeafCheckerNames };

/**** releaseCode -- releases whatever is in R ****/

PROC( releaseCode )

    releaseTagLock1Barrier( (volatile taggedAval *)(R.u.a), 32 );
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne releaseFD = { releaseCode, 329, TRUE, alwaysTrue, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** blockSymbolBindingCode -- Symbol in R, returns binding pair, if any ****/

PROC( blockSymbolBindingCode )

    PUSH
    GET_PNAME
    CALL( boundStringP )
    JUMP_FALSE( noSuchSymbol )
    DROP
    spliceOutNotCarForwardingPointers( R );
    if( ! blockOnTagLock1( &R, 17 ) )
        resetError( 206 );
    PUSH
    CALL( carRegardlessOfBlocking )
    SWAP
    NILCONS
    CONS
    CONTINUE

  noSuchSymbol:
    POP
    DISPLAY
    resetError( 10 );
    CONTINUE

END_PROC

functionDescriptorTypeOne blockSymbolBindingFD = { blockSymbolBindingCode, 330, TRUE, pairP, pairTag, 1, symbolChecker, symbolCheckerNames };

/**** setBlockedBindingCode -- new value in R, blocked binding pair on stack ****/

PROC( setBlockedBindingCode )

    CALL( setCarWithPairAlreadyBlockedKeepBlock )
    RETURN_DEFAULT
    CONTINUE

END_PROC 

functionDescriptorTypeOne setBlockedBindingFD = { setBlockedBindingCode, 335, FALSE, alwaysTrue, trueBooleanTag, 2, anyAnyChecker, anyAnyCheckerNames };

/**** blockAnyBindingCode -- (foo . whatever) in R, returns a list whose car is foo immediately after the lock,
                             and whose cadr is (foo . whatever), but now locked.  ****/

PROC( blockAnyBindingCode )

    spliceOutNotCarForwardingPointers( R );
    if( ! blockOnTagLock1( &R, 52 ) )
        resetError( 256 );
    PUSH
    CALL( carRegardlessOfBlocking )
    SWAP
    NILCONS
    CONS
    CONTINUE

END_PROC

functionDescriptorTypeOne blockAnyBindingFD = { blockAnyBindingCode, 426, TRUE, pairP, pairTag, 1, pairChecker, pairCheckerNames };

/**** booleanPCode -- Is the quantity in R a Boolean? ****/

PROC( booleanPCode) 

    BOOLEAN_P
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne booleanPFD = { booleanPCode, 21, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** boundInstancePCode -- Does the specific binding in R have a value? ****/

PROC( boundInstancePCode )

    BOUND_SYMBOL_P
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne boundInstancePFD = { boundInstancePCode, 231, TRUE, booleanP, trueBooleanTag, 1, symbolChecker, symbolCheckerNames };

/**** boundInstanceInEnvironmentPCode -- Does the specific binding in R have a value in the given environment? ****/

PROC( boundInstanceInEnvironmentPCode )

    boundSymbolInEnvironmentP();
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne boundInstanceInEnvironmentPFD
    = { boundInstanceInEnvironmentPCode, 306, TRUE, booleanP, trueBooleanTag, 2, symbolPairChecker, symbolPairCheckerNames };

/**** car, cdr ... -- Canonical primitives, act on R and return in R. ****/

PROC( cadrCode )        CDR           CAR CONTINUE                              END_PROC
PROC( cddrCode )        CDR           CDR CONTINUE                              END_PROC
PROC( cdarCode )        CAR           CDR CONTINUE                              END_PROC
PROC( caarCode )        CAR           CAR CONTINUE                              END_PROC

PROC( caddrCode )       CDR           CDR CAR CONTINUE                          END_PROC
PROC( cdddrCode )       CDR           CDR CDR CONTINUE                          END_PROC
PROC( cdadrCode )       CDR           CAR CDR CONTINUE                          END_PROC
PROC( caadrCode )       CDR           CAR CAR CONTINUE                          END_PROC
PROC( cadarCode )       CAR           CDR CAR CONTINUE                          END_PROC
PROC( cddarCode )       CAR           CDR CDR CONTINUE                          END_PROC
PROC( cdaarCode )       CAR           CAR CDR CONTINUE                          END_PROC
PROC( caaarCode )       CAR           CAR CAR CONTINUE                          END_PROC

PROC( cadddrCode )      CDR           CDR CDR CAR CONTINUE                      END_PROC
PROC( cddddrCode )      CDR           CDR CDR CDR CONTINUE                      END_PROC
PROC( cdaddrCode )      CDR           CDR CAR CDR CONTINUE                      END_PROC
PROC( caaddrCode )      CDR           CDR CAR CAR CONTINUE                      END_PROC
PROC( cadadrCode )      CDR           CAR CDR CAR CONTINUE                      END_PROC
PROC( cddadrCode )      CDR           CAR CDR CDR CONTINUE                      END_PROC
PROC( cdaadrCode )      CDR           CAR CAR CDR CONTINUE                      END_PROC
PROC( caaadrCode )      CDR           CAR CAR CAR CONTINUE                      END_PROC
PROC( caddarCode )      CAR           CDR CDR CAR CONTINUE                      END_PROC
PROC( cdddarCode )      CAR           CDR CDR CDR CONTINUE                      END_PROC
PROC( cdadarCode )      CAR           CDR CAR CDR CONTINUE                      END_PROC
PROC( caadarCode )      CAR           CDR CAR CAR CONTINUE                      END_PROC
PROC( cadaarCode )      CAR           CAR CDR CAR CONTINUE                      END_PROC
PROC( cddaarCode )      CAR           CAR CDR CDR CONTINUE                      END_PROC
PROC( cdaaarCode )      CAR           CAR CAR CDR CONTINUE                      END_PROC
PROC( caaaarCode )      CAR           CAR CAR CAR CONTINUE                      END_PROC

functionDescriptorTypeOne carFD    = { car,          37, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne cdrFD    = { cdr,          53, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne cadrFD   = { cadrCode,     35, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne cddrFD   = { cddrCode,     52, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne caarFD   = { caarCode,     28, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne cdarFD   = { cdarCode,     45, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne caddrFD  = { caddrCode,    34, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne cdddrFD  = { cdddrCode,    51, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne caadrFD  = { caadrCode,    27, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne cdadrFD  = { cdadrCode,    44, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne cadarFD  = { cadarCode,    31, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne cddarFD  = { cddarCode,    48, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne caaarFD  = { caaarCode,    24, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne cdaarFD  = { cdaarCode,    41, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne cadddrFD = { cadddrCode,   33, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne cddddrFD = { cddddrCode,   50, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne caaddrFD = { caaddrCode,   26, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne cdaddrFD = { cdaddrCode,   43, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne cadadrFD = { cadadrCode,   30, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne cddadrFD = { cddadrCode,   47, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne caaadrFD = { caaadrCode,   23, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne cdaadrFD = { cdaadrCode,   40, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne caddarFD = { caddarCode,   32, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne cdddarFD = { cdddarCode,   49, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne caadarFD = { caadarCode,   25, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne cdadarFD = { cdadarCode,   42, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne cadaarFD = { cadaarCode,   29, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne cddaarFD = { cddaarCode,   46, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne caaaarFD = { caaaarCode,   22, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };
functionDescriptorTypeOne cdaaarFD = { cdaaarCode,   39, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };

/**** clearPermanentCode -- Make the given symbol permanent.     ****/

PROC( clearPermanentCode )      

    applicativeCheckDropNone( NameStrings[ 268 ] );

    CALL( clearPermanentSymbol ) 
    RETURN_DEFAULT 
    CONTINUE

END_PROC

functionDescriptorTypeOne clearPermanentFD = { clearPermanentCode, 268, TRUE, symbolP, symbolTag, 1, symbolChecker, symbolCheckerNames };

/**** closedPortPCode -- Is the quantity in R a closed port? ****/

PROC( closedPortPCode ) 

    PORT_P
    JUMP_FALSE( no )
    PORT_NUMBER
    PUSH
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( closedFileIndex, R ) 
    NUM_EQ_P
    SET_R_FROM_B
    DROP
    CONTINUE

  no:
    LOAD_R_TAG( falseBooleanTag )
    CONTINUE

END_PROC

functionDescriptorTypeOne closedPortPFD = { closedPortPCode, 220, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** compileCode -- Do nothing -- this procedure gets overwritten when the real compiler is installed.                ****/

PROC( compileCode )
    CONTINUE
END_PROC

functionDescriptorTypeOne compileCodeFD = { compileCode, 293, TRUE, alwaysTrue, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** compilingP -- Are "define"s supposed to be compiled? ****/

PROC( compilingP )
    SET_R_FROM( CompileDefinesFlag )
    CONTINUE
END_PROC

functionDescriptorTypeOne ccompilingPFD = { compilingP, 294, TRUE, booleanP, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** consCode -- Return in R the pair ( STACK( 0 ) . R ) ****/

functionDescriptorTypeOne consFD = { cons, 76, TRUE, pairP, pairTag, 2, anyTypeBinaryChecker, anyTypeBinaryCheckerNames };

/**** cwCCode -- Cons the given form onto the continuation.  ****/

PROC( cwCCode )

    PUSH_CONTINUATION
    CONTINUE	

END_PROC

functionDescriptorTypeOne cwCFD = { cwCCode, 229, TRUE, alwaysTrue, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** evalInEnvironmentPCode -- R is an environment,  STACK( 0 ) is something to be evaluated therein. ****/

PROC( eIECode )         

    PUSH                                                                      
    MOVE( E, R )                                                              // R: current E; S(0): new E; S(1): To be evaluated.
    EXCHANGE( R, STACK( 1 ) )                                                 // R: To be evaluated; S(0): new E; S(1): current E.

    PUSH  // Make space for manipulating stuff to push into C.

    R.u.c = popToEFunction_index; R.t = schemeMachinePCIndexTag; CSPush( R ); // E gets environment that obtained on entry.
    R.u.c = cwCCode_index;        R.t = schemeMachinePCIndexTag; CSPush( R ); // R gets result of evaluating.
    R.u.c = popToEFunction_index; R.t = schemeMachinePCIndexTag; CSPush( R ); // E gets environment to be used.

    POP                                                                       // R: To be evaluated; S(0): new E; S(1): current E.

    CONTINUE

END_PROC

functionDescriptorTypeOne eIEFD = { eIECode, 315, TRUE, alwaysTrue, trueBooleanTag, 2, anyTypeBinaryChecker, anyTypeBinaryCheckerNames };

/**** cwccCode -- Call the given function with the current continuation, environment and stack. ****/

PROC( cwccCode )                /* R: proc */

    /* first, mucho arity checking */

    PUSH
    CALL( builtInFunctionP )
    JUMP_TRUE( builtInArityCheck )
    MOVE( STACK( 0 ), R )
    CALL( lambdaPairP )
    MOVE( STACK( 0 ), R )
    JUMP_TRUE( lambdaArityCheck )
    POP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 66 );

  builtInArityCheck:
    MOVE( STACK( 0 ), R )
    CALL( getBuiltInType )
    PUSH
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( FIXED_ARITY, R )
    NUM_EQ_P
    JUMP_TRUE( fixedArity )
    LOAD_OBJECT( ANY_ARITY_ONE_TYPE_STACK, R )
    NUM_EQ_P
    JUMP_TRUE( anyArityOneTypeStack )
    LOAD_OBJECT( POSITIVE_ARITY_TWO_TYPES_STACK, R )
    NUM_EQ_P
    JUMP_TRUE( positiveArityTwoTypesStack )
    LOAD_OBJECT( ANY_ARITY_TWO_TYPES_STACK_2, R )
    NUM_EQ_P
    JUMP_TRUE( anyArityTwoTypesStack2 )
    LOAD_OBJECT( ANY_ARITY_TWO_TYPES_STACK, R )
    NUM_EQ_P
    JUMP_TRUE( anyArityTwoTypesStack )
    POP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 67 );

  fixedArity:
    DROP
    MOVE( STACK( 0 ), R )
    CALL( getBuiltInNArgs )
    ONE_FIXNUM_P
    JUMP_FALSE( arityError )
    JUMP_LABEL( OK )

  anyArityOneTypeStack:
    DROP
    MOVE( STACK( 0 ), R )
    CALL( getBuiltInMinArgs )
    PUSH
    MOVE( one, R )
    NUM_LE_P
    JUMP_FALSE( arityError )
    DROP
    JUMP_LABEL( OK )

  lambdaArityCheck:
    MOVE( STACK( 0 ), R )
    LOAD_R_TAG( pairTag )
    PAIR_P
    JUMP_FALSE( OK )
    CAR          
    LENGTH
    ONE_FIXNUM_P
    JUMP_FALSE( arityError )
    JUMP_LABEL( OK )

  positiveArityTwoTypesStack:
  anyArityTwoTypesStack:
  anyArityTwoTypesStack2:
    DROP

  OK:
    POP
    MAKE_FRAME
    LOAD_FRAME( 0, 0 )
    PUSH

    PUSH
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( MiscStrings[ 146 ], R )
    CALL( intern );
    CPushFrame( R );                    // Just added a frame containing just "val", to match the remains of the "apply" later.
    POP

    LOAD_TAG( exactShortFixnumBitsMask, STACK( 0 ) )
    LOAD_OBJECT( CF, STACK( 0 ) )
    CSPush( STACK( 0 ) );
    CALL( saveContinuation );           /* R: proc; A: savedContinuation */
    CSDrop();
    CSDrop();
    MOVE( A, SavedE )                   /* R: proc; SavedE: savedContinuation (temporarily); A: <dont care> */
    LOAD_TAG( exactShortFixnumBitsMask, STACK( 0 ) )
    LOAD_OBJECT( F, STACK( 0 ) )
    CALL( saveStack );                  /* R: proc; SavedE: savedContinuation; A: savedStack */
    DROP                                /* clean up the fake stack frame -- CHANGE THIS CODE FOR NEW FRAME SIZE */
    DROP
//    DROP                                // Excising "C"
    MOVE( (taggedAval *)(OBJECT( STACK( 0 ) )), F )
    DROP                                /* This line added when changing frame size from 4 to 5. */
    MOVE( (taggedAval *)(OBJECT( STACK( 0 ) )), CF )
//    DROP                                /* This line added when changing frame size from 5 to 6. */// Excising NArgs and ArgCounter.
//    DROP                                /* This line added when changing frame size from 6 to 7. */// Excising NArgs and ArgCounter.
    DROP                                /* This line added when changing frame size from 7 to 8. */
    DROP
    DROP
    PUSH                                /* R: proc; SavedE: savedContinuation; A: savedStack; S: proc ... */
    MOVE( SavedE, R )
    LOAD_TAG( emptyListTag, SavedE )
    PUSH                                /* R: savedContinuation; A: savedStack; S: savedContinuation proc ... */
    MOVE( A, R )
    CONS
    NILCONS
    PUSH                                /* R: ((savedContinuation .  savedStack)); S: ((savedContinuation . savedStack)) proc ... */
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( NameStrings[ 281 ], R )
    CALL( intern )                      /* R: 'c::quote-simple';                   S: ((savedContinuation . savedStack)) proc ... */
    SWAP
    CONS                                /* R: (c::quote-simple (savedContinuation . savedStack);  S: proc ... */
    PUSH

    /* build up a lambda pair to be passed to the given procedure */

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( MiscStrings[ 146 ], R )
    CALL( intern )                      /* R: 'val'; S: (c::quote-simple (savedContinuation . savedStack)) proc ... */
    SWAP
    PUSH                                /* R: (c::quote-simple ( . ); S: (c::quote-simple ( . )) 'val' proc ... */
    LOAD_R_TAG( schemeMachinePCIndexTag )  
    LOAD_OBJECT( restoreForCWCC_index, R )
    PUSH
    MOVE( STACK( 2 ), R )               /* R: 'val'; S: restoreForCWCC (c::quote-simple ( . )) 'val' proc ... */
    NILCONS
    CONS
    CONS                                /* R: lambdaBody;       S: 'val' proc ... */

    /* the lambda body is ((c::quote-simple ( . )) restoreForCWCC 'val') */

    SWAP                                /* R: 'val';            S: lambdaBody proc ... */
    NILCONS
    SWAP
    PUSH

    MOVE( E, R )

    NILCONS
    CONS                                /* R: (lambdaBody env);                  S: ('val') proc ... */
    CONS                                /* R: (('val') . (lambdaBody env));      S: proc ... */
    LOAD_R_TAG( lambdaPairTag )         /* R: lambdaPair;                        S: proc ... */

    /* R now contains a lambda pair which is to be called with a single value, and which upon
       being called, will restore the saved state of the world (bound into it as literals)
       and then return the value it was called with, in R. */

    SWAP
    PUSH
    PUSH
    MOVE( one, STACK( 1 ) )             /* R: <don't care>; S: proc 1 lambdaPair ... */

    APPLY
    CONTINUE

arityError:
    resetError( 68 );
    CONTINUE

END_PROC

functionDescriptorTypeOne cwccFD = { cwccCode, 36, TRUE, alwaysTrue, trueBooleanTag, 1, procedureChecker, procedureCheckerNames };

/**** dECode -- Print a useful representation of the symbols in the lexically-smallest
                subenvironment; that is, in the headmost element of the environment list.  ****/

PROC( dECode )

    CALL( printEnvironment )
    RETURN_DEFAULT
    CONTINUE        

END_PROC

functionDescriptorTypeOne dEFD = { dECode, 84, FALSE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** dELCode -- Print a useful representation of all of
                            the symbols in the entire environment list.  ****/

PROC( dELCode )

    CALL( printEnvironmentList )
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne dELFD = { dELCode, 85, FALSE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** dDELCode -- Print a useful representation of all of the symbols in nontoplevel environments.  ****/

PROC( dDELCode )

    CALL( printDynamicEnvironmentList )
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne dDELFD = { dDELCode, 278, FALSE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** describeMemoryCode -- Print a (looonng!) low-level description of everything in main storage.                       ****/

PROC( describeMemoryCode )

    DESCRIBE_SPACE
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne describeMemoryFD = { describeMemoryCode, 87, FALSE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** describeActiveMemoryCode -- Print a (looonng!) low-level description of everything in the active generation    ****/

PROC( describeActiveMemoryCode )

    theHeap->describeCurrentActiveGeneration();
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne describeActiveMemoryFD = { describeActiveMemoryCode, 432, FALSE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** describeAgingMemoryCode -- Print a (looonng!) low-level description of everything in the aging generation     ****/

PROC( describeAgingMemoryCode )

    theHeap->describeCurrentAgingGeneration();
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne describeAgingMemoryFD = { describeAgingMemoryCode, 433, FALSE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** describeLocksCode -- Print a low-level description of everything in main storage that is locked.                       ****/

PROC( describeLocksCode )

    theHeap->describeLocksInCurrentMainSpace();
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne describeLocksFD = { describeLocksCode, 327, FALSE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** releaseLocksCode -- Print a low-level description of everything in main storage that is locked, then release the locks.   ****/

PROC( releaseLocksCode )

    theHeap->describeLocksInCurrentMainSpace();
    theHeap->releaseLocksInCurrentMainSpace();
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne releaseLocksFD = { releaseLocksCode, 337, FALSE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** letDeadlockTimeOutCode -- Allow any deadlock in a given kitten to time out and produce an error. ****/

PROC( letDeadlockTimeOutCode )

    PUSH
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( theNumberOfKittens, R )
    NUM_GE_P
    JUMP_TRUE( error )
    MOVE( zero, R )
    NUM_LT_P
    JUMP_TRUE( error )
    POP
    theClowder->setBreakKittenDeadlockFlag( (Word)(R.u.c) );
    RETURN_DEFAULT
    CONTINUE

  error:
    POP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 108 );    
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne letDeadlockTimeOutFD
    = { letDeadlockTimeOutCode, 338, FALSE, alwaysTrue, trueBooleanTag, 1, exactIntegerChecker, exactIntegerCheckerNames };

/**** describeRoomCode -- Tell how much room is available in main storage. ****/

PROC( describeRoomCode )

    DESCRIBE_ROOM
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne describeRoomFD = { describeRoomCode, 97, FALSE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** describeRoomCode -- Tell how much room is available in main storage. ****/

PROC( describeStackCode )

    DESCRIBE_STACK
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne describeStackFD = { describeStackCode, 121, FALSE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** Displaycode -- Implement the Scheme "display" primitive. ****/

PROC( displayCode )

    ONE_FIXNUM_P
    JUMP_FALSE( onPort )
    POP
    DISPLAY
    RETURN_DEFAULT
    CONTINUE

  onPort:
    POP
    SWAP
    DISPLAY_ON_PORT 
    DROP
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeSix displayFD = { displayCode, 80, FALSE, alwaysTrue, trueBooleanTag, 2, alwaysTrue,
                                        outputPortP, ObjectErrorNameStrings[ 0 ], ObjectErrorNameStrings[ 22 ] };

/**** environmentCode -- Return the current contents of E. ****/

PROC( environmentCode ) 

    MOVE( E, R )
    CONTINUE

END_PROC

functionDescriptorTypeOne environmentFD = { environmentCode, 79, TRUE, properListP, pairTag, 0, anyChecker, anyCheckerNames };

/**** interruptVectorRegisterCode -- Return the current contents of IVR. ****/

PROC( interruptVectorRegisterCode) 

    MOVE( IVR, R )
    CONTINUE

END_PROC

PROC( interruptVectorRegisterCode_OLD ) 

    UnsignedLongWord counter = 0;

    MOVE( numberOfInterrupts, R )
    CALL( makeVectorOfSharpF )
    PUSH                            // R: Copy; Stack: Copy
    for( counter = 0; counter < NUMBER_OF_INTERRUPTS; ++counter ) {
        PUSH                        // Stack: Copy, Copy
        MOVE( IVR, R )
        vectorRefN( counter );      // R: counterth element of IVR; Stack: Copy, Copy
        PUSH                        
        LOAD_R_TAG( exactShortFixnumBitsMask )
        R.u.c = counter;            // R: taggedAval of counter; Stack: counterth element of IVR, Copy
        EXCHANGE( R, STACK( 0 ) )   // R: counterth element of IVR; Stack: taggedAval of counter, Copy
        VSET                        // Stack: Copy
        DUP                         // R: Copy; Stack: Copy
        }    
    POP
    CONTINUE

END_PROC

functionDescriptorTypeOne interruptVectorRegisterFD
    = { interruptVectorRegisterCode, 398, TRUE, properListP, pairTag, 0, anyChecker, anyCheckerNames };

/**** topLevelEnvironmentCode -- Return the current contents of TopLevelE. ****/

PROC( topLevelEnvironmentCode ) 

    MOVE( TopLevelE, R )
    CONTINUE

END_PROC

functionDescriptorTypeOne topLevelEnvironmentFD = { topLevelEnvironmentCode, 312, TRUE, properListP, pairTag, 0, anyChecker, anyCheckerNames };

/**** nullEnvironmentCode -- Return the current contents of NullE. ****/

PROC( nullEnvironmentCode ) 

    MOVE( TopLevelE, R )
    CDR
    CDR
    CONTINUE

END_PROC

functionDescriptorTypeOne nullEnvironmentFD = { nullEnvironmentCode, 313, TRUE, properListP, pairTag, 0, anyChecker, anyCheckerNames };

/**** schemeReportEnvironmentCode -- Return the current contents of SchemeReportE. ****/

PROC( schemeReportEnvironmentCode ) 

    MOVE( TopLevelE, R )
    CDR
    CONTINUE

END_PROC

functionDescriptorTypeOne schemeReportEnvironmentFD = { schemeReportEnvironmentCode, 314, TRUE, properListP, pairTag, 0, anyChecker, anyCheckerNames };

/**** vRefCode -- Return the (0-based) nth element of the given vector.  ****/

PROC( environmentNDownCode )

    UnsignedLongWord n = 0;

    EXACT_P
    JUMP_FALSE( needExact )
    COERCE_R_SHORT_FIXNUM
    PUSH
    MOVE( zero, R )
    NUM_GE_P
    JUMP_FALSE( lowRange )
    NUM_EQ_P
    JUMP_TRUE( zeroDown )
    POP

    n = R.u.c;

    for( R = *(F + FRAME_FP_OFFSET); n-- > 0; R = *(R.u.a + FRAME_FP_OFFSET) )
        ;
    R = *(R.u.a + FRAME_E_OFFSET);

    CONTINUE

  lowRange:
    POP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 288 );
    CONTINUE

  needExact:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 289 );
    CONTINUE

  zeroDown:
    POP
    MOVE( E, R )
    CONTINUE
    

END_PROC

functionDescriptorTypeOne environmentNDownFD = { environmentNDownCode, 308, TRUE, properListP, pairTag, 1, integerChecker, integerCheckerNames };

/**** environmentRefCode -- Return the referenced variable. ****/

PROC( environmentRefCode ) 

    ENV_REF
    CONTINUE

END_PROC

functionDescriptorTypeOne environmentRefFD = { environmentRefCode, 290, TRUE, alwaysTrue, trueBooleanTag, 1, eRefChecker, eRefCheckerNames };

/**** topLevelEnvironmentRefCode -- Return the value of the referenced variable. ****/

PROC( topLevelEnvironmentRefCode ) 

    TOP_LEVEL_ENV_REF
    CONTINUE

END_PROC

functionDescriptorTypeOne topLevelEnvironmentRefFD =
    { topLevelEnvironmentRefCode, 291, TRUE, alwaysTrue, trueBooleanTag, 1, topLevelERefChecker, topLevelERefCheckerNames };

/**** topLevelEnvironmentRefForSymbolCode -- Return the referenced variable -- the symbol, not the value. ****/

PROC( topLevelEnvironmentRefForSymbolCode ) 

    TOP_LEVEL_ENV_REF_SYMBOL
    CONTINUE

END_PROC

functionDescriptorTypeOne topLevelEnvironmentRefForSymbolFD =
    { topLevelEnvironmentRefForSymbolCode, 309, TRUE, alwaysTrue, trueBooleanTag, 1, topLevelERefChecker, topLevelERefCheckerNames };

/**** eofPCode -- Is the quantity in R an end-of-file object? ****/

PROC( eofPCode ) 

    PUSH
    LOAD_R_TAG( peculiarLeafTag )
    LOAD_OBJECT( 0, R ) 
    orIntoLeafByteNonAtomic( R.u.c, eofObjectPLTag );
    EQ_P
    SET_R_FROM_B
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne eofPFD = { eofPCode, 214, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** eqPCode -- Are the contents of R and of STACK( 0 ) eq?  Pops the stack. ****/

PROC( eqPCode )         

    EQ_P
    SET_R_FROM_B
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne eqPFD = { eqPCode, 102, TRUE, booleanP, trueBooleanTag, 2, anyTypeBinaryChecker, anyTypeBinaryCheckerNames };

/**** equalPCode -- Are the contents of R and of STACK( 0 ) equal?  Pops the stack. ****/

PROC( equalPCode )              

    EQUAL_P
    SET_R_FROM_B
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne equalPFD = { equalPCode, 103, TRUE, booleanP, trueBooleanTag, 2, anyTypeBinaryChecker, anyTypeBinaryCheckerNames };

/**** eqvPCode -- Are the contents of R and of STACK( 0 ) eqv?  Pops the stack. ****/

PROC( eqvPCode )                

    EQV_P
    SET_R_FROM_B
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne eqvPFD = { eqvPCode, 104, TRUE, booleanP, trueBooleanTag, 2, anyTypeBinaryChecker, anyTypeBinaryCheckerNames };

/**** expandMacroAux -- Expects R: maybe a macro; S: (macroCall ...).   ****/

PROC( expandMacroAux )

    MACRO_P
    JUMP_FALSE( notMacro )
    LOAD_R_TAG( lambdaPairTag )
    SWAP
    PUSH
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( NameStrings[ 281 ], R )
    CALL( intern )
    SWAP               /* R: macroCall; S:(c::quote-simple macro ...) */
    NILCONS
    CONS                                    /* R: (c::quote-simple macroCall); S:(macro ...) */
    NILCONS
    CONS
    PUSH_CONTINUATION
    CONTINUE

  notMacro:
    POP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 235 );
    CONTINUE

END_PROC

/**** expandMacroCode -- Return what you get by expanding (but not evaluating) the macro call that is passed in R.                               ****/

PROC( expandMacroCode )

    PUSH
    CAR
    PUSH
    LOAD_R_TAG( schemeMachinePCIndexTag )
    LOAD_OBJECT( expandMacroAux_index, R )
    PUSH_CONTINUATION
    POP
    PUSH_CONTINUATION

    /* C: (shouldEvalToMacro testOnWhetherItDid ...) */

    CONTINUE

END_PROC

functionDescriptorTypeOne expandMacroFD = { expandMacroCode, 250, TRUE, alwaysTrue, trueBooleanTag, 1, pairChecker, pairCheckerNames };

/**** forceCode -- Force a promise, merely return anything else. ****/

PROC( forceCode )

    PROMISE_P
    JUMP_TRUE( gotPromise )
    CONTINUE

  gotPromise:
    LOAD_R_TAG( pairTag )
    PUSH                            /* save promise's location for overwriting when forced */
    CDR          
    TRUE_P
    JUMP_TRUE( doForce )
    POP
    CAR                             /* the promise has already been forced, to this value */
    CONTINUE

  doForce:
    MOVE( STACK( 0 ), R )
    CAR          
    MAKE_FRAME
    PUSH                            /* S: (lambdaPair <don't care> <don't care> <don't care> forceLoc ...) */
    PUSH
    MOVE( zero, STACK( 1 ) )        /* R: <don't care>; S: (lambdaPair zero <don't care> <don't care> <don't care> forceLoc ...) */

    /* Stuff continuation with (rebindForce) */

    LOAD_R_TAG( schemeMachinePCIndexTag )
    LOAD_OBJECT( rebindForce_index, R )
    PUSH_CONTINUATION       

    LOAD_FRAME( 2, 0 )              /* R: <don't care>; S: (lambdaPair zero savedE savedC zero forceLoc ...) */

    LOAD_R_TAG( schemeMachinePCIndexTag )
    LOAD_OBJECT( apply_index, R )
    CSPush( R );
//    NILCONS
//    MOVE( R, C )

    CONTINUE

END_PROC

functionDescriptorTypeOne forceFD = { forceCode, 112, TRUE, alwaysTrue, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** forcedPCode -- Has this promise been forced yet? ****/

PROC( forcedPCode) 

    PUSH
    LOAD_R_TAG( pairTag )
    CDR          
    EMPTY_LIST_P
    JUMP_FALSE( notForced )
    JUMP_LABEL( forced )

  notForced:
    POP
    LOAD_R_TAG( falseBooleanTag )
    CONTINUE

  forced:
    POP
    LOAD_R_TAG( trueBooleanTag )
    CONTINUE

END_PROC

functionDescriptorTypeOne forcedPFD = { forcedPCode, 91, TRUE, booleanP, trueBooleanTag, 1, promiseChecker, promiseCheckerNames };

/**** fullGCType2Code -- Do a full garbage-collection. ****/

PROC( fullGCType2Code )

    theClowder->setFullGCIsRequired();
    theClowder->setGCIsRequired();
    theHeap->triggerGC();
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne fullGC2FD = { fullGCType2Code, 92, FALSE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

functionDescriptorTypeOne genSymFD = { genSym, 93, TRUE, symbolP, symbolTag, 0, anyChecker, anyCheckerNames };

/**** getTagCode -- Get the tag of the passed object ****/

PROC( getTagCode )

    LOAD_OBJECT( TAG( R ), R )
    LOAD_R_TAG( exactShortFixnumBitsMask )
    CONTINUE

END_PROC

functionDescriptorTypeOne getTagFD = { getTagCode, 258, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 1, anyChecker, anyCheckerNames };

/**** hashStringCode -- Return the hash of the string. ****/

PROC( hashStringCode ) 

    HASH_STRING
    CONTINUE

END_PROC

functionDescriptorTypeOne hashStringFD = { hashStringCode, 292, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 1, stringChecker, stringCheckerNames };

/**** infPCode -- Is the quantity in R an IEEE infinity? ****/

PROC( infPCode) 

    INF_P
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne infPFD = { infPCode, 221, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** inspectCode -- "Inspect" the passed object; this is an enhancement that prints out
                     more information (and perhaps lower-level information) than does
                     either "display" or "write".                            ****/

PROC( inspectCode )

    ONE_FIXNUM_P
    JUMP_FALSE( onPort )
    MOVE( STACK( 0 ), R )
    INSPECT 
    POP
    CONTINUE

  onPort:
    MOVE( STACK( 0 ), R )
    MOVE( R, STACK( 0 ) )
    MOVE( STACK( 1 ), R )
    INSPECT_ON_PORT 
    DROP
    POP
    CONTINUE

END_PROC

functionDescriptorTypeSix inspectFD = { inspectCode, 94, TRUE, alwaysTrue, trueBooleanTag, 2, alwaysTrue,
                                        outputPortP, ObjectErrorNameStrings[ 0 ], ObjectErrorNameStrings[ 22 ] };

/**** uSleepCode -- Sleep the number of microseconds that is in R. ****/

PROC( uSleepCode )

    theHeap->triggerGC();  // Reduce chances that other processes will spin waiting for GC.
    USLEEP
    eventLoopOnce( 0 );    // If anything has happened while we were sleeping, deal with it promptly.
    RETURN_DEFAULT

END_PROC

functionDescriptorTypeOne uSleepFD = { uSleepCode, 303, FALSE, alwaysTrue, trueBooleanTag, 1, integerChecker, integerCheckerNames };

/**** compilerOnCode ****/

PROC( compilerOnCode )

    CompileDefinesFlag = TRUE;
    reportEvent( COMPILE_DEFINES_EVENT );
    usleep( 3 * STANDARD_TIMER_INTERVAL_US );   // Allow event to propagate back through to the evaluator thread.
    eventLoopOnce( 0 );                         // Allow the new event to register.
    RETURN_DEFAULT 
    CONTINUE

END_PROC

functionDescriptorTypeOne compilerOnFD = { compilerOnCode, 317, FALSE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** compilerOffCode ****/

PROC( compilerOffCode )

    CompileDefinesFlag = FALSE;
    reportEvent( DO_NOT_COMPILE_DEFINES_EVENT );
    usleep( 3 * STANDARD_TIMER_INTERVAL_US );   // Allow event to propagate back through to the evaluator thread.
    eventLoopOnce( 0 );                         // Allow the new event to register.
    RETURN_DEFAULT 
    CONTINUE

END_PROC

functionDescriptorTypeOne compilerOffFD = { compilerOffCode, 318, FALSE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** fullPrecisionOnCode ****/

PROC( fullPrecisionOnCode )

    useFullNumericPrecision = TRUE;
    reportEvent( USE_HIGH_PRECISION_EVENT );
    usleep( 3 * STANDARD_TIMER_INTERVAL_US );   // Allow event to propagate back through to the evaluator thread.
    eventLoopOnce( 0 );                         // Allow the new event to register.
    RETURN_DEFAULT 
    CONTINUE

END_PROC

functionDescriptorTypeOne fullPrecisionOnFD = { fullPrecisionOnCode, 319, FALSE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** fullPrecisionOffCode ****/

PROC( fullPrecisionOffCode )

    useFullNumericPrecision = FALSE;
    reportEvent( USE_LOW_PRECISION_EVENT );
    usleep( 3 * STANDARD_TIMER_INTERVAL_US );   // Allow event to propagate back through to the evaluator thread.
    eventLoopOnce( 0 );                         // Allow the new event to register.
    RETURN_DEFAULT 
    CONTINUE

END_PROC

functionDescriptorTypeOne fullPrecisionOffFD = { fullPrecisionOffCode, 320, FALSE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** compilingPCode ****/

PROC( compilingPCode )

    SET_R_FROM( CompileDefinesFlag )
    CONTINUE

END_PROC

functionDescriptorTypeOne compilingPFD = { compilingPCode, 321, TRUE, booleanP, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** fullPrecisionPCode ****/

PROC( fullPrecisionPCode )

    SET_R_FROM( useFullNumericPrecision )
    CONTINUE

END_PROC

functionDescriptorTypeOne fullPrecisionPFD = { fullPrecisionPCode, 322, TRUE, booleanP, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** momcatPCode ****/

PROC( momcatPCode )

    SET_R_FROM( isMomCatP )
    CONTINUE

END_PROC

functionDescriptorTypeOne momcatPFD = { momcatPCode, 323, TRUE, booleanP, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** kittenNumberCode ****/

PROC( kittenNumberCode )

    LOAD_R_TAG( exactShortFixnumBitsMask )     
    LOAD_OBJECT( theKittenNumber, R )
    CONTINUE

END_PROC

functionDescriptorTypeOne kittenNumberFD = { kittenNumberCode, 324, FALSE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** maxKittenNumberCode ****/

PROC( maxKittenNumberCode )

    LOAD_R_TAG( exactShortFixnumBitsMask )     
    LOAD_OBJECT( theNumberOfKittens, R )
    PUSH
    MOVE( one, R )
    SUBTRACT
    CONTINUE

END_PROC

functionDescriptorTypeOne maxKittenNumberFD = { maxKittenNumberCode, 325, FALSE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** showKittensCode ****/

PROC( showKittensCode )

    UnsignedWord i = 0;

    tprintf2( &CurrentOutputPort, "  Who State  ObList Message  Message  Block  Process\n");
    tprintf2( &CurrentOutputPort, "             In Reg   From     From    Number    ID  \n");
    tprintf2( &CurrentOutputPort, "                      Mom      Kitten          Number\n");
    tprintf2( &CurrentOutputPort, "\n");
    for( i = 0; i < theNumberOfKittens; ++i ) {
        if( i == 0 )
            tprintf2( &CurrentOutputPort, "Mom:  " );
        else
            tprintf3( &CurrentOutputPort, "Kit %d:", i );
        tprintf4( &CurrentOutputPort, "    %d     %d  ",
                  theClowder->getKittenEvaluatorActivity( i ),
                  theClowder->getObListInRegister( i )
                  );
        tprintf6( &CurrentOutputPort, "%6d   %6d   %6ju   %6d\n",
                  theClowder->getMomSays( i ),
                  theClowder->getKittenSays( i ),
                  (intmax_t)theClowder->getKittenBlock( i ),
                  theClowder->getKittenPID( i )
                  );
        }
   
    RETURN_DEFAULT 
    CONTINUE

END_PROC

functionDescriptorTypeOne showKittensFD = { showKittensCode, 326, FALSE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** getPIDCode ****/

PROC( getPIDCode )
    

    R.u.c =(theClowder->getKittenPID( theKittenNumber ));
    R.t = exactShortFixnumBitsMask;

    CONTINUE

END_PROC

functionDescriptorTypeOne getPIDFD = { getPIDCode, 336, FALSE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** nonPrintingObjectPCode -- Is the quantity in R a non-printing object? ****/

PROC( nonPrintingObjectPCode ) 

    NON_PRINTING_OBJECT_P
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne nonPrintingObjectPFD = { nonPrintingObjectPCode, 439, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

