/****************************************************************************************

SchemeFunctions3.c -- Copyright (c) by Jay Reynolds Freeman 1988, 1989, 1990, 1991, 2006

Scheme built-in functions for Wraith, and all code that needs to know about the
"functionDescriptor" structures.  This is file no. 3 of several ...

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import "WraithHeader.h"
#import "MemoryManager.h"
#import "OpCodes.h"
#import "SchemeFunctions.h"
#import "DataTypes.e"
#import "Globals.e"
#import "Intern.e"
#import "IO.e"
#import "SchemeAux.e"
#import "SchemeFunctionStatics.h"

/******** Stuff for Scheme built-in functions; code and associated statics ********/

/**** acosCode -- Return the acos of the number in R.   ****/

functionDescriptorTypeOne acosFD = { acosMC, 11, TRUE, numberP, numberBitsMask, 1, numberChecker, numberCheckerNames };

/**** asinCode -- Return the asin of the number in R.   ****/

functionDescriptorTypeOne asinFD = { asinMC, 15, TRUE, numberP, numberBitsMask, 1, numberChecker, numberCheckerNames };

/**** addCode -- Add the numbers stacked and return the sum,
                 except that if there are no numbers, return zero.                           ****/

PROC( addCode )

    ZERO_FIXNUM_P
    JUMP_FALSE( oneOrMore )
    CONTINUE

  oneOrMore:
    ONE_FIXNUM_P
    JUMP_TRUE( noMore )
    EXCHANGE( R, STACK( 1 ) )
    ADD
    SWAP
    DECREMENT_R
    JUMP_LABEL( oneOrMore )

  noMore:
    POP
    CONTINUE

END_PROC

functionDescriptorTypeFive addFD = { addCode, 2, TRUE, numberP, numberBitsMask, 0, numberP, ObjectErrorNameStrings[ 18 ] };

/**** bitAndCode ****/

PROC( bitAndCode )

    COERCE_R_SHORT_FIXNUM
    COERCE_S0_SHORT_FIXNUM
    BIT_AND
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne bitAndFD = { bitAndCode, 282, TRUE, numberP, numberBitsMask, 2, signedFixnumBinaryChecker, signedFixnumBinaryCheckerNames };

/**** bitNotCode ****/

PROC( bitNotCode )

    COERCE_R_SHORT_FIXNUM
    BIT_NOT
    CONTINUE

END_PROC

functionDescriptorTypeOne bitNotFD = { bitNotCode, 283, TRUE, numberP, numberBitsMask, 1, signedFixnumChecker, signedFixnumCheckerNames };

/**** bitOrCode ****/

PROC( bitOrCode )

    COERCE_R_SHORT_FIXNUM
    COERCE_S0_SHORT_FIXNUM
    BIT_OR
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne bitOrFD = { bitOrCode, 284, TRUE, numberP, numberBitsMask, 2, signedFixnumBinaryChecker, signedFixnumBinaryCheckerNames };

/**** bitXorCode ****/

PROC( bitXorCode )

    COERCE_R_SHORT_FIXNUM
    COERCE_S0_SHORT_FIXNUM
    BIT_XOR
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne bitXorFD = { bitXorCode, 285, TRUE, numberP, numberBitsMask, 2, signedFixnumBinaryChecker, signedFixnumBinaryCheckerNames };

/**** bitShiftLeftCode ****/

PROC( bitShiftLeftCode )

    COERCE_R_SHORT_FIXNUM
    COERCE_S0_SHORT_FIXNUM
    BIT_SHIFT_LEFT
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne bitShiftLeftFD
    = { bitShiftLeftCode, 286, TRUE, numberP, numberBitsMask, 2, signedFixnumUnsignedFixnumChecker, signedFixnumUnsignedFixnumCheckerNames };

/**** bitShiftRightArithmeticCode ****/

PROC( bitShiftRightArithmeticCode )

    COERCE_R_SHORT_FIXNUM
    COERCE_S0_SHORT_FIXNUM
    BIT_SHIFT_RIGHT_ARITHMETIC
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne bitShiftRightArithmeticFD
    = { bitShiftRightArithmeticCode, 287, TRUE, numberP, numberBitsMask, 2, signedFixnumUnsignedFixnumChecker, signedFixnumUnsignedFixnumCheckerNames };

/**** bitShiftRightLogicalCode ****/

PROC( bitShiftRightLogicalCode )

    COERCE_R_SHORT_FIXNUM
    COERCE_S0_SHORT_FIXNUM
    BIT_SHIFT_RIGHT_LOGICAL
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne bitShiftRightLogicalFD
    = { bitShiftRightLogicalCode, 288, TRUE, numberP, numberBitsMask, 2, signedFixnumUnsignedFixnumChecker, signedFixnumUnsignedFixnumCheckerNames };

/**** ceilingCode -- Return the ceiling of the number in R. ****/

PROC( ceilingCode )

    FIXNUM_P
    JUMP_TRUE( done )
    CEILING 
  done:
    CONTINUE

END_PROC

functionDescriptorTypeOne ceilingFD =  { ceilingCode, 54, TRUE, integerP, numberBitsMask, 1, numberChecker, numberCheckerNames };

/**** complexPCode -- Is the number in R complex? ****/

PROC( complexPCode )

    NAN_OR_INF_P
    JUMP_FALSE( maybe )
    LOAD_R_TAG( falseBooleanTag )
    CONTINUE

  maybe:
    COMPLEX_P 
    SET_R_FROM_B 
    CONTINUE

END_PROC

functionDescriptorTypeOne complexPFD = { complexPCode, 74, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/******** Stuff for Scheme built-in functions; code and associated statics ********/

/**** cosCode -- Return the cos of the number in R.     ****/

functionDescriptorTypeOne cosFD = { cosMC, 77, TRUE, numberP, numberBitsMask, 1, numberChecker, numberCheckerNames };

/**** divideCode -- STACK( 0 ) is a list:  If it contains only one element,
                    return the quotient of
                    the elements in the list, associating to the left.
                    (Thus if the list is (a b c d), consider the infix
                    expression (a / b / c / d), associate as (((a / b) / c) /d)
                    and re-express in prefix notation as (/ (/ (/ a b) c) d)         ****/

PROC( divideCode )

    ONE_FIXNUM_P
    JUMP_FALSE( twoOrMore )
    SWAP
    DIVIDE
    CONTINUE

  twoOrMore:
    REVERSE_TOP_N

  twoOrMoreLoop:
    EXCHANGE( R, STACK( 1 ) )
    DIVIDE
    SWAP
    DECREMENT_R
    ONE_FIXNUM_P
    JUMP_TRUE( noMore )
    JUMP_LABEL( twoOrMoreLoop )

  noMore:
    POP
    CONTINUE

END_PROC

functionDescriptorTypeSix divideFD = { divideCode, 4, TRUE, numberP, numberBitsMask, -1,
                                       numberP, numberP, ObjectErrorNameStrings[ 18 ], ObjectErrorNameStrings[ 18 ] };

/**** evenPCode -- Is the integer in R even? ****/

PROC( evenPCode )       

    COERCE_R_SHORT_FIXNUM
    EVEN_P 
    SET_R_FROM_B 
    CONTINUE

END_PROC

functionDescriptorTypeOne evenPFD = { evenPCode, 105, TRUE, booleanP, trueBooleanTag, 1, integerChecker, integerCheckerNames };

/**** expCode -- Return the exp of the number in R.     ****/

functionDescriptorTypeOne expFD = { expMC, 108, TRUE, numberP, numberBitsMask, 1, numberChecker, numberCheckerNames };

/**** exptCode -- Return STACK( 0 ) raised to the power R.      ****/

PROC( exptCode )

    INTEGER_P
    JUMP_FALSE( moreTests )
  goAhead:
    EXPT
    DROP
    CONTINUE

  moreTests:
    SWAP
    PUSH
    MOVE( zero, R )
    NUM_LT_P
    JUMP_TRUE( complexResult )
    POP
    SWAP
    JUMP_LABEL( goAhead )

  complexResult:
    POP
    OUTPUT_TO_ERROR
    SWAP
    DISPLAY
    WRITE_NEWLINE_ERROR
    POP
    DISPLAY
    resetError( 69 );
    CONTINUE

END_PROC

functionDescriptorTypeOne exptFD = { exptCode, 109, TRUE, numberP, numberBitsMask, 2, numberBinaryChecker, numberBinaryCheckerNames };

/**** exactPCode -- Is the number in R exact? ****/

PROC( exactPCode )      

    EXACT_P 
    SET_R_FROM_B 
    CONTINUE

END_PROC

functionDescriptorTypeOne exactPFD = { exactPCode, 107, TRUE, booleanP, trueBooleanTag, 1, numberChecker, numberCheckerNames };

/**** exactToInexactCode -- Change representation of the number in R as if
                            it were inexact.  This action, of course, does not
                            actually make the number inexact                                ****/

PROC( exactToInexactCode )      

    LOAD_R_TAG( (R.t) & (~(exactBitMask)) )
    CONTINUE

END_PROC

functionDescriptorTypeOne exactToInexactFD = { exactToInexactCode, 106, TRUE, numberP, numberBitsMask, 1, numberChecker, numberCheckerNames };

/**** fixnumPCode -- Is the number in R a fixnum?  ("Fixnum" is not a Scheme numeric type.) ****/

PROC( fixnumPCode )             

    FIXNUM_P 
    SET_R_FROM_B 
    CONTINUE

END_PROC

functionDescriptorTypeOne fixnumPFD = { fixnumPCode, 89, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** floatPCode -- Is the number in R a float?  ("Float" is not a Scheme numeric type.) ****/

PROC( floatPCode )              

    FLOAT_P 
    SET_R_FROM_B 
    CONTINUE

END_PROC

functionDescriptorTypeOne floatPFD = { floatPCode, 90, TRUE, numberP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** floorCode -- Return the floor of the number in R. ****/

PROC( floorCode )

    FIXNUM_P
    JUMP_TRUE( done )
    FLOOR 
  done:
    CONTINUE

END_PROC

functionDescriptorTypeOne floorFD = { floorCode, 110, TRUE, integerP, numberBitsMask, 1, numberChecker, numberCheckerNames };

/**** GCDCode -- The arguments are stacked.  Return their gcd and pop the stack.  ****/

PROC( GCDCode )

    ZERO_FIXNUM_P
    JUMP_TRUE( noArgs )

  testAndDoMore:                  /* R: numberToGo;                       S: (number* ) */
    ONE_FIXNUM_P
    JUMP_TRUE( allDone )
    DECREMENT_R
    EXCHANGE( R, STACK( 1 ) )
    GCD
    SWAP
    JUMP_LABEL( testAndDoMore )

  allDone:
    POP
    CONTINUE

  noArgs:
    CONTINUE

END_PROC

functionDescriptorTypeFive GCDFD = { GCDCode, 224, TRUE, integerP, numberBitsMask, 0,
                                     rangeCheckAsSignedFixnumP, ObjectErrorNameStrings[ 26 ] };

/**** inexactPCode -- Is the number in R inexact? ****/

PROC( inexactPCode )    

    INEXACT_P 
    SET_R_FROM_B 
    CONTINUE

END_PROC

functionDescriptorTypeOne inexactPFD = { inexactPCode, 115, TRUE, booleanP, trueBooleanTag, 1, numberChecker, numberCheckerNames };

/**** inexactToExactCode -- Change representation of the number in R as if
                            it were exact.  This action, of course, does not
                            actually make the number exact ...                                 ****/

PROC( inexactToExactCode )      

    LOAD_R_TAG( (R.t) | exactBitMask )
    CONTINUE

END_PROC

functionDescriptorTypeOne inexactToExactFD = { inexactToExactCode, 114, TRUE, numberP, numberBitsMask, 1, numberChecker, numberCheckerNames };

/**** integerPCode -- Is the number in R an integer? ****/

PROC( integerPCode )    

    NAN_OR_INF_P
    JUMP_FALSE( maybe )
    LOAD_R_TAG( falseBooleanTag )
    CONTINUE

  maybe:
    INTEGER_P 
    SET_R_FROM_B 
    CONTINUE

END_PROC

functionDescriptorTypeOne integerPFD = { integerPCode, 117, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** LCMCode -- The arguments are stacked. Return their lcm and pop the stack.  ****/

PROC( LCMCode )

    ZERO_FIXNUM_P
    JUMP_TRUE( noArgs )

  testAndDoMore:                  /* R: numberToGo;                       S: (number* ) */
    ONE_FIXNUM_P
    JUMP_TRUE( allDone )
    DECREMENT_R
    EXCHANGE( R, STACK( 1 ) )
    LCM
    SWAP
    JUMP_LABEL( testAndDoMore )

  allDone:
    POP
    CONTINUE

  noArgs:
    MOVE( one, R )
    CONTINUE

END_PROC

functionDescriptorTypeFive LCMFD = { LCMCode, 225, TRUE, integerP, numberBitsMask, 0,
                                     rangeCheckAsSignedFixnumP, ObjectErrorNameStrings[ 26 ] };

/**** logCode -- Return the log of the number in R.     ****/

functionDescriptorTypeOne logFD =       { logMC, 129, TRUE, numberP, numberBitsMask, 1, numberChecker, numberCheckerNames };

/**** shortFloatPCode -- Is the number in R a short float?  ("Short Float" is not a Scheme numeric type.) ****/

PROC( shortFloatPCode )          

    FLOAT_P 
    JUMP_FALSE( notAShortFloat ) 
    LONG_P
    SET_R_FROM_B
    CONTINUE

  notAShortFloat:
    LOAD_R_TAG( falseBooleanTag )
    CONTINUE

END_PROC

functionDescriptorTypeOne shortFloatPFD = { shortFloatPCode, 269, TRUE, numberP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** moduloCode -- Return STACK( 0 ) modulo R.  Pop the stack.
                    Do not check for division by zero.                                       ****/

PROC( moduloCode )

    SWAP
    PUSH
    MOVE( STACK( 1 ), R )
    REMAINDER
    PUSH
    MOVE( zero, R )
    NUM_EQ_P
    JUMP_TRUE( remainderZero )
    NUM_GE_P
    JUMP_TRUE( remainderPositive )

//  remainderNegative:
    POP
    SWAP
    PUSH        
    MOVE( zero, R )
    NUM_GE_P
    JUMP_TRUE( signsDifferent )
    JUMP_LABEL( signsSame )

  remainderPositive:
    POP
    SWAP
    PUSH        
    MOVE( zero, R )
    NUM_GE_P
    JUMP_TRUE( signsSame )

  signsDifferent:
    POP
    ADD
    CONTINUE

  signsSame:
    DROP
    POP
    CONTINUE

  remainderZero:
    POP
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne moduloFD = { moduloCode, 138, TRUE, integerP, numberBitsMask, 2, signedFixnumBinaryChecker, signedFixnumBinaryCheckerNames };

/**** multiplyCode -- Multiply the numbers in STACK( 0 ) and return the
                      product, except that if STACK( 0 ) is empty, return one. ****/

PROC( multiplyCode )

    ZERO_FIXNUM_P
    JUMP_FALSE( oneOrMore )
    MOVE( one, R )
    CONTINUE

 oneOrMore:
    ONE_FIXNUM_P
    JUMP_TRUE( noMore )
    EXCHANGE( R, STACK( 1 ) )
    MULTIPLY
    SWAP
    DECREMENT_R
    JUMP_LABEL( oneOrMore )

  noMore:
    POP
    CONTINUE

END_PROC

functionDescriptorTypeFive multiplyFD = { multiplyCode, 1, TRUE, numberP, numberBitsMask, 0, numberP, ObjectErrorNameStrings[ 18 ] };

/****  numAbsCode -- Return the absolute value of the number in R. ****/

PROC( numAbsCode )

    PUSH
    MOVE( zero, R )
    NUM_GE_P
    JUMP_TRUE( nonNegative )
    SWAP
    SUBTRACT
    CONTINUE

  nonNegative:
    POP
    CONTINUE

END_PROC

functionDescriptorTypeOne numAbsFD = { numAbsCode, 10, TRUE, numberP, numberBitsMask, 1, numberChecker, numberCheckerNames };

/**** numEqualCode -- Are the numbers stacked all equal? ****/

PROC( numEqualCode )

  testAndDoMore:
    ONE_FIXNUM_P
    JUMP_TRUE( itsTrue )
    EXCHANGE( R, STACK( 1 ) )
    NUM_EQ_P
    JUMP_FALSE( itsFalse )
    DROP
    SWAP
    DECREMENT_R
    JUMP_LABEL( testAndDoMore )

  itsTrue:
    DROP
    LOAD_R_TAG( trueBooleanTag )
    CONTINUE

  itsFalse:
    DROP
    POP
    DECREMENT_R
    DECREMENT_R

  dropLoop:
    ZERO_FIXNUM_P
    JUMP_TRUE( allDone )
    DROP
    DECREMENT_R
    JUMP_LABEL( dropLoop )

  allDone:
    LOAD_R_TAG( falseBooleanTag )
    CONTINUE

END_PROC

functionDescriptorTypeFive numEqualPFD = { numEqualCode, 7, TRUE, booleanP, trueBooleanTag, 2,
                                           numberP, ObjectErrorNameStrings[ 18 ] };

/**** numGreaterCode -- Are the numbers stacked monotone absolutely descending? ****/

PROC( numGreaterCode )

  testAndDoMore:
    ONE_FIXNUM_P
    JUMP_TRUE( itsTrue )
    EXCHANGE( R, STACK( 1 ) )
    NUM_LT_P
    JUMP_FALSE( itsFalse )
    DROP
    SWAP
    DECREMENT_R
    JUMP_LABEL( testAndDoMore )

  itsTrue:
    DROP
    LOAD_R_TAG( trueBooleanTag )
    CONTINUE

  itsFalse:
    DROP
    POP
    DECREMENT_R
    DECREMENT_R

  dropLoop:
    ZERO_FIXNUM_P
    JUMP_TRUE( allDone )
    DROP
    DECREMENT_R
    JUMP_LABEL( dropLoop )

  allDone:
    LOAD_R_TAG( falseBooleanTag )
    CONTINUE

END_PROC

functionDescriptorTypeFive numGreaterPFD = { numGreaterCode, 8, TRUE, booleanP, trueBooleanTag, 2,
                                             numberP, ObjectErrorNameStrings[ 18 ] };

/**** numGreaterEqualCode -- Are the numbers stacked monotone descending? ****/

PROC( numGreaterEqualCode )

  testAndDoMore:
    ONE_FIXNUM_P
    JUMP_TRUE( itsTrue )
    EXCHANGE( R, STACK( 1 ) )
    NUM_LE_P
    JUMP_FALSE( itsFalse )
    DROP
    SWAP
    DECREMENT_R
    JUMP_LABEL( testAndDoMore )

  itsTrue:
    DROP
    LOAD_R_TAG( trueBooleanTag )
    CONTINUE

  itsFalse:
    DROP
    POP
    DECREMENT_R
    DECREMENT_R

  dropLoop:
    ZERO_FIXNUM_P
    JUMP_TRUE( allDone )
    DROP
    DECREMENT_R
    JUMP_LABEL( dropLoop )

  allDone:
    LOAD_R_TAG( falseBooleanTag )
    CONTINUE

END_PROC

functionDescriptorTypeFive numGreaterEqualPFD = { numGreaterEqualCode, 9, TRUE, booleanP, trueBooleanTag, 2,
                                                  numberP, ObjectErrorNameStrings[ 18 ] };

/**** numLessCode -- Are the numbers stacked monotone absolutely increasing? ****/

PROC( numLessCode )

  testAndDoMore:
    ONE_FIXNUM_P
    JUMP_TRUE( itsTrue )
    EXCHANGE( R, STACK( 1 ) )
    NUM_GT_P
    JUMP_FALSE( itsFalse )
    DROP
    SWAP
    DECREMENT_R
    JUMP_LABEL( testAndDoMore )

  itsTrue:
    DROP
    LOAD_R_TAG( trueBooleanTag )
    CONTINUE

  itsFalse:
    DROP
    POP
    DECREMENT_R
    DECREMENT_R

  dropLoop:
    ZERO_FIXNUM_P
    JUMP_TRUE( allDone )
    DROP
    DECREMENT_R
    JUMP_LABEL( dropLoop )

  allDone:
    LOAD_R_TAG( falseBooleanTag )
    CONTINUE

END_PROC

functionDescriptorTypeFive numLessPFD = { numLessCode, 5, TRUE, booleanP, trueBooleanTag, 2,
                                          numberP, ObjectErrorNameStrings[ 18 ] };

/**** numLessEqualCode -- Are the numbers stacked monotone increasing? ****/

PROC( numLessEqualCode )

  testAndDoMore:
    ONE_FIXNUM_P
    JUMP_TRUE( itsTrue )
    EXCHANGE( R, STACK( 1 ) )
    NUM_GE_P
    JUMP_FALSE( itsFalse )
    DROP
    SWAP
    DECREMENT_R
    JUMP_LABEL( testAndDoMore )

  itsTrue:
    DROP
    LOAD_R_TAG( trueBooleanTag )
    CONTINUE

  itsFalse:
    DROP
    POP
    DECREMENT_R
    DECREMENT_R

  dropLoop:
    ZERO_FIXNUM_P
    JUMP_TRUE( allDone )
    DROP
    DECREMENT_R
    JUMP_LABEL( dropLoop )

  allDone:
    LOAD_R_TAG( falseBooleanTag )
    CONTINUE

END_PROC

functionDescriptorTypeFive numLessEqualPFD = { numLessEqualCode, 6, TRUE, booleanP, trueBooleanTag, 2,
                                               numberP, ObjectErrorNameStrings[ 18 ] };

/**** numMaxCode -- Return the maximum of the numbers stacked. ****/

PROC( numMaxCode )

    Boolean allExact = TRUE;

    SWAP
    EXACT_P
    allExact &= B;
    SWAP

  testAndDoMore:
    ONE_FIXNUM_P
    JUMP_TRUE( noMore )
    EXCHANGE( R, STACK( 1 ) )
    EXACT_P
    allExact &= B;
    NUM_GE_P
    JUMP_TRUE( maxOnStack )
    DROP
    SWAP
    DECREMENT_R
    JUMP_LABEL( testAndDoMore )

  maxOnStack:
    POP
    SWAP
    DECREMENT_R
    JUMP_LABEL( testAndDoMore )

  noMore:
    POP
    EXACT_P
    B &= allExact;
    SET_EXACT_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeFive numMaxFD = { numMaxCode, 133, TRUE, numberP, numberBitsMask, 2,
                                        numberP, ObjectErrorNameStrings[ 18 ] };

/**** numMinCode -- Return the minimum of the numbers stacked. ****/

PROC( numMinCode )

    Boolean allExact = TRUE;

    SWAP
    EXACT_P
    allExact &= B;
    SWAP

  testAndDoMore:
    ONE_FIXNUM_P
    JUMP_TRUE( noMore )
    EXCHANGE( R, STACK( 1 ) )
    EXACT_P
    allExact &= B;
    NUM_LE_P
    JUMP_TRUE( minOnStack )
    DROP
    SWAP
    DECREMENT_R
    JUMP_LABEL( testAndDoMore )

  minOnStack:
    POP
    SWAP
    DECREMENT_R
    JUMP_LABEL( testAndDoMore )

  noMore:
    POP
    EXACT_P
    B &= allExact;
    SET_EXACT_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeFive numMinFD = { numMinCode, 137, TRUE, numberP, numberBitsMask, 2,
                                        numberP, ObjectErrorNameStrings[ 18 ] };

/**** numNegativePCode -- Is the real in R less than zero? ****/

PROC( numNegativePCode )

    PUSH
    MOVE( zero, R )
    NUM_LT_P 
    SET_R_FROM_B
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne numNegativePFD = { numNegativePCode, 139, TRUE, booleanP, trueBooleanTag, 1, realChecker, realCheckerNames };

/**** numPositivePCode -- Is the real in R greater than zero? ****/

PROC( numPositivePCode )

    PUSH
    MOVE( zero, R )
    NUM_GT_P 
    SET_R_FROM_B
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne numPositivePFD = { numPositivePCode, 147, TRUE, booleanP, trueBooleanTag, 1, realChecker, realCheckerNames };

/**** numberPCode -- Is the thing in R a number? ****/

PROC( numberPCode )

    NAN_OR_INF_P
    JUMP_FALSE( maybe )
    LOAD_R_TAG( falseBooleanTag )
    CONTINUE

  maybe:
    NUMBER_P 
    SET_R_FROM_B 
    CONTINUE

END_PROC

functionDescriptorTypeOne numberPFD = { numberPCode, 143, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** oddPCode -- Is the integer in R odd? ****/

PROC( oddPCode )        

    COERCE_R_SHORT_FIXNUM
    ODD_P 
    SET_R_FROM_B 
    CONTINUE

END_PROC

functionDescriptorTypeOne oddPFD = { oddPCode, 144, TRUE, booleanP, trueBooleanTag, 1, integerChecker, integerCheckerNames };

/**** quotientCode -- Return STACK( 0 ) div R.  Pop the stack.
                      Do not check for division by zero.                                 ****/

PROC( quotientCode )

    QUOTIENT
    CONTINUE

END_PROC

functionDescriptorTypeOne quotientFD = { quotientCode, 151, TRUE, integerP, numberBitsMask, 2, signedFixnumBinaryChecker, signedFixnumBinaryCheckerNames };

/**** rationalPCode -- Is the number in R a rational? ****/

PROC( rationalPCode )   

    NAN_OR_INF_P
    JUMP_FALSE( maybe )
    LOAD_R_TAG( falseBooleanTag )
    CONTINUE

  maybe:
    RATIONAL_P 
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne rationalPFD = { rationalPCode, 152, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** realPCode -- Is the number in R a real? ****/

PROC( realPCode )       

    NAN_OR_INF_P
    JUMP_FALSE( maybe )
    LOAD_R_TAG( falseBooleanTag )
    CONTINUE

  maybe:
    REAL_P
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne realPFD = { realPCode, 153, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** remainderCode -- Return the remainder of STACK( 0 ) div R.  Pop the stack.
                       Do not check for division by zero.                                            ****/

PROC( remainderCode )

    REMAINDER
    CONTINUE

END_PROC

functionDescriptorTypeOne remainderFD = { remainderCode, 154, TRUE, integerP, numberBitsMask, 2, signedFixnumBinaryChecker, signedFixnumBinaryCheckerNames };

/**** roundCode -- Return the round of the number in R. ****/

PROC( roundCode )

    FIXNUM_P
    JUMP_TRUE( done )
    ROUND 
  done:
    CONTINUE

END_PROC

functionDescriptorTypeOne roundFD = { roundCode, 156, TRUE, integerP, numberBitsMask, 1, numberChecker, numberCheckerNames };

/**** sinCode -- Return the sin of the number in R.     ****/

functionDescriptorTypeOne sinFD = { sinMC, 160, TRUE, numberP, numberBitsMask, 1, numberChecker, numberCheckerNames };

/**** sqrtCode -- Return the sqrt of the number in R.   ****/

functionDescriptorTypeOne sqrtFD = { sqrtMC, 161, TRUE, numberP, numberBitsMask, 1, numberChecker, numberCheckerNames };

/**** subtractCode -- STACK( 0 ) is a list:  If it contains only one element,
                      return its negation; otherwise return the difference of
                      the elements in the list, associating to the left.
                      (Thus if the list is (a b c d), consider the infix
                      expression (a - b - c - d), associate as (((a - b) - c) -d)
                      and re-express in prefix notation as (- (- (- a b) c) d)               ****/

PROC( subtractCode )

    ONE_FIXNUM_P
    JUMP_FALSE( twoOrMore )
    MOVE( zero, R )
    SWAP
    SUBTRACT
    CONTINUE

  twoOrMore:
    REVERSE_TOP_N

  twoOrMoreLoop:
    EXCHANGE( R, STACK( 1 ) )
    SUBTRACT
    SWAP
    DECREMENT_R
    ONE_FIXNUM_P
    JUMP_TRUE( noMore )
    JUMP_LABEL( twoOrMoreLoop )

  noMore:
    POP
    CONTINUE

END_PROC

functionDescriptorTypeSix subtractFD = { subtractCode, 3, TRUE, numberP, numberBitsMask, -1,
                                         numberP, numberP, ObjectErrorNameStrings[ 18 ], ObjectErrorNameStrings[ 18 ] };

/**** tanCode -- Return the tan of the number in R.     ****/

functionDescriptorTypeOne tanFD = { tanMC, 185, TRUE, numberP, numberBitsMask, 1, numberChecker, numberCheckerNames };

/**** truncateCode -- Return the truncate of the number in R. ****/

PROC( truncateCode )

    FIXNUM_P
    JUMP_TRUE( done )
    TRUNCATE 
  done:
    CONTINUE

END_PROC

functionDescriptorTypeOne truncateFD = { truncateCode, 186, TRUE, integerP, numberBitsMask, 1, numberChecker, numberCheckerNames };

/**** zeroPCode -- Is the number in R zero? ****/

PROC( zeroPCode )

    PUSH
    MOVE( zero, R )
    NUM_EQ_P
    SET_R_FROM_B
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne zeroPFD = { zeroPCode, 197, TRUE, booleanP, trueBooleanTag, 1, numberChecker, numberCheckerNames };
