/****************************************************************************************

SchemeFunctions4.c++ -- Copyright (c) by Jay Reynolds Freeman 1988, 1989, 1990, 1991, 2006

Scheme built-in functions for Wraith Scheme.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import "WraithHeader.h"
#import "MemoryManager.h"
#import "OpCodes.h"
#import "SchemeFunctions.h"
#import "DataTypes.e"
#import "Globals.e"
#import "Intern.e"
#import "IO.e"
#import "MainAux.e"
#import "SchemeAux.e"
#import "SchemeGlobals.e"
#import "SchemeFunctionStatics.h"
//#import "SECDWorldSave.e"

#ifdef USE_COCOA_IO

#import "CocoaInterface.h"

#endif

/******** Stuff for Scheme built-in functions; code and associated statics ********/

/**** forward declarations ****/

void listTailCode();

/**** appendCode -- Append together zero or more lists.  The last thing to be appended
                    may actually be any object.  The result shares structure only with
                    the last object.                                         ****/

PROC( appendCode )

    ZERO_FIXNUM_P
    JUMP_TRUE( noArgs )

  testAndDoMore:                  /* R: listsToGo;        S: (tail list* ...) */
    DECREMENT_R
    ZERO_FIXNUM_P
    JUMP_TRUE( wrapUp )
    EXCHANGE( R, STACK( 1 ) )     /* R: nextList;         S: (tail listsToGo-1 list* ...) */

    /* Let's be sure that this argument to append is a list */

    PAIR_P
    JUMP_TRUE( gotAPair )
    EMPTY_LIST_P
    JUMP_TRUE( gotEmptyList )
    JUMP_LABEL( improperList )

  gotAPair:
    CALL( copyList )
    PUSH                            /* R: nextListCopy  S: (nextListCopy tail listsToGo-1 list* ...) */       

    /* The next few lines find the last pair of the proper list in R,
       or, alternatively, trap if R does not contain a proper list.   */

    CALL( lastPair )                /* R: newLastPair   S: (nextListCopy tail listsToGo-1 list* ...) */
    PUSH
    CDR          
    EMPTY_LIST_P
    JUMP_FALSE( improperList )
    POP                             /* R: newLastPair   S: (nextListCopy tail listsToGo-1 list* ...) */

    /* Now append the tail to the new last pair, and then set up
       the stack for one more iteration through the "testAndDoMore" loop. */

    SWAP   /* R: nextListCopy  S: (newLastPair tail listsToGo-1 list* ...) */
    EXCHANGE( R, STACK( 1 ) )   /* R: tail              S: (lastPair nextListCopy listsToGo-1 list* ...) */
    SETCDR                      /* by side-effect, "nextListCopy" becomes the new tail */
    POP                         /* R: newTail;          S: (listsToGo-1 list* ...) */
    SWAP   /* R: listsToGo-1;      S: (newTail list* ...) */
    JUMP_LABEL( testAndDoMore )

  gotEmptyList:                 /* R: ();               S: (tail listsToGo-1 list* ...) */
                                /* there is nothing to append in the empty list: */
    POP
    SWAP
    JUMP_LABEL( testAndDoMore )

  wrapUp:                       /* R: 0;                S: (lastTail ...) */
    POP                         /* R: lastTail */
    CONTINUE

  improperList:
    resetError( 71 );
    CONTINUE

  noArgs:
    LOAD_R_TAG( emptyListTag )
    CONTINUE

END_PROC

functionDescriptorTypeEight appendFD = { appendCode, 13, TRUE, listP, trueBooleanTag, -1,
                                         properListP, alwaysTrue, 0, ObjectErrorNameStrings[ 4 ], ObjectErrorNameStrings[ 0 ] };

/**** lastPairCode ****/

PROC( lastPairCode )

    CALL( lastPair )
    CONTINUE

END_PROC

functionDescriptorTypeOne lastPairFD = { lastPairCode, 119, TRUE, pairP, trueBooleanTag, 1, pairChecker, pairCheckerNames };

/**** lengthCode -- Find the length of a list.   ****/

functionDescriptorTypeOne lengthFD = { lengthOp, 120, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 1, listChecker, listCheckerNames };

/**** listRefCode -- Find the (zero-based) nth element of a list.        ****/

PROC( listRefCode )

    EXACT_P
    JUMP_FALSE( needExact )
    COERCE_R_SHORT_FIXNUM
    CALL( listTail )
    PAIR_P
    JUMP_FALSE( offTheEnd )
    CAR          
    CONTINUE

  offTheEnd:
    resetError( 72 );
    CONTINUE

  needExact:
    resetError( 73 );
    CONTINUE

END_PROC        

functionDescriptorTypeOne listRefFD = { listRefCode, 127, TRUE, alwaysTrue, trueBooleanTag, 2, listIntegerChecker, listIntegerCheckerNames };

/**** listTailCode -- Find the (zero-based) nth cdr of a list.   ****/

PROC( listTailCode )

    PUSH
    MOVE( STACK( 0 ), R )
    MOVE( STACK( 1 ), STACK( 0 ) )
    CALL( listTail )
    DROP
    CONTINUE

END_PROC        

functionDescriptorTypeOne listTailFD = { listTailCode, 128, TRUE, alwaysTrue, trueBooleanTag, 2, listIntegerChecker, listIntegerCheckerNames };

/**** listToVectorCode -- Coerce a list to a vector.     ****/

PROC( listToVectorCode )

    PUSH
    LENGTH
    JUMP_FUNCTION( makeVectorAndStuff )

END_PROC

functionDescriptorTypeOne listToVectorFD = { listToVectorCode, 126, TRUE, vectorP, vectorTag, 1, listChecker, listCheckerNames };

/**** getFileNameForWorldLoad ****/

void getFileNameForWorldLoad()
{
    Text buffer[ MAX_PATHNAME_LENGTH + 1 ];

    buffer[ 0 ] = 0;
    eventLoopOnce( 0 );
    performDialogAndReturnString( MiscStrings[ 349 ], buffer, MAX_PATHNAME_LENGTH );
    makeString( buffer);
    }    

/**** loadWorldCode -- Load a saved world.       ****/

    PROC( loadWorldCode )

    applicativeCheckDropNone( NameStrings[ 266 ] );

    if( ! isMomCatP )
        resetError( 212 );

    ONE_FIXNUM_P
    JUMP_TRUE( givenFileName )

    getFileNameForWorldLoad();
    JUMP_LABEL( nameInR )

  givenFileName:
    POP

  nameInR:

    LOAD_WORLD
    resetEverybody();
    CONTINUE

END_PROC

functionDescriptorTypeSeven loadWorldFD = { loadWorldCode, 266, FALSE, alwaysTrue, trueBooleanTag, 1,
                                            stringP, alwaysTrue, 0, ObjectErrorNameStrings[ 11 ], ObjectErrorNameStrings[ 0 ] };

/**** getFileNameForWorldExamine ****/

void getFileNameForWorldExamine()
{
    Text buffer[ MAX_PATHNAME_LENGTH + 1 ];

    buffer[ 0 ] = 0;
    eventLoopOnce( 0 );
    performDialogAndReturnString( MiscStrings[ 386 ], buffer, MAX_PATHNAME_LENGTH );
    makeString( buffer);
    }    

/**** examineWorldCode -- examine a saved world. ****/

PROC( examineWorldCode )

    ONE_FIXNUM_P
    JUMP_TRUE( givenFileName )

    getFileNameForWorldExamine();
    JUMP_LABEL( nameInR )

  givenFileName:
    POP

  nameInR:
    EXAMINE_WORLD
    resetScheme();
    CONTINUE

END_PROC

functionDescriptorTypeSeven examineWorldFD = { examineWorldCode, 299, FALSE, alwaysTrue, trueBooleanTag, 1,
                                               stringP, alwaysTrue, 0, ObjectErrorNameStrings[ 11 ], ObjectErrorNameStrings[ 0 ] };

/**** macroPCode -- Return #t if R is a macro, else return #f.   ****/

PROC( macroPCode )      

    MACRO_P 
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne macroPFD = { macroPCode, 249, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** memberCode -- Using "equal?", is the element in STACK( 0 ) a member of the list in R?      ****/

PROC( memberCode )

  begin:
    EMPTY_LIST_P
    JUMP_TRUE( notThere )
    PAIR_P
    JUMP_FALSE( improperList )
    CALL( carEqualP )
    JUMP_TRUE( foundIt )
    CDR          
    JUMP_LABEL( begin )

  notThere:
    DROP
    LOAD_R_TAG( falseBooleanTag )
    CONTINUE

  foundIt:
    DROP
    CONTINUE

  improperList:
    resetError( 77 );
    CONTINUE

END_PROC

functionDescriptorTypeOne memberFD = { memberCode, 134, TRUE, sharpFOrProperListP, trueBooleanTag, 2, anyProperListChecker, anyProperListCheckerNames };

/**** memqCode -- Using "eq?", is the element in STACK( 0 ) a member of the list in R?   ****/

PROC( memqCode )

  begin:
    EMPTY_LIST_P
    JUMP_TRUE( notThere )
    PAIR_P
    JUMP_FALSE( improperList )
    CALL( carEqP )
    JUMP_TRUE( foundIt )
    CDR          
    JUMP_LABEL( begin )

  notThere:
    DROP
    LOAD_R_TAG( falseBooleanTag )
    CONTINUE

  foundIt:
    DROP
    CONTINUE

  improperList:
    resetError( 78 );
    CONTINUE

END_PROC

functionDescriptorTypeOne memqFD = { memqCode, 135, TRUE, sharpFOrProperListP, trueBooleanTag, 2, anyProperListChecker, anyProperListCheckerNames };

/**** memvCode -- Using "eqv?", is the element in STACK( 0 ) a member of the list in R?  ****/

PROC( memvCode )

  begin:
    EMPTY_LIST_P
    JUMP_TRUE( notThere )
    PAIR_P
    JUMP_FALSE( improperList )
    CALL( carEqvP )
    JUMP_TRUE( foundIt )
    CDR          
    JUMP_LABEL( begin )

  notThere:
    DROP
    LOAD_R_TAG( falseBooleanTag )
    CONTINUE

  foundIt:
    DROP
    CONTINUE

  improperList:
    resetError( 79 );
    CONTINUE

END_PROC

functionDescriptorTypeOne memvFD = { memvCode, 136, TRUE, sharpFOrProperListP, trueBooleanTag, 2, anyProperListChecker, anyProperListCheckerNames };

/**** nanPCode -- Is the quantity in R an IEEE NAN? ****/

PROC( nanPCode) 

    NAN_P
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne nanPFD = { nanPCode, 222, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** newlineCode -- Output a newline to standard output. ****/

PROC( newlineCode )

    ZERO_FIXNUM_P
    JUMP_FALSE( onPort )
    MOVE( newlineChar, R )
    DISPLAY
    RETURN_DEFAULT
    CONTINUE

  onPort:
    POP
    PUSH
    MOVE( newlineChar, R )
    DISPLAY_ON_PORT 
    DROP
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeSeven newlineFD = { newlineCode, 140, FALSE, alwaysTrue, trueBooleanTag, 1,
                                          outputPortP, alwaysTrue, 0, ObjectErrorNameStrings[ 22 ], ObjectErrorNameStrings[ 0 ] };

/**** notPCode -- Return #t if R is #f, else return #f.  ****/

PROC( notPCode )        

    FALSE_P 
    SET_R_FROM_B
    CONTINUE            

END_PROC

functionDescriptorTypeOne notPFD = { notPCode, 141, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** nullPCode -- Return #t if R is (), else return #f. ****/

PROC( nullPCode )       

    EMPTY_LIST_P 
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne nullPFD = { nullPCode, 142, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** notNullPCode -- Return #f if R is (), else return #t.      ****/

PROC( notNullPCode )    

    EMPTY_LIST_P 
    CLEAR_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne notNullPFD = { notNullPCode, 298, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** pairPCode -- Return #t if R is a pair, else return #f.     ****/

PROC( pairPCode )       

    PAIR_P 
    SET_R_FROM_B 
    CONTINUE

END_PROC

functionDescriptorTypeOne pairPFD = { pairPCode, 146, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** permanentPCode -- Return #t if R is permanent, else return #f.     ****/

PROC( permanentPCode )  

    CALL( permanentSymbolP ) 
    SET_R_FROM_B 
    CONTINUE

END_PROC

functionDescriptorTypeOne permanentPFD = { permanentPCode, 254, TRUE, booleanP, trueBooleanTag, 1, symbolChecker, symbolCheckerNames };

/**** printDepthListGet ****/

PROC( printDepthListGet )

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( printDepth, R )
    CONTINUE

END_PROC

functionDescriptorTypeOne printDepthListGetFD
    = { printDepthListGet, 234, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 0, anyChecker, anyCheckerNames };

/**** printDepthVectorGet ****/

PROC( printDepthVectorGet )

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( vPrintDepth, R )
    CONTINUE

END_PROC

functionDescriptorTypeOne printDepthVectorGetFD =
    { printDepthVectorGet, 235, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 0, anyChecker, anyCheckerNames };

/**** printLengthListGet ****/

PROC( printLengthListGet )

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( printLength, R )
    CONTINUE

END_PROC

functionDescriptorTypeOne printLengthListGetFD =
    { printLengthListGet, 236, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 0, anyChecker, anyCheckerNames };

/**** printLengthVectorGet ****/

PROC( printLengthVectorGet )

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( vPrintLength, R )
    CONTINUE

END_PROC

functionDescriptorTypeOne printLengthVectorGetFD =
    { printLengthVectorGet, 237, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 0, anyChecker, anyCheckerNames };

/**** printDepthListSet ****/

PROC( printDepthListSet )

    applicativeCheckDropNone( NameStrings[ 238 ] );

    PUSH
    MOVE( three, R )
    NUM_GE_P
    JUMP_TRUE( setIt )
    POP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 225 );

  setIt:
    POP
#ifdef USE_COCOA_IO
    reportEvent( LIST_PRINT_DEPTH_EVENT );
#endif 
    MOVE( (LongWord)OBJECT( R ), printDepth )
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne printDepthListSetFD = { printDepthListSet, 238, FALSE, alwaysTrue, trueBooleanTag, 1, integerChecker, integerCheckerNames };

/**** printDepthVectorSet ****/

PROC( printDepthVectorSet )

    applicativeCheckDropNone( NameStrings[ 239 ] );

    PUSH
    MOVE( three, R )
    NUM_GE_P
    JUMP_TRUE( setIt )
    POP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 225 );

  setIt:
    POP
#ifdef USE_COCOA_IO
    reportEvent( VECTOR_PRINT_DEPTH_EVENT );
#endif 
    MOVE( (LongWord)OBJECT( R ), vPrintDepth )
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne printDepthVectorSetFD = { printDepthVectorSet, 239, FALSE, alwaysTrue, trueBooleanTag, 1, integerChecker, integerCheckerNames };

/**** printLengthListSet ****/

PROC( printLengthListSet )

    applicativeCheckDropNone( NameStrings[ 240 ] );

    PUSH
    MOVE( three, R )
    NUM_GE_P
    JUMP_TRUE( setIt )
    POP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 225 );

  setIt:
    POP
#ifdef USE_COCOA_IO
    reportEvent( LIST_PRINT_LENGTH_EVENT );
#endif 
    MOVE( (LongWord)OBJECT( R ), printLength )
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne printLengthListSetFD = { printLengthListSet, 240, FALSE, alwaysTrue, trueBooleanTag, 1, integerChecker, integerCheckerNames };

/**** printLengthVectorSet ****/

PROC( printLengthVectorSet )

    applicativeCheckDropNone( NameStrings[ 241 ] );

    PUSH
    MOVE( three, R )
    NUM_GE_P
    JUMP_TRUE( setIt )
    POP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 225 );

  setIt:
    POP
#ifdef USE_COCOA_IO
    reportEvent( VECTOR_PRINT_LENGTH_EVENT );
#endif 
    MOVE( (LongWord)OBJECT( R ), vPrintLength )
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne printLengthVectorSetFD = { printLengthVectorSet, 241, FALSE, alwaysTrue, trueBooleanTag, 1, integerChecker, integerCheckerNames };

/**** procedurePCode -- Return #t if R is a procedure, else return #f.   ****/

PROC( procedurePCode )  

    PROCEDURE_P 
    SET_R_FROM_B 
    CONTINUE

END_PROC

functionDescriptorTypeOne procedurePFD = { procedurePCode, 148, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** promisePCode -- Return #t if R is a promise, else return #f.       ****/

PROC( promisePCode )    

    PROMISE_P 
    SET_R_FROM_B 
    CONTINUE

END_PROC

functionDescriptorTypeOne promisePFD = { promisePCode, 95, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** properListPCode -- Return #t if R is a proper list or the empty list, else return #f.      ****/

PROC( properListPCode ) 

    CALL( properListCheck )
    SET_R_FROM_B 
    CONTINUE

END_PROC

functionDescriptorTypeOne properListPFD = { properListPCode, 296, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** quitCode -- Exit from Scheme.      ****/

PROC( quitCode )        

    JUMP_FUNCTION( exitTopLoop )

END_PROC

functionDescriptorTypeOne quitFD = { quitCode, 88, FALSE, alwaysTrue, emptyListTag, 0, anyChecker, anyCheckerNames };

/**** randomCode -- Generate a random number using "random". ****/ 

PROC( randomCode) 

    LOAD_R_TAG( exactShortFixnumBitsMask );
    LOAD_OBJECT( random(), R )
    CONTINUE

END_PROC

functionDescriptorTypeOne randomFD = { randomCode, 301, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 0, anyChecker, anyCheckerNames };

/**** resetCode -- Reset Scheme without exiting.         ****/

PROC( resetCode )

    resetError( 0 );

END_PROC

functionDescriptorTypeOne resetFD = { resetCode, 96, FALSE, alwaysTrue, emptyListTag, 0, anyChecker, anyCheckerNames };

functionDescriptorTypeOne reverseFD = { reverseListCode, 155, TRUE, properListP, trueBooleanTag, 1, properListChecker, properListCheckerNames };

/**** returnRoomCode -- Return the amount of room remaining in the current main store.           ****/

functionDescriptorTypeOne returnRoomFD = { returnRoomOp, 226, TRUE, integerP, numberBitsMask, 0, anyChecker, anyCheckerNames };

/**** returnActiveRoomCode -- Return the amount of room remaining in the current main store.           ****/

functionDescriptorTypeOne returnActiveRoomFD = { returnActiveRoomOp, 429, TRUE, integerP, numberBitsMask, 0, anyChecker, anyCheckerNames };

/**** returnAgingRoomCode -- Return the amount of room remaining in the current main store.           ****/

functionDescriptorTypeOne returnAgingRoomFD = { returnAgingRoomOp, 430, TRUE, integerP, numberBitsMask, 0, anyChecker, anyCheckerNames };

/**** returnStoreSizeCode -- Return the (total) size of the current main store.          ****/

functionDescriptorTypeOne returnStoreSizeFD = { returnStoreSizeOp, 227, TRUE, integerP, numberBitsMask, 0, anyChecker, anyCheckerNames };

/**** returnActiveStoreSizeCode -- Return the (total) size of the current active store.          ****/

functionDescriptorTypeOne returnActiveStoreSizeFD = { returnActiveStoreSizeOp, 431, TRUE, integerP, numberBitsMask, 0, anyChecker, anyCheckerNames };

/**** return90PercentFlagCode -- Return whether 90 percent or more of main store is in use.          ****/

functionDescriptorTypeOne return90PercentFlagFD = { return90PercentFlagOp, 362, TRUE, integerP, numberBitsMask, 0, anyChecker, anyCheckerNames };

/**** getFileNameForWorldSave ****/

void getFileNameForWorldSave()
{
    Text buffer[ MAX_PATHNAME_LENGTH + 1 ];

    buffer[ 0 ] = 0;
    eventLoopOnce( 0 );
    performDialogAndReturnString( MiscStrings[ 350 ], buffer, MAX_PATHNAME_LENGTH );
    makeString( buffer);
    }    

/**** saveWorldCode -- Save the world.   ****/

PROC( saveWorldCode )

    if( ! isMomCatP )
        resetError( 213 );

    ONE_FIXNUM_P
    JUMP_TRUE( givenFileName )

    getFileNameForWorldSave();
    JUMP_LABEL( nameInR )

  givenFileName:
    POP

  nameInR:
    SAVE_WORLD
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeSeven saveWorldFD = { saveWorldCode, 267, FALSE, alwaysTrue, trueBooleanTag, 1,
                                            stringP, alwaysTrue, 0, ObjectErrorNameStrings[ 11 ], ObjectErrorNameStrings[ 0 ] };

/**** setCarCode -- Set the car of the pair in STACK( 0 ), to the object in R.  Return #t.       ****/

PROC( setCarCode )

    applicativeCheckDropOne( NameStrings[ 158 ] );

    SWAP
    IMMUTABLE_P
    JUMP_TRUE( setCarImmutableError )
    PUSH
    R = spliceOutNotCarForwardingPointers( R );
    IMMUTABLE_P
    POP
    JUMP_TRUE( setCarImmutableError )
    SWAP
    SETCAR
    RETURN_DEFAULT
    CONTINUE

  setCarImmutableError:
    DROP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 245 );
    CONTINUE

END_PROC

functionDescriptorTypeOne setCarFD = { setCarCode, 158, FALSE, alwaysTrue, trueBooleanTag, 2, pairAnyChecker, pairAnyCheckerNames };

/**** setCdrCode -- Set the cdr of the pair in STACK( 0 ), to the object in R.  Return #t.       ****/

PROC( setCdrCode )

    applicativeCheckDropOne( NameStrings[ 159 ] );

    SWAP
    IMMUTABLE_P
    JUMP_TRUE( setCdrImmutableError )
    PUSH
    R = spliceOutNotCarForwardingPointers( R );
    IMMUTABLE_P
    POP
    JUMP_TRUE( setCdrImmutableError )
    SWAP
    SETCDR
    RETURN_DEFAULT
    CONTINUE

  setCdrImmutableError:
    DROP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 246 );
    CONTINUE

END_PROC

functionDescriptorTypeOne setCdrFD = { setCdrCode, 159, FALSE, alwaysTrue, trueBooleanTag, 2, pairAnyChecker, pairAnyCheckerNames };

/**** setPermanentCode -- Make the given symbol permanent.       ****/

PROC( setPermanentCode )        

    applicativeCheckDropNone( NameStrings[ 255 ] );

    CALL( makePermanentSymbol ) 
    RETURN_DEFAULT 
    CONTINUE

END_PROC

functionDescriptorTypeOne setPermanentFD = { setPermanentCode, 255, FALSE, alwaysTrue, trueBooleanTag, 1, symbolChecker, symbolCheckerNames };

/**** setTagCode -- Set the tag of the object passed in STACK( 0 ), to the
                    value passed in R; return the newly-tagged object.               ****/

PROC( setTagCode )

//    if object is not leaf
//        if object is locked
//            change tag while locked
//        else
//            lock object
//            change tag
//            unlock    
//    else
//        change tag

    Tag toTransfer = 0;

    applicativeCheckDropOne( NameStrings[ 251 ] );

    if( ! leaf_p( (S[0]).t ) ) {
        if( lockedS0() ) {
            toTransfer = R.u.c & (~(cdrCodingBitsMask | administrationBitsMask));
            toTransfer |= ((S[0]).t & (cdrCodingBitsMask | administrationBitsMask));
            (S[0]).t = toTransfer;
            }
        else {
            if( ! blockOnTagLock1( &(S[0]), 18 ) )
                resetError( 293 );

            toTransfer = R.u.c & (~(cdrCodingBitsMask | administrationBitsMask));
            toTransfer |= ((S[0]).t & (cdrCodingBitsMask | administrationBitsMask)) ;
            (S[0]).t = toTransfer;

            releaseTagLock1( (volatile taggedAval *)((S[0]).u.a), 33 );
            }
        }
    else {
        toTransfer = R.u.c & (~(cdrCodingBitsMask | administrationBitsMask));
        toTransfer |= ((S[0]).t & (cdrCodingBitsMask | administrationBitsMask));
        (S[0]).t = toTransfer;
        }
    POP
    R.t &= ~pointsToActiveBitMask;  // TEST
    CONTINUE

END_PROC

functionDescriptorTypeOne setTagFD = { setTagCode, 251, TRUE, alwaysTrue, trueBooleanTag, 2, anyNumberChecker, anyNumberCheckerNames };

/**** cSetTagCode -- Set the tag of the object passed in STACK( 0 ), to the
                     value passed in R; return the newly-tagged object.               ****/

PROC( cSetTagCode )

//    if object is not leaf
//        if object is locked
//            change tag while locked
//        else
//            lock object
//            change tag
//            unlock    
//    else
//        change tag

    Tag toTransfer = 0;

//    applicativeCheckDropOne( NameStrings[ 251 ] );  // This line being commented out distinguishes c::set-tag! from e::set-tag!

    if( ! leaf_p( (S[0]).t ) ) {
        if( lockedS0() ) {
            toTransfer = R.u.c & (~(cdrCodingBitsMask | administrationBitsMask));
            toTransfer |= ((S[0]).t & (cdrCodingBitsMask | administrationBitsMask));
            (S[0]).t = toTransfer;
            }
        else {
            if( ! blockOnTagLock1( &(S[0]), 56 ) )
                resetError( 335 );

            toTransfer = R.u.c & (~(cdrCodingBitsMask | administrationBitsMask));
            toTransfer |= ((S[0]).t & (cdrCodingBitsMask | administrationBitsMask)) ;
            (S[0]).t = toTransfer;

            releaseTagLock1( (volatile taggedAval *)((S[0]).u.a), 74 );
            }
        }
    else {
        toTransfer = R.u.c & (~(cdrCodingBitsMask | administrationBitsMask));
        toTransfer |= ((S[0]).t & (cdrCodingBitsMask | administrationBitsMask));
        (S[0]).t = toTransfer;
        }
    POP
    R.t &= ~pointsToActiveBitMask;  // TEST
    CONTINUE

END_PROC

functionDescriptorTypeOne cSetTagFD = { cSetTagCode, 456, TRUE, alwaysTrue, trueBooleanTag, 2, anyNumberChecker, anyNumberCheckerNames };

/**** sRandomCode -- Seed "random". ****/

PROC( sRandomCode ) 

    COERCE_R_SHORT_FIXNUM
    srandom( (UnsignedWord)((UnsignedLongWord)(R.u.c) & WORD_MASK) );
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne sRandomFD = { sRandomCode, 302, FALSE, alwaysTrue, trueBooleanTag, 1, unsignedFixnumChecker, unsignedFixnumCheckerNames };

/**** stackDepth -- Return the depth of the stack ****/

functionDescriptorTypeOne stackDepthFD = { stackDepth, 244, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 0, anyChecker, anyCheckerNames };

/**** stringSymbolCode -- Coerce a string to a symbol, and intern that symbol.   ****/

PROC( stringSymbolCode )        

    CALL( extractByteBlockString ) 
    CALL( intern ) 
    CONTINUE

END_PROC

functionDescriptorTypeOne stringSymbolFD = {stringSymbolCode, 164, TRUE, symbolP, symbolTag, 1, stringChecker, stringCheckerNames };

/**** syntaxBindingPCode -- Is the object in R a syntax-binding?        ****/

PROC( syntaxBindingPCode )     

    SYNTAX_BINDING_P 
    SET_R_FROM_B 
    CONTINUE

END_PROC

functionDescriptorTypeOne syntaxBindingPFD = { syntaxBindingPCode, 82, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** multipleValuesPCode -- Is the object in R a multipleValues?        ****/

PROC( multipleValuesPCode )     

    MULTIPLE_VALUES_P 
    SET_R_FROM_B 
    CONTINUE

END_PROC

functionDescriptorTypeOne multipleValuesPFD = { multipleValuesPCode, 307, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** symbolPCode -- Is the object in R a symbol?        ****/

PROC( symbolPCode )     

    SYMBOL_P 
    SET_R_FROM_B 
    CONTINUE

END_PROC

functionDescriptorTypeOne symbolPFD = { symbolPCode, 184, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** symbolStringCode -- Return the string that names the given symbol. ****/

functionDescriptorTypeOne symbolStringFD = { getPName, 183, TRUE, stringP, stringTag, 1, symbolChecker, symbolCheckerNames };

/**** symevalCode -- Evaluate the given symbol in the current environment.  ****/

functionDescriptorTypeOne symevalFD = { lookUp, 256, TRUE, alwaysTrue, symbolTag, 1, symbolChecker, symbolCheckerNames };

/**** symevalInEnvironmentCode  -- Evaluate the given symbol in the given environment.  ****/

functionDescriptorTypeOne symevalInEnvironmentFD = { lookUpInEnvironment, 297, TRUE, alwaysTrue, symbolTag, 2, symbolPairChecker, symbolPairCheckerNames };

/**** systemCode ****/

PROC( systemCode )

#ifndef PIXIE_SCHEMES
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( system( (Text *)((R.u.a) + 1) ), R )   // Not enabled for the Pixie Schemes.
#endif
    CONTINUE

END_PROC

functionDescriptorTypeOne systemFD = { systemCode, 305, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 1, stringChecker, stringCheckerNames };

/**** tickCode -- Get the Macintosh "tickcount". ****/

PROC( tickCode )

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( time( 0 ), R )
    CONTINUE

END_PROC

functionDescriptorTypeOne tickFD = { tickCode, 242, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 0, anyChecker, anyCheckerNames };

/**** vectorPCode -- Is the object in R a vector?        ****/

PROC( vectorPCode )

    VECTOR_P
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne vectorPFD = { vectorPCode, 195, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** vectorFillCode -- Fill the given vector with the given object.     ****/

PROC( vectorFillCode )

    applicativeCheckDropOne( NameStrings[ 191 ] );

    IMMUTABLE_P_S0
    JUMP_TRUE( vectorFillImmutableError )
    SWAP
    PUSH
    R = spliceOutNotCarForwardingPointers( R );
    R = *(R.u.a);
    IMMUTABLE_P
    POP
    SWAP
    JUMP_TRUE( vectorFillImmutableError )
    IMMUTABLE_P
    CALL( vectorFillAux )
    DROP
    RETURN_DEFAULT
    CONTINUE

  vectorFillImmutableError:
    POP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 249 );
    CONTINUE

END_PROC

functionDescriptorTypeOne vectorFillFD = { vectorFillCode, 191, FALSE, alwaysTrue, trueBooleanTag, 2, vectorAnyChecker, vectorAnyCheckerNames };

/**** vectorToListCode -- Coerce the given vector to a list.     ****/

PROC( vectorToListCode )

    LOAD_R_TAG( pairTag )
    CDR          
    EMPTY_LIST_P
    JUMP_TRUE( exit )
    JUMP_FUNCTION( copyList )

  exit:  
    CONTINUE

END_PROC

functionDescriptorTypeOne vectorToListFD = { vectorToListCode, 190, TRUE, properListP, trueBooleanTag, 1, vectorChecker, vectorCheckerNames };

/**** versionGetCode ****/

PROC( versionGetCode )

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( versionString, R )
    MAKE_STRING
    CONTINUE

END_PROC

functionDescriptorTypeOne versionGetFD =
    { versionGetCode, 98, TRUE, stringP, stringTag, 0, anyChecker, anyCheckerNames };

/**** vLengthCode ****/

functionDescriptorTypeOne vLengthFD = { vectorLength, 192, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 1, vectorChecker, vectorCheckerNames };

/**** vRefCode -- Return the (0-based) nth element of the given vector.  ****/

PROC( vRefCode )

    EXACT_P
    JUMP_FALSE( needExact )
    COERCE_R_SHORT_FIXNUM
    PUSH
    MOVE( STACK( 1 ), R )
    VLENGTH
    NUM_LT_P
    JUMP_FALSE( highRange )
    POP
    VREF
    CONTINUE

  highRange:
    POP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 81 );
    CONTINUE

  needExact:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 82 );
    CONTINUE

END_PROC

functionDescriptorTypeOne vRefFD = { vRefCode, 193, TRUE, alwaysTrue, trueBooleanTag, 2, vectorIntegerChecker, vectorIntegerCheckerNames };

/**** vSetCode -- Set the (0-based) nth element of the given vector to the given object. ****/

PROC( vSetCode )

    applicativeCheckDropTwo( NameStrings[ 194 ] );

    IMMUTABLE_P_S1
    JUMP_TRUE( vectorSetImmutableError )
    EXCHANGE( R, STACK( 1 ) )
    PUSH
    R = spliceOutNotCarForwardingPointers( R );
    R = *(R.u.a);
    IMMUTABLE_P
    POP
    EXCHANGE( R, STACK( 1 ) )
    JUMP_TRUE( vectorSetImmutableError )
    COERCE_S0_SHORT_FIXNUM 
    EXCHANGE( R, STACK( 1 ) )           /* R: vector; S: (index new ...) */
    PUSH                                        
    MOVE( STACK( 1 ), R )               /* R: index; S: (vector index new ...) */
    EXACT_P
    JUMP_FALSE( needExact )
    PUSH
    MOVE( STACK( 1 ), R )
    VLENGTH
    NUM_LT_P
    JUMP_FALSE( highRange )
    DROP
    POP
    EXCHANGE( R, STACK( 1 ) )
    VSET
    RETURN_DEFAULT
    CONTINUE

  highRange:
    POP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 84 );
    CONTINUE

  needExact:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 85 );
    CONTINUE

  vectorSetImmutableError:
    DROP
    POP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 250 );
    CONTINUE

END_PROC

functionDescriptorTypeOne vSetFD = { vSetCode, 194, FALSE, alwaysTrue, trueBooleanTag, 3, vectorIntegerAnyChecker, vectorIntegerAnyCheckerNames };

/**** writeCharCode -- Write out the character in R.     ****/

PROC( writeCharCode )   

    ONE_FIXNUM_P
    JUMP_FALSE( onPort )
    POP
    WRITE_CHAR 
    RETURN_DEFAULT
    CONTINUE

  onPort:
    POP
    SWAP
    WRITE_CHAR_ON_PORT 
    DROP
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeSix writeCharFD = { writeCharCode, 205, FALSE, alwaysTrue, trueBooleanTag, 2,
                                          characterP, outputPortP, ObjectErrorNameStrings[ 5 ], ObjectErrorNameStrings[ 22 ] };

/**** writeCode -- Write out the object in R.  ("Write" outputs a  different
                   format than "display", at least for some objects.                 ****/

PROC( writeCode )       

    ONE_FIXNUM_P
    JUMP_FALSE( onPort )
    POP
    WRITE 
    RETURN_DEFAULT
    CONTINUE

  onPort:
    POP
//    GET_OUTPUT_DESCRIPTOR
    SWAP
    WRITE_ON_PORT 
    DROP
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeSix writeFD = { writeCode, 196, FALSE, alwaysTrue, trueBooleanTag, 2,
                                      alwaysTrue, outputPortP, ObjectErrorNameStrings[ 0 ], ObjectErrorNameStrings[ 22 ] };

/**** writeContinuationCode -- Write out the current contents of the continuation.       ****/

PROC( writeContinuationCode )

    saveContinuation();
    MOVE( A, R )
    WRITE
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne writeContinuationFD = { writeContinuationCode, 99, FALSE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** writeStackCode Write out the current contents of the stack.       ****/

PROC( writeStackCode )

    CALL( saveStack )
    MOVE( A, R )
    WRITE
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne writeStackFD = { writeStackCode, 100, FALSE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

