/****************************************************************************************

SchemeFunctions5.c -- Copyright (c) by Jay Reynolds Freeman 1988, 1989, 1990, 1991, 2006

Scheme built-in character-manipulation functions for Wraith Scheme.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import "WraithHeader.h"
#import "MemoryManager.h"
#import "OpCodes.h"
#import "SchemeFunctions.h"
#import "DataTypes.e"
#import "Globals.e"
#import "Intern.e"
#import "IO.e"
#import "SchemeAux.e"
#import "SchemeFunctionStatics.h"

/******** Stuff for Scheme built-in functions; code and associated statics ********/

/**** charPCode -- Is the object in R a character? ****/

PROC( charPCode )

    CHARACTER_P 
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne charPFD = { charPCode, 73, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** charAlphaPCode -- Is the character in R an alphabetic character? ****/

PROC( charAlphaPCode )

    CHAR_ALPHA_P 
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne charAlphaPFD = { charAlphaPCode, 56, TRUE, booleanP, trueBooleanTag, 1, charChecker, charCheckerNames };

/**** charLowerPCode -- Is the character in R a lower-case letter? ****/

PROC( charLowerPCode )

    CHAR_LOWER_P 
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne charLowerPFD = { charLowerPCode, 63, TRUE, booleanP, trueBooleanTag, 1, letterChecker, letterCheckerNames };

/**** charNumberPCode -- Is the character in R a decimal digit? ****/

PROC( charNumberPCode )

    CHAR_NUMERIC_P 
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne charNumberPFD = { charNumberPCode, 64, TRUE, booleanP, trueBooleanTag, 1, charChecker, charCheckerNames };

/**** charUpperPCode -- Is the character in R an upper-case letter? ****/

PROC( charUpperPCode )

    CHAR_UPPER_P 
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne charUpperPFD = { charUpperPCode, 66, TRUE, booleanP, trueBooleanTag, 1, letterChecker, letterCheckerNames };

/**** charWhitespacePCode -- Is the character in R a whitespace character? ****/

PROC( charWhitespacePCode )

    CHAR_WHITESPACE_P 
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne charWhitespacePFD = { charWhitespacePCode, 67, TRUE, booleanP, trueBooleanTag, 1, charChecker, charCheckerNames };

/**** charEqualPCode -- Are the characters in R and in STACK( 0 ) the same? ****/

PROC( charEqualPCode )

    OBJECT_EQ_P( STACK( 0 ), R )
    DROP
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne charEqualPFD = { charEqualPCode, 70, TRUE, booleanP, trueBooleanTag, 2, charBinaryChecker, charBinaryCheckerNames };

/**** charGreaterPCode -- Is the character in STACK( 0 ) lexicographically
                          greater than the character in R? ****/

PROC( charGreaterPCode )

    OBJECT_GT_P( STACK( 0 ), R )
    DROP
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne charGreaterPFD = { charGreaterPCode, 72, TRUE, booleanP, trueBooleanTag, 2, charBinaryChecker, charBinaryCheckerNames };

/**** charGreaterEqualPCode -- Is the character in STACK( 0 ) lexicographically
                               greater than or equal to the character in R?              ****/

PROC( charGreaterEqualPCode )

    OBJECT_GE_P( STACK( 0 ), R )
    DROP
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne charGreaterEqualPFD = { charGreaterEqualPCode, 71, TRUE, booleanP, trueBooleanTag, 2, charBinaryChecker, charBinaryCheckerNames };

/**** charLessPCode -- Is the character in STACK( 0 ) lexicographically
                       less than the character in R?  ****/

PROC( charLessPCode )

    OBJECT_LT_P( STACK( 0 ), R )
    DROP
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne charLessPFD = { charLessPCode, 69, TRUE, booleanP, trueBooleanTag, 2, charBinaryChecker, charBinaryCheckerNames };

/**** charLessEqualPCode -- Is the character in STACK( 0 ) lexicographically
                            less than or equal to the character in R?                ****/

PROC( charLessEqualPCode )

    OBJECT_LE_P( STACK( 0 ), R )
    DROP
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne charLessEqualPFD = { charLessEqualPCode, 68, TRUE, booleanP, trueBooleanTag, 2, charBinaryChecker, charBinaryCheckerNames };

/**** charCIEqualPCode -- Ignoring case differences, are the characters
                          in R and in STACK( 0 ) the same?                                   ****/

PROC( charCIEqualPCode )

    CHAR_TO_LOWER
    SWAP
    CHAR_TO_LOWER
    OBJECT_EQ_P( R, STACK( 0 ) )
    DROP
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne charCIEqualPFD = { charCIEqualPCode, 59, TRUE, booleanP, trueBooleanTag, 2, charBinaryChecker, charBinaryCheckerNames };

/**** charCIGreaterPCode -- Ignoring case differences, is the character in STACK( 0 )
                            lexicographically greater than the character in R?                       ****/

PROC( charCIGreaterPCode )

    CHAR_TO_LOWER
    SWAP
    CHAR_TO_LOWER
    OBJECT_GT_P( R, STACK( 0 ) )
    DROP
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne charCIGreaterPFD = { charCIGreaterPCode, 61, TRUE, booleanP, trueBooleanTag, 2, charBinaryChecker, charBinaryCheckerNames };

/**** charCIGreaterEqualPCode -- Ignoring case differences, is the character in STACK( 0 )
                                 lexicographically greater than or equal to the character
                                 in R?                                               ****/

PROC( charCIGreaterEqualPCode )

    CHAR_TO_LOWER
    SWAP
    CHAR_TO_LOWER
    OBJECT_GE_P( R, STACK( 0 ) )
    DROP
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne charCIGreaterEqualPFD =
    { charCIGreaterEqualPCode, 60, TRUE, booleanP, trueBooleanTag, 2, charBinaryChecker, charBinaryCheckerNames };

/**** charCILessPCode -- Ignoring case differences, is the character in STACK( 0 )
                         lexicographically less than the character in R?                     ****/

PROC( charCILessPCode )

    CHAR_TO_LOWER
    SWAP
    CHAR_TO_LOWER
    OBJECT_LT_P( R, STACK( 0 ) )
    DROP
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne charCILessPFD = { charCILessPCode, 58, TRUE, booleanP, trueBooleanTag, 2, charBinaryChecker, charBinaryCheckerNames };

/**** charCILessEqualPCode -- Ignoring case differences, is the character in STACK( 0 )
                              lexicographically greater than or equal to the character in R? ****/

PROC( charCILessEqualPCode )

    CHAR_TO_LOWER
    SWAP
    CHAR_TO_LOWER
    OBJECT_LE_P( R, STACK( 0 ) )
    DROP
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne charCILessEqualPFD = { charCILessEqualPCode, 57, TRUE, booleanP, trueBooleanTag, 2, charBinaryChecker, charBinaryCheckerNames };

/**** charToIntegerCode -- What is the integer (ASCII) that represents the character in R? ****/

PROC( charToIntegerCode )

    LOAD_R_TAG( exactShortFixnumBitsMask )
    CONTINUE

END_PROC

functionDescriptorTypeOne charToIntegerFD = { charToIntegerCode, 55, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 1, charChecker, charCheckerNames };

/**** integerToCharCode -- What ASCII character is represented by the integer in R? ****/

PROC( integerToCharCode )

    COERCE_R_SHORT_FIXNUM
    PUSH
    MOVE( zero, R )
    NUM_GE_P
    JUMP_FALSE( rangeError )
    MOVE( maxChar, R )
    NUM_LE_P
    JUMP_FALSE( rangeError )
    POP
    LOAD_R_TAG( characterTag )
    CONTINUE

  rangeError:
    POP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 86 );
    CONTINUE

END_PROC

functionDescriptorTypeOne integerToCharFD = { integerToCharCode, 116, TRUE, characterP, characterTag, 1, exactIntegerChecker, exactIntegerCheckerNames };

/**** charUpcaseCode -- If the character in R is alphabetic and lower-case,
                        convert it to upper case, otherwise leave it alone.      ****/

PROC( charUpCaseCode )

    CHAR_TO_UPPER
    CONTINUE

END_PROC

functionDescriptorTypeOne charUpcaseFD = { charUpCaseCode, 65, TRUE, characterP, characterTag, 1, charChecker, charCheckerNames };

/**** charDowncaseCode -- If the character in R is alphabetic and upper-case,
                          convert it to upper case, otherwise leave it alone.    ****/

PROC( charDowncaseCode )

    CHAR_TO_LOWER
    CONTINUE

END_PROC

functionDescriptorTypeOne charDowncaseFD = { charDowncaseCode, 62, TRUE, characterP, characterTag, 1, charChecker, charCheckerNames };

