/****************************************************************************************

SchemeFunctions6.c -- Copyright (c) by Jay Reynolds Freeman 1988, 1989, 1990, 1991, 2006

Scheme built-in functions with variable arity for Wraith Scheme.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import <signal.h>
#import "WraithHeader.h"
#import "MemoryManager.h"
#import "OpCodes.h"
#import "SchemeFunctions.h"
#import "SchemeFunctions.e"
#import "DataTypes.e"
#import "Globals.e"
#import "Intern.e"
#import "IO.e"
#import "Reader.e"
#import "SchemeAux.e"
#import "SchemeFunctionStatics.h"
#import "StreamHandling.e"

/******** Stuff for Scheme built-in functions; code and associated statics ********/

/**** listCode -- The arguments are stacked; their number is in R. ****/

functionDescriptorTypeFive listFD = { listTopN, 124, TRUE, properListP, trueBooleanTag, 0, alwaysTrue, ObjectErrorNameStrings[ 0 ] };

/**** callWithInputPortCode -- Call the given function with an input file. ****/ 

PROC( callWithInputPortCode )

    ONE_FIXNUM_P
    JUMP_FALSE( givenFileName )

    OPEN_INPUT_SF
    JUMP_LABEL( doTheCall )

  givenFileName:
    POP
    SWAP
    OPEN_INPUT_NAMED
        
  doTheCall:                                    /* R: descriptor;       S: (procedure ...) */
    EMPTY_LIST_P
    JUMP_FALSE( goodDescriptor )
    resetError( 172 );

  goodDescriptor:       
    MAKE_FRAME
    MOVE( STACK( FrameSize ), STACK( 0 ) )
    SWAP                   /* R: procedure;        S: (descriptor ...) */
    PUSH
    PUSH
    PUSH                                        /* S: (procedure procedure procedure descriptor ...) */
    MOVE( STACK( 3 ), STACK( 2 ) )
    MOVE( STACK( 3 ), STACK( 3 + FrameSize ) )
    MOVE( one, STACK( 1  ) )                    /* S: (procedure one descriptor X X X X X X X X descriptor ...) */

    LOAD_R_TAG( schemeMachinePCIndexTag )
    LOAD_OBJECT( closeStackedPort_index, R )
    PUSH_CONTINUATION
    LOAD_FRAME( 3, 0 )
    LOAD_R_TAG( schemeMachinePCIndexTag )
    LOAD_OBJECT( apply_index, R )
    CSPush( R );
//    NILCONS
//    MOVE( R, C )

    /* C: (apply closeStackedPort ...) */

    CONTINUE

END_PROC

functionDescriptorTypeEight callWithInputPortFD =
    { callWithInputPortCode, 206, TRUE, alwaysTrue, trueBooleanTag, 2,
      stringP, procedureP, 1, ObjectErrorNameStrings[ 11 ], ObjectErrorNameStrings[ 23 ] };

/**** callWithOutputPortCode -- Call the given function with an output port. ****/       

PROC( callWithOutputPortCode )

    ONE_FIXNUM_P
    JUMP_FALSE( givenFileName )

    OPEN_OUTPUT_SF
    JUMP_LABEL( doTheCall )

  givenFileName:
    POP
    SWAP
    OPEN_OUTPUT_NAMED
        
  doTheCall:                                            /* R: descriptor;       S: (procedure ...) */
    EMPTY_LIST_P
    JUMP_FALSE( goodDescriptor ) 
    resetError( 172 );

  goodDescriptor:       
    MAKE_FRAME
    MOVE( STACK( FrameSize ), STACK( 0 ) )
    SWAP                           /* R: procedure;        S: (descriptor ...) */
    PUSH
    PUSH
    PUSH                                                /* S: (procedure procedure procedure descriptor ...) */
    MOVE( STACK( 3 ), STACK( 2 ) )
    MOVE( STACK( 3 ), STACK( 3 + FrameSize ) )
    MOVE( one, STACK( 1  ) )                            /* S: (procedure one descriptor X X X X X X X X descriptor ...) */

    LOAD_R_TAG( schemeMachinePCIndexTag )
    LOAD_OBJECT( closeStackedPort_index, R )
    PUSH_CONTINUATION
    LOAD_FRAME( 3, 0 )
    LOAD_R_TAG( schemeMachinePCIndexTag )
    LOAD_OBJECT( apply_index, R )
    CSPush( R );
//    NILCONS
//    MOVE( R, C )

    CONTINUE

END_PROC

functionDescriptorTypeEight callWithOutputPortFD =
    { callWithOutputPortCode, 207, TRUE, alwaysTrue, trueBooleanTag, 2, 
      stringP, procedureP, 1, ObjectErrorNameStrings[ 11 ], ObjectErrorNameStrings[ 23 ] };

/**** charReadyPCode -- Is a character ready to be read on the given port?. ****/

PROC( charReadyPCode )

    ZERO_FIXNUM_P
    JUMP_FALSE( portGiven )
    CHAR_READY_P
    SET_R_FROM_B
    CONTINUE

  portGiven:
    POP
    PUSH
    MOVE( DefaultInputPort, R )
    SWAP
    MOVE( R, DefaultInputPort )
    CHAR_READY_P
    POP
    MOVE( R, DefaultInputPort )
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeSeven charReadyPFD = { charReadyPCode, 219, TRUE, booleanP, trueBooleanTag, 1,
                                             inputPortP, alwaysTrue, 0, ObjectErrorNameStrings[ 9 ], ObjectErrorNameStrings[ 0 ] };

/**** closePortCode -- Close any port ****/

functionDescriptorTypeOne closeInputPortFD = { closePort, 200, FALSE, alwaysTrue, trueBooleanTag, 1, inputPortChecker, inputPortCheckerNames };

functionDescriptorTypeOne closeOutputPortFD = { closePort, 201, FALSE, alwaysTrue, trueBooleanTag, 1, outputPortChecker, outputPortCheckerNames };

/**** currentInputPortCode -- Retrieve the current input port. ****/

PROC( currentInputPortCode )

    MOVE( CurrentInputPort, R )
    CONTINUE

END_PROC

functionDescriptorTypeOne currentInputPortFD = { currentInputPortCode, 215, TRUE, inputPortP, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** currentOutputPortCode -- Retrieve the current output port. ****/

PROC( currentOutputPortCode )

    MOVE( CurrentOutputPort, R )
    CONTINUE

END_PROC

functionDescriptorTypeOne currentOutputPortFD = { currentOutputPortCode, 216, TRUE, outputPortP, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** errorPortCode -- Retrieve the error output port. ****/

PROC( errorPortCode )

    MOVE( ErrorPort, R )
    CONTINUE

END_PROC

functionDescriptorTypeOne errorPortFD = { errorPortCode, 246, TRUE, outputPortP, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** kittenBusyCode -- Is the kitten busy? ****/

PROC( kittenBusyCode )

    UnsignedLongWord whichKitten = (UnsignedLongWord)(R.u.c);

    SET_R_FROM( theClowder->getKittenEvaluating( whichKitten )  )
    CONTINUE

END_PROC

functionDescriptorTypeOne kittenBusyFD  = {  kittenBusyCode, 261, TRUE, booleanP, trueBooleanTag, 1, kittenChecker, kittenCheckerNames };

/**** inputQueueCode -- Retrieve the input queue. ****/

PROC( inputQueueCode )

    UnsignedLongWord whichKitten = (UnsignedLongWord)(R.u.c);

    MOVE( *(theClowder->getKIQPointer( whichKitten )), R ) 
    CONTINUE

END_PROC

functionDescriptorTypeOne inputQueueFD  = {  inputQueueCode, 331, TRUE, properListP, trueBooleanTag, 1, kittenChecker, kittenCheckerNames };

/**** kittenDequeueCode -- if Q is empty, return '(); else pop Q and return a list containing what was its first element. ****/

PROC( kittenDequeueCode )

    blockQOnTagLock1( theKittenNumber, 19 );
    if( leaf_p( KIQ.t ) ) {
        R.t = emptyListTag;
        releaseQueue( theKittenNumber, 1 );
        CONTINUE
        }
    if( pair_p( KIQ.t ) )
        KIQ = spliceOutNotCarForwardingPointers( KIQ );
    MOVE( KIQ, R )
    CAR
    NILCONS
    PUSH
    MOVE( KIQ, R )
    CDR
    MOVE( R, KIQ )
    POP
    releaseQueue( theKittenNumber, 2 );

    CONTINUE

END_PROC

/**** kittenEmptyQueueCode -- return whether the queue is empty. ****/

PROC( kittenEmptyQueueCode )

    UnsignedLongWord whichKitten = (UnsignedLongWord)(R.u.c);

    MOVE( *(theClowder->getKIQPointer( whichKitten )), R ) 
    EMPTY_LIST_P
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne kittenEmptyQueueFD  = {  kittenEmptyQueueCode, 339, TRUE, alwaysTrue, trueBooleanTag, 1, kittenChecker, kittenCheckerNames };

/**** purgeKittenQueueCode -- empty the queue. ****/

PROC( purgeKittenQueueCode )

    UnsignedLongWord whichKitten = (UnsignedLongWord)(R.u.c);

    blockQOnTagLock1( theKittenNumber, 20 );
    if( points_to_active_bit( (theClowder->getKIQPointer( whichKitten ))->t ) )
//        (theClowder->getKIQPointer( whichKitten ))->t = (emptyListTag | pointsToActiveBitMask);  // OUT FOR TEST 2009.12.26
        (theClowder->getKIQPointer( whichKitten ))->t = emptyListTag;
    else
        (theClowder->getKIQPointer( whichKitten ))->t = emptyListTag;
    releaseQueue( theKittenNumber, 3 );
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne purgeKittenQueueFD  = {  purgeKittenQueueCode, 333, TRUE, alwaysTrue, trueBooleanTag, 1, kittenChecker, kittenCheckerNames };

/**** tellKittenCode -- Add the entity in R to the tail of the queue.

       This version uses a passed argument to determine which kitten is being told.

       ****/

PROC(  tellKittenCode )

    Word whichKitten = (Word)(((S[0]).u.c) & WORD_MASK);

    if( applicativeMode() ) {

        //  If "X" is what the kitten is to be told, then what we must actually tell is
        //    ((lambda () (c::set-applicative-bit-n-down! 0) X)).  The compiled lambda for this is
        //
        //    '(() ((c::set-applicative-bit-n-down! 0) X) <top-level-environment> )

        PUSH                                     //  S: whatToDo ...
        LOAD_R_TAG( emptyListTag )
        PUSH                                     //  S: '() whatToDo ...

        LOAD_R_TAG( exactShortFixnumBitsMask )
        LOAD_OBJECT( NameStrings[ 454 ], R );    //  R: "c::set-applicative-bit-n-down!"
        CALL( intern )

        PUSH                                     //  S: c::set-applicative-bit-n-down! '() whatToDo ...
        MOVE( zero, R )
        NILCONS                                  //  R: '(0)
        CONS                                     //  R: (c::set-applicative-bit-n-down! 0)  S: '() whatToDo ...
        PUSH                                     //  S: (c::set-applicative-bit-n-down! 0) '() whatToDo ...
        MOVE( S[2], R )
        NILCONS                                  //  R: (whatToDo)
        CONS                                     //  R: ((c::set-applicative-bit-n-down! 0) whatToDo)  S: '() whatToDo ...
        PUSH                                     //  S: ((c::set-applicative-bit-n-down! 0) whatToDo) '() whatToDo ...

        MOVE( TopLevelE, R )
        
        NILCONS                                  //  R: (TopLevelE)
        CONS                                     //  S: '() whatToDo ...
        CONS                                     //  S: whatToDo ...
        DROP

        //  R is now ((() ((c::set-applicative-bit-n-down! 0) whatToDo) topLevelE))
        LOAD_R_TAG( lambdaPairTag )
        NILCONS
        }

    blockQOnTagLock1( whichKitten, 21 );
    DROP
    if( leaf_p( (theClowder->getKIQPointer( whichKitten ))->t ) ) { 
        NILCONS 
        MOVE( R, *(theClowder->getKIQPointer( whichKitten )) ) 
        releaseQueue( whichKitten, 4 );
        RETURN_DEFAULT
        CONTINUE 
        } 
    NILCONS 
    PUSH 
    if( pair_p( (theClowder->getKIQPointer( whichKitten ))->t ) )
        theClowder->spliceOutKIQPointer( whichKitten );
    MOVE( (*((theClowder->getKIQPointer( whichKitten )))), R )         // R: the queue; S: the new last pair
    CALL( lastPairDangerous ) 
    if( R.u.a == ((theClowder->getKIQPointer( whichKitten ))->u).a ) { 
        // Special case -- R is Q, which is blocked, so we can't just set-cdr! it
        CAR
        SWAP                                                 // R: the new last pair; S: the first item of the old queue
        CONS                                                 // R: the new queue
        PUSH 
        MOVE( *(theClowder->getKIQPointer( whichKitten )), R )   
        SWAP                                                 // R: the new queue; S: the old queue
        MOVE( R, *(theClowder->getKIQPointer( whichKitten )) ) 
        theClowder->addToLiveAddressStack( theClowder->getKIQPointer( whichKitten ) );
        POP 
        releaseQueue( whichKitten, 5 );
        } 
    else {                                    
        SWAP                                                 // R: the new last pair; S: the old last pair
        SETCDR 
        releaseQueue( whichKitten, 6 );
        } 
    RETURN_DEFAULT
    CONTINUE 

END_PROC

functionDescriptorTypeOne tellKittenFD  = {  tellKittenCode, 356, TRUE, alwaysTrue, trueBooleanTag, 2, kittenAnyChecker, kittenAnyCheckerNames };

/**** fileExistsCode -- Does the named file exist? ****/

PROC( fileExistsPCode )

    FILE_EXISTS_P
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne fileExistsPFD = { fileExistsPCode, 289, TRUE, booleanP, trueBooleanTag, 1, stringChecker, stringCheckerNames };

/**** getStartupFolderCode -- Retrieve the startup folder. ****/

PROC( getStartupFolderCode )

    GET_STARTUP_WORKING_FOLDER
    CONTINUE

END_PROC

functionDescriptorTypeOne getStartupFolderFD = { getStartupFolderCode, 81, TRUE, stringP, stringTag, 0, anyChecker, anyCheckerNames };

/**** getWorkingFolderCode -- Retrieve the current working folder. ****/

PROC( getWorkingFolderCode )

    GET_WORKING_FOLDER 
    CONTINUE

END_PROC

functionDescriptorTypeOne getWorkingFolderFD = { getWorkingFolderCode, 38, TRUE, stringP, stringTag, 0, anyChecker, anyCheckerNames };

/**** getInitFilePathCode -- Retrieve the path to the directory where the program was. ****/

PROC( getInitFilePathCode )

    GET_INIT_FILE_PATH
    CONTINUE

END_PROC

functionDescriptorTypeOne getInitFilePathFD = { getInitFilePathCode, 300, TRUE, stringP, stringTag, 0, anyChecker, anyCheckerNames };

/**** inputPortPCode -- Is this an input port? ****/

PROC( inputPortPCode )

    INPUT_PORT_P
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne inputPortPFD = { inputPortPCode, 202, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** assertFileLoadSucceeded ****/

Text lastFileLoadedName[ MAX_PATHNAME_LENGTH ];

void assertFileLoadSucceeded() { reportEvent( FILE_LOAD_SUCCEEDED_EVENT ); }

/**** loadCode -- Load a file. ****/             

PROC( loadCode )

    strcpy( lastFileLoadedName, "");

    ONE_FIXNUM_P
    JUMP_TRUE( givenFileName )

    OPEN_INPUT_SF
    JUMP_LABEL( fileIsOpen )

  givenFileName:
    POP
    OPEN_INPUT_NAMED                    /* R: inputPort */

  fileIsOpen:
    EMPTY_LIST_P
    JUMP_FALSE( goodDescriptor )
    resetError( 172 );

  goodDescriptor:       
    PUSH
    strncpy( lastFileLoadedName, (getPortFD( &R ))->pathName, MAX_PATHNAME_LENGTH - 1 );
    reportEvent( FILE_LOAD_BEGINNING_EVENT );

    R.u.c = MakeRTrueFunction_index;            R.t = schemeMachinePCIndexTag;    CSPush( R );
    R.u.c = assertFileLoadSucceeded_index;      R.t = schemeMachinePCIndexTag;    CSPush( R );
    R.u.c = popToEFunction_index;               R.t = schemeMachinePCIndexTag;    CSPush( R );
    R.u.c = SPopRFunction_index;                R.t = schemeMachinePCIndexTag;    CSPush( R );
    R.u.c = restoreInputPort_index;             R.t = schemeMachinePCIndexTag;    CSPush( R );
    R = DefaultInputPort;                                                         CSPush( R );
    R.u.c = closePort_index;                    R.t = schemeMachinePCIndexTag;    CSPush( R );
    POP
    PUSH
    /* R is the load port */                                                      CSPush( R );
    R.u.c = peekLoop_index;                     R.t = schemeMachinePCIndexTag;    CSPush( R );
    R.u.c = restoreInputPort_index;             R.t = schemeMachinePCIndexTag;    CSPush( R );

    MOVE( E, R )
    MOVE( TopLevelE, E )
    SWAP
    PUSH

//    POP
//    PUSH

    DefaultInputPort = R;

    /*  Continuation:
          restoreInputPort (from R, which is loadPort )
          peekLoop (which still has the load port in R, since restoreInputPort does not change it)
          loadPort
          closePort (will close loadPort, in R)
          DefaultInputPort
          restoreInputPort (from R, which is defaultInputFD )
          SPopRFunction (which will pull off the stack, the "loadPort" that was pushed herein)
          popToEFunction (which will restore the saved value of E from the top of the stack)
          assertFileLoadSucceded (which will report success to the I/O mechanism)
          MakeRTrueFunction (which will return #t
     */

    CONTINUE

END_PROC

functionDescriptorTypeSeven loadFD = { loadCode, 204, FALSE, alwaysTrue, trueBooleanTag, 1,
                                       stringP, alwaysTrue, 0, ObjectErrorNameStrings[ 11 ], ObjectErrorNameStrings[ 0 ] };

/**** openInputFileCode -- Open a file for input.  If there is one argument, it had better be
                           a string that names the file.  If there are no arguments, call an
                           interactive routine to get the file name.                           ****/

PROC( openInputFileCode )

    ONE_FIXNUM_P
    JUMP_TRUE( givenFileName )

    OPEN_INPUT_SF
    CONTINUE

  givenFileName:
    POP
    OPEN_INPUT_NAMED
    CONTINUE

END_PROC

functionDescriptorTypeSeven openInputFileFD = { openInputFileCode, 198, TRUE, inputPortP, portTag, 1,
                                                stringP, alwaysTrue, 0, ObjectErrorNameStrings[ 11 ], ObjectErrorNameStrings[ 0 ] };

/**** openOutputFileCode -- Open a file for output.  If there is one argument, it had better be
                            a string that names the file.  If there are no arguments, call an
                            interactive routine to get the file name.                           ****/

PROC( openOutputFileCode )

    ONE_FIXNUM_P
    JUMP_TRUE( givenFileName )

    OPEN_OUTPUT_SF
    CONTINUE

  givenFileName:
    POP
    OPEN_OUTPUT_NAMED
    CONTINUE

END_PROC

functionDescriptorTypeSeven openOutputFileFD =
    { openOutputFileCode, 199, TRUE, outputPortP, portTag, 1,
      stringP, alwaysTrue, 0, ObjectErrorNameStrings[ 11 ], ObjectErrorNameStrings[ 0 ] };

/**** outputPortPCode -- Is this an output port? ****/

PROC( outputPortPCode )

    OUTPUT_PORT_P
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne outputPortPFD = 
    { outputPortPCode, 203, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** peekCharCode -- Peek one character from the port given (or default to the current one). ****/

PROC( peekCharCode )

    ZERO_FIXNUM_P
    JUMP_FALSE( portGiven )
    CHAR_READY_P
    JUMP_FALSE( noChar2 )
    PEEKC
    EOF_P
    JUMP_TRUE( noChar2 )
    CONTINUE

  portGiven:
    R = DefaultInputPort;
    SWAP
    DefaultInputPort = R;
    CHAR_READY_P
    JUMP_FALSE( noChar1 )
    PEEKC
    EOF_P
    JUMP_TRUE( noChar1 )
    DefaultInputPort = STACK( 0 );
    DROP    
    CONTINUE

  noChar1:
    DefaultInputPort = STACK( 0 );
    DROP

  noChar2:
    LOAD_R_TAG( peculiarLeafTag )
    LOAD_OBJECT( 0, R ) 
    orIntoLeafByteNonAtomic( R.u.c, eofObjectPLTag );
    CONTINUE

END_PROC

functionDescriptorTypeSeven peekCharFD = { peekCharCode, 304, TRUE, characterP, characterTag, 1,
                                           inputPortP, alwaysTrue, 0, ObjectErrorNameStrings[ 9 ], ObjectErrorNameStrings[ 0 ] };

/**** readCharCode -- Read one character from the port given (or default to the current one). ****/

PROC( readCharCode )

    ZERO_FIXNUM_P
    JUMP_FALSE( portGiven )
    CHAR_READY_P
    JUMP_FALSE( noChar2 )
    GETC_RAW
    EOF_P
    JUMP_TRUE( noChar2 )
    CONTINUE

  portGiven:
    R = DefaultInputPort;
    SWAP
    DefaultInputPort = R;
    CHAR_READY_P
    JUMP_FALSE( noChar1 )
    GETC_RAW
    EOF_P
    JUMP_TRUE( noChar1 )
    DefaultInputPort = STACK( 0 );
    DROP    
    CONTINUE

  noChar1:
    DefaultInputPort = STACK( 0 );
    DROP

  noChar2:
    LOAD_R_TAG( peculiarLeafTag )
    LOAD_OBJECT( 0, R ) 
    orIntoLeafByteNonAtomic( R.u.c, eofObjectPLTag );
    CONTINUE

END_PROC

functionDescriptorTypeSeven readCharFD = { readCharCode, 218, TRUE, characterP, characterTag, 1,
                                           inputPortP, alwaysTrue, 0, ObjectErrorNameStrings[ 9 ], ObjectErrorNameStrings[ 0 ] };

/**** readCode -- Read one S-expression from the port given (or default to the current one). ****/

PROC( readCode )

    ZERO_FIXNUM_P
    JUMP_FALSE( portGiven )
    CALL( readSexp )
    CONTINUE

  portGiven:
    CALL( readSexpFromPort )
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeSeven readFD = { readCode, 213, TRUE, alwaysTrue, trueBooleanTag, 1,
                                       inputPortP, alwaysTrue, 0, ObjectErrorNameStrings[ 9 ], ObjectErrorNameStrings[ 0 ] };

/**** setInputPortCode -- Redirect input from the given input file. ****/

PROC( setInputPortCode )

    applicativeCheckDropNone( NameStrings[ 212 ] );

    ZERO_FIXNUM_P
    JUMP_FALSE( givenFileName )

    OPEN_INPUT_SF
    JUMP_LABEL( doTheCall )

  givenFileName:
    POP
    OPEN_INPUT_NAMED                                /* R: port */

  doTheCall:                                        /* R: port */
    EMPTY_LIST_P
    JUMP_FALSE( goodDescriptor )
    resetError( 172 );

  goodDescriptor:       
    DefaultInputPort = R;
    CurrentInputPort = R;
    MasterDefaultInputPort = R;
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeSeven setInputPortFD = { setInputPortCode, 212, FALSE, alwaysTrue, trueBooleanTag, 1,
                                               stringP, alwaysTrue, 0, ObjectErrorNameStrings[ 11 ], ObjectErrorNameStrings[ 0 ] };

/**** setWorkingFolderCode -- Set the working folder. ****/              

PROC( setWorkingFolderCode )

    applicativeCheckDropNone( NameStrings[ 12 ] );

    ONE_FIXNUM_P
    JUMP_TRUE( givenFolderName )

    OPEN_DIRECTORY_SF
    SET_WORKING_FOLDER                      /* R: folderName */
    RETURN_DEFAULT
    CONTINUE

  givenFolderName:
    POP
    SET_WORKING_FOLDER                      /* R: folderName */
    RETURN_DEFAULT

    CONTINUE

END_PROC

functionDescriptorTypeSeven setWorkingFolderFD = 
    { setWorkingFolderCode, 12, FALSE, alwaysTrue, trueBooleanTag, 1,
      stringP, alwaysTrue, 0, ObjectErrorNameStrings[ 11 ], ObjectErrorNameStrings[ 0 ] };

/**** transcriptOffCode -- Close the transcript file, if any. ****/

PROC( transcriptOffCode )

    CLOSE_TRANSCRIPT
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne transcriptOffFD = { transcriptOffCode, 211, FALSE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** transcriptOnCode -- Open a file for transcript.  If there is one argument, it had better be
                          a string that names the file.  If there are no arguments, call an
                          interactive routine to get the file name.                            ****/

PROC( transcriptOnCode )

    ONE_FIXNUM_P
    JUMP_TRUE( givenFileName )

    OPEN_TRANSCRIPT_SF
    RETURN_DEFAULT
    CONTINUE

  givenFileName:
    POP
    OPEN_TRANSCRIPT_NAMED
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeSeven transcriptOnFD = { transcriptOnCode, 210, FALSE, alwaysTrue, trueBooleanTag, 1,
                                               stringP, alwaysTrue, 0, ObjectErrorNameStrings[ 11 ], ObjectErrorNameStrings[ 0 ] };

/**** withInputFileCode -- Call the given function with an input file. ****/     

PROC( withInputFileCode )

    ONE_FIXNUM_P
    JUMP_FALSE( givenFileName )

    OPEN_INPUT_SF
    JUMP_LABEL( doTheCall )

  givenFileName:
    POP
    SWAP
    OPEN_INPUT_NAMED                                /* R: port;     S: (procedure ...) */

  doTheCall:                                        /* R: port;     S: (procedure ...) */
    EMPTY_LIST_P
    JUMP_FALSE( goodDescriptor )
    resetError( 172 );

  goodDescriptor:
    MAKE_FRAME                                      /* CHANGE FOR NEW FRAME SIZE */
    PUSH
    PUSH
    PUSH
    PUSH                                            /* R: port;       S: (P P P P X X X X X X X X procedure ...) */
    EXCHANGE( STACK( 4 + FrameSize ), STACK( 2 + FrameSize ) )     // S: (P P P P X X X X X X procedure X X ...)
    MOVE( CurrentInputPort, STACK( 3 + FrameSize ) )               // S: (P P P P X X X X X X procedure savedCIP X ...)
    MOVE( DefaultInputPort, STACK( 4 + FrameSize ) )               // S: (P P P P X X X X X X procedure savedCIP savedDIP ...)
    MOVE( R, CurrentInputPort )                     // R: port;       S: (P P P P X X X X X X procedure savedCIP savedDIP ...)
    MOVE( R, DefaultInputPort )                     // R: port;       S: (P P P P X X X X X X procedure savedCIP savedDIP ...)
    EXCHANGE( STACK( 0 ), STACK( 2 + FrameSize ) )  //                S: (procedure X X X X X X X X X P savedCIP savedDIP ...)
    MOVE( zero, STACK( 1  ) )                       //                S: (procedure zero X X X X X X X X P savedCIP savedDIP ...)

    LOAD_R_TAG( schemeMachinePCIndexTag )
    LOAD_OBJECT( closeStackedPortRestoreInput_index, R )
    PUSH_CONTINUATION
    LOAD_FRAME( 2, 0 )
    LOAD_R_TAG( schemeMachinePCIndexTag )
    LOAD_OBJECT( apply_index, R )
    CSPush( R );
//    NILCONS
//    MOVE( R, C )

    CONTINUE

END_PROC

functionDescriptorTypeEight withInputFileFD = { withInputFileCode, 208, TRUE, alwaysTrue, trueBooleanTag, 2,
                                                stringP, procedureP, 1, ObjectErrorNameStrings[ 11 ], ObjectErrorNameStrings[ 23 ] };

/**** withOutputFileCode -- Call the given function with an output file. ****/ 

PROC( withOutputFileCode )

    ONE_FIXNUM_P
    JUMP_FALSE( givenFileName )

    OPEN_OUTPUT_SF
    JUMP_LABEL( doTheCall )

  givenFileName:
    POP
    SWAP
    OPEN_OUTPUT_NAMED                               /* R: port;     S: (procedure ...) */

  doTheCall:                                        /* R: port;     S: (procedure ...) */
    EMPTY_LIST_P
    JUMP_FALSE( goodDescriptor )
    resetError( 172 );

  goodDescriptor:
    MAKE_FRAME                                      /* CHANGE FOR NEW FRAME SIZE */
    PUSH
    PUSH
    PUSH                                            /* R: port;    S: (port port port X X X X X X X X procedure ...) */
    EXCHANGE( STACK( 3 + FrameSize ), STACK( 2 + FrameSize ) )
    MOVE( CurrentOutputPort, STACK( 3 + FrameSize ) )
    MOVE( R, CurrentOutputPort )                    /* R: port;    S: (port port port X X X X X X X procedure savedOutputPort ...) */
    EXCHANGE( STACK( 0 ), STACK( 2 + FrameSize ) )  /*             S: (procedure port port X X X X X X X port savedOutputPort ...) */
    MOVE( zero, STACK( 1  ) )                       /*             S: (procedure zero port X X X X X X X port savedOutputPort ...) */

    LOAD_R_TAG( schemeMachinePCIndexTag )
    LOAD_OBJECT( closeStackedPortRestoreOutput_index, R )
    PUSH_CONTINUATION
    LOAD_FRAME( 2, 0 )
    LOAD_R_TAG( schemeMachinePCIndexTag )
    LOAD_OBJECT( apply_index, R )
    CSPush( R );
//    NILCONS
//    MOVE( R, C )

    CONTINUE

END_PROC

functionDescriptorTypeEight withOutputFileFD = { withOutputFileCode, 209, TRUE, alwaysTrue, trueBooleanTag, 2,
                                                 stringP, procedureP, 1, ObjectErrorNameStrings[ 11 ], ObjectErrorNameStrings[ 23 ] };

/**** vectorCode -- There is some common code in an auxiliary that does what we need. ****/

functionDescriptorTypeFive vectorFD = { makeVectorAndStuffFromStack, 189, TRUE, vectorP, vectorTag, 0,
                                        alwaysTrue, ObjectErrorNameStrings[ 0 ] };

// Short routines to enable and disable console output, and to check whether console output is enabled.

PROC( disableConsoleOutputCode )

    consoleOutputEnabledFlag = FALSE;
    RETURN_DEFAULT

END_PROC

functionDescriptorTypeOne disableConsoleOutputFD
    = {  disableConsoleOutputCode, 363, TRUE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

PROC( enableConsoleOutputCode )

    consoleOutputEnabledFlag = TRUE;
    RETURN_DEFAULT

END_PROC

functionDescriptorTypeOne enableConsoleOutputFD
    = {  enableConsoleOutputCode, 364, TRUE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

PROC( consoleOutputEnabledPCode )

    SET_R_FROM( consoleOutputEnabledFlag )

END_PROC

functionDescriptorTypeOne consoleOutputEnabledPFD
    = { consoleOutputEnabledPCode, 365, TRUE, booleanP, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** kittenResetCode -- Post a request reset event to the kitten in question. ****/

PROC( kittenResetCode )

    Word whichKitten = (Word)(R.u.c);
    Text systemBuffer[ 256 ];

    sprintf( systemBuffer, "kill -%d %d", SIGINT, theClowder->getKittenPID( whichKitten ) );
    system( systemBuffer );
    RETURN_DEFAULT

END_PROC

functionDescriptorTypeOne kittenResetFD  = {  kittenResetCode, 366, TRUE, alwaysTrue, trueBooleanTag, 1, kittenChecker, kittenCheckerNames };

/**** loadFromStringCode -- take input from the Scheme string in R. ****/

PROC( loadFromStringCode )

    loadFromString( startString( &R ) );

    RETURN_DEFAULT

END_PROC

functionDescriptorTypeOne loadFromStringFD  = {  loadFromStringCode, 423, TRUE, alwaysTrue, trueBooleanTag, 1, stringChecker, stringCheckerNames };

/******** Socket stuff starts here ********/

#import <sys/socket.h>
#import <sys/un.h>

typedef struct socketaddr_un *pSAun;
extern int errno;

/****  For "c::socket-create-and-listen" -- socketCreateAndListen ****/

PROC( socketCreateAndListen )

    // String to name socket is on top of stack, backlog is in R

    struct sockaddr_un answerAddr;

    strcpy( answerAddr.sun_path, startString( &(S[ 0 ]) ) );
    answerAddr.sun_family = AF_UNIX;

    SWAP
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( socket( AF_UNIX, SOCK_STREAM, 0 ), R )

    if( R.u.c == -1 ) {
        tprintf5( &ErrorPort, MiscStrings[ 445 ], answerAddr.sun_path, errno, strerror( errno ) );
        POP
        resetError( 216 );
        CONTINUE
        }
    if( bind( (Word)(R.u.c & WORD_MASK), 
              (const sockaddr *)&answerAddr,
              (Word)(strlen( answerAddr.sun_path ) + sizeof( answerAddr.sun_family ) + 1) )
        == -1 ) {
        tprintf5( &ErrorPort, MiscStrings[ 446 ], answerAddr.sun_path, errno, strerror( errno ) );
        POP
        resetError( 217 );
        CONTINUE
        }
    if( listen( (Word)(R.u.c & WORD_MASK), (Word)(S[0].u.c & WORD_MASK) ) == -1 ) {
        tprintf5( &ErrorPort, MiscStrings[ 403 ], answerAddr.sun_path, errno, strerror( errno ) );
        POP
        resetError( 218 );
        CONTINUE
        }

    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne socketCreateAndListenFD = { socketCreateAndListen, 357, TRUE, integerP,
                                                      shortFixnumBitsMask, 2, stringIntegerChecker, stringIntegerCheckerNames };

/****  For "c::socket-accept" -- socketAccept ****/

PROC( socketAccept )

    // Socket number is in R.

    struct sockaddr_un theAddr;
    int theSocket = (Word)(R.u.c & WORD_MASK);
    socklen_t theLength = (Word)(sizeof( struct sockaddr_un ));
    
    LOAD_OBJECT( accept( (Word)(R.u.c & WORD_MASK), (sockaddr *)(&theAddr), &theLength ), R )

    if( R.u.c == -1 ) {
        tprintf5( &ErrorPort, MiscStrings[ 513 ], theSocket, errno, strerror( errno ) );
        resetError( 219 );
        CONTINUE
        }
    CONTINUE

END_PROC

functionDescriptorTypeOne socketAcceptFD = { socketAccept, 358, TRUE, integerP, shortFixnumBitsMask, 1, integerChecker, integerCheckerNames };

/****  For "c::socket-write" -- socketWrite ****/

PROC( socketWrite )

    // Socket number on stack, string in R.

    write( (Word)((S[0]).u.c & WORD_MASK), startString( &R ), strlen( startString( &R ) ) );
    RETURN_DEFAULT

END_PROC

functionDescriptorTypeOne socketWriteFD = { socketWrite, 359, FALSE, alwaysTrue, trueBooleanTag, 2, integerStringChecker, integerStringCheckerNames };

/****  For "c::open-input-socket" -- openInputSocketCode -- Open a known socket for input. ****/

PROC( openInputSocketCode )

    OPEN_READ_SOCKET_GIVEN
    CONTINUE

END_PROC

functionDescriptorTypeOne openInputSocketFD = { openInputSocketCode, 361, TRUE, inputPortP, portTag, 1, integerChecker, integerCheckerNames };

/****  For "c::socket-fgets" -- socketFGets ****/

#define SOCKET_BUFFER_LENGTH 128

PROC( socketFGets )

    // Socket read port in R.  Does not check for length.

    Text buffer[ SOCKET_BUFFER_LENGTH ];
    UnsignedText *cs = (UnsignedText *)buffer;
    Word n = SOCKET_BUFFER_LENGTH;
    FILE *fp = getPortFilePointer( &R );
    Word c = 0;

//    fprintf( kittenLogFile, "Enter socket loop for socket 0x%016lu, n is %d\n", (UnsignedWord)fp, n); fflush( kittenLogFile );
    *cs = '\0';
    theHeap->triggerGC();
    fp = getPortFilePointer( &R );

    while( TRUE ) {
//        fprintf( kittenLogFile, "Socket loop for socket 0x%016lu, n is %d\n", (UnsignedWord)fp, n); fflush( kittenLogFile );

        if( theHeap->triggerGC() )
            fp = getPortFilePointer( &R );

        if( (c = fgetc( fp )) >= 0 ) {
            if( --n <= 0 )
                break;
            *cs++ = c;
            if( c == '\n')
                break;
            }
        else
            clearerr( fp );
        }
//    fprintf( kittenLogFile, "Leave socket loop for socket 0x%016lu, n is %d\n", (UnsignedWord)fp, n); fflush( kittenLogFile );
        
    *cs++ = '\0';
    makeString( buffer );

END_PROC

functionDescriptorTypeOne socketFGetsFD = { socketFGets, 360, TRUE, stringP, stringTag, 1, inputPortChecker, inputPortCheckerNames };

