/****************************************************************************************

SchemeFunctions7.c -- Copyright (c) by Jay Reynolds Freeman 1988, 1989, 1990, 1991, 2006

Scheme built-in string-manipulation functions for Wraith Scheme.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import "WraithHeader.h"
#import "MemoryManager.h"
#import "OpCodes.h"
#import "Numbers.h"
#import "SchemeFunctions.h"
#import "DataTypes.e"
#import "Globals.e"
#import "Intern.e"
#import "IO.e"
#import "SchemeAux.e"
#import "SchemeFunctionStatics.h"

/******** Stuff for Scheme built-in functions; code and associated statics ********/

/**** stringPCode -- Is R a string? ****/

PROC( stringPCode )

    STRING_P 
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne stringPFD = { stringPCode, 181, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** listToStringCode -- Coerce a list of characters to the obvious string.    ****/

PROC( listToStringCode )

    PUSH
    LENGTH
    MAKE_EMPTY_STRING
    PUSH
    MOVE( zero, R)
    PUSH
    MOVE( STACK( 2 ), R )       /* R: list; S: (position str list ) */

  doneYet:
    EMPTY_LIST_P
    JUMP_TRUE( done )
    CAR                         /* R: char; S: (position str list ) */
    CHARACTER_P
    JUMP_FALSE( notChar )
    STRING_SET
    MOVE( one, R )
    ADD                         /* R: newPosition; S: (str list ) */
    PUSH
    MOVE( STACK( 2 ), R )       /* R: list; S: (newPposition str list, ) */
    CDR
    MOVE( R, STACK( 2 ) )       /* R: newList; S: (newPosition str newList, ) */
    JUMP_LABEL( doneYet )

  done:
    DROP
    POP
    DROP
    CONTINUE

  notChar:
    OUTPUT_TO_ERROR
    WRITE
    resetError( 281 );
    CONTINUE        

END_PROC

functionDescriptorTypeOne listToStringFD = { listToStringCode, 125, TRUE, stringP, stringTag, 1, listChecker, listCheckerNames };

/**** numberToStringViaSPRintfCode -- Convert the given number in STACK( 1 ) to a string, using the
                                      "C" format string that is in STACK( 0 ).  ****/

PROC( numberToStringViaSPrintfCode )

#define NUMBER_BUFFER_SIZE 256

    static Text scratch[ NUMBER_BUFFER_SIZE ];

    POP
    SWAP
    CALL( coerceRToShortFloat )
    snprintf( scratch, NUMBER_BUFFER_SIZE, startString( &(STACK( 0 )) ), R.u.f );
    LOAD_OBJECT( scratch, R )
    MAKE_STRING
    DROP
    CONTINUE
   
END_PROC    

functionDescriptorTypeSeven numberToStringWithCFormatFD = 
    { numberToStringViaSPrintfCode, 245, TRUE, stringP, stringTag, 2,
      numberP, stringP, 2, ObjectErrorNameStrings[ 18 ], ObjectErrorNameStrings[ 11 ] };

/**** numberToStringCode -- Convert the number to a string.     ****/

PROC( numberToStringCode )

    ONE_FIXNUM_P
    POP
    JUMP_FALSE( radixGiven )
    PUSH
    MOVE( ten, R )

  radixGiven:
    SWAP
    NUMBER_TO_STRING
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeSeven numberToStringFD = 
    { numberToStringCode, 223, TRUE, stringP, stringTag, 2,
      numberP, radixP, 1, ObjectErrorNameStrings[ 18 ], ObjectErrorNameStrings[ 30 ] };

/**** stringAppendCode -- Append together some strings in a newly-allocated destination.  ****/

PROC( stringAppendCode )

    REVERSE_TOP_N
    PUSH                                        /* R: <X>;      S: (nStr str* ) */
    STRING_LENGTH_MANY
    MAKE_EMPTY_STRING                           /* destination for all the copies */
                                                /* R: destSt;   S: (nStr str* ) */
    SWAP                   /* R: nStr;     S: (destStr str* ) */
    PUSH                                        /* R: nStr;     S: (nStr destStr str* ) */
    MOVE( zero, STACK( 0 ) )                    /* R: nStr;     S: (zero destStr str* ) */
    EXCHANGE( STACK( 0 ), STACK( 1 ) )          /* R: nStr;     S: (destStr zero str* ) */

  insertLoop:                                   /* R: nStr;     S: (destStr pos str* ) */
    ZERO_FIXNUM_P
    JUMP_TRUE( done )
    EXCHANGE( R, STACK( 2 ) )                   /* R: thisStr;  S: (destStr pos nStr str* ) */
    STRING_INSERT
    STRING_LENGTH                               /* R: thisLnth; S: (destStr pos nStr str* ) */
    PUSH
    MOVE( STACK( 2 ), R )                       /* R: pos;      S: (thisLnth destStr pos nStr str* ) */
    ADD                                         /* R: newPos;   S: (destStr pos nStr str* ) */
    EXCHANGE( R, STACK( 2 ) )                   /* R: nStr;     S: (destStr pos newPos str* ) */
    EXCHANGE( STACK( 0 ), STACK( 1 ) )          /* R: nStr;     S: (pos destStr newPos str* ) */
    DROP                                        /* R: nStr;     S: (destStr newPos str* ) */
    DECREMENT_R                                 /* R: newNStr;  S: (destStr newPos str* ) */
    JUMP_LABEL( insertLoop )

  done:
    POP
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeFive stringAppendFD = { stringAppendCode, 165, TRUE, stringP, stringTag, 0,
                                              stringP, ObjectErrorNameStrings[ 11 ] };

/**** stringCopyCode -- Make a newly-allocated copy of the given string.        ****/

functionDescriptorTypeOne stringCopyFD = { stringCopy, 171, TRUE, stringP, stringTag, 1, stringChecker, stringCheckerNames };

/**** stringFillCode -- There is an auxiliary for this; the string is in 
                        STACK( 0 ) and the character to fill it with is in R.            ****/

PROC( stringFillCode )

    applicativeCheckDropOne( NameStrings[ 172 ] );

    SWAP
    IMMUTABLE_STRING_P
    SWAP
    JUMP_TRUE( stringFillImmutableError )
    CALL( stringFillAux )
    DROP
    RETURN_DEFAULT
    CONTINUE

  stringFillImmutableError:
    POP
    OUTPUT_TO_ERROR
    WRITE
    resetError( 247 );
    CONTINUE

END_PROC

functionDescriptorTypeOne stringFillFD = { stringFillCode, 172, FALSE, alwaysTrue, trueBooleanTag, 2, stringCharChecker, stringCharCheckerNames };

/**** downInFlamesCode -- Crash Wraith Scheme with the error message given. ****/

PROC( downInFlamesCode )

    fatalError( 120 );

 foo:
    JUMP_LABEL( foo )

    CONTINUE

END_PROC

functionDescriptorTypeOne downInFlamesFD = { downInFlamesCode, 316, FALSE, alwaysTrue, emptyListTag, 0, anyChecker, anyCheckerNames };

/**** stringLengthCode -- How long is the string in R? ****/

functionDescriptorTypeOne stringLengthFD = { stringLength, 173, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 1, stringChecker, stringCheckerNames };

/**** stringRefCode -- String in STACK( 0 ), zero-based cursor into it in R.    ****/

PROC( stringRefCode )

    EXACT_P
    JUMP_FALSE( needExact )
    COERCE_R_SHORT_FIXNUM
    PUSH
    MOVE( STACK( 1 ), R )
    STRING_LENGTH
    NUM_LT_P
    JUMP_FALSE( highRange )
    POP
    STRING_REF
    DROP
    CONTINUE

  highRange:
    POP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 88 );
    CONTINUE

  needExact:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 89 );
    CONTINUE

END_PROC

functionDescriptorTypeOne stringRefFD = { stringRefCode, 174, TRUE, characterP, characterTag, 2, stringNonNegativeIntegerChecker, stringNonNegativeIntegerCheckerNames };

/**** stringSetCode  -- String in STACK( 1 ), zero-based cursor into it in STACK( 1 ),
                        value to be placed in that position of the string in R.          ****/

PROC( stringSetCode )

    applicativeCheckDropTwo( NameStrings[ 175 ] );

    EXCHANGE( R, STACK( 1 ) )
    IMMUTABLE_STRING_P
    EXCHANGE( R, STACK( 1 ) )
    JUMP_TRUE( stringSetImmutableError )
    COERCE_S0_SHORT_FIXNUM
    EXCHANGE( R, STACK( 1 ) )       /* R: string; S: (index char ) */
    PUSH                                        
    MOVE( STACK( 1 ), R )           /* R: index; S: (string index char ) */
    EXACT_P
    JUMP_FALSE( needExact )
    PUSH
    MOVE( STACK( 1 ), R )
    STRING_LENGTH
    NUM_LT_P
    JUMP_FALSE( highRange )
    DROP
    POP
    EXCHANGE( R, STACK( 1 ) )
    STRING_SET
    RETURN_DEFAULT
    DROP
    DROP
    CONTINUE

  highRange:
    POP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 91 );
    CONTINUE

  needExact:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 92 );
    CONTINUE

  stringSetImmutableError:
    DROP
    POP
    OUTPUT_TO_ERROR
    WRITE
    resetError( 248 );
    CONTINUE

END_PROC

functionDescriptorTypeOne stringSetFD = { stringSetCode, 175, FALSE, alwaysTrue, trueBooleanTag, 3, stringNonNegativeIntegerCharChecker, stringNonNegativeIntegerCharCheckerNames };

/**** stringEqualPCode -- Are the two strings lexicographically equal?  ****/

PROC( stringEqualPCode )

    STRING_EQUAL_P
    SET_R_FROM_B
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne stringEqualPFD = { stringEqualPCode, 178, TRUE, booleanP, trueBooleanTag, 2, stringBinaryChecker, stringBinaryCheckerNames };

/**** stringCIEqualPCode -- Neglecting case differences, are the
                            two strings lexicographically equal?             ****/

PROC( stringCIEqualPCode )

    STRING_TO_LOWER
    SWAP
    STRING_TO_LOWER
    STRING_EQUAL_P
    SET_R_FROM_B
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne stringCIEqualPFD = { stringCIEqualPCode, 168, TRUE, booleanP, trueBooleanTag, 2, stringBinaryChecker, stringBinaryCheckerNames };

/**** stringGreaterEqualPCode -- Is the string in STACK( 0 ) lexicographically
                                 greater than or equal to the string in R?               ****/

PROC( stringGreaterEqualPCode )

    STRING_GE_P
    SET_R_FROM_B
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne stringGreaterEqualPFD =
    { stringGreaterEqualPCode, 179, TRUE, booleanP, trueBooleanTag, 2, stringBinaryChecker, stringBinaryCheckerNames };

/**** stringGreaterPCode -- Is the string in STACK( 0 )
                            lexicographically greater than the string in R?          ****/

PROC( stringGreaterPCode )

    STRING_GT_P
    SET_R_FROM_B
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne stringGreaterPFD = { stringGreaterPCode, 180, TRUE, booleanP, trueBooleanTag, 2, stringBinaryChecker, stringBinaryCheckerNames };

/**** stringLessEqualPCode-- Is the string in STACK( 0 ) lexicographically
                             less than or equal to the string in R?                      ****/

PROC( stringLessEqualPCode )

    STRING_LE_P
    SET_R_FROM_B
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne stringLessEqualPFD = { stringLessEqualPCode, 176, TRUE, booleanP, trueBooleanTag, 2, stringBinaryChecker, stringBinaryCheckerNames };

/**** stringLessPCode-- Is the string in STACK( 0 )
                        lexicographically less than the string in R?             ****/

PROC( stringLessPCode )

    STRING_LT_P
    SET_R_FROM_B
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne stringLessPFD = { stringLessPCode, 177, TRUE, booleanP, trueBooleanTag, 2, stringBinaryChecker, stringBinaryCheckerNames };

/**** stringCIGreaterEqualPCode -- Neglecting case differences, is the string in
                                   STACK( 0 ) lexicographically greater than or equal
                                   to the string in R?                               ****/

PROC( stringCIGreaterEqualPCode )

    STRING_GE_CI_P
    SET_R_FROM_B
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne stringCIGreaterEqualPFD = 
    { stringCIGreaterEqualPCode, 169, TRUE, booleanP, trueBooleanTag, 2, stringBinaryChecker, stringBinaryCheckerNames };

/**** stringCIGreaterPCode -- Neglecting case differences, is the string in
                              STACK( 0 ) lexicographically greater than
                              the string in R?                                           ****/

PROC( stringCIGreaterPCode )

    STRING_GT_CI_P
    SET_R_FROM_B
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne stringCIGreaterPFD = { stringCIGreaterPCode, 170, TRUE, booleanP, trueBooleanTag, 2, stringBinaryChecker, stringBinaryCheckerNames };

/**** stringCILessEqualPCode -- Neglecting case differences, is the string in
                                STACK( 0 ) lexicographically less than or equal
                                to the string in R?                                      ****/

PROC( stringCILessEqualPCode )

    STRING_LE_CI_P
    SET_R_FROM_B
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne stringCILessEqualPFD = 
    { stringCILessEqualPCode, 166, TRUE, booleanP, trueBooleanTag, 2, stringBinaryChecker, stringBinaryCheckerNames };

/**** stringCILessPCode -- Neglecting case differences, is the string in
                           STACK( 0 ) lexicographically less than the string in R?   ****/

PROC( stringCILessPCode )

    STRING_LT_CI_P
    SET_R_FROM_B
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne stringCILessPFD = { stringCILessPCode, 167, TRUE, booleanP, trueBooleanTag, 2, stringBinaryChecker, stringBinaryCheckerNames };

/**** stringToListCode -- Return a list of the characters that comprise the string in R.        ****/

PROC( stringToListCode )

    PUSH
    STRING_LENGTH
    PUSH
    PUSH
    LOAD_R_TAG( emptyListTag )              /* R: list;        S: (position position str ) */
    EXCHANGE( R, STACK( 1 ) )               /* R: position;    S: (position list str ) */
    EXCHANGE( STACK( 0 ), STACK( 2 ) )      /* R: position;    S: (str list position ) */
  doneYet:
    ZERO_FIXNUM_P
    JUMP_TRUE( done )
    PUSH
    MOVE( one, R )
    SUBTRACT
    MOVE( R, STACK( 2 ) )                   /* R: newPosition; S: (str list newPosition ) */
    STRING_REF                              /* R: char;        S: (str list newPosition ) */
    PUSH
    MOVE( STACK( 2 ), R )                   /* R: list;        S: (char str list newPosition ) */
    CONS
    MOVE( R, STACK( 1 ) )                   /* R: list;        S: (str list newPosition ) */
    MOVE( STACK( 2 ), R )                   /* R: newPosition; S: (str list newPosition ) */
    JUMP_LABEL( doneYet )

  done:
    DROP
    POP
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeOne stringToListFD = { stringToListCode, 162, TRUE, properListP, trueBooleanTag, 1, stringChecker, stringCheckerNames };

/**** stringToNumberCode -- Parse the given string as a number and return its value.    ****/

PROC( stringToNumberCode )

    ONE_FIXNUM_P
    POP
    JUMP_FALSE( radixGiven )
    PUSH
    MOVE( ten, R )

  radixGiven:
    SWAP
    STRING_TO_NUMBER
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeSeven stringToNumberFD = { stringToNumberCode, 163, TRUE, numberP, numberBitsMask, 2, 
                                                 stringP, radixP, 1, ObjectErrorNameStrings[ 11 ], ObjectErrorNameStrings[ 30 ] } ;

/**** substringCode -- The string is in STACK( 1 ), the (inclusive) (zero-based) starting
                       cursor is in STACK( 0 ), and the (exclusive) (zero-based) ending
                       cursor is in R.  Construct a newly-allocated copy of the indicated
                       substring.                                                   ****/

PROC( substringCode )

    EXACT_P
    JUMP_FALSE( needExactEnd )
    SWAP
    EXACT_P
    JUMP_FALSE( needExactStart )
    SWAP
    COERCE_R_SHORT_FIXNUM
    COERCE_S0_SHORT_FIXNUM
    PUSH
    MOVE( STACK( 2 ), R )
    STRING_LENGTH
    NUM_LE_P
    JUMP_FALSE( endTooBig )
    EXCHANGE( STACK( 0 ), STACK( 1 ) )
    POP
    SWAP
    NUM_LE_P
    JUMP_FALSE( startExceedsEnd )

    SUBSTRING
    DROP
    DROP
    CONTINUE

  endTooBig:
    POP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 93 );
    CONTINUE

  startExceedsEnd:
    POP
    NILCONS
    CONS
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 95 );
    CONTINUE

  needExactEnd:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 96 );
    CONTINUE

  needExactStart:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 97 );
    CONTINUE

END_PROC

functionDescriptorTypeOne substringFD = { substringCode, 182, TRUE, stringP, stringTag, 3, stringNonNegativeIntegerNonNegativeIntegerChecker, stringNonNegativeIntegerNonNegativeIntegerCheckerNames };

