/****************************************************************************************

SchemeFunctions8.c -- Copyright (c) by Jay Reynolds Freeman 1988, 1989, 1990, 1991, 2006

Scheme built-in functions with variable arity from one up, for Wraith Scheme.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import <stdio.h>
#import "LocalTypes.h"
#import "WraithHeader.h"
#import "MemoryManager.h"
#import "OpCodes.h"
#import "SchemeFunctions.h"
#import "DataTypes.e"
#import "Globals.e"
#import "Intern.e"
#import "IO.e"
#import "SchemeAux.e"
#import "SchemeFunctionStatics.h"

/******** Stuff for Scheme built-in functions; code and associated statics ********/

/**** atanCode -- Return the atan of the number in R.   ****/

PROC( atanCode )

    ONE_FIXNUM_P
    JUMP_FALSE( twoArgs )
    POP
    ATAN
    CONTINUE

  twoArgs:
    POP
    ATAN2
    DROP
    CONTINUE

END_PROC

functionDescriptorTypeSix atanFD = { atanCode, 19, TRUE, numberP, numberBitsMask, 2,
                                     numberP, numberP, ObjectErrorNameStrings[ 18 ], ObjectErrorNameStrings[ 18 ] };

/**** applyCode -- Apply a procedure to the (unevaluated) arguments given.
                   This function handles "rest" arguments, etcetera.                       ****/

functionDescriptorTypeSeven applyFD = { setUpApply, 14, TRUE, alwaysTrue, trueBooleanTag, -1,
                                        procedureP, alwaysTrue, 2, ObjectErrorNameStrings[ 23 ], ObjectErrorNameStrings[ 0 ] };

/**** forEachCode -- Canonical code for "for-each"; apply the given function to the
                     consecutive elements of the given lists, in order.                    ****/

PROC( forEachCode )

    DECREMENT_R
    TOP_N_SAME_LENGTH_P
    JUMP_FALSE( differentLengths )

    ZERO_FIXNUM_P
    JUMP_TRUE( noArgsInLists )

    PUSH
    RETURN_DEFAULT                     /* "for-each" returns #t */
    PUSH_CONTINUATION
    LOAD_R_TAG( schemeMachinePCIndexTag )                /* ... after dropping its arguments ... */
    LOAD_OBJECT( dropNFromStack_index, R )
    PUSH_CONTINUATION
    POP

  pushContinuationLoop:
    ZERO_FIXNUM_P
    JUMP_TRUE( done )
    PUSH
    LOAD_R_TAG( schemeMachinePCIndexTag )
    LOAD_OBJECT( oneForEach_index, R )                    /* ... after applying the function many times ... */
    PUSH_CONTINUATION
    POP
    DECREMENT_R
    JUMP_LABEL( pushContinuationLoop )

  done:
    POP
    INCREMENT_R
    PUSH
    CONTINUE

  differentLengths:
    resetError( 231 );
    CONTINUE

  noArgsInLists:
    POP
    INCREMENT_R
    PUSH
    DROP_N
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeSeven forEachFD = { forEachCode, 111, FALSE, alwaysTrue, trueBooleanTag, -1, 
                                          procedureP, properListP, 2, ObjectErrorNameStrings[ 23 ], ObjectErrorNameStrings[ 4 ] };

/**** makeStringCode -- Make a string of the given length, and fill it with any character the user
                        may optionally have provided.  Read string from Dialog Box if there are no arguments.  ****/

PROC( makeStringCode )

    ONE_FIXNUM_P
    JUMP_FALSE( notOneArgument )
    POP
    EXACT_P
    JUMP_FALSE( needExact )
    COERCE_R_SHORT_FIXNUM
    PUSH
    MOVE( zero, R )
    NUM_GE_P
    JUMP_FALSE( tooSmall )
    POP
    MAKE_EMPTY_STRING
    PUSH
    MOVE( blankChar, R )
    CALL( stringFillAux )
    POP
    CONTINUE

  notOneArgument:
    ZERO_FIXNUM_P
    JUMP_TRUE( noArguments )
    POP
    SWAP
    EXACT_P
    JUMP_FALSE( needExact )
    COERCE_R_SHORT_FIXNUM
    PUSH
    MOVE( zero, R )
    NUM_GE_P
    JUMP_FALSE( tooSmall )
    POP
    MAKE_EMPTY_STRING
    SWAP
    CALL( stringFillAux )
    POP
    CONTINUE

  noArguments:
    POP
    readDialogString( MiscStrings[ 254 ] );
    CONTINUE

  tooSmall:
    POP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 101 );
    CONTINUE

  needExact:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 102 );
    CONTINUE

END_PROC

functionDescriptorTypeNine makeStringFD = { makeStringCode, 130, TRUE, stringP, stringTag, 2,
                                            integerP, characterP, ObjectErrorNameStrings[ 10 ], ObjectErrorNameStrings[ 5 ] };

/**** makeStringWithPromptCode -- get a string from the dialog box, using the prompt provided.  ****/

PROC( makeStringWithPromptCode )

    readDialogString( startString( & R ) );
    CONTINUE

END_PROC

functionDescriptorTypeOne makeStringWithPromptFD = { makeStringWithPromptCode, 424, TRUE, stringP, stringTag, 1, stringChecker, stringCheckerNames };

/**** mapCode -- Canonical code for "map"; apply the given function to the
                 consecutive elements of the given lists, cons up the results.                   ****/

PROC( mapCode )

    DECREMENT_R
    TOP_N_SAME_LENGTH_P
    JUMP_FALSE( differentLengths )

    ZERO_FIXNUM_P
    JUMP_TRUE( noArgsInLists )
    PUSH
    LOAD_R_TAG( schemeMachinePCIndexTag )
    LOAD_OBJECT( dropNFromStack_index, R )
    PUSH_CONTINUATION
    LOAD_R_TAG( schemeMachinePCIndexTag )
    LOAD_OBJECT( reverseListCode_index, R )
    PUSH_CONTINUATION
    LOAD_R_TAG( schemeMachinePCIndexTag )
    LOAD_OBJECT( SwapCons_index, R )
    PUSH_CONTINUATION
    POP

  pushContinuationLoop:
    ONE_FIXNUM_P
    JUMP_TRUE( done )
    PUSH
    LOAD_R_TAG( schemeMachinePCIndexTag )
    LOAD_OBJECT( oneMap_index, R )
    PUSH_CONTINUATION
    POP
    DECREMENT_R
    JUMP_LABEL( pushContinuationLoop )

  done:
    PUSH
    LOAD_R_TAG( schemeMachinePCIndexTag )
    LOAD_OBJECT( oneMapFirst_index, R )
    PUSH_CONTINUATION
    POP
    POP
    INCREMENT_R
    PUSH
    LOAD_R_TAG( emptyListTag )
    PUSH
    CONTINUE

  differentLengths:
    resetError( 232 );
    CONTINUE

  noArgsInLists:
    POP
    INCREMENT_R
    PUSH
    DROP_N
    LOAD_R_TAG( emptyListTag )
    CONTINUE

END_PROC

functionDescriptorTypeSeven mapFD = { mapCode, 132, TRUE, properListP, trueBooleanTag,  -1,
                                      procedureP, properListP, 2, ObjectErrorNameStrings[ 23 ], ObjectErrorNameStrings[ 4 ] };

functionDescriptorTypeSeven mapNoTypeCheckingFD = { mapCode, 334, TRUE, properListP, trueBooleanTag,  -1,
                                                    procedureP, alwaysTrue, 2,
                                                    ObjectErrorNameStrings[ 23 ], ObjectErrorNameStrings[ 0 ] };

/**** makeVectorCode -- Make a vector of the given length, and fill it with
                        any object the user may have optionally provided.            ****/

PROC( makeVectorCode )

    ONE_FIXNUM_P
    JUMP_FALSE( twoArguments )
    POP
    EXACT_P
    JUMP_FALSE( needExact )
    COERCE_R_SHORT_FIXNUM
    JUMP_FUNCTION( makeVectorOfSharpF )

  twoArguments:
    POP
    SWAP
    EXACT_P
    JUMP_FALSE( needExact )
    COERCE_R_SHORT_FIXNUM
    CALL( makeVectorOfSharpF )
    SWAP
    CALL( vectorFillAux )
    POP
    CONTINUE

  needExact:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 104 );
    CONTINUE

END_PROC

functionDescriptorTypeSix makeVectorFD = { makeVectorCode, 131, TRUE, vectorP, trueBooleanTag, 2,
                                           integerP, alwaysTrue, ObjectErrorNameStrings[ 1 ], ObjectErrorNameStrings[ 0 ] };

