/****************************************************************************************

SchemeFunctions9.c++ -- Copyright (c) by Jay Reynolds Freeman 1988, 1989, 1990, 1991, 2006

Scheme built-in functions for Wraith Scheme.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import <time.h>
#import "WraithHeader.h"

#ifdef USE_COCOA_IO

#import "CocoaInterface.h"

#endif

#import "DataTypes.h"
#import "MemoryManager.h"
#import "OpCodes.h"
#import "SchemeFunctions.h"
#import "SenseLightCommands.h"
#import "DataTypes.e"
#import "Globals.e"
#import "Intern.e"
#import "IO.e"
#import "MainAux.e"
#import "MMapStuff.e"
#import "SchemeAux.e"
#import "SchemeGlobals.e"
#import "SchemeFunctionStatics.h"

extern int errno;

#ifdef USE_COCOA_IO

#import "CocoaInterface.h"

#endif

/******** Stuff for Scheme built-in functions; code and associated statics ********/

/**** forward declarations ****/

/**** makeForgettableCode  -- object in S[1], time in s[0], strategy in R.  ****/

PROC( makeForgettableCode )

    PUSH
    MOVE( three, R )
    CALL( makeVectorAndStuffFromStack )
    LOAD_R_TAG( forgettableUnexpiredTag )

END_PROC

functionDescriptorTypeOne makeForgettableFD = { makeForgettableCode, 367, TRUE, forgettableP, forgettableUnexpiredTag, 3, 
                                                anyIntegerProbabilityChecker, anyIntegerProbabilityCheckerNames };

/**** forgettablePCode -- Return #t if R is a forgettable, else return #f.   ****/

PROC( forgettablePCode )      

    FORGETTABLE_P 
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne forgettablePFD = { forgettablePCode, 368, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** forgettableForgottenPCode -- Return #t if R is a forgettableForgotten, else return #f.   ****/

PROC( forgettableForgottenPCode )      

    FORGETTABLE_FORGOTTEN_P 
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne forgettableForgottenPFD 
    = { forgettableForgottenPCode, 369, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** forgettableRememberedPCode -- Return #t if R is a forgettableRemembered, else return #f.   ****/

PROC( forgettableRememberedPCode )      

    FORGETTABLE_REMEMBERED_P 
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne forgettableRememberedPFD
    = { forgettableRememberedPCode, 370, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** forgettableUnexpiredPCode -- Return #t if R is a forgettableUnexpired, else return #f.   ****/

PROC( forgettableUnexpiredPCode )      

    FORGETTABLE_UNEXPIRED_P 
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne forgettableUnexpiredPFD 
    = { forgettableUnexpiredPCode, 371, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** forgettableObjectCode -- return the object, if possible.  ****/

PROC( forgettableObjectCode )

    FORGETTABLE_FORGOTTEN_P
    JUMP_TRUE( gone )
    LOAD_R_TAG( vectorTag )
    PUSH
    MOVE( zero, R )
    VREF
    PUSH
    LOAD_R_TAG( trueBooleanTag )
    SWAP
    NILCONS
    CONS
    LOAD_R_TAG( multipleValuesTag )
    CONTINUE

  gone:
    LOAD_R_TAG( falseBooleanTag )
    PUSH
    LOAD_R_TAG( trueBooleanTag )
    NILCONS
    CONS
    LOAD_R_TAG( multipleValuesTag )
    CONTINUE

END_PROC

functionDescriptorTypeOne forgettableObjectFD
    = { forgettableObjectCode, 372, TRUE, alwaysTrue, trueBooleanTag, 1, forgettableChecker, forgettableCheckerNames };

/**** forgettableTimeCode -- return the expiration time, if possible.  ****/

PROC( forgettableTimeCode )

    FORGETTABLE_FORGOTTEN_P
    JUMP_TRUE( gone )
    LOAD_R_TAG( vectorTag )
    PUSH
    MOVE( one, R )
    VREF
    PUSH
    LOAD_R_TAG( trueBooleanTag )
    SWAP
    NILCONS
    CONS
    LOAD_R_TAG( multipleValuesTag )
    CONTINUE

  gone:
    LOAD_R_TAG( exactShortFixnumBitsMask )
    PUSH
    LOAD_R_TAG( falseBooleanTag )
    SWAP
    NILCONS
    CONS
    LOAD_R_TAG( multipleValuesTag )
    CONTINUE

END_PROC

functionDescriptorTypeOne forgettableTimeFD
    = { forgettableTimeCode, 373, TRUE, integerP, numberBitsMask, 1, forgettableChecker, forgettableCheckerNames };

/**** forgettableRememberProbabilityCode -- return the remember probability, if possible.  ****/

PROC( forgettableRememberProbabilityCode )

    FORGETTABLE_FORGOTTEN_P
    JUMP_TRUE( gone )
    LOAD_R_TAG( vectorTag )
    PUSH
    MOVE( two, R )
    VREF
    PUSH
    LOAD_R_TAG( trueBooleanTag )
    SWAP
    NILCONS
    CONS
    LOAD_R_TAG( multipleValuesTag )
    CONTINUE

  gone:
    LOAD_R_TAG( falseBooleanTag )
    PUSH
    LOAD_R_TAG( trueBooleanTag )
    NILCONS
    CONS
    LOAD_R_TAG( multipleValuesTag )
    CONTINUE

END_PROC

functionDescriptorTypeOne forgettableRememberProbabilityFD
    = { forgettableRememberProbabilityCode, 374, TRUE, probabilityP, numberBitsMask, 1, forgettableChecker, forgettableCheckerNames };

/**** setForgettableTimeCode -- set the expiration time, if possible.  ****/

PROC( setForgettableTimeCode )
 
    PUSH                                    //  R: new time;    S: new time, forgettable ...
    MOVE( STACK( 1 ), R )                   //  R: forgettable; S: new time, forgettable ...
    FORGETTABLE_FORGOTTEN_P
    JUMP_TRUE( gone )
    FORGETTABLE_REMEMBERED_P
    JUMP_TRUE( gone )
    LOAD_R_TAG( vectorTag )
    MOVE( R, STACK( 1 ) )
    MOVE( one, R )
    SWAP                                    //  R: new time;    S: one, forgettable-as-vector ...
    VSET
    LOAD_R_TAG( trueBooleanTag )
    CONTINUE

  gone:
    DROP
    DROP
    LOAD_R_TAG( falseBooleanTag )
    CONTINUE

END_PROC

functionDescriptorTypeOne setForgettableTimeFD
    = { setForgettableTimeCode, 375, TRUE, booleanP, trueBooleanTag, 2, forgettableIntegerChecker, forgettableIntegerCheckerNames };

/**** setForgettableRememberProbabilityCode -- set the expiration time, if possible.  ****/

PROC( setForgettableRememberProbabilityCode )
 
    PUSH                                    //  R: new remember probability;    S: new remember probability, forgettable ...
    MOVE( STACK( 1 ), R )                   //  R: forgettable;                 S: new remember probability, forgettable ...
    FORGETTABLE_FORGOTTEN_P
    JUMP_TRUE( gone )
    FORGETTABLE_REMEMBERED_P
    JUMP_TRUE( gone )
    LOAD_R_TAG( vectorTag )
    MOVE( R, STACK( 1 ) )
    MOVE( two, R )
    SWAP                                    //  R: new remember probability;    S: one, forgettable-as-vector ...
    VSET
    LOAD_R_TAG( trueBooleanTag )
    CONTINUE

  gone:
    DROP
    DROP
    LOAD_R_TAG( falseBooleanTag )
    CONTINUE

END_PROC

functionDescriptorTypeOne setForgettableRememberProbabilityFD
    = { setForgettableRememberProbabilityCode, 376, TRUE, booleanP, trueBooleanTag, 2, forgettableProbabilityChecker, forgettableProbabilityCheckerNames };

/**** makeMMappedBlockCode -- create an mmapped block, open and stuff the file. ****/

PROC( makeMMappedBlockCode )

    memoryBlockSize pathLength = lengthString( &R );
    Text *theString = 0;
    Text *whereTheStringGoes = 0;
    mmappedBlockDescriptor *theDescriptor = 0;
    memoryBlockSize sizeWanted = sizeof( mmappedBlockDescriptor ) + pathLength + 1;
    UnsignedLongWord kitten = 0;
    void *blockLocation = 0;
    Byte *scratchWriteBuffer = 0;
    int scratchWriteBufferSize = 100000000;
    size_t virtualBytesWritten = 0;
    LongWord writeResult = 0;

    // First, set up a descriptor appropriate for "nothing has happened yet", in main memory...

    if( sizeWanted & (sizeof( taggedAval ) - 1) )
        sizeWanted = (sizeWanted & (~(sizeof( taggedAval ) - 1))) + sizeof( taggedAval );

    theHeap->allocateByteBlock( sizeWanted );
    theDescriptor = (mmappedBlockDescriptor *)((Byte *)(A.u.a) + sizeof( taggedAval ));
    whereTheStringGoes = (Text *)((Byte *)theDescriptor + sizeof( mmappedBlockDescriptor ));
    
    theString = (Text *)(R.u.a + 1);
    strcpy( whereTheStringGoes, theString );
    POP
    theDescriptor->size = (size_t)(R.u.c);

    theDescriptor->absoluteAddress = (void *)0xDEADBEEFLL;
    theDescriptor->prot = (PROT_READ | PROT_WRITE);
    theDescriptor->flags = (MAP_SHARED | MAP_FILE | MAP_FIXED | MAP_HASSEMAPHORE);
    theDescriptor->fd = 0;
    theDescriptor->offset = 0;

    for( kitten = 0; kitten < ABSOLUTE_MAX_KITTENS; ++kitten ) {
        (theDescriptor->mmapWasSuccessful)[ kitten ] = FALSE;
        (theDescriptor->regionHasBeenUnmapped)[ kitten ] = FALSE;
        }

    theDescriptor->fileHasBeenWritten = FALSE;

    // Now get the address where the block will be (and incidentally check for enough space remaining).

    blockLocation = theClowder->incrementNextFreeMMappableSpace( theDescriptor->size );

    // Now, try to open the file...

    theDescriptor->fd = open( whereTheStringGoes, (O_CREAT | O_RDWR), 0666 );

    if( theDescriptor->fd == -1 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 439 ], whereTheStringGoes );
        tprintf4( &CurrentOutputPort, MiscStrings[ 440 ], errno, strerror( errno ) );
        resetError( 258 );
        }

    // Allocate a buffer to write to the file with.

    scratchWriteBuffer = (Byte *)calloc( scratchWriteBufferSize, sizeof( Byte ) );
    if( scratchWriteBuffer == 0 )
        fatalError( 315 );

    // Write out the file.

    virtualBytesWritten = 0;
    while( virtualBytesWritten < theDescriptor->size ) {        
        if( (theDescriptor->size - virtualBytesWritten) >= (UnsignedLongWord)scratchWriteBufferSize )
            writeResult = write( theDescriptor->fd, scratchWriteBuffer, scratchWriteBufferSize );
        else
            writeResult = write( theDescriptor->fd, scratchWriteBuffer, theDescriptor->size - virtualBytesWritten );
        if( writeResult == -1 ) {
            useErrorOutput();
            tprintf3( &CurrentOutputPort, MiscStrings[ 438 ], whereTheStringGoes );
            free( scratchWriteBuffer );
            resetError( 304 );
            }
        virtualBytesWritten += scratchWriteBufferSize;
        }

    theDescriptor->fileHasBeenWritten = TRUE;

    // Perform the mmap.

    theDescriptor->absoluteAddress = mmap( blockLocation, theDescriptor->size, theDescriptor->prot,
                                           theDescriptor->flags, theDescriptor->fd, theDescriptor->offset );
    if( theDescriptor->absoluteAddress == MAP_FAILED ) {
        useErrorOutput();
        tprintf4( &CurrentOutputPort, MiscStrings[ 438 ], errno, strerror( errno ) );
        free( scratchWriteBuffer );
        resetError( 314 );
        }

    free( scratchWriteBuffer );

    (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] = TRUE;

    R.u.c = A.u.c;
    LOAD_R_TAG( mmappedBlockTag );
    (A.u.a)->t = byteBlockTag;

    CONTINUE

END_PROC

functionDescriptorTypeOne makeMMappedBlockFD
    = { makeMMappedBlockCode, 386, TRUE, mmappedBlockP, mmappedBlockTag, 2, positiveIntegerStringChecker, positiveIntegerStringCheckerNames };

/**** mmapBlockInCurrentKittenCode -- Map the given block in the current kitten. ****/

PROC( mmapBlockInCurrentKittenCode )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )(R.u.a + 1);
    Text *theString = (Text *)(theDescriptor + 1);

    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == TRUE ) {
        RETURN_DEFAULT
        CONTINUE
        }

    theDescriptor->fd = open( theString, (O_CREAT | O_RDWR), 0666 );
 
    if( theDescriptor->fd == -1 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 439 ], theString );
        tprintf4( &CurrentOutputPort, MiscStrings[ 440 ], errno, strerror( errno ) );
        resetError( 258 );
        }

    R = spliceOutNotCarForwardingPointers( R );
    blockOnTagLock1( &R, 22 );

    // Perform the mmap.

    theDescriptor->absoluteAddress = mmap( theDescriptor->absoluteAddress, theDescriptor->size, theDescriptor->prot,
                                           theDescriptor->flags, theDescriptor->fd, theDescriptor->offset );

    if( theDescriptor->absoluteAddress == MAP_FAILED ) {
        releaseTagLock1( (volatile taggedAval *)(R.u.a), 34 );
        useErrorOutput();
        tprintf4( &CurrentOutputPort, MiscStrings[ 438 ], errno, strerror( errno ) );
        resetError( 314 );
        }

    releaseTagLock1( (volatile taggedAval *)(R.u.a), 35 );

    (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] = TRUE;

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne mmapBlockInCurrentKittenFD
    = { mmapBlockInCurrentKittenCode, 387, TRUE, alwaysTrue, trueBooleanTag, 1, mmapBlockChecker, mmapBlockCheckerNames };

/**** getMMappableSpaceCode ****/

PROC( getMMappableSpaceCode )

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( theClowder->getMMappableSpaceSize(), R )
    CONTINUE

END_PROC

functionDescriptorTypeOne getMMappableSpaceFD
    = { getMMappableSpaceCode, 388, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 0, anyChecker, anyCheckerNames };

/**** poke%VectorCode ****//*

PROC( poke%VectorCode )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[1]).u.a + 1);
    UnsignedLongWord theOffset = ((S[0]).u.c);
    UnsignedLongWord vLength = 0;
    UnsignedLongWord index = 0;

    PUSH           // R: vector; S: vector, offset, mmapped-block

    R.t = pairTag;
    R.t &= ~cdrCodingBitsMask;
    PUSH           // R: vector-as-pair; S: vector-as-pair, vector, offset, mmapped-block

    CAR
    vLength = R.u.c;
   
    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 316 );

    if( (theOffset + (vLength * sizeof( Byte% )) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 389% ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( Byte% ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 389% ] );
        resetError( 319 );
        }

    MOVE( STACK( 0 ), R )

    S[3] = spliceOutNotCarForwardingPointers( S[3] );
    blockOnTagLock1( &(S[3]), 53 );

    // R: vector; S: vector-as-pair, vector, offset, mmapped-block

    for( index = 0; index < vLength; ++index ) {
        R.u.a = R.u.a + index + 1;                // The "+ 1" skips the vector length position
        PUSH
        car();
        *((Byte% *)((Byte% *)theDescriptor->absoluteAddress + theOffset + index)) = R.u.c;
        MOVE( STACK( 0 ), R )
        }

    // S: vector-as-pair, vector, offset, mmapped-block

    releaseTagLock1( (volatile taggedAval *)((S[3]).u.a), 36% );

    DROP
    DROP
    DROP
    DROP

    RETURN_DEFAULT
    CONTINUE

END_PROC

*/

/**** peek8Code ****/

PROC( peek8Code )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[0]).u.a + 1);
    UnsignedLongWord theOffset = (R.u.c);

    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 316 );

    if( (theOffset + sizeof( Byte ) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 389 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( Byte ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 389 ] );
        resetError( 319 );
        }

    S[0] = spliceOutNotCarForwardingPointers( S[0] );
    blockOnTagLock1( &(S[0]), 23 );

    LOAD_R_TAG( exactShortFixnumBitsMask )
    R.u.c = *((Byte *)((Byte *)(theDescriptor->absoluteAddress) + theOffset));

    releaseTagLock1( (volatile taggedAval *)((S[0]).u.a), 36 );

    DROP

    CONTINUE

END_PROC

functionDescriptorTypeOne peek8FD
    = { peek8Code, 389, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 2, mmapBlockUnsigned, mmapBlockUnsignedNames };

/**** peek16Code ****/

PROC( peek16Code )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[0]).u.a + 1);
    UnsignedLongWord theOffset = (R.u.c);

    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 316 );

    if( (theOffset + sizeof( UnsignedShort ) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 390 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( UnsignedShort ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 390 ] );
        resetError( 319 );
        }

    S[0] = spliceOutNotCarForwardingPointers( S[0] );
    blockOnTagLock1( &(S[0]), 24 );

    LOAD_R_TAG( exactShortFixnumBitsMask )
    R.u.c = *((UnsignedShort *)((Byte *)(theDescriptor->absoluteAddress) + theOffset));

    releaseTagLock1( (volatile taggedAval *)((S[0]).u.a), 37 );

    DROP

    CONTINUE

END_PROC

functionDescriptorTypeOne peek16FD
    = { peek16Code, 390, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 2, mmapBlockUnsigned, mmapBlockUnsignedNames };

/**** peek32Code ****/

PROC( peek32Code )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[0]).u.a + 1);
    UnsignedLongWord theOffset = (R.u.c);

    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 316 );

    if( (theOffset + sizeof( UnsignedWord ) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 391 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( UnsignedWord ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 391 ] );
        resetError( 319 );
        }

    S[0] = spliceOutNotCarForwardingPointers( S[0] );
    blockOnTagLock1( &(S[0]), 25 );

    LOAD_R_TAG( exactShortFixnumBitsMask )
    R.u.c = *((UnsignedWord *)((Byte *)(theDescriptor->absoluteAddress) + theOffset));

    releaseTagLock1( (volatile taggedAval *)((S[0]).u.a), 38 );

    DROP

    CONTINUE

END_PROC

functionDescriptorTypeOne peek32FD
    = { peek32Code, 391, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 2, mmapBlockUnsigned, mmapBlockUnsignedNames };

/**** peek64Code ****/

PROC( peek64Code )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[0]).u.a + 1);
    UnsignedLongWord theOffset = (R.u.c);

    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 316 );

    if( (theOffset + sizeof( LongWord ) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 392 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( LongWord ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 392 ] );
        resetError( 319 );
        }

    S[0] = spliceOutNotCarForwardingPointers( S[0] );
    blockOnTagLock1( &(S[0]), 26 );

    LOAD_R_TAG( exactShortFixnumBitsMask )
    R.u.c = *((LongWord *)((Byte *)(theDescriptor->absoluteAddress) + theOffset));

    releaseTagLock1( (volatile taggedAval *)((S[0]).u.a), 39 );

    DROP

    CONTINUE

END_PROC

functionDescriptorTypeOne peek64FD
    = { peek64Code, 392, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 2, mmapBlockUnsigned, mmapBlockUnsignedNames };

/**** peek8SignExtendCode ****/

PROC( peek8SignExtendCode )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[0]).u.a + 1);
    UnsignedLongWord theOffset = (R.u.c);

    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 316 );

    if( (theOffset + sizeof( Byte ) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 403 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( Byte ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 403 ] );
        resetError( 319 );
        }

    S[0] = spliceOutNotCarForwardingPointers( S[0] );
    blockOnTagLock1( &(S[0]), 31 );

    LOAD_R_TAG( exactShortFixnumBitsMask )
    R.u.c = *((Byte *)((Byte *)(theDescriptor->absoluteAddress) + theOffset));
    if( R.u.c & MSB_8 )       // Sign-extend
        R.u.c = (R.u.c) | BYTE_SIGN_EXTENDER;

    releaseTagLock1( (volatile taggedAval *)((S[0]).u.a), 44 );
 
    DROP

    CONTINUE

END_PROC

functionDescriptorTypeOne peek8SignExtendFD
    = { peek8SignExtendCode, 403, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 2, mmapBlockUnsigned, mmapBlockUnsignedNames };

/**** peek16SignExtendCode ****/

PROC( peek16SignExtendCode )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[0]).u.a + 1);
    UnsignedLongWord theOffset = (R.u.c);

    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 316 );

    if( (theOffset + sizeof( UnsignedShort ) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 404 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( UnsignedShort ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 404 ] );
        resetError( 319 );
        }

    S[0] = spliceOutNotCarForwardingPointers( S[0] );
    blockOnTagLock1( &(S[0]), 32 );

    LOAD_R_TAG( exactShortFixnumBitsMask )
    R.u.c = *((UnsignedShort *)((Byte *)(theDescriptor->absoluteAddress) + theOffset));
    if( R.u.c & MSB_16 )      // Sign-extend
        R.u.c = R.u.c | SHORT_SIGN_EXTENDER;

    releaseTagLock1( (volatile taggedAval *)((S[0]).u.a), 45 );

    DROP

    CONTINUE

END_PROC

functionDescriptorTypeOne peek16SignExtendFD
    = { peek16SignExtendCode, 404, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 2, mmapBlockUnsigned, mmapBlockUnsignedNames };

/**** peek32SignExtendCode ****/

PROC( peek32SignExtendCode )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[0]).u.a + 1);
    UnsignedLongWord theOffset = (R.u.c);

    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 316 );

    if( (theOffset + sizeof( UnsignedWord ) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 405 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( UnsignedWord ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 405 ] );
        resetError( 319 );
        }

    S[0] = spliceOutNotCarForwardingPointers( S[0] );
    blockOnTagLock1( &(S[0]), 33 );

    LOAD_R_TAG( exactShortFixnumBitsMask )
    R.u.c = *((UnsignedWord *)((Byte *)(theDescriptor->absoluteAddress) + theOffset));
    if( R.u.c & MSB_32 )      // Sign-extend
        R.u.c = R.u.c | WORD_SIGN_EXTENDER;

    releaseTagLock1( (volatile taggedAval *)((S[0]).u.a), 46 );

    DROP

    CONTINUE

END_PROC

functionDescriptorTypeOne peek32SignExtendFD
    = { peek32SignExtendCode, 405, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 2, mmapBlockUnsigned, mmapBlockUnsignedNames };

/**** poke8Code ****/

PROC( poke8Code )

    Byte theValue = 0;
    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[1]).u.a + 1);
    UnsignedLongWord theOffset = ((S[0]).u.c);

    if( ! short_fixnum_p( R.t ) )
        if( ! coerceRToShortFixnumIfPossible() ) {
            tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 393 ] );
            resetError( 320 );
            }

    theValue = (Byte)((UnsignedLongWord)(R.u.c) & justOneByteMask);

    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 317 );

    if( (theOffset + sizeof( Byte ) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 393 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( Byte ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 393 ] );
        resetError( 319 );
        }

    S[1] = spliceOutNotCarForwardingPointers( S[1] );
    blockOnTagLock1( &(S[1]), 27 );

    *((Byte *)(theDescriptor->absoluteAddress) + theOffset) = theValue;

    releaseTagLock1( (volatile taggedAval *)((S[1]).u.a), 40 );

    DROP
    DROP

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne poke8FD
    = { poke8Code, 393, FALSE, alwaysTrue, trueBooleanTag, 3, mmapBlockUnsignedInteger, mmapBlockUnsignedIntegerNames };

/**** poke16Code ****/

PROC( poke16Code )

    UnsignedShort theValue = 0;
    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[1]).u.a + 1);
    UnsignedLongWord theOffset = ((S[0]).u.c);

    if( ! short_fixnum_p( R.t ) )
        if( ! coerceRToShortFixnumIfPossible() ) {
            tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 394 ] );
            resetError( 320 );
            }

    theValue = (UnsignedShort)((UnsignedLongWord)(R.u.c) & shortMask);

    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 317 );

    if( (theOffset + sizeof( UnsignedShort ) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 394 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( UnsignedShort ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 394 ] );
        resetError( 319 );
        }

    S[1] = spliceOutNotCarForwardingPointers( S[1] );
    blockOnTagLock1( &(S[1]), 28 );

    *((UnsignedShort *)((Byte *)(theDescriptor->absoluteAddress) + theOffset)) = theValue;

    releaseTagLock1( (volatile taggedAval *)((S[1]).u.a), 41 );

    DROP
    DROP

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne poke16FD
    = { poke16Code, 394, FALSE, alwaysTrue, trueBooleanTag, 3, mmapBlockUnsignedInteger, mmapBlockUnsignedIntegerNames };

/**** poke32Code ****/

PROC( poke32Code )

    UnsignedWord theValue = 0;
    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[1]).u.a + 1);
    UnsignedLongWord theOffset = ((S[0]).u.c);

    if( ! short_fixnum_p( R.t ) )
        if( ! coerceRToShortFixnumIfPossible() ) {
            tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 395 ] );
            resetError( 320 );
            }

    theValue = (UnsignedWord)((UnsignedLongWord)(R.u.c) & lowerWordMask);

    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 317 );

    if( (theOffset + sizeof( UnsignedWord ) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 395 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( UnsignedWord ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 395 ] );
        resetError( 319 );
        }

    S[1] = spliceOutNotCarForwardingPointers( S[1] );
    blockOnTagLock1( &(S[1]), 29 );

    *((UnsignedWord *)((Byte *)(theDescriptor->absoluteAddress) + theOffset)) = theValue;

    releaseTagLock1( (volatile taggedAval *)((S[1]).u.a), 42 );

    DROP
    DROP

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne poke32FD
    = { poke32Code, 395, FALSE, alwaysTrue, trueBooleanTag, 3, mmapBlockUnsignedInteger, mmapBlockUnsignedIntegerNames };

/**** poke64Code ****/

PROC( poke64Code )

    LongWord theValue = 0;
    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[1]).u.a + 1);
    UnsignedLongWord theOffset = ((S[0]).u.c);

    if( ! short_fixnum_p( R.t ) )
        if( ! coerceRToShortFixnumIfPossible() ) {
            tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 396 ] );
            resetError( 320 );
            }

    theValue = (UnsignedLongWord)(R.u.c);

    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 317 );

    if( (theOffset + sizeof( LongWord ) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 396 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( LongWord ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 396 ] );
        resetError( 319 );
        }

    S[1] = spliceOutNotCarForwardingPointers( S[1] );
    blockOnTagLock1( &(S[1]), 30 );

    *((LongWord *)((Byte *)(theDescriptor->absoluteAddress) + theOffset)) = theValue;

    releaseTagLock1( (volatile taggedAval *)((S[1]).u.a), 43 );

    DROP
    DROP

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne poke64FD
    = { poke64Code, 396, FALSE, alwaysTrue, trueBooleanTag, 3, mmapBlockUnsignedInteger, mmapBlockUnsignedIntegerNames };

/**** peek8VectorCode ****/

PROC( peek8VectorCode )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[1]).u.a + 1);
    UnsignedLongWord theOffset = ((S[0]).u.c);
    UnsignedLongWord vLength = 0;
    UnsignedLongWord index = 0;

    PUSH           // R: vector; S: vector, offset, mmapped-block

    R.t = pairTag;
    R.t &= ~cdrCodingBitsMask;
    PUSH           // R: vector-as-pair; S: vector-as-pair, vector, offset, mmapped-block

    CAR
    vLength = R.u.c;

    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 316 );

    if( (theOffset + (vLength * sizeof( Byte )) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 410 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( Byte ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 410 ] );
        resetError( 319 );
        }

    MOVE( STACK( 0 ), R )

    S[3] = spliceOutNotCarForwardingPointers( S[3] );
    blockOnTagLock1( &(S[3]), 34 );

    // R: vector; S: vector-as-pair, vector, offset, mmapped-block

    for( index = 0; index < vLength; ++index ) {
        R.u.a = R.u.a + index + 1;                // The "+ 1" skips the vector length position
        PUSH
        LOAD_R_TAG( exactShortFixnumBitsMask )
        R.u.c = *((Byte *)((Byte *)(theDescriptor->absoluteAddress) + theOffset + (index * sizeof( Byte ))));
        setCar();
        MOVE( STACK( 0 ), R )
        }

    // S: vector-as-pair, vector, offset, mmapped-block

    releaseTagLock1( (volatile taggedAval *)((S[3]).u.a), 47 );

    DROP
    POP
    DROP
    DROP

    CONTINUE

END_PROC

functionDescriptorTypeOne peek8VectorFD
    = { peek8VectorCode, 410, TRUE, vectorP, vectorTag, 3, mmapBlockUnsignedVector, mmapBlockUnsignedVectorNames };

/**** peek16VectorCode ****/

PROC( peek16VectorCode )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[1]).u.a + 1);
    UnsignedLongWord theOffset = ((S[0]).u.c);
    UnsignedLongWord vLength = 0;
    UnsignedLongWord index = 0;

    PUSH           // R: vector; S: vector, offset, mmapped-block

    R.t = pairTag;
    R.t &= ~cdrCodingBitsMask;
    PUSH           // R: vector-as-pair; S: vector-as-pair, vector, offset, mmapped-block

    CAR
    vLength = R.u.c;

    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 316 );

    if( (theOffset + (vLength * sizeof( UnsignedShort )) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 411 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( UnsignedShort ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 411 ] );
        resetError( 319 );
        }

    MOVE( STACK( 0 ), R )

    S[3] = spliceOutNotCarForwardingPointers( S[3] );
    blockOnTagLock1( &(S[3]), 35 );

    // R: vector; S: vector-as-pair, vector, offset, mmapped-block

    for( index = 0; index < vLength; ++index ) {
        R.u.a = R.u.a + index + 1;                // The "+ 1" skips the vector length position
        PUSH
        LOAD_R_TAG( exactShortFixnumBitsMask )
        R.u.c = *((UnsignedShort *)((Byte *)(theDescriptor->absoluteAddress) + theOffset + (index * sizeof( UnsignedShort ))));
        setCar();
        MOVE( STACK( 0 ), R )
        }

    // S: vector-as-pair, vector, offset, mmapped-block

    releaseTagLock1( (volatile taggedAval *)((S[3]).u.a), 48 );

    DROP
    POP
    DROP
    DROP

    CONTINUE

END_PROC

functionDescriptorTypeOne peek16VectorFD
    = { peek16VectorCode, 411, TRUE, vectorP, vectorTag, 3, mmapBlockUnsignedVector, mmapBlockUnsignedVectorNames };

/**** peek32VectorCode ****/

PROC( peek32VectorCode )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[1]).u.a + 1);
    UnsignedLongWord theOffset = ((S[0]).u.c);
    UnsignedLongWord vLength = 0;
    UnsignedLongWord index = 0;

    PUSH           // R: vector; S: vector, offset, mmapped-block

    R.t = pairTag;
    R.t &= ~cdrCodingBitsMask;
    PUSH           // R: vector-as-pair; S: vector-as-pair, vector, offset, mmapped-block

    CAR
    vLength = R.u.c;

    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 316 );

    if( (theOffset + (vLength * sizeof( UnsignedWord )) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 412 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( UnsignedWord ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 412 ] );
        resetError( 319 );
        }

    MOVE( STACK( 0 ), R )

    S[3] = spliceOutNotCarForwardingPointers( S[3] );
    blockOnTagLock1( &(S[3]), 36 );

    // R: vector; S: vector-as-pair, vector, offset, mmapped-block

    for( index = 0; index < vLength; ++index ) {
        R.u.a = R.u.a + index + 1;                // The "+ 1" skips the vector length position
        PUSH
        LOAD_R_TAG( exactShortFixnumBitsMask )
        R.u.c = *((UnsignedWord *)((Byte *)(theDescriptor->absoluteAddress) + theOffset + (index * sizeof( UnsignedWord ))));
        setCar();
        MOVE( STACK( 0 ), R )
        }

    // S: vector-as-pair, vector, offset, mmapped-block

    releaseTagLock1( (volatile taggedAval *)((S[3]).u.a), 49 );

    DROP
    POP
    DROP
    DROP

    CONTINUE

END_PROC

functionDescriptorTypeOne peek32VectorFD
    = { peek32VectorCode, 412, TRUE, vectorP, vectorTag, 3, mmapBlockUnsignedVector, mmapBlockUnsignedVectorNames };

/**** peek64VectorCode ****/

PROC( peek64VectorCode )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[1]).u.a + 1);
    UnsignedLongWord theOffset = ((S[0]).u.c);
    UnsignedLongWord vLength = 0;
    UnsignedLongWord index = 0;

    PUSH           // R: vector; S: vector, offset, mmapped-block

    R.t = pairTag;
    R.t &= ~cdrCodingBitsMask;
    PUSH           // R: vector-as-pair; S: vector-as-pair, vector, offset, mmapped-block

    CAR
    vLength = R.u.c;

    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 316 );

    if( (theOffset + (vLength * sizeof( UnsignedLongWord )) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 413 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( UnsignedLongWord ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 413 ] );
        resetError( 319 );
        }

    MOVE( STACK( 0 ), R )

    S[3] = spliceOutNotCarForwardingPointers( S[3] );
    blockOnTagLock1( &(S[3]), 37 );

    // R: vector; S: vector-as-pair, vector, offset, mmapped-block

    for( index = 0; index < vLength; ++index ) {
        R.u.a = R.u.a + index + 1;                // The "+ 1" skips the vector length position
        PUSH
        LOAD_R_TAG( exactShortFixnumBitsMask )
        R.u.c = *((UnsignedLongWord *)((Byte *)(theDescriptor->absoluteAddress) + theOffset + (index * sizeof( UnsignedLongWord ))));
        setCar();
        MOVE( STACK( 0 ), R )
        }

    // S: vector-as-pair, vector, offset, mmapped-block

    releaseTagLock1( (volatile taggedAval *)((S[3]).u.a), 50 );

    DROP
    POP
    DROP
    DROP

    CONTINUE

END_PROC

functionDescriptorTypeOne peek64VectorFD
    = { peek64VectorCode, 413, TRUE, vectorP, vectorTag, 3, mmapBlockUnsignedVector, mmapBlockUnsignedVectorNames };

/**** peek8VectorSignExtendCode ****/

PROC( peek8VectorSignExtendCode )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[1]).u.a + 1);
    UnsignedLongWord theOffset = ((S[0]).u.c);
    UnsignedLongWord vLength = 0;
    UnsignedLongWord index = 0;

    PUSH           // R: vector; S: vector, offset, mmapped-block

    R.t = pairTag;
    R.t &= ~cdrCodingBitsMask;
    PUSH           // R: vector-as-pair; S: vector-as-pair, vector, offset, mmapped-block

    CAR
    vLength = R.u.c;

    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 316 );

    if( (theOffset + (vLength * sizeof( Byte )) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 418 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( Byte ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 418 ] );
        resetError( 319 );
        }

    MOVE( STACK( 0 ), R )

    S[3] = spliceOutNotCarForwardingPointers( S[3] );
    blockOnTagLock1( &(S[3]), 38 );

    // R: vector; S: vector-as-pair, vector, offset, mmapped-block

    for( index = 0; index < vLength; ++index ) {
        R.u.a = R.u.a + index + 1;                // The "+ 1" skips the vector length position
        PUSH
        LOAD_R_TAG( exactShortFixnumBitsMask )
        R.u.c = *((Byte *)((Byte *)(theDescriptor->absoluteAddress) + theOffset + (index * sizeof( Byte ))));
        if( R.u.c & MSB_8 )       // Sign-extend
            R.u.c = R.u.c | BYTE_SIGN_EXTENDER;
        setCar();
        MOVE( STACK( 0 ), R )
        }

    // S: vector-as-pair, vector, offset, mmapped-block

    releaseTagLock1( (volatile taggedAval *)((S[3]).u.a), 51 );

    DROP
    POP
    DROP
    DROP

    CONTINUE

END_PROC

functionDescriptorTypeOne peek8VectorSignExtendFD
    = { peek8VectorSignExtendCode, 418, TRUE, vectorP, vectorTag, 3, mmapBlockUnsignedVector, mmapBlockUnsignedVectorNames };

/**** peek16VectorSignExtendCode ****/

PROC( peek16VectorSignExtendCode )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[1]).u.a + 1);
    UnsignedLongWord theOffset = ((S[0]).u.c);
    UnsignedLongWord vLength = 0;
    UnsignedLongWord index = 0;

    PUSH           // R: vector; S: vector, offset, mmapped-block

    R.t = pairTag;
    R.t &= ~cdrCodingBitsMask;
    PUSH           // R: vector-as-pair; S: vector-as-pair, vector, offset, mmapped-block

    CAR
    vLength = R.u.c;

    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 316 );

    if( (theOffset + (vLength * sizeof( UnsignedShort )) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 419 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( UnsignedShort ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 419 ] );
        resetError( 319 );
        }

    MOVE( STACK( 0 ), R )

    S[3] = spliceOutNotCarForwardingPointers( S[3] );
    blockOnTagLock1( &(S[3]), 39 );

    // R: vector; S: vector-as-pair, vector, offset, mmapped-block

    for( index = 0; index < vLength; ++index ) {
        R.u.a = R.u.a + index + 1;                // The "+ 1" skips the vector length position
        PUSH
        LOAD_R_TAG( exactShortFixnumBitsMask )
        R.u.c = *((UnsignedShort *)((Byte *)(theDescriptor->absoluteAddress) + theOffset + (index * sizeof( UnsignedShort ))));
        if( R.u.c & MSB_16 )       // Sign-extend
            R.u.c = R.u.c | SHORT_SIGN_EXTENDER;
        setCar();
        MOVE( STACK( 0 ), R )
        }

    // S: vector-as-pair, vector, offset, mmapped-block

    releaseTagLock1( (volatile taggedAval *)((S[3]).u.a), 52 );

    DROP
    POP
    DROP
    DROP

    CONTINUE

END_PROC

functionDescriptorTypeOne peek16VectorSignExtendFD
    = { peek16VectorSignExtendCode, 419, TRUE, vectorP, vectorTag, 3, mmapBlockUnsignedVector, mmapBlockUnsignedVectorNames };

/**** peek32VectorSignExtendCode ****/

PROC( peek32VectorSignExtendCode )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[1]).u.a + 1);
    UnsignedLongWord theOffset = ((S[0]).u.c);
    UnsignedLongWord vLength = 0;
    UnsignedLongWord index = 0;

    PUSH           // R: vector; S: vector, offset, mmapped-block

    R.t = pairTag;
    R.t &= ~cdrCodingBitsMask;
    PUSH           // R: vector-as-pair; S: vector-as-pair, vector, offset, mmapped-block

    CAR
    vLength = R.u.c;

    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 316 );

    if( (theOffset + (vLength * sizeof( UnsignedWord )) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 420 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( UnsignedWord ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 420 ] );
        resetError( 319 );
        }

    MOVE( STACK( 0 ), R )

    S[3] = spliceOutNotCarForwardingPointers( S[3] );
    blockOnTagLock1( &(S[3]), 40 );

    // R: vector; S: vector-as-pair, vector, offset, mmapped-block

    for( index = 0; index < vLength; ++index ) {
        R.u.a = R.u.a + index + 1;                // The "+ 1" skips the vector length position
        PUSH
        LOAD_R_TAG( exactShortFixnumBitsMask )
        R.u.c = *((UnsignedWord *)((Byte *)(theDescriptor->absoluteAddress) + theOffset + (index * sizeof( UnsignedWord ))));
        if( R.u.c & MSB_32 )       // Sign-extend
            R.u.c = R.u.c | WORD_SIGN_EXTENDER;
        setCar();
        MOVE( STACK( 0 ), R )
        }

    // S: vector-as-pair, vector, offset, mmapped-block

    releaseTagLock1( (volatile taggedAval *)((S[3]).u.a), 53 );

    DROP
    POP
    DROP
    DROP

    CONTINUE

END_PROC

functionDescriptorTypeOne peek32VectorSignExtendFD
    = { peek32VectorSignExtendCode, 420, TRUE, vectorP, vectorTag, 3, mmapBlockUnsignedVector, mmapBlockUnsignedVectorNames };

/**** peekFlonumCode ****/

PROC( peekFlonumCode )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[0]).u.a + 1);
    UnsignedLongWord theOffset = (R.u.c);

    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 316 );

    if( (theOffset + sizeof( ShortFloat ) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 408 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( ShortFloat ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 408 ] );
        resetError( 319 );
        }

    S[0] = spliceOutNotCarForwardingPointers( S[0] );
    blockOnTagLock1( &(S[0]), 41 );

    LOAD_R_TAG( inexactShortFloatBitsMask )
    R.u.f = *((ShortFloat *)((Byte *)(theDescriptor->absoluteAddress) + theOffset));

    releaseTagLock1( (volatile taggedAval *)((S[0]).u.a), 54 );

    DROP

    CONTINUE

END_PROC

functionDescriptorTypeOne peekFlonumFD
    = { peekFlonumCode, 408, TRUE, exactShortFloatP, exactShortFloatBitsMask, 2, mmapBlockUnsigned, mmapBlockUnsignedNames };

/**** pokeFlonumCode ****/

PROC( pokeFlonumCode )

    ShortFloat theValue = 0.0;

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[1]).u.a + 1);
    UnsignedLongWord theOffset = ((S[0]).u.c);

    if( short_fixnum_p( R.t ) )
        coerceRToShortFloat();

    theValue = (R.u.f);

    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 317 );

    if( (theOffset + sizeof( ShortFloat ) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 409 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( ShortFloat ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 409 ] );
        resetError( 319 );
        }

    S[1] = spliceOutNotCarForwardingPointers( S[1] );
    blockOnTagLock1( &(S[1]), 42 );

    *((ShortFloat *)((Byte *)(theDescriptor->absoluteAddress) + theOffset)) = theValue;

    releaseTagLock1( (volatile taggedAval *)((S[1]).u.a), 55 );

    DROP
    DROP

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne pokeFlonumFD
    = { pokeFlonumCode, 409, FALSE, alwaysTrue, trueBooleanTag, 3, mmapBlockUnsignedNumber, mmapBlockUnsignedNumberNames };

/**** poke8VectorCode ****/

PROC( poke8VectorCode )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[1]).u.a + 1);
    UnsignedLongWord theOffset = ((S[0]).u.c);
    UnsignedLongWord vLength = 0;
    UnsignedLongWord index = 0;

    PUSH           // R: vector; S: vector, offset, mmapped-block

    R.t = pairTag;
    R.t &= ~cdrCodingBitsMask;
    PUSH           // R: vector-as-pair; S: vector-as-pair, vector, offset, mmapped-block

    CAR
    vLength = R.u.c;
   
    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 317 );

    if( (theOffset + (vLength * sizeof( Byte )) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 414 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( Byte ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 414 ] );
        resetError( 319 );
        }

    MOVE( STACK( 0 ), R )

    S[3] = spliceOutNotCarForwardingPointers( S[3] );
    blockOnTagLock1( &(S[3]), 43 );

    // R: vector; S: vector-as-pair, vector, offset, mmapped-block

    for( index = 0; index < vLength; ++index ) {
        R.u.a = R.u.a + index + 1;                // The "+ 1" skips the vector length position
        car();
        if( ! short_fixnum_p( R.t ) )
            if( ! coerceRToShortFixnumIfPossible() ) {
                releaseTagLock1( (volatile taggedAval *)((S[3]).u.a), 60 );
                tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 414 ] );
                tprintf3( &CurrentOutputPort, MiscStrings[ 443 ], index );
                resetError( 320 );
                }
        *((Byte *)((Byte *)(theDescriptor->absoluteAddress) + theOffset + (index * sizeof( Byte )))) = (Byte)(R.u.c);
        MOVE( STACK( 0 ), R )
        }

    // S: vector-as-pair, vector, offset, mmapped-block

    releaseTagLock1( (volatile taggedAval *)((S[3]).u.a), 56 );

    DROP
    DROP
    DROP
    DROP

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne poke8VectorFD
    = { poke8VectorCode, 414, TRUE, alwaysTrue, trueBooleanTag, 3, mmapBlockUnsignedVector, mmapBlockUnsignedVectorNames };

/**** poke16VectorCode ****/

PROC( poke16VectorCode )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[1]).u.a + 1);
    UnsignedLongWord theOffset = ((S[0]).u.c);
    UnsignedLongWord vLength = 0;
    UnsignedLongWord index = 0;

    PUSH           // R: vector; S: vector, offset, mmapped-block

    R.t = pairTag;
    R.t &= ~cdrCodingBitsMask;
    PUSH           // R: vector-as-pair; S: vector-as-pair, vector, offset, mmapped-block

    CAR
    vLength = R.u.c;
   
    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 317 );

    if( (theOffset + (vLength * sizeof( UnsignedShort )) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 415 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( UnsignedShort ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 415 ] );
        resetError( 319 );
        }

    MOVE( STACK( 0 ), R )

    S[3] = spliceOutNotCarForwardingPointers( S[3] );
    blockOnTagLock1( &(S[3]), 44 );

    // R: vector; S: vector-as-pair, vector, offset, mmapped-block

    for( index = 0; index < vLength; ++index ) {
        R.u.a = R.u.a + index + 1;                // The "+ 1" skips the vector length position
        car();
        if( ! short_fixnum_p( R.t ) )
            if( ! coerceRToShortFixnumIfPossible() ) {
                releaseTagLock1( (volatile taggedAval *)((S[3]).u.a), 61 );
                tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 415 ] );
                tprintf3( &CurrentOutputPort, MiscStrings[ 443 ], index );
                resetError( 320 );
                }
        *((UnsignedShort *)((Byte *)(theDescriptor->absoluteAddress) + theOffset + (index * sizeof( UnsignedShort ))))
            = (UnsignedShort)(R.u.c);
        MOVE( STACK( 0 ), R )
        }

    // S: vector-as-pair, vector, offset, mmapped-block

    releaseTagLock1( (volatile taggedAval *)((S[3]).u.a), 57 );

    DROP
    DROP
    DROP
    DROP

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne poke16VectorFD
    = { poke16VectorCode, 415, TRUE, alwaysTrue, trueBooleanTag, 3, mmapBlockUnsignedVector, mmapBlockUnsignedVectorNames };

/**** poke32VectorCode ****/

PROC( poke32VectorCode )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[1]).u.a + 1);
    UnsignedLongWord theOffset = ((S[0]).u.c);
    UnsignedLongWord vLength = 0;
    UnsignedLongWord index = 0;

    PUSH           // R: vector; S: vector, offset, mmapped-block

    R.t = pairTag;
    R.t &= ~cdrCodingBitsMask;
    PUSH           // R: vector-as-pair; S: vector-as-pair, vector, offset, mmapped-block

    CAR
    vLength = R.u.c;
   
    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 317 );

    if( (theOffset + (vLength * sizeof( UnsignedWord )) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 416 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( UnsignedWord ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 416 ] );
        resetError( 319 );
        }

    MOVE( STACK( 0 ), R )

    S[3] = spliceOutNotCarForwardingPointers( S[3] );
    blockOnTagLock1( &(S[3]), 45 );

    // R: vector; S: vector-as-pair, vector, offset, mmapped-block

    for( index = 0; index < vLength; ++index ) {
        R.u.a = R.u.a + index + 1;                // The "+ 1" skips the vector length position
        car();
        if( ! short_fixnum_p( R.t ) ) 
            if( ! coerceRToShortFixnumIfPossible() ) {
                releaseTagLock1( (volatile taggedAval *)((S[3]).u.a), 62 );
                tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 416 ] );
                tprintf3( &CurrentOutputPort, MiscStrings[ 443 ], index );
                resetError( 320 );
                }
        *((UnsignedWord *)((Byte *)(theDescriptor->absoluteAddress) + theOffset + (index * sizeof( UnsignedWord ))))
            = (UnsignedWord)(R.u.c);
        MOVE( STACK( 0 ), R )
        }

    // S: vector-as-pair, vector, offset, mmapped-block

    releaseTagLock1( (volatile taggedAval *)((S[3]).u.a), 58 );

    DROP
    DROP
    DROP
    DROP

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne poke32VectorFD
    = { poke32VectorCode, 416, TRUE, alwaysTrue, trueBooleanTag, 3, mmapBlockUnsignedVector, mmapBlockUnsignedVectorNames };

/**** poke64VectorCode ****/

PROC( poke64VectorCode )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[1]).u.a + 1);
    UnsignedLongWord theOffset = ((S[0]).u.c);
    UnsignedLongWord vLength = 0;
    UnsignedLongWord index = 0;

    PUSH           // R: vector; S: vector, offset, mmapped-block

    R.t = pairTag;
    R.t &= ~cdrCodingBitsMask;
    PUSH           // R: vector-as-pair; S: vector-as-pair, vector, offset, mmapped-block

    CAR
    vLength = R.u.c;
   
    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 317 );

    if( (theOffset + (vLength * sizeof( UnsignedLongWord )) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 417 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( UnsignedLongWord ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 417 ] );
        resetError( 319 );
        }

    MOVE( STACK( 0 ), R )

    S[3] = spliceOutNotCarForwardingPointers( S[3] );
    blockOnTagLock1( &(S[3]), 46 );

    // R: vector; S: vector-as-pair, vector, offset, mmapped-block

    for( index = 0; index < vLength; ++index ) {
        R.u.a = R.u.a + index + 1;                // The "+ 1" skips the vector length position
        car();
        if( ! short_fixnum_p( R.t ) )
            if( ! coerceRToShortFixnumIfPossible() ) {
                releaseTagLock1( (volatile taggedAval *)((S[3]).u.a), 63 );
                tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 417 ] );
                tprintf3( &CurrentOutputPort, MiscStrings[ 443 ], index );
                resetError( 320 );
                }
        *((UnsignedLongWord *)((Byte *)(theDescriptor->absoluteAddress) + theOffset + (index * sizeof( UnsignedLongWord ))))
            = (UnsignedLongWord)(R.u.c);
        MOVE( STACK( 0 ), R )
        }

    // S: vector-as-pair, vector, offset, mmapped-block

    releaseTagLock1( (volatile taggedAval *)((S[3]).u.a), 59 );

    DROP
    DROP
    DROP
    DROP

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne poke64VectorFD
    = { poke64VectorCode, 417, TRUE, alwaysTrue, trueBooleanTag, 3, mmapBlockUnsignedVector, mmapBlockUnsignedVectorNames };

/**** peekFlonumVectorCode ****/

PROC( peekFlonumVectorCode )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[1]).u.a + 1);
    UnsignedLongWord theOffset = ((S[0]).u.c);
    UnsignedLongWord vLength = 0;
    UnsignedLongWord index = 0;

    PUSH           // R: vector; S: vector, offset, mmapped-block

    R.t = pairTag;
    R.t &= ~cdrCodingBitsMask;
    PUSH           // R: vector-as-pair; S: vector-as-pair, vector, offset, mmapped-block

    CAR
    vLength = R.u.c;

    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 317 );

    if( (theOffset + (vLength * sizeof( ShortFloat )) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 421 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( ShortFloat ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 421 ] );
        resetError( 319 );
        }

    MOVE( STACK( 0 ), R )

    S[3] = spliceOutNotCarForwardingPointers( S[3] );
    blockOnTagLock1( &(S[3]), 47 );

    // R: vector; S: vector-as-pair, vector, offset, mmapped-block

    for( index = 0; index < vLength; ++index ) {
        R.u.a = R.u.a + index + 1;                // The "+ 1" skips the vector length position
        PUSH
        LOAD_R_TAG( exactShortFloatBitsMask )
        R.u.f = *((ShortFloat *)((Byte *)(theDescriptor->absoluteAddress) + theOffset + (index * sizeof( ShortFloat ))));
        setCar();
        MOVE( STACK( 0 ), R )
        }

    // S: vector-as-pair, vector, offset, mmapped-block

    releaseTagLock1( (volatile taggedAval *)((S[3]).u.a), 64 );

    DROP
    POP
    DROP
    DROP

    CONTINUE

END_PROC

functionDescriptorTypeOne peekFlonumVectorFD
    = { peekFlonumVectorCode, 421, TRUE, vectorP, vectorTag, 3, mmapBlockUnsignedVector, mmapBlockUnsignedVectorNames };

/**** pokeFlonumVectorCode ****/

PROC( pokeFlonumVectorCode )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[1]).u.a + 1);
    UnsignedLongWord theOffset = ((S[0]).u.c);
    UnsignedLongWord vLength = 0;
    UnsignedLongWord index = 0;

    PUSH           // R: vector; S: vector, offset, mmapped-block

    R.t = pairTag;
    R.t &= ~cdrCodingBitsMask;
    PUSH           // R: vector-as-pair; S: vector-as-pair, vector, offset, mmapped-block

    CAR
    vLength = R.u.c;
   
    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 317 );

    if( (theOffset + (vLength * sizeof( ShortFloat )) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 422 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( ShortFloat ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 422 ] );
        resetError( 319 );
        }

    MOVE( STACK( 0 ), R )

    S[3] = spliceOutNotCarForwardingPointers( S[3] );
    blockOnTagLock1( &(S[3]), 48 );

    // R: vector; S: vector-as-pair, vector, offset, mmapped-block

    for( index = 0; index < vLength; ++index ) {
        R.u.a = R.u.a + index + 1;                // The "+ 1" skips the vector length position
        car();
        if( short_fixnum_p( R.t ) )
            coerceRToShortFloat();
        *((ShortFloat *)((Byte *)(theDescriptor->absoluteAddress) + theOffset + (index * sizeof( ShortFloat ))))
            = (ShortFloat)(R.u.f);
        MOVE( STACK( 0 ), R )
        }

    // S: vector-as-pair, vector, offset, mmapped-block

    releaseTagLock1( (volatile taggedAval *)((S[3]).u.a), 65 );

    DROP
    DROP
    DROP
    DROP

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne pokeFlonumVectorFD
    = { pokeFlonumVectorCode, 422, TRUE, alwaysTrue, trueBooleanTag, 3, mmapBlockUnsignedVector, mmapBlockUnsignedVectorNames };

/**** peekCStringCode ****/

PROC( peekCStringCode )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[0]).u.a + 1);
    UnsignedLongWord theOffset = (R.u.c);
    UnsignedLongWord sLength = 0;
    Byte *position = 0;
    Byte *pastEnd = (Byte *)(theDescriptor->absoluteAddress) + (theDescriptor->size);

    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 316 );

    if( (theOffset & (sizeof( Byte ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 406 ] );
        resetError( 319 );
        }

    blockOnTagLock1( &(S[0]), 49 );

    for( position = (Byte *)(theDescriptor->absoluteAddress) + theOffset; position < pastEnd; ++position )
        if( *position == 0 )
            break;

    if( position >= pastEnd ) {
        releaseTagLock1( (volatile taggedAval *)((S[0]).u.a), 66 );
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 406 ] );
        tprintf3( &CurrentOutputPort, MiscStrings[ 444 ], theOffset );
        resetError( 318 );
        }

    sLength = position - ((Byte *)(theDescriptor->absoluteAddress) + theOffset);
    makeString( (Text *)(theDescriptor->absoluteAddress) + theOffset );
    releaseTagLock1( (volatile taggedAval *)((S[0]).u.a), 67 );

    DROP

    CONTINUE

END_PROC

functionDescriptorTypeOne peekCStringFD
    = { peekCStringCode, 406, TRUE, stringP, stringTag, 2, mmapBlockUnsigned, mmapBlockUnsignedNames };

/**** pokeCStringCode ****/

PROC( pokeCStringCode )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[1]).u.a + 1);
    UnsignedLongWord theOffset = (S[0]).u.c;
    UnsignedLongWord sLength = lengthString( &R );

    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 317 );

    if( (theOffset + (sLength * sizeof( Byte )) + 1 - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 407 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 394 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( Byte ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 405 ] );
        resetError( 319 );
        }

    blockOnTagLock1( &(S[1]), 50 );

    strcpy( (Text *)(theDescriptor->absoluteAddress) + theOffset, (Text*)(R.u.a + 1) );

    releaseTagLock1( (volatile taggedAval *)((S[1]).u.a), 68 );

    DROP
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne pokeCStringFD
    = { pokeCStringCode, 407, TRUE, alwaysTrue, trueBooleanTag, 3, mmapBlockUnsignedString, mmapBlockUnsignedStringNames };

/**** peekPokeAtomicCode ****/

PROC( peekPokeAtomicCode )

    mmappedBlockDescriptor *theDescriptor = (mmappedBlockDescriptor* )((S[0]).u.a + 1);
    UnsignedLongWord theOffset = (R.u.c);
    void *theAddress = 0;
    Boolean theResult = FALSE;

    if( (theDescriptor->mmapWasSuccessful)[ theKittenNumber ] == FALSE )
        resetError( 321 );

    if( (theOffset + sizeof( UnsignedLongWord ) - 1) > (theDescriptor->size - 1) ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 441 ], NameStrings[ 425 ] );
        tprintf4( &CurrentOutputPort, MiscStrings[ 424 ], theOffset, theDescriptor->size );
        resetError( 318 );
        }

    if( (theOffset & (sizeof( UnsignedLongWord ) - 1) ) != 0 ) {
        useErrorOutput();
        tprintf3( &CurrentOutputPort, MiscStrings[ 442 ], NameStrings[ 425 ] );
        resetError( 319 );
        }

    S[0] = spliceOutNotCarForwardingPointers( S[0] );

    theAddress = (void *)((Byte *)(theDescriptor->absoluteAddress) + theOffset);

    blockOnTagLock1( &(S[0]), 51 );

    theResult = OSAtomicTestAndSetBarrier( 7, theAddress );

    releaseTagLock1( (volatile taggedAval *)((S[0]).u.a), 69 );

    B = (theResult == FALSE);    // Set B if the original content of the bit tested-and-set was zero;
                                 //   that is, if we "got" the bit.

    SET_R_FROM_B

    DROP

    CONTINUE

END_PROC

functionDescriptorTypeOne peekPokeAtomicFD
    = { peekPokeAtomicCode, 425, TRUE, booleanP, trueBooleanTag, 2, mmapBlockUnsigned, mmapBlockUnsignedNames };

/**** cBooleanToSchemeBooleanCode ****/

PROC( cBooleanToSchemeBooleanCode )

    ZERO_FIXNUM_P
    JUMP_TRUE( returnFalse )
    LOAD_R_TAG( trueBooleanTag )
    CONTINUE

  returnFalse:
    LOAD_R_TAG( falseBooleanTag )
    CONTINUE

END_PROC    

functionDescriptorTypeOne cBooleanToSchemeBooleanFD
    = { cBooleanToSchemeBooleanCode, 427, TRUE, booleanP, trueBooleanTag, 1, numberChecker, numberCheckerNames };

/**** schemeBooleanToCBooleanCode ****/

PROC( schemeBooleanToCBooleanCode )

    FALSE_P
    JUMP_TRUE( setZero )
    MOVE( one, R )
    CONTINUE

  setZero:
    MOVE( zero, R )
    CONTINUE

END_PROC    

functionDescriptorTypeOne schemeBooleanToCBooleanFD
    = { schemeBooleanToCBooleanCode, 428, TRUE, numberP, numberBits, 1, anyChecker, anyCheckerNames };

/**** memoryMappedBlockPCode ****/

PROC( memoryMappedBlockPCode )

    CALL( mmappedBlockP ) 
    SET_R_FROM_B 
    CONTINUE

END_PROC

functionDescriptorTypeOne memoryMappedBlockPFD
    = { memoryMappedBlockPCode, 397, TRUE, booleanP, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** setInterruptCode ****/

PROC( setInterruptCode )

    UnsignedLongWord interruptNumber = (S[1]).u.c;

    theClowder->lockInterrupt( interruptNumber );

    theClowder->clearInterruptRequested( interruptNumber );   // Clear any pending requests to this interrupt.
    CONS                         //  R: (kitten-to-execute . handler); S: interrupt-number
    PUSH                         //  R: (kitten-to-execute . handler); S: (kitten-to-execute . handler), interrupt-number
    MOVE( IVR, R )
    EXCHANGE( R, STACK( 1 ) )    //  R: interrupt-number; S: (kitten-to-execute . handler), IVR
    EXCHANGE( R, STACK( 0 ) )    //  R: (kitten-to-execute-interrupt . handler); S: interrupt-number, IVR
    VSET
    theClowder->clearInterruptRequested( interruptNumber );   // Clear any pending requests to this interrupt.
    theClowder->setInterruptInstalled( interruptNumber );     // Record that a handler is installed.

    theClowder->unlockInterrupt( interruptNumber );

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne setInterruptFD
    = { setInterruptCode, 399, FALSE, alwaysTrue, trueBooleanTag, 3, interruptKittenAnyChecker, interruptKittenAnyCheckerNames };

/**** clearInterruptCode ****/

PROC( clearInterruptCode )

    UnsignedLongWord interruptNumber = R.u.c;

    theClowder->lockInterrupt( interruptNumber );

    theClowder->clearInterruptInstalled( interruptNumber );   // Record that no handler is installed.
    theClowder->clearInterruptRequested( interruptNumber );   // Clear any pending requests to this interrupt.
	PUSH                         //  S: interrupt-number
    MOVE( IVR, R )
    EXCHANGE( R, STACK( 0 ) )    //  R: interrupt-number; S: IVR
    PUSH                         //  S: interrupt-number, IVR
    LOAD_R_TAG( emptyListTag )
    VSET
    theClowder->clearInterruptRequested( interruptNumber );   // Clear any pending requests to this interrupt.

    theClowder->unlockInterrupt( interruptNumber );

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne clearInterruptFD
    = { clearInterruptCode, 401, FALSE, alwaysTrue, trueBooleanTag, 1, interruptChecker, interruptCheckerNames };

/**** clearInterruptRequestedCode ****/

PROC( clearInterruptRequestedCode )

    UnsignedLongWord interruptNumber = R.u.c;

    theClowder->lockInterrupt( interruptNumber );

    theClowder->clearInterruptRequested( interruptNumber );   // Clear any pending requests to this interrupt.

    theClowder->unlockInterrupt( interruptNumber );

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne clearInterruptRequestedFD
    = { clearInterruptRequestedCode, 438, FALSE, alwaysTrue, trueBooleanTag, 1, interruptChecker, interruptCheckerNames };

/**** interruptCode ****/

PROC( interruptCode )

#define SYSTEM_BUFFER_SIZE 100    

    Text systemBuffer[ SYSTEM_BUFFER_SIZE ];

#ifdef USE_COCOA_IO
    theClowder->setInterruptRequested( R.u.c );
#endif

    snprintf( systemBuffer, SYSTEM_BUFFER_SIZE -1, "kill -30 %d", theClowder->getKittenPID( (Word)((S[0]).u.c) ) );
    system( systemBuffer );

    DROP

    RETURN_DEFAULT
    CONTINUE

END_PROC    

functionDescriptorTypeOne interruptFD
    = { interruptCode, 400, FALSE, alwaysTrue, trueBooleanTag, 2, kittenInterruptChecker, kittenInterruptCheckerNames };

/**** getInterruptFileInformationCode ****/

PROC( getInterruptFileInformationCode )

    makeString( theClowder->getInterruptInformationFileName() );
    PUSH
    LOAD_R_TAG( exactShortFixnumBitsMask )
    R.u.c = theClowder->getInterruptInformationFileSize();

    NILCONS
    CONS

    LOAD_R_TAG( multipleValuesTag )
    CONTINUE

END_PROC

functionDescriptorTypeOne getInterruptFileInformationFD
    = { getInterruptFileInformationCode, 402, TRUE, multipleValuesP, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** setAlarmCode ****/

PROC( setAlarmCode )

    if( R.u.c == 0 ) {
        alarm( 0 );
        isAlarmSet = FALSE;
        R.t = trueBooleanTag;
        }
    else if( (alarmTime = time( 0 )) == -1 ) {
        alarm( 0 );
        isAlarmSet = FALSE;
        R.t = falseBooleanTag;
        }
    else {
        isAlarmSet = TRUE;
        alarmTime += R.u.c;
        alarm( (int)(R.u.c) );
        R.t = trueBooleanTag;
        }

    CONTINUE

END_PROC

functionDescriptorTypeOne setAlarmFD
    = { setAlarmCode, 435, TRUE, booleanP, trueBooleanTag, 1, nonNegativeIntegerChecker, nonNegativeIntegerCheckerNames };

/**** alarmPCode ****/

PROC( alarmPCode )

    if( isAlarmSet )
        R.t = trueBooleanTag;
    else
        R.t = falseBooleanTag;

    CONTINUE

END_PROC

functionDescriptorTypeOne alarmPFD
    = { alarmPCode, 436, TRUE, booleanP, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** alarmTimeCode ****/

PROC( alarmTimeCode )

    R.t = exactShortFixnumBitsMask;
    if( isAlarmSet )
        R.u.c = alarmTime;
    else
        R.u.c = -1;

    CONTINUE

END_PROC

functionDescriptorTypeOne alarmTimeFD
    = { alarmTimeCode, 437, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 0, anyChecker, anyCheckerNames };

// Commands for sense-light operations follow.

// Primitive to load one (byte-sized) command

UnsignedLongWord packOneCommand( Boolean isCommand, Boolean isSetIllumination, Boolean isVisible, UnsignedLongWord color )
{
    UnsignedLongWord answer = 0;

    if( isCommand )
        answer |= COMMAND_BIT_MASK;

    if( isSetIllumination )
        answer |= COMMAND_TYPE_BIT_MASK;

    if( isVisible )
        answer |= VISIBLE_BIT_MASK;

    answer |= (Byte)(color & ILLUMINATION_BITS_MASK );

    return answer;
    }

// Primitive to load a given full (64-bit) command with the specific command data for a particular light.

void loadCommandForLight( LongWord *destination, UnsignedLongWord light,
                          Boolean isCommand, Boolean isSetIllumination, Boolean isVisible, UnsignedLongWord color )
{
    UnsignedLongWord thisCommand = packOneCommand( isCommand, isSetIllumination, isVisible, color );

    *destination |= (thisCommand << (light * BITS_IN_BYTE) );
    }

// A few primitives to obtain constants of interest.

LongWord showAllLightsCommand()    { return 0xA0A0A0A0A0A0A0A0ll; }

LongWord hideAllLightsCommand()    { return 0x8080808080808080ll; }

LongWord turnOffAllLightsCommand() { return 0xC0C0C0C0C0C0C0C0ll; }

/**** SetSenseLightNIlluminationCode -- Set the illumination state of a given sense light: On
                                        entry, S[0] has the sense light, R has the illumination state.  ****/

PROC( setSenseLightNIlluminationCode )

#ifdef USE_COCOA_IO  // If not, there are no sense lights

    LongWord wholeCommand = 0;
    
    loadCommandForLight( &wholeCommand, (UnsignedLongWord)(S[0].u.c), TRUE, TRUE, FALSE, (UnsignedLongWord)(R.u.c) );
    tellController( E_SENSE_LIGHT_COMMAND, wholeCommand, "" );

#endif

    DROP
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne setSenseLightNIlluminationFD
    = { setSenseLightNIlluminationCode, 440, FALSE, alwaysTrue, trueBooleanTag, 2,
        senseLightSenseLightIlluminationChecker, senseLightSenseLightIlluminationCheckerNames };

/**** SetSenseLightNVisibilityCode -- Set the visibility state of a given sense light: On
                                      entry, S[0] has the sense light, R has the visibility state.  ****/

PROC( setSenseLightNVisibilityCode )

#ifdef USE_COCOA_IO  // If not, there are no sense lights

    LongWord wholeCommand = 0;
    
    loadCommandForLight( &wholeCommand, (UnsignedLongWord)(S[0].u.c), TRUE, FALSE, true_p( R.t ), 0 );
    tellController( E_SENSE_LIGHT_COMMAND, wholeCommand, "" );

#endif

    DROP
    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne setSenseLightNVisibilityFD
    = { setSenseLightNVisibilityCode, 441, FALSE, alwaysTrue, trueBooleanTag, 2,
        senseLightBooleanChecker, senseLightBooleanCheckerNames };

/**** senseLightPCode -- is the content of R an integer in the range [ 0, N_SENSE_LIGHTS )? ****/

PROC( senseLightPCode )

    CALL( senseLightP )
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne senseLightPFD
    = { senseLightPCode, 442, FALSE, alwaysTrue, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** senseLightIlluminationPCode -- is the content of R an integer in the range [ 0, N_SENSE_LIGHTS )? ****/

PROC( senseLightIlluminationPCode )

    CALL( senseLightIlluminationP )
    SET_R_FROM_B
    CONTINUE

END_PROC

functionDescriptorTypeOne senseLightIlluminationPFD
    = { senseLightIlluminationPCode, 443, FALSE, alwaysTrue, trueBooleanTag, 1, anyChecker, anyCheckerNames };

/**** showSenseLightsCode -- make all the sense lights visible ****/

PROC( showSenseLightsCode )

#ifdef USE_COCOA_IO  // If not, there are no sense lights

    LongWord wholeCommand = showAllLightsCommand();

    tellController( E_SENSE_LIGHT_COMMAND, wholeCommand, "" );

#endif

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne showSenseLightsFD
    = { showSenseLightsCode, 444, TRUE, booleanP, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** hideSenseLightsCode -- make all the sense lights invisible ****/

PROC( hideSenseLightsCode )

#ifdef USE_COCOA_IO  // If not, there are no sense lights

    LongWord wholeCommand = hideAllLightsCommand();

    tellController( E_SENSE_LIGHT_COMMAND, wholeCommand, "" );
    
#endif

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne hideSenseLightsFD
    = { hideSenseLightsCode, 445, TRUE, booleanP, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** setSenseLightsOffCode -- turn all the sense lights off ****/

PROC( setSenseLightsOffCode )

#ifdef USE_COCOA_IO  // If not, there are no sense lights

    LongWord wholeCommand = turnOffAllLightsCommand();

    tellController( E_SENSE_LIGHT_COMMAND, wholeCommand, "" );
    
#endif

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne setSenseLightsOffFD
    = { setSenseLightsOffCode, 446, TRUE, booleanP, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** setSenseLightIlluminationsCode -- Set the illumination state of all sense lights: On
                                        entry, R has the illumination state.  ****/

PROC( setSenseLightIlluminationsCode )

#ifdef USE_COCOA_IO  // If not, there are no sense lights

    LongWord wholeCommand = 0;
    int light = 0;

    for( light = 0; light < N_SENSE_LIGHTS; ++light )
        loadCommandForLight( &wholeCommand, light, TRUE, TRUE, 0, (UnsignedLongWord)(R.u.c) );

    tellController( E_SENSE_LIGHT_COMMAND, wholeCommand, "" );

#endif

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne setSenseLightIlluminationsFD
    = { setSenseLightIlluminationsCode, 447, FALSE, alwaysTrue, trueBooleanTag, 1,
        senseLightIlluminationChecker, senseLightIlluminationCheckerNames };

/**** rotateSenseLightsCode -- rotate the pattern of all sense lights as shown -- positive is to the left ****/

PROC( rotateSenseLightsCode )

#ifdef USE_COCOA_IO  // If not, there are no sense lights

    tellController( E_SENSE_LIGHT_ROTATE, R.u.c, "" );

#endif

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne rotateSenseLightsFD
    = { rotateSenseLightsCode, 448, FALSE, alwaysTrue, trueBooleanTag, 1,
        exactIntegerChecker, exactIntegerCheckerNames };

/**** applicativeModeCheckSetBangCode -- test and fail if the appicative mode bit is set ****/

PROC( applicativeModeCheckSetBangCode )

    applicativeCheckDropNone( NameStrings[ 157 ] );

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne applicativeModeCheckSetBangFD
    = { applicativeModeCheckSetBangCode, 450, FALSE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** applicativeModeCheckCode -- test and fail if the appicative mode bit is set ****/

PROC( applicativeModeCheckCode )

    applicativeCheckDropNone( startString( &R ) );

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne applicativeModeCheckFD
    = { applicativeModeCheckCode, 451, FALSE, alwaysTrue, trueBooleanTag, 1, stringChecker, stringCheckerNames };

/**** applicativeBitPCode  --  return whether the applicative bit is on  ****/

PROC( applicativeBitPCode )

    if( applicativeModeP() )
        LOAD_R_TAG( trueBooleanTag )
    else
        LOAD_R_TAG( falseBooleanTag )

    CONTINUE

END_PROC

functionDescriptorTypeOne applicativeBitPFD
    = { applicativeBitPCode, 452, TRUE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** setApplicativeBitCode  --  set the applicative bit in the evaluation status register ****/

PROC( setApplicativeBitCode )

    setApplicativeMode( TRUE );

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne setApplicativeBitFD
    = { setApplicativeBitCode, 453, FALSE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** setApplicativeBitNDownCode  --  set the applicative bit in the (zero-based) Nth stack frame down  ****/

PROC( setApplicativeBitNDownCode )

    setApplicativeModeNDown( R.u.c );

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne setApplicativeBitNDownFD
    = { setApplicativeBitNDownCode, 454, FALSE, alwaysTrue, trueBooleanTag, 1, nonNegativeIntegerChecker, nonNegativeIntegerCheckerNames };

/**** clearApplicativeBitCode  --  clear the applicative bit in the evaluation status register ****/

PROC( clearApplicativeBitCode )

    setApplicativeMode( FALSE );

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne clearApplicativeBitFD
    = { clearApplicativeBitCode, 455, FALSE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** clearApplicativeBitNDownCode  --  clear the applicative bit in the (zero-based) Nth stack frame down  ****/

PROC( clearApplicativeBitNDownCode )

    clearApplicativeModeNDown( R.u.c );

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne clearApplicativeBitNDownFD
    = { clearApplicativeBitNDownCode, 457, FALSE, alwaysTrue, trueBooleanTag, 1, nonNegativeIntegerChecker, nonNegativeIntegerCheckerNames };

/**** senseSwitchesCode -- get the sense switch settings ****/

PROC( senseSwitchesCode )

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( senseSwitchPositions, R )
    
    CONTINUE

END_PROC

functionDescriptorTypeOne senseSwitchesFD
    = { senseSwitchesCode, 458, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 0, anyChecker, anyCheckerNames };

/**** sliderCode -- get the sense switch settings ****/

PROC( sliderCode )

#ifdef USE_COCOA_IO  // If not, there are no sliders

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( sliderValues[ R.u.c ], R )
    
#else

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( bigRedButtonInterruptNumber, zero )
    
#endif

    CONTINUE

END_PROC

functionDescriptorTypeOne sliderFD
    = { sliderCode, 459, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 1, sliderChecker, sliderCheckerNames };

/**** enablePushbuttonCode -- takes a pushbutton number and an interrupt number. ****/

PROC( enablePushbuttonCode )

#ifdef USE_COCOA_IO  // If not, there are no pushbuttons.

    applicativeCheckDropOne( NameStrings[ 460 ] );

    UnsignedLongWord pushbuttonNo = (S[0]).u.c;
    UnsignedLongWord interruptNo  = R.u.c;

    pushbuttonInterruptNumbers[ pushbuttonNo ] = interruptNo;
    pushbuttonEnabled[ pushbuttonNo ] = TRUE;

#endif

    DROP

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne enablePushbuttonFD
    = { enablePushbuttonCode, 460, TRUE, alwaysTrue, trueBooleanTag, 2, pushbuttonInterruptChecker, pushbuttonInterruptCheckerNames };

/**** pushbuttonInterruptNumberCode -- takes a pushbutton number.  ****/

PROC( pushbuttonInterruptNumberCode )

#ifdef USE_COCOA_IO  // If not, there are no pushbuttons

    UnsignedLongWord pushbuttonNo = R.u.c;

    if( ! pushbuttonEnabled[ pushbuttonNo ] ) {
        tprintf3( &CurrentOutputPort, MiscStrings[ 513 ], pushbuttonNo );
        resetError( 340 );
        }

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( pushbuttonInterruptNumbers[ pushbuttonNo ], R )
    
#else

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( 0, R )
    
#endif

    CONTINUE

END_PROC

functionDescriptorTypeOne pushbuttonInterruptFD
    = { pushbuttonInterruptNumberCode, 461, TRUE, exactShortFixnumP, exactShortFixnumBitsMask,
        1, pushbuttonChecker, pushbuttonCheckerNames };

/**** disablePushbuttonCode -- takes a pushbutton number.  ****/

PROC( disablePushbuttonCode )

#ifdef USE_COCOA_IO  // If not, there are no pushbuttons

    UnsignedLongWord pushbuttonNo = R.u.c;

    applicativeCheckDropNone( NameStrings[ 462 ] );

    pushbuttonEnabled[ pushbuttonNo ] = FALSE;
    pushbuttonInterruptNumbers[ pushbuttonNo ] = 0;
    
#endif

    RETURN_DEFAULT

END_PROC

functionDescriptorTypeOne disablePushbuttonFD
    = { disablePushbuttonCode, 462, TRUE, alwaysTrue, trueBooleanTag, 1, pushbuttonChecker, pushbuttonCheckerNames };

/**** enableBigRedButtonCode -- takes an interrupt number. ****/

PROC( enableBigRedButtonCode )

#ifdef USE_COCOA_IO  // If not, there are no pushbuttons

    applicativeCheckDropNone( NameStrings[ 463 ] );

    bigRedButtonInterruptNumber = R.u.c;
    bigRedButtonEnabled = TRUE;

#endif

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne enableBigRedButtonFD
    = { enableBigRedButtonCode, 463, TRUE, alwaysTrue, trueBooleanTag, 1, interruptChecker, interruptCheckerNames };

/**** bigRedButtonInterruptNumberCode  ****/

PROC( bigRedButtonInterruptNumberCode )

#ifdef USE_COCOA_IO  // If not, there are no pushbuttons

    if( ! bigRedButtonEnabled )
        resetError( 341 );

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( bigRedButtonInterruptNumber, R )
    
#else
		
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( 0, R )
    
#endif

    CONTINUE

END_PROC

functionDescriptorTypeOne bigRedButtonInterruptFD
    = { bigRedButtonInterruptNumberCode, 464, TRUE, exactShortFixnumP, exactShortFixnumBitsMask, 0, anyChecker, anyCheckerNames };

/**** disableBigRedButtonCode.  ****/

PROC( disableBigRedButtonCode )

#ifdef USE_COCOA_IO  // If not, there are no pushbuttons

    applicativeCheckDropNone( NameStrings[ 465 ] );

    bigRedButtonEnabled = FALSE;
    bigRedButtonInterruptNumber = 0;
    
#endif

    RETURN_DEFAULT

END_PROC

functionDescriptorTypeOne disableBigRedButtonFD
    = { disableBigRedButtonCode, 465, TRUE, alwaysTrue, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** pushbuttonEnabledCode -- takes a pushbutton number.  ****/

PROC( pushbuttonEnabledCode )

#ifdef USE_COCOA_IO  // If not, there are no pushbuttons

    if( pushbuttonEnabled[ R.u.c ] )
        LOAD_R_TAG( trueBooleanTag )
    else
        LOAD_R_TAG( falseBooleanTag )
    
#else

    LOAD_R_TAG( falseBooleanTag )

#endif

    CONTINUE

END_PROC

functionDescriptorTypeOne pushbuttonEnabledFD
    = { pushbuttonEnabledCode, 466, TRUE, booleanP, trueBooleanTag, 1, pushbuttonChecker, pushbuttonCheckerNames };

/**** bigRedButtonEnabledCode ****/

PROC( bigRedButtonEnabledCode )

#ifdef USE_COCOA_IO  // If not, there are no pushbuttons

    if( bigRedButtonEnabled )
        LOAD_R_TAG( trueBooleanTag )
    else
        LOAD_R_TAG( falseBooleanTag )
    
#else

    LOAD_R_TAG( falseBooleanTag )

#endif

    CONTINUE

END_PROC

functionDescriptorTypeOne bigRedButtonEnabledFD
    = { bigRedButtonEnabledCode, 467, TRUE, booleanP, trueBooleanTag, 0, anyChecker, anyCheckerNames };

/**** setLevelIndicatorCode -- takes a level number and an integer in the range [0..1024]. ****/

PROC( setLevelIndicatorCode )

#ifdef USE_COCOA_IO  // If not, there are no level indicators

    applicativeCheckDropOne( NameStrings[ 468 ] );

    UnsignedLongWord levelIndicatorNo = (S[0]).u.c;

    DROP

    switch( levelIndicatorNo ) {
        case 0: tellController( E_LEVEL_INDICATOR_0_VALUE, R.u.c, ""); break;
        case 1: tellController( E_LEVEL_INDICATOR_1_VALUE, R.u.c, ""); break;
        case 2: tellController( E_LEVEL_INDICATOR_2_VALUE, R.u.c, ""); break;
        case 3: tellController( E_LEVEL_INDICATOR_3_VALUE, R.u.c, ""); break;
        }

#endif

    RETURN_DEFAULT
    CONTINUE

END_PROC

functionDescriptorTypeOne setLevelIndicatorFD
    = { setLevelIndicatorCode, 468, TRUE, alwaysTrue, trueBooleanTag,
        2, levelIndicatorNumberValueChecker, levelIndicatorNumberValueCheckerNames };

