/****************************************************************************************

SchemeGlobals.c -- Copyright (c) by Jay Reynolds Freeman 1988, 1989, 1990, 1991, 2006

Stuff to create load, bind, etcetera, various Scheme system globals for Wraith Scheme.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import "WraithHeader.h"
#import "OpCodes.h"
#import "Events.e"
#import "Globals.e"
#import "Intern.e"
#import "IO.e"
#import "Reader.e"
#import "SchemeAux.e"
#import "SchemeSpecials.e"
#import "SECDEval.e"
#import "SECDRegisters.e"
#import "StreamHandling.e"
#import "StringLoader.e"

/******** Stuff for Scheme system globals ********/

Text versionString[ 256 ];

/**** forward declarations ****/

/**** statics: ****/

/**** functions: ****/

/**** kittenizeString -- append "-<theKittenNumber>" to a string. Uses a static buffer, caller must copy if required. ****/

#define KITTENIZE_STRING_BUFFER_LENGTH 256

const Text *kittenizeString( const Text *str, Word aKitten, Boolean kittenizeMomP )
{
    static Text scratchText[ KITTENIZE_STRING_BUFFER_LENGTH ];

    if( kittenizeMomP || (aKitten > 0) ) {
        if( (strlen( str ) + 10) >= KITTENIZE_STRING_BUFFER_LENGTH )
            fatalError( 140 );
        sprintf( scratchText, "%s-%d", str, aKitten );
        return &(scratchText[ 0 ]);
        }
    else
        return str;
    }

/**** bindToOptionMinus -- Bind R to the OPTION_MINUS symbol, while saving R. ****/

void bindToOptionMinus()
{
    PUSH
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( kittenizeString( loadNameString( 271 ), theKittenNumber, FALSE ), R )
    CALL( intern )
    PUSH
    MOVE( STACK( 1 ), R )
    CALL( bindOrAssign )
    POP
    }

/**** rebindTopLoopVariables -- rebind ">+<", ">++<", ">+++<", ">*<", ">**<" and ">***<",
                                (or whatever equivalent symbols do their jobs), after one trip through a top-level loop.

                                The new value of ">*<" is assumed to be in R, and R is saved.             ****/

void rebindTopLoopVariables()
{
    PUSH
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( kittenizeString( loadNameString( 277 ), theKittenNumber, FALSE ), R )
    CALL( intern )
    PUSH                                                    /*                S: ( symbol >***< new >*< ...) */

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( kittenizeString( loadNameString( 276 ), theKittenNumber, FALSE ), R )
    CALL( intern )
    PUSH                                                    /*                S: (symbol >**< symbol >***< new >*< ...) */
    CALL( lookUp )                                          /* R: old >**<;   S: (symbol >**<  symbol >***< new >*< ...) */
    EXCHANGE( STACK( 0 ), STACK( 1 ) )                      /* R: old >**<;   S: (symbol >***< symbol >**< new >*< ...) */
    CALL( bindOrAssignTopLevel )                            /*                S: (symbol >**< new >*< ...) */

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( kittenizeString( loadNameString( 275 ), theKittenNumber, FALSE ), R )
    CALL( intern )
    PUSH                                                    /*                S: (symbol >*< symbol >**< new >*< ...) */
    CALL( lookUp )                                          /* R: old>*<;     S: (symbol >*< symbol >**< new >*< ...) */
    EXCHANGE( STACK( 0 ), STACK( 1 ) )                      /* R: old>*<;     S: (symbol >**< symbol >*< new >*< ...) */
    CALL( bindOrAssignTopLevel )                            /*                S: (symbol >*< new >*< ...) */

    MOVE( STACK( 1 ), R )                                   /* R: new >*<;    S: (symbol >*< new >*< ...) */
    CALL( bindOrAssignTopLevel )  

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( kittenizeString( loadNameString( 274 ), theKittenNumber, FALSE ), R )
    CALL( intern )
    PUSH                                                    /*                S: ( symbol >+++< new >*< ...) */

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( kittenizeString( loadNameString( 273 ), theKittenNumber, FALSE ), R )
    CALL( intern )
    PUSH                                                    /*                S: (symbol >++< symbol >+++< new >*< ...) */
    CALL( lookUp )                                          /* R: old >++<;   S: (symbol >++< symbol >+++< new >*< ...) */
    EXCHANGE( STACK( 0 ), STACK( 1 ) )                      /* R: old >++<;   S: (symbol >+++< symbol >++< new >*< ...) */
    CALL( bindOrAssignTopLevel )                            /*                S: (symbol >++< new >*< ...) */

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( kittenizeString( loadNameString( 272 ), theKittenNumber, FALSE ), R )
    CALL( intern )
    PUSH                                                    /*                S: (symbol >+< symbol >++< new >*< ...) */
    CALL( lookUp )                                          /* R: old >+<;    S: (symbol >+< symbol >++< new >*< ...) */
    EXCHANGE( STACK( 0 ), STACK( 1 ) )                      /* R: old >+<;    S: (symbol >++< symbol >+< new >*< ...) */
    CALL( bindOrAssignTopLevel )                            /* S: (symbol >+< new >*< ...) */

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( kittenizeString( loadNameString( 271 ), theKittenNumber, FALSE ), R )
    CALL( intern )
    CALL( lookUp )                                          /* R: new >-<;    S: (symbol >+< new >*< ...) */
    CALL( bindOrAssignTopLevel )                            /*                S: (new >*< ...) */

    POP
    }

/**** loadTaggedAvalGlobal -- Install a named taggedAval into the environment. ****/

void loadTaggedAvalGlobal( const Text *pName, taggedAval *initialValue )
{
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( pName, R )
    CALL( intern )
    PUSH
    PUSH
    R = *initialValue;
    SWAP
    CALL( bindScheme )
    POP
    CALL( makePermanentSymbol )
    }

/**** loadTaggedAvalGlobalNonPermanent -- Install a named, non-permanent taggedAval into the environment. ****/

void loadTaggedAvalGlobalNonPermanent( const Text *pName, taggedAval *initialValue )
{
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( pName, R )
    CALL( intern )
    PUSH
    R = *initialValue;
    SWAP
    CALL( bindScheme )
    }

/**** loadCharacterGlobal -- Install a named character into the environment. ****/

void loadCharacterGlobal( const Text *pName, Text c )
{
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( pName, R )
    CALL( intern )
    PUSH
    PUSH
    LOAD_R_TAG( characterTag )
    LOAD_CHARACTER_OBJECT( c, R )
    SWAP
    CALL( bindScheme )
    POP
    CALL( makePermanentSymbol )
    }

/**** loadStringGlobal -- Install a named string into the environment. ****/

void loadStringGlobal( const Text *pName, const Text *stringText )
{
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( pName, R )
    CALL( intern )
    PUSH
    PUSH
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( stringText, R )
    MAKE_STRING;
    SWAP
    CALL( bindScheme )
    POP
    CALL( makePermanentSymbol )
    }

/**** loadSymbolGlobal  -- Install an alternate name for a symbol into the environment. ****/

void loadSymbolGlobal( const Text *newPName, const Text *pNameToLookUp )
{
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( newPName, R )
    CALL( intern )
    PUSH
    PUSH
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( pNameToLookUp, R )
    CALL( intern );
    SWAP
    CALL( bindScheme )
    POP
    CALL( makePermanentSymbol )
    }

/**** assertEvaluating ****/

void assertEvaluating()
{
    theClowder->setKittenEvaluating();
    reportEvent( START_EVALUATION_EVENT );
    eventLoopOnce( 0 );
    }

/**** assertNotEvaluating ****/

void assertNotEvaluating()
{
    theClowder->clearKittenEvaluating();
    reportEvent( END_EVALUATION_EVENT );
    eventLoopOnce( 0 );
    }

/**** newlineIfPrinting  --  used only in top-level loop code, as loaded in "loadSchemeGlobals", below.  ****/

void newlineIfPrinting()
{
//    tprintf4( &CurrentOutputPort, "Peculiar leaf? %d; number %lu", (peculiar_leaf_p( R.t ) ? 1 : 0), leaf_number( R ) );  // DEBUG
    if( peculiar_leaf_p( R.t ) && (leaf_number( R ) == nonPrintingObjectTag ) ) {
        CSDrop();
        CSDrop();
        }
    }

/**** loadSchemeGlobals -- Load a few special globals into the environment. ****/

void loadSchemeGlobals( Boolean showListP )
{
    const Text *scratchString;
    taggedAval foo;
    Word aKitten = 0;

    if( showListP )
        tprintf2( &CurrentOutputPort, MiscStrings[ 69 ] );
    if( showListP )
        tprintf2( &CurrentOutputPort, MiscStrings[ 124 ] );
    if( showListP )
        tprintf2( &CurrentOutputPort, MiscStrings[ 70 ] );
    versionString[ 0 ] = 0;
    strcat( versionString, MiscStrings[ 508 ] );
    strcat( versionString, " ");
#ifndef PIXIE_SCHEMES
    strcat( versionString, MiscStrings[ 510 ] );
    strcat( versionString, " ");
#endif
    strcat( versionString, releaseDate );
    strcat( versionString, " ");
    strcat( versionString, releaseTime );

    if( showListP )
        tprintf2( &CurrentOutputPort, MiscStrings[ 124 ] );
    if( showListP )
        tprintf2( &CurrentOutputPort, MiscStrings[ 71 ] );
    scratchString = loadNameString( 101 );                                          /* else */
    loadSymbolGlobal( scratchString, scratchString );
    scratchString = loadNameString( 311 );                                          /* => */
    loadSymbolGlobal( scratchString, scratchString );
    if( showListP )
        tprintf2( &CurrentOutputPort, MiscStrings[ 124 ] );
    if( showListP )
        tprintf2( &CurrentOutputPort, MiscStrings[ 72 ] );
    foo.t = schemeMachinePCIndexTag;
    foo.u.c = SPushFunction_index;
    loadTaggedAvalGlobal( loadNameString( 252 ), &foo );                            /* c::push */
    foo.t = schemeMachinePCIndexTag;
    foo.u.c = apply_index;
    loadTaggedAvalGlobal( loadNameString( 253 ), &foo );                            /* c::apply */
    foo.t = schemeMachinePCIndexTag;
    foo.u.c = resetScheme_index;
    loadTaggedAvalGlobal( loadNameString( 257 ), &foo );                            /* c::reset */
    foo.t = schemeMachinePCIndexTag;
    foo.u.c = applyEnd_index;
    loadTaggedAvalGlobal( loadNameString( 259 ), &foo );                            /* c::apply-end */
    foo.t = schemeMachinePCIndexTag;
    foo.u.c = noop_index;
    loadTaggedAvalGlobal( loadNameString( 262 ), &foo );                            /* c::noop */
    foo.t = schemeMachinePCIndexTag;
    foo.u.c = S0toC_index;
    loadTaggedAvalGlobal( loadNameString( 263 ), &foo );                            /* c::s0-to-c */
    foo.t = schemeMachinePCIndexTag;
    foo.u.c = dereferenceStackReferenceMC_index;
    loadTaggedAvalGlobal( loadNameString( 264 ), &foo );                            /* c::stack-ref */

    for( aKitten = 0; aKitten < ABSOLUTE_MAX_KITTENS; ++aKitten ) {
        loadTaggedAvalGlobalNonPermanent( kittenizeString( loadNameString( 271 ), aKitten, FALSE ), &empty ); /* >-< */
        loadTaggedAvalGlobalNonPermanent( kittenizeString( loadNameString( 272 ), aKitten, FALSE ), &empty ); /* >+< */
        loadTaggedAvalGlobalNonPermanent( kittenizeString( loadNameString( 273 ), aKitten, FALSE ), &empty ); /* >++< */
        loadTaggedAvalGlobalNonPermanent( kittenizeString( loadNameString( 274 ), aKitten, FALSE ), &empty ); /* >+++< */
        loadTaggedAvalGlobalNonPermanent( kittenizeString( loadNameString( 275 ), aKitten, FALSE ), &empty ); /* >*< */
        loadTaggedAvalGlobalNonPermanent( kittenizeString( loadNameString( 276 ), aKitten, FALSE ), &empty ); /* >**< */
        loadTaggedAvalGlobalNonPermanent( kittenizeString( loadNameString( 277 ), aKitten, FALSE ), &empty ); /* >***< */
        foo.t = schemeMachinePCIndexTag;
        foo.u.c = kittenDequeueCode_index;
        loadTaggedAvalGlobal( kittenizeString( loadNameString( 332 ), aKitten, TRUE ), &foo );  /* c::kitten-dequeue */
        }

    //  This is where we create the code fragment that repeatedly runs the top-level loop ...

    R.u.c = ClearLFunction_index;              R.t = schemeMachinePCIndexTag;    SPushR();
    R.u.c = readSexpOrQueueTopLevel_index;     R.t = schemeMachinePCIndexTag;    SPushR();
    R.u.c = SPushFunction_index;               R.t = schemeMachinePCIndexTag;    SPushR();
    R.u.c = assertEvaluating_index;            R.t = schemeMachinePCIndexTag;    SPushR();	
    R.u.c = SPopRFunction_index;               R.t = schemeMachinePCIndexTag;    SPushR();
    R.u.c = bindToOptionMinus_index;           R.t = schemeMachinePCIndexTag;    SPushR();       
    R.u.c = CPushFunction_index;               R.t = schemeMachinePCIndexTag;    SPushR();
    R.u.c = rebindTopLoopVariables_index;      R.t = schemeMachinePCIndexTag;    SPushR();       
    R.u.c = writeObject_index;                 R.t = schemeMachinePCIndexTag;    SPushR();
    R.u.c = newlineIfPrinting_index;           R.t = schemeMachinePCIndexTag;    SPushR();
    R = newlineChar;                                                             SPushR();
    R.u.c = displayObject_index;               R.t = schemeMachinePCIndexTag;    SPushR();       
    R.u.c = assertNotEvaluating_index;         R.t = schemeMachinePCIndexTag;    SPushR();	
    R.u.c = topLoopCode_index;                 R.t = schemeMachinePCIndexTag;

    nilCons();
    cons();
    cons();
    cons();
    cons();
    cons();
    cons();
    cons();
    cons();
    cons();
    cons();
    cons();
    cons();
    cons();

    foo = R;
    TLC = R;

    loadTaggedAvalGlobal( loadNameString( 75 ), &foo );                             /* e::top-loop-code */

    if( showListP )
        tprintf2( &CurrentOutputPort, MiscStrings[ 124 ] );
    }

/**** loadKittenizedSchemeGlobals -- Load a few special globals into the environment, for just one kitten. ****/

void loadKittenizedSchemeGlobals( Word aKitten )
{
    loadTaggedAvalGlobalNonPermanent( kittenizeString( loadNameString( 271 ), aKitten, FALSE ), &empty ); /* >-< */
    loadTaggedAvalGlobalNonPermanent( kittenizeString( loadNameString( 272 ), aKitten, FALSE ), &empty ); /* >+< */
    loadTaggedAvalGlobalNonPermanent( kittenizeString( loadNameString( 273 ), aKitten, FALSE ), &empty ); /* >++< */
    loadTaggedAvalGlobalNonPermanent( kittenizeString( loadNameString( 274 ), aKitten, FALSE ), &empty ); /* >+++< */
    loadTaggedAvalGlobalNonPermanent( kittenizeString( loadNameString( 275 ), aKitten, FALSE ), &empty ); /* >*< */
    loadTaggedAvalGlobalNonPermanent( kittenizeString( loadNameString( 276 ), aKitten, FALSE ), &empty ); /* >**< */
    loadTaggedAvalGlobalNonPermanent( kittenizeString( loadNameString( 277 ), aKitten, FALSE ), &empty ); /* >***< */

    }

