/****************************************************************************************

SchemeSpecials.c -- Copyright (c) by Jay Reynolds Freeman 1988, 1989, 1990, 1991, 2006

Scheme special forms for Wraith, and all code that needs
to know about the "specialFormDescriptor" structure.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import "WraithHeader.h"
#import "Opcodes.h"
#import "SchemeSpecials.h"
#import "DataTypes.e"
#import "Globals.e"
#import "Intern.e"
#import "IO.e"
#import "StringLoader.e"
#import "SchemeAux.e"
#import "SchemeFunctions.e"
#import "StringLoader.e"

/**** forward declarations ****/

void lambdaEvalAux();
void quoteSimpleEvalCode();
void setBangEvalCode();

/******** Stuff for Scheme built-in special forms ********/

/**** beginEvalCode -- Deal with a "begin" special form:  Load the continuation from
                       the cdr of the "begin" form, pushing the last item of that cdr
                       first.  ****/

PROC( beginEvalCode )           /* on entry, R: (begin . body) */

    CDR          
    EMPTY_LIST_P
    JUMP_FALSE( nonEmpty )
    resetError( 106 );

  nonEmpty:
    LIST_P
    JUMP_FALSE( improperBody )  
    PUSH
    CSPushProperList();
    POP
    CONTINUE

  improperBody:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 243 );
    CONTINUE
        
END_PROC

/**** compiledBlockEvalCode ****/

PROC( compiledBlockEvalCode )

    CDR          
    CSPushProperList();
    CONTINUE

END_PROC

/**** defineEvalCode -- Code for the "define" special form.  "Define"s are
                        allowed only in the top-level environment.  Note that
                        "bindOrAssign" returns the symbol in question, so that
                        "define" also returns the symbol as its result.                                      ****/

PROC( defineEvalCode )          /* on entry, R: (define . argList) */

    PUSH
    MOVE( E, R )
    LENGTH
    PUSH
    MOVE( three, R )
    NUM_EQ_P

    JUMP_FALSE( notTopLevel )

    POP
    POP
    CDR                             /* R: args */
    PUSH
    LENGTH
    PUSH
    MOVE( two, R )
    NUM_LT_P
    JUMP_TRUE( badArity )
    NUM_EQ_P
    DROP
    MOVE( STACK( 0 ), R )
    JUMP_FALSE( mustBeSugar )

    CAR
    CALL( pairP )
    JUMP_TRUE( isSugar )
    POP     
    LOAD_STACK
    SWAP       /* R: <sym>; S: (<valExp> ...) */
    CALL( symbolP )
    JUMP_FALSE( notSymbol )

    /* C <- (<valExp> bindOrAssign . C) */

    SWAP       /* S: (<sym> ...) */
    PUSH                            /* S: (<valExp> <sym> ...) */
    LOAD_OBJECT( bindOrAssign_index, R )
    LOAD_R_TAG( schemeMachinePCIndexTag )
    PUSH_CONTINUATION
    POP                             /* S: (<sym> ...) */
    PUSH_CONTINUATION
    CONTINUE

  mustBeSugar:                      /* R: ((mustBePair) . body); S: ((mustBePair) . body) ...) */
    CAR
    CALL( pairP )
    JUMP_FALSE( needPair )

  isSugar:
    SWAP       /* R: ((mustBePair) . body); S: (<pair> ...) */
    CDR          
    SWAP       /* R: <pair>;           S: (body ...) */
    PUSH
    CAR          
    SYMBOL_P
    JUMP_FALSE( notSymbolSugar )
    SWAP
    CDR                             /* R: lambda-list;      S: (<symbol> body ...) */
    EXCHANGE( STACK( 0 ), STACK( 1 ) )              /* R: lambda-list;      S: (body <symbol> ...) */
    CALL( lambdaEvalAux )                                   /* R: lambdaPair;       S: (<symbol> ...) */
    JUMP_FUNCTION( bindOrAssign )   

  badArity:
    OUTPUT_TO_ERROR
    POP
    DISPLAY
    resetError( 121 );
    CONTINUE

  notTopLevel:
 
    POP
    POP
    CDR                             /* R: args */
    PUSH
    LENGTH
    PUSH
    MOVE( two, R )
    NUM_LT_P
    JUMP_TRUE( badArity )
    NUM_EQ_P
    DROP
    MOVE( STACK( 0 ), R )
    JUMP_FALSE( mustBeSugar2 )

    CAR
    CALL( pairP )
    JUMP_TRUE( isSugar2 )
    POP     
    LOAD_STACK
    SWAP       /* R: <sym>; S: (<valExp> ...) */
    CALL( symbolP )
    JUMP_FALSE( notSymbol )

    /* C <- (<sym> bindScheme . C) */

    PUSH                            /* S: (<sym> <valExp> ...) */
    LOAD_OBJECT( bindScheme_index, R )
    LOAD_R_TAG( schemeMachinePCIndexTag )
    PUSH_CONTINUATION
    POP                             /* R: <sym>; S: (<valExp> ...) */
    NILCONS                         /* R: (<sym>); S: (<valExp> ...) */
    PUSH                            
    LOAD_OBJECT( loadNameString( 281 ), R )
    CALL( getSymbol )
    SWAP             /* R: (<sym>); S: (c::quote-simple <valExp> ...) */
    CONS                                  /* R: (c::quote-simple sym); S: (<valExp> ?)   */

    PUSH_CONTINUATION

    LOAD_OBJECT( SPushFunction_index, R )      // Create code to push the evaluated <valExp>
    LOAD_R_TAG( schemeMachinePCIndexTag )
    PUSH_CONTINUATION

    POP

    PUSH_CONTINUATION
    CONTINUE

  mustBeSugar2:                      /* R: ((mustBePair) . body); S: ((mustBePair) . body) ...) */
    CAR
    CALL( pairP )
    JUMP_FALSE( needPair )

  isSugar2:
    SWAP       /* R: ((mustBePair) . body); S: (<pair> ...) */
    CDR          
    SWAP       /* R: <pair>;           S: (body ...) */
    PUSH
    CAR          
    SYMBOL_P
    JUMP_FALSE( notSymbolSugar )
    SWAP
    CDR                             /* R: lambda-list;      S: (<symbol> body ...) */
    EXCHANGE( STACK( 0 ), STACK( 1 ) )              /* R: lambda-list;      S: (body <symbol> ...) */
    CALL( lambdaEvalAux )                                   /* R: lambdaPair;       S: (<symbol> ...) */
    SWAP       /* S: (<sym> ...) */
    JUMP_FUNCTION( bindScheme )   

  notSymbol:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 123 );
    CONTINUE

  notSymbolSugar:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 198 );
    CONTINUE

  needPair:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 226 );
    CONTINUE

END_PROC

/**** ifAuxCode -- Code to help with evaluating an "if" by the interpreter ****/

PROC( ifAuxCode )               /* on entry: R: evaluated predicate; C: (alternative consequent ...) */

    FALSE_P
    JUMP_TRUE( doAlternative )

    CSDrop();

    CONTINUE

  doAlternative:
    CSPopR();
    CSDrop();
    CSPush( R );

    CONTINUE

END_PROC

/**** if3EvalCode -- Code to deal with an "if3" special form -- an
                     "if" that is known to have exactly three arguments.         ****/

PROC( if3EvalCode )                     /* on entry, R: (if . argList) */
 
    /* C <- (test ifAuxCode alternative consequent) */

    /*
       The change in the order of "consequent" and "alternative" is based
       on the folklore that the consequents of if-statements get chosen more
       often than the alternatives:  The present software requires consing to
       take whatever choice is first in the continuation, but does not require
       consing to take the other choice.  This strategy thereby produces less
       overall consing.
     */

    CDR                                                     /* R: (test consequent alternative) */
    LOAD_STACK                                              /* R: alternative;      S: (consequent test ...) */
    SWAP
    PUSH_CONTINUATION
    POP
    PUSH_CONTINUATION
    LOAD_OBJECT( ifAuxCode_index, R )
    LOAD_R_TAG( schemeMachinePCIndexTag )
    PUSH_CONTINUATION
    POP
    PUSH_CONTINUATION
    CONTINUE

END_PROC

/**** lambdaEvalAux -- Auxiliary for handling lambda expressions, also used
                       when there is syntactic sugar on top of "define".                     ****/

PROC( lambdaEvalAux )

    /* R: lambdaList; S: (lambdaBody ...) */

    CALL( lambdaListCheck )
    SWAP                       /* R: lambdaBody; S: (lambdaList ...) */
    EMPTY_LIST_P
    JUMP_TRUE( nullBody )
    CALL( lambdaBodyCheck )
    JUMP_FALSE( malformedBody )

    PUSH
    MOVE( E, R )
    NILCONS
    CONS
    CONS

    LOAD_R_TAG( lambdaPairTag )

    CONTINUE

  nullBody:
    resetError( 238 );
    CONTINUE

  malformedBody:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 239 );

    CONTINUE

END_PROC

/**** lambdaEvalCode -- This is what to do on seeing a lambda expression, not how to apply one. ****/

PROC( lambdaEvalCode )

    CDR          
    EMPTY_LIST_P
    JUMP_FALSE( hasLambdaList )
    resetError( 127 );
        
  hasLambdaList:
    PUSH
    CDR          
    SWAP
    CAR                                                     /* R: lambdaList; S: (lambdaBody ...) */
    JUMP_FUNCTION( lambdaEvalAux )

END_PROC

/**** macroBodyAux -- Assume what's in R is a macro and so set its tag.
                      Type check first that R does contain a procedure. ****/

PROC( macroBodyAux )            /* R: macro? */

    CALL( procedureP )
    JUMP_FALSE( notProcedure )
    LOAD_R_TAG( macroTag )
    CONTINUE

  notProcedure:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 234 );
    CONTINUE

END_PROC        

/**** macroBodyCode -- Deal with a "macro" special form:  Take the cdr and call it a macro.     */      

PROC( macroBodyCode )           /* on entry, R: (macro-body) */

    PUSH
    LOAD_R_TAG( schemeMachinePCIndexTag )
    LOAD_OBJECT( macroBodyAux_index, R )
    PUSH_CONTINUATION
    POP                             /* R: (body) */
    CDR
    PAIR_P
    JUMP_FALSE( malformed )
    LOAD_CONTINUATION
    CONTINUE

  malformed:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 236 );
    CONTINUE        
        
END_PROC

/**** macroEvalAux -- Assume what's in R is a macro and so set its tag.
                      Type check first that R does contain a procedure. ****/

// This procedure should differ from "macroSetAux" only in using "defineEvalCode"
// where the other uses "setBangEvalCode".

PROC( macroEvalAux )            /* R: macro?; S: (name ...) */

    CALL( procedureP )
    JUMP_FALSE( notProcedure )
    LOAD_R_TAG( macroTag )
    NILCONS
    CONS
    PUSH
    LOAD_R_TAG( falseBooleanTag )
    SWAP
    CONS                                    /* R: (X name procedure) -- "X" substitutes for "define" */
    PUSH
    LOAD_R_TAG( schemeMachinePCIndexTag )
    LOAD_OBJECT( defineEvalCode_index, R )
    PUSH_CONTINUATION
    POP     
    CONTINUE

  notProcedure:
    POP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 234 );
    CONTINUE

END_PROC        

/**** macroEvalCode -- Deal with a "macro" special form:  Take the cdr and call it a macro.     */      

PROC( macroEvalCode )           /* on entry, R: (macro name . body) */

    PUSH
    LOAD_R_TAG( schemeMachinePCIndexTag )
    LOAD_OBJECT( macroEvalAux_index, R )
    PUSH_CONTINUATION
    POP
    CDR                             /* R: (name . body ) */
    PAIR_P
    JUMP_FALSE( malformed )
    PUSH
    CAR          
    SYMBOL_P
    JUMP_FALSE( needSymbol )
    SWAP
    CDR          
    LOAD_CONTINUATION
    CONTINUE

  malformed:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 236 );
    CONTINUE        
        
  needSymbol:
    DROP
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 237 );
    CONTINUE        
        
END_PROC

/**** quoteSimpleEvalCode -- Merely surpress evaluation; don't fuss the immutable bit:  ****/

PROC( quoteSimpleEvalCode )             /* on entry, R: (quote . argList) */
 
    CDR                                 /* R: args */
    PUSH
    MOVE( one, R )
    ARITY_CHECK
    JUMP_FALSE( wrongArity )

    DROP
    POP
    CAR
    CONTINUE

  wrongArity:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 43 );
    CONTINUE

END_PROC

/**** quoteEvalCode -- Deal with a "quote" special form:                        ****/

PROC( quoteEvalCode )           /* on entry, R: (quote . argList) */
 
    CDR                     /* R: args */
    PUSH
    MOVE( one, R )
    ARITY_CHECK
    JUMP_FALSE( wrongArity )

    DROP
    POP
    CAR
    CALL( makeImmutable )

    CONTINUE

  wrongArity:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 136 );
    CONTINUE

END_PROC

/**** setBangEvalCode -- Deal with a "set!" special form. ****/

PROC( setBangEvalCode )         /* on entry, R: (set! . argsList) */
 
    CDR                         /* R: argList */
    PUSH
    MOVE( two, R )
    ARITY_CHECK
    JUMP_FALSE( wrongArity )

    DROP
    POP
    LOAD_STACK
    SWAP
    CALL( symbolP )
    JUMP_FALSE( notSymbol )

    applicativeCheckDropOne( NameStrings[ 157 ] );

    /* C <- (<valExp> assign . C) */

    SWAP
    PUSH                        /* S: (<valExp> <sym> ...) */
    LOAD_R_TAG( schemeMachinePCIndexTag )
    LOAD_OBJECT( assign_index, R )
    PUSH_CONTINUATION
    POP                         /* S: (<sym> ...) */
    PUSH_CONTINUATION
    CONTINUE

  wrongArity:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 137 );
    CONTINUE

  notSymbol:
    TRUE_P
    JUMP_TRUE( veritas )
    FALSE_P
    JUMP_TRUE( nihil )
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 138 );
    CONTINUE

  veritas:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 268 );
    CONTINUE

  nihil:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 269 );
    CONTINUE

END_PROC

/**** cSetBangEvalCode -- Deal with a "set!" special form. ****/

PROC( cSetBangEvalCode )         /* on entry, R: (set! . argsList) */
 
    CDR                         /* R: argList */
    PUSH
    MOVE( two, R )
    ARITY_CHECK
    JUMP_FALSE( wrongArity )

    DROP
    POP
    LOAD_STACK
    SWAP
    CALL( symbolP )
    JUMP_FALSE( notSymbol )

    /* C <- (<valExp> assign . C) */

    SWAP
    PUSH                        /* S: (<valExp> <sym> ...) */
    LOAD_R_TAG( schemeMachinePCIndexTag )
    LOAD_OBJECT( assign_index, R )
    PUSH_CONTINUATION
    POP                         /* S: (<sym> ...) */
    PUSH_CONTINUATION
    CONTINUE

  wrongArity:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 333 );
    CONTINUE

  notSymbol:
    TRUE_P
    JUMP_TRUE( veritas )
    FALSE_P
    JUMP_TRUE( nihil )
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 334 );
    CONTINUE

  veritas:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 268 );
    CONTINUE

  nihil:
    OUTPUT_TO_ERROR
    DISPLAY
    resetError( 269 );
    CONTINUE

END_PROC

/**** Statics: ****/

static specialFormDescriptor beginSFD                   = { beginEvalCode, 243 };
static specialFormDescriptor compiledBlockSFD           = { compiledBlockEvalCode, 265 };
static specialFormDescriptor cSetBangSFD                = { cSetBangEvalCode, 449 };
static specialFormDescriptor defineSFD                  = { defineEvalCode, 78 };
static specialFormDescriptor if3SFD                     = { if3EvalCode, 260 };
static specialFormDescriptor lambdaSFD                  = { lambdaEvalCode, 119 };
static specialFormDescriptor macroSFD                   = { macroEvalCode, 247 };
static specialFormDescriptor quoteSFD                   = { quoteEvalCode, 150 };
static specialFormDescriptor quoteSimpleSFD             = { quoteSimpleEvalCode, 281 };
static specialFormDescriptor setBangSFD                 = { setBangEvalCode, 157 };
static specialFormDescriptor macroBodySFD               = { macroBodyCode, 310 };
        
/**** The Special Form List -- which an initialization routine uses to construct
      a more rapidly accessed structure for actual use at run-time.                     ****/

static simpleListCell specialFormList[] = {
    { 1, &beginSFD },
    { 1, &compiledBlockSFD },
    { 1, &cSetBangSFD },
    { 1, &defineSFD },
    { 1, &if3SFD },
    { 1, &lambdaSFD },
    { 1, &macroSFD },
    { 1, &quoteSFD },
    { 1, &quoteSimpleSFD },
    { 1, &setBangSFD },
    { 1, &macroBodySFD },
    { 0, (void *)NULL }
    };

/**** More functions: ****/

/**** describeSpecialFormDescriptor -- Output a description of a special-form
                                       descriptor for the user to peruse.            ****/

void describeSpecialFormDescriptor( specialFormDescriptor *d )
{       
    if( d )
        tprintf4( &CurrentOutputPort, MiscStrings[ 86 ], d, d->pNameNumber );
    else
        tprintf2( &CurrentOutputPort, MiscStrings[ 103 ] );
    }

/**** describeSpecialFormList -- Describe all the special-form descriptors in the given list. ****/

void describeSpecialFormList( simpleList m )
{
    tprintf2( &CurrentOutputPort, MiscStrings[ 76 ] );
    while( m->flag ) {
        describeSpecialFormDescriptor( (specialFormDescriptor *)(m->pointer) );
        ++m;
        }
    }

/**** getSpecialEvalCodeIndex -- Get the code for the given special-form descriptor. ****/

void getSpecialEvalCodeIndex()
{
    R.t = vectorTag;
    vectorRefN( (LongWord)0 );
    }

/**** getSpecialPName -- Get the name of the given special-form descriptor. ****/
        
void getSpecialPName()
{
    R.t = vectorTag;
    vectorRefN( (LongWord)1 );
    }
        
/**** loadSchemeSpecialForm -- Install the given special form into the environment. ****/

extern FILE *logFile;

void loadSchemeSpecialForm( specialFormDescriptor *f )
{
    LongWord vectorLnth = (LongWord)2;
    taggedAval theVector;

    /* cons up list of things to put into a vector */

    R.u.c = (theWorldHandler->indexForPC( (UnsignedLongWord)(f->evalCode) ));
    R.t = schemeMachinePCIndexTag;
    NILCONS
    PUSH                                                    /* push (index of code to execute function) */
    PUSH                                                    /* push list of code to execute the special */
    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( loadNameString( f->pNameNumber ), R )
    CALL( intern )                                          /* interning now avoids storing two copies of this string */
    GET_PNAME 
    PUSH

    R.u.c = vectorLnth;
    R.t = exactShortFixnumBitsMask;
    CALL( makeVectorAndStuffFromStack )
    theVector = R;

    /* intern the print name of the special form, and assign the vector to it */

    LOAD_R_TAG( exactShortFixnumBitsMask )
    LOAD_OBJECT( loadNameString( f->pNameNumber ), R )
    CALL( intern )
    PUSH
    PUSH
    MOVE( theVector, R )

    LOAD_R_TAG( specialFormTag )
    SWAP
    CALL( bindScheme )
    POP
    JUMP_FUNCTION( makePermanentSymbol )
    }

/**** Conventions about the vector into which the stuff from the specialFormDescriptor gets
      stuffed, documented here for want of any obvious other place:
          
      All slots contain Scheme objects (tagged Avals); not C objects.

      Slot #                Contents

       0                    scheme-machine pc index to code to execute the special form
       1                    print name              
       2...                 more to come ...

       ****/

/**** loadSchemeSpecialForms -- Load all the special forms on the (global) specialFormList
                                into the environment, optionally describing each one in turn. ****/

#define LOADS_PER_SPIN          2

void loadSchemeSpecialForms( Boolean showListP )
{
    simpleListCell *l = specialFormList;

    if( showListP ) {
        describeSpecialFormList( l );
        tprintf2( &CurrentOutputPort, MiscStrings[ 73 ] );
        }
    while( l->flag ) {
        loadSchemeSpecialForm( (specialFormDescriptor *)(l->pointer) );
        ++l;
        }
    if( showListP )
        tprintf2( &CurrentOutputPort, MiscStrings[ 124 ] );
    }
