/****************************************************************************************

StreamHandling.c -- Copyright (c) by Jay Reynolds Freeman 1987, 1988, 1989, 1990, 1991, 2006

Stream handling utilities for Wraith Scheme.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import "WraithHeader.h"
#import "Globals.e"
#import "IO.e"

/******** Statics and externals: ********/

Boolean descriptorsInitializedP = false;

fileDescriptor AllFileDescriptors[ nBigArray ];

fileDescriptor *SchemeInputFiles      = &(AllFileDescriptors[0]);
fileDescriptor *SchemeOutputFiles     = (fileDescriptor *)((Byte *)SchemeInputFiles + nInputFileDescriptors * sizeof( fileDescriptor ));
fileDescriptor *conIn                 = (fileDescriptor *)((Byte *)SchemeOutputFiles + nOutputFileDescriptors * sizeof( fileDescriptor ));
fileDescriptor *conOut                = (fileDescriptor *)((Byte *)conIn + 1 * sizeof( fileDescriptor ));
fileDescriptor *conErr                = (fileDescriptor *)((Byte *)conOut + 1 * sizeof( fileDescriptor ));
fileDescriptor *transcriptFD          = (fileDescriptor *)((Byte *)conErr + 1 * sizeof( fileDescriptor ));
fileDescriptor *worldFD               = (fileDescriptor *)((Byte *)transcriptFD + 1 * sizeof( fileDescriptor ));
fileDescriptor *closedFile            = (fileDescriptor *)((Byte *)worldFD + 1 * sizeof( fileDescriptor ));

/**** File descriptors that are widely used. ****/

fileDescriptor *masterDefaultInputFD = conIn;

/**** describeFileDescriptor ****/

void describeFileDescriptor( taggedAval *how, fileDescriptor *which )
{
    tprintf8( how, MiscStrings[ 308 ],
              which->pathName, which->oldPathName, (UnsignedLongWord)(which->fp),
              which->fdOpen, which->type, which->kittenNumber );
    }

/**** initializeDefaultPorts ****/

void initializeDefaultPorts()
{
    }

/**** initializeFileDescriptor ****/

void initializeFileDescriptor( fileDescriptor *fd, FILE *theFP, Text *name )
{
    Text *p;
  
    fd->fp = theFP;
    fd->fdOpen = FALSE;
    fd->fdInUse = FALSE;
    fd->kittenNumber = theKittenNumber;
    p = (Text *)(malloc( MAX_PATHNAME_LENGTH * sizeof( Text ) ));
    if( p == (Text *)NULL )
        fatalError( 19 );
    fd->pathName = p;
    p = (Text *)(malloc( MAX_PATHNAME_LENGTH * sizeof( Text ) ));
    if( p == (Text *)NULL )
        fatalError( 19 );
    fd->oldPathName = p;
    p = (Text *)malloc( MAX_TYPE_LENGTH * sizeof( Text ) );
    if( p == (Text *)NULL )
        fatalError( 21 );
    fd->type = p;
    strcpy( fd->pathName, "" );
    strcpy( fd->oldPathName, "");
    strcpy( fd->type, "" );
    }

/**** initializeFileDescriptorsIfNecessary ****/

void initializeFileDescriptorsIfNecessary()
{
    UnsignedLongWord i = 0;

    if( ! descriptorsInitializedP ) {
        for( i = 0; i < nBigArray; ++i )
            (AllFileDescriptors[ i ]).placeInBigArray = i;

        for( i = 0; i < nInputFileDescriptors; ++i )
            initializeFileDescriptor( &(SchemeInputFiles[i]), 0, MiscStrings[ 302 ] );
        for( i = 0; i < nOutputFileDescriptors; ++i )
            initializeFileDescriptor( &(SchemeOutputFiles[i]), 0, MiscStrings[ 303 ] );
        initializeFileDescriptor( transcriptFD, 0, 0 );
        initializeFileDescriptor( conIn, stdin, MiscStrings[ 205 ] );
        initializeFileDescriptor( conOut, stdout, MiscStrings[ 206 ] );
        initializeFileDescriptor( conErr, stderr, MiscStrings[ 304 ] );
        initializeFileDescriptor( worldFD, 0, 0 );

        descriptorsInitializedP = true;
        }
    }

/**** reinitializeFileDescriptor ****/

void reinitializeFileDescriptor( fileDescriptor *fd )
{
    if( (fd != conIn) && (fd != conOut) && (fd != conErr) ) {
        fd->fdInUse = FALSE;
        fd->fdOpen = FALSE;
        if( strcmp( fd->pathName, "" ) != 0 ) {
            strcpy( fd->oldPathName, fd->pathName );
            strcpy( fd->pathName, "" );
            }
        strcpy( fd->type, "" );
        fd->fp = NULL;
        }
    }

/**** fflushConsoleOutput -- must be callable as a scheme primitive. ****/

void fflushConsoleOutput() { fflush( conOut->fp ); }

