/****************************************************************************************

StringLoader.c -- Copyright (c) by Jay Reynolds Freeman 1988, 1989, 1990, 1991, 2006

Code to load strings from Macintosh resources.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import "WraithHeader.h"

#import "Globals.e"
#import "IO.e"
#import "StringLoader.e"

Text MiscChars[ 50 ];

/**** Statics: ****/

/**** Some forward declarations: ****/

/**** loadAllocateDataTypePNameString ****/

const Text *loadAllocateDataTypePNameString( LongWord n, const Text **destinationStr )
{
    *destinationStr = DataTypeStrings[ n ];
    return *destinationStr;
    }

/**** loadNameString ****/

const Text *loadNameString( LongWord n ) { return NameStrings[ n ]; }

/**** initializeMiscChars ****/

void initializeMiscChars()
{
    MiscChars[0] = *(MiscStrings[1]);               /* cons dot */
    MiscChars[1] = *(MiscStrings[22]);              /* space */
    MiscChars[2] = *(MiscStrings[151]);             /* sharp */
    MiscChars[3] = *(MiscStrings[61]);              /* list-opener */
    MiscChars[4] = *(MiscStrings[62]);              /* list-closer */
    MiscChars[5] = *(MiscStrings[106]);             /* double-quote */
    MiscChars[6] = *(MiscStrings[112]);             /* backslash */
    MiscChars[7] = *(MiscStrings[115]);             /* newline */
    MiscChars[8] = *(MiscStrings[152]);             /* semicolon */
    MiscChars[9] = *(MiscStrings[154]);             /* backquote */
    MiscChars[10] = *(MiscStrings[155]);            /* unquote */ 
    MiscChars[11] = *(MiscStrings[156]);            /* unquote-splicing #1 */
    MiscChars[12] = *((MiscStrings[156]) + 1);      /* unquote-splicing #2 */
    MiscChars[13] = *(MiscStrings[153]);            /* quote */
    MiscChars[14] = *(MiscStrings[157]);            /* return */
    MiscChars[15] = *(MiscStrings[158]);            /* tab */
    MiscChars[16] = *(MiscStrings[159]);            /* t is for #t */
    MiscChars[17] = *(MiscStrings[160]);            /* f is for #f */
    MiscChars[18] = *(MiscStrings[1]);              /* non-printing character dot */
    MiscChars[19] = *(MiscStrings[151]);            /* sharp for non-significant digits */
    MiscChars[20] = *(MiscStrings[164]);            /* prefix indicating exponent */
    MiscChars[21] = *(MiscStrings[166]);            /* binary base */
    MiscChars[22] = *((MiscStrings[166]) + 1);      /* octal base */
    MiscChars[23] = *((MiscStrings[166]) + 2);      /* decimal base */
    MiscChars[24] = *((MiscStrings[166]) + 3);      /* hex base */
    MiscChars[25] = *((MiscStrings[167]) + 1);      /* plus sign */
    MiscChars[26] = *((MiscStrings[167]) + 2);      /* minus sign */
    MiscChars[27] = *(MiscStrings[162]);            /* radix point */
    MiscChars[28] = *((MiscStrings[163]) + 0);      /* exact */
    MiscChars[29] = *((MiscStrings[163]) + 1);      /* inexact */
    MiscChars[30] = *((MiscStrings[165]) + 0);      /* short */
    MiscChars[31] = *((MiscStrings[165]) + 1);      /* long */
    MiscChars[32] = *((MiscStrings[162]) + 1);      /* fraction slash */
    MiscChars[33] = *(MiscStrings[223]);            /* express */
    MiscChars[34] = *((MiscStrings[223]) + 1);      /* surpress */
    MiscChars[35] = *(MiscStrings[221]);            /* heuristic */
    MiscChars[36] = *(MiscStrings[251]);            /* option-plus */
    MiscChars[37] = *((MiscStrings[251]) + 1);      /* option-minus */
    MiscChars[38] = *((MiscStrings[251]) + 2);      /* option-star */
    MiscChars[39] = *(MiscStrings[323]);            /* prefix for indicating short-float exponent */
    MiscChars[40] = *(MiscStrings[329]);            /* '.', as in the only peculiar subsequent */
    MiscChars[41] = *(MiscStrings[167]);            /* '.', as in peculiar initial */
    MiscChars[42] = *((MiscStrings[167]) + 1);      /* '+', as in peculiar initial */
    MiscChars[43] = *((MiscStrings[167]) + 2);      /* '-', as in peculiar initial */
    MiscChars[44] = *((MiscStrings[388]));          /* u is for #u */
    MiscChars[45] = *((MiscStrings[389]));          /* s is for #s */
    MiscChars[46] = *(MiscStrings[373]);            /* Unix pathname separator, '/' */
    MiscChars[47] = *(MiscStrings[469]);            /* n is for #n */
    }

