/****************************************************************************************

TTYIO.c -- Copyright (c) by Jay Reynolds Freeman 2006.

Routines that talk directly to stdin and stdout.  (The "stdout" stuff is not here at
present -- 2006.11.12).

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import <stdio.h>
#import <sys/types.h>
#import <unistd.h>
#import <setjmp.h>
#import <string.h>
#import "LocalTypes.h"
#import "ASCIITypes.h"
#import "DataTypes.h"
#import "TTYIO.h"

#import "OpCodes.h"
#import "DataTypes.e"
#import "Events.e"
#import "Globals.e"
#import "PutBack.e"
#import "SchemeAux.e"
#import "StreamHandling.e"

extern void eventLoopOnce( LongWord sleepTimeInMicroseconds );
extern fileDescriptor *transcriptFD;
extern Boolean consoleOutputEnabledFlag;

Boolean nostalgiaMode = FALSE;

Text TTYMessageBuffer[ TTYMB_SIZE + 1 ];

/**** clearNostalgiaMode ****/

void clearNostalgiaMode()
{
    nostalgiaMode = FALSE;
    }

/**** setNostalgiaMode ****/

void setNostalgiaMode()
{
    nostalgiaMode = TRUE;
    }

/**** emitStatusMessage ****/

extern Text *MiscStrings[];

Boolean emitStatusMessage( const Text *msg )
{ 
    fprintf( stderr, "\n%s %s\n", MiscStrings[ 301 ], msg );
    return TRUE;
    }

/**** performDialogAndReturnString ****/

Boolean performDialogAndReturnString( const Text *prompt, Text *result, UnsignedWord resultLength )
{
    Boolean answer = FALSE;
    Word charsGotten = 0;

    result[ 0 ] = 0;
    fprintf( stdout, "\n%s%s\n", MiscStrings[ 301 ], prompt );
    if( (answer = (fgets( result, resultLength, stdin ) != NULL)) ) {
        if( (charsGotten = strlen( result )) > 0 )
            result[ charsGotten - 1 ] = 0;                  // Strip terminal newline.        
        }
    return answer;
    }

/**** initializeLowLevelIO ****/

void initializeLowLevelIO()
{
    initializePutBack();
    }

/**** reinitializeLowLevelIO -- nothing to do for the tty ****/

void reinitializeLowLevelIO( Text *reinitializationAction )
{
    reinitializePutBack();
    }


/**** cleanUpLowLevelIO -- nothing to do for the tty ****/

void cleanUpLowLevelIO() {;}

/**** dealWithResize -- nothing to do for the tty ****/

void dealWithResize() {;}

/**** switchToSystemIO -- nothing to do for the tty ****/

void switchToSystemIO() {}

/**** switchFromSystemIO -- nothing to do for the tty ****/

void switchFromSystemIO() {}

#define CI_BUFSIZE 1024

static Text CIBuffer[ CI_BUFSIZE + 1 ];
static Text *CIBp = CIBuffer;
static Word CIn = 0;

/**** tgetchar -- low-level console read routine ****/

extern jmp_buf ExitBuffer;

Word tgetchar()
{
    eventLoopOnce( 0 );
    if( hasPutBack() )
        return getPutBack();
    if( CIn == 0 ) {     // Empty buffer.
        fgets( CIBuffer, CI_BUFSIZE, stdin );
        if( (CIn = strlen( CIBuffer )) == 0 )
            longjmp( ExitBuffer, 2 );                
        CIBp = CIBuffer;
        if( (CIn > 0) && transcriptFD->fdOpen ) {
            fprintf( transcriptFD->fp, MiscStrings[ 201 ], CIBuffer );
            fflush( transcriptFD->fp );
            }
        }
    return( (--CIn >= 0) ? ((Word)(*CIBp++)) : EOF );
    }

/**** tcharreadyConsole -- Is there a character ready on the console? ****/

Boolean tcharreadyConsole()
{
    eventLoopOnce( 0 );
    return ( (CIn >0) || hasPutBack() );   
    }

/**** tfflushConsoleInput ****/

void tfflushConsoleInput()
{ 
    CIn = 0;
    CIBp = CIBuffer;
    clearPutBack();
    }

/**** fatalErrorWithText ****/

static Boolean recursiveCall = FALSE;

void fatalErrorWithText( Text *fatalErrorMessage )
{
    useFatalErrorOutput();
    tprintf3( &ErrorPort, MiscStrings[ 234 ], fatalErrorMessage );
    tprintf2( &ErrorPort, MiscStrings[ 115 ] );
    if( !recursiveCall ) {                  /* try to provide helpful information */
        recursiveCall = TRUE;
#if CODE_CHECK
        describeMemorySpaces();
        tprintf2( &ErrorPort, MiscStrings[ 115 ] );
#endif
        }
    if( strstr( (const char *)fatalErrorMessage, MiscStrings[ 339 ] ) )
        tprintf2( &ErrorPort, MiscStrings[ 338 ] );
    if( strstr( (const char *)fatalErrorMessage, MiscStrings[ 341 ] ) )
        tprintf2( &ErrorPort, MiscStrings[ 340 ] );
    resetCleanUp();
    longjmp( ExitBuffer, 2 );
    }

/**** fatalError ****/

void fatalError( UnsignedLongWord n )
{
    useFatalErrorOutput();
    if( descriptorsInitializedP ) {
        fflush( getPortFilePointer( &CurrentOutputPort ) );
        fflush( getPortFilePointer( &ErrorPort ) );
        }
    if( (n < 1) || (n > NumberOfFatalErrorStrings) ) {
        tprintf3( &ErrorPort, MiscStrings[ 93 ], n );
        exit( 1 );
        }               
    fatalErrorWithText( FatalErrorStrings[ n ] );
    }

/**** useFatalErrorOutput ****/

void useFatalErrorOutput()
{
    cleanUpLowLevelIO();
    CurrentOutputPort = ErrorPort;
    writeNewline( &CurrentOutputPort );
    }

Boolean tprintfX( taggedAval *f )
{
    FILE *fp = getPortFilePointer( f );

    if( (fp == stdout) || (fp == stderr) ) {
        if( transcriptFD->fdOpen ) {
            fputs( TTYMessageBuffer, transcriptFD->fp );
            fflush( transcriptFD->fp );
            }
        return fputs( TTYMessageBuffer, fp );
        }
    else
        return fputs( TTYMessageBuffer, fp );
    return TRUE;
    }

