/****************************************************************************************

WorldHandler.c++ -- Copyright (c) by Jay Reynolds Freeman 1987, 1988, 1989, 1990, 1991, 2006

Storage-allocation system for Wraith Scheme.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact information:

        EMail:       Jay_Reynolds_Freeman@mac.com
        Web Site:    http://web.mac.com/Jay_Reynolds_Freeman

        Paper Mail:  Jay Reynolds Freeman
                     Post Office Box 60628
                     Palo Alto, CA, 94306-0628

****************************************************************************************/

#import "LocalTypes.h"
#import "WraithHeader.h"
#import "OpCodes.h"
#import "WorldHandler.h"
#import "GarbageCollector.h"
#import "MemoryManager.h"
#import "Registers.h"
#import "Apply.e"
#import "DataTypes.e"
#import "Events.e"
#import "FunctionAndSpecialNames.e"
#import "Globals.e"
#import "Intern.e"
#import "IO.e"
#import "MainAux.e"
#import "Reader.e"
#import "Registers.e"
#import "SchemeAux.e"
#import "SchemeFunctions.e"
#import "SchemeFunctionExterns.e"
#import "SchemeGlobals.e"
#import "SchemeSpecials.e"
#import "SECDEval.e"
#import "SECDIO.e"
#import "SECDMachine.e"
#import "SECDRegisters.e"
#import "StreamHandling.e"

#define RUNNING_AS_BIG_ENDIAN (BigEndianFlag == TRUE)
#define WORLDS_ARE_LITTLE_ENDIAN TRUE

#define currentWorldType            ((sizeof( ShortFloat ) == 12) ? 11 : 12)
#define currentShortFloatSize        (sizeof( ShortFloat))

/**** Visible externally ****/

Text *theWorldString = 0;

/**** statics ****/

static pFVoid pFVoidTable[] = {
/*   0 */    CDropFunction,
/*   1 */    CPopRFunction,
/*   2 */    CPushFunction,
/*   3 */    ClearLFunction,
/*   4 */    GCDCode,
/*   5 */    LCMCode,
/*   6 */    MakeRTrueFunction,
/*   7 */    S0toC,
/*   8 */    SDropFunction,
/*   9 */    SPopEFunction,
/*  10 */    SPopRFunction,
/*  11 */    SPushFunction,
/*  12 */    SwapCons,
/*  13 */    acosMC,
/*  14 */    addCode,
/*  15 */    alwaysFalse,
/*  16 */    alwaysTrue,
/*  17 */    appendCode,
/*  18 */    apply,
/*  19 */    applyEnd,
/*  20 */    asinMC,
/*  21 */    assertEvaluating,
/*  22 */    assertFileLoadSucceeded,
/*  23 */    assertNotEvaluating,
/*  24 */    assign,
/*  25 */    assocCode,
/*  26 */    assqCode,
/*  27 */    assvCode,
/*  28 */    atanCode,
/*  29 */    beginEvalCode,
/*  30 */    bindOrAssign,
/*  31 */    bindScheme,
/*  32 */    bindToOptionMinus,
/*  33 */    bitAndCode,
/*  34 */    bitNotCode,
/*  35 */    bitOrCode,
/*  36 */    bitShiftLeftCode,
/*  37 */    bitShiftRightArithmeticCode,
/*  38 */    bitShiftRightLogicalCode,
/*  39 */    bitXorCode,
/*  40 */    booleanP,
/*  41 */    booleanPCode,
/*  42 */    boundInstanceInEnvironmentPCode,
/*  43 */    boundInstancePCode,
/*  44 */    builtInFunctionP,
/*  45 */    byteBlockP,
/*  46 */    caaaarCode,
/*  47 */    caaadrCode,
/*  48 */    caaarCode,
/*  49 */    caadarCode,
/*  50 */    caaddrCode,
/*  51 */    caadrCode,
/*  52 */    caarCode,
/*  53 */    cadaarCode,
/*  54 */    cadadrCode,
/*  55 */    cadarCode,
/*  56 */    caddarCode,
/*  57 */    cadddrCode,
/*  58 */    caddrCode,
/*  59 */    cadrCode,
/*  60 */    callWithInputPortCode,
/*  61 */    callWithOutputPortCode,
/*  62 */    car,
/*  63 */    carEqP,
/*  64 */    carEqualP,
/*  65 */    carEqvP,
/*  66 */    cdaaarCode,
/*  67 */    cdaadrCode,
/*  68 */    cdaarCode,
/*  69 */    cdadarCode,
/*  70 */    cdaddrCode,
/*  71 */    cdadrCode,
/*  72 */    cdarCode,
/*  73 */    cddaarCode,
/*  74 */    cddadrCode,
/*  75 */    cddarCode,
/*  76 */    cdddarCode,
/*  77 */    cddddrCode,
/*  78 */    cdddrCode,
/*  79 */    cddrCode,
/*  80 */    cdr,
/*  81 */    ceilingCode,
/*  82 */    charAlphaP,
/*  83 */    charAlphaPCode,
/*  84 */    charCIEqualPCode,
/*  85 */    charCIGreaterEqualPCode,
/*  86 */    charCIGreaterPCode,
/*  87 */    charCILessEqualPCode,
/*  88 */    charCILessPCode,
/*  89 */    charDowncaseCode,
/*  90 */    charEqualPCode,
/*  91 */    charGreaterEqualPCode,
/*  92 */    charGreaterPCode,
/*  93 */    charLessEqualPCode,
/*  94 */    charLessPCode,
/*  95 */    charLowerP,
/*  96 */    charLowerPCode,
/*  97 */    charNumberPCode,
/*  98 */    charNumericP,
/*  99 */    charPCode,
/*  00 */    charReadyPCode,
/* 101 */    charToIntegerCode,
/* 102 */    charUpCaseCode,
/* 103 */    charUpperP,
/* 104 */    charUpperPCode,
/* 105 */    charWhitespaceP,
/* 106 */    charWhitespacePCode,
/* 107 */    characterP,
/* 108 */    clearPermanentCode,
/* 109 */    closePort,
/* 110 */    closeStackedPort,
/* 111 */    closeStackedPortRestoreInput,
/* 112 */    closeStackedPortRestoreOutput,
/* 113 */    closedPortPCode,
/* 114 */    compileCode,
/* 115 */    compiledBlockEvalCode,
/* 116 */    compilerOffCode,
/* 117 */    compilerOnCode,
/* 118 */    compilingP,
/* 119 */    compilingPCode,
/* 120 */    complexP,
/* 121 */    complexPCode,
/* 122 */    cons,
/* 123 */    cosMC,
/* 124 */    currentInputPortCode,
/* 125 */    currentOutputPortCode,
/* 126 */    cwCCode,
/* 127 */    cwccCode,
/* 128 */    dDELCode,
/* 129 */    dECode,
/* 130 */    dELCode,
/* 131 */    defineEvalCode,
/* 132 */    dereferenceEnvironmentReferenceMC,
/* 133 */    dereferenceStackReferenceMC,
/* 134 */    dereferenceTopLevelEnvironmentReferenceForSymbolMC,
/* 135 */    dereferenceTopLevelEnvironmentReferenceMC,
/* 136 */    describeMemoryCode,
/* 137 */    describeRoomCode,
/* 138 */    describeStackCode,
/* 139 */    displayCode,
/* 140 */    displayObject,
/* 141 */    divideCode,
/* 142 */    downInFlamesCode,
/* 143 */    dropNFromStack,
/* 144 */    eIECode,
/* 145 */    environmentCode,
/* 146 */    environmentNDownCode,
/* 147 */    environmentRefCode,
/* 148 */    environmentReferenceP,
/* 149 */    eofPCode,
/* 150 */    eqPCode,
/* 151 */    equalPCode,
/* 152 */    eqvPCode,
/* 153 */    errorPortCode,
/* 154 */    evenPCode,
/* 155 */    exactIntegerP,
/* 156 */    exactShortFloatP,
/* 157 */    exactP,
/* 158 */    exactPCode,
/* 159 */    exactShortFixnumP,
/* 160 */    nonNegativeIntegerP,
/* 161 */    exactToInexactCode,
/* 162 */    examineWorldCode,
/* 163 */    expMC,
/* 164 */    expandMacroAux,
/* 165 */    expandMacroCode,
/* 166 */    exptCode,
/* 167 */    falseP,
/* 168 */    fileExistsPCode,
/* 169 */    fixnumP,
/* 170 */    fixnumPCode,
/* 171 */    floatP,
/* 172 */    floatPCode,
/* 173 */    floorCode,
/* 174 */    forEachCode,
/* 175 */    forceCode,
/* 176 */    forcedPCode,
/* 177 */    forwardingPointerP,
/* 178 */    fullGCType2Code,
/* 179 */    fullPrecisionOffCode,
/* 180 */    fullPrecisionOnCode,
/* 181 */    fullPrecisionPCode,
/* 182 */    genSym,
/* 183 */    getInitFilePathCode,
/* 184 */    getPName,
/* 185 */    getStartupFolderCode,
/* 186 */    getTagCode,
/* 187 */    getWorkingFolderCode,
/* 188 */    hashString,
/* 189 */    hashStringCode,
/* 190 */    if3EvalCode,
/* 191 */    ifAuxCode,
/* 192 */    forgettableP,
/* 193 */    inexactShortFloatP,
/* 194 */    inexactP,
/* 195 */    inexactPCode,
/* 196 */    inexactShortFixnumP,
/* 197 */    makeForgettableCode,
/* 198 */    inexactToExactCode,
/* 199 */    infP,
/* 100 */    infPCode,
/* 201 */    inputPortP,
/* 202 */    inputPortPCode,
/* 203 */    inspectCode,
/* 204 */    integerP,
/* 205 */    integerPCode,
/* 206 */    integerToCharCode,
/* 207 */    lambdaEvalAux,
/* 208 */    lambdaEvalCode,
/* 209 */    lambdaPairP,
/* 210 */    lastPair,
/* 211 */    lastPairCode,
/* 212 */    lengthOp,
/* 213 */    letterP,
/* 214 */    listP,
/* 215 */    listRefCode,
/* 216 */    listTail,
/* 217 */    listTailCode,
/* 218 */    listToStringCode,
/* 219 */    listToVectorCode,
/* 220 */    listTopN,
/* 221 */    loadCode,
/* 222 */    loadWorldCode,
/* 223 */    logMC,
/* 224 */    shortFloatPCode,
/* 225 */    longP,
/* 226 */    lookUp,
/* 227 */    lookUpInEnvironment,
/* 228 */    macroBodyAux,
/* 229 */    macroBodyCode,
/* 230 */    macroEvalAux,
/* 231 */    macroEvalCode,
/* 232 */    macroP,
/* 233 */    macroPCode,
/* 234 */    macroPartTwo,
/* 235 */    makeStringCode,
/* 236 */    makeVectorAndStuffFromStack,
/* 237 */    makeVectorCode,
/* 238 */    mapCode,
/* 239 */    memberCode,
/* 240 */    memqCode,
/* 241 */    memvCode,
/* 242 */    moduloCode,
/* 243 */    multipleValuesP,
/* 244 */    multipleValuesPCode,
/* 245 */    multiplyCode,
/* 246 */    nanOrInfP,
/* 247 */    nanP,
/* 248 */    nanPCode,
/* 249 */    newlineCode,
/* 250 */    noop,
/* 251 */    notNullPCode,
/* 252 */    notPCode,
/* 253 */    nullEnvironmentCode,
/* 254 */    nullPCode,
/* 255 */    numAbsCode,
/* 256 */    numEqualCode,
/* 257 */    numGreaterCode,
/* 258 */    numGreaterEqualCode,
/* 259 */    numLessCode,
/* 260 */    numLessEqualCode,
/* 261 */    numMaxCode,
/* 262 */    numMinCode,
/* 263 */    numNegativePCode,
/* 264 */    numPositivePCode,
/* 265 */    numberP,
/* 266 */    numberPCode,
/* 267 */    numberToStringCode,
/* 268 */    numberToStringMC,
/* 269 */    numberToStringViaSPrintfCode,
/* 270 */    oddPCode,
/* 271 */    oneForEach,
/* 272 */    oneMap,
/* 273 */    oneMapFirst,
/* 274 */    openInputFileCode,
/* 275 */    openOutputFileCode,
/* 276 */    outputPortP,
/* 277 */    outputPortPCode,
/* 278 */    pairP,
/* 279 */    pairPCode,
/* 280 */    peculiarLeafP,
/* 281 */    peculiarNonLeafP,
/* 282 */    peekCharCode,
/* 283 */    peekLoop,
/* 284 */    permanentPCode,
/* 285 */    pointerToByteBlockP,
/* 286 */    popToEFunction,
/* 287 */    printDepthListGet,
/* 288 */    printDepthListSet,
/* 289 */    printDepthVectorGet,
/* 290 */    printDepthVectorSet,
/* 291 */    printLengthListGet,
/* 292 */    printLengthListSet,
/* 293 */    printLengthVectorGet,
/* 294 */    printLengthVectorSet,
/* 295 */    procedureP,
/* 296 */    procedurePCode,
/* 297 */    promiseP,
/* 298 */    promisePCode,
/* 299 */    properListCheck,
/* 200 */    properListP,
/* 301 */    properListPCode,
/* 302 */    quitCode,
/* 303 */    quoteEvalCode,
/* 304 */    quoteSimpleEvalCode,
/* 305 */    quotientCode,
/* 306 */    radixP,
/* 307 */    randomCode,
/* 308 */    rangeCheckAsSignedFixnumP,
/* 309 */    rangeCheckAsUnsignedFixnumP,
/* 310 */    rationalP,
/* 311 */    rationalPCode,
/* 312 */    readCharCode,
/* 313 */    readCode,
/* 314 */    readSexp,
/* 315 */    readSexpFromPort,
/* 316 */    inputQueueCode,
/* 317 */    realP,
/* 318 */    realPCode,
/* 319 */    rebindForce,
/* 320 */    rebindTopLoopVariables,
/* 321 */    remainderCode,
/* 322 */    resetCode,
/* 323 */    resetScheme,
/* 324 */    restoreInputPort,
/* 325 */    restoreStack,
/* 326 */    restoreStackAndFrame,
/* 327 */    returnRoomOp,
/* 328 */    returnStoreSizeOp,
/* 329 */    reverseListCode,
/* 330 */    roundCode,
/* 331 */    sRandomCode,
/* 332 */    saveStack,
/* 333 */    saveWorldCode,
/* 334 */    schemeMachinePCP,
/* 335 */    schemeReportEnvironmentCode,
/* 336 */    setBangEvalCode,
/* 337 */    setCarCode,
/* 338 */    setCdrCode,
/* 339 */    setInputPortCode,
/* 340 */    setPermanentCode,
/* 341 */    setTagCode,
/* 342 */    setUpApply,
/* 343 */    setWorkingFolderCode,
/* 344 */    sharpFOrProperListP,
/* 345 */    shortFixnumP,
/* 346 */    shortFloatPCode,
/* 347 */    sinMC,
/* 348 */    specialFormP,
/* 349 */    sqrtMC,
/* 350 */    stackDepth,
/* 351 */    stackReferenceP,
/* 352 */    stringAppendCode,
/* 353 */    stringCIEqualPCode,
/* 354 */    stringCIGreaterEqualPCode,
/* 355 */    stringCIGreaterPCode,
/* 356 */    stringCILessEqualPCode,
/* 357 */    stringCILessPCode,
/* 358 */    stringCopy,
/* 359 */    stringEqualPCode,
/* 360 */    stringFillCode,
/* 361 */    stringGreaterEqualPCode,
/* 362 */    stringGreaterPCode,
/* 363 */    stringLength,
/* 364 */    stringLessEqualPCode,
/* 365 */    stringLessPCode,
/* 366 */    stringP,
/* 367 */    stringPCode,
/* 368 */    stringRefCode,
/* 369 */    stringSetCode,
/* 370 */    stringSymbolCode,
/* 371 */    stringToListCode,
/* 372 */    stringToNumberCode,
/* 373 */    substringCode,
/* 374 */    subtractCode,
/* 375 */    symbolP,
/* 376 */    symbolPCode,
/* 377 */    syntaxBindingP,
/* 378 */    syntaxBindingPCode,
/* 379 */    systemCode,
/* 380 */    tanMC,
/* 381 */    tfflushConsoleOutput,
/* 382 */    tickCode,
/* 383 */    topLevelEnvironmentCode,
/* 384 */    topLevelEnvironmentRefCode,
/* 385 */    topLevelEnvironmentRefForSymbolCode,
/* 386 */    topLevelEnvironmentReferenceP,
/* 387 */    topLoopCode,
/* 388 */    transcriptOffCode,
/* 389 */    transcriptOnCode,
/* 390 */    trueP,
/* 391 */    truncateCode,
/* 392 */    uSleepCode,
/* 393 */    vRefCode,
/* 394 */    vSetCode,
/* 395 */    vectorFillCode,
/* 396 */    vectorLength,
/* 397 */    vectorP,
/* 398 */    vectorPCode,
/* 399 */    vectorToListCode,
/* 300 */    versionGetCode,
/* 401 */    withInputFileCode,
/* 402 */    withOutputFileCode,
/* 403 */    writeCharCode,
/* 404 */    writeCode,
/* 405 */    writeContinuationCode,
/* 406 */    writeObject,
/* 407 */    writeStackCode,
/* 408 */    zeroPCode,
/* 409 */    momcatPCode,
/* 410 */    kittenNumberCode,
/* 411 */    maxKittenNumberCode,
/* 412 */    showKittensCode,
/* 413 */    blockCode,
/* 414 */    releaseCode,
/* 415 */    blockSymbolBindingCode,
/* 416 */    readSexpOrQueueTopLevel,
/* 417 */    setBlockedBindingCode,
/* 418 */    getPIDCode,
/* 419 */    nonLeafP,
/* 420 */    describeLocksCode,
/* 421 */    releaseLocksCode,
/* 422 */    letDeadlockTimeOutCode,
/* 423 */    kittenDequeueCode,
/* 424 */    kittenEmptyQueueCode,
/* 425 */    purgeKittenQueueCode,
/* 426 */    tellKittenCode,
/* 427 */    kittenRangeCheckP,
/* 428 */    socketCreateAndListen,
/* 429 */    socketAccept,
/* 430 */    socketWrite,
/* 431 */    openInputSocketCode,
/* 432 */    socketFGets,
/* 433 */    return90PercentFlagOp,
/* 434 */    restoreContinuation,
/* 435 */    restoreContinuationAndFrame,
/* 436 */    saveContinuation,
/* 437 */    restoreForCWCC,
/* 438 */    saveForCWCC,
/* 439 */    disableConsoleOutputCode,
/* 440 */    enableConsoleOutputCode,
/* 441 */    consoleOutputEnabledPCode,
/* 442 */    kittenResetCode,
/* 443 */    kittenBusyCode,
/* 444 */    probabilityP,
/* 445 */    forgettableUnexpiredP,
/* 446 */    forgettableRememberedP,
/* 447 */    forgettableForgottenP,
/* 448 */    forgettablePCode,
/* 449 */    forgettableUnexpiredPCode,
/* 450 */    forgettableRememberedPCode,
/* 451 */    forgettableForgottenPCode,
/* 452 */    forgettableObjectCode,
/* 453 */    forgettableTimeCode,
/* 454 */    forgettableRememberProbabilityCode,
/* 455 */    setForgettableTimeCode,
/* 456 */    setForgettableRememberProbabilityCode,
/* 457 */    leafPCode,
/* 458 */    mmappedBlockP,
/* 459 */    makeMMappedBlockCode,
/* 460 */    mmapBlockInCurrentKittenCode,
/* 461 */    getMMappableSpaceCode,
/* 462 */    peek8Code,
/* 463 */    poke8Code,
/* 464 */    peek16Code,
/* 465 */    poke16Code,
/* 466 */    peek32Code,
/* 467 */    poke32Code,
/* 468 */    peek64Code,
/* 469 */    poke64Code,
/* 470 */    memoryMappedBlockPCode,
/* 471 */    interruptVectorRegisterCode,
/* 472 */    interruptRangeCheckP,
/* 473 */    setInterruptCode,
/* 474 */    clearInterruptCode,
/* 475 */    interruptCode,
/* 476 */    getInterruptFileInformationCode,
/* 477 */    peek8SignExtendCode,
/* 478 */    peek16SignExtendCode,
/* 479 */    peek32SignExtendCode,
/* 480 */    peek8VectorCode,
/* 481 */    peek16VectorCode,
/* 482 */    peek32VectorCode,
/* 483 */    peek64VectorCode,
/* 484 */    peek8VectorSignExtendCode,
/* 485 */    peek16VectorSignExtendCode,
/* 486 */    peek32VectorSignExtendCode,
/* 487 */    peekFlonumCode,
/* 488 */    pokeFlonumCode,
/* 489 */    poke8VectorCode,
/* 490 */    poke16VectorCode,
/* 491 */    poke32VectorCode,
/* 492 */    poke64VectorCode,
/* 493 */    peekFlonumVectorCode,
/* 494 */    pokeFlonumVectorCode,
/* 495 */    peekCStringCode,
/* 496 */    pokeCStringCode,
/* 497 */    loadFromStringCode,
/* 498 */    makeStringWithPromptCode,
/* 499 */    positiveIntegerP,
/* 500 */    peekPokeAtomicCode,
/* 501 */    blockAnyBindingCode,
/* 502 */    cBooleanToSchemeBooleanCode,
/* 503 */    schemeBooleanToCBooleanCode,
/* 504 */    ggcForwardingPointerP,
/* 505 */    returnActiveRoomOp,
/* 506 */    returnAgingRoomOp,
/* 507 */    returnActiveStoreSizeOp,
/* 508 */    describeActiveMemoryCode,
/* 509 */    describeAgingMemoryCode,
/* 510 */    setAlarmCode,
/* 511 */    alarmPCode,
/* 512 */    alarmTimeCode,
/* 513 */    clearInterruptRequestedCode,
/* 514 */    newlineIfPrinting,
/* 515 */    nonPrintingObjectPCode,
/* 516 */    senseLightPCode,
/* 517 */    senseLightIlluminationPCode,
/* 518 */    setSenseLightNIlluminationCode,
/* 519 */    setSenseLightNVisibilityCode,
/* 520 */    showSenseLightsCode,
/* 521 */    hideSenseLightsCode,
/* 522 */    setSenseLightsOffCode,
/* 523 */    senseLightP,
/* 524 */    senseLightIlluminationP,
/* 525 */    setSenseLightIlluminationsCode,
/* 526 */    rotateSenseLightsCode,
/* 527 */    applicativeModeCheckSetBangCode,
/* 528 */    applicativeModeCheckCode,
/* 529 */    applicativeBitPCode,
/* 530 */    setApplicativeBitCode,
/* 531 */    setApplicativeBitNDownCode,
/* 532 */    clearApplicativeBitCode,
/* 533 */    cSetBangEvalCode,
/* 534 */    cSetTagCode,
/* 535 */    clearApplicativeBitNDownCode,
/* 536 */    senseSwitchesCode,
/* 537 */    sliderNumberRangeCheckP,
/* 538 */    sliderCode,
/* 539 */    pushbuttonNumberRangeCheckP,
/* 540 */    enablePushbuttonCode,
/* 541 */    pushbuttonInterruptNumberCode,
/* 542 */    disablePushbuttonCode,
/* 543 */    enableBigRedButtonCode,
/* 544 */    bigRedButtonInterruptNumberCode,
/* 545 */    disableBigRedButtonCode,
/* 546 */    pushbuttonEnabledCode,
/* 547 */    bigRedButtonEnabledCode,
/* 548 */    levelIndicatorNumberRangeCheckP,
/* 549 */    levelIndicatorValueRangeCheckP,
/* 550 */    setLevelIndicatorCode,
    };

size_t pFVoidTableSize = sizeof( pFVoidTable ) / sizeof( pFVoid );

static cursorAndCode *theTableStart = 0;
static cursorAndCode *theTableEnd = 0;

/**** compareCursorAndCodes ****/

int compareCursorAndCodes( const void *first, const void *second )
{
    if( (((cursorAndCode *)first)->codePointer) < (((cursorAndCode *)second)->codePointer) )
        return -1;
    else if( (((cursorAndCode *)first)->codePointer) > (((cursorAndCode *)second)->codePointer) )
        return 1;
    else
        return 0;
    }

void WorldHandler::initializeWH( Boolean bEFlag, Text *externalNameString, size_t externalNameStringSize )
{   
    lastWorldLoadedName = externalNameString;
    lastWorldLoadedNameSize = externalNameStringSize;
    UnsignedLongWord counter = 0;

    for( counter = 0; counter < pFVoidTableSize; ++counter )
        codePointerTable[ counter ] = (UnsignedLongWord)(pFVoidTable[ counter ]);

    theUnsortedTableSize = pFVoidTableSize;
    memset( &theUserSetup, 0, sizeof( userSetup ) );
    theString = 0;
    BigEndianFlag = bEFlag;

    theTableStart = &(theUnsortedTable[0]);
    theTableEnd   = theUnsortedTable + theUnsortedTableSize;

    createUnsortedTable();

    memcpy( &(theSortedTable[0]), &(theUnsortedTable[0]), ((size_t)theUnsortedTableSize) * sizeof( cursorAndCode ) );

//    for( UnsignedLongWord counter = 0; counter < theUnsortedTableSize; ++counter ) {
//        (theSortedTable[ counter ]).tableCursor = (theUnsortedTable[ counter ]).tableCursor;
//        (theSortedTable[ counter ]).codePointer = (theUnsortedTable[ counter ]).codePointer;
//        }

    qsort( &(theSortedTable[0]), (size_t)theUnsortedTableSize, sizeof( cursorAndCode ), compareCursorAndCodes );
    theSimpleTable = codePointerTable;
    }

/**** createUnsortedTable -- Create an unsorted table of non-heap things that we need to keep track of.       ****/

void WorldHandler::createUnsortedTable()
{
    UnsignedLongWord tabCursor = 0;
        
    /* fill table */
        
    for( tabCursor = 0; tabCursor < theUnsortedTableSize; ++tabCursor ) {
        (theUnsortedTable[ tabCursor ]).tableCursor = tabCursor;
        (theUnsortedTable[ tabCursor ]).codePointer = codePointerTable[ tabCursor ];
        }
    }

/**** pcForIndex ****/

UnsignedLongWord WorldHandler::pcForIndex( UnsignedLongWord anIndex )
{
    if( anIndex < theUnsortedTableSize )
        return theSimpleTable[ anIndex ];
    else
        fatalError( 136 );
    return 0;
    }

/**** indexForPC ****/

UnsignedLongWord WorldHandler::indexForPC( UnsignedLongWord aPC )
{
    LongWord Low = 0;
    LongWord U = (LongWord)(theUnsortedTableSize - 1);
    LongWord M = 0;

    while( TRUE ) {
        if( Low > U ) {
            useFatalErrorOutput();
            tprintf3( &CurrentOutputPort, MiscStrings[ 346 ], aPC );
            fatalError( 137 );
            }
        M = (Low + U) / 2;
        if( (theSortedTable[ M ]).codePointer < aPC )
            Low = M + 1;
        else if( (theSortedTable[ M ]).codePointer > aPC )
            U = M - 1;
        else {                  /* found it! */
            return (theSortedTable[ M ]).tableCursor;
            }
        }
    }

/**** describeTableHeader ****/

void WorldHandler::describeTableHeader( struct tableHeader *th )
{
    tprintf2( &CurrentOutputPort, "Table Header:\n");
    tprintf3( &CurrentOutputPort, "    Version:                   %s\n",  th->versionString );
    tprintf3( &CurrentOutputPort, "    topECursor:                %jd\n", (intmax_t)(th->topECursor) );
    tprintf3( &CurrentOutputPort, "    heapSizeWanted:            %jd\n", (intmax_t)(th->heapSizeWanted) );
    tprintf3( &CurrentOutputPort, "    nKittensExpected:          %jd\n", (intmax_t)(th->nKittensExpected) );
    tprintf3( &CurrentOutputPort, "    tableSizeExpected:         %jd\n", (intmax_t)(th->tableSizeExpected) );
    tprintf3( &CurrentOutputPort, "    prDepth:                   %jd\n", (intmax_t)(th->prDepth) );  
    tprintf3( &CurrentOutputPort, "    prLength:                  %jd\n", (intmax_t)(th->prLength) );  
    tprintf3( &CurrentOutputPort, "    vPrDepth:                  %jd\n", (intmax_t)(th->vPrDepth) );
    tprintf3( &CurrentOutputPort, "    vPrLength:                 %jd\n", (intmax_t)(th->vPrLength) );
    tprintf3( &CurrentOutputPort, "    worldType:                 %jd\n", (intmax_t)(th->worldType) );
    tprintf3( &CurrentOutputPort, "    shortFloatSize:            %jd\n", (intmax_t)(th->shortFloatSize) );
    tprintf3( &CurrentOutputPort, "    ObListCursor:              %jd\n", (intmax_t)(th->ObListCursor) );
    tprintf3( &CurrentOutputPort, "    TLCCursor:                 %jd\n", (intmax_t)(th->TLCCursor) );
    tprintf3( &CurrentOutputPort, "    extra1:                    %jd\n", (intmax_t)(th->extra1) );
    tprintf3( &CurrentOutputPort, "    extra2:                    %jd\n", (intmax_t)(th->extra2) );
    tprintf3( &CurrentOutputPort, "    extra3:                    %jd\n", (intmax_t)(th->extra3) );
    tprintf3( &CurrentOutputPort, "    stringLength:              %jd\n", (intmax_t)(th->stringLength) );
    tprintf2( &CurrentOutputPort, "\n");
    // Add code here to describe savedUserSetup;  // FIXME
    }

/**** describeTable -- describe a pre-existing table. ****/

void WorldHandler::describeTable( UnsignedLongWord *tab )
{
    UnsignedLongWord counter = 0;
    UnsignedLongWord tabSize = sizeof( pFVoidTable ) / sizeof( pFVoid );

    for( counter = 0; counter < tabSize; ++counter ) 
        tprintf4( &CurrentOutputPort, "%jd: 0x%016jx\n", (intmax_t)counter, (intmax_t)(tab[counter]) );

    }

/**** badPointerAfterLoad -- Verify that any non-scheme-machine-PC non-leaves point into the heap. ****/

Boolean WorldHandler::badPointerAfterLoad( volatile memorySpace *where, taggedAval *tp )
{
    Tag tg = pure_type_code( tp->t );
    Byte *ptr = (Byte *)((tp->u).a);

    if( ! leaf_p( tg ) ) {
        if( (ptr < where->space) || (ptr >= where->end) )
            return true;        
        }
    return false;
    }

/**** fixPointerForLoad -- Change the pointer of a taggedAval appropriately for performing a world load. ****/

void WorldHandler::fixPointerForLoad( volatile memorySpace *where, taggedAval *tp )
{
    Tag tg = pure_type_code( tp->t );

    if( ! leaf_p( tg ) )
        (tp->u).c = (LongWord)(where->end - (UnsignedLongWord)((tp->u).c));
    }

/**** fixPointerForSave -- Change the pointer of a taggedAval appropriately for performing a world save. ****/

void WorldHandler::fixPointerForSave( volatile memorySpace *where, taggedAval *tp )
{
    Tag tg = pure_type_code( tp->t );

    if( ! leaf_p( tg ) )
        (tp->u).c = (LongWord)(where->end - (Byte *)((tp->u).c));
    }

/**** notFlippedYet -- uses the GC bits in the alternate space ****/

Boolean notFlippedYet( taggedAval *aTag, volatile memorySpace *real, volatile memorySpace *scratch )
{
    Byte *aByte = (Byte *)aTag - ((real->space) - (scratch->space));
    taggedAval *alternateTag = (taggedAval *)aByte;

    return( gc_bit( alternateTag->t ) == 0 );
    }

/**** markFlipped -- uses the GC bits in the alternate space ****/

void markFlipped( taggedAval *aTag, volatile memorySpace *real, volatile memorySpace *scratch )
{
    Byte *aByte = (Byte *)aTag - ((real->space) - (scratch->space));
    taggedAval *alternateTag = (taggedAval *)aByte;

    setGCBitNonAtomic( alternateTag->t );
    }

/**** loadWorldGuts -- "Innards" of simple means to load all Pixie state from a file. ****/

void WorldHandler::loadWorldGuts( fileDescriptor *wrldFile, volatile memorySpace *where, volatile memorySpace *alternate, Boolean describeP )
{
    struct tableHeader tblHdr;
    UnsignedLongWord tableSize;
    UnsignedLongWord currentLength = 0;

    volatile Byte *heapPointer;
    taggedAval *cTA = NULL;

    taggedAval scratchIVR, scratchObList, scratchTopLevelE, scratchTLC;

    Boolean badPointerP = false;

    if( wrldFile->fp == NULL )
        resetError( 116 );
    if( fread( (void *)&tblHdr, sizeof( tblHdr ), 1, wrldFile->fp ) != 1 ) {
        fclose( wrldFile->fp );
        reinitializeFileDescriptor( wrldFile );
        resetError( 117 );
        }

    // Just read a table.

    if( tblHdr.heapSizeWanted > where->allocateSize ) {
        fclose( wrldFile->fp );
        reinitializeFileDescriptor( wrldFile );
        resetError( 118 );
        }
    tableSize = sizeof( pFVoidTable ) / sizeof( pFVoid );
    if( tblHdr.tableSizeExpected != tableSize ) {
        fclose( wrldFile->fp );
        reinitializeFileDescriptor( wrldFile );
        resetError( 119 );
        }
    if( tblHdr.worldType != currentWorldType ) {
        fclose( wrldFile->fp );
        reinitializeFileDescriptor( wrldFile );
        resetError( 322 );
        }
    if( tblHdr.shortFloatSize != currentShortFloatSize ) {
        fclose( wrldFile->fp );
        reinitializeFileDescriptor( wrldFile );
        resetError( 323 );
        }
    scratchIVR.t = IVR.t;
    scratchIVR.u.c = (LongWord)(where->end - (UnsignedLongWord)tblHdr.IVRCursor);
    if( badPointerAfterLoad( where, &scratchIVR ) )
        fatalError( 61 );
    scratchTopLevelE.t = TopLevelE.t;
    scratchTopLevelE.u.c = (LongWord)(where->end - (UnsignedLongWord)tblHdr.topECursor);
    if( badPointerAfterLoad( where, &scratchTopLevelE ) )
        fatalError( 47 );
    scratchObList.t = ObList.t;
    scratchObList.u.c = (LongWord)(where->end - (UnsignedLongWord)tblHdr.ObListCursor);
    if( badPointerAfterLoad( where, &scratchObList ) )
        fatalError( 48 );
    scratchTLC.t = TLC.t;
    scratchTLC.u.c = (LongWord)(where->end - (UnsignedLongWord)tblHdr.TLCCursor);
    if( badPointerAfterLoad( where, &scratchTLC ) )
        fatalError( 142 );
    if( ! describeP ) {  // Don't change state if we are just examining a world.
        closeAllPorts();
        errorRecovery( 0 );
        IVR.u.a = scratchIVR.u.a;
        TopLevelE.u.a = scratchTopLevelE.u.a;
        E.t = TopLevelE.t;
        E.u.a = TopLevelE.u.a;
        ObList.u.a = scratchObList.u.a;
        TLC.u.a = scratchTLC.u.a;
        printDepth   = tblHdr.prDepth; 
        printLength  = tblHdr.prLength; 
        vPrintDepth  = tblHdr.vPrDepth;
        vPrintLength = tblHdr.vPrLength;
        // Add code here to use extra1.
        // Add code here to use extra2.
        // Add code here to use extra3.

        theUserSetup = tblHdr.savedUserSetup;

        // Add code here to use theUserSetup;  // FIXME
        // Add code here to use the string.
        }
    
    where->free = where->end - tblHdr.heapSizeWanted;

    if( fread( (void *)(where->free), sizeof( Byte ), (size_t)(tblHdr.heapSizeWanted), wrldFile->fp ) != tblHdr.heapSizeWanted ) {
        fclose( wrldFile->fp );
        reinitializeFileDescriptor( wrldFile );
        fatalError( 33 );
        }

    Text *stringBuffer = (Text *)calloc( (size_t)(tblHdr.stringLength) + 1, sizeof( Text ) );

    if( stringBuffer == 0 )
        fatalError( 193 );
    if( tblHdr.stringLength > 0 ) {
        if(    fread( (void *)stringBuffer, sizeof( Byte ), (size_t)(tblHdr.stringLength) + 1, wrldFile->fp )
            != tblHdr.stringLength + 1 ) {
            fclose( wrldFile->fp );
            reinitializeFileDescriptor( wrldFile );
            fatalError( 33 );
            }
        }
    else
        stringBuffer[ 0 ] = 0;        

    theString = stringBuffer;

    strncpy( lastWorldLoadedName, wrldFile->pathName, lastWorldLoadedNameSize );
    fclose( wrldFile->fp );
    reinitializeFileDescriptor( wrldFile );

    // Read a world into main memory.

    for( heapPointer = where->free; heapPointer < where->end; heapPointer += currentLength ) {
        cTA = (taggedAval *)heapPointer;
        fixPointerForLoad( where, cTA );
        if( badPointerAfterLoad( where, cTA ) )
            badPointerP = true;
        currentLength = lengthTaggedAvalPointer( cTA );
        }

    for( heapPointer = where->free; heapPointer < where->end; heapPointer += currentLength ) {
        cTA = (taggedAval *)heapPointer;
        if( badPointerAfterLoad( where, cTA ) )
            badPointerP = true;
        currentLength = lengthTaggedAvalPointer( cTA );
        }
    if( describeP ) {
        describeTableHeader( &tblHdr );
        describeTable( codePointerTable );
        describeMemorySpace( where, true );
        }
    if( badPointerP )
        fatalError( 46 );

    theClowder->setObListCount( countObList() );
    aWorldHasJustBeenLoaded = TRUE;
    reportEvent( WORLD_LOAD_SUCCEEDED_EVENT );
    }

/**** worldIsUseable -- just check and see if we can use it ****/

Boolean WorldHandler::worldIsUseable( Text *pathToWorld, volatile memorySpace *where )
{
    struct tableHeader tblHdr;
    UnsignedLongWord tableSize;

    worldFD->fp = fopen( pathToWorld, "r" );
    strcpy( worldFD->oldPathName, worldFD->pathName );
    strcpy( worldFD->pathName, pathToWorld );
    strcpy( worldFD->type, "r" );

    if( worldFD->fp == NULL )
        return FALSE;
    if( fread( (void *)&tblHdr, sizeof( tblHdr ), 1, worldFD->fp ) != 1 ) {
        fclose( worldFD->fp );
        reinitializeFileDescriptor( worldFD );
        return FALSE;
        }

    // Just read a table.

    if( tblHdr.heapSizeWanted > where->allocateSize ) {
        fclose( worldFD->fp );
        reinitializeFileDescriptor( worldFD );
        return FALSE;
        }
    tableSize = sizeof( pFVoidTable ) / sizeof( pFVoid );
    if( tblHdr.tableSizeExpected != tableSize ) {
        fclose( worldFD->fp );
        reinitializeFileDescriptor( worldFD );
        return FALSE;
        }
    if( tblHdr.worldType != currentWorldType ) {
        fclose( worldFD->fp );
        reinitializeFileDescriptor( worldFD );
        return FALSE;
        }
    if( tblHdr.shortFloatSize != currentShortFloatSize ) {
        fclose( worldFD->fp );
        reinitializeFileDescriptor( worldFD );
        return FALSE;
        }
    return TRUE;
    }

/**** loadWorld -- Simple means to load all Pixie state from a file. ****/

void WorldHandler::loadWorld( volatile memorySpace *where, volatile memorySpace *alternate )
{
    if( isMomCatP ) {

        theClowder->setMomSaysToEverybody( MOM_SAYS_WORLD_OPERATION );
        while( ! theClowder->allKittensSay( KITTEN_ACKNOWLEDGES_WORLD_OPERATION ) )
            usleep( WORLD_OPERATION_DELAY );

        worldFD->fp = fopen( startString( &R ), "r" );
        strcpy( worldFD->oldPathName, worldFD->pathName );
        strcpy( worldFD->pathName, startString( &R ) );
        strcpy( worldFD->type, "r" );
        tprintf3( &CurrentOutputPort, MiscStrings[ 292 ], worldFD->pathName );
        loadWorldGuts( worldFD, where, alternate, false );

        theClowder->setMomSaysToEverybody( MOM_SAYS_RESET );

        }
    else
        resetError( 212 );
    }

// Save *just* the user setup to a given file.  Requires theUserSetup and theString to have been initialized.

void WorldHandler::saveUserSetupToFileNamed( Text *setupFileName )
{
    const Text *emptyString = "";
    FILE *setupFile = fopen( setupFileName, "w" );

    if( theString != 0 )
        theUserSetup.strLength = strlen( theString );
    else
        theUserSetup.strLength = 0;

    if( setupFile == NULL )
        resetError( 337 );

    if( (LongWord)fwrite( (void *)&theUserSetup, sizeof( Byte ), sizeof( userSetup ), setupFile ) != sizeof( userSetup ) ) {
        fclose( setupFile );
        resetError( 336 );
        }

    if( theString != 0 ) {
        if( fwrite( (void *)theString, strlen( theString ) + 1, 1, setupFile ) != 1 ) {
            fclose( setupFile );
            resetError( 336 );
            }
        }
    else {
        if( fwrite( (void *)emptyString, 1, 1, setupFile ) != 1 ) {
            fclose( setupFile );
            resetError( 336 );
            }
        }
    fclose( setupFile );
    }

/**** saveWorldGuts -- Simple means to save all Pixie state into a file. ****/

void WorldHandler::saveWorldGuts( fileDescriptor *wrldFile, volatile memorySpace *where, volatile memorySpace *alternate )
{
    struct tableHeader tblHdr;
    UnsignedLongWord tableSize;
    volatile Byte *heapPointer = NULL;
    taggedAval *cTA;
    UnsignedLongWord currentLength = 0;
    LongWord nWritten = 0;

    if( wrldFile->fp == NULL )
        resetError( 126 );
    tableSize = sizeof( pFVoidTable ) / sizeof( pFVoid );
    (tblHdr.versionString)[0] = 0;
    strcat( tblHdr.versionString, releaseDate );
    strcat( tblHdr.versionString, " ");
    strcat( tblHdr.versionString, releaseTime );
    tblHdr.IVRCursor = where->end - (Byte *)(IVR.u.a);
    tblHdr.topECursor = where->end - (Byte *)(TopLevelE.u.a);
    tblHdr.ObListCursor = where->end - (Byte *)(ObList.u.a);
    tblHdr.TLCCursor = where->end - (Byte *)(TLC.u.a);
    tblHdr.heapSizeWanted = where->end - where->free;
    tblHdr.nKittensExpected = theNumberOfKittens;
    tblHdr.tableSizeExpected = tableSize;
    tblHdr.prDepth = printDepth; 
    tblHdr.prLength = printLength; 
    tblHdr.vPrDepth = vPrintDepth;
    tblHdr.vPrLength = vPrintLength;
    tblHdr.extra1 = 0;  // Dummy value, not presently in use.
    tblHdr.extra2 = 0;  // Dummy value, not presently in use.
    tblHdr.extra3 = 0;  // Dummy value, not presently in use.
    tblHdr.worldType = (UnsignedLongWord)currentWorldType;

    tblHdr.savedUserSetup = theUserSetup;  // Yes, structure copy.
    if( theString )
        tblHdr.stringLength = strlen( theString );
    else
        tblHdr.stringLength = 0;

    tblHdr.shortFloatSize = (UnsignedLongWord)currentShortFloatSize;

    // About to write a table.

    if( (nWritten = fwrite( (void *)( &tblHdr ), sizeof( tblHdr ), 1, wrldFile->fp )) != 1 ) {
        fclose( wrldFile->fp );
        reinitializeFileDescriptor( wrldFile );
        resetError( 115 );
        }

    for( heapPointer = where->free; heapPointer < where->end; heapPointer += currentLength ) {
        cTA = (taggedAval *)heapPointer;
        fixPointerForSave( where, cTA );
        currentLength = lengthTaggedAvalPointer( cTA );
        }

    // About to write a main memory.

    if(    (LongWord)fwrite( (void *)(where->free),
                                     sizeof( Byte ),
                                     where->end - where->free,
                                     wrldFile->fp )
        != (where->end - where->free) ) {
        fclose( wrldFile->fp );
        reinitializeFileDescriptor( wrldFile );
        resetError( 115 );
        }

    for( heapPointer = where->free; heapPointer < where->end; heapPointer += currentLength ) {
        cTA = (taggedAval *)heapPointer;
        fixPointerForLoad( where, cTA );
        currentLength = lengthTaggedAvalPointer( cTA );
        }

    if( tblHdr.stringLength > 0 ) 
        if( (nWritten = fwrite( (void *)theString, strlen( theString ) + 1, 1, wrldFile->fp )) != 1 ) {
            fclose( wrldFile->fp );
            reinitializeFileDescriptor( wrldFile );
            resetError( 115 );
            }

    fclose( wrldFile->fp );
    reinitializeFileDescriptor( wrldFile );
    reportEvent( WORLD_SAVE_SUCCEEDED_EVENT );
    }

/**** saveWorld -- Simple means to save all Pixie state to a file. ****/

void WorldHandler::saveWorld( volatile memorySpace *where, volatile memorySpace *alternate )
{
    Text fileName[ MAX_PATHNAME_LENGTH ];

    if( isMomCatP ) {

        theClowder->setMomSaysToEverybody( MOM_SAYS_WORLD_OPERATION );
        while( ! theClowder->allKittensSay( KITTEN_ACKNOWLEDGES_WORLD_OPERATION ) )
            usleep( WORLD_OPERATION_DELAY );

        B = TRUE;
        
        if( anyOpenPortsP() )
            resetError( 253 );
        strcpy( fileName, startString( &R ) );
        strcpy( worldFD->oldPathName, worldFD->pathName );
        strcpy( worldFD->pathName, startString( &R ) );
#ifndef PIXIE_SCHEME_III
        tprintf3( &CurrentOutputPort, MiscStrings[ 286 ], worldFD->pathName );
#endif
        worldFD->fp = fopen( worldFD->pathName, "w" );
        strcpy( worldFD->type, "w" );

#if USE_COCOA_IO

extern void setWorldAttributesForPath( const char *path );

        setWorldAttributesForPath( worldFD->pathName );

#endif

        saveWorldGuts( worldFD, where, alternate );

        theClowder->setMomSaysToEverybody( MOM_SAYS_NOTHING );

        }
    else
        resetError( 213 );
    }

/******** Stuff to examine a world without loading it. ********/

/**** examineWorld -- Simple means to examine all Wraith Scheme state in a file. ****/

void WorldHandler::examineWorld( Boolean compareP, Text *fileName, volatile memorySpace *where, volatile memorySpace *alternate )
{
    volatile Byte *cwp, *sp, *xcwp, *xsp;
    taggedAval cwt, st;
    UnsignedLongWord currentLength;
    Tag tg;
    MemoryManager *localMemoryManager = new MemoryManager;

    localMemoryManager->initialize();
    volatile memorySpace *scratchSpace = localMemoryManager->initializeMemorySpace( (size_t)where->size );

    if( compareP ) {
        worldFD->fp = fopen( fileName, "r" );
        strcpy( worldFD->oldPathName, worldFD->pathName );
        strcpy( worldFD->pathName, fileName );
        }
    else {
        worldFD->fp = fopen( startString( &R ), "r" );
        strcpy( worldFD->pathName, startString( &R ) );
       }
    strcpy( worldFD->type, "r" );
    loadWorldGuts( worldFD, scratchSpace, alternate, true );

    if( compareP ) {
        cwp = where->free;
        sp = scratchSpace->free;
        while( cwp < where->end ) {
            cwt = *(taggedAval *)cwp;
            st = *(taggedAval *)sp;
            tg = pure_type_code( ((taggedAval *)sp)->t );
            currentLength = lengthTaggedAvalPointer( (taggedAval *)cwp );
            if( ! leaf_p( tg ) ) 
                st.u.c = (LongWord)(st.u.c) - (scratchSpace->free - where->free);
            if( cwt.u.c != st.u.c ) {
                tprintf5( &CurrentOutputPort, MiscStrings[ 287 ],
                         (leaf_p( tg ) ? MiscStrings[ 288 ] : MiscStrings[ 289 ] ),
                         cwp - where->free, where->end - where->free );
                describeMemorySpace( where, true );
                break;
                }
            if( cwt.t != st.t ) {
                tprintf4( &CurrentOutputPort, MiscStrings[ 290 ],
                         cwp - where->free, where->end - where->free );
                describeMemorySpace( where, true );
                break;
                }
            for( xcwp = cwp + sizeof( taggedAval ), xsp = sp + sizeof( taggedAval ); xcwp < cwp + currentLength; ++xcwp, ++xsp ) {
                if( *xcwp != *xsp ) {
                    tprintf4( &CurrentOutputPort, MiscStrings[ 291 ],
                             cwp - where->free, where->end - where->free );
                    describeMemorySpace( where, true );
                    break;
                    }
                }
            cwp += currentLength;
            sp += currentLength;
            }
        describeMemorySpace( where, true );
        }
    ::free( (void *)scratchSpace );

    scratchSpace = NULL;     // Defensive

    delete localMemoryManager;
    }

/**** setUserSetup ****/

void WorldHandler::setUserSetup( userSetup *aUserSetup ) { memcpy( &theUserSetup, aUserSetup, sizeof( userSetup ) ); }

// Read user setup from a named file:

void WorldHandler::readUserSetupFromFileNamed( Text *setupFileName )
{
    FILE *setupFile = fopen( setupFileName, "r" );
    size_t strLen = 0;
    Text *stringBuffer = 0;

    if( setupFile == NULL )
        resetError( 339 );

    if( (LongWord)fread( (void *)&theUserSetup, sizeof( Byte ), sizeof( userSetup ), setupFile ) != sizeof( userSetup ) ) {
        fclose( setupFile );
        resetError( 338 );
        }

    strLen = (size_t)theUserSetup.strLength;
    stringBuffer = (Text *)calloc( strLen + 1, sizeof( Text ) );
    if( stringBuffer == 0 )
        fatalError( 193 );

    if( fread( (void *)stringBuffer, strLen + 1, 1, setupFile ) != 1 ) {
        fclose( setupFile );
        resetError( 338 );
        }

    theString = stringBuffer;
    }

/**** retrieveUserSetup ****/

void WorldHandler::retrieveUserSetup( userSetup *aUserSetup) { memcpy( aUserSetup, &theUserSetup, sizeof( userSetup ) ); }

/**** setTheString ****/

void WorldHandler::setTheString( Text *aString )
{
    Text *stringBuffer = (Text *)calloc( strlen( aString ) + 1, sizeof( Text ) );

    if( stringBuffer == 0 )
        fatalError( 193 );

    strcpy( stringBuffer, aString );
    theString = stringBuffer;
    }

/**** freeTheString ****/

void WorldHandler::freeTheString() 
{
    if( theString != 0 ) {
        ::free( theString );
        theString = 0;
        }
    }

/**** getTheString ****/

Text *WorldHandler::getTheString() { return theString; }

/**** printfCodePointerTable -- describe a codePointerTable. ****/

void WorldHandler::printfCodePointerTable( cursorAndCode *table, UnsignedLongWord tableSize )
{
    UnsignedLongWord counter = 0;

#ifndef PIXIE_SCHEME_III
    printf( "Table of size %lu\n", tableSize ); fflush( stdout );
#else
    printf( "Table of size %llu\n", tableSize ); fflush( stdout );
#endif
    for( counter = 500; counter < tableSize; ++counter ) {
#ifdef PIXIE_SCHEME_III
        printf( " %llu: At %p, %llu 0x%016llx", counter, &(table[counter]), (table[counter]).tableCursor, (table[counter]).codePointer );
#else
        printf( " %lu: At %p, %lu 0x%016lx", counter, &(table[counter]), (table[counter]).tableCursor, (table[counter]).codePointer );
#endif
        fflush( stdout );
        }
    }

